/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Base Frame View-Object Routines
-end*/
 
/*-Intro:
The base frame view-object is just an empty 'top level' window on which other 
view-objects may be laid out. The window manager will usually add a frame of 
some kind round such a window and its exact appearance will therefore depend on
the system on which the program is being run. Frequently the added frame will 
display a title which has been supplied by the application programmer. Any 
operations on the base frame (e.g. moving it or resizing it) are functions of 
the particular window manager which is running.
-end*/
 
/*-Section: Creating and Handling a Base Frame View-Object
-end*/
 

/*Routines List:
The base frame view-object has the following routines available:

   xdl_base_frame              Create a new base frame view-object
*/

/*-Routine: Create a base frame view-object - xdl_base_frame
The routine xdl_base_frame (xdlf_base_frame) is used to create a new base
frame view-object. The size of the frame is given and a title and icon
label are supplied. A preferred position may be specified if required
though this is just a request to the window manager which may not
necesssarily be honoured.
-end*/
/*
***************************
**     xdl_base_frame    **
***************************

Purpose: Create a new base frame

Author:  John W. Campbell, April 1990

*/
/*-Fortran:
         CALL XDLF_BASE_FRAME (IVH, IWIDTH, IHEIGHT, XDLSTR(TITLE), 
        +                      ITLEN, ICON_LABL, ILEN, IX, IY)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
IWIDTH     (R)  Frame width (see width)
IHEIGHT    (R)  Frame height (see height)
TITLE      (R)  Fortran character string containing the title
                ** Pass address using the XDLSTR function **
                (cf title)
ITITL      (R)  The title length (> 0) (cf title_len)
ICON_LABL  (R)  Fortran character string containing the icon label
                ** Pass address using the XDLSTR function **
                (cf icon_label)
ILEN       (R) The icon label length (> 0) (cf icon_len)
IX         (R) The requested 'x' position on the root window
               for the frame (-1 allow window manager to choose)
               (see x)
IY         (R) The requested 'y' position c.f. IX
IY         (R) The requested 'y' position c.f. IX
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

/* Define routine types */
void xdl_base_frame();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_base_frame xdlf_base_frame_

#endif

#if  LINKTYP == 2

#define xdlf_base_frame XDLF_BASE_FRAME

#endif

void xdlf_base_frame();
/*-C:*/
void xdl_base_frame (vh, width, height, title, title_len, 
                     icon_label, icon_len, x, y)
/*end*/
 
/*-Parameters:*/
int vh;             /* User selected view-object handle (R)*/
int width;          /* Frame width (including border width of 1 at each 
                       side but excluding any window manager additions) 
                       (R)*/
int height;         /* Frame height (including border width of 1 at top
                       and bottom but excluding any window manager 
                       additions) (R)*/
char * title;       /* Title string (R)*/
int title_len;      /* The length of the title string; if the string is
                       null terminated then 0 may be given and the program
                       will determine the string length (R)*/
char * icon_label;  /* Icon label string (R)*/
int icon_len;       /* The length of the icon label string; if the string 
                       is null terminated then 0 may be given and the 
                       program will determine the string length (R)*/
int x;              /* Requested root window 'x' position. This acts
                       as a hint to the window manager (if x=-1 and
                       y=-1 then the choice of position is left to
                       the window manager (R)*/
int y;              /* Requested root window 'y' position. c.f. 'x' (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   Window wid;                 /* Window id for the frame */
   int len;                    /* String length */
   char * temp_title;          /* Temporary copy of the title (null term) */
   char * temp_icon;           /* Temporary copy of icon label (null term) */

   /* Create the new base frame */

   /* Make copy of title in case it was not null terminated */
   len = title_len;
   if (len<=0) len = strlen(title);
   temp_title = (char *) malloc (len+1);
   strncpy (temp_title, title, len);
   temp_title[len] = 0;

   /* Make copy of icon label in case it was not null terminated */
   len = icon_len;
   if (len<=0) len = strlen(icon_label);
   temp_icon = (char *) malloc (len+1);
   strncpy (temp_icon, icon_label, len);
   temp_icon[len] = 0;

   wid = xdl_create_base_frame_window (temp_title, temp_icon, x, y,
                                       width, height);

   xdl_new_view_object (vh, wid, 0, NULL, 1001, NULL, NULL);

   free (temp_title);
   free (temp_icon);

   return;
}

/* Fortran binding: xdlf_base_frame */

void xdlf_base_frame (ivh, iwidth, iheight, title, itlen, icon_labl, ilen,
                      ix, iy)
int *ivh, *iwidth, *iheight, *title, *itlen, *icon_labl, *ilen, *ix, *iy;
{
   xdl_base_frame (*ivh, *iwidth, *iheight, XDL_pointers[*title], *itlen,
                  XDL_pointers[*icon_labl], *ilen, *ix, *iy);
   return;
}
