/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: Image Background Calculation Routines
-end*/
 
/*-Intro:
These are a set of routines for calculating a background image and
background values from a single crystal X-ray diffraction image. The routines 
are written in C. Fortran interfaces are provided. They were written, in 
particular, for use with programs using XDL_VIEW routines, in particular 
xdl_image (xdlf_image) and related routines.
-end*/
 
/*-Section: Background Calculation Routines
-end*/
 

/*Routines List:
The following background image calculating routines are available:

   xdl_bg_calc       Calculate background image (2-D search)
   xdl_bg_calc_prog  As xdl_bg_calc but with progress bar
   xdl_bg_strip      Calculate radial background function
   xdl_bg_radimg     Form radially averaged background image
   xdl_bg_val        Get background value from radial function
   xdl_bg_valimg     Get background value from background image
*/

#include "xdl_systyp.h"
#include <math.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

/* Define Integer & Integer*2 image data types */

typedef int Intpix;
typedef unsigned short I2pix;
typedef short I2squash;

/* Define unsquash/squash macros for squashed short integers */

#define unsq(i) i<0?-i<<3:i
#define sqsh(i) i>32767?-i>>3:i

/* Define routine types */
int xdl_bg_calc();
int xdl_bg_calc_prog();
int xdl_bg_strip();
int xdl_bg_radimg();
int xdl_bg_val();
int xdl_bg_valimg();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_bg_calc xdlf_bg_calc_
#define xdlf_bg_calc_prog xdlf_bg_calc_prog_
#define xdlf_bg_strip xdlf_bg_strip_
#define xdlf_bg_radimg xdlf_bg_radimg_
#define xdlf_bg_val xdlf_bg_val_
#define xdlf_bg_valimg xdlf_bg_valimg_

#endif

#if  LINKTYP == 2

#define xdlf_bg_calc XDLF_BG_CALC
#define xdlf_bg_calc_prog XDLF_BG_CALC_PROG
#define xdlf_bg_strip XDLF_BG_STRIP
#define xdlf_bg_radimg XDLF_BG_RADIMG
#define xdlf_bg_val XDLF_BG_VAL
#define xdlf_bg_valimg XDLF_BG_VALIMG

#endif

void xdlf_bg_calc();
void xdlf_bg_calc_prog();
void xdlf_bg_strip();
void xdlf_bg_radimg();
void xdlf_bg_val();
void xdlf_bg_valimg();

/*-Routine: Calculate background image using a 2-D search - xdl_bg_calc
The routine xdl_bg_calc (xdlf_bg_calc) is used to calculate
a background image from an image. This is for single 
crystal X-ray diffraction images (i.e. one with spots over a fairly slowly 
changing background). The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. To optimise the calculation, the
box is walked through the image moving a pixel at a time starting
at the top left. It then repeats the process of left from to right,
then one pixel down, then from right to left, and then one
pixel down etc. until the film image has been covered. (In this
description the slow moving image index is visualised as being from top to 
bottom and the fast one from left to right). The routines will cope with
a maximum background level up to around 65000.
-end*/
/*
*****************************
**   xdl_bg_calc           **
*****************************

Purpose: Calculate a background image for an image using a 2-D search

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_CALC (IMG, ITYPE, NSRASTS, NFRASTS, NFOFF, 
        +                   NCMP, NPBOX, IPCNT, IMG_BG, NFBGOFF, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the image data 
                  (normally packed into an integer array) 
                  (see 'img')
ITYPE       (R)   Image data type =1 unsigned byte, 
                                  =2 unsigned two-byte, 
                                  =3 signed integer 
                                  =4 squashed i2
                  (see 'type')
NSRASTS     (R)   No. rasters along slow image axis 
                  (see 'ns_rasts')
NFRASTS     (R)   No. rasters along fast image axis
                  (see 'nf_rasts')
NFOFF       (R)   Offset (in pixels) between successive slow
                  rasters (see 'nf_off')
NCMP        (R)   Compress (sample) NCMP into 1 pixel in each 
                  direction for background image (see ncmp)
NPBOX       (R)   Dimension of background calculation box in
                  pixels of compressed imagee (preferably odd) 
                  (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background.
                  If -IPCNT is given, then pixel values of zero
                  will be omitted from the calculation.
                  (see 'percent')
IMG_BG      (W)   Array to hold the returned background image.
                  (pass an integer array large enough to hold 
                  the returned image data) (see 'img_bg')
NFBGOFF     (R/W) Offset between start of lines in returned
                  background image (allows for padding if 
                  required). If zero on input then value will
                  be returned for no padding cases (see nf_bgoff)
IERR        (W)   Returns the status from the xdl_bg_calc
                  call
-end*/

/*-C:*/
int xdl_bg_calc (img, type, ns_rasts, nf_rasts, nf_off, ncmp, npbox, 
                 percent, img_bg, nf_bgoff)
/*end*/
 
/*-Parameters:*/
char *img;                 /* Image from which to calculate the background 
                              image (R)*/
int type;                  /* Image (and backgound image) data type
                              =1 unsigned byte
                              =2 unsigned two-byte data
                              =3 signed integer data
                              =4 'squashed i2' data (if Intensity>32767
                                 store as 65536-Intensity/8)
                              (R)*/
int ns_rasts;              /* No. of slow moving rasters in the image (R)*/
int nf_rasts;              /* No. of fast moving rasters in the image (R)*/
int nf_off;                /* Offset in the image (in pixels) between start
                              of each line of the image as stored (allows
                              for padding at the end of each line if 
                              used) (R)*/
int ncmp;                  /* No. of pixels to compress into 1 in each
                              direction for the background image. The
                              size of the background image will be
                              ns_rasts/ncmp * nf_rasts/ncmp (integer
                              divisions) (R)*/
int npbox;                 /* Size of box (after compression of the image)
                              for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              If -percent is given, then pixel values of 
                              zero will be omitted from the calculation
                              (R)*/
char *img_bg;              /* Returns background image in same data format
                              as the input image. The size of the 
                              background image will be
                              ns_rasts/ncmp * nf_rasts/ncmp (integer
                              divisions) (see also mf_bgoff) (W)*/
int *nf_bgoff;             /* Offset between start of image lines in
                              returned background image. If value
                              given is less than nf_rasts/ncmp then
                              it will be reset to nf_rasts/ncmp (R/W)*/
                              
/*end*/
/*-Doc:
Return:  =0 OK; =1 background too high (> 65000); =2 box too big for image
-end*/
{
   return xdl_bg_calc_prog (img, type, ns_rasts, nf_rasts, nf_off, ncmp, 
                            npbox, percent, img_bg, nf_bgoff, 0, 0);
}
/* Fortran binding: xdlf_bg_calc */

void xdlf_bg_calc (img, itype, nsrasts, nfrasts, nfoff, ncmp, npbox, 
                   ipcnt, img_bg, nfbgoff, ierr)
int *img, *itype, *nsrasts, *nfrasts, *nfoff, *ncmp, *npbox;
int *ipcnt, *img_bg, *nfbgoff, *ierr;
{
   *ierr = xdl_bg_calc ((char *) img, *itype, *nsrasts, *nfrasts, *nfoff, *ncmp,
                        *npbox, *ipcnt, (char *) img_bg, nfbgoff);
   return;
}
   
/*-Routine: Background 2-D search with progress bar - xdl_bg_calc_prog
The routine xdl_bg_calc_ptrog (xdlf_bg_calc_prog) is the same as xdl_bg_calc
(xdlf_bg_calc) except that it has allows for routines to be passed to it
to monitor the progress of the background calculation and to cancel i
the calculation if required.
-end*/
/*
*******************************
**   xdl_bg_calc_prog        **
*******************************

Purpose: Calculate a background image for an image using a 2-D search

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_CALC_PROG (IMG, ITYPE, NSRASTS, NFRASTS, NFOFF, 
        +                   NCMP, NPBOX, IPCNT, IMG_BG, NFBGOFF, 
        +                   PROGRESS, CANCEL, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the image data 
                  (normally packed into an integer array) 
                  (see 'img')
ITYPE       (R)   Image data type =1 unsigned byte, 
                                  =2 unsigned two-byte, 
                                  =3 signed integer 
                                  =4 squashed i2
                  (see 'type')
NSRASTS     (R)   No. rasters along slow image axis 
                  (see 'ns_rasts')
NFRASTS     (R)   No. rasters along fast image axis
                  (see 'nf_rasts')
NFOFF       (R)   Offset (in pixels) between successive slow
                  rasters (see 'nf_off')
NCMP        (R)   Compress (sample) NCMP into 1 pixel in each 
                  direction for background image (see ncmp)
NPBOX       (R)   Dimension of background calculation box in
                  pixels of compressed imagee (preferably odd) 
                  (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background.
                  If -IPCNT is given, then pixel values of zero
                  will be omitted from the calculation.
                  (see 'percent')
IMG_BG      (W)   Array to hold the returned background image.
                  (pass an integer array large enough to hold 
                  the returned image data) (see 'img_bg')
NFBGOFF     (R/W) Offset between start of lines in returned
                  background image (allows for padding if 
                  required). If zero on input then value will
                  be returned for no padding cases (see nf_bgoff)
PROGRESS    (R)   Subroutine to monitor progress. Will be
                  called after each processing 'strip' 
                  as follows:
                  as follows:
                  CALL PROGRESS (IFLAG, NSTRIP, ISTRIP)
                  CALL PROGRESS (IFLAG, NSTRIP, ISTRIP)
                  IFLAG    (R)   =0 start, =1 in progress, 
                                 =2 done
                  NSTRIP   (R)   Total no. of strips of image to 
                                 process
                  ISTRIP   (R)   Current strip (1 to NSTRIP)
                  ISTRIP   (R)   Current strip (1 to NSTRIP)
CANCEL      (R)   This routine will be called at intervals to
                  see if a cancel state has been set.
                  see if a cancel state has been set.
                  The call is:
                  The call is:
                  CALL CANCEL (ICANCL)
                  CALL CANCEL (ICANCL)
                  ICANCL   (W)  Return 0 to continue, 1 to cancel
                  ICANCL   (W)  Return 0 to continue, 1 to cancel
IERR        (W)   Returns the status from the xdl_bg_calc_prog
                  call
-end*/

/*-C:*/
int xdl_bg_calc_prog (img, type, ns_rasts, nf_rasts, nf_off, ncmp, npbox, 
                 percent, img_bg, nf_bgoff, progress, cancel)
/*end*/
 
/*-Parameters:*/
char *img;                 /* Image from which to calculate the background 
                              image (R)*/
int type;                  /* Image (and backgound image) data type
                              =1 unsigned byte
                              =2 unsigned two-byte data
                              =3 signed integer data
                              =4 'squashed i2' data (if Intensity>32767
                                 store as 65536-Intensity/8)
                              (R)*/
int ns_rasts;              /* No. of slow moving rasters in the image (R)*/
int nf_rasts;              /* No. of fast moving rasters in the image (R)*/
int nf_off;                /* Offset in the image (in pixels) between start
                              of each line of the image as stored (allows
                              for padding at the end of each line if 
                              used) (R)*/
int ncmp;                  /* No. of pixels to compress into 1 in each
                              direction for the background image. The
                              size of the background image will be
                              ns_rasts/ncmp * nf_rasts/ncmp (integer
                              divisions) (R)*/
int npbox;                 /* Size of box (after compression of the image)
                              for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              If -percent is given, then pixel values of 
                              zero will be omitted from the calculation
                              (R)*/
char *img_bg;              /* Returns background image in same data format
                              as the input image. The size of the 
                              background image will be
                              ns_rasts/ncmp * nf_rasts/ncmp (integer
                              divisions) (see also mf_bgoff) (W)*/
int *nf_bgoff;             /* Offset between start of image lines in
                              returned background image. If value
                              given is less than nf_rasts/ncmp then
                              it will be reset to nf_rasts/ncmp (R/W)*/
void (*progress)();        /* Function passed to the routine to monitor
                              the progress of the background calculation.
                              called as follows: 

                              progress (iflag, nstrip, istrip);

                              int *iflag;    =0 start, =1 in progress, 
                                             =2 done
                              int *nstrip;   Total no. of strips of image to 
                                             process
                              int *istrip;   Current strip (1 to nstrip)

                              (May be a NULL)
                              (R)*/
void (*cancel)();          /* Function passed to the routine to enable
                              cancellation. Called as follows:

                              cancel (icancl);

                              int *icancl;  Function must return 0 to 
                                            continue, or 1 to cancel.
                              (May be a NULL)
                              (R)*/                              
/*end*/
/*-Doc:
Return:  =0 OK; =1 background too high (> 65000); =2 box too big for image
         -1 interrupted
-end*/
{

   int count[2048];   /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box (compressed) 
                         from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_f;         /* The number of fast axis steps required */
   int max_s;         /* The number of slow axis steps required */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset1;       /* Offset addition for sampled data */
   int offset2;       /* Pixel offset from top to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values included in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   enum step {down,up,right} nxt, last; /*Next and last step directions */
   int kf;            /* Step counter in fast direction */
   int ks;            /* Step counters in slow direction */
   int ii;            /* Temp intensity value */
   int jj;            /* Offset into count array for intensities>1023 */
   int iii;           /* Temp offset value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int min_foff;      /* Minimum fast axis offset for which background 
                         pixel found */
   int max_foff;      /* Maximum fast axis offset for which background 
                         pixel found */
   int min_soff;      /* Minimum slow axis offset for which background 
                         pixel found */
   int max_soff;      /* Maximum slow axis offset for which background 
                         pixel found */
   int too_high;      /* Background level too high flag */
   int iflag;         /* Flag for progress routiine */
   int istrip;        /* Strip no. for progress routine */
   int icancl;        /* Cancel routine flag */
   int start;
   int use_zero;
   unsigned char *byte_data; /* Pointer to unsigned byte image data */
   I2pix *i2_data;    /* Pointer to unsigned two-byte image data */
   I2squash *i2_squash; /* Pointer to 'squashed i2' image data */
   Intpix *int_data;  /* Pointer to signed integer image data */
   unsigned char *byte_bgdata; /* Pointer to unsigned byte background 
                                  image data */
   I2pix *i2_bgdata;  /* Pointer to unsigned two-byte background image data */
   I2squash *i2_bgsquash; /* Pointer to 'squased i2' background image data */
   Intpix *int_bgdata;/* Pointer to signed integer background image data */

   use_zero = percent < 0 ? 0 : 1;
   percent  = percent < 0 ? -percent : percent;

   too_high = 0;
   for(i=0;i<2048;++i) count[i]=0;
   nbgmin = (npbox*npbox*percent)/100;
   nf_rasts = ncmp*(nf_rasts/ncmp);
   ns_rasts = ncmp*(ns_rasts/ncmp);
   if (npbox>nf_rasts/ncmp||npbox>ns_rasts/ncmp) return 2;
   offset = npbox/2;
   max_f = nf_rasts/ncmp - npbox;
   max_s = ns_rasts/ncmp - npbox;
   min_foff = offset;
   max_foff = nf_rasts/ncmp + min_foff - npbox;
   min_soff = offset;
   max_soff = ns_rasts/ncmp + min_soff - npbox;
   offset1 = (ncmp/2)*nf_off + (ncmp/2);
   offset2 = npbox*ncmp*nf_off;
   int_data = (Intpix *) img;
   i2_data = (I2pix *) img;
   i2_squash = (I2squash *) img;
   byte_data = (unsigned char *) img;
   int_bgdata = (Intpix *) img_bg;
   i2_bgdata = (I2pix *) img_bg;
   i2_bgsquash = (I2squash *) img_bg;
   byte_bgdata = (unsigned char *) img_bg;
   if (*nf_bgoff<nf_rasts/ncmp) *nf_bgoff = nf_rasts/ncmp;

   /* Initial call to progress monitor */

   iflag = 0;
   istrip = 1;
   if (progress!=0) (*progress)(&iflag, &max_f, &istrip);
   iflag = 1;

   /* Full scan of image */

   /* Process initial background box */
   k=offset1;
   for (i=0;i<npbox;i++)
   {
      j = k;
      for (l=0;l<npbox;l++) 
      {
         switch (type)
         {
            case 1:
               ii = byte_data[j]; j+=ncmp;
            break;

            case 2:
               ii = i2_data[j]; j+=ncmp;
            break;

            case 3:
               ii = int_data[j]; j+=ncmp;
            break;

            case 4:
               ii = unsq(i2_squash[j]); j+=ncmp;
            break;
         }
         if (ii>1023)
         {
            jj = 1024 + ((ii-992)>>6);
            if (jj>2047) jj = 2047;
            count[jj]++;
         }
         else
         {
            count[ii]++;
         }
      }
      k += ncmp*nf_off;
   }
   nbgmin = use_zero ? nbgmin : ( npbox*npbox - count[0] ) * percent / 100;
   start = use_zero ? 0 : 1;
   ni=0; intens_tot=0;
   for (i=start,nl=start;i<2048;++i,++nl)
   {
      ni += count[i];
      if (i>1023)
      {
         intens_tot += count[i]*(1024+((i-1024)<<6));
         if (i==2047) too_high = 1;
      }
      else
      {
         intens_tot += count[i]*i;
      }
      if (ni>=nbgmin) break;
   }
   ipx = offset*(*nf_bgoff) + offset;
   switch (type)
   {
      case 1:
      	byte_bgdata[ipx] = ni ? intens_tot/ni : 0;
      break;

      case 2:
        i2_bgdata[ipx] = ni ? intens_tot/ni : 0;
      break;

      case 3:
        int_bgdata[ipx] = ni ? intens_tot/ni : 0;
      break;

      case 4:
        i2_bgsquash[ipx] = ni ? sqsh(intens_tot/ni) : 0;
      break;
   }
   ipx += *nf_bgoff;

   /* Start walk through the image */
   
   kf = 0;
   ks = 1;
   ip = offset1;
   jp = offset1 + offset2;
   nxt=down;

   next_box:

   /* Remove intensities for left (or top or right) row of current box
      depending on the direction the box is being moved */

   /* Add intensities for right (or bottom or left) row of shifted box */

   if (nxt!=right)   
   {
      for (i=0;i<npbox;++i)
      {
         switch (type)
         {
            case 1:
               ii = byte_data[ip]; ip+=ncmp;
            break;

            case 2:
               ii = i2_data[ip]; ip+=ncmp;
            break;

            case 3:
               ii = int_data[ip]; ip+=ncmp;
            break;

            case 4:
               ii = unsq(i2_squash[ip]); ip+=ncmp;
            break;
         }
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            if (jj>2047) jj = 2047;
            count[jj]--;
            if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
         }
         else
         {
            count[ii]--;
            if (ii<=nl) {intens_tot-=ii; ni--;}
            if (ii<=nl && !ii && !use_zero) {intens_tot+=ii; ni++;}
         }
         switch (type)
         {
            case 1:
               ii = byte_data[jp]; jp+=ncmp;
            break;

            case 2:
               ii = i2_data[jp]; jp+=ncmp;
            break;

            case 3:
               ii = int_data[jp]; jp+=ncmp;
            break;

            case 4:
               ii = unsq(i2_squash[jp]); jp+=ncmp;
            break;
         }
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            if (jj>2047) jj = 2047;
            count[jj]++;
            if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
         }
         else
         {
            count[ii]++;
            if (ii<=nl) {intens_tot+=ii; ni++;}
            if (ii<=nl && !ii && !use_zero) {intens_tot-=ii; ni--;}
         }
      }
   }
   else
   {
      for (i=0;i<npbox;++i)
      {
         switch (type)
         {
            case 1:
               ii = byte_data[ip]; ip+=ncmp*nf_off;
            break;

            case 2:
               ii = i2_data[ip]; ip+=ncmp*nf_off;
            break;

            case 3:
               ii = int_data[ip]; ip+=ncmp*nf_off;
            break;

            case 4:
               ii = unsq(i2_squash[ip]); ip+=ncmp*nf_off;
            break;
         }
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            if (jj>2047) jj = 2047;
            count[jj]--;
            if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
         }
         else
         {
            count[ii]--;
            if (ii<=nl) {intens_tot-=ii; ni--;}
            if (ii<=nl && !ii && !use_zero) {intens_tot+=ii; ni++;}
         }
         switch (type)
         {
            case 1:
               ii = byte_data[jp]; jp+=ncmp*nf_off;
            break;

            case 2:
               ii = i2_data[jp]; jp+=ncmp*nf_off;
            break;

            case 3:
               ii = int_data[jp]; jp+=ncmp*nf_off;
            break;

            case 4:
               ii = unsq(i2_squash[jp]); jp+=ncmp*nf_off;
            break;
         }
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            if (jj>2047) jj = 2047;
            count[jj]++;
            if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
         }
         else
         {
            count[ii]++;
            if (ii<=nl) {intens_tot+=ii; ni++;}
            if (ii<=nl && !ii && !use_zero) {intens_tot-=ii; ni--;}
         }
      }
   }

   /* Move up or down 'count' array to find new background level */

   nbgmin = use_zero ? nbgmin : ( npbox*npbox - count[0] ) * percent / 100;
   if (ni>=nbgmin)
   {
      nc = count[nl];
      while (ni-nc>=nbgmin)
      {
         if (nl>1023)
         {
            intens_tot -= nc*(1024+((nl-1024)<<6));
         }
         else
         {
            intens_tot -= nc*nl;
         }
         ni -= nc;
         nc = count[--nl];
      }
   }
   else
   {
      while (ni<nbgmin)
      {
         nc = count[++nl];
         ni += nc;
         if (nl>1023)
         {
            intens_tot += nc*(1024+((nl-1024)<<6));
            if (nl==2047) too_high = 1;
         }
         else
         {
            intens_tot += nc*nl;
         }
      }
   }

   /* Set background pixel value */

   ne = ni - nbgmin;
   if (nl>1023)
   {
      switch (type)
      {
         case 1:
            byte_bgdata[ipx] = nbgmin ? (intens_tot-ne*(1024+((nl-1024)<<6)))
                               /nbgmin : 0;
         break;

         case 2:
            i2_bgdata[ipx] = nbgmin ? (intens_tot-ne*(1024+((nl-1024)<<6)))
                             /nbgmin : 0;
         break;

         case 3:
            int_bgdata[ipx] = nbgmin ? (intens_tot-ne*(1024+((nl-1024)<<6)))
            /nbgmin : 0;
         break;

         case 4:
            i2_bgsquash[ipx]= nbgmin ? sqsh((intens_tot-ne
                              *(1024+((nl-1024)<<6)))/nbgmin) : 0;
         break;
     }
   }
   else
   {
      switch (type)
      {
         case 1:
            byte_bgdata[ipx] = nbgmin ? (intens_tot-ne*nl)/nbgmin : 0;
         break;

         case 2:
            i2_bgdata[ipx] = nbgmin ? (intens_tot-ne*nl)/nbgmin : 0;
         break;

         case 3:
            int_bgdata[ipx] = nbgmin ? (intens_tot-ne*nl)/nbgmin : 0;
         break;

         case 4:
            i2_bgsquash[ipx] = nbgmin ? sqsh((intens_tot-ne*nl)/nbgmin) : 0;
         break;
      }
   }

   /* Get direction for next step and key pixel positions */

   last = nxt;
   switch (last)
   {
      case down:
         if (ks<max_s) 
         {
            ip = offset1 + ks*ncmp*nf_off + kf*ncmp;
            jp = ip + offset2;
            ipx += *nf_bgoff;
            ks++;
         }
         else
         {
            if (kf>=max_f) goto done;
            nxt = right;
            ip = offset1 + ks*ncmp*nf_off + kf*ncmp;
            jp = ip + npbox*ncmp;
            ipx++;
            kf++;
         }
         goto next_box;
      
      case up:
         if (ks>0) 
         {
            ks--;
            jp = offset1 + ks*ncmp*nf_off + kf*ncmp;
            ip = jp + offset2;
            ipx -= *nf_bgoff;
         }
         else
         {
            if (kf>=max_f) goto done;
            nxt = right;
            ip = offset1 + kf*ncmp;
            jp = ip + npbox*ncmp;
            ipx++;
            kf++;
            if (progress!=0) 
            {
               istrip = kf;
               (*progress)(&iflag, &max_f, &istrip);
	    }
            if (cancel!=0)
	    {
               (*cancel)(&icancl);
                if (icancl==1) return -1;
            }
         }
         goto next_box;

      case right:
         if (ks==0)
         {
            nxt = down;
            ip = offset1 + kf*ncmp;
            jp = ip + offset2;
            ipx += *nf_bgoff;
            ks = 1;
         }
         else
         {
            nxt = up;
            ks--;
            jp = offset1 + ks*ncmp*nf_off + kf*ncmp;
            ip = jp + offset2;
            ipx -= *nf_bgoff;
         }
         goto next_box;
   }
   done:

   /* Fill in strip around edge */

   for (ks=0;ks<ns_rasts/ncmp;++ks)
   {
      ip = ks*(*nf_bgoff);
      for (kf=0;kf<min_foff;++kf)
      {
         if (ks<min_soff)
         {
            jp = min_soff*(*nf_bgoff)+min_foff;
         }
         else if (ks>max_soff)
         {
            jp = max_soff*(*nf_bgoff)+min_foff;
         }
         else
         {
            jp = ks*(*nf_bgoff)+min_foff;
         }
         switch (type)
         {
            case 1:
               byte_bgdata[ip] = byte_bgdata[jp];
            break;

            case 2:
               i2_bgdata[ip] = i2_bgdata[jp];
            break;

            case 3:
               int_bgdata[ip] = int_bgdata[jp];
            break;

            case 4:
               i2_bgsquash[ip] = i2_bgsquash[jp];
            break;
         }
         ip++;
      }
      ip = ks*(*nf_bgoff) + max_foff + 1;
      for (kf=max_foff+1;kf<nf_rasts/ncmp;++kf)
      {
         if (ks<min_soff)
         {
            jp = min_soff*(*nf_bgoff)+max_foff;
         }
         else if (ks>max_soff)
         {
            jp = max_soff*(*nf_bgoff)+max_foff;
         }
         else
         {
            jp = ks*(*nf_bgoff)+max_foff;
         }
         switch (type)
         {
            case 1:
               byte_bgdata[ip] = byte_bgdata[jp];
            break;

            case 2:
               i2_bgdata[ip] = i2_bgdata[jp];
            break;

            case 3:
               int_bgdata[ip] = int_bgdata[jp];
            break;

            case 4:
               i2_bgsquash[ip] = i2_bgsquash[jp];
            break;
         }
         ip++;
      }
   }
   for (ks=0;ks<min_soff;++ks)
   {
      ip = ks*(*nf_bgoff) + min_foff;
      jp = min_soff*(*nf_bgoff) + min_foff;
      for (kf=min_foff;kf<=max_foff;++kf)
      {
         switch (type)
         {
            case 1:
               byte_bgdata[ip++] = byte_bgdata[jp++];
            break;

            case 2:
               i2_bgdata[ip++] = i2_bgdata[jp++];
            break;

            case 3:
               int_bgdata[ip++] = int_bgdata[jp++];
            break;

            case 4:
               i2_bgsquash[ip++] = i2_bgsquash[jp++];
            break;
         }
      }
   }
   for (ks=max_soff+1;ks<ns_rasts/ncmp;++ks)
   {
      ip = ks*(*nf_bgoff) + min_foff;
      jp = max_soff*(*nf_bgoff) + min_foff;
      for (kf=min_foff;kf<=max_foff;++kf)
      {
         switch (type)
         {
            case 1:
               byte_bgdata[ip++] = byte_bgdata[jp++];
            break;

            case 2:
               i2_bgdata[ip++] = i2_bgdata[jp++];
            break;

            case 3:
               int_bgdata[ip++] = int_bgdata[jp++];
            break;
            case 4:
               i2_bgsquash[ip++] = i2_bgsquash[jp++];
            break;
         }
      }
   }
   if (progress!=0)
   {
      iflag = 2;
      istrip = max_f;
      (*progress)(&iflag, &max_f, &istrip);
   }

   return too_high;
}
/* Fortran binding: xdlf_bg_calc_prog */

void xdlf_bg_calc_prog (img, itype, nsrasts, nfrasts, nfoff, ncmp, npbox, 
                   ipcnt, img_bg, nfbgoff, progress, cancel, ierr)
int *img, *itype, *nsrasts, *nfrasts, *nfoff, *ncmp, *npbox;
int *ipcnt, *img_bg, *nfbgoff, *ierr;
void (*progress)();
void (*cancel)();
{
   *ierr = xdl_bg_calc_prog ((char *) img, *itype, *nsrasts, *nfrasts, *nfoff, *ncmp,
                        *npbox, *ipcnt, (char *) img_bg, nfbgoff, progress, cancel);
   return;
}

/*-Routine: Calculate radially averaged background - xdl_bg_strip
The routine xdl_bg_strip (xdlf_bg_strip) is used to calculate
a radially averaged background from an image. 
The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. The background values are found for
a horizontal (fast axis) or vertical (slow axis) strip of pixels running 
through the centre of the image and the values from the two halves of the 
strip are averaged. The routine will only cope with background levels
up to around 65000.
-end*/
/*
*****************************
**   xdl_bg_strip          **
*****************************

Purpose: Calculate a radially averaged background from an image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_STRIP (IMG, ITYPE, NSRASTS, NFRASTS, NFOFF, NCMP, 
        +                    NPBOX, IPCNT, IOPT, IBGAVE, NBG, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the image data .
                  (normally packed into an integer array) 
                  (see 'img')
ITYPE       (R)   Image data type 1 = unsigned byte, 2 =
                  unsigned two-byte, 3 = signed integer,
                  4 = 'squashed i2' (see 'type')
NSRASTS     (R)   No. rasters in slow image direction 
                  (see 'ns_rasts')
NFRASTS     (R)   No. rasters in fast image direction 
                  (see 'nf_rasts')
NFOFF       (R)   Offset (in pixels) between successive slow
                  rasters (see 'nf_off')
NCMP        (R)   Compress NCMP into 1 pixel in each direction
                  for background image (actually sample)
                  (see ncmp)
NPBOX       (R)   Dimension of background calculation box in
                  pixels of compressed image (preferably odd) 
                  (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background
                  (see 'percent')
IOPT        (R)   =1 strip across the centre along fast axis
                  =2 strip through the centre along slow axis
IBGAVE      (W)   Array returning radially averaged background
                  values (see bgave)
NBG         (W)   The number of values returned in IBGAVE
                  (maxr_off+1, see maxr_off)
IERR        (W)   Returns the status from the xdl_bg_strip
                  call
-end*/

/*-C:*/
int xdl_bg_strip (img, type, ns_rasts, nf_rasts, nf_off, ncmp, npbox, 
                  percent, iopt, bgave, maxr_off)
/*end*/
 
/*-Parameters:*/
char *img;                 /* Full image data (R) */
int type;                  /* Data type =1 unsigned byte data
                                        =2 unsigned two-byte data
                                        =3 signed integer data
                                        =4 'squashed i2' data
                              (R)*/
int ns_rasts;              /* No. of slow rasters in image (R) */
int nf_rasts;              /* No. of fast rasters in image (R) */
int nf_off;                /* Offset in rasters between lines of
                              stored image (allows for padding) (R)*/
int ncmp;                  /* Compress ncmp pixels in each direction
                              into 1 before calculating background 
                              (data are actually sampled) (R)*/
int npbox;                 /* Size of box for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (box is for compressed
                              image) (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              (R)*/
int iopt;                  /* =1, strip through centre along fast axis
                              =2, strip through centre along slow axis 
                              (R)*/
int bgave[];               /* Returns radially averaged background function
                              in intervals of pixels of the compressed
                              image and starting from the centre of that
                              image; dimension to >= (nf_rasts/ncmp)/2
                              for iopt 1 and (ns_rasts/ncmp)/2 for iopt 2 
                              (W)*/
int *maxr_off;             /* Maximum offset in returned 'bgave' array 
                              (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
         =1 Background too high (>65000)
         =2 box too big for image
         =3 Unable to allocate memory for temporary arrays;
-end*/
{

   int count[2048];   /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_f;         /* The number of fast axis steps required */
   int max_s;         /* The number of slow axis steps required */
   int offset1;       /* Offset addition for sampled data */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset2;       /* Pixel offset from top to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values includen in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   int kf;            /* Step counter in fast axis */
   int ks;            /* Step counters in slow axis */
   int ii;            /* Temp intensity value */
   int jj;            /* Offset into count array for intensities>1023 */
   int iii;           /* Temp offset value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */
   int *bg_val;       /* Array of background pixel values in line across
                         centre of the image */
   int too_high;      /* Background too high flag */
   unsigned char *byte_data; /* Pointer to unsigned byte image data */
   I2pix *i2_data;    /* Pointer to unsigned two-byte image data */
   I2squash *i2_squash; /* Pointer to 'squashed i2' image data */
   Intpix *int_data;  /* Pointer to signed integer image data */

   too_high = 0;
   for(i=0;i<2048;++i) count[i]=0;
   if (npbox>nf_rasts/ncmp||npbox>ns_rasts/ncmp) return 2;
   offset = npbox/2;
   nbgmin = (npbox*npbox*percent)/100;
   max_f = nf_rasts/ncmp - npbox;
   max_s = ns_rasts/ncmp - npbox;
   offset1 = (ncmp/2)*nf_off + (ncmp/2);
   offset2 = npbox*ncmp*nf_off;
   int_data = (Intpix *) img;
   i2_data = (I2pix *) img;
   i2_squash = (I2squash *) img;
   byte_data = (unsigned char *) img;

   switch (iopt)
   {
      case 1:

         /* Strip across centre of image along fast axis direction */

         *maxr_off = (nf_rasts/ncmp-npbox)/2;
         bg_val = (int *) malloc ((nf_rasts/ncmp)*sizeof(int));
         if (bg_val==0) return 3;
         /* Process initial background box */

         k=offset1+(ns_rasts/2-npbox*ncmp/2)*nf_off;

         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) 
            {  
               switch (type)
               {
                   case 1:
                     ii = byte_data[j]; j+=ncmp;
                   break;

                   case 2:
                     ii = i2_data[j]; j+=ncmp;
                   break;

                   case 3:
                     ii = int_data[j]; j+=ncmp;
                   break;

                   case 4:
                     ii = unsq(i2_squash[j]); j+=ncmp;
                   break;
               }
               if (ii>1023)
               {
                  jj = 1024 + ((ii-992)>>6);
                  if (jj>2047) jj = 2047;
                  count[jj]++;
               }
               else
               {
                  count[ii]++;
               }
            }
            k += ncmp*nf_off;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<2048;++i,++nl)
         {
            ni += count[i];
            if (i>1023)
            {
               intens_tot += count[i]*(1024+((i-1024)<<6));
               if (i==2047) too_high = 1;
            }
            else
            {
               intens_tot += count[i]*i;
            }
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
         kf = 1;
         ks = (ns_rasts/ncmp)/2-npbox/2;
         ip = offset1 + ks*ncmp*nf_off;
         jp = ip + npbox*ncmp;

         next_box_opt1:

         /* Remove intensities for left column of current box */

         /* Add intensities for right column of shifted box */

         for (i=0;i<npbox;++i)
         {
            switch (type)
            {
               case 1:
                  ii = byte_data[ip]; ip += ncmp*nf_off;
               break;

               case 2:
                  ii = i2_data[ip]; ip += ncmp*nf_off;
               break;

               case 3:
                  ii = int_data[ip]; ip += ncmp*nf_off;
               break;

               case 4:
                  ii = unsq(i2_squash[ip]); ip += ncmp*nf_off;
               break;
            }
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               if (jj>2047) jj = 2047;
               count[jj]--;
               if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
            }
            else
            {
               count[ii]--;
               if (ii<=nl) {intens_tot-=ii; ni--;}
            }
            switch (type)
            {
               case 1:
                  ii = byte_data[jp]; jp += ncmp*nf_off;
               break;

               case 2:
                  ii = i2_data[jp]; jp += ncmp*nf_off;
               break;

               case 3:
                  ii = int_data[jp]; jp += ncmp*nf_off;
               break;

               case 4:
                  ii = unsq(i2_squash[jp]); jp += ncmp*nf_off;
               break;
            }
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               if (jj>2047) jj = 2047;
               count[jj]++;
               if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
            }
            else
            {
               count[ii]++;
               if (ii<=nl) {intens_tot+=ii; ni++;}
            }
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               if (nl>1023)
               {
                  intens_tot -= nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot -= nc*nl;
               }
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               if (nl>1023)
               {
                  intens_tot += nc*(1024+((nl-1024)<<6));
                  if (nl==2047) too_high = 1;
               }
               else
               {
                  intens_tot += nc*nl;
               }
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        if (nl>1023)
        {
           bg_val[ipx] = (intens_tot-ne*(1024+((nl-1024)<<6)))/nbgmin;
        }
        else
        {
            bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;
        }

        /* Get key pixel positions for next step */

        if (kf>=max_f) 
        {
           hx = (nf_rasts/ncmp)/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hx-i]+bg_val[hx+i])/2;
           free (bg_val);
           return 0;
        }
        ip = offset1 + ks*ncmp*nf_off + kf*ncmp;
        jp = ip + npbox*ncmp;
        ipx++;
        kf++;
        goto next_box_opt1;

      case 2:

         /* Strip across centre of image along slow axis */
         *maxr_off = (ns_rasts/ncmp-npbox)/2;
         bg_val = (int *) malloc ((ns_rasts/ncmp)*sizeof(int));
         if (bg_val==0) return 1;
         /* Process initial background box */

         k=offset1+(nf_rasts/2-npbox*ncmp/2);
         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) 
            {
               switch (type)
               {
                   case 1:
                     ii = byte_data[j]; j+=ncmp;
                   break;

                   case 2:
                     ii = i2_data[j]; j+=ncmp;
                   break;

                   case 3:
                     ii = int_data[j]; j+=ncmp;
                   break;

                   case 4:
                     ii = unsq(i2_squash[j]); j+=ncmp;
                   break;
               }
               if (ii>1023)
               {
                  jj = 1024 + ((ii-992)>>6);
                  if (jj>2047) jj = 2047;
                  count[jj]++;
               }
               else
               {
                  count[ii]++;
               }
            }
            k += ncmp*nf_off;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<2048;++i,++nl)
         {
            ni += count[i];
            if (i>1023)
            {
               intens_tot += count[i]*(1024+((i-1024)<<6));
               if (i==2047) too_high = 1;
            }
            else
            {
               intens_tot += count[i]*i;
            }
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
   
         kf = ((nf_rasts/ncmp)/2-npbox/2);
         ks = 1;
         ip = offset1 + kf*ncmp;
         jp = ip + offset2;

         next_box_opt2:

         /* Remove intensities for top row of current box */

         /* Add intensities for bottom row of shifted box */

         for (i=0;i<npbox;++i)
         {
            switch (type)
            {
                case 1:
                  ii = byte_data[ip]; ip+=ncmp;
                break;

                case 2:
                  ii = i2_data[ip]; ip+=ncmp;
                break;

                case 3:
                  ii = int_data[ip]; ip+=ncmp;
                break;

                case 4:
                  ii = unsq(i2_squash[ip]); ip+=ncmp;
                break;
            }
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               if (jj>2047) jj = 2047;
               count[jj]--;
               if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
            }
            else
            {
               count[ii]--;
               if (ii<=nl) {intens_tot-=ii; ni--;}
            }
            switch (type)
            {
                case 1:
                  ii = byte_data[jp]; jp+=ncmp;
                break;

                case 2:
                  ii = i2_data[jp]; jp+=ncmp;
                break;

                case 3:
                  ii = int_data[jp]; jp+=ncmp;
                break;

                case 4:
                  ii = unsq(i2_squash[jp]); jp+=ncmp;
                break;
            }
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               if (jj>2047) jj = 2047;
               count[jj]++;
               if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
            }
            else
            {
               count[ii]++;
               if (ii<=nl) {intens_tot+=ii; ni++;}
            }
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               if (nl>1023)
               {
                  intens_tot -= nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot -= nc*nl;
               }
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               if (nl>1023)
               {
                  intens_tot += nc*(1024+((nl-1024)<<6));
                  if (nl==2047) too_high = 1;
               }
               else
               {
                  intens_tot += nc*nl;
               }
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        if (nl>1023)
        {
           bg_val[ipx] = (intens_tot-ne*(1024+((nl-1024)<<6)))/nbgmin;
        }
        else
        {
            bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;
        }

        /* Get key pixel positions for next step */

        if (ks>=max_s) 
        {
           hy = (ns_rasts/ncmp)/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hy-i]+bg_val[hy+i])/2;
           free (bg_val);
           return too_high;
        }
        ip = offset1 + ks*ncmp*nf_off + kf*ncmp;
        jp = ip + offset2;
        ipx++;
        ks++;
        goto next_box_opt2;
   }
   return 0;
}
/* Fortran binding: xdlf_bg_strip */

void xdlf_bg_strip (img, type, nsrasts, nfrasts, nfoff, ncmp, npbox, 
                    ipcnt, iopt, ibgave, nbg, ierr)
int *img, *type, *nsrasts, *nfrasts, *nfoff, *ncmp, *npbox;
int *ipcnt, *iopt, *ibgave, *nbg, *ierr;
{
   int maxr_off;

   *ierr = xdl_bg_strip ((char *) img, *type, *nsrasts, *nfrasts, *nfoff, *ncmp,
                         *npbox, *ipcnt, *iopt, ibgave, &maxr_off);
   *nbg = maxr_off + 1;
   return;
}
/*-Routine: Form radially averaged background image - xdl_bg_radimg
The routine xdl_bg_radimg (xdlf_bg_radimg) is used to form a
a radially averaged background image from the radially averaged
background function returned by xdl_bg_strip (xdlf_bg_strip).
-end*/
/*
*****************************
**   xdl_bg_radimg         **
*****************************

Purpose: Form radially averaged background image from radial background 
         function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_RADIMG (ITYPE, NS_BG, NF_BG, NF_BGOFF, IBGAVE, 
        +                     NBG, IMG_BG, IERR)
-end*/
 
/*-Parameters:
ITYPE       (R)   Required data type for background image
                  =1 unsigned byte
                  =2 unsigned two-byte
                  =3 signed integer
                  =4 'squashed i2'
                  (see type)
NS_BG       (R)   No. of rasters in background image along
                  slow moving axis (see ns_bg)
NF_BG       (R)   No. of rasters in background image along
                  fast moving axis (see nf_bg)
NF_BGOFF    (R)   Offset required between successive lines of
                  returned image (allows for padding if required)
                  (see nf_bgoff)
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (W)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
IMG_BG      (W)   Integer array returning background image
                  (must be large enough to hold 
                  NS_BG*NF_BGOFF data pixels) (see img_bg)
IERR        (W)   Returns the status from the xdl_bg_radimg
                  call
-end*/

/*-C:*/
int xdl_bg_radimg (type, ns_bg, nf_bg, nf_bgoff, bgave, maxr_off, 
                   img_bg)
/*end*/
 
/*-Parameters:*/
int type;               /* Background image data type 
                           =1 unsigned byte
                           =2 unsigned two-byte
                           =3 signed integer
                           =4 'squashed i2'
                           (R) */
int ns_bg;              /* No. rasters in backgound image along slow 
                           moving axis (R) */
int nf_bg;              /* No. rasters in backgound image along fast
                           moving axis (R) */
int nf_bgoff;           /* Required offset between successive lines of
                           returned background image (allows for padding
                           if required) (R) */
int bgave[];            /* Radially average background values from
                           xdl_bg_strip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_bg_strip 
                           (R)*/
char *img_bg;           /* Returns the compressed radially averaged 
                           background image (ns_bg*nf_bg padded if 
                           required) (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;           /* Temp/loop variables */
   int ip;            /* Index of pixel */
   int jp;            /* Index of pixel */
   int ix;            /* Step counter in x (fast axis)*/
   int iy,ky;         /* Step counters in y (slow axis)*/
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */
   unsigned char *byte_bgdata; /* Pointer to unsigned byte background 
                                  image data */
   I2pix *i2_bgdata;  /* Pointer to unsigned two-byte background image data */
   I2squash *i2_bgsquash; /* Pointer to 'squashed i2' background image data */
   Intpix *int_bgdata;/* Pointer to signed integer background image data */

   hx = nf_bg/2;
   hy = ns_bg/2;
   max_off = hx; if (hy>hx) max_off = hy;
   byte_bgdata = (unsigned char *) img_bg;
   i2_bgdata = (I2pix *) img_bg;
   i2_bgsquash = (I2squash *) img_bg;
   int_bgdata = (Intpix *) img_bg;

   /* Form background image */
   for (i=0;i<=max_off;++i)
   {
      for (j=0;j<=i;++j)
      {
         dsq = i*i + j*j;
         if (dsq>1)
         {
            /*Get approx square root*/
            d = i+j>>1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
         }
         else
         {
            d=dsq;
         }
         if (d>maxr_off) d = maxr_off;
         x = hx - j; 
         if (x>=0)
         {
            y = hy - i;
            if (y>=0)
            {
               switch (type)
               {
                  case 1: 
                     byte_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 2: 
                     i2_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 3: 
                     int_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 4: 
                     i2_bgsquash[y*nf_bgoff+x]=sqsh(bgave[d]);
                  break;
              }
           }
         }
         x = hx - i; 
         if (x>=0)
         {
            y = hy - j;
            if (y>=0)
            {
               switch (type)
               {
                  case 1: 
                     byte_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 2: 
                     i2_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 3: 
                     int_bgdata[y*nf_bgoff+x]=bgave[d];
                  break;

                  case 4: 
                     i2_bgsquash[y*nf_bgoff+x]=sqsh(bgave[d]);
                  break;
              }
           }
         }
      }  
   }           
   /* Form top right quarter */
   for (iy=0;iy<=hy;++iy)
   {
      ip = (iy*nf_bgoff + hx) + 1;
      jp = ip - 2;
      switch (type)
      {
         case 1:
            for (ix=hx+1;ix<nf_bg;++ix) byte_bgdata[ip++] = byte_bgdata[jp--];
         break;

         case 2:
            for (ix=hx+1;ix<nf_bg;++ix) i2_bgdata[ip++] = i2_bgdata[jp--];
         break;

         case 3:
            for (ix=hx+1;ix<nf_bg;++ix) int_bgdata[ip++] = int_bgdata[jp--];
         break;

         case 4:
            for (ix=hx+1;ix<nf_bg;++ix) i2_bgsquash[ip++] = i2_bgsquash[jp--];
         break;
     }
   }
   /* Form lower half */
   for (iy=hy+1,ky=hy-1;iy<ns_bg;++iy,--ky)
   {
      ip = iy*nf_bgoff;
      jp = ky*nf_bgoff;
      switch (type)
      {
         case 1:
            for (ix=0;ix<nf_bg;++ix) byte_bgdata[ip++] = byte_bgdata[jp++];
         break;

         case 2:
            for (ix=0;ix<nf_bg;++ix) i2_bgdata[ip++] = i2_bgdata[jp++];
         break;

         case 3:
            for (ix=0;ix<nf_bg;++ix) int_bgdata[ip++] = int_bgdata[jp++];
         break;

         case 4:
            for (ix=0;ix<nf_bg;++ix) i2_bgsquash[ip++] = i2_bgsquash[jp++];
         break;
      }
   }
   return 0;
}
/* Fortran binding: xdlf_bg_radimg */

void xdlf_bg_radimg (itype, ns_bg, nf_bg, nf_bgoff, ibgave, nbg, img_bg, ierr)
int *itype, *ns_bg, *nf_bg, *nf_bgoff, *ibgave, *nbg, *img_bg, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_bg_radimg (*itype, *ns_bg, *nf_bg, *nf_bgoff, ibgave, 
                          maxr_off, (char *) img_bg);
   return;
}

/*-Routine: Get background value from background function - xdl_bg_val
The routine xdl_bg_val (xdlf_bg_val) is used to get a background
value at a given pixel position in the full image from a radially averaged 
background function as returned by xdl_bg_strip (xdlf_bg_strip).
If the pixel is outside the range of the background function the value
at the maximum available radius will be returned.
-end*/
/*
*****************************
**     xdl_bg_val          **
*****************************

Purpose: Get background value at a pixel position in the full image from
         radially averaged background function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_VAL (ISLOW, IFAST, IBGAVE, NBG, NSRASTS, NFRASTS,
                           NCMP, IVAL, IERR)
-end*/
 
/*-Parameters:
ISLOW       (R)   Pixel  position along slow axis in full image 
                  (from 1 up)
IFAST       (R)   Pixel  position along fast axis in full image 
                  (from 1 up)
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (R)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
NSRASTS     (R)   No. rasters along slow axis in full image from 
                  which the background was calculated
NFRASTS     (R)   No. rasters along fast axis in full image from 
                  which the background was calculated
NCMP        (R)   No. pixels in each direction compressed into
                  1 for the background calculation.
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_bg_val
                  call
-end*/

/*-C:*/
int xdl_bg_val (islow, ifast, bgave, maxr_off, ns_rasts, nf_rasts,
                ncmp, ival)
/*end*/
 
/*-Parameters:*/
int islow;              /* The pixel position in the full image
                           along the slow axis (from 1 up) (R)*/
int ifast;              /* The pixel position in the full image
                           along the fast axis (from 1 up) (R)*/
int bgave[];            /* Radially average background values from
                           xdl_bg_strip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_bg_strip 
                           (R)*/
int ns_rasts;           /* No. of rasters along the slow axis of the image 
                           used to calculate background function (R)*/
int nf_rasts;           /* No. of rasters along the fast axis of the image 
                           used to calculate background function (R)*/
int ncmp;               /* No. of pixels in each direction compressed into
                           1 for the background calcn. (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;             /* Offsets from centre of compressed background 
                           image from which background function was 
                           determined */
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */
   int d, dsq;          /* Distance and d**2 from centre */
   int hx, hy;          /* Offsets of central pixel in compressed background
                           image from which background function was
                           determined */

   xoff = (ifast-1)/ncmp;
   yoff = (islow-1)/ncmp;
   hx = (nf_rasts/ncmp)/2;
   hy = (ns_rasts/ncmp)/2;
   i = abs(hx-xoff);
   j = abs(hy-yoff);
   dsq = i*i + j*j;
   if (dsq>1)
   {
      d = i + j>>1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
   }
   else
   {
      d = dsq;
   }
   if (d>maxr_off) d = maxr_off;
   *ival = bgave[d];
   return 0;
}    
/* Fortran binding: xdlf_bg_val */

void xdlf_bg_val (islow, ifast, ibgave, nbg, nsrasts, nfrasts, ncmp,
                      ival, ierr)
int *islow, *ifast, *ibgave, *nbg, *nsrasts, *nfrasts;
int *ncmp, *ival, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_bg_val (*islow, *ifast, ibgave, maxr_off, *nsrasts, *nfrasts,
                           *ncmp, ival);
   return;
}
/*-Routine: Get background value from background image - xdl_bg_valimg
The routine xdl_bg_valimg (xdlf_bg_valimg) is used to get a background
value at a given pixel position in the full image from a previously calculated
background image (e.g. from xdl_bg_radimg (xdlf_bg_radimg) or
xdl_bg_calc (xdlf_bg_calc)). If the pixel is outside the range of the 
background image the background value at the edge will be returned.
-end*/
/*
*****************************
**   xdl_bg_valimg         **
*****************************

Purpose: Get background value at a pixel position in the full image from
         a background image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_BG_VALIMG (ISLOW, IFAST, IMG_BG, ITYPE, NF_BGOFF,
        +                         NCMP, IVAL, IERR)
-end*/
 
/*-Parameters:
ISLOW       (R)   Pixel  position along slow axis in full image 
                  (from 1 up)
IFAST       (R)   Pixel  position along fast axis in full image 
                  (from 1 up)
IMG_BG      (R)   Background image (e.g. from xdl_bg_calc
                  or xdl_bg_radimg (see img_bg)
ITYPE       (R)   Data type for background image
                  =1 unsigned byte
                  =2 unsigned two-byte
                  =3 signed integer
                  =4 'squashed i2'
                  (see type)
NF_BGOFF    (R)   Offset between successive lines of background
                  image (allowing for padding if used)
                  (see nf_bgoff)
NCMP        (R)   No. pixels in each direction compressed into
                  1 for the background calculation.
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_bg_valimg
                  call
-end*/

/*-C:*/
int xdl_bg_valimg (islow, ifast, img_bg, type, nf_bgoff,
                    ncmp, ival)
/*end*/
 
/*-Parameters:*/
int islow;              /* The pixel position in the full image
                           along the slow axis (from 1 up) (R)*/
int ifast;              /* The pixel position in the full image
                           along the fast axis (from 1 up) (R)*/
char *img_bg;           /* Background image (R)*/
int type;               /* Background image data type 
                           =1 unsigned byte
                           =2 unsigned two-byte
                           =3 signed integer
                           =4 'squashed i2'
                           (R) */
int nf_bgoff;           /* Offset between successive lines of
                           background image (allowing for padding
                           if used) (R)*/
int ncmp;               /* No. of pixels in each direction compressed into
                           1 for the background calcn. (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */
   unsigned char *byte_bgdata; /* Pointer to unsigned byte background 
                                  image data */
   I2pix *i2_bgdata;  /* Pointer to unsigned two-byte background image data */
   I2squash *i2_bgsquash; /* Pointer to 'squashed i2' background image data */
   Intpix *int_bgdata;/* Pointer to signed integer background image data */

   byte_bgdata = (unsigned char *) img_bg;
   i2_bgdata = (I2pix *) img_bg;
   i2_bgsquash = (I2squash *) img_bg;
   int_bgdata = (Intpix *) img_bg;
   xoff = (ifast-1)/ncmp;
   yoff = (islow-1)/ncmp;
   switch (type)
   {
      case 1:
         *ival = byte_bgdata[yoff*nf_bgoff+xoff];
      break;

      case 2:
         *ival = i2_bgdata[yoff*nf_bgoff+xoff];
      break;

      case 3:
         *ival = int_bgdata[yoff*nf_bgoff+xoff];
      break;

      case 4:
         *ival = unsq(i2_bgsquash[yoff*nf_bgoff+xoff]);
      break;
   }
   return 0;
}    
/* Fortran binding: xdlf_bg_valimg */

void xdlf_bg_valimg (islow, ifast, img_bg, itype, nf_bgoff, ncmp,
                      ival, ierr)
int *islow, *ifast, *img_bg, *itype, *nf_bgoff;
int *ncmp, *ival, *ierr;
{
   *ierr = xdl_bg_valimg (*islow, *ifast, (char *) img_bg, *itype, *nf_bgoff,
                           *ncmp, ival);
   return;
}
