/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: Image Background Calculation Routines
-end*/
 
/*-Intro:
These are a set of routines for calculating a background image and
background values from a (compressed) film or image-plate (i2) single 
crystal X-ray diffraction image. The routines are written in C. Fortran 
interfaces are provided. They were written, in particular, for use with 
programs using XDL_VIEW routines, in particular xdl_film_image
(xdlf_film_image) and related routines.
-end*/
 
/*-Section: Background Calculation Routines
-end*/
 

/*Routines List:
The following background image calculating routines are available:

   xdl_film_bgcalc       Calculate background image for a film image
   xdl_film_bgstrip      Calculate radial background function (film)
   xdl_film_bgradimg     Form radially averaged background image (film)
   xdl_film_bgval        Get background value from radial function (film)
   xdl_film_bgvalimg     Get background value from background image (film)
   xdl_i2_bgcalc         Calculate background image for an 'i2' image
   xdl_i2_bgstrip        Calculate radial background function ('i2' image)
   xdl_i2_bgradimg       Form radially averaged background image ('i2')
   xdl_i2_bgval          Get background value from radial function ('i2')
   xdl_i2_bgvalimg       Get background value from background image ('i2')
*/

#include "xdl_systyp.h"
#include <math.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

/* Define routine types */
int xdl_film_bgcalc();
int xdl_film_bgstrip();
int xdl_film_bgradimg();
int xdl_film_bgval();
int xdl_film_bgvalimg();
int xdl_i2_bgcalc();
int xdl_i2_bgstrip();
int xdl_i2_bgradimg();
int xdl_i2_bgval();
int xdl_i2_bgvalimg();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_film_bgcalc xdlf_film_bgcalc_
#define xdlf_film_bgstrip xdlf_film_bgstrip_
#define xdlf_film_bgradimg xdlf_film_bgradimg_
#define xdlf_film_bgval xdlf_film_bgval_
#define xdlf_film_bgvalimg xdlf_film_bgvalimg_
#define xdlf_i2_bgcalc xdlf_i2_bgcalc_
#define xdlf_i2_bgstrip xdlf_i2_bgstrip_
#define xdlf_i2_bgradimg xdlf_i2_bgradimg_
#define xdlf_i2_bgval xdlf_i2_bgval_
#define xdlf_i2_bgvalimg xdlf_i2_bgvalimg_

#endif

#if  LINKTYP == 2

#define xdlf_film_bgcalc XDLF_FILM_BGCALC
#define xdlf_film_bgstrip XDLF_FILM_BGSTRIP
#define xdlf_film_bgradimg XDLF_FILM_BGRADIMG
#define xdlf_film_bgval XDLF_FILM_BGVAL
#define xdlf_film_bgvalimg XDLF_FILM_BGVALIMG
#define xdlf_i2_bgcalc XDLF_I2_BGCALC
#define xdlf_i2_bgstrip XDLF_I2_BGSTRIP
#define xdlf_i2_bgradimg XDLF_I2_BGRADIMG
#define xdlf_i2_bgval XDLF_I2_BGVAL
#define xdlf_i2_bgvalimg XDLF_I2_BGVALIMG

#endif

void xdlf_film_bgcalc();
void xdlf_film_bgstrip();
void xdlf_film_bgradimg();
void xdlf_film_bgval();
void xdlf_film_bgvalimg();
void xdlf_i2_bgcalc();
void xdlf_i2_bgstrip();
void xdlf_i2_bgradimg();
void xdlf_i2_bgval();
void xdlf_i2_bgvalimg();

/*-Routine: Calculate a background image for a film image - xdl_film_bgcalc
The routine xdl_film_bgcalc (xdlf_film_bgcalc) is used to calculate
a background image from a film image. This is for single crystal X-ray
diffraction images (i.e. one with spots over a fairly slowly changing
background. The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. To optimise the calculation, the
box is walked through the image moving a pixel at a time starting
at the top left. It then repeats the process of moving from to bottom, 
then one pixel to the right, then from bottom to top, and then one
pixel to the right until the film image has been covered. 
The routine will normally be used to find a 
background from a compressed image as returned by a routine such
as xdl_read_film_file (xdlf_read_film_file).
-end*/
/*
*****************************
**   xdl_film_bgcalc       **
*****************************

Purpose: Calculate a background image for a (compressed) film image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_BGCALC (IMG, IMGW, IMGH, NPBOX, IPCNT,
        +                       IMG_BG, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the (compressed) film image data 
                  (normally packed into an integer array) 
                  (see 'img')
IMGW        (R)   Image width (see 'img_width')
IMGH        (R)   Image width (see 'img_height')
NPBOX       (R)   Dimension of background calculation box in
                  pixels (preferably odd) (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background
                  (see 'percent')
IMG_BG      (W)   Array to hold the returned background image.
                  (pass an integer array large enough to hold the
                   returned image data) (see 'img_bg')
IERR        (W)   Returns the status from the xdl_film_bgcalc
                  call
-end*/

/*-C:*/
int xdl_film_bgcalc (img, img_width, img_height, npbox, percent, img_bg)
/*end*/
 
/*-Parameters:*/
unsigned char img[];       /* Film image from which to calculate the 
                              background image (R)*/
int img_width;             /* Image width (fastest index in 'img' (R)*/
int img_height;            /* Image height (slowest index in 'img' (R)*/
int npbox;                 /* Size of box for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              (R)*/
unsigned char img_bg[];    /* Returns background image in same form as input
                              image (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; =2 box too big for image
-end*/
{

   int count[256];    /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_x;         /* The number of x steps required */
   int max_y;         /* The number of y steps required */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset2;       /* Pixel offset from to to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values includen in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   enum step {down,up,right} nxt, last; /*Next and last step directions */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int ii;            /* Temp intensity value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int min_xoff;      /* Minimum x offset for which background pixel found */
   int max_xoff;      /* Maximum x offset for which background pixel found */
   int min_yoff;      /* Minimum y offset for which background pixel found */
   int max_yoff;      /* Maximum y offset for which background pixel found */

   for(i=0;i<=255;++i) count[i]=0;
   if (npbox>img_width||npbox>img_height) return 2;
   offset = npbox/2;
   nbgmin = (npbox*npbox*percent)/100;
   max_x = img_width - npbox;
   max_y = img_height - npbox;
   min_xoff = offset;
   max_xoff = img_width + min_xoff - npbox;
   min_yoff = offset;
   max_yoff = img_height + min_yoff - npbox;
   offset2 = npbox*img_width; 

   /* Full scan of image */

   /* Process initial background box */
   k=0;
   for (i=0;i<npbox;++i)
   {
      j = k;
      for (l=0;l<npbox;++l) count[img[j++]]++;
      k += img_width;
   }
   ni=0; intens_tot=0;
   for (i=0,nl=0;i<=255;++i,++nl)
   {
      ni += count[i];
      intens_tot += count[i]*i;
      if (ni>=nbgmin) break;
   }
   ipx = offset*img_width + offset;
   img_bg[ipx] = intens_tot/ni;
   ipx += img_width;

   /* Start walk through the image */
   
   ix = 0;
   iy = 1;
   ip = 0;
   jp = offset2;
   nxt=down;

   next_box:

   /* Remove intensities for top (or left or bottom) row of current box
      depending on the direction the box is being moved */

   /* Add intensities for bottom (or right or top) row of shifted box */

   if (nxt!=right)   
   {
      for (i=0;i<npbox;++i)
      {
         ii = img[ip++]; count[ii]--;
         if (ii<=nl) {intens_tot-=ii; ni--;}
         ii = img[jp++]; count[ii]++;
         if (ii<=nl) {intens_tot+=ii; ni++;}
      }
   }
   else
   {
      for (i=0;i<npbox;++i)
      {
         ii = img[ip]; count[ii]--; ip += img_width;
         if (ii<=nl) {intens_tot-=ii; ni--;}
         ii = img[jp]; count[ii]++; jp += img_width;
         if (ii<=nl) {intens_tot+=ii; ni++;}
      }
   }

   /* Move up or down 'count' array to find new background level */

   if (ni>=nbgmin)
   {
      nc = count[nl];
      while (ni-nc>=nbgmin)
      {
         intens_tot -= nc*nl;
         ni -= nc;
         nc = count[--nl];
      }
   }
   else
   {
      while (ni<nbgmin)
      {
         nc = count[++nl];
         ni += nc;
         intens_tot += nc*nl;
      }
   }

   /* Set background pixel value */

   ne = ni - nbgmin;
   img_bg[ipx] = (intens_tot-ne*nl)/nbgmin;

   /* Get direction for next step and key pixel positions */

   last = nxt;
   switch (last)
   {
      case down:
         if (iy<max_y) 
         {
            ip = iy*img_width + ix;
            jp = ip + offset2;
            ipx += img_width;
            iy++;
         }
         else
         {
            if (ix>=max_x) goto done;
            nxt = right;
            ip = iy*img_width + ix;
            jp = ip + npbox;
            ipx++;
            ix++;
         }
         goto next_box;
      
      case up:
         if (iy>0) 
         {
            iy--;
            jp = iy*img_width + ix;
            ip = jp + offset2;
            ipx -= img_width;
         }
         else
         {
            if (ix>=max_x) goto done;
            nxt = right;
            ip = ix;
            jp = ip + npbox;
            ipx++;
            ix++;
         }
         goto next_box;

      case right:
         if (iy==0)
         {
            nxt = down;
            ip = ix;
            jp = ip + offset2;
            ipx += img_width;
            iy = 1;
         }
         else
         {
            nxt = up;
            iy--;
            jp = iy*img_width + ix;
            ip = jp + offset2;
            ipx -= img_width;
         }
         goto next_box;
   }
   done:

   /* Fill in strip around edge */

   for (iy=0;iy<img_height;++iy)
   {
      ip = iy*img_width;
      for (ix=0;ix<min_xoff;++ix)
      {
         if (iy<min_yoff)
         {
            img_bg[ip] = img_bg[min_yoff*img_width+min_xoff];
         }
         else if (iy>max_yoff)
         {
            img_bg[ip] = img_bg[max_yoff*img_width+min_xoff];
         }
         else
         {
            img_bg[ip] = img_bg[iy*img_width+min_xoff];
         }
         ip++;
      }
      ip = iy*img_width + max_xoff+1;
      for (ix=max_xoff+1;ix<img_width;++ix)
      {
         if (iy<min_yoff)
         {
            img_bg[ip] = img_bg[min_yoff*img_width+max_xoff];
         }
         else if (iy>max_yoff)
         {
            img_bg[ip] = img_bg[max_yoff*img_width+max_xoff];
         }
         else
         {
            img_bg[ip] = img_bg[iy*img_width+max_xoff];
         }
         ip++;
      }
   }
   for (iy=0;iy<min_yoff;++iy)
   {
      ip = iy*img_width + min_xoff;
      jp = min_yoff*img_width + min_xoff;
      for (ix=min_xoff;ix<=max_xoff;++ix)
      {
         img_bg[ip++] = img_bg[jp++];
      }
   }
   for (iy=max_yoff+1;iy<img_height;++iy)
   {
      ip = iy*img_width + min_xoff;
      jp = max_yoff*img_width + min_xoff;
      for (ix=min_xoff;ix<=max_xoff;++ix)
      {
         img_bg[ip++] = img_bg[jp++];
      }
   }

   return 0;
}
/* Fortran binding: xdlf_film_bgcalc */

void xdlf_film_bgcalc (img, imgw, imgh, npbox, ipcnt, img_bg, ierr)
int *img, *imgw, *imgh, *npbox, *ipcnt, *img_bg, *ierr;
{
   *ierr = xdl_film_bgcalc (img, *imgw, *imgh, *npbox, *ipcnt, img_bg);
   return;
}

/*-Routine: Calculate radial background function (film) - xdl_film_bgstrip
The routine xdl_film_bgstrip (xdlf_film_bgstrip) is used to calculate
a radially averaged background from a (compressed) film image. 
The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. The background values are found for
a horizontal or vertical strip of pixels running through the centre of
the image and the values from the two halves of the strip are averaged.
The routine will normally be used to find a 
background from a compressed image as returned by a routine such
as xdl_read_film_file (xdlf_read_film_file).
-end*/
/*
*****************************
**   xdl_film_bgstrip      **
*****************************

Purpose: Calculate a radially averaged background from a (compressed) 
         film image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_BGSTRIP (IMG, IMGW, IMGH, NPBOX, IPCNT,
        +                       IOPT, IBGAVE, NBG, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the (compressed) film image data 
                  (normally packed into an integer array) 
                  (see 'img')
IMGW        (R)   Image width (see 'img_width')
IMGH        (R)   Image width (see 'img_height')
NPBOX       (R)   Dimension of background calculation box in
                  pixels (preferably odd) (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background
                  (see 'percent')
IOPT        (R)   =1 Horizontal strip across the centre
                  =2 Vertical strip through the centre
IBGAVE      (W)   Array returning radially averaged background
                  values (see bgave)
NBG         (W)   The number of values returned in IBGAVE
                  (maxr_off+1, see maxr_off)
IERR        (W)   Returns the status from the xdl_film_bgstrip
                  call
-end*/

/*-C:*/
int xdl_film_bgstrip (img, img_width, img_height, npbox, percent, iopt,
                      bgave, maxr_off)
/*end*/
 
/*-Parameters:*/
unsigned char img[];       /* Compressed image from which to calculate the 
                              background strip (R)*/
int img_width;             /* Image width (fastest index in 'img' (R)*/
int img_height;            /* Image height (slowest index in 'img' (R)*/
int npbox;                 /* Size of box for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              (R)*/
int iopt;                  /* =1, horizontal strip (along x)
                              =2, vertical strip (along y) (R)*/
int bgave[];               /* Returns radially averaged background function
                              in intervals of pixels of the compressed
                              image and starting from the centre of that
                              image (W)*/
int *maxr_off;             /* Maximum offset in returned 'bgave' array (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to allocate memory for temporary arrays;
         =2 box too big for image
-end*/
{

   int count[256];    /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_x;         /* The number of x steps required */
   int max_y;         /* The number of y steps required */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset2;       /* Pixel offset from to to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values includen in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int ii;            /* Temp intensity value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */
   unsigned char *bg_val; /* Array of background pixel values in line across
                             centre of the image */

   for(i=0;i<=255;++i) count[i]=0;
   if (npbox>img_width||npbox>img_height) return 2;
   offset = npbox/2;
   nbgmin = (npbox*npbox*percent)/100;
   max_x = img_width - npbox;
   max_y = img_height - npbox;
   offset2 = npbox*img_width; 

   switch (iopt)
   {
      case 1:

         /* Horizontal strip across centre of film */

         *maxr_off = (img_width-npbox)/2;
         bg_val = (unsigned char *) malloc (img_width);
         if (bg_val==0) return 1;
         /* Process initial background box */
         k=(img_height/2-npbox/2)*img_width;

         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) count[img[j++]]++;
            k += img_width;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<=255;++i,++nl)
         {
            ni += count[i];
            intens_tot += count[i]*i;
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
   
         ix = 1;
         iy = (img_height/2-npbox/2);
         ip = iy*img_width;
         jp = ip + npbox;

         next_box_opt1:

         /* Remove intensities for left column of current box */

         /* Add intensities for right column of shifted box */

         for (i=0;i<npbox;++i)
         {
            ii = img[ip]; count[ii]--; ip += img_width;
            if (ii<=nl) {intens_tot-=ii; ni--;}
            ii = img[jp]; count[ii]++; jp += img_width;
            if (ii<=nl) {intens_tot+=ii; ni++;}
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               intens_tot -= nc*nl;
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               intens_tot += nc*nl;
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;

        /* Get key pixel positions for next step */

        if (ix>=max_x) 
        {
           hx = img_width/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hx-i]+bg_val[hx+i])/2;
           free (bg_val);
           return 0;
        }
        ip = iy*img_width + ix;
        jp = ip + npbox;
        ipx++;
        ix++;
        goto next_box_opt1;

      case 2:

         /* Vertical strip across centre of film */

         *maxr_off = (img_height-npbox)/2;
         bg_val = (unsigned char *) malloc (img_height);
         if (bg_val==0) return 1;
         /* Process initial background box */
         k=(img_width/2-npbox/2);
         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) count[img[j++]]++;
            k += img_width;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<=255;++i,++nl)
         {
            ni += count[i];
            intens_tot += count[i]*i;
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
   
         ix = (img_width/2-npbox/2);
         iy = 1;
         ip = ix;
         jp = ip + offset2;

         next_box_opt2:

         /* Remove intensities for top row of current box */

         /* Add intensities for bottom row of shifted box */

         for (i=0;i<npbox;++i)
         {
            ii = img[ip++]; count[ii]--;
            if (ii<=nl) {intens_tot-=ii; ni--;}
            ii = img[jp++]; count[ii]++;
            if (ii<=nl) {intens_tot+=ii; ni++;}
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               intens_tot -= nc*nl;
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               intens_tot += nc*nl;
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;

        /* Get key pixel positions for next step */

        if (iy>=max_y) 
        {
           hy = img_height/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hy-i]+bg_val[hy+i])/2;
           free (bg_val);
           return 0;
        }
        ip = iy*img_width + ix;
        jp = ip + offset2;
        ipx++;
        iy++;
        goto next_box_opt2;
   }
}
/* Fortran binding: xdlf_film_bgstrip */

void xdlf_film_bgstrip (img, imgw, imgh, npbox, ipcnt, iopt, ibgave, nbg,
                        ierr)
int *img, *imgw, *imgh, *npbox, *ipcnt, *iopt, *ibgave, *nbg, *ierr;
{
   int maxr_off;

   *ierr = xdl_film_bgstrip (img, *imgw, *imgh, *npbox, *ipcnt, *iopt,
                            ibgave, &maxr_off);
   *nbg = maxr_off;
   return;
}
/*-Routine: Form radially averaged background image (film) - xdl_film_bgradimg
The routine xdl_film_bgradimg (xdlf_film_bgradimg) is used to form a
a radially averaged background image from the radially averaged
background function returned by xdl_film_bgstrip (xdlf_film_bgstrip).
-end*/
/*
*****************************
**   xdl_film_bgradimg     **
*****************************

Purpose: Form radially averaged background image from radial background 
         function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_BGRADIMG (IMGW, IMGH, IBGAVE, NBG, IMG_BG, IERR)
-end*/
 
/*-Parameters:
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (W)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
IMG_BG      (W)   Integer array returning background image
                  (must be large enough to hold 
                  img_width*img_height bytes) (see img_bg)
IERR        (W)   Returns the status from the xdl_film_bgradimg
                  call
-end*/

/*-C:*/
int xdl_film_bgradimg (img_width, img_height, bgave, maxr_off, img_bg)
/*end*/
 
/*-Parameters:*/
int img_width;          /* Width of the compressed image and background 
                           image (R)*/
int img_height;         /* Height of the compressed image and background 
                           image (R)*/
int bgave[];            /* Radially average background values from
                           xdl_film_bgstrip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_film_bgstrip 
                           (R)*/
unsigned char img_bg[]; /* Returns the compressed radially averaged 
                           background image (img_width*img_height) (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;           /* Temp/loop variables */
   int ip;            /* Index of pixel */
   int jp;            /* Index of pixel */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */

   hx = img_width/2;
   hy = img_height/2;
   max_off = hx; if (hy>hx) max_off = hy;
   /* Form background image */
   for (i=0;i<=max_off;++i)
   {
      for (j=0;j<=i;++j)
      {
         dsq = i*i + j*j;
         if (dsq>1)
         {
            /*Get approx square root*/
            d = i+j>>1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
         }
         else
         {
            d=dsq;
         }
         if (d>maxr_off) d = maxr_off;
         x = hx - j; 
         if (x>=0)
         {
            y = hy - i;
            if (y>=0) img_bg[y*img_width+x]=bgave[d];
         }
         x = hx - i; 
         if (x>=0)
         {
            y = hy - j;
            if (y>=0) img_bg[y*img_width+x]=bgave[d];
         }
      }  
   }           
   /* Form top right quarter */
   for (iy=0;iy<=hy;++iy)
   {
      ip = (iy*img_width + hx) + 1;
      jp = ip - 2;
      for (ix=hx+1;ix<img_width;++ix) img_bg[ip++] = img_bg[jp--];
   }
   /* Form lower half */
   for (iy=hy+1,ky=hy-1;iy<img_height;++iy,--ky)
   {
      ip = iy*img_width;
      jp = ky*img_width;
      for (ix=0;ix<img_width;++ix) img_bg[ip++] = img_bg[jp++];
   }
   return 0;
}
/* Fortran binding: xdlf_film_bgradimg */

void xdlf_film_bgradimg (imgw, imgh, ibgave, nbg, img_bg, ierr)
int *imgw, *imgh, *ibgave, *nbg, *img_bg, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_film_bgradimg (*imgw, *imgh, ibgave, maxr_off, img_bg);
   return;
}

/*-Routine: Get background value from background function - xdl_film_bgval
The routine xdl_film_bgval (xdlf_film_bgval) is used to get a background
value at a given pixel position in the full image from a radially averaged 
background function as returned by xdl_film_bgstrip (xdlf_film_bgstrip).
If the pixel is outside the range of the background function the value
at the maximum available radius will be returned.
-end*/
/*
*****************************
**   xdl_film_bgval        **
*****************************

Purpose: Get background value at a pixel position in the full image from
         radially averaged background function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_BGVAL (IX, IY, IBGAVE, NBG, IMGW, IMGH, 
                               IXOPIX, IYOPIX, NXP, NYP, IVAL, IERR)
-end*/
 
/*-Parameters:
IX          (R)   Pixel x position in full image (from 1 up)
IY          (R)   Pixel y position in full image (from 1 up)
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (W)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IXOPIX      (R)   Origin x pixel of background image wrt full
                  image (from 1 up) (see xo_pix)
IYOPIX      (R)   Origin y pixel of background image wrt full
                  image (from 1 up) (see yo_pix)
NXP         (R)   No. of x pixels from full image compressed
                  into 1 in background image (see nxp)
NYP         (R)   No. of y pixels from full image compressed
                  into 1 in background image (see nyp)
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_film_bgval
                  call
-end*/

/*-C:*/
int xdl_film_bgval (ix, iy, bgave, maxr_off, img_width, img_height,
                    xo_pix, yo_pix, nxp, nyp, ival)
/*end*/
 
/*-Parameters:*/
int ix;                 /* The x pixel position in the full image
                           (from 1 up) (R)*/
int iy;                 /* The y pixel position in the full image
                           (from 1 up) (R)*/
int bgave[];            /* Radially average background values from
                           xdl_film_bgstrip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_film_bgstrip 
                           (R)*/
int img_width;          /* Width of the background image used to calculate 
                           background function (R)*/
int img_height;         /* Height of the background image used to calculate 
                           background function (R)*/
int xo_pix;             /* Origin x pixel of background image wrt full
                           image (from 1 up) (R)*/
int yo_pix;             /* Origin y pixel of background image wrt full
                           image (from 1 up) (R)*/
int nxp;                /* No. of x pixels from full image compressed
                           into 1 in background image (R)*/
int nyp;                /* No. of y pixels from full image compressed
                           into 1 in background image (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;             /* Offsets from centre of compressed background 
                           image from which background function was 
                           determined */
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */
   int d, dsq;          /* Distance and d**2 from centre */
   int hx, hy;          /* Offsets of central pixel in compressed background
                           image from which background function was
                           determined */

   xoff = (ix-xo_pix)/nxp;
   yoff = nyp*img_height + yo_pix - 1 - iy;
   yoff /= nyp;
   hx = img_width/2;
   hy = img_height/2;
   i = abs(hx-xoff);
   j = abs(hy-yoff);
   dsq = i*i + j*j;
   if (dsq>1)
   {
      d = i + j>>1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
   }
   else
   {
      d = dsq;
   }
   if (d>maxr_off) d = maxr_off;
   *ival = bgave[d];
   return 0;
}    
/* Fortran binding: xdlf_film_bgval */

void xdlf_film_bgval (ix, iy, ibgave, nbg, imgw, imgh, ixopix, iyopix,
                      nxp, nyp, ival, ierr)
int *ix, *iy, *ibgave, *nbg, *imgw, *imgh, *ixopix, *iyopix;
int *nxp, *nyp, *ival, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_film_bgval (*ix, *iy, ibgave, maxr_off, *imgw, *imgh, 
                           *ixopix, *iyopix, *nxp, *nyp, ival);
   return;
}
/*-Routine: Get background value from background image (film) - xdl_film_bgvalimg
The routine xdl_film_bgvalimg (xdlf_film_bgvalimg) is used to get a background
value at a given pixel position in the full image from a previously calculated
background image (e.g. from xdl_film_bgradimg (xdlf_film_bgradimg) or
xdl_film_bgcalc (xdlf_film_bgcalc)). If the pixel is outside the range of the 
background image the background value at the edge will be returned.
-end*/
/*
*****************************
**   xdl_film_bgvalimg     **
*****************************

Purpose: Get background value at a pixel position in the full image from
         a background image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_BGVALIMG (IX, IY, IMG_BG, IMGW, IMGH, 
        +                         IXOPIX, IYOPIX, NXP, NYP, IVAL, IERR)
-end*/
 
/*-Parameters:
IX          (R)   Pixel x position in full image (from 1 up)
IY          (R)   Pixel y position in full image (from 1 up)
IMG_BG      (R)   Background image (e.g. from xdl_film_bgcalc
                  or xdl_film_bgradimg (see img_bg)
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IXOPIX      (R)   Origin x pixel of background image wrt full
                  image (from 1 up) (see xo_pix)
IYOPIX      (R)   Origin y pixel of background image wrt full
                  image (from 1 up) (see yo_pix)
NXP         (R)   No. of x pixels from full image compressed
                  into 1 in background image (see nxp)
NYP         (R)   No. of y pixels from full image compressed
                  into 1 in background image (see nyp)
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_film_bgvalimg
                  call
-end*/

/*-C:*/
int xdl_film_bgvalimg (ix, iy, img_bg, img_width, img_height,
                    xo_pix, yo_pix, nxp, nyp, ival)
/*end*/
 
/*-Parameters:*/
int ix;                 /* The x pixel position in the full image
                           (from 1 up) (R)*/
int iy;                 /* The y pixel position in the full image
                           (from 1 up) (R)*/
unsigned char img_bg[]; /* Background image (R)*/
int img_width;          /* Width of the background image (R)*/
int img_height;         /* Height of the background image (R)*/
int xo_pix;             /* Origin x pixel of background image wrt full
                           image (from 1 up) (R)*/
int yo_pix;             /* Origin y pixel of background image wrt full
                           image (from 1 up) (R)*/
int nxp;                /* No. of x pixels from full image compressed
                           into 1 in background image (R)*/
int nyp;                /* No. of y pixels from full image compressed
                           into 1 in background image (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */

   xoff = (ix-xo_pix)/nxp;
   yoff = nyp*img_height + yo_pix - 1 - iy;
   yoff /= nyp;
   *ival = img_bg[yoff*img_width+xoff];
   return 0;
}    
/* Fortran binding: xdlf_film_bgvalimg */

void xdlf_film_bgvalimg (ix, iy, img_bg, imgw, imgh, ixopix, iyopix,
                      nxp, nyp, ival, ierr)
int *ix, *iy, *img_bg, *imgw, *imgh, *ixopix, *iyopix;
int *nxp, *nyp, *ival, *ierr;
{
   *ierr = xdl_film_bgvalimg (*ix, *iy, img_bg, *imgw, *imgh, 
                           *ixopix, *iyopix, *nxp, *nyp, ival);
   return;
}
/*-Routine: Calculate background image for an 'i2' image - xdl_i2_bgcalc
The routine xdl_i2_bgcalc (xdlf_i2_bgcalc) is used to calculate
a background image from an image-plate 'i2' image. This is for single 
crystal X-ray diffraction images (i.e. one with spots over a fairly slowly 
changing backgroundz0. The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. To optimise the calculation, the
box is walked through the image moving a pixel at a time starting
at the top left. It then repeats the process of moving from to bottom, 
then one pixel to the right, then from bottom to top, and then one
pixel to the right until the film image has been covered. 
The routine will normally be used to find a 
background from a compressed image as returned by a routine such
as xdl_read_i2 (xdlf_read_i2).
-end*/
/*
*****************************
**   xdl_i2_bgcalc         **
*****************************

Purpose: Calculate a background image for a (compressed) image-plate
         'i2' image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_I2_BGCALC (IMG, IMGW, IMGH, NPBOX, IPCNT,
        +                       IMG_BG, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the (compressed) 'i2' image data 
                  (normally packed into an integer array) 
                  (see 'img')
IMGW        (R)   Image width (see 'img_width')
IMGH        (R)   Image width (see 'img_height')
NPBOX       (R)   Dimension of background calculation box in
                  pixels (preferably odd) (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background
                  (see 'percent')
IMG_BG      (W)   Array to hold the returned background image.
                  (pass an integer array large enough to hold the
                   returned image data) (see 'img_bg')
IERR        (W)   Returns the status from the xdl_i2_bgcalc
                  call
-end*/

/*-C:*/
int xdl_i2_bgcalc (img, img_width, img_height, npbox, percent, img_bg)
/*end*/
 
/*-Parameters:*/
unsigned short img[];      /* Image-plate 'i2' image from which to 
                              calculate the background image (R)*/
int img_width;             /* Image width (fastest index in 'img' (R)*/
int img_height;            /* Image height (slowest index in 'img' (R)*/
int npbox;                 /* Size of box for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              (R)*/
unsigned short img_bg[];   /* Returns background image in same form as 
                              input image (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; =2 box too big for image
-end*/
{

   int count[2048];   /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_x;         /* The number of x steps required */
   int max_y;         /* The number of y steps required */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset2;       /* Pixel offset from to to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values includen in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   enum step {down,up,right} nxt, last; /*Next and last step directions */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int ii;            /* Temp intensity value */
   int jj;            /* Offset into count array for intensities>1023 */
   int iii;           /* Temp offset value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int min_xoff;      /* Minimum x offset for which background pixel found */
   int max_xoff;      /* Maximum x offset for which background pixel found */
   int min_yoff;      /* Minimum y offset for which background pixel found */
   int max_yoff;      /* Maximum y offset for which background pixel found */

   for(i=0;i<2048;++i) count[i]=0;
   if (npbox>img_width||npbox>img_height) return 2;
   offset = npbox/2;
   nbgmin = (npbox*npbox*percent)/100;
   max_x = img_width - npbox;
   max_y = img_height - npbox;
   min_xoff = offset;
   max_xoff = img_width + min_xoff - npbox;
   min_yoff = offset;
   max_yoff = img_height + min_yoff - npbox;
   offset2 = npbox*img_width; 

   /* Full scan of image */

   /* Process initial background box */
   k=0;
   for (i=0;i<npbox;++i)
   {
      j = k;
      for (l=0;l<npbox;++l) 
      {
         ii = img[j++];
         if (ii>1023)
         {
            jj = 1024 + ((ii-992)>>6);
            count[jj]++;
         }
         else
         {
            count[ii]++;
         }
      }
      k += img_width;
   }
   ni=0; intens_tot=0;
   for (i=0,nl=0;i<2048;++i,++nl)
   {
      ni += count[i];
      if (i>1023)
      {
         intens_tot += count[i]*(1024+((i-1024)<<6));
      }
      else
      {
         intens_tot += count[i]*i;
      }
      if (ni>=nbgmin) break;
   }
   ipx = offset*img_width + offset;
   img_bg[ipx] = intens_tot/ni;
   ipx += img_width;

   /* Start walk through the image */
   
   ix = 0;
   iy = 1;
   ip = 0;
   jp = offset2;
   nxt=down;

   next_box:

   /* Remove intensities for top (or left or bottom) row of current box
      depending on the direction the box is being moved */

   /* Add intensities for bottom (or right or top) row of shifted box */

   if (nxt!=right)   
   {
      for (i=0;i<npbox;++i)
      {
         ii = img[ip++]; 
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            count[jj]--;
            if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
         }
         else
         {
            count[ii]--;
            if (ii<=nl) {intens_tot-=ii; ni--;}
         }
         ii = img[jp++]; 
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            count[jj]++;
            if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
         }
         else
         {
            count[ii]++;
            if (ii<=nl) {intens_tot+=ii; ni++;}
         }
      }
   }
   else
   {
      for (i=0;i<npbox;++i)
      {
         ii = img[ip]; ip += img_width;
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            count[jj]--;
            if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
         }
         else
         {
            count[ii]--;
            if (ii<=nl) {intens_tot-=ii; ni--;}
         }
         ii = img[jp]; jp += img_width;
         if (ii>1023)
         {
            iii = (ii-992)>>6;
            jj = 1024+iii;
            count[jj]++;
            if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
         }
         else
         {
            count[ii]++;
            if (ii<=nl) {intens_tot+=ii; ni++;}
         }
      }
   }

   /* Move up or down 'count' array to find new background level */

   if (ni>=nbgmin)
   {
      nc = count[nl];
      while (ni-nc>=nbgmin)
      {
         if (nl>1023)
         {
            intens_tot -= nc*(1024+((nl-1024)<<6));
         }
         else
         {
            intens_tot -= nc*nl;
         }
         ni -= nc;
         nc = count[--nl];
      }
   }
   else
   {
      while (ni<nbgmin)
      {
         nc = count[++nl];
         ni += nc;
         if (nl>1023)
         {
            intens_tot += nc*(1024+((nl-1024)<<6));
         }
         else
         {
            intens_tot += nc*nl;
         }
      }
   }

   /* Set background pixel value */

   ne = ni - nbgmin;
   if (nl>1023)
   {
      img_bg[ipx] = (intens_tot-ne*(1024+((nl-1024)<<6)))/nbgmin;
   }
   else
   {
      img_bg[ipx] = (intens_tot-ne*nl)/nbgmin;
   }

   /* Get direction for next step and key pixel positions */

   last = nxt;
   switch (last)
   {
      case down:
         if (iy<max_y) 
         {
            ip = iy*img_width + ix;
            jp = ip + offset2;
            ipx += img_width;
            iy++;
         }
         else
         {
            if (ix>=max_x) goto done;
            nxt = right;
            ip = iy*img_width + ix;
            jp = ip + npbox;
            ipx++;
            ix++;
         }
         goto next_box;
      
      case up:
         if (iy>0) 
         {
            iy--;
            jp = iy*img_width + ix;
            ip = jp + offset2;
            ipx -= img_width;
         }
         else
         {
            if (ix>=max_x) goto done;
            nxt = right;
            ip = ix;
            jp = ip + npbox;
            ipx++;
            ix++;
         }
         goto next_box;

      case right:
         if (iy==0)
         {
            nxt = down;
            ip = ix;
            jp = ip + offset2;
            ipx += img_width;
            iy = 1;
         }
         else
         {
            nxt = up;
            iy--;
            jp = iy*img_width + ix;
            ip = jp + offset2;
            ipx -= img_width;
         }
         goto next_box;
   }
   done:

   /* Fill in strip around edge */

   for (iy=0;iy<img_height;++iy)
   {
      ip = iy*img_width;
      for (ix=0;ix<min_xoff;++ix)
      {
         if (iy<min_yoff)
         {
            img_bg[ip] = img_bg[min_yoff*img_width+min_xoff];
         }
         else if (iy>max_yoff)
         {
            img_bg[ip] = img_bg[max_yoff*img_width+min_xoff];
         }
         else
         {
            img_bg[ip] = img_bg[iy*img_width+min_xoff];
         }
         ip++;
      }
      ip = iy*img_width + max_xoff+1;
      for (ix=max_xoff+1;ix<img_width;++ix)
      {
         if (iy<min_yoff)
         {
            img_bg[ip] = img_bg[min_yoff*img_width+max_xoff];
         }
         else if (iy>max_yoff)
         {
            img_bg[ip] = img_bg[max_yoff*img_width+max_xoff];
         }
         else
         {
            img_bg[ip] = img_bg[iy*img_width+max_xoff];
         }
         ip++;
      }
   }
   for (iy=0;iy<min_yoff;++iy)
   {
      ip = iy*img_width + min_xoff;
      jp = min_yoff*img_width + min_xoff;
      for (ix=min_xoff;ix<=max_xoff;++ix)
      {
         img_bg[ip++] = img_bg[jp++];
      }
   }
   for (iy=max_yoff+1;iy<img_height;++iy)
   {
      ip = iy*img_width + min_xoff;
      jp = max_yoff*img_width + min_xoff;
      for (ix=min_xoff;ix<=max_xoff;++ix)
      {
         img_bg[ip++] = img_bg[jp++];
      }
   }

   return 0;
}
/* Fortran binding: xdlf_i2_bgcalc */

void xdlf_i2_bgcalc (img, imgw, imgh, npbox, ipcnt, img_bg, ierr)
int *img, *imgw, *imgh, *npbox, *ipcnt, *img_bg, *ierr;
{
   *ierr = xdl_i2_bgcalc (img, *imgw, *imgh, *npbox, *ipcnt, img_bg);
   return;
}

/*-Routine: Calculate radially averaged background function 'i2' - xdl_i2_bgstrip
The routine xdl_i2_bgstrip (xdlf_i2_bgstrip) is used to calculate
a radially averaged background from a (compressed) image-plate 'i2' image. 
The method calculates the background at a pixel by taking
the average of the lowest pixel values in a box surrounding the pixel 
such that the number of such values is a requested percentage of the
total number of pixels in the box. The background values are found for
a horizontal or vertical strip of pixels running through the centre of
the image and the values from the two halves of the strip are averaged.
The routine will normally be used to find a 
background from a compressed image as returned by a routine such
as xdl_read_i2 (xdlf_read_i2).
-end*/
/*
*****************************
**   xdl_i2_bgstrip        **
*****************************

Purpose: Calculate a radially averaged background from a (compressed) 
         image-plate image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_I2_BGSTRIP (IMG, IMGW, IMGH, NPBOX, IPCNT,
        +                       IOPT, IBGAVE, NBG, IERR)
-end*/
 
/*-Parameters:
IMG         (R)   Array holding the (compressed) image-plate image 
                  data .
                  (normally packed into an integer array) 
                  (see 'img')
IMGW        (R)   Image width (see 'img_width')
IMGH        (R)   Image width (see 'img_height')
NPBOX       (R)   Dimension of background calculation box in
                  pixels (preferably odd) (see 'npbox')
IPCNT       (R)   Percentage of pixels to treat as background
                  (see 'percent')
IOPT        (R)   =1 Horizontal strip across the centre
                  =2 Vertical strip through the centre
IBGAVE      (W)   Array returning radially averaged background
                  values (see bgave)
NBG         (W)   The number of values returned in IBGAVE
                  (maxr_off+1, see maxr_off)
IERR        (W)   Returns the status from the xdl_i2_bgstrip
                  call
-end*/

/*-C:*/
int xdl_i2_bgstrip (img, img_width, img_height, npbox, percent, iopt,
                      bgave, maxr_off)
/*end*/
 
/*-Parameters:*/
unsigned short img[];      /* Compressed image from which to calculate the 
                              background strip (R)*/
int img_width;             /* Image width (fastest index in 'img' (R)*/
int img_height;            /* Image height (slowest index in 'img' (R)*/
int npbox;                 /* Size of box for each background calculation
                              in pixels (box is npbox*npbox) - preferably
                              an odd number (R)*/
int percent;               /* Percentage of pixels to treat as background 
                              (R)*/
int iopt;                  /* =1, horizontal strip (along x)
                              =2, vertical strip (along y) (R)*/
int bgave[];               /* Returns radially averaged background function
                              in intervals of pixels of the compressed
                              image and starting from the centre of that
                              image (W)*/
int *maxr_off;             /* Maximum offset in returned 'bgave' array (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to allocate memory for temporary arrays;
         =2 box too big for image
-end*/
{

   int count[2048];   /* Array to hold intensity counts */
   int i,j,k,l;       /* Temp/loop variables */
   int offset;        /* Offset to center of background box from its origin */
   int nbgmin;        /* The minimum number of background pixels required */
   int max_x;         /* The number of x steps required */
   int max_y;         /* The number of y steps required */
   int ip;            /* Index of pixel being removed in current step */
   int jp;            /* Index of pixel being added in current step */
   int ipx;           /* Index of current pixel for which background is being
                         found */
   int offset2;       /* Pixel offset from to to bottom of box */
   int intens_tot;    /* Total intensity up to maximum level used in
                         background calculation */
   int ni;            /* No. of intensity values includen in intens_tot */
   int nl;            /* Max intensity currently used in background
                         calculation (index into 'count') */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int ii;            /* Temp intensity value */
   int jj;            /* Offset into count array for intensities>1023 */
   int iii;           /* Temp offset value */
   int nc;            /* Count of intensities in top level bin used for
                         background */
   int ne;            /* No. of extra intensities in intens_tot over those
                         required */
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */
   unsigned short *bg_val; /* Array of background pixel values in line across
                              centre of the image */

   for(i=0;i<2048;++i) count[i]=0;
   if (npbox>img_width||npbox>img_height) return 2;
   offset = npbox/2;
   nbgmin = (npbox*npbox*percent)/100;
   max_x = img_width - npbox;
   max_y = img_height - npbox;
   offset2 = npbox*img_width; 

   switch (iopt)
   {
      case 1:

         /* Horizontal strip across centre of image */

         *maxr_off = (img_width-npbox)/2;
         bg_val = (unsigned short *) malloc (img_width*sizeof(unsigned short));
         if (bg_val==0) return 1;
         /* Process initial background box */
         k=(img_height/2-npbox/2)*img_width;

         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) 
            {
               ii = img[j++];
               if (ii>1023)
               {
                  jj = 1024 + ((ii-992)>>6);
                  count[jj]++;
               }
               else
               {
                  count[ii]++;
               }
            }
            k += img_width;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<2048;++i,++nl)
         {
            ni += count[i];
            if (i>1023)
            {
               intens_tot += count[i]*(1024+((i-1024)<<6));
            }
            else
            {
               intens_tot += count[i]*i;
            }
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
   
         ix = 1;
         iy = (img_height/2-npbox/2);
         ip = iy*img_width;
         jp = ip + npbox;

         next_box_opt1:

         /* Remove intensities for left column of current box */

         /* Add intensities for right column of shifted box */

         for (i=0;i<npbox;++i)
         {
            ii = img[ip]; ip += img_width;
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               count[jj]--;
               if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
            }
            else
            {
               count[ii]--;
               if (ii<=nl) {intens_tot-=ii; ni--;}
            }
            ii = img[jp]; jp += img_width;
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               count[jj]++;
               if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
            }
            else
            {
               count[ii]++;
               if (ii<=nl) {intens_tot+=ii; ni++;}
            }
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               if (nl>1023)
               {
                  intens_tot -= nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot -= nc*nl;
               }
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               if (nl>1023)
               {
                  intens_tot += nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot += nc*nl;
               }
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        if (nl>1023)
        {
           bg_val[ipx] = (intens_tot-ne*(1024+((nl-1024)<<6)))/nbgmin;
        }
        else
        {
            bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;
        }

        /* Get key pixel positions for next step */

        if (ix>=max_x) 
        {
           hx = img_width/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hx-i]+bg_val[hx+i])/2;
           free (bg_val);
           return 0;
        }
        ip = iy*img_width + ix;
        jp = ip + npbox;
        ipx++;
        ix++;
        goto next_box_opt1;

      case 2:

         /* Vertical strip across centre of film */

         *maxr_off = (img_height-npbox)/2;
         bg_val = (unsigned short *) malloc (img_height*sizeof(unsigned short));
         if (bg_val==0) return 1;
         /* Process initial background box */
         k=(img_width/2-npbox/2);
         for (i=0;i<npbox;++i)
         {
            j = k;
            for (l=0;l<npbox;++l) 
            {
               ii = img[j++];
               if (ii>1023)
               {
                  jj = 1024 + ((ii-992)>>6);
                  count[jj]++;
               }
               else
               {
                  count[ii]++;
               }
            }
            k += img_width;
         }
         ni=0; intens_tot=0;
         for (i=0,nl=0;i<2048;++i,++nl)
         {
            ni += count[i];
            if (i>1023)
            {
               intens_tot += count[i]*(1024+((i-1024)<<6));
            }
            else
            {
               intens_tot += count[i]*i;
            }
            if (ni>=nbgmin) break;
         }
         ipx = offset;
         bg_val[ipx] = intens_tot/ni;
         ipx++;

         /* Start strip through the image */
   
         ix = (img_width/2-npbox/2);
         iy = 1;
         ip = ix;
         jp = ip + offset2;

         next_box_opt2:

         /* Remove intensities for top row of current box */

         /* Add intensities for bottom row of shifted box */

         for (i=0;i<npbox;++i)
         {
            ii = img[ip++]; 
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               count[jj]--;
               if (jj<=nl){intens_tot-=(1024+(iii<<6)); ni--;}
            }
            else
            {
               count[ii]--;
               if (ii<=nl) {intens_tot-=ii; ni--;}
            }
            ii = img[jp++]; 
            if (ii>1023)
            {
               iii = (ii-992)>>6;
               jj = 1024+iii;
               count[jj]++;
               if (jj<=nl){intens_tot+=(1024+(iii<<6)); ni++;}
            }
            else
            {
               count[ii]++;
               if (ii<=nl) {intens_tot+=ii; ni++;}
            }
         }

         /* Move up or down 'count' array to find new background level */

         if (ni>=nbgmin)
         {
            nc = count[nl];
            while (ni-nc>=nbgmin)
            {
               if (nl>1023)
               {
                  intens_tot -= nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot -= nc*nl;
               }
               ni -= nc;
               nc = count[--nl];
            }
         }
         else
         {
            while (ni<nbgmin)
            {
               nc = count[++nl];
               ni += nc;
               if (nl>1023)
               {
                  intens_tot += nc*(1024+((nl-1024)<<6));
               }
               else
               {
                  intens_tot += nc*nl;
               }
            }
         }

        /* Set background pixel value */

        ne = ni - nbgmin;
        if (nl>1023)
        {
           bg_val[ipx] = (intens_tot-ne*(1024+((nl-1024)<<6)))/nbgmin;
        }
        else
        {
            bg_val[ipx] = (intens_tot-ne*nl)/nbgmin;
        }

        /* Get key pixel positions for next step */

        if (iy>=max_y) 
        {
           hy = img_height/2;
           for (i=0;i<=*maxr_off;++i) bgave[i]=(bg_val[hy-i]+bg_val[hy+i])/2;
           free (bg_val);
           return 0;
        }
        ip = iy*img_width + ix;
        jp = ip + offset2;
        ipx++;
        iy++;
        goto next_box_opt2;
   }
}
/* Fortran binding: xdlf_i2_bgstrip */

void xdlf_i2_bgstrip (img, imgw, imgh, npbox, ipcnt, iopt, ibgave, nbg,
                        ierr)
int *img, *imgw, *imgh, *npbox, *ipcnt, *iopt, *ibgave, *nbg, *ierr;
{
   int maxr_off;

   *ierr = xdl_i2_bgstrip (img, *imgw, *imgh, *npbox, *ipcnt, *iopt,
                            ibgave, &maxr_off);
   *nbg = maxr_off;
   return;
}
/*-Routine: Form radially averaged background image 'i2' - xdl_i2_bgradimg
The routine xdl_i2_bgradimg (xdlf_i2_bgradimg) is used to form a
a radially averaged background 'i2' image from the radially averaged
background function returned by xdl_i2_bgstrip (xdlf_i2_bgstrip).
-end*/
/*
*****************************
**   xdl_i2_bgradimg       **
*****************************

Purpose: Form radially averaged background image from radial background 
         function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_I2_BGRADIMG (IMGW, IMGH, IBGAVE, NBG, IMG_BG, IERR)
-end*/
 
/*-Parameters:
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (W)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
IMG_BG      (W)   Integer array returning background image
                  (must be large enough to hold 
                  img_width*img_height 'i2' short integers) 
                  (see img_bg)
IERR        (W)   Returns the status from the xdl_i2_bgradimg
                  call
-end*/

/*-C:*/
int xdl_i2_bgradimg (img_width, img_height, bgave, maxr_off, img_bg)
/*end*/
 
/*-Parameters:*/
int img_width;          /* Width of the compressed image and background 
                           image (R)*/
int img_height;         /* Height of the compressed image and background 
                           image (R)*/
int bgave[];            /* Radially average background values from
                           xdl_i2_bgstrip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_i2_bgstrip 
                           (R)*/
unsigned short img_bg[]; /* Returns the compressed radially averaged 'i2'
                            background image (img_width*img_height) (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;           /* Temp/loop variables */
   int ip;            /* Index of pixel */
   int jp;            /* Index of pixel */
   int ix;            /* Step counter in x */
   int iy,ky;         /* Step counters in y */
   int hx, hy;        /* Mid point pixel of image (offsets) */
   int d;             /* Distance of current pixel from mid-point pixel */
   int dsq;           /* d**2 */
   int x, y;          /* Current pixel position (offsets) */
   int max_off;       /* Maximum x or y offset from centre of image to edge */

   hx = img_width/2;
   hy = img_height/2;
   max_off = hx; if (hy>hx) max_off = hy;
   /* Form background image */
   for (i=0;i<=max_off;++i)
   {
      for (j=0;j<=i;++j)
      {
         dsq = i*i + j*j;
         if (dsq>1)
         {
            /*Get approx square root*/
            d = i+j>>1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
            d += dsq/d;
            d >>=1;
         }
         else
         {
            d=dsq;
         }
         if (d>maxr_off) d = maxr_off;
         x = hx - j; 
         if (x>=0)
         {
            y = hy - i;
            if (y>=0) img_bg[y*img_width+x]=bgave[d];
         }
         x = hx - i; 
         if (x>=0)
         {
            y = hy - j;
            if (y>=0) img_bg[y*img_width+x]=bgave[d];
         }
      }  
   }           
   /* Form top right quarter */
   for (iy=0;iy<=hy;++iy)
   {
      ip = (iy*img_width + hx) + 1;
      jp = ip - 2;
      for (ix=hx+1;ix<img_width;++ix) img_bg[ip++] = img_bg[jp--];
   }
   /* Form lower half */
   for (iy=hy+1,ky=hy-1;iy<img_height;++iy,--ky)
   {
      ip = iy*img_width;
      jp = ky*img_width;
      for (ix=0;ix<img_width;++ix) img_bg[ip++] = img_bg[jp++];
   }
   return 0;
}
/* Fortran binding: xdlf_i2_bgradimg */

void xdlf_i2_bgradimg (imgw, imgh, ibgave, nbg, img_bg, ierr)
int *imgw, *imgh, *ibgave, *nbg, *img_bg, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_i2_bgradimg (*imgw, *imgh, ibgave, maxr_off, img_bg);
   return;
}

/*-Routine: Get background value from background function - xdl_i2_bgval
The routine xdl_i2_bgval (xdlf_i2_bgval) is used to get a background
value at a given pixel position in the full image from a radially averaged 
background function as returned by xdl_i2_bgstrip (xdlf_i2_bgstrip).
If the pixel is outside the range of the background function the value
at the maximum available radius will be returned.
-end*/
/*
*****************************
**   xdl_i2_bgval          **
*****************************

Purpose: Get background value at a pixel position in the full image from
         radially averaged background function

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_I2_BGVAL (IX, IY, IBGAVE, NBG, IMGW, IMGH, 
                               IXOPIX, IYOPIX, NXP, NYP, IVAL, IERR)
-end*/
 
/*-Parameters:
IX          (R)   Pixel x position in full image (from 1 up)
IY          (R)   Pixel y position in full image (from 1 up)
IBGAVE      (R)   Array with radially averaged background
                  values (see bgave)
NBG         (W)   The number of values in IBGAVE
                  (maxr_off+1, see maxr_off)
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IXOPIX      (R)   Origin x pixel of background image wrt full
                  image (from 1 up) (see xo_pix)
IYOPIX      (R)   Origin y pixel of background image wrt full
                  image (from 1 up) (see yo_pix)
NXP         (R)   No. of x pixels from full image compressed
                  into 1 in background image (see nxp)
NYP         (R)   No. of y pixels from full image compressed
                  into 1 in background image (see nyp)
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_i2_bgval
                  call
-end*/

/*-C:*/
int xdl_i2_bgval (ix, iy, bgave, maxr_off, img_width, img_height,
                    xo_pix, yo_pix, nxp, nyp, ival)
/*end*/
 
/*-Parameters:*/
int ix;                 /* The x pixel position in the full image
                           (from 1 up) (R)*/
int iy;                 /* The y pixel position in the full image
                           (from 1 up) (R)*/
int bgave[];            /* Radially average background values from
                           xdl_i2_bgstrip (R)*/
int maxr_off;           /* Maximum offset in 'bgave' from xdl_i2_bgstrip 
                           (R)*/
int img_width;          /* Width of the background image used to calculate 
                           background function (R)*/
int img_height;         /* Height of the background image used to calculate 
                           background function (R)*/
int xo_pix;             /* Origin x pixel of background image wrt full
                           image (from 1 up) (R)*/
int yo_pix;             /* Origin y pixel of background image wrt full
                           image (from 1 up) (R)*/
int nxp;                /* No. of x pixels from full image compressed
                           into 1 in background image (R)*/
int nyp;                /* No. of y pixels from full image compressed
                           into 1 in background image (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;             /* Offsets from centre of compressed background 
                           image from which background function was 
                           determined */
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */
   int d, dsq;          /* Distance and d**2 from centre */
   int hx, hy;          /* Offsets of central pixel in compressed background
                           image from which background function was
                           determined */

   xoff = (ix-xo_pix)/nxp;
   yoff = nyp*img_height + yo_pix - 1 - iy;
   yoff /= nyp;
   hx = img_width/2;
   hy = img_height/2;
   i = abs(hx-xoff);
   j = abs(hy-yoff);
   dsq = i*i + j*j;
   if (dsq>1)
   {
      d = i + j>>1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
      d += dsq/d;
      d >>= 1;
   }
   else
   {
      d = dsq;
   }
   if (d>maxr_off) d = maxr_off;
   *ival = bgave[d];
   return 0;
}    
/* Fortran binding: xdlf_i2_bgval */

void xdlf_i2_bgval (ix, iy, ibgave, nbg, imgw, imgh, ixopix, iyopix,
                      nxp, nyp, ival, ierr)
int *ix, *iy, *ibgave, *nbg, *imgw, *imgh, *ixopix, *iyopix;
int *nxp, *nyp, *ival, *ierr;
{
   int maxr_off;

   maxr_off = *nbg - 1;
   *ierr = xdl_i2_bgval (*ix, *iy, ibgave, maxr_off, *imgw, *imgh, 
                           *ixopix, *iyopix, *nxp, *nyp, ival);
   return;
}
/*-Routine: Get background value from background image 'i2' - xdl_i2_bgvalimg
The routine xdl_i2_bgvalimg (xdlf_i2_bgvalimg) is used to get a background
value at a given pixel position in the full image from a previously calculated
background 'i2' image (e.g. from xdl_i2_bgradimg (xdlf_i2_bgradimg) or
xdl_i2_bgcalc (xdlf_i2_bgcalc)). If the pixel is outside the range of the 
background image the background value at the edge will be returned.
-end*/
/*
*****************************
**   xdl_i2_bgvalimg       **
*****************************

Purpose: Get background value at a pixel position in the full image from
         a background 'i2' image

Author:  John Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_I2_BGVALIMG (IX, IY, IMG_BG, IMGW, IMGH, 
        +                         IXOPIX, IYOPIX, NXP, NYP, IVAL, IERR)
-end*/
 
/*-Parameters:
IX          (R)   Pixel x position in full image (from 1 up)
IY          (R)   Pixel y position in full image (from 1 up)
IMG_BG      (R)   Background image (e.g. from xdl_i2_bgcalc
                  or xdl_i2_bgradimg (see img_bg)
IMGW        (R)   Compressed and background image width 
                  (see 'img_width')
IMGH        (R)   Compressed and background image height
                  (see 'img_height')
IXOPIX      (R)   Origin x pixel of background image wrt full
                  image (from 1 up) (see xo_pix)
IYOPIX      (R)   Origin y pixel of background image wrt full
                  image (from 1 up) (see yo_pix)
NXP         (R)   No. of x pixels from full image compressed
                  into 1 in background image (see nxp)
NYP         (R)   No. of y pixels from full image compressed
                  into 1 in background image (see nyp)
IVAL        (W)   Returned background value (see ival)
IERR        (W)   Returns the status from the xdl_i2_bgvalimg
                  call
-end*/

/*-C:*/
int xdl_i2_bgvalimg (ix, iy, img_bg, img_width, img_height,
                    xo_pix, yo_pix, nxp, nyp, ival)
/*end*/
 
/*-Parameters:*/
int ix;                 /* The x pixel position in the full image
                           (from 1 up) (R)*/
int iy;                 /* The y pixel position in the full image
                           (from 1 up) (R)*/
unsigned short img_bg[]; /* Background 'i2' image (R)*/
int img_width;          /* Width of the background image (R)*/
int img_height;         /* Height of the background image (R)*/
int xo_pix;             /* Origin x pixel of background image wrt full
                           image (from 1 up) (R)*/
int yo_pix;             /* Origin y pixel of background image wrt full
                           image (from 1 up) (R)*/
int nxp;                /* No. of x pixels from full image compressed
                           into 1 in background image (R)*/
int nyp;                /* No. of y pixels from full image compressed
                           into 1 in background image (R)*/
int *ival;              /* Returns background value (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int xoff, yoff;      /* Offsets in compressed background image from which
                           background function was determined */

   xoff = (ix-xo_pix)/nxp;
   yoff = nyp*img_height + yo_pix - 1 - iy;
   yoff /= nyp;
   *ival = img_bg[yoff*img_width+xoff];
   return 0;
}    
/* Fortran binding: xdlf_i2_bgvalimg */

void xdlf_i2_bgvalimg (ix, iy, img_bg, imgw, imgh, ixopix, iyopix,
                      nxp, nyp, ival, ierr)
int *ix, *iy, *img_bg, *imgw, *imgh, *ixopix, *iyopix;
int *nxp, *nyp, *ival, *ierr;
{
   *ierr = xdl_i2_bgvalimg (*ix, *iy, img_bg, *imgw, *imgh, 
                           *ixopix, *iyopix, *nxp, *nyp, ival);
   return;
}
