/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Blank Object View-Object Routines
-end*/
 
/*-Intro:
The blank object view-object is merely a window with a surrounding border 
similar to that used in other view-objects which may be output on a display 
for aesthetic reasons to indicate the position on a tiled display which will be 
occupied by another view-object later.
-end*/
 
/*-Section: Creating and Handling a Blank View-Object
-end*/
 

/*Routines List:
The blank object view-object has the following routines available:

   xdl_blank_object            Create a new blank object view-object
*/

/*-Routine: Create a blank object view-object - xdl_blank_object
THe routine xdl_blank_object (xdlf_blank_object) is used to create a new
blank object view-object. The size of the object is supplied by the user.
-end*/
/*
***************************
**    xdl_blank_object   **
***************************

Purpose: Display a blank object with a frame

Author:  John W. Campbell, June 1990

*/
/*-Fortran:
         CALL XDLF_blank_OBJECT (IVH, IVHPARENT, IX, IY, ICSET, IWIDTH, 
        +                        IHEIGHT, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
IWIDTH    (R)   Width in pixels (see width)
IHEIGHT   (R)   Height in pixels (see height)
IERR      (W)   Returns status from xdl_blank_object call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wblank;                    /* Blank object window id */
   GC          paint_gc;             /* GC for text paint window */
   int         width;                /* Object width */
   int         height;               /* Object height */
}  
   XDl_blank_object;

/* Define routine types */
int xdl_blank_object();
void xdl_blank_object_rp1();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_blank_object xdlf_blank_object_

#endif

#if  LINKTYP == 2

#define xdlf_blank_object XDLF_blank_object

#endif

void xdlf_blank_object();

/*-C:*/
int xdl_blank_object (vh, vh_parent, x, y, cset, width, height)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame,
                             if 0 then a base  level frame is created to
                             enclose the blank view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - 
                             normally 0 (R)*/
int       width;          /* Window width in pixels (R)*/
int       height;         /* Window height in pixels (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
-end*/
{
   Window wframe;              /* Primary window id */
   Window wparent;             /* Parent window id */
   Status st;                  /* Status return from Xlib calls */
   XDl_blank_object * gd;      /* Pointer to view-object's global data 
                                  structure */
   char * p;                   /* Temporary pointer */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */

   err = 0;

/* Create the top level frame (or base frame) window for the routine */

/* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent window was specified */
      wframe = xdl_create_base_frame_window ("xdl_blabk_object", "xdl_blank",
                                      x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_blank_object));
   gd = (XDl_blank_object *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, p, 1091, NULL, NULL);

   gd->wframe = wframe;
   gd->width = width;
   gd->height = height;

   /* Create blank paint window */
  
   gd->wblank = xdl_create_paint_window (wframe, 0, 0, 0, width, height, cset);
   xdl_add_window (vh, gd->wblank, xdl_blank_object_rp1, NULL);

   XSelectInput (XDL_display, gd->wblank, ExposureMask|StructureNotifyMask);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wblank, 0, NULL);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return 0;
}

/* Fortran binding: xdf_blank_object */

void xdlf_blank_object (ivh, ivhparent, ix, iy, icset, iwidth, iheight, 
                     ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *iwidth, *iheight, *ierr;
{
   *ierr = xdl_blank_object (*ivh, *ivhparent, *ix, *iy, *icset, *iwidth, 
                            *iheight);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
************************************
**     xdl_blank_object_rp1       **
************************************

Purpose: Repaint the area

Return:  None

Author:  John W. Campbell, July 1990

*/

void xdl_blank_object_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_blank_object * gd;     /* Pointer to global data area */

   gd = (XDl_blank_object *) global;
   XClearWindow (XDL_display, gd->wblank);
   XDrawRectangle (XDL_display, gd->wblank, gd->paint_gc,
                  BORDER_WIDTH, BORDER_HEIGHT,
                  gd->width - 2*BORDER_WIDTH - 3,
                  gd->height - 2*BORDER_HEIGHT - 3);
   return;
}
