/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Control Panel Items View-Object Routines
-end*/
 
/*-Intro:
The control panel view-objects give access from an application to the
individual Panel Items within XDL_VIEW (Panel choice, button, slider
etc.). The documentation on these items should be used in conjunction
with this documentation. For those items which have an input to the
application program, an active strip may be included within the control
item and should be used where possible to fit in with the general
philosophy of input to application programs via XDL_VIEW view-objects.
-end*/
 
 

/*Routines List:
The following sets of routines are available:

   Control view-object with Panel Choice item

      xdl_control_choice                 Create a Panel Choice object
      xdl_control_choice_reset           Reset selected item
      xdl_control_choice_getdata         Get selected item number
      xdl_control_choice_getsize         Get size requirements

   Control view-object with Panel Button item

      xdl_control_button                 Create a Panel Button object
      xdl_control_button_set             Set new button label
      xdl_control_button_getsize         Get size requirements

   Control view-object with Panel Label item

      xdl_control_label                  Create a Panel Label object
      xdl_control_label_set              Set new label
      xdl_control_label_getsize          Get size requirements

   Control view-object with Panel Value item

      xdl_control_value                  Create a Panel Value object
      xdl_control_value_set              Set new value
      xdl_control_value_getdata          Get input value string
      xdl_control_value_getsize          Get size requirements


   Control view-object with Panel Slider item

      xdl_control_slider                 Create a Panel Slider object
      xdl_control_slider_reset           Set new slider position
      xdl_control_slider_getdata         Get input slider position
      xdl_control_slider_getsize         Get size requirements

   Control view-object with Panel I/O item

      xdl_control_io                     Create a Panel I/O object
      xdl_control_io_prompt              Set a new prompt
      xdl_control_io_clear               Clear the I/O item
      xdl_control_io_kbfocus             Set keyboard focus
      xdl_control_io_getdata             Get input string
      xdl_control_io_getsize             Get size requirements
*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"

#define MIN_ACT_STRIP 100     /* Minimum acceptable active strip width */

/*==================================================*/
/*         PANEL CHOICE                             */
/*==================================================*/

/*-Section: Panel Choice Item Routines
These are routines for creating an handling a view-object containing a
panel choice item.
-end*/

/*-Routine: View_object with a Panel Choice Item - xdl_control_choice
The routine xdl_control_choice (xdlf_control_choice) is used to create a
view object containing a Panel Choice item (See Panel Item Routines).
An active strip may be displayed if desired.
-end*/
/*
***************************
**    xdl_control_choice **
***************************

Purpose: View-object with panel choice item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_CHOICE (IVH, IVHPARENT, IX, IY, IACT,
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          XDLSTR(LABEL), LLEN,
        +                          NITEMS, XDLSTR(STRINGS), LENSTRS,
        +                          IDFITEM, MSG, LENMSG,
        +                          MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IACT      (R)   Display active strip flag 1=yes, 0=no (see
                act_strip)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   Flag =1 border, =0 no border (see brdr)
LABEL     (R)   Address of label string for panel choice.
                ** Pass address using the XDLSTR function **
LLEN      (R)   Length of the label string (>0)
NITEMS    (R)   No. of panel choice item strings (see
                num_items)
STRINGS   (R)   Fortran array of character strings cotaining
                the panel choice item strings.
                [CHARACTER*LENSTRS STRINGS(NITEMS)].
                ** Pass address using XDLSTR function **
                (see strings)
LENSTRS   (R)   Length of the panel choice item strings
                (>0 (see len_strs)
IDFITEM   (R)   Default item number (see df_item)
MSG       (R)   Address of character string for active strip
                message. May be a blank string if IACT = 0.
                ** Pass address using XDLSTR function **
                (see msg)
LENMSG    (R)   Length of active strip message string (>0)
                (see len_msg)
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_choice call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object choice;              /* Panel Choice object */
   int act_strip;                    /* Active strip present flag */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        **strs;               /* Pointers to null terminated panel
                                        choice item strings (copies) */
   int         num_items;            /* No. of panel choice item strings */
   char        *label;               /* Null terminated label string (copy) */
   char        *msg;                 /* Null terminated active strip message
                                        string */
   int         item;                 /* Last selected item */
}  
   XDl_control_choice;

/* Define routine types */

int xdl_control_choice();
void xdl_control_choice_rps();
void xdl_control_choice_cp();
void xdl_control_choice_on_off();
void xdl_control_choice_tidy_up();
int xdl_control_choice_reset();
int xdl_control_choice_getdata();
void xdl_control_choice_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_choice xdlf_control_choice_
#define xdlf_control_choice_reset xdlf_control_choice_reset_
#define xdlf_control_choice_getdata xdlf_control_choice_getdata_
#define xdlf_control_choice_getsize xdlf_control_choice_getsize_
#endif

#if  LINKTYP == 2

#define xdlf_control_choice XDLF_CONTROL_CHOICE
#define xdlf_control_choice_reset XDLF_CONTROL_CHOICE_RESET
#define xdlf_control_choice_getdata XDLF_CONTROL_CHOICE_GETDATA
#define xdlf_control_choice_getsize XDLF_CONTROL_CHOICE_GETSIZE

#endif

void xdlf_control_choice();
void xdlf_control_choice_reset();
void xdlf_control_choice_getdata();
void xdlf_control_choice_getsize();

/*-C:*/
int xdl_control_choice (vh, vh_parent, x, y, act_strip, hcen, vcen,
                        font_type, brdr, label, llen, 
                        num_items, strings, len_strs, 
                        df_item, msg, len_msg, minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* Flag =1 border, =0 no border to be drawn round
                             panel item (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       num_items;      /* No. of panel choice item strings (R) */
char    * strings;        /* Panel choice item strings. This may 
                             either be an array of pointers to num_items 
                             null terminated character strings (len_strs=0)
                             or a character string num_items*len_strs in 
                             length with each string occupying len_strs 
                             characters (left justified and padded to the 
                             right with blanks if needed). (R)*/
int       len_strs;       /* Strings length (0 = array of pointers to
                             null terminated character strings (e.g. for 
                             'C' programs) or the length of each string 
                             (e.g. for Fortran programs). See also the 
                             previous item. (R) */
int       df_item;        /* Default item number (R)*/
char    * msg;            /* Message for active strip, may be null if
                             no active strip required (R)*/
int       len_msg;        /* Length of active strip message string; may
                             be 0 is a null terminated string given for
                             msg (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_choice * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   char * item_str;            /* Temp pointer to Fortran type strings */
   char ** item_strs;          /* Temp pointer to character strings list */
   int max_name_len;           /* Maximum length of choice item string */
   int itm;                    /* Items loop variable */
   int ioff;                   /* Offset in Fortran type strings list */
   int len;                    /* String length */
   int l;                      /* Loop variable */
   int actstrip_width;         /* Active strip width */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   if (llen==0) llen = strlen(label);

   max_name_len = 0;
   if (len_strs>0)
   {
      item_str = (char *) strings;
      ioff = 0;
      for (itm=0;itm<num_items;++itm)
      {
         len = 0;
         for (l=len_strs-1;l>=0;--l)
         {
            if (item_str[ioff+l]!=' ')
            {
               len = l+1;
               break;
            }
         }
         if (len>max_name_len) max_name_len = len;
         ioff += len_strs;
      }
   }
   else
   {
      item_strs = (char **) strings;
      for (itm=0;itm<num_items;++itm)
      {
         len = strlen (item_strs[itm]);
         if (len>max_name_len) max_name_len = len;
      }
   }

   xdl_panel_choice_size (llen, max_name_len, brdr, font_type,
                          &w, &h);

   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) w = MIN_ACT_STRIP;
      h = h + ACT_STRIP_HEIGHT;
   }
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      if (act_strip)
      {
         y_pos = (height-ACT_STRIP_HEIGHT-h)/2;
      }
      else
      {
         y_pos = (height-h)/2;
      }
   }
   else
   {
      y_pos = BORDER_HEIGHT;
      if (act_strip) y_pos = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_choice", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_choice *) malloc (sizeof(XDl_control_choice));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2101, 
                        xdl_control_choice_on_off,
                        xdl_control_choice_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->act_strip = act_strip;
   gd->on_off = 0;
   gd->msg = 0;
   gd->label = 0;
   gd->strs = 0;
   gd->item = 0;

   /* Make copy of active strip string */

   if (act_strip)
   {
      len = len_msg;
      if (len_msg==0) len = strlen(msg);
      p = malloc(len+1);
      if (p==0) {err |= 2; return err;}
      gd->msg = p;
      p[0] = 0;
      strncat (p,msg,len);
   }

   /* Make copy of label string */

   len = llen;
   if (llen) len = strlen(label);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->label = p;
   p[0] = 0;
   strncat (p,label,len);

   /* Make copy of items strings */

   gd->num_items = num_items;
   gd->strs = (char **) malloc(num_items*sizeof(char *));
   if (gd->strs==0) {err |= 2; return err;}
   if (len_strs>0)
   {
      item_str = (char *) strings;
      ioff = 0;
      for (itm=0;itm<num_items;++itm)
      {
         len = 0;
         for (l=len_strs-1;l>=0;--l)
         {
            if (item_str[ioff+l]!=' ')
            {
               len = l+1;
               break;
            }
         }
         p = malloc(len+1);
         if (p==0) {err |= 2; return err;}
         gd->strs[itm] = p;
         p[0] = 0;
         strncat (p, &item_str[ioff], len);
         ioff += len_strs;
      }
   }
   else
   {
      item_strs = (char **) strings;
      for (itm=0;itm<num_items;++itm)
      {
         len = strlen (item_strs[itm]);
         p = malloc(len+1);
         if (p==0) return 2;
         gd->strs[itm] = p;
         strcpy (p, item_strs[itm]);
      }
   }

   /* Create active strip window (if required) */

   if (act_strip)
   {
      actstrip_width = width - 2*BORDER_WIDTH - 2;
      gd->wact = xdl_create_active_strip (gd->wframe, vh, actstrip_width,
                                          xdl_control_choice_rps);
   }

   /* Create panel item */

   gd->choice = xdl_panel_choice (vh, gd->wframe, x_pos, y_pos, gd->label,
                     num_items, gd->strs,
                     df_item, font_type, brdr, 0, 0,
                     xdl_control_choice_cp);

   return 0;
}

/* Fortran binding: xdlf_control_choice */

void xdlf_control_choice (ivh, ivhparent, ix, iy, iact, ihcen, ivcen,
                        ifont, ibrdr, label, llen,
                        nitems, strings, lenstrs, idfitem, msg, lenmsg,
                        minw, minh, ierr)

int *label, *strings, *msg;
int *ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *llen, *nitems, *lenstrs, *idfitem, *lenmsg, *minw, *minh;
{
   *ierr = xdl_control_choice (*ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen,
                             *ifont, *ibrdr, XDL_pointers[*label], *llen,
                             *nitems, XDL_pointers[*strings], *lenstrs,
                             *idfitem, XDL_pointers[*msg], *lenmsg, 
                             *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_control_choice_rps  **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_choice_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_choice * gd;  /* Pointer to global data area */

   gd = (XDl_control_choice *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->msg, 0);
   }
}
/*
***********************************
**     xdl_control_choice_cp     **
***********************************

Purpose: Callback for panel choice item

Author:  John W. Campbell, Jan 1994

*/

void xdl_control_choice_cp (iv, item)

int iv;
int item;

{
   XDl_control_choice *gd;          /* Pointer to global data area */

   gd = (XDl_control_choice *) XDL_view_objects[iv]->gd;
   gd->item = item;
   if (gd->on_off)
   {
       XDL_return_data_vh = XDL_view_objects[iv]->vh;
   }
   return;
}

/*
****************************************
**     xdl_control_choice_on_off      **
****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_choice_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_choice *gd;        /* Pointer to global data area */

   gd = (XDl_control_choice *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->msg, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}

/*
*****************************************
**     xdl_control_choice_tidy_up      **
*****************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_choice_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_choice *gd;    /* Pointer to the global data area */
   int i;                     /* Temp/loop variable */

   gd = (XDl_control_choice *) global;
   if (gd->label!=NULL) free(gd->label);
   if (gd->msg!=NULL) free(gd->msg);
   if (gd->strs!=NULL)
   {
      for(i=0;i<gd->num_items;++i)
      {
         if (gd->strs[i]!=NULL) free (gd->strs[i]);
      }
      free (gd->strs);
   }
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Reset selected choice - xdl_control_choice_reset
The routine xdl_control_choice_reset (xdlf_control_choice_reset) is
used to reset the selecte item number for a control Panel Choice view-object.
-end*/
/*
*****************************************
**     xdl_control_choice_reset        **
*****************************************

Purpose: Reset selected choice item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_CHOICE_RESET (IVH, ITEM, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
ITEM      (R)  New selected item number
IERR      (W)  Returns status from xdl_control_choice_reset call
-end*/

/*-C:*/
int xdl_control_choice_reset (vh, item)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
int item;                 /* Rest item number (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_choice *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_choice_reset",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2101)
   {
      xdl_view_type_errmsg ("xdl_control_choice_reset", vh,
                            "xdl_control_choice");
      return 2;
   }

   gd = (XDl_control_choice *) XDL_view_objects[iv]->gd;

   /* Set the new value */

   xdl_panel_choice_reset (gd->choice, item);

   return 0;
}

/* Fortran binding xdlf_control_choice_reset */

void xdlf_control_choice_reset (ivh, item, ierr)
int *ivh, *item, *ierr;
{
   *ierr = xdl_control_choice_reset (*ivh, *item);
   return;
}
/*-Routine: Get selected item number - xdl_control_choice_getdata
The routine xdl_control_choice_getdata (xdlf_control_choice_getdata) is
used to determine the selected item number after a return from an
XDL_VIEW event handling routine such as xdl_get_events (xdlf_get_events)
indicates that there was input from the view-object in question.
-end*/
/*
****************************************
**     xdl_control_choice_getdata     **
****************************************

Purpose: Get returned data from the control Panel Choice view-object

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_CHOICE_GETDATA (IVH, ITEM, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ITEM     (W)  Selected choice item number
IERR     (W)  Returns status from xdl_control_choice_getdata call
-end*/

/*-C:*/
int xdl_control_choice_getdata (vh, item)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *item;       /* Returns selected item number (W)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_choice *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_choice_getdata",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2101)
   {
      xdl_view_type_errmsg ("xdl_control_choice_getdata", vh,
                            "xdl_control_choice");
      return 2;
   }

   gd = (XDl_control_choice *) XDL_view_objects[iv]->gd;

   /* Return the required data */

   *item = gd->item;

   return 0;
}

/* Fortran binding xdlf_control_choice_getdata */

void xdlf_control_choice_getdata (ivh, item, ierr)
int *ivh, *item, *ierr;
{
   *ierr = xdl_control_choice_getdata (*ivh, item);
   return;
}

/*-Routine: Get required size - xdl_control_choice_getzize
The routine xdl_control_choice_getsize (xdlf_control_choice_getsize) is
used to determine the required size for a control view-object containing
a Panel Choice Item given various details about the requirements of the
item.
-end*/
/*
****************************************
**     xdl_control_choice_getsize     **
****************************************

Purpose: Get size of control choice item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_CHOICE_GETSIZE (IACT, IFONT, IBRDR,
        +                          LLEN, LENSTRS, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IACT      (R)   =1 include active strip, =0 do not
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   Flag =1 border, =0 no border (see brdr)
LLEN      (R)   Length of the label string (>0)
LENSTRS   (R)   Length (maximum) of the panel choice item 
                strings (>0 (see len_strs)
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_choice_getsize (act_strip, font_type, brdr, llen,
                                 len_strs, width, height)
/*end*/
 
/*-Parameters:*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* Flag =1 border, =0 no border to be drawn round
                             panel item (R)*/
int       llen;           /* Required label length (>0) (R)*/
int       len_strs;       /* Panel choice strings length (maximum) 
                             (>0) (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_choice_size (llen, len_strs, brdr, font_type,
                          width, height);

   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP;
      *height = *height + ACT_STRIP_HEIGHT;
   }
   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   return;
}
/* Fortran binding: xdlf_control_choice_getsize */

void xdlf_control_choice_getsize (iact, ifont, ibrdr, llen, lenstrs, 
                                  iwidth, iheight)

int *iact, *ifont, *ibrdr, *llen, *lenstrs, *iwidth, *iheight;
{
   xdl_control_choice_getsize (*iact, *ifont, *ibrdr, *llen, *lenstrs,
                               iwidth, iheight);
   return;
}

/*==================================================*/
/*         PANEL BUTTON                             */
/*==================================================*/


/*-Section: Panel Button Item Routines
These are routines for creating an handling a view-object containing a
panel button item.
-end*/

/*-Routine: View_object with a Panel Button Item - xdl_control_button
The routine xdl_control_button (xdlf_control_button) is used to create a
view object containing a Panel Button item (See Panel Item Routines).
An active strip may be displayed if desired.
-end*/
/*
***************************
**    xdl_control_button **
***************************

Purpose: View-object with panel button item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_BUTTON (IVH, IVHPARENT, IX, IY, IACT,
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          XDLSTR(LABEL), LLEN, MAXLEN,
        +                          XDLSTR(MSG), LENMSG,
        +                          IBOLD, ICENT,
        +                          MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IACT      (R)   Display active strip flag 1=yes, 0=no (see
                act_strip)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   Button border width (>0)
LABEL     (R)   Address of label string for panel choice.
                ** Pass address using the XDLSTR function **
LLEN      (R)   Length of the label string (>0)
MAXLEN    (R)   Maximum length to allow for a label string
                (label may be changed)
MSG       (R)   Address of character string for active strip
                message. May be a blank string if IACT = 0.
                ** Pass address using XDLSTR function **
                (see msg)
LENMSG    (R)   Length of active strip message string (>0)
                (see len_msg)
IBOLD     (R)   = 1 bold print, = 0 normal print for label
ICENT     (R)   = 1 centre label in button, = 0 do not
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_button call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object button;              /* Button panel item */
   int act_strip;                    /* Active strip present flag */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        *label;               /* Null terminated label string (copy) */
   char        *msg;                 /* Active strip message string (copy) */
}  
   XDl_control_button;

/* Define routine types */

int xdl_control_button();
void xdl_control_button_rps();
void xdl_control_button_cp();
void xdl_control_button_on_off();
void xdl_control_button_tidy_up();
int xdl_control_button_set();
void xdl_control_button_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_button xdlf_control_button_
#define xdlf_control_button_set xdlf_control_button_set_
#define xdlf_control_button_getsize xdlf_control_button_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_control_button XDLF_control_button
#define xdlf_control_button_set XDLF_CONTROL_BUTTON_SET
#define xdlf_control_button_getsize XDLF_CONTROL_BUTTON_GETSIZE

#endif

void xdlf_control_button();
void xdlf_control_button_set();
void xdlf_control_button_getsize();

/*-C:*/
int xdl_control_button (vh, vh_parent, x, y, act_strip, hcen, vcen,
                        font_type, brdr, label, llen, maxlen,
                        msg, len_msg,
                        bold, cent, minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* Border width for the button (>0) (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       maxlen;         /* Maximum length required for a label (button
                             label may be reset (R)*/
char    * msg;            /* Message for active strip, may be null if
                             no active strip required (R)*/
int       len_msg;        /* Length of active strip message string; may
                             be 0 is a null terminated string given for
                             msg (R)*/
int       bold;           /* = 1 bold print, = 0 normal print for label
                             (R)*/
int       cent;           /* = 1 centre label in button, = 0 do not (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_button * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int len;                    /* String length */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   int actstrip_width;         /* Active strip width */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   xdl_panel_button_size (maxlen, font_type, brdr, &w, &h);

   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) w = MIN_ACT_STRIP;
      h = h + ACT_STRIP_HEIGHT;
   }
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      if (act_strip)
      {
         y_pos = (height-ACT_STRIP_HEIGHT-h)/2;
      }
      else
      {
         y_pos = (height-h)/2;
      }
   }
   else
   {
      y_pos = BORDER_HEIGHT;
      if (act_strip) y_pos = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_button", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_button *) malloc (sizeof(XDl_control_button));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2102, 
                        xdl_control_button_on_off,
                        xdl_control_button_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->act_strip = act_strip;
   gd->on_off = 0;
   gd->label = 0;
   gd->msg = 0;

   /* Make copy of active strip string */

   if (act_strip)
   {
      len = len_msg;
      if (len_msg==0) len = strlen(msg);
      p = malloc(len+1);
      if (p==0) {err |= 2; return err;}
      gd->msg = p;
      p[0] = 0;
      strncat (p,msg,len);
   }

   /* Make copy of label string */

   len = llen;
   if (llen==0) len = strlen(label);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->label = p;
   p[0] = 0;
   strncat (p,label,len);

   /* Create active strip window (if required) */

   if (act_strip)
   {
      actstrip_width = width - 2*BORDER_WIDTH - 2;
      gd->wact = xdl_create_active_strip (gd->wframe, vh, actstrip_width,
                                          xdl_control_button_rps);
   }

   /* Create panel item */

   gd->button = xdl_panel_button (vh, gd->wframe, x_pos, y_pos, 
                     gd->label, maxlen,
                     font_type, brdr, 0, 0, bold, cent, 0, 
                     xdl_control_button_cp);

   return 0;
}

/* Fortran binding: xdlf_control_button */

void xdlf_control_button (ivh, ivhparent, ix, iy, iact, ihcen, ivcen,
                        ifont, ibrdr, label, llen, maxlen, 
                        msg, lenmsg, ibold, icent,
                        minw, minh, ierr)

int *label, *msg;
int *ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *llen, *maxlen, *lenmsg, *ibold, *icent, *minw, *minh;
{
   *ierr = xdl_control_button (*ivh, *ivhparent, *ix, *iy, *iact, 
                             *ihcen, *ivcen,
                             *ifont, *ibrdr, XDL_pointers[*label], *llen, 
                             *maxlen,
                             XDL_pointers[*msg], *lenmsg, *ibold, *icent, 
                             *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_control_button_rps  **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_button_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_button * gd;  /* Pointer to global data area */

   gd = (XDl_control_button *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->msg, 0);
   }
}
/*
***********************************
**     xdl_control_button_cp     **
***********************************

Purpose: Callback for panel choice item

Author:  John W. Campbell, Jan 1994

*/

void xdl_control_button_cp (iv, item)

int iv;
int item;

{
   XDl_control_button *gd;          /* Pointer to global data area */

   gd = (XDl_control_button *) XDL_view_objects[iv]->gd;
   if (gd->on_off)
   {
       XDL_return_data_vh = XDL_view_objects[iv]->vh;
   }
   return;
}

/*
****************************************
**     xdl_control_button_on_off      **
****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_button_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_button *gd;        /* Pointer to global data area */

   gd = (XDl_control_button *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->msg, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}

/*
*****************************************
**     xdl_control_button_tidy_up      **
*****************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_button_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_button *gd;    /* Pointer to the global data area */

   gd = (XDl_control_button *) global;
   if (gd->label!=NULL) free(gd->label);
   if (gd->msg!=NULL) free(gd->msg);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set new label string - xdl_control_button_set
The routine xdl_control_button_set (xdlf_control_button_set) is
used to set a new label string for a control Panel Button view-object.
-end*/
/*
****************************************
**     xdl_control_button_set         **
****************************************

Purpose: Set new label string in button

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_BUTTON_SET (IVH, XDLSTR(LABEL), LLEN, 
        +                                IBOLD, ICENT, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
LABEL     (R)  Address of new label string
               ** Pass address using the XDLSTR function **
LLEN      (R)  Length of the label string (>0)
IBOLD     (R)   = 1 bold print, = 0 normal print for label
ICENT     (R)   = 1 centre label in button, = 0 do not
IERR      (W)  Returns status from xdl_control_button_set call
-end*/

/*-C:*/
int xdl_control_button_set (vh, label, llen, bold, cent)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       bold;           /* = 1 bold print, = 0 normal print for label
                             (R)*/
int       cent;           /* = 1 centre label in button, = 0 do not (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_button *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_button_set",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2102)
   {
      xdl_view_type_errmsg ("xdl_control_button_set", vh,
                            "xdl_control_button");
      return 2;
   }

   gd = (XDl_control_button *) XDL_view_objects[iv]->gd;

   /* Set the new label */

   xdl_panel_button_set (gd->button, label, llen, bold, cent);

   return 0;
}

/* Fortran binding xdlf_control_button_set */

void xdlf_control_button_set (ivh, label, llen, ibold, icent, ierr)
int *label;
int *ivh, *llen, *ibold, *icent, *ierr;
{
   *ierr = xdl_control_button_set (*ivh, XDL_pointers[*label], *llen, *ibold, 
                                    *icent);
   return;
}

/*-Routine: Get required size - xdl_control_button_getzize
The routine xdl_control_button_getsize (xdlf_control_button_getsize) is
used to determine the required size for a control view-object containing
a Panel Button item given various details about the requirements of the
item.
-end*/
/*
****************************************
**     xdl_control_button_getsize     **
****************************************

Purpose: Get size of button item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_BUTTON_GETSIZE (IACT, IFONT, IBRDR,
        +                          MAXLEN, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IACT      (R)   =1 include active strip, =0 do not
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   Required border width (>0)
MAXLEN    (R)   Maximum required length of the label string (>0)
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_button_getsize (act_strip, font_type, brdr, maxlen,
                                 width, height)
/*end*/
 
/*-Parameters:*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* Button border width (>0) (R)*/
int       maxlen;         /* maximum required label length (>0) (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_button_size (maxlen, font_type, brdr, width, height);

   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP;
      *height = *height + ACT_STRIP_HEIGHT;
   }
   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   return;
}
/* Fortran binding: xdlf_control_button_getsize */

void xdlf_control_button_getsize (iact, ifont, ibrdr, maxlen, 
                                  iwidth, iheight)

int *iact, *ifont, *ibrdr, *maxlen, *iwidth, *iheight;
{
   xdl_control_button_getsize (*iact, *ifont, *ibrdr, *maxlen, 
                               iwidth, iheight);
   return;
}

/*==================================================*/
/*         PANEL LABEL                              */
/*==================================================*/


/*-Section: Panel Label Item Routines
These are routines for creating an handling a view-object containing a
panel label item.
-end*/

/*-Routine: View_object with a Panel Label Item - xdl_control_label
The routine xdl_control_label (xdlf_control_label) is used to create a
view object containing a Panel Label item (See Panel Item Routines).
-end*/
/*
***************************
**    xdl_control_label  **
***************************

Purpose: View-object with panel label item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_LABEL (IVH, IVHPARENT, IX, IY, 
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          XDLSTR(LABEL), LLEN, MAXLEN,
        +                          IBOLD, MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, =0 do not
LABEL     (R)   Address of label string for panel choice.
                ** Pass address using the XDLSTR function **
LLEN      (R)   Length of the label string (>0)
MAXLEN    (R)   Maximum length to allow for a label string
                (label may be changed)
IBOLD     (R)   = 1 bold print, = 0 normal print for label
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_label call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object labl;                /* Label panel item */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        *label;               /* Null terminated label string (copy) */
}  
   XDl_control_label;

/* Define routine types */

int xdl_control_label();
void xdl_control_label_rps();
void xdl_control_label_on_off();
void xdl_control_label_tidy_up();
int xdl_control_label_set();
void xdl_control_label_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_label xdlf_control_label_
#define xdlf_control_label_set xdlf_control_label_set_
#define xdlf_control_label_getsize xdlf_control_label_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_control_label XDLF_CONTROL_LABEL
#define xdlf_control_label_set XDLF_CONTROL_LABEL_SET
#define xdlf_control_label_getsize XDLF_CONTROL_LABEL_GETSIZE

#endif

void xdlf_control_label();
void xdlf_control_label_set();
void xdlf_control_label_getsize();

/*-C:*/
int xdl_control_label (vh, vh_parent, x, y, hcen, vcen,
                        font_type, brdr, label, llen, maxlen,
                        bold, minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       maxlen;         /* Maximum length required for a label 
                             (label may be reset) (R)*/
int       bold;           /* = 1 bold print, = 0 normal print for label
                             (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_label * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int len;                    /* String length */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   xdl_panel_label_size (maxlen, font_type, brdr, &w, &h);

   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;

   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      y_pos = (height-h)/2;
   }
   else
   {
      y_pos = BORDER_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_label", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_label *) malloc (sizeof(XDl_control_label));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2103, 
                        xdl_control_label_on_off,
                        xdl_control_label_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->on_off = 0;
   gd->label = 0;

   /* Make copy of label string */

   len = llen;
   if (llen==0) len = strlen(label);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->label = p;
   p[0] = 0;
   strncat (p,label,len);

   /* Create panel item */

   gd->labl = xdl_panel_label (vh, gd->wframe, x_pos, y_pos, 
                     gd->label, maxlen,
                     font_type, brdr, 0, 0, bold);
   return 0;
}

/* Fortran binding: xdlf_control_label */

void xdlf_control_label (ivh, ivhparent, ix, iy, ihcen, ivcen,
                        ifont, ibrdr, label, llen, maxlen, 
                        ibold, minw, minh, ierr)

int *label;
int *ivh, *ivhparent, *ix, *iy, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *llen, *maxlen, *ibold, *minw, *minh;
{
   *ierr = xdl_control_label (*ivh, *ivhparent, *ix, *iy, 
                             *ihcen, *ivcen,
                             *ifont, *ibrdr, XDL_pointers[*label], *llen, 
                             *maxlen,
                             *ibold, *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_control_label_rps   **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_label_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_label * gd;  /* Pointer to global data area */

   gd = (XDl_control_label *) global;

   return;
}

/*
****************************************
**     xdl_control_label_on_off       **
****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_label_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_label *gd;        /* Pointer to global data area */

   gd = (XDl_control_label *) XDL_view_objects[iv]->gd;

   gd->on_off = i;

   return;
}

/*
*****************************************
**     xdl_control_label_tidy_up       **
*****************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_label_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_label *gd;    /* Pointer to the global data area */

   gd = (XDl_control_label *) global;
   if (gd->label!=NULL) free(gd->label);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set new label string - xdl_control_label_set
The routine xdl_control_label_set (xdlf_control_label_set) is
used to set a new label string for a control Panel Label view-object.
-end*/
/*
****************************************
**     xdl_control_label_set          **
****************************************

Purpose: Set new label string in button

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_LABEL_SET (IVH, XDLSTR(LABEL), LLEN, 
        +                                IBOLD, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
LABEL     (R)  Address of new label string
               ** Pass address using the XDLSTR function **
LLEN      (R)  Length of the label string (>0)
IBOLD     (R)   = 1 bold print, = 0 normal print for label
IERR      (W)  Returns status from xdl_control_label_set call
-end*/

/*-C:*/
int xdl_control_label_set (vh, label, llen, bold)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       bold;           /* = 1 bold print, = 0 normal print for label
                             (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_label *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_label_set",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2103)
   {
      xdl_view_type_errmsg ("xdl_control_label_set", vh,
                            "xdl_control_label");
      return 2;
   }

   gd = (XDl_control_label *) XDL_view_objects[iv]->gd;

   /* Set the new label */

   xdl_panel_label_set (gd->labl, label, llen, bold);

   return 0;
}

/* Fortran binding xdlf_control_label_set */

void xdlf_control_label_set (ivh, label, llen, ibold, ierr)
int *label;
int *ivh, *llen, *ibold, *ierr;
{
   *ierr = xdl_control_label_set (*ivh, XDL_pointers[*label], *llen, *ibold);
   return;
}

/*-Routine: Get required size - xdl_control_label_getzize
The routine xdl_control_label_getsize (xdlf_control_label_getsize) is
used to determine the required size for a control view-object containing
a Panel Label item given various details about the requirements of the
item.
-end*/
/*
****************************************
**     xdl_control_label_getsize      **
****************************************

Purpose: Get size of label item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_LABEL_GETSIZE (IFONT, IBRDR,
        +                          MAXLEN, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
MAXLEN    (R)   Maximum required length of the label string (>0)
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_label_getsize (font_type, brdr, maxlen,
                                 width, height)
/*end*/
 
/*-Parameters:*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
int       maxlen;         /* maximum required label length (>0) (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_label_size (maxlen, font_type, brdr, width, height);

   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   return;
}
/* Fortran binding: xdlf_control_label_getsize */

void xdlf_control_label_getsize (ifont, ibrdr, maxlen, 
                                  iwidth, iheight)

int *ifont, *ibrdr, *maxlen, *iwidth, *iheight;
{
   xdl_control_label_getsize (*ifont, *ibrdr, *maxlen, 
                               iwidth, iheight);
   return;
}

/*==================================================*/
/*         PANEL VALUE                              */
/*==================================================*/

/*-Section: Panel Value Item Routines
These are routines for creating an handling a view-object containing a
panel value item.
-end*/

/*-Routine: View_object with a Panel Value Item - xdl_control_value
The routine xdl_control_value (xdlf_control_value) is used to create a
view object containing a Panel Value item (See Panel Item Routines).
An active strip may be displayed if desired.
-end*/
/*
***************************
**    xdl_control_value  **
***************************

Purpose: View-object with panel value item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_VALUE (IVH, IVHPARENT, IX, IY, IACT,
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          XDLSTR(LABEL), LLEN, 
        +                          XDLSTR(VALUE), IVLEN, MAXVLEN,
        +                          XDLSTR(MSG), LENMSG,
        +                          MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IACT      (R)   Display active strip flag 1=yes, 0=no (see
                act_strip)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
LABEL     (R)   Address of label string for panel choice.
                ** Pass address using the XDLSTR function **
LLEN      (R)   Length of the label string (>0)
VALUE     (R)   Address of value string for panel value.
                ** Pass address using the XDLSTR function **
IVLEN     (R)   Length of the value string (>0)
MAXVLEN   (R)   Maximum length to allow for a value string
                (value may be changed)
MSG       (R)   Address of character string for active strip
                message. May be a blank string if IACT = 0.
                ** Pass address using XDLSTR function **
                (see msg)
LENMSG    (R)   Length of active strip message string (>0)
                (see len_msg)
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_value call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object val;                 /* Button value item */
   int act_strip;                    /* Active strip present flag */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        *value;               /* Null terminated value string (copy) */
   char        *label;               /* Null terminated label string (copy) */
   char        *msg;                 /* Active strip message string (copy) */
   char        *valinp;              /* String to hold last input value */
}  
   XDl_control_value;

/* Define routine types */

int xdl_control_value();
void xdl_control_value_rps();
void xdl_control_value_cp();
void xdl_control_value_on_off();
void xdl_control_value_tidy_up();
int xdl_control_value_set();
int xdl_control_value_getdata();
void xdl_control_value_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_value xdlf_control_value_
#define xdlf_control_value_set xdlf_control_value_set_
#define xdlf_control_value_getdata xdlf_control_value_getdata_
#define xdlf_control_value_getsize xdlf_control_value_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_control_value XDLF_CONTROL_VALUE
#define xdlf_control_value_set XDLF_CONTROL_VALUE_SET
#define xdlf_control_value_getdata XDLF_CONTROL_VALUE_GETDATA
#define xdlf_control_value_getsize XDLF_CONTROL_VALUE_GETSIZE

#endif

void xdlf_control_value();
void xdlf_control_value_set();
void xdlf_control_value_getdata();
void xdlf_control_value_getsize();

/*-C:*/
int xdl_control_value (vh, vh_parent, x, y, act_strip, hcen, vcen,
                        font_type, brdr, label, llen, 
                        value, vlen, maxvlen,
                        msg, len_msg,
                        minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
char    * value;          /* Value string (R)*/
int       vlen;           /* Value length, may be 0 if null terminated
                             string passed (R)*/
int       maxvlen;        /* Maximum length required for a value string
                             (value may be reset) (R)*/
char    * msg;            /* Message for active strip, may be null if
                             no active strip required (R)*/
int       len_msg;        /* Length of active strip message string; may
                             be 0 is a null terminated string given for
                             msg (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_value * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int len;                    /* String length */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   int actstrip_width;         /* Active strip width */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   if (llen==0) llen = strlen(label);
   xdl_panel_value_size (llen, maxvlen, font_type, brdr, &w, &h);

   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) w = MIN_ACT_STRIP;
      h = h + ACT_STRIP_HEIGHT;
   }
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      if (act_strip)
      {
         y_pos = (height-ACT_STRIP_HEIGHT-h)/2;
      }
      else
      {
         y_pos = (height-h)/2;
      }
   }
   else
   {
      y_pos = BORDER_HEIGHT;
      if (act_strip) y_pos = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_value", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_value *) malloc (sizeof(XDl_control_value));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2104, 
                        xdl_control_value_on_off,
                        xdl_control_value_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->act_strip = act_strip;
   gd->on_off = 0;
   gd->label = 0;
   gd->msg = 0;
   gd->value = 0;
   gd->valinp = 0;

   /* Make copy of active strip string */

   if (act_strip)
   {
      len = len_msg;
      if (len_msg==0) len = strlen(msg);
      p = malloc(len+1);
      if (p==0) {err |= 2; return err;}
      gd->msg = p;
      p[0] = 0;
      strncat (p,msg,len);
   }

   /* Make copy of label string */

   len = llen;
   if (llen==0) len = strlen(label);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->label = p;
   p[0] = 0;
   strncat (p,label,len);

   /* Make copy of value string */

   len = vlen;
   if (vlen==0) len = strlen(value);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->value = p;
   p[0] = 0;
   strncat (p,value,len);

   /* String for input values */

   p = malloc(maxvlen+1);
   if (p==0) {err |= 2; return err;}
   gd->valinp = p;

   /* Create active strip window (if required) */

   if (act_strip)
   {
      actstrip_width = width - 2*BORDER_WIDTH - 2;
      gd->wact = xdl_create_active_strip (gd->wframe, vh, actstrip_width,
                                          xdl_control_value_rps);
   }

   /* Create panel item */

   gd->val = xdl_panel_value (vh, gd->wframe, x_pos, y_pos, 
                     gd->label, gd->value, maxvlen,
                     font_type, brdr, 0, 0, 
                     xdl_control_value_cp);

   return 0;
}

/* Fortran binding: xdlf_control_value */

void xdlf_control_value (ivh, ivhparent, ix, iy, iact, ihcen, ivcen,
                        ifont, ibrdr, label, llen, value, ivlen, maxvlen, 
                        msg, lenmsg, minw, minh, ierr)

int *label, *value, *msg;
int *ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *llen, *ivlen, *maxvlen, *lenmsg, *minw, *minh;
{
   *ierr = xdl_control_value (*ivh, *ivhparent, *ix, *iy, *iact, 
                             *ihcen, *ivcen,
                             *ifont, *ibrdr, XDL_pointers[*label], *llen, 
                             XDL_pointers[*value], *ivlen, *maxvlen,
                             XDL_pointers[*msg], *lenmsg, *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_control_value_rps   **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_value_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_value * gd;  /* Pointer to global data area */

   gd = (XDl_control_value *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->msg, 0);
   }
}
/*
***********************************
**     xdl_control_value_cp      **
***********************************

Purpose: Callback for panel choice item

Author:  John W. Campbell, Jan 1994

*/

void xdl_control_value_cp (iv, valstr, len, maxvlen)

int iv;
char * valstr;
int len;
int maxvlen;

{
   XDl_control_value *gd;          /* Pointer to global data area */

   gd = (XDl_control_value *) XDL_view_objects[iv]->gd;

   gd->valinp[0] = 0;
   strncat (gd->valinp, valstr, len);

   if (gd->on_off)
   {
       XDL_return_data_vh = XDL_view_objects[iv]->vh;
   }
   return;
}

/*
****************************************
**     xdl_control_value_on_off       **
****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_value_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_value *gd;        /* Pointer to global data area */

   gd = (XDl_control_value *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->msg, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}

/*
*****************************************
**     xdl_control_value_tidy_up       **
*****************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_value_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_value *gd;    /* Pointer to the global data area */

   gd = (XDl_control_value *) global;
   if (gd->label!=NULL) free(gd->label);
   if (gd->msg!=NULL) free(gd->msg);
   if (gd->value!=NULL) free(gd->value);
   if (gd->valinp!=NULL) free(gd->valinp);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set new value string - xdl_control_value_set
The routine xdl_control_value_set (xdlf_control_value_set) is
used to set a new value string for a control Panel Value view-object.
-end*/
/*
****************************************
**     xdl_control_value_set          **
****************************************

Purpose: Set new value string

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_VALUE_SET (IVH, XDLSTR(VALUE), IVLEN, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
VALUE     (R)  Address of new value string
               ** Pass address using the XDLSTR function **
IVLEN     (R)  Length of the value string (>0)
IERR      (W)  Returns status from xdl_control_value_set call
-end*/

/*-C:*/
int xdl_control_value_set (vh, value, vlen)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char    * value;          /* Value string (R)*/
int       vlen;           /* Value length, may be 0 if null terminated
                             string passed (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_value *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_value_set",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2104)
   {
      xdl_view_type_errmsg ("xdl_control_value_set", vh,
                            "xdl_control_value");
      return 2;
   }

   gd = (XDl_control_value *) XDL_view_objects[iv]->gd;

   /* Set the new value */

   xdl_panel_value_set (gd->val, value, vlen);

   return 0;
}

/* Fortran binding xdlf_control_value_set */

void xdlf_control_value_set (ivh, value, ivlen, ierr)
int *value;
int *ivh, *ivlen, *ierr;
{
   *ierr = xdl_control_value_set (*ivh, XDL_pointers[*value], *ivlen);
   return;
}

/*-Routine: Get input value string - xdl_control_value_getdata
The routine xdl_control_value_getdata (xdlf_control_value_getdata) is
used to return the input value string after a return from an
XDL_VIEW event handling routine such as xdl_get_events (xdlf_get_events)
indicates that there was input from the view-object in question.
-end*/
/*
****************************************
**     xdl_control_value_getdata      **
****************************************

Purpose: Get returned data from the control Panel Value view-object

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_VALUE_GETDATA (IVH, XDLSTR(VALSTR), MAXV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
VALSTR   (W)  Address of character string in which to place
              returned value string
               ** Pass address using the XDLSTR function **
MAXV     (W)  Max length of VALSTR (>0)
IERR     (W)  Returns status from xdl_control_value_getdata call
-end*/

/*-C:*/
int xdl_control_value_getdata (vh, valstr, maxv)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char * valstr;   /* String in which to return value (W)*/
int maxv;        /* if >0 Maximum length of returned string allowed 
                          (excluding terminating null - string must be 
                          at least max_len + 1 characters in length)
                    if <0 abs(max_len) characters will be returned padded 
                          with blanks if needed (for 'fortran' use) (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_value *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_value_getdata",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2104)
   {
      xdl_view_type_errmsg ("xdl_control_value_getdata", vh,
                            "xdl_control_value");
      return 2;
   }

   gd = (XDl_control_value *) XDL_view_objects[iv]->gd;

   /* Return the required data */

   xdl_copy_chars (gd->valinp, valstr, maxv);

   return 0;
}

/* Fortran binding xdlf_control_value_getdata */

void xdlf_control_value_getdata (ivh, valstr, maxv, ierr)

int *valstr;
int *ivh, *maxv, *ierr;
{
   int max_len;

   max_len = -(*maxv);
   if (max_len>0) max_len = -max_len;

   *ierr = xdl_control_value_getdata (*ivh, XDL_pointers[*valstr], max_len);
   return;
}
/*-Routine: Get required size - xdl_control_value_getzize
The routine xdl_control_value_getsize (xdlf_control_value_getsize) is
used to determine the required size for a control view-object containing
a Panel value item given various details about the requirements of the
item.
-end*/
/*
****************************************
**     xdl_control_value_getsize      **
****************************************

Purpose: Get size of value item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_VALUE_GETSIZE (IACT, IFONT, IBRDR,
        +                        LLEN,  MAXVLEN, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IACT      (R)   =1 include active strip, =0 do not
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
LLEN      (R)   Length of label ((>0)
MAXVLEN   (R)   Maximum required length of the value string (>0)
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_value_getsize (act_strip, font_type, brdr, llen, maxvlen,
                                 width, height)
/*end*/
 
/*-Parameters:*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
int       llen;           /* Label length (>0) (R)*/
int       maxvlen;        /* Maximum required value length (>0) (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_value_size (llen, maxvlen, font_type, brdr, width, height);

   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP;
      *height = *height + ACT_STRIP_HEIGHT;
   }
   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   return;
}
/* Fortran binding: xdlf_control_value_getsize */

void xdlf_control_value_getsize (iact, ifont, ibrdr, llen, maxvlen, 
                                  iwidth, iheight)

int *iact, *ifont, *ibrdr, *llen, *maxvlen, *iwidth, *iheight;
{
   xdl_control_value_getsize (*iact, *ifont, *ibrdr, *llen, *maxvlen, 
                               iwidth, iheight);
   return;
}

/*==================================================*/
/*         PANEL SLIDER                             */
/*==================================================*/

/*-Section: Panel Slider Item Routines
These are routines for creating an handling a view-object containing a
panel slider item.
-end*/

/*-Routine: View_object with a Panel Slider Item - xdl_control_slider
The routine xdl_control_slider (xdlf_control_slider) is used to create a
view object containing a Panel Slider item (See Panel Item Routines).
An active strip may be displayed if desired.
-end*/
/*
****************************
**    xdl_control_slider  **
****************************

Purpose: View-object with panel slider item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_SLIDER (IVH, IVHPARENT, IX, IY, IACT,
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          XDLSTR(LABEL), LLEN, 
        +                          ITRAVEL, ISTART,
        +                          XDLSTR(MSG), LENMSG,
        +                          MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IACT      (R)   Display active strip flag 1=yes, 0=no (see
                act_strip)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
LABEL     (R)   Address of label string for panel slider.
                ** Pass address using the XDLSTR function **
LLEN      (R)   Length of the label string (>0)
ITRAVEL   (R)   Travel length of slider (in pixels)
                (see travel_len)
ISTART    (R)   Start position of slider from 0 to ITRAVEL
                (see startpos)
MSG       (R)   Address of character string for active strip
                message. May be a blank string if IACT = 0.
                ** Pass address using XDLSTR function **
                (see msg)
LENMSG    (R)   Length of active strip message string (>0)
                (see len_msg)
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_slider call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object slider;              /* Slider item */
   int act_strip;                    /* Active strip present flag */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        *label;               /* Null terminated label string (copy) */
   char        *msg;                 /* Active strip message string (copy) */
   int         ipos;                 /* Last selected slider position */
   int         travel_len;           /* Travel length requested */
}  
   XDl_control_slider;

/* Define routine types */

int xdl_control_slider();
void xdl_control_slider_rps();
void xdl_control_slider_cp();
void xdl_control_slider_on_off();
void xdl_control_slider_tidy_up();
int xdl_control_slider_reset();
int xdl_control_slider_getdata();
void xdl_control_slider_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_slider xdlf_control_slider_
#define xdlf_control_slider_reset xdlf_control_slider_reset_
#define xdlf_control_slider_getdata xdlf_control_slider_getdata_
#define xdlf_control_slider_getsize xdlf_control_slider_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_control_slider XDLF_CONTROL_SLIDER
#define xdlf_control_slider_reset XDLF_CONTROL_SLIDER_RESET
#define xdlf_control_slider_getdata XDLF_CONTROL_SLIDER_GETDATA
#define xdlf_control_slider_getsize XDLF_CONTROL_SLIDER_GETSIZE

#endif

void xdlf_control_slider();
void xdlf_control_slider_reset();
void xdlf_control_slider_getdata();
void xdlf_control_slider_getsize();

/*-C:*/
int xdl_control_slider (vh, vh_parent, x, y, act_strip, hcen, vcen,
                        font_type, brdr, label, llen, 
                        travel_len, startpos,
                        msg, len_msg,
                        minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
char    * label;          /* Label string (R)*/
int       llen;           /* Label length, may be 0 if null terminated
                             string passed (R)*/
int       travel_len;     /* Slider travel length in pixels. The slider
                             positions will represent travel_len + 1
                             discrete values (R)*/
int       startpos;       /* Start position of slider from 0 to travel_len
                             (R)*/
char    * msg;            /* Message for active strip, may be null if
                             no active strip required (R)*/
int       len_msg;        /* Length of active strip message string; may
                             be 0 is a null terminated string given for
                             msg (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_slider * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int len;                    /* String length */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   int actstrip_width;         /* Active strip width */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   if (llen==0) llen = strlen(label);
   xdl_panel_slider_size (llen, font_type, travel_len, brdr, &w, &h);

   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) w = MIN_ACT_STRIP;
      h = h + ACT_STRIP_HEIGHT;
   }
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      if (act_strip)
      {
         y_pos = (height-ACT_STRIP_HEIGHT-h)/2;
      }
      else
      {
         y_pos = (height-h)/2;
      }
   }
   else
   {
      y_pos = BORDER_HEIGHT;
      if (act_strip) y_pos = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_slider", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_slider *) malloc (sizeof(XDl_control_slider));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2105, 
                        xdl_control_slider_on_off,
                        xdl_control_slider_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->act_strip = act_strip;
   gd->on_off = 0;
   gd->label = 0;
   gd->msg = 0;
   gd->travel_len = travel_len;
   gd->ipos = startpos;

   /* Make copy of active strip string */

   if (act_strip)
   {
      len = len_msg;
      if (len_msg==0) len = strlen(msg);
      p = malloc(len+1);
      if (p==0) {err |= 2; return err;}
      gd->msg = p;
      p[0] = 0;
      strncat (p,msg,len);
   }

   /* Make copy of label string */

   len = llen;
   if (llen==0) len = strlen(label);
   p = malloc(len+1);
   if (p==0) {err |= 2; return err;}
   gd->label = p;
   p[0] = 0;
   strncat (p,label,len);

   /* Create active strip window (if required) */

   if (act_strip)
   {
      actstrip_width = width - 2*BORDER_WIDTH - 2;
      gd->wact = xdl_create_active_strip (gd->wframe, vh, actstrip_width,
                                          xdl_control_slider_rps);
   }

   /* Create panel item */

   gd->slider = xdl_panel_slider (vh, gd->wframe, x_pos, y_pos, 
                     gd->label, font_type, travel_len, startpos, brdr, 0, 0, 
                     xdl_control_slider_cp);

   return 0;
}

/* Fortran binding: xdlf_control_slider */

void xdlf_control_slider (ivh, ivhparent, ix, iy, iact, ihcen, ivcen,
                        ifont, ibrdr, label, llen, itravel, istart,
                        msg, lenmsg, minw, minh, ierr)

int *label, *msg;
int *ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *llen, *itravel, *istart, *lenmsg, *minw, *minh;
{
   *ierr = xdl_control_slider (*ivh, *ivhparent, *ix, *iy, *iact, 
                             *ihcen, *ivcen,
                             *ifont, *ibrdr, XDL_pointers[*label], *llen, 
                             *itravel, *istart,
                             XDL_pointers[*msg], *lenmsg, *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
**********************************
**     xdl_control_slider_rps   **
**********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_slider_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_slider * gd;  /* Pointer to global data area */

   gd = (XDl_control_slider *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->msg, 0);
   }
}
/*
************************************
**     xdl_control_slider_cp      **
************************************

Purpose: Callback for panel slider item

Author:  John W. Campbell, Jan 1994

*/

void xdl_control_slider_cp (iv, curpos, travel_len)

int iv;
int curpos;
int travel_len;

{
   XDl_control_slider *gd;          /* Pointer to global data area */

   gd = (XDl_control_slider *) XDL_view_objects[iv]->gd;

   gd->ipos = curpos;

   if (gd->on_off)
   {
       XDL_return_data_vh = XDL_view_objects[iv]->vh;
   }
   return;
}

/*
*****************************************
**     xdl_control_slider_on_off       **
*****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_slider_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_slider *gd;        /* Pointer to global data area */

   gd = (XDl_control_slider *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->msg, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}

/*
******************************************
**     xdl_control_slider_tidy_up       **
******************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_slider_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_slider *gd;    /* Pointer to the global data area */

   gd = (XDl_control_slider *) global;
   if (gd->label!=NULL) free(gd->label);
   if (gd->msg!=NULL) free(gd->msg);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set new slider position - xdl_control_slider_reset
The routine xdl_control_slider_reset (xdlf_control_slider_reset) is
used to set a new slider position for a control Panel Slider view-object.
-end*/
/*
*****************************************
**     xdl_control_slider_reset        **
*****************************************

Purpose: Set new slider position

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_SLIDER_RESET (IVH, ISLIDE, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
ISLIDE    (R)  New slider position (0 to ITRAVEL)
IERR      (W)  Returns status from xdl_control_slider_reset call
-end*/

/*-C:*/
int xdl_control_slider_reset (vh, islide)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
int islide;               /* new slider position (0 to travel_len) (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_slider *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_slider_reset",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2105)
   {
      xdl_view_type_errmsg ("xdl_control_slider_reset", vh,
                            "xdl_control_slider");
      return 2;
   }

   gd = (XDl_control_slider *) XDL_view_objects[iv]->gd;

   /* Set the new value */

   xdl_panel_slider_reset (gd->slider, islide);

   return 0;
}

/* Fortran binding xdlf_control_slider_reset */

void xdlf_control_slider_reset (ivh, islide, ierr)
int *ivh, *islide, *ierr;
{
   *ierr = xdl_control_slider_reset (*ivh, *islide);
   return;
}

/*-Routine: Get input slider position - xdl_control_slider_getdata
The routine xdl_control_slider_getdata (xdlf_control_slider_getdata) is
used to return the input slider position after a return from an
XDL_VIEW event handling routine such as xdl_get_events (xdlf_get_events)
indicates that there was input from the view-object in question.
-end*/
/*
*****************************************
**     xdl_control_slider_getdata      **
*****************************************

Purpose: Get returned data from the control Panel Slider view-object

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_SLIDER_GETDATA (IVH, IPOS, ITRAVEL, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IPOS     (W)  The returned slider position (0 to ITRAVEL)
ITRAVEL  (W)  The travel length for reference
IERR     (W)  Returns status from xdl_control_slider_getdata call
-end*/

/*-C:*/
int xdl_control_slider_getdata (vh, ipos, travel_len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *ipos;       /* returns the current slider position (W)*/
int *travel_len; /* Returns the travel length for reference (W)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_slider *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_slider_getdata",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2105)
   {
      xdl_view_type_errmsg ("xdl_control_slider_getdata", vh,
                            "xdl_control_slider");
      return 2;
   }

   gd = (XDl_control_slider *) XDL_view_objects[iv]->gd;

   /* Return the required data */

   *ipos = gd->ipos;
   *travel_len = gd->travel_len;
   return 0;
}

/* Fortran binding xdlf_control_slider_getdata */

void xdlf_control_slider_getdata (ivh, ipos, itravel, ierr)

int *ivh, *ipos, *itravel, *ierr;
{

   *ierr = xdl_control_slider_getdata (*ivh, ipos, itravel);
   return;
}
/*-Routine: Get required size - xdl_control_slider_getzize
The routine xdl_control_slider_getsize (xdlf_control_slider_getsize) is
used to determine the required size for a control view-object containing
a Panel Slider item given various details about the requirements of the
item.
-end*/
/*
*****************************************
**     xdl_control_slider_getsize      **
*****************************************

Purpose: Get size of slider item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_SLIDER_GETSIZE (IACT, IFONT, IBRDR,
        +                        LLEN,  ITRAVEL, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IACT      (R)   =1 include active strip, =0 do not
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
LLEN      (R)   Length of label ((>0)
ITRAVEL   (R)   Travel length
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_slider_getsize (act_strip, font_type, brdr, llen, 
                                 travel_len, width, height)
/*end*/
 
/*-Parameters:*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
int       llen;           /* Label length (>0) (R)*/
int       travel_len;     /* Travel length (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_slider_size (llen, font_type, travel_len, brdr, width, height);

   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP;
      *height = *height + ACT_STRIP_HEIGHT;
   }
   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   return;
}
/* Fortran binding: xdlf_control_slider_getsize */

void xdlf_control_slider_getsize (iact, ifont, ibrdr, llen, itravel,
                                  iwidth, iheight)

int *iact, *ifont, *ibrdr, *llen, *itravel, *iwidth, *iheight;
{
   xdl_control_slider_getsize (*iact, *ifont, *ibrdr, *llen, *itravel, 
                               iwidth, iheight);
   return;
}

/*==================================================*/
/*         PANEL I/O                                */
/*==================================================*/

/*-Section: Panel I/O Item Routines
These are routines for creating an handling a view-object containing a
panel input/output item.
-end*/

/*-Routine: View_object with a Panel I/O Item - xdl_control_io
The routine xdl_control_io (xdlf_control_io) is used to create a
view object containing a Panel I/O item (See Panel Item Routines).
An active strip may be displayed if desired.
-end*/
/*
***************************
**    xdl_control_io     **
***************************

Purpose: View-object with panel I/O item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO (IVH, IVHPARENT, IX, IY, IACT,
        +                          IHCEN, IVCEN, IFONT, IBRDR,
        +                          MAX_PR, MIN_RP, MAX_RP, ICH,
        +                          XDLSTR(MSG), LENMSG,
        +                          MINW, MINH,IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
IACT      (R)   Display active strip flag 1=yes, 0=no (see
                act_strip)
IHCEN     (R)   =1 Centre panel item horizontally, = 0 do
                not (left justify instead) (see hcen)
IVCEN     (R)   =1 Centre panel item vertically, = 0 do
                not (top justify instead) (see vcen)
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
MAX_PR    (R)   Maximum length for prompt field
MIN_RP    (R)   Minimum length for reply field
MAX_RP    (R)   Maximum length for reply
ICH       (R)   = 1 show character count, = 0 do not
MSG       (R)   Address of character string for active strip
                message. May be a blank string if IACT = 0.
                ** Pass address using XDLSTR function **
                (see msg)
LENMSG    (R)   Length of active strip message string (>0)
                (see len_msg)
MINW      (R)   Minimum width required (may be 0) 
MINH      (R)   Minimum height required (may be 0)
IERR      (W)   Returns status from xdl_control_io call
-end*/


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window */
   Panel_object io;                  /* I/O item */
   int act_strip;                    /* Active strip present flag */
   int         on_off;               /* Program awaiting input =1, not =0 */
   char        *msg;                 /* Active strip message string (copy) */
   char        *str;                 /* String to hold last input */
   int         iret;                 /* Return flag with input string */
   int         iflag;                /* Flag identifying prompt (user 
                                        supplied)*/
}  
   XDl_control_io;

/* Define routine types */

int xdl_control_io();
void xdl_control_io_rps();
void xdl_control_io_cp();
void xdl_control_io_on_off();
void xdl_control_io_tidy_up();
int xdl_control_io_prompt();
int xdl_control_io_clear();
int xdl_control_io_kbfocus();
int xdl_control_io_getdata();
void xdl_control_io_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_control_io xdlf_control_io_
#define xdlf_control_io_prompt xdlf_control_io_prompt_
#define xdlf_control_io_clear xdlf_control_io_clear_
#define xdlf_control_io_kbfocus xdlf_control_io_kbfocus_
#define xdlf_control_io_getdata xdlf_control_io_getdata_
#define xdlf_control_io_getsize xdlf_control_io_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_control_io XDLF_CONTROL_IO
#define xdlf_control_io_prompt XDLF_CONTROL_IO_PROMPT
#define xdlf_control_io_clear XDLF_CONTROL_IO_CLEAR
#define xdlf_control_io_kbfocus XDLF_CONTROL_IO_KBFOCUS
#define xdlf_control_io_getdata XDLF_CONTROL_IO_GETDATA
#define xdlf_control_io_getsize XDLF_CONTROL_IO_GETSIZE

#endif

void xdlf_control_io();
void xdlf_control_io_prompt();
void xdlf_control_io_clear();
void xdlf_control_io_kbfocus();
void xdlf_control_io_getdata();
void xdlf_control_io_getsize();

/*-C:*/
int xdl_control_io (vh, vh_parent, x, y, act_strip, hcen, vcen,
                        font_type, brdr, 
                        max_pr, min_rp, max_rp, ch_count,
                        msg, len_msg,
                        minw, minh)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       hcen;           /* =1 Centre panel item horizontally, = 0 do
                             not (left justify instead) (R)*/
int       vcen;           /* =1 Centre panel item vertically, = 0 do
                             not (top justify instead) (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
int       max_pr;         /* Maximum length for prompt field (R)*/
int       min_rp;         /* Minimum length for reply field (R)*/
int       max_rp;         /* Maximum length for reply (R)*/
int       ch_count;       /* = 1 show character count, = 0 do not (R)*/
char    * msg;            /* Message for active strip, may be null if
                             no active strip required (R)*/
int       len_msg;        /* Length of active strip message string; may
                             be 0 is a null terminated string given for
                             msg (R)*/
int minw;                 /* Minimum width required (may be 0) (R)*/
int minh;                 /* Minimum height required (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Cannot allocate memory required
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_control_io * gd;    /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int w;                      /* Width of panel item */
   int h;                      /* Height of panel item */
   int len;                    /* String length */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int x_pos;                  /* x position of panel item within 
                                  the view-object*/
   int y_pos;                  /* y position of panel item within
                                  the view-object */
   int actstrip_width;         /* Active strip width */
   char *p;                    /* Temporary pointer to character string */

   err = 0;

   /* Get size requirements */

   xdl_panel_io_size (max_pr, min_rp, max_rp, font_type, brdr, ch_count, 
                      &w, &h);

   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) w = MIN_ACT_STRIP;
      h = h + ACT_STRIP_HEIGHT;
   }
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (w<MIN_ACT_STRIP) width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   if (minw>width) width = minw;
   if (minh>height) height = minh;
   if (hcen)
   {
      x_pos = (width-w)/2;
   }
   else
   {
      x_pos = BORDER_WIDTH;
   }
   if (vcen)
   {
      if (act_strip)
      {
         y_pos = (height-ACT_STRIP_HEIGHT-h)/2;
      }
      else
      {
         y_pos = (height-h)/2;
      }
   }
   else
   {
      y_pos = BORDER_HEIGHT;
      if (act_strip) y_pos = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, 0);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_control_io", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_control_io *) malloc (sizeof(XDl_control_io));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2106, 
                        xdl_control_io_on_off,
                        xdl_control_io_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->act_strip = act_strip;
   gd->on_off = 0;
   gd->msg = 0;
   gd->str = 0;
   gd->iret = 0;

   /* Make copy of active strip string */

   if (act_strip)
   {
      len = len_msg;
      if (len_msg==0) len = strlen(msg);
      p = malloc(len+1);
      if (p==0) {err |= 2; return err;}
      gd->msg = p;
      p[0] = 0;
      strncat (p,msg,len);
   }

   /* String for input values */

   p = malloc(max_rp+1);
   if (p==0) {err |= 2; return err;}
   gd->str = p;

   /* Create active strip window (if required) */

   if (act_strip)
   {
      actstrip_width = width - 2*BORDER_WIDTH - 2;
      gd->wact = xdl_create_active_strip (gd->wframe, vh, actstrip_width,
                                          xdl_control_io_rps);
   }

   /* Create panel item */

   gd->io = xdl_panel_io (vh, gd->wframe, x_pos, y_pos, 
                     max_pr, min_rp, max_rp,
                     font_type, brdr, ch_count, 0, 0, 
                     xdl_control_io_cp);

   return 0;
}

/* Fortran binding: xdlf_control_io */

void xdlf_control_io (ivh, ivhparent, ix, iy, iact, ihcen, ivcen,
                        ifont, ibrdr, max_pr, min_rp, max_rp, ich,
                        msg, lenmsg, minw, minh, ierr)

int *msg;
int *ivh, *ivhparent, *ix, *iy, *iact, *ihcen, *ivcen, *ifont, *ierr;
int *ibrdr, *max_pr, *min_rp, *max_rp, *ich, *lenmsg, *minw, *minh;
{
   *ierr = xdl_control_io (*ivh, *ivhparent, *ix, *iy, *iact, 
                             *ihcen, *ivcen,
                             *ifont, *ibrdr, *max_pr, *min_rp, *max_rp, *ich,
                             XDL_pointers[*msg], *lenmsg, *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
**********************************
**     xdl_control_io_rps       **
**********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, January 1994

*/

void xdl_control_io_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_control_io * gd;  /* Pointer to global data area */

   gd = (XDl_control_io *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->msg, 0);
   }
}
/*
***********************************
**     xdl_control_io_cp         **
***********************************

Purpose: Callback for panel I/O item

Author:  John W. Campbell, Jan 1994

*/

void xdl_control_io_cp (iv, reply, len, max_rp, flag, iret)

int iv;
char * reply;
int len;
int max_rp;
int flag;
int iret;

{
   XDl_control_io *gd;          /* Pointer to global data area */

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;

   gd->str[0] = 0;
   strncat (gd->str, reply, len);
   gd->iret = iret;
   gd->iflag = flag;

   if (gd->on_off)
   {
       XDL_return_data_vh = XDL_view_objects[iv]->vh;
   }
   return;
}

/*
****************************************
**     xdl_control_io_on_off          **
****************************************

Purpose: Routine called when switching application input on/off

Author:  John Campbell, January 1994

*/

void xdl_control_io_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_control_io *gd;        /* Pointer to global data area */

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->msg, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}

/*
*****************************************
**     xdl_control_io_tidy_up          **
*****************************************

Purpose: Routine called to tidy up before view-object is deleted

Author: John W. Campbell, January 1994

*/

void xdl_control_io_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_control_io *gd;    /* Pointer to the global data area */

   gd = (XDl_control_io *) global;
   if (gd->msg!=NULL) free(gd->msg);
   if (gd->str!=NULL) free(gd->str);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set new prompt string - xdl_control_io_prompt
The routine xdl_control_io_prompt (xdlf_control_io_prompt) is
used to set a new prompt string for a control Panel I/O view-object.
-end*/
/*
****************************************
**     xdl_control_io_prompt          **
****************************************

Purpose: Set new prompt string

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO_PROMPT (IVH, XDLSTR(PROMPT), IPLEN,
        +                             IFLAG, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
PROMPT    (R)  Address of new prompt string
               ** Pass address using the XDLSTR function **
IPLEN     (R)  Length of the prompt string (>0)
IFLAG     (R)  Flag to be passed to callback identifying
               prompt (if required)
IERR      (W)  Returns status from xdl_control_io_prompt call
-end*/

/*-C:*/
int xdl_control_io_prompt (vh, prompt, plen, iflag)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char    * prompt;         /* Prompt string (R)*/
int       plen;           /* Prompt length, may be 0 if null terminated
                             string passed (R)*/
int       iflag;          /* User selected flag to identify flag (will
                             be passed to callback (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_io *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_io_prompt",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2106)
   {
      xdl_view_type_errmsg ("xdl_control_io_prompt", vh,
                            "xdl_control_io");
      return 2;
   }

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;

   /* Set the new value */

   xdl_panel_io_prompt (gd->io, prompt, plen, iflag);

   return 0;
}

/* Fortran binding xdlf_control_io_prompt */

void xdlf_control_io_prompt (ivh, prompt, iplen, iflag, ierr)
int *prompt;
int *ivh, *iplen, *iflag, *ierr;
{
   *ierr = xdl_control_io_prompt (*ivh, XDL_pointers[*prompt], *iplen, *iflag);
   return;
}
/*-Routine: Clear I/O item - xdl_control_io_clear
The routine xdl_control_io_clear (xdlf_control_io_clear) is
used to clear a control Panel I/O view-object. It will remain
inactive till another prompt is output via xdl_control_io_prompt
(xdlf_control_io_prompt).
-end*/
/*
****************************************
**     xdl_control_io_clear           **
****************************************

Purpose: Clear I/O item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO_CLEAR (IVH, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
IERR      (W)  Returns status from xdl_control_io_clear call
-end*/

/*-C:*/
int xdl_control_io_clear (vh)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_io *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_io_clear",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2106)
   {
      xdl_view_type_errmsg ("xdl_control_io_clear", vh,
                            "xdl_control_io");
      return 2;
   }

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;

   /* Clear the panel i/o item */

   xdl_panel_io_clear (gd->io);

   return 0;
}

/* Fortran binding xdlf_control_io_clear */

void xdlf_control_io_clear (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_control_io_clear (*ivh);
   return;
}
/*-Routine: Set keyboard focus for I/O item - xdl_control_io_kbfocus
The routine xdl_control_io_kbfocus (xdlf_control_io_kbfocus) is
used to set the keyboard focus for a control Panel I/O view-object. 
-end*/
/*
****************************************
**     xdl_control_io_kbfocus         **
****************************************

Purpose: Set keyboard focus for i/o item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO_KBFOCUS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
IERR      (W)  Returns status from xdl_control_io_kbfocus call
-end*/

/*-C:*/
int xdl_control_io_kbfocus (vh)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_io *gd;        /* pointer to the global data area */
   Window wid;                /* Window id */ 
   XWindowAttributes attrib;  /* Window attributes structure */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_io_kbfocus",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2106)
   {
      xdl_view_type_errmsg ("xdl_control_io_kbfocus", vh,
                            "xdl_control_io");
      return 2;
   }

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;

   /* Set the keyboard focus */

   xdl_panel_io_wid (gd->io, &wid);
   XGetWindowAttributes (XDL_display, wid, &attrib);
   if (attrib.map_state==IsViewable)
   {
      XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
   }
   return 0;
}

/* Fortran binding xdlf_control_io_kbfocus */

void xdlf_control_io_kbfocus (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_control_io_kbfocus (*ivh);
   return;
}

/*-Routine: Get input string - xdl_control_io_getdata
The routine xdl_control_io_getdata (xdlf_control_io_getdata) is
used to return the input string after a return from an
XDL_VIEW event handling routine such as xdl_get_events (xdlf_get_events)
indicates that there was input from the view-object in question.
-end*/
/*
****************************************
**     xdl_control_io_getdata         **
****************************************

Purpose: Get returned data from the control Panel I/O view-object

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO_GETDATA (IVH, REPLY, MAXR, IRET,
        +                              IFLAG, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
REPLY    (W)  Address of character string in which to place
              returned input string
               ** Pass address using the XDLSTR function **
MAXR     (W)  Max length of REPLY (>0)
IRET     (W)  Returns input status flag = 1 non-blank string
                                        = 0 blank string
                                        =-1 escape pressed
IFLAG    (W)  User flag defined when setting prompt
IERR     (W)  Returns status from xdl_control_io_getdata call
-end*/

/*-C:*/
int xdl_control_io_getdata (vh, reply, maxr, iret, iflag)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char * reply;    /* String in which to return value (W)*/
int maxr;        /* if >0 Maximum length of returned string allowed 
                          (excluding terminating null - string must be 
                          at least max_len + 1 characters in length)
                    if <0 abs(max_len) characters will be returned padded 
                          with blanks if needed (for 'fortran' use) (R)*/
int * iret;      /* Returns input status flag = 1 non-blank string
                                              = 0 blank string
                                              =-1 escape pressed (W)*/
int *iflag;      /* User flag defined when setting prompt (W)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                    /* Offset in XDL_view_objects array */
   XDl_control_io *gd;        /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_control_io_getdata",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2106)
   {
      xdl_view_type_errmsg ("xdl_control_io_getdata", vh,
                            "xdl_control_io");
      return 2;
   }

   gd = (XDl_control_io *) XDL_view_objects[iv]->gd;

   /* Return the required data */

   xdl_copy_chars (gd->str, reply, maxr);
   *iret = gd->iret;
   *iflag = gd->iflag;

   return 0;
}

/* Fortran binding xdlf_control_io_getdata */

void xdlf_control_io_getdata (ivh, reply, maxr, iret, iflag, ierr)

int *reply;
int *ivh, *maxr, *iret, *iflag, *ierr;
{
   int max_len;

   max_len = -(*maxr);
   if (max_len>0) max_len = -max_len;

   *ierr = xdl_control_io_getdata (*ivh, XDL_pointers[*reply], max_len,
                                   iret, iflag);
   return;
}
/*-Routine: Get required size - xdl_control_io_getzize
The routine xdl_control_io_getsize (xdlf_control_io_getsize) is
used to determine the required size for a control view-object containing
a Panel I/O item given various details about the requirements of the
item.
-end*/
/*
****************************************
**     xdl_control_io_getsize         **
****************************************

Purpose: Get size of i/o item

Author:  John W. Campbell, January 1994

*/
/*-Fortran:
         CALL XDLF_CONTROL_IO_GETSIZE (IACT, IFONT, IBRDR,
        +                        MAX_PR, MIN_RP, MAX_RP, ICH, 
        +                        IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IACT      (R)   =1 include active strip, =0 do not
IFONT     (R)   Font type (1-5) very small to very large (see
                font_type)
IBRDR     (R)   = 1 draw border, = 0 do not
MAX_PR    (R)   Maximum length for prompt field
MIN_RP    (R)   Minimum length for reply field
MAX_RP    (R)   Maximum length for reply
ICH       (R)   = 1 show character count, = 0 do not
IWIDTH    (W)   Width required for view-object
IHEIGHT   (W)   Height required for view-object
-end*/

/*-C:*/
void xdl_control_io_getsize (act_strip, font_type, brdr, max_pr, min_rp,
                             max_rp, ch_count, width, height)
/*end*/
 
/*-Parameters:*/
int       act_strip;      /* Display active strip flag 1=yes, 0=no (R)*/
int       font_type;      /* Font type (1-5) very small to very large (R)*/
int       brdr;           /* = 1 draw border, = 0 do not (R)*/
int       max_pr;         /* Maximum length for prompt field (R)*/
int       min_rp;         /* Minimum length for reply field (R)*/
int       max_rp;         /* Maximum length for reply (R)*/
int       ch_count;       /* = 1 show character count, = 0 do not (R)*/
int     * width;          /* Returns the required width in pixels (W)*/
int     * height;         /* Returns the required height in pixels (W)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   xdl_panel_io_size (max_pr, min_rp, max_rp, font_type, brdr, 
                      ch_count, width, height);

   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP;
      *height = *height + ACT_STRIP_HEIGHT;
   }
   *width = *width + 2*BORDER_WIDTH + 2;
   *height = *height + 2*BORDER_HEIGHT + 2;
   if (act_strip)
   {
      if (*width<MIN_ACT_STRIP) *width = MIN_ACT_STRIP + 2*BORDER_WIDTH + 2;
   }
   return;
}
/* Fortran binding: xdlf_control_io_getsize */

void xdlf_control_io_getsize (iact, ifont, ibrdr, max_pr, min_rp, max_rp,
                              ich, iwidth, iheight)

int *iact, *ifont, *ibrdr, *max_pr, *min_rp, *max_rp, *ich, *iwidth, *iheight;
{
   xdl_control_io_getsize (*iact, *ifont, *ibrdr, *max_pr, *min_rp,
                           *max_rp, *ich, iwidth, iheight);
   return;
}
