/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: Image File Reading Routines
-end*/
 
/*-Intro:
These are a set of routines for reading film image and image plate (i2)
files (without header information) or MAR image plate files. The routines 
are written in C. Fortran interfaces are provided. They
were written, in particular, for use with programs using XDL_VIEW routines.
-end*/

/*-Section_order:
1 2 4 3
-end*/ 

/*Routines List:
The following film file handling routines are available:

   xdl_open_film_file          Open a film image file
   xdl_read_full_film          Read a full film image
   xdl_close_film_file         Close a film image file
   xdl_open_i2                 Open an image plate (i2) file
   xdl_read_full_i2            Read a full image-plate (i2) image
   xdl_close_i2                Close an image plate (i2) file
   xdl_read_mar                Read a full MAR image-plate image
   xdl_rdsquash_mar            Read and squash a full MAR image
   xdl_read_pfbyte             Read Photon Factory log byte image
   xdl_axord                   Interpret string defining axis order
*/

#include "xdl_view_ext.h"
#ifdef VMS
#include <file.h>
#else
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>
#endif
#include <math.h>

/* Define macro for squashing i2 data */

#define sqsh(i) i>32767?65536-(i>>3):i

/* Define nearest integer macro */

#define nint(x)  x>=0.0?(int)(x+0.5):(int)(x-0.5)

/* Define routine types */
int xdl_open_film_file();
int xdl_read_full_film();
int xdl_close_film_file();
int xdl_open_i2();
int xdl_read_full_i2();
int xdl_close_i2();
int xdl_read_mar();
int xdl_rdsquash_mar();
int xdl_read_pfbyte();
int xdl_axord();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_open_film_file xdlf_open_film_file_
#define xdlf_read_full_film xdlf_read_full_film_
#define xdlf_close_film_file xdlf_close_film_file_
#define xdlf_open_i2 xdlf_open_i2_
#define xdlf_read_full_i2 xdlf_read_full_i2_
#define xdlf_close_i2 xdlf_close_i2_
#define xdlf_read_mar xdlf_read_mar_
#define xdlf_rdsquash_mar xdlf_rdsquash_mar_
#define xdlf_read_pfbyte xdlf_read_pfbyte_
#define xdlf_axord xdlf_axord_

#endif

#if  LINKTYP == 2

#define xdlf_open_film_file XDLF_OPEN_FILM_FILE
#define xdlf_read_full_film XDLF_READ_FULL_FILM
#define xdlf_close_film_file XDLF_CLOSE_FILM_FILE
#define xdlf_open_i2 XDLF_OPEN_I2
#define xdlf_read_full_i2 XDLF_READ_FULL_I2
#define xdlf_close_i2 XDLF_CLOSE_I2
#define xdlf_read_mar XDLF_READ_MAR
#define xdlf_rdsquash_mar XDLF_RDSQUASH_MAR
#define xdlf_read_pfbyte XDLF_READ_PFBYTE
#define xdlf_axord XDLF_AXORD

#endif

void xdlf_open_film_file();
void xdlf_read_full_film();
void xdlf_close_film_file();
void xdlf_open_i2();
void xdlf_read_full_i2();
void xdlf_close_i2();
void xdlf_read_mar();
void xdlf_rdsquash_mar();
void xdlf_read_pfbyte();
void xdlf_axord();
 
/*-Section: Film Image File Routines
These are routines for reading a film image binary file without header
information and with one unsigned byte per pixel.
-end*/

/*-Routine: Open a film image file - xdl_open_film_file
The routine xdl_open_film_file (xdlf_open_film_file) is used to open a 
film image file and return the file descriptor. 
-end*/
/*
*****************************
**   xdl_open_film_file    **
*****************************

Purpose: Open a film image file and return the file descriptor. 

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_OPEN_FILM_FILE (XDLSTR(FILNAM), LENF, IFD, IERR)
-end*/
 
/*-Parameters:
FILNAM      (R)   Character string holding the filename 
                  (see filename)
                  ** Pass address using the XDLSTR routine **
LENF        (R)   Length of the filename string (see lenf)
IFD         (W)   Returns the file descriptor (see fd)
IERR        (W)   Returns the status from the xdl_open_film_file 
                  call
-end*/

/*-C:*/
int xdl_open_film_file (filename, lenf, fd)
/*end*/
 
/*-Parameters:*/
char  * filename;       /* The name of the film image file (R)*/
int     lenf;           /* Length of the file name. If the filename is a
                           null terminated string then the length will be
                           determined by the routine if lenf is 0 (R)*/
int   * fd;             /* Returns the file descriptor (W)*/

/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to open the file;
-end*/
{

   int len;
   char *fnam;

   /* Copy file name string in case not null terminated */
   len = lenf;
   if (len==0) len = strlen (filename);
   fnam = (char *) malloc (len+1);
   strncpy (fnam, filename, len);
   fnam[len] = 0;

   /* Open the file */
   *fd = open (fnam, O_RDONLY, 0);
   free (fnam);
   if (*fd <0) return 1;

   return 0;
}

/* Fortran binding: xdlf_open_film_file */

void xdlf_open_film_file (filnam, lenf, ifd, ierr)
int *filnam, *lenf, *ifd, *ierr;
{
   *ierr = xdl_open_film_file (XDL_pointers[*filnam], *lenf, ifd);
   return;
}
/*-Routine: Read a full film image file - xdl_read_full_film
The routine xdl_read_full_film (xdlf_read_full_film) is used to read a complete
film image file and return the image data in an array. 
The film file may have its data in any of the eight possible orders (the
default is assumed to  have yf moving fastest and 
xf slowest) and is in unsigned byte format without any header. The number 
of x rasters and the number of y rasters must be supplied.
-end*/
/*
*****************************
**   xdl_read_full_film    **
*****************************

Purpose: Read a film image file and return the image data in an array. 
         The film_file may have its data in any of the eight possible
         orders (the default is assumed to  have yf moving fastest and xf 
         slowest) and is in unsigned byte format without any header. The 
         number of x rasters and number of y rasters must be supplied.

Author:  John Campbell, Sept 1991

*/
/*-Fortran:
         CALL XDLF_READ_FULL_FILM (IFD, IORD, NXRASTS, NYRASTS,
        +                         IVH_BAR, IXROOT, IYROOT, IMG_FULL,
        +                         IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film image file (1 to 8)
               (see order)
NXRASTS  (R)   The number of x rasters (see nxrasts)
NYRASTS  (R)   The number of y rasters (see nyrasts)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG_FULL (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img_full)
IERR     (W)   Returns the status from the xdl_read_full_film call
-end*/

/*-C:*/
int xdl_read_full_film (fd, order,nxrasts, nyrasts, vh_bar, 
                        xroot, yroot, img_full)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8.
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int nxrasts;      /* The number of x rasters (R)*/
int nyrasts;      /* The number of y rasters (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned char img_full[];
                  /* Array returning the read image data; 
                     The slower moving index in
                     the returned data is x going from low x to
                     high x and the faster moving index is y going 
                     from low y to high y. (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred; -1 unable
         to allocate temporary buffer space.
-end*/
{
   int i, j, k;
   int irec;
   int step;
   int nrecs, nrasts;
   unsigned char *buf;
   int ir, ix;

   /* Initialisations */

   if(order<1||order>8) order=1;
   nrecs = nxrasts;
   nrasts = nyrasts;
   if (order>4)
   {
      nrecs = nyrasts;
      nrasts = nxrasts;
   }

   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nrecs, 2, 5);
   
   /* Read data */

   lseek (fd, 0L, 0);
   buf = (unsigned char *) malloc(nrasts);
   if (buf==0) return -1;
   irec = 1;
   for (ir=0;ir<nrecs;++ir)
   {
      i = read (fd, buf, nrasts);
      if (i!=nrasts) goto tidy;
      switch (order)
      {
         case 1:
            j = ir*nrasts;
            step = 1;
            break;
         case 2:
            j = (ir+1)*nrasts - 1;
            step = -1;
            break;
         case 3:
            j = (nrecs-ir-1)*nrasts;
            step = 1;
            break;
         case 4:
            j = (nrecs-ir)*nrasts - 1;
            step = -1;
            break;
         case 5:
            j = ir;
            step = nrecs;
            break;
         case 6:
            j = nrecs - 1 - ir;
            step = nrecs;
            break;
         case 7:
            j = (nrasts-1)*nrecs + ir;
            step = -nrecs;
            break;
         case 8:
            j = nrasts*nrecs - 1 - ir;
            step = -nrecs;
            break;
      }
      for (k=0;k<nrasts;++k)
      {
          img_full[j] = buf[k];
          j += step;
      }
      if (vh_bar>0) xdl_progress_bar_value (vh_bar, ir+1);
      irec++;
   }
   irec = 0;   
   
   /* Tidy up */

   tidy:
   free (buf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_full_film */

void xdlf_read_full_film (ifd, iord, nxrasts, nyrasts,ivh_bar, ixroot, iyroot,
                          img_full, ierr)
int *ifd, *iord, *nxrasts, *nyrasts;
int *ivh_bar, *ixroot, *iyroot, *img_full, *ierr;
{
   *ierr = xdl_read_full_film (*ifd, *iord, *nxrasts, *nyrasts,
                               *ivh_bar, *ixroot, *iyroot, 
                               (unsigned char *) img_full);
   return;
}
/*-Routine: Close a film image file - xdl_close_film_file
The routine xdl_close_film_file (xdlf_close_film_file) is used to close a
film image file.
-end*/
/*
******************************
**   xdl_close_film_file    **
******************************

Purpose: Close a film image file 

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_CLOSE_FILM_FILE (IFD, IERR)
-end*/
 
/*-Parameters:
IFD         (R)   The file descriptor (see fd)
IERR        (W)   Returns the status from the xdl_close_film_file 
                  call
-end*/

/*-C:*/
int xdl_close_film_file (fd)
/*end*/
 
/*-Parameters:*/
int   fd;             /* The file descriptor (R)*/

/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to close the file;
-end*/
{
   int ierr;   /* Error flag */

   /* Close the file */
   ierr = close (fd);
   if (ierr!=0) return 1;
   return 0;
}

/* Fortran binding: xdlf_close_film_file */

void xdlf_close_film_file (ifd, ierr)
int *ifd, *ierr;
{
   *ierr = xdl_close_film_file (*ifd);
   return;
}
 
/*-Section: Image-Plate Image File Routines
These are routines for reading a image-plate image binary file without header
information and with one unsigned two-byte integer per pixel.
-end*/

/*-Routine: Open an image plate (i2) file - xdl_open_i2
The routine xdl_open_i2 (xdlf_open_i2) is used to open an 'i2' unsigned 16 
bit integer format image file and return the file descriptor. It can
in fact be used with any format (image) file.
-end*/
/*
***********************
**   xdl_open_i2     **
***********************

Purpose: Open a 'i2' unsigned 16 bit integer format image file and return the
         file descriptor.

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_OPEN_I2 (XDLSTR(FILNAM), LENF, IFD, IERR)
-end*/
 
/*-Parameters:
FILNAM      (R)   Character string holding the filename 
                  (see filename)
                  ** Pass address using the XDLSTR routine **
LENF        (R)   Length of the filename string (see lenf)
IFD         (W)   Returns the file descriptor (see fd)
IERR        (W)   Returns the status from the xdl_open_i2 call
-end*/

/*-C:*/
int xdl_open_i2 (filename, lenf, fd)
/*end*/
 
/*-Parameters:*/
char  * filename;       /* The name of the 'i2' format image file (R)*/
int     lenf;           /* Length of the file name. If the filename is a
                           null terminated string then the length will be
                           determined by the routine if lenf is 0 (R)*/

int   * fd;             /* Returns the file descriptor (W)*/

/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to open the file,
         =2 error in reading the file header
-end*/
{
   int len;
   char *fnam;

   /* Copy file name string in case not null terminated */
   len = lenf;
   if (len==0) len = strlen (filename);
   fnam = (char *) malloc (len+1);
   strncpy (fnam, filename, len);
   fnam[len] = 0;


   *fd = open (fnam, O_RDONLY, 0);
   free (fnam);
   if (*fd <0) return 1;

   return 0;
}

/* Fortran binding: xdlf_open_i2 */

void xdlf_open_i2 (filnam, lenf, ifd, ierr)
int *filnam, *lenf, *ifd, *ierr;
{
   *ierr = xdl_open_i2 (XDL_pointers[*filnam], *lenf, ifd);
   return;
}
/*-Routine: Read a full image-plate image file - xdl_read_full_i2
The routine xdl_read_full_i2 (xdlf_read_full_i2) is used to read a complete
image-plate image file and return the image data in an array. 
The image-plate file may have its data in any of the eight possible orders
and may be byte swapped if necessary (the default is assumed to  have yf 
moving fastest and xf slowest) and is in unsigned two byte integer format 
without any header.  The number of x rasters and the number of y rasters must 
be supplied.
-end*/
/*
*****************************
**   xdl_read_full_i2      **
*****************************

Purpose: Read an image-plate 'i2' image file and return the image data in 
         an array. The file may have its data in any of the eight
         possible orders and may be byte swapped if needed (the default
         is assumed to  have yf moving fastest and xf 
         slowest) and is in unsigned two byte integer format without any 
         header. The number of x rasters and number of y rasters must be 
         supplied.

Author:  John Campbell, Sept 1991

*/
/*-Fortran:
         CALL XDLF_READ_FULL_I2 (IFD, IORD, NXRASTS, NYRASTS,
        +                         IVH_BAR, IXROOT, IYROOT, IMG_FULL,
        +                         MINVAL, MAXVAL, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film image file (1 to 8)
               (-IORD can be given to swap bytes in the data as
               well as re-order it ) (see order)
NXRASTS  (R)   The number of x rasters (see nxrasts)
NYRASTS  (R)   The number of y rasters (see nyrasts)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG_FULL (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img_full)
MINVAL   (W)   The minimum pixel value in the image (see minval)
MAXVAL   (W)   The maximum pixel value in the image (see maxval)
IERR     (W)   Returns the status from the xdl_read_full_film call
-end*/

/*-C:*/
int xdl_read_full_i2 (fd, order, nxrasts, nyrasts, vh_bar, xroot, yroot, 
                      img_full, minval, maxval)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8. (-order can be given to
                     byte swap the data as well as change the order)
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int nxrasts;      /* The number of x rasters (R)*/
int nyrasts;      /* The number of y rasters (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned short img_full[];
                  /* Array returning the read image data; 
                     The slower moving index in
                     the returned data is x going from low x to
                     high x and the faster moving index is y going 
                     from low y to high y i.e. the same order as in the
                     file. (W)*/
int *minval;      /* Returns the minimum pixel value in the image (W)*/
int *maxval;      /* Returns the maximum pixel value in the image (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred.
-end*/
{
   int i, j, k, l;
   int swap;
   int irec;
   int step;
   int nrecs, nrasts;
   unsigned short *buf;
   unsigned char *cbuf;
   unsigned char tempc;
   int ir, ix;

   /* Initialisations */

   *minval = 1000000;
   *maxval = 0;
   swap = 0;
   if (order<0)
   {
      swap = 1;
      order = -order;
   }
   if(order<1||order>8) order=1;
   nrecs = nxrasts;
   nrasts = nyrasts;
   if (order>4)
   {
      nrecs = nyrasts;
      nrasts = nxrasts;
   }

   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nrecs, 2, 5);
   
   /* Read data */

   lseek (fd, 0L, 0);
   buf = (unsigned short *) malloc(nrasts*sizeof(unsigned short));
   if (buf==0) return -1;
   cbuf = (unsigned char *) buf;
   irec = 1;
   for (ir=0;ir<nrecs;++ir)
   {
      i = read (fd, buf, 2*nrasts);
      if (i!=2*nrasts) goto tidy;
      switch (order)
      {
         case 1:
            j = ir*nrasts;
            step = 1;
            break;
         case 2:
            j = (ir+1)*nrasts - 1;
            step = -1;
            break;
         case 3:
            j = (nrecs-ir-1)*nrasts;
            step = 1;
            break;
         case 4:
            j = (nrecs-ir)*nrasts - 1;
            step = -1;
            break;
         case 5:
            j = ir;
            step = nrecs;
            break;
         case 6:
            j = nrecs - 1 - ir;
            step = nrecs;
            break;
         case 7:
            j = (nrasts-1)*nrecs + ir;
            step = -nrecs;
            break;
         case 8:
            j = nrasts*nrecs - 1 - ir;
            step = -nrecs;
            break;
      }
      if (swap)
      {
         l=0;
         for (k=0;k<nrasts;++k)
         {
             tempc = cbuf[l];
             cbuf[l] = cbuf[l+1];
             cbuf[l+1] = tempc;
             l += 2;
         }
      }
      for (k=0;k<nrasts;++k)
      {
          img_full[j] = buf[k];
          if (buf[k]<*minval) *minval = buf[k];
          if (buf[k]>*maxval) *maxval = buf[k];
          j += step;
      }
      if (vh_bar>0) xdl_progress_bar_value (vh_bar, ir+1);
      irec++;
   }
   irec = 0;   
   
   /* Tidy up */

   tidy:
   free (buf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_full_i2 */

void xdlf_read_full_i2 (ifd, iord, nxrasts, nyrasts,ivh_bar, ixroot, iyroot,
                          img_full, minval, maxval, ierr)
int *ifd, *iord, *nxrasts, *nyrasts;
int *ivh_bar, *ixroot, *iyroot, *img_full, *minval, *maxval, *ierr;
{
   *ierr = xdl_read_full_i2 (*ifd, *iord, *nxrasts, *nyrasts,
                               *ivh_bar, *ixroot, *iyroot, 
                               (unsigned short *) img_full,
                               minval, maxval);
   return;
}
/*-Routine: Close an image plate (i2) file - xdl_close_i2
The routine xdl_close_i2 (xdlf_close_i2) is used to close an image plate
'i2' format file. It can in fact be used with any format (image) file.
-end*/
/*
************************
**   xdl_close_i2     **
************************

Purpose: Close an 'i2' image file 

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_CLOSE_I2 (IFD, IERR)
-end*/
 
/*-Parameters:
IFD         (R)   The file descriptor (see fd)
IERR        (W)   Returns the status from the xdl_close_i2 call
-end*/

/*-C:*/
int xdl_close_i2 (fd)
/*end*/
 
/*-Parameters:*/
int   fd;             /* The file descriptor (R)*/

/*end*/
/*-Doc:
Return:  =0 OK; =1 Unable to close the file;
-end*/
{
   int ierr;      /* Error flag */

   /* Close the file */
   ierr = close (fd);
   if (ierr!=0) return 1;
   return 0;
}

/* Fortran binding: xdlf_close_i2 */

void xdlf_close_i2 (ifd, ierr)
int *ifd, *ierr;
{
   *ierr = xdl_close_i2 (*ifd);
   return;
}
 
/*-Section: Axis Order Decoding Routines
These are routines for decoding the axis order codes use to indicate the
order of the data in an image file.
-end*/

/*-Routine: Interpret axis order string - xdl_axord
The routine xdl_axord (xdlf_axord) is used to interpret a user supplied
string defining the axis order in an input film or image-plate file.
An option to request or nullify byte swapping (only relevant for
image plate data) may also be given. The string defines the axis
order in terms of x and y (the film xf, yf axes). The slower moving axis is
given first followed by the faster moving axis. Minus signs are used
when the order of an axis is reversed. The code 's' if present indicates
that byte swapping is to be done and the code 'n' indicates that no
byte swapping is to be done (byte swapping only relevant for image plate
data). Characters, other than 'x', 'y', '-', '+', 's' and 'n', or their
upper case equivalents, are ignored. Some examples follow and the returned
flags are indicated:
.
      xy                     order = 1, swaptyp = 0
      +x+y                   order = 1, swaptyp = 0
      -xy                    order = 3, swaptyp = 0
      x-y s                  order = 2, swaptyp = 2
      -Y, +X, n              order = 7, swaptyp = 1
      S                      order = 0, swaptyp = 2
      +yf, +xf               order = 4, swaptyp = 0

Invalid strings: +x-x      (Same axis defined twice)
                 +x+y-x    (More than two axes defined)
                 xysn      (Byte swapping and no byte swapping both 
                            defined)
.
-end*/
/*
***************************
**     xdl_axord         **
***************************

Purpose: Interpret image axes order and byte swapping option string

Author:  John W. Campbell, November 1991

*/
/*-Fortran:
         CALL XDLF_AXORD (STR, LEN, IORD, ISWAP, IERR)
-end*/
 
/*-Parameters:
STR         (R)   Character string containing the axis order
                  (and byte swapping option) definition
                  (see str)
                  ** Pass address using the XDLSTR routine **
LEN         (R)   The length of the character string
IORD        (W)   Returns the axis order as a number from 1-8
                  (0 if not specified or syntax error) 
                  (see order)
ISWAP       (W)   Returns the byte swap flag =1 no byte swap
                                             =2 byte swap data
                                             =0 not specified (or
                                                syntax error)
IERR        (W)   Returns the status from the xdl_axord call
-end*/

/*-C:*/
int xdl_axord (str,len,order,swaptyp)
/*end*/
 
/*-Parameters:*/
char *str;       /* String containing the axes order and byte swap flags
                    (R)*/
int len;         /* Length of the string; if 0 then assume a null
                    terminated string was passed (R) */
int *order;      /* Returns the axis order given as a number from 1 to 8
                    (If none specified (or syntax error) return 0)
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (W)*/
int *swaptyp;    /* Returns byte swap type   =1 do not swap bytes
                                             =2 swap bytes
                                             =0 not specified (or syntax
                                                error)
                    (W)*/

/*end*/
/*-Doc:
Return:  Error flag =0 no error; 
                    >0 syntax error   =1 Only one axis defined.
                                      =2 Both axes the same.
                                      =3 More than two axes defined.
                                      =4 Byte swapping and no byte
                                         swapping defined.
-end*/
{
   int saxis, ssign;
   int faxis, fsign;
   int n;
   int l;
   int err;

   /* Initialisations */

   if (len<=0) len=strlen(str);
   saxis = 0;
   ssign = 1;
   faxis = 0;
   fsign = 1;
   err = 0;
   *swaptyp = 0;
   *order = 0;
   if (len==0) return 0;
   n = 1;

   /* Decode the string */

   for (l=0;l<len;++l)
   {
      switch (str[l])
      {
         case '-':
            if (n==1) { ssign = -1; }else{ fsign = -1; }
            break;
         case '+':
            if (n==1) { ssign = 1; }else{ fsign = 1; }
            break;
         case 'x':
         case 'X':
            if (n==1) 
            {
               saxis = 1;
               n = 2;
            }
            else
            {
               if (n==3) { err=3; goto synterr; }
               faxis = 1;
               n = 3;
            }
            break;
         case 'y':
         case 'Y':
            if (n==1) 
            {
               saxis = 2;
               n = 2;
            }
            else
            {
               if (n==3) { err=3; goto synterr; }
               faxis = 2;
               n = 3;
            }
            break;
         case 's':
         case 'S':
            if (*swaptyp>0) { err=4; goto synterr; }
            *swaptyp = 2;
            break;
         case 'n':
         case 'N':
            if (*swaptyp>0) goto synterr;
            *swaptyp = 1;
            break;
      }
   }
   if (saxis==0&&faxis==0) return 0;
   if (saxis==0||faxis==0) { err=1; goto synterr; }
   if (saxis==faxis) { err=2; goto synterr; }
   *order = 4*saxis - 3;
   if (ssign<0) *order += 2;
   if (fsign<0) *order += 1;
   return 0;
   synterr:
   *order = 0;
   *swaptyp = 0;
   return err;
}
/* Fortran binding: xdlf_axord */

void xdlf_axord (str, len, iord, iswap, ierr)
int *str, *len, *iord, *iswap, *ierr;
{
   *ierr = xdl_axord (XDL_pointers[*str], *len, iord, iswap);
   return;
}
 
/*-Section: MAR Image-Plate File Routines
These are routines for reading a MAR image-plate system image file including
the header information and overflow data.
-end*/

/*-Routine: Read a MAR image plate file - xdl_read_mar
The routine xdl_read_mar (xdlf_read_mar) is used to read a complete MAR
image-plate image file and return the image data in an array. Overload
data are include. The returned image contains signed integer*4 data.
The data will be byte swapped on input if required.
-end*/
/*
*************************
**   xdl_read_mar      **
*************************

Purpose: Read a Mar image plate file into an integer*4 array and 
         include overload data.

Author:  John Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_READ_MAR (IFD, IVH_BAR, IXROOT, IYROOT, 
        +                    IMG_FULL, NXPIX, NYPIX, NS_RAS, NF_RAS, 
        +                    MINVAL, MAXVAL, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG_FULL (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img_full)
NXPIX    (W)   Returns no. of 'x' pixels
NYPIX    (W)   Returns no. of 'y' pixels
NS_RAS   (W)   Returns no. of slow rasters
NF_RAS   (W)   Returns no. of fast rasters
MINVAL   (W)   The minimum pixel value in the image (see minval)
MAXVAL   (W)   The maximum pixel value in the image (see maxval)
IERR     (W)   Returns the status from the xdl_read_mar call
-end*/

/*-C:*/
int xdl_read_mar (fd, vh_bar, xroot, yroot, img_full, 
                  nxpix, nypix, ns_ras, nf_ras, minval, maxval)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
int img_full[];   /* Array returning the read image data (W) */
int *nxpix;       /* Returns the no. of 'x' pixels (W)*/
int *nypix;       /* Returns the no. of 'y' pixels (W)*/
int *ns_ras;      /* Returns the no. of 'slow' rasters */
int *nf_ras;      /* Returns the no. of 'fast' rasters */
int *minval;      /* Returns the minimum pixel value in the image (W)*/
int *maxval;      /* Returns the maximum pixel value in the image (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred.
         -1 error in allocating memory
         -2 error reading header
         -3 error reading overflow data
-end*/
{
   int i, j, k, l;
   int irec;
   int step;
   int nrecs, nrasts, novfl;
   unsigned short *buf;
   unsigned char *cbuf;
   unsigned char tempc;
   int hdr[6];
   int ir, ix;
   long offset;
   int ipix;
   int ovfl[2];
   int swap;

   /* Initialisations */

   *minval = 1000000;
   *maxval = 0;
   buf = 0;
   swap = 0;

   /* Read header */   

   irec = -2;
   lseek (fd, 0L, 0);
   cbuf = (unsigned char *) hdr;
   i = read (fd, hdr, 24);
   if (i!=24) goto tidy;
   if (hdr[0]!=1200&&hdr[0]!=2000) swap = 1;
   if (swap)
   {
      l=0;
      for (k=0;k<6;++k)
      {
          tempc = cbuf[l];
          cbuf[l] = cbuf[l+3];
          cbuf[l+3] = tempc;
          tempc = cbuf[l+1];
          cbuf[l+1] = cbuf[l+2];
          cbuf[l+2] = tempc;
          l += 4;
      }
   }
   nrecs = hdr[0];
   nrasts = hdr[1];
   *nxpix = nrecs;
   *ns_ras = nrecs;
   *nypix = nrasts;
   *nf_ras = nrasts;
   novfl = hdr[4];
   offset = 2*nrasts;

   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nrecs, 2, 5);

   /* Read data */

   lseek (fd, offset, 0);
   buf = (unsigned short *) malloc(nrasts*sizeof(unsigned short));
   if (buf==0) return -1;
   cbuf = (unsigned char *) buf;
   irec = 1;
   for (ir=0;ir<nrecs;++ir)
   {
      i = read (fd, buf, 2*nrasts);
      if (i!=2*nrasts) goto tidy;
            j = ir*nrasts;
      if (swap)
      {
         l=0;
         for (k=0;k<nrasts;++k)
         {
             tempc = cbuf[l];
             cbuf[l] = cbuf[l+1];
             cbuf[l+1] = tempc;
             l += 2;
         }
      }
      for (k=0;k<nrasts;++k)
      {
          img_full[j] = (int) buf[k];
          if (buf[k]<*minval) *minval = buf[k];
          if (buf[k]>*maxval) *maxval = buf[k];
          j ++;
      }
      if (vh_bar>0) xdl_progress_bar_value (vh_bar, ir+1);
      irec++;
   }
   irec = 0;   

   /* Overloads */

   offset = 2*nrasts*(nrecs+1);
   lseek (fd, offset, 0);
   
   irec = -3;
   for (j=0;j<novfl;++j)
   {
      i = read (fd, ovfl, 8);
      if (i!=8) goto tidy;
      if (swap)
      {
         l=0;
         for (k=0;k<2;++k)
         {
             tempc = cbuf[l];
             cbuf[l] = cbuf[l+3];
             cbuf[l+3] = tempc;
             tempc = cbuf[l+1];
             cbuf[l+1] = cbuf[l+2];
             cbuf[l+2] = tempc;
             l += 4;
        }
      }
      ipix = ovfl[0];
      img_full[ipix-1] = ovfl[1];
      if (ovfl[1]>*maxval) *maxval = ovfl[1];
   }
   irec = 0;

   /* Tidy up */

   tidy:
   if (buf!=0) free (buf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_mar */

void xdlf_read_mar (ifd, ivh_bar, ixroot, iyroot,
                          img_full, nxpix, nypix, ns_ras, nf_ras,
                          minval, maxval, ierr)
int *ifd;
int *ivh_bar, *ixroot, *iyroot, *img_full, *minval, *maxval, *ierr;
int *nxpix, *nypix, *ns_ras, *nf_ras;
{
   *ierr = xdl_read_mar (*ifd, *ivh_bar, *ixroot, *iyroot, img_full,
                         nxpix, nypix, ns_ras, nf_ras, minval, maxval);
   return;
}
/*-Routine: Read and squash a MAR image plate file - xdl_rdsquash_mar
The routine xdl_rdsquash_mar (xdlf_rdsquash_mar) is used to read a complete MAR
image-plate image file and return the image data in an array. Overload
data are included. The returned image contains each data item packed into
two bytes with intensities>32767 being stored as (65536-intensity/8) (unsigned)
values. The data will be byte swapped on input if required.
-end*/
/*
*****************************
**   xdl_rdsquash_mar      **
*****************************

Purpose: Read a Mar image plate file into an 'squashed' two byte array and 
         include overload data.

Author:  John Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_RDSQUASH_MAR (IFD, IVH_BAR, IXROOT, IYROOT, 
        +                    IMG_FULL, NXPIX, NYPIX, NS_RAS, NF_RAS, 
        +                    MINVAL, MAXVAL, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG_FULL (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img_full)
NXPIX    (W)   Returns no. of 'x' pixels
NYPIX    (W)   Returns no. of 'y' pixels
NS_RAS   (W)   Returns no. of slow rasters
NF_RAS   (W)   Returns no. of fast rasters
MINVAL   (W)   The minimum pixel value in the image (see minval)
MAXVAL   (W)   The maximum pixel value in the image (see maxval)
IERR     (W)   Returns the status from the xdl_rdsquash_mar call
-end*/

/*-C:*/
int xdl_rdsquash_mar (fd, vh_bar, xroot, yroot, img_full, 
                      nxpix, nypix, ns_ras, nf_ras, minval, maxval)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned short img_full[];   /* Array returning the read image data as 
                                squashed two-byte data (W) */
int *nxpix;       /* Returns the no. of 'x' pixels (W)*/
int *nypix;       /* Returns the no. of 'y' pixels (W)*/
int *ns_ras;      /* Returns the no. of 'slow' rasters */
int *nf_ras;      /* Returns the no. of 'fast' rasters */
int *minval;      /* Returns the minimum pixel value in the image (W)*/
int *maxval;      /* Returns the maximum pixel value in the image (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred.
         -1 error in allocating memory
         -2 error reading header
         -3 error reading overflow data
-end*/
{
   int i, j, k, l;
   int irec;
   int step;
   int nrecs, nrasts, novfl;
   unsigned short *buf;
   unsigned char *cbuf;
   unsigned char tempc;
   int hdr[6];
   int ir, ix;
   long offset;
   int ipix;
   int ovfl[2];
   int swap;

   /* Initialisations */

   *minval = 1000000;
   *maxval = 0;
   buf = 0;
   swap = 0;

   /* Read header */   

   irec = -2;
   lseek (fd, 0L, 0);
   cbuf = (unsigned char *) hdr;
   i = read (fd, hdr, 24);
   if (i!=24) goto tidy;
   if (hdr[0]!=1200&&hdr[0]!=2000) swap = 1;
   if (swap)
   {
      l=0;
      for (k=0;k<6;++k)
      {
          tempc = cbuf[l];
          cbuf[l] = cbuf[l+3];
          cbuf[l+3] = tempc;
          tempc = cbuf[l+1];
          cbuf[l+1] = cbuf[l+2];
          cbuf[l+2] = tempc;
          l += 4;
      }
   }
   nrecs = hdr[0];
   nrasts = hdr[1];
   *nxpix = nrecs;
   *ns_ras = nrecs;
   *nypix = nrasts;
   *nf_ras = nrasts;
   novfl = hdr[4];
   offset = 2*nrasts;

   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nrecs, 2, 5);

   /* Read data */

   lseek (fd, offset, 0);
   buf = (unsigned short *) malloc(nrasts*sizeof(unsigned short));
   if (buf==0) return -1;
   cbuf = (unsigned char *) buf;
   irec = 1;
   for (ir=0;ir<nrecs;++ir)
   {
      i = read (fd, buf, 2*nrasts);
      if (i!=2*nrasts) goto tidy;
            j = ir*nrasts;
      if (swap)
      {
         l=0;
         for (k=0;k<nrasts;++k)
         {
             tempc = cbuf[l];
             cbuf[l] = cbuf[l+1];
             cbuf[l+1] = tempc;
             l += 2;
         }
      }
      for (k=0;k<nrasts;++k)
      {
          img_full[j] = (unsigned short) sqsh(buf[k]);
          if (buf[k]<*minval) *minval = buf[k];
          if (buf[k]>*maxval) *maxval = buf[k];
          j ++;
      }
      if (vh_bar>0) xdl_progress_bar_value (vh_bar, ir+1);
      irec++;
   }
   irec = 0;   

   /* Overloads */

   offset = 2*nrasts*(nrecs+1);
   lseek (fd, offset, 0);
   
   irec = -3;
   for (j=0;j<novfl;++j)
   {
      i = read (fd, ovfl, 8);
      if (i!=8) goto tidy;
      if (swap)
      {
         l=0;
         for (k=0;k<2;++k)
         {
             tempc = cbuf[l];
             cbuf[l] = cbuf[l+3];
             cbuf[l+3] = tempc;
             tempc = cbuf[l+1];
             cbuf[l+1] = cbuf[l+2];
             cbuf[l+2] = tempc;
             l += 4;
        }
      }
      ipix = ovfl[0];
      img_full[ipix-1] = (unsigned short) (sqsh(ovfl[1]));
      if (ovfl[1]>*maxval) *maxval = ovfl[1];
   }
   irec = 0;

   /* Tidy up */

   tidy:
   if (buf!=0) free (buf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_rdsquash_mar */

void xdlf_rdsquash_mar (ifd, ivh_bar, ixroot, iyroot,
                        img_full, nxpix, nypix, ns_ras, nf_ras,
                        minval, maxval, ierr)
int *ifd;
int *ivh_bar, *ixroot, *iyroot, *img_full, *minval, *maxval, *ierr;
int *nxpix, *nypix, *ns_ras, *nf_ras;
{
   *ierr = xdl_rdsquash_mar (*ifd, *ivh_bar, *ixroot, *iyroot, 
                         (unsigned short *) img_full,
                         nxpix, nypix, ns_ras, nf_ras, minval, maxval);
   return;
}

/*-Routine: Read a Photon Factory Log Byte File - xdl_read_pfbyte
The routine xdl_read_pfbyte (xdlf_read_pfbyte) is used to read a complete
Photon Fcatory image plate file and return the image data in an array. 
The file may have its data in any of the eight possible orders (the default 
is assumed to  have yf moving fastest and  xf slowest). Each pixel stores
a log value for the pixel intensity (unsigned byte). The file has no
header. The number of x rasters and the number of y rasters must be supplied.
The data are returned in a packed integer array with
-end*/
/*
*****************************
**   xdl_read_pfbyte       **
*****************************

Purpose: Read a Photon Factory image plate data with log byte pixels. 

Author:  John Campbell, July 1997

*/
/*-Fortran:
         CALL XDLF_READ_PFBYTE (IFD, IORD, NXRASTS, NYRASTS,
        +                       IVH_BAR, IXROOT, IYROOT, IMG_FULL,
        +                       IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the image file (1 to 8)
               (see order)
NXRASTS  (R)   The number of x rasters (see nxrasts)
NYRASTS  (R)   The number of y rasters (see nyrasts)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG_FULL (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img_full)
IERR     (W)   Returns the status from the xdl_read_pfbyte call
-end*/

/*-C:*/
int xdl_read_pfbyte (fd, order, nxrasts, nyrasts, vh_bar, 
                     xroot, yroot, img_full)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8.
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int nxrasts;      /* The number of x rasters (R)*/
int nyrasts;      /* The number of y rasters (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned short img_full[];
                  /* Array returning the read image data as two byte
                     unsigned quantities; 
                     The slower moving index in
                     the returned data is x going from low x to
                     high x and the faster moving index is y going 
                     from low y to high y. (W)*/
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred; -1 unable
         to allocate temporary buffer space.
-end*/
{
   int i, j, k;
   int irec;
   int step;
   int nrecs, nrasts;
   unsigned char *buf;
   int ir, ix;

   /* Initialisations */

   if(order<1||order>8) order=1;
   nrecs = nxrasts;
   nrasts = nyrasts;
   if (order>4)
   {
      nrecs = nyrasts;
      nrasts = nxrasts;
   }

   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nrecs, 2, 5);
   
   /* Read data */

   lseek (fd, 0L, 0);
   buf = (unsigned char *) malloc(nrasts);
   if (buf==0) return -1;
   irec = 1;
   for (ir=0;ir<nrecs;++ir)
   {
      i = read (fd, buf, nrasts);
      if (i!=nrasts) goto tidy;
      switch (order)
      {
         case 1:
            j = ir*nrasts;
            step = 1;
            break;
         case 2:
            j = (ir+1)*nrasts - 1;
            step = -1;
            break;
         case 3:
            j = (nrecs-ir-1)*nrasts;
            step = 1;
            break;
         case 4:
            j = (nrecs-ir)*nrasts - 1;
            step = -1;
            break;
         case 5:
            j = ir;
            step = nrecs;
            break;
         case 6:
            j = nrecs - 1 - ir;
            step = nrecs;
            break;
         case 7:
            j = (nrasts-1)*nrecs + ir;
            step = -nrecs;
            break;
         case 8:
            j = nrasts*nrecs - 1 - ir;
            step = -nrecs;
            break;
      }
      for (k=0;k<nrasts;++k)
      {
          img_full[j] = nint(pow(10.0,4.0*(float)buf[k]/255.0)-1.0);
          j += step;
      }
      if (vh_bar>0) xdl_progress_bar_value (vh_bar, ir+1);
      irec++;
   }
   irec = 0;   
   
   /* Tidy up */

   tidy:
   free (buf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_pfbyte */

void xdlf_read_pfbyte (ifd, iord, nxrasts, nyrasts,ivh_bar, ixroot, iyroot,
                       img_full, ierr)
int *ifd, *iord, *nxrasts, *nyrasts;
int *ivh_bar, *ixroot, *iyroot, *img_full, *ierr;
{
   *ierr = xdl_read_pfbyte (*ifd, *iord, *nxrasts, *nyrasts,
                            *ivh_bar, *ixroot, *iyroot, 
                            (unsigned short *) img_full);
   return;
}
