/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The I/O Window View-Object Routines
-end*/
 
/*-Intro:
The I/O window view-object provides an area for the output of
text from the program and input of text strings by the user when
required (e.g. for a question and answer sequence). The I/O window has 
an 'active strip' at the top to indicate whether or not the program is 
waiting for a reply string to be input by the user. When the program is 
waiting for input, then the message 'Input reply' is displayed in the 
active strip. Options are available for text editing, selecting/pasting,
command recall and window scrolling.
-end*/
 
/*-Section: Creating and Handling an I/O Window View-Object
-end*/
 

/*Routines List:
The I/O window view-object has the following routines available:

   xdl_io_window               Create a new I/O window view-object
   xdl_io_window_print         Output text to an I/O window
   xdl_io_window_getstring     Get input text string from an I/O window
   xdl_io_window_input_message Change message in active strip
   xdl_io_window_set_focus     Set keyboard focus
   xdl_io_window_rootxy        Get root window position of input string
   xdl_io_window_getsize       Calculate size requirements
*/

/*-Routine: Create an I/O window view-object - xdl_io_window
The routine xdl_io_window (xdlf_io_window) is used to create a new I/O
window view-object. The user may specify the size of the window in
pixels or in terms of the required numbers of rows and columns of
characters.
-end*/
/*
************************
**    xdl_io_window   **
************************

Purpose: Display a window for program i/o

Author:  John W. Campbell, June 1990, August 1993

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW (IVH, IVHPARENT, IX, IY, ICSET, NCOLS, 
        +                     NROWS, MINW, MINH, IFONT, NSCROLL, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
MINW      (R)   Minimum width in pixels (see min_width)
MINH      (R)   Minimum height in pixels (see min_height)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
NSCROLL   (R)   The number of pages (of nrows lines) to be
                saved for scrolling option (0 if no scrolling)
IERR      (W)   Returns status from xdl_io_window call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"

#define MAXINSTRING 300        /* Must be a multiple of 4*/
#define MAXHIST  50
#define MARG_W 2
#define MARG_H 2
#define MIN_ACT_STRIP 100      /* Minimum width for active strip */
#define PANEL_WIDTH 130        /* Width required for scrolling control panel */

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wio;                       /* I/o area window id */
   Window wact;                      /* Active strip window */
   Window wpanel;                    /* Control panel (for scroll) window */
   Panel_object button1;             /* Page up button */
   Panel_object button2;             /* Page down button */
   Panel_object slider;              /* Scroll slider */
   int travel_len;                   /* Travel length for slider */
   GC          paint_gc;             /* GC for text paint window */
   char        **texp;               /* Storage of the text for the io-window
                                        (array of pointers to text strings) */
   int         ch_width;             /* Character width in pixels */
   int         ch_height;            /* Character height in pixels */
   int         ncols;                /* No. of columns of characters */
   int         nrows;                /* No. of rows of characters */
   int         curs_col;             /* Cursor column number (1 upwards) */
   int         curs_row;             /* Cursor row number (1 upwards) */
   int         start_row;            /* Row on which input string starts */
   int         start_col;            /* Column on which input string starts */
   int         font_descent;         /* Character descent (from font) */
   int         font_ascent;          /* Character ascent (from font) */
   int         max_width;            /* Maximum character width (from font) */
   unsigned int copy_w;              /* Width of area to copy on scrolling */
   unsigned int copy_h;              /* Height of area to copy on scrolling */
   char        instring[MAXINSTRING+1]; /* Array to hold input string */
   int         max_instring;         /* Max. allowed length of input string */
   int         last_in;              /* Offset of last input character in 
                                        'instring' */
   int         current_pos;          /* Current insert position offset in
                                        'instring' */
   int         max_room;             /* Maximum room available for input
                                        string based on i/o window size;
                                        if < MAXINSTRING, will protect
                                        against start of input being
                                        lost off top of screen & avoids
                                        code having to treat such a case */
   char *      hist_strings[MAXHIST];/* List of history string pointers */
   int         num_hist;             /* Number of history strings stored */
   int         last_hist;            /* Offset of last history recalled */
   char        sav_instring[MAXINSTRING+1]; /* Save current input string
                                               during history request */
   int         on_off;               /* Program awaiting input =1, not =0 */
   int         kbd_focus;            /* View-object has keyboard focus flag */
   char *      paste_buf;            /* Pointer to paste buffer area */
   char *      select_buf;           /* Pointer to 'selected' text */
   int         select_len;           /* Length of selected text */
   char        input_message[61];    /* Message for active strip */
   int         select_start;         /* Start offset in text array for
                                        text selection */
   int         select_end;           /* End offset in text array for
                                        text selection */
   int         select;               /* Select in progress =1 yes, =0 no */
   int         select_page;          /* Page on which current selection (if
                                        any) has been done */
   int         select_owner;         /* Selection owner =1 yes, =0 no */
   int         nscroll;              /* No. of pages for back scrolling */
   int         zp_off;               /* Offset in 'texp' for start of
                                        'Zero Page' i.e. the latest page */
   int         current_page;         /* Current page; 0 = zero page; back
                                        pages from -1 to -nscroll */
   int         nback;                /* Number of backward lines currently
                                        stored */
   int         maxback;              /* Maximum no. of backward lines which
                                        may be stored */
}  
   XDl_io_window;

/* Define routine types */
int xdl_io_window();
void xdl_io_window_rps();
void xdl_io_window_rp1();
void xdl_io_window_ep1();
void xdl_io_window_rp2();
void xdl_io_window_cp1();
void xdl_io_window_cp2();
void xdl_io_window_clc();
void xdl_io_window_cursor_on();
void xdl_io_window_shft();
void xdl_io_window_curchar();
void xdl_io_window_rest_out();
void xdl_io_window_delch();
void xdl_io_window_insch();
void xdl_io_window_cursfw();
void xdl_io_window_cursbk();
void xdl_io_window_savhist();
void xdl_io_window_gethist();
void xdl_io_window_xysoff();
void xdl_io_window_select_set();
void xdl_io_window_select_clear();
void xdl_io_window_select_show();
void xdl_io_window_select_receive();
void xdl_io_window_select_string();
void xdl_io_window_ch();
void xdl_io_window_ch_rev();
void xdl_io_window_getch();
void xdl_io_window_rewrite();
void xdl_io_window_on_off();
void xdl_io_window_tidy_up();
int xdl_io_window_print();
int xdl_io_window_getstring();
int xdl_io_window_input_message();
int xdl_io_window_rootxy();
void xdl_io_window_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_io_window xdlf_io_window_
#define xdlf_io_window_print xdlf_io_window_print_
#define xdlf_io_window_getstring xdlf_io_window_getstring_
#define xdlf_io_window_input_message xdlf_io_window_input_message_
#define xdlf_io_window_set_focus  xdlf_io_window_set_focus_
#define xdlf_io_window_rootxy xdlf_io_window_rootxy_
#define xdlf_io_window_getsize xdlf_io_window_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_io_window XDLF_IO_WINDOW
#define xdlf_io_window_print XDLF_IO_WINDOW_PRINT
#define xdlf_io_window_getstring XDLF_IO_WINDOW_GETSTRING
#define xdlf_io_window_input_message XDLF_IO_WINDOW_INPUT_MESSAGE
#define xdlf_io_window_set_focus  XDLF_IO_WINDOW_SET_FOCUS
#define xdlf_io_window_rootxy XDLF_IO_WINDOW_ROOTXY
#define xdlf_io_window_getsize XDLF_IO_WINDOW_GETSIZE

#endif

void xdlf_io_window();
void xdlf_io_window_print();
void xdlf_io_window_getstring();
void xdlf_io_window_input_message();
void xdlf_io_window_set_focus();
void xdlf_io_window_rootxy();
void xdlf_io_window_getsize();

/*-C:*/
int xdl_io_window (vh, vh_parent, x, y, cset, ncols, nrows, min_width, 
                   min_height, font_type, nscroll)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the i/o window view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - 
                             normally 0 (R)*/
int       ncols;          /* Number of columns of characters (if 0 then
                             this will be calculated from min_width which
                             must then be given a value > 0. (R)*/
int       nrows;          /* Number of rows of characters (if 0 then
                             this will be calculated from min_height which
                             must then be given a value > 0. (R)*/
int       min_width;      /* Minimum window width in pixels. If this is
                             greater than the width calculated from the
                             number of character columns requirement then 
                             this value will be used as the window width 
                             (R)*/
int       min_height;     /* Minimum window height in pixels. If this is
                             greater than the height calculated from the
                             number of character rows requirement then 
                             this value will be used as the window height
                             (R)*/
int       font_type;      /* Font type (1-5) for very_small, small, medium,
                             large or extra-large font. (R)*/
int       nscroll;        /* No. of pages which may be back scrolled;
                             0 if no scrolling */
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Memory allocation error
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   Status st;                  /* Status return from Xlib calls */
   XDl_io_window * gd;         /* Pointer to view-object's global data 
                                  structure */
   XFontStruct *fontinfo;      /* Font structure pointer */
   char * p;                   /* Temporary pointer */
   char ** pp;                 /* Temporary pointer */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int i;                      /* Loop counter */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int area_width;             /* Text area width */
   int area_height;            /* Text area height */
   int x_area;                 /* x position of text area */
   int y_area;                 /* y position of text area */
   int actwidth;               /* Active strip width */
   int min_strip_width;        /* Minimum width required for active strip
                                  + scrolling if used */
   int panel_width;            /* Scrolling control panel width */
   int panel_x;                /* x position in control panel */
   int panel_y;                /* y position in control panel */

   err = 0;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (nscroll<0) nscroll = 0;

   /* Get size requirements */

   min_strip_width = MIN_ACT_STRIP;
   panel_width = 0;
   min_strip_width = min_strip_width + 2*BORDER_WIDTH + 2;
   if (nscroll>0) 
   {
      panel_width = PANEL_WIDTH;
      min_strip_width += panel_width + 4;
   }

   fontinfo = XDL_fonts[font_type-1];
   ch_width = fontinfo->max_bounds.width;
   ch_height = fontinfo->ascent + fontinfo->descent;
   x_area = BORDER_WIDTH;
   y_area = BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   if (ncols>0)
   {
      area_width = ncols*ch_width + 2*MARG_W  + 2;
      width = area_width + 2*BORDER_WIDTH + 2;
      if (min_width>width) 
      {
         width = min_width;
         x_area = (width - 2 - area_width) / 2;
      }
   }
   else
   {
      width = min_width;
      if (width<2*MARG_W+2*BORDER_WIDTH+ch_width+4)
          width = 2*MARG_W + 2*BORDER_WIDTH + ch_width + 4;
      area_width = width - 2*BORDER_WIDTH - 2;
      ncols = (area_width - 2*MARG_W - 2) / ch_width;
   }
   if (width < min_strip_width)
   {
      width = min_strip_width;
      area_width = width - 2*BORDER_WIDTH - 2;
      ncols = (area_width - 2*MARG_W - 2) / ch_width;
   }
   if (nrows>0)
   {
      area_height = nrows*ch_height + 2*MARG_H + 2;
      height = area_height + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2;
      if (min_height>height)
      {
         height = min_height;
         y_area = ACT_STRIP_HEIGHT + (height-2-ACT_STRIP_HEIGHT-area_height)/2;
      }
   }
   else
   {
      height = min_height;
      if (height<2*MARG_H+2*BORDER_HEIGHT+ACT_STRIP_HEIGHT+ch_height+4)
          height = 2*MARG_H + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT 
                   + ch_height + 4;
      area_height = height - 2*BORDER_HEIGHT - ACT_STRIP_HEIGHT -2;
      nrows = (area_height - 2*MARG_H - 2) / ch_height;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_io_window", "xdl_io",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_io_window));
   gd = (XDl_io_window *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, p, 1031, xdl_io_window_on_off,
                        xdl_io_window_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->on_off = 0;
   gd->kbd_focus = 0;
   gd->font_descent = fontinfo->descent;
   gd->font_ascent = fontinfo->ascent;
   gd->max_width = (int) fontinfo->max_bounds.width;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->ncols = ncols;
   gd->nrows = nrows;
   gd->curs_col = 1;
   gd->curs_row = 1;
   gd->start_col = gd->curs_col;
   gd->start_row = gd->curs_row;
   gd->max_room = (gd->nrows)*(gd->ncols) - gd->start_col;
   gd->copy_w = (unsigned int) area_width - 2;
   gd->copy_h = (unsigned int) (gd->nrows-1)*(gd->ch_height);
   gd->max_instring = MAXINSTRING;
   if (gd->max_instring>(gd->nrows)*(gd->ncols))
   {
      gd->max_instring = (gd->nrows)*(gd->ncols);
   }
   gd->last_in = -1;
   gd->current_pos = 0;
   gd->instring[0] = 0;
   gd->num_hist = 0;
   gd->last_hist = 1;
   for (i=0; i<MAXHIST; ++i) gd->hist_strings[i] = 0;
   strcpy (gd->input_message, "Input reply");
   gd->nscroll = nscroll;
   gd->zp_off = nscroll*nrows;
   gd->nback = 0;
   gd->maxback = nscroll*ncols;
   gd->current_page = 0;
   pp = (char **) malloc (((nscroll+1)*nrows+1)*sizeof(char**));
   if (pp==0) return 2;
   gd->texp = pp;
   for (i=0; i<=(nscroll+1)*nrows; i++) gd->texp[i] = 0;
   for (i=0;i<nrows;++i)
   {
       p = malloc ((ncols+2)*sizeof(char));
       gd->texp[gd->zp_off+i] = p;
       if (p!=0) p[0]=0;
   }
   p = (char *) malloc (((gd->ncols)*(gd->nrows)+1)*sizeof(char));
   gd->select_buf = p;
   if (p==0)
   {
      for (i=0;i<nrows;++i)
      {
         p = gd->texp[gd->zp_off+i];
         if (p!=0) free(p);
      }
      free (gd->texp);
      return 2;
   }
   gd->select_start = 0;
   gd->select_end = 0;
   gd->select_page = 0;
   gd->select = 0;
   gd->select_owner = 0;
   gd->select_buf[0] = 0;
   gd->select_len = 0;
   gd->paste_buf = NULL;

   /* Create active strip window */

   actwidth = width-2*BORDER_WIDTH-2;
   if (nscroll>0)
   {
      if (nscroll>0) actwidth = actwidth - panel_width - 4;
   }

   gd->wact = xdl_create_active_strip (gd->wframe, vh, actwidth, 
                                       xdl_io_window_rps);

   /* Add scrolling control panel & panel items if required */

   if (nscroll>0)
   {
      gd->wpanel = xdl_create_paint_window (wframe, 
                   (width - panel_width - BORDER_WIDTH - 2), 
                   BORDER_HEIGHT, 1,
                   panel_width, ACT_STRIP_H, 0);

      xdl_add_window (vh, gd->wpanel, xdl_io_window_rp2, NULL);

      panel_x = 2;
      panel_y = 2;
      gd->button1 = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                    "Up", 2, 1, 2, 0, 0, 0, 1, 1, xdl_io_window_cp1);

      panel_x = 102;
      panel_y = 2;
      gd->button2 = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                    "Dn", 2, 1, 2, 0, 0, 0, 1, 2, xdl_io_window_cp1);

      panel_x = 26;
      panel_y = 2;
      gd->travel_len = 50;
      gd->slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                    "", 2, gd->travel_len, gd->travel_len, 0, 0, 0, 
                    xdl_io_window_cp2);

   }

   /* Create i/o paint window */

   gd->wio = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                              area_width, area_height, cset);

   xdl_add_window (vh, gd->wio, xdl_io_window_rp1, xdl_io_window_ep1);

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wio, ExposureMask|StructureNotifyMask|
                         PointerMotionMask|ButtonPressMask|ButtonReleaseMask|
                         KeyPressMask|FocusChangeMask);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wio, 0, NULL);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetFont (XDL_display, gd->paint_gc, fontinfo->fid);

   return 0;
}

/* Fortran binding: xdf_io_window */

void xdlf_io_window (ivh, ivhparent, ix, iy, icset, ncols, nrows, minw, minh,
                     ifont, nscroll, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *ncols, *nrows, *minw, *minh,
    *ifont, *nscroll, *ierr;
{
   *ierr = xdl_io_window (*ivh, *ivhparent, *ix, *iy, *icset, *ncols, *nrows,
                          *minw, *minh, *ifont, *nscroll);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_io_window_rps       **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_io_window * gd;  /* Pointer to global data area */

   gd = (XDl_io_window *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->input_message, gd->kbd_focus);
   }
}
/*
*********************************
**     xdl_io_window_rp1       **
*********************************

Purpose: Repaint text area

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_io_window * gd;     /* Pointer to global data area */

   gd = (XDl_io_window *) global;
   xdl_io_window_rewrite (gd);

   return;
}
/*
*********************************
**     xdl_io_window_rp2       **
*********************************

Purpose: Repaint the control panel surround

Author:  John W. Campbell, August 1993

*/

void xdl_io_window_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}
/*
*********************************
**     xdl_io_window_cp1       **
*********************************

Purpose: Callback for scrolling panel buttons

Author:  John W. Campbell, August 1993

*/

void xdl_io_window_cp1 (iv, button)

int iv;
int button;

{
   int npage;          /* No. of back pages currently present */
   int islide;         /* New slider position */
   XDl_io_window *gd;

   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;

   npage = 0;
   if (gd->nback>0)
   {
      npage = (gd->nback-1)/(gd->nrows) + 1;
   }

   if (button==1)
   /* Page up */
   {
      if (npage>0)
      {
         if (gd->current_page>(-npage)&&gd->current_page>(-gd->nscroll))
         {
            gd->current_page --;
            xdl_io_window_rewrite (gd);
         }
      }
   }
   else
   /* Page down */
   {
      if (gd->current_page<0)
      {
         gd->current_page ++;
         xdl_io_window_rewrite (gd);
      }
   }
   if (npage>0)
   {
      islide = (int) (((float) gd->travel_len * (float) -gd->current_page
                     / (float) npage) + 0.5);
      islide = gd->travel_len - islide;
   }
   else
   {
      islide = gd->travel_len;
   }
   xdl_panel_slider_reset (gd->slider, islide);
   return;
}
/*
*********************************
**     xdl_io_window_cp2       **
*********************************

Purpose: Callback for scrolling panel slider

Author:  John W. Campbell, August 1993

*/

void xdl_io_window_cp2 (iv, curpos, travel_len)

int iv;
int curpos;
int travel_len;

{
   int npage;          /* No. of back pages currently present */
   int page;           /* Saves value of current page */
   int ipage;          /* page offset from 0 to npage */
   int islide;         /* Slider position (reset to calc. position for
                          new current page */
   XDl_io_window *gd;

   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;

   page = gd->current_page;
   npage = 0;
   if (gd->nback>0) npage = (gd->nback-1)/(gd->nrows) + 1;
   ipage = (int) (((float) npage * (float) curpos /(float) gd->travel_len)+0.5);
   if (ipage<0) ipage = 0;
   if (ipage>npage) ipage = npage;
   ipage = npage - ipage;
   gd->current_page = -ipage;
   if (npage>0)
   {
      islide = (int) (((float) gd->travel_len * (float) -gd->current_page
                     / (float) npage) + 0.5);
      islide = gd->travel_len - islide;
   }
   else
   {
      islide = gd->travel_len;
   }
   xdl_panel_slider_reset (gd->slider, islide);
   if (gd->current_page!=page) xdl_io_window_rewrite(gd);
   return;
}
/*
*********************************
**     xdl_io_window_ep1       **
*********************************

Purpose: Event handling from text area

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_io_window *gd;      /* Pointer to the global data area */
   int i;                  /* Loop variable */
   int x, y;               /* Character position */
   KeySym keysym;          /* Keysym for keyboard input */
   XComposeStatus compose; /* Compose status */
   int nc;                 /* No. of characters decoded fro key press */
   char chbuf[2];          /* Buffer for ascii translated characters */
   int ioff;               /* Offset in string for o/p of text */
   int nsp;                /* Number of spaces to o/p (/clear) after string */
   XEvent xevent;          /* Temporary XEvent */
   int toff;               /* Offset into text pointers array */
   char *p;                /* Pointer to string */
   int nitems;             /* No. of items received from select */

   gd = (XDl_io_window *) global;

   /* Check for keyboard focus lost */

   if (event->type==FocusOut)
   {
      if (gd->on_off==1)
      {
          if (gd->kbd_focus==0) return;
          xdl_active_strip_on (gd->wact, gd->input_message, 0);
          gd->kbd_focus = 0;
          return;
      }
   }

   /* Check for keyboard focus gained */

   if (event->type==FocusIn)
   {
      if (gd->on_off==1)
      {
          if (gd->kbd_focus==1) return;
          xdl_active_strip_on (gd->wact, gd->input_message, 1);
          gd->kbd_focus = 1;
          return;
      }
   }

   /* Check for mouse select to get keyboard focus */

   if (event->type==ButtonPress&&(event->xbutton.button==Button1||
      event->xbutton.button==Button2))
   {
      if (gd->on_off==1)
      {
         XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
         xdl_active_strip_on (gd->wact, gd->input_message, 1);
         gd->kbd_focus = 1;
      }
   }

   /* Start selection area if Button 1 pressed (clearing any old one) */

   if (event->type==ButtonPress&&event->xbutton.button==Button1)
   {
      xdl_io_window_select_clear (gd);
      x = event->xbutton.x; y = event->xbutton.y;
      xdl_io_window_xysoff (gd, x, y, &gd->select_start);
      gd->select_end = gd->select_start;
      gd->select = 1;
      gd->select_page = gd->current_page;
   }

   /* Stop selection area if Button 1 released */

   if (event->type==ButtonRelease&&event->xbutton.button==Button1)
   {
      xdl_io_window_select_string (gd);
      gd->select = 0;
      if (gd->select_end!=gd->select_start)
      {
         xdl_select_owner (gd->wio);
         gd->select_owner = 1;
      }
      else
      {
         if (gd->select_owner==1)
         {
            xdl_select_cancel();
            gd->select_owner = 0;
         }
      }
   }

   /* Check Middle button to paste in selection */

   if (event->type==ButtonPress&&event->xbutton.button==Button2)
   {
      if (gd->on_off==1)
      {
         xdl_select_request (event, gd->wio);
         return;
      }
   }

   /* Selection Request received */

   if (event->type==SelectionRequest)
   {
      xdl_select_respond (event, gd->select_buf, gd->select_len);
   }

   /* Selection Clear received */

   if (event->type==SelectionClear)
   {
      xdl_io_window_select_clear (gd);
      gd->select_start = 0;
      gd->select_end = 0;
      gd->select_owner = 0;
   }

   /* Pointer motion & select area */

   if (event->type==MotionNotify)
   {
      x = event->xmotion.x;
      y = event->xmotion.y;
      while (XCheckTypedWindowEvent (XDL_display, gd->wio,
             MotionNotify, &xevent))
      {
         x = xevent.xmotion.x;
         y = xevent.xmotion.y;
      }
      xdl_io_window_select_set (gd, x, y);
   }

   /* Process keyboard input */

   if (gd->on_off==0) return;

   if (event->type==KeyPress)
   {
      if (gd->select_owner==1)
      {
         xdl_select_cancel ();
      }
      xdl_io_window_select_clear (gd);

      if (gd->current_page!=0)
      {
         gd->current_page = 0;
         xdl_panel_slider_reset (gd->slider, gd->travel_len);
         xdl_io_window_rewrite (gd);
      }
      gd->select_start = 0;
      gd->select_end = 0;

      nc = XLookupString ((XKeyEvent *)event, chbuf, 2, &keysym, NULL);
      if (keysym==XK_BackSpace||keysym==XK_Delete)
      {
         if (gd->last_in>=0&&gd->current_pos>0)
         {
            xdl_io_window_delch (gd);
            nsp = 1;
            if (gd->current_pos>gd->last_in) nsp = 2;
            xdl_io_window_rest_out (gd, gd->current_pos, nsp);
            xdl_io_window_cursor_on (gd, 0);
         }
      }
      else if (keysym==XK_Return||keysym==XK_KP_Enter||keysym==XK_Linefeed)
      {
         gd->current_pos = gd->last_in + 1;
         gd->curs_row = gd->start_row + 
                        (gd->start_col + gd->current_pos - 1)/(gd->ncols);
         gd->curs_col = (gd->start_col + gd->current_pos - 1)%(gd->ncols) + 1;
         toff = gd->zp_off + gd->curs_row - 1;
         p = gd->texp[toff];
         strcat (p,"\n");
         xdl_io_window_clc (gd, 0);
         gd->curs_col = 1;
         gd->curs_row += 1;
         if (gd->curs_row>gd->nrows)
         {
            xdl_io_window_shft(gd);
         }
         gd->start_col = gd->curs_col;
         gd->start_row = gd->curs_row;
         gd->max_room = (gd->nrows)*(gd->ncols) - gd->start_col;
         xdl_io_window_savhist (gd);
         XDL_return_data_vh = XDL_view_objects[iv]->vh;
         return;
      }
      else if (keysym>=XK_space&&keysym<=XK_asciitilde)
      {
         if (gd->last_in<(gd->max_instring-1))
         {
            xdl_io_window_insch (gd, chbuf[0]);
            ioff = gd->current_pos - 1;
            xdl_io_window_rest_out (gd, ioff, 1);
            xdl_io_window_cursor_on (gd, 0);
         }
      }
      else if (keysym==XK_Left)
      {
         if (gd->last_in>=0&&gd->current_pos>0)
         {
            xdl_io_window_curchar (gd);
            gd->current_pos--;
            xdl_io_window_cursbk (gd);
            xdl_io_window_cursor_on (gd, 0);
         }
      }
      else if (keysym==XK_Right)
      {
         if (gd->current_pos<gd->last_in+1)
         {
            xdl_io_window_curchar (gd);
            gd->current_pos++;
            xdl_io_window_cursfw (gd);
            xdl_io_window_cursor_on (gd, 0);
         }
      }
      else if (keysym==XK_Up)
      {
         xdl_io_window_gethist (gd, 1);
      }
      else if (keysym==XK_Down)
      {
         xdl_io_window_gethist (gd, 2);
      }
   }

   /* Selection notify following paste request (ignore if not zero page) */

   if (event->type==SelectionNotify)
   {
      if (xdl_select_receive (event, gd->wio, &gd->paste_buf, &nitems,
          MAXINSTRING)) xdl_io_window_select_receive (gd, nitems);
   }
   return;
}

/*
*********************************
**     xdl_io-window_clc       **
*********************************

Purpose: Clear a character position in the text area (on screen and
         in text store if required)

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_clc (gd, it)

XDl_io_window *gd;
int it;             /* =1 clear character in text arrays, =0 do not */
{
   int toff;      /* Offset into text pointers array */
   int ic;        /* Column offset */
   int ll;        /* Length of character string */
   char *p;       /* String pointer */

   if (gd->current_page!=0) return;

   if (it)
   {
      toff = gd->zp_off + gd->curs_row - 1;
      ic = gd->curs_col - 1;
      p = gd->texp[toff];
      if (p!=0)
      {
         ll = strlen(p);
         if (ic<ll) p[ic] = ' ';
      }
   }

   XClearArea (XDL_display, gd->wio,
               (gd->curs_col-1)*(gd->ch_width) + MARG_W,
               (gd->curs_row-1)*(gd->ch_height) + MARG_H,
               gd->ch_width, gd->ch_height, False);
   return;
}

/*
*********************************
**     xdl_io_window_delch     **
*********************************

Purpose: Delete a character from the input string (prior to current position)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_delch (gd)

XDl_io_window *gd;
{
   int i;    /*Loop counter*/

   if(gd->last_in<0) return;
   if (gd->current_pos<=gd->last_in)
   {
      for (i=gd->current_pos;i<=gd->last_in;++i)
      {
         gd->instring[i-1] = gd->instring[i];
      }
   }
   gd->instring[gd->last_in] = 0;
   gd->last_in--;
   gd->current_pos--;
   return;
}

/*
*********************************
**     xdl_io_window_insch     **
*********************************

Purpose: Insert a character into the input string (at current position)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_insch (gd, ch)

XDl_io_window *gd;
char ch;             /*Character to be inserted*/
{
   int i;    /*Loop counter*/

   if(gd->last_in<(gd->max_instring-1)&&gd->last_in<(gd->max_room-1))
   {
      if (gd->current_pos<=gd->last_in)
      {
         for (i=gd->last_in;i>=gd->current_pos;--i)
         {
            gd->instring[i+1] = gd->instring[i];
         }
      }
      gd->instring[gd->current_pos] = ch;
      gd->last_in++;
      gd->current_pos++;
      gd->instring[gd->last_in+1] = 0;
   }
   return;
}
/*
***********************************
**     xdl_io_window_rest_out    **
***********************************

Purpose: Output rest of input string from requested position (offset) onwards;
         (Returns with cursor position at current_pos in string though
         does not display cursor)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_rest_out (gd, ioff, nsp)

XDl_io_window *gd;
int ioff;            /* Offset for start of o/p */
int nsp;             /* No. of additional spaces to be o/p after end of
                        input string */
{
   int i;         /*Loop counter*/
   int toff;      /*Offset into text pointers array*/
   int x;         /*'x' position for character output*/
   int y;         /*'y' position for character output*/
   int first;     /* first char has been o/p flag */
   int ic;        /* Column offset */
   int ll;        /* Current length of a text string */
   char *p;       /* String pointer */

   first = 0;
   gd->curs_row = gd->start_row + 
                  (gd->start_col + ioff - 1)/(gd->ncols);
   gd->curs_col = (gd->start_col + ioff - 1)%(gd->ncols) + 1;

   if (ioff<=gd->last_in)
   {
      for (i=ioff;i<=gd->last_in;++i)
      {
         if (first) xdl_io_window_cursfw (gd);
         first = 1;
         xdl_io_window_clc (gd, 1);
         toff = gd->zp_off + gd->curs_row - 1;
         p = gd->texp[toff];
         ic = gd->curs_col -1;
         ll = strlen(p);
         if (ic<ll)
         {
            p[ic] = gd->instring[i];
         }
         else if (ic==ll)
         {
            p[ic] = gd->instring[i];
            p[ic+1] = 0;
         }
         else
         {
            for (i=ll;i<=ic;++i) p[ic] = ' ';
            p[ic] = gd->instring[i];
            p[ic+1] = 0;
         }
         x = (gd->ch_width)*(gd->curs_col-1) + MARG_W;
         y = (gd->ch_height)*(gd->curs_row) - gd->font_descent+MARG_H;
         XDrawString (XDL_display, gd->wio, gd->paint_gc, x, y, 
                      &gd->instring[i], 1);
      }
   }
   if (nsp>0)
   {
      for (i=0;i<nsp;++i)
      {
         if (first) xdl_io_window_cursfw (gd);
         first = 1;
         xdl_io_window_clc (gd, 1);
      }
   }
   gd->curs_row = gd->start_row + 
                  (gd->start_col + gd->current_pos - 1)/(gd->ncols);
   gd->curs_col = (gd->start_col + gd->current_pos - 1)%(gd->ncols) + 1;
   return;
}
/*
***********************************
**     xdl_io_window_curchar     **
***********************************

Purpose: Re-output current position character without cursor (zero page only)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_curchar (gd)

XDl_io_window *gd;
{
   int x;         /*'x' position for character output*/
   int y;         /*'y' position for character output*/
   char ch;       /* Current character */
   int within;    /* Character within defined string flag */

   xdl_io_window_getch (gd, gd->curs_row, gd->curs_col, &ch, &within);
   x = (gd->ch_width)*(gd->curs_col-1) + MARG_W;
   y = (gd->ch_height)*(gd->curs_row) - gd->font_descent+MARG_H;
   XDrawImageString (XDL_display, gd->wio, gd->paint_gc, x, y, 
                &ch, 1);
   return;
}
/*
******************************
**     xdl_io_window_ch     **
******************************

Purpose: Output a character from 'text' in normal video

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_ch (gd, soff)

XDl_io_window *gd;
int soff;      /*Screen offset of charcater position*/
{
   int x;         /*'x' position for character output*/
   int y;         /*'y' position for character output*/
   int ir;        /*Row number*/
   int icoff;     /*Column offset*/
   char ch;       /*Text character*/
   int within;    /*Character within defined string flag*/

   ir = soff/gd->ncols + 1;
   icoff = soff%gd->ncols;
   x = (gd->ch_width)*icoff + MARG_W;
   y = (gd->ch_height)*ir - gd->font_descent + MARG_H;
   xdl_io_window_getch (gd, ir, icoff+1, &ch, &within);
   XDrawImageString (XDL_display, gd->wio, gd->paint_gc, x, y, 
                    &ch, 1);
   return;
}
/*
**********************************
**     xdl_io_window_ch_rev     **
**********************************

Purpose: Output a character from 'text' in reverse video if within
         defined string (or normal blank if beyond end of string)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_ch_rev (gd, soff)

XDl_io_window *gd;
int soff;      /*Character offset into screen*/
{
   int x;         /*'x' position for character output*/
   int y;         /*'y' position for character output*/
   int ir;        /*Row number*/
   int icoff;     /*Column offset*/
   char ch;       /*Text character*/
   int within;    /*=1 char is within defined string; =0 beyond end of string */

   ir = soff/gd->ncols + 1;
   icoff = soff%gd->ncols;
   x = (gd->ch_width)*icoff + MARG_W;
   y = (gd->ch_height)*ir - gd->font_descent + MARG_H;
   xdl_io_window_getch (gd, ir, icoff+1, &ch, &within);
   if (within)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
      XSetBackground (XDL_display, gd->paint_gc, XDL_blackpixel);
   }
   XDrawImageString (XDL_display, gd->wio, gd->paint_gc, x, y, 
                    &ch, 1);
   if (within)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   }
   return;
}
/*
**********************************
**     xdl_io_window_getch      **
**********************************

Purpose: Get character for a given row and columns number (any newline
         characters are returned as spaces)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_getch (gd, ir, ic, ch, within)

XDl_io_window *gd;
int ir;            /* Row number */
int ic;            /* Column number */
char *ch;          /* Returns the character */
int *within;       /* Returns 1 if charcater within defined string; otherwise
                      returns 0 */
{
   int toff;     /* Offset in text pointers array */
   char *p;      /* String pointer */
   int ll;       /* String length */

   *ch = ' ';
   *within = 0;
   if (gd->current_page==0)
   {
      toff = gd->zp_off + ir - 1;
   }
   else
   {
      toff = (gd->nscroll+gd->current_page)*(gd->nrows) + ir - 1;
   }
   p = gd->texp[toff];
   if (p==0) return;
   ll = strlen(p);
   if (ic>ll) return;
   *within = 1;
   *ch = p[ic-1];
   if (*ch=='\n') *ch = ' ';
   return;
}
/*
**********************************
**     xdl_io_window_rewrite    **
**********************************

Purpose: Rewrite a page

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_rewrite (gd)

XDl_io_window *gd;
{
   int ir, ic;             /* Row and column loop counters (offsets)*/
   int x, y;               /* Character x, y position */
   int toff;               /* Text pointers array offset */
   char str[2];            /* Temp character string array */
   char *p;                /* Temp string pointer */
   int ll;                 /* Length of character string */

   str[1] = 0;
   XClearWindow (XDL_display, gd->wio);
   toff = (gd->nscroll+gd->current_page)*(gd->nrows);
   for (ir=0; ir<gd->nrows; ++ir)
   {
      p = gd->texp[toff+ir];
      if (p==0) continue;
      ll = strlen (p);
      for (ic=0; ic<gd->ncols; ++ic)
      {
         if (ic<ll)
         {
            str[0] = p[ic];
            if (str[0]=='\n') str[0]=' ';
         }
         else
         {
            str[0] = ' ';
         }
         x = (gd->ch_width)*ic + MARG_W;
         y = (gd->ch_height)*(ir+1) - gd->font_descent + MARG_H;
         XDrawString (XDL_display, gd->wio, gd->paint_gc,
                      x, y, str, 1);
      }
   }
   if (gd->on_off==1&&gd->current_page==0) xdl_io_window_cursor_on (gd, 0);
   if (gd->select_page==gd->current_page) xdl_io_window_select_show (gd);
   return;
}

/*
*****************************************
**     xdl_io_window_ch_select_clear   **
*****************************************

Purpose: Clear current select highlighting

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_select_clear (gd)

XDl_io_window *gd;
{
   int i;         /* Offset into 'text' array */
   int coff;      /* Offset in 'text' array of cursor position */
   int hl_start;  /* Start offset in 'text' array of highlighting */
   int hl_end;    /* End offset in 'text' array of highlighting */

   if (gd->select_end==gd->select_start) return;

   coff = (gd->curs_row-1)*(gd->ncols) + gd->curs_col -1;
   if (gd->select_end>=gd->select_start)
   {
      hl_start = gd->select_start;
      hl_end = gd->select_end - 1;
   }
   else
   {
      hl_start = gd->select_end;
      hl_end = gd->select_start-1;
   }
   if (hl_end>=hl_start)
   {
      for (i=hl_start;i<=hl_end;++i)
      {
         xdl_io_window_ch (gd, i);
         if (gd->on_off==1&&i==coff) xdl_io_window_cursor_on (gd, 0);
      }
   }

   return;
}
/*
*****************************************
**     xdl_io_window_ch_select_show    **
*****************************************

Purpose: Show current select highlighting (for repaint)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_select_show (gd)

XDl_io_window *gd;
{
   int i;         /* Offset into 'text' array */
   int coff;      /* Offset in 'text' array of cursor position */
   int hl_start;  /* Start offset in 'text' array of highlighting */
   int hl_end;    /* End offset in 'text' array of highlighting */

   if (gd->select_end==gd->select_start) return;

   coff = (gd->curs_row-1)*(gd->ncols) + gd->curs_col -1;
   if (gd->select_end>=gd->select_start)
   {
      hl_start = gd->select_start;
      hl_end = gd->select_end - 1;
   }
   else
   {
      hl_start = gd->select_end;
      hl_end = gd->select_start-1;
   }
   if (hl_end>=hl_start)
   {
      for (i=hl_start;i<=hl_end;++i)
      {
         xdl_io_window_ch_rev (gd, i);
         if (gd->on_off==1&&i==coff) xdl_io_window_cursor_on (gd, 1);
      }
   }

   return;
}
/*
*********************************
**     xdl_io_window_xysoff    **
*********************************

Purpose: Convert pointer x, y to screen offset character position

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_xysoff (gd, x, y, soff)

XDl_io_window *gd;
int x;             /* Pointer 'x' position */
int y;             /* Pointer 'y' position */
int * soff;        /* Character position offset on screen offset */
{
   int row_offset;  /*Row offset*/
   int col_offset;  /*Column offset*/

   row_offset = y/gd->ch_height;
   if (row_offset>=gd->nrows) row_offset = gd->nrows-1;
   col_offset = x/gd->ch_width;
   if (col_offset>=gd->ncols) col_offset = gd->ncols-1;
   *soff = row_offset*(gd->ncols) + col_offset;
   return;
}
/*
************************************
**     xdl_io_window_select_set   **
************************************

Purpose: Set current selection

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_select_set (gd, x, y)

XDl_io_window *gd;
int x;             /* Pointer 'x' position */
int y;             /* Pointer 'y' position */
{
   int kend;       /* Current end position from pointer */
   int low;        /* Start offset in 'text' or area affected */
   int high;       /* End offset in 'text' of area affected */
   int hl_start;   /* Start offset in 'text' for highlighting */
   int hl_end;     /* End offset in 'text' for highlighting (may
                      be <hl_low if no highlighting */
   int i;          /* Loop variable */
   int coff;       /* Offset in 'text' array of current cursor position */

   if (gd->select)
   {
      xdl_io_window_xysoff (gd, x, y, &kend);
      low = gd->select_start;
      if (gd->select_end<low) low = gd->select_end;
      if (kend<low) low = kend;
      high = gd->select_start;
      if (gd->select_end>high) high = gd->select_end;
      if (kend>high) high = kend;
      gd->select_end = kend;
      coff = (gd->curs_row-1)*(gd->ncols) + gd->curs_col -1;
      if (gd->select_end>=gd->select_start)
      {
         hl_start = gd->select_start;
         hl_end = gd->select_end - 1;
      }
      else
      {
         hl_start = gd->select_end;
         hl_end = gd->select_start-1;
      }
      for (i=low;i<=high;++i)
      {
         if (i>=hl_start&&i<=hl_end)
         {
            xdl_io_window_ch_rev (gd, i);
            if (gd->on_off==1&&i==coff) xdl_io_window_cursor_on (gd, 1);
         }
         else
         {
            xdl_io_window_ch (gd, i);
         }
         if (coff>=gd->select_start&&coff<=gd->select_end)
         {
            xdl_io_window_cursor_on (gd, 1);
            if (gd->on_off==1&&i==coff) xdl_io_window_cursor_on (gd, 0);
         }
      }
   }
   return;
}
/*
****************************************
**     xdl_io_window_select_receive   **
****************************************

Purpose: Deal with received selection

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_select_receive (gd, nitems)

XDl_io_window *gd;
int nitems;
{
   int ioff;
   int nmax;
   int i;

   if (gd->current_page!=0)
   {
      gd->current_page = 0;
      xdl_panel_slider_reset (gd->slider, gd->travel_len);
      xdl_io_window_rewrite (gd);
   }

   if (nitems>0)
   {
      ioff = gd->last_in;
      if (ioff<0) ioff = 0;
      nmax = MAXINSTRING;
      if (gd->max_room<nmax) nmax = gd->max_room;
      nmax -= gd->last_in - 1;
      if (nitems>nmax) nitems = nmax;
      if (nitems>0)
      {
         for (i=0;i<nitems;++i)
         {
            if (gd->paste_buf[i]=='\n') break;
            gd->last_in ++;
            gd->instring[gd->last_in] = gd->paste_buf[i];
         }
         gd->current_pos = gd->last_in + 1;
         gd->instring[gd->last_in+1] = 0;
         xdl_io_window_rest_out (gd, ioff, 1);
         xdl_io_window_cursor_on (gd, 0);
      }
   }
   if (gd->paste_buf!=0) free (gd->paste_buf);
   return;
}
/*
****************************************
**     xdl_io_window_select_string    **
****************************************

Purpose: Prepare select string from selected area

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_select_string (gd)

XDl_io_window *gd;
{
   XEvent sel_event;       /* Selection event */
   int hl_start;           /* Start offset in 'text' of selection area */
   int hl_end;             /* End offset in 'text' of selection area */
   int irow1;              /* First row containing highlighting */
   int irow2;              /* Final row containing highlighting */
   int ir;                 /* Row counter */
   int ll;                 /* Length of string */
   char *p;                /* String pointer */
   int ic;                 /* Start column offset */
   int jc;                 /* End column offset */
   int toff;               /* Offset in text pointers array */

   gd->select_buf[0] = 0;
   if (gd->select_end>=gd->select_start)
   {
      hl_start = gd->select_start;
      hl_end = gd->select_end - 1;
   }
   else
   {
      hl_start = gd->select_end;
      hl_end = gd->select_start-1;
   }
   gd->select_len = hl_end - hl_start + 1;
   if (gd->select_len>0)
  {
      irow1 = hl_start/(gd->ncols) + 1;
      irow2 = hl_end/(gd->ncols) + 1;
      for (ir=irow1;ir<=irow2;++ir)
      {
         if (gd->current_page==0)
         {
            toff = gd->zp_off + ir - 1;
         }
         else
         {
            toff = (gd->nscroll+gd->current_page)*(gd->nrows) + ir - 1;
         }
         p = gd->texp[toff];
         if (p==0) continue;
         ll = strlen(p);
         ic = 0;
         if (ir==irow1) ic = hl_start%gd->ncols;
         if (ic<ll)
         {
            if (ir==irow2)
            {
               jc = hl_end%gd->ncols;
               if (jc<ll)
               {
                  strncat (gd->select_buf, &p[ic], jc-ic+1);
               }
               else
               {
                  strcat (gd->select_buf, &p[ic]);
               }
            }
            else
            {
               strcat (gd->select_buf, &p[ic]);
            }
         }
      }
      gd->select_len = strlen(gd->select_buf);
   }
   return;
}
/*
***********************************
**     xdl_io_window_gethist     **
***********************************

Purpose: Get and display input string recalled from history buffer

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_gethist (gd, iupd)

XDl_io_window *gd;
int iupd;           /*=1 up_arrow request, =2 down_arrow request */
{
   int ll;         /* String length*/
   int nsp;        /* Number of extra spaces to o/p */
   char * str;     /* Pointer to character string */

   ll = 0;
   if (iupd==1)
   {
      if (gd->num_hist>0)
      {
         if (gd->last_hist==gd->num_hist) /*If first access save 
                                            current input string*/
         {
            strcpy (gd->sav_instring, gd->instring);
         }

         if (gd->last_hist>0) gd->last_hist--;
         str = gd->hist_strings[gd->last_hist];
         if (str!=0) ll = strlen(str);
      }
   }
   else
   {
      if (gd->num_hist>0)
      {
         if (gd->last_hist<gd->num_hist) gd->last_hist++;
         if (gd->last_hist<gd->num_hist)
         {
            str = gd->hist_strings[gd->last_hist];
            if (str!=0) ll = strlen(str);
         }
         else
         {
            str = gd->sav_instring;
            if (str!=0) ll = strlen(str);
         }
      }
   }
   nsp = 1;
   if (gd->last_in+1>ll) nsp = gd->last_in + 2 - ll;
   gd->instring[0] = 0;
   gd->last_in = -1;
   gd->current_pos = 0;
   if (ll>0)
   {
      strcat (gd->instring, str);
      gd->last_in = strlen (gd->instring) - 1;
      gd->current_pos = gd->last_in + 1;
   }
   xdl_io_window_rest_out (gd, 0, nsp);
   xdl_io_window_cursor_on (gd, 0);
   return;
}
/*
***********************************
**     xdl_io_window_savhist     **
***********************************

Purpose: Save last input string in history buffer & reset last history
         pointer

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_savhist (gd)

XDl_io_window *gd;
{
   char * p;       /* Temp pointer to character string */
   int i;          /* Loop counter */
   int ll;         /* Length of character string */

   if (gd->num_hist>=MAXHIST)
   {
      p = gd->hist_strings[0];
      if (p!=0) free(p);
      for (i=0;i<=gd->num_hist-1;++i)
      {
         gd->hist_strings[i] = gd->hist_strings[i+1];
      }
      gd->hist_strings[gd->num_hist-1] = 0;
      gd->num_hist --;
   }

   ll = strlen(gd->instring);
   p = (char *) malloc (ll+1);
   gd->hist_strings[gd->num_hist] = p;
   if (p!=0) strcpy (p, gd->instring);
   gd->num_hist ++;

   gd->last_hist = gd->num_hist;
   return;
}
/*
*********************************
**     xdl_io_window_cursfw    **
*********************************

Purpose: Move current cursor 1 position forward (no cursor display by
         this routine)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_cursfw (gd)

XDl_io_window *gd;
{
   gd->curs_col += 1;
   if (gd->curs_col>gd->ncols)
   {
      gd->curs_col = 1;
      gd->curs_row += 1;
      if (gd->curs_row>gd->nrows)
      {
         xdl_io_window_shft(gd);
      }
   }
   return;
}
/*
*********************************
**     xdl_io_window_cursbk    **
*********************************

Purpose: Move current cursor 1 position  backward (no cursor display by
         this routine)

Author:  John W. Campbell, July 1993

*/

void xdl_io_window_cursbk (gd)

XDl_io_window *gd;
{
   gd->curs_col -= 1;
   if (gd->curs_col<1)
   {
      gd->curs_col = gd->ncols;
      gd->curs_row -= 1;
   }
   return;
}

/*
**************************************
**     xdl_io_window_cursor_on      **
**************************************

Purpose: Display a cursor for text input

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_cursor_on (gd, rev)

XDl_io_window *gd;   /* Pointer to the global data area */
int rev;             /* Reverse video flag =1 yes, =0 no */
{
   int x, y;    /* Position for the cursor */
   if (rev)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
      XSetBackground (XDL_display, gd->paint_gc, XDL_blackpixel);
   }
   x = (gd->curs_col-1)*(gd->ch_width) + MARG_W;
   y = (gd->curs_row)*(gd->ch_height) - 1 + MARG_H;
   XDrawLine (XDL_display, gd->wio, gd->paint_gc,
   x+1, y-3, x+1, y-1);
   XDrawLine (XDL_display, gd->wio, gd->paint_gc,
   x+1, y-1, x+gd->ch_width-2, y-1);
   XDrawLine (XDL_display, gd->wio, gd->paint_gc,
   x+gd->ch_width-2, y-3, x+gd->ch_width-2, y-1);
   if (rev)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   }
   return;
}

/*
*********************************
**     xdl_io_window_shft      **
*********************************

Purpose: Shift up text lines when bottom of window reached (ie scrolling)

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_shft (gd)

XDl_io_window *gd;    /* Pointer to the global data area */
{
   int i;    /* Loop counter */
   int ll;   /* String length */
   char *p;  /* String pointer */

   gd->curs_row -= 1;
   if (gd->start_row>1) gd->start_row -=1;
   XCopyArea (XDL_display, gd->wio, gd->wio, gd->paint_gc,
              0, gd->ch_height + MARG_H, gd->copy_w, gd->copy_h,
              0, MARG_H);
   XClearArea (XDL_display, gd->wio,
               0, (gd->nrows-1)*(gd->ch_height) + MARG_H,
               gd->copy_w, gd->ch_height, False);


   if (gd->nscroll>0)
   {
      if (gd->texp[0]!=0) free(gd->texp[0]);
      for (i=0;i<gd->zp_off-1;++i) gd->texp[i] = gd->texp[i+1];
      gd->texp[gd->zp_off-1] = 0;
      ll =strlen (gd->texp[gd->zp_off]);
      p = malloc ((ll+1)*sizeof(char));
      gd->texp[gd->zp_off-1] = p;
      if (p!=0) strcpy (p, gd->texp[gd->zp_off]);
      gd->nback ++;
   }
   p = gd->texp[gd->zp_off];
   for (i=0;i<gd->nrows-1;++i)
   {  
      gd->texp[gd->zp_off+i] = gd->texp[gd->zp_off+i+1];
   }
   gd->texp[gd->zp_off+gd->nrows-1]=p;
   p[0] = 0;
   
   gd->select_start = gd->select_start - gd->ncols;
   if (gd->select_start<0) gd->select_start = 0;
   gd->select_end = gd->select_end - gd->ncols;
   if (gd->select_end<0) gd->select_end = 0;
   return;
}

/*
***********************************
**     xdl_io_window_on_off      **
***********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_io_window *gd;        /* Pointer to global data area */
   XWindowAttributes attrib; /* Window attributes structure */

   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      gd->last_in = -1;
      gd->current_pos = 0;
      gd->instring[0] = 0;
      gd->current_page = 0;
      xdl_io_window_cursor_on (gd, 0);
      XGetWindowAttributes (XDL_display, gd->wio, &attrib);
      if (attrib.map_state==IsViewable)
      {
         XSetInputFocus (XDL_display, gd->wio, RevertToParent, CurrentTime);
         gd->kbd_focus = 1;
      }
      xdl_active_strip_on (gd->wact, gd->input_message, gd->kbd_focus);
   }
   else
   {
      gd->on_off = 0;
      if (gd->current_page==0) xdl_io_window_clc (gd, 1);
      xdl_active_strip_off (gd->wact);
   }
   return;
}

/*
************************************
**     xdl_io_window_tidy_up      **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, July 1990

*/

void xdl_io_window_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_io_window *gd;    /* Pointer to the global data area */
   int i;                /* Loop counter */

   gd = (XDl_io_window *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   if (gd->num_hist>0)
   {
      for (i=0;i<gd->num_hist;++i)
      {
         if (gd->hist_strings[i]!=0) free (gd->hist_strings[i]);
      }
   }
   if (gd->paste_buf!=0) free (gd->paste_buf);
   for (i=0;i<(gd->nscroll+1)*(gd->nrows)+1;++i)
   {
       if (gd->texp[i]!=0) free (gd->texp[i]);
   }
   if (gd->texp!=0) free (gd->texp);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Output a text string - xdl_io_window_print
The routine xdl_io_window_print (xdlf_io_window_print) is used to output a 
text string to an I/O window view object. The string for the C routine may 
contain newline characters and the Fortran routine has options for newlines 
at the start and/or end of the string.
-end*/
/*
*******************************
**     xdl_io_window_print   **
*******************************

Purpose: Print string on the io_window

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_PRINT (IVH, XDLSTR(STRING), LEN, NEWLIN, 
        +                           IERR)
-end*/
 
/*-Parameters:
IVH     (R)  View-object handle (see vh)
STRING  (R)  The string to be printed (see string)
             ** Pass the address using the XDLSTR function **
LEN     (R)  Length of the string (>0) (cf len_str)
NEWLIN  (R)  =0 no newlines, =1 newline before printing the 
                string,
             =2 newline after printing the string, =3 newlines
             before and after printing the string
IERR    (W)  Returns the status from xdl_io_window_print call
-end*/

/*-C:*/
int xdl_io_window_print (vh, string, len_str)
/*end*/
 
/*-Parameters:*/
int vh;               /* The xdl_io_window view-object handle (R)*/
char string[];        /* The string to be printed 
                         Note: \f, \r and \v will all be treated as \n
                               (newline)
                               \b is ignored
                               \t (tab) is converted to  a space
                         (R)*/
int len_str;          /* The length of the string. If the string is null
                         terminated, then 0 may be given and the length
                         of the string will then be determined by the
                         routine (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   XDl_io_window *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int nch;                /* No. of characters to output */
   int is;                 /* Loop counter through string characters */
   int toff;               /* Offset in the text array */
   int x, y;               /* Position of a character */
   char str[2];            /* Temporary string to hold 1 character and a null */
   char *p;                /* Pointer to string */
   int ic;                 /* Column offset */
   int i;                  /* Loop variable */
   int ll;                 /* String length */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_io_window_print",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1031)
   {
      xdl_view_type_errmsg ("xdl_io_window_print", vh, "xdl_io_window");
      return 2;
   }
   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;
   if (gd->on_off==1) return 0;
   str[1] = 0;

   /* Reset to 'zero page' if required */

   if (gd->current_page!=0)
   {
      gd->current_page = 0;
      xdl_io_window_rewrite (gd);
   }

   /* Output and store the string */

   nch = len_str;
   if (nch==0) nch = strlen (string);

   toff = (gd->zp_off+gd->curs_row-1);
   p = gd->texp[toff];

   for (is=0; is<nch; ++is)
   {
      str[0] = string[is];
      if (str[0]=='\b') continue;
      if (str[0]=='\f'||str[0]=='\r'||str[0]=='\v') str[0] = '\n';
      if (str[0]=='\t') str[0] = ' ';
      if (str[0]=='\n')
      {
         strcat (p,"\n");
         gd->curs_col = 1;
         gd->curs_row += 1;
         if (gd->curs_row>gd->nrows)
         {
            xdl_io_window_shft(gd);
         }
         toff = (gd->zp_off+gd->curs_row-1);
         p = gd->texp[toff];
      }
      else if (str[0]>31&&str[0]<127)
      {
         ic = gd->curs_col -1;
         ll = strlen(p);
         if (ic<ll)
         {
            p[ic] = str[0];
         }
         else if (ic==ll)
         {
            p[ic] = str[0];
            p[ic+1] = 0;
         }
         else
         {
            for (i=ll;i<=ic;++i) p[ic] = ' ';
            p[ic] = str[0];
            p[ic+1] = 0;
         }
         x = (gd->ch_width)*(gd->curs_col-1) + MARG_W;
         y = (gd->ch_height)*(gd->curs_row) - gd->font_descent + MARG_H;
         XDrawString (XDL_display, gd->wio, gd->paint_gc,
                      x, y, str, 1);
         gd->curs_col += 1;
         if (gd->curs_col>gd->ncols)
         {
            gd->curs_col = 1;
            gd->curs_row += 1;
            if (gd->curs_row>gd->nrows)
            {
               xdl_io_window_shft(gd);
            }
            toff = (gd->zp_off+gd->curs_row-1);
            p = gd->texp[toff];
        }
      }
   }
   gd->start_col = gd->curs_col;
   gd->start_row = gd->curs_row;
   gd->max_room = (gd->nrows)*(gd->ncols) - gd->start_col;
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_io_window_print */

void xdlf_io_window_print (ivh, string, len, newlin, ierr)
int *ivh, *string, *len, *newlin, *ierr;
{
   if (*newlin==1||*newlin==3) *ierr = xdl_io_window_print (*ivh, "\n", 0);
   *ierr = xdl_io_window_print (*ivh, XDL_pointers[*string], *len);
   if (*newlin==2||*newlin==3) *ierr = xdl_io_window_print (*ivh, "\n", 0);
   return;
}

/*-Routine: Get an input text string - xdl_io_window_getstring
The routine xdl_io_window_getstring (xdlf_io_window_getstring) is used to
return an input text string. It will normally be called after the return 
from a call to xdl_get_events (xdlf_get_events) indicates that an I/O
window view-object has returned input.
-end*/
/*
***********************************
**     xdl_io_window_getstring   **
***********************************

Purpose: Get returned input string from the io_window

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_GETSTRING (IVH, XDLSTR(STRING), MAXLEN, 
        +                               IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
STRING   (W)  Returned string (see string)
              ** Pass address using the XDLSTR function **
MAXLEN   (R)  Maximum allowed length for the returned string 
              ( > 0) (cf max_len)
IERR     (W)  Returns status from xdl_io_window_getstring call
-end*/

/*-C:*/
int xdl_io_window_getstring (vh, string, max_len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char string[];   /* Returned string (W)*/
int max_len;     /* if >0 Maximum length of returned string allowed 
                          (excluding terminating null - string must be 
                          at least max_len + 1 characters in length)
                    if <0 abs(max_len) characters will be returned padded 
                          with blanks if needed (for 'fortran' use) (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_io_window *gd;    /* pointer to the global data area */
   char *p;              /* Temp. pointer for string functions */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_io_window_getstring",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1031)
   {
      xdl_view_type_errmsg ("xdl_io_window_getstring", vh, "xdl_io_window");
      return 2;
   }

   /* Copy the string to be returned */

   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;
   xdl_copy_chars (gd->instring, string, max_len);
   return 0;
}

/* Fortran binding xdlf_io_window_getstring */

void xdlf_io_window_getstring (ivh, string, maxlen, ierr)
int *ivh, *string, *maxlen, *ierr;
{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ierr = xdl_io_window_getstring (*ivh, XDL_pointers[*string], max_len);
   return;
}
/*-Routine: Reset active strip message - xdl_io_window_input_message
The routine xdl_io_window_input_message (xdlf_io_window_input_message) is
used to reset the message which will appear in the active strip when input
is requested When the I/O window view-object is created, the default message 
is 'Input reply'.
-end*/
/*
*****************************************
**     xdl_io_window_input_message     ** 
*****************************************

Purpose: Reset 'Input reply' message for the active strip area

Author:  John W. Campbell, July 1993

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_INPUT_MESSAGE (IVH, XDLSTR(MESSAGE), 
        +                                    LEN, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MESSAGE  (R)  Character string containing the message 
              (see message)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the message string - must be >0 
              (cf len) (max 60 chars) 
IERR     (W)  Returns status from xdl_io_window_input_message 
              call
-end*/

/*-C:*/
int xdl_io_window_input_message (vh, message, len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *message;   /* Message string (max of 60 chars long) (R)*/
int len;         /* Length of the message string; if 0 then length 
                    will be found assuming a null terminated string 
                    (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_io_window  *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_io_window_input_message",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1031)
   {
      xdl_view_type_errmsg ("xdl_io_window_input_message", vh, "xdl_io_window");
      return 2;
   }
   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;
   if (len==0) len = strlen (message);
   if (len>60) len = 60;
   strncpy (gd->input_message, message, len);
   gd->input_message[len] = 0;
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_io_window_input_message */

void xdlf_io_window_input_message (ivh, message, len, ierr)
int *ivh, *message, *len, *ierr;
{
   *ierr = xdl_io_window_input_message (*ivh, XDL_pointers[*message], *len);
   return;
}
/*-Routine: Set keyboard focus - xdl_io_window_set_focus
The routine xdl_io_window_set_focus (xdlf_io_window_set_focus) is
used to reset the keyboard focus to the i/o window. If the window is
not currently viewable, an attempt will be made again after a requested
number of milliseconds and this will be repeated if needed up to a maximum
requested number of times. This procedure may be needed, for example, if
a new i/o window view-object has just been created.
-end*/
/*
*****************************************
**     xdl_io_window_set_focus         ** 
*****************************************

Purpose: Set keyboard focus (window must be viewable)

Author:  John W. Campbell, April 1996

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_SET_FOCUS (IVH, MSEC, NTRY, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MSEC     (R)  If window not currently visable, retry after 'MSEC' 
              milliseconds
NTRY     (R)  Maximum no. of retries to attempt.
IERR     (W)  Returns status from xdl_io_window_set_focus call
-end*/

/*-C:*/
int xdl_io_window_set_focus (vh, msec, ntry)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int msec;        /* If window not currently visable, retry after  
                    'msec' milliseconds (R)*/
int ntry;        /* Maximum no. of retries to attempt (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_io_window  *gd;       /* Pointer to global data area */
   XWindowAttributes attrib; /* Window attributes structure */
   int ntried;               /* No. of attempts tried */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_io_window_set_focus",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1031)
   {
      xdl_view_type_errmsg ("xdl_io_window_set_focus", vh, "xdl_io_window");
      return 2;
   }
   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;
   ntried=0;
   retry:
   XGetWindowAttributes (XDL_display, gd->wio, &attrib);
   if (attrib.map_state==IsViewable)
   {
      XSetInputFocus (XDL_display, gd->wio, RevertToParent, CurrentTime);
      gd->kbd_focus = 1;
   }
   else
   {
      if (ntried>=ntry) return 0;
      xdl_delay (msec);
      ntried++;
      goto retry;
   }
   return 0;
}

/* Fortran binding: xdlf_io_window_set_focus */

void xdlf_io_window_set_focus (ivh, msec, ntry, ierr)
int *ivh, *msec, *ntry, *ierr;
{
   *ierr = xdl_io_window_set_focus (*ivh, *msec, *ntry);
   return;
}

/*-Routine: Get a root window position - xdl_io_window_rootxy
The routine xdl_io_window_rootxy (xdlf_io_windowrootxy) gets the root
window x, y position at the right hand side of the io_window on the line 
above the one currently containing the cursor. It is basically intended 
to be used as a position for an error message popup-notice if the
program finds some kind of error in an input text string.
-end*/
/*
*********************************
**     xdl_io_window_rootxy    **
*********************************

Purpose: Get the rootxy position at the right hand side of the io_window
         on the line above the one currently containing the cursor

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_ROOTXY (IVH, IXROOT, IYROOT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXROOT   (W)  Returns the required x root position (see xroot)
IYROOT   (W)  Returns the required y root position (see yroot)
IERR     (W)  Returns status from xdl_io_window_rootxy call
-end*/

/*-C:*/
int xdl_io_window_rootxy (vh, xroot, yroot)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *xroot;      /* Returns the required xroot position e.g for use with
                    an error notice after the input of an incorrect string
                    (W)*/
int *yroot;      /* Returns the required yroot position (W)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_io_window *gd;    /* Pointer to the global data area */
   int row;              /* Row for yroot calculation */
   int x;                /* x position to convert to xroot */
   int y;                /* y position to convert to yroot */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_io_window_rootxy",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1031)
   {
      xdl_view_type_errmsg ("xdl_io_window_rootxy", vh, "xdl_io_window");
      return 2;
   }

   /* Get the required position */

   gd = (XDl_io_window *) XDL_view_objects[iv]->gd;

   row = gd->curs_row - 1;
   if (row<1) row = 1;
   x = (gd->ch_width)*(gd->ncols) + 2*MARG_W;
   y = (row-1)*(gd->ch_height) + (gd->ch_height)/2 + MARG_H;
   xdl_rootxy (gd->wio, x, y, xroot, yroot);
   
   return 0;
}

/* Fortran binding xdlf_io_window_rootxy */

void xdlf_io_window_rootxy (ivh, ixroot, iyroot, ierr)
int *ivh, *ixroot, *iyroot, *ierr;
{
   *ierr = xdl_io_window_rootxy (*ivh, ixroot, iyroot);
   return;
}

/*-Routine: Calculate size requirements - xdl_io_window_getsize
The routine xdl_io_window_getsize (xdlf_io_window_getsize) is used to
get the required size for an I/O window view-object given the numbers
of rows and columns of characters required.
-end*/
/*
************************************
**     xdl_io_window_getsize      **
************************************

Purpose: Return the size requirements of the io-window view-object based
         on sizing specified by number of columns and rows of characters

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IO_WINDOW_GETSIZE (NCOLS, NROWS, IFONT, NSCROLL, IWIDTH, 
        +                             IHEIGHT)
-end*/
 
/*-Parameters:
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
NSCROLL   (R)   No. of scrolling pages or 0 if no scrolling;
                (any value >0 has the same effect)
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_io_window_getsize (ncols, nrows, font_type, nscroll, w, h)
/*end*/
 
/*-Parameters:*/
int       ncols;          /* Number of columns of characters ( > 0 ) (R)*/
int       nrows;          /* Number of rows of characters ( > 0 ) (R)*/
int       font_type;      /* Font type (1-5) for very-small, small, medium,
                             large or extra-large font. (R)*/
int       nscroll;        /* No. of scrolling pages to save */
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /* Font structure pointer */
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int min_strip_width;        /* Minimum width required for active strip
                                  + scrolling if used */
   int panel_width;            /* Scrolling control panel width */

   /* Calculate the size requirements */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (nscroll<0) nscroll = 0;

   /* Get size requirements */

   min_strip_width = MIN_ACT_STRIP;
   panel_width = 0;
   min_strip_width = min_strip_width + 2*BORDER_WIDTH + 2;
   if (nscroll>0) 
   {
      panel_width = PANEL_WIDTH;
      min_strip_width += panel_width + 4;
   }

   fontinfo = XDL_fonts[font_type-1];
   ch_width = fontinfo->max_bounds.width;
   ch_height = fontinfo->ascent + fontinfo->descent;
   *w = ncols*ch_width + 2*MARG_W + 2 + 2*BORDER_WIDTH + 2;
   if (*w<min_strip_width) *w = min_strip_width;
   *h = nrows*ch_height + 2*MARG_H + 2 + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2;
   return;
}

/* Fortran binding: xdlf_io_window_getsize */

void xdlf_io_window_getsize (ncols, nrows, ifont, nscroll, iwidth, iheight)
int *ncols, *nrows, *ifont, *nscroll, *iwidth, *iheight;
{
   xdl_io_window_getsize (*ncols, *nrows, *ifont, *nscroll, iwidth, iheight);
   return;
}
