/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: XDL_VIEW Layout Routines
-end*/
 
/*-Intro:
The xdl_view layout routines are designed to assist the layout of view-objects
for a program where several objects are to be fitted together to give
a tidy looking layout. The basic look of the layout may be preserved
and new positions and sizes automatically calculated if the size
requirements of any of the individual objects are changed. The basic
procedure requires that the layout is defined in terms of pairs of
objects and/or previously defined pairs. Each newly defined pair
may be in a horizontal or vertical direction. Minimum size
requirements are given for the individual objects and recursive
procedures are used to calculate the overall size requirement for the
layout and to reset the sizes and positions of the individual objects.
There is also an option to include a table (regular rows and columns
of single objects) as an item of a pair.
-end*/
 
/*-Section: Determining XDL_VIEW Layout Parameters
-end*/
 

/*Routines List:
The xdl_view layout routines available to the user are as follows:

   xdl_layout_init              Initialise the lists of 
                                singles/tables/pairs
   xdl_layout set_single        Give details for a single object
   xdl_layout_set_table         Give details for a table
   xdl_layout_set_pair          Give details for a pair
   xdl_layout                   Determine the layout
   xdl_layout_get_position      Get position/size of object in layout
   xdl_layout_reset_table       Reset object positions/sizes in a table
   xdl_layout_reset_single      Reset object position/size
*/

#include "xdl_view_ext.h"

/* Define routine types */

int xdl_layout_init();
int xdl_layout_set_single();
int xdl_layout_set_table();
int xdl_layout_set_pair();
int xdl_layout();
int xdl_layout_get_position();
int xdl_layout_reset_table();
int xdl_layout_reset_single();

void xdl_layout_get_pair_size();
int xdl_layout_find_name();
void xdl_layout_reset_sizes();


/* Define Fortran binding names */

#if   LINKTYP == 1

#define xdlf_layout_init xdlf_layout_init_
#define xdlf_layout_set_single xdlf_layout_set_single_
#define xdlf_layout_set_table xdlf_layout_set_table_
#define xdlf_layout_set_pair xdlf_layout_set_pair_
#define xdlf_layout xdlf_layout_
#define xdlf_layout_get_position xdlf_layout_get_position_
#define xdlf_layout_reset_table xdlf_layout_reset_table_
#define xdlf_layout_reset_single xdlf_layout_reset_single_

#endif

#if   LINKTYP == 2

#define xdlf_layout_init XDLF_LAYOUT_INIT
#define xdlf_layout_set_single XDLF_LAYOUT_SET_SINGLE
#define xdlf_layout_set_table XDLF_LAYOUT_SET_TABLE
#define xdlf_layout_set_pair XDLF_LAYOUT_SET_PAIR
#define xdlf_layout XDLF_LAYOUT
#define xdlf_layout_get_position XDLF_LAYOUT_GET_POSITION
#define xdlf_layout_reset_table XDLF_LAYOUT_RESET_TABLE
#define xdlf_layout_reset_single XDLF_LAYOUT_RESET_SINGLE

#endif

void xdlf_layout_init();
void xdlf_layout_set_single();
void xdlf_layout_set_table();
void xdlf_layout_set_pair();
void xdlf_layout();
void xdlf_layout_get_position();
void xdlf_layout_reset_table();
void xdlf_layout_reset_single();


#define MAX_IN_TABLE 100  /* Max no. of objects in a table of objects */
#define MAX_SINGLES 250   /* Max no. of single objects allowed */
#define MAX_PAIRS 24      /* Max no. of pairs allowed */
#define MAX_TABLES 10     /* Max no. of tables allowed */

/* Define Structures */

  typedef struct {
  char name[21];     /* Object name */
  int  hjust;        /* Horizontal justification if expand requested 
                        0=expand allowed, 
                        1=left justify,
                        2=right justify,
                        3=centre            */
  int vjust;         /* Vertical justification if expand requested */
  int used;          /* =1 used. =0 not yet used (to check that not used
                        more than once) */
  int x;             /* x offset to be used when creating the object */
  int y;             /* y offset to be used when creating the object */
  int w;             /* width value to use when creating the object */
  int h;             /* height value to use when creating the object */

                     /* When defining object, set x=0, y=0, w=minimum
                        width needed (or desired (if greater)), 
                        h=minimum height needed (or desired (if greater))
                     */
} Single;

  typedef struct {
  char name[21];     /* Name by which to identify the pair */
  int h_or_v;        /* = 0 horizontal pair, = 1 vertical pair */
  int just;          /* Justification code for the two children
                        = 1 Expand both children
                        = 2 Fix left (top), expand right (bottom)
                        = 3 Expand left (top), fix right (bottom)
                        = 4 Left/left (top/top) justify 
                        = 5 Left/right (top/bottom) justify
                        = 6 Right/right) (bottom/bottom) justify
                     */
  void * child1;     /* Pointer to child 1 structure */
  int type1;         /* Child 1 type 0=pair, 1=single */
  void * child2;     /* Pointer to child 2 structure */
  int type2;         /* Child 2 type 0=pair, 1=single */
  int sep;           /* Minimum separation bewteen children */
  int percent;       /* Percentage of width/height to be used by the
                        first if both are of justification 'expand' in the
                        the direction of the pairing */
  int used;          /* 1=used, 0=not used (to check that not used twice) */
  int w;             /* width needed (set to 0 or minimum desired width
                        when defining) */
  int h;             /* height needed (set to 0 or minimum desired height
                        when defining) */
  int x1;            /* x position of child 1 */
  int y1;            /* y position of child 1 */
  int w1;            /* width of child 1 */
  int h1;            /* height of child 1 */
  int x2;            /* x position of child 2 */
  int y2;            /* y position of child 2 */
  int w2;            /* width of child 2 */
  int h2;            /* height of child 2 */
                     /* When defining pair x1,y1,w1,h1,x2,y2,w2,h2 may all
                        be set to 0 */
} Pair;

typedef struct {
  char name[21];      /* Name of the table */
  int nrows;          /* Number of rows of objects in the table */
  int ncols;          /* Number of columns of objects in the table */
  int order;          /* Order of objects 1=column fastest, 2=row fastest */
  Single *singles[MAX_IN_TABLE]; /* Pointers to the objects in table (may have
                                    values of 0 for absent entries */
  int w;              /* Minimum width required for the table (may be 0) */
  int h;              /* Minimum height required for the table (may be 0) */
  int used;           /* 1=used, 0=not used (to check that not used twice) */

                      /* Justification codes: 0=expand
                                             1=left/top justify
                      */
}Table;

/* Counters and lists */

int ns;                   /* No. of single objects defined */
int np;                   /* No. of pairs defined */
int nt;                   /* No. of tables defined */

Single singles[MAX_SINGLES];
Pair pairs[MAX_PAIRS];
Table tables[MAX_TABLES];

/*-Routine: Initialise lists - xdl_layout_init
The routine xdl_layout_init (xdlf_layout_init) is used to initialise the
lists of single objects, tables of objects and layout pairs. It must be
called before using any other routine and again if it is desired to
re-initialise the lists.
-end*/
/*
*****************************
**     xdl_layout_init     ** 
*****************************

Purpose: Initialise counters for lists of single objects, pairs & tables

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_INIT (IERR)
-end*/
 
/*-Parameters:
IERR     (W)  Returns status from xdl_layout_init call
-end*/

/*-C:*/
int xdl_layout_init ()
/*end*/
 
/*-Parameters:*/
{
   ns = 0;
   nt = 0;
   np = 0;
   return 0;
}

/* Fortran binding: xdlf_layout_init */

void xdlf_layout_init (ierr)
int *ierr;
/*end*/
/*-Doc:
Return:  Status = 0 OK
-end*/
{
   *ierr = xdl_layout_init ();
   return;
}
/*-Routine: Define a single object - xdl_layout_set_single
The routine xdl_layout_set_single (xdlf_layout_set_single) is used to give
details of each single object to be included in the layout. The minimum
needed with and height must be given. Details are also given for what is
to happen if the space available for the object is expanded as the
layout is determined.
-end*/
/*
********************************
**     xdl_layout_set_single  ** 
********************************

Purpose: Add a single object to the list of single objects

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_SET_SINGLE (XDLSTR(NAME), LEN, IW, IH, 
        +                             IHJUST, IVJUST, IERR)
-end*/
 
/*-Parameters:
NAME     (R)  Name by which to identify the single object
              (max 20 chars)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the name (>0)
IW       (R)  Minimum width needed (e.g. from a ..getsize routine)
IH       (R)  Minimum height needed
IHJUST   (R)  Horizontal justification required if width is
              expanded.
              0=expand allowed, 
              1=left justify,
              2=right justify,
              3=centre     
IVJUST   (R)  Vertical justification required if height is
              expanded.
              0=expand allowed, 
              1=top justify,
              2=bottom justify,
              3=centre     
IERR     (W)  Returns status from xdl_layout_set_single call
-end*/

/*-C:*/
int xdl_layout_set_single(name, len, w, h, hjust, vjust)
/*end*/
 
/*-Parameters:*/
char *name;       /* Name used to identify the object (max 20 chars) (R)*/
int len;          /* Length of the name string (may be 0 if a null terminated
                     string is passed) (R)*/
int w;            /* Minimum width needed for the object (R)*/
int h;            /* Minimum height needed for the object (R)*/
int hjust;        /* Horizontal justification required if width is
                     expanded.
                     0=expand allowed, 
                     1=left justify,
                     2=right justify,
                     3=centre           (R)*/
int vjust;        /* Vertical justification required if height is
                     expanded.
                     0=expand allowed, 
                     1=top justify,
                     2=bottom justify,
                     3=centre           (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 name already used
                        =2 list of single objects full
-end*/
{
  void * pointer;    /* Temp pointer */
  char str[21];      /* Temp string */

  str[0] = 0;
  if (len>20) len = 20;
  if (len==0)
  {
     strcat (str, name);
  }
  else
  {
     strncat (str, name, len);
  }

  if (xdl_layout_find_name (str, &pointer)>=0) return 1;

  if (ns>=MAX_SINGLES) return 2;

  strcpy (singles[ns].name, str);
  singles[ns].hjust = hjust;
  singles[ns].vjust = vjust;
  singles[ns].w = w;
  singles[ns].h = h;
  singles[ns].x = 0;
  singles[ns].y = 0;
  singles[ns].used = 0;

  ns++;
  return 0;
}

/* Fortran binding: xdlf_layout_set_single */

void xdlf_layout_set_single (name, len, iw, ih, ihjust, ivjust, ierr)
int *name, *len, *iw, *ih, *ihjust, *ivjust, *ierr;
{
   *ierr = xdl_layout_set_single (XDL_pointers[*name], *len, *iw, *ih,
                                  *ihjust, *ivjust);
   return;
}
/*-Routine: Give details for a table - xdl_layout_set_table
The routine xdl_layout_set_table (xdlf_layout_set_table) is used to to give
details of a table of single objects set out in a series of rows and
columns. An equal amount of space will be allocated to each object though
the placing of the object within that space will depend on the justification
codes given when that single object is defined using xdl_layout_set_single
(xdlf_layout_set_single).
-end*/
/*
*******************************
**     xdl_layout_set_table  ** 
*******************************

Purpose: Give details of a table of objects

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_SET_TABLE (XDLSTR(NAME), LEN, NROWS, NCOLS,
        +                            XDLSTR(NAMES), NAMLEN, IORDER, 
        +                            IWIDTH, IHEIGHT,
        +                            IERR)
-end*/
 
/*-Parameters:
NAME       (R)  Name used to identify the table (max 20
                chars) 
LEN        (R)  Length of the naem string (>0)
NROWS      (R)  Number of rows of items
NCOLS      (R)  Number of columns of items
NAMES      (R)  Fortran array of character strings containing
                the single object names.
                [CHARACTER*NAMLEN NAMES(NITEMS)].
                ** Pass address using XDLSTR function **
                Can pass a blank string for an absent item.
                (see names) 
NAMLEN     (R)  Length of the single object names character strings
                (Must be >0 cf name_len)
IORDER     (R)  Order of items in the names list =1 column fastest,
                =2 row fastest 
IWIDTH     (R)  Mimimum width required for the table (may be 0) 
IHEIGHT    (R)  Minimum height required for the table (may be 0) 
IERR       (W)  Returns status from xdl_layout_set_table call
-end*/

/*-C:*/
int xdl_layout_set_table (name, len, nrows, ncols, names, name_len, 
                          order, w, h)
/*end*/
 
/*-Parameters:*/
char *name;      /* Name used to identify the table (max 20 chars) (R)*/
int len;         /* Length of the name (may be 0 if a null terminated
                    string is passed to the routine (R)*/
int nrows;       /* Number of rows of objects in the table (R)*/
int ncols;       /* Number of columns of objects in the table (R)*/
char * names;    /* Single object names - this may either an array of 
                    pointers to num_items null terminated character strings 
                    (len_name=0) or a character string num_items*len_name
                    in length with each name occupying len_name characters 
                    (left justified and padded to the right with blanks 
                    if needed) Can pass blank name string or null string 
                    for an absent item (R)*/
int name_len;    /* Item name length (0 = array of pointers to 
                    null-terminated character strings (e.g. for 'C' 
                    programs) or the length of each name string 
                    (e.g. for Fortran programs). See also the previous 
                    item (R) */
int order;       /* Order of items in the names list =1 column fastest,
                    =2 row fastest (R) */
int w;           /* Mimimum width required for the table (may be 0) (R)*/
int h;           /* Minimum height required for the table (may be 0) (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 name already used
                        =2 list of tables full
                        =3 single not defined
                        =4 single already used
-end*/
{
   void * pointer;          /* Temp pointer */
   Single * single;         /* Pointer to singles stricture */
   int i;                   /* Temp variable */
   int n;                   /* Loop counter through single objects */
   int nn;                  /* Total number of single objects */
   int wmax;                /* Max width of a single object */
   int hmax;                /* Max height of a single object */
   int blank;               /* Flag for blank or null string */
   int ioff;                /* Offset into names strings (Fortran type) */
   char str[21];            /* Temp string for name */  
   char * item_str;         /* Pointer to name strings for Fortran type
                               input */
   char ** item_strs;       /* Pointer to name strings for list of
                               null terminated strings type input */

  str[0] = 0;
  if (len>20) len = 20;
  if (len==0)
  {
     strcat (str, name);
  }
  else
  {
     strncat (str, name, len);
  }

  if (xdl_layout_find_name (str, &pointer)>=0) return 1;

  if (nt>=MAX_TABLES) return 2;

  strcpy (tables[nt].name, name);
  tables[nt].nrows = nrows;
  tables[nt].ncols = ncols;
  tables[nt].order = order;
  tables[nt].used = 0;
  nn = ncols*nrows;
  hmax = 0;
  wmax = 0;
  if (name_len>0) item_str = (char *) names;
  if (name_len==0) item_strs = (char **) names;
  ioff = 0;
  for (n=0;n<nn;++n)
  {
     if (name_len>0)
     {
        str[0] = 0;
        strncat (str, &item_str[ioff], name_len);
        blank = xdl_blank_string (str);
        ioff += name_len;
     }
     else
     {
        blank = 0;
        if (item_strs[n] ==0)
        {
           blank = 1;
        }
        else
        {
            strcpy (str, item_strs[n]);
            blank = xdl_blank_string (str);
        }
     }
     if (blank)
     {
        tables[nt].singles[n] = 0;
     }
     else
     {
        i = xdl_layout_find_name (str, &single);
        if (i<0) return 3;
        if (single->used==1) return 4;
        single->used = 1;
        tables[nt].singles[n] = single;
        if (single->w>wmax) wmax = single->w;
        if (single->h>hmax) hmax = single->h;
    }
  }
  tables[nt].w = ncols*wmax;
  tables[nt].h = nrows*hmax;
  if (w>tables[nt].w) tables[nt].w = w;
  if (h>tables[nt].h) tables[nt].h = h;
  nt++;
  return 0;
}

/* Fortran binding: xdlf_layout_set_table */

void xdlf_layout_set_table (name, len, nrows, ncols, names, namlen,
                            iorder, iwidth, iheight, ierr)
int *name, *len, *nrows, *ncols, *names, *namlen, *iorder, *iwidth, *iheight;
int *ierr;
{
   *ierr = xdl_layout_set_table (XDL_pointers[*name], *len, *nrows, *ncols,
                                XDL_pointers[*names], *namlen, 
                                *iorder, *iwidth, *iheight);
   return;
}
/*-Routine: Give details of layout pair - xdl_layout_set_pair
The routine xdl_layout_set_pair (xdlf_layout_set_pair) is used to go give 
details of a layout pair. The full layout is made by combining pairs
in succession. Each pair in the layout may be either horizontal or
vertical. Each of the two children of a pair may be another pair,
a single object or a table of single objects. 
-end*/
/*
*****************************
**     xdl_layout_set_pair ** 
*****************************

Purpose: Define details of a layout pair

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_SET_PAIR (IHORV, JUST, XDLSTR(NAME), LEN,
        +                          XDLSTR(NAME_CH1), LEN1, 
        +                          XDLSTR(NAME_CH2), LEN2,
        +                          IWIDTH, IHEIGHT, ISEP, IPCNT, IERR)
-end*/
 
/*-Parameters:
IHORV    (R)  =0 horizontal pair, =1 vertical pair
JUST     (R)  Layout/justification code for the two children
               = 1 Expand both children
               = 2 Fix left (top), expand right (bottom)
               = 3 Expand left (top), fix right (bottom)
               = 4 Left/left (top/top) justify 
               = 5 Left/right (top/bottom) justify
               = 6 Right/right) (bottom/bottom) justify
NAME     (R)  Name string used to identify the pair (max 20 chars)
              ** Pass address using XDLSTR function **
LEN      (R)  Length of the name string (>0)
NAME_CH1 (R)  Name string of the first child (previously set
              single, table or another pair)
              ** Pass address using XDLSTR function **
LEN1     (R)  Length of name for child 1 (>0)
NAME_CH2 (R)  Name string of the second child (previously set
              single, table or another pair)
              ** Pass address using XDLSTR function **
LEN2     (R)  Length of name for child 2 (>0)
IWIDTH   (R)  Minumum width required (may be 0)
IHEIGHT  (R)  Minimum height required (may be 0)
ISEP     (R)  Separation required between the halves of the pair
IPCNT    (R)  Suggested allocation of width/height between the
              two halves of the pair if both expanded.
IERR     (W)  Returns status from xdl_layout_set_pair call
-end*/

/*-C:*/
int xdl_layout_set_pair (h_or_v, just, name, len, name_child1, len1,
                         name_child2, len2, w, h, sep, percent)
/*end*/
 
/*-Parameters:*/
int h_or_v;            /* =0 horizontal pair, =1 vertical pair (R) */
int just;              /* Layout/justification code for the two children
                        = 1 Expand both children
                        = 2 Fix left (top), expand right (bottom)
                        = 3 Expand left (top), fix right (bottom)
                        = 4 Left/left (top/top) justify 
                        = 5 Left/right (top/bottom) justify
                        = 6 Right/right) (bottom/bottom) justify
                        (R) */
char *name;            /* Name used to identify the pair (max 20 chars) 
                          (R)*/
int len;               /* Length of name; may be 0 if 'name' is a null 
                          terminated string (R) */
char *name_child1;     /* Name of first (previously defined) child (R)*/
int len1;              /* Length of this name; may be 0 if name_child1
                          is a null terminated string (R)*/
char *name_child2;     /* Name of second (previously defined) child (R)*/
int len2;              /* Length of this name; may be 0 if name_child2
                          is a null terminated string (R)*/
int w;                 /* Minimum width required (may be 0) (R)*/
int h;                 /* Minimum height required (may be 0) (R)*/
int sep;               /* Separation required between the halves of the
                          pair (R)*/
int percent;           /* Suggested allocation of width/height between the
                          two halves of the pair if both expanded (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 name already used
                        =2 list of pairs full
                        =3 child 1 not yet defined
                        =4 child 1 already used
                        =5 child 2 not yet defined
                        =6 child 2 already used
-end*/
{
   void * pointer;     /* Temp pointer */
   Single * single;    /* Temp pointer to single */
   Table *table;       /* Temp pointer to table */
   Pair *pair;         /* Temp pointer to pair */
   int i;              /* Temp variable */
   char str[21];       /* Temp string for a name */

   str[0] = 0;
   if (len>20) len = 20;
   if (len==0)
   {
      strcat (str, name);
   }
   else
   {
      strncat (str, name, len);
   }

   if (xdl_layout_find_name (str, &pointer)>=0) return 1;

   if (np>=MAX_PAIRS) return 2;

   if (h_or_v!=0) h_or_v = 1;
   pairs[np].h_or_v = h_or_v;
   pairs[np].just = just;
   strcpy (pairs[np].name, str);
   pairs[np].w = w;
   pairs[np].h = h;

   str[0] = 0;
   if (len1>20) len1 = 20;
   if (len1==0)
   {
      strcat (str, name_child1);
   }
   else
   {
      strncat (str, name_child1, len1);
   }

   i = xdl_layout_find_name (str, &pointer);
   if (i<0) return 3;
   pairs[np].type1 = i;
   switch (i)
   {
      case 0:
        pair = (Pair *) pointer;
        if (pair->used==1) return 4;
        pair->used = 1;
      break;
      case 1:
        single = (Single *) pointer;
        if (single->used==1) return 4;
        single->used = 1;
      break;
      case 2:
        table = (Table *) pointer;
        if (table->used==1) return 4;
        table->used = 1;
      break;
   }
   pairs[np].type1 = i;
   pairs[np].child1 = pointer;
 
   str[0] = 0;
   if (len2>20) len2 = 20;
   if (len2==0)
   {
      strcat (str, name_child2);
   }
   else
   {
      strncat (str, name_child2, len2);
   }
 
   i = xdl_layout_find_name (str, &pointer);
   if (i<0) return 5;
   pairs[np].type2 = i;
   switch (i)
   {
      case 0:
        pair = (Pair *) pointer;
        if (pair->used==1) return 6;
        pair->used = 1;
      break;
      case 1:
        single = (Single *) pointer;
        if (single->used==1) return 6;
        single->used = 1;
      break;
      case 2:
        table = (Table *) pointer;
        if (table->used==1) return 6;
        table->used = 1;
      break;
   }
   pairs[np].type2 = i;
   pairs[np].child2 = pointer;
 
   pairs[np].sep = sep;
   pairs[np].percent = percent;
   pairs[np].x1 = 0;
   pairs[np].y1 = 0;
   pairs[np].w1 = 0;
   pairs[np].h1 = 0;
   pairs[np].x2 = 0;
   pairs[np].y2 = 0;
   pairs[np].w2 = 0;
   pairs[np].h2 = 0;
   pairs[np].used = 0;
 
   np++;
   return 0;
}

/* Fortran binding: xdlf_layout_set_pair */

void xdlf_layout_set_pair (ihorv, just, name, len, name_ch1, len1,
                           name_ch2, len2, iwidth, iheight,
                           isep, ipcnt, ierr)
int *ihorv, *just, *name, *len, *name_ch1, *len1;
int *name_ch2, *len2, *iwidth, *iheight, *isep, *ipcnt;
int *ierr;
{
   *ierr = xdl_layout_set_pair (*ihorv, *just, XDL_pointers[*name], *len,
                                XDL_pointers[*name_ch1], *len1,
                                XDL_pointers[*name_ch2], *len2,
                                *iwidth, *iheight, *isep, *ipcnt);
   return;
}
/*-Routine: Determine the layout - xdl_layout
The routine xdl_layout (xdlf_layout) is used to determine the layout
from the previously defined layout pairs, tables of objects and single
objects. The position and size parameters for each individual object
can then be found using the routine xdl_layout_get_position
(xdlf_layout_get_position)
-end*/
/*
*************************
**     xdl_layout      ** 
*************************

Purpose: Determine the layout

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT (XDLSTR(NAME), LEN, IX, IY, IWIDTH, IHEIGHT,
        +                  IERR)
-end*/
 
/*-Parameters:
NAME     (R)  Name string used to identify the top level
              pair  for the layout (max 20 chars)
              ** Pass address using XDLSTR function **
LEN      (R)  Length of the name string (>0)
IX       (R)  Required x coordinate for the top left of the layout 
IY       (R)  Required y coordinate for the top left of the layout 
IWIDTH   (W)  Returns the calculated width of the layout 
IHEIGHT  (W)  Returns the calculated height of the layout 
IERR     (W)  Returns status from xdl_layout call
-end*/

/*-C:*/
int xdl_layout (name, len, x, y, w, h)
/*end*/
 
/*-Parameters:*/
char *name;       /* Name of the top level pair for the layout (R)*/
int len;          /* Length of the name (may be 0 if the name string is
                     null terminated) (R)*/
int x;            /* Required x coordinate for the top left of the layout 
                     (R)*/
int y;            /* Required y coordinate for the top left of the layout 
                     (R)*/
int *w;           /* Returns the calculated width of the layout (W)*/
int *h;           /* Returns the calculated height of the layout (W)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 top level pair not found
-end*/
{
   char str[21];      /* Temp character string */
   void * pointer;    /* Temp pointer */
   Pair *pair;        /* Temp pointer to pair */

   str[0] = 0;
   if (len>20) len = 20;
   if (len==0)
   {
      strcat (str, name);
   }
   else
   {
      strncat (str, name, len);
   }

   if (xdl_layout_find_name (str, &pointer)!=0) return 1;

   pair = (Pair *) pointer;

   xdl_layout_get_pair_size (pair, w, h);

   xdl_layout_reset_sizes (pair, x, y);

   return 0;
}

/* Fortran binding: xdlf_layout */

void xdlf_layout (name, len, ix, iy, iwidth, iheight, ierr)
int *name, *len, *ix, *iy, *iwidth, *iheight, *ierr;
{
   *ierr = xdl_layout (XDL_pointers[*name], *len, *ix, *iy, iwidth, iheight);
   return;
}
/*-Routine: Get object position and size - xdl_layout_get_position
The routine xdl_layout_get_position (xdlf_layout_get_position) is used to 
get the position and size for each individual single object after the
layout has been determined using the xdl_layout (xdlf_layout) routine.
-end*/
/*
**********************************
**     xdl_layout_get_position  ** 
**********************************

Purpose: Get the calculated position and size of a single object

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_GET_POSITION (XDLSTR(NAME), LEN, IX, IY,
                                       IWIDTH, IHEIGHT, IERR)
-end*/
 
/*-Parameters:
NAME     (R)  Name of the single object
              ** Pass address using XDLSTR function **
LEN      (R)  Length of the name string (>0)
IX       (W)  Returns the x coordinate for the top left of the 
              single object
IY       (W)  Returns the y coordinate for the top left of the 
              single object
IWIDTH   (W)  Returns the calculated width of the single object
IHEIGHT  (W)  Returns the calculated height of the single object
IERR     (W)  Returns status from xdl_layout_get_position call
-end*/

/*-C:*/
int xdl_layout_get_position (name, len, x, y, w, h)
/*end*/
 
/*-Parameters:*/
char *name;     /* The name of the single object (R)*/
int len;        /* The length of the string (may be 0 if a null terminated
                   string was given) (R)*/
int *x;         /* Returns the x coordinate for the top left of the 
                   single object (W) */
int *y;         /* Returns the y coordinate for the top left of the 
                   single object (W) */
int *w;         /* Returns the calculated width of the single object (W)*/
int *h;         /* Returns the calculated height of the single object (W)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 object not found (or not a single object)
-end*/
{
   char str[21];      /* Temp character string */
   void * pointer;    /* Temp pointer */
   Single *single;    /* Temp pointer to single */

   str[0] = 0;
   if (len>20) len = 20;
   if (len==0)
   {
      strcat (str, name);
   }
   else
   {
      strncat (str, name, len);
   }

   if (xdl_layout_find_name (str, &pointer)!=1) return 1;

   single = (Single *) pointer;

   *x = single->x;
   *y = single->y;
   *w = single->w;
   *h = single->h;

   return 0;
}

/* Fortran binding: xdlf_layout_get_position */

void xdlf_layout_get_position (name, len, ix, iy, iwidth, iheight, ierr)
int *name, *len, *ix, *iy, *iwidth, *iheight;
int *ierr;
{
   *ierr = xdl_layout_get_position (XDL_pointers[*name], *len, ix, iy,
                                    iwidth, iheight);
   return;
}
/*-Routine: Reset objects in table - xdl_layout_reset_table
The routine xdl_layout_reset_table (xdlf_layout_reset_table) is used to 
reset the positions of the objects within a table after the required size 
and position of the table have been determined. It will normally be
called internally via the xdl_layout (xdlf_layout) routine but is made
available for user access if required just to layout a table.
-end*/
/*
*********************************
**     xdl_layout_reset_table  ** 
*********************************

Purpose: Set the single objects positions for a table

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_RESET_TABLE (XDLSTR(NAME), LEN, IX, IY,
                                      IWIDTH, IHEIGHT, IERR)
-end*/
 
/*-Parameters:
NAME     (R)  Name of the table
              ** Pass address using XDLSTR function **
LEN      (R)  Length of the name string (>0)
IX       (R)  Required x coordinate for the top left of the table
IY       (R)  Required y coordinate for the top left of the table
IWIDTH   (R)  Required width of the table 
              (>= min width required)
IHEIGHT  (R)  Required height of the table 
              (>= min height required)
IERR     (W)  Returns status from xdl_layout_reset_table call
-end*/

/*-C:*/
int xdl_layout_reset_table (name, len, x, y, w, h)
/*end*/
 
/*-Parameters:*/
char *name;       /* Name of the table (R)*/
int len;          /* Length of the name (may be 0 if the name string is
                     null terminated (R)*/
int x;            /* Required x position for the top left of the table (R)*/
int y;            /* Required y position for the top left of the table (R)*/
int w;            /* Required width of the table (must be >= minimum width 
                     needed) (R) */
int h;            /* Required height of the table (must be >= minimum 
                     height needed) (R) */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 name not found or not a table
-end*/
{

   int wcol;          /* Width of a column */
   int hrow;          /* Height of a row */
   int ir;            /* Loop counter through rows */
   int ic;            /* Loop counter through columns */
   int ii;            /* Offset into list of singles */
   char str[21];      /* Temp character string */
   void * pointer;    /* Temp pointer */
   Table *table;      /* Temp pointer to table */

   str[0] = 0;
   if (len>20) len = 20;
   if (len==0)
   {
      strcat (str, name);
   }
   else
   {
      strncat (str, name, len);
   }

   if (xdl_layout_find_name (str, &pointer)!=2) return 1;

   table = (Table *) pointer;

   wcol = w / table->ncols;
   hrow = h / table->nrows;
   for (ir=0;ir<table->nrows;++ir)
   {
      for (ic=0;ic<table->ncols;++ic)
      {
         if (table->order==1)
         {
            ii = ir*table->ncols + ic;
         }
         else
         {
            ii=ic*table->nrows + ir;
         }
         xdl_layout_reset_single (table->singles[ii]->name, 0, 
                                 x+(ic*wcol), y+(ir*hrow), wcol, hrow);
      }
   }
   return 0;
}

/* Fortran binding: xdlf_layout_reset_table */

void xdlf_layout_reset_table (name, len, ix, iy, iwidth, iheight, ierr)
int *name, *len, *ix, *iy, *iwidth, *iheight;
int *ierr;
{
   *ierr = xdl_layout_reset_table (XDL_pointers[*name], *len, *ix, *iy, 
                                 *iwidth, *iheight);
   return;
}
/*-Routine: Reset single object position - xdl_layout_reset_single
The routine xdl_layout_reset_single (xdlf_layout_reset_single) is used to 
reset the position of a single object after the required size 
and position of the object have been determined. It will normally be
called internally via the xdl_layout (xdlf_layout) routine but is made
available for user access if required just to reset the position
of a single object.
-end*/
/*
*********************************
**     xdl_layout_reset_single ** 
*********************************

Purpose: Reset a single object position in an expanded field

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_LAYOUT_RESET_SINGLE (XDLSTR(NAME), LEN, IX, IY,
                                      IWIDTH, IHEIGHT, IERR)
-end*/
 
/*-Parameters:
NAME     (R)  Name of the single object
              ** Pass address using XDLSTR function **
LEN      (R)  Length of the name string (>0)
IX       (R)  Required x coordinate for the top left of the object
IY       (R)  Required y coordinate for the top left of the object
IWIDTH   (R)  Required width of the object 
              (>= min width required)
IHEIGHT  (R)  Required height of the object 
              (>= min height required)
IERR     (W)  Returns status from xdl_layout_reset_single call
-end*/

/*-C:*/
int xdl_layout_reset_single (name, len, x, y, w, h)
/*end*/
 
/*-Parameters:*/
char *name;       /* Name of the single object (R)*/
int len;          /* Length of the name (may be 0 if the name string is
                     null terminated (R)*/
int x;            /* Required x position for the top left of the object (R)*/
int y;            /* Required y position for the top left of the object (R)*/
int w;            /* Required width of the object (must be >= minimum width 
                     needed) (R) */
int h;            /* Required height of the object (must be >= minimum 
                     height needed) (R) */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                        =1 name not found or not a single object
-end*/
{

   char str[21];      /* Temp character string */
   void * pointer;    /* Temp pointer */
   Single *single;    /* Temp pointer to single */

   str[0] = 0;
   if (len>20) len = 20;
   if (len==0)
   {
      strcat (str, name);
   }
   else
   {
      strncat (str, name, len);
   }

   if (xdl_layout_find_name (str, &pointer)!=1) return 1;

   single = (Single *) pointer;

   /*Horizontal setting */
   switch (single->hjust)
   {
      case 0: /*Expand*/
         single->x = x;
         single->w = w;
      break;

      case 1: /*Left justify*/
         single->x = x;
      break;

      case 2: /*Right justify*/
         single->x = x + w - single->w;
      break;

      case 3: /*Centre */
         single->x = x + (w-single->w)/2;
      break;
   }

   /* Vertical setting*/
   switch (single->vjust)
   {
      case 0: /*Expand*/
         single->y = y;
         single->h = h;
      break;

      case 1: /*Top justify*/
         single->y = y;
      break;

      case 2: /*Bottom justify*/
         single->y = y + h - single->h;
      break;

      case 3: /*Centre */
         single->y = y + (h-single->h)/2;
      break;
   }
   return 0;
}

/* Fortran binding: xdlf_layout_reset_single */

void xdlf_layout_reset_single (name, len, ix, iy, iwidth, iheight, ierr)
int *name, *len, *ix, *iy, *iwidth, *iheight;
int *ierr;
{
   *ierr = xdl_layout_reset_single (XDL_pointers[*name], *len, *ix, *iy, 
                                 *iwidth, *iheight);
   return;
}

/*
*-------------------------------------------------------------*
*                   SERVICE  ROUTINES                         *
*-------------------------------------------------------------*
*/

/*
***********************************
**     xdl_layout_get_pair_size  **
***********************************

Purpose: Get pair size (recursively)

Author:  John W. Campbell, August 1993

*/

void xdl_layout_get_pair_size (pair, width, height)

Pair * pair;  /* Pointer to the pair structure (R)*/
int *width;   /* Minimum width required (may be 0) (R)*/
int *height;  /* Minimum height required (may be 0) (R)*/
{
    Pair *pair1;
    Pair *pair2;
    Single *single1;
    Single *single2;
    Table *table1;
    Table *table2;
    int w1, h1, w2, h2, w, h, ww, hh, soff, s2min,s2max;

    switch (pair->type1)
    {
       case 0:
          pair1 = (Pair *) pair->child1;
          xdl_layout_get_pair_size (pair1, &w1, &h1);
       break;
       case 1:
          single1 = (Single *) pair->child1;
          w1 = single1->w;
          h1 = single1->h;
       break;
       case 2:
          table1 = (Table *) pair->child1;
          w1 = table1->w;
          h1 = table1->h;
       break;
    } 
    switch (pair->type2)
    {
       case 0:
          pair2 = (Pair *) pair->child2;
          xdl_layout_get_pair_size (pair2, &w2, &h2);
       break;
       case 1:
          single2 = (Single *) pair->child2;
          w2 = single2->w;
          h2 = single2->h;
       break;
       case 2:
          table2 = (Table *) pair->child2;
          w2 = table2->w;
          h2 = table2->h;
       break;
    } 
    if (pair->h_or_v==0)
    /* Horizontal pairing */
    {
       h = h1;
       if (h2>h1) h = h2;
       if (h>pair->h) pair->h = h;
       pair->h1 = pair->h;
       pair->h2 = pair->h;
       w = w1+w2+pair->sep;
       if (w>pair->w) pair->w = w;
       pair->x1 = 0;
       pair->y1 = 0;
       pair->y2 = 0;

       switch (pair->just)
       {
          case 1:
          /* Expand/expand */
             s2min = w1 + pair->sep;
             s2max = pair->w - w2;
             ww = pair->percent*pair->w/100 + pair->sep/2;
             if (ww>=s2min&&ww<=s2max)
             {
                soff = ww;
             }else{
                soff = s2min + (s2max-s2min)/2;
             }
             pair->x2 = soff;
             pair->w1 = soff - pair->sep;
             pair->w2 = pair->w - soff;
          break;

          case 2:
          /* Left/expand */
             pair->x2 = w1 + pair->sep;
             pair->w1 = w1;
             pair->w2 = pair->w - w1 - pair->sep;
          break;

          case 3:
          /* Expand/right */
             pair->w1 = pair->w - w2 - pair->sep;
             pair->w2 = w2;
             pair->x2 = pair->w - w2;
          break;

          case 4:
          /* Left/left */
             pair->x2 = w1 + pair->sep;
             pair->w1 = w1;
             pair->w2 = w2;
          break;

          case 5:
          /* Left/right */
             pair->x2 = pair->w - w2;
             pair->w1 = w1;
             pair->w2 = w2;
          break;

          case 6:
          /* Right/right */
             pair->x1 = pair->w - w2 - pair->sep - w1;
             pair->x2 = pair->w - w2;
             pair->w1 = w1;
             pair->w2 = w2;
          break;
       }
    }else{
    /* Vertical pairing */
       w = w1;
       if (w2>w1) w = w2;
       if (w>pair->w) pair->w = w;
       pair->w1 = pair->w;
       pair->w2 = pair->w;
       h = h1+h2+pair->sep;
       if (h>pair->h) pair->h = h;
       pair->x1 = 0;
       pair->x2 = 0;
       pair->y1 = 0;
       switch (pair->just)
       {
          case 1:
          /* Expand/expand */
             s2min = h1 + pair->sep;
             s2max = pair->h - h2;
             hh = pair->percent*pair->h/100 + pair->sep/2;
             if (hh>=s2min&&hh<=s2max)
             {
                soff = hh;
             }else{
                soff = s2min + (s2max-s2min)/2;
             }
             pair->y2 = soff;
             pair->h1 = soff - pair->sep;
             pair->h2 =pair->h - soff;
          break;

          case 2:
          /* Top/expand */
             pair->y2 = h1 + pair->sep;
             pair->h1 = h1;
             pair->h2 = pair->h - h1 - pair->sep;
          break;

          case 3:
          /* Expand/bottom */
             pair->h1 = pair->h - h2 - pair->sep;
             pair->h2 = h2;
             pair->y2 = pair->h - h2;
          break;

          case 4:
          /* Top/top */
             pair->y2 = h1 + pair->sep;
             pair->h1 = h1;
             pair->h2 = h2;
          break;

          case 5:
          /* Top/bottom */
             pair->y2 = pair->h - h2;
             pair->h1 = h1;
             pair->h2 = h2;
          break;

          case 6:
          /* Bottom/bottom */
             pair->y1 = pair->h - h2 - pair->sep - h1;
             pair->y2 = pair->h - h2;
             pair->h1 = h1;
             pair->h2 = h2;
          break;
       }
    }

    /* Set updated width/height */
    *width = pair->w;
    *height = pair->h;
    return;
}
/*
*******************************
**     xdl_layout_find_name  **
*******************************

Purpose: Find single/ table/ pair in lists

Author:  John W. Campbell, August 1993

*/

int xdl_layout_find_name (name, point)

char * name;     /* Name to be looked for */
void ** point;   /* Returns pointer to single/table/pair if found */
{
   int i;
   if (nt>0)
   {
      for (i=0;i<nt;++i)
      {
         if (strcmp(tables[i].name,name)==0){
            *point=(void *)&tables[i];
            return 2;
         }
      }
   }
   if (ns>0)
   {
      for (i=0;i<ns;++i)
      {
         if (strcmp(singles[i].name,name)==0){
            *point=(void *)&singles[i];
            return 1;
         }
      }
   }
   if (np>0)
   {
      for (i=0;i<np;++i)
      {
         if (strcmp(pairs[i].name,name)==0){
            *point=(void *) &pairs[i];
            return 0;
         }
      }
   }
   return -1;
}
/*
**********************************
**     xdl_layout_reset_sizes   **
**********************************

Purpose: Reset sizes for the layout (recursively)

Return:  None

Author:  John W. Campbell, August 1993

*/

void xdl_layout_reset_sizes (pair, x, y)

Pair *pair;   /* Top level pair pointer (R)*/
int x;        /* x position for top left of layout (R)*/
int y;        /* y position for top left of layout (R)*/
{
    int ww, hh, soff, s2min,s2max;
    Pair *pair1;
    Pair *pair2;
    Single *single1;
    Single *single2;
    Table *table1;
    Table *table2;
    /*Adjust settings as needed*/
    pair->x1 = pair->x1 + x;
    pair->y1 = pair->y1 + y;
    pair->x2 = pair->x2 + x;
    pair->y2 = pair->y2 + y;
    if (pair->h_or_v==0){
       /*Horizontal pair*/
       if (pair->h>pair->h1) pair->h1 = pair->h;
       if (pair->h>pair->h2) pair->h2 = pair->h;
       
       switch (pair->just)
       {
           case 1:
           /* Expand/expand */
             s2min = pair->w1 + pair->sep;
             s2max = pair->w - pair->w2;
             ww = pair->percent*pair->w/100 + pair->sep/2;
             if (ww>=s2min&&ww<=s2max)
             {
                soff = ww;
             }else{
                soff = s2min + (s2max-s2min)/2;
             }
             pair->x2 = pair->x1+soff;
             pair->w1 = soff - pair->sep;
             pair->w2 =pair->w - soff;
          break;

          case 2:
          /* Left/expand */
             pair->w2=pair->w-pair->w1-pair->sep;
          break;

          case 3:
          /* Expand/right */
             pair->w1 = pair->w-pair->w2-pair->sep;
             pair->x2 = pair->x1+pair->w-pair->w2;
          break;

          case 4:
          /* Left/left */
          case 5:
          /* Left/right */
          case 6:
          /* Right/right */
          break;
       }
    }else{
       /*Vertical pair*/
       if (pair->w>pair->w1) pair->w1 = pair->w;
       if (pair->w>pair->w2) pair->w2 = pair->w;

       switch (pair->just)
       {
          case 1:
          /* Expand/expand */
             s2min = pair->h1 + pair->sep;
             s2max = pair->h - pair->h2;
             hh = pair->percent*pair->h/100 + pair->sep/2;
             if (hh>=s2min&&hh<=s2max)
             {
                soff = hh;
             }else{
                soff = s2min + (s2max-s2min)/2;
             }
             pair->y2 = pair->y1+soff;
             pair->h1 = soff - pair->sep;
             pair->h2 =pair->h - soff;
          break;

          case 2:
          /* Top/expand */
             pair->h2 = pair->h-pair->h1-pair->sep;
          break;

          case 3:
          /* Expand/bottom */
             pair->h1 = pair->h-pair->h2-pair->sep;
             pair->y2 = pair->y1+pair->h-pair->h2;
          break;

          case 4:
          /* Top/top */
          case 5:
          /* Top/bottom */
          case 6:
          /* Bottom/bottom */
          break;
       }
     }
     switch (pair->type1)
     {
        case 0:
           pair1 = (Pair *) pair->child1;
           pair1->w = pair->w1;
           pair1->h = pair->h1;
           xdl_layout_reset_sizes (pair1, pair->x1, pair->y1);
        break;
        case 1:
           single1 = (Single *) pair->child1;
           xdl_layout_reset_single (single1->name, 0, pair->x1, pair->y1, 
                                    pair->w1, pair->h1);
        break;
        case 2:
           table1 = (Table *) pair->child1;
           xdl_layout_reset_table (table1->name, 0, pair->x1, pair->y1, 
                                   pair->w1, pair->h1);
        break;
     } 
     switch (pair->type2)
     {
        case 0:
           pair2 = (Pair *) pair->child2;
           pair2->w = pair->w2;
           pair2->h = pair->h2;
           xdl_layout_reset_sizes (pair2, pair->x2, pair->y2);
        break;
        case 1:
           single2 = (Single *) pair->child2;
           xdl_layout_reset_single (single2->name, 0, pair->x2, pair->y2, 
                                    pair->w2, pair->h2);
        break;
        case 2:
           table2 = (Table *) pair->child2;
           xdl_layout_reset_table (table2->name, 0, pair->x2, pair->y2, 
                                   pair->w2, pair->h2);
        break;
    }  
    return;
}
