/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Menu Area View-Object Routines
-end*/
 
/*-Intro:
The menu area view-object provides an area containing a menu from which
items may be selected by the user by means of the mouse. Typically, an
application will use the same menu area for a series of different
menus. The menu area has an optional title. It also has an optional extra
menu item situated below the other menu items and separated from them.
This item is typically used either as a 'Quit' item or as an item
which will instruct the program to return to the previously displayed
menu. The menu area also has an 'active strip' at the top of the area.
-end*/
 
/*-Section: Creating and Handling a Menu Area View-Object
-end*/
 

/*Routines List:
The menu area view-object has the following routines available:

   xdl_menu_area                 Create a new menu area view-object
   xdl_menu_area_setmenu         Set a new menu in the menu area
   xdl_menu_area_getitem         Get the selected menu item number
   xdl_menu_area_getrootxy       Get the position of the selected item
   xdl_menu_area_getsize         Calculate the size requirements
*/

/*-Routine: Create a menu area view-object - xdl_menu_area
The routine xdl_menu_area (xdlf_menu_area) is used to create a new menu
area view-object. The maximum lengths for text strings and the maximum 
number of items are give. The actual menu items are subsequently added
using the xdl_menu_area_setmenu (xdlf_menu_area_setmenu) routine.
-end*/
/*
*************************
**    xdl_menu_area    **
*************************

Purpose: Display a menu area with replaceable menu's

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_MENU_AREA (IVH, IVHPARENT, IX, IY, ICSET, MAXITEMS, 
        +                     MAXLEN, IFONT, IQUIT, MAXTITL, MINWIDTH, 
        +                     MINHEIGHT, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
MAXITEMS  (R)   Max. no. of menu items (see max_items)
MAXLEN    (R)   Max. no. chars in an item name 
                (see max_name_chars)
IFONT     (R)   Font type (1->5 verysmall->extralarge) 
                (see font_type)
IQUIT     (R)   =1 allow, =0 disallow quit box (see quit_box)
MAXTITL   (R)   Maximum title length (see max_title_len)
MINWIDTH  (R)   Minimum width or 0 (see min_width)
MINHEIGHT (R)   Minimum height or 0 (see min_height)
IERR      (W)   Returns status from xdl_menu_area call

-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

#define MARG_W 2
#define MARG_H 2
#define MIN_ITEM_MARG_W 3    /*Minimum left/right margin around item name -
                               in pixels*/
#define MIN_ITEM_MARG_H 3    /*Minimum top/bottom margin around item name -
                               in pixels*/

/* Subjective layout parameters */

#define MAX_ITEM_MARG_W 4        /*Maximum left/right margin around longest item
                                   name - in characters*/
#define MAX_ITEM_MARG_H 1        /*Maximum top/bottom margin around item name -
                                   in characters*/

/*Spacing ratios if quit box present*/
#define TOP_SPACE 3              /*Spacing ratio (wrt MIDDLE_SPACE, 
                                   BOTTOM_SPACE)*/
#define MIDDLE_SPACE 6           /*Spacing ratio (wrt TOP_SPACE, BOTTOM_SPACE)*/
#define BOTTOM_SPACE 4           /*Spacing ratio (wrt TOP_SPACE, MIDDLE_SPACE)*/

/*Spacing ratios if quit box absent*/
#define T_SP 3                   /*Spacing ratio for top space wrt B_SP*/
#define B_SP 4                   /*Spacing ratio for bottom space wrt T_SP*/

/*Best amount of clear space*/
#define BEST_PERCENT_SPACE 20    /*Best percentage clear space*/

/* Define structure to hold preset info for each item */

typedef struct
{
   int toff_name;          /*Offset in text array to menu item name*/
                           /*The following are offsets in the menu area canvas*/
   int xoff;               /*x position (top left) of item area*/
   int yoff;               /*y position (top left) of item area*/
   int xoff_name;          /*x position (top left) of item name*/
   int yoff_name;          /*y position (top left) of item name*/
   int name_len;           /*Length of the name in characters*/
}
   XDl_menu_area_item_details;

/* Define structure to hold font details */

typedef struct
{
   int         fid;                        /*Font id (normal text)*/
   int         fid_bold;                   /*Font id (bold text)*/
   int         font_descent;               /*Font descent*/
   int         font_ascent;                /*Font ascent*/
   int         ch_width;                   /*Font maximum character width*/
}
   XDl_menu_area_font;

/* Define structure for the global data for the view-object */

typedef struct
{
   Window      wmenu;                      /*Menu (drawn on) window*/
   Window      wact;                       /*Active strip area window*/
   GC          paint_gc;                   /*GC for menu area area use*/
   char        *text;                      /*Array to hold menu item names */
   char        *title;                     /*Menu title array*/
   int         area_width;                 /*Area width in pixels*/
   int         area_height;                /*Area height in pixels*/
   int         max_items;                  /*Maximum number of menu items*/
   int         max_name_chars;             /*Max no. chars in an item name*/
   int         title_allowed;              /*=1, space allowed for title,
                                             =0, not allowed*/
   int         max_title_len;              /*Max allowed title length*/
   int         quit_box_allowed;           /*=1, space allowed for quit box, 
                                             =0, not allowed*/
   XDl_menu_area_font menu_fonts[5];       /*Font details*/
   int         on_off;                     /*Program awaits input T/F flag*/
   int         repaint_done;               /*Initial repaint done flag*/
   unsigned long highlight_pixel;          /*Color (if available) for 
                                             highlighting the last one input*/

                                    /* Following are for the current menu */

   int         num_items;                  /*No. of items in the menu*/
   int         xoff_items_start;           /*x offset in area to start of items
                                             */
   int         xoff_items_end;             /*x offset in area to end of items */
   int         yoff_items_start;           /*y offset in area to top of first
                                             item*/
   int         yoff_items_end;             /*y offset in area to end of items*/
   int         yoff_quit_start;            /*y offset in area to start of quit
                                             box*/
   int         yoff_quit_end;              /*y offset in area to end of quit
                                             box*/
   
   int         col_width;                  /*Width of an item in pixels*/
   int         row_height;                 /*Height of an item in pixels*/
   int         font_type;                  /*Font type for the items and
                                             title; 1-5 very small, small, 
                                             medium, large, extra-large*/
   int         title_present;              /*=1 if title present; =0 if not*/
   int         title_len;                  /*Title length*/
   int         title_xoff;                 /*x offset for start of title*/
   int         title_yoff;                 /*y offset for start of title*/
   int         quit_box;                   /*=1, Menu has quit box, =0 has not*/
   int         quit_item;                  /*Item no. for the quit box item*/
   int         selected_item;              /*Last selected item no.*/
   int         highlight_item;             /*Number of the currently highlighted
                                             item - 0 if none*/
   XDl_menu_area_item_details * item_details; /*Status and preset details area
                                             for the items (see structure
                                             definition for details*/
 
}  
   XDl_menu_area;

/* Define routine types */
int xdl_menu_area();
void xdl_menu_area_rps();
void xdl_menu_area_rp1();
void xdl_menu_area_repaint();
void xdl_menu_area_ep1();
void xdl_menu_area_on_off();
void xdl_menu_area_tidy_up();
void xdl_menu_area_drawitem();
void xdl_menu_area_drawstring();
int xdl_menu_area_setmenu();
int xdl_menu_area_getitem();
int xdl_menu_area_getrootxy();
void xdl_menu_area_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_menu_area xdlf_menu_area_
#define xdlf_menu_area_setmenu xdlf_menu_area_setmenu_
#define xdlf_menu_area_getitem xdlf_menu_area_getitem_
#define xdlf_menu_area_getrootxy xdlf_menu_area_getrootxy_
#define xdlf_menu_area_getsize xdlf_menu_area_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_menu_area XDLF_MENU_AREA
#define xdlf_menu_area_setmenu XDLF_MENU_AREA_SETMENU
#define xdlf_menu_area_getitem XDLF_MENU_AREA_GETITEM
#define xdlf_menu_area_getrootxy XDLF_MENU_AREA_GETROOTXY
#define xdlf_menu_area_getsize XDLF_MENU_AREA_GETSIZE

#endif

void xdlf_menu_area();
void xdlf_menu_area_setmenu();
void xdlf_menu_area_getitem();
void xdlf_menu_area_getrootxy();
void xdlf_menu_area_getsize();


/* 
   Format of text storage
   ----------------------
   
   The menu item names names are stored in a single text array 'text'
   For each menu item there are max_name_chars characters.

*/
/*-C:*/
int xdl_menu_area (vh, vh_parent, x, y, cset, max_items, max_name_chars, 
              font_type, quit_box, max_title_len, min_width, min_height)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent view-object,
                             if 0 then a base frame is created to
                             enclose the menu area view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - normally 0 
                             (R)*/
int       max_items;      /* Maximum number of possible items in the 
                             menu (R)*/
int       max_name_chars; /* Maximum number of characters in a menu item 
                             name string (R)*/
int       font_type;      /* Font type for menu items and title to be used 
                             in calculating the size requirements:
                             1 = very small 
                             1 = small
                             2 = medium
                             3 = large 
                             4 = extra-large (R)*/
int       quit_box;       /* = 1, Allow room for a menu 'quit/return' 
                             item box in addition to the max_items other 
                             menu items.
                             The string for the quit box must not exceed
                             max_name_chars in length; = 0, do not (R)*/
int       max_title_len;  /* Maximum length of the title string in 
                             characters; 0 if no title will be used (R)*/
int       min_width;      /* The minimum width for the Menu area 
                             view-object. This will be used only if it 
                             exceeds the width calculated from the details 
                             supplied above. If 0 then the calculated 
                             width will be used (R)*/
int       min_height;     /* The minimum height for the Menu area 
                             view-object. This will be used only if it 
                             exceeds the height calculated from the 
                             details supplied above. If 0 then the 
                             calculated height will be used (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to get memory for global data structure
                bit 2 set; Unable to get memory for 'text' array
                bit 3 set; Unable to get memory for menu item details array
                bit 4 set; Unable to get memory for copy of title
                bit 5 set; A required font not available
-end*/
{
   Window wparent;        /* Parent window id */
   Window wid;            /* Window id for primary window */
   Status st;             /* Status return from Xlib calls */
   XDl_menu_area * gd;    /* Pointer to view-object's global data structure */
   char * p;             /* Temporary pointer */
   int err;              /* Error return flag */
   int iv;               /* Offset into XDL_view_objects array */
   int i;                /* Temp/loop variable */
   int area_width;       /* Menu area canvas width */
   int area_height;      /* Menu area canvas height */
   int ch_width;         /* Character width */
   int ch_height;        /* Character height */
   int ascent;           /* Font acsent */
   int descent;          /* Font descent */
   int col_width;        /* Column width */
   int row_height;       /* Row height */
   int title_width;      /* Title width */
   int title_height;     /* Title height */
   int items_width;      /* Menu items width */
   int items_height;     /* Menu items height */
   int nch_max;          /* Max no. of characters to store in text array */
   int maxq_items;       /* Max no.of items including quit if allowed */
   XDl_menu_area_item_details *id; /* Temp. pointer to parameter details 
                                   structure */
   XDl_menu_area_font menu_fonts[5]; /* Menu area fonts details */
   XFontStruct * fontinfo; /*Temporary font structure pointer*/

   err = 0;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Size information: *WARNING* update xdl_menu_area_getsize if next
      few sections changed (down to setting up parent frame) */

   /* Get font details */

   for (i=0;i<5;++i)
   {
      if (XDL_fonts[i]==NULL)
      {
         xdl_view_errmsg ("xdl_menu_area","Required fonts not available");
         err |= 32;
         return err;
      }
      fontinfo = XDL_fonts[i];
      menu_fonts[i].fid = fontinfo->fid;
      if (XDL_bold_fonts[i]==NULL)
      {
         xdl_view_errmsg ("xdl_menu_area","Required fonts not available");
         err |= 32;
         return err;
      }
      fontinfo = XDL_bold_fonts[i];
      menu_fonts[i].fid_bold = fontinfo->fid;
      xdl_ch_size (i+1, 2, &ch_width, &ch_height, &ascent, &descent);
      menu_fonts[i].font_ascent = ascent;
      menu_fonts[i].font_descent = descent;
      menu_fonts[i].ch_width = ch_width;
   }

   /* Calculate size requirements for the windows */

   title_width = 0;
   title_height = 0;

   /* Get character sizes */

   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);

   /* See if title allowed and get title area dimensions */

   if (max_title_len>0)
   {
      title_width = max_title_len * ch_width + 2*MARG_W;
      title_height = 2*ch_height;
   }

   /* Get items area requirements */

   items_width = 0;
   items_height = 0;
   items_width = max_name_chars*ch_width + 2*MIN_ITEM_MARG_W + 2*MARG_W;
   i = max_items;
   if (quit_box!=0) i += 2;
   items_height = i*(ch_height+2*MIN_ITEM_MARG_H) + 2*MARG_H + 2;

   area_width = title_width;
   if (items_width>area_width) area_width = items_width;
   if (min_width-2*BORDER_WIDTH-2>area_width)
       area_width = min_width - 2*BORDER_WIDTH - 2;
   area_height = title_height;
   area_height += items_height;
   if (min_height-ACT_STRIP_HEIGHT-2*BORDER_HEIGHT-2>area_height) 
       area_height = min_height - ACT_STRIP_HEIGHT - 2*BORDER_HEIGHT - 2;

   /* Create a primary window (paint or top level) for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wid = xdl_create_frame_window (wparent, x, y, 
                           area_width + 2*BORDER_WIDTH + 2,
                           area_height + 2*BORDER_HEIGHT +ACT_STRIP_HEIGHT + 2,
                           cset);
   }else{
   /* Create base frame window as no parent was specified */
      wid = xdl_create_base_frame_window ("xdl_menu_area", "xdl_menu", x, y,
                           area_width + 2*BORDER_WIDTH + 2,
                           area_height + 2*BORDER_HEIGHT +ACT_STRIP_HEIGHT + 2);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_menu_area));
   if (p==0) {err |= 2; return err;}
   gd = (XDl_menu_area *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wid, vh_parent, p, 1051, xdl_menu_area_on_off,
                        xdl_menu_area_tidy_up);

   /* Some initialisations */

   gd->on_off = 0;
   gd->repaint_done = 0;
   gd->quit_box_allowed = 0;
   if (quit_box!=0) gd->quit_box_allowed = 1;
   gd->title_allowed = 0;
   if (max_title_len>0) gd->title_allowed = 1;
   gd->max_title_len = max_title_len;
   gd->num_items = -1;
   gd->highlight_pixel = XDL_cyanpixel;
   gd->highlight_item = 0;
   gd->selected_item = 0;
   for (i=0;i<5;++i)
   {
      gd->menu_fonts[i].fid = menu_fonts[i].fid;
      gd->menu_fonts[i].fid_bold = menu_fonts[i].fid_bold;
      gd->menu_fonts[i].font_ascent = menu_fonts[i].font_ascent;
      gd->menu_fonts[i].font_descent = menu_fonts[i].font_descent;
      gd->menu_fonts[i].ch_width = menu_fonts[i].ch_width;
   }
   gd->area_width = area_width;
   gd->area_height = area_height;
   gd->max_items = max_items;
   gd->max_name_chars = max_name_chars;
   maxq_items = max_items;
   gd->quit_item = 0;
   if (quit_box!=0)
   {
       maxq_items++;
       gd->quit_item = maxq_items;
   }
   nch_max = maxq_items * max_name_chars;
   p = (char *) malloc (nch_max);
   if (p==0) {err |= 4; return err;}
   gd->text = p;
   for (i=0; i<nch_max; i++) gd->text[i] = ' ';
   p = (char *) malloc (maxq_items*sizeof(XDl_menu_area_item_details));
   if (p==0) {err |= 8; return err;}
   gd->item_details = (XDl_menu_area_item_details *) p;
   gd->title = NULL;
   if (max_title_len>0)
   {
      p = (char *) malloc (max_title_len+1);
      if (p==0) {err |= 16; return err;};
      gd->title = p;
   }

   /* Create active strip window */

   gd->wact = xdl_create_active_strip (wid, vh, area_width, xdl_menu_area_rps);

   /* Create menu area paint window */

   gd->wmenu = xdl_create_paint_window (wid, BORDER_WIDTH, 
                                        BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1,
                                        area_width, area_height, cset);

   xdl_add_window (vh, gd->wmenu, xdl_menu_area_rp1, xdl_menu_area_ep1);

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wmenu, ExposureMask|StructureNotifyMask|
           ButtonPressMask|PointerMotionMask|LeaveWindowMask);
  
   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wmenu, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, XDL_fontinfo->fid);

   return 0;
}

/* Fortran binding: xdlf_menu_area */

void xdlf_menu_area (ivh, ivhparent, ix, iy, icset,maxitems, maxlen, ifont, 
                     iquit, maxtitl, minwidth, minheight, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *maxitems, *maxlen, *ifont, *iquit;
int *maxtitl, *minwidth, *minheight, *ierr;
{
   *ierr = xdl_menu_area (*ivh, *ivhparent, *ix, *iy, *icset, *maxitems, 
                          *maxlen, *ifont, *iquit, *maxtitl, *minwidth, 
                          *minheight);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
********************************
**     xdl_menu_area_rps      **
********************************

Purpose: Repaint active strip area

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_menu_area * gd;    /* Pointer to global data area */

   gd = (XDl_menu_area *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, "Select item", 0);
   }
   return;
}

/*
*********************************
**     xdl_menu_area_rp1       **
*********************************

Purpose: Repaint menu area on expose (calls xdl_menu_area_repaint)

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_menu_area * gd;             /* Pointer to global data area */
   XDl_menu_area_item_details *id; /* pointer to item details structure */
   int hl;                         /* Highlight flag 1=yes, 0=no */
   int i;                          /* Loop counter */

   gd = (XDl_menu_area *) global;
   gd->repaint_done = 1;
   xdl_menu_area_repaint (gd);
   return;
}

/*
************************************
**     xdl_menu_area_repaint      **
************************************

Purpose: Repaint menu area

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_repaint (gd)

XDl_menu_area *gd;     /* Pointer to global data area */

{
   int item;                         /* Menu item number */
   XDl_menu_area_item_details *id;   /* Pointer to item details structure */

   XClearWindow (XDL_display, gd->wmenu);
   if (gd->num_items<0) return;

   /* Draw title if present */

   if (gd->title_present)
   {
      xdl_menu_area_drawstring (gd, gd->title_xoff, gd->title_yoff, gd->title,
                                  gd->title_len, 1);
   }


   /* Output menu items */

   if (gd->num_items>0)
   {
      XDrawRectangle (XDL_display, gd->wmenu, gd->paint_gc,
                      gd->xoff_items_start, gd->yoff_items_start-1,
                      gd->col_width-1, gd->num_items*(gd->row_height)+1);
      for (item=1;item<=gd->num_items;++item)
      {
         id = &(gd->item_details[item-1]);
         XDrawRectangle (XDL_display, gd->wmenu, gd->paint_gc,
                         id->xoff+1, id->yoff,
                         gd->col_width-3, gd->row_height-1);
         xdl_menu_area_drawstring (gd, id->xoff_name, id->yoff_name,
                                   &(gd->text[id->toff_name]), id->name_len, 0);
      }
   }

   /* Output quit box if present */

   if (gd->quit_box)
   {
      id = &(gd->item_details[gd->quit_item-1]);
      XDrawRectangle (XDL_display, gd->wmenu, gd->paint_gc,
                      id->xoff, id->yoff-1,
                      gd->col_width-1, gd->row_height+1);
      XDrawRectangle (XDL_display, gd->wmenu, gd->paint_gc,
                      id->xoff+1, id->yoff,
                      gd->col_width-3, gd->row_height-1);
      xdl_menu_area_drawstring (gd, id->xoff_name, id->yoff_name,
                                &(gd->text[id->toff_name]), id->name_len, 0);
   }
   return;
}

/*
********************************
**     xdl_menu_area_ep1      **
********************************

Purpose: Event handling routine for the menu area

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_menu_area *gd;            /* Pointer to the global data area */
   int x, y;                     /* Mouse x,y position */
   int row_offset;               /* Row offset in menu */
   int item;                     /* Menu item number */

   gd = (XDl_menu_area *) global;

   if (gd->on_off==0) return;

   /* Check for mouse select to select input item */

   if (event->type==ButtonPress&&event->xbutton.button==Button1)
   {
      x = event->xbutton.x;
      y = event->xbutton.y;
      if (x>=gd->xoff_items_start&&x<=gd->xoff_items_end)
      {
         if (y>=gd->yoff_items_start&&y<=gd->yoff_items_end)
         {
            row_offset = (y - gd->yoff_items_start) / gd->row_height;
            gd->selected_item = row_offset + 1;
            XDL_return_data_vh = XDL_view_objects[iv]->vh;
            return;
         }
         if (gd->quit_box)
         {
            if (y>=gd->yoff_quit_start&&y<=gd->yoff_quit_end)
            {
               gd->selected_item = gd->quit_item;
               XDL_return_data_vh = XDL_view_objects[iv]->vh;
               return;
            }
         }
      }
      return;
   }

   /* Mouse movement */

   if (event->type==MotionNotify)
   {
      x = event->xmotion.x;
      y = event->xmotion.y;
      item = -1;
      if (x>=gd->xoff_items_start&&x<=gd->xoff_items_end)
      {
         if (y>=gd->yoff_items_start&&y<=gd->yoff_items_end)
         {
            row_offset = (y - gd->yoff_items_start) / gd->row_height;
            item = row_offset + 1;
         }
         if (gd->quit_box)
         {
            if (y>=gd->yoff_quit_start&&y<=gd->yoff_quit_end)
            {
               item = gd->quit_item;
            }
         }
      }
      if (item==gd->highlight_item) return;
      if (gd->highlight_item>0) xdl_menu_area_drawitem (gd, 
                                                 gd->highlight_item, 0);
      gd->highlight_item = 0;
      if (item<0) return;
      gd->highlight_item = item;
      xdl_menu_area_drawitem (gd, item, 1);
      return; 
   }

   /* Check for mouse exit from the window */

   if (event->type==LeaveNotify)
   {
      if (gd->highlight_item==0) return;
      xdl_menu_area_drawitem (gd, gd->highlight_item, 0);
      gd->highlight_item = 0;
      return;
   }

   return;
}

/*
***********************************
**     xdl_menu_area_on_off      **
***********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_on_off (iv, i)

int iv;         /* Offset in XDL_view_objects array */
int i;          /* flag =1 on, =0 off */

{
   XDl_menu_area *gd;                /* Pointer to the global data area */
   XDl_menu_area_item_details *id;   /* Pointer to item details structure */

   gd = (XDl_menu_area *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      xdl_active_strip_on (gd->wact, "Select item", 0);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
      if (gd->highlight_item>0)
      {
         xdl_menu_area_drawitem (gd, gd->highlight_item, 0);
         gd->highlight_item = 0;
      }
   }
   return;
}

/*
************************************
**     xdl_menu_area_tidy_up      **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_tidy_up (global)

char *global;          /* Pointer to the view-object global data area */

{
   XDl_menu_area *gd;    /* Pointer to global data area */

   gd = (XDl_menu_area *) global;
   free (gd->text);
   free ((char *) gd->item_details);
   free (gd->title);
   XFreeGC (XDL_display, gd->paint_gc);
   return;
}

/*
*************************************
**     xdl_menu_area_drawitem      **
*************************************

Purpose: Draw a menu item

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_drawitem (gd, item, hl)

XDl_menu_area *gd;     /* Pointer to global data area */
int item;              /* Item number */
int hl;                /* Highlight flag =1 yes, =0 no */

{
   XDl_menu_area_item_details *id;  /* Pointer to item details structure */
   int bold;                        /* Flag =1 bold, =0 normal print */

   bold = 0;
   id = &(gd->item_details[item-1]);
   XClearArea (XDL_display, gd->wmenu, id->xoff+2, id->yoff+1,
               (gd->col_width-4), (gd->row_height-2), False);

   if (hl)
   {
      if (XDL_color > 1)
      {
         XSetForeground (XDL_display, gd->paint_gc, gd->highlight_pixel);
         XFillRectangle (XDL_display, gd->wmenu, gd->paint_gc,
                         id->xoff+2, id->yoff+1,
                         (gd->col_width-4), (gd->row_height-2));
      }
      bold = 1;
   }

   xdl_menu_area_drawstring (gd, id->xoff_name, id->yoff_name,
                               &(gd->text[id->toff_name]), 
                               id->name_len, bold);
   return;
}

/*
***************************************
**     xdl_menu_area_drawstring      **
***************************************

Purpose: Output a text string

Author:  John W. Campbell, July 1990

*/

void xdl_menu_area_drawstring (gd, x, y, string, len, bold)

XDl_menu_area *gd;      /* Pointer to global data area */
int x;                  /* x coordinate for start of string */
int y;                  /* y coordinate for start of string */
char * string;          /* String to output */
int len;                /* Number of characters to output */
int bold;               /* Bold print flag =1 yes, =0 no */

{
   int i;             /* Loop counter */
   int fnt_offset;    /* Font type offset */
   int ch_width;      /* Character width in pixels */
   int xx;            /* Character position x */
   int yy;            /* Character position y */

   if (len<=0) return;

   fnt_offset = gd->font_type -1;
   ch_width = gd->menu_fonts[fnt_offset].ch_width;

   xx = x;
   yy = y + gd->menu_fonts[fnt_offset].font_ascent;

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   if (bold)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->menu_fonts[fnt_offset].fid_bold);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->menu_fonts[fnt_offset].fid);
   }

   for (i=0;i<len;++i)
   {
      if (string[i]!=' ') XDrawString (XDL_display, gd->wmenu,
                                       gd->paint_gc, xx, yy, &string[i], 1);
      xx += ch_width;
   }
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set up a new menu - xdl_menu_area_setmenu
The routine xdl_menu_area_setmenu (xdlf_menu_area_setmenu) is used to set
an initial or new menu within a previously created menu area view-object.
The length of the text strings and the number of items must be within the
limits set for the menu area but the routine takes care of the layout of
the items on the displayed menu.
-end*/
/*
***********************************
**     xdl_menu_area_setmenu     **
***********************************

Purpose: Set a new menu in the menu area

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_MENU_AREA_SETMENU (IVH, NITEMS, XDLSTR(NAMES), NAMLEN, 
        +                          XDLSTR(TITLE), ITLEN, XDLSTR(QUITNAM),
        +                          IQLEN, IFONT, IERR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
NITEMS     (R)  Number of menu items (see num_items)
NAMES      (R)  Fortran array of character strings containing
                the menu item names.
                [CHARACTER*NAMLEN NAMES(NITEMS)].
                ** Pass address using XDLSTR function **
                (see names) 
NAMLEN     (R)  Length of the menu item names character strings
                (Must be >0 cf name_len)
TITLE      (R)  Title character string (see title)
                ** Pass address using XDLSTR function **
ITLEN      (R)  Length of title string if > 0; -1 if no title 
                (cf title_len)
QUITNAM    (R)  Quit-box character string (see quit_name)
                ** Pass address using XDLSTR function **
IQLEN      (R)  Length of quit-box string if > 0; -1 if no 
                quit-box (cf quit_len)
IFONT      (R)  Font type (1 to 4 small to extralarge or 0 for 
                largest that there is room for) (see font_type)
IERR       (W)  Returns the status from xdl_menu_area_setmenu call
-end*/

/*-C:*/
int xdl_menu_area_setmenu (vh, num_items, names, name_len, title, 
                           title_len, quit_name, quit_len, font_type)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int num_items;   /* No. of items in the menu (R)*/
char * names;    /* Item names - this may either an array of pointers to
                    num_items null terminated character strings 
                    (len_name=0) or a character string num_items*len_name
                    in length with each name occupying len_name characters 
                    (left justified and padded to the right with blanks 
                    if needed) (R)*/
int name_len;    /* Item name length (0 = array of pointers to 
                    null-terminated character strings (e.g. for 'C' 
                    programs) or the length of each name string 
                    (e.g. for Fortran programs). See also the previous 
                    item (R) */
char * title;    /* Title string (R)*/
int title_len;   /* If 0 then the length of the title string is 
                    determined assuming that 'title' is a null terminated
                    string; if > 0 it is the length of the title string; 
                    if = -1 then do not output a title ('title' ignored 
                    in this case) (R) */
char * quit_name;/* Quit box name string (R)*/
int quit_len;    /* If 0 then the length of the quit box string is 
                    determined assuming that 'quit_name' is a null 
                    terminated string; if > 0 it is the length of the 
                    title string; if = -1 then do not output a quit 
                    box ('quit_name' ignored in this case) (R) */
int font_type;   /* Font type: if 1-5, use very-small/small/medium/large/
                    extra-large font respectively if space permitting. 
                    (If not the the font size will be reduced till a
                    suitable one is found); if 0, use the largest
                    font which fits (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                          /* Offset in XDL_viee_objects array */
   XDl_menu_area *gd;               /* Pointer to the global data area */
   XDl_menu_area_item_details *id;  /* Pointer to an item details structure */
   int len;                         /* String length */
   int max_name_len;                /* Maximum length of a menu item name */
   int i, j, l;                     /* Temp/loop variables */
   int itm;                         /* Loop counter through menu items */
   int font_max;                    /* Maximum requested font size */
   int num_rows;                    /* Number of rows of items */
   int fnt;                         /* Font type loop counter */
   int toff;                        /* Offset in text array */
   int ioff;                        /* Offset in input item name strings 
                                       array */
   int joff, koff;                  /* y offsets for start of items and start
                                       of quit box */
   int xoff, yoff;                  /* Item x,y offsets */
   int extra_w, extra_h;            /* Extra width and height around menu
                                       text strings */
   int h, w;                        /* Height and width required for the output
                                       of the title/items/quit box */
   int space, best_space;           /* Free space (in y direction) and best
                                       amount of free space in an 'ideal'
                                       layout */
   int title_height, title_width;   /* Title height and width */
   int item_area_height, item_area_width; /* Height and width for the menu
                                             items area */
   int ch_height, ch_width;         /* Character height and width in pixels */
   char * item_str;                 /* Pointer to item strings for Fortran type
                                       input */
   char ** item_strs;               /* Pointer to item strings for list of
                                       null terminated strings type input */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_menu_area_setmenu",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1051)
   {
      xdl_view_type_errmsg ("xdl_menu_area_setmenu", vh, "xdl_menu_area");
      return 2;
   }
   gd = (XDl_menu_area *) XDL_view_objects[iv]->gd;

  /* Initialise menu items parameters */

   gd->num_items = 0;
   gd->title_present = 0;
   gd->quit_box = 0;
   gd->highlight_item = 0;
   gd->selected_item = 0;

   /* Menu title */

   if (gd->title_allowed)
   {
      if (title_len>=0)
      {
         gd->title_present = 1;
         len = title_len;
         if (len==0) len = strlen (title);
         if (len>gd->max_title_len) len = gd->max_title_len;
         strncpy (gd->title, title, len);
         gd->title_len = 0;
         for (l=len-1;l>=0;--l)
         {
            if (gd->title[l]!=' ')
            {
               gd->title_len = l+1;
               break;
            }
         }
      }
   }

   /* Item names */

   if (num_items>gd->max_items) num_items = gd->max_items;
   gd->num_items = num_items;
   max_name_len = 0;
   if (name_len>0)
   {
      if (name_len>gd->max_name_chars) name_len = gd->max_name_chars;
      item_str = (char *) names;
      toff = 0;
      ioff = 0;
      for (itm=0;itm<num_items;++itm)
      {
         id = &(gd->item_details[itm]);
         id->toff_name = toff;
         strncpy (&gd->text[toff], &item_str[ioff], name_len);
         len = 0;
         for (l=name_len-1;l>=0;--l)
         {
            if (item_str[ioff+l]!=' ')
            {
               len = l+1;
               break;
            }
         }
         id->name_len = len;
         if (len>max_name_len) max_name_len = len;
         toff += gd->max_name_chars;
         ioff += name_len;
      }
   }
   else
   {
      item_strs = (char **) names;
      toff = 0;
      for (itm=0;itm<num_items;++itm)
      {
         id = &(gd->item_details[itm]);
         id->toff_name = toff;
         len = strlen (item_strs[itm]);
         if (len>gd->max_name_chars) len = gd->max_name_chars;
         strncpy (&gd->text[toff], item_strs[itm], len);
         id->name_len = len;
         if (len>max_name_len) max_name_len = len;
         toff += gd->max_name_chars;
      }
   }

   /* Quit box string */

   if (gd->quit_box_allowed)
   {
      if (quit_len>=0)
      {
         gd->quit_box = 1;
         gd->quit_item = gd->num_items + 1;
         len = quit_len;
         if (len==0) len = strlen (quit_name);
         if (len>gd->max_name_chars) len = gd->max_name_chars;
         toff = (gd->quit_item-1)*(gd->max_name_chars);
         strncpy (&gd->text[toff], quit_name, len);
         id = &(gd->item_details[gd->quit_item-1]);
         id->toff_name = toff;
         id->name_len = 0;
         for (l=len-1;l>=0;--l)
         {
            if (quit_name[l]!=' ')
            {
               id->name_len = l+1;
               break;
            }
         }
         if (id->name_len>max_name_len) max_name_len = id->name_len;
      }
   }
   if (font_type<0) font_type = 0;
   if (font_type>5) font_type = 5;
   font_max = font_type;
   if (font_type==0) font_max = 5;

   /* Check space requirements and decrease font size if needed */

   num_rows = num_items;
   if (gd->quit_box) num_rows+=1;
   for (fnt=font_max;fnt>0;--fnt)
   {
      gd->font_type = fnt;
      ch_height = gd->menu_fonts[fnt-1].font_ascent
                  + gd->menu_fonts[fnt-1].font_descent;

      /*Check title width*/
      if (gd->title_present)
      {
         w = gd->title_len * (gd->menu_fonts[fnt-1].ch_width) + 2*MARG_W;
         if (w>gd->area_width) continue;
      }

      /*Check items width*/
      
      w = max_name_len * (gd->menu_fonts[fnt-1].ch_width) + 2*MIN_ITEM_MARG_W
          + 2*MARG_W;
      if (w>gd->area_width) continue;

      /*Check height*/

      h = 0;
      if (gd->title_present) h += 2*ch_height;
      h += num_rows * (ch_height + 2*MIN_ITEM_MARG_H) + 2*MARG_H + 2;
      if (gd->quit_box)
      {
         if (h+ch_height<=gd->area_height) break;
      }
      else
      {
         if (h<=gd->area_height) break;
      }
   }

   /* Font type now set - set layout parameters */

   ch_width = gd->menu_fonts[gd->font_type-1].ch_width;
   ch_height = gd->menu_fonts[gd->font_type-1].font_ascent
               + gd->menu_fonts[gd->font_type-1].font_descent;

   /* Vertical Layout */

   title_height = 0;
   if (gd->title_present) title_height = 2*ch_height;
   item_area_height = gd->area_height - title_height -2*MARG_H;
   space = gd->area_height - h;
   extra_h = 0;
   best_space = (item_area_height*BEST_PERCENT_SPACE)/100;
   if (space>best_space) extra_h = (space-best_space)/(2*num_rows);
   if (extra_h >MAX_ITEM_MARG_H*ch_height) extra_h = MAX_ITEM_MARG_H*ch_height;
   space = item_area_height
           - num_rows * (ch_height+2*MIN_ITEM_MARG_H + 2*extra_h) - 2;
   if (gd->quit_box)
   {
      if (space>25)
      {
         joff = (space*TOP_SPACE)/(TOP_SPACE + BOTTOM_SPACE + MIDDLE_SPACE);
         koff = (space*MIDDLE_SPACE)/(TOP_SPACE + BOTTOM_SPACE + MIDDLE_SPACE);
      }
      else
      {
         joff = space/3;
         koff = joff;
      }
   }
   else
   {
      joff = (T_SP*space)/(T_SP + B_SP);
   }

   /* Horizontal Layout */

   w = max_name_len*ch_width + 2*MIN_ITEM_MARG_W;
   extra_w = 0;
   space = gd->area_width - 2*MARG_W -w;
   extra_w = space/4;
   if (extra_w>MAX_ITEM_MARG_W*ch_width) extra_w = MAX_ITEM_MARG_W*ch_width;

   /* Set offsets etc.*/

   if (gd->title_present)
   {
      gd->title_xoff = (gd->area_width - 2 - gd->title_len*ch_width) / 2;
      if (gd->title_xoff<0) gd->title_xoff = 0;
      gd->title_yoff = (2*ch_height)/3 + MARG_H;
   }

   gd->col_width = max_name_len*ch_width + 2*MIN_ITEM_MARG_W + 2*extra_w;
   gd->row_height = ch_height + 2*MIN_ITEM_MARG_H + 2*extra_h;

   xoff = (gd->area_width - 2 - gd->col_width) / 2;
   if (xoff<0) xoff = 0;
   yoff = title_height + MARG_H + joff;
   gd->yoff_items_start = yoff;
   gd->xoff_items_start = xoff;
   gd->xoff_items_end = xoff + gd->col_width - 1;
   for (itm=0;itm<num_items;++itm)
   {
      id = &(gd->item_details[itm]);
      id->yoff = yoff;
      id->yoff_name = yoff + MIN_ITEM_MARG_H + extra_h;
      id->xoff = xoff;
      id->xoff_name = (gd->area_width -2 - id->name_len*ch_width) / 2;
      yoff += gd->row_height;
   }
   gd->yoff_items_end = yoff - 1;
   if (gd->quit_box)
   {
      yoff += koff;
      gd->yoff_quit_start = yoff + 1;
      gd->yoff_quit_end = yoff + gd->row_height -1;
      id = &(gd->item_details[gd->quit_item-1]);
      id->yoff = yoff;
      id->yoff_name = yoff + MIN_ITEM_MARG_H + extra_h;
      id->xoff = xoff;
      id->xoff_name = (gd->area_width - 2 - id->name_len*ch_width) / 2;
   }

   if (gd->repaint_done==1) xdl_menu_area_repaint (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_menu_area_setmenu */

void xdlf_menu_area_setmenu (ivh, nitems, names, namlen, title, itlen, quitnam,
                             iqlen, ifont, ierr)
int *ivh, *nitems, *names, *namlen, *title, *itlen, *quitnam, *iqlen, *ifont;
int *ierr;
{
   *ierr = xdl_menu_area_setmenu (*ivh, *nitems, XDL_pointers[*names], *namlen,
                                  XDL_pointers[*title], *itlen, 
                                   XDL_pointers[*quitnam],
                                  *iqlen, *ifont);
   return;
}

/*-Routine: Get the selected item number - xdl_menu_area_getitem
The routine xdl_menu_area_getitem (xdlf_menu_area_getitem) returns the
item number of the last selected menu item. It will normally be called
after the return from a call to xdl_get_events (xdlf_get_events) indicates
that a menu_area view-object has returned input.
-end*/
/*
***********************************
**     xdl_menu_area_getitem     **
***********************************

Purpose: Get the number of the last selected menu item

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_MENU_AREA_GETITEM (IVH, ITEM, IQUIT)
-end*/
 
/*-Parameters:
IVH    (R)  The view-object handle (see vh)
ITEM   (W)  Returns the menu item number (see item)
IQUIT  (W)  Returns 0 for normal item, 1 for quit box (see quit)
-end*/

/*-C:*/
int xdl_menu_area_getitem (vh, item, quit)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *item;       /* Returns the selected item number (W) */
int *quit;       /* Returns 0 for a normal item, 1 if quit box (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_menu_area *gd;    /* Pointer to global data area */

   *item = 0;
   *quit = 0;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_menu_area_getitem",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1051)
   {
      xdl_view_type_errmsg ("xdl_menu_area_getitem", vh, "xdl_menu_area");
      return 2;
   }
   gd = (XDl_menu_area *) XDL_view_objects[iv]->gd;
   *item = gd->selected_item;
   if (gd->quit_box&&gd->selected_item==gd->quit_item) *quit = 1;
   return 0;
}

/* Fortran binding: xdlf_menu_area_getitem */

void xdlf_menu_area_getitem (ivh, item, iquit)
int *ivh, *item, *iquit;
{
   xdl_menu_area_getitem (*ivh, item, iquit);
   return;
}
/*-Routine: Get position of selected item - xdl_menu_area_getrootxy
The routine xdl_menu_area_getrootxy (xdlf_menu_area_getrootxy) gets the root 
x, y position of the last selected menu item; this is taken as being half 
way up right hand side of the item box. It may be used for example as a
focus for an error message if required. It will normally be called after
a call to the routine xdl_menu_area_getitem (xdlf_menu_area_getitem).
-end*/
/*
***********************************
**     xdl_menu_area_getrootxy   **
***********************************

Purpose: Get the root x, y position of the last selected menu item
         (half way up right hand side of the item box - for use a
          an error message focus if required)

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_MENU_AREA_GETROOTXY (IVH, IXROOT, IYROOT)
-end*/
 
/*-Parameters:
IVH    (R)  The view-object handle (see vh)
IXROOT (W)  Returns the root x position (see xroot)
IYROOT (W)  Returns the root y position (see yroot)
-end*/
/*-C:*/
int xdl_menu_area_getrootxy (vh, xroot, yroot)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *xroot;      /* Returns the x root position (half way up right hand
                    side of item box) of the last selected menu item) (W)*/
int *yroot;      /* Returns the y root position cf xroot (W)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_menu_area *gd;    /* Pointer to global data area */
   int item;             /* Item number of last selected item */
   int x;                /* Required x position wrt menu area */
   int y;                /* Required y position wrt menu area */
   XDl_menu_area_item_details *id;  /* Pointer to an item details structure */

   *xroot = 0;
   *yroot = 0;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_menu_area_getrootxy",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1051)
   {
      xdl_view_type_errmsg ("xdl_menu_area_getrootxy", vh, "xdl_menu_area");
      return 2;
   }
   gd = (XDl_menu_area *) XDL_view_objects[iv]->gd;
   item = gd->selected_item;
   if (item==0) return 0;
   id = &(gd->item_details[item-1]);
   x = gd->xoff_items_end;
   y = id->yoff + gd->row_height / 2;
   xdl_rootxy (gd->wmenu, x, y, xroot, yroot);
   return 0;
}

/* Fortran binding: xdlf_menu_area_getrootxy */

void xdlf_menu_area_getrootxy (ivh, ixroot, iyroot)
int *ivh, *ixroot, *iyroot;
{
   xdl_menu_area_getrootxy (*ivh, ixroot, iyroot);
   return;
}
/*-Routine: Get menu area size requirements - xdl_menu_area_getsize
The routine xdl_menu_area_getsize (xdlf_menu_area_getsize) is used
to calculate the size requirements for a menu area based on the maximum
lengths of text strings required and the maximum number of menu items
required.
-end*/
/*
************************************
**     xdl_menu_area_getsize      **
************************************

Purpose: Return the size requirements of the menu area view-object

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_MENU_AREA_GETSIZE (MAXITEMS, MAXNAM, IFONT, IQUIT,
        +                             MAXTITL, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
MAXITEMS   (R)  Max. no. of menu items (see max_items)
MAXNAM     (R)  Max. item name length (chars) 
                (see max_name_chars)
IFONT      (R)  Font type (1-4) (see font_type)
IQUIT      (R)  =1 allow quit box , =0 do not (see quit_box)
MAXTITL    (R)  Max. title length, 0 if no title 
                (see max_title_len)
IWIDTH     (W)  Returns the width required (see w)
IHEIGHT    (W)  Returns the height required (see h)
-end*/

/*-C:*/
void xdl_menu_area_getsize (max_items, max_name_chars, font_type, 
                            quit_box, max_title_len, w, h)
/*end*/
 
/*-Parameters:*/
int       max_items;      /* Maximum number of possible items in the 
                             menu (R)*/
int       max_name_chars; /* Maximum number of characters in a menu item 
                             name string (R)*/
int       quit_box;       /* = 1, Allow room for a menu 'quit/return' 
                             item box in addition to the max_items other 
                             menu items.
                             The string for the quit box must not exceed
                             max_name_chars in length; = 0, do not (R)*/
int       font_type;      /* Font type for menu items and title to be 
                             used in calculating the size requirements:
                             1 = very small font,
                             2 = small font,
                             3 = medium font,
                             4 = large font,
                             5 = extra-large font (R)*/
int       max_title_len;  /* Maximum length of the title string in 
                             characters; 0 if no title will be used (R)*/
int *w;                   /* Returns the width required (0 if font not 
                             found) (W)*/
int *h;                   /* Returns the height required (0 if font not 
                             found) (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int i;                /* Temp/loop variable */
   int area_width;       /* Menu area canvas width */
   int area_height;      /* Menu area canvas height */
   int ch_width;         /* Character width */
   int ch_height;        /* Character height */
   int ascent;           /* Font ascent */
   int descent;          /* Font descent */
   int title_width;      /* Title width */
   int title_height;     /* Title height */
   int items_width;      /* Menu items width */
   int items_height;     /* Menu items height */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   *w = 0;
   *h = 0;

   /* Get font details */

   i = font_type - 1;
   if (XDL_fonts[i]==NULL)
   {
      xdl_view_errmsg ("xdl_menu_area_getsize","Required fonts not available");
      return;
   }
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);

   /* Calculate size requirements for the windows */

   title_width = 0;
   title_height = 0;

   /* See if title allowed and get title area dimensions */

   if (max_title_len>0)
   {
      title_width = max_title_len * ch_width + 2*MARG_W;
      title_height = 2*ch_height;
   }

   /* Get items area requirements */

   items_width = 0;
   items_height = 0;
   items_width = max_name_chars*ch_width + 2*MIN_ITEM_MARG_W + 2*MARG_W;
   i = max_items;
   if (quit_box!=0) i += 2;
   items_height = i*(ch_height+2*MIN_ITEM_MARG_H) + 2*MARG_H + 2;

   area_width = title_width;
   if (items_width>area_width) area_width = items_width;
   area_height = title_height;
   area_height += items_height;
   *w = area_width + 2*BORDER_WIDTH + 2;
   *h = area_height + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2;
   return;
}

/* Fortran binding: xdlf_film_image_getsize */

void xdlf_menu_area_getsize (maxitems, maxnam, ifont, iquit, maxtitl, 
                             iwidth, iheight)
int *maxitems, *maxnam, *ifont, *iquit, *maxtitl, *iwidth, *iheight;
{
   xdl_menu_area_getsize (*maxitems, *maxnam, *ifont, *iquit, *maxtitl,
                          iwidth, iheight);
   return;
}
