/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Panel Item Routines and Callbacks
-end*/
 
/*====================================*/
/*        Panel item routines         */
/*====================================*/

/*-Intro:
The xdl_panel_item routines provide standard window items which may be used
within the coding of view-objects. They differ from the main view-objects in
that they are used to serve the internal needs of a view-object rather than
being directly called from an application program.
-end*/

/*Routines List:
The following panel items and routines are available:

   Panel Choice:   xdl_panel_choice        Create a panel choice item
                   xdl_panel_choice_reset  Reset selected item
                   xdl_panel_choice_wid    Get window id.
                   xdl_panel_choice_size   Get size

   Panel Button:   xdl_panel_button        Create a panel button item
                   xdl_panel_button_set    Set a new button label string
                   xdl_panel_button_wid    Get window id.
                   xdl_panel_button_size   Get size

   Panel Label:    xdl_panel_label         Create a panel label item
                   xdl_panel_label_set     Set a new panel label string
                   xdl_panel_label_wid     Get window id.
                   xdl_panel_label_size    Get size

   Panel Value:    xdl_panel_value         Create a panel value item
                   xdl_panel_value_set     Set a new panel value string
                   xdl_panel_value_wid     Get window id.
                   xdl_panel_value_size    Get size

   Panel Slider:   xdl_panel_slider        Create a panel slider item
                   xdl_panel_slider_reset  Reset slider position
                   xdl_panel_slider_wid    Get window id.
                   xdl_panel_slider_size   Get size

   Panel I/O:      xdl_panel_io            Create a panel i/o item
                   xdl_panel_io_prompt     Set prompt and input mode
                   xdl_panel_io_clear      Clear/reset the i/o item
                   xdl_panel_io_wid        Get window id.
                   xdl_panel_io_size       Get size
*/

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"
#ifdef VMS
#include <decw$include/keysym.h>
#else
#include <X11/keysym.h>
#endif

#define MARG_W 4        /*Margin width and item separating width*/

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel Choice Item Routines
These are routines for creating and handling a panel choice item.
-end*/
 

/*-Routine: Create a panel choice item - xdl_panel_choice
The routine xdl_panel_choice is used to create a panel choice item with a
popup menu. The item has a label, an indicator box to indicate a drop
down menu and a display of the currently selected menu item. A callback
routine is invoked when a menu item is selected.
-end*/
/*
******************************
**     xdl_panel_choice     **
******************************

Purpose: Create a panel choice item with popup menu (library routine for
         use within view-objects)

Author:  John W. Campbell, September 1990

*/

typedef struct 
{
   Window wlabl;        /*Window id of window for choice label etc.*/
   char * label;        /*Pointer to label string passed to the routine*/
   char **strings;      /*Pointer to choice strings passed to the routine*/
   int nstrings;        /*Number of choice strings*/
   int area_height;     /*Height of label/inicatorbox/current-string area*/
   int area_width;      /*Width  "  "                                "   */
   int df_item;         /*Default item number*/
   int cur_item;        /*Current item number*/
   GC paint_gc;         /*Paint GC*/
   int indicator_size;  /*Indicator box size -1 */
   int xlabel;          /*x position of start of label string*/
   int ylabel;          /*y position of start of label string*/
   int xcuropt;         /*x position of start of current option string*/
   int ycuropt;         /*y position of start of current option string*/
   int xind;            /*x position of top left of indicator box*/
   int yind;            /*y position of top left of indicator box*/
   int x1;              /*Indicator triangle coordinates */
   int y1;              /* " */
   int x2;              /* " */
   int y2;              /* " */
   int x3;              /* " */
   int y3;              /* " */
   int xmenu;           /*Best x position for drop down menu (relative to
                          panel choice window)*/
   int ymenu;           /*Best y position for drop down menu (relative to
                          panel choice window)*/
   int xstring;         /*x offset for string in menu area*/
   int ystring;         /*y offset for string in menu area from top of item*/
   int ch_width;        /*Character width*/
   int ch_height;       /*Character height*/
   Font fid;            /*Normal font*/
   Font fid_bold;       /*Bold font*/
   int maxlen;          /*Maximum options string length*/
   int highlight_item;  /*Highlighted item on popup menu; 0 if none*/
}
  XDl_panel_choice;

/*Define routine types*/

void xdl_panel_choice_rp();
void xdl_panel_choice_ehp();
void xdl_panel_choice_menstr();
void xdl_panel_choice_curstr();
void xdl_panel_choice_tidy_up();

/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_choice (vh, wid, x, y, label, nstrings, strings, 
                               df_item, font_type, brdr, 
                               min_width, min_height, callback)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel choice's parent window (R)*/
int x;              /* x position of the panel choice area (R)*/
int y;              /* y position of the panel choice area (R)*/
char *label;        /* Label for panel choice (null terminated string)
                       (R)*/
int nstrings;       /* Number of items in choice options menu (R)*/
char **strings;     /* Pointer to list of null terminated strings with 
                       choice options (R)*/
int df_item;        /* Default item number (1 to nstrings) (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int min_width;      /* Minimum width of the panel choice item; this will 
                       be used if it is greater than the width calculated
                       using the length of the strings, font type etc. 
                       (R)*/
int min_height;     /* Minimum height of the panel choice item; this will 
                       be used if it is greater than the height calculated 
                       using the font type etc. (R)*/
void (*callback)(); /* Callback routine to take returned choice 
                       selection (R)*/

/*
Callback:  (*callback)(int iv, int item)

           iv     The offset in the XDL_view_objects array for the parent
                  view-object to which the panel choice item belongs.

           item   The selected item number from the popup menu.
*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_choice *gd;       /*Pointer to global data area*/
   Window wlabl;               /*Window id*/
   int nl;                     /*Length of label string*/
   int maxlen;                 /*maximum length of an option string*/
   int i;                      /*Loop counter*/
   int len;                    /*Length of a character string*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int brdw;                   /*Border width*/
   static int indpar[5][7] = 
   {
     {10,   3, 3,    7, 3,     5,  8},
     {12,   3, 3,    9, 3,     6,  9},
     {14,   4, 4,   10, 4,     7, 11},
     {18,   5, 5,   13, 5,     9, 14},
     {24,   5, 5,   18, 5,    12, 18}
   }; /* Indicator box parameters for 5 font sizes; values for each are
         the rectangle size (box will be one larger than this in total),
         and 3 x,y pairs of offsets for the corners of a triangle with the
         origin at the top left of the box */

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Allocate global data area */

   gd = (XDl_panel_choice *) malloc (sizeof(XDl_panel_choice));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_choice",
                       "Unable to allocate memory for global data");
      exit(1);
   }
   /* Calculate size requirements */

   nl = strlen (label);
   maxlen = 0;
   for (i=0;i<nstrings;++i)
   {
      len = strlen (strings[i]);
      if (len>maxlen) maxlen = len;
   }
   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   width = (maxlen + nl) * ch_width + indpar[font_type-1][0] + 1 + 4*MARG_W;
   if (brdr) width += 2;
   if (min_width>width) width = min_width;
   height = indpar[font_type-1][0] + 1;
   if (ch_height>height) height = ch_height;
   if (brdr) height += 6;
   if (min_height>height) height = min_height;
   gd->area_width = width;
   if (brdr) gd->area_width -= 2;
   gd->area_height = height;
   if (brdr) gd->area_height -= 2;

   /* Set global data area parameters */

   gd->label = label;
   gd->strings = strings;
   gd->nstrings = nstrings;
   gd->df_item = df_item;
   gd->cur_item = df_item;
   gd->indicator_size = indpar[font_type-1][0];
   brdw = 0;
   if (brdr) brdw = 1;
   gd->xlabel = MARG_W;
   gd->ylabel = (height-ch_height-2*brdw)/2 + ascent;
   gd->xind = 2*MARG_W + nl*ch_width;
   gd->yind = (height-gd->indicator_size-1-2*brdw)/2;
   gd->xcuropt = 3*MARG_W + nl*ch_width + gd->indicator_size + 1;
   gd->ycuropt = gd->ylabel;
   gd->x1 = gd->xind + indpar[font_type-1][1];
   gd->y1 = gd->yind + indpar[font_type-1][2];
   gd->x2 = gd->xind + indpar[font_type-1][3];
   gd->y2 = gd->yind + indpar[font_type-1][4];
   gd->x3 = gd->xind + indpar[font_type-1][5];
   gd->y3 = gd->yind + indpar[font_type-1][6];
   gd->xmenu = gd->xind;
   gd->ymenu = gd->yind + gd->indicator_size + 2;
   if (brdr) gd->ymenu += 2;
   gd->xstring = 2;
   gd->ystring = ascent + 1;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->fid = fontinfo->fid;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->maxlen = maxlen;
   
   /* Create paint window for the label, indicator square and current option */

   wlabl = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   xdl_add_libwin (vh, wlabl, (char *) gd, xdl_panel_choice_rp, 
                   xdl_panel_choice_ehp,
                   callback, xdl_panel_choice_tidy_up);
   XSelectInput (XDL_display, wlabl, ExposureMask|StructureNotifyMask|
                 ButtonPressMask);
   gd->wlabl = wlabl;

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wlabl, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_choice_rp    **
********************************

Purpose: Repaint routine for panel choice

Author:  John W. Campbell, September 1990

*/

void xdl_panel_choice_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_choice *gd;       /*Pointer to global data area*/
   int len;                    /*String length*/
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/


   gd = (XDl_panel_choice *) global;

   XClearWindow (XDL_display, wid); 

   /* Draw label string */

   XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   len = strlen (gd->label);
   xx = gd->xlabel;
   for (i=0;i<len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }

   /* Draw Indicator Box */

   XDrawRectangle (XDL_display, wid, gd->paint_gc, 
                   gd->xind, gd->yind, 
                   gd->indicator_size, gd->indicator_size);
   XDrawLine (XDL_display, wid, gd->paint_gc, gd->x1, gd->y1, gd->x2, gd->y2);
   XDrawLine (XDL_display, wid, gd->paint_gc, gd->x2, gd->y2, gd->x3, gd->y3);
   XDrawLine (XDL_display, wid, gd->paint_gc, gd->x3, gd->y3, gd->x1, gd->y1);

   /* Draw Current Label string */

   xdl_panel_choice_curstr (wid, gd);
   return;
}

/*
*********************************
**     xdl_panel_choice_ehp    **
*********************************

Purpose: Event handling routine (with pop-up menu code and event handling)

Author:  John W. Campbell, September 1990

*/

void xdl_panel_choice_ehp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_choice *gd;           /* Pointer to global data area */
   XSetWindowAttributes attrib;    /* Window attributes structure */
   int x, y;                       /* Pointer position */
   int xroot, yroot;               /* Position wrt. root window */
   int w, h;                       /* Width and height of popup menu */
   Window wmenu;                   /* Window id. of popup menu */
   XEvent ev;                      /* Event return for popup menu handling */
   int i;                          /* Loop counter */
   int yoff;                       /* y offset for menu item separating lines */
   int item;                       /* Menu item selected on popup menu */
   int first_up;                   /* First button up flag for popup menu */

   gd = (XDl_panel_choice *) global;

   if (event->type==ButtonPress)
   {
      switch (event->xbutton.button)
      {

         /* Select default item, draw default item string, call callback*/

         case Button1:
            gd->cur_item = gd->df_item;
            xdl_panel_choice_curstr (wid, gd);
            (*callback)(iv, gd->cur_item);
            break;

         /* Create and service Pop-up menu */

         case Button3:
            x = event->xbutton.x;
            y = event->xbutton.y;
            xroot = event->xbutton.x_root;
            yroot = event->xbutton.y_root;
            xroot = gd->xmenu + xroot - x;
            yroot = gd->ymenu + yroot - y;
            w = 12 + (gd->maxlen)*(gd->ch_width);
            h = 3 + (gd->nstrings)*(gd->ch_height + 5);
            if ((xroot + w)>=DisplayWidth(XDL_display,XDL_screen))
                 xroot = DisplayWidth(XDL_display,XDL_screen) - w;
            if ((yroot + h)>=DisplayHeight(XDL_display,XDL_screen))
                 yroot = DisplayHeight(XDL_display,XDL_screen) - h;
            /* Create menu window */
            attrib.background_pixel = XDL_whitepixel;
            attrib.border_pixel = XDL_blackpixel;
            attrib.colormap = XDL_colormap;
            attrib.override_redirect = True;
            attrib.save_under = True;

            wmenu = XCreateWindow (XDL_display, 
                            RootWindow(XDL_display, XDL_screen), xroot, yroot,
                            w-4, h-4, 2, XDL_pw_depth, InputOutput, XDL_visual,
                            CWBackPixel|CWBorderPixel|CWColormap|
                            CWSaveUnder|CWOverrideRedirect,
                            &attrib);

            XSelectInput (XDL_display, wmenu, ExposureMask|StructureNotifyMask);
    
            XMapWindow (XDL_display, wmenu);

            XGrabPointer (XDL_display, wmenu, False, PointerMotionMask|
                         ButtonReleaseMask, GrabModeAsync, GrabModeAsync,
                         None, None, CurrentTime);
            gd->highlight_item = 0;
            first_up = 1;
            /* Handle events for the pop-up menu */
            next:
            XNextEvent (XDL_display, &ev);
            if (ev.xany.window==wmenu)
            {
               switch (ev.type)
               {
                  /* Repaint Menu */
                  case Expose:
                  case ConfigureNotify:
                     XClearWindow (XDL_display, wmenu);
                     if (gd->nstrings>1)
                     {
                        for (i=1;i<gd->nstrings;++i)
                        {
                           yoff = i * (gd->ch_height + 5) - 1;
                           XDrawLine (XDL_display, wmenu, gd->paint_gc,
                                     0, yoff, w-5, yoff);
                        }
                     }
                     for (i=1;i<=gd->nstrings;++i)
                     {
                        xdl_panel_choice_menstr (wmenu, gd, i, 0);
                     }
                     break;

                  /* Select item and/or cancel menu */
                  case ButtonRelease:
                     if (ev.xbutton.button==Button1||ev.xbutton.button==Button3)
                     {
                        x = ev.xbutton.x;
                        y = ev.xbutton.y;
                        item = 0;
                        if (x>=0&&x<=w-5)
                        {
                           if (y>=0)
                           {
                              item = y/(gd->ch_height+5) + 1;
                              if (item>gd->nstrings) item = 0;
                           }
                        }
                        if (item>0)
                        {
                           gd->cur_item = item;
                           xdl_panel_choice_curstr (wid, gd);
                           XUngrabPointer (XDL_display, CurrentTime);
                           XDestroyWindow (XDL_display, wmenu);
                           (*callback)(iv, gd->cur_item);
                           return;
                        }
                        else
                        {
                           if (first_up==0||ev.xbutton.button==Button1)
                           {
                              XUngrabPointer (XDL_display, CurrentTime);
                              XDestroyWindow (XDL_display, wmenu);
                              return;
                           }
                           first_up = 0;
                        }
                     }
                     break;

                  /* Menu item highlighting (via bold text) */
                  case MotionNotify:
                     x = ev.xmotion.x;
                     y = ev.xmotion.y;
                     item = 0;
                     if (x>=0&&x<=w-5)
                     {
                        if (y>=0)
                        {
                           item = y/(gd->ch_height+5) + 1;
                           if (item>gd->nstrings) item = 0;
                        }
                     }
                     if (item==gd->highlight_item) break;
                     if (gd->highlight_item>0)
                     {
                        xdl_panel_choice_menstr (wmenu, gd, 
                                                 gd->highlight_item, 0);
                     }
                     gd->highlight_item = 0;
                     if (item==0) break;
                     gd->highlight_item = item;
                     xdl_panel_choice_menstr (wmenu, gd, item, 1);
                     break;
                  case LeaveNotify:
                     if (gd->highlight_item>0)
                     {
                        xdl_panel_choice_menstr (wmenu, gd, gd->highlight_item,
                                                 0);
                     }
                     gd->highlight_item = 0;
                     break;
               }
            }
            goto next;
      }
   }
   return;
}

/*
*************************************
**     xdl_panel_choice_menstr     **
*************************************

Purpose: Draw a menu string

Author:  John W. Campbell, September 1990

*/

void xdl_panel_choice_menstr (wmenu, gd, item, hl)

Window wmenu;           /* Menu window id */
XDl_panel_choice *gd;   /* Pointer to panel choice global data area */
int item;               /* Item number */
int hl;                 /* Highlight =1, =0 do not */

{
   int xx;      /* x offset for current character */
   int len;     /* String length */
   char *p;     /* Pointer to current string */
   int i;       /* Loop counter */
   int yoff;    /* y offset for string */


   XClearArea (XDL_display, wmenu, gd->xstring, (item-1)*(gd->ch_height+5),
              (gd->ch_width)*(gd->maxlen), (gd->ch_height+4), 0);
   if (hl==1)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid);
   }
   p = gd->strings[item-1];
   len = strlen (p);
   if (len==0) return;
   yoff = (item-1) * (gd->ch_height + 5) + gd->ystring;
   xx = gd->xstring;
   for (i=0;i<len;++i)
   {
      XDrawString (XDL_display, wmenu, gd->paint_gc, xx, yoff, &p[i], 1);
      xx += gd->ch_width;
   }
   return;
}
/*
*************************************
**     xdl_panel_choice_curstr     **
*************************************

Purpose: Draw the currently selected string

Author:  John W. Campbell, September 1990

*/

void xdl_panel_choice_curstr (wid, gd)

Window wid;             /*Window id*/
XDl_panel_choice *gd;   /*Pointer to panel choice global data area*/

{
   int xx;      /*x offset for current character */
   int len;     /*String length*/
   char *p;     /*Pointer to current string*/
   int i;       /*Loop counter*/

   XClearArea (XDL_display, wid, gd->xcuropt, 0,
              (gd->maxlen)*(gd->ch_width), gd->area_height, 0);

   XSetFont (XDL_display, gd->paint_gc, gd->fid);
   xx = gd->xcuropt;
   p = gd->strings[gd->cur_item-1];
   len = strlen (p);
   if (len==0) return;
   for (i=0;i<len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, 
                   gd->ycuropt, &p[i], 1);
      xx += gd->ch_width;
   }
   return;
}

/*
***************************************
**     xdl_panel_choice_tidy_up      **
***************************************

Purpose: Tidy up routine for panel choice on deletion

Author:  John W. Campbell, September 1990

*/

void xdl_panel_choice_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_choice *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_choice *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   return;
}
/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Reset panel choice item - xdl_panel_choice_reset
The routine xdl_panel_choice_reset is used to reset the selected item
for a panel choice item.
-end*/
/*
************************************
**     xdl_panel_choice_reset     **
************************************

Purpose: Reset chosen item from panel choice

Author:  John W. Campbell, August 1991

*/

/*-C:*/
void xdl_panel_choice_reset (panel_object, item)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               choice item was created using 
                               xdl_panel_choice (R)*/
int item;                    /*The number of the item to select from 
                               1 up (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_choice *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_choice *) panel_object;

   if (item<1||item>gd->nstrings) return;

   gd->cur_item = item;
   xdl_panel_choice_curstr (gd->wlabl, gd);

   XFlush (XDL_display);

   return;
}
/*-Routine: Get window id - xdl_panel_choice_wid
The routine xdl_panel_choice_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
**********************************
**     xdl_panel_choice_wid     **
**********************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_choice_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_choice *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_choice *) panel_object;
   *wid  = gd->wlabl;
   return;
}
/*-Routine: Get size - xdl_panel_choice_size
The routine xdl_panel_choice_size is used to predetermine the size of a
panel choice item.
-end*/
/*
**********************************
**     xdl_panel_choice_size    **
**********************************

Purpose: Get panel choice size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_choice_size (max_len, max_string, brdr, font_type, 
                            width, height)
/*end*/
 
/*-Parameters:*/
int max_len;    /* Maximum length of label string */
int max_string; /* Maximum length of menu choice string */
int brdr;       /* =1 draw border (1 thick), =0 do not (R)*/
int font_type;  /* Font size (1-5) (R)*/
int *width;     /* Returns the required width (W)*/
int *height;    /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   static int indpar[5][7] = 
   {
     {10,   3, 3,    7, 3,     5,  8},
     {12,   3, 3,    9, 3,     6,  9},
     {14,   4, 4,   10, 4,     7, 11},
     {18,   5, 5,   13, 5,     9, 14},
     {24,   5, 5,   18, 5,    12, 18}
   }; /* see xdl_panel_choice routine */

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *width = (max_len + max_string) * ch_width + indpar[font_type-1][0] 
            + 1 + 4*MARG_W;
   if (brdr) *width += 2;
   *height = indpar[font_type-1][0] + 1;
   if (ch_height>(*height)) *height = ch_height;
   if (brdr) *height += 6;
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel Button Item Routines
These are routines for creating and handling a panel button item.
-end*/
 

/*-Routine: Create a panel button item - xdl_panel_button
The routine xdl_panel_button creates a panel button item. A callback
routine is invoked when the panel button is selected.
-end*/
/*
******************************
**     xdl_panel_button     **
******************************

Purpose: Create a panel button item (library routine for use within 
         view-objects)

Author:  John W. Campbell, September 1990

*/

typedef struct 
{
   Window wlabl;           /*Button window id*/
   int button_number;      /*Button number*/
   int area_height;        /*Height of label area (excluding border)*/
   int area_width;         /*Width  "  "     "     "         "      */
   GC paint_gc;            /*Paint GC*/
   int ch_width;           /*Character width*/
   int ch_height;          /*Character height*/
   Font fid_normal;        /*Normal font*/
   Font fid_bold;          /*Bold font*/
   char * label;           /*Pointer to label string copy*/
   int max_len;            /*Max. length of label string*/
   int xlabel;             /*x position of start of label string*/
   int ylabel;             /*y position of start of label string*/
   int nchars;             /*No. of chars in label excluding trailing blanks*/
   int bold;               /* =0 normal, =1 bold */
   int inwin;              /* =1 pointer in window, =0 not */
   unsigned long highlight_pixel; /*Highlight pixel if colour available*/
}
  XDl_panel_button;

/*Define routine types*/

void xdl_panel_button_rp();
void xdl_panel_button_ehp();
void xdl_panel_button_drawlabl();
void xdl_panel_button_tidy_up();
void xdl_panel_button_set();


/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_button (vh, wid, x, y, label, max_len, 
                              font_type, brdw, min_width, min_height, 
                              bold, cntr, button_number, callback)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel button's parent window (R)*/
int x;              /* x position of the panel button area (R)*/
int y;              /* y position of the panel button area (R)*/
char *label;        /* Label for panel button (null terminated string) 
                       (copy will be made) (R)*/
int max_len;        /* Maximum no. of characters to be allowed for the
                       label string (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdw;           /* Border width for button (0 upwards) (R)*/
int min_width;      /* Minimum width of the panel button item; this 
                       will be used if it is greater than the width 
                       calculated using the length of the label, font 
                       type etc. (R)*/
int min_height;     /* Minimum height of the panel button item; this 
                       will be used if it is greater than the height 
                       calculated using the font type etc. (R)*/
int bold;           /* =1 bold print, =0 normal (R)*/
int cntr;           /* =0 left justified, =1 centered (R)*/
int button_number;  /* Button number to pass onto callback routine;
                       enables the same callback routine to be used
                       for several buttons if required provided that
                       each button is assigned a different number (R)*/
void (*callback)(); /* Callback routine when button selected (R)*/

/*
Callback:  (*callback)(int iv, int button_number)

           iv               The offset in the XDL_view_objects array 
                            for the parent view-object to which the 
                            panel button item belongs.

           button_number    The button number given when the button was
                            created.
*/

/*end*/
/*-Doc:
Return:  Panel_object for created item.
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_button *gd;       /*Pointer to global data area*/
   Window wlabl;               /*Window id*/
   int nl;                     /*Length of label string*/
   int i;                      /*Loop counter*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   char *p;                    /*Temporary pointer for mallocs*/

   /* Initialisations */

   if (brdw<0) brdw = 0;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Allocate global data area */

   gd = (XDl_panel_button *) malloc (sizeof(XDl_panel_button));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_button",
                       "Unable to allocate memory for global data");
      exit(1);
   }
   /* Calculate size requirements */

   nl = strlen (label);
   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   width = max_len * ch_width  + 2*MARG_W + 2*brdw;
   if (min_width>width) width = min_width;
   height = ch_height + 2*brdw;
   if (min_height>height) height = min_height;
   gd->area_width = width - 2*brdw;
   gd->area_height = height - 2*brdw;

   /* Set other global data area parameters */

   p = (char *) malloc (max_len);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_button", 
           "Unable to allocate memory for label");
      exit (1);
   }
   gd->label = p;
   gd->nchars = 1;
   for (i=0;i<max_len;++i)
   {
      if (i<nl)
      {
         gd->label[i] = label[i];
         if (label[i]!=' ') gd->nchars = i + 1;
      }
      else
      {
         gd->label[i] = ' ';
      }
   }
   gd->max_len = max_len;
   gd->xlabel = MARG_W;
   if (cntr==1) gd->xlabel = (gd->area_width - (gd->nchars)*ch_width) / 2;
   gd->ylabel = (gd->area_height-ch_height)/2 + ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->fid_normal = XDL_fonts[font_type-1]->fid;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->bold = bold;
   gd->highlight_pixel = XDL_cyanpixel;
   gd->inwin = 0;
   gd->button_number = button_number;

   /* Create paint window for the button */

   wlabl = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   gd->wlabl = wlabl;
   xdl_add_libwin (vh, wlabl, (char *) gd, xdl_panel_button_rp, 
                   xdl_panel_button_ehp,
                   callback, xdl_panel_button_tidy_up);
   XSelectInput (XDL_display, wlabl, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|EnterWindowMask|LeaveWindowMask|
                 PointerMotionMask);

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wlabl, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_button_rp    **
********************************

Purpose: Repaint routine for panel button

Author:  John W. Campbell, September 1990

*/

void xdl_panel_button_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_button *gd;       /*Pointer to global data area*/

   gd = (XDl_panel_button *) global;

   xdl_panel_button_drawlabl (gd, wid);

   return;
}
/*
**********************************
**     xdl_panel_button_ehp     **
**********************************

Purpose: Event handling routine

Author:  John W. Campbell, September 1990

*/

void xdl_panel_button_ehp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_button *gd;           /* Pointer to global data area */

   gd = (XDl_panel_button *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            gd->inwin = 0;
            xdl_panel_button_drawlabl (gd, wid);
            (*callback) (iv, gd->button_number);
         }
         return;
      case MotionNotify:
         if (gd->inwin==0)
         {
            gd->inwin = 1;
            xdl_panel_button_drawlabl (gd, wid);
         }
         return;
      case EnterNotify:
         gd->inwin = 1;
         xdl_panel_button_drawlabl (gd, wid);
         return;
      case LeaveNotify:
         gd->inwin = 0;
         xdl_panel_button_drawlabl (gd, wid);
         return;
   }
   return;
}

/*
**************************************
**     xdl_panel_button_drawlabl    **
**************************************

Purpose: Draw label string

Author:  John W. Campbell, September 1990

*/

void xdl_panel_button_drawlabl (gd, wid)

XDl_panel_button *gd;      /* Pointer to the global data area */
Window wid;               /* Window id */

{
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/

   XClearWindow (XDL_display, wid); 

   if (gd->inwin==1&&XDL_color>1)
   {
         XSetForeground (XDL_display, gd->paint_gc, gd->highlight_pixel);
         XFillRectangle (XDL_display, wid, gd->paint_gc, 0, 0,
                         gd->area_width, gd->area_height);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   }


   /* Draw label string */
   
   if (gd->bold==1||gd->inwin==1)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   }
   xx = gd->xlabel;
   for (i=0;i<gd->nchars;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }
   return;
}

/*
***************************************
**     xdl_panel_button_tidy_up      **
***************************************

Purpose: Tidy up routine for panel button on deletion

Author:  John W. Campbell, September 1990

*/

void xdl_panel_button_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_button *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_button *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   free (gd->label);
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Set a new button label string - xdl_panel_button_set
The routine xdl_panel_button_set is used to output a new label string in
a panel button.
-end*/
/*
**********************************
**     xdl_panel_button_set     **
**********************************

Purpose: Set new panel button label string

Author:  John W. Campbell, April 1990

*/

/*-C:*/
void xdl_panel_button_set (panel_object, label, len, bold, cntr)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the label
                               was created using xdl_panel_button (R)*/
char * label;                /*String containing the new label (R)*/
int len;                     /*Length of string; if 0 then length will 
                               be determined assuming a null terminated 
                               string (R)*/
int bold;                    /*Bold print flag =1 bold print, 
                               =0 normal (R)*/
int cntr;                    /*=0 left justified, =1 centered (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_button *gd;     /*Pointer to global data area*/
   int nl;                   /*No. of characters in label string*/
   int i;                    /*Temp/loop variable*/

   gd = (XDl_panel_button *) panel_object;

   gd->bold = bold;

   /* Copy label */

   nl = len;
   if (nl==0) nl = strlen (label);
   gd->nchars = 1;
   for (i=0;i<gd->max_len;++i)
   {
      if (i<nl)
      {
         gd->label[i] = label[i];
         if (label[i]!=' ') gd->nchars = i + 1;
      }
      else
      {
         gd->label[i] = ' ';
      }
   }

   gd->xlabel = MARG_W;
   if (cntr==1) gd->xlabel = (gd->area_width - (gd->nchars)*(gd->ch_width)) / 2;

   /* Draw label */

   xdl_panel_button_drawlabl (gd, gd->wlabl);

   XFlush (XDL_display);

   return;
}
/*-Routine: Get window id - xdl_panel_button_wid
The routine xdl_panel_button_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
**********************************
**     xdl_panel_button_wid     **
**********************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_button_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_button *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_button *) panel_object;
   *wid  = gd->wlabl;
   return;
}
/*-Routine: Get size - xdl_panel_button_size
The routine xdl_panel_button_size is used to predetermine the size of a
panel button item.
-end*/
/*
**********************************
**     xdl_panel_button_size    **
**********************************

Purpose: Get panel button size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_button_size (max_len, font_type, brdw, width, height)
/*end*/
 
/*-Parameters:*/
int max_len;    /* Maximum length of label string */
int font_type;  /* Font size (1-5) (R)*/
int brdw;       /* Border width (R)*/
int *width;     /* Returns the required width (W)*/
int *height;    /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/

   /* Initialisations */

   if (brdw<0) brdw = 0;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *width = max_len * ch_width  + 2*MARG_W + 2*brdw;
   *height = ch_height + 2*brdw;
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel Label Item Routines
These are routines for creating and handling a panel label item.
-end*/
 

/*-Routine: Create a panel label item - xdl_panel_label
The routine xdl_panel_label is used to create a panel label item. This
merely displays a text string which may be changed when required.
-end*/
/*
******************************
**     xdl_panel_label      **
******************************

Purpose: Create a panel label item (library routine for use within 
         view-objects)

Author:  John W. Campbell, September 1990

*/

typedef struct 
{
   Window wlabl;           /* Label window id */
   int area_height;        /*Height of label area (excluding border)*/
   int area_width;         /*Width  "  "     "     "         "      */
   GC paint_gc;            /*Paint GC*/
   int ch_width;           /*Character width*/
   int ch_height;          /*Character height*/
   Font fid_normal;        /*Normal font*/
   Font fid_bold;          /*Bold font*/
   char * label;           /*Pointer to label string copy*/
   int max_len;            /*Max. length of label string*/
   int xlabel;             /*x position of start of label string*/
   int ylabel;             /*y position of start of label string*/
   int bold;               /* =0 normal, =1 bold */
}
  XDl_panel_label;

/*Define routine types*/

void xdl_panel_label_rp();
void xdl_panel_label_tidy_up();
void xdl_panel_label_set();

/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_label (vh, wid, x, y, label, max_len, 
                              font_type, brdr, 
                              min_width, min_height, bold)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel label's parent window (R)*/
int x;              /* x position of the panel label area (R)*/
int y;              /* y position of the panel label area (R)*/
char *label;        /* Label for panel label (null terminated string) 
                       (copy will be made) (R)*/
int max_len;       /* Maximum no. of characters to be allowed for the
                       label string (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int min_width;      /* Minimum width of the panel label item; this will 
                       be used if it is greater than the width calculated 
                       using the length of the label, font type etc. (R)*/
int min_height;     /* Minimum height of the panel label item; this will 
                       be used if it is greater than the height calculated 
                       using the font type etc. (R)*/
int bold;           /* =1 bold print, =0 normal (R)*/

/*end*/
/*-Doc:
Return:  Panel_object for created item.
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_label *gd;        /*Pointer to global data area*/
   Window wlabl;               /*Window id*/
   int nl;                     /*Length of label string*/
   int i;                      /*Loop counter*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int brdw;                   /*Border width around panel item (0 or 1)*/
   char *p;                    /*Temporary pointer for mallocs*/

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Allocate global data area */

   gd = (XDl_panel_label *) malloc (sizeof(XDl_panel_label));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_label",
                       "Unable to allocate memory for global data");
      exit(1);
   }
   /* Calculate size requirements */

   nl = strlen (label);
   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   width = max_len * ch_width  + 2*MARG_W;
   if (brdr) width += 2;
   if (min_width>width) width = min_width;
   height = ch_height;
   if (brdr) height += 2;
   if (min_height>height) height = min_height;
   gd->area_width = width;
   if (brdr) gd->area_width -= 2;
   gd->area_height = height;
   if (brdr) gd->area_height -= 2;

   /* Set other global data area parameters */

   p = (char *) malloc (max_len);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_label", 
           "Unable to allocate memory for label");
      exit (1);
   }
   gd->label = p;
   for (i=0;i<max_len;++i)
   {
      if (i<nl)
      {
         gd->label[i] = label[i];
      }
      else
      {
         gd->label[i] = ' ';
      }
   }
   gd->max_len = max_len;
   gd->xlabel = MARG_W;
   gd->ylabel = (gd->area_height-ch_height)/2 + ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->fid_normal = XDL_fonts[font_type-1]->fid;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->bold = bold;

   /* Create paint window for the label */

   brdw = 0;
   if (brdr) brdw = 1;
   wlabl = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   gd->wlabl = wlabl;
   xdl_add_libwin (vh, wlabl, (char *) gd, xdl_panel_label_rp, NULL,
                   NULL, xdl_panel_label_tidy_up);
   XSelectInput (XDL_display, wlabl, ExposureMask|StructureNotifyMask);

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wlabl, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_label_rp     **
********************************

Purpose: Repaint routine for panel label

Author:  John W. Campbell, September 1990

*/

void xdl_panel_label_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_label *gd;        /*Pointer to global data area*/
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/


   gd = (XDl_panel_label *) global;

   XClearWindow (XDL_display, wid); 

   /* Draw label string */
   
   if (gd->bold)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   }
   xx = gd->xlabel;
   for (i=0;i<gd->max_len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }

   return;
}

/*
***************************************
**     xdl_panel_label_tidy_up       **
***************************************

Purpose: Tidy up routine for panel label on deletion

Author:  John W. Campbell, September 1990

*/

void xdl_panel_label_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_label *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_label *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   free (gd->label);
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Set a new panel label string - xdl_panel_label_set
The routine xdl_panel_label_set is used to output a new label text string
for a panel label item.
-end*/
/*
**********************************
**     xdl_panel_label_set      **
**********************************

Purpose: Set new panel label string

Author:  John W. Campbell, April 1990

*/

/*-C:*/
void xdl_panel_label_set (panel_object, label, len, bold)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the label
                               was created using xdl_panel_label (R)*/
char * label;                /*String containing the new label (R)*/
int len;                     /*Length of string; if 0 then length will be
                               determined assuming a null terminated 
                               string (R)*/
int bold;                    /*Bold print flag =1 bold print, 
                               =0 normal (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_label *gd;     /*Pointer to global data area*/
   int nl;                  /*No. of characters in label string*/
   int i;                   /*Temp/loop variable*/
   int xx;                  /*x position for character output*/

   gd = (XDl_panel_label *) panel_object;
   gd->bold = bold;

   /* Copy label */

   nl = len;
   if (nl==0) nl = strlen (label);
   for (i=0;i<gd->max_len;++i)
   {
      if (i<nl)
      {
         gd->label[i] = label[i];
      }
      else
      {
         gd->label[i] = ' ';
      }
   }

   /* Draw label */

   XClearWindow (XDL_display, gd->wlabl); 

   if (gd->bold)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   }
   xx = gd->xlabel;
   for (i=0;i<gd->max_len;++i)
   {
      XDrawString (XDL_display, gd->wlabl, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }

   XFlush (XDL_display);

   return;
}
/*-Routine: Get window id - xdl_panel_label_wid
The routine xdl_panel_label_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
*********************************
**     xdl_panel_label_wid     **
*********************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_label_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_label *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_label *) panel_object;
   *wid  = gd->wlabl;
   return;
}
/*-Routine: Get size - xdl_panel_label_size
The routine xdl_panel_label_size is used to predetermine the size of a
panel label item.
-end*/
/*
**********************************
**     xdl_panel_label_size     **
**********************************

Purpose: Get panel label size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_label_size (max_len, font_type, brdr, width, height)
/*end*/
 
/*-Parameters:*/
int max_len;    /* Maximum length of label string */
int font_type;  /* Font size (1-5) (R)*/
int brdr;       /* =1 draw border (1 thick), =0 do not (R)*/
int *width;     /* Returns the required width (W)*/
int *height;    /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *width = max_len * ch_width  + 2*MARG_W;
   if (brdr) *width += 2;
   *height = ch_height;
   if (brdr) *height += 2;
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel Value Item Routines
These are routines for creating and handling a panel value item.
-end*/
 

/*-Routine: Create a panel value item - xdl_panel_value
The routine xdl_panel_value is used to create a panel value item. This
displays a label and an value string. The value string may be changed
by the user causing a callback routine to be invoked.
-end*/
/*
******************************
**     xdl_panel_value      **
******************************

Purpose: Create a panel value item (library routine for use within 
         view-objects)

Author:  John W. Campbell, September 1990

*/

typedef struct 
{
   Window wlabl;           /*Window id*/
   int area_height;        /*Height of label/value area (excluding border)*/
   int area_width;         /*Width  "  "            "     "         "      */
   GC paint_gc;            /*Paint GC*/
   int ch_width;           /*Character width*/
   int ch_height;          /*Character height*/
   Font fid_bold;          /*Bold font*/
   Font fid_normal;        /*Normal font*/
   char * label;           /*Pointer to label string copy*/
   char * valstr;          /*Value string (current)*/
   char * oldval;          /*Old value string*/
   int maxv_len;           /*Max. length of value string*/
   int xlabel;             /*x position of start of label string*/
   int ylabel;             /*y position of start of label string*/
   int xvalstr;            /*x position of start of value string*/
   int yvalstr;            /*y position of start of value string 
                            (for XDrawString)*/
   int xline1;             /*x start of value underline line*/
   int xline2;             /*x end of value underline line*/
   int yline;              /*y for value underline line*/
   int yvaltop;            /*y position of top of characters*/
   int ycurs;              /*y position for the cursor*/
   int last_in;            /*Offset of last value character input*/
   int curs_offset;        /*Current cursor position offset when a value is
                             being input*/
   int item_input;         /* =1 item value being input, =0 not*/
   int inwin;              /* =1 pointer in window, =0 not*/
   unsigned long highlight_pixel; /*Color (if available) for highlighting*/
   char * paste_buf;       /* Paste buffer pointer */
   int ignore_focout;      /* Ignore Focus out events flag */
}
  XDl_panel_value;

/*Define routine types*/

void xdl_panel_value_rp();
void xdl_panel_value_ehp();
void xdl_panel_value_drawval();
void xdl_panel_value_drawcursor();
void xdl_panel_value_tidy_up();

/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_value (vh, wid, x, y, label, valstr, maxv_len, 
                              font_type, brdr, 
                              min_width, min_height, callback)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel value's parent window (R)*/
int x;              /* x position of the panel value area (R)*/
int y;              /* y position of the panel value area (R)*/
char *label;        /* Label for panel value (null terminated string) (R)*/
char *valstr;       /* Initial value string (null terminated) (R)*/
int maxv_len;       /* Maximum no. of characters to be allowed for the
                       value string (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int min_width;      /* Minimum width of the panel value item; this will be
                       used if it is greater than the width calculated 
                       using the length of the label, length of the value, 
                       font type etc. (R)*/
int min_height;     /* Minimum height of the panel value item; this will be
                       used if it is greater than the height calculated 
                       using the font type etc. (R)*/
void (*callback)(); /* Callback routine to take returned value 
                       position (R)*/

/*
Callback:  (*Callback)(int iv, char * valstr, int len, int max_len)

           iv        The offset in the XDL_view_objects array for the 
                     parent view-object to which the panel value item 
                     belongs.

           valstr    Pointer to the returned value string

           len       Length of the returned value string

           max_len   Maximum allowed length of a value string
*/

/*end*/
/*-Doc:
Return:  Panel_object for created item.
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_value *gd;        /*Pointer to global data area*/
   Window wlabl;               /*Window id*/
   int nl;                     /*Length of label string*/
   int i;                      /*Loop counter*/
   int len;                    /*Length of a character string*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int brdw;                   /*Border width around panel item (0 or 1)*/
   char *p;                    /*Temporary pointer for mallocs*/

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Allocate global data area */

   gd = (XDl_panel_value *) malloc (sizeof(XDl_panel_value));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_value",
                       "Unable to allocate memory for global data");
      exit(1);
   }

   /* Calculate size requirements */

   nl = strlen (label);
   fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   width = (nl + maxv_len) * ch_width  + 3*MARG_W;
   if (brdr) width += 2;
   if (min_width>width) width = min_width;
   height = ch_height + 2;
   if (brdr) height += 2;
   if (min_height>height) height = min_height;
   gd->area_width = width;
   if (brdr) gd->area_width -= 2;
   gd->area_height = height;
   if (brdr) gd->area_height -= 2;

   /* Set other global data area parameters */

   p = (char *) malloc (nl+1);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_value", 
           "Unable to allocate memory for label");
      exit (1);
   }
   gd->label = p;
   p = strcpy (gd->label, label);
   p = (char *) malloc (maxv_len+1);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_value", 
           "Unable to allocate memory for value_string");
      exit (1);
   }
   gd->valstr = p;
   gd->valstr[maxv_len] = 0;
   p = (char *) malloc (maxv_len+1);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_value", 
           "Unable to allocate memory for value string");
      exit (1);
   }
   gd->oldval = p;
   gd->oldval[maxv_len] = 0;
   gd->maxv_len = maxv_len;
   len = strlen (valstr);
   for (i=0;i<maxv_len;++i)
   {
      if (i<len)
      {
         gd->valstr[i] = valstr[i];
      }
      else
      {
         gd->valstr[i] = ' ';
      }
   }
   gd->xlabel = MARG_W;
   gd->ylabel = (gd->area_height-ch_height-2)/2 + ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->fid_normal = XDL_fonts[font_type-1]->fid;
   gd->xvalstr = 2*MARG_W + nl*ch_width;
   gd->yvalstr = gd->ylabel;
   gd->xline1 = gd->xvalstr;
   gd->xline2 = gd->xline1 + maxv_len*ch_width;
   gd->yline = gd->yvalstr + descent + 1;
   gd->yvaltop = (gd->area_height-ch_height)/2;
   gd->ycurs = gd->yvaltop;
   gd->last_in = -1;
   gd->curs_offset = -1;
   gd->item_input = 0;
   gd->inwin = 0;
   gd->highlight_pixel = XDL_cyanpixel;
   gd->ignore_focout = 0;

   /* Create paint window for the label and value */

   brdw = 0;
   if (brdr) brdw = 1;
   wlabl = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   gd->wlabl = wlabl;
   xdl_add_libwin (vh, wlabl, (char *) gd, xdl_panel_value_rp, 
                   xdl_panel_value_ehp,
                   callback, xdl_panel_value_tidy_up);
   XSelectInput (XDL_display, wlabl, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|KeyPressMask|FocusChangeMask|EnterWindowMask|
                 LeaveWindowMask);

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wlabl, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_value_rp     **
********************************

Purpose: Repaint routine for panel value

Author:  John W. Campbell, September 1990

*/

void xdl_panel_value_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_value *gd;        /*Pointer to global data area*/
   int len;                    /*String length*/
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/
   int hl;                     /* =1 highlight, =0 do not*/


   gd = (XDl_panel_value *) global;

   XClearWindow (XDL_display, wid); 

   /* Draw label string */

   XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   len = strlen (gd->label);
   xx = gd->xlabel;
   for (i=0;i<len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }

   /* Draw value string */

   hl = 0;
   if (gd->item_input==1&&gd->inwin==1) hl = 1;
   xdl_panel_value_drawval (gd, wid, hl);
   if (gd->item_input==1)
   {
      xx = gd->xvalstr + (gd->curs_offset)*(gd->ch_width);
      xdl_panel_value_drawcursor (gd, wid, xx, gd->ycurs);
   }

   return;
}

/*
*********************************
**     xdl_panel_value_ehp     **
*********************************

Purpose: Event handling routine

Author:  John W. Campbell, September 1990

*/

void xdl_panel_value_ehp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_value *gd;           /* Pointer to global data area */
   int i;                         /* Temp/loop variable */
   int j;                         /* Temp/loop variable */
   KeySym keysym;                 /* KeySym structure for keyboard input */
   int xx;                        /* x coordinate for character/cursor o/p */
   int yy;                        /* y coordinate for character/cursor o/p */
   char chbuf[2];                 /* Buffer for ascii translated characters */
   int nc;                        /* No. of characters decoded from keypress */
   int nitems;                    /* No. items in receive buffer */
   Window wfoc;                   /* Keyboard focus window */
   int revert;                    /* Focus revert flag */

   gd = (XDl_panel_value *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            XGetInputFocus (XDL_display, &wfoc, &revert);
            if (wfoc!=wid) 
            {
               XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
               gd->ignore_focout = 1;
	    }
            if (gd->item_input==0)
	    {
               for (i=0;i<gd->maxv_len;++i)
               {
                  gd->oldval[i] = gd->valstr[i];
                  gd->valstr[i] = ' ';
               }
               gd->last_in = -1;
               gd->curs_offset = 0;
               gd->item_input = 1;
               xdl_panel_value_drawval (gd, wid, 0);
               xdl_panel_value_drawcursor (gd, wid, gd->xvalstr, gd->ycurs);
	    }
         }
         if (event->xbutton.button==Button2)
         {
            if (gd->item_input==0) break;
            xdl_select_request (event, wid);
         }   
         break;
      case KeyPress:
         if (gd->item_input==0) return;
         nc = XLookupString ((XKeyEvent *) event, chbuf, 2, &keysym, NULL);
         if (keysym==XK_BackSpace||keysym==XK_Delete)
         {
            if (gd->last_in>=0)
            {
               gd->valstr[gd->last_in] = ' ';
               xx = gd->xvalstr + (gd->curs_offset)*(gd->ch_width);
               yy = gd->yvaltop;
               XClearArea (XDL_display, wid, xx, yy,
                           gd->ch_width, gd->ch_height, False);
               if (gd->last_in>=0)
               {
                  xx = gd->xvalstr + (gd->last_in)*(gd->ch_width);
                  XClearArea (XDL_display, wid, xx, yy,
                              gd->ch_width, gd->ch_height, False);
               }
               if ((gd->curs_offset>0)&&(gd->last_in<gd->maxv_len-1))
               {
                  gd->curs_offset--;
               }
               xx = gd->xvalstr + (gd->curs_offset)*(gd->ch_width);
               xdl_panel_value_drawcursor (gd, wid, xx, gd->ycurs);
               if (gd->last_in>=0) gd->last_in--;
            }
            return;
         }
         else if (keysym==XK_Escape)
         {
            for (i=0;i<gd->maxv_len;++i) gd->valstr[i] = gd->oldval[i];
            xdl_panel_value_drawval (gd, wid, gd->inwin);
            gd->curs_offset = -1;
            gd->last_in = -1;
            gd->item_input = 0;
            return;
         }
         else if (keysym==XK_Return||keysym==XK_KP_Enter||keysym==XK_Linefeed)
         {
            xdl_panel_value_drawval (gd, wid, gd->inwin);
            (*callback) (iv, gd->valstr, gd->last_in+1, gd->maxv_len);
            gd->curs_offset = -1;
            gd->last_in = -1;
            gd->item_input = 0;
            return;
         }
         else if (keysym>=XK_space&&keysym<=XK_asciitilde)
         {
            if (gd->last_in<(gd->maxv_len-1))
            {
               gd->last_in += 1;
               gd->valstr[gd->last_in] = chbuf[0];
               xx = gd->xvalstr + (gd->last_in)*(gd->ch_width);
               yy = gd->yvalstr;
               XClearArea (XDL_display, wid, xx, gd->yvaltop, gd->ch_width,
                           gd->ch_height, False);
               XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
               XDrawString (XDL_display, wid, gd->paint_gc, xx, yy,
                   &chbuf[0], 1);
               if (gd->curs_offset<(gd->maxv_len-1)) gd->curs_offset++;
               xx = gd->xvalstr + (gd->curs_offset)*(gd->ch_width);
               xdl_panel_value_drawcursor (gd, wid, xx, gd->ycurs);
            }
         }
         return;
      case FocusOut:
         if (gd->item_input==0) return;
         if (gd->ignore_focout) return;
         for (i=0;i<gd->maxv_len;++i) gd->valstr[i] = gd->oldval[i];
         xdl_panel_value_drawval (gd, wid, gd->inwin);
         gd->curs_offset = -1;
         gd->last_in = -1;
         gd->item_input = 0;
         return;
      case FocusIn:
         gd->ignore_focout = 0;
         return;
      case EnterNotify:
         gd->inwin = 1;
         if (gd->item_input==0) xdl_panel_value_drawval (gd, wid, 1);
         return;
      case LeaveNotify:
         gd->inwin = 0;
         if (gd->item_input==0) xdl_panel_value_drawval (gd, wid, 0);
         return;
      case SelectionNotify:
         if (xdl_select_receive(event, wid, &gd->paste_buf, &nitems,
             gd->maxv_len+1)==0) break;
         if (nitems>0&&gd->item_input)
         {
            for (j=0;j<nitems;++j)
            {
               if (gd->paste_buf[j]=='\n') break;
               if (gd->last_in<(gd->maxv_len-1))
               {
                  gd->last_in += 1;
                  gd->valstr[gd->last_in] = gd->paste_buf[j];
                  xx = gd->xvalstr + (gd->last_in)*(gd->ch_width);
                  yy = gd->yvalstr;
                  XClearArea (XDL_display, wid, xx, gd->yvaltop, gd->ch_width,
                              gd->ch_height, False);
                  XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
                  XDrawString (XDL_display, wid, gd->paint_gc, xx, yy,
                      &gd->paste_buf[j], 1);
                  if (gd->curs_offset<(gd->maxv_len-1)) gd->curs_offset++;
                  xx = gd->xvalstr + (gd->curs_offset)*(gd->ch_width);
                  xdl_panel_value_drawcursor (gd, wid, xx, gd->ycurs);
               }
            }
         }
         if (gd->paste_buf!=0) free (gd->paste_buf);
         return;
   }
   return;
}

/*
*************************************
**     xdl_panel_value_drawval     **
*************************************

Purpose: Draw value string

Author:  John W. Campbell, September 1990

*/

void xdl_panel_value_drawval (gd, wid, hl)

XDl_panel_value *gd;      /* Pointer to the global data area */
Window wid;               /* Window id */
int hl;                   /* =1 highlight, =0 do not*/

{
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/

   XClearArea (XDL_display, wid, gd->xvalstr, 0, 
              (gd->area_width - gd->xvalstr), gd->area_height, False);

   /* Draw value string */
   if (hl==0)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   }
   else
   {
      if (XDL_color>1)
      {
         XSetForeground (XDL_display, gd->paint_gc, gd->highlight_pixel);
         XFillRectangle (XDL_display, wid, gd->paint_gc, gd->xvalstr, 0,
              (gd->area_width - gd->xvalstr), gd->area_height);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      }
      XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   }
   xx = gd->xvalstr;
   for (i=0;i<gd->maxv_len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->yvalstr, 
                   &(gd->valstr[i]), 1);
      xx += gd->ch_width;
   }
   XDrawLine (XDL_display, wid, gd->paint_gc, gd->xline1, gd->yline,
              gd->xline2, gd->yline);
   return;
}

/*
*****************************************
**     xdl_panel_value_drawcursor      **
*****************************************

Purpose: Draw text input cursor

Author:  John W. Campbell, September 1990

*/

void xdl_panel_value_drawcursor (gd, wid, x, y)

XDl_panel_value *gd;       /* Pointer to the global data area */
Window wid;                /* Window id.*/
int x;                     /* x position for the cursor */
int y;                     /* y position for the cursor */

{
   y += gd->ch_height;
   XDrawLine (XDL_display, wid, gd->paint_gc,
   x+1, y-3, x+1, y-1);
   XDrawLine (XDL_display, wid, gd->paint_gc,
   x+1, y-1, x+gd->ch_width-2, y-1);
   XDrawLine (XDL_display, wid, gd->paint_gc,
   x+gd->ch_width-2, y-3, x+gd->ch_width-2, y-1);
   return;
}

/*
***************************************
**     xdl_panel_value_tidy_up       **
***************************************

Purpose: Tidy up routine for panel value on deletion

Author:  John W. Campbell, September 1990

*/

void xdl_panel_value_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_value *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_value *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   free (gd->label);
   free (gd->valstr);
   free (gd->oldval);
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Set a new panel value string - xdl_panel_value_set
The routine xdl_panel_value_set is used to replace the current panel value
string with a new value string.
-end*/
/*
**********************************
**     xdl_panel_value_set      **
**********************************

Purpose: Set new panel value string

Author:  John W. Campbell, September 1990

*/

/*-C:*/
void xdl_panel_value_set (panel_object, valstr, len)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel 
                               value was created using xdl_panel_value 
                               (R)*/
char * valstr;               /*String containing the new value (R)*/
int len;                     /*Length of string; if 0 then length will be
                               determined assuming a null terminated 
                               string (R)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_value *gd;     /*Pointer to global data area*/
   int nl;                  /*No. of characters in value string*/
   int i;                   /*Temp/loop variable*/
   int hl;                  /*=1 highlight value string, =0 do not*/

   gd = (XDl_panel_value *) panel_object;

   /* Copy value string */

   nl = len;
   if (nl==0) nl = strlen (valstr);
   for (i=0;i<gd->maxv_len;++i)
   {
      if (i<nl)
      {
         gd->valstr[i] = valstr[i];
      }
      else
      {
         gd->valstr[i] = ' ';
      }
   }

   /* Draw value string */

   hl = 0;
   if (gd->item_input==1&&gd->inwin==1) hl = 1;
   xdl_panel_value_drawval (gd, gd->wlabl, hl);

   XFlush (XDL_display);

   return;
}
/*-Routine: Get window id - xdl_panel_value_wid
The routine xdl_panel_value_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
*********************************
**     xdl_panel_value_wid     **
*********************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_value_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_value *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_value *) panel_object;
   *wid  = gd->wlabl;
   return;
}
/*-Routine: Get size - xdl_panel_value_size
The routine xdl_panel_value_size is used to predetermine the size of a
panel value item.
-end*/
/*
**********************************
**     xdl_panel_value_size     **
**********************************

Purpose: Get panel value size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_value_size (max_len, maxv_len, font_type, brdr, width, 
                           height)
/*end*/
 
/*-Parameters:*/
int max_len;    /* Maximum length of label string */
int maxv_len;   /* Maximum no. of characters to be allowed for the
                   value string (R)*/
int font_type;  /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;       /* =1 draw border (1 thick), =0 do not (R)*/
int *width;     /* Returns the required width (W)*/
int *height;    /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *width = (max_len + maxv_len) * ch_width  + 3*MARG_W;
   if (brdr) *width += 2;
   *height = ch_height + 2;
   if (brdr) *height += 2;
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel Slider Item Routines
These are routines for creating and handling a panel slider item.
-end*/
 

/*-Routine: Create a panel slider item - xdl_panel_slider
The routine xdl_panel_slider is used to create a panel slider item. This
has a label and a slider bar with a moveable grip box. When the grip
box on the slider is moved a callback routine is invoked.
-end*/
/*
******************************
**     xdl_panel_slider     **
******************************

Purpose: Create a panel slider item (library routine for use within 
         view-objects)

Author:  John W. Campbell, September 1990

*/

typedef struct 
{
   char * label;           /*Pointer to label string passed to the routine*/
   int area_height;        /*Height of label/slider area (excluding border)*/
   int area_width;         /*Width  "  "            "     "         "      */
   GC paint_gc;            /*Paint GC*/
   Window wlabl;           /*Window for label and slider */
   int xlabel;             /*x position of start of label string*/
   int ylabel;             /*y position of start of label string*/
   int ch_width;           /*Character width*/
   int ch_height;          /*Character height*/
   Font fid_bold;          /*Bold font*/
   int slider_start_x;     /*x offset of slider start*/
   int slider_end_x;       /*x offset of slider end*/
   int slider_endw;        /*slider end width*/
   int slider_grip_ytop;   /*slider grip top y offset*/
   int slider_grip_width;  /*width of slider grip*/
   int slider_grip_height; /*height of slider grip*/
   int slider_bar_ytop;    /*Slider bar top y offset*/
   int slider_bar_depth;   /*Slider bar depth (thickness)*/
   int travel_len;         /*Slider travel length*/
   int curpos;             /*Current slider position 0 to travel_len*/
   int lastpos;            /*Last drawn slider position*/
   int grip_held;          /*Grip held flag =1 yes, =0 no*/
   int xgrip;              /*Last x gripped position reference point*/
   int grip_delta;         /*x offset into grip where button was pressed*/
}
  XDl_panel_slider;

/*Define routine types*/

void xdl_panel_slider_rp();
void xdl_panel_slider_ehp();
void xdl_panel_slider_draw();
void xdl_panel_slider_tidy_up();
void xdl_panel_slider_reset();

/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_slider (vh, wid, x, y, label, font_type, 
                               travel_len, startpos, brdr, 
                               min_width, min_height, callback)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel slider's parent window (R)*/
int x;              /* x position of the panel slider area (R)*/
int y;              /* y position of the panel slider area (R)*/
char *label;        /* Label for panel slider (null terminated string) 
                       (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int travel_len;     /* Travel length of slider (in pixels). The
                       slider positions will represent travel_len+1
                       discrete values (R)*/
int startpos;       /* Start position of the slider from 0 to 
                       travel_len (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int min_width;      /* Minimum width of the panel slider item; this 
                       will be used if it is greater than the width 
                       calculated using the length of the label, 
                       length of the slider, font type etc. (R)*/
int min_height;     /* Minimum height of the panel slider item; this 
                       will be used if it is greater than the height 
                       calculated using the font type etc. (R)*/
void (*callback)(); /* Callback routine to take returned slider 
                       position (R)*/

/*
Callback:  (*callback)(int iv, int curpos, int travel_len)

           iv           The offset in the XDL_view_objects array for 
                        the parent view-object to which the panel 
                        slider item belongs.

           cupos        The current position of the grip box along
                        the slider.

           travel_len   The total travel length along the slider.
*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_slider *gd;       /*Pointer to global data area*/
   Window wlabl;               /*Window id*/
   int nl;                     /*Length of label string*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int slider_width;           /*Width of actual slider*/
   int brdw;                   /*Border width around panel item (0 or 1)*/
   static int griph[5] = {13,15,17,21,27}; /*grip heights for 5 font types (must
                                            be odd numbers)*/
   static int gripw[5] = {7,9,9,11,13}; /*grip widths for 5 font types (must be
                                         odd numbers)*/
   static int bar_depth[5] = {3,5,5,7,7}; /*slider bar depths for 5 font types
                                            (must be odd numbers)*/


   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (startpos<0) startpos = 0;
   if (startpos>travel_len) startpos = travel_len;

   /* Allocate global data area */

   gd = (XDl_panel_slider *) malloc (sizeof(XDl_panel_slider));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_slider",
                       "Unable to allocate memory for global data");
      exit(1);
   }
   /* Calculate size requirements */

   gd->travel_len = travel_len;
   gd->curpos = startpos;
   gd->slider_endw = 2;
   gd->slider_grip_width = gripw[font_type-1];
   gd->slider_grip_height = griph[font_type-1];
   gd->slider_bar_depth = bar_depth[font_type-1];

   nl = strlen (label);
   slider_width = travel_len + gd->slider_grip_width + 2*(gd->slider_endw);
   fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 1, &ch_width, &ch_height, &ascent, &descent);
   width = nl * ch_width + slider_width + 3*MARG_W;
   if (brdr) width += 2;
   if (min_width>width) width = min_width;
   height = gd->slider_grip_height;
   if (ch_height>height) height = ch_height;
   if (brdr) height += 6;
   if (min_height>height) height = min_height;
   gd->area_width = width;
   if (brdr) gd->area_width -= 2;
   gd->area_height = height;
   if (brdr) gd->area_height -= 2;

   /* Set other global data area parameters */

   gd->label = label;
   gd->xlabel = MARG_W;
   gd->ylabel = (gd->area_height-ch_height)/2 + ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->slider_start_x = 2*MARG_W + nl*ch_width;
   gd->slider_end_x = gd->slider_start_x + slider_width - 1; 
   gd->slider_grip_ytop = (gd->area_height - gd->slider_grip_height) / 2;
   gd->slider_bar_ytop = gd->slider_grip_ytop 
              + (gd->slider_grip_height - gd->slider_bar_depth) / 2;
   gd->grip_held = 0;
   gd->lastpos = 0;
   
   /* Create paint window for the label and slider */

   brdw = 0;
   if (brdr) brdw = 1;
   wlabl = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   gd->wlabl = wlabl;
   xdl_add_libwin (vh, wlabl, (char *) gd, xdl_panel_slider_rp, 
                   xdl_panel_slider_ehp,
                   callback, xdl_panel_slider_tidy_up);
   XSelectInput (XDL_display, wlabl, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|ButtonMotionMask|ButtonReleaseMask);

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wlabl, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_slider_rp    **
********************************

Purpose: Repaint routine for panel choice

Author:  John W. Campbell, September 1990

*/

void xdl_panel_slider_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_slider *gd;       /*Pointer to global data area*/
   int len;                    /*String length*/
   int i;                      /*Loop counter*/
   int xx;                     /*x position for character output*/


   gd = (XDl_panel_slider *) global;

   XClearWindow (XDL_display, wid); 

   /* Draw label string */

   XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   len = strlen (gd->label);
   xx = gd->xlabel;
   for (i=0;i<len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ylabel, 
                   &(gd->label[i]), 1);
      xx += gd->ch_width;
   }

   /* Draw Slider */

   xdl_panel_slider_draw (wid, gd);
   gd->lastpos = gd->curpos;

   return;
}

/*
*********************************
**     xdl_panel_slider_ehp    **
*********************************

Purpose: Event handling routine

Author:  John W. Campbell, September 1990

*/

void xdl_panel_slider_ehp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_slider *gd;           /* Pointer to global data area */
   XEvent xevent;                  /* Temp XEvent structure */
   int x;                          /* x position of pointer */
   int y;                          /* y position of pointer */
   int xg;                         /* x offset of start of grip */

   gd = (XDl_panel_slider *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1&&gd->grip_held==0)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            if (y<gd->slider_grip_ytop) break;
            if (y>=(gd->slider_grip_ytop+gd->slider_grip_height)) break;
            xg = gd->slider_start_x + gd->curpos + gd->slider_endw;
            if (x<xg) break;
            if (x>=xg+gd->slider_grip_width) break;
            gd->xgrip = x;
            gd->grip_delta = gd->xgrip - gd->slider_start_x - gd->slider_endw 
                             - gd->curpos;
            gd->grip_held = 1;
         }
         break;
      case ButtonRelease:
         if (event->xbutton.button==Button1&&gd->grip_held==1)
         {
            x = event->xbutton.x;
            gd->curpos += x - gd->xgrip;
            if (gd->curpos<0) gd->curpos = 0;
            if (gd->curpos>gd->travel_len) gd->curpos = gd->travel_len;
            if (gd->curpos!=gd->lastpos) xdl_panel_slider_draw (wid, gd);
            gd->lastpos = gd->curpos;
            gd->grip_held = 0;
            (*callback) (iv, gd->curpos, gd->travel_len);
         }
         break;
      case MotionNotify:
         x = event->xmotion.x;
         while (XCheckTypedWindowEvent (XDL_display, wid, MotionNotify,
                &xevent))
         {
            x = xevent.xmotion.x;
         }
         if (gd->grip_held==1)
         {
            gd->curpos += x - gd->xgrip;
            if (gd->curpos<0) gd->curpos = 0;
            if (gd->curpos>gd->travel_len) gd->curpos = gd->travel_len;
            gd->xgrip = gd->slider_start_x + gd->slider_endw + gd->curpos
                        + gd->grip_delta;
            if (gd->curpos!=gd->lastpos) xdl_panel_slider_draw (wid, gd);
            gd->lastpos = gd->curpos;
         }
         break;
   }
   return;
}
/*
************************************
**     xdl_panel_slider_draw      **
************************************

Purpose: Draw panel slider

Author:  John W. Campbell, September 1990

*/

void xdl_panel_slider_draw (wid, gd)

Window wid;                 /*Window id of the panel slider area*/
XDl_panel_slider *gd;       /*Pointer to global data area*/

{
   int xx;     /*Temp x position offset*/
   int yy;     /*Temp y position offset*/
   int w;      /*Temp width*/
   int h;      /*Temp height*/

   w = gd->slider_end_x - gd->slider_start_x +1;
   XClearArea (XDL_display, wid, gd->slider_start_x, gd->slider_grip_ytop,
               w, gd->slider_grip_height, 0);
   xx = gd->slider_start_x;
   yy = gd->slider_bar_ytop;
   w = gd->curpos + gd->slider_endw;
   h = gd->slider_bar_depth;
   XDrawRectangle (XDL_display, wid, gd->paint_gc, xx, yy, w, h-1);
   XFillRectangle (XDL_display, wid, gd->paint_gc, xx+1, yy+1, w-1, h-2);
   xx = gd->slider_start_x + gd->curpos + gd->slider_endw;
   XDrawRectangle (XDL_display, wid, gd->paint_gc, xx, gd->slider_grip_ytop,
                   gd->slider_grip_width-1, gd->slider_grip_height-1);
   xx = gd->slider_start_x + gd->curpos + gd->slider_endw 
        + gd->slider_grip_width - 1;
   w = gd->slider_end_x - xx + 1;
   XDrawRectangle (XDL_display, wid, gd->paint_gc, xx, yy, w, h-1);
   return;
}

/*
***************************************
**     xdl_panel_slider_tidy_up      **
***************************************

Purpose: Tidy up routine for panel choice on deletion

Author:  John W. Campbell, September 1990

*/

void xdl_panel_slider_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_slider *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_slider *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   return;
}
/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Reset a panel slider position - xdl_panel_slider_reset
The routine xdl_panel_slider_reset is used to set a new slider position for
a panel slider item.
-end*/
/*
*************************************
**     xdl_panel_slider_reset      **
*************************************

Purpose: Reset position of slider

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_slider_reset (panel_object, islide)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel 
                               i/o item was created using xdl_panel_io
                               (R)*/
int islide;                  /*The slider position from 0 to 'travel_len' 
                               (R)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_slider *gd;      /*Pointer to global data area*/

   gd = (XDl_panel_slider *) panel_object;

   gd->curpos = islide;
   if (gd->curpos<0) gd->curpos = 0;
   if (gd->curpos>gd->travel_len) gd->curpos = gd->travel_len;
   if (gd->curpos!=gd->lastpos) xdl_panel_slider_draw (gd->wlabl, gd);
   gd->lastpos = gd->curpos;
   return;
}
/*-Routine: Get window id - xdl_panel_slider_wid
The routine xdl_panel_slider_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
**********************************
**     xdl_panel_slider_wid     **
**********************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_slider_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_slider *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_slider *) panel_object;
   *wid  = gd->wlabl;
   return;
}
/*-Routine: Get size - xdl_panel_slider_size
The routine xdl_panel_slider_size is used to predetermine the size of a
panel slider item.
-end*/
/*
**********************************
**     xdl_panel_slider_size    **
**********************************

Purpose: Get panel slider size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_slider_size (max_len, font_type, travel_len, brdr, 
                            width, height)
/*end*/
 
/*-Parameters:*/
int max_len;    /* Maximum length of label string */
int font_type;  /* Font size (1-5) (R)*/
int travel_len; /* Travel length of slider (in pixels). The
                   slider positions will represent travel_len+1
                   discrete values (R)*/
int brdr;       /* =1 draw border (1 thick), =0 do not (R)*/
int *width;     /* Returns the required width (W)*/
int *height;    /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   static int griph[5] = {13,15,17,21,27}; /*grip heights for 5 font types (must
                                            be odd numbers)*/
   static int gripw[5] = {7,9,9,11,13}; /*grip widths for 5 font types (must be
                                         odd numbers)*/
   int slider_endw;            /* End width after slider */
   int slider_grip_width;      /* Slider grip width */
   int slider_grip_height;     /* Slider grip height */
   int slider_width;           /* Slider width */
   

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   slider_endw = 2;
   slider_grip_width = gripw[font_type-1];
   slider_grip_height = griph[font_type-1];

   slider_width = travel_len + slider_grip_width + 2*(slider_endw);
   fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 1, &ch_width, &ch_height, &ascent, &descent);
   *width = max_len * ch_width + slider_width + 3*MARG_W;
   if (brdr) *width += 2;
   *height = slider_grip_height;
   if (ch_height>(*height)) *height = ch_height;
   if (brdr) *height += 6;
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/
 
/*-Section: Panel I/O Item Routines
These are routines for creating and handling a panel i/o item.
-end*/
 

/*-Routine: Create a panel io item - xdl_panel_io
The routine xdl_panel_io is used to create a panel i/o item. This
displays a prompt string and an input reply area. This reply area
enables the user to input a character string. If this reply is
longer than the reply space available then scrolling will take place.
When a string is terminated by the Return or Enter key or if
the Escape key is pressed, a callback routine will be invoked.
-end*/
/*
******************************
**     xdl_panel_io         **
******************************

Purpose: Create a panel io item (library routine for use within 
         view-objects)

Author:  John W. Campbell, July 1992

*/

typedef struct 
{
   Window wio;             /*Window id*/
   int area_height;        /*Height of prompt/reply area (excluding border)*/
   int area_width;         /*Width  "  "            "     "         "      */
   GC paint_gc;            /*Paint GC*/
   int ch_width;           /*Character width*/
   int ch_height;          /*Character height*/
   int chcount_w;          /*Character width for character count output*/
   Font fid_bold;          /*Bold font*/
   Font fid_normal;        /*Normal font*/
   Font fid_count;         /*Font for character count*/
   char * prompt;          /*Pointer to prompt string copy*/
   char * instring;        /*Input reply string*/
   int max_pr;             /*Max allowed length of prompt string*/
   int min_rp;             /*Min width for displayed reply field*/
   int max_rp;             /*Max. allowed of input reply string*/
   int rp_width;           /*Current width (chars) of displayed
                             reply field*/
   int xprompt;            /*x position of start of prompt string*/
   int yprompt;            /*y position of start of prompt string*/
   int xreply;             /*x position of start of input reply string*/
   int yreply;             /*y position of start of input reply string 
                            (for XDrawString)*/
   int ytop;               /*y position of top of characters*/
   int ycurs;              /*y position for the cursor*/
   int xcount;             /*x position for character count*/
   int ycount;             /*y position for character count*/
   int last_in;            /*Offset of last reply character input*/
   int cur_offset;         /*Current(/cursor) position offset in the
                             input reply string (may be within the string
                             or one past the end of the string */
   int curs_offset;        /*Current cursor position offset in reply window*/
   int start_offset;       /*Offset in input reply string of first character
                             currently displayed in reply window*/
   int kbd_focus;          /*=1, keyboard focus in =0, out*/
   int active;             /*=1 input active state; =0 inactive*/
   int flag;               /*User flag to pass to callback routine*/
   int ch_count;           /*=1 show character count, =0 do not*/
   int maxdig;             /*Maximum no. digits in character count*/
   char * paste_buf;       /*Pointer to paste buffer*/
}
  XDl_panel_io;

/*Define routine types*/

void xdl_panel_io_rp();
void xdl_panel_io_ehp();
void xdl_panel_io_replystr();
void xdl_panel_io_drawcursor();
void xdl_panel_io_drawpr();
void xdl_panel_io_drawcount();
void xdl_panel_io_tidy_up();
void xdl_panel_io_prompt();
void xdl_panel_io_clear();

/*User callable routine*/

/*-C:*/
Panel_object xdl_panel_io (vh, wid, x, y, max_pr, min_rp, max_rp,
                              font_type, brdr, ch_count,
                              min_width, min_height, callback)
/*end*/
 
/*-Parameters:*/
int vh;             /* View-object handle (R)*/
Window wid;         /* Window id of panel i/o item's parent window (R)*/
int x;              /* x position of the panel i/o item area (R)*/
int y;              /* y position of the panel i/o item area (R)*/
int max_pr;         /* Max. no. of chars in prompt (R)*/
int min_rp;         /* Minimum width of displayed reply field (in 
                       characters) (R)*/
int max_rp;         /* Maximum allowed reply string length (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int ch_count;       /* =1 show character count, =0 do not (R)*/
int min_width;      /* Minimum width of the panel i/o item; this will be
                       used if it is greater than the width calculated 
                       using the maximum length of the prompt, mimimum
                       length of the reply field, font type etc. (R)*/
int min_height;     /* Minimum height of the panel i/o item; this will be
                       used if it is greater than the height calculated 
                       using the font type etc. (R)*/
void (*callback)(); /* Callback routine to take returned reply (R)*/

/*
Callback:  (*Callback)(int iv, char * reply, int len, int max_rp, 
                       int flag, int iret)

           iv        The offset in the XDL_view_objects array for the 
                     parent view-object to which the panel value item 
                     belongs.

           reply     Pointer to the returned value string

           len       Length of the returned reply string

           max_rp    Maximum allowed length of a reply string

           flag      Flag passed on from call to xdl_panel_io_prompt

           iret      = 1 non-blank string
                     = 0 blank string
                     =-1 escape pressed
         
*/

/*end*/
/*-Doc:
Return:  Panel_object for created item.
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XDl_panel_io *gd;           /*Pointer to global data area*/
   Window wio;                 /*Window id*/
   int width;                  /*Width of area required*/
   int height;                 /*Height of area required*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int chcount_w;              /*Character width for character count output*/
   int brdw;                   /*Border width around panel item (0 or 1)*/
   char *p;                    /*Temporary pointer for mallocs*/

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Allocate global data area */

   gd = (XDl_panel_io *) malloc (sizeof(XDl_panel_io));
   if (gd==0)
   {
      xdl_view_errmsg ("xdl_panel_io",
                       "Unable to allocate memory for global data");
      exit(1);
   }

   /* Calculate size requirements */

   fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   chcount_w = XDL_fonts[0]->max_bounds.width;
   width = (max_pr + min_rp) * ch_width  + 3*MARG_W;
   gd->maxdig = xdl_ndigit(max_rp);
   if (ch_count) width += gd->maxdig*chcount_w + 2;
   if (brdr) width += 2;
   if (min_width>width) 
   {
      min_rp += (min_width-width)/ch_width;
      width = min_width;
   }
   height = ch_height + 2;
   if (brdr) height += 2;
   if (min_height>height) height = min_height; 
   gd->area_width = width;
   if (brdr) gd->area_width -= 2;
   gd->area_height = height;
   if (brdr) gd->area_height -= 2;

   /* Set other global data area parameters */

   p = (char *) malloc (max_pr+1);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_io", 
           "Unable to allocate memory for prompt");
      exit (1);
   }
   gd->prompt = p;
   gd->prompt[0] = 0;
   p = (char *) malloc (max_rp+1);
   if (p==0)
   {
      xdl_view_errmsg ("xdl_panel_io", 
           "Unable to allocate memory for reply string");
      exit (1);
   }
   gd->instring = p;
   gd->xprompt = MARG_W;
   gd->yprompt = (gd->area_height-ch_height-2)/2 + ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->chcount_w = chcount_w;
   gd->fid_bold = XDL_bold_fonts[font_type-1]->fid;
   gd->fid_normal = XDL_fonts[font_type-1]->fid;
   gd->fid_count = XDL_fonts[0]->fid;
   gd->xreply = 2*MARG_W + max_pr*ch_width;
   gd->yreply = gd->yprompt;
   gd->xcount = 3*MARG_W + (max_pr + min_rp)*ch_width + 1;
   gd->ycount = gd->area_height - 2;
   gd->rp_width = min_rp;
   gd->ytop = (gd->area_height-ch_height)/2;
   gd->ycurs = gd->ytop;
   gd->max_pr = max_pr;
   gd->min_rp = min_rp;
   gd->max_rp = max_rp;
   gd->active = 0;
   gd->kbd_focus = 0;
   gd->ch_count = ch_count;

   /* Initialize reply string variables */

   gd->instring[0] = 0;
   gd->last_in = -1;
   gd->start_offset = 0;
   gd->cur_offset = 0;
   gd->curs_offset = 0;

   /* Create paint window for the prompt and reply */

   brdw = 0;
   if (brdr) brdw = 1;
   wio = xdl_create_paint_window (wid, x, y, brdw, width, height, 0);
   gd->wio = wio;
   xdl_add_libwin (vh, wio, (char *) gd, xdl_panel_io_rp, xdl_panel_io_ehp,
                   callback, xdl_panel_io_tidy_up);
   XSelectInput (XDL_display, wio, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|KeyPressMask|FocusChangeMask|EnterWindowMask|
                 LeaveWindowMask);

   /* Create and set paint_gc */
   
   gd->paint_gc = XCreateGC (XDL_display, wio, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);

   return (Panel_object) gd;
}

/*================================*/
/*  'Private' service routines    */
/*================================*/

/*
********************************
**     xdl_panel_io_rp        **
********************************

Purpose: Repaint routine for panel i/o item

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_io *gd;           /*Pointer to global data area*/


   gd = (XDl_panel_io *) global;

   XClearWindow (XDL_display, wid); 

   /* Draw prompt string */

   xdl_panel_io_drawpr (gd);

   /* Draw reply string */

   xdl_panel_io_replystr (gd);
   return;
}

/*
*********************************
**     xdl_panel_io_ehp        **
*********************************

Purpose: Event handling routine

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_ehp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_panel_io *gd;              /* Pointer to global data area */
   int i;                         /* Temp/loop variable */
   int j;                         /* Temp/loop variable */
   KeySym keysym;                 /* KeySym structure for keyboard input */
   XComposeStatus compose;        /* Compose status */
   int xx;                        /* x coordinate for character/cursor o/p */
   int yy;                        /* y coordinate for character/cursor o/p */
   char chbuf[2];                 /* Buffer for ascii translated characters */
   int nc;                        /* No. of characters decoded from keypress */
   int iret;                      /* Return flag 0 = normal, -1 = escape */
   int nitems;                    /* No. of items received from select */


   gd = (XDl_panel_io *) global;
   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
            gd->kbd_focus = 1;
            xdl_panel_io_replystr(gd);
         }
         if (event->xbutton.button==Button2)
         {
            if (gd->active==0) break;
            xdl_select_request (event, wid);
         }   
         break;
      case FocusIn:
         if (gd->active==0) return;
         gd->kbd_focus = 1;
         xdl_panel_io_replystr(gd);
         break;
      case FocusOut:
         if (gd->active==0) return;
         gd->kbd_focus = 0;
         xdl_panel_io_replystr(gd);
         break;
      case KeyPress:
         if (gd->active==0) return;
         nc = XLookupString ((XKeyEvent *) event, chbuf, 2, &keysym, NULL);
         if (keysym==XK_BackSpace||keysym==XK_Delete)
         {
            if (gd->last_in>=0)
            {
               gd->curs_offset = gd->cur_offset - gd->start_offset;
               if (gd->curs_offset>0)
               {
                  if (gd->cur_offset>gd->last_in)
                  {
                     xx = gd->xreply + gd->curs_offset*gd->ch_width;
                     yy = gd->ytop;
                     XClearArea (XDL_display, wid, xx, yy,
                                 gd->ch_width, gd->ch_height, False);
                     xx = gd->xreply + (gd->last_in-gd->start_offset)
                          *(gd->ch_width);
                     XClearArea (XDL_display, wid, xx, yy,
                                 gd->ch_width, gd->ch_height, False);
                     gd->cur_offset--;
                     gd->curs_offset--;
                     xx = gd->xreply + gd->curs_offset*(gd->ch_width);
                     xdl_panel_io_drawcursor (wid, gd->paint_gc, xx, 
                                              gd->ycurs,
                                              gd->ch_width, gd->ch_height,
                                              gd->kbd_focus);
                     gd->last_in--;
                     xdl_panel_io_drawcount (gd);
                  }
                  else
                  {
                     for (i=gd->cur_offset;i<=gd->last_in;++i)
                     {
                        gd->instring[i-1] = gd->instring[i];
                     }
                     gd->last_in--;
                     gd->cur_offset--;
                     xdl_panel_io_replystr (gd);
                  }
               }
               else
               {
                  if (gd->cur_offset<=gd->last_in)
                  {
                     for (i=gd->cur_offset;i<=gd->last_in;++i)
                     {
                        gd->instring[i-1] = gd->instring[i];
                     }
                  }
                  gd->last_in--;
                  gd->cur_offset--;
                  gd->start_offset--;
                  xdl_panel_io_replystr (gd);
               }
            }
            break;
         }
         else if (keysym==XK_Escape)
         {
            iret = -1;
            gd->instring[0] = 0;
            gd->last_in = -1;
            goto callbk;
         }
         else if (keysym==XK_Return||keysym==XK_KP_Enter
                  ||keysym==XK_Linefeed)
         {
            XClearArea (XDL_display, wid, gd->xreply, gd->ytop,
                       (gd->rp_width*(gd->ch_width)), gd->ch_height, False);
            xx = gd->xreply;
            iret = 0;
            if (gd->last_in>=0)
            {
               for (i=gd->start_offset;i<=gd->last_in;++i)
               {
                   XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->yreply,
                                &gd->instring[i], 1);
                   xx += gd->ch_width;
                   if (gd->instring[i]!=' ') iret = 1;
               }
            }
            gd->instring[gd->last_in+1] = 0;
            goto callbk;
         }
         else if (keysym>=XK_space&&keysym<=XK_asciitilde)
         {
            if (gd->last_in<(gd->max_rp-1))
            {
               if (gd->cur_offset>gd->last_in)
               {
                  gd->last_in += 1;
                  gd->instring[gd->last_in] = chbuf[0];
                  gd->curs_offset = gd->cur_offset - gd->start_offset;
                  if (gd->curs_offset<gd->rp_width-2)
                  {
                     xx = gd->xreply + (gd->last_in-gd->start_offset)
                                     * (gd->ch_width);
                     yy = gd->yreply;
                     XClearArea (XDL_display, wid, xx, gd->ytop, 
                                 gd->ch_width, gd->ch_height, False);
                     XDrawString (XDL_display, wid, gd->paint_gc, xx, yy,
                         &chbuf[0], 1);
                     gd->cur_offset++;
                     gd->curs_offset++;
                     xx = gd->xreply + gd->curs_offset*(gd->ch_width);
                     xdl_panel_io_drawcursor (wid, gd->paint_gc, xx, 
                               gd->ycurs, gd->ch_width, gd->ch_height,
                               gd->kbd_focus);
                     xdl_panel_io_drawcount (gd);
                  }
                  else
                  {
                     gd->cur_offset++;
                     gd->start_offset++;
                     xdl_panel_io_replystr (gd);
                  }
               }
               else
               {
                  gd->last_in++;
                  for (i=gd->last_in;i>gd->cur_offset;--i)
                  {
                     gd->instring[i]=gd->instring[i-1];
                  }
                  gd->instring[gd->cur_offset] = chbuf[0];
                  gd->cur_offset++;
                  gd->curs_offset = gd->cur_offset - gd->start_offset;
                  if (gd->curs_offset>=gd->rp_width)
                  {
                     gd->start_offset++;
                  }
                  xdl_panel_io_replystr (gd);
               }
            }
         }
         else if (keysym>=XK_Right)
         {
            if (gd->cur_offset<=gd->last_in)
            {
               gd->cur_offset++;
               gd->curs_offset = gd->cur_offset - gd->start_offset;
               if (gd->curs_offset>=gd->rp_width)
               {
                  gd->start_offset++;
               }
               xdl_panel_io_replystr (gd);
            }
         }
         else if (keysym>=XK_Left)
         {
            if (gd->last_in>=0)
            {
               if (gd->cur_offset>0)
               {
                  gd->cur_offset--;
                  gd->curs_offset = gd->cur_offset - gd->start_offset;
                  if (gd->curs_offset<0&&gd->start_offset>0)
                  {
                     gd->start_offset--;
                  }
                  xdl_panel_io_replystr (gd);
               }
            }
         }
         break;
      case SelectionNotify:
         if (xdl_select_receive(event, wid, &gd->paste_buf, &nitems,
             gd->max_rp+1)==0) break;
         if (nitems>0&&gd->active)
         {
            for (j=0;j<nitems;++j)
            {
               if (gd->paste_buf[j]=='\n') break;
               if (gd->last_in<(gd->max_rp-1))
               {
                  if (gd->cur_offset>gd->last_in)
                  {
                     gd->last_in += 1;
                     gd->instring[gd->last_in] = gd->paste_buf[j];
                     gd->curs_offset = gd->cur_offset - gd->start_offset;
                     if (gd->curs_offset<gd->rp_width-2)
                     {
                        gd->cur_offset++;
                        gd->curs_offset++;
                     }
                     else
                     {
                        gd->cur_offset++;
                        gd->start_offset++;
                     }
                  }
                  else
                  {
                     gd->last_in++;
                     for (i=gd->last_in;i>gd->cur_offset;--i)
                     {
                        gd->instring[i]=gd->instring[i-1];
                     }
                     gd->instring[gd->cur_offset] = gd->paste_buf[j];
                     gd->cur_offset++;
                     gd->curs_offset = gd->cur_offset - gd->start_offset;
                     if (gd->curs_offset>=gd->rp_width)
                     {
                        gd->start_offset++;
                     }
                  }
               }
            }
            xdl_panel_io_replystr (gd);
         }
         if (gd->paste_buf!=0) free (gd->paste_buf);
         break;
   }
   return;

   /* Callback required */
   callbk:
   gd->active = 0;
   xdl_panel_io_replystr(gd);
   (*callback) (iv, gd->instring, gd->last_in+1, gd->max_rp, gd->flag, iret);
   return;
}

/*
*************************************
**     xdl_panel_io_drawcursor     **
*************************************

Purpose: Draw text input cursor

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_drawcursor (wid, gc, x, y, ch_width, ch_height, focus)

Window wid;                /* Window id.*/
GC gc;                     /* Graphics context */
int x;                     /* x position for the cursor */
int y;                     /* y position for the cursor */
int ch_width;              /* Character width */
int ch_height;             /* Character height */
int focus;                 /* =1, draw keyboard focus in cursor, =0, draw
                              keyboard focus out cursor*/

{
   if (focus)
   {
      y += ch_height;
      XDrawLine (XDL_display, wid, gc,
      x+1, y-3, x+1, y-1);
      XDrawLine (XDL_display, wid, gc,
      x+1, y-1, x+ch_width-2, y-1);
      XDrawLine (XDL_display, wid, gc,
      x+ch_width-2, y-3, x+ch_width-2, y-1);
   }
   else
   {
      XDrawRectangle (XDL_display, wid, gc, x, y, ch_width-1, ch_height-1);
   }
   return;
}
/*
*********************************
**     xdl_panel_io_drawpr     **
*********************************

Purpose: Draw prompt

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_drawpr (gd)

XDl_panel_io *gd;
{
   int i;    /*Temp/loop variable*/
   int len;  /*Length of prompt string*/
   int xx;   /*Current character x position*/

   XSetFont (XDL_display, gd->paint_gc, gd->fid_bold);
   len = strlen (gd->prompt);
   xx = gd->xprompt;
   if (len>0)
   {
      for (i=0;i<len;++i)
      {
         XDrawString (XDL_display, gd->wio, gd->paint_gc, xx, gd->yprompt, 
                      &(gd->prompt[i]), 1);
         xx += gd->ch_width;
      }
   }
   XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   return;
}
/*
*************************************
**     xdl_panel_io_drawcount      **
*************************************

Purpose: Draw character count if required

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_drawcount (gd)

XDl_panel_io *gd;
{
   int i;         /*Temp/loop variable*/
   int len;       /*Length of character count output string*/
   int xx;        /*Current character x position*/
   char str[20];  /*Temporary character count string*/

   if (gd->ch_count==0) return;
   XClearArea (XDL_display, gd->wio, gd->xcount, 0,
               gd->maxdig*(gd->chcount_w), gd->area_height, False);
   if(gd->last_in<0) return;
   sprintf(str,"%d",gd->last_in+1);
   len = strlen(str);
   XSetFont (XDL_display, gd->paint_gc, gd->fid_count);
   xx = gd->xcount;
   if (len>0)
   {
      for (i=0;i<len;++i)
      {
         XDrawString (XDL_display, gd->wio, gd->paint_gc, xx, gd->ycount, 
                      &str[i], 1);
         xx += gd->chcount_w;
      }
   }
   XSetFont (XDL_display, gd->paint_gc, gd->fid_normal);
   return;
}
/*
*************************************
**     xdl_panel_io_replystr       **
*************************************

Purpose: Redraw reply string in its current state

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_replystr (gd)

XDl_panel_io *gd;  /*Pointer to global data area*/
{
   int i, j;
   int xx;
   int nch;

   XClearArea (XDL_display, gd->wio, gd->xreply, gd->ytop,
              (gd->rp_width*(gd->ch_width)), gd->ch_height, False);
   if (gd->last_in>=0)
   {
      nch = gd->last_in - gd->start_offset + 1;
      if (nch>gd->rp_width) nch = gd->rp_width;
      if (nch>0)
      {   
         xx = gd->xreply;
         for (i=0,j=gd->start_offset;i<nch;++i,++j)
         {
            XDrawString (XDL_display, gd->wio, gd->paint_gc, xx, gd->yreply,
                         &gd->instring[j], 1);
            xx += gd->ch_width;
         }
      }
   }
   xx = gd->xreply + (gd->cur_offset-gd->start_offset)*(gd->ch_width);
   if (gd->active) xdl_panel_io_drawcursor (gd->wio, gd->paint_gc, xx, 
                                            gd->ycurs, 
                                            gd->ch_width, gd->ch_height,
                                            gd->kbd_focus);
   xdl_panel_io_drawcount(gd);
   return;
}
/*
***************************************
**     xdl_panel_io_tidy_up          **
***************************************

Purpose: Tidy up routine for panel i/o item on deletion

Author:  John W. Campbell, July 1992

*/

void xdl_panel_io_tidy_up (global)

char *global;      /* Pointer to panel choice global data area */
{
   XDl_panel_io *gd;    /* Pointer to panel choice global data area */

   gd = (XDl_panel_io *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   free (gd->prompt);
   free (gd->instring);
   return;
}

/*==================================*/
/*         'Public Routines'        */
/*==================================*/

/*-Routine: Set a new panel i/o item prompt - xdl_panel_io_prompt
The routine xdl_panel_io_prompt is used to set a new prompt and prepare
for a new input reply string.
-end*/
/*
**********************************
**     xdl_panel_io_prompt      **
**********************************

Purpose: Set new panel prompt and prepare for reply

Author:  John W. Campbell, July 1992

*/

/*-C:*/
void xdl_panel_io_prompt (panel_object, prompt, len, flag)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel 
                               i/o item was created using xdl_panel_io
                               (R)*/
char * prompt;               /*String containing the new prompt (R)*/
int len;                     /*Length of prompt; if 0 then length will be
                               determined assuming a null terminated 
                               string (R)*/
int flag;                    /*Flag to be passed to callback routine to
                               identify current prompt etc. */
/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_io *gd;          /*Pointer to global data area*/
   int nl;                    /*No. of characters in prompt string*/
   XWindowAttributes attrib;  /* Window attributes structure */

   gd = (XDl_panel_io *) panel_object;

   /* Copy prompt string */

   nl = len;
   if (nl==0) nl = strlen (prompt);
   if (nl>gd->max_pr) nl = gd->max_pr;
   gd->prompt[0] = 0;
   strncat (gd->prompt, prompt, nl);
   gd->instring[0] = 0;
   gd->last_in = -1;
   gd->start_offset = 0;
   gd->cur_offset = 0;
   gd->curs_offset = 0;
   gd->rp_width = gd->max_pr + gd->min_rp - nl;
   gd->xreply = 2*MARG_W + nl*(gd->ch_width);
   XGetWindowAttributes (XDL_display, gd->wio, &attrib);
   if (attrib.map_state==IsViewable)
   {
      XSetInputFocus (XDL_display, gd->wio, RevertToParent, CurrentTime);
      gd->kbd_focus = 1;
   }
   gd->active = 1;
   gd->flag = flag;

   /* Draw prompt string & reply cursor*/

   XClearWindow (XDL_display, gd->wio); 

   xdl_panel_io_drawpr (gd);

   xdl_panel_io_replystr (gd);

   XFlush (XDL_display);

   return;
}
/*-Routine: Clear/reset a panel i/o item - xdl_panel_io_clear
The routine xdl_panel_io_clear is used to clear a panel i/o item. It
will remain inactive until another call of the routine xdl_panel_io_prompt.
-end*/
/*
**********************************
**     xdl_panel_io_clear       **
**********************************

Purpose: Clear/reset panel io area

Author:  John W. Campbell, July 1992

*/

/*-C:*/
void xdl_panel_io_clear (panel_object)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel 
                               i/o item was created using xdl_panel_io
                               (R)*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_io *gd;        /*Pointer to global data area*/

   gd = (XDl_panel_io *) panel_object;

   /* Clear window */

   XClearWindow (XDL_display, gd->wio); 
   gd->prompt[0] = 0;
   gd->instring[0] = 0;
   gd->last_in = -1;
   gd->start_offset = 0;
   gd->cur_offset = 0;
   gd->curs_offset = 0;
   gd->rp_width = gd->min_rp;
   gd->xreply = 2*MARG_W + gd->max_pr*(gd->ch_width);
   gd->active = 0;
   XFlush (XDL_display);

   return;
}
/*-Routine: Get window id - xdl_panel_io_wid
The routine xdl_panel_io_wid is used to return the window identifier
of the panel item's window.
-end*/
/*
******************************
**     xdl_panel_io_wid     **
******************************

Purpose: Get window id

Author:  John W. Campbell, August 1993

*/

/*-C:*/
void xdl_panel_io_wid (panel_object, wid)
/*end*/
 
/*-Parameters:*/
Panel_object panel_object;   /*The panel object returned when the panel
                               item was created (R) */
Window *wid;                 /*Returns the window identifier (W) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XDl_panel_io *gd;    /*Pointer to global data area*/

   gd = (XDl_panel_io *) panel_object;
   *wid  = gd->wio;
   return;
}
/*-Routine: Get size - xdl_panel_io_size
The routine xdl_panel_io_size is used to predetermine the size of a
panel I/O item.
-end*/
/*
**********************************
**     xdl_panel_io_size        **
**********************************

Purpose: Get panel I/O item size

Author:  John W. Campbell, October 1993

*/

/*-C:*/
void xdl_panel_io_size (max_pr, min_rp, max_rp, font_type, brdr, ch_count,
                        width, height)
/*end*/
 
/*-Parameters:*/
int max_pr;         /* Max. no. of chars in prompt (R)*/
int min_rp;         /* Minimum width of displayed reply field (in 
                       characters) (R)*/
int max_rp;         /* Maximum allowed reply string length (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int brdr;           /* =1 draw border (1 thick), =0 do not (R)*/
int ch_count;       /* =1 show character count, =0 do not (R)*/
int *width;         /* Returns the required width (W)*/
int *height;        /* Returns the required height (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   int chcount_w;              /*Width for character count o/p */
   int maxdig;                 /*Max no. of digits for character count */

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   fontinfo = XDL_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   chcount_w = XDL_fonts[0]->max_bounds.width;
   *width = (max_pr + min_rp) * ch_width  + 3*MARG_W;
   maxdig = xdl_ndigit(max_rp);
   if (ch_count) *width += maxdig*chcount_w + 2;
   if (brdr) *width += 2;
   *height = ch_height + 2;
   if (brdr) *height += 2;
   return;
}
