/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Parameter Table View-Object Routines
-end*/
 
/*-Intro:
The parameter table view-object provides an area containing an
editable table of parameter values. The parameter table has an
optional title. The table itself consists of one or more columns,
each with a number of rows containing a parameter name and a
parameter value. The parameter table has an 'active strip' at the top 
to indicate when the entries may be edited. When the program is 
prepared to service parameter value edits, then the message 
'Edits allowed' is displayed in the active strip. 

In some cases, parameter values may be displayed as 'toggle' values.
These are either a 'Yes/No' toggle or an 'On/Off' toggle. The toggle
value is displayed enclosed in a box. Label items with no values are
also allowed; these may be useful to group parameters under different
headings.

A parameter table item may have a popup values menu associated with it
to enable the selection of a value from a predefined list of values.
The presence of a menu associated with a parameter value is indicated
by a rectangle containing a downward pointing arrow positioned at the
right hand side of the value field. As an alternative, a step-button
may be displayed; this has two parts the upper right portion and the
lower left portion which may be selected to enable stepping forward
or backward through a series of items (e.g. incrementing or
decrementing a count); the actual action taken is under the control of 
the calling program. (For a given parameter item, toggle values and a 
popup menu (or step-button) are mutually exclusive)

Sometimes, the application may choose to emphasise certain items by
displaying them in bold print, perhaps to bring particularly important
items to the attention of the user. Also, sometimes the application 
may choose to make some items 'silent'. These are effectively removed 
from the display and cannot be seen or modified until restored by the 
program.

If parameter values are too long to be displayed, then scrolling may
take place. 

When a value is being input, text may be 'pasted in' from another window
by pressing the middle mouse button.
-end*/
 
/*-Section: Creating and Handling a Parameter Table View-Object
-end*/
 

/*Routines List:
The parameter table view-object has the following routines available:

   xdl_param_table               Create a new parameter table view-object
   xdl_param_table_getvalue      Get an input value from the table
   xdl_param_table_error         Display error message & replace value
   xdl_param_table_setitem       Set an item (with value) in the table
   xdl_param_table_setvalue      Set a new value in the table
   xdl_param_table_setmenu       Set a values menu for the parameter
   xdl_param_table_setbutton     Set a step-button for the parameter
   xdl_param_table_clrmenu       Clear (remove) a values menu /step-button
   xdl_param_table_silent        Make selected items 'silent'
   xdl_param_table_normal        Make selected items 'normal'
   xdl_param_table_bold          Make selected items 'bold'
   xdl_param_table_all_normal    Reset all items to 'normal'
   xdl_param_table_delitem       Delete an item from the table
   xdl_param_table_clearitems    Clear all items from the table
   xdl_param_table_newtitle      Output a new title
   xdl_param_table_getsize       Calculate the size requirements
*/
   
/*-Routine: Create a parameter table view-object - xdl_param_table
The routine xdl_param_table (xdlf_param_table) is used to create a new
parameter table view-object. The requirements for the table such as
the numbers of rows and columns of items and the maximum lengths of
item name and value strings are given together with a title if
required. The actual items are set individually at a later stage
using for example xdl_param_table_setitem (xdlf_param_table_setitem).
-end*/
/*
**************************
**    xdl_param_table   **
**************************

Purpose: Display a parameter table with editable values

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE (IVH, IVHPARENT, IX, IY, ICSET, NCOLS, 
        +                       NROWS, LENNAM, LENVAL, MAXVAL,
        +                       XDLSTR(TITLE), 
        +                       ITLEN, IFONT, IMENU, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
NCOLS     (R)   Max. no. of cols of items (see num_cols)
NROWS     (R)   Max. no. rows of items (see num_rows)
LENNAM    (R)   Maximum parameter name length in characters (see
                max_name_chars)
LENVAL    (R)   Maximum length in characters of a parameter value
                string (displayed) (see max_value_disp)
MAXVAL    (R)   Maximum length of value which may be saved
                (if > lenval, then scrolling allowed) (see
                max_value_chars)
TITLE     (R)   Title string for the parameter table (see title)
                ** Pass address using the XDLSTR function **
ITLEN     (R)   The title length > 0 or -1 if no title required 
                (see nt)
IFONT     (R)   Font type (1->5 verysmall->extralarge) 
                (see font_type)
IMENU     (R)   = 1 Allow popup values menus and/or step-button, 
                = 0 do not.
IERR      (W)   Returns status from xdl_param_table call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#ifdef VMS
#include <decw$include/keysym.h>
#else
#include <X11/keysym.h>
#endif

#define MARG_W 2     /*Margin width around item - must be at least 2*/
#define MARG_H 2
#define SEP_W 4      /*Space between end of name and start of value*/
#define SEP_H 1      /*Additional space between each row*/
#define SEP_M 2      /*Space between end of value and menu/step button (if
                       present)*/

/* Define structure to hold preset and status info for each item */

typedef struct
{
   unsigned char status;   /*Parameter status =0 not set up, =1 set up*/
   int emphasize;          /*0 = normal, 1 = emphasize, -1 = 'silent'*/
   int toggle;             /*0 = normal value; 1,2 = on/off toggle 1=on,
                             2=no; 3,4 = yes/no toggle 3=yes, 4=no;
                             = -1 label item (no value)*/
   int menu;               /* = 1 values menu present, 
                              = 2 step-button present
                              = 0 no menu/step-button present*/
   char *menu_vals;        /*Pointer to menu items (/step-button) values 
                             strings*/
   int len_menu_vals;      /*Length/type of menu value (/step-button) strings*/
   int num_menu_vals;      /*Number of menu item values  in the menu or
                             2 for a step-button option*/
   int toff_name;          /*Offset in text array to parameter name*/
   int toff_val;           /*Offset in text array to parameter value*/
   int toff_oldval;        /*Offset in text array to parameter old value*/
                           /*The following are offsets in the table canvas*/
   int xoff;               /*x position (top left) of item area*/
   int yoff;               /*y position (top left) of item area*/
   int xoff_name;          /*x position (top left) of item name*/
   int yoff_name;          /*y position (top left) of item name*/
   int xoff_val;           /*x position (top left) of item value*/
   int yoff_val;           /*y position (top left) of item value*/
   int xoff_ind;           /*x position (top left) of menu indicator box*/
   int yoff_ind;           /*y position (top left) of menu indicator box*/
   int xoff_menu;          /*x position (top left) for popup menu*/
   int yoff_menu;          /*y position (top left) for popup menu*/
   int sc1;                /*Value scrolled off left side =1, yes, =0 no*/
   int sc2;                /*Value scrolled off right side =1, yes, =0 no*/
   int minsc;              /*Offset of start character in displayed value
                             string (for scrolling option) */
   int nsc;                /*No of value characters to display for scrolling
                             option */
}
   XDl_param_table_details;

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                          /*Primary window id*/
   Window wparam;                          /*Parameter table window*/
   Window wact;                            /*Active strip window*/
   GC          paint_gc;                   /*GC for parameter table use*/
   char        *text;                      /*Array to hold parameter names and
                                             values*/
   int         repaint_done;               /*Initial repaint done flag*/
   int         ch_width;                   /*Character width in pixels*/
   int         ch_height;                  /*Character height in pixels*/
   int         table_width;                /*Table width in pixels*/
   int         table_height;               /*Table height in pixels*/
   int         num_cols;                   /*No. of columns of parameters*/
   int         num_rows;                   /*No. of rows of parameters*/
   int         num_params;                 /*Maximum number of parameters*/
   int         max_name_chars;             /*Max no. chars in a parameter name*/
   int         max_value_disp;             /*Max no. chars win a parameter value
                                             which may be displayed*/
   int         max_value_chars;            /*Max no. chars in a param value
                                             which may be saved*/
   int         font_type;                  /*Font type size 1-5*/
   XFontStruct *fontinfo;                  /*Pointer to structure for normal
                                             print font*/
   XFontStruct *bold_fontinfo;             /*Pointer to structure for bold
                                             print font*/
   int         font_descent;               /*Font descent*/
   int         font_ascent;                /*Font ascent*/
   int         max_width;                  /*Font maximum character width*/
   int         last_in;                    /*Offset of last value char input*/
   int         curs_offset;                /*Current cursor position offset
                                             when a value is being input*/
   int         on_off;                     /*Program awaits input T/F flag*/
   int         kbd_focus;                  /*Table has kbd focus T/F flag*/
   int         yo_start;                   /*y offset in table to top of first
                                             item*/
   int         col_width;                  /*Width of a table column in pixels*/
   int         row_height;                 /*Height of a table row in pixels*/
   char        *title;                     /*Table title*/
   int         title_height;               /*Height of title (0 if none)*/
   int         title_len;                  /*Title length*/
   int         title_xoff;                 /*x offset for start of title*/
   int         highlight_item;             /*Number of the currently highlighted
                                             item - 0 if none*/
   int         highlight_item_but;         /*Highlight status of button of
                                             currently highlighted item (1 or
                                             2 if on button, 0 if off button) */
   unsigned long highlight_pixel;          /*Color (if available) for 
                                             highlighting*/
   int         input_item;                 /*Number of the item whose value is
                                             currently being input - 0 if none*/
   int         last_input_item;            /*Number of the item whose value was
                                             the last one input*/
   int         step_button;                /*If last input item was step-button
                                             then hold the button number 1 or 2;
                                             otherwise zero */
   int         sav_sc1;                    /*Saved scrolling params during
                                             value input*/
   int         sav_sc2;                    /* " */
   int         sav_minsc;                  /* " */
   int         sav_nsc;                    /* " */
   int         scroll_item;                /* Item number for which value
                                              scrolling is in progress; 0
                                              if none */
   int         menus_allowed;              /*=1 values menus allowed, = 0 not*/
   int         ind_x1;                     /*Menu indicator triangle offsets*/
   int         ind_y1;                     /* " */
   int         ind_x2;                     /* " */
   int         ind_y2;                     /* " */
   int         ind_x3;                     /* " */
   int         ind_y3;                     /* " */
   int         ind_w;                      /*Indicator box width-1*/
   int         ind_h;                      /*Indicator box height-1*/
   XDl_param_table_details * item_details; /*Status and preset details table
                                             for the items (see structure
                                             definition for details*/
 
}  
   XDl_param_table;

/* Define routine types */
int xdl_param_table();
void xdl_param_table_rps();
void xdl_param_table_rp1();
void xdl_param_table_ep1();
void xdl_param_table_on_off();
void xdl_param_table_tidy_up();
void xdl_param_table_drawitem();
void xdl_param_table_drawstring();
void xdl_param_table_drawcursor();
int xdl_param_table_lenstr();
void xdl_param_table_smark();
void xdl_param_table_scroll();
int xdl_param_table_getvalue();
int xdl_param_table_error();
int xdl_param_table_clearitems();
int xdl_param_table_setitem();
int xdl_param_table_setvalue();
int xdl_param_table_setmenu();
int xdl_param_table_setbutton();
int xdl_param_table_clrmenu();
int xdl_param_table_silent();
int xdl_param_table_normal();
int xdl_param_table_bold();
int xdl_param_table_all_normal();
int xdl_param_table_delitem();
void xdl_param_table_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_param_table xdlf_param_table_
#define xdlf_param_table_getvalue xdlf_param_table_getvalue_
#define xdlf_param_table_error xdlf_param_table_error_
#define xdlf_param_table_clearitems xdlf_param_table_clearitems_
#define xdlf_param_table_setitem xdlf_param_table_setitem_
#define xdlf_param_table_setvalue xdlf_param_table_setvalue_
#define xdlf_param_table_setmenu xdlf_param_table_setmenu_
#define xdlf_param_table_setbutton xdlf_param_table_setbutton_
#define xdlf_param_table_clrmenu xdlf_param_table_clrmenu_
#define xdlf_param_table_silent xdlf_param_table_silent_
#define xdlf_param_table_normal xdlf_param_table_normal_
#define xdlf_param_table_bold xdlf_param_table_bold_
#define xdlf_param_table_all_normal xdlf_param_table_all_normal_
#define xdlf_param_table_delitem xdlf_param_table_delitem_
#define xdlf_param_table_newtitle xdlf_param_table_newtitle_
#define xdlf_param_table_getsize xdlf_param_table_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_param_table XDLF_PARAM_TABLE
#define xdlf_param_table_getvalue XDLF_PARAM_TABLE_GETVALUE
#define xdlf_param_table_error XDLF_PARAM_TABLE_ERROR
#define xdlf_param_table_clearitems XDLF_PARAM_TABLE_CLEARITEMS
#define xdlf_param_table_setitem XDLF_PARAM_TABLE_SETITEM
#define xdlf_param_table_setvalue XDLF_PARAM_TABLE_SETVALUE
#define xdlf_param_table_setmenu XDLF_PARAM_TABLE_SETMENU
#define xdlf_param_table_setbutton XDLF_PARAM_TABLE_SETBUTTON
#define xdlf_param_table_clrmenu XDLF_PARAM_TABLE_CLRMENU
#define xdlf_param_table_silent XDLF_PARAM_TABLE_SILENT
#define xdlf_param_table_normal XDLF_PARAM_TABLE_NORMAL
#define xdlf_param_table_bold XDLF_PARAM_TABLE_BOLD
#define xdlf_param_table_all_normal XDLF_PARAM_TABLE_ALL_NORMAL
#define xdlf_param_table_delitem XDLF_PARAM_TABLE_DELITEM
#define xdlf_param_table_newtitle XDLF_PARAM_TABLE_NEWTITLE
#define xdlf_param_table_getsize XDLF_PARAM_TABLE_GETSIZE

#endif

void xdlf_param_table();
void xdlf_param_table_getvalue();
void xdlf_param_table_error();
void xdlf_param_table_clearitems();
void xdlf_param_table_setitem();
void xdlf_param_table_setvalue();
void xdlf_param_table_setmenu();
void xdlf_param_table_setbutton();
void xdlf_param_table_clrmenu();
void xdlf_param_table_silent();
void xdlf_param_table_normal();
void xdlf_param_table_bold();
void xdlf_param_table_all_normal();
void xdlf_param_table_delitem();
void xdlf_param_table_newtitle();
void xdlf_param_table_getsize();


/* 
   Format of text storage
   ----------------------
   
   The parameter names and values are stored in a single text array 'text'
   For each parameter there are max_name_chars characters (for the
   parameter name) followed by two sets of max_value_chars (the first
   for the value being input [or the last one input] and the second for 
   the previous value which may need to be restored if the user abandons
   the input with an escape or if the program rejects the newly input
   value.

*/

/*-C:*/
int xdl_param_table (vh, vh_parent, x, y, cset, num_cols, num_rows, 
                     max_name_chars, max_value_disp, max_value_chars, 
                     title, nt, font_type, menus_allowed)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame,
                             if 0 then a base  level frame is created to
                             enclose the i/o window view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - 
                             normally 0 (R)*/
int       num_cols;       /* Maximum number of columns of items in the 
                             parameter table (R)*/
int       num_rows;       /* Maximum number of rows of items in the 
                             parameter table (R)*/
int       max_name_chars; /* Maximum number of characters in a parameter 
                             name string (R)*/
int       max_value_disp; /* Maximum number of characters in a parameter
                             value string which may be displayed (R)*/
int       max_value_chars;/* Maximum number of characters in a parameter 
                             value string which may be stored (R)*/
char      *title;         /* Title string (R)*/
int       nt;             /* Length of title string; if 0 then length 
                             found assuming that 'title' is null 
                             terminated; if -1 then assume that no title 
                             is required (R)*/
int       font_type;      /* Font type (1-5) for very-small, small, medium,
                             large or extra-large font (R)*/
int       menus_allowed;  /* =1 if popup values menus and/or a step-buttons 
                                may be required,
                             =0 if no popup menus or step-buttons are to be 
                                used (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to get memory for global data structure
                bit 2 set; Unable to get memory for 'text' array
                bit 3 set; Unable to get memory for parameter details array
                bit 4 set; Unable to get memory for copy of title
-end*/
{
   Window wframe;           /* Primary window id */
   Window wparent;          /* parent window id */
   Status st;               /* Status return from Xlib calls */
   XDl_param_table * gd;    /* Pointer to view-object's global data structure */
   char * p;                /* Temporary pointer */
   int err;                 /* Error return flag */
   int iv;                  /* Offset into XDL_view_objects array */
   int i;                   /* Loop counter */
   int table_width;         /* Parameter table canvas width */
   int table_height;        /* Parameter table canvas height */
   int ch_width;            /* Character width */
   int ch_height;           /* Character height */
   int ascent;              /* Font ascent */
   int descent;             /* Font descent */
   int col_width;           /* Column width */
   int row_height;          /* Row height */
   int title_height;        /* Title height */
   int nch_max;             /* Max no. of characters to store in text array */
   int ioff;                /* Offset into parameters list */
   int toff;                /* Offset into text array */
   int xoff;                /* x offset */
   int yoff;                /* y offset */
   int icol;                /* Columns loop counter */
   int irow;                /* Rows loop counter */
   XDl_param_table_details *id; /* Temp. pointer to parameter details 
                                   structure */
   XFontStruct *fontinfo;       /* Pointer to normal print font structure */
   XFontStruct *bold_fontinfo;  /* Pointer to bold print font structure */

   err = 0;

   /* Calculate size requirements for the windows */
   /* *WARNING* update xdl_param_table_getsize if this section changed */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (menus_allowed!=1) menus_allowed = 0;
   fontinfo = XDL_fonts[font_type-1];
   bold_fontinfo = XDL_bold_fonts[font_type-1];
   if (nt==0) nt = strlen (title);
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   title_height = 0;
   if (nt>=0) title_height = ch_height + 1;
   col_width = (max_name_chars + max_value_disp)*ch_width 
               + 2*MARG_W + SEP_W + 1;
   if (menus_allowed) col_width += ch_width + SEP_M;
   table_width = num_cols*col_width + 2;
   row_height = ch_height + SEP_H;
   table_height = num_rows*row_height + title_height + 2*MARG_H + 2;

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y,
                    table_width + 2*BORDER_WIDTH + 2,
                    table_height + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2,
                    cset);
   }else{
   /* Create base frame window as no parent frame window specified */

      wframe = xdl_create_base_frame_window ("xdl_param_table", "xdl_param",
                    x, y,
                    table_width + 2*BORDER_WIDTH + 2,
                    table_height + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_param_table));
   if (p==0) {err |= 2; return err;}
   gd = (XDl_param_table *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, p, 1041, xdl_param_table_on_off,
                        xdl_param_table_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->on_off = 0;
   gd->kbd_focus = 0;
   gd->repaint_done = 0;
   gd->highlight_pixel = XDL_cyanpixel;
   gd->font_type = font_type;
   gd->fontinfo = fontinfo;
   gd->bold_fontinfo = bold_fontinfo;
   gd->font_descent = descent;
   gd->font_ascent = ascent;
   gd->max_width = ch_width;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->table_width = table_width;
   gd->table_height = table_height;
   gd->num_cols = num_cols;
   gd->num_rows = num_rows;
   gd->num_params = num_cols*num_rows;
   gd->max_name_chars = max_name_chars;
   gd->max_value_disp = max_value_disp;
   gd->max_value_chars = max_value_chars;
   gd->col_width = col_width;
   gd->row_height = row_height;
   gd->title_height = title_height;
   gd->yo_start = title_height + MARG_H;
   gd->title_len = nt;
   gd->title_xoff = 0;
   if (nt>0) gd->title_xoff = (table_width - nt*ch_width) / 2;
   gd->last_in = -1;
   gd->curs_offset = -1;
   gd->highlight_item = 0;
   gd->highlight_item_but = 0;
   gd->input_item = 0;
   gd->last_input_item =0;
   gd->step_button = 0;
   gd->sav_sc1 = 0;
   gd->sav_sc2 = 0;
   gd->sav_minsc = 0;
   gd->sav_nsc = 0;
   gd->scroll_item = 0;
   gd->menus_allowed = menus_allowed;
   gd->ind_w = ch_width;
   gd->ind_h = ch_height - 2;
   gd->ind_x1 = ch_width/3 - 1;
   gd->ind_y1 = ch_height/3 - 1;
   gd->ind_x2 = ch_width/2;
   gd->ind_y2 = ch_height - ch_height/3;
   gd->ind_x3 = 2*(gd->ind_x2) - gd->ind_x1;
   gd->ind_y3 = gd->ind_y1;

   nch_max = gd->num_params * (max_name_chars + 2*max_value_chars);
   p = (char *) malloc (nch_max);
   if (p==0) {err |= 4; return err;}
   gd->text = p;
   for (i=0; i<nch_max; i++) gd->text[i] = ' ';
   p = (char *)malloc (gd->num_params*sizeof(XDl_param_table_details));
   if (p==0) {err |= 8; return err;}
   gd->item_details = (XDl_param_table_details *) p;
   gd->title = NULL;
   if (nt>0) 
   {
      p = (char *) malloc (nt+1);
      if (p==0) {err |= 16; return err;};
      gd->title = p;
      strncpy (gd->title, title, nt);
      gd->title[nt] = 0;
   }

   /* Initialise parameter presets/status structures */

   ioff = 0;
   toff = 0;
   for (icol=0;icol<num_cols;++icol)
   {
      for (irow=0;irow<num_rows;++irow)
      {
         id = &(gd->item_details[ioff]);
         id->status = 0;
         id->emphasize = 0;
         id->toggle = 0;
         id->menu = 0;
         id->toff_name = toff;
         toff += max_name_chars;
         id->toff_val = toff;
         toff += max_value_chars;
         id->toff_oldval = toff;
         toff += max_value_chars;
         xoff = icol*(gd->col_width);
         yoff = gd->yo_start + irow*(gd->row_height);
         id->xoff = xoff;
         id->yoff = yoff;
         id->xoff_name = xoff + MARG_W;
         id->yoff_name = yoff + SEP_H/2;
         id->xoff_val = xoff + MARG_W + (max_name_chars)*(gd->ch_width) + SEP_W;
         id->yoff_val = id->yoff_name;
         id->xoff_ind = id->xoff_val + (max_value_disp)*(gd->ch_width) + SEP_M;
         id->yoff_ind = id->yoff_val + 1;
         id->xoff_menu = id->xoff_ind + ch_width + 3;
         id->yoff_menu = id->yoff_ind;
         ioff ++;
      }
   }

   /* Create active strip window */

   gd->wact = xdl_create_active_strip (gd->wframe, vh, table_width, 
                                    xdl_param_table_rps);

   /* Create param table paint window */

   gd->wparam = xdl_create_paint_window (wframe, BORDER_WIDTH,
                           BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1,
                           table_width, table_height, cset);
   xdl_add_window (vh, gd->wparam, xdl_param_table_rp1, xdl_param_table_ep1);

   /* Prepare for event handling */
  
   XSelectInput (XDL_display, gd->wparam, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|KeyPressMask|ButtonReleaseMask|
                 FocusChangeMask|LeaveWindowMask|PointerMotionMask);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wparam, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, fontinfo->fid);

   return 0;
}

/* Fortran binding: xdlf_param_table */

void xdlf_param_table (ivh, ivhparent, ix, iy, icset, ncols, nrows,
                       lennam, lenval, maxlen, title, itlen, ifont, 
                       imenu, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *ncols, *nrows;
int *lennam, *lenval, *maxlen, *title, *itlen, *ifont, *imenu, *ierr;
{
   *ierr = xdl_param_table (*ivh, *ivhparent, *ix, *iy, *icset, *ncols, *nrows,
                            *lennam, *lenval, *maxlen, 
                            XDL_pointers[*title], *itlen, 
                            *ifont, *imenu);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
**********************************
**     xdl_param_table_rps      **
**********************************

Purpose: Repaint active strip area

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_param_table * gd;     /* Pointer to global data area */

   gd = (XDl_param_table *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, "Edits allowed", gd->kbd_focus);
   }
}
/*
**********************************
**     xdl_param_table_rp1      **
**********************************

Purpose: Repaint parameter table area

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_param_table * gd;            /* Pointer to global data area */
   XDl_param_table_details *id;     /* pointer to item-details structure */
   int hl;                          /* Highlight flag 1=yes, 0=no */
   int i;                           /* Loop counter */
   int xx;                          /* Cursor position (x) */
   int yy;                          /* Cursor position (y) */

   gd = (XDl_param_table *) global;
   gd->repaint_done = 1;
   
   /* Draw title if present */

   if (gd->title_height>0)
   {
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 0, (gd->title_height-1), 
                 gd->table_width, (gd->title_height-1));
      xdl_param_table_drawstring (gd, gd->title_xoff, 0, gd->title,
                                  gd->title_len, 1);
   }

   /* Draw column separating lines */

   if (gd->num_cols>1)
   {
      for (i=1;i<gd->num_cols;++i)
      {
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 i*(gd->col_width)-1, gd->title_height, 
                 i*(gd->col_width)-1, gd->table_height);
      }
   }

   /* Output parameter names and values */

   for (i=1;i<=gd->num_params;++i)
   {
      hl = 0;
      if (i==gd->highlight_item) hl = 1;
      xdl_param_table_drawitem (gd, i, hl, 0);
   }

   /* Output cursor if required */

   if (gd->input_item>0)
   {
      id = &(gd->item_details[gd->input_item-1]);
      xx = id->xoff_val + (gd->curs_offset)*(gd->ch_width);
      yy = id->yoff_val;
      xdl_param_table_drawcursor (gd, xx, yy);
   }

   return;
}

/*
**********************************
**     xdl_param_table_ep1      **
**********************************

Purpose: Event handling routine  for the parameter table area

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();
{
   XDl_param_table *gd;            /* Pointer to the global data area */
   XDl_param_table_details *id;    /* Pointer to an item details structure */
   int x, y;                       /* Mouse position */
   int xx, yy;                     /* Character position */
   int xroot, yroot;               /* Root window position for popup menu */
   int col_offset;                 /* Parameter table column offset from
                                      mouse position */
   int row_offset;                 /* parameter table row offset from
                                      mouse position */
   int i;                          /* Loop counter */
   int item;                       /* Parameter item number */
   static char *toggle_str[4] = {"On ","Off","Yes","No "};  /* Toggle strings */
   KeySym keysym;                  /* Keysym structure for keyboard input */
   XComposeStatus compose;         /* Compose status */
   char chbuf[2];                  /* Buffer for ascii translated characters */
   int nc;                         /* No. of characters decoded from key 
                                      press */
   int kitem;                      /* Selected popup menu item no. */
   int kk, ll;                     /* Temp variables */
   int hl_button;                  /* Highlight button flag =1 or 2 button
                                      number, = 0 no, =-1 no button present */
   int hl;                         /* Temp highlight item flag */
   char **strs;                    /* Temp pointer to array of strings */
   XEvent xevent;                  /* Temp XEvent */
   float ytemp;                    /* Temp y value for step-button calcn. */
   float xq;                       /* x offset of pointer from button origin */
   float yq;                       /* y offset of pointer from button origin */
   char *str;                      /* String pointer for selection receive */
   int nitems;                     /* No. of items (characters) for selection
                                      receive */

   gd = (XDl_param_table *) global;

   if (gd->on_off==0) return;

   /* Check for keyboard focus lost */

   if (event->type==FocusOut)
   {
       if (gd->kbd_focus==0) return;
       xdl_active_strip_on (gd->wact, "Edits allowed", 0);
       gd->kbd_focus = 0;
       return;
   }

   /* Check for keyboard focus gained */

   if (event->type==FocusIn)
   {
       if (gd->kbd_focus==1) return;
       xdl_active_strip_on (gd->wact, "Edits allowed", 1);
       gd->kbd_focus = 1;
       return;
   }

   /* Check for mouse select to get keyboard focus and select input item */

   if (event->type==ButtonPress&&event->xbutton.button==Button1)
   {
      XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
      if (gd->kbd_focus==0)
      {
         xdl_active_strip_on (gd->wact, "Edits allowed", 1);
         gd->kbd_focus = 1;
      }

      if (gd->input_item>0) return;

      x = event->xbutton.x;
      y = event->xbutton.y;      
      col_offset = x / gd->col_width;
      if (col_offset>=gd->num_cols) col_offset = gd->num_cols - 1;
      if (y<gd->yo_start) return;
      row_offset = (y - gd->yo_start) / gd->row_height;
      if (row_offset>=gd->num_rows) return;
      item = col_offset*(gd->num_rows) + row_offset + 1;
      id = &(gd->item_details[item-1]);
      if (id->status==0) return;
      if (id->emphasize<0) return;
      if (id->toggle<0) return;
      gd->step_button = 0;

      /* Service step-button if required */
      if (id->menu==2&&id->toggle==0)
      {
         if (x>=id->xoff_ind&&x<=id->xoff_ind+gd->ind_w
            &&y>=id->yoff_ind&&y<=id->yoff_ind+gd->ind_h)
         {
            gd->step_button = 1;
            xq = x - id->xoff_ind;
            yq = y - id->yoff_ind;
            ytemp = xq * (float) (gd->ind_h+1) / (float) (gd->ind_w+1);
            if (yq>ytemp) gd->step_button = 2;
            gd->last_input_item = item;
            gd->input_item = 0;
            XDL_return_data_vh = XDL_view_objects[iv]->vh;
            return;
         }
      }

      gd->last_input_item = 0;
      for (i=0;i<gd->max_value_chars;++i)
      {
         gd->text[id->toff_oldval+i] = gd->text[id->toff_val+i];
         gd->text[id->toff_val+i] = ' ';
      }
      gd->sav_sc1 = id->sc1;
      gd->sav_sc2 = id->sc2;
      gd->sav_minsc = id->minsc;
      gd->sav_nsc = id->nsc;
      id->sc1 = 0;
      id->sc2 = 0;
      id->minsc = 0;
      id->nsc = 0;
      if (id->toggle>0)
      {
         if (id->toggle<=2)
         {
            id->toggle = 3 - id->toggle;
         }
         else
         {
            id->toggle = 7 - id->toggle;
         }
         xdl_param_table_drawitem (gd, item, 0, 1);
         gd->last_input_item = item;
         gd->input_item = 0;
         gd->step_button = 0;
         strncpy (&(gd->text[id->toff_val]), toggle_str[id->toggle-1], 3);
         XDL_return_data_vh = XDL_view_objects[iv]->vh;
         return;
      }
      else
      {
         gd->input_item = item;
         gd->highlight_item = 0;
         gd->highlight_item_but = 0;
         gd->last_in = -1;
         gd->curs_offset = 0;
         xdl_param_table_drawitem (gd, item, 0, 1);
         return;
      }
   }

   /* Mouse Button2 for scrolling value (or pasting)*/

   if (event->type==ButtonPress&&event->xbutton.button==Button2)
   {
      if (gd->input_item==0)
      {
         /* Scroll request - item not being input */
         x = event->xbutton.x;
         y = event->xbutton.y;
         col_offset = x / gd->col_width;
         if (col_offset>=gd->num_cols) col_offset = gd->num_cols - 1;
         if (y<gd->yo_start) return;
         row_offset = (y - gd->yo_start) / gd->row_height;
         if (row_offset>=gd->num_rows) return;
         item = col_offset*(gd->num_rows) + row_offset + 1;
         gd->scroll_item = item;
         xdl_param_table_scroll (gd, x, y);
         return;
      }
      else
      {
         /* Paste request - item being input */
         xdl_select_request (event, gd->wparam);
         XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
         if (gd->kbd_focus==0)
         {
            xdl_active_strip_on (gd->wact, "Edits allowed", 1);
            gd->kbd_focus = 1;
         }
         return;
      }
   }

   if (event->type==ButtonRelease&&event->xbutton.button==Button2)
   {
      if (gd->scroll_item>0)
      {
         hl = 0;
         if (gd->scroll_item==gd->highlight_item) hl = 1;
         xdl_param_table_drawitem (gd, gd->scroll_item, hl, 0);
         gd->scroll_item = 0;
      }
      return;
   }

   /* Mouse Button3 for Popup Values Menu */

   if (event->type==ButtonPress&&event->xbutton.button==Button3)
   {
      if (gd->input_item>0) return;
      x = event->xbutton.x;
      y = event->xbutton.y;
      col_offset = x / gd->col_width;
      if (col_offset>=gd->num_cols) col_offset = gd->num_cols - 1;
      if (y<gd->yo_start) return;
      row_offset = (y - gd->yo_start) / gd->row_height;
      if (row_offset>=gd->num_rows) return;
      item = col_offset*(gd->num_rows) + row_offset + 1;
      id = &(gd->item_details[item-1]);
      if (id->menu==0) return;
      if (id->status==0) return;
      if (id->emphasize<0) return;
      xroot = event->xbutton.x_root;
      yroot = event->xbutton.y_root;
      xroot = id->xoff_menu + xroot - x;
      yroot = id->yoff_menu + yroot - y;
      /* Create and service Pop-up menu */
      kitem = xdl_popup_menu (xroot, yroot, id->num_menu_vals, id->menu_vals,
                      id->len_menu_vals, gd->font_type, 0, 0);
      if (kitem==0) return;
      for (i=0;i<gd->max_value_chars;++i)
      {
         gd->text[id->toff_oldval+i] = gd->text[id->toff_val+i];
         gd->text[id->toff_val+i] = ' ';
      }
      gd->last_input_item = item;
      gd->input_item = 0;
      gd->step_button = 0;
      if (id->len_menu_vals>0)
      {
         ll = id->len_menu_vals;
         if (ll>gd->max_value_chars) ll = gd->max_value_chars;
         kk = (kitem-1)*(id->len_menu_vals);
         strncpy (&(gd->text[id->toff_val]),&(id->menu_vals[kk]),ll);
      }
      else
      {
         strs = (char**) id->menu_vals;
         ll = strlen (strs[kitem-1]);
         if (ll>gd->max_value_chars) ll = gd->max_value_chars;
         strncpy (&(gd->text[id->toff_val]),strs[kitem-1],ll);
      }
      XDL_return_data_vh = XDL_view_objects[iv]->vh;
      return;
   }

   /* Mouse movement */

   if (event->type==MotionNotify)
   {
      if (gd->input_item>0) return;

      x = event->xmotion.x;
      y = event->xmotion.y;
      while (XCheckTypedWindowEvent (XDL_display, gd->wparam,
             MotionNotify, &xevent))
      {
         x = xevent.xmotion.x;
         y = xevent.xmotion.y;
      }
      if (gd->scroll_item>0)
      /* Value scrolling */
      {
         xdl_param_table_scroll (gd, x, y);
      }
      else
      /* Item Highlighting */
      {
         col_offset = x / gd->col_width;
         if (col_offset>=gd->num_cols) col_offset = gd->num_cols - 1;
         row_offset = (y - gd->yo_start) / gd->row_height;
         item = col_offset*(gd->num_rows) + row_offset + 1;
         if ((y<gd->yo_start)||(row_offset>=gd->num_rows)) item = -1;

         /* See if button highlight required */
         hl_button = -1;
         if (item>0)
         {
            id = &(gd->item_details[item-1]);
            if (id->menu==2&&id->toggle==0)
            {
               hl_button = 0;
               if (x>=id->xoff_ind&&x<=id->xoff_ind+gd->ind_w
                  &&y>=id->yoff_ind&&y<=id->yoff_ind+gd->ind_h)
               {
                  hl_button = 1;
                  xq = x - id->xoff_ind;
                  yq = y - id->yoff_ind;
                  ytemp = xq * (float) (gd->ind_h+1) / (float) (gd->ind_w+1);
                  if (yq>ytemp) hl_button = 2;
               }
            }
         }
         if (item==gd->highlight_item) 
         {
            if (hl_button==-1) return;
            if (hl_button==gd->highlight_item_but) return;
            gd->highlight_item_but = hl_button;
            xdl_param_table_drawitem (gd, item, 1, 0);
            return;
         }
         gd->highlight_item_but = 0;
         if (gd->highlight_item>0) xdl_param_table_drawitem (gd, 
                                           gd->highlight_item, 0, 0);
         gd->highlight_item = 0;
         if (item<0) return;
         id = &(gd->item_details[item-1]);
         if (id->status==0) return;
         if (id->emphasize<0) return;
         if (id->toggle<0) return;
         if (hl_button>0) gd->highlight_item_but = hl_button;
         gd->highlight_item = item;
         xdl_param_table_drawitem (gd, item, 1, 0);
      }
      return; 
   }

   /* Check for mouse exit from the window */

   if (event->type==LeaveNotify)
   {
      if (gd->highlight_item==0) return;
      xdl_param_table_drawitem (gd, gd->highlight_item, 0, 0);
      gd->highlight_item = 0;
      return;
   }

   /* Process keyboard input */

   if (gd->input_item==0) return;
   if (event->type==KeyPress)
   {
      nc = XLookupString ((XKeyEvent *) event, chbuf, 2, &keysym, NULL);
      id = &(gd->item_details[gd->input_item-1]);

      if (keysym==XK_BackSpace||keysym==XK_Delete)
      {
         if (gd->last_in>=0)
         {
            gd->text[id->toff_val+gd->last_in] = ' ';
            gd->last_in--;
            xdl_param_table_drawitem (gd, gd->input_item, 0, 1);
         }
         return;
      }
      else if (keysym==XK_Escape)
      {
         /* Escape pressed */
         for (i=0;i<gd->max_value_chars;++i)
         {
            gd->text[id->toff_val+i] = gd->text[id->toff_oldval+i];
         }
         id->sc1 = gd->sav_sc1;
         id->sc2 = gd->sav_sc2;
         id->minsc = gd->sav_minsc;
         id->nsc = gd->sav_nsc;
         xdl_param_table_drawitem (gd, gd->input_item, 0, 0);
         gd->input_item = 0;
         return;
      }
      else if (keysym==XK_Return||keysym==XK_KP_Enter||keysym==XK_Linefeed)
      {
         /* Carriage return/line feed to enter new value */
         if (id->minsc>0)
         {
            ll = xdl_param_table_lenstr (&gd->text[id->toff_val],
                                         gd->max_value_chars);
            if (ll<=gd->max_value_disp)
            {
               id->sc1 = 0;
               id->sc2 = 0;
               id->minsc = 0;
               id->nsc = ll;
            }
            else
            {
               id->sc1 = 1;
               id->sc2 = 0;
               id->minsc = ll - gd->max_value_disp + 1;
               id->nsc = gd->max_value_disp - 1;
            }
         }
         xdl_param_table_drawitem (gd, gd->input_item, 0, 0);
         gd->last_input_item = gd->input_item;
         gd->input_item = 0;
         gd->step_button = 0;
         gd->curs_offset = -1;
         gd->last_in = -1;
         XDL_return_data_vh = XDL_view_objects[iv]->vh;
         return;
      }
      else if (keysym>=XK_space&&keysym<=XK_asciitilde)
      {
         /* Text character input */
         if (gd->last_in<(gd->max_value_chars-1))
         {
            gd->last_in += 1;
            gd->text[id->toff_val+gd->last_in] = chbuf[0];
            xx = id->xoff_val + (gd->last_in)*(gd->ch_width);
            yy = id->yoff_val;
            if (gd->last_in<(gd->max_value_disp-1))
            {
               XClearArea (XDL_display, gd->wparam, xx, yy,
                           gd->ch_width, gd->ch_height, False);
               xdl_param_table_drawstring (gd, xx, yy, chbuf, 1, 0);
               if (gd->curs_offset<(gd->max_value_chars-1))
               {
                  gd->curs_offset++;
               }
               xx = id->xoff_val + (gd->curs_offset)*(gd->ch_width);
               xdl_param_table_drawcursor (gd, xx, yy);
            }
            else
            {
               xdl_param_table_drawitem (gd, gd->input_item, 0, 1);
            }
         }
      }
      return;
   }
   /* Check for paste */
   if (event->type==SelectionNotify)
   {
      if (xdl_select_receive (event, gd->wparam, &str, &nitems, 
          gd->max_value_chars+4))
      {
         if (nitems>0)
         {
            id = &(gd->item_details[gd->input_item-1]);
            for(i=0;i<nitems;++i)
            {
               if (str[i]=='\n') break;
               if (gd->last_in<(gd->max_value_chars-1))
               {
                  gd->last_in += 1;
                  gd->text[id->toff_val+gd->last_in] = str[i];
                  xx = id->xoff_val + (gd->last_in)*(gd->ch_width);
                  yy = id->yoff_val;
                  if (gd->last_in<(gd->max_value_disp-1))
                  {
                     XClearArea (XDL_display, gd->wparam, xx, yy,
                                 gd->ch_width, gd->ch_height, False);
                     xdl_param_table_drawstring (gd, xx, yy, &str[i], 1, 0);
                     if (gd->curs_offset<(gd->max_value_chars-1))
                     {
                        gd->curs_offset++;
                     }
                     xx = id->xoff_val + (gd->curs_offset)*(gd->ch_width);
                     xdl_param_table_drawcursor (gd, xx, yy);
                  }
                  else
                  {
                     xdl_param_table_drawitem (gd, gd->input_item, 0, 1);
                  }
               }
            }
         }
         if (str!=0) free(str);
      }
      return;
   }
   return;
}

/*
*****************************************
**     xdl_param_table_drawcursor      **
*****************************************

Purpose: Draw text input cursor

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_drawcursor (gd, x, y)

XDl_param_table *gd;       /* Pointer to the global data area */
int x;                     /* x position for the cursor */
int y;                     /* y position for the cursor */

{
   y += gd->ch_height;
   XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
   x+1, y-3, x+1, y-1);
   XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
   x+1, y-1, x+gd->ch_width-2, y-1);
   XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
   x+gd->ch_width-2, y-3, x+gd->ch_width-2, y-1);
   return;
}
/*
*************************************
**     xdl_param_table_on_off      **
*************************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_on_off (iv, i)

int iv;          /* Offset in XDL_view_objects array */
int i;           /* Flag =1 on, =0 off */

{
   XDl_param_table *gd;          /* Pointer to global data */
   XDl_param_table_details *id;  /* Pointer to item details structure */

   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      gd->last_in = -1;
      xdl_active_strip_on (gd->wact, "Edits allowed", 1);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
      if (gd->highlight_item>0)
      {
         xdl_param_table_drawitem (gd, gd->highlight_item, 0, 0);
         gd->highlight_item = 0;
      }
      if (gd->input_item>0)
      {
         id = &(gd->item_details[gd->input_item-1]);
         for (i=0;i<gd->max_value_chars;++i)
         {
             gd->text[id->toff_val+i] = gd->text[id->toff_oldval+i];
         }
         id->sc1 = gd->sav_sc1;
         id->sc2 = gd->sav_sc2;
         id->minsc = gd->sav_minsc;
         id->nsc = gd->sav_nsc;
         xdl_param_table_drawitem (gd, gd->input_item, 0, 0);
         gd->input_item = 0;
         gd->last_in = -1;
         gd->curs_offset = -1;
      }
   }
   return;
}

/*
**************************************
**     xdl_param_table_tidy_up      **
**************************************

Purpose: Routine called to tidy up before view object is deleted

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_tidy_up (global)

char * global;   /* Pointer to global data structure */

{
   XDl_param_table *gd;    /* Pointer to global data area */

   gd = (XDl_param_table *) global;
   free (gd->text);
   free ((char *) gd->item_details);
   free (gd->title);
   XFreeGC (XDL_display, gd->paint_gc);
   return;
}

/*
***************************************
**     xdl_param_table_drawitem      **
***************************************

Purpose: Draw a parameter table item

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_drawitem (gd, item, hl, inp)

XDl_param_table *gd;     /* Pointer to the global data area */
int item;                /* The item number */
int hl;                  /* Highlight flag =1 highlight, =0 do not */
int inp;                 /* Parameter value being input flag; true = yes,
                            false = no */

{
   XDl_param_table_details *id;   /* Pointer to item details structure */
   int bold;                      /* Bold print flag 1=yes, 0=no */
   static char *toggle_str[4] = {"On ","Off","Yes","No "};  /* Toggle strings */
   XPoint xy[3];                  /* Points for indicator triangle */
   int ll;                        /* Significant length of value string */
   int xx;                        /* Start 'x' position for string/cursor */

   id = &(gd->item_details[item-1]);
   XClearArea (XDL_display, gd->wparam, id->xoff, id->yoff,
               (gd->col_width-1), gd->row_height, False);

   if (id->status==0) return;
   if (id->emphasize<0) return;

   bold = id->emphasize;

   if (hl)
   {
      if (XDL_color > 1)
      {
         XSetForeground (XDL_display, gd->paint_gc, gd->highlight_pixel);
         XFillRectangle (XDL_display, gd->wparam, gd->paint_gc,
                         id->xoff, id->yoff,
                         (gd->col_width-1), gd->row_height);
      }
      bold = 1;
   }

   xdl_param_table_drawstring (gd, id->xoff_name, id->yoff_name,
                               &(gd->text[id->toff_name]), 
                               gd->max_name_chars, bold);
   if (id->toggle>0)
   {
      xdl_param_table_drawstring (gd, id->xoff_val+gd->ch_width/2,
                                  id->yoff_val, toggle_str[id->toggle-1],
                                  3, bold);
      XDrawRectangle (XDL_display, gd->wparam, gd->paint_gc,
                 id->xoff_val, id->yoff_val,
                 4*(gd->ch_width)-1, gd->row_height-1);
   }

   /* Output value string; allow for scrolling; allow for input mode with
      cursor*/

   if (id->toggle==0)
   {
      if (inp)
      {
         ll = gd->last_in + 1;   /*Some trailing spaces may be significant*/
         if (ll<gd->max_value_disp)
         {
            xdl_param_table_drawstring (gd, id->xoff_val, id->yoff_val,
                                        &(gd->text[id->toff_val]), 
                                        ll, bold);
            xx = id->xoff_val + ll*(gd->ch_width);
            xdl_param_table_drawcursor (gd, xx, id->yoff_val);
            gd->curs_offset = ll;
            id->sc1 = 0;
            id->sc2 = 0;
            id->minsc = 0;
            id->nsc = ll;
         }
         else
         {
            id->sc1 = 1;
            id->sc2 = 0;
            id->minsc = ll - gd->max_value_disp + 2;
            id->nsc = gd->max_value_disp - 2;
            xx = id->xoff_val;
            xdl_param_table_smark (gd, xx, id->yoff_val, 0);
            xx += gd->ch_width;
            xdl_param_table_drawstring (gd, xx, id->yoff_val,
                                        &(gd->text[id->toff_val+id->minsc]), 
                                        id->nsc, bold);
            xx = id->xoff_val + (gd->max_value_disp-1)*(gd->ch_width);
            xdl_param_table_drawcursor (gd, xx, id->yoff_val);
            gd->curs_offset = id->nsc + 1;
         }
      }
      else
      {
         ll = xdl_param_table_lenstr (&(gd->text[id->toff_val]), 
                                      gd->max_value_chars);
         if (ll<=gd->max_value_disp)
         {
            xdl_param_table_drawstring (gd, id->xoff_val, id->yoff_val,
                                        &(gd->text[id->toff_val]), 
                                        gd->max_value_disp, bold);
            id->sc1 = 0;
            id->sc2 = 0;
            id->minsc = 0;
            id->nsc = ll;
         }
         else
         {
            xx = id->xoff_val;
            if (id->sc1>0)
            {
               xdl_param_table_smark (gd, xx, id->yoff_val, 0);
               xx = xx + gd->ch_width;
            }
            xdl_param_table_drawstring (gd, xx, id->yoff_val,
                                        &(gd->text[id->toff_val+id->minsc]), 
                                        id->nsc, bold);
            if (id->sc2>0)
            {

               xx = id->xoff_val + (gd->max_value_disp-1)*(gd->ch_width);
               xdl_param_table_smark (gd, xx, id->yoff_val, 1);
            }
         }
      }
   }

   /* Draw menu indicator if required */

   if (id->menu==1)
   {
      XDrawRectangle (XDL_display, gd->wparam, gd->paint_gc,
                      id->xoff_ind, id->yoff_ind, gd->ind_w, gd->ind_h);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 id->xoff_ind + gd->ind_x1, id->yoff_ind + gd->ind_y1,
                 id->xoff_ind + gd->ind_x2, id->yoff_ind + gd->ind_y2);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 id->xoff_ind + gd->ind_x2, id->yoff_ind + gd->ind_y2,
                 id->xoff_ind + gd->ind_x3, id->yoff_ind + gd->ind_y3);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 id->xoff_ind + gd->ind_x3, id->yoff_ind + gd->ind_y3,
                 id->xoff_ind + gd->ind_x1, id->yoff_ind + gd->ind_y1);
   }

   /* Draw step-button if required */

   if (id->menu==2&&id->toggle>=0)
   {
      XDrawRectangle (XDL_display, gd->wparam, gd->paint_gc,
                      id->xoff_ind, id->yoff_ind, gd->ind_w, gd->ind_h);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
                 id->xoff_ind, id->yoff_ind,
                 id->xoff_ind + gd->ind_w, id->yoff_ind + gd->ind_h);
      if (item==gd->highlight_item&&gd->highlight_item_but>0)
      {
         if (gd->highlight_item_but==1)
         {
            if (XDL_color>0) XSetForeground (XDL_display, gd->paint_gc, 
                             XDL_redpixel);
            xy[0].x = id->xoff_ind;
            xy[0].y = id->yoff_ind;
            xy[1].x = id->xoff_ind + gd->ind_w;
            xy[1].y = id->yoff_ind;
            xy[2].x = id->xoff_ind + gd->ind_w;
            xy[2].y = id->yoff_ind + gd->ind_h;
         }
         else
         {
            if (XDL_color>0) XSetForeground (XDL_display, gd->paint_gc, 
                             XDL_bluepixel);
            xy[0].x = id->xoff_ind;
            xy[0].y = id->yoff_ind;
            xy[1].x = id->xoff_ind;
            xy[1].y = id->yoff_ind + gd->ind_h;
            xy[2].x = id->xoff_ind + gd->ind_w;
            xy[2].y = id->yoff_ind + gd->ind_h;
         }
 
         XFillPolygon (XDL_display, gd->wparam, gd->paint_gc,
                    xy, 3, Convex, CoordModeOrigin);
      }
   }
   return;
}

/*
*****************************************
**     xdl_param_table_drawstring      **
*****************************************

Purpose: Output a text string to the parameter table area

Author:  John W. Campbell, July 1990

*/

void xdl_param_table_drawstring (gd, x, y, string, len, bold)

XDl_param_table *gd;       /* Pointer to global data area */
int x;                     /* x position for start of text string */
int y;                     /* y position for start of text string */
char * string;             /* The string to be output */
int len;                   /* The number of characters to be output */
int bold;                  /* Print type =1 bold, =0 normal */

{
   int i;         /* Loop counter */
   int xx;        /* Character position x */
   int yy;        /* Character position y */

   if (len<=0) return;

   xx = x;
   yy = y +gd->font_ascent + 1;
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   if (bold)
   {
      XSetFont (XDL_display, gd->paint_gc, gd->bold_fontinfo->fid);
   }
   else
   {
      XSetFont (XDL_display, gd->paint_gc, gd->fontinfo->fid);
   }

   for (i=0;i<len;++i)
   {
      if (string[i]!=' ') XDrawString (XDL_display, gd->wparam,
                                       gd->paint_gc, xx, yy, &string[i], 1);
      xx += gd->ch_width;
   }
}
/*
*****************************************
**     xdl_param_table_lenstr          **
*****************************************

Purpose: Find length of a character string excluding trailing blanks

Author:  John W. Campbell, July 1993

*/

int xdl_param_table_lenstr (string, len)

char * string;             /* The string to be examined */
int len;                   /* Length of string to be examined */

{
   int i;         /* Loop counter */
 
   if (len<=0) return 0;
   for (i=len-1;i>=0;--i)
   {
      if (string[i]!=' ') return i+1;
   }
   return 0;
}
/*
************************************
**     xdl_param_table_smark      **
************************************

Purpose: Output a scrolling indicator

Author:  John W. Campbell, July 1993

*/

void xdl_param_table_smark (gd, x, y, ityp)

XDl_param_table *gd;       /* Pointer to global data area */
int x;                     /* x position for start of mark */
int y;                     /* y position for start of mark */
int ityp;                  /* =0, left; =1 right hand scroll mark */

{
   int yy;      /*y position for scrolling mark */

   yy = y + gd->ch_height;
   if (ityp==0)
   {
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+3, yy-1, x+gd->ch_width-2, yy-1);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+1, yy-3, x+3, yy-5);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+1, yy-3, x+3, yy-1);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+3, yy-5, x+gd->ch_width-2, yy-5);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+gd->ch_width-2, yy-5, x+gd->ch_width-2, yy-1);
   }
   else
   {
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+1, yy-1, x+gd->ch_width-4, yy-1);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+gd->ch_width-4, yy-1, x+gd->ch_width-2, yy-3);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+gd->ch_width-4, yy-5, x+gd->ch_width-2, yy-3);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+1, yy-5, x+gd->ch_width-4, yy-5);
      XDrawLine (XDL_display, gd->wparam, gd->paint_gc,
      x+1, yy-1, x+1, yy-5);
   }
   return;
}      
/*
***********************************
**     xdl_param_table_scroll    **
***********************************

Purpose: Service parameter value scrolling request

Author:  John W. Campbell, July 1993

*/

void xdl_param_table_scroll (gd, x, y)

XDl_param_table *gd;       /* Pointer to global data area */
int x;                     /* x position of pointer */
int y;                     /* y position of pointer */
{
   XDl_param_table_details *id;   /* Pointer to item details structure */
   int xmin;                      /* Min x pixel in value field */
   int xmax;                      /* Max x pixel in value field */
   float fr;                      /* Fraction along value field for scroll */
   int ll;                        /* Significant length of value string */
   int nch;                       /* Number of characters from selected
                                     position to end of string */

   id = &(gd->item_details[gd->scroll_item-1]);
   if (id->toggle!=0) return;
   ll = xdl_param_table_lenstr (&gd->text[id->toff_val], gd->max_value_chars);
   if (ll<=gd->max_value_disp) return;
   xmin = id->xoff_val;
   xmax = xmin + gd->max_value_disp*(gd->ch_width) - 1;
   if (x<=xmin)
   {
      id->sc1 = 0;
      id->sc2 = 1;
      id->minsc = 0;
      id->nsc = gd->max_value_disp - 1;
   } 
   else if (x>=xmax)
   {
      id->sc1 = 1;
      id->sc2 = 0;
      id->minsc = ll - gd->max_value_disp + 1;
      id->nsc = gd->max_value_disp - 1;
   }
   else
   {
      fr =(float) (x-xmin)/ (float) (xmax-xmin);
      id->minsc = fr*(ll-gd->max_value_disp+1);
      if (id->minsc==0)
      {
         id->sc1 = 0;
         id->sc2 = 1;
         id->nsc = gd->max_value_disp - 1;
      }
      else if (id->minsc>=(ll-gd->max_value_disp+1))
      {
         id->sc1 = 1;
         id->sc2 = 0;
         id->minsc = ll - gd->max_value_disp + 1;
         id->nsc = gd->max_value_disp - 1;
      }
      else
      {
         id->sc1 = 1;
         nch = ll - id->minsc;
         if (nch>gd->max_value_disp-1)
         {
            id->sc2 = 1;
            id->nsc = gd->max_value_disp - 2;
         }
         else
         {
            id->sc2 = 0;
            id->nsc = nch;
         }
      }
   }
   xdl_param_table_drawitem (gd, gd->scroll_item, 0, 0);
   return;
}
    
/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Get an input value string - xdl_param_table_getvalue
The routine xdl_param_table_getvalue (xdlf_param_table_getvalue) is used
to return the new value string after a parameter value has been edited.
It will normally be called after the return from a call to xdl_get_events
(xdlf_get_events) indicates that a parameter table view-object has
returned input. (If a step-button was selected, then the step-button string
will be returned)
-end*/
/*
************************************
**     xdl_param_table_getvalue   **
************************************

Purpose: Get returned input value string from the parameter table

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_GETVALUE (IVH, ITEM, XDLSTR(STRING), 
        +                                MAXLEN, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (W)   Returned item number (see item)
STRING  (W)   Returned value character string (see string)
              ** Pass address using the XDLSTR function **
MAXLEN  (R)   Maximum allowed length for returned string ( > 0)
              (cf max_len)
IERR    (W)   Returned status from xdl_param_table_getvalue call
-end*/

/*-C:*/
int xdl_param_table_getvalue (vh, item, string, max_len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *item;       /* Returns the parameter item number associated with
                    the input value string, 0 if none (W)*/
char string[];   /* Returned string (W)*/
int max_len;     /* If >0 Maximum length of returned string allowed 
                          (excluding terminating null - string must be 
                          at least max_len + 1 characters in length).
                    if <0 abs(max_len) characters will be returned padded
                          with blanks if needed (for 'fortran' use) (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                       /* Offset in the XDL_view_objects array */
   int i;                        /* Loop counter */
   int len;                      /* String length */
   int kk;                       /* Offset for step-button string */
   XDl_param_table *gd;          /* Pointer to global data area */
   XDl_param_table_details *id;  /* Pointer to item details structure */
   char **strs;                  /* Temp pointer to array of menu value
                                    (button value) strings */
   char *bstr;                   /* Temp pointer to button string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_getvalue",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_getvalue", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   *item = gd->last_input_item;
   if (*item==0) return 0;
   id = &(gd->item_details[*item-1]);
   if (max_len==0) return 0;

   /* Step-button selection */

   if (gd->step_button>0)
   {
      if (id->len_menu_vals>0)
      {
         len = id->len_menu_vals;
         kk = (gd->step_button-1)*(id->len_menu_vals);
         bstr = &(id->menu_vals[kk]);
      }
      else
      {
         strs = (char**) id->menu_vals;
         bstr = strs[gd->step_button-1];
         len = strlen (bstr);
      }
      if (max_len>0)
      {
         if (len>max_len) len = max_len;
         strncpy (string, bstr, len);
         string[len] = 0;
         return 0;
      }
      else
      {
         max_len = - max_len;
         if (len>max_len) len = max_len;
         strncpy (string, bstr, len);
         if (max_len>len)
         {
            for (i=len;i<max_len;++i) string[i] = ' ';
         }
      }
   }
   else
   {
   /* Normal or menu value input */

      len =gd->max_value_chars;
      if (max_len>0)
      {
         if (len>max_len) len = max_len;
         strncpy (string, &(gd->text[id->toff_val]), len);
         string[len] = 0;
         return 0;
      }
      else
      {
         max_len = - max_len;
         if (len>max_len) len = max_len;
         strncpy (string, &(gd->text[id->toff_val]), len);
         if (max_len>len)
         {
            for (i=len;i<max_len;++i) string[i] = ' ';
         }
      }
   }
   return 0;
}

/* Fortran binding: xdlf_param_table_getvalue */

void xdlf_param_table_getvalue (ivh, item, string, maxlen, ierr)
int *ivh, *item, *string, *maxlen, *ierr;
{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ierr = xdl_param_table_getvalue (*ivh, item, XDL_pointers[*string], 
                                     max_len);
   return;
}

/*-Routine: Parameter table error handling - xdl_param_table_error
The routine xdl_param_table_error (xdlf_param_table_error) is used to
display an error notice (using a popup-notice) and replace the parameter
value with the old value old value. It will normally be used after a call
to xdl_param_table_getvalue (xdlf_param_table_getvalue) if the returned 
value was found to be invalid. (Note: that error checking is the responsibility
of the application program; the parameter table merely returns the string as
input.
-end*/
/*
*********************************
**     xdl_param_table_error   **
*********************************

Purpose: Display error notice and replace value with old value (for use
         after xdl_param_table_getvalue if the returned value is not valid)

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_ERROR (IVH, ITEM, XDLSTR(ERRSTR), LEN, 
        +                             IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Item number (see item)
ERRSTR  (R)   Error message character string (see errstr)
              ** Pass address using the XDLSTR function **
LEN     (R)   Length of the error string ( > 0 ) (see len)
IERR    (W)   Returned status from xdl_param_table_error call
-end*/

/*-C:*/
int xdl_param_table_error (vh, item, errstr, len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int item;        /* The item number (R) */
char errstr[];   /* The error message for the notice (R)*/
int len;         /* The length of the string; If 0 is given then the 
                    length will be determined assuming a null terminated 
                    string (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;             /* Offset in XDL_view_objects array */
   int i;              /* Loop counter */
   int xx, yy;         /* Position for error notice focus */
   int xroot, yroot;   /* Root x, y position for notice focus */

   XDl_param_table *gd;           /* Pointer to global data area */
   XDl_param_table_details *id;   /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_error",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_error", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   if (item<1||item>gd->num_params) return 0;
   if (len==0) len = strlen (errstr);
   id = &(gd->item_details[item-1]);
   for (i=0;i<gd->max_value_chars;++i)
   {
      gd->text[id->toff_val+i] = gd->text[id->toff_oldval+i];
   }
   id->sc1 = gd->sav_sc1;
   id->sc2 = gd->sav_sc2;
   id->minsc = gd->sav_minsc;
   id->nsc = gd->sav_nsc;
   if (id->toggle>0)
   {
      if (id->toggle<=2)
      {
         id->toggle = 3 - id->toggle;
      }
      else
      {
         id->toggle = 7 - id->toggle;
      }
   }
   xx = id->xoff_val + (gd->max_value_disp)*(gd->ch_width);
   yy = id->yoff + (gd->row_height)/2;
   xdl_rootxy (gd->wparam, xx, yy, &xroot, &yroot);
   i = xdl_popup_notice (xroot, yroot, errstr, len, 
                         "** Old value will be replaced **", 0,
                         "Continue", 0, NULL, -1, gd->font_type, 1);
   xdl_param_table_drawitem (gd, item, 0, 0);
   return 0;
}

/* Fortran binding: xdl_param_table_error */

void xdlf_param_table_error (ivh, item, errstr, len, ierr)
int *ivh, *item, *errstr, *len, *ierr;
{
   *ierr = xdl_param_table_error (*ivh, *item, XDL_pointers[*errstr], *len);
   return;
}

/*-Routine: Set a parameter table item - xdl_param_table_setitem
The routine xdl_param_table_setitem (xdlf_param_table_setitem) is used
to set a parameter name and value string in a parameter table. Items
with toggle values may also be set using this routine. Label items
(items without values) may also be set. Any value menus required are
subsequently set using calls to xdlf_param_table_setmenu 
(xdl_param_table_setmnu) or any step-buttons using xdlf_param_table_setbutton
(xdl_param_table_setbutton).
-end*/
/*
*************************************
**     xdl_param_table_setitem     **
*************************************

Purpose: Set a parameter item in the parameter table

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_SETITEM (IVH, ITEM, XDLSTR(NAME), LENNAM, 
        +                              XDLSTR(VALUE), LENVAL, ITOG, 
        +                              IEMPH, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Item number (see item)
NAME    (R)   Parameter name character string (see name)
              ** Pass address using the XDLSTR function **
LENNAM  (R)   Length of the name string ( > 0 ) (see len_name)
VALUE   (R)   Parameter value character string (see value)
              ** Pass address using the XDLSTR function **
LENVAL  (R)   Length of the value string ( > 0 ) (see len_value)
ITOG    (R)   Toggle type flag: 0 = normal value, 1,2 on/off 
              toggle, 3,4 yes/no toggle; =-1 Label item 
              (see toggle)
IEMPH   (R)   =0 normal print, =1 bold, =-1 silent (see emphasize)
IERR    (W)   Returned status from xdl_param_table_setitem call
-end*/

/*-C:*/
int xdl_param_table_setitem (vh, item, name, len_name, value, 
                             len_value, toggle, emphasize)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int item;        /* Item number (R) */
char * name;     /* Parameter name string (R) */
int len_name;    /* Length of parameter name string; if 0 then
                    length will be found assuming a null terminated 
                    string (R) */
char * value;    /* Parameter value string (ignored for a toggle or label
                    item) (R) */
int len_value;   /* Length of parameter value string; if 0 then
                    length will be found assuming a null terminated 
                    string (ignored for a toggle item) (R) */
int toggle;      /* =0 normal item with a value, =1,2 on/off toggle 
                    (1=on by default, 2=off), =3,4 yes/no toggle 
                    (3=yes by default 4=no), =-1 label item. If toggle is
                     non-zero then the value string is ignored (R) */
int emphasize;   /* =0 normal print, =1 emphasized, =-1 'silent'. Silent
                    items will not appear in the table. They may be made
                    to appear by calling xdl_param_table_all_normal or
                    xdl_param_table_normal or xdl_param_table_bold (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                          /* Offset in XDL_view_objects array */
   XDl_param_table *gd;             /* Pointer to global data area */
   XDl_param_table_details *id;     /* Pointer to item details structure */
   int len;                         /* String length */
   int i, j;                        /* Temp/loop variables */
   int ll;                          /* significant length of value string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_setitem",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_setitem", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   if (toggle<-1||toggle>4) toggle = 0;
   if (emphasize<-1||emphasize>1) emphasize = 0;
   if (item<1||item>gd->num_params) return 0;
   id = &(gd->item_details[item-1]);
   id->status = 1;
   id->emphasize = emphasize;
   for (i=0;i<(gd->max_name_chars+2*(gd->max_value_chars));++i) 
   {
      gd->text[id->toff_name+i] = ' ';
   }
   len = len_name;
   if (len==0) len = strlen (name);
   if (len>gd->max_name_chars) len = gd->max_name_chars;
   for (i=0,j=id->toff_name;i<len;++i,++j) gd->text[j] = name[i];
   id->toggle = toggle;
   if (toggle==0)
   {
      id->toggle = 0;
      len = len_value;
      if (len==0) len = strlen (value);
      if (len>gd->max_value_chars) len = gd->max_value_chars;
      for (i=0,j=id->toff_val;i<len;++i,++j) gd->text[j] = value[i];
      ll = xdl_param_table_lenstr (value, len);
      id->sc1 = 0;
      id->minsc = 0;
      if (ll<=gd->max_value_disp)
      {
         id->sc2 = 0;
         id->nsc = ll;
      }
      else
      {
         id->sc2 = 1;
         id->nsc = gd->max_value_disp - 1;
      }
   }
   id->menu = 0;
   id->num_menu_vals = 0;
   id->menu_vals = 0;
   id->len_menu_vals = 0;
   if (gd->repaint_done==1) xdl_param_table_drawitem (gd, item, 0, 0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_setitem */

void xdlf_param_table_setitem (ivh, item, name, lennam, value, lenval, 
                               itog, iemph, ierr)
int *ivh, *item, *name, *lennam, *value, *lenval, *itog, *iemph, *ierr;
{
   *ierr = xdl_param_table_setitem (*ivh, *item, XDL_pointers[*name], *lennam,
                                   XDL_pointers[*value], *lenval, 
                                   *itog, *iemph);
   return;
}

/*-Routine: Set a parameter value string - xdl_param_table_setvalue
The routine xdl_param_table_setvalue (xdlf_param_table_setvalue) is used
to set or change the value string for a parameter item. A toggle value
may be set if required or an item may be converted to a label item.
If a toggle value is set, then any popup menu currently defined for the 
item will be removed.
-end*/
/*
*************************************
**     xdl_param_table_setvalue    **
*************************************

Purpose: Set a parameter item value in the parameter table.

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_SETVALUE (IVH, ITEM, XDLSTR(VALUE), 
        +                                LENVAL, ITOG, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Item number (see item)
VALUE   (R)   Parameter value character string (see value)
              ** Pass address using the XDLSTR function **
LENVAL  (R)   Length of the value string ( > 0 ) (see len_value)
ITOG    (R)   Toggle type flag: 0 = normal value, 1,2 on/off 
              toggle, 3,4 yes/no toggle, =-1 change item to
              a label item. (if not 0, value is ignored)
              (see toggle)
IERR    (W)   Returned status from xdl_param_table_setvalue call
-end*/

/*-C:*/
int xdl_param_table_setvalue (vh, item, value, len_value, toggle)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R) */
int item;        /* Item number (R) */
char * value;    /* Parameter value string (ignored if a toggle value set)
                    (R) */
int len_value;   /* Length of parameter value string; if 0 then
                    length will be found assuming a null terminated 
                    string (R) */
int toggle;      /* =0 normal item with a value, =1,2 on/off toggle 
                    (1=on, 2=off), =3,4 yes/no toggle (3=yes, 4=no),
                    =-1 convert to a label item. If toggle is
                    non-zero, then the value string is ignored (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                         /* Offset in XDL_view_objects array */
   XDl_param_table *gd;            /* Pointer to global data area */
   XDl_param_table_details *id;    /* Pointer to item details structure */
   int len;                        /* String length */
   int i, j;                       /* Temp/loop variables */
   int ll;                         /* Significant length of value string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_setvalue",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_setvalue", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   if (toggle<-1||toggle>4) toggle = 0;
   if (item<1||item>gd->num_params) return 0;
   id = &(gd->item_details[item-1]);
   for (i=0;i<(2*(gd->max_value_chars));++i) 
   {
      gd->text[id->toff_val+i] = ' ';
   }
   id->toggle = toggle;
   if (toggle==0)
   {
      id->toggle = 0;
      len = len_value;
      if (len==0) len = strlen (value);
      if (len>gd->max_value_chars) len = gd->max_value_chars;
      for (i=0,j=id->toff_val;i<len;++i,++j) gd->text[j] = value[i];
      ll = xdl_param_table_lenstr (value, len);
      id->sc1 = 0;
      id->minsc = 0;
      if (ll<=gd->max_value_disp)
      {
         id->sc2 = 0;
         id->nsc = ll;
      }
      else
      {
         id->sc2 = 1;
         id->nsc = gd->max_value_disp - 1;
      }
   }
   else
   {
      id->menu = 0;
      id->num_menu_vals = 0;
      id->menu_vals = 0;
      id->len_menu_vals = 0;
   }
   xdl_param_table_drawitem (gd, item, 0, 0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_setvalue */

void xdlf_param_table_setvalue (ivh, item, value, lenval, itog, ierr)
int *ivh, *item, *value, *lenval, *itog, *ierr;
{
   *ierr = xdl_param_table_setvalue (*ivh, *item, XDL_pointers[*value], *lenval,
                                     *itog);
   return;
}

/*-Routine: Set a parameter values menu - xdl_param_table_setmenu
The routine xdl_param_table_setmenu (xdlf_param_table_setmenu) is used
to set or change a popup values menu for a parameter item. No menu will
be set if the parameter item has a toggle value though one is allowed
for a label item.
-end*/
/*
*************************************
**     xdl_param_table_setmenu     **
*************************************

Purpose: Set a parameter item values menu in the parameter table.

Author:  John W. Campbell, April 1991

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_SETMENU (IVH, ITEM, NVALS, 
        +                               XDLSTR(VALUES), LENV, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Parameter table item number (see item)
NVALS   (R)   Number of menu items in the menu (see num_menu_vals)
VALUES  (R)   Fortran array of character strings containing the
              menu item value strings.
              [CHARACTER*LENV VALUES(NVALS)].
              ** Pass address using XDLSTR function **
              (see values)
LENV    (R)   Length of the menu item value character strings
              (must be >0 cf len_vals)
IERR    (W)   Returned status from xdl_param_table_setmenu call
-end*/

/*-C:*/
int xdl_param_table_setmenu (vh, item, num_menu_vals, values, len_vals)
/*end*/
 
/*-Parameters:*/
int vh;            /* View-object handle (R) */
int item;          /* Item number (R) */
int num_menu_vals; /* Number of value strings in the menu (R)*/
char * values;     /* Value strings - this may either be an array of
                      pointers to num_menu_vals null terminated character
                      strings (len_vals=0) or a character string 
                      num_menu_vals*len_vals in length with each value
                      string occupying len_vals characters (left 
                      justified and padded to the right with blanks if 
                      needed. The strings must remain in place during the 
                      lifetime of the menu in the parameter table (i.e. 
                      until the menu is cleared or the parameter table 
                      deleted) as no copies are made of the strings (R)*/
int len_vals;      /* Values strings length (0 = array of pointers to
                      null terminated character strings (e.g. for 'C'
                      programs) or the length of each value string 
                      (e.g. for Fortran programs). See also the previous
                      item. (R) */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Parameter table not set up to allow menus
-end*/
{
   int iv;                         /* Offset in XDL_view_objects array */
   XDl_param_table *gd;            /* Pointer to global data area */
   XDl_param_table_details *id;    /* Pointer to item details structure */
   int len;                        /* String length */
   int i, j, k;                    /* Temp/loop variables */
   char **strs;                    /* Temp pointer to array of null term
                                      strings */
   char *str;                      /* Temp pointer to character string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_setmenu",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_setmenu", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   if (item<1||item>gd->num_params) return 0;
   if (gd->menus_allowed==0) 
   {
      xdl_view_errmsg ("xdl_param_table_setmenu",
                   "Parameter table not set up to allow menus");
      return 4;
   }
   if (len_vals<0) return 0;
   id = &(gd->item_details[item-1]);
   if (id->toggle>0) return 0;
   id->menu = 1;
   id->num_menu_vals = num_menu_vals;
   id->menu_vals = values;
   id->len_menu_vals = len_vals;
   xdl_param_table_drawitem (gd, item, 0, 0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_setmenu */

void xdlf_param_table_setmenu (ivh, item, nvals, values, lenv, ierr)
int *ivh, *item, *nvals, *values, *lenv, *ierr;
{
   *ierr = xdl_param_table_setmenu (*ivh, *item, *nvals, XDL_pointers[*values],
                                   *lenv);
   return;
}
/*-Routine: Set a parameter values button - xdl_param_table_setbutton
The routine xdl_param_table_setbutton (xdlf_param_table_setbutton) is used
to set or change a step-button for a parameter item. The step-button
allows one of two value flags to be returned to the calling program
which will then adjust the parameter value. It is intended to enable
forward or backward stepping through item values (e.g. incrementing or
decrementing a count). No step-button will be set if the parameter item is a 
label or has a toggle value.
-end*/
/*
***************************************
**     xdl_param_table_setbutton     **
***************************************

Purpose: Set a parameter item step-button in the parameter table.

Author:  John W. Campbell, April 1991

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_SETBUTTON (IVH, ITEM,
        +                               XDLSTR(VALUES), LENV, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Parameter table item number (see item)
VALUES  (R)   Fortran array of character strings containing the
              two step button values which may be returned to
              indicate which part of the step-button has been 
              selected (first=top right, 2'nd=bottom left)
              [CHARACTER*LENV VALUES(NVALS)].
              ** Pass address using XDLSTR function **
              (see values)
LENV    (R)   Length of the step-button item value character 
              strings (must be >0 cf len_vals)
IERR    (W)   Returned status from xdl_param_table_setbutton call
-end*/

/*-C:*/
int xdl_param_table_setbutton (vh, item, values, len_vals)
/*end*/
 
/*-Parameters:*/
int vh;            /* View-object handle (R) */
int item;          /* Item number (R) */
char * values;     /* Value strings - this may either be an array of
                      pointers to two null terminated character
                      strings (len_vals=0) or a character string 
                      2*len_vals in length with each value
                      string occupying len_vals characters (left 
                      justified and padded to the right with blanks if 
                      needed. The strings must remain in place during the 
                      lifetime of the button in the parameter table (i.e. 
                      until the button is cleared or the parameter table 
                      deleted) as no copies are made of the strings (R)*/
int len_vals;      /* Values strings length (0 = array of pointers to
                      null terminated character strings (e.g. for 'C'
                      programs) or the length of each value string 
                      (e.g. for Fortran programs). See also the previous
                      item. (R) */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Parameter table not set up to allow menus
-end*/
{
   int iv;                         /* Offset in XDL_view_objects array */
   XDl_param_table *gd;            /* Pointer to global data area */
   XDl_param_table_details *id;    /* Pointer to item details structure */
   int len;                        /* String length */
   int i, j, k;                    /* Temp/loop variables */
   char **strs;                    /* Temp pointer to array of null term
                                      strings */
   char *str;                      /* Temp pointer to character string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_setbutton",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_setbutton", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   if (item<1||item>gd->num_params) return 0;
   if (gd->menus_allowed==0) 
   {
      xdl_view_errmsg ("xdl_param_table_setbutton",
                   "Parameter table not set up to allow buttons");
      return 4;
   }
   if (len_vals<0) return 0;
   id = &(gd->item_details[item-1]);
   if (id->toggle!=0) return 0;
   id->menu = 2;
   id->num_menu_vals = 2;
   id->menu_vals = values;
   id->len_menu_vals = len_vals;
   xdl_param_table_drawitem (gd, item, 0, 0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_setbuttons */

void xdlf_param_table_setbutton (ivh, item, values, lenv, ierr)
int *ivh, *item, *values, *lenv, *ierr;
{
   *ierr = xdl_param_table_setbutton (*ivh, *item, XDL_pointers[*values],
                                   *lenv);
   return;
}

/*-Routine: Clear a parameter values menu - xdl_param_table_clrmenu
The routine xdl_param_table_clrmenu (xdlf_param_table_clrmenu) is used
to clear (i.e. remove) a popup values menu (or button) for a parameter item.
-end*/
/*
*************************************
**     xdl_param_table_clrmenu     **
*************************************

Purpose: Clear a parameter item values menu (or button) from the 
         parameter table.

Author:  John W. Campbell, April 1991

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_CLRMENU (IVH, ITEM, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Parameter table item number (see item)
IERR    (W)   Returned status from xdl_param_table_clrmenu call
-end*/

/*-C:*/
int xdl_param_table_clrmenu (vh, item)
/*end*/
 
/*-Parameters:*/
int vh;            /* View-object handle (R) */
int item;          /* Item number (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                         /* Offset in XDL_view_objects array */
   XDl_param_table *gd;            /* Pointer to global data area */
   XDl_param_table_details *id;    /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_clrmenu",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_clrmenu", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;
   if (item<1||item>gd->num_params) return 0;
   id = &(gd->item_details[item-1]);
   id->menu = 0;
   id->num_menu_vals = 0;
   id->menu_vals = 0;
   id->len_menu_vals = 0;
   xdl_param_table_drawitem (gd, item, 0, 0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_clrmenu */

void xdlf_param_table_clrmenu (ivh, item, ierr)
int *ivh, *item, *ierr;
{
   *ierr = xdl_param_table_clrmenu (*ivh, *item);
   return;
}

/*-Routine: Set item(s) to 'silent' - xdl_param_table_silent
The routine xdl_param_table_silent (xdlf_param_table_silent) is used to
reset the text_emphasis mode for one or more parameter table items to
'silent' mode. These items will no longer be displayed and cannot be
edited whilst in this mode. However the parameter item name and value
are still stored and the item may be reinstated using one of the
routines: xdl_param_table_normal (xdlf_param_table_normal),
xdl_param_table_bold (xdlf_param_table_bold) or xdl_param_table_all_normal
(xdlf_param_table_all_normal).
-end*/
/*
*************************************
**     xdl_param_table_silent      **
*************************************

Purpose: Reset the text emphasis option for a set of parameter table items
         to 'silent'

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_SILENT (IVH, NITEMS, ITEMS, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NITEMS   (R)  Number of items to set (see num_items)
ITEMS    (R)  Integer array of NITEMS item numbers (see items)
IERR     (W)  Returns status from xdl_param_table_silent call
-end*/

/*-C:*/
int xdl_param_table_silent (vh, num_items, items)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R) */
int num_items;   /* Number of items to be set (R) */
int items[];     /* Array of the num_items item numbers to be set (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                      /* Offset in XDL_view_objects array */
   int i;                       /* Loop counter */
   int item;                    /* Item number */
   XDl_param_table *gd;         /* Pointer to global data area */
   XDl_param_table_details *id; /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_silent",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_silent", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   for (i=0;i<num_items;++i)
   {
      item = items[i];
      if (item<1||item>gd->num_params) continue;
      id = &(gd->item_details[item-1]);
      if (id->status==0) continue;
      id->emphasize = -1;
      xdl_param_table_drawitem (gd, item, 0, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_silent */

void xdlf_param_table_silent (ivh, nitems, items, ierr)
int *ivh, *nitems, *items, *ierr;
{
   *ierr = xdl_param_table_silent (*ivh, *nitems, items);
   return;
}

/*-Routine: Set item(s) to 'normal' - xdl_param_table_normal
The routine xdl_param_table_normal (xdlf_param_table_normal) is used to
reset the text_emphasis mode for one or more parameter table items to
'normal' mode.
-end*/
/*
*************************************
**     xdl_param_table_normal      **
*************************************

Purpose: Reset the text emphasis option for a set of parameter table items
         to normal

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_NORMAL (IVH, NITEMS, ITEMS, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NITEMS   (R)  Number of items to set (see num_items)
ITEMS    (R)  Integer array of NITEMS item numbers (see items)
IERR     (W)  Returns status from xdl_param_table_normal call
-end*/

/*-C:*/
int xdl_param_table_normal (vh, num_items, items)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R) */
int num_items;   /* Number of items to be set (R) */
int items[];     /* Array of the num_items item numbers to be set (R) */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                       /* Offset in XDL_view_objects array */
   int i;                        /* Loop counter */
   int item;                     /* Item number */
   XDl_param_table *gd;          /* Pointer to global data area */
   XDl_param_table_details *id;  /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_normal",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_normal", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   for (i=0;i<num_items;++i)
   {
      item = items[i];
      if (item<1||item>gd->num_params) continue;
      id = &(gd->item_details[item-1]);
      if (id->status==0) continue;
      id->emphasize = 0;
      xdl_param_table_drawitem (gd, item, 0, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_normal */

void xdlf_param_table_normal (ivh, nitems, items, ierr)
int *ivh, *nitems, *items, *ierr;
{
   *ierr = xdl_param_table_normal (*ivh, *nitems, items);
   return;
}

/*-Routine: Set item(s) to 'bold' - xdl_param_table_bold
The routine xdl_param_table_bold (xdlf_param_table_bold) is used to
reset the text_emphasis mode for one or more parameter table items to
'bold' mode. These items are the same as 'normal' items except that
the text is in bold print.
-end*/
/*
*************************************
**     xdl_param_table_bold        **
*************************************

Purpose: Reset the text emphasis option for a set of parameter table items
         to bold

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_BOLD (IVH, NITEMS, ITEMS, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NITEMS   (R)  Number of items to set (see num_items)
ITEMS    (R)  Integer array of NITEMS item numbers (see items)
IERR     (W)  Returns status from xdl_param_table_bold call
-end*/

/*-C:*/
int xdl_param_table_bold (vh, num_items, items)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R) */
int num_items;   /* Number of items to be set (R) */
int items[];     /* Array of the num_items item numbers to be set (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                           /* Offset in XDL_view_objects array */
   int i;                            /* Loop counter */
   int item;                         /* Item number */
   XDl_param_table *gd;              /* Pointer to global data area */
   XDl_param_table_details *id;      /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_bold",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_bold", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   for (i=0;i<num_items;++i)
   {
      item = items[i];
      if (item<1||item>gd->num_params) continue;
      id = &(gd->item_details[item-1]);
      if (id->status==0) continue;
      id->emphasize = 1;
      xdl_param_table_drawitem (gd, item, 0, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_bold */

void xdlf_param_table_bold (ivh, nitems, items, ierr)
int *ivh, *nitems, *items, *ierr;
{
   *ierr = xdl_param_table_bold (*ivh, *nitems, items);
   return;
}

/*-Routine: Reset all items to 'normal' - xdl_param_table_all_normal
The routine xdl_param_table_all_normal (xdlf_param_table_all_normal) is 
used to reset the text_emphasis mode for all the parameter table items to
'normal' mode. 
-end*/
/*
*************************************
**     xdl_param_table_all_normal  **
*************************************

Purpose: Reset the text emphasis option to normal text for all parameter 
         table items

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_ALL_NORMAL (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_param_table_all_normal call
-end*/

/*-C:*/
int xdl_param_table_all_normal (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                       /* Offset in XDL_view_objects array */
   int i;                        /* Loop counter */
   XDl_param_table *gd;          /* Pointer to global data area */
   XDl_param_table_details *id;  /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_all_normal",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_all_normal", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   for (i=0;i<gd->num_params;++i)
   {
      id = &(gd->item_details[i]);
      if (id->status==0) continue;
      id->emphasize = 0;
      xdl_param_table_drawitem (gd, i+1, 0, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_all_normal */

void xdlf_param_table_all_normal (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_param_table_all_normal (*ivh);
   return;
}

/*-Routine: Delete a parameter table item - xdl_param_table_delitem
The routine xdl_param_table_delitem (xdlf_param_table_delitem) is used to
delete (remove) a parameter item from the parameter table. The position
in the table may subsequently be used for another item if required.
-end*/
/*
*************************************
**     xdl_param_table_delitem     **
*************************************

Purpose: Delete a parameter item from the parameter table

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_DELITEM (IVH, ITEM, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
ITEM    (R)   Item number (see item)
IERR    (W)   Returned status from xdl_param_table_delitem call
-end*/

/*-C:*/
int xdl_param_table_delitem (vh, item)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int item;        /* Item number (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                          /* Offset in XDL_view_objects array */
   XDl_param_table *gd;             /* Pointer to global data area */
   XDl_param_table_details *id;     /* Pointer to item details structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_delitem",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_delitem", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   if (item<1||item>gd->num_params) return 0;
   id = &(gd->item_details[item-1]);
   id->status = 0;
   id->emphasize = 0;
   id->toggle = 0;
   XClearArea (XDL_display, gd->wparam, id->xoff, id->yoff,
              (gd->col_width-1), gd->row_height, False);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_delitem */

void xdlf_param_table_delitem (ivh, item, ierr)
int *ivh, *item, *ierr;
{
   *ierr = xdl_param_table_delitem (*ivh, *item);
   return;
}

/*-Routine: Clear the parameter table - xdl_param_table_clearitems
The routine xdl_param_table_clearitems (xdlf_param_table_clearitems) is
used to clear all parameter items from the parameter table. New items
may subsequently be set up if required.
-end*/
/*
*************************************
**     xdl_param_table_clearitems  **
*************************************

Purpose: Clear all the parameter items from the parameter table

Author:  John W. Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_CLEARITEMS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_param_table_clearitems call
-end*/

/*-C:*/
int xdl_param_table_clearitems (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                        /* Offset in XDL_view_objects array */
   XDl_param_table *gd;           /* Pointer to global data area */
   XDl_param_table_details *id;   /* Pointer to item details structure */
   int nch_max;                   /* Maximum number of characters */
   int i;                         /* Loop counter */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_clearitems",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_clearitems", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   nch_max = gd->num_params * (gd->max_name_chars + 2*(gd->max_value_chars));
   for (i=0; i<nch_max; i++) gd->text[i] = ' ';
   
   for (i=1;i<=gd->num_params;++i)
   {
      id = &(gd->item_details[i-1]);
      id->status = 0;
      id->emphasize = 0;
      id->toggle = 0;
      XClearArea (XDL_display, gd->wparam, id->xoff, id->yoff,
                  (gd->col_width-1), gd->row_height, False);
   }
   gd->highlight_item = 0;
   gd->input_item = 0;
   gd->last_input_item = 0;
   gd->curs_offset = -1;
   gd->last_in = -1;
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_clearitems */

void xdlf_param_table_clearitems (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_param_table_clearitems (*ivh);
   return;
}

/*-Routine: Output a new title - xdl_param_table_newtitle
The routine xdl_param_table_newtitle (xdlf_param_table_newtitle) is used to
output a new title for the parameter table. Note that a title string
(possibly blank) must have been specified when the parameter table view-object
was created and the size of the view-object may depend on this original
title length. Thus if a longer title is to be output at a later stage, it
would be wise to create the view-object with a title string of the maximum
length required; this could be blank and then reset using this new title
routine if desired. If the new title string is too long for the space
available, it will be truncated.
-end*/
/*
*************************************
**     xdl_param_table_newtitle    **
*************************************

Purpose: Output a new title string

Author:  John W. Campbell, April 1996

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_NEWTITLE (IVH, XDLSTR(TITLE), 
        +                                ITLEN, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
TITLE     (R)   Title string for the parameter table (see title)
                ** Pass address using the XDLSTR function **
ITLEN     (R)   The title length (> 0) 
IERR      (W)   Returned status from xdl_param_table_newtitle call
-end*/

/*-C:*/
int xdl_param_table_newtitle (vh, title, nt)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *title;     /* Title string (R)*/
int nt;          /* Length of title string; if 0 then length 
                    found assuming that 'title' is null 
                    terminated (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  cannot allocate memory for title
-end*/
{
   int iv;                          /* Offset in XDL_view_objects array */
   XDl_param_table *gd;             /* Pointer to global data area */
   XDl_param_table_details *id;     /* Pointer to item details structure */
   int lt;                          /* Title length */
   char *p;                         /* Pointer to character string */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_param_table_newtitle",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1041)
   {
      xdl_view_type_errmsg ("xdl_param_table_newtitle", vh, "xdl_param_table");
      return 2;
   }
   gd = (XDl_param_table *) XDL_view_objects[iv]->gd;

   if (nt<0) return 0;
   if (gd->title_height==0) return 0;
   lt = nt;
   if (lt==0) lt = strlen(title);
   if (lt<=0) return 0;
   p = (char *) malloc (lt+1);
   if (p==0) return 4;
   gd->title = p;
   strncpy (gd->title, title, lt);
   gd->title[lt] = 0;
   gd->title_len = lt;
   gd->title_xoff = (gd->table_width - lt*gd->ch_width) / 2;
   if ( gd->title_xoff<0)  gd->title_xoff = 0;
   XClearArea (XDL_display, gd->wparam, 0, 0,
              (gd->table_width), gd->title_height-1, False);
   xdl_param_table_drawstring (gd, gd->title_xoff, 0, gd->title,
                               gd->title_len, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_param_table_newtitle */

void xdlf_param_table_newtitle (ivh, title, itlen, ierr)
int *ivh, *title, *itlen, *ierr;
{
   *ierr = xdl_param_table_newtitle (*ivh, XDL_pointers[*title], *itlen);
   return;
}

/*-Routine: Calculate size requirements - xdl_param_table_getsize
The routine xdl_param_table_getsize (xdlf_param_table_getsize) is used to
get the required size of a parameter table view-object given the required
number of rows and columns of items and maximum lengths of item name
and value strings.
-end*/
/*
**************************************
**     xdl_param_table_getsize      **
**************************************

Purpose: Return the size requirements of a parameter table view-object

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_PARAM_TABLE_GETSIZE (NCOLS, NROWS, MAXNAM, MAXVDISP, 
     +                               NT, IFONT, IMENU, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
NCOLS      (R)  No. of columns of parameters (see num_cols)
NROWS      (R)  No. of rows of parameters (see num_rows)
MAXNAM     (R)  Maximum parameter name length (chars) 
                (see max_name_chars)
MAXVDISP   (R)  Maximum value string length which may be displayed
                (chars) (see max_value_disp)
NT         (R)  >=0 title present; =-1 no title (see nt)
IFONT      (R)  Font type 1-4 (see font_type)
IMENU      (R)  =1 popup values menus allowed, =0 not 
                (see menus_allowed)
IWIDTH     (W)  Returns the width required (see w)
IHEIGHT    (W)  Returns the height required (see h)
-end*/

/*-C:*/
void xdl_param_table_getsize (num_cols, num_rows, max_name_chars,
                              max_value_disp, nt, font_type, 
                              menus_allowed, w, h)
/*end*/
 
/*-Parameters:*/
int       num_cols;       /* Maximum number of columns of items in the 
                             parameter table (R)*/
int       num_rows;       /* Maximum number of rows of items in the 
                             parameter table (R)*/
int       max_name_chars; /* Maximum number of characters in a parameter 
                             name string (R)*/
int       max_value_disp; /* Maximum number of characters in a parameter 
                             value string which may be displayed (R)*/
int       nt;             /* >=0 title present; =-1 no title (R)*/
int       font_type;      /* Font type (1-5) for very_small, small, medium,
                             large or extra-large font. (R)*/
int       menus_allowed;  /* =1 popup values menus allowed, =0 not (R)*/
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int table_width;            /* Calc table width */
   int table_height;           /* Calc table height */
   XFontStruct *fontinfo;      /* Pointer to normal font info */
   XFontStruct *bold_fontinfo; /* Pointer to bold font info */
   int row_height;             /* Row height in pixels */
   int col_width;              /* Column width in pixels */
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int ascent;                 /* Font ascent */
   int descent;                /* Font descent */
   int title_height;           /* Title height */

   /* Calculate size requirements for the windows */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (menus_allowed!=1) menus_allowed = 0;
   fontinfo = XDL_fonts[font_type-1];
   bold_fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   title_height = 0;
   if (nt>=0) title_height = ch_height + 1;
   col_width = (max_name_chars + max_value_disp)*ch_width 
               + 2*MARG_W + SEP_W + 1;
   if (menus_allowed) col_width += ch_width + SEP_M;
   table_width = num_cols*col_width + 2;
   row_height = ch_height + SEP_H;
   table_height = num_rows*row_height + title_height + 2*MARG_H + 2;
   *w = table_width + 2*BORDER_WIDTH + 2;
   *h = table_height + 2*BORDER_HEIGHT + ACT_STRIP_HEIGHT + 2;
   return;
}

/* Fortran binding: xdlf_param_table_getsize */

void xdlf_param_table_getsize (ncols, nrows, maxnam, maxvdisp, nt,
                               ifont, imenu, iwidth, iheight)
int *ncols, *nrows, *maxnam, *maxvdisp, *nt, *ifont, *imenu, *iwidth, *iheight;
{
   xdl_param_table_getsize (*ncols, *nrows, *maxnam, *maxvdisp, *nt, *ifont,
                            *imenu, iwidth, iheight);
   return;
}
