/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Pop-up Dialogue Box Routines
-end*/
 
/*-Intro:
The pop-up dialog box view-object is used to request the immediate input
of a text string in response to a prompt. The program will only proceed after 
the user has input a reply (or pressed the Escape key to abort the input).
It is typically used to input information such as a file name. If possible,
the use of this view-object should be avoided as it 'grabs the keyboard and
the pointer' and alternatives such as a control view-object with a Panel
I/O item should be considered.
-end*/
 
/*-Section: Creating and Handling a Pop-up Dialogue Box View-Object
-end*/
 

/*Routines List:
The pop-up dialog box view-object has the following routines available:

   xdl_popup_dialog           Display a pop-up dialog box
*/

/*-Routine: Display a pop-up dialog box - xdl_popup_dialog
The routine xdl_popup_dialog (xdlf_popup_dialog) is used to display a
pop-up dialog box containing a prompt and to wait till a reply is
input (or the Escape key is pressed to abort the input).
Note that the pop-up dialog box is rather different from most other 
view-objects in that it is a temporary object and does not have a view-object 
handle.
-end*/
/*
******************************
**     xdl_popup_dialog     **
******************************

Purpose: Create a dialog popup box

Author:  John W. Campbell, April 1991

*/
/*-Fortran:
         CALL XDLF_POPUP_DIALOG (IXROOT, IYROOT, PROMPT, LENP, 
        +                        IWIDTH,
        +                        REPLY, LENR,
        +                        IFONT, IARROW, IRETURN)
-end*/
 
/*-Parameters:
IXROOT    (R)  X (rootwindow) position for dialog (top left) 
               (see xroot)
IYROOT    (R)  Y (rootwindow) position for dialog (top left)
               (see yroot)
PROMPT    (R)  Address of character string containing the prompt.
               ** Pass address using the XDLSTR function **
               (see prompt)
LENP      (R)  Length of the PROMPT character string (>0) 
               (cf lenp)
IWIDTH    (R)  Width of box as the number of characters in
               the prompt string + the number to be displayed
               in the reply string (note: the reply string can 
               be scrolled using the cursor left and right keys)
               (see width)
REPLY     (W)  Address of character string to contain the reply
               message.
               ** Pass address using the XDLSTR function **
               (see reply)
MAXLEN    (R)  Maximum length for the returned reply string (>0).
               (cf max_len)
IFONT     (R)  Font type (1 to 5, very small to extra large) 
               (see font_type)
IBOLD     (R)  =0 Prompt and input string in normal text
               =1 Prompt bold, input string normal
               =2 Prompt normal, input string bold
               =3 Both strings bold
               (see bold)
IARROW    (R)  =1 display an arrow pointing to the top left corner
               of the dialog box provided that it can be placed 
               in the requested position, =0 do not. (see arrow)
IRETURN   (W)  Return flag = 0 Blank string
                           = 1 String returned (non-blank)
                           =-1 Input aborted (via Escape key)
                           =-2 Unable to allocate memory for
                               temporary input string
                           =-3 No room for an input string
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#ifdef VMS
#include <decw$include/keysym.h>
#else
#include <X11/keysym.h>
#endif

/*Define routine types*/
int xdl_popup_dialog();
void xdl_popup_dialog_drawcursor();
void xdl_popup_dialog_replystr();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_popup_dialog xdlf_popup_dialog_

#endif

#if  LINKTYP == 2

#define xdlf_popup_dialog XDLF_POPUP_DIALOG

#endif

void xdlf_popup_dialog();

/*User callable routine*/

/*-C:*/
int xdl_popup_dialog (xroot, yroot, prompt, lenp, width, reply, max_len, 
                      font_type, bold, arrow)
/*end*/
 
/*-Parameters:*/
int xroot;          /* x position of the dialog popup area 
                       (wrt root) (R)*/
int yroot;          /* y position of the dialog popup area 
                       (wrt root) (R)*/
char * prompt;      /* Prompt string (R)*/
int lenp;           /* Length of prompt string. If 0 then the routine will
                       find the length assuming a null terminated 
                       string (R)*/
int width;          /* Width of box as the number of characters in
                       the prompt string + the number to be displayed
                       in the reply string (note: the reply string can 
                       be scrolled using the cursor left and right keys)
                       (R)*/
char * reply;       /* Address for returned string (W)*/
int max_len;        /* if >0 Maximum length of return string allowed
                             (excluding terminating null - reply must be
                             at least max_len+1 characters in length)
                       if <0 abs(max_len) characters will be returned
                             padded with blanks if needed (for 'fortran'
                             use) (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int bold;           /*  =0 Prompt and input string in normal text
                        =1 Prompt bold, input string normal
                        =2 Prompt normal, input string bold 
                        =3 Both strings bold  (R)*/
int arrow;          /* =1 draw arrow pointing to origin (provided that
                       dialog may be positioned where requested), =0 do
                       not (R)*/
/*end*/
/*-Doc:
Return:  = 0,  Blank reply
         = 1,  Text string was input
         =-1,  Input aborted
         =-2,  Unable to allocate memory for temporary input string
         =-3,  No room for an input string
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XFontStruct *fontinfo_bold; /*Temp pointer to bold font structure*/
   XSetWindowAttributes attrib;/*Temp window attributes structure*/
   XEvent ev;                  /*Event return for dialog event handling*/
   Window wid;                 /*Window id*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   GC paint_gc;                /*Graphics context for window*/
   int area_width;             /*Width of area required*/
   int area_height;            /*Height of area required*/
   int lenpr;                  /*Length of prompt to output*/
   int reply_width;            /*Length of reply field*/
   int maxlen;                 /*Maximum input string length*/
   int xx, yy;                 /*Current character output position*/
   int xprompt, yprompt;       /*Coordinates of start of prompt string*/
   int xreply, yreply;         /*Coordinates of start of reply string*/
   int yreplytop;              /*Top of reply characters*/
   int ycurs;                  /*Cursor y coordinate*/
   KeySym keysym;              /*KeySym for keyboard input*/
   XComposeStatus compose;     /*Compose status*/
   int nc;                     /*No. chars decoded from key press*/
   char chbuf[2];              /*Buffer for ascii translated characters*/
   int iret;                   /*Return flag*/
   int i;                      /*Temp/loop variable*/
   int last_in;                /*Offset of last input character*/
   int cur_offset;             /*Current(/cursor) position offset in
                                 input reply string (may be within the
                                 string or one past the end of the string)*/
   int curs_offset;            /*Cursor position offset in reply window*/
   int start_offset;           /*Offset in input reply string of first
                                 character displayed in the reply window*/
   char * instring;            /*Input string*/
   Font fid_normal;            /*Normal font*/
   Font fid_bold;              /*Bold font*/

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Calculate size requirements */

   maxlen = max_len;
   if (maxlen<0) maxlen = -maxlen;
   instring = (char *) malloc(maxlen+1);
   if (instring==0)
   {
      xdl_view_errmsg("xdl_popup_dialog",
                      "Unable to allocate memory for temporary input string");
   }
   if (lenp==0) lenp = strlen (prompt);
   lenpr = lenp;
   if (lenpr>0)
   {
      for (i=lenpr-1;i>=0;--i)
      {
          lenpr = i + 1;
          if (prompt[i]!=' ') break;
      }
      if (prompt[lenp-1]==' ') lenpr++;
      if (lenpr>lenp) lenpr = lenp;
   }
       
   fontinfo = XDL_fonts[font_type-1];
   fontinfo_bold = XDL_bold_fonts[font_type-1];
   fid_normal = fontinfo->fid;
   fid_bold = fontinfo_bold->fid;
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);

   area_width = (width + 2)*ch_width + 8;
   reply_width = width - lenpr;
   if (reply_width<1) return -4;

   area_height = 2*ch_height + 8;

   /* Check position and adjust if needed */

   if ((xroot+area_width+2)>=DisplayWidth (XDL_display,XDL_screen))
   {
      xroot = DisplayWidth (XDL_display,XDL_screen) - area_width - 2;
      arrow = 0;
   }
   if ((yroot+area_height+2)>=DisplayHeight (XDL_display,XDL_screen))
   {
      yroot = DisplayHeight (XDL_display,XDL_screen) - area_height - 2;
      arrow = 0;
   }

   /* Get string positions */

   xprompt = ch_width + 4;
   yprompt = (area_height-ch_height)/2 + ascent;
   xreply = (lenpr+1)*ch_width + 4;
   yreply = yprompt;
   yreplytop = (area_height-ch_height)/2;
   ycurs = yreplytop;

   /* Create dialog box window */

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.override_redirect = True;
   attrib.save_under = True;

   wid = XCreateWindow (XDL_display, 
                       RootWindow(XDL_display, XDL_screen), xroot, yroot,
                       area_width, area_height, 1, XDL_pw_depth, InputOutput, 
                       XDL_visual,
                       CWBackPixel|CWBorderPixel|CWColormap|
                       CWSaveUnder|CWOverrideRedirect,
                       &attrib);

   XSelectInput (XDL_display, wid, ExposureMask|StructureNotifyMask);
  
   XMapWindow (XDL_display, wid);

   /* Create and set paint_gc */
   
   paint_gc = XCreateGC (XDL_display, wid, 0, NULL);
   XSetBackground (XDL_display, paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, paint_gc, XDL_blackpixel);

   /* Grab pointer and keyboard */

   XGrabPointer (XDL_display, wid, False, ButtonPressMask, 
                GrabModeAsync, GrabModeAsync,
                None, 0, CurrentTime);

   XGrabKeyboard (XDL_display, wid, False, GrabModeAsync, GrabModeAsync,
                  CurrentTime);

   /* Initialise input string */

   instring[0] = 0;
   last_in = -1;
   start_offset = 0;
   cur_offset = 0;
   curs_offset = 0;

   /* Handle events for the dialog box window */
   next:
   XNextEvent (XDL_display, &ev);
   if (ev.xany.window==wid)
   {
      switch (ev.type)
      {
         /* Repaint dialog */
         case Expose:
         case ConfigureNotify:

            /* Draw box and arrow */

            XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
            XClearWindow (XDL_display, wid);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           3, 3, area_width-7, area_height-7);
            if (XDL_color>1)
            {
               XSetForeground (XDL_display, paint_gc, XDL_bluepixel);
            }
            XDrawRectangle (XDL_display, wid, paint_gc,
                           0, 0, area_width-1, area_height-1);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           1, 1, area_width-3, area_height-3);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           2, 2, area_width-5, area_height-5);
            if (arrow)
            {
               XClearArea (XDL_display, wid, 0, 0, 7, 7, False);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 4, 0);
               XDrawLine (XDL_display, wid, paint_gc, 0, 1, 4, 1);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 0, 4);
               XDrawLine (XDL_display, wid, paint_gc, 1, 0, 1, 4);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 9, 9);
               XDrawLine (XDL_display, wid, paint_gc, 4, 3, 9, 8);
               XDrawLine (XDL_display, wid, paint_gc, 3, 4, 8, 9);
            }

            /* Draw prompt string */

            XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
            if (bold==1||bold==3)
            {
               XSetFont (XDL_display, paint_gc, fid_bold);
            }
            else
            {
               XSetFont (XDL_display, paint_gc, fid_normal);
            }
            xx = xprompt;
            for (i=0;i<lenpr;++i)
            {
                XDrawString (XDL_display, wid, paint_gc, xx, yprompt,
                             &prompt[i], 1);
                xx += ch_width;
            }
            /* Draw reply/cursor */
            if (bold==2||bold==3)
            {
               XSetFont (XDL_display, paint_gc, fid_bold);
            }
            else
            {
               XSetFont (XDL_display, paint_gc, fid_normal);
            }
            xdl_popup_dialog_replystr (wid, paint_gc, xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
            break;
         case ButtonPress:
            if (ev.xbutton.button==Button1)
            {
               XSetInputFocus (XDL_display, wid, RevertToParent, CurrentTime);
            }
            break;
         case KeyPress:
            nc = XLookupString ((XKeyEvent *) &ev, chbuf, 2, &keysym, NULL);
            if (keysym==XK_BackSpace||keysym==XK_Delete)
            {
               if (last_in>=0)
               {
                  curs_offset = cur_offset - start_offset;
                  if (curs_offset>0)
                  {
                     if (cur_offset>last_in)
                     {
                        xx = xreply + curs_offset*ch_width;
                        yy = yreplytop;
                        XClearArea (XDL_display, wid, xx, yy,
                                    ch_width, ch_height, False);
                        xx = xreply + (last_in-start_offset)*ch_width;
                        XClearArea (XDL_display, wid, xx, yy,
                                    ch_width, ch_height, False);
                        cur_offset--;
                        curs_offset--;
                        xx = xreply + curs_offset*ch_width;
                        xdl_popup_dialog_drawcursor (wid, paint_gc, xx, ycurs,
                                                  ch_width, ch_height);
                        last_in--;
                     }
                     else
                     {
                        for (i=cur_offset;i<=last_in;++i)
                        {
                           instring[i-1] = instring[i];
                        }
                        last_in--;
                        cur_offset--;
                        xdl_popup_dialog_replystr (wid, paint_gc, 
                                       xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
                     }
                  }
                  else
                  {
                     if (cur_offset<=last_in)
                     {
                        for (i=cur_offset;i<=last_in;++i)
                        {
                           instring[i-1] = instring[i];
                        }
                     }
                     last_in--;
                     cur_offset--;
                     start_offset--;
                     xdl_popup_dialog_replystr (wid, paint_gc, xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
                  }
               }
               goto next;
            }
            else if (keysym==XK_Escape)
            {
               iret = -1;
               instring[0] = 0;
               goto tidyup;
            }
            else if (keysym==XK_Return||keysym==XK_KP_Enter
                     ||keysym==XK_Linefeed)
            {
               XClearArea (XDL_display, wid, xreply, yreplytop,
                          (reply_width*ch_width), ch_height, False);
               xx = xreply;
               iret = 0;
               if (last_in>=0)
               {
                  for (i=start_offset;i<=last_in;++i)
                  {
                      XDrawString (XDL_display, wid, paint_gc, xx, yreply,
                                   &instring[i], 1);
                      xx += ch_width;
                      if (instring[i]!=' ') iret = 1;
                  }
               }
               instring[last_in+1] = 0;
               goto tidyup;
            }
            else if (keysym>=XK_space&&keysym<=XK_asciitilde)
            {
               if (last_in<(maxlen-1))
               {
                  if (cur_offset>last_in)
                  {
                     last_in += 1;
                     instring[last_in] = chbuf[0];
                     curs_offset = cur_offset - start_offset;
                     if (curs_offset<reply_width-2)
                     {
                       xx = xreply + (last_in-start_offset)*ch_width;
                       yy = yreply;
                       XClearArea (XDL_display, wid, xx, yreplytop, ch_width,
                                   ch_height, False);
                       XDrawString (XDL_display, wid, paint_gc, xx, yy,
                           &chbuf[0], 1);
                       cur_offset++;
                       curs_offset++;
                       xx = xreply + curs_offset*ch_width;
                       xdl_popup_dialog_drawcursor (wid, paint_gc, xx, ycurs,
                                               ch_width, ch_height);
                     }
                     else
                     {
                        cur_offset++;
                        start_offset++;
                        xdl_popup_dialog_replystr (wid, paint_gc, 
                                       xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
                     }
                  }
                  else
                  {
                     last_in++;
                     for (i=last_in;i>cur_offset;--i)
                     {
                        instring[i]=instring[i-1];
                     }
                     instring[cur_offset] = chbuf[0];
                     cur_offset++;
                     curs_offset = cur_offset - start_offset;
                     if (curs_offset>=reply_width)
                     {
                        start_offset++;
                     }
                     xdl_popup_dialog_replystr (wid, paint_gc, xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
                  }
               }
            }
            else if (keysym>=XK_Right)
            {
               if (cur_offset<=last_in)
               {
                  cur_offset++;
                  curs_offset = cur_offset - start_offset;
                  if (curs_offset>=reply_width)
                  {
                     start_offset++;
                  }
                  xdl_popup_dialog_replystr (wid, paint_gc, xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
               }
            }
            else if (keysym>=XK_Left)
            {
               if (last_in>=0)
               {
                  if (cur_offset>0)
                  {
                     cur_offset--;
                     curs_offset = cur_offset - start_offset;
                     if (curs_offset<0&&start_offset>0)
                     {
                        start_offset--;
                     }
                     xdl_popup_dialog_replystr (wid, paint_gc, 
                                       xreply, yreply,
                                       yreplytop, reply_width,
                                       instring, last_in, 
                                       start_offset, cur_offset,
                                       ycurs, ch_width, ch_height);
                  }
               }
            }
            goto next;
      }
   }
   goto next;

   tidyup:
   XUngrabKeyboard (XDL_display, CurrentTime);
   XUngrabPointer (XDL_display, CurrentTime);
   XDestroyWindow (XDL_display, wid);
   XFreeGC (XDL_display, paint_gc);
   XFlush (XDL_display);
   xdl_copy_chars (instring, reply, max_len);
   free (instring);
   return iret;
}
/* Fortran binding: xdlf_popup_dialog */

void xdlf_popup_dialog (ixroot, iyroot, prompt, lenp, iwidth, reply, maxlen,
                        ifont, ibold, iarrow, ireturn)

int *ixroot, *iyroot, *prompt, *lenp, *iwidth, *reply, *maxlen;
int *ifont, *ibold, *iarrow, *ireturn;

{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ireturn = xdl_popup_dialog (*ixroot, *iyroot, XDL_pointers[*prompt], *lenp,
               *iwidth, XDL_pointers[*reply], max_len, *ifont, *ibold, *iarrow);
   return;
}
/*=======================================================*/
/*             'Private' SERVICE ROUTINES                */
/*=======================================================*/

/*
*****************************************
**     xdl_popup_dialog_drawcursor     **
*****************************************

Purpose: Draw text input cursor

Author:  John W. Campbell, April 1991

*/

void xdl_popup_dialog_drawcursor (wid, gc, x, y, ch_width, ch_height)

Window wid;                /* Window id.*/
GC gc;                     /* Graphics context */
int x;                     /* x position for the cursor */
int y;                     /* y position for the cursor */
int ch_width;              /* Character width */
int ch_height;             /* Character height */

{
   y += ch_height;
   XDrawLine (XDL_display, wid, gc,
   x+1, y-3, x+1, y-1);
   XDrawLine (XDL_display, wid, gc,
   x+1, y-1, x+ch_width-2, y-1);
   XDrawLine (XDL_display, wid, gc,
   x+ch_width-2, y-3, x+ch_width-2, y-1);
   return;
}
/*
*****************************************
**     xdl_popup_dialog_replystr       **
*****************************************

Purpose: Redraw reply string in its current state

Return:  None

Author:  John W. Campbell, November 1991

*/

void xdl_popup_dialog_replystr (wid, gc, xreply, yreply, 
                                yreplytop, reply_width,
                                instring, last_in, start_offset, cur_offset,
                                ycurs, ch_width, ch_height)

Window wid;                /* Window id.*/
GC gc;                     /* Graphics context */
int xreply;                /* x posistion of start of reply */
int yreply;                /* y position of start of reply */
int yreplytop;             /* y position of top of reply area */
int reply_width;           /* Width of reply area (no. of chars) */
char instring[];           /* Input string */
int last_in;               /* Offset of last character in 'instring' */
int start_offset;          /* Offset into instring for first displayed 
                              character */
int cur_offset;            /* Offset of current cursor position relative to
                              start of instring */
int ycurs;                 /* y position ofcursor */
int ch_width;              /* Character width */
int ch_height;             /* Character height */
{
   int i, j;
   int xx;
   int nch;

   XClearArea (XDL_display, wid, xreply, yreplytop,
              (reply_width*ch_width), ch_height, False);
   if (last_in>=0)
   {
      nch = last_in - start_offset + 1;
      if (nch>reply_width) nch = reply_width;
      if (nch>0)
      {   
         xx = xreply;
         for (i=0,j=start_offset;i<nch;++i,++j)
         {
            XDrawString (XDL_display, wid, gc, xx, yreply,
                         &instring[j], 1);
            xx += ch_width;
         }
      }
   }
   xx = xreply + (cur_offset-start_offset)*ch_width;
   xdl_popup_dialog_drawcursor (wid, gc, xx, ycurs, ch_width, ch_height);
   return;
}
