/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Pop-up Frame View-Object Routines
-end*/
 
/*-Intro:
The popup frame view-object is just an empty 'top level' window on which other 
view-objects may be laid out. As it is a popup window, there will be no 
added frame etc. by the window manager. The window will have an external black
border 1 pixel thick and, optionally, an internal coloured border 4 pixels
thick. A popup window cannot be moved or resized and should normally only
be used in an essentially temporary fashion. (Note: Only one such popup
frame must be used at a time; Pointer and Keyboard events are reported only
to the client creating the popup frame while it is displayed)
-end*/
 
/*-Section: Creating and Handling a Pop-up Frame View-Object
-end*/
 

/*Routines List:
The popup frame view-object has the following routines available:

   xdl_popup_frame              Create a new popup frame view-object
*/

/*-Routine: Create a popup frame view-object - xdl_popup_frame
The routine xdl_popup_frame (xdlf_popup_frame) is used to create a new popup
frame view-object. The position, size and border requirements of the frame 
are specified.
-end*/
/*
****************************
**     xdl_popup_frame    **
****************************

Purpose: Create a new popup frame

Author:  John W. Campbell, June 1991

*/
/*-Fortran:
         CALL XDLF_POPUP_FRAME (IVH, IXROOT, IYROOT, IWIDTH, IHEIGHT, 
        +                      IBRDR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
IXROOT     (R)  X (rootwindow) position for frame (top left)
                (see xroot)
IYROOT     (R)  Y (rootwindow) position for frame (top left)
                (see yroot)
IWIDTH     (R)  Frame width (including internal border) 
                (see width)
IHEIGHT    (R)  Frame height (including internal border)
                (see height)
IBRDR      (R)  =0 No internal coloured (just an external 1 
                   pixel width black border)
                =1 Internal Red border (4 pixels wide)
                =2 Internal Yellow border (4 pixels wide)
                =3 Internal Green border (4 pixels wide)
                =4 Internal Cyan border (4 pixels wide)
                =5 Internal Blue border (4 pixels wide)
                   (preferred option if frame used for
                    a temporary menu, parameter table etc.)
                =6 Internal Magenta border (4 pixels wide)
                   (see brd_colr)
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

/* Define routine types */
void xdl_popup_frame();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_popup_frame xdlf_popup_frame_

#endif

#if  LINKTYP == 2

#define xdlf_popup_frame XDLF_POPUP_FRAME

#endif

void xdlf_popup_frame();
/*-C:*/
void xdl_popup_frame (vh, xroot, yroot, width, height, brdr_colr)
/*end*/
 
/*-Parameters:*/
int vh;             /* User selected view-object handle (R)*/
int xroot;          /* x position of the popup frame (wrt Root)
                       (May be adjusted to get frame within display) (R)*/
int yroot;          /* y position of the popup frame (wrt Root)
                       (May be adjusted to get frame within display) (R)*/
int width;          /* Frame width (including any internal border but
                       excluding the external border of width 1 pixel)
                       (R)*/
int height;         /* Frame height (including any internal border but
                       excluding the external border of height 1 pixel)
                       (R)*/
int brdr_colr;      /* =0 No internal coloured (just an external 1 
                          pixel width black border)
                       =1 Internal Red border (4 pixels wide)
                       =2 Internal Yellow border (4 pixels wide)
                       =3 Internal Green border (4 pixels wide)
                       =4 Internal Cyan border (4 pixels wide)
                       =5 Internal Blue border (4 pixels wide)
                          (preferred option if frame used for
                           a temporary menu, parameter table etc.)
                       =6 Internal Magenta border (4 pixels wide) (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   Window wid;                 /* Window id for the frame */
   Window wbrdr;               /* Window id for 'internal border' */
   XSetWindowAttributes attrib; /*Temp window attributes structure */

   if (brdr_colr<0||brdr_colr>6) brdr_colr = 0;
   
   /* Check position and adjust if needed */

   if ((xroot+width+2)>=DisplayWidth (XDL_display,XDL_screen))
   {
      xroot = DisplayWidth (XDL_display,XDL_screen) - width - 2;
   }
   if ((yroot+height+2)>=DisplayHeight (XDL_display,XDL_screen))
   {
      yroot = DisplayHeight (XDL_display,XDL_screen) - height - 2;
   }
   if (xroot<1) xroot = 1;
   if (yroot<1) yroot = 1;

   /* Create popup frame window */

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.override_redirect = True;
   attrib.save_under = True;

   wid = XCreateWindow (XDL_display, 
                RootWindow(XDL_display, XDL_screen), xroot, yroot,
                width, height, 1, XDL_pw_depth, InputOutput, 
                XDL_visual,
                CWBackPixel|CWBorderPixel|CWColormap|
                CWSaveUnder|CWOverrideRedirect,
                &attrib);
 
   XMapWindow (XDL_display, wid);

   XGrabPointer (XDL_display, wid, True, 0, GrabModeAsync, GrabModeAsync,
                 None, None, CurrentTime);
   XGrabKeyboard (XDL_display, wid, True, GrabModeAsync, GrabModeAsync,
                  CurrentTime);

   xdl_new_view_object (vh, wid, 0, NULL, 1002, NULL, NULL);

   /* Add border window if needed */
   
   if (brdr_colr==0) return;

   attrib.background_pixel = XDL_whitepixel;
   switch (brdr_colr)
   {
       case 1:
       attrib.border_pixel = XDL_redpixel;
       break;
       case 2:
       attrib.border_pixel = XDL_yellowpixel;
       break;
       case 3:
       attrib.border_pixel = XDL_greenpixel;
       break;
       case 4:
       attrib.border_pixel = XDL_cyanpixel;
       break;
       case 5:
       attrib.border_pixel = XDL_bluepixel;
       break;
       case 6:
       attrib.border_pixel = XDL_magentapixel;
       break;
   }
   attrib.colormap = XDL_colormap;
   attrib.cursor = XDL_default_cursor;

   wbrdr = XCreateWindow (XDL_display, wid, 0, 0, 
                          width-8, height-8, 4, XDL_pw_depth,
                          InputOutput, XDL_visual,
                          CWBackPixel|CWBorderPixel|CWColormap|CWCursor,
                          &attrib);
   XMapWindow (XDL_display, wbrdr);
   xdl_add_window (vh, wbrdr, NULL, NULL);

   return;
}

/* Fortran binding: xdlf_popup_frame */

void xdlf_popup_frame (ivh, ixroot, iyroot, iwidth, iheight, ibrdr)
int *ivh, *ixroot, *iyroot, *iwidth, *iheight, *ibrdr;
{
   xdl_popup_frame (*ivh, *ixroot, *iyroot, *iwidth, *iheight, *ibrdr);
   return;
}
