/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Pop-up Menu View-Object Routines
-end*/
 
/*-Intro:
The pop-up menu view-object is used to request the immediate input
of an option via a popup menu. The program will only proceed after 
the user has selected an option or cancelled the menu.
-end*/
 
/*-Section: Creating and Handling a Pop-up Menu View-Object
-end*/
 

/*Routines List:
The pop-up menu view-object has the following routines available:

   xdl_popup_menu           Display a pop-up menu
*/

/*-Routine: Display a pop-up menu - xdl_popup_menu
The routine xdl_popup_menu (xdlf_popup_menu) is used to display a
pop-up menu containing a set of user selectable options.
Note that the pop-up menu is rather different from most other view-objects in 
that it is a temporary object and does not have a view-object handle.
-end*/

/*-Doc:
If Button1 or Button3 held down when menu popped up then:

     a) First release of the button on the menu selects item.

     b) First release of the button off the menu leaves menu displayed.

        then 1) Click Button1 or Button3 on the menu to select item.

             2) Click Button1 or Button3 off the menu to dismiss menu.

If Button1 and Button3 are up when the menu is popped up then

    a) Click Button1 or Button3 on the menu to select item.

    b) Click Button1 or Button3 off the menu to dismiss menu.

-end*/
/*
******************************
**     xdl_popup_menu       **
******************************

Purpose: Create a popup menu

Author:  John W. Campbell, April 1991

*/
/*-Fortran:
         CALL XDLF_POPUP_MENU (IXROOT, IYROOT, NITEMS, XDLSTR(STRNGS),
        +                      LENS, IFONT, IBRDR, ICENT, ITEM)
-end*/
 
/*-Parameters:
IXROOT    (R)  X (rootwindow) position for menu (top left) 
               (see xroot)
IYROOT    (R)  Y (rootwindow) position for menu (top left)
               (see yroot)
NITEMS   (R)   Number of menu items in the menu (see num_items)
STRNGS   (R)   Fortran array of character strings containing the
               menu item strings.
               [CHARACTER*LENS STRNGS(NITEMS)].
               ** Pass address using XDLSTR function **
               (see strings)
LENS     (R)   Length of the menu item character strings
               (must be >0 cf len_strs)
IFONT    (R)   Font type (1->5 verysmall->extralarge)
               (see font_type)
IBRDR    (R)   =0, no coloured border added (just a narrow black
                   border will be drawn)
               =1, Red border
               =2, Yellow border
               =3, Green border
               =4, Cyan border
               =5, Blue border (Preferred option when border used)
               =6, Magenta border 
               (see brdr_colr)
ICENT    (R)   =1, centre menu text strings
               =0, do not (strings will be left justified)
               (see cent)
ITEM     (W)   =0, no item selected
               >0, selected menu item number (1 to NITEMS)
               (the return from xdl_popup_menu)
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

/*Define routine types*/
int xdl_popup_menu();
void xdl_popup_menu_drawstr();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_popup_menu xdlf_popup_menu_

#endif

#if  LINKTYP == 2

#define xdlf_popup_menu XDLF_POPUP_MENU

#endif

void xdlf_popup_menu();

/*User callable routine*/

/*-C:*/
int xdl_popup_menu (xroot, yroot, num_items, strings, len_strs, 
                    font_type, brdr_colr, cent)
/*end*/
 
/*-Parameters:*/
int xroot;          /* x position of the menu popup area 
                       (wrt root) (R)*/
int yroot;          /* y position of the menu popup area 
                       (wrt root) (R)*/
int num_items;      /* Number of strings in the menu (R)*/
char * strings;     /* Menu item strings - this may either be an array of
                       pointers to num_items null terminated character
                       strings (len_strs=0) or a character string 
                       num_items*len_strs in length with each string
                       occupying len_strs characters (left justified and
                       padded to the right with blanks if needed). (R)*/
int len_strs;       /* Strings length (0 = array of pointers to
                       null terminated character strings (e.g. for 'C'
                       programs) or the length of each string 
                       (e.g. for Fortran programs). See also the previous
                       item. (R) */
int font_type;      /* Font type (1-5) for very-small, small, medium,
                       large or Extra-large font (R)*/
int brdr_colr;      /* =0, no coloured border added (just a narrow black
                           border will be drawn)
                       =1, Red border
                       =2, Yellow border
                       =3, Green border
                       =4, Cyan border
                       =5, Blue border (Preferred option when border used)
                       =6, Magenta border (R)*/
int cent;           /* =1, centre menu text strings
                       =0, do not (strings will be left justified)
                       (R)*/
/*end*/
/*-Doc:
Return:  = 0,  No item selected
         > 0,  The selected item number
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XFontStruct *fontinfo_bold; /*Temp pointer to bold font structure*/
   XSetWindowAttributes attrib;/*Temp window attributes structure*/
   XEvent ev;                  /*Event return for menu event handling*/
   XEvent xevent;              /*Temp XEvent structure*/
   Window wmenu;               /*Window id for the menu*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   GC paint_gc;                /*Graphics context for window*/
   int area_width;             /*Width of area required*/
   int area_height;            /*Height of area required*/
   int brdr;                   /*Coloured border flag =1 yes, =0 no*/
   int maxlen;                 /*Maximum length of a menu string*/
   int string_type;            /*Strings type =0 'C', =1 'Fortran'*/
   int len;                    /*Length of a character string*/
   int highlight_item;         /*Highlighted item number*/
   int first_up;               /*Indicates first button (1 or 3) event*/
   int xo, yo;                 /*Origin of menu area within all borders*/
   int xmin, xmax;             /*Min and max offsets in x for menu items*/
   Font fid_normal;            /*Normal font*/
   Font fid_bold;              /*Bold font*/
   char **strs;                /*Temp pointer to array of character strings*/
   char *str;                  /*Temp pointer to a character string*/
   int i, j, k;                /*Temp/loop variables*/
   int xstr_off;               /*Offset for character string within menu item
                                 box*/
   int ystr_off;               /*Offset for character string within menu item
                                 box*/
   int str_width;              /*Max string width in pixels*/
   int yoff;                   /*y offset for current item*/
   int item;                   /*Menu item number*/
   int x, y;                   /*Mouse position*/

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (brdr_colr<0||brdr_colr>6) brdr_colr = 0;
   if (len_strs<0) return 0;
   if (num_items<=0) return 0;
   brdr = 0;
   if (brdr_colr>0) brdr=1;

   /* Calculate size requirements */

   fontinfo = XDL_fonts[font_type-1];
   fontinfo_bold = XDL_bold_fonts[font_type-1];
   fid_normal = fontinfo->fid;
   fid_bold = fontinfo_bold->fid;
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);

   if (len_strs==0)
   {
      string_type = 0;
      maxlen = 0;
      strs = (char**) strings;
      for (i=0;i<num_items;++i)
      {
         str = strs[i];
         len = strlen(str);
         if (len>0)
         {
            for (j=len-1,k=len;j>=0;--j,--k)
            {
               if (str[j]!=' ')
               {
                  if (k>maxlen) maxlen = k;
                  break;
               }
            }
         }
      }
   }
   else
   {
      string_type = 1;
      maxlen = 0;
      for (i=0;i<num_items;++i)
      {
         for (j=len_strs*i+len_strs-1,k=len_strs;j>=len_strs*i;--j,--k)
         {
            if (strings[j]!=' ')
            {
               if (k>maxlen) maxlen = k;
               break;
            }
         }
      }
   }
   if (maxlen==0) return 0;

   area_width = maxlen*ch_width + ch_width + 2 + 6*brdr;
   area_height = num_items*(ch_height + 5) + 1 + 6*brdr;

   /* Check position and adjust if needed */

   if ((xroot+area_width+2)>=DisplayWidth (XDL_display,XDL_screen))
   {
      xroot = DisplayWidth (XDL_display,XDL_screen) - area_width - 2;
   }
   if ((yroot+area_height+2)>=DisplayHeight (XDL_display,XDL_screen))
   {
      yroot = DisplayHeight (XDL_display,XDL_screen) - area_height - 2;
   }

   xo = 1;
   yo = 1;
   if (brdr)
   {
      xo = 4;
      yo = 4;
   }
   xmin = xo;
   xmax = xmin + (maxlen+1)*ch_width - 1;
   xstr_off = ch_width/2;
   ystr_off = ascent + 1;
   str_width = maxlen*ch_width;

   /* Create and service Pop-up menu */

   /* Create menu window */
   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.override_redirect = True;
   attrib.save_under = True;

   wmenu = XCreateWindow (XDL_display, 
                   RootWindow(XDL_display, XDL_screen), xroot, yroot,
                   area_width, area_height, 1, XDL_pw_depth, InputOutput, 
                   XDL_visual,
                   CWBackPixel|CWBorderPixel|CWColormap|
                   CWSaveUnder|CWOverrideRedirect,
                   &attrib);
   XSelectInput (XDL_display, wmenu, ExposureMask|StructureNotifyMask);
 
   XMapWindow (XDL_display, wmenu);

   paint_gc = XCreateGC (XDL_display, wmenu, 0, NULL);
   XSetBackground (XDL_display, paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, paint_gc, XDL_blackpixel);

   XGrabPointer (XDL_display, wmenu, False, PointerMotionMask|
                ButtonPressMask|ButtonReleaseMask, 
                GrabModeAsync, GrabModeAsync,
                None, 0, CurrentTime);
   highlight_item = 0;
   first_up = 1;

   /* Handle events for the pop-up menu */
   next:
   XNextEvent (XDL_display, &ev);
   if (ev.xany.window==wmenu)
   {
      switch (ev.type)
      {
         /* Repaint Menu */
         case Expose:
         case ConfigureNotify:
            XClearWindow (XDL_display, wmenu);
            if (brdr)
            {
               if (XDL_color>1)
               {
                  switch (brdr_colr)
                  {
                     case 1:
                     XSetForeground (XDL_display, paint_gc, XDL_redpixel);
                     break;
                     case 2:
                     XSetForeground (XDL_display, paint_gc, XDL_yellowpixel);
                     break;
                     case 3:
                     XSetForeground (XDL_display, paint_gc, XDL_greenpixel);
                     break;
                     case 4:
                     XSetForeground (XDL_display, paint_gc, XDL_cyanpixel);
                     break;
                     case 5:
                     XSetForeground (XDL_display, paint_gc, XDL_bluepixel);
                     break;
                     case 6:
                     XSetForeground (XDL_display, paint_gc, XDL_magentapixel);
                     break;
                  }
               }
               XDrawRectangle (XDL_display, wmenu, paint_gc,
                              0, 0, area_width-1, area_height-1);
               XDrawRectangle (XDL_display, wmenu, paint_gc,
                              1, 1, area_width-3, area_height-3);
               XDrawRectangle (XDL_display, wmenu, paint_gc,
                              2, 2, area_width-5, area_height-5);
               XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
               XDrawRectangle (XDL_display, wmenu, paint_gc,
                              3, 3, area_width-7, area_height-7);
            }
            else
            {
               XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
               XDrawRectangle (XDL_display, wmenu, paint_gc,
                              0, 0, area_width-1, area_height-1);
            }
            if (num_items>1)
            {
               for (i=1;i<num_items;++i)
               {
                  yoff = yo + i*(ch_height + 5) - 1;
                  XDrawLine (XDL_display, wmenu, paint_gc,
                            xmin, yoff, xmax, yoff);
               }
            }
            yoff = yo;
            for (i=1;i<=num_items;++i)
            {
               XClearArea (XDL_display, wmenu, xo+xstr_off, yoff, str_width,
                           ch_height+4, 0);
               XSetFont (XDL_display, paint_gc, fid_normal);
               if (string_type==1)
               {
                  str = &strings[(i-1)*len_strs];
               }
               else
               {
                  str = strs[i-1];
               }
               xdl_popup_menu_drawstr (wmenu, paint_gc, str, maxlen, ch_width, 
                                     xo+xstr_off, yoff+ystr_off, cent);
               yoff += ch_height + 5;
            }
            break;

         /* Select item and/or cancel menu */
         case ButtonRelease:
            if (ev.xbutton.button==Button1||ev.xbutton.button==Button3)
            {
               x = ev.xbutton.x;
               y = ev.xbutton.y;
               item = 0;
               if (x>=xmin&&x<=xmax)
               {
                  if (y>=yo)
                  {
                     item = (y-yo)/(ch_height+5) + 1;
                     if (item>num_items) item = 0;
                  }
               }
               if (item>0||first_up==0) goto tidyup;
               first_up = 0;
            }
            break;

         case ButtonPress:
            if (ev.xbutton.button==Button1||ev.xbutton.button==Button3)
            {
               first_up = 0;
            }
            break;

         /* Menu item highlighting (via bold text) */
         case MotionNotify:
            x = ev.xmotion.x;
            y = ev.xmotion.y;
            while (XCheckTypedWindowEvent (XDL_display, wmenu,
                   MotionNotify, &xevent))
            {
               x = xevent.xmotion.x;
               y = xevent.xmotion.y;
            }
            item = 0;
            if (x>=xmin&&x<=xmax)
            {
               if (y>=yo)
               {
                  item = (y-yo)/(ch_height+5) + 1;
                  if (item>num_items) item = 0;
               }
            }
            if (item==highlight_item) break;
            if (highlight_item>0)
            {
               yoff = yo + (highlight_item-1)*(ch_height+5);
               i = highlight_item;
               XClearArea (XDL_display, wmenu, xo+xstr_off, yoff, str_width,
                           ch_height+4, 0);
               XSetFont (XDL_display, paint_gc, fid_normal);
               if (string_type==1)
               {
                  str = &strings[(i-1)*len_strs];
               }
               else
               {
                  str = strs[i-1];
               }
               xdl_popup_menu_drawstr (wmenu, paint_gc, str, maxlen, ch_width, 
                                     xo+xstr_off, yoff+ystr_off, cent);
            }
            highlight_item = 0;
            if (item==0) break;
            highlight_item = item;
            yoff = yo + (item-1)*(ch_height+5);
            i = item;
            XClearArea (XDL_display, wmenu, xo+xstr_off, yoff, str_width,
                        ch_height+4, 0);
            XSetFont (XDL_display, paint_gc, fid_bold);
            if (string_type==1)
            {
               str = &strings[(i-1)*len_strs];
            }
            else
            {
               str = strs[i-1];
            }
            xdl_popup_menu_drawstr (wmenu, paint_gc, str, maxlen, ch_width, 
                                  xo+xstr_off, yoff+ystr_off, cent);
            break;
         case LeaveNotify:
            if (highlight_item>0)
            {
               yoff = yo + (highlight_item-1)*(ch_height+5);
               i = highlight_item;
               XClearArea (XDL_display, wmenu, xo+xstr_off, yoff, str_width,
                           ch_height+4, 0);
               XSetFont (XDL_display, paint_gc, fid_normal);
               if (string_type==1)
               {
                  str = &strings[(i-1)*len_strs];
               }
               else
               {
                  str = strs[i-1];
               }
               xdl_popup_menu_drawstr (wmenu, paint_gc, str, maxlen, ch_width, 
                                     xo+xstr_off, yoff+ystr_off, cent);
            }
            highlight_item = 0;
            break;
      }
   }
   goto next;

   tidyup:
   XUngrabPointer (XDL_display, CurrentTime);
   XDestroyWindow (XDL_display, wmenu);
   XFreeGC (XDL_display, paint_gc);
   XFlush (XDL_display);
   return item;
}
/* Fortran binding: xdlf_popup_menu */

void xdlf_popup_menu (ixroot, iyroot, nitems, strings, lens, ifont,
                     ibrdr, icent, item)

int *ixroot, *iyroot, *nitems, *strings, *lens;
int *ifont, *ibrdr, *icent, *item;

{
   *item = xdl_popup_menu (*ixroot, *iyroot, *nitems, XDL_pointers[*strings],
                           *lens, *ifont, *ibrdr, *icent);
   return;
}
/*=======================================================*/
/*             'Private' SERVICE ROUTINES                */
/*=======================================================*/

/*
************************************
**     xdl_popup_menu_drawstr     **
************************************

Purpose: Draw a menu string

Return:  None

Author:  John W. Campbell, April 1991

*/

void xdl_popup_menu_drawstr (wmenu, gc, string, maxlen, ch_width,
                             x, y, cent)

Window wmenu;              /* Window id.*/
GC gc;                     /* Graphics context */
char *string;              /* Menu item string */
int maxlen;                /* Maximum string length */
int ch_width;              /* Character width */
int x;                     /* x position for the string */
int y;                     /* y position for the string */
int cent;                  /* =1 centre string, =0 do not */

{
   int i;       /* Temp/loop variable */
   int xx;      /* x position for current character */
   int ll;      /* length of current string */

   if (cent)
   {
      ll = 0;
      for (i=0;i<maxlen;++i)
      {
         if (string[i]==0) break;
         if (string[i]!=' ') ll = i+1;
      }
      if (ll>0)
      {
         xx = x + ((maxlen-ll)*ch_width)/2;
         for (i=0;i<ll;++i)
         {
            if (string[i]!=' ') XDrawString (XDL_display, wmenu, gc, xx, y,
                                             &string[i],1);
            xx += ch_width;
         }
      }
   }
   else
   {
      xx = x;
      for (i=0;i<maxlen;++i)
      {
         if (string[i]==0) break;
         if (string[i]!=' ') XDrawString (XDL_display, wmenu, gc, xx, y,
                                          &string[i],1);
         xx += ch_width;
      }
   }
   return;
}
