/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Pop-up Notice View-Object Routines
-end*/
 
/*-Intro:
The pop-up notice view-object is used to bring important information
to the user's attention. The program will only proceed after the
user has selected one of the buttons displayed within the notice.
It is typically used to indicate error conditions or to request
confirmation for a possibly dangerous choice made by the user. The
notice may contain one or two messages and may have one or two
buttons.
-end*/
 
/*-Section: Create and Handle a Pop-up Notice View-Object
-end*/
 

/*Routines List:
The pop-up notice view-object has the following routines available:

   xdl_popup_notice           Display a pop-up notice
*/

/*-Routine: Display a pop-up notice - xdl_popup_notice
The routine xdl_popup_notice (xdlf_popup_notice) is used to display a
pop-up notice containing one or two messages and wait till its button
(or one of it's two buttons) is 'pressed'.  Note that the pop-up notice
is rather different from other view-objects in that it is a temporary
object and does not have a view-object handle.
-end*/
/*
******************************
**     xdl_popup_notice     **
******************************

Purpose: Create a notice popup box (library routine for
         use within view-objects)

Author:  John W. Campbell, September 1990

*/
/*-Fortran:
         CALL XDLF_POPUP_NOTICE (IXROOT, IYROOT, MSG1, LENMSG1, 
        +                        MSG2, LENMSG2,
        +                        BUTSTR1, LENBUTSTR1, 
        +                        BUTSTR2, LENBUTSTR2,
        +                        IFONT, IARROW, IBUTTON)
-end*/
 
/*-Parameters:
IXROOT    (R)  X (rootwindow) position for notice (top left) 
               (see xroot)
IYROOT    (R)  Y (rootwindow) position for notice (top left)
               (see yroot)
MSG1      (R)  Address of character string containing first
               message.
               ** Pass address using the XDLSTR function **
               (see msg1)
LENMSG1   (R)  Length of the MSG1 character string (>0) 
               (cf lenmsg1)
MSG2      (R)  Address of character string containing second 
               message.
               ** Pass address using the XDLSTR function **
               (see msg2)
LENMSG2   (R)  Length of the MSG2 character string (>0). If 
               there is no second message to output then give 
               a value of -1. (cf lenmsg2)
BUTSTR1   (R)  Address of character string for the first button
               ** Pass address using the XDLSTR function **
               (see butstr1)
LENBUTSTR1 (R) Length of the BUTSTR1 character string (>0) 
               (cf lenbutstr1)
BUTSTR2   (R)  Address of character string for the second button
               ** Pass address using the XDLSTR function **
               (see butstr2)
LENBUTSTR2 (R) Length of the BUTSTR2 character string (>0). If no
               second button is required then give a value of -1.
               (cf lenbutstr2)
IFONT     (R)  Font type (1 to 5, very small to extra large) 
               (see font_type)
IARROW    (R)  =1 display an arrow pointing to the top left corner
               of the notice provided that it can be placed in the
               requested position, =0 do not. (see arrow)
IBUTTON   (W)  Returns the button number selected (1 or 2)
-end*/

#include "xdl_view_ext.h"

/*Define routine types*/
int xdl_popup_notice();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_popup_notice xdlf_popup_notice_

#endif

#if  LINKTYP == 2

#define xdlf_popup_notice XDLF_POPUP_NOTICE

#endif

void xdlf_popup_notice();

/*User callable routine*/

/*-C:*/
int xdl_popup_notice (xroot, yroot, msg1, lenmsg1, msg2, lenmsg2, 
                      butstr1, lenbutstr1, butstr2, lenbutstr2, 
                      font_type, arrow)
/*end*/
 
/*-Parameters:*/
int xroot;          /* x position of the notice popup area 
                       (wrt root) (R)*/
int yroot;          /* y position of the notice popup area 
                       (wrt root) (R)*/
char * msg1;        /* First message (null terminated) string (R)*/
int lenmsg1;        /* Length of message 1. If 0 then the routine will
                       find the length assuming a null terminated 
                       string (R)*/
char * msg2;        /* Second message (null terminated) string - may be 
                       null if no second message required (R)*/
int lenmsg2;        /* Length of message 2. -1 if no second message. 
                       If 0 then the routine will find the length 
                       assuming a null terminated string. (R)*/
char * butstr1;     /* Label string for first button (null terminated) 
                       (R)*/
int lenbutstr1;     /* Length of button 1 string. If 0 then the routine 
                       will find the length assuming a null terminated 
                       string (R)*/
char * butstr2;     /* Label string for second button (null terminated) - 
                       may be null if only one button required (R)*/
int lenbutstr2;     /* Length of button 2 string. -1 if no second button. 
                       If 0 then the routine will find the length assuming 
                       a null terminated string. (R)*/
int font_type;      /* Font type 1 to 5, very-small to extra-large (R)*/
int arrow;          /* =1 draw arrow pointing to origin (provided that
                       notice may be positioned where requested), =0 do
                       not (R)*/
/*end*/
/*-Doc:
Return:  =1 button one from notice presssed, 
         =2 button two from notice pressed
-end*/
{
   XFontStruct *fontinfo;      /*Temp pointer to font structure*/
   XFontStruct *fontinfo_bold; /*Temp pointer to bold font structure*/
   XSetWindowAttributes attrib;/*Temp window attributes structure*/
   XEvent ev;                  /*Event return for notice event handling*/
   Window wid;                 /*Window id*/
   int wbutton1;               /*Width of button 1*/
   int wbutton2;               /*Width of button 2*/
   int ch_width;               /*Character width*/
   int ch_height;              /*Character height*/
   int ascent;                 /*Font ascent*/
   int descent;                /*Font descent*/
   GC paint_gc;                /*Graphics context for window*/
   int nmesg;                  /*No. of messages*/
   int nbuttons;               /*No. of buttons*/
   int area_width;             /*Width of area required*/
   int area_height;            /*Height of area required*/
   int xmsg1;                  /*x for start of message 1 string*/
   int ymsg1;                  /*y for message 1 string*/
   int xmsg2;                  /*x for start of message 2 string*/
   int ymsg2;                  /*y for message 2 string*/
   int xbox1;                  /*x origin of button 1 box*/
   int xbox2;                  /*x origin of button 2 box*/
   int ybox;                   /*y origin for button boxes*/
   int xbutstr1;               /*x for start of button 1 string*/
   int ybutstr1;               /*y for button 1 string*/
   int xbutstr2;               /*x for start of button 2 string*/
   int ybutstr2;               /*y for button 2 string*/
   int button;                 /*Button being pointed to, 0=none*/
   int button_highlight;       /*No. of button being highlighted, 0=none*/
   int i;                      /*Temp/loop variable*/
   int xx;                     /*x offset for current o/p character*/
   int x;                      /*Mouse x position*/
   int y;                      /*Mouse y position*/
   int maxlen;                 /*No. characters in the longer message*/
   int buttw;                  /*Minimum width required for the buttons*/

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Calculate size requirements */

   if (lenmsg1==0&&msg1!=0) lenmsg1 = strlen (msg1);
   if (lenmsg1>0)
   {
      for (i=lenmsg1-1;i>=0;--i)
      {
          lenmsg1 = i + 1;
          if (msg1[i]!=' ') break;
      }
   }
   nmesg = 1;
   if (lenmsg2==0&&msg2!=0) lenmsg2 = strlen (msg2);
   if (lenmsg2>0)
   {
      nmesg = 2;
      for (i=lenmsg2-1;i>=0;--i)
      {
          lenmsg2 = i + 1;
          if (msg2[i]!=' ') break;
      }
   }
   
   if (lenbutstr1==0&&butstr1!=0) lenbutstr1 = strlen (butstr1);
   nbuttons = 1;
   if (lenbutstr1>0)
   {
      for (i=lenbutstr1-1;i>=0;--i)
      {
          lenbutstr1 = i + 1;
          if (butstr1[i]!=' ') break;
      }
   }
   if (lenbutstr2==0&&butstr2!=0) lenbutstr2 = strlen (butstr2);
   if (lenbutstr2>0)
   {
      nbuttons = 2;
      for (i=lenbutstr2-1;i>=0;--i)
      {
          lenbutstr2 = i + 1;
          if (butstr2[i]!=' ') break;
      }
   }
       
   fontinfo = XDL_fonts[font_type-1];
   fontinfo_bold = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);

   maxlen = lenmsg1;
   if (lenmsg2>maxlen) maxlen = lenmsg2;
   area_width = 20 + (maxlen*ch_width);

   wbutton1 = lenbutstr1*ch_width + 12;
   wbutton2 = lenbutstr2*ch_width + 8;
   buttw = 20 + wbutton1;
   if (nbuttons==2) buttw += 10 + wbutton2;

   if (buttw>area_width) area_width = buttw;

   area_height = 44 + 2*ch_height;
   if (nmesg==2) area_height += 6 + ch_height;

   xmsg1 = (area_width - (lenmsg1*ch_width)) / 2;
   ymsg1 = 16 + ascent;
   xmsg2 = (area_width - (lenmsg2*ch_width)) / 2;
   ymsg2 = ymsg1 + 6 + ch_height;
   if (nbuttons==1)
   {
      xbox1 = (area_width - wbutton1) / 2;
      xbutstr1 = xbox1 + 6;
   }
   else
   {
      xbox1 = (area_width - wbutton1 - wbutton2) / 3;
      xbutstr1 = xbox1 + 6;
      xbox2 = xbox1 + wbutton1 + (area_width - xbox1 - wbutton1 - wbutton2) / 2;
      xbutstr2 = xbox2 + 4;
   }
   ybox = 22 + ch_height;
   if (nmesg==2) ybox += 6 + ch_height;
   ybutstr1 = ybox + 6 + ascent;
   ybutstr2 = ybutstr1;

   /* Check position and adjust if needed */

   if ((xroot+area_width+2)>=DisplayWidth (XDL_display,XDL_screen))
   {
      xroot = DisplayWidth (XDL_display,XDL_screen) - area_width - 2;
      arrow = 0;
   }
   if ((yroot+area_height+2)>=DisplayHeight (XDL_display,XDL_screen))
   {
      yroot = DisplayHeight (XDL_display,XDL_screen) - area_height - 2;
      arrow = 0;
   }
   button_highlight = 0;

   /* Create notice window */

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.override_redirect = True;
   attrib.save_under = True;

   wid = XCreateWindow (XDL_display, 
                       RootWindow(XDL_display, XDL_screen), xroot, yroot,
                       area_width, area_height, 1, XDL_pw_depth, InputOutput, 
                       XDL_visual,
                       CWBackPixel|CWBorderPixel|CWColormap|
                       CWSaveUnder|CWOverrideRedirect,
                       &attrib);

   XSelectInput (XDL_display, wid, ExposureMask|StructureNotifyMask|
                 PointerMotionMask|ButtonPressMask);
  
   XMapWindow (XDL_display, wid);

   /* Create and set paint_gc */
   
   paint_gc = XCreateGC (XDL_display, wid, 0, NULL);
   XSetBackground (XDL_display, paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, paint_gc, XDL_blackpixel);


   /* Handle events for the notice window */
   next:
   XWindowEvent (XDL_display, wid, ExposureMask|StructureNotifyMask|
                 PointerMotionMask|ButtonPressMask, &ev);
   if (ev.xany.window==wid)
   {
      switch (ev.type)
      {
         /* Repaint Notice */
         case Expose:
         case ConfigureNotify:

            /* Draw box and arrow */

            XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
            XClearWindow (XDL_display, wid);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           3, 3, area_width-7, area_height-7);
            if (XDL_color>1)
            {
               XSetForeground (XDL_display, paint_gc, XDL_redpixel);
            }
            XDrawRectangle (XDL_display, wid, paint_gc,
                           0, 0, area_width-1, area_height-1);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           1, 1, area_width-3, area_height-3);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           2, 2, area_width-5, area_height-5);
            if (arrow)
            {
               XClearArea (XDL_display, wid, 0, 0, 7, 7, False);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 4, 0);
               XDrawLine (XDL_display, wid, paint_gc, 0, 1, 4, 1);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 0, 4);
               XDrawLine (XDL_display, wid, paint_gc, 1, 0, 1, 4);
               XDrawLine (XDL_display, wid, paint_gc, 0, 0, 9, 9);
               XDrawLine (XDL_display, wid, paint_gc, 4, 3, 9, 8);
               XDrawLine (XDL_display, wid, paint_gc, 3, 4, 8, 9);
            }

            /* Draw message string(s) */

            XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
            XSetFont (XDL_display, paint_gc, fontinfo->fid);
            xx = xmsg1;
            for (i=0;i<lenmsg1;++i)
            {
                XDrawString (XDL_display, wid, paint_gc, xx, ymsg1,
                             &msg1[i], 1);
                xx += ch_width;
            }
            if (nmesg>1)
            {
               xx = xmsg2;
               for (i=0;i<lenmsg2;++i)
               {
                   XDrawString (XDL_display, wid, paint_gc, xx, ymsg2,
                                &msg2[i], 1);
                   xx += ch_width;
               }
            }

            /* Draw buttons */

            button_highlight = 0;
            XSetFont (XDL_display, paint_gc, fontinfo_bold->fid);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           xbox1, ybox, wbutton1-1, ch_height+11);
            XDrawRectangle (XDL_display, wid, paint_gc,
                           xbox1+1, ybox+1, wbutton1-3, ch_height+9);

            xx = xbutstr1;
            for (i=0;i<lenbutstr1;++i)
            {
                XDrawString (XDL_display, wid, paint_gc, xx, ybutstr1,
                             &butstr1[i], 1);
                xx += ch_width;
            }
            if (nbuttons>1)
            {
               XDrawRectangle (XDL_display, wid, paint_gc,
                              xbox2, ybox, wbutton2-1, ch_height+11);
   
               xx = xbutstr2;
               for (i=0;i<lenbutstr2;++i)
               {
                   XDrawString (XDL_display, wid, paint_gc, xx, ybutstr2,
                                &butstr2[i], 1);
                   xx += ch_width;
               }
            }
            break;
         case ButtonPress:
            if (ev.xbutton.button==Button1)
            {
               x = ev.xbutton.x;
               y = ev.xbutton.y;
               button = 0;
               if (y>=ybox&&y<=ybox+ch_height+11)
               {
                  if (x>=xbox1&&x<=xbox1+wbutton1-1) button = 1;
                  if (nbuttons>1)
                  {
                     if (x>=xbox2&&x<=xbox2+wbutton2-1) button = 2;
                  }
               }
               if (button==0) break;
               XDestroyWindow (XDL_display, wid);
               goto tidyup;
            }

         /* Button highlighting/unhighlighting */

         case MotionNotify:
            if (XDL_color<2) break;
            x = ev.xmotion.x;
            y = ev.xmotion.y;
            button = 0;
            if (y>=ybox&&y<=ybox+ch_height+11)
            {
               if (x>=xbox1&&x<=xbox1+wbutton1-1) button = 1;
               if (nbuttons>1)
               {
                  if (x>=xbox2&&x<=xbox2+wbutton2-1) button = 2;
               }
            }
            if (button!=button_highlight)
            {
               if (button_highlight==1)
               {
                  XClearArea (XDL_display, wid, xbox1+2, ybox+2,
                              wbutton1-4, ch_height+8, False);
                  XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
                  XSetFont (XDL_display, paint_gc, fontinfo_bold->fid);
                  xx = xbutstr1;
                  for (i=0;i<lenbutstr1;++i)
                  {
                      XDrawString (XDL_display, wid, paint_gc, xx, ybutstr1,
                                   &butstr1[i], 1);
                      xx += ch_width;
                  }
               }
               else if (button_highlight==2)
               {
                  XClearArea (XDL_display, wid, xbox2+1, ybox+1,
                              wbutton2-2, ch_height+10, False);
                  XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
                  XSetFont (XDL_display, paint_gc, fontinfo_bold->fid);
                  xx = xbutstr2;
                  for (i=0;i<lenbutstr2;++i)
                  {
                      XDrawString (XDL_display, wid, paint_gc, xx, ybutstr2,
                                   &butstr2[i], 1);
                      xx += ch_width;
                  }
               }
            }
            else
            {
               break;
            }
            if (button==0)
            {
               button_highlight = 0;
               break;
            }
            if (button==1)
            {
               button_highlight = 1;
               XSetForeground (XDL_display, paint_gc, XDL_greenpixel);
               XFillRectangle (XDL_display, wid, paint_gc, xbox1+2, ybox+2,
                               wbutton1-4, ch_height+8);
               XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
               XSetFont (XDL_display, paint_gc, fontinfo_bold->fid);
               xx = xbutstr1;
               for (i=0;i<lenbutstr1;++i)
               {
                   XDrawString (XDL_display, wid, paint_gc, xx, ybutstr1,
                                &butstr1[i], 1);
                   xx += ch_width;
               }
            }
            else if (button==2)
            {
               button_highlight = 2;
               XSetForeground (XDL_display, paint_gc, XDL_redpixel);
               XFillRectangle (XDL_display, wid, paint_gc, xbox2+1, ybox+1,
                               wbutton2-2, ch_height+10);
               XSetForeground (XDL_display, paint_gc, XDL_blackpixel);
               XSetFont (XDL_display, paint_gc, fontinfo_bold->fid);
               xx = xbutstr2;
               for (i=0;i<lenbutstr2;++i)
               {
                   XDrawString (XDL_display, wid, paint_gc, xx, ybutstr2,
                                &butstr2[i], 1);
                   xx += ch_width;
               }
            }
            break;
      }
   }
   goto next;

   tidyup:
   XFreeGC (XDL_display, paint_gc);
   return button;
}
/* Fortran binding: xdlf_popup_notice */

void xdlf_popup_notice (ixroot, iyroot, msg1, lenmsg1, msg2, lenmsg2,
                        butstr1, lenbutstr1, butstr2, lenbutstr2,
                        ifont, iarrow, ibutton)

int *ixroot, *iyroot, *msg1, *lenmsg1, *msg2, *lenmsg2;
int *butstr1, *lenbutstr1, *butstr2, *lenbutstr2, *ifont, *iarrow, *ibutton;

{
   *ibutton = xdl_popup_notice (*ixroot, *iyroot, XDL_pointers[*msg1], *lenmsg1,
                 XDL_pointers[*msg2], *lenmsg2, 
                 XDL_pointers[*butstr1], *lenbutstr1,
                 XDL_pointers[*butstr2], *lenbutstr2, *ifont, *iarrow);
   return;
}
