/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Progress Bar View-Object
-end*/
 
/*-Intro:
The progress bar view-object is used to indicate the progress of a
time consuming operation e.g. reading in of a large file or performing
a lengthy calculation. The progress bar is a window containing a title
and the bar itself. The bar starts a an empty rectangle which is gradually
filled in colour (or black on a black and white display) as the operation
proceeds. The program needs to be able to pass the current degree of
progress to the view-object at reasonably frequent intervals for its
use to be effective.
-end*/
 
/*-Section: Create and Handle a Progress Bar View-Object
-end*/
 

/*Routines List:
The progress bar view-object has the following routines available:

   xdl_progress_bar             Create a progress bar view-object
   xdl_progress_bar_value       Set the current value
*/

/*-Routine: Create a progress bar view-object - xdl_progress_bar
The routine xdl_progress_bar (xdlf_progress_bar) is used to create a
progress bar view-object.
-end*/
/*
*****************************
**     xdl_progress_bar    **
*****************************

Purpose: Create a popup progress bar indicator window

Author:  John W. Campbell, April 1990

*/
/*-Fortran:
         CALL XDLF_PROGRESS_BAR (IVH, IXROOT, IYROOT, 
        +                        XDLSTR(TITLE), ITLEN,
        +                        IND_LEN, MAX_VAL, IFONT, ICOLR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
IXROOT     (R)  Root x position for the window (see xroot)
IYROOT     (R)  Root y position for the window (see yroot)
TITLE      (R)  Fortran character string containing the title
                ** Pass address using the XDLSTR function **
                (cf title)
ITITL      (R)  The title length (> 0) (cf title_len)
IND_LEN    (R)  Length of the 'indicator bar' in pixels 
                (see ind_len)
MAX_VAL    (R)  Maximum value indicating completion (see max_val)
IFONT      (R)  Font type 1 to 5 very-small to extra-large (see 
                font_type)
ICOLR      (R)  Colour type 1 to 6 for red, yellow, green, cyan, 
                blue, magenta (see colr)
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

typedef struct
{
   Window wpop;             /* Window id for the popup object */
   Window wpaint;           /* Paint window */
   int max_val;             /* Maximum value */
   char *title;             /* Pointer to copy of title */
   int title_len;           /* Title length */
   GC paint_gc;             /* Paint graphics context */
   int xtitle_off;          /* x offset for start of title */
   int ytitle_off;          /* y offset for title */
   int ind_len;             /* Length of indicator bar (excluding ends) */
   int lfill;               /* Length filled currently */
   int ch_height;           /* Character height */
   int ch_width;            /* Character width */
   int xbar_start;          /* x offset for top left bar pixel */
   int ybar_start;          /* y offset for top left bar pixel */
   unsigned long barpixel;  /* Colour (if available) for bar */
   int exposed;             /* Exposed flag */
}
   XDl_progress_bar;

/* Define routine types */
void xdl_progress_bar();
void xdl_progress_bar_rp();
void xdl_progress_bar_tidy_up();
int xdl_progress_bar_value();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_progress_bar xdlf_progress_bar_
#define xdlf_progress_bar_value xdlf_progress_bar_value_

#endif

#if  LINKTYP == 2

#define xdlf_progress_bar XDLF_PROGRESS_BAR
#define xdlf_progress_bar_value XDLF_PROGRESS_BAR_VALUE

#endif

void xdlf_progress_bar();
void xdlf_progress_bar_value();

/*-C:*/
void xdl_progress_bar (vh, xroot, yroot, title, title_len, ind_len, 
                       max_val, font_type, colr)
/*end*/
 
/*-Parameters:*/
int vh;             /* User selected view-object handle (R)*/
int xroot;          /* x root position for top left of window (may be
                       repositioned if necessary) (R) */
int yroot;          /* y root position for top left of window (may be
                       repositioned if necessary) (R) */
char * title;       /* Title string (R)*/
int title_len;      /* The length of the title string; if the string is
                       null terminated then 0 may be given and the 
                       program will determine the string length (R)*/
int ind_len;        /* Length of the indicator bar in pixels. This will
                       start as an open bar which will be filled up as
                       values indicating progress are supplied (R)*/
int max_val;        /* The maximum value which indicates completion of the 
                       task (R)*/
int font_type;      /* Font type 1 to 5 very-small to extra-large */
int colr;           /* Colour type for filling the indicator bar (if 
                       colour available) 1 to 6 for Red, Yellow, Green, 
                       Cyan, Blue, Magenta (R)*/
/*
     Note: Calls xdl_flush_events to enable initial painting.
*/
/*end*/
/*-Doc:
Return:  None
-end*/
{
   Window wid;                   /* Window id for the frame */
   XFontStruct *fontinfo;        /* Font structure pointer */
   XDl_progress_bar *gd;         /* Pointer to global data area */
   int ch_width;                 /* Character width */
   int ch_height;                /* Character height */
   int width;                    /* Width for the popup window */
   int height;                   /* Height for the popup window */
   XSetWindowAttributes attrib;  /* Window attributes structure */
   char *p;                      /* Temp character pointer */
   int len;                      /* Actual title length */
   XEvent event;                 /* Event */
   int iv;                       /* Index in view objects list */

   /* Initialisations */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   fontinfo = XDL_bold_fonts[font_type-1];
   ch_width = fontinfo->max_bounds.width;
   ch_height = fontinfo->ascent + fontinfo->descent;

   /* Create global data area */

   p = (char *) malloc (sizeof(XDl_progress_bar));
   if (p==0)
   {
      xdl_view_errmsg ("xdl_progress_bar", 
                       "Unable to allocate memory for global data");
      return;
   }
   gd = (XDl_progress_bar *) p;

   /* Make copy of title */

   len = title_len;
   if (len<=0) len = strlen(title);
   gd->title = (char *) malloc (len+1);
   strncpy (gd->title, title, len);
   gd->title[len] = 0;


   /* Calculate Size Requirements */

   width = 18 + len*ch_width + ind_len;
   height = 12 + ch_height;

   /* Reposition if necessary */

   if (xroot+width>=DisplayWidth (XDL_display, XDL_screen))
   {
      xroot = DisplayWidth (XDL_display, XDL_screen) - width;
   }
   if (yroot+height>=DisplayHeight (XDL_display, XDL_screen))
   {
      yroot = DisplayHeight (XDL_display, XDL_screen) - height;
   }

   /* Create Popup */

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.override_redirect = True;
   attrib.save_under = True;
   attrib.cursor = 0;

   gd->wpop = XCreateWindow (XDL_display,
                             RootWindow (XDL_display, XDL_screen),
                             xroot, yroot,
                             width, height, 2, XDL_pw_depth, InputOutput,
                             XDL_visual,
                             CWBackPixel|CWBorderPixel|CWColormap|
                             CWSaveUnder|CWOverrideRedirect|CWCursor,
                             &attrib);

   XMapWindow (XDL_display, gd->wpop);

   xdl_new_view_object (vh, gd->wpop, 0, (char *) gd, 2001, 
                        NULL, xdl_progress_bar_tidy_up);

   gd->wpaint = xdl_create_paint_window (gd->wpop, 0, 0, 0,
                width-4, height-4, 0);


   xdl_add_window (vh, gd->wpaint, xdl_progress_bar_rp, NULL);

   /* Create paint GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wpaint, 0, NULL);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, fontinfo->fid);

   /* Set up global data values */

   gd->title_len = len;
   gd->max_val = max_val;
   gd->xtitle_off = 4;
   gd->ytitle_off = 4 + fontinfo->ascent;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->lfill = 0;
   gd->xbar_start = 8 + len*ch_width;
   gd->ybar_start = 4;
   gd->ind_len = ind_len;
   gd->exposed = 0;
   gd->barpixel = XDL_blackpixel;
   switch (colr)
   {
      case 1:
         gd->barpixel = XDL_redpixel;
         break;
      case 2:
         gd->barpixel = XDL_yellowpixel;
         break;
      case 3:
         gd->barpixel = XDL_greenpixel;
         break;
      case 4:
         gd->barpixel = XDL_cyanpixel;
         break;
      case 5:
         gd->barpixel = XDL_bluepixel;
         break;
      case 6:
         gd->barpixel = XDL_magentapixel;
         break;
   }
   
   if (gd->exposed==0)
   {
       XWindowEvent (XDL_display, gd->wpaint, ExposureMask, &event);
       iv = xdl_getiv(vh);
       xdl_progress_bar_rp (gd->wpaint, &event, iv, (char *) gd, NULL);
   }
}

/* Fortran binding: xdlf_progress_bar */

void xdlf_progress_bar (ivh, ixroot, iyroot, title, itlen, ind_len, 
                        max_val, ifont, icolr)

int *ivh, *ixroot, *iyroot, *title, *itlen, *ind_len, *max_val, *ifont, *icolr;

{
   xdl_progress_bar (*ivh, *ixroot, *iyroot, XDL_pointers[*title], *itlen, 
                     *ind_len, *max_val, *ifont, *icolr);
   return;
}

/*
*********************************
**     xdl_progress_bar_rp     **
*********************************

Purpose: Repaint progress bar indicator window

Author:  John W. Campbell, October 1990

*/

void xdl_progress_bar_rp (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_progress_bar *gd;        /* Pointer to global data area */
   int xx;                      /* x offset for current character */
   int i;                       /* Loop counter */

   gd = (XDl_progress_bar *) global;

   XClearWindow (XDL_display, wid);

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   xx = gd->xtitle_off;
   for (i=0;i<gd->title_len;++i)
   {
      XDrawString (XDL_display, wid, gd->paint_gc, xx, gd->ytitle_off,
                  &(gd->title[i]), 1);
      xx += gd->ch_width;
   }

   XDrawRectangle (XDL_display, wid, gd->paint_gc,
                   gd->xbar_start, gd->ybar_start, 
                   gd->ind_len+1, gd->ch_height + 1);

   if (XDL_color>1)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->barpixel);
   }

   if (gd->lfill>0)
   {
      XFillRectangle (XDL_display, gd->wpaint, gd->paint_gc,
                      gd->xbar_start+1, gd->ybar_start+1,
                      gd->lfill, gd->ch_height);
   }
   gd->exposed = 1;
   return;
}

/*
**************************************
**     xdl_progress_bar_tidy_up     **
**************************************

Purpose: Tidy up prior to deletion

Author:  John W. Campbell, October 1990

*/

void xdl_progress_bar_tidy_up (global)

char * global;

{
   XDl_progress_bar *gd;       /*Pointer to global data */

   gd = (XDl_progress_bar *) global;

   free (gd->title);
   XFreeGC (XDL_display, gd->paint_gc);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set the current value - xdl_progress_bar_value
The routine xdl_progress_bar_value (xdlf_progress_bar_value) is used to
pass the value indicating the current state of progress to the progress
bar view-object. It must be called at reasonably frequent intervals to
be effective. Note that each call also calls xdl_flush_events which
therefore need not be called during a time consuming process monitored
by using the progress bar as described.
-end*/
/*
*************************************
**     xdl_progress_bar_value      **
*************************************

Purpose: Notify progress bar indicator routine of current value

Author:  John W. Campbell, October 1990

*/
/*-Fortran:
         CALL XDLF_PROGRESS_BAR_VALUE (IVH, IVAL, IERR)
-end*/
 
/*-Parameters:
IVH    (R)   View-object handle (see vh)
IVAL   (R)   The current value (see value)
IERR   (W)   The return from the xdl_progress_bar_value call
-end*/

/*-C:*/
int xdl_progress_bar_value (vh, value)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle for the progress bar object (R)*/
int value;       /* The current value indicating the progress (up to the
                    max_val value passed to xdl_progress_bar) (R)*/
/*
   Note: Calls xdl_flush_events on each valid call.
*/
/*end*/
/*-Doc:
Return:  = 0 OK, >0 error
           bit 0 set: View-object handle not found
           bit 1 set: View handle does not match view object;
-end*/
{
   XDl_progress_bar *gd;            /* Pointer to the global data area */
   int lfill_old;                   /* The previous fill position */
   int iv;                          /* Offset in the XDL_view_objects array */

   /* Check validity of the call */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_progress_bar_value",vh); return 1;}
   if (XDL_view_objects[iv]->type != 2001)
   {
      xdl_view_type_errmsg ("xdl_progress_bar_value", vh,
                            "xdl_progress_bar");
      return 2;
   }
   
   gd = (XDl_progress_bar *) XDL_view_objects[iv]->gd;

   lfill_old = gd->lfill;
   if (gd->lfill>gd->max_val) gd->lfill = gd->max_val;
   if (gd->lfill<0) gd->lfill = 0;
   gd->lfill = (value * gd->ind_len) / gd->max_val;

   if (gd->lfill==lfill_old) return 0;
   if (gd->exposed==0) return 0;
   if (XDL_color>1)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->barpixel);
   }
   if (gd->lfill>lfill_old)
   {
      XFillRectangle (XDL_display, gd->wpaint, gd->paint_gc,
                      gd->xbar_start+1, gd->ybar_start+1,
                      gd->lfill, gd->ch_height);
   }
   else
   {
      XClearArea (XDL_display, gd->wpaint,
                  gd->xbar_start+1+gd->lfill, gd->ybar_start+1,
                  lfill_old - gd->lfill, gd->ch_height, False);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdl_progress_bar_value */

void xdlf_progress_bar_value (ivh, ival, ierr)

int *ivh, *ival, *ierr;
{
   *ierr = xdl_progress_bar_value (*ivh, *ival);
   return;
}
