/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Image File Handling Routines
-end*/
 
/*-Intro:
These are a set of routines for reading film image and image plate (i2)
files. The routines are written in C. Fortran interfaces are provided. They
were written, in particular, for use with programs using XDL_VIEW routines.

***OLD ones only here now ****
-end*/
 
/*-Section: Reading and Handling Image Files - Additional Old Routines
-end*/
 

/*Routines List:
The following film file handling routines are available:

   xdl_compress_film           Compress a film image
   xdl_read_film_file          Read and compress a film image file
   xdl_read_film_file_box      Read a section of a film image file
   xdl_compress_i2             Compress an image-plate (i2) image
   xdl_read_i2                 Read and compress an image plate (i2) file
   xdl_read_i2_box             Read a section of an image plate (i2) file
*/

#include "xdl_view_ext.h"
#ifdef VMS
#include <file.h>
#else
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>
#endif

/* Define routine types */
int xdl_compress_film();
int xdl_read_film_file();
int xdl_read_film_file_box();
int xdl_compress_i2();
int xdl_read_i2();
int xdl_read_i2_box();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_compress_film xdlf_compress_film_
#define xdlf_read_film_file xdlf_read_film_file_
#define xdlf_read_film_file_box xdlf_read_film_file_box_
#define xdlf_compress_i2 xdlf_compress_i2_
#define xdlf_read_i2 xdlf_read_i2_
#define xdlf_read_i2_box xdlf_read_i2_box_

#endif

#if  LINKTYP == 2

#define xdlf_compress_film XDLF_COMPRESS_FILM
#define xdlf_read_film_file XDLF_READ_FILM_FILE
#define xdlf_read_film_file_box XDLF_READ_FILM_FILE_BOX
#define xdlf_compress_i2 XDLF_COMPRESS_I2
#define xdlf_read_i2 XDLF_READ_I2
#define xdlf_read_i2_box XDLF_READ_I2_BOX

#endif

void xdlf_compress_film();
void xdlf_read_film_file();
void xdlf_read_film_file_box();
void xdlf_compress_i2();
void xdlf_read_i2();
void xdlf_read_i2_box();

/*-Routine: Compress a film image - xdl_compress_film
The routine xdl_compress_film (xdlf_compress_film) is used to compress a 
film image and return the compressed image data in an array. The image may be 
compressed in the 'x' and 'y' directions. The user may
select the rectangular area of the image to be read. The returned image 
data is in the form required by Daresbury XDL_VIEW routines such as 
xdl_film_image. The full image is assumed to  have yf moving fastest and 
xf slowest and is in unsigned byte format. Pixels may be averaged or
every nxp'th, nyp'th pixel selected.
-end*/
/*
*****************************
**   xdl_compress_film     **
*****************************

Purpose: Compress a film image and return the image data in an array. 
         The image may be compressed in the 'x' and 'y' directions. 
         The user may select the rectangular area of the image to be 
         compressed.read. The returned image data is in the form required 
         by routines such as xdl_film_image. The input image is assumed to  
         have yf moving fastest and xf slowest and is in unsigned byte format.

Author:  John Campbell, September 1991

*/
/*-Fortran:
         CALL XDLF_COMPRESS_FILM (IMG_FULL, NXRASTS, NYRASTS, IOPT,
        +                         IXOPIX, IYOPIX,
        +                         NXPIXELS, NYPIXELS, NXP, NYP, 
        +                         IVH_BAR, IXROOT, IYROOT, 
        +                         IMG, IMGW, IMGH, IERR)
-end*/
 
/*-Parameters:
IMG_FULL (R)   The full image (normally passed as an integer array
               in which the data have been packed) (see img_full)
NXRASTS  (R)   The number of x-rasters in the full image 
               (see nxrasts)
NYRASTS  (R)   The number of y-rasters in the full film image
               (see nyrasts)
IOPT     (R)   =1 average pixels, =2 sample (see iopt)
IXOPIX   (R)   The origin x pixel (see xo_pix)
IYOPIX   (R)   The origin y pixel (see yo_pix)
NXPIXELS (R)   The number of x pixels (see nx_pixels)
NYPIXELS (R)   The number of y pixels (see ny_pixels)
NXP      (R)   Compress NXP x pixels into 1 (see nxp)
NYP      (R)   Compress NYP y pixels into 1 (see nyp)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG      (W)   Array to hold the compressed data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img)
IMGW     (W)   Returns the image width in IMG (see img_width)
IMGH     (W)   Returns the image height in IMG (see img_height)
IERR     (W)   Returns the status from the xdl_compress_film call
-end*/

/*-C:*/
int xdl_compress_film (img_full, nxrasts, nyrasts, iopt, xo_pix, yo_pix, 
                       nx_pixels, ny_pixels, nxp, nyp, 
                       vh_bar, xroot, yroot, 
                       img, img_width, img_height)
/*end*/
 
/*-Parameters:*/
unsigned char img_full[]; /* The full film image nxrasts*nyrasts with
                             y fastest and x slowest (R)*/
int nxrasts;              /* The number of x-rasters in the full image 
                             (R)*/
int nyrasts;              /* The number of y-rasters in the full image 
                             (R)*/
int iopt;                 /* =1, Average pixels nxp*nyp into 1
                             =2, Sample every nxp'th, nyp'th pixel
                             (R)*/
int xo_pix;               /* The x value of the required origin pixel 
                             (pixels numbered from 1 up). If this is less 
                             than 1 it will be reset to 1 and, if it is 
                             greater than the maximum x pixel number 
                             present, it will be set to the maximum x 
                             pixel number (R)*/
int yo_pix;               /* The y value of the required origin pixel 
                             (pixels numbered from 1 up). If this is less 
                             than 1 it will be reset to 1 and, if it is 
                             greater than the maximum y pixel number 
                             present, it will be set to the maximum y 
                             pixel number (R)*/
int nx_pixels;            /* The 'x' width of the image section to be 
                             read. If a value of less than 1 is given it 
                             will be reset to 1. If a value is given 
                             which takes the section beyond the image 
                             then it will be reset to be within bounds 
                             (R)*/
int ny_pixels;            /* The 'y' height of the image section to be 
                             read. If a value of less than 1 is given it 
                             will be reset to 1. If a value is given 
                             which takes the section beyond the image 
                             then it will be reset to be within bounds 
                             (R)*/
int nxp;                  /* The number of pixels in the 'x' direction to 
                             compress to 1 pixel (R)*/
int nyp;                  /* The number of pixels in the 'y' direction to 
                             compress to 1 pixel (R)*/
int vh_bar;               /* View-object handle (>0) for display bar 
                             showing the progress of the compress, 
                             0 if none required (R) */
int xroot;                /* The root x position for the top left of 
                             the display bar area if required (R)*/
int yroot;                /* The root y position for the top left of 
                             the display bar area if required (R)*/
unsigned char img[];      /* Array returning the compressed image data;
                             The pixels will have been compressed in 'x' 
                             and 'y' as requested. The values will be 
                             averaged over nxp*nyp pixels or sampled at 
                             intervals of nxp, nyp. The slower moving 
                             index in the returned data is y going from 
                             high y to low y and the faster moving index 
                             is x going from low x to high x (W)*/
int * img_width;          /* Returns image width as a number of elements. 
                             Fastest moving index in 'img' (= x in full 
                             image). (W)*/
int * img_height;         /* Returns image height as a number of elements. 
                             Slowest moving index in 'img' (= y in full 
                             image). (W)*/

/* Note if nxrasts (in the full image) is not a multiple of nxp then up to
   nxp-1 pixels may be lost from the high end of 'x' and similarly for
   nyrasts and nyp. */
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;
   int ix, iix;
   int iy, iiy;
   int yoff;
   int nx, ny;
   long *ibuf;
   long ival;
   int nx_els, ny_els;
   int compress_xy;
   int nr;
   int j_save;
   int done;

   /* Initialisations */

   if (iopt!=2) iopt = 1;
   if (nxp<1) nxp = 1;
   if (nyp<1) nyp = 1;
   compress_xy = 0;
   if ((nxp>1)||(nyp>1)) compress_xy = 1;
   if (xo_pix<1) xo_pix=1;
   if (yo_pix<1) yo_pix=1;
   if (nx_pixels<1) nx_pixels=1;
   if (ny_pixels<1) ny_pixels=1;

   nx = nxrasts;
   ny = nyrasts;

   /* Further checks on dimensions */

   if (xo_pix>nx) xo_pix = nx;
   if (yo_pix>ny) yo_pix = ny;
   if ((xo_pix+nx_pixels-1)>nx) nx_pixels = nx - xo_pix + 1;
   if ((yo_pix+ny_pixels-1)>ny) ny_pixels = ny - yo_pix + 1;

   /* Allocate buffer space for collecting data */

   if (compress_xy==1&&iopt==1) ibuf = (long *) malloc (ny*sizeof(long));

   /* Compress data */

   nx_els = nx_pixels/nxp;
   *img_width = nx_els;
   ny_els = ny_pixels/nyp;
   *img_height = ny_els;
   nr = nx_els * nxp;  
   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Compress progress", 
                                   0, 120, nr, 2, 5);
   
   if (compress_xy==0)
   {
      /* Simple case, no xy compression required */
      j_save = (xo_pix-1)*ny;
      for (ix=0;ix<nx_pixels;++ix)
      {
         if (vh_bar>0) xdl_progress_bar_value (vh_bar, ix);
         for (i=ny_pixels-1,j=yo_pix-1+j_save;i>=0;--i,++j)
         {
            img[i*nx_pixels+ix] = img_full[j];
         }
         j_save += ny;
      }
   }
   else
   {
      switch (iopt)
      {
         case 1:
            /* Compress image in xy */
            j_save = (xo_pix-1)*ny;
            done=0;
            for (ix=0;ix<nx_els;++ix)
            {
               for (i=0;i<ny_pixels;++i) ibuf[i] = 0;
               for (iix=0;iix<nxp;++iix)
               {
                  if (vh_bar>0) xdl_progress_bar_value (vh_bar, done);
                  done++;
                  for (i=ny_pixels-1,j=yo_pix-1+j_save;i>=0;--i,++j)
                  {
                     ibuf[i] +=  (int) img_full[j];
                  }
                  j_save += ny;
               }
               yoff = 0;
               for (iy=0;iy<ny_els;++iy)
               {
                  ival = 0;
                  for (iiy=0;iiy<nyp;++iiy)
                  {
                     ival += ibuf[yoff];
                     yoff++;
                  }
                  ival = ival / (nxp*nyp);
                  img[iy*nx_els+ix] = (unsigned char) (ival);
               }
            }
            break;

         case 2:

            /* Sample */
            j_save = (xo_pix-1+nxp/2)*ny;
            for (ix=0;ix<nx_els;++ix)
            {
               for (i=ny_els-1,j=yo_pix-1+nyp/2+j_save;i>=0;i--,j+=nyp)
               {
                  img[i*nx_els+ix] = img_full[j];
               }
               if (vh_bar>0) xdl_progress_bar_value (vh_bar, (ix+1)*nxp);
               j_save += nxp*ny;
            }
            break;
      }
   }
      
   /* Tidy up */

   if (compress_xy==1&&iopt==1) free (ibuf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return 0;
}

/* Fortran binding: xdlf_compress_film */

void xdlf_compress_film  (img_full, nxrasts, nyrasts, iopt, ixopix, iyopix, 
                          nxpixels, nypixels, nxp, nyp, ivh_bar, 
                          ixroot, iyroot,img, imgw, imgh, ierr)
int *img_full, *nxrasts, *nyrasts, *iopt, *ixopix, *iyopix;
int *nxpixels, *nypixels;
int *nxp, *nyp, *ivh_bar, *ixroot, *iyroot, *img, *imgw, *imgh, *ierr;
{
   *ierr = xdl_compress_film  (img_full, *nxrasts, *nyrasts, *iopt, 
                               *ixopix, *iyopix,
                               *nxpixels, *nypixels, *nxp, *nyp,
                               *ivh_bar, *ixroot, *iyroot, img, imgw, imgh);
   return;
}

/*-Routine: Read a film image file - xdl_read_film_file
The routine xdl_read_film_file (xdlf_read_film_file) is used to read a 
film image file and return the image data in an array. The image may be 
compressed in the 'x' and 'y' directions. The file must be opened before 
this routine is called and it is not closed by this routine. The user may
select the rectangular area of the image to be read. The returned image 
data is in the form required by Daresbury XDL_VIEW routines such as 
xdl_film_image. The film_file may have the data in any of the eight possible
orders (the default is assumed to  have yf moving fastest and 
xf slowest) and is in byte format without any header. The number of x-rasters
and the number of y-rasters must be supplied.
-end*/
/*
*****************************
**   xdl_read_film_file    **
*****************************

Purpose: Read a film image file and return the image data in an array. 
         The image may be compressed in the 'x' and 'y' directions. 
         The file must be opened before this routine
         is called and it is not closed by this routine. The user may
         select the rectangular area of the image to be read. The returned
         image data is in the form required by Daresbury X-windows routines 
         such as xdl_film_image. The film_file can have any of the eight
         possible orders (the default order is assumed to  have yf
         moving fastest and xf slowest) and is in byte format without any
         header. The number of x-rasters and the number of y-rasters
         must be supplied.

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_READ_FILM_FILE (IFD, IORD, NXRASTS, NYRASTS, 
        +                         IXOPIX, IYOPIX,
        +                         NXPIXELS, NYPIXELS, NXP, NYP, 
        +                         IVH_BAR, IXROOT, IYROOT, 
        +                         IMG, IMGW, IMGH, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film image file 
               (see order)
NXRASTS  (R)   The number of x-rasters (see nxrasts)
NYRASTS  (R)   The number of y-rasters (see nyrasts)
IXOPIX   (R)   The origin x pixel (see xo_pix)
IYOPIX   (R)   The origin y pixel (see yo_pix)
NXPIXELS (R)   The number of x pixels (see nx_pixels)
NYPIXELS (R)   The number of y pixels (see ny_pixels)
NXP      (R)   Compress NXP x pixels into 1 (see nxp)
NYP      (R)   Compress NYP y pixels into 1 (see nyp)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG      (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img)
IMGW     (W)   Returns the image width in IMG (see img_width)
IMGH     (W)   Returns the image height in IMG (see img_height)
IERR     (W)   Returns the status from the xdl_read_film_file call
-end*/

/*-C:*/
int xdl_read_film_file (fd, order, nxrasts, nyrasts, xo_pix, yo_pix, 
                        nx_pixels, ny_pixels, nxp, nyp, 
                        vh_bar, xroot, yroot, 
                        img, img_width, img_height)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8.
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int nxrasts;      /* The number of x-rasters (R)*/
int nyrasts;      /* The number of y-rasters (R)*/
int xo_pix;       /* The x value of the required origin pixel (pixels 
                     numbered from 1 up). If this is less than 1 it 
                     will be reset to 1 and, if it is greater than the 
                     maximum x pixel number present, it will be set to 
                     the maximum x pixel number (R)*/
int yo_pix;       /* The y value of the required origin pixel (pixels 
                     numbered from 1 up). If this is less than 1 it 
                     will be reset to 1 and, if it is greater than the 
                     maximum y pixel number present, it will be set to 
                     the maximum y pixel number (R)*/
int nx_pixels;    /* The 'x' width of the image section to be read. 
                     If a value of less than 1 is given it will be 
                     reset to 1. If a value is given which takes the 
                     section beyond the image then it will be reset to 
                     be within bounds (R)*/
int ny_pixels;    /* The 'y' height of the image section to be read. 
                     If a value of less than 1 is given it will be 
                     reset to 1. If a value is given which takes the 
                     section beyond the image then it will be reset to 
                     be within bounds (R)*/
int nxp;          /* The number of pixels in the 'x' direction to 
                     compress to 1 pixel (R)*/
int nyp;          /* The number of pixels in the 'y' direction to 
                     compress to 1 pixel (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned char img[];
                  /* Array returning the read image data; The pixels
                     will have been compressed in 'x' and 'y' as 
                     requested. The values will be averaged over 
                     nxp*nyp pixels. The slower moving index in
                     the returned data is y going from high y to 
                     low y and the faster moving index is x going 
                     from low x to high x (W)*/
int * img_width;  /* Returns image width as a number of elements. 
                     Fastest moving index in 'img' (= x in film file). 
                     (W)*/
int * img_height; /* Returns image height as a number of elements. 
                     Slowest moving index in 'img' (= y in film file). 
                     (W)*/

/* Note if nx (in the film file) is not a multiple of nxp then up to
   nxp-1 pixels may be lost from the high end of 'x' and similarly for
   ny and nyp. */
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred, -1 cannot
         allocate temporary buffer(s)
-end*/
{
   int i,j;
   int ix, iix;
   int iy, iiy;
   int xoff, yoff;
   int nx, ny;
   unsigned char *buf;
   long *ibuf;
   long ival;
   int nx_els, ny_els;
   int irec;
   int ioff;
   int nr;
   int rec_step;

   /* Initialisations */

   if (order<1||order>8) order = 1;
   if (nxp<1) nxp = 1;
   if (nyp<1) nyp = 1;
   if (xo_pix<1) xo_pix=1;
   if (yo_pix<1) yo_pix=1;
   if (nx_pixels<1) nx_pixels=1;
   if (ny_pixels<1) ny_pixels=1;

   nx = nxrasts;
   ny = nyrasts;

   /* Further checks on dimensions */

   if (xo_pix>nx) xo_pix = nx;
   if (yo_pix>ny) yo_pix = ny;
   if ((xo_pix+nx_pixels-1)>nx) nx_pixels = nx - xo_pix + 1;
   if ((yo_pix+ny_pixels-1)>ny) ny_pixels = ny - yo_pix + 1;

   /* Read data */

   nx_els = nx_pixels/nxp;
   *img_width = nx_els;
   ny_els = ny_pixels/nyp;
   *img_height = ny_els;

   /* Compress image in xy */

   if (order<5) /*xf slow, yf fast*/
   {
      nr = nx_els * nxp;  
      if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nr, 2, 5);
      buf = (unsigned char *) malloc (ny);
      if (buf==0) return -1;
      ibuf = (long *) malloc (ny*sizeof(long));
      if (ibuf==0) return -1;
      if (order==1||order==2)
      {
         irec = xo_pix;
         rec_step = 1;
      }
      else
      {
         irec = nxrasts + 1 - xo_pix;
         rec_step = -1;
      }
      for (ix=0;ix<nx_els;++ix)
      {
         for (i=0;i<ny_pixels;++i) ibuf[i] = 0;
         for (iix=0;iix<nxp;++iix)
         {
            i = lseek(fd, (long) (irec-1)*nyrasts,0);
            i = read (fd, buf, nyrasts);
            if (i!=nyrasts) goto tidy;
            if (vh_bar>0) xdl_progress_bar_value (vh_bar, ix*nxp+iix+1);
            if (order==1||order==3)
            {
               for (i=ny_pixels-1,j=yo_pix-1;i>=0;--i,++j)
               {
                  ibuf[i] +=  (long) buf[j];
               }
            }
            else
            {
               for (i=0,j=nyrasts+1-ny_pixels-yo_pix;i<ny_pixels;++i,++j)
               {
                  ibuf[i] += (long) buf[j];
               }
            }
            irec += rec_step;
         }
         yoff = 0;
         for (iy=0;iy<ny_els;++iy)
         {
            ival = 0;
            for (iiy=0;iiy<nyp;++iiy)
            {
               ival += ibuf[yoff];
               yoff++;
            }
            ival = ival / (nxp*nyp);
            img[iy*nx_els+ix] = (unsigned char) (ival);
         }
      }
   }
   else  /*yf slow, xf fast*/
   {
      nr = ny_els * nyp;  
      if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nr, 2, 5);
      buf = (unsigned char *) malloc (nx);
      if (buf==0) return -1;
      ibuf = (long *) malloc (nx*sizeof(long));
      if (ibuf==0) return -1;
      ioff = 0;
      if (order==5||order==6)
      {
         irec = yo_pix + ny_pixels - 1;
         rec_step = -1;
      }
      else
      {
         irec = nyrasts + 2 - ny_pixels - yo_pix;
         rec_step = 1;
      }
      for (iy=0;iy<ny_els;++iy)
      {
         for (i=0;i<nx_pixels;++i) ibuf[i] = 0;
         for (iiy=0;iiy<nyp;++iiy)
         {
            i = lseek(fd, (long) (irec-1)*nxrasts,0);
            i = read (fd, buf, nxrasts);
            if (i!=nxrasts) goto tidy;
            if (vh_bar>0) xdl_progress_bar_value (vh_bar, iy*nyp+iiy+1);
            if (order==5||order==7)
            {
               for (i=0,j=xo_pix-1;i<nx_pixels;++i,++j)
               {
                  ibuf[i] +=  (long) buf[j];
               }
            }
            else
            {
               for (i=0,j=nxrasts-xo_pix;i<nx_pixels;++i,--j)
               {
                  ibuf[i] += (long) buf[j];
               }
            }
            irec += rec_step;
         }
         xoff = 0;
         for (ix=0;ix<nx_els;++ix)
         {
            ival = 0;
            for (iix=0;iix<nxp;++iix)
            {
               ival += ibuf[xoff];
               xoff++;
            }
            ival = ival / (nxp*nyp);
            img[ioff] = (unsigned char) (ival);
            ioff++;
         }
      }
   }
   irec = 0;
      
   /* Tidy up */

   tidy:
   free (buf);
   free (ibuf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_film_file */

void xdlf_read_film_file (ifd, iord, nxrasts, nyrasts, ixopix, iyopix, 
                          nxpixels, nypixels, nxp, nyp, ivh_bar, 
                          ixroot, iyroot,img, imgw, imgh, ierr)
int *ifd, *iord, *nxrasts, *nyrasts, *ixopix, *iyopix, *nxpixels, *nypixels;
int *nxp, *nyp, *ivh_bar, *ixroot, *iyroot, *img, *imgw, *imgh, *ierr;
{
   *ierr = xdl_read_film_file (*ifd, *iord, *nxrasts, *nyrasts,
                               *ixopix, *iyopix,
                               *nxpixels, *nypixels, *nxp, *nyp,
                               *ivh_bar, *ixroot, *iyroot, img, imgw, imgh);
   return;
}

/*-Routine: Read a section of a film file image - xdl_read_film_file_box
The routine xdl_read_film_file_box (xdlf_read_film_file_box) is used to
read a rectangular section from a film image file and return the data in an 
integer array. The file must be opened before this routine is called and it 
is not closed by this routine. The film file may have its data in any
of the eight possible orders (the default is assumed to  have yf moving 
fastest and xf slowest).
-end*/
/*
*********************************
**   xdl_read_film_file_box    **
*********************************

Purpose: Read a rectangular section from a film image file and return the 
         data in an integer array. 
         The file must be opened before this routine is called and it is not 
         closed by this routine. 
         The film_file may have its data in any of the eight possible
         orders (the default is assumed to  have yf moving fastest and xf 
         slowest).

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_READ_FILM_FILE_BOX (IFD, IORD, IXC, IYC, ISIZX, ISIZY, 
        +                              NXRAST, NYRAST, IDATA, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film file (1-8)
               (see order)
IXC      (R)   The centre x pixel (see xc)
IYC      (R)   The centre y pixel (see yc)
IXSIZ    (R)   The box 'half' size in x (actual size 2*IXSIZ+1)
               (see xsiz)
IYSIZ    (R)   The box 'half' size in y (actual size 2*IYSIZ+1)
               (see ysiz)
NXRAST   (R)   The number of x pixels (see nxrast)
NYRAST   (R)   The number of y pixels (see nyrast)
IDATA    (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see idata)
IERR     (W)   Returns the status from the xdl_read_film_file_box
               call
-end*/

/*-C:*/
int xdl_read_film_file_box (fd, order, xc, yc, xsiz, ysiz, 
                            nxrast, nyrast, idata)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened film format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8.
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int xc;           /* The box centre (x) pixel (R)*/
int yc;           /* The box centre (y) pixel (R)*/
int xsiz;         /* The box 'half' x size (actual box size = 2*xsiz+1) 
                     (R)*/
int ysiz;         /* The box 'half' y size (actual box size = 2*ysiz+1) 
                     (R)*/
int nxrast;       /* The total number of x pixels in the image (R)*/
int nyrast;       /* The total number of y pixels in the image (R)*/
int idata[];      /* Integer array returning the read image data with one
                     pixel value per integer. Out of limits pixels will
                     be returned with zero values (Fastest moving y) (W)*/
/*end*/
/*-Doc:
Return:  =  0 OK; 
         = -1 Error reading file
         = -2 Cannot allocate memory for file reading buffer
         =  1 Partly out of limits
         =  2 Completely out of limits
-end*/
{
   unsigned char *buf;      /* Buffer for reading in record */
   int ierr;                /* Error flag */
   int xstart;              /* Start of x range */
   int xend;                /* End of x range */
   int ystart;              /* Start of y range */
   int yend;                /* End of y range */
   int ix;                  /* Loop counter over x rasters */
   int iy;                  /* Loop counter over y rasters */
   int ii;                  /* Index into 'idata' array */
   int i;                   /* Temp variable */
   int xrec;                /* Record number for x (slow x) */
   int yrec;                /* Record number for y (slow y) */
   int xoff;                /* Offset into record buffer (slow y) */
   int yoff;                /* Offset into record buffer (slow x) */

   /* Initialisations */

   if (order<1||order>8) order = 1;
   ierr = 0;
   xstart = xc - xsiz;
   xend = xc + xsiz;
   ystart = yc - ysiz;
   yend = yc + ysiz;

   if (order<5)
   {
      /* Allocate buffer space for file reads and collecting data */

      buf = (unsigned char *) malloc (nyrast);
      if (buf==0) return -2;

      /* Read data */

      ii = 0;
      for (ix=xstart;ix<=xend;++ix)
      {
         xrec = ix;
         if (order==3||order==4) xrec = nxrast + 1 - xrec;
         if (xrec<1||xrec>nxrast)
         {
            ierr = 1;
            for (iy=ystart;iy<=yend;++iy)
            {
               idata[ii] = 0;
               ii++;
            }
         }
         else
         {
            lseek (fd, (long) (xrec-1)*nyrast, 0);
            i = read (fd, buf, nyrast);
            if (i!=nyrast) 
            {
               ierr = -2;
               goto tidy;
            }
            for (iy=ystart;iy<=yend;++iy)
            {
               if (iy<1||iy>nyrast)
               {
                  ierr = 1;
                  idata[ii] = 0;
               }
               else
               {
                  yoff = iy - 1;
                  if (order==2||order==4) yoff = nyrast - 1 - yoff;
                  idata[ii] = (int) buf[yoff];
               }
               ii++;
            }
         }
      }
   }
   else
   {
      /* Allocate buffer space for file reads and collecting data */

      buf = (unsigned char *) malloc (nxrast);
      if (buf==0) return -2;

      /* Read data */

      for (iy=ystart;iy<=yend;++iy)
      {
         yrec = iy;
         if (order==7||order==8) yrec = nyrast + 1 - yrec;
         if (yrec<1||yrec>nyrast)
         {
            ierr = 1;
            ii = iy - ystart;
            for (ix=xstart;ix<=xend;++ix)
            {
               idata[ii] = 0;
               ii += nyrast;
            }
         }
         else
         {
            lseek (fd, (long) (yrec-1)*nxrast, 0);
            i = read (fd, buf, nxrast);
            if (i!=nxrast) 
            {
               ierr = -2;
               goto tidy;
            }
            for (ix=xstart;ix<=xend;++ix)
            {
               if (ix<1||ix>nyrast)
               {
                  ierr = 1;
                  idata[ii] = 0;
               }
               else
               {
                  xoff = ix - 1;
                  if (order==6||order==8) xoff = nxrast - 1 - xoff;
                  idata[ii] = (int) buf[xoff];
               }
               ii += nyrast;
            }
         }
      }
   }
   if (xend<1||yend<1||xstart>nxrast||ystart>nyrast) ierr = 2;
      
   /* Tidy up */

   tidy:
   free (buf);
   return ierr;
}

/* Fortran binding: xdlf_read_film_file_box */

void xdlf_read_film_file_box (ifd, iord, ixc, iyc, ixsiz, iysiz, nxrast, nyrast,
                              idata, ierr)
int *ifd, *iord, *ixc, *iyc, *ixsiz, *iysiz, *nxrast, *nyrast, *idata, *ierr;
{
   *ierr = xdl_read_film_file_box (*ifd, *iord, *ixc, *iyc, *ixsiz, *iysiz,
                                   *nxrast, *nyrast, idata);
   return;
}
/*-Routine: Compress an image-plate image - xdl_compress_i2
The routine xdl_compress_i2 (xdlf_compress_i2) is used to compress an
image-plate (i2) image and return the compressed image data in an array. 
The image may be compressed in the 'x' and 'y' directions. The user may
select the rectangular area of the image to be read. The returned image 
data is in the form required by Daresbury XDL_VIEW routines such as 
xdl_film_image. The full image is assumed to  have yf moving fastest and 
xf slowest and is in unsigned two byte integer format. Pixels may be 
averaged or every nxp'th, nyp'th pixel selected.
-end*/
/*
*****************************
**   xdl_compress_i2       **
*****************************

Purpose: Compress an image-plate 'i2' image and return the image data in an 
         array. The image may be compressed in the 'x' and 'y' directions. 
         The user may select the rectangular area of the image to be 
         compressed.read. The returned image data is in the form required 
         by routines such as xdl_film_image. The input image is assumed to  
         have yf moving fastest and xf slowest and is in unsigned two byte
         integer format.

Author:  John Campbell, September 1991

*/
/*-Fortran:
         CALL XDLF_COMPRESS_I2 (IMG_FULL, NXRASTS, NYRASTS, IOPT,
        +                         IXOPIX, IYOPIX,
        +                         NXPIXELS, NYPIXELS, NXP, NYP, 
        +                         IVH_BAR, IXROOT, IYROOT, 
        +                         IMG, IMGW, IMGH, IERR)
-end*/
 
/*-Parameters:
IMG_FULL (R)   The full image (normally passed as an integer array
               in which the data have been packed) (see img_full)
NXRASTS  (R)   The number of x-rasters in the full image 
               (see nxrasts)
NYRASTS  (R)   The number of y-rasters in the full film image
               (see nyrasts)
IOPT     (R)   =1 average pixels, =2 sample (see iopt)
IXOPIX   (R)   The origin x pixel (see xo_pix)
IYOPIX   (R)   The origin y pixel (see yo_pix)
NXPIXELS (R)   The number of x pixels (see nx_pixels)
NYPIXELS (R)   The number of y pixels (see ny_pixels)
NXP      (R)   Compress NXP x pixels into 1 (see nxp)
NYP      (R)   Compress NYP y pixels into 1 (see nyp)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG      (W)   Array to hold the compressed data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img)
IMGW     (W)   Returns the image width in IMG (see img_width)
IMGH     (W)   Returns the image height in IMG (see img_height)
IERR     (W)   Returns the status from the xdl_compress_i2 call
-end*/

/*-C:*/
int xdl_compress_i2 (img_full, nxrasts, nyrasts, iopt, xo_pix, yo_pix, 
                       nx_pixels, ny_pixels, nxp, nyp, 
                       vh_bar, xroot, yroot, 
                       img, img_width, img_height)
/*end*/
 
/*-Parameters:*/
unsigned short img_full[]; /* The full image-plate (i2) image 
                              nxrasts*nyrasts with y fastest and 
                              x slowest (R)*/
int nxrasts;              /* The number of x-rasters in the full image 
                             (R)*/
int nyrasts;              /* The number of y-rasters in the full image 
                             (R)*/
int iopt;                 /* =1, Average pixels nxp*nyp into 1
                             =2, Sample every nxp'th, nyp'th pixel
                             (R)*/
int xo_pix;               /* The x value of the required origin pixel 
                             (pixels numbered from 1 up). If this is 
                             less than 1 it will be reset to 1 and, if 
                             it is greater than the maximum x pixel 
                             number present, it will be set to the 
                             maximum x pixel number (R)*/
int yo_pix;               /* The y value of the required origin pixel 
                             (pixels numbered from 1 up). If this is 
                             less than 1 it will be reset to 1 and, if 
                             it is greater than the maximum y pixel 
                             number present, it will be set to the 
                             maximum y pixel number (R)*/
int nx_pixels;            /* The 'x' width of the image section to be 
                             read. If a value of less than 1 is given 
                             it will be reset to 1. If a value is given 
                             which takes the section beyond the image 
                             then it will be reset to be within bounds 
                             (R)*/
int ny_pixels;            /* The 'y' height of the image section to be 
                             read. If a value of less than 1 is given it 
                             will be reset to 1. If a value is given 
                             which takes the section beyond the image 
                             then it will be reset to be within bounds 
                             (R)*/
int nxp;                  /* The number of pixels in the 'x' direction to 
                             compress to 1 pixel (R)*/
int nyp;                  /* The number of pixels in the 'y' direction to 
                             compress to 1 pixel (R)*/
int vh_bar;               /* View-object handle (>0) for display bar 
                             showing the progress of the compress, 
                             0 if none required (R) */
int xroot;                /* The root x position for the top left of the 
                             display bar area if required (R)*/
int yroot;                /* The root y position for the top left of the 
                             display bar area if required (R)*/
unsigned short img[];     /* Array returning the compressed image data; 
                             The pixels will have been compressed in 
                             'x' and 'y' as requested. The values will 
                             be averaged over nxp*nyp pixels or sampled 
                             at intervals of nxp, nyp. The slower moving 
                             index in the returned data is y going from 
                             high y to low y and the faster moving index 
                             is x going from low x to high x (W)*/
int * img_width;          /* Returns image width as a number of elements. 
                             Fastest moving index in 'img' 
                             (= x in full image). (W)*/
int * img_height;         /* Returns image height as a number of elements. 
                             Slowest moving index in 'img' 
                             (= y in full image). (W)*/

/* Note if nxrasts (in the full image) is not a multiple of nxp then up to
   nxp-1 pixels may be lost from the high end of 'x' and similarly for
   nyrasts and nyp. */
/*end*/
/*-Doc:
Return:  =0 OK; 
-end*/
{
   int i,j;
   int ix, iix;
   int iy, iiy;
   int yoff;
   int nx, ny;
   long *ibuf;
   long ival;
   int nx_els, ny_els;
   int compress_xy;
   int nr;
   int j_save;
   int done;

   /* Initialisations */

   if (iopt!=2) iopt = 1;
   if (nxp<1) nxp = 1;
   if (nyp<1) nyp = 1;
   compress_xy = 0;
   if ((nxp>1)||(nyp>1)) compress_xy = 1;
   if (xo_pix<1) xo_pix=1;
   if (yo_pix<1) yo_pix=1;
   if (nx_pixels<1) nx_pixels=1;
   if (ny_pixels<1) ny_pixels=1;

   nx = nxrasts;
   ny = nyrasts;

   /* Further checks on dimensions */

   if (xo_pix>nx) xo_pix = nx;
   if (yo_pix>ny) yo_pix = ny;
   if ((xo_pix+nx_pixels-1)>nx) nx_pixels = nx - xo_pix + 1;
   if ((yo_pix+ny_pixels-1)>ny) ny_pixels = ny - yo_pix + 1;

   /* Allocate buffer space for collecting data */

   if (compress_xy==1&&iopt==1) ibuf = (long *) malloc (ny*sizeof(long));

   /* Compress data */

   nx_els = nx_pixels/nxp;
   *img_width = nx_els;
   ny_els = ny_pixels/nyp;
   *img_height = ny_els;
   nr = nx_els * nxp;  
   if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Compress progress", 
                                   0, 120, nr, 2, 5);
   
   if (compress_xy==0)
   {
      /* Simple case, no xy compression required */
      j_save = (xo_pix-1)*ny;
      for (ix=0;ix<nx_pixels;++ix)
      {
         if (vh_bar>0) xdl_progress_bar_value (vh_bar, ix);
         for (i=ny_pixels-1,j=yo_pix-1+j_save;i>=0;--i,++j)
         {
            img[i*nx_pixels+ix] = img_full[j];
         }
         j_save += ny;
      }
   }
   else
   {
      switch (iopt)
      {
         case 1:
            /* Compress image in xy */
            j_save = (xo_pix-1)*ny;
            done=0;
            for (ix=0;ix<nx_els;++ix)
            {
               for (i=0;i<ny_pixels;++i) ibuf[i] = 0;
               for (iix=0;iix<nxp;++iix)
               {
                  if (vh_bar>0) xdl_progress_bar_value (vh_bar, done);
                  done++;
                  for (i=ny_pixels-1,j=yo_pix-1+j_save;i>=0;--i,++j)
                  {
                     ibuf[i] +=  (int) img_full[j];
                  }
                  j_save += ny;
               }
               yoff = 0;
               for (iy=0;iy<ny_els;++iy)
               {
                  ival = 0;
                  for (iiy=0;iiy<nyp;++iiy)
                  {
                     ival += ibuf[yoff];
                     yoff++;
                  }
                  ival = ival / (nxp*nyp);
                  img[iy*nx_els+ix] = (unsigned short) (ival);
               }
            }
            break;

         case 2:

            /* Sample */
            j_save = (xo_pix-1+nxp/2)*ny;
            for (ix=0;ix<nx_els;++ix)
            {
               for (i=ny_els-1,j=yo_pix-1+nyp/2+j_save;i>=0;i--,j+=nyp)
               {
                  img[i*nx_els+ix] = img_full[j];
               }
               if (vh_bar>0) xdl_progress_bar_value (vh_bar, (ix+1)*nxp);
               j_save += nxp*ny;
            }
            break;
      }
   }
      
   /* Tidy up */

   if (compress_xy==1&&iopt==1) free (ibuf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return 0;
}

/* Fortran binding: xdlf_compress_i2 */

void xdlf_compress_i2  (img_full, nxrasts, nyrasts, iopt, ixopix, iyopix, 
                        nxpixels, nypixels, nxp, nyp, ivh_bar, 
                        ixroot, iyroot,img, imgw, imgh, ierr)
int *img_full, *nxrasts, *nyrasts, *iopt, *ixopix, *iyopix;
int *nxpixels, *nypixels;
int *nxp, *nyp, *ivh_bar, *ixroot, *iyroot, *img, *imgw, *imgh, *ierr;
{
   *ierr = xdl_compress_i2  (img_full, *nxrasts, *nyrasts, *iopt, 
                             *ixopix, *iyopix,
                             *nxpixels, *nypixels, *nxp, *nyp,
                             *ivh_bar, *ixroot, *iyroot, img, imgw, imgh);
   return;
}

/*-Routine: Read an image plate (i2) file - xdl_read_i2
The routine xdl_read_i2 (xdlf_read_i2) is used to read an 'i2' unsigned 16 bit 
integer format image file and return the image data in an array. The image may 
be compressed in the 'x' and 'y' directions. The file must be opened before 
this routine is called and it is not closed by this routine. The user may
select the rectangular area of the image to be read. The returned image data 
is in the form required by Daresbury XDL_VIEW routines such as xdl_film_image.
The data in the input file may be in any of the eight possible orders and
may be byte swapped if required.
-end*/
/*
***********************
**   xdl_read_i2     **
***********************

Purpose: Read a 'i2' unsigned 16 bit integer format image file and return the
         image data in an array. The image may be compressed in the 'x'
         and 'y' directions. The file must be opened before this routine
         is called and it is not closed by this routine. The user may
         select the rectangular area of the image to be read. The returned
         image data is in the form required by Daresbury X-windows routines 
         such as xdl_film_image.

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_READ_I2 (IFD, IORD, NXRASTS, NYRASTS, IXOPIX, IYOPIX, 
        +                   NXPIXELS, NYPIXELS, NXP, NYP, 
        +                   IVH_BAR, IXROOT, IYROOT, 
        +                   IMG, IMGW, IMGH, MINVAL, MAXVAL, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film image file (1 to 8)
               (-IORD can be given to swap bytes in the data as
               well as re-order it ) (see order)
NXRASTS  (R)   The number of x-rasters (see nxrasts)
NYRASTS  (R)   The number of y-rasters (see nyrasts)
IXOPIX   (R)   The origin x pixel (see xo_pix)
IYOPIX   (R)   The origin y pixel (see yo_pix)
NXPIXELS (R)   The number of x pixels (see nx_pixels)
NYPIXELS (R)   The number of y pixels (see ny_pixels)
NXP      (R)   Compress NXP x pixels into 1 (see nxp)
NYP      (R)   Compress NYP y pixels into 1 (see nyp)
IVH_BAR  (R)   View-object handle for displaying a read progress
               bar >0 on 0 if none required (see vh_bar)
IXROOT   (R)   x root position for progress bar display if 
               required (see xroot)
IYROOT   (R)   y root position for progress bar display if 
               required (see yroot)
IMG      (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see img)
IMGW     (W)   Returns the image width in IMG (see img_width)
IMGH     (W)   Returns the image height in IMG (see img_height)
MINVAL   (W)   Minimum pixel value in portion of the image read
               (see minval)
MAXVAL   (W)   Maximum pixel value in portion of the image read
               (see maxval)
IERR     (W)   Returns the status from the xdl_read_i2 call
-end*/

/*-C:*/
int xdl_read_i2 (fd, order, nxrasts, nyrasts, xo_pix, yo_pix, 
                 nx_pixels, ny_pixels, 
                 nxp, nyp, vh_bar, xroot, yroot, 
                 img, img_width, img_height, minval, maxval)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened 'i2' format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8. (-order can be given to
                     byte swap the data as well as change the order)
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int nxrasts;      /* The number of x-rasters (R)*/
int nyrasts;      /* The number of y-rasters (R)*/
int xo_pix;       /* The x value of the required origin pixel (pixels 
                     numbered from 1 up). If this is less than 1 it 
                     will be reset to 1 and, if it is greater than the 
                     maximum x pixel number present, it will be set to 
                     the maximum x pixel number (R)*/
int yo_pix;       /* The y value of the required origin pixel (pixels 
                     numbered from 1 up). If this is less than 1 it 
                     will be reset to 1 and, if it is greater than the 
                     maximum y pixel number present, it will be set to 
                     the maximum y pixel number (R)*/
int nx_pixels;    /* The 'x' width of the image section to be read. 
                     If a value of less than 1 is given it will be 
                     reset to 1. If a value is given which takes the 
                     section beyond the image then it will be reset to
                     be within bounds (R)*/
int ny_pixels;    /* The 'y' height of the image section to be read. 
                     If a value of less than 1 is given it will be 
                     reset to 1. If a value is given which takes the 
                     section beyond the image then it will be reset to 
                     be within bounds (R)*/
int nxp;          /* The number of pixels in the 'x' direction to 
                     compress to 1 pixel (R)*/
int nyp;          /* The number of pixels in the 'y' direction to 
                     compress to 1 pixel (R)*/
int vh_bar;       /* View-object handle (>0) for display bar showing the
                     progress of the read, 0 if none required (R) */
int xroot;        /* The root x position for the top left of the display
                     bar area if required (R)*/
int yroot;        /* The root y position for the top left of the display
                     bar area if required (R)*/
unsigned short img[];/* Array returning the read image data; The pixels 
                     will have been compressed in 'x' and 'y' as 
                     requested. The values will be averaged over 
                     nxp*nyp pixels and will be in unsigned 16 bit 
                     integer format. The slower moving index in the 
                     returned data is y going from high y to low y and
                     the faster moving index is x going from low x to 
                     high x (W)*/
int * img_width;  /* Returns image width as a number of elements. 
                     Fastest moving index in 'img' (= x in i2 file). (W)*/
int * img_height; /* Returns image height as a number of elements. 
                     Slowest moving index in 'img' (= y in i2 file). (W)*/
int * minval;     /* Returns minimum pixel value in the portion of the
                     image read (W)*/
int * maxval;     /* Returns maximum pixel value in the portion of the
                     image read (W)*/

/* Note if nx (in the i2 file) is not a multiple of nxp then up to
   nxp-1 pixels may be lost from the high end of 'x' and similarly for
   ny and nyp. */
/*end*/
/*-Doc:
Return:  =0 OK; >0 Record number where a read error occurred.
-end*/
{
   int i,j,k,l;
   int ix, iix;
   int iy, iiy;
   int xoff, yoff;
   int nx, ny;
   unsigned short *buf;
   unsigned char *cbuf;
   unsigned char tempc;
   long *ibuf;
   long ival;
   int nx_els, ny_els;
   int irec;
   int ioff;
   int nr;
   int rec_step;
   int swap;

   /* Initialisations */

   swap = 0;
   if (order<0)
   {
      swap = 1;
      order = -order;
   }
   if (order<1||order>8) order = 1;
   if (nxp<1) nxp = 1;
   if (nyp<1) nyp = 1;
   if (xo_pix<1) xo_pix=1;
   if (yo_pix<1) yo_pix=1;
   if (nx_pixels<1) nx_pixels=1;
   if (ny_pixels<1) ny_pixels=1;

   nx = nxrasts;
   ny = nyrasts;

   /* Further checks on dimensions */

   if (xo_pix>nx) xo_pix = nx;
   if (yo_pix>ny) yo_pix = ny;
   if ((xo_pix+nx_pixels-1)>nx) nx_pixels = nx - xo_pix + 1;
   if ((yo_pix+ny_pixels-1)>ny) ny_pixels = ny - yo_pix + 1;

   /* Read data */

   nx_els = nx_pixels/nxp;
   *img_width = nx_els;
   ny_els = ny_pixels/nyp;
   *img_height = ny_els;
   *minval = 1000000;
   *maxval = 0;

   /* Compress image in xy */

   if (order<5) /*xf slow, yf fast*/
   {
      nr = nx_els * nxp;  
      if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nr, 2, 5);
      buf = (unsigned short *) malloc (ny*sizeof(unsigned short));
      if (buf==0) return -1;
      cbuf = (unsigned char *) buf;
      ibuf = (long *) malloc (ny*sizeof(long));
      if (ibuf==0) return -1;
      if (order==1||order==2)
      {
         irec = xo_pix;
         rec_step = 1;
      }
      else
      {
         irec = nxrasts + 1 - xo_pix;
         rec_step = -1;
      }
      for (ix=0;ix<nx_els;++ix)
      {
         for (i=0;i<ny_pixels;++i) ibuf[i] = 0;
         for (iix=0;iix<nxp;++iix)
         {
            i = lseek(fd, (long) (irec-1)*2*nyrasts,0);
            i = read (fd, buf, 2*nyrasts);
            if (i!=2*nyrasts) goto tidy;
            if (swap)
            {
               l=0;
               for (k=0;k<nyrasts;++k)
               {
                   tempc = cbuf[l];
                   cbuf[l] = cbuf[l+1];
                   cbuf[l+1] = tempc;
                   l += 2;
               }
            }
            if (vh_bar>0) xdl_progress_bar_value (vh_bar, ix*nxp+iix+1);
            if (order==1||order==3)
            {
               for (i=ny_pixels-1,j=yo_pix-1;i>=0;--i,++j)
               {
                  ibuf[i] +=  (long) buf[j];
                  if (buf[j]>*maxval) *maxval = buf[j];
                  if (buf[j]<*minval) *minval = buf[j];
               }
            }
            else
            {
               for (i=0,j=nyrasts+1-ny_pixels-yo_pix;i<ny_pixels;++i,++j)
               {
                  ibuf[i] += (long) buf[j];
                  if (buf[j]>*maxval) *maxval = buf[j];
                  if (buf[j]<*minval) *minval = buf[j];
               }
            }
            irec += rec_step;
         }
         yoff = 0;
         for (iy=0;iy<ny_els;++iy)
         {
            ival = 0;
            for (iiy=0;iiy<nyp;++iiy)
            {
               ival += ibuf[yoff];
               yoff++;
            }
            ival = ival / (nxp*nyp);
            img[iy*nx_els+ix] = (unsigned short) (ival);
         }
      }
   }
   else  /*yf slow, xf fast*/
   {
      nr = ny_els * nyp;  
      if (vh_bar>0) xdl_progress_bar (vh_bar, xroot, yroot, "Read progress", 0, 
                                   120, nr, 2, 5);
      buf = (unsigned short *) malloc (nx*sizeof(unsigned short));
      if (buf==0) return -1;
      cbuf = (unsigned char *) buf;
      ibuf = (long *) malloc (nx*sizeof(long));
      if (ibuf==0) return -1;
      ioff = 0;
      if (order==5||order==6)
      {
         irec = yo_pix + ny_pixels - 1;
         rec_step = -1;
      }
      else
      {
         irec = nyrasts + 2 - ny_pixels - yo_pix;
         rec_step = 1;
      }
      for (iy=0;iy<ny_els;++iy)
      {
         for (i=0;i<nx_pixels;++i) ibuf[i] = 0;
         for (iiy=0;iiy<nyp;++iiy)
         {
            i = lseek(fd, (long) (irec-1)*2*nxrasts,0);
            i = read (fd, buf, 2*nxrasts);
            if (i!=2*nxrasts) goto tidy;
            if (swap)
            {
               l=0;
               for (k=0;k<nxrasts;++k)
               {
                   tempc = cbuf[l];
                   cbuf[l] = cbuf[l+1];
                   cbuf[l+1] = tempc;
                   l += 2;
               }
            }
            if (vh_bar>0) xdl_progress_bar_value (vh_bar, iy*nyp+iiy+1);
            if (order==5||order==7)
            {
               for (i=0,j=xo_pix-1;i<nx_pixels;++i,++j)
               {
                  ibuf[i] +=  (long) buf[j];
                  if (buf[j]>*maxval) *maxval = buf[j];
                  if (buf[j]<*minval) *minval = buf[j];
               }
            }
            else
            {
               for (i=0,j=nxrasts-xo_pix;i<nx_pixels;++i,--j)
               {
                  ibuf[i] += (long) buf[j];
                  if (buf[j]>*maxval) *maxval = buf[j];
                  if (buf[j]<*minval) *minval = buf[j];
               }
            }
            irec += rec_step;
         }
         xoff = 0;
         for (ix=0;ix<nx_els;++ix)
         {
            ival = 0;
            for (iix=0;iix<nxp;++iix)
            {
               ival += ibuf[xoff];
               xoff++;
            }
            ival = ival / (nxp*nyp);
            img[ioff] = (unsigned short) (ival);
            ioff++;
         }
      }
   }
   irec = 0;
      
   /* Tidy up */

   tidy:
   free (buf);
   free (ibuf);
   if (vh_bar>0) xdl_delete_view_object (vh_bar);
   return irec;
}

/* Fortran binding: xdlf_read_i2 */

void xdlf_read_i2 (ifd, iord, nxrasts, nyrasts, ixopix, iyopix, 
                   nxpixels, nypixels, 
                   nxp, nyp, ivh_bar, ixroot, iyroot, img, imgw, imgh, 
                   minval, maxval, ierr)
int *ifd, *iord, *nxrasts, *nyrasts, *ixopix, *iyopix, *nxpixels, *nypixels;
int *nxp, *nyp, *ivh_bar, *ixroot, *iyroot, *img, *imgw, *imgh, *ierr;
int *minval, *maxval;
{
   *ierr = xdl_read_i2 (*ifd, *iord, *nxrasts, *nyrasts, *ixopix, *iyopix, 
                        *nxpixels, *nypixels, *nxp, *nyp, 
                        *ivh_bar, *ixroot, *iyroot, 
                        img, imgw, imgh, minval, maxval);
   return;
}

/*-Routine: Read a section of an image plate (i2) file - xdl_read_i2_box
The routine xdl_read_i2_box (xdlf_read_i2_box) is used to read a rectangular 
section from a 'i2' image file and return the data in an integer array. 
The file must be opened before this routine is called and it is not closed 
by this routine. The file may have its data in any of the eight possible
orders and may be byte swapped if needed (the default is assumed to  have 
yf moving fastest and xf slowest).
-end*/
/*
***************************
**   xdl_read_i2_box     **
***************************

Purpose: Read a rectangular section from a 'i2' image file and return the 
         data in an integer array. 
         The file must be opened before this routine is called and it is not 
         closed by this routine. 
         The file may have its data in any of the eight possible orders
         and may be byte swapped (the default is assumed to  have yf moving 
         fastest and xf slowest).

Author:  John Campbell, July 1990

*/
/*-Fortran:
         CALL XDLF_READ_I2_BOX (IFD, IORD, IXC, IYC, ISIZX, ISIZY, NXRAST,
        +                       NYRAST, IDATA, IERR)
-end*/
 
/*-Parameters:
IFD      (R)   The file descriptor (see fd)
IORD     (R)   Order of the data in the film image file (1 to 8)
               (-IORD can be given to swap bytes in the data as
               well as re-order it ) (see order)
IXC      (R)   The centre x pixel (see xc)
IYC      (R)   The centre y pixel (see yc)
IXSIZ    (R)   The box 'half' size in x (actual size 2*IXSIZ+1) 
               (see xsiz)
IYSIZ    (R)   The box 'half' size in y (actual size 2*IYSIZ+1)
               (see ysiz)
NXRAST   (R)   The number of x pixels (see nxrast)
NYRAST   (R)   The number of y pixels (see nyrast)
IDATA    (W)   Array to hold the read data (pass as an
               integer array sufficiently large to hold the
               returned image data) (see idata)
IERR     (W)   Returns the status from the xdl_read_i2_box call
-end*/

/*-C:*/
int xdl_read_i2_box (fd, order, xc, yc, xsiz, ysiz, nxrast, nyrast, 
                     idata)
/*end*/
 
/*-Parameters:*/
int fd;           /* The file descriptor of the opened 'i2' format 
                     file (R)*/
int order;        /* Order of the data in the input film image file
                     in terms of the xf, yf coordinate system as
                     a number from 1 to 8. (-order can be given to
                     byte swap the data as well as change the order)
                     1   +xf slow   +yf fast
                     2   +xf slow   -yf fast
                     3   -xf slow   +yf fast
                     4   -xf slow   -yf fast
                     5   +yf slow   +xf fast
                     6   +yf slow   -xf fast
                     7   -yf slow   +xf fast
                     8   -yf slow   -xf fast
                     (R)*/
int xc;           /* The box centre (x) pixel (R)*/
int yc;           /* The box centre (y) pixel (R)*/
int xsiz;         /* The box 'half' x size (actual box size = 2*xsiz+1)
                     (R)*/
int ysiz;         /* The box 'half' y size (actual box size = 2*ysiz+1) 
                     (R)*/
int nxrast;       /* The total number of x pixels in the image (R)*/
int nyrast;       /* The total number of y pixels in the image (R)*/
int idata[];      /* Integer array returning the read image data with one
                     pixel value per integer. Out of limits pixels will
                     be returned with zero values (Fastest moving y) (W)*/
/*end*/
/*-Doc:
Return:  =  0 OK; 
         = -1 Error reading file
         = -2 Cannot allocate memory for file reading buffer
         =  1 Partly out of limits
         =  2 Completely out of limits
-end*/
{
   unsigned short *buf;     /* Buffer for reading in record */
   unsigned char *cbuf;     /* Buffer for byte swapping */
   unsigned char tempc;     /* Tempoary character for byte swapping */
   int swap;                /* Byte swap flag =1 yes, =0 no */
   int ierr;                /* Error flag */
   int xstart;              /* Start of x range */
   int xend;                /* End of x range */
   int ystart;              /* Start of y range */
   int yend;                /* End of y range */
   int ix;                  /* Loop counter over x rasters */
   int iy;                  /* Loop counter over y rasters */
   int ii;                  /* Index into 'idata' array */
   int i, k, l;             /* Temp variables */
   int xrec;                /* Record number for x (slow x) */
   int yrec;                /* Record number for y (slow y) */
   int xoff;                /* Offset into record buffer (slow y) */
   int yoff;                /* Offset into record buffer (slow x) */

   /* Initialisations */

   swap = 0;
   if (order<0)
   {
      swap = 1;
      order = -order;
   }
   if (order<1||order>8) order = 1;
   ierr = 0;
   xstart = xc - xsiz;
   xend = xc + xsiz;
   ystart = yc - ysiz;
   yend = yc + ysiz;

   if (order<5)
   {
      /* Allocate buffer space for file reads and collecting data */

      buf = (unsigned short *) malloc (nyrast*sizeof(unsigned short));
      if (buf==0) return -2;
      cbuf = (unsigned char *) buf;

      /* Read data */

      ii = 0;
      for (ix=xstart;ix<=xend;++ix)
      {
         xrec = ix;
         if (order==3||order==4) xrec = nxrast + 1 - xrec;
         if (xrec<1||xrec>nxrast)
         {
            ierr = 1;
            for (iy=ystart;iy<=yend;++iy)
            {
               idata[ii] = 0;
               ii++;
            }
         }
         else
         {
            lseek (fd, (long) (xrec-1)*2*nyrast, 0);
            i = read (fd, buf, 2*nyrast);
            if (i!=2*nyrast) 
            {
               ierr = -2;
               goto tidy;
            }
            if (swap)
            {
               l=0;
               for (k=0;k<nyrast;++k)
               {
                   tempc = cbuf[l];
                   cbuf[l] = cbuf[l+1];
                   cbuf[l+1] = tempc;
                   l += 2;
               }
            }
            for (iy=ystart;iy<=yend;++iy)
            {
               if (iy<1||iy>nyrast)
               {
                  ierr = 1;
                  idata[ii] = 0;
               }
               else
               {
                  yoff = iy - 1;
                  if (order==2||order==4) yoff = nyrast - 1 - yoff;
                  idata[ii] = (int) buf[yoff];
               }
               ii++;
            }
         }
      }
   }
   else
   {
      /* Allocate buffer space for file reads and collecting data */

      buf = (unsigned short *) malloc (nxrast*sizeof(unsigned short));
      if (buf==0) return -2;
      cbuf = (unsigned char *) buf;

      /* Read data */

      for (iy=ystart;iy<=yend;++iy)
      {
         yrec = iy;
         if (order==7||order==8) yrec = nyrast + 1 - yrec;
         if (yrec<1||yrec>nyrast)
         {
            ierr = 1;
            ii = iy - ystart;
            for (ix=xstart;ix<=xend;++ix)
            {
               idata[ii] = 0;
               ii += nyrast;
            }
         }
         else
         {
            lseek (fd, (long) (yrec-1)*2*nxrast, 0);
            i = read (fd, buf, 2*nxrast);
            if (i!=2*nxrast) 
            {
               ierr = -2;
               goto tidy;
            }
            if (swap)
            {
               l=0;
               for (k=0;k<nxrast;++k)
               {
                   tempc = cbuf[l];
                   cbuf[l] = cbuf[l+1];
                   cbuf[l+1] = tempc;
                   l += 2;
               }
            }
            for (ix=xstart;ix<=xend;++ix)
            {
               if (ix<1||ix>nyrast)
               {
                  ierr = 1;
                  idata[ii] = 0;
               }
               else
               {
                  xoff = ix - 1;
                  if (order==6||order==8) xoff = nxrast - 1 - xoff;
                  idata[ii] = (int) buf[xoff];
               }
               ii += nyrast;
            }
         }
      }
   }
   if (xend<1||yend<1||xstart>nxrast||ystart>nyrast) ierr = 2;
      
   /* Tidy up */

   tidy:
   free (buf);
   return ierr;
}
/* Fortran binding: xdlf_read_i2_box */

void xdlf_read_i2_box (ifd, iord, ixc, iyc, ixsiz, iysiz, nxrast, nyrast,
                        idata, ierr)
int *ifd, *iord, *ixc, *iyc, *ixsiz, *iysiz, *nxrast, *nyrast, *idata, *ierr;
{
   *ierr = xdl_read_i2_box (*ifd, *iord, *ixc, *iyc, *ixsiz, *iysiz,
                                   *nxrast, *nyrast, idata);
   return;
}

