/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Show Unique Data View-Object Routines
-end*/
 
/*-Intro:
The show unique view-object is used to give a pictorial representation
of sections of the reciprocal lattice showing the unique data and which
parts of that data have been measured (or potentially measured) for
simulated data,
-end*/
 
/*-Section: Creating and Handling a Show Unique Data View-Object
-end*/
 

/*Routines List:
The show unique view-object has the following routines available:

    xdl_show_unq           Create a 'show unique' view-object
    xdl_show_unq_getsize   Calculate size requirements
    xdl_show_unq_getdata   Get cursored in position from reciprocal lattice
*/

/*-Routine: Create a show unique view-object - xdl_show_unq
This routine is used to create a view-object for showing the coverage
of the reciprocal space unique data.
-end*/
/*
**************************
**    xdl_show_unq      **
**************************

Purpose: Show unique section of reciprocal lattice and data coverage

Author:  John W. Campbell, july 1995

*/
/*-Fortran:
         CALL XDLF_SHOW_UNQ (IVH, IVHPARENT, IX, IY, ICSET, IRAD,
        +                    MINDX_UNQ, DSTMAX, BMATR, 
        +                    XDLSTR(TITLE), LENT, MINW, MINH, IERR)
-end*/
 
/*-Parameters:
IVH        (R)   View-object handle (see vh)
IVHPARENT  (R)   View-object handle for parent 0=none 
                 (see vh_parent)
IX         (R)   X position (may be -1 if no parent) (see x)
IY         (R)   Y position (may be -1 if no parent) (see y)
ICSET      (R)   Colorset number (see cset)
ISIZE      (R)   Half width/height for displayed pattern in 
                 pixels (see size)
MINDX_UNQ  (R)   Index to unique data list (unq.c functions)
                 (preferably sorted on 'l' to give much better
                 performance)
DSTMAX     (R)   Maximum value of d*
BMATR(3,3) (R)   Orthogonalising 'B' matrix (Fortran order
                 BMATR(1,1) BMATR(2,1) BMATR(3,1) BMATR(1,2)...)
TITLE      (R)   Title string (max 120 characters)
                 ** Pass address using the XDLSTR function **
LENT       (R)   Length of title string
MINW       (R)   Minimum width for view-object or 0 
                 (see min_width)
MINH       (R)   Minimum height for view-object or 0 
                 (see min_height)
IERR       (W)   Returns status from xdl_show_unq call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"
#include "unq.h"
#include <math.h>

#define PANEL_WIDTH  595  /* Minimum width required for control panel items */
#define PANEL_HEIGHT 70   /* Height required for control panel items */
#define SEP_H_MIN 4       /* Minimum separation between contol panels and
                             unique data display area */

#define nint(x)  x>=0.0?(int)(x+0.5):(int)(x-0.5)

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wpaint;                    /* Paint window id */
   Window wact;                      /* Active strip window */
   Window wpanel;                    /* Control panel paint window */
   Panel_object fulls_choice;        /* Fulls choice menu */
   Panel_object partials_choice;     /* Partials choice menu */
   Panel_object anom_choice;         /* Anomalous choice menu */
   Panel_object spat_choice;         /* Spatials choice menu */
   Panel_object up_button;           /* Layer 'up' button */
   Panel_object down_button;         /* Layer 'down' button */
   Panel_object layer_slider;        /* Layer slider */
   Panel_object layer_label;         /* Layer label */
   Panel_object cycle_button;        /* Cycle through layers label */
   Panel_object cycle_slider;        /* Cycle speed slider */
   Panel_object ps_button;           /* Postscript button */
   Panel_object ps_io;               /* POstscript i/o item */
   GC          paint_gc;             /* GC for paint window */
   int         on_off;               /* Program awaiting input =1, not =0 */
   int         exposed;              /* Set after 1'st expose event to
                                        main display window */
   int slider_len;                   /* Length for layers slider */
   int area_width;                   /* Width of unique data display area */
   int area_height;                  /* Height of unique data display area */
   int xcen;                         /* Centre 'x' pixel of unique data display
                                        area */
   int ycen;                         /* Centre 'y' pixel of unique data display
                                        area */
   int rad_surround;                 /* Radius of main surround circle */
   int lw_surround;                  /* Line width of main surround circle */
   int mindx_unq;                    /* Index to unique data list */
   float dstmax;                     /* Max d* value */
   float bmatr[3][3];                /* 'B' matrix */
   char title[121];                  /* Title string */
   float scal;                       /* Scale factor r.l. coords to pixel
                                        offset coords */
   int x_selected;                   /* Button selected 'x' position */
   int y_selected;                   /* Button selected 'y' position */
   int * layers;                     /* Start reflection number for each
                                        layer */
   int lmin;                         /* Minimum 'l' index */
   int lmax;                         /* Maximum 'l' index */
   int sorted;                       /* Data sorted on 'l' flag 1=yes, 0=no */
   int cur_layer;                    /* Current layer number */
   int cycle_on;                     /* Automatic cycling on 1=yes, 0=no*/
   XtIntervalId timer_id;            /* ID for layer cycling timer */
   int pause_done;                   /* Pause end routine done =1 yes, =0 no */
   unsigned long cycle_time;         /* Cycle time for layers (pause on each 
                                       layer */
   int fulls_flag;                   /* =1 include fulls, = 0 exclude */
   int partials_flag;                /* =0 exclude,  = 1 include <=2,
                                        =2 include <= 3, =3 include <=nwmax */
   int anom_flag;                    /* =0 merge, =1 keep separate anomalous
                                        data */
   int spat_flag;                    /* =0 exclude spatials, =1 include 
                                        spatials */
   unsigned long unique_pixel;       /* Pixel colour for unique spots */
   unsigned long measured_pixel;     /* Pixel colour for measured spots */
   unsigned long plus_pixel;         /* Pixel colour for I+ spots */
   unsigned long minus_pixel;        /* Pixel colour for I- spots */
   unsigned long unk_pixel;          /* Pixel colour for unknown sign spots */
   unsigned long circle_pixel;       /* Pixel colour for circles */
   unsigned long axis_pixel;         /* Pixel colour for lattice axes */
   unsigned long layer_pixel;        /* Pixel colour for layer number */

   int         ps_on;                /* Postscript option in progress
                                        flag =1 yes, =0 no */
   FILE        *psfil;               /* File pointer for Postscript file */
   int         color_plot;           /* Postscript o/p colour plot flag
                                        =1 yes, =0 no */
   int         mult_sect;            /* Postscript o/p all sections flag
                                        =1 multiple sections,
                                        =0 one section only */
   int         ps_minsec;            /* Minimum (or single) section no. for
                                        Postscript plot */
   int         ps_maxsec;            /* Maximum section no. for Postscript 
                                        plot */
   int         ps_incsec;            /* Section no. increment for Postscript 
                                        plot */
   float       ps_symscal;           /* Symbol scaling factor */
   char        *ps_comment;          /* Pointer to comment for postscript file
                                        (last reply from panel i/o item) */
}  
   XDl_show_unq;

/* Define routine types */

void xdl_show_unq_rps();
void xdl_show_unq_rp1();
void xdl_show_unq_ep1();
void xdl_show_unq_rp2();
void xdl_show_unq_cp1();
void xdl_show_unq_cp2();
void xdl_show_unq_cp3();
void xdl_show_unq_cp4();
void xdl_show_unq_cp5();
void xdl_show_unq_cp6();
void xdl_show_unq_cp7();
void xdl_show_unq_cp8();
void xdl_show_unq_cp9();
void xdl_show_unq_ps();
void xdl_show_unq_psdraw();
void xdl_show_unq_pstitles();
void xdl_show_unq_pscomments();
void xdl_show_unq_psoutlines();
void xdl_show_unq_pslayer();
void xdl_show_unq_cp10();
void xdl_show_unq_on_off();
void xdl_show_unq_tidy_up();
void xdl_show_unq_cycle_dummy();
void xdl_show_unq_cycle_off();
void xdl_show_unq_sendmsg();
void xdl_show_unq_arrow_coords();
void xdl_show_unq_refresh();
int xdl_show_unq_getdata();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_show_unq xdlf_show_unq_
#define xdlf_show_unq_getdata xdlf_show_unq_getdata_

#endif

#if  LINKTYP == 2

#define xdlf_show_unq XDLF_SHOW_UNQ
#define xdlf_show_unq_getdata XDLF_SHOW_UNQ_GETDATA

#endif

void xdlf_show_unq();
void xdlf_show_unq_getdata();

/*-C:*/
int xdl_show_unq (int vh, int vh_parent, int x, int y, int cset, int size, 
                  int mindx_unq, float dstmax, float bmatr[3][3], 
                  char * title, int lent, int min_width, int min_height)
/*end*/
 
/*-Parameters:
int       vh;             User selected view-object handle (R)
int       vh_parent;      View-object handle for the parent base frame, 
                          if 0 then a base  level frame is created to
                          enclose the view-object (R)
int       x;              x coordinate for the view-object.
                          If no parent may be -1 to give default x (R)
int       y;              y coordinate for the view-object.
                          If no parent may be -1 to give default y (R)
int       cset;           Number of the colorset to be used; needs
                          at least 8 colorcells allocated  (R) 
int       size;           Pattern display area half width/height in
                          pixels (R)
int       mindx_unq;      Index to unique data list (see unq.c functions) 
                          preferably sorted on 'l' as this will give
                          much better performance (R)
float     dstmax;         Maximum d* value (R)
float     bmatr[3][3];    'B' matrix (C order) (R) 
char      *title;         Title string (max of 120 characters) (R)
int       lent;           Length of title string (may give 0 if a null 
                          terminated string is passed) (R)
int       min_width;      The program will calculate the width required
                          for the view-object based on the display area
                          width, control panel width etc. If this is
                          less than min_width then min_width will be
                          used instead and the layout altered to reflect
                          this (R)
int       min_height;     minimum height cf min_width (R)
-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         ...
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_show_unq * gd;          /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int panel_width;            /* Width for control panel */
   int panel_height;           /* Height for control panel */
   int x_panel;                /* 'x' position for control panel */
   int y_panel;                /* 'y' position for control panel */
   int area_width;             /* Paint area width */
   int area_height;            /* Paint area height */
   int x_area;                 /* x position of paint area within 
                                  the view-object*/
   int y_area;                 /* y position of paint area within
                                  the view-object */
   int ytop;                   /* Height for act strip + control panel */
   int panel_x;                /* 'x' position of item within control panel */
   int panel_y;                /* 'y' position of item within control panel */
   int font;                   /* Font no. for panel items */
   int i;                      /* Temp/loop variable */
   int h;                      /* 'h' index */
   int k;                      /* 'k' index */
   int l;                      /* 'l' index */
   int nlayers;                /* No. of layers */
   int last_l;                 /* Last 'l' value */
   int ll;                     /* Offset into layers list for current 'l' 
                                  value */
   int islide;                 /* Slider position */
   int nref;                   /* No. of reflections in unique reflections
                                  list */
   char str[10];               /* Temporary string */
   XColor colr_cell;           /* Temporary Xcolor structure */
   int lstr;                   /* String length */

   static char *full_strings[] =
   {"included","excluded"};            /* Fulls menu strings */

   static char *partial_strings[] =
   {"none","<=2","<=3","<=nwmax"};     /* Partials menu strings */

   static char *anom_strings[] =
   {"merged","separate"};              /* Anomalous menu strings */

   static char *spat_strings[] =
   {"excluded","included"};            /* Spatials menu strings */

   err = 0;
   font = 2;

   /* Get size requirements */

   area_width = 2*size + 1 + 2;
   area_height = 2*size + 1 + 2;
   panel_width = PANEL_WIDTH;
   panel_height = PANEL_HEIGHT;
   if (panel_width>area_width)
   {
      width = panel_width + 2*BORDER_WIDTH + 2;
   }
   else
   {
      width = area_width + 2*BORDER_WIDTH + 2;
   }
   height = area_height + SEP_H_MIN + panel_height 
            + ACT_STRIP_HEIGHT
            + 2*BORDER_HEIGHT + 2;
   if (width<min_width) width = min_width;
   if (height<min_height) height = min_height;
   area_width = width - 2*BORDER_WIDTH-2;
   area_height = height - SEP_H_MIN - panel_height - ACT_STRIP_HEIGHT 
                 -2*BORDER_HEIGHT - 2;

   x_panel = BORDER_WIDTH;
   y_panel = ACT_STRIP_HEIGHT+ BORDER_HEIGHT;
   x_area = (width-2-area_width)/2;
   ytop = ACT_STRIP_HEIGHT + PANEL_HEIGHT + SEP_H_MIN;
   y_area = ytop + (height - 2 - area_height - ytop)/2;

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_show_unq", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_show_unq *) malloc (sizeof(XDl_show_unq));

   /* Add new view-object to the list of view-objects */
 
   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2112, 
                        xdl_show_unq_on_off, xdl_show_unq_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->on_off = 0;
   gd->exposed = 0;
   gd->slider_len = 150;
   gd->x_selected = -1;
   gd->y_selected = -1;
   gd->area_width = area_width;
   gd->area_height = area_height;
   gd->xcen = (area_width-2)/2;
   gd->ycen = (area_height-2)/2;
   gd->lw_surround = 2;
   gd->rad_surround = (area_width-2)/2;
   if ((area_height-2)/2<gd->rad_surround) 
        gd->rad_surround = (area_height-2)/2;
   gd->rad_surround = gd->rad_surround - 25 - gd->lw_surround;
   gd->ps_on = 0;
   gd->cycle_on = 0;
   gd->timer_id = 0;
   gd->cycle_time = 500;
   gd->mindx_unq = mindx_unq;
   gd->dstmax = dstmax;
   gd->bmatr[0][0] = bmatr[0][0];
   gd->bmatr[0][1] = bmatr[0][1];
   gd->bmatr[0][2] = bmatr[0][2];
   gd->bmatr[1][0] = bmatr[1][0];
   gd->bmatr[1][1] = bmatr[1][1];
   gd->bmatr[1][2] = bmatr[1][2];
   gd->bmatr[2][0] = bmatr[2][0];
   gd->bmatr[2][1] = bmatr[2][1];
   gd->bmatr[2][2] = bmatr[2][2];
   lstr = lent;
   if (lstr==0) lstr = strlen(title);
   if (lstr>120) lstr = 120;
   gd->title[0] = 0;
   strncat (gd->title, title, lstr);
   gd->scal = gd->rad_surround/dstmax; 

   gd->fulls_flag = 1;
   gd->partials_flag = 2;
   gd->anom_flag = 0;
   gd->spat_flag = 0;

   /* Create active strip window */

   gd->wact = xdl_create_active_strip (gd->wframe, vh, area_width, 
                                       xdl_show_unq_rps);

   /* Create paint window */

   gd->wpaint = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                                         area_width, area_height, cset);

   xdl_add_window (vh, gd->wpaint, xdl_show_unq_rp1, xdl_show_unq_ep1);

   XSetWindowBackground (XDL_display, gd->wpaint, XDL_blackpixel);

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wpaint, ExposureMask|StructureNotifyMask|
                 ButtonPressMask|ButtonReleaseMask);

   /* Paint window for control panel */

   gd->wpanel = xdl_create_paint_window (wframe, x_panel, y_panel, 1, 
                width - 2*BORDER_WIDTH -2, PANEL_HEIGHT, cset);

   xdl_add_window (vh, gd->wpanel, xdl_show_unq_rp2, NULL);

   XSelectInput (XDL_display, gd->wpanel, ExposureMask|StructureNotifyMask);

   /* Panel items for control panel */
   
      /* Fulls menu */
      panel_x = 4;
      panel_y = 24;

      gd->fulls_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Fulls ", 2, full_strings, 1, font, 0, 0, 0,
                      xdl_show_unq_cp1);

      /* Partials menu */
      panel_x = 134;
      panel_y = 24;

      gd->partials_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Partials ", 4, partial_strings, 3, font, 0, 0, 0,
                      xdl_show_unq_cp2);

      /* Anomalous menu */
      panel_x = 279;
      panel_y = 24;

      gd->anom_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Anomalous ", 2, anom_strings, 1, font, 0, 0, 0,
                      xdl_show_unq_cp3);

      /* Spatials menu */
      panel_x = 434;
      panel_y = 24;

      gd->spat_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Spatials ", 2, spat_strings, 1, font, 0, 0, 0,
                      xdl_show_unq_cp4);

      /* Layer up and down buttons */

      panel_x = 8;
      panel_y = 44;
      gd->down_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                     "DOWN", 4, 2, 1, 0, 0, 0, 1, 1, xdl_show_unq_cp5);

      panel_x = 265;
      panel_y = 44;
      gd->up_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                     " UP ", 4, 2, 1, 0, 0, 0, 1, 2, xdl_show_unq_cp5);

      /* Layer slider */

      panel_x = 50;
      panel_y = 44;
      gd->layer_slider =  xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                         "Layer", font, gd->slider_len, 0, 0,
                          0, 0, xdl_show_unq_cp6);

      /* Layer label */

      panel_x = 308;
      panel_y = 44;
      gd->layer_label = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                       "l=0", 8, font, 0, 0, 0, 1);

      /* Cycle  button */
      panel_x = 530;
      panel_y = 44;
      gd->cycle_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                      "CYCLE", 6, 2, 2, 0, 0, 1, 1, 1, xdl_show_unq_cp7);

      /* Cycle slider */

      panel_x = 410;
      panel_y = 44;
      gd->cycle_slider =  xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                         "Speed", font, 50, 40, 0,
                          0, 0, xdl_show_unq_cp10);
      /* Postscript button */
      panel_x = 8;
      panel_y = 4;
      gd->ps_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                      "PS", 2, 2, 2, 0, 0, 1, 1, 1, xdl_show_unq_cp8);

      /* Postscript i/o area */
      panel_x = 36;
      panel_y = 4;
      gd->ps_io = xdl_panel_io (vh, gd->wpanel, panel_x, panel_y,
                  50, 20, 200, font, 1, 1, 550, 0, 
                  xdl_show_unq_cp9);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wpaint, 0, NULL);

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);

   /* Check order of unique data list; set up pointers to layers if
      sorted on 'l' */

   gd->layers = 0;
   nref = unq_reflist_numrefs(gd->mindx_unq);
   if (nref<=0) return 0;
   unq_reflist_hkl (gd->mindx_unq, 1, &h, &k, &l);
   gd->lmin = l;
   gd->lmax = l;
   gd->sorted = 1;
   last_l = l;
   for (i=1;i<=nref;++i)
   {
      unq_reflist_hkl (gd->mindx_unq, i, &h, &k, &l);
      if (l<last_l) gd->sorted = 0;
      last_l = l;
      if (l>gd->lmax) gd->lmax = l;
      if (l<gd->lmin) gd->lmin = l;
   }
   if (gd->sorted)
   {
      nlayers = gd->lmax - gd->lmin + 1;
      gd->layers = (int *) malloc (nlayers*sizeof(int));
      if (gd->layers==0) gd->sorted = 0;
   }
   if (gd->sorted)
   {
      for (i=0;i<nlayers;++i) gd->layers[i] = 0;
      for (i=1;i<=nref;++i)
      {
         unq_reflist_hkl (gd->mindx_unq, i, &h, &k, &l);
         ll = l - gd->lmin;
         if (gd->layers[ll]==0) gd->layers[ll] = i;
      }
   }
   gd->cur_layer = gd->lmin;
   sprintf (str,"l=%d",gd->cur_layer);
   xdl_panel_label_set (gd->layer_label, str, 0, 1);
   islide = 0;
   if (gd->lmax>gd->lmin)
   {
      islide = ((gd->cur_layer-gd->lmin)*gd->slider_len)/(gd->lmax-gd->lmin);
   }
   xdl_panel_slider_reset (gd->layer_slider, islide);

   /* Allocate colors */

   if (XDL_color>1)
   {
      colr_cell.flags = DoRed|DoGreen|DoBlue;

      /* Unique reflection  pixel colour */
      colr_cell.red = 10000;
      colr_cell.green = 65535;
      colr_cell.blue = 40000;;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[0];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->unique_pixel = colr_cell.pixel;

      /* Measured spot pixel colour */
      colr_cell.red = 65535;
      colr_cell.green = 65535;
      colr_cell.blue = 2000;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[1];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->measured_pixel = colr_cell.pixel;

      /* I+ spot pixel colour */
      colr_cell.red = 65535;
      colr_cell.green = 6000;
      colr_cell.blue = 3000;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[2];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->plus_pixel = colr_cell.pixel;

      /* I- spot pixel colour */
      colr_cell.red = 20000;
      colr_cell.green = 20000;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[3];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->minus_pixel = colr_cell.pixel;

      /* Unknown sign spot pixel colour */
      colr_cell.red = 65535;
      colr_cell.green = 65535;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[4];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->unk_pixel = colr_cell.pixel;

      /* Circle pixel colour */
      colr_cell.red = 65535;
      colr_cell.green = 0;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[5];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->circle_pixel = colr_cell.pixel;

      /* Axis pixel colour */
      colr_cell.red = 40000;
      colr_cell.green = 40000;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[6];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }

      gd->axis_pixel = colr_cell.pixel;

      /* Layer number pixel  colour */
      colr_cell.red = 65535;
      colr_cell.green = 65535;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[7];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->layer_pixel = colr_cell.pixel;
   }
   else
   {
      gd->unique_pixel = XDL_whitepixel;
      gd->measured_pixel = XDL_whitepixel;
      gd->plus_pixel = XDL_whitepixel;
      gd->minus_pixel = XDL_whitepixel;
      gd->circle_pixel = XDL_whitepixel;
      gd->axis_pixel = XDL_whitepixel;
      gd->layer_pixel = XDL_whitepixel;
   }

   return 0;
}

/* Fortran binding: xdlf_show_unq */

void xdlf_show_unq (ivh, ivhparent, ix, iy, icset, isize, mindx_unq, dstmax,
                    bmatr, title, lent, minw, minh, ierr)

int *ivh, *ivhparent, *ix, *iy, *icset, *isize, *mindx_unq;
float *dstmax, bmatr[3][3];
int *title, *lent, *minw, *minh, *ierr;
{
   float bmat[3][3];
   bmat[0][0] = bmatr[0][0];
   bmat[0][1] = bmatr[1][0];
   bmat[0][2] = bmatr[2][0];
   bmat[1][0] = bmatr[0][1];
   bmat[1][1] = bmatr[1][1];
   bmat[1][2] = bmatr[2][1];
   bmat[2][0] = bmatr[0][2];
   bmat[2][1] = bmatr[1][2];
   bmat[2][2] = bmatr[2][2];
   *ierr = xdl_show_unq (*ivh, *ivhparent, *ix, *iy, *icset, *isize, 
                         *mindx_unq, *dstmax, bmat, 
                         XDL_pointers[*title], *lent, *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_show_unq_rps       **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_show_unq * gd;  /* Pointer to global data area */

   gd = (XDl_show_unq *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, "Select Point on Reciprocal Lattice", 0);
   }
}
/*
*********************************
**     xdl_show_unq_rp1        **
*********************************

Purpose: Repaint paint window (unique data display area)

Author:  John W. Campbell July 1995

*/

void xdl_show_unq_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_show_unq * gd;     /* Pointer to global data area */

   gd = (XDl_show_unq *) global;

   /* Repaint code */

   gd->exposed = 1;
   xdl_show_unq_refresh (gd);

   return;
}

/*
*********************************
**     xdl_show_unq_ep1       **
*********************************

Purpose: Event handling from paint window (unique data display area)

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   int x, y;              /* Mouse position */
   XDl_show_unq *gd;      /* Pointer to the global data area */
   char str[10];          /* Temporary string */
   int islide;            /* Slider position */

   gd = (XDl_show_unq *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
	 {
            printf("***Button 1 pressed****\n");
            gd->x_selected = event->xbutton.x;
            gd->y_selected = event->xbutton.y;
	 }
      break;

      case ClientMessage:
         if (gd->cycle_on)
	 {
            gd->cur_layer ++;
            if (gd->cur_layer>gd->lmax) gd->cur_layer = gd->lmin;
            sprintf (str,"l=%d",gd->cur_layer);
            xdl_panel_label_set (gd->layer_label, str, 0, 1);
            islide = 0;
            if (gd->lmax>gd->lmin)
            {
               islide = ((gd->cur_layer-gd->lmin)*gd->slider_len)
                        /(gd->lmax-gd->lmin);
            }
            xdl_panel_slider_reset (gd->layer_slider, islide);
            xdl_show_unq_refresh (gd);
            gd->pause_done = 0;
            gd->timer_id = XtAppAddTimeOut(XDL_app, gd->cycle_time, 
                     (XtTimerCallbackProc) xdl_show_unq_cycle_dummy,
                     (XtPointer) gd);
            while (gd->pause_done==0) XtAppProcessEvent (XDL_app,XtIMTimer);
            gd->timer_id = 0;
            xdl_show_unq_sendmsg(gd);
	 }
      break;
     
   }
   return;
}

/*
*********************************
**     xdl_show_unq_rp2        **
*********************************

Purpose: Repaint control panel window

Author:  John W. Campbell July 1995

*/

void xdl_show_unq_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}
/*
*********************************
**     xdl_show_unq_cp1        **
*********************************

Purpose: Callback for control panel fulls choice menu

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp1 (iv, item)

int iv;
int item;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   int iflag_cur;        /* Save current flag value */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   iflag_cur = gd->fulls_flag;
   gd->fulls_flag = 2 - item;
   xdl_show_unq_cycle_off(gd);
   if (gd->fulls_flag==iflag_cur) return;
   xdl_show_unq_refresh(gd);
   return;
}
/*
*********************************
**     xdl_show_unq_cp2        **
*********************************

Purpose: Callback for control panel partials choice menu

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp2 (iv, item)

int iv;
int item;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   int iflag_cur;        /* Save current flag value */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   iflag_cur = gd->partials_flag;
   gd->partials_flag = item - 1;
   xdl_show_unq_cycle_off(gd);
   if (gd->partials_flag==iflag_cur) return; 
   xdl_show_unq_refresh(gd);
   return;
}
/*
*********************************
**     xdl_show_unq_cp3        **
*********************************

Purpose: Callback for control panel anomalous choice menu

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp3 (iv, item)

int iv;
int item;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   int iflag_cur;        /* Save current flag value */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   iflag_cur = gd->anom_flag;
   gd->anom_flag = item - 1;
   xdl_show_unq_cycle_off(gd);
   if (gd->anom_flag==iflag_cur) return;
   xdl_show_unq_refresh(gd);
   return;
}
/*
*********************************
**     xdl_show_unq_cp4        **
*********************************

Purpose: Callback for control panel spatials choice menu

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp4 (iv, item)

int iv;
int item;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   int iflag_cur;        /* Save current flag value */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   iflag_cur = gd->spat_flag;
   gd->spat_flag = item - 1;
   xdl_show_unq_cycle_off(gd);
   if (gd->spat_flag==iflag_cur) return;
   xdl_show_unq_refresh(gd);
   return;
}
/*
*********************************
**     xdl_show_unq_cp5        **
*********************************

Purpose: Callback for control panel up/down buttons

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp5 (iv, button)

int iv;
int button;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   int lsav;             /* Save value of current layer */
   char str[10];         /* Temporary string */
   int islide;           /* Slider position */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;

   lsav = gd->cur_layer;
   if (button==1) {gd->cur_layer--;}else{gd->cur_layer++;}
   if (gd->cur_layer>gd->lmax) gd->cur_layer = gd->lmax;
   if (gd->cur_layer<gd->lmin) gd->cur_layer = gd->lmin;
   if (gd->cur_layer==lsav) return;
   sprintf (str,"l=%d",gd->cur_layer);
   xdl_panel_label_set (gd->layer_label, str, 0, 1);

   islide = 0;
   if (gd->lmax>gd->lmin)
   {
      islide = ((gd->cur_layer-gd->lmin)*gd->slider_len)/(gd->lmax-gd->lmin);
   }
   xdl_show_unq_cycle_off(gd);
   xdl_panel_slider_reset (gd->layer_slider, islide);
   xdl_show_unq_refresh (gd);
  
   return;
}

/*
*********************************
**     xdl_show_unq_cp6        **
*********************************

Purpose: Callback for control panel layer slider

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp6 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_show_unq *gd;     /* Pointer to global data area */
   char str[10];         /* Temporary string */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   gd->cur_layer = gd->lmin + value*(gd->lmax-gd->lmin)/slider_len;
   sprintf (str,"l=%d",gd->cur_layer);
   xdl_show_unq_cycle_off(gd);
   xdl_panel_label_set (gd->layer_label, str, 0, 1);
   xdl_show_unq_refresh (gd);
   return;
}
/*
*********************************
**     xdl_show_unq_cp7        **
*********************************

Purpose: Callback for control panel 'cycle'  button

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp7 (iv, button)

int iv;
int button;

{
   XDl_show_unq *gd;     /* Pointer to global data area */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   if (gd->cycle_on)
   {
      gd->cycle_on = 0;
      xdl_panel_button_set (gd->cycle_button, "CYCLE", 0, 1,1);
   }
      else
   {
      gd->cycle_on = 1;
      xdl_panel_button_set (gd->cycle_button, "STATIC", 0, 1,1);
      gd->pause_done = 0;
      gd->timer_id = XtAppAddTimeOut(XDL_app, gd->cycle_time, 
                     (XtTimerCallbackProc) xdl_show_unq_cycle_dummy,
                     (XtPointer) gd);
      while (gd->pause_done==0) XtAppProcessEvent (XDL_app,XtIMTimer);
      gd->timer_id = 0;
      xdl_show_unq_sendmsg(gd);
   }
   return;
}
/*
*********************************
**     xdl_show_unq_cp8        **
*********************************

Purpose: Callback for control panel Postscript button

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp8 (iv, button)

int iv;
int button;

{
   XDl_show_unq *gd;     /* Pointer to global data area */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   xdl_show_unq_cycle_off(gd);
   gd->ps_on = 1;
   xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
   return;
}
/*
*********************************
**     xdl_show_unq_cp9        **
*********************************

Purpose: Callback for control panel Postscript i/o item

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp9 (iv, reply, len, max_rp, flag, iret)

int iv;
char *reply;
int len;
int max_rp;
int flag;
int iret;

{
   XDl_show_unq *gd;     /* Pointer to global data area */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;

   xdl_show_unq_ps (gd, reply, len, flag, iret);

   return;
}
/*
*****************************
**     xdl_show_unq_ps     **
*****************************

Purpose: Postscript output processing

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_ps (gd, reply, len, prompt_num, iret)

XDl_show_unq *gd;    /* Pointer to global data area */
char *reply;         /* Reply to last prompt */
int len;             /* Length of reply */
int prompt_num;      /* Postscript prompt number */
int iret;            /* Return flag =0 blank string
                                    =1 non-blank string
                                    =-1 escape */
{
   int xroot;       /* Root x position for error message */
   int yroot;       /* Root y position for error message */
   int i, j;        /* Temp/loop variables */
   int inc;
   int prompt_only; /* =1 prompt without processing,=0 process then prompt */
   char prompt[67]; /* Temporary prompt string */
   float ss;        /* Symbol scale factor */

   /* Process current reply */
    
   prompt_only = 0;
   xdl_rootxy (gd->wpanel, 200, 20, &xroot, &yroot);
   if (iret==-1)
   {
      if (gd->ps_on>0) fclose (gd->psfil);
      gd->ps_on = 0;
      xdl_panel_io_clear (gd->ps_io);
      return;
   }
   if (prompt_num==1)
   {
      if (iret==0)
      {
         gd->ps_on = 0;
         xdl_panel_io_clear (gd->ps_io);
         return;
      }
      gd->psfil = fopen (reply, "w");
      if (gd->psfil==0)
      {
        i = xdl_popup_notice (xroot, yroot, 
                             "** Cannot open file **", 0, 
                             NULL, -1,
                             "Retry", 0, "Cancel", 0, 2, 1);
        xdl_panel_io_clear (gd->ps_io);
        if (i==1)
        {
           gd->ps_on = 1;
           xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
        }
        else
        {
           gd->ps_on = 0;
        }
        return;
      }
   }
   labl:
   switch (prompt_num)
   {
      case 1:
         prompt_num ++;
         sprintf (prompt,"Colour [y]:");
         xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
         break;
      case 2:
         if (prompt_only==0)
         {
            gd->color_plot = 1;
            if (iret>0)
            {
               if (reply[0]!='y'&&reply[0]!='Y'&&
                   reply[0]!='n'&&reply[0]!='N')
               {
                   i = xdl_popup_notice (xroot, yroot, 
                       "**Error**", 0, 
                       "Reply must be 'y' or 'n'", 0,
                       "Continue", 0, NULL, -1, 2, 1);
                   prompt_num --; prompt_only = 1; goto labl;
               }
               if (reply[0]=='n'||reply[0]=='N') gd->color_plot = 0;
            }
	 }
         prompt_num ++;
         xdl_panel_io_prompt (gd->ps_io, 
                "Current (c), all (a) sections, no. or range [c]:",
                             0, prompt_num);
         break;
      case 3:
         if (prompt_only==0)
         {
            gd->mult_sect = 0;
            gd->ps_minsec = gd->cur_layer;
            gd->ps_incsec = 1;
            if (iret>0)
            {
               i = sscanf (reply, "%d %d %d", &gd->ps_minsec,&gd->ps_maxsec,
                           &inc);
               if (i>0)
	       {
                  if (i>1)
		  {
                     gd->mult_sect = 1;
                     if (gd->ps_maxsec<gd->ps_minsec) 
                         gd->ps_maxsec = gd->ps_minsec;
		  }
                  if (i>2&&inc>1) gd->ps_incsec = inc;
	       }
               else
	       {
                  if (reply[0]!='c'&&reply[0]!='C'&&
                      reply[0]!='a'&&reply[0]!='A')
                  {
                      i = xdl_popup_notice (xroot, yroot, 
                          "**Error**", 0, 
                          "Reply must be 'c', 'a', no. or range", 0,
                         "Continue", 0, NULL, -1, 2, 1);
                      prompt_num --; prompt_only = 1; goto labl;
                  }
                  if (reply[0]=='a'||reply[0]=='A')
                  {
                      gd->mult_sect = 1;
                      gd->ps_minsec = gd->lmin;
                      gd->ps_maxsec = gd->lmax;
		  }
	       }
            }
         }
         prompt_num ++;
         xdl_panel_io_prompt (gd->ps_io, 
                   "Symbol scale factor [1.0]:", 0, prompt_num);
         break;
      case 4:
         if (prompt_only==0)
         {
            gd->ps_symscal = 1.0;
            if (iret>0)
            {
               i = sscanf (reply, "%f", &ss);
               if (i>0)
	       {
                  gd->ps_symscal = ss;
	       }
               else
	       {
                  i = xdl_popup_notice (xroot, yroot, 
                       "**Error**", 0, 
                       "Invalid number", 0,
                       "Continue", 0, NULL, -1, 2, 1);
                  prompt_num --; prompt_only = 1; goto labl;
               }
	    }
         }
         prompt_num ++;
         xdl_panel_io_prompt (gd->ps_io, "Comment:", 0, prompt_num);
         break;
      case 5:
         gd->ps_comment = reply;
         xdl_show_unq_psdraw (gd);  
         xdl_panel_io_clear (gd->ps_io);
         gd->ps_on = 0;
         break;
   }
   return;
}
/*
**********************************
**     xdl_show_unq_psdraw      **
**********************************

Purpose: Write postscript output file

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_psdraw (gd)

XDl_show_unq *gd;    /* Pointer to global data area */

{
   int isec;
   int ixl;
   int iyl;
   float rad;
   float xcent;
   float ycent;
   float srad;
   float rdot;
   float scal;
   float xo;
   float yo;
   float ddx;
   float ddy;

   srad = .75;
   rdot = .3;


   /* Header Information */
   /* ================== */

   /* Ouput start of header */
   fprintf (gd->psfil,"%%!\nnewpath\n");
   fprintf (gd->psfil,"/M {moveto} def\n");
   fprintf (gd->psfil,"/L {lineto} def\n");
   fprintf (gd->psfil,"/R {rlineto} def\n");
   fprintf (gd->psfil,"/S {stroke} def\n");
   fprintf (gd->psfil,"/mm_to_p 2.835 def\n");

   /* Colour table */
   fprintf (gd->psfil,"/black {0 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/red {1 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/unq_colr {.15 1 .61 setrgbcolor} def\n");
   fprintf (gd->psfil,"/meas_colr {1 1 .03 setrgbcolor} def\n");
   fprintf (gd->psfil,"/plus_colr {1 .09 .045 setrgbcolor} def\n");
   fprintf (gd->psfil,"/minus_colr {.31 .31 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/unk_colr {1 1 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/axis_colr {.61 .61 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/layer_colr {1 1 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/prcolr   {0.8 0.8 0.8 setrgbcolor} def\n");
   fprintf (gd->psfil,"/titl_colr {.15 1 .61 setrgbcolor} def\n");

   /* Spot print */
   fprintf (gd->psfil,"\n%% Output spot (xs ys rad_mm S1) %%\n");
   fprintf (gd->psfil,"/S1 {/rad_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/rad {rad_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs ys M xs ys rad 0 360 arc fill S} def\n");

   /* Plus print */
   fprintf (gd->psfil,"\n%% Output plus sign (xs ys hs_mm S2) %%\n");
   fprintf (gd->psfil,"/S2 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"xs ys hs sub M xs ys hs add L\n");
   fprintf (gd->psfil,"S} def\n");

   /* Minus print */
   fprintf (gd->psfil,"\n%% Output minus sign (xs ys hs_mm S3) %%\n");
   fprintf (gd->psfil,"/S3 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"S} def\n");

   /* Unknown sign  print */
   fprintf (gd->psfil,"\n%% Output minus sign (xs ys hs_mm S3) %%\n");
   fprintf (gd->psfil,"/S4 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys hs sub M xs hs add ys hs add L\n");
   fprintf (gd->psfil,"xs hs sub ys hs add M xs hs add ys hs sub L\n");
   fprintf (gd->psfil,"S} def\n");

   /*Break into lines procedure */
   fprintf (gd->psfil,"\n%% Break into lines procedure %%\n");
   fprintf (gd->psfil,"/wordbreak ( ) def /BreakIntoLines\n");
   fprintf (gd->psfil,
            "{/wrlin exch def /linewidth exch def /textstring exch def\n");
   fprintf (gd->psfil,"/breakwidth wordbreak stringwidth pop def\n");
   fprintf (gd->psfil,"/curwidth 0 def /lastwordbreak 0 def\n");
   fprintf (gd->psfil,"/startchar 0 def /restoftext textstring def\n");
   fprintf (gd->psfil,"{restoftext wordbreak search\n");
   fprintf (gd->psfil,"{/nextword exch def pop /restoftext exch def\n");
   fprintf (gd->psfil,"/wordwidth nextword stringwidth pop def\n");
   fprintf (gd->psfil,"curwidth wordwidth add linewidth gt\n");
   fprintf (gd->psfil,"{textstring startchar lastwordbreak startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin /startchar lastwordbreak def\n");
   fprintf (gd->psfil,"/curwidth wordwidth breakwidth add def}\n");
   fprintf (gd->psfil,"{/curwidth curwidth wordwidth add\n");
   fprintf (gd->psfil,"breakwidth add def } ifelse\n");
   fprintf (gd->psfil,
            "/lastwordbreak lastwordbreak nextword length add 1 add def}\n");
   fprintf (gd->psfil,"{pop exit} ifelse }loop\n");
   fprintf (gd->psfil,"/lastchar textstring length def\n");
   fprintf (gd->psfil,"textstring startchar lastchar startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin } def\n");

   if (gd->color_plot)
   {
      fprintf (gd->psfil,"initclip clippath 0 0 0 setrgbcolor fill\n");
   }

   xdl_show_unq_pstitles(gd);

   ddx = 180.0;
   ddy = 180.0;
   xo = 115.0;
   yo = 600.0;

   if (gd->ps_minsec<gd->lmin)gd->ps_minsec = gd->lmin; 
   if (gd->ps_maxsec>gd->lmax)gd->ps_maxsec = gd->lmax; 

   if (gd->mult_sect)   /* Multiple sections */
   {
      iyl = 0;
      ixl = -1;
      rad = 80.0;
      scal = rad/gd->dstmax;
      for (isec=gd->ps_minsec;isec<=gd->ps_maxsec;isec+=gd->ps_incsec)
      {
         ixl++;
         if (ixl>2)
	 {
            ixl = 0;
            iyl++;
            if (iyl>2)
	    {
               xdl_show_unq_pscomments(gd);
               fprintf (gd->psfil, "showpage\n");
               if (gd->color_plot)
               {
                  fprintf (gd->psfil,
                     "initclip clippath 0 0 0 setrgbcolor fill\n");
               }
               xdl_show_unq_pstitles(gd);
               iyl = 0;
	    }
	 }
         xcent = xo + ixl*ddx;
         ycent = yo - iyl*ddy;
         xdl_show_unq_psoutlines (gd, isec, xcent, ycent, scal, rad);
         xdl_show_unq_pslayer (gd, isec, xcent, ycent, scal,
                             .20, .05, .2, .2);
       }
   }
   else  /* Current section only */
   {
       rad = 250.0;
       scal = rad/gd->dstmax;
       xcent = 297.6;
       ycent = 400.0;
       xdl_show_unq_psoutlines (gd, gd->ps_minsec, xcent, ycent, scal, rad);
       xdl_show_unq_pslayer (gd, gd->ps_minsec, xcent, ycent, scal,
                             .3, .05, .3, .5);
   }

   xdl_show_unq_pscomments(gd);

   /* Show page */

   fprintf (gd->psfil,"showpage\n");
   fclose (gd->psfil);
   return;
}
/*
***********************************
**     xdl_show_unq_pstitles     **
***********************************

Purpose: Postscript output of titles etc. for current page

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_pstitles (gd)

XDl_show_unq *gd;    /* Pointer to global data area (R)*/
{
   time_t tm;                /* Structure for timer routines */

   time(&tm);
   fprintf (gd->psfil,"black\n");
   if (gd->color_plot) fprintf (gd->psfil,"titl_colr\n");
   fprintf(gd->psfil,"/Helvetica findfont 16 scalefont setfont\n");
   fprintf(gd->psfil,"297.6 765.0 M\n");
   fprintf(gd->psfil,"(Unique Data Coverage Plot: %s)\n",ctime(&tm));
   fprintf(gd->psfil,"dup stringwidth pop 2 div neg 0 rmoveto show\n");
   if (strlen(gd->title)>0)
   {
      if (gd->color_plot) fprintf (gd->psfil,"prcolr\n");
      fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 740 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->title);
      fprintf (gd->psfil,"490 {297.6 ycomm M dup stringwidth pop 2 div neg 0 rmoveto show /ycomm ycomm 14 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }
   return;
}
/*
***********************************
**     xdl_show_unq_pscomments   **
***********************************

Purpose: Postscript output of comments etc. for current page

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_pscomments (gd)

XDl_show_unq *gd;    /* Pointer to global data area (R)*/
{
   char tmpstr[120];
   static char *full_strings[] =
   {"excluded","included"};            /* Fulls strings */

   static char *partial_strings[] =
   {"excluded","<=2 included","<=3 included","<=nwmax included"};     
                                       /* Partials strings */

   static char *anom_strings[] =
   {"merged","separated"};             /* Anomalous strings */

   static char *spat_strings[] =
   {"excluded","included"};            /* Spatials strings */

   sprintf (tmpstr, "Fulls %s; Partials %s; Anomalous %s; Spatials %s;",
            full_strings[gd->fulls_flag], 
            partial_strings[gd->partials_flag], 
            anom_strings[gd->anom_flag],
            spat_strings[gd->spat_flag]);

   if (gd->color_plot)
   {
      fprintf (gd->psfil, "titl_colr\n");
   }
   else
   {
      fprintf (gd->psfil, "black\n");
   }
   fprintf(gd->psfil,"/Helvetica findfont 15 scalefont setfont\n");
   fprintf(gd->psfil,"297.6 90.0 M\n");
   fprintf(gd->psfil,"(%s)\n",tmpstr);
   fprintf(gd->psfil,"dup stringwidth pop 2 div neg 0 rmoveto show\n");

   if (strlen(gd->ps_comment)>0)
   {
      if (gd->color_plot)
      {
         fprintf (gd->psfil, "prcolr\n");
      }
      else
      {
         fprintf (gd->psfil, "black\n");
      }
      fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 56 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->ps_comment);
      fprintf (gd->psfil,"490 {297.6 ycomm M dup stringwidth pop 2 div neg 0 rmoveto show /ycomm ycomm 14 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }

   return;
}
/*
*************************************
**     xdl_show_unq_psoutlines     **
*************************************

Purpose: Postscript output of outlines etc. for requested layer

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_psoutlines (gd, ll, xcent, ycent, scal, radius)

XDl_show_unq *gd;    /* Pointer to global data area (R)*/
int ll;              /* Layer number (R)*/
float xcent;         /* 'x' centre of current plot on PS page (R)*/
float ycent;         /* 'y' centre of current plot on PS page (R)*/
float scal;          /* Scaling factor based on gd->dstmax (R)*/
float radius;        /* Radius of main circle (R)*/
{
   float d;
   float angl;
   float xend, yend;
   float xa1, ya1;
   float xa2, ya2;
   char str[20];

   fprintf (gd->psfil,"black\n");
   if (gd->color_plot)fprintf (gd->psfil,"red\n"); 

   if (ll==0)
   {
      fprintf (gd->psfil,"/Helvetica findfont 10 scalefont setfont\n");
      fprintf (gd->psfil,"1 setlinewidth\n");
      if (gd->color_plot) fprintf (gd->psfil,"axis_colr\n");
      angl = atan2 ((double) gd->bmatr[1][0], (double) gd->bmatr[0][0]);
      xdl_show_unq_arrow_coords ((radius + 20.0),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent, ycent,
               xcent+xend, ycent+yend);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent+xa1, ycent+ya1,
               xcent+xend, ycent+yend);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent+xa2, ycent+ya2,
               xcent+xend, ycent+yend);
      angl = angl + 0.05;
      xdl_show_unq_arrow_coords ((radius + 12),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      fprintf (gd->psfil, "%.2f %.2f M (a*) show S\n", xcent+xend-8.0,
                          ycent+yend-2.0);
      angl = atan2 ((double) gd->bmatr[1][1], (double) gd->bmatr[0][1]);
      xdl_show_unq_arrow_coords ((radius + 20),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent, ycent,
               xcent+xend, ycent+yend);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent+xa1, ycent+ya1,
               xcent+xend, ycent+yend);
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", xcent+xa2, ycent+ya2,
               xcent+xend, ycent+yend);
      angl = angl + 0.05;
      xdl_show_unq_arrow_coords ((radius + 12),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      fprintf (gd->psfil, "%.2f %.2f M (b*) show S\n", xcent+xend-8.0,
                          ycent+yend-2.0);
   }

   /* Output layer number */

   fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
   if (gd->color_plot) fprintf (gd->psfil,"layer_colr\n");
   sprintf (str,"l=%d",ll);
   fprintf (gd->psfil, "%.2f %.2f M (%s) show S\n", xcent-radius+5.0,
            ycent-radius+5.0, str);

   /* Plot circles */
   fprintf(gd->psfil,"%d setlinewidth black\n",gd->lw_surround);
   if (gd->color_plot)
   {
      fprintf (gd->psfil,"red\n");
   }
   fprintf(gd->psfil,"%.2f %.2f M\n",
           xcent+scal*gd->dstmax+gd->lw_surround-1,ycent);
   fprintf(gd->psfil,"%.2f %.2f %.2f 0 360 arc S\n", xcent,ycent,
           scal*gd->dstmax+gd->lw_surround-1);
   d = ll*gd->bmatr[2][2];
   d = sqrt(gd->dstmax*gd->dstmax - d*d);
   fprintf(gd->psfil,"1 setlinewidth\n");
   fprintf(gd->psfil,"%.2f %.2f M\n",xcent+scal*d,ycent);
   fprintf(gd->psfil,"%.2f %.2f %.2f 0 360 arc S\n", xcent,ycent,
           scal*d);
   return;
}
/*
**********************************
**     xdl_show_unq_pslayer     **
**********************************

Purpose: Postscript output of  spots for requested layer

Author:  John W. Campbell, September 1995

*/

void xdl_show_unq_pslayer (gd, ll, xcen, ycen, scal, srad, rdot, sym_lw,
                           sym_wid)

XDl_show_unq *gd;    /* Pointer to global data area (R)*/
int ll;              /* Layer number (R)*/
float xcen;          /* 'x' centre of current plot on PS page (R)*/
float ycen;          /* 'y' centre of current plot on PS page (R)*/
float scal;          /* r.l. units to PS points scale factor (R)*/
float srad;          /* Radius/half-size for symbols in mm (R)*/
float rdot;          /* Radius for unique dots in mm (R)*/
float sym_lw;        /* Line width for + and - symbols (R)*/
float sym_wid;       /* Half width for + and - symbols (R)*/

{
   int i;
   int nref;
   int icolr;
   int ifirst;
   int h, k, l;
   int nfull_p;
   int nfull_m;
   int npart2_p;
   int npart2_m;
   int npart3_p;
   int npart3_m;
   int npartn_p;
   int npartn_m;
   int nunk_p;
   int nunk_m;
   int nunk_u;
   int nsfull_p;
   int nsfull_m;
   int nspart2_p;
   int nspart2_m;
   int nspart3_p;
   int nspart3_m;
   int nspartn_p;
   int nspartn_m;
   int iplus;
   int iminus;
   int iunknown;
   int disp;
   float xr, yr, zr;
   float xp, yp;

   /* Draw unique reflections data */

   nref = unq_reflist_numrefs(gd->mindx_unq);
   if (nref<=0) return;
   fprintf (gd->psfil,"black %.2f setlinewidth\n", sym_lw);
   if (gd->color_plot) fprintf (gd->psfil,"unq_colr\n");
   icolr = 5;
   ifirst = 1;
   if (gd->sorted)
   { 
      ifirst = gd->layers[ll-gd->lmin];
      if (ifirst==0) return;
   }
   for (i=ifirst;i<=nref;++i)
   {
      unq_reflist_hkl (gd->mindx_unq, i, &h, &k, &l);
      if (gd->sorted&&l>ll) break;
      if (l!=ll) continue;
      unq_reflist_xyz (gd->mindx_unq, i, &xr, &yr, &zr);
      xp = xcen + xr*scal;
      yp = ycen + yr*scal;
      unq_reflist_get (gd->mindx_unq, i, &nfull_p, &nfull_m,
                       &npart2_p, &npart2_m, &npart3_p, &npart3_m,
                       &npartn_p, &npartn_m,
                       &nunk_p, &nunk_m, &nunk_u,
                       &nsfull_p, &nsfull_m,
                       &nspart2_p, &nspart2_m, &nspart3_p, &nspart3_m,
                       &nspartn_p, &nspartn_m);
      iplus = 0;
      iminus = 0;
      iunknown = 0;
      if (gd->fulls_flag)
      {
         if (nfull_p>0) iplus = 1;
         if (nunk_p>0) iplus = 1;
         if (gd->spat_flag&&nsfull_p>0) iplus = 1;
         if (nfull_m>0) iminus = 1;
         if (nunk_m>0) iminus = 1;
         if (gd->spat_flag&&nsfull_m>0) iminus = 1;
         if (nunk_u>0) iunknown = 1;
      }
      if (gd->partials_flag>0)
      {
         switch (gd->partials_flag)
	 {
	     case 1:
             if (npart2_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p>0) iplus = 1;
             if (npart2_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m>0) iminus = 1;
             break;

	     case 2:
             if (npart2_p+npart3_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p+nspart3_p>0) iplus = 1;
             if (npart2_m+npart3_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m+nspart3_m>0) iminus = 1;
             break;

	     case 3:
             if (npart2_p+npart3_p+npartn_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p+nspart3_p+nspartn_p>0) iplus = 1;
             if (npart2_m+npart3_m+npartn_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m+nspart3_m+nspartn_m>0) iminus = 1;
             break;
          }
      }
      disp = 0;
      if (gd->anom_flag)
      {
         if (iplus>0&&iminus>0)
	 {
            if (gd->color_plot)
	    {
               if (icolr!=1)
               {
                  fprintf(gd->psfil,"meas_colr\n");
                  icolr = 1;
	       }
               fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                       srad*gd->ps_symscal);
	    }
            else
	    {
               fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                       srad*gd->ps_symscal);
            }
            disp = 1;        
         }
         else
	 {
            if (iplus>0&&iunknown==0)
	    {
               if (gd->color_plot)
	       {
                  if (icolr!=2)
                  {
                     fprintf(gd->psfil,"plus_colr\n");
                     icolr = 2;
	          }
                  fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                          srad*gd->ps_symscal);
	       }
               else
	       {
                  fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n", xp, yp, 
                          sym_wid*gd->ps_symscal);
               }
               disp = 1;
	    }
            if (iminus>0&&iunknown==0)
	    {
               if (gd->color_plot)
	       {
                  if (icolr!=3)
                  {
                     fprintf(gd->psfil,"minus_colr\n");
                     icolr = 3;
	          }
                  fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                           srad*gd->ps_symscal);
	       }
               else
	       {
                  fprintf (gd->psfil,"%.2f %.2f %.3f S3 S\n", xp, yp, 
                           sym_wid*gd->ps_symscal);
               }
               disp = 1;
	    }
            if (iunknown>0)
	    {
               if (gd->color_plot)
	       {
                  if (icolr!=4)
                  {
                     fprintf(gd->psfil,"unk_colr\n");
                     icolr = 4;
	          }
                  fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                           srad*gd->ps_symscal);
	       }
               else
	       {
                  fprintf (gd->psfil,"%.2f %.2f %.3f S4 S\n", xp, yp, 
                           sym_wid*gd->ps_symscal);
               }
               disp = 1;
	    }
	 }
      }
      else
      {
         if (iplus+iminus+iunknown>0)
	 {
            if (gd->color_plot)
	    {
               if (icolr!=1)
               {
                  fprintf(gd->psfil,"meas_colr\n");
                  icolr = 1;
	       }
               fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                        srad*gd->ps_symscal);
	    }
            else
	    {
               fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                        srad*gd->ps_symscal);
            }
            disp = 1;
	 }
      }
      if (disp==0)
      {
         if (gd->color_plot)
	 {
            if (icolr!=5)
            {
               fprintf(gd->psfil,"unq_colr\n");
               icolr = 5;
	    }
            fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                     rdot*gd->ps_symscal);
	 }
         else
	 {
            fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", xp, yp, 
                     rdot*gd->ps_symscal);
         }
      }
   }
   return;
}

/*
*********************************
**     xdl_show_unq_cp10       **
*********************************

Purpose: Callback for control panel cycle speed slider

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cp10 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_show_unq *gd;     /* Pointer to global data area */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   gd->cycle_time =  2500 - value*(2400)/slider_len;
   return;
}
/*
***********************************
**     xdl_show_unq_on_off       **
***********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_show_unq *gd;        /* Pointer to global data area */

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      xdl_active_strip_on (gd->wact, "Select Point on Reciprocal Lattice", 0);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
   }
   return;
}
/*
************************************
**     xdl_show_unq_tidy_up       **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  ...

*/

void xdl_show_unq_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_show_unq *gd;    /* Pointer to the global data area */

   gd = (XDl_show_unq *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   if (gd->layers!=0) free (gd->layers);
   return;
}
/*
**************************************
**     xdl_show_unq_cycle_dummy     **
**************************************

Purpose: Dummy routine for timer callback

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cycle_dummy (client_data, tim_id)

XtPointer client_data;
XtIntervalId * tim_id;
{
   XDl_show_unq *gd;    /* Pointer to the global data area */

   gd = (XDl_show_unq *) client_data;
   gd->pause_done = 1;
   return;
}
/*
**************************************
**     xdl_show_unq_cycle_off        **
**************************************

Purpose: Turn off automatic cycling

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_cycle_off (gd)

XDl_show_unq *gd;          /* Pointer to global data area */
{

   if (gd->cycle_on)
   {
      gd->cycle_on = 0;
      xdl_panel_button_set (gd->cycle_button, "CYCLE", 0, 1,1);
   }
   return;
}
/*
*********************************
**     xdl_show_unq_sendmsg    **
*********************************

Purpose: Send message event to paint window

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_sendmsg (gd)

XDl_show_unq *gd;          /* Pointer to global data area */
{
   XClientMessageEvent mesg;      /* Client message use in stepping through
                                     displayed layers */
   mesg.type = ClientMessage;
   mesg.window = gd->wpaint;
   mesg.message_type = XA_STRING;
   mesg.format = 8;
   XSendEvent (XDL_display, gd->wpaint, False, 0L, (XEvent *) &mesg);
   return;
}
/*
**************************************
**     xdl_show_unq_arrow_coords    **
**************************************

Purpose: Calculate relative coordinates for drawing an arrow

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_arrow_coords (alen, angl, hlen, hwid, xend, yend,
                                xa1, ya1, xa2, ya2)

float alen;           /* Arrow length (R)*/
float angl;           /* Arrow angle in radians (0.0=along x) (R)*/
float hlen;           /* Length of arrow head (R)*/
float hwid;           /* Width of arrow head (R)*/
float *xend, *yend;   /* Returns coordinates of arrow end (W)*/
float *xa1, *ya1;     /* Coordinates of start of 1'st arrow head tail (W)*/
float *xa2, *ya2;     /* Coordinates of start of 2'nd arrow head tail (W)*/
{
   float xx, yy;
   float dx, dy;
   float cth, sth;;

   cth = cos((double)angl);
   sth = sin((double)angl);
   xx = alen;
   yy = 0.0;
   dx = xx*cth - yy*sth;
   dy = xx*sth + yy*cth;
   *xend = dx;
   *yend = dy;
   xx = alen - hlen;
   yy = hwid/2.0;
   dx = xx*cth - yy*sth;
   dy = xx*sth + yy*cth;
   *xa1 = dx;
   *ya1 = dy;
   xx = alen - hlen;
   yy = -hwid/2.0;
   dx = xx*cth - yy*sth;
   dy = xx*sth + yy*cth;
   *xa2 = dx;
   *ya2 = dy;
   return;
}
/*
********************************
**     xdl_show_unq_refresh   **
********************************

Purpose: Refresh displayed unique data display area

Author:  John W. Campbell, July 1995

*/

void xdl_show_unq_refresh (gd)

XDl_show_unq *gd;          /* Pointer to global data area */
{
   int i;
   int nref;
   int h,k,l;
   int xpix;
   int ypix;
   int ifirst;
   int rad_cur;
   int nfull_p;
   int nfull_m;
   int npart2_p;
   int npart2_m;
   int npart3_p;
   int npart3_m;
   int npartn_p;
   int npartn_m;
   int nunk_p;
   int nunk_m;
   int nunk_u;
   int nsfull_p;
   int nsfull_m;
   int nspart2_p;
   int nspart2_m;
   int nspart3_p;
   int nspart3_m;
   int nspartn_p;
   int nspartn_m;
   int iplus;
   int iminus;
   int iunknown;
   int disp;
   int icolr;
   int font_off;
   int ascent;
   int descent;
   int xt;
   int yt;
   int tw;
   float xr;
   float yr;
   float zr;
   float d;
   float angl;
   float xend, yend;
   float xa1, ya1;
   float xa2, ya2;
   char str[20];

   /* Clear window (black background) */

   XClearWindow (XDL_display, gd->wpaint);

   /* Axis annotation if zero layer */
   if (gd->cur_layer==0)
   {
      font_off = 1;
      ascent = XDL_fonts[font_off]->ascent;
      descent = XDL_fonts[font_off]->descent;
      XSetForeground (XDL_display, gd->paint_gc, gd->axis_pixel);
      angl = atan2 ((double) gd->bmatr[1][0], (double) gd->bmatr[0][0]);
      xdl_show_unq_arrow_coords ((float) (gd->rad_surround + 22),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 gd->xcen, gd->ycen, 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 nint(gd->xcen+xa1), nint(gd->ycen-ya1), 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 nint(gd->xcen+xa2), nint(gd->ycen-ya2), 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      angl = angl + 0.05;
      xdl_show_unq_arrow_coords ((float) (gd->rad_surround + 14),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      xt = nint(gd->xcen + xend);
      yt = nint(gd->ycen - yend);
      tw = XTextWidth (XDL_fonts[font_off], "a*", 2);
      XDrawString (XDL_display, gd->wpaint, gd->paint_gc, 
                   xt - tw/2, yt + (ascent-descent)/2,
                   "a*", 2);
      angl = atan2 ((double) gd->bmatr[1][1], (double) gd->bmatr[0][1]);
      xdl_show_unq_arrow_coords ((float) (gd->rad_surround + 22),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 gd->xcen, gd->ycen, 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 nint(gd->xcen+xa1), nint(gd->ycen-ya1), 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      XDrawLine (XDL_display, gd->wpaint, gd->paint_gc,
                 nint(gd->xcen+xa2), nint(gd->ycen-ya2), 
                 nint(gd->xcen+xend), nint(gd->ycen-yend));
      angl = angl + 0.05;
      xdl_show_unq_arrow_coords ((float) (gd->rad_surround + 14),
                                angl, 8.0, 6.0, &xend, &yend, 
                                &xa1, &ya1, &xa2, &ya2);
      xt = nint(gd->xcen + xend);
      yt = nint(gd->ycen - yend);
      tw = XTextWidth (XDL_fonts[font_off], "b*", 2);
      XDrawString (XDL_display, gd->wpaint, gd->paint_gc, 
                   xt - tw/2, yt + (ascent-descent)/2,
                   "b*", 2);
   }

   /* Output layer number */

   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[2]->fid);
   XSetForeground (XDL_display, gd->paint_gc, gd->layer_pixel);
   sprintf (str,"l=%d",gd->cur_layer);
   XDrawString (XDL_display, gd->wpaint, gd->paint_gc, 
                20, gd->area_height-20, str, strlen(str));

   /* Reset default font */

   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);

   /* Draw circles */

   XSetForeground (XDL_display, gd->paint_gc, gd->circle_pixel);
   XSetLineAttributes (XDL_display, gd->paint_gc, gd->lw_surround,
                       LineSolid, CapButt, JoinBevel);
   XDrawArc (XDL_display, gd->wpaint, gd->paint_gc,
             gd->xcen - gd->rad_surround - gd->lw_surround,
             gd->ycen - gd->rad_surround - gd->lw_surround,
             2*(gd->rad_surround + gd->lw_surround),
             2*(gd->rad_surround + gd->lw_surround),
             0, 23040); 
   d = gd->cur_layer*gd->bmatr[2][2];
   d = sqrt(gd->dstmax*gd->dstmax - d*d);
   rad_cur = nint(d*gd->scal);
   XSetLineAttributes (XDL_display, gd->paint_gc, 1,
                       LineSolid, CapButt, JoinBevel);
   XDrawArc (XDL_display, gd->wpaint, gd->paint_gc,
             gd->xcen - rad_cur - 1,
             gd->ycen - rad_cur - gd->lw_surround,
             2*(rad_cur + 1),
             2*(rad_cur + 1),
             0, 23040);

   /* Draw unique reflections data */

   nref = unq_reflist_numrefs(gd->mindx_unq);
   if (nref<=0) return;
   XSetForeground (XDL_display, gd->paint_gc, gd->unique_pixel);
   icolr = 5;
   ifirst = 1;
   if (gd->sorted)
   { 
      ifirst = gd->layers[gd->cur_layer-gd->lmin];
      if (ifirst==0) return;
   }
   for (i=ifirst;i<=nref;++i)
   {
      unq_reflist_hkl (gd->mindx_unq, i, &h, &k, &l);
      if (gd->sorted&&l>gd->cur_layer) break;
      if (l!=gd->cur_layer) continue;
      unq_reflist_xyz (gd->mindx_unq, i, &xr, &yr, &zr);
      xpix = nint (gd->xcen + gd->scal*xr);
      ypix = nint (gd->ycen - gd->scal*yr);
      unq_reflist_get (gd->mindx_unq, i, &nfull_p, &nfull_m,
                       &npart2_p, &npart2_m, &npart3_p, &npart3_m,
                       &npartn_p, &npartn_m,
                       &nunk_p, &nunk_m, &nunk_u, 
                       &nsfull_p, &nsfull_m,
                       &nspart2_p, &nspart2_m, &nspart3_p, &nspart3_m,
                       &nspartn_p, &nspartn_m);
      iplus = 0;
      iminus = 0;
      iunknown = 0;
      if (gd->fulls_flag)
      {
         if (nfull_p>0) iplus = 1;
         if (nunk_p>0) iplus = 1;
         if (gd->spat_flag&&nsfull_p>0) iplus = 1;
         if (nfull_m>0) iminus = 1;
         if (nunk_m>0) iminus = 1;
         if (gd->spat_flag&&nsfull_m>0) iminus = 1;
         if (nunk_u>0) iunknown = 1;
      }
      if (gd->partials_flag>0)
      {
         switch (gd->partials_flag)
	 {
	     case 1:
             if (npart2_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p>0) iplus = 1;
             if (npart2_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m>0) iminus = 1;
             break;

	     case 2:
             if (npart2_p+npart3_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p+nspart3_p>0) iplus = 1;
             if (npart2_m+npart3_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m+nspart3_m>0) iminus = 1;
             break;

	     case 3:
             if (npart2_p+npart3_p+npartn_p>0) iplus = 1;
             if (gd->spat_flag&&nspart2_p+nspart3_p+nspartn_p>0) iplus = 1;
             if (npart2_m+npart3_m+npartn_m>0) iminus = 1;
             if (gd->spat_flag&&nspart2_m+nspart3_m+nspartn_m>0) iminus = 1;
             break;
          }
      }
      disp = 0;
      if (gd->anom_flag)
      {
         if (iplus>0&&iminus>0)
	 {
            if (icolr!=1)
            {
               XSetForeground (XDL_display, gd->paint_gc, gd->measured_pixel);
               icolr = 1;
	    }
            XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                      xpix-1, ypix-1, 3, 3, 0, 23040);
            disp = 1;        
         }
         else
	 {
            if (iplus>0&&iunknown==0)
	    {
               if (icolr!=2)
               {
                  XSetForeground (XDL_display, gd->paint_gc, gd->plus_pixel);
                  icolr = 2;
	       }
               XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                         xpix-1, ypix-1, 3, 3, 0, 23040);
               disp = 1;
	    }
            if (iminus>0&&iunknown==0)
	    {
               if (icolr!=3)
               {
                  XSetForeground (XDL_display, gd->paint_gc, gd->minus_pixel);
                  icolr = 3;
	       }
               XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                         xpix-1, ypix-1, 3, 3, 0, 23040);
               disp = 1;
	    }
            if (iunknown>0)
	    {
               if (icolr!=4)
               {
                  XSetForeground (XDL_display, gd->paint_gc, gd->unk_pixel);
                  icolr = 4;
	       }
               XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                         xpix-1, ypix-1, 3, 3, 0, 23040);
               disp = 1;
            }
	 }
      }
      else
      {
         if (iplus+iminus+iunknown>0)
	 {
            if (icolr!=1)
            {
               XSetForeground (XDL_display, gd->paint_gc, gd->measured_pixel);
               icolr = 1;
	    }
            XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                      xpix-1, ypix-1, 3, 3, 0, 23040);
            disp = 1;
	 }
      }
      if (disp==0)
      {
         if (icolr!=5)
         {
            XSetForeground (XDL_display, gd->paint_gc, gd->unique_pixel);
            icolr = 5;
	 }
      /*   XFillArc (XDL_display, gd->wpaint, gd->paint_gc, 
                   xpix-1, ypix-1, 3, 3, 0, 23040); */
           XDrawPoint (XDL_display, gd->wpaint, gd->paint_gc, xpix, ypix);
      }
   }
   XFlush (XDL_display);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Return selected spot position - xdl_show_unq_getdata
This routine returns the position of the last mouse selected (spot) position
from a show unique data view-object.
-end*/
/*
***********************************
**     xdl_show_unq_getdata     **
***********************************

Purpose: Get returned data ... from the show_unq

Author:  John W. Campbell, July 1995

*/
/*-Fortran:
         CALL XDLF_SHOW_UNQ_GETDATA (IVH, X, Y, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
X        (R)  'X' coordinate
Y        (W)  'Y' coordinate
IERR     (W)  Returns status from xdl_show_unq_gedata call
-end*/

/*-C:*/
int xdl_show_unq_getdata (int vh, float * x, float * y)
/*end*/
 
/*-Parameters:
int vh;          View-object handle (R)
float *x;        Returned 'x' coordinate (W)
float *y;        Returned 'y' coordinate (W)
-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  ...
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_show_unq *gd;    /* pointer to the global data area */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_show_unq_getdata",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2112)
   {
      xdl_view_type_errmsg ("xdl_show_unq_getdata", vh, "xdl_show_unq");
      return 2;
   }

   gd = (XDl_show_unq *) XDL_view_objects[iv]->gd;

   /* Return the required data */

   *x = (float) gd->x_selected;
   *y = (float) gd->y_selected;

   return 0;
}

/* Fortran binding xdlf_show_unq_getdata */

void xdlf_show_unq_getdata (ivh, x, y, ierr)
int *ivh, *ierr;
float *x, *y;
{
   *ierr = xdl_show_unq_getdata (*ivh, x, y);
   return;
}
