/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Text Table View-Object Routines
-end*/
 
/*-Intro:
The text table view-object provides an area for the simple output of
text from a program. Text strings may be output at any character position
within the window. There is no scrolling. An alternative option allows
for the definition of active cells which respond to mouse input.
-end*/
 
/*-Section: Creating and Handling a Text Table View-Object
-end*/
 

/*Routines List:
The table view-object has the following routines available:

   xdl_text_table               Create a new text table view-object
   xdl_text_table_cell          Create a new text cell view-object 
   xdl_text_table_text          Output a text string to a text table
   xdl_text_table_symbol        Output a symbol to a text table
   xdl_text_table_clear         Clear the text table
   xdl_text_table_clearsect     Clear a section of the text table
   xdl_text_table_input_message Change message in active strip
   xdl_text_table_define_cell   Define an input cell
   xdl_text_table_clear_cell    Clear input cell(s)
   xdl_text_table_getinp        Get selected input cell number
   xdl_text_table_getsize       Get size of a text table view-object
   xdl_text_table_getcsize      Get size of a text cell view-object 
*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"

#define MARG_W 2
#define MARG_H 2
#define MIN_ACT_STRIP 100      /* Minimum width for active strip */

/* Define routine types */
int xdl_text_table();
int xdl_text_table_cell();
void xdl_text_table_rps();
void xdl_text_table_rp1();
void xdl_text_table_ep1();
void xdl_text_table_clc();
void xdl_text_table_drawch();
void xdl_text_table_drawsymb();
int xdl_text_table_incell();
void xdl_text_table_drawcell();
void xdl_text_table_on_off();
void xdl_text_table_tidy_up();
int xdl_text_table_text();
int xdl_text_table_symbol();
int xdl_text_table_clear();
int xdl_text_table_clearsect();
int xdl_text_table_input_message();
int xdl_text_table_define_cell();
int xdl_text_table_clear_cell();
int xdl_text_table_getinp();
void xdl_text_table_getsize();
void xdl_text_table_getcsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_text_table xdlf_text_table_
#define xdlf_text_table_cell xdlf_text_table_cell_
#define xdlf_text_table_text xdlf_text_table_text_
#define xdlf_text_table_symbol xdlf_text_table_symbol_
#define xdlf_text_table_clear xdlf_text_table_clear_
#define xdlf_text_table_clearsect xdlf_text_table_clearsect_
#define xdlf_text_table_input_message xdlf_text_table_input_message_
#define xdlf_text_table_define_cell xdlf_text_table_define_cell_
#define xdlf_text_table_clear_cell xdlf_text_table_clear_cell_
#define xdlf_text_table_getinp xdlf_text_table_getinp_
#define xdlf_text_table_getsize xdlf_text_table_getsize_
#define xdlf_text_table_getcsize xdlf_text_table_getcsize_

#endif

#if  LINKTYP == 2

#define xdlf_text_table XDLF_TEXT_TABLE
#define xdlf_text_table_cell XDLF_TEXT_TABLE_CELL
#define xdlf_text_table_text XDLF_TEXT_TABLE_TEXT
#define xdlf_text_table_symbol XDLF_TEXT_TABLE_SYMBOL
#define xdlf_text_table_clear XDLF_TEXT_TABLE_CLEAR
#define xdlf_text_table_clearsect XDLF_TEXT_TABLE_CLEARSECT
#define xdlf_text_table_input_message XDLF_TEXT_TABLE_INPUT_MESSAGE
#define xdlf_text_table_define_cell XDLF_TEXT_TABLE_DEFINE_CELL
#define xdlf_text_table_clear_cell XDLF_TEXT_TABLE_CLEAR_CELL
#define xdlf_text_table_getinp XDLF_TEXT_TABLE_GETINP
#define xdlf_text_table_getsize XDLF_TEXT_TABLE_GETSIZE
#define xdlf_text_table_getcsize XDLF_TEXT_TABLE_GETCSIZE

#endif

void xdlf_text_table();
void xdlf_text_table_cell();
void xdlf_text_table_text();
void xdlf_text_table_symbol();
void xdlf_text_table_clear();
void xdlf_text_table_clearsect();
void xdlf_text_table_input_message();
void xdlf_text_table_define_cell();
void xdlf_text_table_clear_cell();
void xdlf_text_table_getinp();
void xdlf_text_table_getsize();
void xdlf_text_table_getcsize();


/*-Routine: Create a text table view-object - xdl_text_table
The routine xdl_text_table (xdlf_text_table) is used to create a new text
table window view-object. The user may specify the size of the window in
pixels or in terms of the required numbers of rows and columns of
characters.
-end*/
/*
*****************************
**    xdl_text_table       **
*****************************

Purpose: Display a window for simple program text table output

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE (IVH, IVHPARENT, IX, IY, ICSET, NCOLS, 
        +                      NROWS, MINW, MINH, IFONT, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
MINW      (R)   Minimum width in pixels (see min_width)
MINH      (R)   Minimum height in pixels (see min_height)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
IERR      (W)   Returns status from xdl_text_table call
-end*/

/*-C:*/
int xdl_text_table (vh, vh_parent, x, y, cset, ncols, nrows, min_width, 
                   min_height, font_type)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the i/o window view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - 
                             normally 0 (R)*/
int       ncols;          /* Number of columns of characters (if 0 then
                             this will be calculated from min_width which
                             must then be given a value > 0. (R)*/
int       nrows;          /* Number of rows of characters (if 0 then
                             this will be calculated from min_height which
                             must then be given a value > 0. (R)*/
int       min_width;      /* Minimum window width in pixels. If this is
                             greater than the width calculated from the
                             number of character columns requirement then 
                             this value will be used as the window width 
                             (R)*/
int       min_height;     /* Minimum window height in pixels. If this is
                             greater than the height calculated from the
                             number of character rows requirement then 
                             this value will be used as the window height
                             (R)*/
int       font_type;      /* Font type (1-5) for very_small, small, medium,
                             large or extra-large font. (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to allocate required memory
-end*/
{
   return xdl_text_table_cell (vh, vh_parent, x, y, cset, 0, 0,
              ncols, nrows, min_width, min_height, font_type, 0);
}

/* Fortran binding: xdlf_text_table */

void xdlf_text_table (ivh, ivhparent, ix, iy, icset, ncols, nrows, minw, minh,
                     ifont, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *ncols, *nrows, *minw, *minh,
    *ifont, *ierr;
{
   *ierr = xdl_text_table_cell (*ivh, *ivhparent, *ix, *iy, *icset, 0, 0,
                          *ncols, *nrows, *minw, *minh, *ifont, 0);
   return;
}
/*-Routine: Create a text cell view-object - xdl_text_table_cell
The routine xdl_text_table_cell (xdlf_text_table_cell) is used to create 
a new text table window view-object. In contrast with the standard text table,
the user may define active cells within the window which respond
to mouse input. An active strip may be displayed if required. The user may 
specify the size of the window in pixels or in terms of the required numbers 
of rows and columns of characters.
-end*/
/*
*****************************
**    xdl_text_table_cell  **
*****************************

Purpose: Display a window for simple program text cell output/input

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_CELL (IVH, IVHPARENT, IX, IY, ICSET, IACT, 
        +                MAX_CELLS, NCOLS, NROWS, MINW, MINH, IFONT, 
        +                HIGHL_FN, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
IACT      (R)   Display active strip flag =1 yes, =0 no
MAX_CELLS (R)   Maximum no. of input cells needed (0 if none)
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
MINW      (R)   Minimum width in pixels (see min_width)
MINH      (R)   Minimum height in pixels (see min_height)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
HIGHL_FN  (R)   Subroutine to call when highlighting/unhighlighting 
                takes place in a cell to allow additional user defined
                actions to be carried out.
                
                Call wiil be:    CALL HIGHL_FN (IVH, ICELL, IFLAG)

                Parameters:   IVH    (R)     The file handle
                              ICELL  (R)     The cell number
                              IFLAG  (R)     Flag =1 highlight on 
                                                  =0 highlight off
                Note: Will still be called when not in input selection 
                      mode even if the highlight option selected for this 
                      mode via CALL XDLF_TEXT_TABLE_DEFINE_CELL is 0.

IERR      (W)   Returns status from xdl_text_table call
-end*/


/* Define structure to hold cells info */

typedef struct
{
   unsigned char defined;          /* Cell defined flag = 1 yes, =0 no */
   unsigned char highlight;        /* Highlight option (when in input 
                                      selection mode.
                                      = 1 coloured bg (underline if not colour)
                                      = 2 coloured bg (surround box if not 
                                          colour)
                                      = 3 underline
                                      = 4 surround box */
   unsigned char highlight_nosel;  /* Highlight option when not in input
                                      selection mode (as above but may
                                      be 0 for no highlighting in this
                                      mode */                                      
   unsigned long line_pixel;       /* Line colour pix for underline/box 
                                      highlight */
   unsigned long highlight_pixel;  /* Background highlight pixel */
   unsigned short row1;            /* First row in cell */
   unsigned short col1;            /* First column in cell */
   unsigned short row2;            /* Final row in cell */
   unsigned short col2;            /* Final column in cell */
   unsigned short xmin;            /* Min 'x' pixel */
   unsigned short xmax;            /* Max 'x' pixel */
   unsigned short ymin;            /* Min 'y' pixel */
   unsigned short ymax;            /* Max 'y' pixel */
}
   XDl_text_table_cell;   /* NOTE rows/columns verified to be within range */

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wact;                      /* Active strip window id */
   Window wtab;                      /* Table area window id */
   GC          paint_gc;             /* GC for text paint window */
   GC          paint_bold_gc;        /* GC for text paint window bold text */
   int         vh;                   /* View object handle */
   char        *text;                /* Storage of the text (/symbols) for the 
                                        table */
   char        *flag;                /* Storage or normal/bold/symbol/colour
                                        print flags 
                                            0/1  Black   normal/bold text
                                            2/3  Red     normal/bold text
                                            4/5  Yellow  normal/bold text
                                            6/7  Green   normal/bold text
                                            8/9  Cyan    normal/bold text
                                          10/11  Blue    normal/bold text
                                          12/13  Magenta normal/bold text
                                             20  Black   symbol
                                             21  Red     symbol
                                             22  Yellow  symbol
                                             23  Green   symbol
                                             24  Cyan    symbol
                                             25  Blue    symbol
                                             26  Magenta symbol
                                       */
   int         max_toff;             /* Maximum offset in the text array */
   int         ch_width;             /* Character width in pixels */
   int         ch_height;            /* Character height in pixels */
   int         ncols;                /* No. of columns of characters */
   int         nrows;                /* No. of rows of characters */
   int         nchmax;               /* Maximum no. of characters allowed */
   int         font_descent;         /* Character descent (from font) */
   int         font_ascent;          /* Character ascent (from font) */
   int         max_width;            /* Maximum character width (from font) */
   int         cells_allowed;        /* Max no. of input cells allowed */
   int         num_cells;            /* No. of cells defined */
   XDl_text_table_cell * cells;      /* Pointer to cells table */
   int         act_strip;            /* Active strip present  =1 yes, =0 no */
   char        input_message[61];    /* Message for active strip */
   int         on_off;               /* Program awaiting input =1, not =0 */
   int         highlight_cell;       /* Number of the currently highlighted
                                        cell - 0 if none */
   void (*highlight_fn)();           /* User defined function to to called on
                                        highlight on or off */
   int         last_cell;            /* Last cell selected = if none */
}  
   XDl_text_table;

/*-C:*/
int xdl_text_table_cell (vh, vh_parent, x, y, cset, iact, max_cells,
                         ncols, nrows, min_width, min_height, font_type,
                         highl_fn)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame, 
                             if 0 then a base  level frame is created to
                             enclose the i/o window view-object (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used - 
                             normally 0 (R)*/
int       iact;           /* =1 active strip, =0 no active strip (R)*/
int       max_cells;      /* Maximum no. of cells allowed (R)*/
int       ncols;          /* Number of columns of characters (if 0 then
                             this will be calculated from min_width which
                             must then be given a value > 0. (R)*/
int       nrows;          /* Number of rows of characters (if 0 then
                             this will be calculated from min_height which
                             must then be given a value > 0. (R)*/
int       min_width;      /* Minimum window width in pixels. If this is
                             greater than the width calculated from the
                             number of character columns requirement then 
                             this value will be used as the window width 
                             (R)*/
int       min_height;     /* Minimum window height in pixels. If this is
                             greater than the height calculated from the
                             number of character rows requirement then 
                             this value will be used as the window height
                             (R)*/
int       font_type;      /* Font type (1-5) for very_small, small, medium,
                             large or extra-large font. (R)*/
void (*highl_fn)();       /*  Function to call when highlighting or 
                              unhighlighting takes place in a cell to allow 
                              additional user define actions to be carried 
                              out. (May be a NULL function)
                
                              Call:  highl_fn (ivh, icell, iflag)

                Parameters:   int *ivh       The file handle (R)
                              int *icell     The cell number (R)
                              int *iflag     Flag =1 highlight on 
                                                  =0 highlight off 

                              Pointers used for parameters to allow
                              for Fortran case.

                Note: Will still be called when not in input selection 
                      mode even if the highlight option selected for this 
                      mode via CALL XDLF_TEXT_TABLE_DEFINE_CELL is 0.
                              (R)*/     
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to allocate required memory
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   Status st;                  /* Status return from Xlib calls */
   XDl_text_table * gd;        /* Pointer to view-object's global data 
                                  structure */
   XFontStruct *fontinfo;      /* Font structure pointer */
   XFontStruct *bold_fontinfo; /* Bold font structure pointer */
   char * p;                   /* Temporary pointer */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int i;                      /* Loop counter */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int ascent;                 /* Font ascent */
   int descent;                /* Font descent */
   int area_width;             /* Text area width */
   int area_height;            /* Text area height */
   int x_area;                 /* x position of text area */
   int y_area;                 /* y position of text area */
   int h_act;                  /* Height of active strip or 0 if none */

   err = 0;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Get size requirements */

   fontinfo = XDL_fonts[font_type-1];
   bold_fontinfo = XDL_bold_fonts[font_type-1];
   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   x_area = BORDER_WIDTH;
   y_area = BORDER_HEIGHT;
   if (iact>0&&min_width<MIN_ACT_STRIP) min_width = MIN_ACT_STRIP;
   if (ncols>0)
   {
      area_width = ncols*ch_width + 2*MARG_W  + 2;
      width = area_width + 2*BORDER_WIDTH + 2;
      if (min_width>width) 
      {
         width = min_width;
         x_area = (width - 2 - area_width) / 2;
      }
   }
   else
   {
      width = min_width;
      if (width<2*MARG_W+2*BORDER_WIDTH+ch_width+4)
          width = 2*MARG_W + 2*BORDER_WIDTH + ch_width + 4;
      area_width = width - 2*BORDER_WIDTH - 2;
      ncols = (area_width - 2*MARG_W - 2) / ch_width;
   }
   if (nrows>0)
   {
      h_act = 0;
      if (iact>0) h_act = ACT_STRIP_HEIGHT;
      area_height = nrows*ch_height + 2*MARG_H + 2;
      height = area_height + 2*BORDER_HEIGHT + 2;
      if (min_height>height+h_act)
      {
         height = min_height - h_act;
         y_area = (height - 2 - area_height) / 2;
      }
   }
   else
   {
      height = min_height;
      if (iact>0) height = min_height - ACT_STRIP_HEIGHT; 
      if (height<2*MARG_H+2*BORDER_HEIGHT+ch_height+4)
          height = 2*MARG_H + 2*BORDER_HEIGHT + ch_height + 4;
      area_height = height - 2*BORDER_HEIGHT -2;
      nrows = (area_height - 2*MARG_H - 2) / ch_height;
   }
   if (iact>0) 
   {
      height = height + ACT_STRIP_HEIGHT;
      y_area = y_area + ACT_STRIP_HEIGHT;
   }

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_table", "xdl_table",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_text_table));
   if (p==0) return 2;
   gd = (XDl_text_table *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, p, 1021, xdl_text_table_on_off,
                        xdl_text_table_tidy_up);

   /* Some initialisations */

   gd->wframe = wframe;
   gd->wact = 0;
   gd->vh = vh;
   gd->font_descent = descent;
   gd->font_ascent = ascent;
   gd->max_width = ch_width;
   gd->ch_width = ch_width;
   gd->ch_height = ch_height;
   gd->ncols = ncols;
   gd->nrows = nrows;
   gd->nchmax = (gd->ncols)*(gd->nrows);
   gd->max_toff = gd->nchmax -1;
   p = (char *) malloc ((gd->nchmax)*sizeof(char));
   if (p==0) return 2;
   gd->text = p;
   p = (char *) malloc ((gd->nchmax)*sizeof(char));
   if (p==0)
   {
      free (gd->text);
      return 2;
   }
   gd->flag = p;
   for (i=0; i<=gd->max_toff; i++)
   {
      gd->text[i] = ' ';
      gd->flag[i] = 0;
   }
   gd->act_strip = 0;
   if (iact>0) gd->act_strip = 1; 
   gd->num_cells = 0;
   gd->cells_allowed = 0;
   if (max_cells>0) 
   {
      gd->cells_allowed = max_cells;
      gd->cells = (XDl_text_table_cell *) 
                malloc (max_cells*sizeof(XDl_text_table_cell));
      if (gd->cells==0) gd->cells_allowed = 0;
      if (gd->cells_allowed>0)
      {
         for (i=0;i<gd->cells_allowed;++i) gd->cells[i].defined = 0;
      }
   }
   strcpy (gd->input_message, "Select input");
   gd->on_off = 0;
   gd->highlight_cell = 0;
   gd->highlight_fn = highl_fn;
   gd->last_cell = 0;

   /* Create active strip window (if required) */

   if (iact>0)
   {
      gd->wact = xdl_create_active_strip (gd->wframe, vh, 
                            width - 2*BORDER_WIDTH - 2, xdl_text_table_rps);
   }
   /* Create text table paint window */

   gd->wtab = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                              area_width, area_height, cset);

   xdl_add_window (vh, gd->wtab, xdl_text_table_rp1, xdl_text_table_ep1);

   /* Prepare for event handling */

   if (gd->cells_allowed==0)
   {
      XSelectInput (XDL_display, gd->wtab, ExposureMask|StructureNotifyMask);
   }
   else
   {
      XSelectInput (XDL_display, gd->wtab, ExposureMask|StructureNotifyMask|
                    ButtonPressMask|KeyPressMask|ButtonReleaseMask|
                    LeaveWindowMask|PointerMotionMask);
   }

   /* Create and initialise GCs */

   gd->paint_gc = XCreateGC (XDL_display, gd->wtab, 0, NULL);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, fontinfo->fid);

   gd->paint_bold_gc = XCreateGC (XDL_display, gd->wtab, 0, NULL);
   XSetForeground (XDL_display, gd->paint_bold_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_bold_gc, bold_fontinfo->fid);

   return 0;
}

/* Fortran binding: xdlf_text_table_cell */

void xdlf_text_table_cell (ivh, ivhparent, ix, iy, icset, iact, maxcols,
                           ncols, nrows, minw, minh, ifont, highl_fn, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *iact,*ncols, *nrows, *minw,  *minh,
    *maxcols, *ifont, *ierr;
void (*highl_fn)();
{
   *ierr = xdl_text_table_cell (*ivh, *ivhparent, *ix, *iy, *icset, *iact,
                               *maxcols, *ncols, *nrows, *minw, *minh, *ifont,
                               highl_fn);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_text_table_rps      **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, June 1996

*/

void xdl_text_table_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_text_table * gd;  /* Pointer to global data area */

   gd = (XDl_text_table *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->input_message, 0);
   }
}

/*
*********************************
**     xdl_text_table_rp1      **
*********************************

Purpose: Repaint text area

Author:  John W. Campbell, March 1992

*/

void xdl_text_table_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_text_table * gd;    /* Pointer to global data area */
   int ir, ic;             /* Row and column loop counters */
   int x, y;               /* Character x, y position */
   int toff;               /* Text array offset */
   char str[2];            /* Temp character string array */
   int icolr;              /* Colour flag */
   int bold;               /* Bold print flag */
   unsigned long pixcolr;  /* Pixel colour */

   str[1] = 0;
   gd = (XDl_text_table *) global;
   XClearWindow (XDL_display, gd->wtab);
   for (ir=1; ir<=gd->nrows; ++ir)
   {
      for (ic=1; ic<=gd->ncols; ++ic)
      {
         toff = (ir-1)*(gd->ncols) + ic -1;
         str[0] = gd->text[toff];
         x = (gd->ch_width)*(ic-1) + MARG_W;
         y = (gd->ch_height)*ir - gd->font_descent + MARG_H;
         bold = 0;
         if (gd->flag[toff]<20)
         {
            icolr = gd->flag[toff]/2;
            if (gd->flag[toff]%2) bold = 1;
         }
         else
         {
            /* Symbol */
            xdl_text_table_drawsymb (gd,ir,ic);
            continue;
         }
         
         /* Text */
         if (icolr>0)
         {
            switch (icolr)
            {
                case 1: /* Red */
                    pixcolr = XDL_redpixel;
                break;
                case 2: /* Yellow */
                    pixcolr = XDL_yellowpixel;
                break;
                case 3: /* Green */
                    pixcolr = XDL_greenpixel;
                break;
                case 4: /* Cyan */
                    pixcolr = XDL_cyanpixel;
                break;
                case 5: /* Blue */
                    pixcolr = XDL_bluepixel;
                break;
                case 6: /* Magenta */
                    pixcolr = XDL_magentapixel;
                break;
            }
            if (bold) 
            {
              XSetForeground (XDL_display, gd->paint_bold_gc, pixcolr);
            }
            else
            {
               XSetForeground (XDL_display, gd->paint_gc, pixcolr);
            }
         }

         if (bold)
         {
            XDrawString (XDL_display, gd->wtab, gd->paint_bold_gc,
                         x, y, str, 1);
         }
         else
         {
            XDrawString (XDL_display, gd->wtab, gd->paint_gc,
                         x, y, str, 1);
         }

         if (icolr>0)
         {
            if (bold) 
            {
              XSetForeground (XDL_display, gd->paint_bold_gc, XDL_blackpixel);
            }
            else
            {
               XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
            }
         }

      }
   }
   return;
}

/*
*********************************
**     xdl_text_table_ep1      **
*********************************

Purpose: Event handling from text area

Author:  John W. Campbell, March 1992

*/

void xdl_text_table_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_text_table *gd;            /* Pointer to the global data area */
   int x, y;                      /* Mouse position */
   int icell;                     /* Cell no. -  0 if none */
   XEvent xevent;                 /* Temp XEvent */

   gd = (XDl_text_table *) global;
   if (gd->cells_allowed==0) return;

   /* Check for mouse select */

   if (gd->on_off)
   {
      if (event->type==ButtonRelease&&event->xbutton.button==Button1)
      {
         x = event->xbutton.x;
         y = event->xbutton.y; 
         icell = xdl_text_table_incell (gd, x, y);
         if (icell>0)
         {
            gd->last_cell = icell;
            XDL_return_data_vh = XDL_view_objects[iv]->vh;
#if 0
            if (gd->highlight_cell>0) 
               xdl_text_table_drawcell (gd, gd->highlight_cell, 0, gd->on_off);
            gd->highlight_cell = 0;
#endif
            return;
         }
      }
   }
     
   /* Mouse movement */

   if (event->type==MotionNotify)
   {
      x = event->xmotion.x;
      y = event->xmotion.y;
      while (XCheckTypedWindowEvent (XDL_display, gd->wtab,
             MotionNotify, &xevent))
      {
         x = xevent.xmotion.x;
         y = xevent.xmotion.y;
      }
      /* Item Highlighting */
      icell = xdl_text_table_incell (gd, x, y);
      if (icell>0)
      {
         if (icell!=gd->highlight_cell)
         {
            if (gd->highlight_cell>0) 
               xdl_text_table_drawcell(gd, gd->highlight_cell, 0, gd->on_off);
            xdl_text_table_drawcell (gd, icell, 1, gd->on_off);
            gd->highlight_cell = icell;
            return;
	 }
      }
      else
      {
         if (gd->highlight_cell>0) 
            xdl_text_table_drawcell (gd, gd->highlight_cell, 0, gd->on_off);
         gd->highlight_cell = 0;
         return;
      }
   }

   /* Check for mouse exit from the window */

   if (event->type==LeaveNotify)
   {
      if (gd->highlight_cell<=0) return;
      xdl_text_table_drawcell (gd, gd->highlight_cell, 0, gd->on_off);
      gd->highlight_cell = 0;
      return;
   }

   return;
}

/*
*********************************
**     xdl_text_table_clc      **
*********************************

Purpose: Clear a character position in the text area

Author:  John W. Campbell, March 1992

*/

void xdl_text_table_clc (gd,row,col)

XDl_text_table *gd;
int row;
int col;
{
   int toff;      /* Offset into text array */

   if (row<1||row>gd->nrows) return;
   if (col<1||col>gd->ncols) return;
   toff = (row-1)*(gd->ncols) + col -1;
   gd->text[toff] = ' ';
   gd->flag[toff] = 0;
   XClearArea (XDL_display, gd->wtab,
               (col-1)*(gd->ch_width) + MARG_W,
               (row-1)*(gd->ch_height) + MARG_H,
               gd->ch_width, gd->ch_height, False);
   return;
}
/*
************************************
**     xdl_text_table_drawch      **
************************************

Purpose: Draw a text character or symbol

Author:  John W. Campbell, October 1994

*/

void xdl_text_table_drawch(gd,row,col)

XDl_text_table *gd;
int row;
int col;
{
   int toff;               /* Offset into text array */
   int colr;               /* Colour code */
   unsigned long pixcolr;  /* Pixel colour */
   int x, y;               /* Character x, y position */
   char str[2];            /* Temp character string array */
   int icolr;              /* Colour flag */
   int bold;               /* Bold print flag */


   if (row<1||row>gd->nrows) return;
   if (col<1||col>gd->ncols) return;
   toff = (row-1)*(gd->ncols) + col -1;
   colr = (int) gd->flag[toff]-20;
   str[0] = gd->text[toff];
   x = (gd->ch_width)*(col-1) + MARG_W;
   y = (gd->ch_height)*row - gd->font_descent + MARG_H;
   bold = 0;
   if (gd->flag[toff]<20)
   {
      icolr = gd->flag[toff]/2;
      if (gd->flag[toff]%2) bold = 1;
   }
   else
   {
      /* Symbol */
      xdl_text_table_drawsymb (gd,row,col);
      return;
   }
         
   /* Text */
   if (icolr>0)
   {
      switch (icolr)
      {
          case 1: /* Red */
              pixcolr = XDL_redpixel;
          break;
          case 2: /* Yellow */
              pixcolr = XDL_yellowpixel;
          break;
          case 3: /* Green */
              pixcolr = XDL_greenpixel;
          break;
          case 4: /* Cyan */
              pixcolr = XDL_cyanpixel;
          break;
          case 5: /* Blue */
              pixcolr = XDL_bluepixel;
          break;
          case 6: /* Magenta */
              pixcolr = XDL_magentapixel;
          break;
      }
      if (bold) 
      {
        XSetForeground (XDL_display, gd->paint_bold_gc, pixcolr);
      }
      else
      {
         XSetForeground (XDL_display, gd->paint_gc, pixcolr);
      }
   }

   if (bold)
   {
      XDrawString (XDL_display, gd->wtab, gd->paint_bold_gc,
                   x, y, str, 1);
   }
   else
   {
      XDrawString (XDL_display, gd->wtab, gd->paint_gc,
                   x, y, str, 1);
   }

   if (icolr>0)
   {
      if (bold) 
      {
        XSetForeground (XDL_display, gd->paint_bold_gc, XDL_blackpixel);
      }
      else
      {
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      }
   }

   return;
}
/*
************************************
**     xdl_text_table_drawsymb    **
************************************

Purpose: Draw a symbol

Author:  John W. Campbell, October 1994

*/

void xdl_text_table_drawsymb (gd,row,col)

XDl_text_table *gd;
int row;
int col;
{
   int toff;      /* Offset into text array */
   int colr;      /* Colour code */
   unsigned long pixcolr; /* Pixel colour */
   int xo, yo;    /* Origin of character cell */
   int x, y;      /* Coordinates used in plotting symbol */
   int w, h;      /* Width, height used in plotting symbol */


   if (row<1||row>gd->nrows) return;
   if (col<1||col>gd->ncols) return;
   toff = (row-1)*(gd->ncols) + col -1;
   colr = (int) gd->flag[toff]-20;
   switch (colr)
   {
       case 0: /* Black */
          pixcolr = XDL_blackpixel;
       break;
       case 1: /* Red */
           pixcolr = XDL_redpixel;
       break;
       case 2: /* Yellow */
           pixcolr = XDL_yellowpixel;
       break;
       case 3: /* Green */
           pixcolr = XDL_greenpixel;
       break;
       case 4: /* Cyan */
           pixcolr = XDL_cyanpixel;
       break;
       case 5: /* Blue */
           pixcolr = XDL_bluepixel;
       break;
       case 6: /* Magenta */
           pixcolr = XDL_magentapixel;
       break;
   }

   xo = (gd->ch_width)*(col-1) + MARG_W;
   yo = (gd->ch_height)*(row-1) + MARG_H;

   XSetForeground (XDL_display, gd->paint_gc, pixcolr);

   switch (gd->text[toff])
   {
      case 0: /* Open circle */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XDrawArc (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w, 
                   0, 23040);
      break;

      case 1: /* Filled disk */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XDrawArc (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w, 
                   0, 23040);
         XFillArc (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w, 
                   0, 23040);
      break;

      case 2: /* Filled disk with black rim */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XFillArc (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w, 
                   0, 23040);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
         XDrawArc (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w, 
                   0, 23040);
      break;

      case 3: /* Open square within character */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, w-1);
      break;

      case 4: /* Filled square within character */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w);
      break;

      case 5: /* Filled square with black border within character */
         w = gd->ch_width - 2;
         x = xo + 1;
         y = yo + (gd->ch_height - w)/2;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, w-1);
      break;

      case 6: /* Open square (full character width) */
         w = gd->ch_width;
         x = xo;
         y = yo + (gd->ch_height - w)/2;
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, w-1); 
      break;

      case 7: /* Filled square (full character width) */
         w = gd->ch_width;
         x = xo;
         y = yo + (gd->ch_height - w)/2;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w);
      break;

      case 8: /* Filled square with black border (full character width) */
         w = gd->ch_width;
         x = xo;
         y = yo + (gd->ch_height - w)/2;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, w);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, w-1); 
      break;

      case 9: /* Open rectangle (full character size) */
         w = gd->ch_width;
         h = gd->ch_height;
         x = xo;
         y = yo;
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, h-1); 
      break;

      case 10: /* Filled rectangle (full character size) */
         w = gd->ch_width;
         h = gd->ch_height;
         x = xo;
         y = yo;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, h);
      break;

      case 11: /* Filled rectangle with black border (full character size) */
         w = gd->ch_width;
         h = gd->ch_height;
         x = xo;
         y = yo;
         XFillRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, w, h);
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
         XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, x, y, 
                         w-1, h-1); 
      break;

   }

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);

   return;
}
/*
************************************
**     xdl_text_table_incell      **
************************************

Purpose: See if pixel is in a cell

Author:  John W. Campbell, June 1996

*/

int xdl_text_table_incell (gd, x, y)

XDl_text_table *gd;
int x;              /* Pixel 'x' position */
int y;              /* Pixel 'y' position */

/* Return 0 not in cell, >0 cell no. */
{
   int i;

   if (gd->cells_allowed==0) return 0;

   for (i=0;i<gd->cells_allowed;++i)
   {
      if (gd->cells[i].defined)
      {
         if (x>=gd->cells[i].xmin&&x<=gd->cells[i].xmax
             &&y>=gd->cells[i].ymin&&y<=gd->cells[i].ymax) return i+1;
      }
   }
   return 0;          
}
/*
************************************
**     xdl_text_table_drawcell    **
************************************

Purpose: Re-draw a cell with or without highlighting

Author:  John W. Campbell, June 1996

*/

void xdl_text_table_drawcell (gd,icell,highlight,on_off)

XDl_text_table *gd;
int icell;     /* Cell no. */
int highlight; /* = 1 draw highlighted, = 0 draw unhighlighted */
int on_off;    /* Input selection on/off (1/0) flag */
{
   XDl_text_table_cell * cell; /* Pointer to cell structure */
   int i;                      /* Temp/loop variable */
   int j;                      /* Temp/loop variable */
   int highlight_opt;          /* Highlight option */

   if (icell<1||icell>gd->cells_allowed) return;
   cell = &gd->cells[icell-1];
   if (!cell->defined) return;

   if (gd->highlight_fn!=0)
   {
      gd->highlight_fn (&gd->vh, &icell, &highlight);
   }

   if (on_off)
   {
      highlight_opt = cell->highlight;
   }
   else
   {
      highlight_opt = cell->highlight_nosel;
   }
   if (highlight_opt==0) return;

   /* Clear cell */
   XClearArea (XDL_display, gd->wtab, cell->xmin, cell->ymin,
               (cell->xmax-cell->xmin+1), (cell->ymax-cell->ymin+1),
               False);
   if (highlight&&XDL_color>1&&highlight_opt<=2)
   {     
      XSetForeground (XDL_display, gd->paint_gc, cell->highlight_pixel);
      XFillRectangle (XDL_display, gd->wtab, gd->paint_gc,
                     cell->xmin, cell->ymin,
                    (cell->xmax-cell->xmin+1), (cell->ymax-cell->ymin+1));
   }

   /* Redraw text */

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetForeground (XDL_display, gd->paint_bold_gc, XDL_blackpixel);
   for (i=cell->row1;i<=cell->row2;++i)
   {
      for (j=cell->col1;j<=cell->col2;++j)
      {
         xdl_text_table_drawch (gd,i,j);
      }
   }

   if (!highlight) return;

   XSetForeground (XDL_display, gd->paint_gc, cell->line_pixel);

   /* Underline if required */

   if ((XDL_color>1&&highlight_opt==3)||(XDL_color<=1&&highlight_opt==1))
   {
      XDrawLine (XDL_display, gd->wtab, gd->paint_gc, cell->xmin, cell->ymax,
                 cell->xmax, cell->ymax); 
   }

   /* Draw box if required */

   if ((XDL_color>1&&highlight_opt==4)||(XDL_color<=1&&highlight_opt==2))
   {
      XDrawRectangle (XDL_display, gd->wtab, gd->paint_gc, 
                 cell->xmin, cell->ymin,
                 cell->xmax-cell->xmin, cell->ymax-cell->ymin); 
   }
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   return;
}
/*
***********************************
**     xdl_text_table_on_off     **
***********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, June 1996

*/

void xdl_text_table_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_text_table *gd;       /* Pointer to global data area */
   XWindowAttributes attrib; /* Window attributes structure */

   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      if (gd->act_strip)
      {
         xdl_active_strip_on (gd->wact, gd->input_message, 0);
      }
   }
   else
   {
      gd->on_off = 0;
      if (gd->act_strip)
      {
         xdl_active_strip_off (gd->wact);
      }
   }
   return;
}
/*
************************************
**     xdl_text_table_tidy_up     **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, March 1992

*/

void xdl_text_table_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_text_table *gd;    /* Pointer to the global data area */

   gd = (XDl_text_table *) global;
   free (gd->text);
   free (gd->flag);
   XFreeGC (XDL_display, gd->paint_gc);
   XFreeGC (XDL_display, gd->paint_bold_gc);
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Output a text string - xdl_text_table_text
The routine xdl_text_table_text (xdlf_text_table_text) is used to output a 
text string to a text table view object.
-end*/
/*
*******************************
**     xdl_text_table_text   **
*******************************

Purpose: Output a text string to the text table

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_TEXT (IVH, XDLSTR(STRING), LEN, IROW, ICOL,
        +                           IBOLD, IERR)
-end*/
 
/*-Parameters:
IVH     (R)  View-object handle (see vh)
STRING  (R)  The string to be output (see string)
             ** Pass the address using the XDLSTR function **
LEN     (R)  Length of the string (>0) (cf len_str)
IROW    (R)  Row number for the output of the text string 
             (see row)
ICOL    (R)  Column number for the output of the text string
             (see col)
IBOLD   (R)  Flag =0 normal print, =1 bold print (black)
                  = 2/3   Red     normal/bold print
                  = 4/5   Yellow  normal/bold print
                  = 6/7   Green   normal/bold print
                  = 8/9   Cyan    normal/bold print
                  = 10/11 Blue    normal/bold print
                  = 12/13 Magenta normal/bols print
IERR    (W)  Returns the status from xdl_text_table_text call
IERR    (W)  Returns the status from xdl_text_table_text call
-end*/

/*-C:*/
int xdl_text_table_text (vh, string, len_str, row, col, bold)
/*end*/
 
/*-Parameters:*/
int vh;               /* The xdl_text_table view-object handle (R)*/
char string[];        /* The string to be printed (R)*/
int len_str;          /* The length of the string. If the string is null
                         terminated, then 0 may be given and the length
                         of the string will then be determined by the
                         routine (R)*/
int row;              /* The row number for outputting the string (R)*/
int col;              /* The start column number for outputting the
                         text string (R)*/
int bold;             /* Flag =0 normal print, =1 bold print (black)
                              = 2/3   Red     normal/bold print
                              = 4/5   Yellow  normal/bold print
                              = 6/7   Green   normal/bold print
                              = 8/9   Cyan    normal/bold print
                              = 10/11 Blue    normal/bold print
                              = 12/13 Magenta normal/bols print
                         (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Position for start of text string outside
                              window
-end*/
{
   XDl_text_table *gd;     /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int nch;                /* No. of characters to output */
   int is;                 /* Loop counter through string characters */
   int toff;               /* Offset in the text array */
   int x, y;               /* Position of a character */
   char str[2];            /* Temporary string to hold 1 character and a null */
   int nmax;               /* Max no. cols from start column to rhs of window */
   int icolr;              /* Colour flag */
   unsigned long pixcolr;  /* Pixel colour */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_text",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_text", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   str[1] = 0;
   if (bold<0||bold>13) bold = 0;

   /* Output and store the string */

   if (row<1||row>gd->nrows) return 4;
   if (col<1||col>gd->ncols) return 4;
   nmax = gd->ncols - col + 1;

   nch = len_str;
   if (nch==0) nch = strlen (string);
   if (nch>nmax) nch=nmax;

   for (is=0; is<nch; ++is)
   {
      str[0] = string[is];
      xdl_text_table_clc (gd, row, col+is);
      toff = (row-1)*(gd->ncols) + col + is -1;
      gd->text[toff] = str[0];
      gd->flag[toff] = bold;
      x = (gd->ch_width)*(col+is-1) + MARG_W;
      y = (gd->ch_height)*(row) - gd->font_descent + MARG_H;
      icolr = bold/2;
      if (icolr>0)
      {
         switch (icolr)
         {
             case 1: /* Red */
                 pixcolr = XDL_redpixel;
             break;
             case 2: /* Yellow */
                 pixcolr = XDL_yellowpixel;
             break;
             case 3: /* Green */
                 pixcolr = XDL_greenpixel;
             break;
             case 4: /* Cyan */
                 pixcolr = XDL_cyanpixel;
             break;
             case 5: /* Blue */
                 pixcolr = XDL_bluepixel;
             break;
             case 6: /* Magenta */
                 pixcolr = XDL_magentapixel;
             break;
         }
      }
      if (bold%2)
      {
         if (bold>1) XSetForeground (XDL_display, gd->paint_bold_gc, pixcolr);
         XDrawString (XDL_display, gd->wtab, gd->paint_bold_gc,
                      x, y, str, 1);
         if (bold>1)  XSetForeground (XDL_display, gd->paint_bold_gc,
                                     XDL_blackpixel);
      }
      else
      {
         if (bold>1) XSetForeground (XDL_display, gd->paint_gc, pixcolr);
         XDrawString (XDL_display, gd->wtab, gd->paint_gc,
                      x, y, str, 1);
         if (bold>1) XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      }
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_text_table_text */

void xdlf_text_table_text (ivh, string, len, row, col, bold, ierr)
int *ivh, *string, *len, *row, *col, *bold, *ierr;
{
   *ierr = xdl_text_table_text (*ivh, XDL_pointers[*string], *len, *row, *col, 
                                *bold);
   return;
}
/*-Routine: Output a symbol - xdl_text_table_symbol
The routine xdl_text_table_symbol (xdlf_text_table_symbol is used to output a 
symbol to a text table view object. The symbol lies within the space of a
single character and its exact size depends on the character font used for
the text output.
-end*/
/*
*******************************
**     xdl_text_table_symbol **
*******************************

Purpose: Output a symbol to the text table

Author:  John W. Campbell, October 1994

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_SYMBOL (IVH, ISYMB, IROW, ICOL,
        +                             ICOLR, IERR)
-end*/
 
/*-Parameters:
IVH     (R)  View-object handle (see vh)
ISYMB   (R)  Symbol number
             = 0  Open circle (within character)
             = 1  Disk (within character)
             = 2  Disk with black border (within character)
             = 3  Open square (within character)
             = 4  Filled square (within character)
             = 5  Filled square with black border (within character)
             = 6  Open square (full character width)
             = 7  Filled square (full character width)
             = 8  Filled square with black border (full character
                  width)
             = 9  Open rectangle (full character size)
             = 10 Filled rectangle (full character size)
             = 11 Filled rectangle with black border (full character
                  size)
IROW    (R)  Row number for the output of the text string 
             (see row)
ICOL    (R)  Column number for the output of the text string
             (see col)
ICOLR   (R)  Colour flag 0 = black
                         1 = red
                         2 = yellow
                         3 = green
                         4 = cyan
                         5 = blue
                         6 = magenta
IERR    (W)  Returns the status from xdl_text_table_symbol call
IERR    (W)  Returns the status from xdl_text_table_symbol call
-end*/

/*-C:*/
int xdl_text_table_symbol (vh, symb, row, col, colr)
/*end*/
 
/*-Parameters:*/
int vh;               /* The xdl_text_table view-object handle (R)*/
int symb;             /* Symbol code
                         = 0  Open circle (within character)
                         = 1  Disk (within character)
                         = 2  Disk with black border (within character)
                         = 3  Open square (within character)
                         = 4  Filled square (within character)
                         = 5  Filled square with black border (within 
                              character)
                         = 6  Open square (full character width)
                         = 7  Filled square (full character width)
                         = 8  Filled square with black border (full
                              character width)
                         = 9  Open rectangle (full character size)
                         = 10 Filled rectangle (full character size)
                         = 11 Filled rectangle with black border (full
                              character size)
                         (R)*/
int row;              /* The row number for outputting the string (R)*/
int col;              /* The start column number for outputting the
                         text string (R)*/
int colr;             /* Colour code 
                            0 = black
                            1 = red
                            2 = yellow
                            3 = green
                            4 = cyan
                            5 = blue
                            6 = magenta
                         (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Position for symbol outside window
-end*/
{
   XDl_text_table *gd;     /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int toff;               /* Offset in the text/symbols array */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_symbol", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;

   if (symb<0||symb>11) symb = 0;
   if (colr<0||colr>6) colr = 0;

   /* Output and store the symbol */

   if (row<1||row>gd->nrows) return 4;
   if (col<1||col>gd->ncols) return 4;

   xdl_text_table_clc (gd, row, col);

   toff = (row-1)*(gd->ncols) + col  -1;
   gd->text[toff] = (char) symb;
   gd->flag[toff] = 20 + colr;
   xdl_text_table_drawsymb (gd, row, col);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_text_table_symbol */

void xdlf_text_table_symbol (ivh, isymb, row, col, icolr, ierr)
int *ivh, *isymb, *row, *col, *icolr, *ierr;
{
   *ierr = xdl_text_table_symbol (*ivh, *isymb, *row, *col, *icolr);
   return;
}

/*-Routine: Clear the text table - xdl_text_table_clear
The routine xdl_text_table_clear (xdlf_text_table_clear) is used to clear the
text table.
-end*/
/*
***********************************
**     xdl_text_table_clear      **
***********************************

Purpose: Clear text table

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_CLEAR (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_text_table_clear
IERR     (W)  Returns status from xdl_text_table_clear
-end*/

/*-C:*/
int xdl_text_table_clear (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                /* Offset in XDL_view_objects array */
   XDl_text_table *gd;    /* pointer to the global data area */
   int i;                 /* Temp/loop variable */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_clear",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_clear", vh, "xdl_text_table");
      return 2;
   }

   /* Clear text array and text window */

   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   for (i=0; i<=gd->max_toff; i++)
   {
      gd->text[i] = ' ';
      gd->flag[i] = 0;
   }
   XClearWindow (XDL_display, gd->wtab);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding xdlf_text_table_clear */

void xdlf_text_table_clear (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_text_table_clear (*ivh);
   return;
}

/*-Routine: Clear a section - xdl_text_table_clearsect
The routine xdl_text_table_clearsect (xdlf_text_table_clearsect) is used 
to clear a section of the text table.
-end*/
/*
****************************************
**     xdl_text_table_clearsect       **
****************************************

Purpose: Clear a section of the text table

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_CLEARSECT (IVH, IROW1, ICOL1, 
        +                                IROW2, ICOL2, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IROW1    (R)  Row number of top left character of area to be 
              cleared
ICOL1    (R)  Column no. of top left character of area to be 
              cleared
IROW2    (R)  Row number of bottom right character of area to be 
              cleared
ICOL2    (R)  Column no. of bottom right character of area to be 
              cleared
IERR     (W)  Returns status from xdl_text_table_clearsect
IERR     (W)  Returns status from xdl_text_table_clearsect
-end*/

/*-C:*/
int xdl_text_table_clearsect (vh, row1, col1, row2, col2)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int row1;        /* Row number of top left character of area to be 
                    cleared (R)*/
int col1;        /* Column no. of top left character of area to be 
                    cleared (R)*/
int row2;        /* Row number of bottom right character of area to be 
                    cleared (R)*/
int col2;        /* Column no. of bottom right character of area to be 
                    cleared (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                /* Offset in XDL_view_objects array */
   XDl_text_table *gd;    /* pointer to the global data area */
   int i;                 /* Temp/loop variable */
   int j;                 /* Temp/loop variable */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_clearsect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_clearsect", vh, "xdl_text_table");
      return 2;
   }

   /* Clear text array and text window sections */

   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;

   if (row1<1) row1 = 1;
   if (row2>gd->nrows) row2 = gd->nrows;
   if (row2<row1) return 0;
   if (col1<1) col1 = 1;
   if (col2>gd->ncols) col2 = gd->ncols;
   if (col2<col1) return 0;

   for (i=row1;i<=row2;++i)
   {
      for (j=col1;j<=col2;++j)
      {
         xdl_text_table_clc (gd,i,j);
      }
   }     
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding xdlf_text_table_clearsect */

void xdlf_text_table_clearsect (ivh, irow1, icol1, irow2, icol2, ierr)
int *ivh, *irow1, *icol1, *irow2, *icol2, *ierr;
{
   *ierr = xdl_text_table_clearsect (*ivh, *irow1, *icol1, *irow2, *icol2);
   return;
}

/*-Routine: Reset active strip message - xdl_text_table_input_message
The routine xdl_text_table_input_message (xdlf_text_table_input_message) is
used to reset the message which will appear in the active strip when input
is requested When the view-object is created, the default message 
is 'Select input'.
-end*/
/*
*****************************************
**     xdl_text_table_input_message    ** 
*****************************************

Purpose: Reset 'Select input' message for the active strip area

Author:  John W. Campbell, June 1996

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_INPUT_MESSAGE (IVH, XDLSTR(MESSAGE), 
        +                                    LEN, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MESSAGE  (R)  Character string containing the message 
              (see message)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the message string - must be >0 
              (cf len) (max 60 chars) 
IERR     (W)  Returns status from xdl_text_table_input_message 
              call
-end*/

/*-C:*/
int xdl_text_table_input_message (vh, message, len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *message;   /* Message string (max of 60 chars long) (R)*/
int len;         /* Length of the message string; if 0 then length 
                    will be found assuming a null terminated string 
                    (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_text_table  *gd;    /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_input_message",vh);return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_input_message", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   if (len==0) len = strlen (message);
   if (len>60) len = 60;
   strncpy (gd->input_message, message, len);
   gd->input_message[len] = 0;
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_text_table_input_message */

void xdlf_text_table_input_message (ivh, message, len, ierr)
int *ivh, *message, *len, *ierr;
{
   *ierr = xdl_text_table_input_message (*ivh, XDL_pointers[*message], *len);
   return;
}
/*-Routine: Define an input cell - xdl_text_table_define_cell
The routine xdl_text_table_define_cell (xdlf_text_table_define_cell) is
used to define an input cell within the text table. This may be selected
via the mouse and will be highlighted when the pointer lies within the
cell area.
-end*/
/*
*****************************************
**     xdl_text_table_define_cell      ** 
*****************************************

Purpose: Define an active cell

Author:  John W. Campbell, June 1996

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_DEFINE_CELL (IVH, INUM, IROW1, ICOL1, 
        +                                  IROW2, ICOL2, IHIGH, IHIGH_N,
        +                                  ICOLR_BG, ICOLR_L,
        +                                  IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
INUM     (R)  Cell number (must be in range define when text table set up
IROW1    (R)  First row of cell
ICOL1    (R)  First column of cell
IROW2    (R)  Final row of cell
ICOL2    (R)  Final column of cell
IHIGH    (R)  Highlight option (when text table in input selection mode)
              = 1 coloured bg (underline if not colour)
              = 2 coloured bg (surround box if not colour)
              = 3 underline
              = 4 surround box 
IHIGH_2  (R)  Highlight option when text table not in input selection 
              mode. As above but may be 0 for non such highlighting
ICOLR_BG (R)  Colour for background highlght (1-7) Black, Red, Yellow, Green,
              Cyan, Blue, Magenta
ICOLR_L  (R)  Colour for highlght lines(1-7) Black, Red, Yellow, Green,
              Cyan, Blue, Magenta
IERR     (W)  Returns status from xdl_text_table_define_cell
              call
-end*/

/*-C:*/
int xdl_text_table_define_cell (vh, num, row1, col1, row2, col2, hightyp,
                                high_nosel, colr_bg, colr_l)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int num;         /* Cell number (must be in range define when text 
                    table set up (R)*/
int row1;        /* First row of cell (R)*/
int col1;        /* First column of cell (R)*/
int row2;        /* Final row of cell (R)*/
int col2;        /* Final column of cell (R)*/
int hightyp;     /* Highlight option (when text table in input 
                    selection mode)
                    = 1 coloured bg (underline if not colour)
                    = 2 coloured bg (surround box if not colour)
                    = 3 underline
                    = 4 surround box 
                    (R)*/
int high_nosel;  /* Highlight option (when text table not in input 
                    selection mode)
                    As above but may be 0 for no highlighting in this
                    mode (R)*/
int colr_bg;     /* Colour for background highlght (1-7) Black, Red,
                    Yellow, Green, Cyan, Blue, Magenta (R)*/
int colr_l;      /* Colour for highlght lines(1-7) Black, Red, Yellow, 
                    Green, Cyan, Blue, Magenta (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Invalid cell number
                  bit 4 set:  Other invalid parameters
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_text_table  *gd;    /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_define_cell",vh);return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_define_cell", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   if (num<1||num>gd->cells_allowed) return 4;
   gd->cells[num-1].defined = 0;
   if (hightyp<1||hightyp>4) return 8;
   if (high_nosel<0||high_nosel>4) return 8;
   if (colr_bg<1||colr_bg>7) return 8;
   if (colr_l<1||colr_l>7) return 8; 
   if (row1<1) row1 = 1;
   if (col1<1) col1 = 1;
   if (row2>gd->nrows) row2 = gd->nrows;
   if (col2>gd->ncols) col2 = gd->ncols;
   if (row2<row1) return 4;
   if (col2<col1) return 4;
   gd->cells[num-1].defined = 1;
   gd->cells[num-1].highlight = hightyp;
   gd->cells[num-1].highlight_nosel = high_nosel;
   gd->cells[num-1].highlight_pixel = XDL_blackpixel; 
   if (XDL_color>1)
   {
      if (colr_bg==2) gd->cells[num-1].highlight_pixel = XDL_redpixel;
      if (colr_bg==3) gd->cells[num-1].highlight_pixel = XDL_yellowpixel;
      if (colr_bg==4) gd->cells[num-1].highlight_pixel = XDL_greenpixel;
      if (colr_bg==5) gd->cells[num-1].highlight_pixel = XDL_cyanpixel;
      if (colr_bg==6) gd->cells[num-1].highlight_pixel = XDL_bluepixel;
      if (colr_bg==7) gd->cells[num-1].highlight_pixel = XDL_magentapixel;
   }
   gd->cells[num-1].line_pixel = XDL_blackpixel; 
   if (XDL_color>1)
   {
      if (colr_l==2) gd->cells[num-1].line_pixel = XDL_redpixel;
      if (colr_l==3) gd->cells[num-1].line_pixel = XDL_yellowpixel;
      if (colr_l==4) gd->cells[num-1].line_pixel = XDL_greenpixel;
      if (colr_l==5) gd->cells[num-1].line_pixel = XDL_cyanpixel;
      if (colr_l==6) gd->cells[num-1].line_pixel = XDL_bluepixel;
      if (colr_l==7) gd->cells[num-1].line_pixel = XDL_magentapixel;
   }
   gd->cells[num-1].row1 = row1;
   gd->cells[num-1].col1 = col1;
   gd->cells[num-1].row2 = row2;
   gd->cells[num-1].col2 = col2;
   gd->cells[num-1].xmin = (col1-1)*(gd->ch_width) + MARG_W;
   gd->cells[num-1].ymin = (row1-1)*(gd->ch_height) + MARG_H;
   gd->cells[num-1].xmax = (col2-1)*(gd->ch_width) + gd->ch_width-1 + MARG_W;
   gd->cells[num-1].ymax = (row2-1)*(gd->ch_height) + gd->ch_height-1 + MARG_H;
   return 0;
}

/* Fortran binding: xdlf_text_table_define_cell */

void xdlf_text_table_define_cell (ivh, inum, irow1, icol1, irow2, icol2,
                                 ihigh, ihigh_2, icolr_bg, icolr_l, ierr)
int *ivh, *inum, *irow1, *icol1, *irow2, *icol2, *ierr;
int *ihigh, *ihigh_2, *icolr_bg, *icolr_l;
{
   *ierr = xdl_text_table_define_cell (*ivh, *inum, *irow1, *icol1,
               *irow2, *icol2, *ihigh, *ihigh_2, *icolr_bg, *icolr_l);
   return;
}

/*-Routine: Clear input cell(s) - xdl_text_table_clear_cell
The routine xdl_text_table_clear_cell (xdlf_text_table_clear_cell) is
used to clear an input cell or all input cells within the text table. 
This dose not affect the text drawn in such areas just the input control.
-end*/
/*
*****************************************
**     xdl_text_table_clear_cell      ** 
*****************************************

Purpose: Clear an active cell

Author:  John W. Campbell, June 1996

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_CLEAR_CELL (IVH, INUM, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
INUM     (R)  Cell number (must be in range define when text table set up)
              or 0 to clear all cells
IERR     (W)  Returns status from xdl_text_table_clear_cell
              call
-end*/

/*-C:*/
int xdl_text_table_clear_cell (vh, num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int num;         /* Cell number (must be in range define when text 
                    table set up) or 0 to claer all cells (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Invalid cell number
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   int i;                  /* Temp/loop variable */
   XDl_text_table  *gd;    /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_clear_cell",vh);return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_clear_cell", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   if (num<0||num>gd->cells_allowed) return 4;
   if (num>0)
   {
      if (num==gd->highlight_cell)
      {
         xdl_text_table_drawcell (gd, gd->highlight_cell, 0, gd->on_off);
         gd->highlight_cell = 0;
      }
      gd->cells[num-1].defined = 0;
   }
   else
   {
      if (gd->highlight_cell>0) 
      {
         xdl_text_table_drawcell (gd, gd->highlight_cell, 0, gd->on_off);
         gd->highlight_cell = 0;
      }
      for (i=0;i<gd->cells_allowed;++i) gd->cells[i].defined=0;
   }
   return 0;
}

/* Fortran binding: xdlf_text_table_clear_cell */

void xdlf_text_table_clear_cell (ivh, inum, ierr)
int *ivh, *inum, *ierr;
{
   *ierr = xdl_text_table_clear_cell (*ivh, *inum);
   return;
}

/*-Routine: Get selected input cell number - xdl_text_table_getinp
The routine xdl_text_table_getinp (xdlf_text_table_getinp) is
used find the cell number of the last input cell that was selected using
the mouse.
-end*/
/*
*****************************************
**     xdl_text_table_getinp           ** 
*****************************************

Purpose: Get last selected input cell number

Author:  John W. Campbell, June 1996

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_GETINP (IVH, INUM, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
INUM     (W)  Last selected cell number 0 if none
IERR     (W)  Returns status from xdl_text_table_clear_cell
              call
-end*/

/*-C:*/
int xdl_text_table_getinp (vh, num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *num;        /* Last selected cell number 0 if none (W)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   int i;                  /* Temp/loop variable */
   XDl_text_table  *gd;    /* Pointer to global data area */

   *num = 0;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_text_table_getinp",vh);return 1;}
   if (XDL_view_objects[iv]->type!=1021)
   {
      xdl_view_type_errmsg ("xdl_text_table_getinp", vh, "xdl_text_table");
      return 2;
   }
   gd = (XDl_text_table *) XDL_view_objects[iv]->gd;
   *num = gd->last_cell;
   return 0;
}

/* Fortran binding: xdlf_text_table_getinp */

void xdlf_text_table_getinp (ivh, inum, ierr)
int *ivh, *inum, *ierr;
{
   *ierr = xdl_text_table_getinp (*ivh, inum);
   return;
}

/*-Routine: Calculate size requirements - xdl_text_table_getsize
The routine xdl_text_table_getsize (xdlf_text_table_getsize) is used to
get the required size for a text table view-object given the numbers
of rows and columns of characters required.
-end*/
/*
************************************
**     xdl_text_table_getsize     **
************************************

Purpose: Return the size requirements of the text table view-object based
         on sizing specified by number of columns and rows of characters

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_GETSIZE (NCOLS, NROWS, IFONT, IWIDTH, 
        +                             IHEIGHT)
-end*/
 
/*-Parameters:
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_text_table_getsize (ncols, nrows, font_type, w, h)
/*end*/
 
/*-Parameters:*/
int       ncols;          /* Number of columns of characters ( > 0 ) (R)*/
int       nrows;          /* Number of rows of characters ( > 0 ) (R)*/
int       font_type;      /* Font type (1-5) for very-small, small, medium,
                             large or extra-large font. (R)*/
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int ascent;                 /* Font ascent */
   int descent;                /* Font descent */

   /* Calculate the size requirements */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Get size requirements */

   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *w = ncols*ch_width + 2*MARG_W + 2 + 2*BORDER_WIDTH + 2;
   *h = nrows*ch_height + 2*MARG_H + 2 + 2*BORDER_HEIGHT + 2;
   return;
}

/* Fortran binding: xdlf_text_table_getsize */

void xdlf_text_table_getsize (ncols, nrows, ifont, iwidth, iheight)
int *ncols, *nrows, *ifont, *iwidth, *iheight;
{
   xdl_text_table_getsize (*ncols, *nrows, *ifont, iwidth, iheight);
   return;
}
/*-Routine: Calculate cell text size  - xdl_text_table_getcsize
The routine xdl_text_table_getcsize (xdlf_text_table_getcsize) is used to
get the required size for a text cell view-object given the numbers
of rows and columns of characters required. and whether or not an activestip
is to be displayed.
-end*/
/*
************************************
**     xdl_text_table_getcsize    **
************************************

Purpose: Return the size requirements of the text cell view-object based
         on sizing specified by number of columns and rows of characters

Author:  John W. Campbell, June 1996

*/
/*-Fortran:
         CALL XDLF_TEXT_TABLE_GETCSIZE (NCOLS, NROWS, IFONT, IACT,
        +                              IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
NCOLS     (R)   No. of cols of characters or 0 (see ncols)
NROWS     (R)   No. of rows of characters or 0 (see nrows)
IFONT     (R)   Font type (1->5 very-small->extra-large) 
                (see font_type)
IACT      (R)   <=0 no active strip, =1 active strip present
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_text_table_getcsize (ncols, nrows, font_type, iact, w, h)
/*end*/
 
/*-Parameters:*/
int       ncols;          /* Number of columns of characters ( > 0 ) (R)*/
int       nrows;          /* Number of rows of characters ( > 0 ) (R)*/
int       font_type;      /* Font type (1-5) for very-small, small, medium,
                             large or extra-large font. (R)*/
int       iact;           /* <=0 no active strip =1 active strip (R)*/
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int ch_height;              /* Character height */
   int ch_width;               /* Character width */
   int ascent;                 /* Font ascent */
   int descent;                /* Font descent */

   /* Calculate the size requirements */

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;

   /* Get size requirements */

   xdl_ch_size (font_type, 2, &ch_width, &ch_height, &ascent, &descent);
   *w = ncols*ch_width + 2*MARG_W + 2 + 2*BORDER_WIDTH + 2;
   *h = nrows*ch_height + 2*MARG_H + 2 + 2*BORDER_HEIGHT + 2;
   if (iact<=0) return;
   if (*w<MIN_ACT_STRIP) *w = MIN_ACT_STRIP;
   *h = *h + ACT_STRIP_HEIGHT;
   return;
}

/* Fortran binding: xdlf_text_table_getcsize */

void xdlf_text_table_getcsize (ncols, nrows, ifont, iact, iwidth, iheight)
int *ncols, *nrows, *ifont, *iact, *iwidth, *iheight;
{
   xdl_text_table_getcsize (*ncols, *nrows, *ifont, *iact, iwidth, iheight);
   return;
}
