/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The XDL_VIEW Management Routines
-end*/
 
/*===============================*/
/*       XDL_VIEW routines       */
/*===============================*/

/*-Intro:
This file contains a series of routines concerned with the management
and coding of XDL_VIEW view-objects. Some are application callable
and have Fortran interfaces whereas the others are for internal use
within the source code of the view-objects.
-end*/
 

/*Routines List:
The following routines are available:

   Application Callable Routines

         xdl_open_view              Initialise XDL_VIEW
         xdl_get_events             Event handling loop
         xdl_open_file              Open file for input
         xdl_set_file_input         Set up file input
         xdl_clear_file_input       Clear up file input
         xdl_file_events            Event handling loop with file input
         xdl_get_record             Get last record read
         xdl_get_events_timeout     Event handling loop (with timeout)
         xdl_getio_events           Event handling and non-blocking reads
         xdl_timer_events           Timer driven event handling
         xdl_getio_string           Get string from non-blocking read
         xdl_tread                  Non-blocking terminal read
         xdl_flush_events           Service queued events
         xdl_sync                   XSync/XSynchronize calls
         xdl_delete_view_object     Delete a view-object

         xdlstr                     Return the address of a Fortran 
                                    character string

         xdl_view_object_rootxy     Get root x, y position from window x, y

         xdl_view_object_raise      Raise a view-object (to front)
         xdl_view_object_lower      Lower a view-object (to back)
         xdl_delay                  Delay for a requested time

   Routines for Use in Coding View-objects

     Creating Windows:

         xdl_create_base_frame_window    Create a base frame window
         xdl_create_frame_window         Create a frame window
         xdl_create_paint_window         Create a paint window

     Active Strip Windows Routines:

         xdl_create_active_strip     Create an active strip
         xdl_active_strip_on         Set active strip to 'on'
         xdl_active_strip_off        Set active strip to 'off'

     Adding new objects and windows to view-objects list

         xdl_new_view_object         Add new view-object to list
         xdl_add_window              Add window to list
         xdl_add_libwin              Add library routine window to list
         xdl_add_top_callbacks       Add callbacks for top level window

     Getting items from a view-objects list

         xdl_getiv                   Get offset in list from handle
         xdl_get_global              Get global data area from window id
         xdl_get_vh                  Get view-object handle from window id

     Deleting a panel item

         xdl_delete_panel_item       Delete a panel item

     Selection routines for string cut/paste

         xdl_select_owner            Request selection ownership
         xdl_select_cancel           Cancel selection ownership
         xdl_select_respond          Respond to selection request
         xdl_select_request          Make a selection request
         xdl_select_receive          Receive the requested selection

     Error Message Routines

         xdl_view_errmsg             XDL_VIEW error message (general)
         xdl_view_iv_errmsg          View-object not in list error message
         xdl_view_type_errmsg        Handle does not match routine type

     Some Miscellaneous Routines

        xdl_rootxy                  Get root x, y from window x, y
        xdl_copy_chars              Copy back characters to Fortran 
                                    character string
        xdl_blank_string            Test for blank (or null) string
        xdl_chop_blanks             Remove trailing blanks from a C string
        xdl_ndigit                  Find no. of digits in an integer
        xdl_upc                     Convert string to upper case
        xdl_lwc                     Convert string to lower case
        xdl_upcc                    Convert character to upper case
        xdl_lwcc                    Convert character to lower case
        xdl_rand16                  Random number, 0-32767
        xdl_srand16                 Set seed for xdl_rand16
        xdl_ch_size                 Get character size for a standard font
        xdl_set64gray               Set 64 colour gray scale
        xdl_set64gray_yel           Set 64 colour gray with yellow at top
        xdl_set64color              Set 64 colour blue>-red->orange->yellow
        xdl_get64gray               Get 64 colour gray scale
        xdl_get64gray_yel           Get 64 colour gray with yellow at top
        xdl_get64color              Get 64 colour blue>-red->orange->yellow

   Special Service Routines used by the XDL_VIEW routines

        xdl_delete_children         Delete a view-object's children
        xdl_any_children            See if a view-object has any children

*/
                                          
/* xdl-view routines include files */
#include "xdl_view.h"
#include "xdl_view_ext.h"

/* Initialise xdl_rand16 seed */
   unsigned long int xdl_next_seed = 1;

/* Global data XDL view-object handling  routines */
/* ============================================== */

/* The following items are set/initialised by a call to xdl_open_view */

Display      *XDL_display;           /*Display for use with Xdl routines*/
XtAppContext XDL_app;                /*Application Context*/
int          XDL_screen;             /*Screen no.*/
int          XDL_open = 0;           /*XDL_VIEW opened flag*/
int          XDL_num_view_objects;   /*Number of view-objects set up*/
int          XDL_max_alloc_objects;  /*Maximum number of view objects for
                                       which the XDL_view_objects pointers
                                       list has been set up*/
XDl_view_object  **XDL_view_objects; /*Array of pointers to XDL_view_object
                                       structures*/

int          XDL_pw_depth;	     /* Depth for paint windows */
int          XDL_image_depth;	     /* depth of selected image type */
int          XDL_image_bits;	     /* bits per pixel of selected
                                        image type */

XFontStruct  *XDL_fontinfo;          /*Standard (Font1) font*/
XFontStruct  *XDL_fontinfo_bold;     /*Standard (BoldFont1) bold font*/
XFontStruct  *XDL_fonts[5];  /* fonts very small, small, medium,
                                      large and very large*/
XFontStruct  *XDL_bold_fonts[5]; /* bold fonts very small, small,
                                          medium, large and very large */
Cursor       XDL_default_cursor;    /*Default cursor for view_objects*/

int          XDL_color;               /*Colour availability flag
                                        = 0, not available - monochrome
                                        = 1, grays
                                        = 2, colour*/
int          XDL_display_type;        /*Display type to be used 
                                        = StaticGray, GrayScale,
                                          StaticColor, PseudoColor,
                                          TrueColor or DirectColor
                                          (Values defined in X.h)*/
int          XDL_csets_alloc;        /* =1 requested colour sets allocated
                                           with writeable colour cells
                                        =0 requested colour sets not
                                           allocated (static display type
                                           or less than 8 bit planes)*/
int          XDL_display_cells;       /*No. of color cells available*/
Visual       *XDL_visual;             /*Visual for creating windows*/
Colormap     XDL_colormap;            /*Colormap selected for the application*/
int          XDL_num_colorsets;       /*Number of colormap sets (in addition to
                                        the standard short pallette)*/
XDl_colorset XDL_colorsets[MAXCOLORSETS];
                                      /*Array of allocated colormap sets*/
unsigned long  XDL_blackpixel;        /*Black pixel no.*/
unsigned long  XDL_whitepixel;        /*White pixel no.*/
unsigned long  XDL_redpixel;          /*Red pixel no.*/
unsigned long  XDL_yellowpixel;       /*Yellow pixel no.*/
unsigned long  XDL_greenpixel;        /*Green pixel no.*/
unsigned long  XDL_cyanpixel;         /*Cyan pixel no.*/
unsigned long  XDL_bluepixel;         /*Blue pixel no.*/
unsigned long  XDL_magentapixel;      /*Magenta pixel no.*/

GC XDL_active_strip_gc;          /*GC for standard 'view-object active for
                                   input to the application' window use*/

int XDL_return_data_vh;          /*Data returned from 'active' view-object
                                   flag = 0, none returned
                                        > 0, the view-object handle of the
                                             view-object returning the data*/
Atom XDL_paste_buffer;           /*Atom for xdl_view cut/paste string buffers */
int XDL_tmr_events;              /*Timer events handling in progress flag
                                   =0 no, =1 yes */

/* Pointers list for XDLSTR */

char * XDL_pointers[MAXPOINTERS];  /* List of string pointers */
int    XDL_pointers_index;         /* Last index to pointers currently used
                                      (list used in a circular manner */

/* Global data & definitions for non-blocking input handling */

#define MAXINPSTR 250

char XIO_str[MAXINPSTR+1];        /* Input string for non-blocking 
                                     terminal input */
int       XIO_numstr;             /* No. of characters currently in XIO_str */
int       XIO_newstr;             /* =1 start a new input string, =0 do not */
int       XIO_flags;              /* Saved flags for 'fcntl' routine */
int       XIO_func;               /* QIOW function code for VMS */
int       XIO_channel;            /* QIOW channel for VMS */
XClientMessageEvent XIO_mesg;     /* Client message to ensure event on queue on
                                     return from signal handler*/

/* Global data & definitions for file input events handling */

int  XINP_fid;                  /* File id */
char XINP_str[MAXINPSTR+1];     /* Current input string for file input */
char XINP_laststr[MAXINPSTR+1]; /* Last input string for file input */
int       XINP_numstr;          /* No. of characters currently in XINP_str */

#ifdef HAVE_SIGACTION
 struct sigaction           
                    XIO_vec_alarm,        /* Set alarm signal handling */
                    XIO_ovec_alarm,       /* Save old alarm signal handling */
                    XIO_vec_ctrc,         /* Set ctr_C signal handling */
                    XIO_ovec_ctrc;        /* Save old ctr_C signal handling */
#endif

#ifdef VMS
#include <iodef.h>
#include <descrip.h>
$DESCRIPTOR (XIO_terminal,"SYS$COMMAND");   /* Terminal descriptor block */
#endif

/* Application and service routine Declarations */
/* -------------------------------------------- */

int xdl_open_view();
int xdl_get_events();
void xdl_open_file();
void xdl_set_file_input();
void clear_set_file_input();
int xdl_file_events();
int xdl_read_next_rec ();
int xdl_get_record ();
int xdl_get_events_timeout();
int xdl_getio_events();
int xdl_timer_events();
void xdlsys_nonblock();
void xdlsys_ctr_c();
void xdlsys_alarm();
void xdlsys_read();
void xdlevents_alarm();
int xdl_getio_string();
int xdl_tread();
void xdl_flush_events();
void xdl_sync();
int xdl_delete_view_object();
int xdlstr();
int xdl_view_object_rootxy();
int xdl_view_object_raise();
int xdl_view_object_lower();
void xdl_delay();
void xdl_timdum();
Window xdl_create_base_frame_window();
Window xdl_create_frame_window();
Window xdl_create_paint_window();
Window xdl_create_active_strip();
void xdl_active_strip_on();
void  xdl_active_strip_off();
int xdl_new_view_object();
void xdl_add_window();
void xdl_add_libwin();
void xdl_add_top_callbacks();
char * xdl_get_global();
int xdl_get_vh();
int xdl_getiv();
void xdl_delete_panel_item();
void xdl_select_owner();
void xdl_select_cancel();
void xdl_select_respond();
void xdl_select_request();
int xdl_select_receive();
void xdl_view_errmsg();
void xdl_view_iv_errmsg();
void xdl_view_type_errmsg();
void xdl_rootxy();
void xdl_copy_chars();
int xdl_blank_string();
void xdl_chop_blanks();
int xdl_ndigit();
void xdl_upc();
void xdl_lwc();
void xdl_upcc();
void xdl_lwcc();
int xdl_rand16();
void xdl_srand16();
void xdl_ch_size();
int xdl_set64gray();
int xdl_set64gray_yel();
int xdl_set64color();
int xdl_get64gray();
int xdl_get64gray_yel();
int xdl_get64color();
void xdl_delete_children();
int xdl_any_children();
int xdl_any_children();
int IgnoreError();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_open_view xdlf_open_view_
#define xdlf_get_events xdlf_get_events_
#define xdlf_open_file xdlf_open_file_
#define xdlf_set_file_input xdlf_set_file_input_
#define xdlf_clear_file_input xdlf_clear_file_input_
#define xdlf_file_events xdlf_file_events_
#define xdlf_get_record xdlf_get_record_
#define xdlf_get_events_timeout xdlf_get_events_timeout_
#define xdlf_getio_events xdlf_getio_events_
#define xdlf_timer_events xdlf_timer_events_
#define xdlf_getio_string xdlf_getio_string_
#define xdlf_tread xdlf_tread_
#define xdlf_flush_events xdlf_flush_events_
#define xdlf_sync xdlf_sync_
#define xdlf_delete_view_object xdlf_delete_view_object_
#define xdlstr xdlstr_
#define xdlf_view_object_rootxy xdlf_view_object_rootxy_
#define xdlf_view_object_raise xdlf_view_object_raise_
#define xdlf_view_object_lower xdlf_view_objct_lower_
#define xdlf_delay xdlf_delay_

#endif

#if  LINKTYP == 2

#define xdlf_open_view XDLF_OPEN_VIEW
#define xdlf_get_events XDLF_GET_EVENTS
#define xdlf_open_file XDLF_OPEN_FILE
#define xdlf_set_file_input XDLF_SET_FILE_INPUT
#define xdlf_clear_file_input XDLF_CLEAR_FILE_INPUT
#define xdlf_file_events XDLF_FILE_EVENTS
#define xdlf_get_record XDLF_GET_RECORD
#define xdlf_get_events_timeout XDLF_GET_EVENTS_TIMEOUT
#define xdlf_getio_events XDLF_GETIO_EVENTS
#define xdlf_timer_events XDLF_TIMER_EVENTS
#define xdlf_getio_string XDLF_GETIO_STRING
#define xdlf_tread XDLF_TREAD
#define xdlf_flush_events XDLF_FLUSH_EVENTS
#define xdlf_sync XDLF_SYNC
#define xdlf_delete_view_object XDLF_DELETE_VIEW_OBJECT
#define xdlstr XDLSTR
#define xdlf_view_object_rootxy XDLF_VIEW_OBJECT_ROOTXY
#define xdlf_view_object_raise XDLF_VIEW_OBJECT_RAISE
#define xdlf_view_object_lower XDLF_VIEW_OBJECT_LOWER
#define xdlf_delay XDLF_DELAY

#endif

void xdlf_open_view();
void xdlf_get_events();
void xdlf_open_file();
void xdlf_set_file_input();
void xdlf_clear_file_input();
void xdlf_file_events();
void xdlf_get_record();
void xdlf_get_events_timeout();
void xdlf_getio_events();
void xdlf_timer_events();
void xdlf_getio_string();
void xdlf_tread();
void xdlf_flush_events();
void xdlf_sync();
void xdlf_delete_view_object();
void xdlf_view_object_rootxy();
void xdlf_view_object_raise();
void xdlf_view_object_lower();
void xdlf_delay();



/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

 
/*-Section: Application Callable Management Routines
These are routines commonly used by the application programmer making
use of the xdl_view package. There are routines to initialise the XDL_VIEW 
package, handle events and delete view-objects. A routine is provided 
which must be used when passing character strings from a Fortran program.
There are further routines to raise and lower windows and get their positions.
-end*/

/*-Routine: XDL_VIEW initialisation - xdl_open_view
The initialisation routine xdl_open_view (xdlf_open_view) must be called 
before any other XDL_VIEW routine. It initialises the list of view-objects.
It also sets up, in addition to a a small default colour map which contains 
the colours black, white and the six primary and secondary colours, colour 
maps based on the user's requirements for the application if possible. 
In addition to these items, a number of other global data items, which may 
be used within the code for the individual view-objects, are set up by the 
initialisation routine. 

Fonts will normally be defined as X-windows resources Xdl*font1...Xdl*font5 
and Xdl*boldFont1...Xdl*boldFont5. There are five normal and five bold fonts. 
These should normally be fixed width fonts. Each series must be in ascending 
size order. Bold fonts must match normal fonts to within 1 pixel in width 
and two in height. Some of the view-objects assume that the small font does 
not exceed 7x13 in pixels in size; a warning will be output if this size
is exceeded.
-end*/

/*-Doc:
The following defaults are used by xdl_open_view if the font resources 
are not defined:

Normal fonts

    *adobe-courier-medium-r*--8*
    *adobe-courier-medium-r*--12*
    *adobe-courier-medium-r*--14*
    *adobe-courier-medium-r*--18*
    *adobe-courier-medium-r*--24*

Bold fonts

    *adobe-courier-bold-r*--8*
    *adobe-courier-bold-r*--12*
    *adobe-courier-bold-r*--14*
    *adobe-courier-bold-r*--18*
    *adobe-courier-bold-r*--24*
-end*/
/*                                            
************************
**    xdl_open_view   **
************************

Purpose: Initialise xdl_view

Author:  John W. Campbell, April 1990

*/
/*-Fortran:
         CALL XDLF_OPEN_VIEW (NUMC, NCOLORS, NPLANES, IERR)
-end*/
 
/*-Parameters:
NUMC     (R)  No. of colorsets (see numc)
NCOLORS  (R)  Integer array of nos. of colors (see ncolors)
NPLANES  (R)  Integer array of nos. of overlay planes 
              (see nplanes)
IERR     (W)  Returns the status from the xdl_open_view call
-end*/

/*-C:*/
int xdl_open_view (numc, ncolors, nplanes)
/*end*/
 
/*-Parameters:*/
int numc;           /*Number of colorsets to be allocated in addition
                      to the standard short pallette (may be 0) (R)*/
int ncolors[];      /*Array of 'ncolors' values for the XAllocColorCells
                      routines (numc values) (R)*/
int nplanes[];      /*Array of 'nplanes' values for the XAllocColorCells
                      routines (numc values) (R)*/

/*end*/
/*-Doc:
Return:  =0 if OK, >0 if error:
                      bit 0  set unable to allocate the requested 
                             colorsets.
                             (XDL_csets_alloc will be set to 0)
                             non-fatal - view-objects should cope with this
                             case.
-end*/
{
   char fonterr[201];                /*Font error message string*/
   int err;                          /*Error flag*/
   int nvisual;                      /*Number of required visuals available*/
   XVisualInfo visual_info, *vlist;  /*Temporary structures*/
   Visual *visual;                   /*Temporary Visual structure*/
   XColor xcolor, rgb_def;           /*Temporary XColor structures*/
   int retried;                      /*Flag to tell if re-allocation of
                                       colour map already tried =1 yes, =0 no*/
   int i,j;                          /*Temporary/loop variables*/
   int nshort;                       /*No. of entries in short standard
                                       colorset*/
   int status;                       /*Status return from Xlib routines*/
   int ch_w1, ch_w2;                 /*Character widths*/
   int ch_h1, ch_h2;                 /*Character heights*/
   int ascent, descent;              /*Font ascent, descent*/
   int asc;                          /*Character ascending size flag*/
   int good_match;                   /*Bold/normal fonts good size match flag*/
   Window temp_win;                  /*Temporary window for creating GC*/
   XWindowAttributes attrib;         /* Window attributes structure */
   XSetWindowAttributes set_attrib;  /* Set window attributes structure */
   static char *xdl_font_names[5];
   static char *xdl_boldfont_names[5];

   static char *font_msg[] =
   {
      "Xdl*font1",
      "Xdl*font2",
      "Xdl*font3",
      "Xdl*font4",
      "Xdl*font5" }; /* Fontname messages*/

   static char *font_bold_msg[] =
   {
      "Xdl*boldFont1",
      "Xdl*boldFont2",
      "Xdl*boldFont3",
      "Xdl*boldFont4",
      "Xdl*boldFont5" }; /* BoldFontname messages*/

   static char *font_default[] =
     {
       "*adobe-courier-medium-r*--8*", 
       "*adobe-courier-medium-r*--12*", 
       "*adobe-courier-medium-r*--14*", 
       "*adobe-courier-medium-r*--18*", 
       "*adobe-courier-medium-r*--24*"};   /* Default fontnames */

   static char *font_bold_default[] =
     {
       "*adobe-courier-bold-r*--8*", 
       "*adobe-courier-bold-r*--12*", 
       "*adobe-courier-bold-r*--14*", 
       "*adobe-courier-bold-r*--18*", 
       "*adobe-courier-bold-r*--24*"};   /* Default fontnames */


   /*Initialize X*/

   err = 0;
   if (XDL_open) return 0;
   XtToolkitInitialize();
   XDL_app = XtCreateApplicationContext();
   XDL_display = XOpenDisplay(NULL);
   if (XDL_display==NULL)
   {
      xdl_view_errmsg ("xdl_open_view", "Unable to open display");
      exit(1);
   }

   XDL_screen = DefaultScreen (XDL_display);

   /*Load and save fonts and set default font*/
   /* Get fontnames for resource data base */
   xdl_font_names[0] = XGetDefault(XDL_display,"Xdl","font1");
   xdl_font_names[1] = XGetDefault(XDL_display,"Xdl","font2");
   xdl_font_names[2] = XGetDefault(XDL_display,"Xdl","font3");
   xdl_font_names[3] = XGetDefault(XDL_display,"Xdl","font4");
   xdl_font_names[4] = XGetDefault(XDL_display,"Xdl","font5");
   
   xdl_boldfont_names[0] = XGetDefault(XDL_display,"Xdl","boldFont1");
   xdl_boldfont_names[1] = XGetDefault(XDL_display,"Xdl","boldFont2");
   xdl_boldfont_names[2] = XGetDefault(XDL_display,"Xdl","boldFont3");
   xdl_boldfont_names[3] = XGetDefault(XDL_display,"Xdl","boldFont4");
   xdl_boldfont_names[4] = XGetDefault(XDL_display,"Xdl","boldFont5");


   for (i=0;i<5;++i)
   {
      if (xdl_font_names[i]!=NULL)
      {
         XDL_fonts[i] = XLoadQueryFont (XDL_display, xdl_font_names[i]);
         if (XDL_fonts[i]==NULL)
         {
            sprintf (fonterr, 
            "Unable to load %s font,\n   name = %s,\n   default %s used",
            font_msg[i], xdl_font_names[i], font_default[i]);
            xdl_view_errmsg ("xdl_open_view", fonterr);
            xdl_font_names[i] = font_default[i];
            XDL_fonts[i] = XLoadQueryFont (XDL_display, xdl_font_names[i]);
            if (XDL_fonts[i]==NULL)
            {
               sprintf (fonterr, "Unable to load %s font", font_default[i]);
               xdl_view_errmsg ("xdl_open_view", fonterr);
               exit (1);
            }
         }
      }
      else
      {
         xdl_font_names[i] = font_default[i];
         XDL_fonts[i] = XLoadQueryFont (XDL_display, xdl_font_names[i]);
         if (XDL_fonts[i]==NULL)
         {
            sprintf (fonterr, "Unable to load %s font", font_default[i]);
            xdl_view_errmsg ("xdl_open_view", fonterr);
            exit (1);
         }
      }
      if (xdl_boldfont_names[i]!=NULL)
      {
         XDL_bold_fonts[i] = XLoadQueryFont (XDL_display, 
                                             xdl_boldfont_names[i]);
         if (XDL_bold_fonts[i]==NULL)
         {
            sprintf (fonterr, 
                   "Unable to load %s font,\n   name = %s,\n   default %s used",
                   font_bold_msg[i], xdl_boldfont_names[i], 
                   font_bold_default[i]);
            xdl_view_errmsg ("xdl_open_view", fonterr);
            xdl_boldfont_names[i] = font_bold_default[i];
            XDL_bold_fonts[i] = XLoadQueryFont (XDL_display, 
                                                xdl_boldfont_names[i]);
            if (XDL_bold_fonts[i]==NULL)
            {
               sprintf (fonterr, "Unable to load %s font", 
                        font_bold_default[i]);
               xdl_view_errmsg ("xdl_open_view", fonterr);
               exit (1);
            }
         }
      }
      else
      {
         xdl_boldfont_names[i] = font_bold_default[i];
         XDL_bold_fonts[i] = XLoadQueryFont (XDL_display, 
                                             xdl_boldfont_names[i]);
         if (XDL_bold_fonts[i]==NULL)
         {
            sprintf (fonterr, "Unable to load %s font", 
                     font_bold_default[i]);
            xdl_view_errmsg ("xdl_open_view", fonterr);
            exit (1);
         }
      }
   }

   /* Checks on font dimensions*/

   asc = 1;
   for (i=2;i<=5;++i)
   {
      xdl_ch_size (i-1,0,&ch_w1,&ch_h1,&ascent,&descent);
      xdl_ch_size (i,0,&ch_w2,&ch_h2,&ascent,&descent);
      if (ch_w2<ch_w1) asc = 0;
      if (ch_h2<ch_h1) asc = 0;
      xdl_ch_size (i-1,1,&ch_w1,&ch_h1,&ascent,&descent);
      xdl_ch_size (i,1,&ch_w2,&ch_h2,&ascent,&descent);
      if (ch_w2<ch_w1) asc = 0;
      if (ch_h2<ch_h1) asc = 0;
   }
   if (asc==0)
   {
      xdl_view_errmsg ("xdl_open_view","Fonts sizes out of order");
      exit(1);
   }

   good_match = 1;
   for (i=1;i<=5;++i)
   {
      xdl_ch_size (i,0,&ch_w1,&ch_h1,&ascent,&descent);
      xdl_ch_size (i,1,&ch_w2,&ch_h2,&ascent,&descent);
      if (abs(ch_w1-ch_w2)>1) good_match = 0;
      if (abs(ch_h1-ch_h2)>2) good_match = 0;
   }
   if (good_match==0)
   {
      xdl_view_errmsg ("xdl_open_view",
                       "Poor match between normal and bold font sizes");
   }

   xdl_ch_size (2,2,&ch_w1,&ch_h1,&ascent,&descent);
   if (ch_w1>7||ch_h1>13)
   {
      sprintf (fonterr,"Standard small font of %dx%d is too large (max = 7x13)",
               ch_w1, ch_h1);
      xdl_view_errmsg ("xdl_open_view", fonterr);
   }

   XDL_fontinfo = XDL_fonts[1];
   XDL_fontinfo_bold = XDL_bold_fonts[1];

   /*Create and save default cursor*/

   XDL_default_cursor = XCreateFontCursor (XDL_display,XC_left_ptr);

   /* Set default display (colour) parameters */
   
   visual = DefaultVisual (XDL_display, XDL_screen);
   XGetWindowAttributes (XDL_display, RootWindow (XDL_display, XDL_screen),
                         &attrib);   
   visual = attrib.visual;
   XDL_visual = visual;
   XDL_pw_depth = attrib.depth;
   XDL_colormap = DefaultColormap (XDL_display, XDL_screen);
   XDL_display_type = visual->class;
   XDL_display_cells = DisplayCells (XDL_display, XDL_screen);
   XDL_csets_alloc = 0;

   /*Get 8 bit Pseudocolour if not the default and if available*/
   /* If this causes problems for some platforms, comment out
      using COLMAP_TYPE variable set in xdl_systyp.h */

   if (XDL_display_type!=PseudoColor||XDL_pw_depth!=8)
   {
      visual_info.class = PseudoColor;
      visual_info.screen = XDL_screen;
      visual_info.depth = 8;
      vlist = XGetVisualInfo (XDL_display, 
                              VisualScreenMask|VisualClassMask|VisualDepthMask,
                              &visual_info, &nvisual);
      if (nvisual > 0)
      {
         XDL_visual = vlist[0].visual;
         XDL_display_type = PseudoColor;
         XDL_pw_depth = 8;
         XDL_colormap = XCreateColormap (XDL_display, 
                                         RootWindow (XDL_display, XDL_screen),
                                         XDL_visual, 
                                         AllocNone);
	 /* DL: this looks dodgy.  */
	 XSetErrorHandler(IgnoreError);
      }
   }

   XDL_color = 0;
   if (XDL_pw_depth>1)
     switch (XDL_display_type)
       {
       case StaticGray:
       case GrayScale:
         XDL_color = 1;
	 break;
	 /* case StaticColor:
	    case PseudoColor:
	    case TrueColor:
	    case DirectColor: */
       default:
         XDL_color = 2;
       }

   /* Look for a suitable image depth, 8 preferred.  */
   {
     XPixmapFormatValues *pixmap_formats;
     int npix;
     int i, depth;

     pixmap_formats = XListPixmapFormats (XDL_display, &npix);
     if (! pixmap_formats)
       {
	 xdl_view_errmsg ("xdl_open_view",
			  "Can't get of display's pixmap formats");
	 exit (EXIT_FAILURE);
       }
     XDL_image_depth = 0;
     for (i = 0; i < npix; i++)
       {
	 depth = pixmap_formats[i].depth;
	 if (depth == 8)	/* preferred */
	   {
	     XDL_image_depth = 8;
	     XDL_image_bits = pixmap_formats[i].bits_per_pixel;
	     break;
	   }
	 else if (((XDL_image_depth < 8) /* prefer deeper than 8 */
		   && (depth > XDL_image_depth))		  
		  || (depth < XDL_image_depth)) /* but as shallow as possible */
	   {
	     XDL_image_bits = pixmap_formats[i].bits_per_pixel;
	     XDL_image_depth = depth;
	   }
       }
   }

   /* Allocate color cells for a standard short pallette */
   /* If monochrome all colours apart from white will be set to black */
   /* If gray then different grays will be allocated for the colours */

   nshort= 8;
   XDL_num_colorsets = 1;
   XDL_colorsets[0].colormap = XDL_colormap;
   XDL_colorsets[0].ncolors = nshort;
   XDL_colorsets[0].pixels = (unsigned long *) malloc
                             (nshort*sizeof(unsigned long));
   XDL_colorsets[0].nplanes = 0;
   XDL_colorsets[0].plane_masks = 0;

   status = XAllocNamedColor (XDL_display, XDL_colormap, "white",
                              &xcolor, &rgb_def);
   XDL_whitepixel = xcolor.pixel;
   XDL_colorsets[0].pixels[0] = XDL_whitepixel;

   status = XAllocNamedColor (XDL_display, XDL_colormap, "black",
                              &xcolor, &rgb_def);
   XDL_blackpixel = xcolor.pixel;
   XDL_colorsets[0].pixels[1] = XDL_blackpixel;

   if (XDL_color>0)
   {
      status = XAllocNamedColor (XDL_display, XDL_colormap, "red",
                                 &xcolor, &rgb_def);
      XDL_redpixel = xcolor.pixel;
      XDL_colorsets[0].pixels[2] = XDL_redpixel;
   
      status = XAllocNamedColor (XDL_display, XDL_colormap, "yellow",
                                 &xcolor, &rgb_def);
      XDL_yellowpixel = xcolor.pixel;
      XDL_colorsets[0].pixels[3] = XDL_yellowpixel;
  
      status = XAllocNamedColor (XDL_display, XDL_colormap, "green",
                                 &xcolor, &rgb_def);
      XDL_greenpixel = xcolor.pixel;
      XDL_colorsets[0].pixels[4] = XDL_greenpixel;
 
      status = XAllocNamedColor (XDL_display, XDL_colormap, "cyan",
                              &xcolor, &rgb_def);
      XDL_cyanpixel = xcolor.pixel;
      XDL_colorsets[0].pixels[5] = XDL_cyanpixel;
  
      status = XAllocNamedColor (XDL_display, XDL_colormap, "blue",
                                 &xcolor, &rgb_def);
      XDL_bluepixel = xcolor.pixel;
      XDL_colorsets[0].pixels[6] = XDL_bluepixel;

      status = XAllocNamedColor (XDL_display, XDL_colormap, "magenta",
                              &xcolor, &rgb_def);
      XDL_magentapixel = xcolor.pixel;
      XDL_colorsets[0].pixels[7] = XDL_magentapixel;
   }
   else
   {
      for (i=2;i<=7;++i) XDL_colorsets[0].pixels[i] = XDL_blackpixel;
   }

   /* Allocate required colorsets */
   
   retried=0;
   if (numc>0)
   {
      for (i=1;i<=numc;++i)
      {
         XDL_colorsets[i].colormap = XDL_colormap;
         XDL_colorsets[i].ncolors = ncolors[i-1];
         XDL_colorsets[i].pixels = (unsigned long *) malloc
                                   (ncolors[i-1]*sizeof(unsigned long));
         XDL_colorsets[i].nplanes = nplanes[i-1];
         XDL_colorsets[i].plane_masks = 0;
         if (nplanes[i-1]>0)
         {    
            XDL_colorsets[i].plane_masks = (unsigned long *) malloc
                                           (nplanes[i-1]*sizeof(unsigned long));
         }
         status = XAllocColorCells (XDL_display, XDL_colormap, False,
                                    XDL_colorsets[i].plane_masks,
                                    XDL_colorsets[i].nplanes,
                                    XDL_colorsets[i].pixels,
                                    XDL_colorsets[i].ncolors);
         if (status==0)
         {
            if (retried==1)
            {
               err |= 1;
               break;
            }else{
               retried = 1;
               XDL_colormap = XCopyColormapAndFree (XDL_display, XDL_colormap);
               for (j=0;j<i;++j) XDL_colorsets[j].colormap = XDL_colormap;
               status = XAllocColorCells (XDL_display, XDL_colormap, False,
                                          XDL_colorsets[i].plane_masks,
                                          XDL_colorsets[i].nplanes,
                                          XDL_colorsets[i].pixels,
                                          XDL_colorsets[i].ncolors);
               if (status==0)
               {
                  err |= 1;
                  break;
               }
            }
         }
      }
      if ((err&1)==0)
      {
         XDL_num_colorsets = numc + 1;
         XDL_csets_alloc = 1;
      }
   }

   /* Create a 'dummy' paint window for setting up a GC for the active strip*/

   set_attrib.colormap = XDL_colormap;
   temp_win = XCreateWindow (XDL_display, RootWindow (XDL_display, XDL_screen), 
                             0, 0, 10, 10, 1, XDL_pw_depth, InputOutput, 
                             XDL_visual, CWColormap, &set_attrib); 
/*                           CopyFromParent, 0, NULL); */
   XDL_active_strip_gc = XCreateGC (XDL_display, temp_win, 0, NULL);

   /* Get Atom for xdl_view string cut/paste buffers */

   XDL_paste_buffer = XInternAtom (XDL_display, "xdl_paste_buffer",
                                 False);
   
   /*Initialise view-objects table*/

   XDL_num_view_objects = 0;
   XDL_max_alloc_objects = 0;

   /* Initialise for non-blocking reads code */

   XIO_str[0] = 0;
   XIO_numstr = 0;
   XIO_newstr = 1;

   /* Initialise for file input */

   XINP_fid = -1;
   XINP_str[0] = 0;
   XINP_laststr[0] = 0;
   XINP_numstr = 0;
 
   /* Initialise timer events flag */

   XDL_tmr_events = 0;

   /* XDLSTR pointers list initialisation */
   
   for (i=0;i<MAXPOINTERS;++i) XDL_pointers[i] = 0;
   XDL_pointers_index = MAXPOINTERS;

   /* Set xdl_view open flag */

   XDL_open = 1;

   return err;
}

/* Fortran binding: xdlf_open_view */

void xdlf_open_view (numc, ncolors, nplanes, ierr)
int *numc, *ncolors, *nplanes, *ierr;
{
   *ierr =  xdl_open_view (*numc,  ncolors,  nplanes);
   return;
}

/*-Routine: XDL_VIEW event handling - xdl_get_events
An important part of XDL_VIEW is the X event handling mechanism. This is
normally done through a call from the application program to the
routine xdl_get_events (xdlf_get_events). When the routine is called, the 
user specifies a list of the view objects from which the program is ready 
to receive input ('active' view-objects). The routine services all the 
X events, passing them on to the approriate event handling routines until 
an event causes program input from one of the active view-objects; when 
this occurs, a return is made from the routine giving the view-object handle
of the routine from which the input originated.
-end*/
/*
***************************
**     xdl_get_events    **
***************************

Purpose: Set up loop for event processing returning when data are sent
         back from the specified view-objects.

Author:  John W. Campbell, April 1990

*/
/*-Fortran:
         CALL XDLF_GET_EVENTS (NUMIVH, IVHLIST, IVH)
-end*/
 
/*-Parameters:
NUMIVH   (R)  Number of view-objects to return data (see num-vh)
IVHLIST  (R)  Array of NUMIVH view object handles (see vh_list)
IVH      (W)  Returns the view-object handle of object returning
              the data (return from xdl_get_events)
              the data (return from xdl_get_events)
-end*/

/*-C:*/
int xdl_get_events (num_vh, vh_list)
/*end*/
 
/*-Parameters:*/
int num_vh;      /* Number of view objects from which data are to be
                    returned (may be 0 if no return is required)
                    if -num_vh is given then the routine be in a no wait
                    mode and will return when either input is received or
                    all current events have been flushed (R)*/
int vh_list[];   /* Array of the num_vh view-object handles (R)*/

/*end*/
/*-Doc:
Return:  The view-object handle of the view object returning the data.
         (May be 0 in no wait mode)
-end*/
{
   int i;                /* Temporary loop variable */
   int vh;               /* The returned view-object handle */
   int iv;               /* Offset into XDL_view_objects array for the given 
                            view-object handle */
   int num_win;          /* Number of windows */
   int no_wait;          /* Do not wait for view-object input */
   int on_off;           /* Input from view-object on/off flag */
   XEvent event;         /* X event */
   XEvent xevent;        /* X event */
   XAnyEvent xany;       /* XAnyEvent structure */
   Window wid;           /* Window id */
   XDl_window *win_list; /* Array of XDl_window structures */
   XDl_window *wp;       /* Pointer to and individual XDl_window structure */
   char * gd;            /* Pointer to global data area */
   void (*callback)();   /* Pointer to callback routine */

   /* Start event loop and wait for returned data */
   /* Call on-off routines as needed at start and end */

   XDL_return_data_vh = 0;
   no_wait = 0;
   xdl_timer_events(0);
   if (num_vh<0)
   {
      no_wait = 1;
      num_vh = - num_vh;
   }

   if (XDL_num_view_objects>0)
   {
      for (iv=0;iv<XDL_num_view_objects;++iv)
      {
         on_off=0;
         if (num_vh>0)
         {
            for (i=0;i<num_vh;++i)
            {
               if (XDL_view_objects[iv]->vh==vh_list[i])
               {
                  on_off=1;
                  break;
               }
            }
         }
         if (XDL_view_objects[iv]->on_off_state!=on_off)
         {
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, on_off);
            }
            XDL_view_objects[iv]->on_off_state = on_off;
         }
      }
   }
   XFlush (XDL_display);

   next:
   if (no_wait&&XPending(XDL_display)==0)
   {
      XFlush (XDL_display);
      return 0;
   }

   XNextEvent (XDL_display, &event);
   wid = event.xany.window;

  /*  printf ("Event %d, window %d\n", event.type, wid); */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      num_win = XDL_view_objects[iv]->num_win;
      if (num_win>0)
      {
         win_list = XDL_view_objects[iv]->win_list;
         for (i=0;i<num_win;++i)
         {
            wp = &(win_list[i]);
            if (wid==wp->wid)
            {
               gd = XDL_view_objects[iv]->gd;
               callback = NULL;
               if (wp->libdata!=NULL)
               {
                  if (wp->libdata->gd!=NULL) gd = wp->libdata->gd;
                  callback = wp->libdata->callback;
               }
               if (event.type==Expose||event.type==ConfigureNotify)
               {
                  if (wp->repaint!=NULL)
                  {
                     (*wp->repaint) (wid, &event, iv, gd, callback);
                  }
                  xdl_delay(100);
		  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         ConfigureNotify, &xevent));
                  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         Expose, &xevent));
                  goto winfound;
               }
               else
               {
                  if (wp->ehp!=NULL)
                  {
                     (*wp->ehp) (wid, &event, iv, gd, callback);
                  }
                  goto winfound;
               }
            }
         }
      }
   }
   goto next;
  
   winfound:
   if (XDL_return_data_vh==0) goto next;
   if (num_vh<=0) goto next;

   vh = XDL_return_data_vh;
   XDL_return_data_vh = 0;


   /* See if the returning view-object is in the list of view-objects from
      which events are required */

   for (i=0;i<num_vh;++i)
   {
      if (vh == vh_list[i])
      {
         for (i=0;i<num_vh;++i)
         {
            iv = xdl_getiv (vh_list[i]);
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, 0);
            }
            XDL_view_objects[iv]->on_off_state = 0;
         }
         XFlush (XDL_display);
         return vh;
      }
   }
   goto next;
}

/* Fortran binding: xdlf_get_events */

void xdlf_get_events (numivh, ivhlist, ivh)
int *numivh, *ivhlist, *ivh;
{
   *ivh = xdl_get_events (*numivh, ivhlist);
   return;
}

/*-Routine: XDL_VIEW event flushing - xdl_flush_events
The routine xdl_flush_events (xdlf_flush_events) services queued up events.
It may be called at intervals during a long calculation to keep event handling 
going. It is only used when the application is not waiting to receive data 
from a view-object. It returns when the event queue is empty.
-end*/
/*
*****************************
**     xdl_flush_events    **
*****************************

Purpose: Service queued up events (may be called at intervals during a long
         calculation to keep event handling going). Only used when the
         application is not waiting to receive data from a view-object.
         Returns when the queue is empty.

Author:  John W. Campbell, October 1990

*/
/*-Fortran:
         CALL XDLF_FLUSH_EVENTS (IDUM)
-end*/
 
/*-Parameters:
IDUM     (R)  Dummy parameter
-end*/

/*-C:*/
void xdl_flush_events (dummy)
/*end*/
 
/*-Parameters:*/
int dummy;       /* Dummy parameter (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int i;                /* Loop variable */
   int iv;               /* Offset into XDL_view_objects array for the given 
                            view-object handle */
   int num_win;          /* Number of windows */
   XEvent event;         /* X event */
   XEvent xevent;        /* X event */
   XAnyEvent xany;       /* XAnyEvent structure */
   Window wid;           /* Window id */
   XDl_window *win_list; /* Array of XDl_window structures */
   XDl_window *wp;       /* Pointer to and individual XDl_window structure */
   char * gd;            /* Pointer to global data area */
   void (*callback)();   /* Pointer to callback routine */


   while (XPending (XDL_display)>0)
   {
      XNextEvent (XDL_display, &event);
      wid = event.xany.window;

      /* printf ("Event %d, window %d\n", event.type, wid); */

      if (XDL_num_view_objects==0) return;

      for (iv=0;iv<XDL_num_view_objects;++iv)
      {
         num_win = XDL_view_objects[iv]->num_win;
         if (num_win>0)
         {
            win_list = XDL_view_objects[iv]->win_list;
            for (i=0;i<num_win;++i)
            {
               wp = &(win_list[i]);
               if (wid==wp->wid)
               {
                  gd = XDL_view_objects[iv]->gd;
                  callback = NULL;
                  if (wp->libdata!=NULL)
                  {
                     if (wp->libdata->gd!=NULL) gd = wp->libdata->gd;
                     callback = wp->libdata->callback;
                  }
                  if (event.type==Expose||event.type==ConfigureNotify)
                  {
                     if (wp->repaint!=NULL)
                     {
                        (*wp->repaint) (wid, &event, iv, gd, callback);
                     }
                     xdl_delay(100);
		     while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                           ConfigureNotify, &xevent));
                     while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                            Expose, &xevent));
                     goto winfound;
                  }
                  else
                  {
                     if (wp->ehp!=NULL)
                     {
                        (*wp->ehp) (wid, &event, iv, gd, callback);
                     }
                     goto winfound;
                  }
               }
            }
         }
      }
      winfound:
      continue;
   }
   return;
}

/* Fortran binding: xdlf_flush_events */

void xdlf_flush_events (idum)
int *idum;
{
   xdl_flush_events (*idum);
   return;
}

/*-Routine: Delete an XDL_VIEW view-object - xdl_delete_view_object
The routine xdl_delete_view_object (xdlf_delete_view_object) deletes a
view-object and any of its children.
-end*/
/*
************************************
**     xdl_delete_view_object     **
************************************

Purpose: Delete a view-object (and all its children). The tidy-up routines
         (if present) will be called for the view-object and children and
         their global data areas freed. The entries will be removed from
         the list of view objects. The requested view-object will be
         destroyed.
         

Author:  John W. Campbell, April 1990

*/
/*-Fortran:
         CALL XDLF_DELETE_VIEW_OBJECT (IVH, IERR)
-end*/
 
/*-Parameters:
IVH     (R)   View-object handle (see vh)
IERR    (W)   Return from xdl_delete_view_object routine
-end*/

/*-C:*/
int xdl_delete_view_object (vh)
/*end*/
 
/*-Parameters:*/
int vh;               /* The view object handle (R) */

/*end*/
/*-Doc:
Return:  0 = OK, 1 = requested view object not found
-end*/
{
   int iv;         /*Offset in XDL_view_objects array for view-object*/
   int i, j;       /*Loop variables */
   XDl_window *wp; /*Temporary pointer to XDl_window structure*/

   /* Check that view-object is in the list of view-objects */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_delete_view_object",vh); return 1;}

   /* Recursively delete all child view-objects */

   xdl_delete_children (vh);
   iv = xdl_getiv (vh);

   /* Destroy this view-object and hence all dependent view-objects */

   XDestroyWindow (XDL_display, XDL_view_objects[iv]->wid);

   /* Call tidy_up routine(s) if present and free global data areas*/

   if (XDL_view_objects[iv]->tidy_up!=NULL) 
       (*XDL_view_objects[iv]->tidy_up) (XDL_view_objects[iv]->gd);

   for (i=0;i<XDL_view_objects[iv]->num_win;++i)
   {
      wp = &(XDL_view_objects[iv]->win_list[i]);
      if (wp->libdata!=NULL)
      {
         if (wp->libdata->tidy_up!=NULL) 
            (*wp->libdata->tidy_up) (wp->libdata->gd);
         if (wp->libdata->gd!=NULL) free (wp->libdata->gd);
         free (wp->libdata);
      }
   }

   if (XDL_view_objects[iv]->gd!=0) free (XDL_view_objects[iv]->gd);

   /* Remove view-object from the list of view-objects */

   if (iv==XDL_num_view_objects-1)
   {
      if (XDL_view_objects[iv]->win_list!=NULL) 
	free (XDL_view_objects[iv]->win_list);
      if (XDL_view_objects[iv] == NULL)
      {
         fprintf(stderr, "XDL_view_objects[iv] == NULL\n");
         exit(1);
      }
      free (XDL_view_objects[iv]);
      XDL_view_objects[iv] = 0;
      XDL_num_view_objects--;
   }
   else
   {
      if (XDL_view_objects[iv]->win_list!=NULL) 
           free ((char *) XDL_view_objects[iv]->win_list);
      free ((char *) XDL_view_objects[iv]);
      for (i=iv+1,j=iv;i<XDL_num_view_objects;++i,++j)
      {
          XDL_view_objects[j] = XDL_view_objects[i];
      }
      XDL_view_objects[XDL_num_view_objects - 1] = 0;
      XDL_num_view_objects--;
   }

   /* Return */

   return 0;
}

/* Fortran binding: xdlf_delete_view_object */

void xdlf_delete_view_object (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_delete_view_object (*ivh);
   return;
}

/*-Routine: Pointers list index of a Fortran character string - xdlstr
The routine xdlstr is used to return an index to a list of character string
pointers held within XDL_VIEW and used where a Fortran callable routine 
passes character data to or from the routine. The actual paramter(s) expected 
by the C routine are machine dependent.  The Fortran call is of the form:

               XDLSTR (CHS)   where CHS is a Fortran character variable 
                              or string.

The function XDLSTR must be declared in the calling Fortran program as
an INTEGER FUNCTION.

-end*/
/*
*********************
**     xdlstr      **
*********************

Purpose: Return the index in XDL_pointers array of a Fortran character string
         being passed as a parameter from a Fortran program

Author:  John W. Campbell, New version March 1994

*/

#if   CHAR_STRING_TYPE == 1


int xdlstr (ch, len)
char *ch;    /* Address of the character string (R)*/
int len;     /* Length of the character string (R)*/

/*-C:*/
{
   XDL_pointers_index++;
   if (XDL_pointers_index>=MAXPOINTERS) XDL_pointers_index = 0;
   XDL_pointers[XDL_pointers_index] = ch;
   return XDL_pointers_index;
}

#endif

#if CHAR_STRING_TYPE == 2

typedef struct
{
   short int len;
   unsigned char type;
   unsigned char class;
   char * addr;
}
   Vax_ch_desc;

int xdlstr (ch_desc)
/*end*/
 
/*-Parameters:*/
Vax_ch_desc *ch_desc;     /* Pointer to character descriptor (R)*/

/*end*/
/*-Doc:
Return:  Index in the XDL_pointers list from 0 to MAXPOINTERS-1
-end*/
{
   XDL_pointers_index++;
   if (XDL_pointers_index>=MAXPOINTERS) XDL_pointers_index = 0;
   XDL_pointers[XDL_pointers_index] = ch_desc->addr;
   return XDL_pointers_index;
}

#endif

/*--------------------------*/
/*  Root x, y coordinates   */
/*--------------------------*/

/*-Routine: Get root coordinates - xdl_view_object_rootxy
The routine xdl_view_object_rootxy (xdlf_view_object_rootxy)is used to
convert x, y coordinates within a view object to xroot, yroot coordinates 
relative to the root window.
-end*/
/*
*************************************
**     xdl_view_object_rootxy      **
*************************************

Purpose: Convert x, y coordinates within a view object to xroot, yroot
         coordinates relative to the root window.

Author:  John W. Campbell, October 1990

*/
/*-Fortran:
         CALL XDLF_VIEW_OBJECT_ROOTXY (IVH, IX, IY, IXROOT, IYROOT, IERR)
-end*/
 
/*-Parameters:
IVH         (R)  View-object handle (see vh)
IX          (R)  X coordinate within the view-object (see x)
IY          (R)  Y coordinate within the view-object (see y)
IXROOT      (W)  Returns the root window X coordinate (see xroot)
IYROOT      (W)  Returns the root window Y coordinate (see yroot)
IERR        (W)  Return from the xdl_view_object_rootxy call
-end*/

/*-C:*/
int xdl_view_object_rootxy (vh, x, y, xroot, yroot)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int x;           /* x coordinate within view-object (R)*/
int y;           /* y coordinate within view-object (R)*/
int *xroot;      /* Returns x coordinate with respect to the root 
                    window (W)*/
int *yroot;      /* Returns y coordinate with respect to the root 
                    window (W)*/

/*end*/
/*-Doc:
Return:  =0 OK, =1 view-object not found
-end*/
{
   int iv;  /*Offset in XDL_view_objects array*/

   iv = xdl_getiv (vh);
   if (iv<0)
   {
      xdl_view_iv_errmsg ("xdl_view_object_rootxy", vh);
      return 1;
   }
   xdl_rootxy (XDL_view_objects[iv]->wid, x, y, xroot, yroot);

   return 0;
}

/* Fortran binding: xdlf_view_object_rootxy */

void xdlf_view_object_rootxy (ivh, ix, iy, ixroot, iyroot, ierr)

int *ivh, *ix, *iy, *ixroot, *iyroot, *ierr;
{
   *ierr = xdl_view_object_rootxy (*ivh, *ix, *iy, ixroot, iyroot);
   return;
}
/*----------------------------*/
/*  Raise/lower view-objects  */
/*----------------------------*/

/*-Routine: Raise a view-object - xdl_view_object_raise
The routine xdl_view_object_raise (xdlf_view_object_raise) is used to
raise a view-object to the top (i.e to the front) of the stacking order
amongst its siblings (view-objects with the same parent) on the screen.
-end*/
/*
*************************************
**     xdl_view_object_raise       **
*************************************

Purpose: Raise a view-object to the top (front) of the stacking order
         amongst its siblings on the screen

Author:  John W. Campbell, July 1991

*/
/*-Fortran:
         CALL XDLF_VIEW_OBJECT_RAISE (IVH, IERR)
-end*/
 
/*-Parameters:
IVH         (R)  View-object handle (see vh)
IERR        (W)  Return from the xdl_view_object_raise call
-end*/

/*-C:*/
int xdl_view_object_raise (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  =0 OK, =1 view-object not found
-end*/
{
   int iv;  /*Offset in XDL_view_objects array*/

   iv = xdl_getiv (vh);
   if (iv<0)
   {
      xdl_view_iv_errmsg ("xdl_view_object_raise", vh);
      return 1;
   }
   XRaiseWindow (XDL_display, XDL_view_objects[iv]->wid);
   return 0;
}

/* Fortran binding: xdlf_view_object_raise*/

void xdlf_view_object_raise (ivh, ierr)

int *ivh, *ierr;
{
   *ierr = xdl_view_object_raise (*ivh);
   return;
}

/*-Routine: Lower a view-object - xdl_view_object_lower
The routine xdl_view_object_lower (xdlf_view_object_lower) is used to
lower a view-object to the bottom (i.e to the back) of the stacking order
amongst its siblings (view-objects with the same parent) on the screen.
-end*/
/*
*************************************
**     xdl_view_object_lower       **
*************************************

Purpose: Lower a view-object to the bottom (back) of the stacking order
         amongst its siblings on the screen

Author:  John W. Campbell, July 1991

*/
/*-Fortran:
         CALL XDLF_VIEW_OBJECT_LOWER (IVH, IERR)
-end*/
 
/*-Parameters:
IVH         (R)  View-object handle (see vh)
IERR        (W)  Return from the xdl_view_object_lower call
-end*/

/*-C:*/
int xdl_view_object_lower (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  =0 OK, =1 view-object not found
-end*/
{
   int iv;  /*Offset in XDL_view_objects array*/

   iv = xdl_getiv (vh);
   if (iv<0)
   {
      xdl_view_iv_errmsg ("xdl_view_object_lower", vh);
      return 1;
   }
   XLowerWindow (XDL_display, XDL_view_objects[iv]->wid);
   return 0;
}

/* Fortran binding: xdlf_view_object_lower*/

void xdlf_view_object_lower (ivh, ierr)

int *ivh, *ierr;
{
   *ierr = xdl_view_object_lower (*ivh);
   return;
}
 
/*-Section: Special Purpose Application Callable Routines
These are some additional special purpose routines which may be used by the 
application programmer if needed. They include routines for handling 
non-blocking terminal input, handling events at timed interrupts,
synchronising X calls and introducing a program delay. 
-end*/

/*-Routine: XDL_VIEW open file for input - xdl_open_file
The routine xdl_file_open (xdlf_file_open) is used to open a file or FIFO as
a read only file prior to calling xdl_set_file_input (xdlf_set_file_input)
with a view to using xdl_file_events (xdlf_file_events). It is of
particular use in the fortran call as it enables the retur of a file
descriptor id when the file is opened using a 'C' call. The 'C'
routine is mearely an 'open' call with the appropriate flag.
-end*/
/*
**************************
**     xdl_open_file    **
***************************

Purpose: Open input file

Author:  John W. Campbell, February 1998

*/
/*-Fortran:
         CALL XDLF_OPEN_FILE (XDLSTR(FILNAM), LENF, IFID)
-end*/
/*-Parameters:
FILNAM   (R)  The file name string
              ** Pass address using the XDLSTR function **
LENF     (R)  Length of the file name string
IFID     (W)  The 'C' file id for the file. (Returns as -1 if failure
              on opening)
-end*/
/*-C:*/
void xdl_open_file (char * filnam, int lenf, int *fid)
/*end*/

/*-Parameters:
char * filnam;  The file name. (R)
int lenf;       The length of the file name; may be 0 if file name 
                string is null terminated. (R)                
int *fid;       Returns the file id of the opened file 
                (-1 if failure) (W) 
-end*/
{
   char tmpstr[251];
   if (lenf==0)
   {
      *fid = open (filnam, O_RDONLY);
      return;
   }
   else
   {
      tmpstr[0] = 0;
      strncat (tmpstr, filnam, lenf);
      *fid = open (tmpstr, O_RDONLY);
      return;
   }
}
/* Fortran binding: xdlf_open_file */

void xdlf_open_file (int * filnam, int * lenf, int * fid)
{
   xdl_open_file (XDL_pointers[*filnam], *lenf, fid);
   return;
}

/*-Routine: XDL_VIEW set file input - xdl_set_file_input
This routine xdl_set_file_input (xdlf_set_file_input) enables the reading 
of an updatable file or FIFO whilst carrying out event handling. The file 
must be opened with the given file id before calling this routine e.g
using xdl_open_file (xdlf_open_file). The file input 
option may be cleared by calling xdl_clear_file_input (xdlf_clear_file_input).
Only one file may be open at a time and a successive call to
xdl_set_file_input (xdlf_set_file_input) will clear any previous input file
even in the clear routine has not been explicitly called.
-end*/
/*
******************************
**     xdl_set_file_input   **
******************************

Purpose: Set up file input

Author:  John W. Campbell, February 1998

*/
/*-Fortran:
         CALL XDLF_SET_FILE_INPUT (IFID)
-end*/
 
/*-Parameters:
IFID     (R)  The 'C' file id for the file. (Must be retrieved from opening 
              a file using a 'C' based routine)
-end*/

/*-C:*/
void xdl_set_file_input (fid)
/*end*/

/*-Parameters:*/
int fid;      /* The file id for the already opened file (R)*/
/*end*/
{
    XINP_fid = fid; 
    XINP_str[0] = 0;
    XINP_numstr = 0;
}
/* Fortran binding: xdlf_set_file_input */

void xdlf_set_file_input (fid)
int *fid;
{
   xdl_set_file_input (*fid);
   return;
}

/*-Routine: XDL_VIEW clear file input - xdl_clear_file_input
This routine xdl_clear_file_input (xdlf_clear_file_input) clears
any current file input and should be called when such input has been 
completed.
-end*/
/*
********************************
**     xdl_clear_file_input   **
********************************

Purpose: Clear file input

Author:  John W. Campbell, February 1998

*/
/*-Fortran:
         CALL XDLF_CLEAR_FILE_INPUT (IDUM)
-end*/
 
/*-Parameters:
IDUM     (R)  Dummy parameter
-end*/

/*-C:*/
void xdl_clear_file_input ()
/*end*/

/*-Parameters:
none
-end*/
{
    XINP_fid = -1;
    XINP_str[0] = 0;
    XINP_numstr = 0;
}
/* Fortran binding: xdlf_clear_file_input */

void xdlf_clear_file_input (idum)
int *idum;
{
   xdl_clear_file_input ();
   return;
}

/*-Routine: XDL_VIEW event & file handling - xdl_file_events
This is equivalent to xdl_get_events (xdlf_get_events) except that it will 
also handle file input (if set by xdl_set_file input (xdlf_set_file_input)).
If a file record is read, then a view-handle of 0 is returned and the record
can be retrieved using the routine xdl_get_record (xdlf_get_record).
-end*/
/*
***************************
**     xdl_file_events   **
***************************

Purpose: Set up loop for event processing returning when data are sent
         back from the specified view-objects - version with additional
         handlinh\g of file input.

Author:  John W. Campbell, February 1998

*/
/*-Fortran:
         CALL XDLF_FILE_EVENTS (NUMIVH, IVHLIST, IVH)
-end*/
 
/*-Parameters:
NUMIVH   (R)  Number of view-objects to return data (see num-vh)
IVHLIST  (R)  Array of NUMIVH view object handles (see vh_list)
IVH      (W)  Returns the view-object handle of object returning
              the data (return from xdl_file_events) or -1 for
              input of a record from a file.
-end*/

/*-C:*/
int xdl_file_events (num_vh, vh_list)
/*end*/
 
/*-Parameters:*/
int num_vh;      /* Number of view objects from which data are to be
                    returned (may be 0 if no return is required)
                    if -num_vh is given then the routine be in a no wait
                    mode and will return when either input is received or
                    all current events have been flushed (R)*/
int vh_list[];   /* Array of the num_vh view-object handles (R)*/

/*end*/
/*-Doc:
Return:  The view-object handle of the view object returning the data.
         (May be 0 in no wait mode). Returns -1 if a file record read.
-end*/
{
   int i;                /* Temporary loop variable */
   int vh;               /* The returned view-object handle */
   int iv;               /* Offset into XDL_view_objects array for the given 
                            view-object handle */
   int num_win;          /* Number of windows */
   int no_wait;          /* Do not wait for view-object input */
   int on_off;           /* Input from view-object on/off flag */
   XEvent event;         /* X event */
   XEvent xevent;        /* X event */
   XAnyEvent xany;       /* XAnyEvent structure */
   Window wid;           /* Window id */
   XDl_window *win_list; /* Array of XDl_window structures */
   XDl_window *wp;       /* Pointer to and individual XDl_window structure */
   char * gd;            /* Pointer to global data area */
   void (*callback)();   /* Pointer to callback routine */

   /* Start event loop and wait for returned data */
   /* Call on-off routines as needed at start and end */

   XDL_return_data_vh = 0;
   no_wait = 0;
   xdl_timer_events(0);
   if (num_vh<0)
   {
      no_wait = 1;
      num_vh = - num_vh;
   }


   /* See if any data present if file handlin on */

   if (XINP_fid>=0)
   {
      i = xdl_read_next_rec();
      if (i) return -1;
   }
 
   if (XDL_num_view_objects>0)
   {
      for (iv=0;iv<XDL_num_view_objects;++iv)
      {
         on_off=0;
         if (num_vh>0)
         {
            for (i=0;i<num_vh;++i)
            {
               if (XDL_view_objects[iv]->vh==vh_list[i])
               {
                  on_off=1;
                  break;
               }
            }
         }
         if (XDL_view_objects[iv]->on_off_state!=on_off)
         {
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, on_off);
            }
            XDL_view_objects[iv]->on_off_state = on_off;
         }
      }
   }
   XFlush (XDL_display);

   next:
   if (no_wait&&XPending(XDL_display)==0)
   {
      XFlush (XDL_display);
      return 0;
   }
   if (XINP_fid>=0)
   {
      if (XPending(XDL_display)>0)
      {
         XNextEvent (XDL_display, &event);
      }
      else
      {
         i = xdl_read_next_rec();
         if (i) return -1;
         xdl_delay (100);
         goto next;
      }
   }
   else
   {
      XNextEvent (XDL_display, &event);
   }
   wid = event.xany.window;

  /*  printf ("Event %d, window %d\n", event.type, wid); */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      num_win = XDL_view_objects[iv]->num_win;
      if (num_win>0)
      {
         win_list = XDL_view_objects[iv]->win_list;
         for (i=0;i<num_win;++i)
         {
            wp = &(win_list[i]);
            if (wid==wp->wid)
            {
               gd = XDL_view_objects[iv]->gd;
               callback = NULL;
               if (wp->libdata!=NULL)
               {
                  if (wp->libdata->gd!=NULL) gd = wp->libdata->gd;
                  callback = wp->libdata->callback;
               }
               if (event.type==Expose||event.type==ConfigureNotify)
               {
                  if (wp->repaint!=NULL)
                  {
                     (*wp->repaint) (wid, &event, iv, gd, callback);
                  }
                  xdl_delay(100);
		  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         ConfigureNotify, &xevent));
                  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         Expose, &xevent));
                  goto winfound;
               }
               else
               {
                  if (wp->ehp!=NULL)
                  {
                     (*wp->ehp) (wid, &event, iv, gd, callback);
                  }
                  goto winfound;
               }
            }
         }
      }
   }
   goto next;
  
   winfound:
   if (XDL_return_data_vh==0) goto next;

   /* Look for file input */
   
   filinp:
   if (XDL_return_data_vh==-99)
   {
      XDL_return_data_vh = 0;
      return -1;
   }


   if (num_vh<=0) goto next;

   vh = XDL_return_data_vh;
   XDL_return_data_vh = 0;


   /* See if the returning view-object is in the list of view-objects from
      which events are required */

   for (i=0;i<num_vh;++i)
   {
      if (vh == vh_list[i])
      {
         for (i=0;i<num_vh;++i)
         {
            iv = xdl_getiv (vh_list[i]);
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, 0);
            }
            XDL_view_objects[iv]->on_off_state = 0;
         }
         XFlush (XDL_display);
         return vh;
      }
   }
   goto next;
}

/* Fortran binding: xdlf_file_events */

void xdlf_file_events (numivh, ivhlist, ivh)
int *numivh, *ivhlist, *ivh;
{
   *ivh = xdl_file_events (*numivh, ivhlist);
   return;
}
/* Service routines for file input handling */

int xdl_read_next_rec()
{
   char buf[3];

   while (read (XINP_fid,buf,1)>0)
   {
      if (buf[0]=='\n')
      {
         strcpy (XINP_laststr, XINP_str);
         XINP_str[0] = 0;
         XINP_numstr = 0;
         return 1;
      }
      else
      {
         if (XINP_numstr<MAXINPSTR)
         {
            XINP_str[XINP_numstr] = buf[0];
            XINP_str[XINP_numstr+1] = 0;
            XINP_numstr++;
         }  
      }
   }
   return 0;
}  
/*-Routine: Get record after xdl_file_events call - xdl_get_record
The routine xdl_get_record (xdlf_get_record) is called to return
the last record input from the file when the routine xdl_file_events 
(xdlf_file_events) returns a view-object handle of -1.
-end*/
/*
*****************************
**     xdl_get_record      **
*****************************

Purpose: Get last string input xdl_file_events

Author: John W. Campbell, February 1998

*/
/*-Fortran:
         CALL XDLF_GET_RECORD (XDLSTR(STRING), MAXLEN, IERR)
-end*/
 
/*-Parameters:
STRING    (W)      Returned record string (see string)
                   ** Pass address using the XDLSTR function **
MAXLEN    (R)      Maximum allowed length for the returned string
                   ( > 0 ) (cf max_len)
IERR      (W)      Return status from the xdl_get_record call
-end*/

/*-C:*/
int xdl_get_record (char * string, int max_len)
/*end*/
 
/*-Parameters:
char string[];   Returned string (W)
int max_len;     if >0 Maximum length of returned string allowed
                       (excluding terminating null - string must be at
                       least max_len + 1 characters in length)
                 if <0 abs(max_len) characters will be returned padded
                       with blanks if needed (for 'fortran' use) (R)

-end*/
/*-Doc:
Return: = 0 OK string present, =1 blank/null string
-end*/
{
   xdl_copy_chars (XINP_laststr,string,max_len);
   if (xdl_blank_string(XINP_laststr)) return 1;
   return 0;
}
/* Fortran binding: xdlf_get_record */

void xdlf_get_record (string, maxlen, ierr)
int *string, *maxlen, *ierr;
{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ierr = xdl_get_record (XDL_pointers[*string], max_len);
   return;
}

/*-Routine: Event handling with time out- xdl_get_events_timeout
An important part of XDL_VIEW is the X event handling mechanism. This is
normally done through a call from the application program to the
routine xdl_get_events (xdlf_get_events). When the routine is called, the 
user specifies a list of the view objects from which the program is ready 
to receive input ('active' view-objects). The routine services all the 
X events, passing them on to the approriate event handling routines until 
an event causes program input from one of the active view-objects; when 
this occurs, a return is made from the routine giving the view-object handle
of the routine from which the input originated. The xdl_get_events_timeout
(xdlf_get_events_timeout) routine has the additional function that a
return will be made after a requested interval if there has not been a
return occasioned by input to an active window. The time out interval may 
be taken from the invocation of the routine or from the last time an
event occured in any of a list of specified view-objects.
-end*/
/*
***********************************
**     xdl_get_events_timeout    **
***********************************

Purpose: Set up loop for event processing returning when data are sent
         back from the specified view-objects with timeout facility

Author:  John W. Campbell, September 1996

*/
/*-Fortran:
         CALL XDLF_GET_EVENTS_TIMEOUT (NUMIVH, IVHLIST, MSECS, 
        +                              NUMIVH2, IVHLIST2, IVH)
-end*/
 
/*-Parameters:
NUMIVH   (R)  Number of view-objects to return data (see num_vh)
IVHLIST  (R)  Array of NUMIVH view object handles (see vh_list)
MSECS    (R)  Return after this number of milliseconds if no
              input has been done (0 = no time out)
NUMIVH2  (R)  Number of view-objects in list 2 for re-setting
              the time interval (see IVHLIST2); may be 0.
IVHLIST2 (R)  Array of NUMIVH2 view object handles; the time out
              period will be re-started whenever an event takes place
              in any of these listed view-objects so that the
              time out return will only occur when no events have
              taken place in these objects for the time out period.
IVH      (W)  Returns the view-object handle of object returning
              the data (return from xdl_get_events)
-end*/

/*-C:*/
int xdl_get_events_timeout (num_vh, vh_list, msecs, num_vh2, vh_list2)
/*end*/
 
/*-Parameters:*/
int num_vh;      /* Number of view objects from which data are to be
                    returned (may be 0 if no return is required)
                    if -num_vh is given then the routine be in a no wait
                    mode and will return when either input is received or
                    all current events have been flushed (R)*/
int vh_list[];   /* Array of the num_vh view-object handles (R)*/
int msecs;       /* Return after this number of milliseconds if no
                    input has been done (0 = no time out) (R)*/
int num_vh2;     /* Number of view objects in list 2 (see vh_list2)
                    for re-setting the time interval; may be 0 (R)*/
int vh_list2[];  /* Array of NUMIVH2 view object handles; the time out
                    period will be re-started whenever an event takes 
                    place in any of these listed view-objects so that 
                    the time out return will only occur when no events 
                    have taken place in these objects for the time out 
                    period. (R)*/

/*end*/
/*-Doc:
Return:  The view-object handle of the view object returning the data.
         May be 0 in no wait mode or  -1 if the return is due to timing 
         out after the requested interval.
-end*/
{
   int i;                /* Temporary loop variable */
   int vh;               /* The returned view-object handle */
   int iv;               /* Offset into XDL_view_objects array for the given 
                            view-object handle */
   int num_win;          /* Number of windows */
   int no_wait;          /* Do not wait for view-object input */
   int timeout;          /* Timeout flag =1 yes, =0 no */
   int tflag;            /* Time out done flag (main time out)*/
   int tflag2;           /* Time out 2 done flag */
   int on_off;           /* Input from view-object on/off flag */
   unsigned long tim;    /* Time for timeout */
   XEvent event;         /* X event */
   XEvent xevent;        /* X event */
   XAnyEvent xany;       /* XAnyEvent structure */
   Window wid;           /* Window id */
   XDl_window *win_list; /* Array of XDl_window structures */
   XDl_window *wp;       /* Pointer to and individual XDl_window structure */
   char * gd;            /* Pointer to global data area */
   void (*callback)();   /* Pointer to callback routine */
   XtIntervalId tim_id;  /* Timer id */
   XtIntervalId tim_id2; /* Timer id 2 */

   /* Start event loop and wait for returned data */
   /* Call on-off routines as needed at start and end */

   XDL_return_data_vh = 0;
   no_wait = 0;
   timeout = 0;
   tflag = 0;
   tflag2 = 1;
   if (msecs>0)
   {
      timeout = 1;
      tim = msecs;
      tim_id = XtAppAddTimeOut (XDL_app, tim, 
                               (XtTimerCallbackProc) xdl_timdum, 
                               (XtPointer) &tflag);
   }
   xdl_timer_events(0);
   if (num_vh<0)
   {
      no_wait = 1;
      num_vh = - num_vh;
   }

   if (XDL_num_view_objects>0)
   {
      for (iv=0;iv<XDL_num_view_objects;++iv)
      {
         on_off=0;
         if (num_vh>0)
         {
            for (i=0;i<num_vh;++i)
            {
               if (XDL_view_objects[iv]->vh==vh_list[i])
               {
                  on_off=1;
                  break;
               }
            }
         }
         if (XDL_view_objects[iv]->on_off_state!=on_off)
         {
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, on_off);
            }
            XDL_view_objects[iv]->on_off_state = on_off;
         }
      }
   }
   XFlush (XDL_display);

   next:
   if (no_wait&&XtAppPending(XDL_app)==0)
   {
      XFlush (XDL_display);
      if (timeout&&tflag==0) XtRemoveTimeOut (tim_id);
      if (timeout&&tflag2==0) XtRemoveTimeOut (tim_id2);
      return 0;
   }

   if (timeout)
   {
      if (XPending(XDL_display)>0)
      {
         XNextEvent (XDL_display, &event);
      }
      else
      {
         tflag2 = 0;
         tim_id2 = XtAppAddTimeOut (XDL_app, (unsigned long) 50, 
                                   (XtTimerCallbackProc) xdl_timdum, 
                                   (XtPointer) &tflag2);
         XtAppProcessEvent (XDL_app, XtIMTimer);
         if (timeout&&tflag)
         { 
            if (tflag2==0) XtRemoveTimeOut (tim_id2);
            return -1;
	 }
         goto next;
      }
   }
   else
   {
      XNextEvent (XDL_display, &event);
   }
   wid = event.xany.window;

  /*  printf ("Event %d, window %d\n", event.type, wid); */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      num_win = XDL_view_objects[iv]->num_win;
      if (num_win>0)
      {
         win_list = XDL_view_objects[iv]->win_list;
         for (i=0;i<num_win;++i)
         {
            wp = &(win_list[i]);
            if (wid==wp->wid)
            {
               gd = XDL_view_objects[iv]->gd;
               callback = NULL;
               if (wp->libdata!=NULL)
               {
                  if (wp->libdata->gd!=NULL) gd = wp->libdata->gd;
                  callback = wp->libdata->callback;
               }
               if (event.type==Expose||event.type==ConfigureNotify)
               {
                  if (wp->repaint!=NULL)
                  {
                     (*wp->repaint) (wid, &event, iv, gd, callback);
                  }
                  xdl_delay(100);
		  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         ConfigureNotify, &xevent));
                  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         Expose, &xevent));
                  goto winfound;
               }
               else
               {
                  if (wp->ehp!=NULL)
                  {
                     (*wp->ehp) (wid, &event, iv, gd, callback);
                  }
                  goto winfound;
               }
            }
         }
      }
   }
   goto next;
  
   winfound:
   if (timeout&&num_vh2>0)
   { 
      vh = XDL_view_objects[iv]->vh;
      for (i=0;i<num_vh2;++i)
      {
         if (vh == vh_list2[i])
         {
            XtRemoveTimeOut (tim_id);            
            tim_id = XtAppAddTimeOut (XDL_app, tim, 
                               (XtTimerCallbackProc) xdl_timdum, 
                               (XtPointer) &tflag);
	 }
      }
   }
   if (XDL_return_data_vh==0) goto next;
   if (num_vh<=0) goto next;

   vh = XDL_return_data_vh;
   XDL_return_data_vh = 0;


   /* See if the returning view-object is in the list of view-objects from
      which events are required */

   for (i=0;i<num_vh;++i)
   {
      if (vh == vh_list[i])
      {
         for (i=0;i<num_vh;++i)
         {
            iv = xdl_getiv (vh_list[i]);
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, 0);
            }
            XDL_view_objects[iv]->on_off_state = 0;
         }
         XFlush (XDL_display);
         if (timeout&&tflag==0) XtRemoveTimeOut (tim_id);
         if (timeout&&tflag2==0) XtRemoveTimeOut (tim_id2);
         return vh;
      }
   }
   goto next;
}

/* Fortran binding: xdlf_get_events_timeout */

void xdlf_get_events_timeout (numivh, ivhlist, msecs, numivh2, ivhlist2, ivh)
int *numivh, *ivhlist, *msecs, *numivh2, *ivhlist2, *ivh;
{
   *ivh = xdl_get_events_timeout (*numivh, ivhlist, *msecs, 
                                  *numivh2, ivhlist2);
   return;
}

/*-Routine: XDL_VIEW event and standard input handling - xdl_getio_events
An important part of XDL_VIEW is the X event handling mechanism. This is
normally done through a call from the application program to the
routine xdl_get_events (xdlf_get_events). When the routine is called, the 
user specifies a list of the view objects from which the program is ready 
to receive input ('active' view-objects). The routine services all the 
X events, passing them on to the approriate event handling routines until 
an event causes program input from one of the active view-objects; when 
this occurs, a return is made from the routine giving the view-object handle
of the routine from which the input originated. If, in addition to handling
these window events, the user wishes to read data from the standard input
stream then this input must be done in a 'non-blocking' mode or the whole
process may hang. The routine xdl_getio_events (xdlf_getio_events) allows
such input in addition to any other event handling required.
-end*/
/*
*****************************
**     xdl_getio_events    **
*****************************

Purpose: Set up loop for event processing returning when data are sent
         back from the specified view-objects. In addition this routine
         reads input from the standard input stream in a non-blocking
         mode.

Author:  John W. Campbell, November 1991

*/
/*-Fortran:
         CALL XDLF_GETIO_EVENTS (NUMIVH, IVHLIST, IVH)
-end*/
 
/*-Parameters:
NUMIVH   (R)  Number of view-objects to return data (see num-vh)
              (may be 0)
IVHLIST  (R)  Array of NUMIVH view object handles (see vh_list)
IVH      (W)  Returns the view-object handle of object returning
              the data (return from xdl_getio_events) or 0 is 
              there was input from stdin.
-end*/

/*-C:*/
int xdl_getio_events (num_vh, vh_list)
/*end*/
 
/*-Parameters:*/
int num_vh;      /* Number of view objects from which data are to be
                    returned (may be 0) (R)*/
int vh_list[];   /* Array of the num_vh view-object handles (R)*/

/*end*/
/*-Doc:
Return:  The view-object handle of the view object returning the data.
         or 0 if input data was from stdin.
-end*/
{
   int i;                /* Temporary loop variable */
   int vh;               /* The returned view-object handle */
   int iv;               /* Offset into XDL_view_objects array for the given 
                            view-object handle */
   int on_off;           /* Input mode on/off flag */
   int num_win;          /* Number of windows */
   XEvent event;         /* X event */
   XEvent xevent;        /* X event */
   XAnyEvent xany;       /* XAnyEvent structure */
   Window wid;           /* Window id */
   XDl_window *win_list; /* Array of XDl_window structures */
   XDl_window *wp;       /* Pointer to and individual XDl_window structure */
   char * gd;            /* Pointer to global data area */
   void (*callback)();   /* Pointer to callback routine */

   /* Start event loop and wait for returned data */
   /* Call on-off routines as needed at start and end */

   XDL_return_data_vh = 0;
   xdl_timer_events(0);

   if (XIO_newstr==1)
   {
      XIO_numstr = 0;
      XIO_str[0] = 0;
   }
   XIO_newstr = 0;

   fflush(stdout);

   /*If no view objects present the use a 'normal' read*/

   if (XDL_num_view_objects==0)
   {
      gets(XIO_str);
      XIO_numstr = strlen(XIO_str);
      XIO_newstr = 1;
      return 0;
   }

   /* Set terminal read to non-blocking mode */

   xdlsys_nonblock(1);

   if (XDL_num_view_objects>0)
   {
      for (iv=0;iv<XDL_num_view_objects;++iv)
      {
         on_off=0;
         if (num_vh>0)
         {
            for (i=0;i<num_vh;++i)
            {
               if (XDL_view_objects[iv]->vh==vh_list[i])
               {
                  on_off=1;
                  break;
               }
            }
         }
         if (XDL_view_objects[iv]->on_off_state!=on_off)
         {
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, on_off);
            }
            XDL_view_objects[iv]->on_off_state = on_off;
         }
      }
   }
   XFlush (XDL_display);

   /* See if any input on stdin and set timer if needed */

   xdlsys_read(0);

   if (XDL_return_data_vh==-99) goto infound;

   /* Process events */

   next:
   XNextEvent (XDL_display, &event);
   if (XDL_return_data_vh==-99) goto infound;

   wid = event.xany.window;

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      num_win = XDL_view_objects[iv]->num_win;
      if (num_win>0)
      {
         win_list = XDL_view_objects[iv]->win_list;
         for (i=0;i<num_win;++i)
         {
            wp = &(win_list[i]);
            if (wid==wp->wid)
            {
               gd = XDL_view_objects[iv]->gd;
               callback = NULL;
               if (wp->libdata!=NULL)
               {
                  if (wp->libdata->gd!=NULL) gd = wp->libdata->gd;
                  callback = wp->libdata->callback;
               }
               if (event.type==Expose||event.type==ConfigureNotify)
               {
                  if (wp->repaint!=NULL)
                  {
                     (*wp->repaint) (wid, &event, iv, gd, callback);
                  }
                  xdl_delay(100);
		  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         ConfigureNotify, &xevent));
                  while (XCheckTypedWindowEvent (XDL_display, wp->wid,
                         Expose, &xevent));
                  goto winfound;
               }
               else
               {
                  if (wp->ehp!=NULL)
                  {
                     (*wp->ehp) (wid, &event, iv, gd, callback);
                  }
                  goto winfound;
               }
            }
         }
      }
   }
   goto next;
  
   winfound:
   if (XDL_return_data_vh==0) goto next;
   if (num_vh<=0) goto next;
   vh = XDL_return_data_vh;
   XDL_return_data_vh = 0;


   /* See if the returning view-object is in the list of view-objects from
      which events are required */

   for (i=0;i<num_vh;++i)
   {
      if (vh == vh_list[i])
      {
         for (i=0;i<num_vh;++i)
         {
            iv = xdl_getiv (vh_list[i]);
            if (XDL_view_objects[iv]->on_off!=NULL)
            {
               (*XDL_view_objects[iv]->on_off)(iv, 0);
            }
            XDL_view_objects[iv]->on_off_state = 0;
         }
         XFlush (XDL_display);
         xdlsys_nonblock(0);
         return vh;
      }
   }
   goto next;

   /* Input completed from standard input stream */

   infound:
   XDL_return_data_vh = 0;
   if (num_vh>0)
   {
      for (i=0;i<num_vh;++i)
      {
         iv = xdl_getiv (vh_list[i]);
         if (XDL_view_objects[iv]->on_off!=NULL)
         {
            (*XDL_view_objects[iv]->on_off)(iv, 0);
         }
         XDL_view_objects[iv]->on_off_state = 0;
      }
      XFlush (XDL_display);
   }
   xdlsys_nonblock(0);
   XIO_newstr = 1;
   return 0;
}

/* Service routines for xdl_getio_events */

/* Set/unset non-blocking read and alarm/ctr_C handlers */

void xdlsys_nonblock(i)

int i;  /*Flag =1 set non-block; =0 reset to previous mode*/
{
   int result;
   int flags;

   if (i==1)
   {

#ifdef HAVE_SIGACTION
      sigemptyset (&(XIO_vec_alarm.sa_mask));
      XIO_vec_alarm.sa_flags = 0;
      XIO_vec_alarm.sa_handler = xdlsys_alarm;
      sigaction(SIGALRM,&XIO_vec_alarm,&XIO_ovec_alarm);
      sigemptyset (&(XIO_vec_ctrc.sa_mask));
      XIO_vec_ctrc.sa_flags = 0;
      XIO_vec_ctrc.sa_handler = xdlsys_ctr_c;
      sigaction(SIGINT,&XIO_vec_ctrc,&XIO_ovec_ctrc);
#else
      signal(SIGINT,xdlsys_ctr_c);
      signal(SIGALRM,xdlsys_alarm);
#endif

#ifdef HAVE_O_NONBLOCK
      XIO_flags=fcntl(0,F_GETFL,0);
      flags = XIO_flags;
      fcntl(0,F_SETFL,flags|O_NONBLOCK);
#else
      XIO_flags=fcntl(0,F_GETFL,0);
      flags = XIO_flags;
      fcntl(0,F_SETFL,flags|O_NDELAY);
#endif

#ifdef VMS
      result = SYS$ASSIGN(&XIO_terminal,&XIO_channel,0,0);
      XIO_func = IO$_READVBLK|IO$M_TIMED|IO$M_NOFILTR;
#endif

   }
   else
   {
      alarm (0);

#ifndef VMS
      fcntl(0,F_SETFL,XIO_flags);
#endif

#ifdef HAVE_SIGACTION
      sigaction(SIGALRM,&XIO_ovec_alarm,0);
      sigaction(SIGINT,&XIO_ovec_ctrc,0);
#else
      signal(SIGALRM,SIG_DFL);
      signal(SIGINT,SIG_DFL);
#endif

   }
   return;
}

/* Control C handling routine */

void
xdlsys_ctr_c(sig,code,scp)

int sig, code;
struct sigcontext *scp;
{
    xdlsys_nonblock(0);
    exit(1);
}

/* Signal handler routine for alarm */

void
xdlsys_alarm(sig,code,scp)

int sig, code;
struct sigcontext *scp;
{
   xdlsys_read(1);
   return;
}

/* Non-blocking read routine*/

void xdlsys_read(send_msg)

int send_msg; /*If True send client message event before returning with
                completed input; if False do not */
{
   int n;
   char ch;

#ifdef VMS
   struct
   {
      short cond_value;
      short count;
      int info;
   } status_block;
#endif

#if NONBLK_TYPE==1||NONBLK_TYPE==2
   lab:
   n=read(0,&ch,1);
   if (n==1)
   {
      if (ch!=10)
      {
         if (XIO_numstr<MAXINPSTR)
         {
            XIO_str[XIO_numstr] = ch;
            XIO_numstr++;
         }
         goto lab;
      }
   }
   else
   {
      alarm(1);
      return;
   }
#endif

#ifdef VMS
   lab:
   ch = 0;
   n=SYS$QIOW(0,XIO_channel,XIO_func,&status_block,0,0,&ch,1,0,0,0,0);
   if (ch!=0)
   {
      if (ch!=13)
      {
         if (ch==127)
         {
            if (XIO_numstr==0) goto lab;
            printf("\b \b");
            XIO_numstr -= 1;
            goto lab;
         }
         else if (ch=='\b')
         {
            if (XIO_numstr==0) goto lab;
            printf(" \b");
            XIO_numstr -= 1;
            goto lab;
         }
         else
         {
            XIO_str[XIO_numstr] = ch;
            XIO_numstr++;
         }
         goto lab;
      }
   }
   else
   {
      signal(SIGALRM,xdlsys_alarm);
      alarm(1);
      return;
   }
#endif

   /* Input line completed */

   XDL_return_data_vh = -99;
   XIO_str[XIO_numstr] = 0;
   if (send_msg)
   {
      if (XDL_num_view_objects>0)
      {
         XIO_mesg.type = ClientMessage;
         XIO_mesg.message_type = XA_STRING;
         XIO_mesg.format = 8;
         XSendEvent (XDL_display,XDL_view_objects[0]->wid,
                     True,0L,(XEvent*) &XIO_mesg);
         XFlush(XDL_display);
      }
      return;
   }
   else
   {
      return;
   }
}
        
/* Fortran binding: xdlf_getio_events */

void xdlf_getio_events (numivh, ivhlist, ivh)
int *numivh, *ivhlist, *ivh;
{
   *ivh = xdl_getio_events (*numivh, ivhlist);
   return;
}
/*-Routine: Get string after xdl_getio_events call - xdl_getio_string
The routine xdl_getio_string (xdlf_getio_string) is called to return
the last string input from the terminal (in a non-blocking read mode)
when the routine xdl_getio_events (xdlf_getio_events) returns a
view-object handle of 0.
-end*/
/*
*****************************
**     xdl_getio_string    **
*****************************

Purpose: Get last string input from non-blocking read via xdl_getio_events

Author: John W. Campbell, November 1991

*/
/*-Fortran:
         CALL XDLF_GETIO_STRING (XDLSTR(STRING), MAXLEN, IERR)
-end*/
 
/*-Parameters:
STRING    (W)      Returned string (see string)
                   ** Pass address using the XDLSTR function **
MAXLEN    (R)      Maximum allowed length for the returned string
                   ( > 0 ) (cf max_len)
IERR      (W)      Return status from the xdl_getio_string call
-end*/

/*-C:*/
int xdl_getio_string (string, max_len)
/*end*/
 
/*-Parameters:*/
char string[];   /* Returned string (W)*/
int max_len;     /* if >0 Maximum length of returned string allowed
                          (excluding terminating null - string must be at
                          least max_len + 1 characters in length)
                    if <0 abs(max_len) characters will be returned padded
                          with blanks if needed (for 'fortran' use) (R)*/

/*end*/
/*-Doc:
Return: = 0 OK string present, =1 blank/null string
-end*/
{
   xdl_copy_chars (XIO_str,string,max_len);
   if (XIO_numstr==0) return 1;
   return 0;
}
/* Fortran binding: xdlf_getio_string */

void xdlf_getio_string (string, maxlen, ierr)
int *string, *maxlen, *ierr;
{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ierr = xdl_getio_string (XDL_pointers[*string], max_len);
   return;
}

/*-Routine: Read a string from the terminal - xdl_tread
The routine xdl_tread (xdlf_tread) is used to read a string from
the terminal (standard input). The routine performs reads in a
non-blocking mode and the background handling of X events for
the view-objects will continue. Normal reads should not be used
as this will block event handling and can cause a hang-up.
-end*/
/*
*****************************
**     xdl_tread           **
*****************************

Purpose: Non-blocking terminal read while processing background events

Author: John W. Campbell, November 1991

*/
/*-Fortran:
         CALL XDLF_TREAD (XDLSTR(STRING), MAXLEN, IERR)
-end*/
 
/*-Parameters:
STRING    (W)      Returned string (see string)
                   ** Pass address using the XDLSTR function **
MAXLEN    (R)      Maximum allowed length for the returned string
                   ( > 0 ) (cf max_len)
IERR      (W)      Return status from the xdl_tread call
-end*/

/*-C:*/
int xdl_tread (string, max_len)
/*end*/
 
/*-Parameters:*/
char string[];   /* Returned string (W)*/
int max_len;     /* if >0 Maximum length of returned string allowed
                          (excluding terminating null - string must be at
                          least max_len + 1 characters in length)
                    if <0 abs(max_len) characters will be returned padded
                          with blanks if needed (for 'fortran' use) (R)*/

/*end*/
/*-Doc:
Return: = 0 OK string returned, =1 blank/null string returned
-end*/
{
   int vh_list[2];
   int i;

   i = xdl_getio_events (0,vh_list);
   i = xdl_getio_string (string, max_len);
   return i;
}
/* Fortran binding: xdlf_tread */

void xdlf_tread (string, maxlen, ierr)
int *string, *maxlen, *ierr;
{
   int max_len;

   max_len = -(*maxlen);
   if (max_len>0) max_len = -max_len;
   *ierr = xdl_tread (XDL_pointers[*string], max_len);
   return;
}
/*-Routine: XDL_VIEW timer driven event handling - xdl_timer_events
An important part of XDL_VIEW is the X event handling mechanism. This is
normally done through a call from the application program to the
routine xdl_get_events (xdlf_get_events). In some applications, it
may not be convenient to handle events in this way and this routine
provides a mechanism to keep the event queue serviced at regular
intervals using a system timer. It may also be used during lengthy
calculations to keep the event queue serviced. The routine is
only used when the program is not waiting for input from one or
more view-objects. It should be noted that a call to xdl_get_events
(xdlf_get_events) or xdl_getio_events (xdlf_getio_events) switches
off the timer driven event handling so that it will need to be
switched on again after such a call if required. Where possible,
the use of this routine should be avoided.
-end*/
/*
*****************************
**     xdl_timer_events    **
*****************************

Purpose: Switch on/off handling of events at regular intervals based
         using the alarm signal handling.

Author:  John W. Campbell, December 1993

*/
/*-Fortran:
         CALL XDLF_TIMER_EVENTS (IOPT)
-end*/
 
/*-Parameters:
IOPT     (R)  = 0 turn off timer driven event handling, 
              = 1 turn on timer driven event handling
-end*/

/*-C:*/
int xdl_timer_events (on_off)
/*end*/
 
/*-Parameters:*/
int on_off;      /* = 1 turn timer driven event handling on,
                    = 0 turn timer driven event handling off (R)*/
/*end*/
/*-Doc:
Return:  
-end*/
{
   if (on_off)
   {

#ifdef HAVE_SIGACTION
      sigemptyset (&(XIO_vec_alarm.sa_mask));
      XIO_vec_alarm.sa_flags = 0;
      XIO_vec_alarm.sa_handler = xdlevents_alarm;
      sigaction(SIGALRM,&XIO_vec_alarm,&XIO_ovec_alarm);
#else
      signal(SIGALRM,xdlevents_alarm);
#endif
      XDL_tmr_events = 1;
      alarm(1);
   }
   else
   {
      if (XDL_tmr_events)
      {
         alarm(0);
#ifdef HAVE_SIGACTION
      sigaction(SIGALRM,&XIO_ovec_alarm,0);
#else
      signal(SIGALRM,SIG_DFL);
#endif
         XDL_tmr_events = 0;
      }
   }
   return 0;
}
/* Signal handler routine for alarm */

void
xdlevents_alarm(sig,code,scp)

int sig, code;
struct sigcontext *scp;
{
   alarm(0);
   xdl_flush_events(0);
   alarm(1);
   return;
}

/* Fortran binding: xdlf_timer_events */

void xdlf_timer_events (on_off)
int *on_off;
{
   xdl_timer_events (*on_off);
   return;
}
/*-Routine: XDL_VIEW XSync/XSyschronise calls - xdl_sync
The routine xdl_sync (xdlf_sync) routine enables access to the X-windows
XSync and XSynchronize functions in case these are needed.
-end*/
/*
*****************************
**     xdl_sync            **
*****************************

Purpose: Make calls to XSync or XSynchronize routines.

Author:  John W. Campbell, October 1994

*/
/*-Fortran:
         CALL XDLF_SYNC (IOPT)
-end*/
 
/*-Parameters:
IOPT     (R)  Flag =1 XSync, no discard
                   =2 XSync, discard
                   =3 XSynchronize on
                   =4 XSynchronize off
-end*/

/*-C:*/
void xdl_sync (opt)
/*end*/
 
/*-Parameters:*/
int opt;       /* Flage =1 XSync, no discard
                        =2 XSync, discard
                        =3 XSynchronize on
                        =4 XSynchronize off (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   switch (opt)
   {
      case 1:
         XSync (XDL_display, False);
      break;

      case 2:
         XSync (XDL_display, True);
      break;

      case 3:
         XSynchronize (XDL_display, 1);
      break;

      case 4:
         XSynchronize (XDL_display, 0);
      break;
   }
   return;
}

/* Fortran binding: xdlf_sync */

void xdlf_sync (iopt)
int *iopt;
{
   xdl_sync (*iopt);
   return;
}

/*----------------------------*/
/*  Delay for requested time  */
/*----------------------------*/

/*-Routine: Delay - xdl_delay
The routine xdl_delay (xdlf_delay) is used to introduce a program delay
for a requested no. of milliseconds e.g. to wait for new windows to 
be exposed.
-end*/
/*
************************
**     xdl_delay      **
************************

Purpose: Delay for a requested time in milliseconds

Author:  John W. Campbell, December 1995

*/
/*-Fortran:
         CALL XDLF_DELAY (MSEC)
-end*/
 
/*-Parameters:
MSEC       (R)  Time of delay in milliseconds
-end*/

/*-C:*/
void xdl_delay (msec)
/*end*/
 
/*-Parameters:*/
int msec;          /* Delay in milliseconds (R)*/

/*end*/
/*-Doc:
Return:  none
-end*/
{
   int done;
   unsigned long tim; 

   tim = msec;
   
   done = 0;
   XtAppAddTimeOut (XDL_app, tim, (XtTimerCallbackProc) xdl_timdum, 
                   (XtPointer) &done);
   while (done==0) XtAppProcessEvent (XDL_app,XtIMTimer);
   return;
}

/* Fortran binding: xdlf_delay*/

void xdlf_delay (msec)

int *msec;
{
   xdl_delay (*msec);
   return;
}
/* Dummy routine for timer callback (for xdl_delay & xdl_get_events_timeout */
void xdl_timdum (client_data, tim_id)

XtPointer client_data;
XtIntervalId * tim_id;
{
   int *i;
   i = (int *) client_data;
   *i = 1;
   return;
}

/*===========================================================*/
/*         ROUTINES FOR USE IN CODING VIEW-OBJECTS           */
/*===========================================================*/

/*--------------------------*/
/*     Creating Windows     */
/*--------------------------*/

 
/*-Section: Creating Windows when Coding View-Objects
These are routines used to create windows when coding a new view-object.
-end*/

/*-Routine: Create a base frame window - xdl_create_base_frame_window
The routine xdl_create_base_frame_window is used to create (and map) a base 
frame window. This window is a child of the Rootwindow. The routine sends a 
title, icon label, hints etc. to the window manager.
-end*/
/*
*******************************************
**     xdl_create_base_frame_window      **
*******************************************

Purpose: Create (and map) a base frame window (child of Rootwindow - sends
         title, icon label, hints etc. to window manager)

Author:  John W. Campbell, September 1990
*/

/*-C:*/
Window xdl_create_base_frame_window (title, icon_label, x, y, 
                                     width, height)
/*end*/
 
/*-Parameters:*/
char * title;       /*Title for passing to window manager (null term) (R)*/
char * icon_label;  /*Icon label for passing to window manager 
                      (null term) (R)*/
int x;              /*Root x coordinate for window (-1 to leave selection
                      to the window manager) (R)*/
int y;              /*Root y coordinate for window (-1 to leave selection
                      to the window manager) (R)*/
int width;          /*Window width (including border of 1 at each side) 
                      (R)*/
int height;         /*Window height (including border of 1 at top and 
                      bottom (R)*/

/*end*/
/*-Doc:
Return:  The window id
-end*/
{
   XSizeHints size_hints;        /*Size hints structure*/
   XSetWindowAttributes attrib;  /*Window attributes structure*/
   Window wid;                   /*Window id of created window*/
   int xx,yy;

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = XDL_colormap;
   attrib.cursor = XDL_default_cursor;
   xx = 100;
   yy = 100;
   if (x>=0&&y>=0)
   {
      xx = x;
      yy = y;
   }
  
   wid = XCreateWindow (XDL_display, RootWindow (XDL_display, XDL_screen),
                        xx, yy, width-2, height-2, 1, 
                        XDL_pw_depth, InputOutput,
/*                        CopyFromParent, InputOutput, */
                        XDL_visual, 
                        CWBackPixel|CWBorderPixel|CWColormap|CWCursor,
                        &attrib);

   size_hints.flags = PSize|PMinSize;
   size_hints.width = width;
   size_hints.height = height;
   if (width>100)
   {
      size_hints.min_width = 100;
   }
   else
   {
      size_hints.min_width = width;
   }
   if (height>100)
   {
      size_hints.min_height = 100;
   }
   else
   {
      size_hints.min_height = height;
   }
   if (x>=0&&y>=0)
   {
       size_hints.x = x;
       size_hints.y = y;
       size_hints.flags |= USPosition;
   }

   XSetStandardProperties (XDL_display, wid, title, icon_label, None,
                          0, 0, &size_hints); 

   XMapWindow (XDL_display, wid);

   return wid;
}

/*-Routine: Create a frame window - xdl_create_frame_window
The routine xdl_create_frame_window is used to create (and map) a frame 
window. Such a window is normally used as the top level window of a 
view-object when it has a parent view-object.
-end*/
/*
**************************************
**     xdl_create_frame_window      **
**************************************

Purpose: Create (and map) a frame window

Author:  John W. Campbell, September 1990
*/
/*-C:*/
Window xdl_create_frame_window (wparent, x, y, width, height, cset)
/*end*/
 
/*-Parameters:*/
Window wparent;     /*Window id of the parent window (R)*/
int x;              /*x position of top left pixel (of border) (R)*/
int y;              /*y position of top left pixel (of border) (R)*/
int width;          /*Window width (including border of 1 at each side) 
                      (R)*/
int height;         /*Window height (including border of 1 at top and 
                      bottom (R)*/
int cset;           /*XDL_VIEW Colorset number (R)*/

         /* Note: Origin pixel at top left within border */
/*end*/
/*-Doc:
Return:  The window id
-end*/
{
   XSetWindowAttributes attrib;  /*Window attributes structure*/
   Window wid;                   /*Window id of created window*/
   Colormap cmap;                /*Colormap*/

   cmap = XDL_colorsets[0].colormap;
   if ((cset>0) && (cset<=XDL_num_colorsets))
   {
       cmap = XDL_colorsets[cset].colormap;
   }

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = cmap;
   attrib.cursor = XDL_default_cursor;
  
   wid = XCreateWindow (XDL_display, wparent,
                        x, y, width-2, height-2, 1, XDL_pw_depth, InputOutput,
/*                        x, y, width-2, height-2, 1, 0, InputOutput, */
                        XDL_visual, 
                        CWBackPixel|CWBorderPixel|CWColormap|CWCursor,
                        &attrib);

   XMapWindow (XDL_display, wid);

   return wid;
}

/*-Routine: Create a paint window - xdl_create_paint_window
The routine xdl_create_paint_window is used to create (and map) an 
input/output paint window.
-end*/
/*
**************************************
**     xdl_create_paint_window      **
**************************************

Purpose: Create (and map) an input/output paint window

Author:  John W. Campbell, September 1990
*/

/*-C:*/
Window xdl_create_paint_window (wparent, x, y, brdw, width, height, 
                                cset)
/*end*/
 
/*-Parameters:*/
Window wparent;     /*Window id of the parent window (R)*/
int x;              /*x position of top left pixel (of border) (R)*/
int y;              /*y position of top left pixel (of border) (R)*/
int brdw;           /*Border width in pixels (normally 1)*/
int width;          /*Window width (including border at each side) (R)*/
int height;         /*Window height (including border at top and 
                      bottom (R)*/
int cset;           /*XDL_VIEW Colorset number (R)*/

         /* Note: Origin pixel at top left within border */
         /*       Paintable width  = width - 2*brdw      */
         /*       Paintable height = height - 2*brdw     */
/*end*/
/*-Doc:
Return:  The window id
-end*/
{
   XSetWindowAttributes attrib;  /*Window attributes structure*/
   Window wid;                   /*Window id of created window*/
   Colormap cmap;                /*Colormap*/

   cmap = XDL_colorsets[0].colormap;
   if ((cset>0) && (cset<=XDL_num_colorsets))
   {
       cmap = XDL_colorsets[cset].colormap;
   }

   attrib.background_pixel = XDL_whitepixel;
   attrib.border_pixel = XDL_blackpixel;
   attrib.colormap = cmap;
   attrib.cursor = XDL_default_cursor;
  
   wid = XCreateWindow (XDL_display, wparent,
                        x, y, width-2*brdw, height-2*brdw, brdw, XDL_pw_depth,
                        InputOutput, XDL_visual, 
                        CWBackPixel|CWBorderPixel|CWColormap|CWCursor,
                        &attrib);

   XSelectInput (XDL_display, wid, ExposureMask|StructureNotifyMask);

   XMapWindow (XDL_display, wid);

   return wid;
}

/*----------------------------------*/
/*   Active Strip Window Routines   */
/*----------------------------------*/

 
/*-Section: Creating Active Strips when Coding View-Objects
These are routines used to create active strips when coding a new view-object.
-end*/

/*-Routine: Create an active strip - xdl_create_active_strip
The routine xdl_create_active_strip is used to create an active strip 
window and add it to the view-object's window list. This strip indicates 
whether or not the application is waiting for input from a view object. 
The on/off states are set by the routines xdl_active_strip_on and 
xdl_active_strip_off. A filled square in green (or black if b/w display) 
indicates that input is awaited and a text string indicates the nature of 
the required input. An open square indicates that input is not awaited. 
The strip will be placed with its origin at 4, 4 of the parent window with 
a depth of 20. The required width (normally the parent window width - 8) 
is passed to the routine. Bold text should be used when the window has 
the keyboard focus and normal text be used otherwise.
-end*/
/*
************************************
**     xdl_create_active_strip    **
************************************

Purpose: Create an active strip window and add it to the view-object's
         window list.

Author:  John W. Campbell, July 1990
*/

/*-C:*/
Window xdl_create_active_strip (wparent, vh, width, repaint)
/*end*/
 
/*-Parameters:*/
Window wparent;        /* The parent window for which the strip is to be 
                          created (R) */
int vh;                /* View-object handle (The view object must have
                          been added to the list of view-objects using 
                          the routine xdl_add_new_view_object before 
                          xdl_create_active_strip is called (R) */
int width;             /* The width for the strip (R) */
void (*repaint)();     /* Active strip repaint function (R)*/

/*end*/
/*-Doc:
Return:  The created window
-end*/
{
   Window wid;         /* Window identifier */

   wid = xdl_create_paint_window (wparent, BORDER_WIDTH, BORDER_HEIGHT, 1,
                                  width, ACT_STRIP_H, 0);

   XSelectInput (XDL_display, wid, ExposureMask|StructureNotifyMask);

   xdl_add_window (vh, wid, repaint, NULL);

   return wid;
}

/*-Routine: Set active strip to 'on' - xdl_active_strip_on
The routine xdl_active_strip_on is used to set an active strip to its 'on' 
state.
-end*/
/*
********************************
**     xdl_active_strip_on    **
********************************

Purpose: Set active strip window to its 'on' state.

Author:  John W. Campbell, July 1990
*/

/*-C:*/
void xdl_active_strip_on (wid, str, bold)
/*end*/
 
/*-Parameters:*/
Window wid;          /* The active strip window id (R) */
char * str;          /* The string to be displayed (R) */
int bold;            /* =1 Use bold text (to indicate keyboard focus) or
                        =0 Use normal text (R) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int len;                 /* String length */

   len = strlen (str);

   XSetForeground (XDL_display, XDL_active_strip_gc, XDL_blackpixel);

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);

   XSetLineAttributes (XDL_display, XDL_active_strip_gc, 1, LineSolid,
                       CapButt, JoinMiter);
   XDrawLine (XDL_display, wid, XDL_active_strip_gc, ACT_STRIP_H-2, 0, 
              ACT_STRIP_H-2, ACT_STRIP_H-3);

   XDrawRectangle (XDL_display, wid, XDL_active_strip_gc, 2, 2, 
                  ACT_STRIP_H-7, ACT_STRIP_H-7);

   if (XDL_color>1)
   {
      XSetForeground (XDL_display, XDL_active_strip_gc, XDL_greenpixel);
   }
   XFillRectangle (XDL_display, wid, XDL_active_strip_gc, 3, 3,
                   ACT_STRIP_H-8, ACT_STRIP_H-8);

   XSetForeground (XDL_display, XDL_active_strip_gc, XDL_blackpixel);

   if (bold)
   {
      XSetFont (XDL_display, XDL_active_strip_gc, XDL_fontinfo_bold->fid);
   }
   else
   {
      XSetFont (XDL_display, XDL_active_strip_gc, XDL_fontinfo->fid);
   }
   XDrawString (XDL_display, wid, XDL_active_strip_gc, ACT_STRIP_H+1, 
                13, str, len);

   return;
}

/*-Routine: Set active strip to 'off' - xdl_active_strip_off
The routine xdl_active_strip_off is used to set an active strip to its 'off' 
state.
-end*/
/*
*********************************
**     xdl_active_strip_off    **
*********************************

Purpose: Set active strip window to its 'off' state.

Author:  John W. Campbell, July 1990
*/

/*-C:*/
void xdl_active_strip_off (wid)
/*end*/
 
/*-Parameters:*/
Window wid;      /* The active strip window id (R) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XSetForeground (XDL_display, XDL_active_strip_gc, XDL_blackpixel);

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);

   XSetLineAttributes (XDL_display, XDL_active_strip_gc, 1, LineSolid,
                       CapButt, JoinMiter);

   XDrawLine (XDL_display, wid, XDL_active_strip_gc, ACT_STRIP_H-2, 0, 
              ACT_STRIP_H-2, ACT_STRIP_H-3);

   XDrawRectangle (XDL_display, wid, XDL_active_strip_gc, 2, 2, 
                  ACT_STRIP_H-7, ACT_STRIP_H-7);

   return;
}

/*-----------------------------------------------------*/
/* Adding new objects and windows to view-objects list */
/*-----------------------------------------------------*/
 
/*-Section: Registering View-Objects and Windows when Coding View-Objects
These are routines used to register new view-objects by adding them to an 
internal list and to add window information to that list when coding a new 
view-object.
-end*/

/*-Routine: Add a new view-object - xdl_new_view_object
The routine xdl_new_view_object is used to add a new view-object to the
list of view-objects and to initialiase its associated data.
-end*/
/*
*******************************
**     xdl_new_view_object   **
*******************************

Purpose: Add a new view-object to the list of view-objects and initialise
         their associated data.

Author: John W. Campbell, March 1990

*/

/*-C:*/
int xdl_new_view_object (vh, wid, vh_parent, gd, type, on_off, tidy_up)
/*end*/
 
/*-Parameters:*/
int vh;                 /*The view_object handle for the new view object
                          (vh) (R)*/
Window wid;             /*The parent window of the new view object (R)*/
int vh_parent;          /*The view-object handle for the parent 
                          (0 if none) (R)*/
char * gd;              /*Pointer to the global data area used by the 
                          view-object's routines (R)*/
int type;               /*A unique integer for the view-object type (R)*/
void (*on_off)();       /*Pointer to the input on/off function for the
                          view object. May be NULL if not required or
                          implemented. (Used by xdl_get_events) (R)*/
void (*tidy_up)();      /*Pointer to the tidy up routine for the 
                          view-object. This routine will be called by 
                          xdl_delete_view_object when the view object
                          is deleted. It should be used for example
                          to free any memory areas allocated by the 
                          view-object (except for the global data area 
                          which will be automatically freed by 
                          xdl_delete_view_object). May be NULL if not 
                          required or implemented. (R)*/

/*end*/
/*-Doc:
Return: The offset in the XDL_view_objects array for the new view object.
-end*/
{
   int i;                /*Loop/temporary variable*/
   int iv;               /*Offset into XDL_view_objects pointers array*/
   char msg[121];        /*Array for error message*/
   XDl_view_object **p;  /*Temp pointer for memory allocations*/

   /* Check validity of the new view-object handle */

   if (XDL_num_view_objects>=XDL_max_alloc_objects)
   {
      if (XDL_max_alloc_objects==0)
      {
         p = (XDl_view_object **) malloc (sizeof(XDl_view_object *));
         if (p==NULL)
         {
            xdl_view_errmsg ("xdl_new_view_object",
                             "Cannot allocate space for new view object");
            exit(1);
         }
         XDL_view_objects = p;
         XDL_max_alloc_objects = 1;
      }
      else
      {
         p = (XDl_view_object **) realloc (XDL_view_objects,
                 (XDL_max_alloc_objects+1)*sizeof(XDl_view_object *));
         if (p==NULL)
         {
            xdl_view_errmsg ("xdl_new_view_object",
                             "Cannot allocate space for new view object");
            exit(1);
         }
         XDL_view_objects = p;
         XDL_max_alloc_objects += 1;
      }
   }
   if (vh<=0)
   {
      sprintf (msg, "View object handle (%d) must be greater than zero", vh);
      xdl_view_errmsg ("xdl_new_view_object", msg);
      exit(1);
   }
   if (XDL_num_view_objects>0)
   {
      for (i=0;i<XDL_num_view_objects;++i)
      {
         if (vh==XDL_view_objects[i]->vh)
         {
            sprintf (msg, "view-object handle %d already used", vh);
            xdl_view_errmsg ("xdl_new_view_object", msg);
            exit(1);
         }
      }
   }
   if (vh_parent>0)
   {
      i = xdl_getiv (vh_parent);
      if (i<0)
      {
         sprintf (msg, "Parent view-object handle %d not found", vh_parent);
         xdl_view_errmsg ("xdl_new_view_object", msg);
         exit(1);
      }
   }

   /* Add and initialise the new view-object */

   iv=XDL_num_view_objects;
   XDL_view_objects[iv] = (XDl_view_object *) malloc(sizeof(XDl_view_object));
   XDL_view_objects[iv]->vh = vh;
   XDL_view_objects[iv]->wid = wid;
   XDL_view_objects[iv]->vh_parent = vh_parent;
   XDL_view_objects[iv]->type = type;
   XDL_view_objects[iv]->gd = gd;
   XDL_view_objects[iv]->num_win = 0;
   XDL_view_objects[iv]->on_off_state = 0;
   XDL_view_objects[iv]->win_list = NULL;
   XDL_view_objects[iv]->on_off = on_off;
   XDL_view_objects[iv]->tidy_up = tidy_up;

   XDL_num_view_objects++;

   xdl_add_window (vh, wid, NULL, NULL);

   return iv;
}

/*-Routine: Add a window to the windows list - xdl_add_window
The routine xdl_add_window is used to add a derived (child/subwindow) to 
a list of such windows for the view-object.
-end*/
/*
***************************
**     xdl_add_window    **
***************************

Purpose: To add a derived (child/subwindow) window to a list of 
         such windows for the parent view-object. 

Author:  John W. Campbell, March, 1990

*/

/*-C:*/
void xdl_add_window (vh, wid, repaint, ehp)
/*end*/
 
/*-Parameters:*/
int vh;              /*The view-object handle for the view-object to 
                       whose derived windows list the new window is to 
                       be appended (R)*/
Window wid;          /*The id of the window to be added (R)*/
void (*repaint)();   /*Address of repaint routine for the window 
                       (may be NULL) (R)*/
void (*ehp)();       /*Address of event handling procedure for the window 
                       (may be NULL) (R)*/

/*end*/
/*-Doc:
Return:  None.
-end*/
{
   int iv;         /*Offset for the view-object in the XDL_view_objects array*/
   int n;          /*No. of xview objects in the list*/
   XDl_window *wp; /*Pointer to an XDl_window structure*/

   iv = xdl_getiv (vh);
   if (iv<0) return;

   n = XDL_view_objects[iv]->num_win;
   if (n==0)
   {
      XDL_view_objects[iv]->win_list = (XDl_window *) malloc 
                                        (sizeof(XDl_window));
      XDL_view_objects[iv]->num_win = 1;
      n = 1;
   }
   else
   {
      n++;
      XDL_view_objects[iv]->win_list = (XDl_window *) realloc 
                        (XDL_view_objects[iv]->win_list,n*sizeof(XDl_window));
      XDL_view_objects[iv]->num_win = n;
   }
   wp = &(XDL_view_objects[iv]->win_list[n-1]);
   wp->wid = wid;
   wp->repaint = repaint;
   wp->ehp = ehp;
   wp->libdata = NULL;
   return;
}

/*-Routine: Add a library window to windows list - xdl_add_libwin
The routine xdl_add_libwin is used to add a derived (child/subwindow) 
library routine window to the list of windows for a view-object. 
-end*/
/*
***************************
**     xdl_add_libwin    **
***************************

Purpose: To add a derived (child/subwindow) library routine window to a list of 
         windows for the parent view-object. 

Author:  John W. Campbell, March, 1990

*/

/*-C:*/
void xdl_add_libwin (vh, wid, gd, repaint, ehp, callback, tidy_up)
/*end*/
 
/*-Parameters:*/
int vh;              /*The view-object handle for the view-object to 
                       whose derived windows list the new window is to 
                       be appended (R)*/
Window wid;          /*The id of the window to be added (R)*/
char *gd;            /*The global data area associated with the window -
                       normally that of the parent view-object's global
                       data area for routines forming part of the code
                       for a view-object. Normally a separately defined
                       area for special library routine window objects 
                       (R)*/
void (*repaint)();   /*Address of repaint routine for the window 
                       (may be NULL) (R)*/
void (*ehp)();       /*Address of event handling procedure for the window 
                       (may be NULL) (R)*/
void (*callback)();  /*Address of callback routine if required. 
                       Normally NULL except when using a library routine 
                       window which will return data to the calling 
                       view-object (R)*/
void (*tidy_up)();   /*Routine to be called when a window is deleted to
                       tidy up (e.g. free allocated memory areas or other
                       allocated resources). May be NULL if not required.
                       For a window set up by a routine forming part of 
                       the specific code for the view object, all the 
                       tidying up will normally be done by the one 
                       tidy_up routine associated with its top level 
                       window (R)*/
                       
/*end*/
/*-Doc:
Return:  None.
-end*/
{
   int iv;         /*Offset for the view-object in the XDL_view_objects array*/
   int n;          /*No. of xview objects in the list*/
   XDl_window *wp; /*Pointer to an XDl_window structure*/

   iv = xdl_getiv (vh);
   if (iv<0) return;

   n = XDL_view_objects[iv]->num_win;
   if (n==0)
   {
      XDL_view_objects[iv]->win_list = (XDl_window *) malloc 
                                        (sizeof(XDl_window));
      XDL_view_objects[iv]->num_win = 1;
      n = 1;
   }
   else
   {
      n++;
      XDL_view_objects[iv]->win_list = (XDl_window *) realloc 
                        (XDL_view_objects[iv]->win_list,n*sizeof(XDl_window));
      XDL_view_objects[iv]->num_win = n;
   }
   wp = &(XDL_view_objects[iv]->win_list[n-1]);
   wp->wid = wid;
   wp->repaint = repaint;
   wp->ehp = ehp;
   wp->libdata = (XDl_libdata *) malloc (sizeof(XDl_libdata));
   if (wp->libdata==NULL)
   {
      xdl_view_errmsg ("xdl_add_libwin",
                       "Unable to allocate memory for XDl_libdata structure");
      exit(1);
   }
   wp->libdata->gd = gd;
   wp->libdata->callback = callback;
   wp->libdata->tidy_up = tidy_up;
   return;
}
/*-Routine: Add to level window callbacks - xdl_add_top_callbacks
The routine xdl_add_top_callbacks allows the normally NULL callbacks
(repaint and event handling) to be reset for the top level window of
a view object.
-end*/
/*
********************************
**     xdl_add_top_callbacks  **
********************************

Purpose: This allows the normally NULL callbacks (repaint and event handling)
         routines to be reset for the top level window of a view-object.

Author:  John W. Campbell, November 1993

*/

/*-C:*/
void xdl_add_top_callbacks (vh, repaint, ehp)
/*end*/
 
/*-Parameters:*/
int vh;              /*The view-object handle for the view-object to 
                       whose derived windows list the new window is to 
                       be appended (R)*/
void (*repaint)();   /*Address of repaint routine for the window 
                       (may be NULL) (R)*/
void (*ehp)();       /*Address of event handling procedure for the window 
                       (may be NULL) (R)*/

/*end*/
/*-Doc:
Return:  None.
-end*/
{
   int iv;         /*Offset for the view-object in the XDL_view_objects array*/
   int n;          /*No. of xview objects in the list*/
   XDl_window *wp; /*Pointer to an XDl_window structure*/

   iv = xdl_getiv (vh);
   if (iv<0) return;

   wp = &(XDL_view_objects[iv]->win_list[0]);
   wp->repaint = repaint;
   wp->ehp = ehp;
   return;
}

/*---------------------------------------*/
/*  Getting items from view-objects list */
/*---------------------------------------*/

 
/*-Section: Getting Items from a View-Objects List when Coding View-Objects
These are routines used to get data items from a view-objects internal
list when coding a new view object. 
-end*/

/*-Routine: Get offset in view-objects list from handle - xdl_getiv
The routine xdl_getiv is used to find the offset in the XDL_view_objects 
array for a requested view-object handle.
-end*/
/*
***********************
**     xdl_getiv     **
***********************

Purpose: To find the offset in the XDL_view_objects array for a requested
         view-object handle.

Author:  John W. Campbell, March, 1990

*/

/*-C:*/
int xdl_getiv (vh)
/*end*/
 
/*-Parameters:*/
int vh;              /*The view-object handle (vh) (R)*/

/*end*/
/*-Doc:
Return:  The offset (>=0) or -1 if view object-handle not present.
-end*/
{
   int i;       /*Temp/loop variable*/

   if (XDL_num_view_objects<=0) return -1;

   for (i=0;i<XDL_num_view_objects;++i)
   {
      if (XDL_view_objects[i]->vh==vh) return i;
   }
   return -1;
}

/*-Routine: Get global data area from window id - xdl_get_global
The routine xdl_get_global is used to get a pointer to the main view-object 
global data area for a given window.
-end*/
/*
***************************
**     xdl_get_global    **
***************************

Purpose: Get a pointer to the main view-object global data area for a given 
         window

Author:  John W. Campbell, April 1990
*/

/*-C:*/
char * xdl_get_global (wid)
/*end*/
 
/*-Parameters:*/
Window wid;      /* The window id for which the view-object's data area 
                    is to be found (R)*/

/*end*/
/*-Doc:
Return:  The pointer to the global data (0 if none found)
-end*/
{
   int iv;    /* Offset into the XDL_view_objects array */
   int i;     /* Temporary/loop variable */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      for (i=0;i<XDL_view_objects[iv]->num_win;++i)
      {
         if (XDL_view_objects[iv]->win_list[i].wid==wid)
         {
            return XDL_view_objects[iv]->gd;
         }
      }
   }
   return 0;
}

/*-Routine: Get view-object handle from window id - xdl_get_vh
The routine xdl_get_vh is used to get the view-object handle for the
view-object to which a given window belongs.
-end*/
/*
***********************
**     xdl_get_vh    **
***********************

Purpose: Get the view-object handle for a given window

Author:  John W. Campbell, April 1990
*/

/*-C:*/
int xdl_get_vh (wid)
/*end*/
 
/*-Parameters:*/
Window wid;      /* The window id for which the view-object handle is 
                    to be found (R)*/

/*end*/
/*-Doc:
Return:  The view-object handle (0 if none found)
-end*/
{
   int iv;    /* Offset into the XDL_view_objects array */
   int i;     /* Temporary/loop variable */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
      for (i=0;i<XDL_view_objects[iv]->num_win;++i)
      {
         if (XDL_view_objects[iv]->win_list[i].wid==wid)
         {
            return XDL_view_objects[iv]->vh;
         }
      }
   }
   return 0;
}

/*---------------------------------*/
/*  Delete Panel Item Routine      */
/*---------------------------------*/
 
/*-Section: Deleteing a Panel Item when Coding View-Objects
These are routines used to delete panel items if required when coding 
a new view-object.
-end*/

/*-Routine: Delete a panel item - xdl_delete_panel_item
The routine xdl_delete_panel_item is used to delete a panel item while
the view-object is still in use.
-end*/
/*
**********************************
**     xdl_delete_panel_item    **
**********************************

Purpose: Delete a panel item

Author:  John W. Campbell, August 1993
*/

/*-C:*/
void xdl_delete_panel_item (vh, wid)
/*end*/
 
/*-Parameters:*/
int vh;      /* View-object handle for the view-object to which the
                panel item belongs (R) */
Window wid;  /* The window id. of the panel item (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int iv;          /* Offset into the XDL_view_objects array */
   int i;           /* Temporary/loop variable */
   int ifound;      /* Offset into windows list for panel item window */
   XDl_window *wp;  /* Pointer to window list item */

   iv = xdl_getiv (vh);
   if (iv<0) return;
   ifound = -1;
   for (i=0;i<XDL_view_objects[iv]->num_win;++i)
   {
      wp = &(XDL_view_objects[iv]->win_list[i]);
      if (wp->wid==wid)
      {
         ifound = i;
         if (wp->libdata!=NULL)
         {
            if (wp->libdata->tidy_up!=NULL) 
               (*wp->libdata->tidy_up) (wp->libdata->gd);
            if (wp->libdata->gd!=NULL) free (wp->libdata->gd);
            free (wp->libdata);
         }
         break;
      }
   }
   if (ifound<0) return;
   XDestroyWindow (XDL_display, wid);
   if (ifound<XDL_view_objects[iv]->num_win-1)
   {
      for (i=ifound;i<XDL_view_objects[iv]->num_win-1;++i)
      {
         XDL_view_objects[iv]->win_list[i]=XDL_view_objects[iv]->win_list[i+1];
      }
   }
   XDL_view_objects[iv]->num_win--;
   return;
}

/*---------------------------------*/
/*  Selection Handling  Routines   */
/*---------------------------------*/
 
/*-Section: Handling Selections when Coding View-Objects
These are routines used assit with the handling of text selections from/to a
window if this is required when coding a new view-object.
-end*/

/*-Routine: Request selection ownership - xdl_select_owner
The routine xdl_select_owner is used to get the selection ownership of
the XA_PRIMARY selection (for an example of the use of these selection
routines, see the xdl_io_window.c view-object code).
-end*/
/*
****************************
**     xdl_select_owner   **
****************************

Purpose: Request selection (XA_PRIMARY) ownership

Author:  John W. Campbell, August 1993
*/

/*-C:*/
void xdl_select_owner (wid)
/*end*/
 
/*-Parameters:*/
Window wid;      /* The window id of the window requesting the selection
                    ownership (R) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
    XSetSelectionOwner (XDL_display, XA_PRIMARY, wid, CurrentTime);
    return;
}
/*-Routine: Cancel selection ownership - xdl_select_cancel
The routine xdl_select_cancel is used to cancel the selection ownership of
the XA_PRIMARY selection (for an example of the use of these selection
routines, see the xdl_io_window.c view-object code).
-end*/
/*
****************************
**     xdl_select_cancel  **
****************************

Purpose: Cancel selection (XA_PRIMARY) ownership

Author:  John W. Campbell, August 1993
*/

/*-C:*/
void xdl_select_cancel ()
/*end*/
 
/*-Parameters:*/
{
    XSetSelectionOwner (XDL_display, XA_PRIMARY, None, CurrentTime);
    return;
}
/*
*******************************
**     xdl_select_respond    **
******************************

Purpose: Respond to selection request (XA_PRIMARY)

Return:  None

Author:  John W. Campbell, August 1993
*/

/*Doc:
Respond to selection request - xdl_select_respond

The routine xdl_select_respond is used to respond to a selection request
from another client on XA_PRIMARY. Only string requests are translated; 
otherwise a null reply is given (for an example of the use of these selection
routines, see the xdl_io_window.c view-object code).
*/

void xdl_select_respond (event, str, nitems)

XEvent * event;  /* The selection request event (R)*/
char *str;       /* The string containing the currently selected data (R)*/
int nitems;      /* Number of characters in the string to be sent (excluding
                    any null terminator); may be 0 (R) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XEvent sel_event;   /* Event for reply to requestor */

   sel_event.xselection.type = SelectionNotify;
   sel_event.xselection.send_event = True;
   sel_event.xselection.display = XDL_display;
   sel_event.xselection.requestor = event->xselectionrequest.requestor;
   sel_event.xselection.selection = event->xselectionrequest.selection;
   sel_event.xselection.target = event->xselectionrequest.target;
   sel_event.xselection.property = event->xselectionrequest.property;
   sel_event.xselection.time = event->xselectionrequest.time;
   if (event->xselectionrequest.target==XA_STRING)
   {
       XChangeProperty (XDL_display, event->xselectionrequest.requestor, 
                        event->xselectionrequest.property,
                        XA_STRING, 8, PropModeReplace, 
                        (unsigned char *) str, nitems);
   }
   else
   {
      sel_event.xselection.property = None;
   }
   XSendEvent (XDL_display, event->xselectionrequest.requestor,
              True, 0L, &sel_event); 
   return;
}
/*-Routine: Request a selection - xdl_select_request
The routine xdl_select_request is used to request a (string) selection from
the XA_PRIMARY selection (for an example of the use of these selection
routines, see the xdl_io_window.c view-object code).
-end*/
/*
******************************
**     xdl_select_request   **
******************************

Purpose: Request a (string) selection from XA_PRIMARY

Author:  John W. Campbell, August 1993
*/

/*-C:*/
void xdl_select_request (event, wid)
/*end*/
 
/*-Parameters:*/
XEvent *event;   /* The event triggering the request (for timing) */
Window wid;      /* The window id of the window requesting the selection (R)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   XConvertSelection (XDL_display, XA_PRIMARY, XA_STRING,
                      XDL_paste_buffer, wid, event->xbutton.time);
   return;
}
/*-Routine: Receive selection data - xdl_select_receive
The routine xdl_select_receive is used to get the (string) data received 
following a selection request. This will only be successful if the selection 
owner can translate the request to the required (string) form (for an example 
of the use of these selection routines, see the xdl_io_window.c view-object 
code).
-end*/
/*
******************************
**     xdl_select_receive   **
******************************

Purpose: Receive the string data (if any) following a selection request

Author:  John W. Campbell, August 1993
*/

/*-C:*/
int xdl_select_receive (event,wid,str,nitems,maxinstring)
/*end*/
 
/*-Parameters:*/
XEvent *event;   /* The selection notify event (R)*/
Window wid;      /* The window id of the window requesting the selection
                    ownership (R) */
char **str;      /* Pointer to the received data 

                    NOTE   This data is allocated by the routine
                    ====   and should be freed after it has been 
                           used unless it is a null pointer; this is
                           only needed if the return is 1 (true) as
                           it will already be freed by this routine
                           when the return is 0 (R)*/

int *nitems;     /* Returns the number of items in the string (excluding
                    terminating null (W)*/
int maxinstring; /* Maximum number of items (characters) which may be
                    received */

/*end*/
/*-Doc:
Return:  =1 string received; =0 request could not be satisfied
-end*/
{
   Atom actual_type;          /* Selection property type */
   int actual_format;         /* Selection data format */
   unsigned long bytes_after; /* Selection - additional bytes no returned */

   str[0] = 0;
   if (event->xselection.property!=None)
   {
      XGetWindowProperty (XDL_display, wid, XDL_paste_buffer,
                  0L, (long) (maxinstring/4), True, AnyPropertyType,
                  &actual_type, &actual_format, (unsigned long *) nitems,
                  &bytes_after, (unsigned char **) str);

      if (actual_type==XA_STRING) return 1;
   }
   if (str[0]!=0) free(str[0]);
   return 0;
}

/*---------------------------*/
/*  Error Message Routines   */
/*---------------------------*/

 
/*-Section: Outputting Error Messages when Coding View-Objects
These are routines used to output error messages in a standard format
when coding a new view-object.
-end*/

/*-Routine: Output an XDL_VIEW error message - xdl_view_errmsg
The routine xdl_view_errmsg is used to output an error message for 
xdl_view routines.

The error message will be of the form:
.
    ** xdl_view error in routine xdl_routine_name **
    ** user  supplied  error  message **
.
The stars around the user's message are supplied by the routine.
-end*/
/*
******************************
**      xdl_view_errmsg     **
******************************

Purpose: Output an error message for xdl_view routines.

Author:  John Campbell, August 1990

*/

/*-C:*/
void xdl_view_errmsg (routine_name, message)
/*end*/
 
/*-Parameters:*/
char *routine_name;   /* The name of the routine giving the error (null
                         terminated string (R) */
char *message;        /* A null terminated string describing the error */

/*end*/
{
   fprintf (stderr, "\n\n** xdl_view error in routine %s **", routine_name);
   fprintf (stderr, "\n** %s **\n\n", message);
   return;
}

/*-Routine: 'View-object not found' error message - xdl_view_iv_errmsg
The routine xdl_view_iv_errmsg is used to output an error message for 
xdl_view routines for when a requested view-object handle is not found 
in the list of view objects.

The error message will be of the form:
.
    ** xdl_view error in routine xdl_routine_name **
    ** view-object handle n not found in current list of view-objects **
.
-end*/
/*
*********************************
**      xdl_view_iv_errmsg     **
*********************************

Purpose: Output an error message for xdl_view routines for when a requested
         view-object handle is not found in the list of view objects.

Author:  John Campbell, August 1990

*/

/*-C:*/
void xdl_view_iv_errmsg (routine_name, vh)
/*end*/
 
/*-Parameters:*/
char *routine_name;   /* The name of the routine giving the error 
                         (null terminated string) (R) */
int vh;               /* The view-object handle */
/*end*/
{
   char msg[121];     /* Array to hold error message */

   sprintf (msg,
            "view_object handle %d not found in current list of view-objects",
             vh);
   xdl_view_errmsg (routine_name, msg);
   return;
}

/*-Routine: Error message for wrong type - xdl_view_type_errmsg
The routine xdl_view_type_errmsg is used to output an error message for 
xdl_view routines for when a requested view-object handle is used in a 
routine which corresponds to an incorrect type of view object. 
e.g. an xdl_menu_area_setmenu routine was called with a view handle 
corresponding to an xdl_param_table view-object.

The error message will be of the form:
.
    ** xdl_view error in routine xdl_routine_name **
    ** view-object for handle n is not of type view_object_type **
.
-end*/
/*
***********************************
**      xdl_view_type_errmsg     **
***********************************

Purpose: Output an error message for xdl_view routines for when a requested
         view-object handle is used in a routine which corresponds to an
         incorrect type of view object. 

Author:  John Campbell, August 1990

*/

/*-C:*/
void xdl_view_type_errmsg (routine_name, vh, view_object_type)
/*end*/
 
/*-Parameters:*/
char *routine_name;     /* The name of the routine giving the error (null
                           terminated string)
                           e.g. "xdl_param_table_setvalue" (R) */
int vh;                 /* The view-object handle (R)*/
char *view_object_type; /* Null terminated string giving the view-object
                           type to which the routine belongs
                           e.g. "xdl_param_table" (R) */
/*end*/
{
   char msg[121];     /* Array to hold error message */

   sprintf (msg,
            "view_object for handle %d is not of type %s",
             vh, view_object_type);
   xdl_view_errmsg (routine_name, msg);
   return;
}


/*-----------------------------*/
/* Some Miscellaneous Routines */
/*-----------------------------*/
 
/*-Section: Miscellaneous Routines used when Coding View-Objects
These are some miscellaneous routines which may be used if required
when coding a new view-object.
-end*/

/*-Routine: Get root x, y coordinates - xdl_rootxy
The routine xdl_rootxy is used to get the root window x, y coordinates
from the x, y coordinates within a given window.
-end*/
/*
*************************
**     xdl_rootxy      **
*************************

Purpose: Get root window x and y from a window's x and y

Author:  John W. Campbell, October 1990

*/

/*-C:*/
void xdl_rootxy (wid, x, y, xroot, yroot)
/*end*/
 
/*-Parameters:*/
Window wid;    /*The window id of the window to which x, y coordinates 
                 apply (R)*/
int x;         /*The x coordinate (R)*/
int y;         /*The y coordinate (R)*/
int *xroot;    /*Returns the x coordinate in the root window (W)*/
int *yroot;    /*Returns the y coordinate in the root window (W)*/

/*end*/
/*-Doc:
Return:  xroot, yroot via parameters list
-end*/
{
   Window wroot;           /*Root window id*/
   Window wparent;         /*Returns the parent window id*/
   Window *children;       /*List of child windows*/
   unsigned int nchildren; /*No. of child windows*/
   int xx;                 /*x relative to parent*/
   int yy;                 /*y relative to parent*/
   unsigned int w;         /*window width*/
   unsigned int h;         /*window height*/
   unsigned int brdw;      /*border width*/
   unsigned int depth;     /*window depth*/

   do
   {
      XGetGeometry (XDL_display, wid, &wroot, &xx, &yy, &w, &h, &brdw, &depth);
      XQueryTree (XDL_display, wid, &wroot, &wparent, &children, &nchildren);
      XFree ((void *)children);
      x += xx;
      y += yy;
      wid = wparent;
   } 
   while (wparent!=wroot);

   *xroot = x;
   *yroot = y;
   return;
}

/*-Routine: Copy an output string - xdl_copy_chars
The routine xdl_copy_chars is used to copy an output string from an XDL_VIEW
view-object routine. The output string may be either a 'c' type string
which will be null terminated, or a 'fortran' type string which will be 
padded with blanks if required but will have no terminating null.
-end*/
/*
******************************
**      xdl_copy_chars      **
******************************

Purpose: Copy a string; The output string may be either a 'c' type string
         which will be null terminated, or a 'fortran' type string which
         will be padded with blanks if required but will have no terminating
         null.

Author:  John Campbell, June 1989

*/

/*-C:*/
void xdl_copy_chars (string_in, string_out, max_len)
/*end*/
 
/*-Parameters:*/
char * string_in;    /*The string to be copied (must be null terminated) 
                       (R)*/
char * string_out;   /*The output string address for the copied string 
                       (W) */
int    max_len;      /*If >0 it is the maximum length allowed for the null
                             terminated copied string. string_out must be 
                             at least (max_len + 1) characters in length.
                       if <0 then up to abs(max_len) characters are copied 
                             to the output string. If the number of 
                             characters in the input string is less than 
                             this then the output string will be padded to 
                             abs(max_len) characters with blanks. (R) */
/*end*/
{
   int len;     /* String length */
   int i;       /* Loop counter */

   if (max_len==0) return;

   len =strlen (string_in);
   if (max_len>0)
   {
      if (len>max_len) len = max_len;
      strncpy (string_out, string_in, len);
      string_out[len] = 0;
      return;
   }
   else
   {
      max_len = - max_len;
      if (len>max_len) len = max_len;
      strncpy (string_out, string_in, len);
      if (max_len>len)
      {
         for (i=len;i<max_len;++i) string_out[i] = ' ';
      }
   }
   return;
}

/*-Routine: Test for a blank string - xdl_blank_string
The routine xdl_blank_string is used to test for a blank (/null) character
string.
-end*/
/*
***************************
**    xdl_blank_string   **
***************************

Purpose: Test for a blank(/null) character string

Author:  J. W. Campbell, June 1989

*/

/*-C:*/
int xdl_blank_string (str)
/*end*/
 
/*-Parameters:*/
char str[];   /*String to test (null terminated) (R)*/

/*end*/
/*-Doc:
Return:  =1 (true) blank or null string found,
         =0 not a blank/null string
-end*/
{
   int i;     /*Temp/loop variables*/
   int l;     /*Offset of last character in string*/

   l = strlen (str);
   if (l==0) return 1;
   for (i=0;i<l;++i)
   {
      if (str[i]!=FBLANK) return 0;
   }
   return 1;
}

/*-Routine: Chop trailing blanks - xdl_chop_blanks
The routine xdl_chop_blanks is used to remove trailing blanks from a
character string.
-end*/
/*
**************************
**    xdl_chop_blanks   **
**************************

Purpose: Remove trailing blanks from a character string

Author:  J. W. Campbell, June 1989

*/

/*-C:*/
void xdl_chop_blanks (str)
/*end*/
 
/*-Parameters:*/
char str[];   /*String from which trailing blanks are to be removed 
                (null terminated on input and output) (M)*/

/*end*/
{
   int i;     /*Temp/loop variables*/
   int l;     /*Offset of last character in string*/

   l = strlen (str) - 1;
   if (l<0) return;
   for (i=l;i>=0;--i)
   {
      if (str[i]!=FBLANK)
      {
         if (i!=l) str[i+1]=0;
         return;
      }
   }
   str[0]=0;
   return;
}

/*-Routine: Number of digits in an integer - xdl_ndigit
The routine xdl_ndigit finds the number of decimal digits in an integer
value.
-end*/
/*
************************
**     xdl_ndigit     **
************************

Purpose: Find the number of decimal digits in an integer value.

Author:  John W. Campbell, June 1989

*/

/*-C:*/
int xdl_ndigit (i)
/*end*/
 
/*-Parameters:*/
int i;     /*The integer value for which the number of digits is to 
             be found (R)*/

/*end*/
/*-Doc:
Return:  The number of digits.
-end*/
{
   int k;  /*Temp variable*/
   int n;  /*Count of no. of digits*/

   k = i;
   n = 0;
   do {k=k/10;n++;} while (k!=0);
   return n;
}

/*-Routine: Convert a string to upper case - xdl_upc
The routine xdl_upc is a machine independent routine to convert a
character string to upper case.
-end*/
/*
***********************
**      xdl_upc      **
***********************

Purpose: Convert string to upper case (machine independent version)

Author:  John Campbell, June 1989

*/

/*-C:*/
void xdl_upc (str)
/*end*/
 
/*-Parameters:*/
char str[];    /*String to be converted to upper case 
                 (null terminated) (M)*/

/*end*/
{
   static char uc[27] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; /*Upper case characters*/
   static char lc[27] = "abcdefghijklmnopqrstuvwxyz"; /*Lower case characters*/
   int i,j;             /*Temp/loop variables*/
   int l;               /*Length of string*/

   l = strlen (str);
   if (l==0) return;
   for (i=0;i<l;++i)
   {
      for (j=0;j<26;++j)
      {
         if (str[i]==lc[j]) {str[i]=uc[j]; break;}
      }
   }
   return;
}

/*-Routine: Convert a string to lower case - xdl_lwc
The routine xdl_lwc is a machine independent routine to convert a
character string to lower case.
-end*/
/*
***********************
**      xdl_lwc      **
***********************

Purpose: Convert string to lower case (machine independent version)

Author:  John Campbell, June 1989

*/

/*-C:*/
void xdl_lwc (str)
/*end*/
 
/*-Parameters:*/
char str[];    /*String to be converted to lower case 
                 (null terminated) (M)*/

/*end*/
{
   static char uc[27] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; /*Upper case characters*/
   static char lc[27] = "abcdefghijklmnopqrstuvwxyz"; /*Lower case characters*/
   int i,j;             /*Temp/loop variables*/
   int l;               /*Length of string*/

   l = strlen (str);
   if (l==0) return;
   for (i=0;i<l;++i)
   {
      for (j=0;j<26;++j)
      {
         if (str[i]==uc[j]) {str[i]=lc[j]; break;}
      }
   }
   return;
}

/*-Routine: Convert a character to upper case - xdl_upcc
The routine xdl_upcc is a machine independent routine to convert a
character to upper case.
-end*/
/*
***********************
**      xdl_upcc     **
***********************

Purpose: Convert a character to upper case (machine independent version)

Author:  John Campbell, June 1989

*/

/*-C:*/
void xdl_upcc (ch)
/*end*/
 
/*-Parameters:*/
char *ch;    /*Pointer to character to be converted to upper case (M)*/

/*end*/
{
   static char uc[27] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; /*Upper case characters*/
   static char lc[27] = "abcdefghijklmnopqrstuvwxyz"; /*Lower case characters*/
   int i;               /*Temp/loop variable*/

   for (i=0;i<26;++i)
   {
      if (*ch==lc[i]) {*ch=uc[i]; break;}
   }
   return;
}

/*-Routine: Convert a character to lower case - xdl_lwcc
The routine xdl_lwcc is a machine independent routine to convert a
character to lower case.
-end*/
/*
***********************
**      xdl_lwcc     **
***********************

Purpose: Convert a character to lower case (machine independent version)

Author:  John Campbell, June 1989

*/

/*-C:*/
void xdl_lwcc (ch)
/*end*/
 
/*-Parameters:*/
char *ch;    /*Pointer to character to be converted to lower case (M)*/

/*end*/
{
   static char uc[27] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"; /*Upper case characters*/
   static char lc[27] = "abcdefghijklmnopqrstuvwxyz"; /*Lower case characters*/
   int i;               /*Temp/loop variable*/

   for (i=0;i<26;++i)
   {
      if (*ch==uc[i]) {*ch=lc[i]; break;}
   }
   return;
}

/*-Routine: Random number, 0-32767 - xdl_rand16
The routine xdl_rand16 is a machine independent routine to return a
pseudo-random number in the range 0 to 32767. The seed may be reset
with the routine xdl_srand16; its initial value is 1.
-end*/
/*
*************************
**      xdl_rand16     **
*************************

Purpose: Return pseudo-random number in range 0 to 32767

Author:  John Campbell, July, 1992 (from K&R)

*/

/*-C:*/
int xdl_rand16 ()
/*end*/
 
/*-Parameters:*/
{
   xdl_next_seed = xdl_next_seed*1103515245 + 12345;
   return (unsigned int) (xdl_next_seed/65536)%32768;
}

/*
***************************
**      xdl_srand16      **
***************************

Purpose: Set new seed for random number routine xdl_rand16

Author:  John Campbell, July 1992 (from K&R)

*/

/*Doc:
Set seed for xdl_rand16 - xdl_srand16

The routine xdl_srand16 is used to reset the seed value for the pseudo-random
number routine xdl_rand16.
*/


void xdl_srand16 (seed)

unsigned int seed;  /* New seed value (R)*/

/*end*/
{
   xdl_next_seed = (unsigned long int) seed;
   return;
}
/*-Routine: Get character size - xdl_ch_size
The routine xdl_ch_size is used to get the character size for a character
in one of the XDL_VIEW standard fonts; this may be for a normal font,
a bold font or the larger or smaller dimensions of the two.
-end*/
/*
***************************
**      xdl_ch_size      **
***************************

Purpose: Get character size for one of the standard fonts

Author:  John Campbell, July 1992

*/

/*-C:*/
void xdl_ch_size (font_type, opt, ch_width, ch_height, ascent, descent)
/*end*/
 
/*-Parameters:*/
int font_type;     /* Font type (1 to 5) (R)*/
int opt;           /* = 0, normal font
                      = 1, bold font
                      = 2, larger dimensions of the normal/bold fonts
                      = 3, smaller dimensions of the normal/bold fonts
                      (R)*/
int *ch_width;     /* Returns the character width in pixels (W)*/
int *ch_height;    /* Returns the character height in pixels (W)*/
int *ascent;       /* Returns font ascent (W)*/
int *descent;      /* Returns font descent (W)*/

/*end*/
{
   XFontStruct *fontinfo;
   XFontStruct *bold_fontinfo;

   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   *ch_width = 0;
   *ch_height = 0;
   
   switch (opt)
   {
      case 0:
         fontinfo = XDL_fonts[font_type-1];
         *ch_width = fontinfo->max_bounds.width;
         *ch_height = fontinfo->descent + fontinfo->ascent;
         *ascent = fontinfo->ascent;
         *descent = fontinfo->descent;
         return;
      case 1:
         bold_fontinfo = XDL_bold_fonts[font_type-1];
         *ch_width = bold_fontinfo->max_bounds.width;
         *ch_height = bold_fontinfo->descent + bold_fontinfo->ascent;         
         *ascent = bold_fontinfo->ascent;
         *descent = bold_fontinfo->descent;
         return;
      case 2:
         fontinfo = XDL_fonts[font_type-1];
         bold_fontinfo = XDL_bold_fonts[font_type-1];
         *ch_width = fontinfo->max_bounds.width;
         if (bold_fontinfo->max_bounds.width>*ch_width) 
            *ch_width = bold_fontinfo->max_bounds.width;
         *ch_height = fontinfo->descent + fontinfo->ascent;
         *ascent = fontinfo->ascent;
         *descent = fontinfo->descent;
         if ((bold_fontinfo->descent + bold_fontinfo->ascent)>*ch_height)
         {
            *ch_height = bold_fontinfo->descent + bold_fontinfo->ascent;
            *ascent = bold_fontinfo->ascent;
            *descent = bold_fontinfo->descent;
         }
         return;
      case 3:
         fontinfo = XDL_fonts[font_type-1];
         bold_fontinfo = XDL_bold_fonts[font_type-1];
         *ch_width = fontinfo->max_bounds.width;
         if (bold_fontinfo->max_bounds.width<*ch_width) 
            *ch_width = bold_fontinfo->max_bounds.width;
         *ch_height = fontinfo->descent + fontinfo->ascent;
         *ascent = fontinfo->ascent;
         *descent = fontinfo->descent;
         if ((bold_fontinfo->descent + bold_fontinfo->ascent)<*ch_height)
         {
            *ch_height = bold_fontinfo->descent + bold_fontinfo->ascent;
            *ascent = bold_fontinfo->ascent;
            *descent = bold_fontinfo->descent;
         }
         return;
   }
   return;
}

/*-Routine: Set a 64 level gray scale - xdl_set64gray
The routine xdl_set64gray is used to set up a 64 level
gray scale. The colorset must have been allocated when xdl_open_view
(xdlf_open_view) was called and must have at least 64 colorcells
allocated. 
-end*/
/*
****************************
**     xdl_set64gray      **
****************************

Purpose: Set 64 colour gray scale

Author:  John W. Campbell, April 1990

*/

/*-C:*/
int xdl_set64gray (cset, contrast_val, type)
/*end*/
 
/*-Parameters:*/
int cset;         /* The colorset number (R)*/
int contrast_val; /* Contrast value from 1 to 1024 (R)*/
int type;         /* =1 high density = black, 
                     =2 high density = white (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= colorset not allocated or has <64 cells or 
                  contast parameter out of range
-end*/
{
   XColor colr_cell;      /* Temporary Xcolor structure */
   int i,j;               /* Temporary/loop variables */

   /* Check parameters */

   if (cset<0) return 0;
   if (cset>XDL_num_colorsets) return 0;
   if (XDL_csets_alloc==0) return 0;
   if (XDL_colorsets[cset].ncolors<64) return 0;
   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Set gray scale */

   for (i=0;i<64;++i)
   {
      if(type==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[i];
      }else{
         colr_cell.pixel = XDL_colorsets[cset].pixels[63-i];
      }
      j = 65535-contrast_val*i;if(j<0)j=0;
      colr_cell.red = j;
      colr_cell.green = j;
      colr_cell.blue = j;
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   return 1;
}

/*-Routine: Set a 64 level gray + yellow scale - xdl_set64gray_yel
The routine xdl_set64gray_yel is used to set up a 
64 level gray scale with the top level gray being replaced by yellow. 
The colorset must have been allocated when xdl_open_view (xdlf_open_view) 
was called and must have at least 64 colorcells allocated.
-end*/
/*
********************************
**     xdl_set64gray_yel      **
********************************

Purpose: Set 64 colour gray scale but with top level yellow so that it can
         be used to indicate saturation.

Author:  John W. Campbell, April 1990

*/

/*-C:*/
int xdl_set64gray_yel (cset, contrast_val, type)
/*end*/
 
/*-Parameters:*/
int cset;         /* The colorset number (R)*/
int contrast_val; /* Contrast value from 1 to 1024 (R)*/
int type;         /* =1 high density = black, 
                     =2 high density = white (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= colorset not allocated or has <64 cells 
                  or contrast parameter out of range
-end*/
{
   XColor colr_cell;      /* Temporary Xcolor structure */
   int i,j;               /* Temporary/loop variables */

   /* Check parameters */

   if (cset<0) return 0;
   if (cset>XDL_num_colorsets) return 0;
   if (XDL_csets_alloc==0) return 0;
   if (XDL_colorsets[cset].ncolors<64) return 0;
   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Set gray scale */

   for (i=0;i<64;++i)
   {
      if(type==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[i];
      }else{
         colr_cell.pixel = XDL_colorsets[cset].pixels[63-i];
      }
      j = 65535-contrast_val*i;if(j<0)j=0;
      colr_cell.red = j;
      colr_cell.green = j;
      colr_cell.blue = j;
      if ((type==1&&i==63)||(type==2&&i==0))
      {
         colr_cell.red = 65535;
         colr_cell.green = 65535;
         colr_cell.blue = 0;
      }
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   return 1;
}

/*-Routine: Set a 64 level colour scale - xdl_set64color
The routine xdl_set64color is used to set up a 64 level
colour scale. The colour scale is blue..purple..red..orange..yellow with
increasing brightness or the same colour sequence in reverse order. The
routine. The colorset must have been allocated when xdl_open_view 
(xdlf_open_view) was called and must have at least 64 colorcells allocated.
-end*/
/*
****************************
**     xdl_set64color     **
****************************

Purpose: Set 64 colour graded color scale (blue..purple..red..orange..yellow)

Author:  John W. Campbell, April 1990

*/

/*-C:*/
int xdl_set64color (cset, contrast_val, type)
/*end*/
 
/*-Parameters:*/
int cset;         /* The colorset number (R)*/
int contrast_val; /* Contrast value from 1 to 1024 (R)*/
int type;         /* =1 high density at blue end of scale
                     =2 high density at yellow end of scale (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= colorset not allocated or <64 cells 
                  or contrast parameter out of range
-end*/
{
   XColor colr_cell;                   /* Temporary Xcolor structure */
   int i,j;                            /* Temporary/loop variables */
   static float rd_max=24000.0;        /* Colour setting parameters */
   static float rd_dec=500.0;
   static float gr_max=24000.0;
   static float gr_dec=1500.0;
   static float bl_inc=750.0;
   float wh_max,wh_dec,dw,rd,gr,bl,wh;

   /* Check parameters */

   if (cset<0) return 0;
   if (cset>XDL_num_colorsets) return 0;
   if (XDL_csets_alloc==0) return 0;
   if (XDL_colorsets[cset].ncolors<64) return 0;
   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Set up colour scale */

   wh_max=148000.0;
   wh_dec=200.0+(float)contrast_val;
   rd=rd_max+rd_dec;
   gr=gr_max+gr_dec;
   bl=0.0;wh=wh_max+wh_dec;
   for (i=0;i<64;++i)
   {
      wh-=wh_dec;
      if(type==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[i];
      }else{
         colr_cell.pixel = XDL_colorsets[cset].pixels[63-i];
      }
      rd-=rd_dec;gr-=gr_dec;if(i>=32)bl+=bl_inc;
      if(rd<0.0)rd=0.0;
      if(gr<0.0)gr=0.0;
      dw=(wh-rd-gr-bl)/3.0;
      colr_cell.red = (int)(dw+rd);
      colr_cell.green = (int)(dw+gr);
      colr_cell.blue = (int)(dw+bl);
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   return 1;
}

/*-Routine: Get a 64 level gray scale - xdl_get64gray
The routine xdl_get64gray is used to get a 64 level gray scale. The nearest
colours available will be allocated.
-end*/
/*
****************************
**     xdl_get64gray      **
****************************

Purpose: Get 64 colour gray scale

Author:  John W. Campbell, August 1991

*/

/*-C:*/
int xdl_get64gray (pixels, contrast_val, type)
/*end*/
 
/*-Parameters:*/
unsigned long pixels[64];         /* The returned pixels list (W)*/
int contrast_val;                 /* Contrast value from 1 to 1024 (R)*/
int type;                         /* =1 high density = black, 
                                     =2 high density = white (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= contast parameter out of range
-end*/
{
   XColor colr_cell;      /* Temporary Xcolor structure */
   int i,j,k;             /* Temporary/loop variables */

   /* Check parameters */

   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Get gray scale */

   for (i=0;i<64;++i)
   {
      if(type==1)
      {
         k = i;
      }else{
         k = 63 - i;
      }
      j = 65535-contrast_val*i;if(j<0)j=0;
      colr_cell.red = j;
      colr_cell.green = j;
      colr_cell.blue = j;
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      pixels[k] = colr_cell.pixel;
   }
   return 1;
}

/*-Routine: Get a 64 level gray + yellow scale - xdl_get64gray_yel
The routine xdl_get64gray_yel is used to get a 
64 level gray scale with the top level gray being replaced by yellow. The
nearest available colours are allocated.
-end*/
/*
********************************
**     xdl_get64gray_yel      **
********************************

Purpose: Get 64 colour gray scale but with top level yellow so that it can
         be used to indicate saturation.

Author:  John W. Campbell, August 1991

*/

/*-C:*/
int xdl_get64gray_yel (pixels, contrast_val, type)
/*end*/
 
/*-Parameters:*/
unsigned long pixels[64];         /* The returned pixels list (W)*/
int contrast_val;                 /* Contrast value from 1 to 1024 (R)*/
int type;                         /* =1 high density = black, 
                                     =2 high density = white (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= contrast parameter out of range
-end*/
{
   XColor colr_cell;      /* Temporary Xcolor structure */
   int i,j,k;             /* Temporary/loop variables */

   /* Check parameters */

   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Set gray scale */

   for (i=0;i<64;++i)
   {
      if(type==1)
      {
         k = i;
      }else{
         k = 63 - i;
      }
      j = 65535-contrast_val*i;if(j<0)j=0;
      colr_cell.red = j;
      colr_cell.green = j;
      colr_cell.blue = j;
      if ((type==1&&i==63)||(type==2&&i==0))
      {
         colr_cell.red = 65535;
         colr_cell.green = 65535;
         colr_cell.blue = 0;
      }
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      pixels[k] = colr_cell.pixel;
   }
   return 1;
}

/*-Routine: Get a 64 level colour scale - xdl_get64color
The routine xdl_get64color is used to get a 64 level
colour scale. The colour scale is blue..purple..red..orange..yellow with
increasing brightness or the same colour sequence in reverse order. 
The nearest available colours are allocated.
-end*/
/*
****************************
**     xdl_get64color     **
****************************

Purpose: Get 64 colour graded color scale (blue..purple..red..orange..yellow)

Author:  John W. Campbell, August 1991

*/

/*-C:*/
int xdl_get64color (pixels, contrast_val, type)
/*end*/
 
/*-Parameters:*/
unsigned long pixels[64];         /* The returned pixels list (W)*/
int contrast_val;                 /* Contrast value from 1 to 1024 (R)*/
int type;                         /* =1 high density at blue end of scale
                                     =2 high density at yellow end of scale 
                                        (R)*/

/*end*/
/*-Doc:
Return:  1=OK, 0= contrast parameter out of range
-end*/
{
   XColor colr_cell;                   /* Temporary Xcolor structure */
   int i,j,k;                          /* Temporary/loop variables */
   static float rd_max=24000.0;        /* Colour setting parameters */
   static float rd_dec=500.0;
   static float gr_max=24000.0;
   static float gr_dec=1500.0;
   static float bl_inc=750.0;
   float wh_max,wh_dec,dw,rd,gr,bl,wh;

   /* Check parameters */

   if (contrast_val<1||contrast_val>1024) return 0;
   if (type!=1&&type!=2) return 0;

   /* Set up colour scale */

   wh_max=148000.0;
   wh_dec=200.0+(float)contrast_val;
   rd=rd_max+rd_dec;
   gr=gr_max+gr_dec;
   bl=0.0;wh=wh_max+wh_dec;
   for (i=0;i<64;++i)
   {
      wh-=wh_dec;
      if(type==1)
      {
         k = i;
      }else{
         k = 63 - i;
      }
      rd-=rd_dec;gr-=gr_dec;if(i>=32)bl+=bl_inc;
      if(rd<0.0)rd=0.0;
      if(gr<0.0)gr=0.0;
      dw=(wh-rd-gr-bl)/3.0;
      colr_cell.red = (int)(dw+rd);
      colr_cell.green = (int)(dw+gr);
      colr_cell.blue = (int)(dw+bl);
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      pixels[k] = colr_cell.pixel;
   }
   return 1;
}

/*================================================================*/
/*    SPECIAL SERVICE ROUTINES USED BY THE XDL_VIEW ROUTINES      */
/*================================================================*/
  
/*-Routine: Delete view-object children - xdl_delete_children
The routine xdl_delete_children is used to delete the children of a 
view-object. (Recursively)
-end*/
/*
*********************************
**     xdl_delete_children     **
*********************************

Purpose: Delete the children of a view-object. (Recursively)

Author:  John W. Campbell, April 1990

*/

/*-C:*/
void xdl_delete_children (vh_parent)
/*end*/
 
/*-Parameters:*/
int vh_parent;           /* View-object handle of the parent (R) */

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int i, j;         /*Loop variables*/
   int iv;           /*Offset in XDL_view_objects array*/
   int vh;           /*View-object handle*/
   int vh_p;         /*View-object handle for parent*/
   XDl_window *wp;   /*Temporary pointer to XDl_window structure*/

   /* Loop through view-objects (from the end to the start as entries in
      the table may be deleted during this procedure). For child
      view-objects see if they have any children and if so delete these; if
      not delete the child object */

   for (iv=XDL_num_view_objects-1;iv>=0;--iv)
   {
       vh = XDL_view_objects[iv]->vh;
       vh_p = XDL_view_objects[iv]->vh_parent;
       if (vh_p==vh_parent)
       {
          /* See if any children */
          if (xdl_any_children (vh))
          {
             /* Yes - delete these children */
             xdl_delete_children (vh);
          }
          else
          {
             /* No - Call tidy_ups and delete global data area(s) */
             for (i=0;i<XDL_view_objects[iv]->num_win;++i)
             {
                wp = &(XDL_view_objects[iv]->win_list[i]);
                if (wp->libdata!=NULL)
                {
                   if (wp->libdata->tidy_up!=NULL) 
                      (*wp->libdata->tidy_up) (wp->libdata->gd);
                   if (wp->libdata->gd!=NULL) free (wp->libdata->gd);
                   free (wp->libdata);
                }
             }
             if (XDL_view_objects[iv]->tidy_up!=NULL) 
                (*XDL_view_objects[iv]->tidy_up) (XDL_view_objects[iv]->gd);
             if (XDL_view_objects[iv]->gd!=0) 
                free (XDL_view_objects[iv]->gd);

             /* Remove the view-object from the list of view-objects */
             if (iv==XDL_num_view_objects-1)
             {
                free (XDL_view_objects[iv]);
                XDL_view_objects[iv] = 0;
                XDL_num_view_objects--;
             }
             else
             {
                free (XDL_view_objects[iv]);
                for (i=iv+1,j=iv;i<XDL_num_view_objects;++i,++j)
                {
                    XDL_view_objects[j] = XDL_view_objects[i];
                }
		XDL_view_objects[XDL_num_view_objects] = 0;
		XDL_num_view_objects--;
             }
          }
      }
   }

   /* Return */

   return;
}

/*-Routine: See if a view-object has children - xdl_any_children
The routine xdl_any_children is used to find whether or not a view-object
has any child view-objects.
-end*/
/*
*****************************
**     xdl_any_children    **
*****************************

Purpose: See if a view-object has any children

Author:  John W. Campbell, April 1990

*/

/*-C:*/
int xdl_any_children (vh)
/*end*/
 
/*-Parameters:*/
int vh;           /* View-object handle (R) */

/*end*/
/*-Doc:
Return:  =1 (true) child or children found, =0 no children;
-end*/
{
   int iv;   /* Offset in XDL_view_objects array */

   /* See if any view object is a child of the requested view_object */

   for (iv=0;iv<XDL_num_view_objects;++iv)
   {
       if (XDL_view_objects[iv]->vh_parent==vh) return 1;
   }
   return 0;
}
int IgnoreError(display, xerrorevent )
Display *display;
XErrorEvent *xerrorevent;
{
  int debug, serial_no, error_code, major_req_code, minor_req_code;

  debug = 0;
  if (debug) {
    serial_no = (int)(xerrorevent->serial);
    error_code = (int)(xerrorevent->error_code);
    major_req_code = (int)(xerrorevent->request_code);
    minor_req_code = (int)(xerrorevent->minor_code);
    fprintf(stderr,
     "Caught X server error - this is not necessarily a real problem.\n");
    fprintf (stderr,"  Serial no., error code, request codes: %d %d %d %d\n",
     serial_no,error_code,major_req_code,minor_req_code);
    fflush(stderr);
  }
  return(1);
}

/* Various xdl_view stuff bogusly assumes that a free'd pointer is
   nullified.  Provide our own routine to do that, with `free'
   #defined to it.  */
#undef free
void
xdl_free (void *ptr)
{
  free (ptr);
  ptr = NULL;
}
