/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Film Image View-Object Routines
-end*/
 
/*-Intro:
The film image view-object is designed specifically for displaying film or 
image-plate images used in Protein Crystallography. It may be used for 
measuring spot positions and has facilities for overlaying symbols or
other vectors on top of the displayed film image. A magnifying window within 
the view-object shows a magnified version of a portion of the image from the 
main image display area. A control panel allows for a series of adjustments 
to the display.
-end*/
 
/*-Section: Creating and Handling a Film Image View-Object
-end*/
 

/*Routines List:
The film image view-object has the following routines available:

   xdl_film_image                 Create a film image view-object
   xdl_film_image_getxy           Get selected pixel position
   xdl_film_image_getxy_ovly      Get pixel position from an overlay
   xdl_film_image_getrect         Get selected rectangle
   xdl_film_image_background      Set background image
   xdl_film_image_newimg          Set new image
   xdl_film_image_reset_fd        Reset file descriptor
   xdl_film_image_ovly_offset     Reset overlay offset values
   xdl_film_image_set_colour      Reset an overlay colour
   xdl_film_image_set_colormap    Reset the user defined colour map
   xdl_film_image_symbol          Store and display an overlay symbol
   xdl_film_image_symbols         Store and display overlay symbols
   xdl_film_image_clear_symbols   Clear overlay symbols
   xdl_film_image_del_symbol      Delete an overlay symbol
   xdl_film_image_vect            Store and display an overlay vector
   xdl_film_image_clear_vects     Clear overlay vectors
   xdl_film_image_del_vect        Delete an overlay vector
   xdl_film_image_input_message   Reset active strip message
   xdl_film_image_getsize         Calculate size requirements
*/

/*-Routine: Create a film image view-object - xdl_film_image
The routine xdl_film_image (xdlf_film_image) is used to create a new
film image view object. The parameters give details of the image to
be displayed.
-end*/
/*
**************************
**    xdl_film_image    **
**************************

Purpose: Display an X-ray diffraction film (or image plate) image.

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE (IVH, IVHPARENT, IX, IY, ICSET, ITYPE,
        +                      IORD, IMGDATA, IMGW, IMGH, MAXVAL, 
        +                      MINTHR, MAXTHR, MINW, MINH, 
        +                      IXOPIX, IYOPIX, NXP, NYP, IBG, IOVLY,
        +                      IFULL, IMGFULL,
        +                      IFD, NXRASTS, NYRASTS, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number  (see cset)
ITYPE     (R)   1 = film image data
                2 = image-plate data
                3 = signed integer data
                (see type)
IORD      (R)   Data order in image file or full image 
                in terms of the xf, yf coordinate system as
                a number from 1 to 8.
                1   +xf slow   +yf fast
                2   +xf slow   -yf fast
                3   -xf slow   +yf fast
                4   -xf slow   -yf fast
                5   +yf slow   +xf fast
                6   +yf slow   -xf fast
                7   -yf slow   +xf fast
                8   -yf slow   -xf fast
                If this value is negated then the data will
                also be byte swapped (image plate data only)
                (see order)
IMGDATA   (R)   Array holding compressed data (e.g. pass as an 
                integer array with data packed as needed)  
                (see imgdata)
IMGW      (R)   Displayed image width in pixels (see img_width)
IMGH      (R)   Displayed image height in pixels (see img_height)
MAXVAL    (R)   Maximum value present (see max_val)
MINTHR    (R)   Minimum threshold for scaling (image-plate only)
                (see min_thresh)
MAXTHR    (R)   Maximum threshold for scaling (image-plate only)
                (see max_thresh)
MINW      (R)   Min. width for view-object or 0 (see min_width)
MINH      (R)   Min. height for view-object or 0 (see min_height)
IXOPIX    (R)   X origin pixel (see xo_pix)
IYOPIX    (R)   Y origin pixel (see yo_pix)
NXP       (R)   No. x pixels compressed into 1 (see nxp)
NYP       (R)   No. y pixels compressed into 1 (see nyp)
IBG       (R)   =0 no background choice menu, 
                =1 background choice menu (Set background via
                   xdlf_film_image_background
                   (see bg_typ)
IOVLY     (R)   = 1 Overlay options are on/off/offset
                = 2 Overlay options are on/off/1 on/2 on
                (see ovly_typ)
IFULL     (R)   1 = full image will be passed to the routine (in
                    addition to the compressed image section)
                0 = full image will not be passed.
                (see full)
IMGFULL   (R)   Array holding full image data (e.g. pass as an
                integer array with data packed as needed). Dummy
                array if IFULL=0.
                (see img_full)
IFD       (R)   File descriptor of film image file (-1 if none to
                be read). Ignored if ifull=1. (see fd)
NXRASTS   (R)   No. of x rasters in the complete image 
                (see nxrasts)
NYRASTS   (R)   No. of y rasters in the complete image 
                (see nyrasts)
IERR      (W)   Returns status from xdl_film_image call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"

/* Define some parameters */

                          /*Check gd->magwsizes[] if next 3 values changed */
#define MAGWSIZE 128      /*Maximum size of magnified image window*/
#define MAGWSIZE_W 128    /*Width to allow for magnifying window in control
                            panel */
#define MAGWSIZE_H 64     /*Height to allow for magnifying window in control
                            panel */
#define DEFAULT_MAG 4     /*Default magnification for magnified image window*/
#define MIN_MAG 2         /*Minimum magnification for magnifying window*/
#define MAX_MONO_MAG 9    /*Max magnification for monochrome display magnifying
                            window option*/
#define NSYMB_ALLOC 20    /*No. of symbols structures to allocate at a time*/
#define NVECT_ALLOC 20    /*No. of vector structures to allocate at a time*/
#define MAX_SYMB_SIZE 25  /*Maximum symbol size in pixels*/
#define Y_SEARCH_MAX 64   /*No. of values in 'speed up' y_search array*/

/* Define structure for table of symbols */

typedef struct
{
   unsigned short x_pixel;        /* x pixel posn. (full image, 1 upwards) */
   unsigned short y_pixel;        /* y pixel posn. (full image, 1 upwards) */
   short x_disp_offset;           /* x offset wrt displayed image */
   short y_disp_offset;           /* y offset wrt displayed image */
   unsigned short x_nxp_offset;   /* x offset within a displayed pix 0 to 
                                     nxp-1 */
   unsigned short y_nyp_offset;   /* y offset within a displayed pix 0 to 
                                     nyp-1 */
   unsigned char  symb;           /* symbol type number */
   unsigned char  color_flags;    /* symbol colour & flags 
                                     bits 0-3 colour
                                     bit  4   =0 overlay 1, =1 overlay 2
                                     bit  5   =0 allow offset, =1 fixed */
   unsigned int   indx;           /* Reserved for use in sorting (index) */
   unsigned long  temp;           /* Reserved for use in sorting */
}
   XDl_film_image_symbol;

/* Define structure for table of vectors */

typedef struct
{
   unsigned short vect_id;        /* Identifier for vector or group of vectors
                                     (user defined; used in delete option) */
   short x1_disp_offset;          /* x offset wrt displayed image 1'st point */
   unsigned short x1_nxp_offset;  /* x offset within a displayed pix 0 to 
                                     nxp-1 */
   short y1_disp_offset;          /* y offset wrt displayed image 1'st point
                                     Note: 1'st point always chosen to have
                                           the lower y value */
   unsigned short y1_nyp_offset;  /* y offset within a displayed pix 0 to 
                                     nyp-1 */
   short x2_disp_offset;          /* x offset wrt displayed image 2'nd point */
   unsigned short x2_nxp_offset;  /* x offset within a displayed pix 0 to 
                                     nxp-1 */
   short y2_disp_offset;          /* y offset wrt displayed image 2'nd point */
   unsigned short y2_nyp_offset;  /* y offset within a displayed pix 0 to 
                                     nyp-1 */
   unsigned char  color_flags;    /* vector colour & flags 
                                     bits 0-3 colour
                                     bit  4   =0 overlay 1, =1 overlay 2
                                     bit  5   =0 allow offset, =1 fixed */
}
   XDl_film_image_vect;

/* Define structure for holding details of area frozen in magnifying window */

typedef struct
{
   int            on;             /* =1 frozen area to restore, =0 not */
   int            xo;             /* X origin of rectangle on film display to be
                                     restored */
   int            yo;             /* Y origin */
   int            width;          /* Width of rectangle */
   int            height;         /* Height of rectangle */
}
   XDl_film_image_freeze;

/* Define structure for the global data for the view-object */

typedef struct
{
   int          type;                  /* Image type 1=film, 2=image-plate (i2)
                                          3 = signed integer */
   int          full_image;            /* =1 full image input
                                          =0 full image not input */
   char        *full_imgdata;          /* Pointer to full image data if
                                          present */
   int          xrasts;                /* No. of x rasters in full image
                                          if full image was input or if
                                          file is to be read */
   int          yrasts;                /* No. of y rasters in full image
                                          if full image was input or if
                                          file is to be read */
   int          img_width;             /* Displayed image width */
   int          img_height;            /* Displayed image height */
   int          nxp;                   /* No. x pixels combined into 1 pixel */
   int          nyp;                   /* No. y pixels combined into 1 pixel */
   Window       wfilm;                 /* Top level window */
   Window       wact;                  /* Active strip window id */
   Window       wdisp;                 /* Film image display area window */
   Window       wmag;                  /* magnifying area window */
   Window       wpanel;                /* Control panel window */
   Panel_object contrast_slider;       /* Contrast slider */
   Panel_object thresh_slider;         /* Monochrome threshold slider */
   Panel_object cursor_position;       /* Cursor position label */
   Panel_object color_choice;          /* Color choice menu */
   Panel_object mag_choice;            /* Magnification factor menu */
   Panel_object ovly_choice;           /* Overlay on/off etc. choice item */
   Panel_object bg_choice;             /* Background choice menu */
   Panel_object max_text_item;         /* Maximum threshold text item */
   Panel_object min_text_item;         /* Minimum threshold text item */
   Panel_object ps_button;             /* Postscript o/p button */
   GC           paint_gc;              /* GC for paint windows */
   Cursor       cursor;                /* Cursor for film and magw windows */
   char         *img;                  /* Array to hold pixel data for the 
                                          XImage */
   char         *imgdata;              /* Pointer to the compressed image 
                                          data */
   int          magwsize;              /* Current size for magnifying window */
   int          magwsizes[2];          /* List of magnifying window sizes */
   int          magwsize_opt;          /* Current option for magnifying window
                                          size */
   XImage       *img_image;            /* XImage for the film image */
   XImage       *mag_image;            /* XImage for the magnifying window 
                                          (current) */
   XImage       *mag_images[2];        /* XImages for the magnifying window
                                          small and large */
   unsigned char *mag_data;            /* Magnifying window pixel data */
   int          cset;                  /* Colorset number */
   int          contrast_val;          /* Contast value 1-1024 */
   int          thresh_val;            /* Monochrome threshold value 0-63 */
   int          color_opt;             /* Color option type */
   int          mag;                   /* Current magnification factor */
   int          magnify_on;            /* Magnifier on flag =1 on (follows
                                          pointer) =0 off (frozen) */
   int          xc;                    /* Current x offset on image on which
                                          magnifier image is centered */
   int          yc;                    /* Current y offset on image on which
                                          magnifier image is centered */
   int          xo_pix;                /* Origin pixel (x) */
   int          yo_pix;                /* Origin pixel (y) */
   int          xpix;                  /* Image pixel x at mouse position */
   int          ypix;                  /* Image pixel y at mouse position */
   int          xpix_dn;               /* Last pixel x on button 1 down */
   int          ypix_dn;               /* Last pixel y on button 1 down */
   int          xpix_up;               /* Last pixel x on button 1 up */
   int          ypix_up;               /* Last pixel y on button 1 up */
   int          xpix_img;              /* Returned x pixel no. for selected 
                                          spot or 1'st corner of rectangle*/
   int          ypix_img;              /* Returned y pixel no. for selected 
                                          spot or 1'st corner of rectangle */
   int          xpix_img_up;           /* Returned x pixel no. of 2'nd corner
                                          of selected rectangle */
   int          ypix_img_up;           /* Returned y pixel no. of 2'nd corner
                                          of selected rectangle */
   int          sel_rect;              /* Rectangle being selected on display
                                          =1 yes, =0 no */
   int          sel_rect_xo;           /* x origin of selected rectangle;
                                          button 1 down position */
   int          sel_rect_yo;           /* y origin of selected rectangle;
                                          button 1 down position */
   int          sel_rect_x1;           /* x top left of selected rectangle */
   int          sel_rect_y1;           /* y top left of selected rectangle */
   int          sel_rect_x2;           /* x bottom right of rectangle */
   int          sel_rect_y2;           /* y bottom right of rectangle */
   Pixmap       sel_rect_t_line;       /* Pixmap to save top line of pixels
                                          where selected rectangle will be
                                          drawn */
   Pixmap       sel_rect_b_line;       /* Pixmap to save bottom line */
   Pixmap       sel_rect_l_line;       /* Pixmap to save left line */
   Pixmap       sel_rect_r_line;       /* Pixmap to save right line */
   int          fd;                    /* File descriptor for film or image plate
                                          file. (< 0 if no file to be read) */
   int          read_file;             /* Flag for magnifying window o/p
                                          = 0 use stored compressed image data, 
                                          = 1 read data from image file or
                                              from full image if given */
   char         *read_image;           /* Points to space to store read in data
                                          for magnifying window  (unsigned char
                                          for film data, unsigned short for
                                          image plate data) */
   char         *read_buf;             /* Buffer for reading in an image file
                                          record (unsigned char for film data,
                                          unsigned short for image plate data) 
                                          */
   unsigned short *monodata;           /* Buffer to hold pixel data for
                                          monochrome display on magnifying
                                          window */
   int          max_val;               /* Maximum intensity value in image*/
   int          min_thresh;            /* Minimum threshold for scaling (image 
                                          plate) */
   int          max_thresh;            /* Maximum threshold for scaling (image
                                          plate) */
   int          on_off;                /* Active input flag 1=yes, 0=no */
   char         input_message[61];     /* Active strip message */
   int          ovly_typ;              /* Overlay type from input params.
                                          =1 on/off/offset
                                          =2 on/off/ov1 on/ov2 on */
   int          ovly1;                 /* =1 overlay 1 on, =0 off */
   int          ovly2;                 /* =1 overlay 2 on, =0 off */
   int          ovly_offset;           /* =1 offset overlay (if other
                                             conditions are met), =0 do not */
   int          ovly_opt;              /* Number of latest item selected from
                                          overlay menu */
   int          x_ovly_offset;         /* x offset for overlay when it is to be
                                          offset (in pixels on film display
                                          area) */
   int          y_ovly_offset;         /* y offset for overlay */
   int          max_alloc_symbols;     /* Max space currently allocated for
                                          storing symbols information */
   int          num_symb;              /* Number of symbols in  symbols list */
   XDl_film_image_symbol *symbols;     /* Pointer to array of symbols 
                                          structures */
   int          y_search[Y_SEARCH_MAX];/* Indexing array to speed up search for
                                          symbols */
   int          max_alloc_vects;       /* Max space currently allocated for
                                          storing vectors information */
   int          num_vect;              /* Number of vectors in vectors list */
   XDl_film_image_vect *vects;         /* Pointer to array of vectors 
                                          structures */
   int         *nxp_to_mag;            /* Pointer to array of conversions from
                                          pixel offsets (0 to nxp-1) to 
                                          (0 to mag-1) */
   int         *nyp_to_mag;            /* Pointer to array of conversions from
                                          pixel offsets (0 to nyp-1) to 
                                          (0 to mag-1) */
   unsigned long graypixels[64];       /* Pixels for gray scales */
   unsigned long ovly_pixels[7];       /* Pixels for overlay colours */
   unsigned short user_rgb[3][64];     /* User defined rgb values for a
                                          color scale */
   XDl_film_image_freeze freeze;       /* Details of 'frozen' rectangle */
   unsigned long freeze_pixel;         /* Colour for frozen area rectangle */
   int          bg_typ;                /* =1 include background options
                                          =0 do not */
   int          bg_present;            /* =1 background image has been supplied
                                          =0 background image not yet given */
   int          bg_opt;                /* =0 included (or bg_typ=0)
                                          =1 subtracted
                                          =2 background only */
   char        *bg_data;               /* Pointer to background data */
   int         bytes_per_zpix;         /* No. of bytes per z-pixel in
                                          XImage image data */
   int         mono_grscal;            /* For monochrome display
                                          =1, Simulate grayscale in mag window.
                                          =0, Do not   */
   int         order;                  /* File /full image data order 1-8 */
   int         swap;                   /* Swap bytes flag 1=yes, 0=no */
}  
   XDl_film_image;

/* Define routine types */
int xdl_film_image();
void xdl_film_image_rps();
void xdl_film_image_rp1();
void xdl_film_image_rp2();
void xdl_film_image_rp3();
void xdl_film_image_refresh();
void xdl_film_image_ep1();
void xdl_film_image_ep2();
void xdl_film_image_getpix1();
int xdl_film_image_getpix2();
void xdl_film_image_cp1();
void xdl_film_image_cp2();
void xdl_film_image_cp3();
void xdl_film_image_cp4();
void xdl_film_image_cp5();
void xdl_film_image_cp6();
void xdl_film_image_cp7();
void xdl_film_image_cp8();
void xdl_film_image_rds();
void xdl_film_image_rdv();
void xdl_film_image_magw();
void xdl_film_image_setdata();
void xdl_film_image_set_freeze();
void xdl_film_image_restore_freeze();
void xdl_film_image_set_rect();
void xdl_film_image_restore_rect();
void xdl_film_image_intensity();
void xdl_film_image_drawsymb();
void xdl_film_image_drawvec();
void xdl_film_image_dsppos();
void xdl_film_image_postscript();
void xdl_film_image_on_off();
void xdl_film_image_tidy_up();
int xdl_film_image_getxy();
int xdl_film_image_getxy_ovly();
int xdl_film_image_getrect();
int xdl_film_image_background();
int xdl_film_image_newimg();
int xdl_film_image_reset_fd();
int xdl_film_image_ovly_offset();
int xdl_film_image_set_colour();
int xdl_film_image_set_colormap();
int xdl_film_image_clear_symbols();
int xdl_film_image_del_symbol();
int xdl_film_image_input_message();
int xdl_film_image_symbol();
int xdl_film_image_symbols();
int xdl_film_image_vect();
int xdl_film_image_clear_vects();
int xdl_film_image_del_vect();
void xdl_film_image_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_film_image xdlf_film_image_
#define xdlf_film_image_getxy xdlf_film_image_getxy_
#define xdlf_film_image_getxy_ovly xdlf_film_image_getxy_ovly_
#define xdlf_film_image_getrect xdlf_film_image_getrect_
#define xdlf_film_image_getsize xdlf_film_image_getsize_
#define xdlf_film_image_symbol xdlf_film_image_symbol_
#define xdlf_film_image_symbols xdlf_film_image_symbols_
#define xdlf_film_image_background xdlf_film_image_background_
#define xdlf_film_image_newimg xdlf_film_image_newimg_
#define xdlf_film_image_reset_fd xdlf_film_image_reset_fd_
#define xdlf_film_image_ovly_offset xdlf_film_image_ovly_offset_
#define xdlf_film_image_set_colour xdlf_film_image_set_colour_
#define xdlf_film_image_set_colormap xdlf_film_image_set_colormap_
#define xdlf_film_image_clear_symbols xdlf_film_image_clear_symbols_
#define xdlf_film_image_del_symbol xdlf_film_image_del_symbol_
#define xdlf_film_image_vect xdlf_film_image_vect_
#define xdlf_film_image_clear_vects xdlf_film_image_clear_vects_
#define xdlf_film_image_del_vect xdlf_film_image_del_vect_
#define xdlf_film_image_input_message xdlf_film_image_input_message_

#endif

#if  LINKTYP == 2

#define xdlf_film_image XDLF_FILM_IMAGE
#define xdlf_film_image_getxy XDLF_FILM_IMAGE_GETXY
#define xdlf_film_image_getxy_ovly XDLF_FILM_IMAGE_GETXY_OVLY
#define xdlf_film_image_getrect XDLF_FILM_IMAGE_GETRECT
#define xdlf_film_image_getsize XDLF_FILM_IMAGE_GETSIZE
#define xdlf_film_image_symbol XDLF_FILM_IMAGE_SYMBOL
#define xdlf_film_image_symbols XDLF_FILM_IMAGE_SYMBOLS
#define xdlf_film_image_background XDLF_FILM_IMAGE_BACKGROUND
#define xdlf_film_image_newimg XDLF_FILM_IMAGE_NEWIMG
#define xdlf_film_image_reset_fd XDLF_FILM_IMAGE_RESET_FD
#define xdlf_film_image_ovly_offset XDLF_FILM_IMAGE_OVLY_OFFSET
#define xdlf_film_image_set_colour XDLF_FILM_IMAGE_SET_COLOUR
#define xdlf_film_image_set_colormap XDLF_FILM_IMAGE_SET_COLORMAP
#define xdlf_film_image_clear_symbols XDLF_FILM_IMAGE_CLEAR_SYMBOLS
#define xdlf_film_image_del_symbol XDLF_FILM_IMAGE_DEL_SYMBOL
#define xdlf_film_image_vect XDLF_FILM_IMAGE_VECT
#define xdlf_film_image_clear_vects XDLF_FILM_IMAGE_CLEAR_VECTS
#define xdlf_film_image_del_vect XDLF_FILM_IMAGE_DEL_VECT
#define xdlf_film_image_input_message XDLF_FILM_IMAGE_INPUT_MESSAGE

#endif

void xdlf_film_image();
void xdlf_film_image_getxy();
void xdlf_film_image_getxy_ovly();
void xdlf_film_image_getrect();
void xdlf_film_image_getsize();
void xdlf_film_image_symbol();
void xdlf_film_image_symbols();
void xdlf_film_image_background();
void xdlf_film_image_newimg();
void xdlf_film_image_reset_fd();
void xdlf_film_image_ovly_offset();
void xdlf_film_image_set_colour();
void xdlf_film_image_set_colormap();
void xdlf_film_image_clear_symbols();
void xdlf_film_image_del_symbol();
void xdlf_film_image_vect();
void xdlf_film_image_clear_vects();
void xdlf_film_image_del_vect();
void xdlf_film_image_input_message();

/*-C:*/
int xdl_film_image (vh, vh_parent, x, y, cset, type, order, imgdata, 
                    img_width, img_height, max_val, 
                    min_thresh, max_thresh, min_width, min_height, 
                    xo_pix, yo_pix, nxp, nyp, bg_typ, ovly_typ,
                    full, img_full, fd, nxrasts, nyrasts)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame,
                             if 0 then a base  level frame is created to
                             enclose the image plate image view-object 
                             (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used for the 
                             film image display; must have 72 or more 
                             colorcells allocated */
int       type;           /* 1 = compressed film image section
                                 (unsigned byte)
                             2 = compressed image-plate image section
                                 (unsigned two byte integer (short))
                             3 = compressed signed integer data image 
                                 section
                             (R)*/
int order;                /* Order of the data in the input image file
                             or in the full image in terms of the xf, yf 
                             coordinate system as a number from 1 to 8.
                             1   +xf slow   +yf fast
                             2   +xf slow   -yf fast
                             3   -xf slow   +yf fast
                             4   -xf slow   -yf fast
                             5   +yf slow   +xf fast
                             6   +yf slow   -xf fast
                             7   -yf slow   +xf fast
                             8   -yf slow   -xf fast
                             If -order is given (for image plate data) then
                             the data will be byte swapped.
                             (R)*/
char      *imgdata;       /* Compressed image data section in unsigned byte 
                             format, unsigned two byte or signed integer
                             format as indicated by the 'type' parameter.
                             The image data must remain in place as long
                             as the view object is being used as it will 
                             be accessed again if the scaling thresholds 
                             etc. are changed. For the compressed data 
                             (nxp*nyp pixels into 1 pixel), the slower 
                             moving index is the film yf going from high 
                             to low and the faster moving index is the film 
                             xf going from low to high. The order of the 
                             compressed image must always be the same 
                             regardless of the order of the full image or 
                             order of data in the input file as indicated 
                             by the 'order' parameter. The compressed image 
                             need only correspond to a section of the full 
                             image (see xo_pix, yo_pix etc.)
                             (R)*/
int       img_width;      /* Image width (x) for the compressed 
                             image section (see type) in pixels (R)*/
int       img_height;     /* Image height (y) for the compressed
                             image section (see type) in pixels (R)*/
int       max_val;        /* Maximum value of a pixel in the image.
                             Must be given a value > 0. Used
                             as upper limit for max_thresh. (R)*/
int       min_thresh;     /* Minimum threshold value for the initial 
                             intensity scaling. 0<=min_thesh<max_val. 
                             (R)*/
int       max_thresh;     /* Maximum threshold value for the initial
                             intensity scaling. 
                             min_thresh<max_thresh<=max_val. (R)*/
int       min_width;      /* minimum width for the displayed view-object.
                             The program will calculate the required width
                             for the view-object based on the image width,
                             the space required for the magnified image,
                             slider bars, menus etc. If this is less than
                             min_width then min_width will be used and the
                             view-object will be set out as appropriate. 
                             (R)*/
int       min_height;     /* minimum height for the displayed view object.
                             cf min_width (R) */
int       xo_pix;         /* The xf origin pixel number (pixels numbered
                             from 1 up) for the section of the film or 
                             image plate image passed to the routine. 
                             This is used so that displayed and returned 
                             pixel numbers are those corresponding to the
                             complete image. (R)*/
int       yo_pix;         /* The yf origin pixel number (pixels numbered
                             from 1 up) for the section of the film or 
                             image plate image passed to the routine. 
                             This is used so that displayed and returned 
                             pixel numbers are those corresponding to the
                             complete image. (R)*/
int       nxp;            /* The number of film or image plate pixels in 
                             the xf direction combined into one pixel in 
                             the  image passed to the routine. e.g. 4 if
                             a 2400*2400 image has been reduced to 
                             600*600 for display (R)*/
int       nyp;            /* The number of film or image plate pixels in 
                             the yf direction combined into one pixel in 
                             the image passed to the routine cf nxp. (R)*/
int       bg_typ;         /* =0 no background choice menu, 
                             =1 background choice menu (Set background via
                                xdl_film_image_background routine) (R)*/
int       ovly_typ;       /* = 1 Overlay options are on/off/offset.
                             = 2 Overlay options are on/off/1 on/2 on.
                                 Two separate sets of overlay symbols may
                                 be drawn.
                                 In case 1, the offset only applies to
                                 to overlay 1; In case 2 no offsetting is
                                 done but it is possible to switch between
                                 the two overlays. (R)*/
int       full;           /* 1 = pass full image to the routine (img_full)
                                 as well as the compressed image. This
                                 is used for magnifyiing window data and
                                 pixel intensities if given.
                             0 = do not pass the full image
                             (R)*/
char      *img_full;      /* Full image data (byte, unsigned i2 or signed
                             integer as indicated by 'type') if required 
                             (see full); otherwise null. The data may be 
                             in any of the eight possible orders as 
                             indicated by the 'order' parameter. The 
                             default order has the film xf as the slower 
                             moving index from low to high and the film yf 
                             as the faster moving index from low to
                             high. (R)*/
int       fd;             /* File descriptor for the film or image plate 
                             file.
                             May be -1 if no file is to be read.
                             (The file reading is used if nxp>1 or nyp>1
                             and the middle mouse button is clicked to
                             freeze an area in the magnifying window enabling 
                             a more detailed image to be displayed there. It 
                             will also be used to find a pixel intensity if 
                             this is needed).
                             Not used if full=1.
                             (R) */
int       nxrasts;        /* The number of xf rasters in the full image 
                             (R)*/
int       nyrasts;        /* The number of yf rasters in the full image 
                             (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to allocate memory for global data area
                bit 2 set: Unable to allocate memory for internal image 
                           data
                bit 3 set: Unable to allocate memory for read file arrays
                bit 4 set: Error in reading file header (image plate file)
                bit 5 set: Unable to allocate memory for nxp_to_mag or
                           nyp_to_mag arrays.
-end*/
{
   Window wfilm;            /* Window id for primary window */
   Window wparent;          /* Window id of parent window */
   Status st;               /* Status return from Xlib calls */
   XDl_film_image * gd;     /* Pointer to view-object's global data structure */
   int i,j,k,kk;            /* Temporary/loop variables */
   int recl;                /* Record length */
   char * p;                /* Temporary pointer */
   int err;                 /* Error return flag */
   int iv;                  /* Offset into XDL_view_objects array */
   int width, height;       /* Full width/height of the object */
   int img_x, img_y;        /* Offsets in view-object for positioning the image
                               display area */
   int panel_x;             /* Offset x in view-object for positioning the
                               control panel area or item */
   int panel_y;             /* Offset y for control panel item */
   int font;                /* Font size 1 to 5 */
   int slider_width;        /* Contrast slider width */
   int small_cursor;        /* Small cursor flag =1 yes, =0 no */
   int start_pos;           /* Start position for slider */
   char valstr[12];         /* Value string for numeric text item */
   int ytop;                /* Height required for top of object items */
   short header[6];         /* Buffer for reading image plate file header 
                               info */
   float fr;                /* mag/nxp or mag/nyp ratio (as floats) */
   XColor colr_cell;        /* Temporary Xcolor structure */
   XColor cursor_fg;        /* Structure for setting cross cursor foreground */
   XColor cursor_bg;        /* Structure for setting cross cursor background */
   Pixmap cursor_source;    /* Pixmap for cursor source */
   Pixmap cursor_mask;      /* Pixmap for cursor mask */
   static unsigned char cursor_source_bits[] = {
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x0F, 0x0F, 0x8F, 0x87, 0x87, 0x07,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00};

   static unsigned char cursor_mask_bits[] = {
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0xFF, 0xFF, 0x8F, 0xFF, 0xFF, 0x07,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00};
   static unsigned char curs15_source_bits[] ={
      0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x33, 0x66, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static unsigned char curs15_mask_bits[] = {
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x3f, 0x7e, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static char *color_strings[] =
   {
     "Black on white",
     "White on black",
     "Yellow if >Max",
     "Colour 1",
     "Colour 2",
     "User defined",
     "Random colour"
   };                                          /* Colour options menu strings */
   static char *mag_strings[] = 
   {"x2", "x3", "x4", "x5", "x6", "x7", "x8"}; /* Magnifying choice strings */
   static char *mag_strings2[] =
   {"x2", "x3", "x4", "x5", "x6", "x7", "x8", "x3G", "x6G", "x9G"};
                                               /* Magnifying choice strings
                                                  for monochrome case */
   static char *ovly_strings[] =
   {"on", "off", "offset"};                    /* Overlay choice strings for
                                                  overlay type 1*/
   static char *ovly_strings2[] =
   {"on", "off", "ov1 on", "ov2 on"};          /* Overlay choice strings for
                                                  overlay type 2*/
   static char *bg_strings[] =
   {"inc.", "sub.", "only"};                    /* Background choice strings */

   err = 0;
   font = 2;
   if (type!=2&&type!=3) type = 1;
   if (max_val<=0) max_val = 1;
   if (min_thresh>=max_val) min_thresh = max_val - 1;
   if (min_thresh<0) min_thresh = 0;
   if (max_thresh>max_val) max_thresh = max_val;
   if (max_thresh<=min_thresh) max_thresh = min_thresh + 1;
   if (ovly_typ!=2) ovly_typ = 1;
   if (bg_typ!=1) bg_typ = 0;
   if (full!=1) full = 0;

   /* Calculate the size requirements. *WARNING* update xdl_film_image_getsize 
      if this section changed */
   
   width = 355 + MAGWSIZE_W;
   if ((img_width+2+2*BORDER_WIDTH+2)>width) 
      width = img_width + 2 + 2*BORDER_WIDTH + 2;
   if (min_width>width) width = min_width;
   panel_x = BORDER_WIDTH + MAGWSIZE_W +2 + (width-2-335-MAGWSIZE_W-2)/2;
   img_x = (width-2-img_width-2)/2;
   ytop = 66 + BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   if (MAGWSIZE_H+2+ACT_STRIP_HEIGHT>ytop) ytop = MAGWSIZE_H+2+ACT_STRIP_HEIGHT;
   height = img_height + 2 + 2*BORDER_HEIGHT + 2 + ytop;
   if (min_height>height) height = min_height;
   img_y = ytop + (height-2-img_height-2-ytop)/2;

   /* Create a primary (top level) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      {
         err |= 1;
         return err;
      }
      /* Create frame window as parent frame was given */
      wparent = XDL_view_objects[iv]->wid;
      wfilm = xdl_create_frame_window (wparent, x, y, width, height,
                                       cset);
   }else{
   /* Create base frame window as no parent was specified */
      wfilm = xdl_create_base_frame_window ("xdl_film_image","xdl_film",
                         x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_film_image));
   if (p==0) {err |= 2; return err;}
   gd = (XDl_film_image *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wfilm, vh_parent, p, 1081, xdl_film_image_on_off,
                        xdl_film_image_tidy_up);

   /* Store image related parameters */

   gd->nxp = nxp;
   gd->nyp = nyp;
   gd->bytes_per_zpix = (XDL_image_bits-1)/8 + 1;
   gd->type = type;
   gd->swap = 0;
   if (order<0)
   {
      gd->swap = 1;
      order = -order;
   }
   if (order<1||order>8) order = 1;
   gd->order = order;
   gd->imgdata = imgdata;
   gd->full_image = full;
   if (full==1)
   {
      gd->full_imgdata = img_full;
   }
   else
   {
      gd->full_imgdata = NULL;
   }
   gd->xrasts = nxrasts;
   gd->yrasts = nyrasts;
   gd->img_width = img_width;
   gd->img_height = img_height;
   gd->xo_pix = xo_pix;
   gd->yo_pix = yo_pix;

   /* Allocate space for magnifying window image data */

   gd->mag_data = (unsigned char *) 
                   malloc (gd->bytes_per_zpix*MAGWSIZE*MAGWSIZE);
   if (gd->mag_data==0) {err|=4; return err;}

   /* Some other initialisations */

   gd->wfilm = wfilm;
   gd->on_off = 0;
   gd->color_opt = 1;
   gd->contrast_val = 1023;
   gd->thresh_val = 32;
   gd->cset = cset;
   if (XDL_color>0)
   {
      gd->mag = DEFAULT_MAG;
   }
   else
   {
      gd->mag = 3;
   }
   gd->magnify_on = 1;
   gd->magwsizes[0] = MAGWSIZE_H;
   gd->magwsizes[1] = 2*(gd->magwsizes[0]);
   gd->magwsize = gd->magwsizes[0];
   gd->magwsize_opt = 1;
   gd->freeze.on = 0;
   gd->xpix = -1;
   gd->ypix = -1;
   gd->xpix_dn = -1;
   gd->ypix_dn = -1;
   gd->xpix_up = -1;
   gd->ypix_up = -1;
   gd->xpix_img = -1;
   gd->ypix_img = -1;
   gd->xpix_img_up = -1;
   gd->ypix_img_up = -1;
   gd->xc = -1;
   gd->yc = -1;
   gd->sel_rect = 0;
   gd->sel_rect_xo = 0;
   gd->sel_rect_yo = 0;
   gd->sel_rect_x1 = 0;
   gd->sel_rect_y1 = 0;
   gd->sel_rect_x2 = 0;
   gd->sel_rect_y2 = 0;
   gd->fd = fd;
   gd->read_file = 0;
   gd->read_image = NULL;
   gd->read_buf = NULL;
   gd->monodata = NULL;
   gd->mono_grscal = 0;
   gd->max_val = max_val;
   gd->min_thresh = min_thresh;
   gd->max_thresh = max_thresh;
   gd->bg_typ = bg_typ;
   gd->bg_present = 0;
   gd->bg_opt = 0;
   gd->bg_data = NULL;
   gd->ovly_typ = ovly_typ;
   gd->ovly_opt = 1;
   gd->ovly1 = 1;
   gd->ovly2 = 1;
   gd->ovly_offset = 0;
   gd->x_ovly_offset = -4;
   gd->y_ovly_offset = 4;
   gd->max_alloc_symbols = 0;
   gd->num_symb = 0;
   gd->symbols = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
   gd->vects = NULL;
   gd->nxp_to_mag = NULL;
   gd->nyp_to_mag = NULL;
   gd->nxp_to_mag = (int *) malloc (nxp*sizeof(int));
   if (gd->nxp_to_mag==0) {err |= 32; return err;}
   gd->nyp_to_mag = (int *) malloc (nyp*sizeof(int));
   if (gd->nyp_to_mag==0) {err |= 32; return err;}
   fr = (float) gd->mag / (float) nxp;
   for (i=0;i<nxp;++i) gd->nxp_to_mag[i] = (int) ((float) i + 0.5) * fr;
   fr = (float) gd->mag / (float) nyp;
   for (i=0;i<nyp;++i) gd->nyp_to_mag[i] = (int) ((float) i + 0.5) * fr;
   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i] = 0;     
   p = strcpy (gd->input_message, "Input spot positions");

   /* Create active strip area */

   gd->wact = xdl_create_active_strip (wfilm, vh, 
                      width-2*BORDER_WIDTH-2, xdl_film_image_rps);

   /* Set initial color map */

   if (XDL_color>0)
   {
      if (XDL_csets_alloc==1)
      {
         xdl_set64gray (gd->cset, gd->contrast_val, 1);
         for (i=0;i<64;++i)
         {
            gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
         }
      }
      else
      {
         xdl_get64gray (gd->graypixels, gd->contrast_val, 1);
      }
   }
   else
   {
      for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_whitepixel;
      for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_blackpixel;
   }

   /* Set overlay colours */

   if (XDL_color>1)
   {
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      /* Red */
      colr_cell.red = 65535;
      colr_cell.green = 0;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[64];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[0] = colr_cell.pixel;
      /* Yellow */
      colr_cell.red = 65535;
      colr_cell.green = 65535;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[65];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[1] = colr_cell.pixel;
      /* Green */
      colr_cell.red = 0;
      colr_cell.green = 65535;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[66];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[2] = colr_cell.pixel;
      /* Cyan */
      colr_cell.red = 0;
      colr_cell.green = 65535;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[67];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[3] = colr_cell.pixel;
      /* Blue */
      colr_cell.red = 0;
      colr_cell.green = 0;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[68];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[4] = colr_cell.pixel;
      /* Magenta */
      colr_cell.red = 65535;
      colr_cell.green = 0;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[69];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[5] = colr_cell.pixel;
      /* Freeze rectangle colour */
      colr_cell.red = 10000;
      colr_cell.green = 50000;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[70];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[6] = colr_cell.pixel;
      gd->freeze_pixel = colr_cell.pixel;
   }
   else
   {
      for (i=0;i<7;++i) gd->ovly_pixels[i] = XDL_blackpixel;
      gd->freeze_pixel = XDL_blackpixel;
   }

   /* Create film display area paint window */

   gd->wdisp = xdl_create_paint_window (wfilm, img_x, img_y, 1,
                    img_width+2, img_height+2, cset);

   xdl_add_window (vh, gd->wdisp, xdl_film_image_rp1, xdl_film_image_ep1);

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wdisp, ExposureMask|StructureNotifyMask|
           ButtonPressMask|ButtonReleaseMask|PointerMotionMask|LeaveWindowMask);

   /* Set Cross cursor */

   small_cursor = 1;
   p = XGetDefault (XDL_display,"xdl_film_image","cursor");
   if (p!=NULL)
   {
      if (p[0]=='l') small_cursor = 0;
   }
   if (small_cursor)
   {
      cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                          (char *) curs15_source_bits, 15, 15);

      cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                          (char *) curs15_mask_bits, 15, 15);
   }
   else
   {
      cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                           (char *) cursor_source_bits, 43, 43);

      cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                           (char *) cursor_mask_bits, 43, 43);
   }

   cursor_fg.red = 0;
   cursor_fg.green = 0;
   cursor_fg.blue = 0;
   cursor_fg.flags = DoRed|DoGreen|DoBlue;

   cursor_bg.red = 65535;
   cursor_bg.green = 65535;
   cursor_bg.blue = 65535;
   cursor_bg.flags = DoRed|DoGreen|DoBlue;
   
   if (small_cursor)
   {
      gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                        &cursor_fg, &cursor_bg, 7, 7);
   }
   else
   {
      gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                     &cursor_fg, &cursor_bg, 21, 21);
   }

   XFreePixmap (XDL_display, cursor_source);
   XFreePixmap (XDL_display, cursor_mask);

   XDefineCursor (XDL_display, gd->wdisp, gd->cursor);

   /* Magnifier paint window */

   gd->wmag = xdl_create_paint_window (wfilm, BORDER_WIDTH, 
            BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1, gd->magwsize+2, 
            gd->magwsize+2, cset);

   xdl_add_window (vh, gd->wmag, xdl_film_image_rp2, xdl_film_image_ep2);

   /* Set Cross cursor */

   XDefineCursor (XDL_display, gd->wmag, gd->cursor);

   /* Prepare for event handling */
  
   XSelectInput (XDL_display, gd->wmag, ExposureMask|StructureNotifyMask|
           ButtonPressMask|ButtonReleaseMask|PointerMotionMask|LeaveWindowMask);

   /* Paint window for control panel */

   gd->wpanel = xdl_create_paint_window (wfilm, panel_x, 
                BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1, 335, 66, cset);

   xdl_add_window (vh, gd->wpanel, xdl_film_image_rp3, NULL);

   XSelectInput (XDL_display, gd->wpanel, ExposureMask|StructureNotifyMask);

   /* Add contrast slider unless monochrome*/

   if (XDL_color>0)
   {
      panel_x = 200;
      panel_y = 24;
      slider_width = 40;
      start_pos = 40;

      gd->contrast_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                            "Contrast", font, slider_width, start_pos, 0,
                            0, 0, xdl_film_image_cp1);
   }
   else
   {
      /* Add threshold slider for monochrome */
      panel_x = 200;
      panel_y = 24;
      slider_width = 64;
      start_pos = 32;

      gd->thresh_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                            "Thresh", font, slider_width, start_pos, 0,
                            0, 0, xdl_film_image_cp8);
   }

   /* Add cursor position string */

   panel_x = 200;
   panel_y = 4;

   gd->cursor_position = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                         "Cursor Position", 16, font, 0,  0, 0, 0);

   /* Add color type choice menu */

   panel_x = 4;
   panel_y = 44;

   i=2;
   if (XDL_color>1) i=7;

   gd->color_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Colour ", i, color_strings, 1, font, 0, 0, 0,
                      xdl_film_image_cp2);

   /* Add magnifying choice menu */

   panel_x = 200;
   panel_y = 44;

   if (XDL_color>0)
   {
      gd->mag_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                       "Mag", 7, mag_strings, 3, font, 0, 0, 0,
                       xdl_film_image_cp3);
   }
   else
   {
      gd->mag_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                       "Mag", 10, mag_strings2, 1, font, 0, 0, 0,
                       xdl_film_image_cp3);
   }

   /* Add Overlay on/off choice item */

   panel_x = 4;
   panel_y = 24;

   if (gd->ovly_typ==1)
   {
      gd->ovly_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                        "Overlay", 3, ovly_strings, 1, font, 0, 0, 0,
                        xdl_film_image_cp6);
   }
   else
   {
      gd->ovly_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                        "Overlay", 4, ovly_strings2, 1, font, 0, 0, 0,
                        xdl_film_image_cp6);
   }

   /* Add threshold numeric text input items image plate display */

   sprintf (valstr, "%d", gd->min_thresh);

   gd->min_text_item = xdl_panel_value (vh, gd->wpanel, 4, 4,
                       "Min", valstr, 6, font, 0, 0, 0, xdl_film_image_cp5);

   sprintf (valstr, "%d", gd->max_thresh);

   gd->max_text_item = xdl_panel_value (vh, gd->wpanel, 102, 4,
                       "Max", valstr, 6, font, 0, 0, 0, xdl_film_image_cp4);

   /* Add background choice menu if required */

   if (bg_typ>0)
   {
      panel_x = 130;
      panel_y = 24;
      gd->bg_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Bg", 3, bg_strings, 1, font, 0, 0, 0,
                       xdl_film_image_cp7);
   }

   /* Add Postscript print button */

   panel_x = 294;
   panel_y = 44;
   gd->ps_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                   "PS", 2, 1, 2, 0, 0, 1, 1, 1, xdl_film_image_postscript);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wdisp, 0, NULL);

   /* Initialise 'User defined' rgb colorscale */

   kk = 0;
   for (i=1;i<=4;++i)
   {
      for (j=1;j<=4;++j)
      {
         for (k=1;k<=4;++k)
         {
            gd->user_rgb[0][kk] = 16000*i;
            gd->user_rgb[1][kk] = 16000*j;
            gd->user_rgb[2][kk] = 16000*k;
            kk++;
         }
      }
   }

   /* Create pixmaps to store saved lines of pixels for select rectangle
      option. (Use this method rather than xor'ing to give control over
      colour of drawn outline */

   gd->sel_rect_t_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    gd->img_width, 1, XDL_image_depth);
   gd->sel_rect_b_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    gd->img_width, 1, XDL_image_depth);
   gd->sel_rect_l_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    1, gd->img_height, XDL_image_depth);
   gd->sel_rect_r_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    1, gd->img_height, XDL_image_depth);

   /* Create data area for image data ZPixmap */

   gd->img = (char *) malloc ((gd->bytes_per_zpix)*img_width
                             *img_height*sizeof(char));
   if (gd->img==0) {err |= 4; return err;}

   /* Set up XImages for film/image-plate data and magnified data*/

   gd->img_image = XCreateImage (XDL_display, XDL_visual,
                                  XDL_image_depth, ZPixmap, 0, gd->img, img_width, 
                                  img_height, 8, 0);

   gd->mag_images[0] = XCreateImage (XDL_display, XDL_visual, 
                                 XDL_image_depth, ZPixmap, 0, 
                                 (char *) gd->mag_data, 
                                 gd->magwsizes[0], gd->magwsizes[0],
                                 8, 0);

   gd->mag_images[1] = XCreateImage (XDL_display, XDL_visual, 
                                 XDL_image_depth, ZPixmap, 0, 
                                 (char *) gd->mag_data, 
                                 gd->magwsizes[1], gd->magwsizes[1],
                                 8, 0);

   gd->mag_image = gd->mag_images[0];

   /* Set up pixels data */

   xdl_film_image_setdata (gd, gd->type);

   /* For monochrome display set up extra array for handling magnifying
      window data */

   if (XDL_color==0)
   {
      i = MAGWSIZE + 2*MAX_MONO_MAG;
      gd->monodata = (unsigned short *) 
           malloc (i*i*sizeof(unsigned short));
      if (gd->monodata==0) {err |= 4; return err;}
   }

   /* Allocate array and buffer for reading in data from image file if needed */

   if (gd->fd>=0)
   {
      i = (MAGWSIZE+MIN_MAG-1)/(2*MIN_MAG);
      i = 2*i + 1;
      i = i*i*nxp*nyp;
      switch (gd->type)
      {
         case 1:
            p = (char *) malloc (i);
            break;
         case 2:
            p = (char *) malloc (i*sizeof(short));
            break;
         case 3:
            p = (char *) malloc (i*sizeof(int));
            break;
      }
      if (p==0) {err |= 8; return err;}
      gd->read_image = (char *) p;

      /* Allocate buffer for reading a record */

      if (gd->full_image==0)
      {
         recl = gd->yrasts;
         if (gd->order>4) recl = gd->xrasts;
         if (gd->type==2) recl *= 2;
         if (gd->type==3) recl *= sizeof(int);
         p = (char *) malloc(recl);
         if (p==0) {err |= 8; return err;}
         gd->read_buf = (char *) p;
      }
   }

   return 0;
}

/* Fortran binding: xdlf_film_image */

void xdlf_film_image (ivh, ivhparent, ix, iy, icset, itype, iord, imgdata, 
                      imgw, imgh, maxval, minthr, maxthr, minw, minh, 
                      ixopix, iyopix, nxp, nyp, ibg, iovly, ifull,
                      imgfull, ifd, nxrasts, nyrasts, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *itype, *iord, *imgdata, *imgw, *imgh;
int *maxval, *minthr, *maxthr, *minw, *minh, *ixopix, *iyopix;
int *nxp, *nyp, *ibg, *iovly, *ifull, *imgfull, *ifd;
int *nxrasts, *nyrasts, *ierr;
{
   *ierr = xdl_film_image (*ivh, *ivhparent, *ix, *iy, *icset, *itype, *iord,
                          (char *) imgdata, *imgw, *imgh, *maxval,
                          *minthr, *maxthr, *minw, *minh, *ixopix, *iyopix, 
                          *nxp, *nyp, *ibg, *iovly, *ifull, (char *) imgfull,
                          *ifd, *nxrasts, *nyrasts);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_film_image_rps      **
*********************************

Purpose: Repaint the active strip

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_film_image * gd;     /* Pointer to the global data area */

   gd = (XDl_film_image *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->input_message, 0);
   }
}
/*
***************************************
**     xdl_film_image_postscript     **
***************************************

Purpose: Callback for Postscript button push

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_postscript (iv, ibutton)

int iv;
int ibutton;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   int vh;                 /* View object handle */
   int xroot, yroot;       /* Root position of film image window */
   char str[101];          /* String read from dialog box */
   static char *ordstr[8]=   /* Axis order strings */
   {
     "+xf +yf",
     "+xf -yf",
     "-xf +yf",
     "-xf -yf",
     "+yf +xf",
     "+yf -xf",
     "-yf +xf",
     "-yf -xf"
   };
   char str2[26];            /* Temporary string */
   char *title;              /* Main (short) title string */
   char *comments[4];        /* Pointers to comment strings */
   char datstr[30];          /* Date and time string */
   char *p;                  /* Temp character pointer */
   int i,j,k,l;              /* Temp/loop variables */
   int ll;                   /* Character string length */
   int ic;                   /* Comments loop counter */
   int color_ps;             /* Colour postscript =1 yes, =0 no */
   int annotate;             /* Additional annotation flag 1=yes 0=no */
   int addate;               /* Output date/time string 1=yes 0=no */
   int addcomment;           /* Output comments 1=yes 0=no */
   int ncomment;             /* No. of comments */
   int icomment;             /* No. of the longest comment (most chars) */
   int lenmax;               /* Max length of a comment (no. of chars) */
   int inscal;               /* Input user defined scaling 1=yes 0=no */
   int addkey;               /* Output key 1=yes 0=no */
   int blank;                /* Blank string 1=yes 0=no */
   int range;                /* Range between min and max scaling thresholds */
   int ih, iw;               /* Loop counters through compressed image */
   int high;                 /* Black high 1=high 0=low */
   int maxels;               /* Max no. of pixels along an axis of compressed
                                image */
   int xs_off;               /* Overlay  x offset for symbol/vector */
   int ys_off;               /* Overlay  y offset for symbol/vector */
   int xs;                   /* Symbol x position */
   int ys;                   /* Symbol y position */
   int symtyp;               /* Symbol type */
   int symsize;              /* Symbol size */
   int icolr;                /* Current overlay colour number */
   int last_colr;            /* Last overlay colour number */
   int yo;                   /* y origin of freeze box */
   int x1, y1, x2, y2;       /* Ends of overlay vector */
   int max_per_line;         /* Max no. of pixels to o/p per line of hex data */

   float xscal, yscal;       /* x, y postscript scaling factors for image */
   float xpcen, ypcen;       /* Page position for centre of image (mm) */
   float ytitl;              /* Page 'y' position for short title (mm) */
   float ydate;              /* Page 'y' position for date/time (mm) */
   float xorg, yorg;         /* Page origin position for image (points) */
   float xlen, ylen;         /* Image size on page (points) */
   float x1key, x2key, x3key;/* Page 'x' position for parts of key (mm) */
   float ykey, dykey;        /* Page 'y' position & lines offset for key (mm) */
   float ycomm, dycomm;      /* Page 'y' position & lines offset for comments
                                (mm) */
   float mmscal;             /* Scale factor mm to points */
   float xwidth;             /* User requested width of image plot (cms) */
   float xx;                 /* Temp floating point value */
   float r,g,b;              /* Red, green blue values fo overlay colour */

   unsigned int ipix;        /* Pixel value (intensity) */
   unsigned char pixval;     /* Pixel value for display (0-255) */
   unsigned char *bytedata;  /* Pointer to film image data */
   unsigned short *i2data;   /* Pointer to image plate 'i2' data */
   int *int_data;            /* Pointer to signed integer data */
   unsigned char *bgdata;    /* Pointer to film background data */
   unsigned short *i2_bgdata;/* Pointer to background data for i2 data */
   int *int_bgdata;          /* Pointer to background data for int data */
   unsigned char rpix[64];   /* Red pixel values 0-255 for colour image */
   unsigned char gpix[64];   /* Green pixel values 0-255 for colour image */
   unsigned char bpix[64];   /* Blue pixel values 0-255 for colour image */

   time_t tm;                /* Structure for timer routines */
   FILE *fpout;              /* FILE pointer for output postscript file */
   XColor colr_cell;         /* Temp XColor structure */
   XDl_film_image_symbol *smb; /* Temp pointer to symbol structure */
   XDl_film_image_vect *vec; /* Temp pointer to vector structure */
   XDl_film_image_freeze *fz;/* Temp pointer to freeze structure */

   /* Initialisations */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   vh = XDL_view_objects[iv]->vh;
   high = 1;
   if (gd->color_opt==2||gd->color_opt==5) high = 0;
   mmscal = 2.8346;
   xpcen = mmscal*105;
   ypcen = mmscal*175;
   ytitl = mmscal*75;
   ydate = mmscal*270;
   x1key = mmscal*8;
   x2key = mmscal*20;
   x3key = mmscal*61;
   ykey = mmscal*35;
   dykey = mmscal*5;
   ycomm = mmscal*60;
   dycomm = mmscal*5;
   addate = 0;
   addkey = 0;
   addcomment = 0;
   color_ps = 0;
   inscal = 0;
   for (i=0;i<4;++i) comments[i] = 0;
   title = 0;

   /* Get position for dialog box */

   xdl_view_object_rootxy (vh, 0, 0, &xroot, &yroot);

   /* Allocate strings memory */

   title = (char *) malloc(81);
   if (title==0)
   {
      xdl_popup_notice (xroot, yroot, 
                        "**Unable to allocate required memory for title**",
                        0, 0, 0, "Continue", 0, 0, 0, 2, 0);
      return;
   }
   for (i=0;i<4;++i)
   {
      comments[i] = (char *) malloc(81);
      if (comments[i]==0)
      {
         xdl_popup_notice (xroot, yroot, 
                          "**Unable to allocate required memory for comments**",
                           0, 0, 0, "Continue", 0, 0, 0, 2, 0);
         goto psend;
      }
   }

   /* Get name of output postscript file */

   psfile:
   i = xdl_popup_dialog (xroot, yroot, "Postscript file name: ", 0,
                     75, str, 80, 2, 1, 0);
   if (i<=0) goto psend;
   fpout = fopen (str,"w");
   if (fpout==0)
   {
      retry:
      i = xdl_popup_dialog (xroot, yroot, 
                        "**Cannot open file** Enter new file name [y]: ",
                        0, 75, str, 80, 2, 1, 0);
      if (i<0) goto psend;
      if (i==0) goto psfile;
      if (i>0)
      {
         j = sscanf(str,"%s",str2);
         if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y')
            goto retry;
         if (str2[0]=='y'||str2[0]=='Y') goto psfile;
      }
      goto psend;
   }

   /* Colour or black and white postscript */

   labcolr:
   str2[0] = 0;
   color_ps = 0;
   i = xdl_popup_dialog (xroot, yroot, 
                     "Colour postscript [n]: ", 0,
                     75, str, 20, 2, 1, 0);
   if (i<0) goto psend;
   if (i>0)
   {
      j = sscanf(str,"%s",str2);
      if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y') goto labcolr;
      if (str2[0]=='y'||str2[0]=='Y') color_ps = 1;
   }

   /* Scaling of image */

   labscal:
   str2[0] = 0;
   inscal = 0;
   i = xdl_popup_dialog (xroot, yroot, 
                     "Automatic (a) or user defined (u) scaling [a]: ", 0,
                     75, str, 20, 2, 1, 0);
   if (i<0) goto psend;
   if (i>0)
   {
      j = sscanf(str,"%s",str2);
      if (str2[0]!='a'&&str2[0]!='A'&&str2[0]!='u'&&str2[0]!='U') goto labscal;
      if (str2[0]=='u'||str2[0]=='U') inscal = 1;
   }
   if (inscal)
   {
      labsc2:
      xwidth = 15.0;
      i = xdl_popup_dialog (xroot, yroot, 
                "Required width (xf) in cms (xf,yf<=18.0 cms) [15.0]: ", 0,
                 75, str, 20, 2, 1, 0);
      if (i<0) goto psend;
      i = sscanf(str,"%f",&xx);
      if (i!=-1&&i!=1) goto labsc2;
      if (i>0) xwidth = xx;
      if (xwidth<=0.0) goto labsc2;
   }

   /* Annotation requirements*/

   i = xdl_popup_dialog (xroot, yroot, 
                 "Short title: ", 0,
                  75, title, 80, 2, 1, 0);
   if (i<0) goto psend;

   labannot:
   str2[0] = 0;
   annotate = 1;
   i = xdl_popup_dialog (xroot, yroot, 
                     "Add further annotation [y]: ", 0,
                     75, str, 20, 2, 1, 0);
   if (i<0) goto psend;
   if (i>0)
   {
      j = sscanf(str,"%s",str2);
      if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y') goto labannot;
      if (str2[0]=='n'||str2[0]=='N') annotate = 0;
   }

   if (annotate)
   {
      labdate:
      str2[0] = 0;
      addate = 1;
      i = xdl_popup_dialog (xroot, yroot, 
                        "Add heading with date [y]: ", 0,
                        75, str, 20, 2, 1, 0);
      if (i<0) goto psend;
      if (i>0)
      {
         j = sscanf(str,"%s",str2);
         if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y') 
             goto labdate;
         if (str2[0]=='n'||str2[0]=='N') addate = 0;
      }
      if (addate)
      {
         time(&tm);
         strcpy(datstr,ctime(&tm));
         datstr[24] = 0;
      }

      labkey:
      str2[0] = 0;
      addkey = 1;
      i = xdl_popup_dialog (xroot, yroot, 
                        "Add key [y]: ", 0,
                        75, str, 20, 2, 1, 0);
      if (i<0) goto psend;
      if (i>0)
      {
         j = sscanf(str,"%s",str2);
         if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y')
             goto labkey;
         if (str2[0]=='n'||str2[0]=='N') addkey = 0;
      }
      labcomment:
      str2[0] = 0;
      addcomment = 1;
      i = xdl_popup_dialog (xroot, yroot, 
                        "Add comments [y]: ", 0,
                        75, str, 20, 2, 1, 0);
      if (i<0) goto psend;
      if (i>0)
      {
         j = sscanf(str,"%s",str2);
         if (str2[0]!='n'&&str2[0]!='N'&&str2[0]!='y'&&str2[0]!='Y')
             goto labcomment;
         if (str2[0]=='n'||str2[0]=='N') addcomment = 0;
      }
      if (addcomment)
      {
         ncomment = 0;
         lenmax = 0;
         for (ic=0;ic<4;++ic)
         {
            i = xdl_popup_dialog (xroot, yroot, 
                              ": ", 0,
                              75, comments[ic], 80, 2, 1, 0);
            if (i<0) goto psend;
            blank = 1;
            ll = strlen(comments[ic]);
            if (ll>0)
            {
               p = comments[ic];
               for (l=0;l<ll;++l)
               {
                  if (p[l]!=' ')
                  {
                     blank = 0;
                     break;
                  }
               }
            }
            if (blank)
            {
               break;
            }
            else
            {
               ncomment = ic + 1;
               if (ll>lenmax)
               {
                  icomment = ncomment;
                  lenmax = ll;
               }
            }
         }
      }
   }

   /* Prepare postscript file */
   /* ======================= */

   xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);

   /* Initialisation */

   fprintf(fpout,"%%!\n");
   fprintf(fpout,"newpath\n");

   /* Short title */

   fprintf(fpout,"/Helvetica findfont 20 scalefont setfont\n");
   fprintf(fpout,"%.2f %.2f moveto\n",xpcen,ytitl);
   fprintf(fpout,"(%s)\n",title);
   fprintf(fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");

   /* Other annotation */

   if (addate)  /* Date/time string */
   {
      fprintf(fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(fpout,"%.2f %.2f moveto\n",xpcen,ydate);
      fprintf(fpout,"(Image Plot: %s)\n",datstr);
      fprintf(fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");
   }

   if (addkey) /* Key */
   {
      fprintf(fpout,"/Helvetica findfont 15 scalefont setfont\n");
      fprintf(fpout,"%.2f %.2f moveto\n",x1key,ykey);
      fprintf(fpout,"(Key) show\n");
      fprintf(fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(fpout,"(Image section details) show\n");
      fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
      fprintf(fpout,
              "(: xf %d to %d of %d,  yf %d to %d of %d) show\n",
              gd->xo_pix,gd->xo_pix+((gd->nxp)*(gd->img_width))-1,gd->xrasts,
              gd->yo_pix,gd->yo_pix+((gd->nyp)*(gd->img_height))-1,gd->yrasts);
      ykey -= dykey;
      fprintf(fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(fpout,"(Image orientation) show\n");
      fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
      fprintf(fpout,"(: xf to the right, yf up) show\n");
      ykey -= dykey;
      fprintf(fpout,"%.2f %.2f moveto\n",x2key,ykey);
      if (gd->swap==0)
      {
         fprintf(fpout,"(Data order in file) show\n");
         fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(fpout,"(: %s) show\n",ordstr[gd->order-1]);
      }
      else
      {
         fprintf(fpout,"(Data order in file) show\n");
         fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(fpout,"(: %s, byte swapped) show\n",ordstr[gd->order-1]);
      }
      ykey -= dykey;
      fprintf(fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(fpout,"(Maximum pixel value) show\n");
      fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
      fprintf(fpout,"(: %d) show\n",gd->max_val);
      ykey -= dykey;
      fprintf(fpout,"%.2f %.2f moveto\n",x2key,ykey);
      if(high)
      {
         fprintf(fpout,"(Scaling thresholds) show\n");
         fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(fpout,"(: minimum = %d, maximum = %d, black is high) show\n",
                 gd->min_thresh, gd->max_thresh);
      }
      else
      {
         fprintf(fpout,"(Scaling thresholds) show\n");
         fprintf(fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(fpout,"(: minimum = %d, maximum = %d, black is low) show\n",
                 gd->min_thresh, gd->max_thresh);
      }
   }
   
   if (addcomment&&ncomment>0) /* Comments */
   {
      fprintf(fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(fpout,"/xoff (%s) stringwidth pop 2 div neg def\n",
              comments[icomment-1]);
      if (ncomment>3) ycomm += dycomm;
      for (i=0;i<ncomment;++i)
      {
         fprintf(fpout,"%.2f %.2f moveto\n",xpcen,ycomm);
         fprintf(fpout,"xoff 0 rmoveto\n");      
         fprintf(fpout,"(%s) show\n",comments[i]);
         ycomm -= dycomm;
      }
   }
   fprintf(fpout,"stroke\n");

   /* Get image size and draw box */

   maxels = gd->img_width;
   if (gd->img_height>maxels) maxels = gd->img_height;
   if (inscal)
   {
      xscal = 10.0*xwidth*mmscal/(float) gd->img_width;
   }
   else
   {
      xscal = 432.0/(float) maxels;
   }
   yscal = xscal;
   xlen = xscal*(gd->img_width);
   ylen = yscal*(gd->img_height);
   xorg = xpcen - xscal*(gd->img_width/2);
   yorg = ypcen - yscal*(gd->img_height/2);
   fprintf(fpout,"%.2f %.2f moveto\n",xorg,yorg);
   fprintf(fpout,"%.2f 0 rlineto\n",xlen);
   fprintf(fpout,"0 %.2f rlineto\n",ylen);
   fprintf(fpout,"%.2f 0 rlineto\n",-xlen);
   fprintf(fpout,"0 %.2f rlineto\n",-ylen);
   fprintf(fpout,"stroke\n");
   fprintf(fpout,"%.2f %.2f moveto\n",xorg,yorg);
   fprintf(fpout,"%.2f 0 rlineto\n",xlen);
   fprintf(fpout,"0 %.2f rlineto\n",ylen);
   fprintf(fpout,"%.2f 0 rlineto\n",-xlen);
   fprintf(fpout,"0 %.2f rlineto\n",-ylen);
   fprintf(fpout,"closepath clip\n");

   /* Reset position */

   fprintf(fpout,"0 0 moveto\n");
   fprintf(fpout,"save\n");
   fprintf(fpout,"%.2f %.2f translate\n",xorg,yorg);
   fprintf(fpout,"gsave\n");

   /* Output the image */

   fprintf(fpout,"%.2f %.2f scale\n",xlen,ylen);

   if (color_ps)
   {
      for (i=0;i<64;++i)
      {
         colr_cell.pixel = gd->graypixels[i];
         colr_cell.flags = DoRed|DoGreen|DoBlue;
         XQueryColor (XDL_display, XDL_colormap, &colr_cell);
         rpix[i] = colr_cell.red/256;
         gpix[i] = colr_cell.green/256;
         bpix[i] = colr_cell.blue/256;
      }
   }

   if (color_ps)
   {
      fprintf(fpout,"/picstr %d string def\n",3*gd->img_width); 
      fprintf(fpout,"%d %d 8\n",gd->img_width,gd->img_height);
      fprintf(fpout,"[ %d 0 0 %d 0 %d ]\n",gd->img_width,-gd->img_height,
              gd->img_height);
      fprintf(fpout,"{ currentfile picstr readhexstring pop }\n");
      fprintf(fpout,"false 3 colorimage\n");
      max_per_line = 12;
   }
   else
   {
      fprintf(fpout,"/picstr %d string def\n",gd->img_width); 
      fprintf(fpout,"%d %d 8\n",gd->img_width,gd->img_height);
      fprintf(fpout,"[ %d 0 0 %d 0 %d ]\n",gd->img_width,-gd->img_height,
              gd->img_height);
      fprintf(fpout,"{ currentfile picstr readhexstring pop }\n");
      fprintf(fpout,"image\n");
      max_per_line = 40;
   }

   if (gd->type==1)
   {
      bytedata = (unsigned char *) gd->imgdata;
      bgdata = (unsigned char *) gd->bg_data;
   }
   else if (gd->type==2)
   {
      i2data = (unsigned short *) gd->imgdata;
      i2_bgdata = (unsigned short *) gd->bg_data;
   }
   else
   {
      int_data = (int *) gd->imgdata;
      int_bgdata = (int *) gd->bg_data;
   }
  
   range = gd->max_thresh - gd->min_thresh;

   i = 0;
   for (ih=0;ih<gd->img_height;++ih)
   {
      j = 0;
      for (iw=0;iw<gd->img_width;++iw)
      {
         if (gd->type==1)
         {
            if (gd->bg_opt==0)
            {
               ipix = (unsigned int) bytedata[i];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (unsigned int) bytedata[i];
                        ipix -= (unsigned int) bgdata[i];
#if 0
                        if (ipix<0) ipix = 0; /* huh?  ipix unsigned */
#endif
                        break;
                     case 2:
                        ipix = (unsigned int) bgdata[i];
                        break;
                  }
               }
               else
               {
                  ipix = (unsigned int) bytedata[i];
               }
            }
         }
         else if (gd->type==2)
         {
            if (gd->bg_opt==0)
            {
               ipix = (unsigned int) i2data[i];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (unsigned int) i2data[i];
                        ipix -= (unsigned int) i2_bgdata[i];
#if 0
                        if (ipix<0) ipix = 0; /* huh?  ipix unsigned */
#endif
                        break;
                     case 2:
                        ipix = (unsigned int) i2_bgdata[i];
                        break;
                  }
               }
               else
               {
                  ipix = (unsigned int) i2data[i];
               }
            }
         }
         else
         {
            if (gd->bg_opt==0)
            {
               ipix = int_data[i];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = int_data[i];
                        ipix -= int_bgdata[i];
#if 0
                        if (ipix<0) ipix = 0; /* huh?  ipix unsigned */
#endif
                        break;
                     case 2:
                        ipix = int_bgdata[i];
                        break;
                  }
               }
               else
               {
                  ipix = int_data[i];
               }
            }
         }
         if (color_ps)
         {
            if (ipix<=gd->min_thresh)
            {
               pixval = 0;
            }
            else if (ipix>=gd->max_thresh)
            {
               pixval = 63;
            }
            else
            {
               pixval = 63*(ipix-gd->min_thresh)/range;
            }
            fprintf(fpout,"%02X%02X%02X",rpix[pixval],gpix[pixval],
                    bpix[pixval]);
         }
         else
         {
            if (ipix<=gd->min_thresh)
            {
               pixval = 0;
            }
            else if (ipix>=gd->max_thresh)
            {
               pixval = 255;
            }
            else
            {
               pixval = 255*(ipix-gd->min_thresh)/range;
            }
            if (high) pixval = 255 - pixval;
            fprintf(fpout,"%02X",pixval);
         }
         j++;
         if (j==max_per_line) {fprintf(fpout,"\n"); j=0;}
         i++; 
      }
      if (gd->img_width%max_per_line!=0) fprintf(fpout,"\n");
   }

   /* Overlays if needed */

   fprintf(fpout,"grestore\n");
   fprintf(fpout,"/M {moveto} def /L {lineto} def /S {stroke} def\n");
   if (color_ps)
   {
      for (i=0;i<7;++i)
      {
         colr_cell.pixel = gd->ovly_pixels[i];
         colr_cell.flags = DoRed|DoGreen|DoBlue;
         XQueryColor (XDL_display, XDL_colormap, &colr_cell);
         r = colr_cell.red/65535.0;
         g = colr_cell.green/65535.0;
         b = colr_cell.blue/65535.0;
         fprintf(fpout,"/color%d {%.4f %.4f %.4f setrgbcolor} def\n",
                 i+1,r,g,b);
      }
   }
   else
   {
      for (i=0;i<7;++i)
      {
         if (high)
         {
            fprintf(fpout,"/color%d {0.0 0.0 0.0 setrgbcolor} def\n",i+1);
         }
         else
         {
            fprintf(fpout,"/color%d {0.0 0.0 0.0 setrgbcolor} def\n",i+1);
         }
      }
   }
   /* Output symbols if needed */

   last_colr = 1;
   fprintf(fpout,"color1\n");
   fprintf(fpout,"%.5f %.5f scale\n",xscal,yscal);
   fprintf(fpout,"0.5 0.5 translate\n"); /*Offset by half a pixel*/
   fprintf(fpout,"1 setlinewidth\n");
   if (gd->ovly1>0||gd->ovly2>0)
   {
      if (gd->num_symb>0)
      {
         /*Define symbol drawing routines*/
         fprintf(fpout,"/P { /ys exch def /xs exch def\n");
         fprintf(fpout,"xs 0.5 sub ys M xs 0.5 add L\n");
         fprintf(fpout,"S } def\n");
         fprintf(fpout,"/VC { /hs exch def /ys exch def /xs exch def\n");
         fprintf(fpout,"xs hs sub ys M xs hs add ys L\n");
         fprintf(fpout,"xs ys hs sub M xs ys hs add L\n");
         fprintf(fpout,"S } def\n");
         fprintf(fpout,"/CX { /hs exch def /ys exch def /xs exch def\n");
         fprintf(fpout,"xs hs sub ys hs sub M\n");
         fprintf(fpout,"xs hs add ys hs add L\n");
         fprintf(fpout,"xs hs add ys hs sub M\n");
         fprintf(fpout,"xs hs sub ys hs add L\n");
         fprintf(fpout,"S } def\n");
         fprintf(fpout,"/SQ { /hs exch def /ys exch def /xs exch def\n");
         fprintf(fpout,"xs hs sub ys hs sub M\n");
         fprintf(fpout,"xs hs sub ys hs add L\n");
         fprintf(fpout,"xs hs add ys hs add L\n");
         fprintf(fpout,"xs hs add ys hs sub L\n");
         fprintf(fpout,"xs hs sub ys hs sub L\n");
         fprintf(fpout,"stroke } def\n");

         /*Draw symbols*/

         for (i=0;i<gd->num_symb;++i)
         {
            smb = &(gd->symbols[i]);
            if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            icolr = smb->color_flags&0XF;
            if (icolr!=last_colr)
            {
               fprintf(fpout,"color%d\n",icolr);
               last_colr = icolr;
            }
            xs = smb->x_disp_offset + xs_off;
            ys = smb->y_disp_offset + ys_off;
            ys = gd->img_height - 1 - ys;
            symtyp = (smb->symb) / 10;
            symsize = (smb->symb) % 10;
            if (symsize==0)
            {
               fprintf(fpout,"%d %d P\n",xs,ys);
            }
            else
            {
               switch (symtyp)
               {
                  case 0: /*Vertical cross*/
                     fprintf(fpout,"%d %d %d VC\n",xs,ys,symsize);
                     break;
                  case 1: /*Cross*/
                     fprintf(fpout,"%d %d %d CX\n",xs,ys,symsize);
                     break;
                  case 2: /*Square*/
                     fprintf(fpout,"%d %d %d SQ\n",xs,ys,symsize);
                     break;
               }
            }
         }
      }

   /* Output vectors if needed */

      if (gd->num_vect>0)
      {
         for (i=0;i<gd->num_vect;++i)
         {
            vec = &(gd->vects[i]);
            if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            icolr = vec->color_flags&0XF;
            if (icolr!=last_colr)
            {
               fprintf(fpout,"color%d\n",icolr);
               last_colr = icolr;
            }
            x1 = vec->x1_disp_offset + xs_off;
            y1 = gd->img_height - 1 - vec->y1_disp_offset - ys_off;
            x2 = vec->x2_disp_offset + xs_off;
            y2 = gd->img_height - 1 - vec->y2_disp_offset - ys_off;
            fprintf(fpout,"%d %d M %d %d L S\n",x1,y1,x2,y2);
         }
      }
   }

   if (gd->magnify_on==0) 
   {
      fz = &(gd->freeze);
      fprintf(fpout,"color7\n");
      yo = gd->img_height - 1 - fz->yo;
      fprintf(fpout,"%d %d M\n",fz->xo, yo);
      fprintf(fpout,"%d %d L\n",fz->xo+fz->width-1, yo);
      fprintf(fpout,"%d %d L\n",fz->xo+fz->width-1, yo-fz->height+1);
      fprintf(fpout,"%d %d L\n",fz->xo, yo-fz->height+1);
      fprintf(fpout,"%d %d L S\n",fz->xo, yo);
   }


   /* Complete output */

   fprintf(fpout,"restore\n");
   fprintf(fpout,"showpage\n");
   fclose (fpout);

   xdl_panel_label_set (gd->cursor_position," ",0,0);
 
   psend:
   if (title!=0) free(title);
   for(i=0;i<4;++i) if (comments[i]!=0) free(comments[i]);
   return;
}

/*
************************************
**     xdl_film_image_on_off      **
************************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_on_off (iv, i)

int iv;      /* Offset in XDL_view_objects array */
int i;       /* Flag =1 on, =0 off */

{
   XDl_film_image *gd;      /* Pointer to global data area */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
   }
   return;
}

/*
*************************************
**     xdl_film_image_tidy_up      **
*************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_tidy_up (global)

char *global;

{
   XDl_film_image *gd;     /* Pointer to global data area */

   gd = (XDl_film_image *) global;
   free (gd->img);
   if (gd->read_image!=NULL) free ((char *) gd->read_image);
   if (gd->read_buf!=NULL) free ((char *) gd->read_buf);
   if (gd->mag_data!=NULL) free ((char *) gd->mag_data);
   if (gd->monodata!=NULL) free ((char *) gd->monodata);
   if (gd->nxp_to_mag!=NULL) free ((char *) gd->nxp_to_mag);
   if (gd->nyp_to_mag!=NULL) free ((char *) gd->nyp_to_mag);
   if (gd->symbols!=NULL) free ((char *) gd->symbols);
   if (gd->vects!=NULL) free ((char *) gd->vects);
   XFreeCursor (XDL_display, gd->cursor);
   XFreeGC (XDL_display, gd->paint_gc);
   XFreePixmap (XDL_display, gd->sel_rect_t_line);
   XFreePixmap (XDL_display, gd->sel_rect_b_line);
   XFreePixmap (XDL_display, gd->sel_rect_l_line);
   XFreePixmap (XDL_display, gd->sel_rect_r_line);
   return;
}

/*
*********************************
**     xdl_film_image_rp1      **
*********************************

Purpose: Repaint displayed image area

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_film_image * gd;        /* Pointer to global data area */

   gd = (XDl_film_image *) global;

   xdl_film_image_refresh (gd, 0);

   return;
}

/*
*********************************
**     xdl_film_image_rp2      **
*********************************

Purpose: Repaint the magnifying window on an expose

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_film_image * gd;      /* Pointer to global data area */

   XClearWindow (XDL_display, wid);  
   gd = (XDl_film_image *) global;
   if (gd->xc<0) return;
   xdl_film_image_magw (gd);
   return;
}

/*
*********************************
**     xdl_film_image_rp3      **
*********************************

Purpose: Repaint the control panel canvas surround

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_rp3 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*************************************
**     xdl_film_image_refresh      **
*************************************

Purpose: Refresh displayed image area and, if required, magnifying window

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_refresh (gd, magw)

XDl_film_image *gd;     /* Pointer to global data area */
int magw;               /* =1, refresh magnifying window (if needed)
                           =0, do not */

{
   XDl_film_image_symbol *smb; /* Temp. pointer to symbol structure */
   XDl_film_image_vect *vec;   /* Temp. pointer to vector structure */
   int i;                      /* Loop counter */
   int xs_off;                 /* Overlay symbol x offset */
   int ys_off;                 /* Overlay symbol y offset */

   XPutImage (XDL_display, gd->wdisp, gd->paint_gc, gd->img_image,
              0, 0, 0, 0, gd->img_width, gd->img_height); 

   if (gd->ovly1>0||gd->ovly2>0)
   {
      if (gd->num_symb>0)
      {
         for (i=0;i<gd->num_symb;++i)
         {
            smb = &(gd->symbols[i]);
            if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            xdl_film_image_drawsymb (gd, gd->wdisp,
                                     smb->x_disp_offset + xs_off,
                                     smb->y_disp_offset + ys_off,
                                     smb->symb, (smb->color_flags&0XF));
         }
      }
      if (gd->num_vect>0)
      {
         for (i=0;i<gd->num_vect;++i)
         {
            vec = &(gd->vects[i]);
            if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            xdl_film_image_drawvec (gd, gd->wdisp,
                                     vec->x1_disp_offset + xs_off,
                                     vec->y1_disp_offset + ys_off,
                                     vec->x2_disp_offset + xs_off,
                                     vec->y2_disp_offset + ys_off,
                                     (vec->color_flags&0XF));
         }
      }
   }

   if (gd->magnify_on==0) xdl_film_image_set_freeze (gd);

   if (magw==1&&gd->xc>=0) xdl_film_image_magw (gd);

   return;
}

/*
*********************************
**     xdl_film_image_ep1      **
*********************************

Purpose: Event handling for the film image display canvas

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   int x, y;                  /* Mouse position */
   XDl_film_image *gd;        /* Pointer to global data area */
   XEvent xevent;             /* X Event structure */
   char str[20];              /* Position label string */

   gd = (XDl_film_image *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_film_image_getpix1 (gd, x, y);
            gd->xpix_dn = gd->xpix;
            gd->ypix_dn = gd->ypix;
            gd->sel_rect = 1;
            gd->sel_rect_xo = x;
            gd->sel_rect_yo = y;
            xdl_film_image_set_rect (gd, x, y);
            break;
         }
         else if (event->xbutton.button==Button3)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_film_image_getpix1 (gd, x, y);
            xdl_film_image_intensity (gd);
            break;
         }
         else if (event->xbutton.button!=Button2)
         {
             break;
         }
         else  /*Button 2*/
         {
            if (gd->sel_rect) /* Option to cancel select rectangle if needed */
            {
               xdl_film_image_restore_rect (gd);
               gd->sel_rect = 0;
               break;
            }
            else   /* Normal button2 function; freeze/free magnifying window */
            {
               gd->magnify_on = 1 - gd->magnify_on;
               if (gd->magnify_on==0)
               {
                  xdl_film_image_set_freeze (gd);
                  if (gd->fd<0&&gd->full_image==0) break;
                  gd->read_file = 1;
                  xdl_film_image_magw (gd);
                  break;
               }
               else
               {
                  xdl_film_image_restore_freeze (gd);
               }
            }
         }
      /* Note: Fall through to next case if magnify_on has just been set */

      case MotionNotify:
         x = event->xmotion.x;
         y = event->xmotion.y;
         /* Get later motion notify event if any, and remove any intermediate
            ones in the event queue */
         while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
                MotionNotify, &xevent))
         {
            x = xevent.xmotion.x;
            y = xevent.xmotion.y;
         }
         if (x<0) x = 0;
         if (x>=gd->img_width) x = gd->img_width - 1;
         if (y<0) y = 0;
         if (y>=gd->img_height) y = gd->img_height - 1;
         xdl_film_image_getpix1 (gd, x, y);
         if (gd->sel_rect)
         {
            xdl_film_image_restore_rect (gd);
            xdl_film_image_set_rect (gd, x, y);
         }
         if (gd->magnify_on==0) break;
         gd->xc = x;
         gd->yc = y;
         gd->read_file = 0;
         xdl_film_image_magw (gd);
         break;
      case LeaveNotify:
         gd->xpix = -1;
         gd->ypix = -1;
         xdl_panel_label_set (gd->cursor_position, "Cursor position", 0, 0);
         break;
      case ButtonRelease:
         if (event->xbutton.button==Button1&&gd->sel_rect)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_film_image_getpix1 (gd, x, y);
            gd->xpix_up = gd->xpix;
            gd->ypix_up = gd->ypix;
            xdl_film_image_restore_rect (gd);
            gd->sel_rect = 0;
            if (gd->on_off==1)
            {
               gd->xpix_img = gd->xpix_dn;
               gd->ypix_img = gd->ypix_dn;
               gd->xpix_img_up = gd->xpix_up;
               gd->ypix_img_up = gd->ypix_up;
               XDL_return_data_vh = XDL_view_objects[iv]->vh;
               return;
            }
         }
         if (event->xbutton.button==Button3)
         {
            sprintf (str, "x=%d, y=%d", gd->xpix, gd->ypix);
            xdl_panel_label_set (gd->cursor_position, str, 0, 0);
         }
         break;
   }
   return;
}

/*
*********************************
**     xdl_film_image_ep2      **
*********************************

Purpose: Event handling for the magnifying window

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_ep2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   int x, y;                /* Mouse position */
   int i;                   /* Temp. variable */
   XDl_film_image *gd;      /* Pointer to global data area */
   XEvent xevent;           /* Temp. XEvent structure */
   char str[20];            /* Position label string */

   gd = (XDl_film_image *) global;

   switch (event->type)
   {
      case MotionNotify:
         x = event->xmotion.x;
         y = event->xmotion.y;
         /* Get later motion notify event if any, and remove any intermediate
            ones in the event queue */
         while (XCheckTypedWindowEvent (XDL_display, gd->wmag,
                MotionNotify, &xevent))
         {
            x = xevent.xmotion.x;
            y = xevent.xmotion.y;
         }
         xdl_film_image_getpix2 (gd, x, y);
         break;
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            i = xdl_film_image_getpix2 (gd, x, y);
            if (i>0) return;
            if (gd->on_off==1)
            {
               gd->xpix_img = gd->xpix;
               gd->ypix_img = gd->ypix;
               XDL_return_data_vh = XDL_view_objects[iv]->vh;
               return;
            }
            break;
         }
         else if (event->xbutton.button==Button2)
         {
            if (gd->magnify_on==0) xdl_film_image_restore_freeze(gd);
            gd->magwsize_opt = 3 - gd->magwsize_opt;
            gd->magwsize = gd->magwsizes[gd->magwsize_opt-1];
            gd->mag_image = gd->mag_images[gd->magwsize_opt-1];
            XResizeWindow (XDL_display, gd->wmag, gd->magwsize, gd->magwsize);
            if (gd->magnify_on==0) xdl_film_image_set_freeze (gd);
            xdl_film_image_magw (gd);
            break;
         }
         else if (event->xbutton.button==Button3)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            i = xdl_film_image_getpix2 (gd, x, y);
            if (i>0) return;
            xdl_film_image_intensity (gd);
            break;
         }
         break;
      case LeaveNotify:
         gd->xpix = -1;
         gd->ypix = -1;
         xdl_panel_label_set (gd->cursor_position, "Cursor position", 0, 0);
         break;
      case ButtonRelease:
         if (event->xbutton.button==Button3)
         {
            sprintf (str, "x=%d, y=%d", gd->xpix, gd->ypix);
            xdl_panel_label_set (gd->cursor_position, str, 0, 0);
         }
         break;
   }
   return;
}

/*
***********************************
**     xdl_film_image_getpix1    **
***********************************

Purpose: Get film image pixel position from x,y mouse position in main film
         display area and show value; Sets gd->xpix, gd->ypix.

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_getpix1 (gd, x, y)

XDl_film_image *gd;   /* Pointer to global data area */
int x;                /* Mouse x position in film display area */
int y;                /* Mouse y position in film display area */

{
   int pixpos;               /* Pixel position */
   char str[20];             /* String to hold position label string */
   pixpos = ((gd->nxp)+1)/2;
   gd->xpix = (gd->nxp)*x + pixpos;
   pixpos = ((gd->nyp)+1)/2;
   gd->ypix = (gd->nyp)*y + gd->nyp + 1 - pixpos;
   gd->ypix = (gd->nyp)*(gd->img_height) + 1 - gd->ypix;
   gd->xpix += gd->xo_pix -1;
   gd->ypix += gd->yo_pix -1;
   sprintf (str, "x=%d, y=%d", gd->xpix, gd->ypix);
   xdl_panel_label_set (gd->cursor_position, str, 0, 0);

   return;
}

/*
************************************
**     xdl_film_image_getpix2     **
************************************

Purpose: Get and display film image pixel position from magnifying window
         mouse x, y position. Set gd->xpix, gd->ypix

Author:  John W. Campbell, July 1990

*/

int xdl_film_image_getpix2 (gd, x, y)

XDl_film_image *gd;        /* Pointer to global data area */
int x;                     /* Mouse x position */
int y;                     /* Mouse y position */

{
   int xm, ym;              /* Centre pixel position on magnifying window */
   int xpix, ypix;          /* Intermediate values in pixel position calcn. */
   int dx, dy, delta;       /* Offsets from centre pixel */
   int pix1, pix2;          /* Temp pixel positions */
   float a;                 /* Used in pixel position calcn */
   char str[20];            /* String to hold position label */

   if (gd->xc<0) return 1;
   xm = (gd->magwsize-1)/2 - (gd->mag-1)/2;
   ym = xm;
   dx = x-xm;
   if (dx>=0)
   {
      delta = dx/(gd->mag);
      pix1 = dx%(gd->mag) + 1;
   }else{
      delta = (dx+1)/(gd->mag) - 1;
      pix1 = gd->mag + (dx+1)%(gd->mag);
   }
   a = ((float)pix1 - 0.5) * ((float)(gd->nxp))/((float)(gd->mag));
   pix2 = 1 + (int) a;
   xpix = (gd->nxp)*(gd->xc+delta) + pix2;
   dy = y-ym;
   if (dy>=0)
   {
      delta = dy/(gd->mag);
      pix1 = dy%(gd->mag) + 1;
   }else{
      delta = (dy+1)/(gd->mag) - 1;
      pix1 = gd->mag + (dy+1)%(gd->mag);
   }
   a = ((float)pix1 - 0.5) * ((float)(gd->nxp))/((float)(gd->mag));
   pix2 = 1 + (int) a;
   ypix = (gd->nyp)*(gd->yc+delta) + pix2;
   ypix = (gd->nyp)*(gd->img_height) + 1 - ypix;
   if (xpix<1) return 1;
   if (xpix>(gd->nxp)*(gd->img_width)) return 1;
   if (ypix<1) return 1;
   if (ypix>(gd->nyp)*(gd->img_height)) return 1;
   xpix += gd->xo_pix -1;
   ypix += gd->yo_pix -1;
   gd->xpix = xpix;
   gd->ypix = ypix;
   sprintf (str, "x=%d, y=%d", gd->xpix, gd->ypix);
   xdl_panel_label_set (gd->cursor_position, str, 0, 0);
 
   return 0;
}
/*
*********************************
**     xdl_film_image_cp1      **
*********************************

Purpose: Callback for control panel contrast slider

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_cp1 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_film_image *gd;     /* Pointer to global data area */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->contrast_val=(1023*value)/slider_len + 1;
   xdl_film_image_cp2 (iv,gd->color_opt);
   return;
}

/*
*********************************
**     xdl_film_image_cp2      **
*********************************

Purpose: Callback for control panel color choice menu

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_cp2 (iv, item)

int iv;
int item;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   int i;                  /* Temp/loop variable */
   XColor colr_cell;       /* Temp XColor structure */
   int fac;                /* Scaling factor for random numbers */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->color_opt=item;
   if (XDL_color>0)
   {
      switch(gd->color_opt)
      {
         case 1:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 2:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray (gd->cset, gd->contrast_val, 2);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray (gd->graypixels, gd->contrast_val, 2);
            }
            break;
         case 3:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray_yel (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray_yel (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 4:
            if (XDL_csets_alloc==1)
            {
               xdl_set64color (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64color (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 5:
            if (XDL_csets_alloc==1)
            {
               xdl_set64color (gd->cset, gd->contrast_val, 2);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64color (gd->graypixels, gd->contrast_val, 2);
            }
            break;
         case 6:  /* User defined color map */
            if (XDL_csets_alloc==1)
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
                  colr_cell.red = gd->user_rgb[0][i];
                  colr_cell.green = gd->user_rgb[1][i];
                  colr_cell.blue = gd->user_rgb[2][i];
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XStoreColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.red = gd->user_rgb[0][i];
                  colr_cell.green = gd->user_rgb[1][i];
                  colr_cell.blue = gd->user_rgb[2][i];
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XAllocColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = colr_cell.pixel;
               }
            }
            break;
         case 7:  /* Random Colour */
            fac = 2;
            if (XDL_csets_alloc==1)
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
                  colr_cell.red = fac * xdl_rand16();
                  colr_cell.green = fac * xdl_rand16();
                  colr_cell.blue = fac * xdl_rand16();
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XStoreColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.red = fac * xdl_rand16();
                  colr_cell.green = fac * xdl_rand16();
                  colr_cell.blue = fac * xdl_rand16();
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XAllocColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = colr_cell.pixel;
               }
            }
            break;
      }
   }
   else
   {
      switch (gd->color_opt)
      {
         case 1:
           for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_whitepixel;
           for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_blackpixel;
           break;
         case 2:
           for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_blackpixel;
           for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_whitepixel;
           break;
      }
   }
   if (XDL_csets_alloc==0||XDL_color==0)
   {
       xdl_film_image_setdata (gd, gd->type);
       xdl_film_image_refresh (gd,1);
   }
   return;
}

/*
*********************************
**     xdl_film_image_cp3      **
*********************************

Purpose: Callback for control panel magnification factor menu

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_cp3 (iv, item)

int iv;
int item;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   int i;                  /* Loop counter */
   float fr;               /* mag/nxp or mag/nyp ratios (as floats) */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (XDL_color>0)
   {
      gd->mag = item + 1;
   }
   else
   {
      gd->mono_grscal = 0;
      gd->mag = item + 1;
      if (item>7)
      {
         gd->mono_grscal = 1;
         gd->mag = 3*(item-7);
      }
   }
   fr = (float) gd->mag / (float) gd->nxp;
   for (i=0;i<gd->nxp;++i) gd->nxp_to_mag[i] = (int) ((float) i + 0.5) * fr;
   fr = (float) gd->mag / (float) gd->nyp;
   for (i=0;i<gd->nyp;++i) gd->nyp_to_mag[i] = (int) ((float) i + 0.5) * fr;
   if (gd->xc<0) return;
   xdl_film_image_magw (gd);
   xdl_film_image_restore_freeze (gd);
   if (gd->magnify_on==0) xdl_film_image_set_freeze (gd);
   return;
}

/*
*********************************
**     xdl_film_image_cp4      **
*********************************

Purpose: Callback for control panel maximum threshold numeric text item

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_cp4 (iv, str, len, maxv_len)

int iv;
char *str;
int len;
int maxv_len;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   char valstr[12];        /* Parameter value string */
   int value;              /* Parameter value as an integer */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   sscanf (str, "%d", &value);
   if (value>gd->max_val) value = gd->max_val;
   if (value<=gd->min_thresh)
   {
      gd->max_thresh = gd->min_thresh + 1;
   }
   else
   {
      gd->max_thresh = value;
   }
   sprintf (valstr, "%d", gd->max_thresh);
   xdl_panel_value_set (gd->max_text_item, valstr, 0);
   xdl_film_image_setdata (gd, gd->type);
   xdl_film_image_refresh (gd, 1);
   return;
}

/*
*********************************
**     xdl_film_image_cp5      **
*********************************

Purpose: Callback for control panel minimum threshold numeric text item

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_cp5 (iv, str, len, maxv_len)

int iv;
char * str;
int len;
int maxv_len;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   char valstr[12];        /* Parameter value string */
   int value;              /* Parameter value as an integer */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   sscanf (str, "%d", &value);
   if (value<0) value = 0;
   if (value>=gd->max_thresh)
   {
      gd->min_thresh = gd->max_thresh - 1;
   }
   else
   {
      gd->min_thresh = value;
   }
   sprintf (valstr, "%d", gd->min_thresh);
   xdl_panel_value_set (gd->min_text_item, valstr, 0);
   xdl_film_image_setdata (gd, gd->type);
   xdl_film_image_refresh (gd, 1);
   return;
}

/*
*********************************
**     xdl_film_image_cp6      **
*********************************

Purpose: Callback for control panel overlay type menu

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_cp6 (iv, item)

int iv;
int item;

{
   XDl_film_image *gd;     /* Pointer to global data area */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   switch (item)
   {
      case 1:
         if (gd->ovly_opt==1) return;
         gd->ovly_opt = 1;
         gd->ovly1 = 1;
         gd->ovly2 = 1;
         gd->ovly_offset = 0;
         break;
      case 2:
         if (gd->ovly_opt==2) return;
         gd->ovly_opt = 2;
         gd->ovly1 = 0;
         gd->ovly2 = 0;
         gd->ovly_offset = 0;
         break;
      case 3:
         if (gd->ovly_opt==3) return;
         gd->ovly_opt = 3;
         if (gd->ovly_typ==1)
         {
            gd->ovly1 = 1;
            gd->ovly2 = 1;
            gd->ovly_offset = 1;
         }
         else
         {
            gd->ovly1 = 1;
            gd->ovly2 = 0;
            gd->ovly_offset = 0;
         }
         break;
      case 4:
         if (gd->ovly_opt==4) return;
         gd->ovly_opt = 4;
         gd->ovly1 = 0;
         gd->ovly2 = 1;
         gd->ovly_offset = 0;
         break;
   }
   xdl_film_image_refresh (gd, 1);
   return;
}


/*
*********************************
**     xdl_film_image_cp7      **
*********************************

Purpose: Callback for background type menu

Author:  John W. Campbell, August 1991

*/

void xdl_film_image_cp7 (iv, item)

int iv;
int item;

{
   XDl_film_image *gd;     /* Pointer to global data area */
   int xroot, yroot;       /* Root coordinates for error notice */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   switch (item)
   {
      case 1:
         if (gd->bg_opt==0) return;
         gd->bg_opt = 0;
         break;
      case 2:
         if (gd->bg_opt==1) return;
         if (gd->bg_present==0)
         {
            xdl_view_object_rootxy (XDL_view_objects[iv]->vh,240,30,
                                    &xroot,&yroot);
            xdl_popup_notice (xroot,yroot,"**No background defined**",0,
                              0,-1,"Continue",0,0,-1,2,0);
            xdl_panel_choice_reset (gd->bg_choice, 1);
            gd->bg_opt = 0;
            return;
         }
         gd->bg_opt = 1;
         break;
      case 3:
         if (gd->bg_opt==2) return;
         if (gd->bg_present==0)
         {
            xdl_view_object_rootxy (XDL_view_objects[iv]->vh,240,30,
                                    &xroot,&yroot);
            xdl_popup_notice (xroot,yroot,"**No background defined**",0,
                              0,-1,"Continue",0,0,-1,2,0);
            xdl_panel_choice_reset (gd->bg_choice, 1);
            gd->bg_opt = 0;
            return;
         }
         gd->bg_opt = 2;
         break;
   }
   xdl_film_image_setdata (gd, gd->type);
   xdl_film_image_refresh (gd, 1);
   return;
}
/*
*********************************
**     xdl_film_image_cp8      **
*********************************

Purpose: Callback for control panel monochrome threshold slider

Author:  John W. Campbell, Sept 1991

*/

void xdl_film_image_cp8 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_film_image *gd;     /* Pointer to global data area */

   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->thresh_val=(63*value)/slider_len;
   xdl_film_image_cp2 (iv,gd->color_opt);
   return;
}
/*
******************************
**     xdl_film_image_rds   **
******************************

Purpose: Redraw symbols

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_rds (gd, xs_min, xs_max, ys_min, ys_max, magw, xm, ym)
XDl_film_image *gd;    /* Pointer to the global data area (R)*/
int xs_min;            /* Limits of area to be refilled (max symbol width
                          already allowed for therefore search for symbol
                          centres within this area (R)*/
int xs_max;            /* " */
int ys_min;            /* " */
int ys_max;            /* " */
int magw;              /* =0 drawing in main window, =1 draw in magnifying 
                             window (R)*/
int xm;                /* x centre position in magnifying window (only
                            needed if magw==1) (R)*/
int ym;                /* y centre position in magnifying window (only
                            needed if magw==1) (R)*/

{
   XDl_film_image_symbol *smb;      /* Temp. pointer to symbol structure */
   int x_d, y_d;                    /* x, y pixel offsets of symbol in main
                                       film display area */
   int x_s, y_s;                    /* Symbol position x, y offsets in 
                                       magnifying window */
   int y_start;                     /* Y offset 0 to Y_SEARCH_MAX-1 for symbols
                                       search  start */
   int s_start;                     /* Start position in symbols list for search
                                       */
   int xs_off;                      /* Overlay symbol x offset */
   int ys_off;                      /* Overlay symbol y offset */
   int i;                           /* Temp/loop variable */

   /* Find and draw symbols as needed */

   y_start = ((ys_min)*(Y_SEARCH_MAX-1)) 
             / gd->img_height;
   if (y_start<0) y_start = 0;
   if (y_start >= Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
   s_start = gd->y_search[y_start];
   if (s_start<gd->num_symb)
   {
      for (i=s_start;i<gd->num_symb;++i)
      {
         smb = &(gd->symbols[i]);
         if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
         if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
         xs_off = 0;
         ys_off = 0;
         if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
         {
            xs_off = gd->x_ovly_offset;
            ys_off = gd->y_ovly_offset;
         }
         x_d = smb->x_disp_offset + xs_off;
         y_d = smb->y_disp_offset + ys_off;
         if (x_d<xs_min) continue;
         if (x_d>xs_max) continue;
         if (y_d<ys_min) continue;
         if (y_d>ys_max) break;
         if (magw)
         {
            x_s = xm + (x_d - gd->xc) * gd->mag 
                  + gd->nxp_to_mag[smb->x_nxp_offset];
            y_s = ym + (y_d - gd->yc) * gd->mag 
                  + gd->nyp_to_mag[smb->y_nyp_offset];
            xdl_film_image_drawsymb (gd, gd->wmag, x_s, y_s,
                                     smb->symb, (smb->color_flags&0XF));
         }
         else
         {
            xdl_film_image_drawsymb (gd, gd->wdisp, x_d, y_d,
                                     smb->symb, (smb->color_flags&0XF));
         }
      }
   }
   return;
}
/*
******************************
**     xdl_film_image_rdv   **
******************************

Purpose: Redraw overlayed vectors

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, magw, xm, ym)
XDl_film_image *gd;    /* Pointer to the global data area (R)*/
int xv_min;            /* Limits of area to be refilled wrt main display (R)*/
int xv_max;            /* " */
int yv_min;            /* " */
int yv_max;            /* " */
int magw;              /* =0 drawing in main window, =1 draw in magnifying 
                             window (R)*/
int xm;                /* x centre position in magnifying window (only
                            needed if magw==1) (R)*/
int ym;                /* y centre position in magnifying window (only
                            needed if magw==1) (R)*/

{
   XDl_film_image_vect *vec;        /* Temp. pointer to vectorstructure */
   int x1_d, y1_d, x2_d, y2_d;      /* x, y pixel offsets of vector in main
                                       film display area */
   int x1_s, y1_s, x2_s, y2_s;      /* Vector position x, y offsets in 
                                       magnifying window */
   int xs_off;                      /* Overlay vector x offset */
   int ys_off;                      /* Overlay vector y offset */
   int i;                           /* Temp/loop variable */

   /* Find and draw vectors as needed */

   if (gd->num_vect==0) return;
   for (i=0;i<gd->num_vect;++i)
   {
      vec = &(gd->vects[i]);
      if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
      if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
      {
         xs_off = gd->x_ovly_offset;
         ys_off = gd->y_ovly_offset;
      }
      x1_d = vec->x1_disp_offset + xs_off;
      y1_d = vec->y1_disp_offset + ys_off;
      if (y1_d>yv_max) continue;
      x2_d = vec->x2_disp_offset + xs_off;
      y2_d = vec->y2_disp_offset + ys_off;
      if (y2_d<yv_min) continue;
      if (x1_d<=x2_d)
      {
         if (x1_d>xv_max) continue;
         if (x2_d<xv_min) continue;
      }
      else
      {
         if (x2_d>xv_max) continue;
         if (x1_d<xv_min) continue;
      }
      if (magw)
      {
         x1_s = xm + (x1_d - gd->xc) * gd->mag 
               + gd->nxp_to_mag[vec->x1_nxp_offset];
         y1_s = ym + (y1_d - gd->yc) * gd->mag 
               + gd->nyp_to_mag[vec->y1_nyp_offset];
         x2_s = xm + (x2_d - gd->xc) * gd->mag 
               + gd->nxp_to_mag[vec->x2_nxp_offset];
         y2_s = ym + (y2_d - gd->yc) * gd->mag 
               + gd->nyp_to_mag[vec->y2_nyp_offset];
         xdl_film_image_drawvec (gd, gd->wmag, x1_s, y1_s, x2_s, y2_s,
                                  (vec->color_flags&0XF));
      }
      else
      {
         xdl_film_image_drawvec (gd, gd->wdisp, x1_d, y1_d, x2_d, y2_d,
                                  (vec->color_flags&0XF));
      }
   }
   return;
}

/*
**********************************
**     xdl_film_image_magw      **
**********************************

Purpose: Paint the magnifying window 
         (for a monochrome display a 10 level 3*3 halftone grayscale is
          simulated)

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_magw (gd)

XDl_film_image *gd;      /* Pointer to the global data area */

{
   int x, y;              /* Selected pixel position from main image */
   int i, j, k, l, ix, iy, jx, jy, kx, ky, lx, ly, ii, jj, kk, ll, jjx, jjy;
                          /* Temp/loop variables */
   int ixx, iyy;          /* Loop variables */
   int max_offset;        /* Maximum possible offset from centre of the
                             magnifying window */
   int xm, ym;            /* Centre position in magnifying window */
   int xoff, yoff;        /* x and y offset loop counters */
   unsigned char idat;    /* Pixel from main image */
   unsigned char * img;   /* Main image pixels data */
   unsigned char *bytedata; /* Pointer to film image data */
   unsigned short *i2data;  /* Pointer to image plate 'i2' data */
   int *int_data;           /* Pointer to signe integer data */
   int nxel, nyel;        /* Number of x, y data points */
   int range;             /* Threshold range */
   int nfx, nfy;          /* No. x,y points in file */
   int ncx, ncy;          /* No. x,y pixels to combine into 1 */
   int isx, isy;          /* Pixel position in read in image (1 to nxp or 
                             1 to nyp corresponding to a magnifying window 
                             pixel position 1 to gd->mag */
   int ns, nf;            /* Start and finish of overlapping pixels */
   int x_file_off, y_file_off; /* Offsets into image file */
   long offset;           /* File offset for current record */
   int ioff;              /* Offset into read in image data array */
   int joff;              /* Offset into record buffer */
   int irec;              /* Record number */
   int recl;              /* Record length */
   int xtyp, ytyp;        /* Flag xtyp = 1, 2, 3 if nxp=mag, nxp>mag, nxp<mag
                             and cf for ytyp */
   float x_fr, y_fr, fr1, fr2, rs, rf; /* Used in calculating pixels required
                                          for non-simple cases */
   double rval;           /* Total intensity of pixels to be combined into 1 */
   long ival;             /* Pixel intensity to be o/p */
   unsigned long *pix;    /* Pointer to pixels array */
   int pixval;            /* Pixel value */
   unsigned short *read_i2_image;   /* Pointer to array to hold read in image
                                       data (gd->read_image) for i2 data or
                                       that obtained from full image */
   unsigned short *read_i2_buf;     /* Pointer to file read buffer 
                                       (gd->read_buf) for i2 data */
   unsigned char *read_byte_image;  /* Pointer to array to hold read in image
                                       data (gd->read_image) for film data or
                                       that obtained from full image */
   unsigned char *read_byte_buf;    /* Pointer to file read buffer
                                       (gd->read_buf) for film data */
   int *read_int_image;             /* Pointer to array to hold read in image
                                       data (gd->read_image) for int data or
                                       that obtained from full image */
   int *read_int_buf;               /* Pointer to file read buffer 
                                       (gd->read_buf) for int data */
   unsigned char *stored_byte_buf;  /* Pointer to full film image data */
   unsigned short *stored_i2_buf;   /* Pointer to full image-plate image */
   int *stored_int_buf;             /* Pointer to full integer data image */
   unsigned char *bgdata;           /* Pointer to background (film) data */
   unsigned short *i2_bgdata;       /* Pointer to i2 background data */
   int *int_bgdata;                 /* Pointer to integer background data */
   int ibx, iby;                    /* Offsets into background data image */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       magnifying window */
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       magnifying window */
   int xv_min, xv_max;              /* x limits (in main film display area)
                                       for vectorswhich may appear in the
                                       magnifying window */
   int yv_min, yv_max;              /* y limits (in main film display area)
                                       for vectors which may appear in the
                                       magnifying window */
   unsigned long pixel;             /* Pixel number */
   int kp, kp_save;                 /* Indices into gd->mag_data array */
   int jp, jp_save;                 /* Indices into gd->monodata array */
   int nmono;                       /* No. of elements in a row of data
                                       in gd->monodata array */
   int ipix_val;                    /* Pixel value */
   static unsigned char grscal[] = 
     { 0,0,0,0,0,0,0,0,0,
       0,0,0,0,1,0,0,0,0,
       0,0,0,1,1,0,0,0,0,
       0,0,0,1,1,0,0,1,0,
       0,0,0,1,1,1,0,1,0,
       0,0,1,1,1,1,0,1,0,
       0,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,1,
       1,1,1,1,1,1,1,1,1 };          /* Grayscale emulations 3*3 half-tone */
   unsigned char *grayp;             /* Pointer to grayscale half-tone pixels
                                        for current pixel value */
   int xr;                           /* No. of x rasters - 1 */
   int yr;                           /* No. of y rasters - 1 */
   unsigned char *swab;              /* Pointer for byte swapping */
   unsigned char tempc;              /* Temp character for byte swapping */
   int rec_step;                     /* Step to next record to be read */

   max_offset = (gd->magwsize+gd->mag-1)/(2*gd->mag);
   xm = (gd->magwsize-1)/2 - (gd->mag-1)/2;
   ym = xm;
   x = gd->xc;
   y = gd->yc;
   
   if (gd->read_file==0)
   {
      if (XDL_color>0||gd->mono_grscal==0)
      {
      /* Expand data from stored image (Colour/grayscale display or
         monochrome not using simulated grayscale) */

         img = (unsigned char *) gd->img_image->data;
         if (XDL_pw_depth==8)
         {
            for (yoff= -max_offset;yoff<=max_offset;++yoff)
            {
               iy = y + yoff;
               ii = ym + gd->mag*yoff;
               for (xoff= -max_offset;xoff<=max_offset;++xoff)
               {
                  ix = x + xoff;
                  jj = xm + gd->mag*xoff;
                  if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
                  {
                     idat = XDL_whitepixel;
                  }
                  else
                  {
                     idat = img[iy*(gd->img_width)+ix];
                  }
                  kp_save = ii * gd->magwsize + jj;
                  for(k=0;k<gd->mag;++k)
                  {
                     kp = kp_save;
                     if ((ii+k)<0||(ii+k)>=gd->magwsize) 
                                           {kp_save+=gd->magwsize; continue;}
                     for(l=0;l<gd->mag;++l)
                     {
                        if ((jj+l)<0||(jj+l)>=gd->magwsize) {kp++; continue;}
                        gd->mag_data[kp]=idat;
                        kp++;
                     }
                     kp_save += gd->magwsize;
                  }
               }
            }
         }
         else
         {
            for (yoff= -max_offset;yoff<=max_offset;++yoff)
            {
               iy = y + yoff;
               ii = ym + gd->mag*yoff;
               for (xoff= -max_offset;xoff<=max_offset;++xoff)
               {
                  ix = x + xoff;
                  jj = xm + gd->mag*xoff;
                  if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
                  {
                     pixel = XDL_whitepixel;
                  }
                  else
                  {
                     pixel = XGetPixel(gd->img_image,ix,iy);
                  }
                  for(k=0;k<gd->mag;++k)
                  {
                     if ((ii+k)<0||(ii+k)>=gd->magwsize) continue;
                     for(l=0;l<gd->mag;++l)
                     {
                        if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                        XPutPixel(gd->mag_image,jj+l,ii+k,pixel);
                     }
                  }
               }
            }
         }
      }
      else
      {

         /* Expand data from stored compressed image for monochrome
            display using 3x3 halftone 10 level gray scale */

         range = gd->max_thresh - gd->min_thresh;
         if (gd->type==1)
         {
            bytedata = (unsigned char *) gd->imgdata;
            bgdata = (unsigned char *) gd->bg_data;
         }
         else if (gd->type==2)
         {
            i2data = (unsigned short *) gd->imgdata;
            i2_bgdata = (unsigned short *) gd->bg_data;
         }
         else
         {
            int_data = (int *) gd->imgdata;
            int_bgdata = (int *) gd->bg_data;
         }
         for (yoff= -max_offset,jjy=0;yoff<=max_offset;++yoff,++jjy)
         {
            iy = y + yoff;
            ii = ym + gd->mag*yoff;
            for (xoff= -max_offset,jjx=0;xoff<=max_offset;++xoff,++jjx)
            {
               ix = x + xoff;
               jj = xm + gd->mag*xoff;
               if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
               {
                  ipix_val = 0;
               }
               else
               {
                  if (gd->type==1)
                  {
                     ival = bytedata[iy*gd->img_width+ix];
                  }
                  else if (gd->type==2)
                  {
                     ival = i2data[iy*gd->img_width+ix];
                  }
                  else
                  {
                     ival = int_data[iy*gd->img_width+ix];
                  }
                  if (gd->bg_opt==0)
                  {
                     ipix_val = ival;
                  }
                  else
                  {
                     if (gd->bg_present==1)
                     {
                        switch (gd->bg_opt)
                        {
                           case 1:
                              ipix_val = ival;
                              if (gd->type==1)
                              {
                                 ipix_val -= bgdata[iy*gd->img_width+ix];
                              }
                              else if (gd->type==2)
                              {
                                 ipix_val -= i2_bgdata[iy*gd->img_width+ix];
                              }
                              else
                              {
                                 ipix_val -= int_bgdata[iy*gd->img_width+ix];
                              }
                              if (ipix_val<0) ipix_val = 0;
                              break;
                           case 2:
                              if (gd->type==1)
                              {
                                 ipix_val =  bgdata[iy*gd->img_width+ix];
                              }
                              else if (gd->type==2)
                              {
                                 ipix_val = i2_bgdata[iy*gd->img_width+ix];
                              }
                              else
                              {
                                 ipix_val = int_bgdata[iy*gd->img_width+ix];
                              }
                              break;
                        }
                     }
                     else
                     {
                        ipix_val = ival;
                     }
                  }
               }
               if (ipix_val<=gd->min_thresh)
               {
                  pixval = 0;
               }
               else if (ipix_val>=gd->max_thresh)
               {
                  pixval = 9;
               }
               else
               {
                  pixval = 9*(ipix_val-gd->min_thresh)/range;
               }
               grayp = &grscal[9*pixval];
               for (k=0;k<gd->mag;++k)
               {
                  if ((ii+k)<0||(ii+k)>=gd->magwsize) continue;
                  kk = k%3;
                  for(l=0;l<gd->mag;++l)
                  {
                     if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                     ll = l%3;
                     if (grayp[3*kk+ll]==0)
                     {
                        XPutPixel (gd->mag_image, jj+l, ii+k, XDL_whitepixel);
                     }
                     else
                     {
                        XPutPixel (gd->mag_image, jj+l, ii+k, XDL_blackpixel);
                     }
                  }
               }
            }
         }
      }
   }

   else

   /* Get image data from full image or read from file */

   {
      xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);
      range = gd->max_thresh - gd->min_thresh;
      if (gd->full_image==1)
      {

         /* Get from full image */
         switch (gd->type)
         {
            case 1:
               read_byte_image = (unsigned char *) gd->read_image;
               stored_byte_buf = (unsigned char *) gd->full_imgdata;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y offsets for top left pixel of section in full 
                  image */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               xr = gd->xrasts - 1;
               yr = gd->yrasts - 1;
               xoff = x_file_off;
               ioff = 0;
               for (ix=0;ix<nxel;++ix)
               {
                  if (xoff<0||xoff>=nfx)
                  {
                     for (i=0;i<nyel;++i)
                     {
                        read_byte_image[ioff] = 0;
                        ioff ++;
                     }
                  }
                  else
                  {
                     yoff = y_file_off;
                     for (iy=0;iy<nyel;++iy)
                     {
                        if (yoff<0||yoff>=nfy)
                        {
                           read_byte_image[ioff] = 0;
                        }
                        else
                        {
                           switch (gd->order)
                           {
                              case 1:
                                 joff = xoff*(gd->yrasts) + yoff;
                                 break;
                              case 2:
                                 joff = xoff*(gd->yrasts) + yr - yoff;
                                 break;
                              case 3:
                                 joff = (xr-xoff)*(gd->yrasts) + yoff;
                                 break;
                              case 4:
                                 joff = (xr-xoff)*(gd->yrasts) + yr - yoff;
                                 break;
                              case 5:
                                 joff = yoff*(gd->xrasts) + xoff;
                                 break;
                              case 6:
                                 joff = yoff*(gd->xrasts) + xr - xoff;
                                 break;
                              case 7:
                                 joff = (yr-yoff)*(gd->xrasts) + xoff;
                                 break;
                              case 8:
                                 joff = (yr-yoff)*(gd->xrasts) + xr - xoff;
                                 break;
                           }
                           read_byte_image[ioff] = stored_byte_buf[joff];
                        }
                        ioff++;
                        yoff--;
                     }
                  }
                  xoff++;
               }
               break;

            case 2:
               read_i2_image = (unsigned short *) gd->read_image;
               stored_i2_buf = (unsigned short *) gd->full_imgdata;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y offsets for top left pixel of section in full
                  image */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               xr = gd->xrasts - 1;
               yr = gd->yrasts - 1;
               xoff = x_file_off;
               ioff = 0;
               for (ix=0;ix<nxel;++ix)
               {
                  if (xoff<0||xoff>=nfx)
                  {
                     for (i=0;i<nyel;++i)
                     {
                        read_i2_image[ioff] = 0;
                        ioff ++;
                     }
                  }
                  else
                  {
                     yoff = y_file_off;
                     for (iy=0;iy<nyel;++iy)
                     {
                        if (yoff<0||yoff>=nfy)
                        {
                           read_i2_image[ioff] = 0;
                        }
                        else
                        {
                           switch (gd->order)
                           {
                              case 1:
                                 joff = xoff*(gd->yrasts) + yoff;
                                 break;
                              case 2:
                                 joff = xoff*(gd->yrasts) + yr - yoff;
                                 break;
                              case 3:
                                 joff = (xr-xoff)*(gd->yrasts) + yoff;
                                 break;
                              case 4:
                                 joff = (xr-xoff)*(gd->yrasts) + yr - yoff;
                                 break;
                              case 5:
                                 joff = yoff*(gd->xrasts) + xoff;
                                 break;
                              case 6:
                                 joff = yoff*(gd->xrasts) + xr - xoff;
                                 break;
                              case 7:
                                 joff = (yr-yoff)*(gd->xrasts) + xoff;
                                 break;
                              case 8:
                                 joff = (yr-yoff)*(gd->xrasts) + xr - xoff;
                                 break;
                           }
                           read_i2_image[ioff] = stored_i2_buf[joff];
                           if (gd->swap)
                           {
                              swab = (unsigned char *) &read_i2_image[ioff];
                              tempc = swab[0];
                              swab[0] = swab[1];
                              swab[1] = tempc;
                           }
                        }
                        ioff++;
                        yoff--;
                     }
                  }
                  xoff++;
               }
               break;

            case 3:
               read_int_image = (int *) gd->read_image;
               stored_int_buf = (int *) gd->full_imgdata;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y offsets for top left pixel of section in full
                  image */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               xr = gd->xrasts - 1;
               yr = gd->yrasts - 1;
               xoff = x_file_off;
               ioff = 0;
               for (ix=0;ix<nxel;++ix)
               {
                  if (xoff<0||xoff>=nfx)
                  {
                     for (i=0;i<nyel;++i)
                     {
                        read_int_image[ioff] = 0;
                        ioff ++;
                     }
                  }
                  else
                  {
                     yoff = y_file_off;
                     for (iy=0;iy<nyel;++iy)
                     {
                        if (yoff<0||yoff>=nfy)
                        {
                           read_int_image[ioff] = 0;
                        }
                        else
                        {
                           switch (gd->order)
                           {
                              case 1:
                                 joff = xoff*(gd->yrasts) + yoff;
                                 break;
                              case 2:
                                 joff = xoff*(gd->yrasts) + yr - yoff;
                                 break;
                              case 3:
                                 joff = (xr-xoff)*(gd->yrasts) + yoff;
                                 break;
                              case 4:
                                 joff = (xr-xoff)*(gd->yrasts) + yr - yoff;
                                 break;
                              case 5:
                                 joff = yoff*(gd->xrasts) + xoff;
                                 break;
                              case 6:
                                 joff = yoff*(gd->xrasts) + xr - xoff;
                                 break;
                              case 7:
                                 joff = (yr-yoff)*(gd->xrasts) + xoff;
                                 break;
                              case 8:
                                 joff = (yr-yoff)*(gd->xrasts) + xr - xoff;
                                 break;
                           }
                           read_int_image[ioff] = stored_int_buf[joff];
                           if (gd->swap)
                           {
                              swab = (unsigned char *) &read_int_image[ioff];
                              tempc = swab[0];
                              swab[0] = swab[3];
                              swab[3] = tempc;
                              tempc = swab[1];
                              swab[1] = swab[2];
                              swab[2] = tempc;
                           }
                        }
                        ioff++;
                        yoff--;
                     }
                  }
                  xoff++;
               }
               break;
         }

      }
      else
      {
         
         /* Read from file */
         switch (gd->type)
         {
            case 1:
               read_byte_image = (unsigned char *) gd->read_image;
               read_byte_buf = (unsigned char *) gd->read_buf;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y file offsets for top left pixel of section to be 
                  read in */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               if (gd->order<5)
               {
                  recl = gd->yrasts;
                  rec_step = 1;
                  irec = 1 + x_file_off;
                  if (gd->order==3||gd->order==4)
                  {
                     irec = gd->xrasts - x_file_off;
                     rec_step = -1;
                  }
                  ioff = 0;
                  for (ix=0;ix<nxel;++ix)
                  {
                     if (irec<1||irec>nfx)
                     {
                        for (i=0;i<nyel;++i)
                        {
                           read_byte_image[ioff]=0;
                           ioff++;
                        }
                     }
                     else
                     {
                        lseek (gd->fd, (irec-1)*recl, 0);
                        read (gd->fd, read_byte_buf, recl);
                        for (iy=0,j=y_file_off;iy<nyel;++iy,--j)
                        { 
                           if (j<0||j>=nfy)
                           {
                              read_byte_image[ioff] = 0;
                           }
                           else
                           {
                              if (gd->order==1||gd->order==3)
                              {
                                 read_byte_image[ioff] = read_byte_buf[j];
                              }
                              else
                              {
                                 read_byte_image[ioff] = 
                                       read_byte_buf[gd->yrasts-1-j];
                              }
                           }
                           ioff++;
                        }
                     }
                     irec += rec_step;
                  }
               }
               else
               {
                  recl = gd->xrasts;
                  irec = 1 + y_file_off;
                  rec_step = -1;
                  if (gd->order==7||gd->order==8)
                  {
                     irec = gd->yrasts - y_file_off;
                     rec_step = 1;
                  }
                  for (iy=0;iy<nyel;++iy)
                  {
                     ioff = iy;
                     if (irec<1||irec>nfy)
                     {
                        for (i=0;i<nxel;++i)
                        {
                           read_byte_image[ioff] = 0;
                           ioff += nyel;
                        }
                     }
                     else
                     {
                        lseek (gd->fd,(irec-1)*recl,0);
                        read (gd->fd,read_byte_buf,recl);
                        for (ix=0,j=x_file_off;ix<nxel;++ix,++j)
                        {
                           if (j<0||j>=nfx)
                           {
                              read_byte_image[ioff] = 0;
                              ioff += nyel;
                           }
                           else
                           {
                              if (gd->order==5||gd->order==7)
                              {
                                 read_byte_image[ioff] = read_byte_buf[j];
                              }
                              else
                              {
                                 read_byte_image[ioff] =
                                          read_byte_buf[gd->xrasts-1-j];
                              }
                              ioff += nyel;
                           }
                        }
                     }
                     irec += rec_step;
                  }
               }
               break;
            case 2:
               read_i2_image = (unsigned short *) gd->read_image;
               read_i2_buf = (unsigned short *) gd->read_buf;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y file offsets for top left pixel of section to be 
                  read in */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               if (gd->order<5)
               {
                  recl = 2*(gd->yrasts);
                  rec_step = 1;
                  irec = 1 + x_file_off;
                  if (gd->order==3||gd->order==4)
                  {
                     rec_step = -1;
                     irec = gd->xrasts - x_file_off;
                  }
                  ioff = 0;
                  for (ix=0;ix<nxel;++ix)
                  {
                     if (irec<2||irec>nfx+1)
                     {
                        for (i=0;i<nyel;++i)
                        {
                           read_i2_image[ioff]=0;
                           ioff++;
                        }
                     }
                     else
                     {
                        lseek (gd->fd, (irec-1)*recl, 0);
                        read (gd->fd, read_i2_buf, recl);
                        for (iy=0,j=y_file_off;iy<nyel;++iy,--j)
                        { 
                           if (j<0||j>=nfy)
                           {
                              read_i2_image[ioff] = 0;
                           }
                           else
                           {
                              if (gd->order==1||gd->order==3)
                              {
                                 read_i2_image[ioff] = read_i2_buf[j];
                              }
                              else
                              {
                                 read_i2_image[ioff] =
                                         read_i2_buf[gd->yrasts-1-j];
                              }
                              if (gd->swap)
                              {
                                 swab = (unsigned char *) &read_i2_image[ioff];
                                 tempc = swab[0];
                                 swab[0] = swab[1];
                                 swab[1] = tempc;
                              }
                           }
                           ioff++;
                        }
                     }
                     irec += rec_step;
                  }
               }
               else
               {
                  recl = 2*(gd->xrasts);
                  irec = 1 + y_file_off;
                  rec_step = -1;
                  if (gd->order==7||gd->order==8)
                  {
                     irec = gd->yrasts - y_file_off;
                     rec_step = 1;
                  }
                  for (iy=0;iy<nyel;++iy)
                  {
                     ioff = iy;
                     if (irec<1||irec>nfy)
                     {
                        for (i=0;i<nxel;++i)
                        {
                           read_i2_image[ioff] = 0;
                           ioff += nyel;
                        }
                     }
                     else
                     {
                        lseek (gd->fd,(irec-1)*recl,0);
                        read (gd->fd,read_i2_buf,recl);
                        for (ix=0,j=x_file_off;ix<nxel;++ix,++j)
                        {
                           if (j<0||j>=nfx)
                           {
                              read_i2_image[ioff] = 0;
                              ioff += nyel;
                           }
                           else
                           {
                              if (gd->order==5||gd->order==7)
                              {
                                 read_i2_image[ioff] = read_i2_buf[j];
                              }
                              else
                              {
                                 read_i2_image[ioff] =
                                          read_i2_buf[gd->xrasts-1-j];
                              }
                              if (gd->swap)
                              {
                                 swab = (unsigned char *) &read_i2_image[ioff];
                                 tempc = swab[0];
                                 swab[0] = swab[1];
                                 swab[1] = tempc;
                              }
                              ioff += nyel;
                           }
                        }
                     }
                     irec += rec_step;
                  }
               }
               break;
            case 3:
               read_int_image = (int *) gd->read_image;
               read_int_buf = (int *) gd->read_buf;
               nfx = gd->xrasts;
               nfy = gd->yrasts;
               nxel = (2*max_offset + 1) * gd->nxp;
               nyel = (2*max_offset + 1) * gd->nyp;
               /* Get x, y file offsets for top left pixel of section to be 
                  read in */
               x_file_off = (gd->xc - max_offset) * (gd->nxp) + gd->xo_pix - 1;
               y_file_off = (gd->yc - max_offset) * (gd->nyp);
               y_file_off = ((gd->img_height)*(gd->nyp))-1 - y_file_off 
                              + gd->yo_pix-1;
               if (gd->order<5)
               {
                  recl = 4*(gd->yrasts);
                  rec_step = 1;
                  irec = 1 + x_file_off;
                  if (gd->order==3||gd->order==4)
                  {
                     rec_step = -1;
                     irec = gd->xrasts - x_file_off;
                  }
                  ioff = 0;
                  for (ix=0;ix<nxel;++ix)
                  {
                     if (irec<2||irec>nfx+1)
                     {
                        for (i=0;i<nyel;++i)
                        {
                           read_int_image[ioff]=0;
                           ioff++;
                        }
                     }
                     else
                     {
                        lseek (gd->fd, (irec-1)*recl, 0);
                        read (gd->fd, read_int_buf, recl);
                        for (iy=0,j=y_file_off;iy<nyel;++iy,--j)
                        { 
                           if (j<0||j>=nfy)
                           {
                              read_int_image[ioff] = 0;
                           }
                           else
                           {
                              if (gd->order==1||gd->order==3)
                              {
                                 read_int_image[ioff] = read_int_buf[j];
                              }
                              else
                              {
                                 read_int_image[ioff] =
                                         read_int_buf[gd->yrasts-1-j];
                              }
                              if (gd->swap)
                              {
                                 swab = (unsigned char *)&read_int_image[ioff];
                                 tempc = swab[0];
                                 swab[0] = swab[3];
                                 swab[3] = tempc;
                                 tempc = swab[1];
                                 swab[1] = swab[2];
                                 swab[2] = tempc;
                              }
                           }
                           ioff++;
                        }
                     }
                     irec += rec_step;
                  }
               }
               else
               {
                  recl = 4*(gd->xrasts);
                  irec = 1 + y_file_off;
                  rec_step = -1;
                  if (gd->order==7||gd->order==8)
                  {
                     irec = gd->yrasts - y_file_off;
                     rec_step = 1;
                  }
                  for (iy=0;iy<nyel;++iy)
                  {
                     ioff = iy;
                     if (irec<1||irec>nfy)
                     {
                        for (i=0;i<nxel;++i)
                        {
                           read_int_image[ioff] = 0;
                           ioff += nyel;
                        }
                     }
                     else
                     {
                        lseek (gd->fd,(irec-1)*recl,0);
                        read (gd->fd,read_int_buf,recl);
                        for (ix=0,j=x_file_off;ix<nxel;++ix,++j)
                        {
                           if (j<0||j>=nfx)
                           {
                              read_int_image[ioff] = 0;
                              ioff += nyel;
                           }
                           else
                           {
                              if (gd->order==5||gd->order==7)
                              {
                                 read_int_image[ioff] = read_int_buf[j];
                              }
                              else
                              {
                                 read_int_image[ioff] =
                                          read_int_buf[gd->xrasts-1-j];
                              }
                              if (gd->swap)
                              {
                                 swab = (unsigned char *)&read_int_image[ioff];
                                 tempc = swab[0];
                                 swab[0] = swab[3];
                                 swab[3] = tempc;
                                 tempc = swab[1];
                                 swab[1] = swab[2];
                                 swab[2] = tempc;
                              }
                              ioff += nyel;
                           }
                        }
                     }
                     irec += rec_step;
                  }
               }
               break;
         }
      }

      /* Get case types for x (nxp=mag, nxp>mag, nxp<mag) */
      xtyp = 1;
      ncx = 1;
      x_fr = (float) (gd->nxp) / (float) (gd->mag);
      if (gd->nxp>gd->mag)
      {
         xtyp = 2;
         ncx = gd->nxp/gd->mag;
      }
      if (gd->nxp<gd->mag)
      {
         xtyp = 3;
         ncx = 1;
      }
      /* Get case types for y (nyp=mag, nyp>mag, nyp<mag) */
      ytyp = 1;
      ncy = 1;
      y_fr = (float) (gd->nyp) / (float) (gd->mag);
      if (gd->nyp>gd->mag)
      {
         ytyp = 2;
         ncy = gd->nyp/gd->mag;
      }
      if (gd->nyp<gd->mag)
      {
         ytyp = 3;
         ncy = 1;
      }

      /* Form image data */
      pix = gd->graypixels;
      for (ix=0;ix<gd->mag;++ix)
      {
         switch (xtyp)
         {
            case 1:
               isx = ix;
               break;
            case 2:
               rs = (float) ix*x_fr;
               rf = (float) (ix+1)*x_fr;
               ns = (int) rs;
               nf = (int) rf;
               if ((nf-ns)>ncx)
               {
                  isx = ns + 1;
               }
               else
               {
                  fr1 = (float) (ns+1) - rs;
                  fr2 = rf - (float) (nf);
                  isx = ns;
                  if (fr2>fr1) isx++;
               }
               break;
            case 3:
               isx = (int) ((float)ix + 0.5) * x_fr;
         }
         for (iy=0;iy<gd->mag;++iy)
         {
            switch (ytyp)
            {
               case 1:
                  isy = iy;
                  break;
               case 2:
                  rs = (float) iy*y_fr;
                  rf = (float) (iy+1)*y_fr;
                  ns = (int) rs;
                  nf = (int) rf;
                  if ((nf-ns)>ncy)
                  {
                     isy = ns + 1;
                  }
                  else
                  {
                     fr1 = (float) (ns+1) - rs;
                     fr2 = rf - (float) (nf);
                     isy = ns;
                     if (fr2>fr1) isy++;
                  }
                  break;
               case 3:
                  isy = (int) ((float)iy + 0.5) * y_fr;
            }

            if (XDL_color>0||gd->mono_grscal==0)
            {

               /* Colour/grayscale display options or monochrome not using
                  simulated grayscale */

               if (gd->type==1)
               {
                  bgdata = (unsigned char *) gd->bg_data;
               }
               else if (gd->type==2)
               {
                  i2_bgdata = (unsigned short *) gd->bg_data;
               }
               else
               {
                  int_bgdata = (int *) gd->bg_data;
               }
               for (xoff= -max_offset,kx=0;xoff<=max_offset;
                    ++xoff,kx+=gd->nxp)
               {
                  ibx = gd->xc + xoff;
                  for (yoff= -max_offset,ky=0;yoff<=max_offset;
                       ++yoff,ky+=gd->nyp)
                  {
                     iby = gd->yc + yoff;
                     ioff = (kx+isx)*nyel + ky + isy;
                     rval = 0.0;
                     for (ixx=0;ixx<ncx;++ixx)
                     {
                        for (iyy=0;iyy<ncy;++iyy)
                        {
                           if (gd->type==1)
                           {
                              rval += (float) read_byte_image[ioff+iyy];
                           }
                           else
                           {
                              rval += (float) read_i2_image[ioff+iyy];
                           }
                        }
                        ioff += nyel;
                     }
                     ival = (long) rval/(float)(ncx*ncy);
                     jy = ym + gd->mag*yoff + iy;
                     jx = xm + gd->mag*xoff + ix;
                     if (jy<0||jy>=gd->magwsize||jx<0||jx>=gd->magwsize) 
                         continue;
                     if (gd->bg_opt==0)
                     {
                        ipix_val = ival;
                     }
                     else
                     {
                        if (gd->bg_present==1)
                        {
                           switch (gd->bg_opt)
                           {
                              case 1:
                                 ipix_val = ival;
                                 if (gd->type==1)
                                 {
                                    ipix_val -= bgdata[iby*gd->img_width+ibx];
                                 }
                                 else if (gd->type==2)
                                 {
                                    ipix_val-=i2_bgdata[iby*gd->img_width+ibx];
                                 }
                                 else
                                 {
                                    ipix_val-=int_bgdata[iby*gd->img_width+ibx];
                                 }
                                 if (ipix_val<0) ipix_val = 0;
                                 break;
                              case 2:
                                 if (gd->type==1)
                                 {
                                    ipix_val =  bgdata[iby*gd->img_width+ibx];
                                 }
                                 else if (gd->type==2)
                                 {
                                    ipix_val = i2_bgdata[iby*gd->img_width+ibx];
                                 }
                                 else
                                 {
                                    ipix_val=int_bgdata[iby*gd->img_width+ibx];
                                 }
                                 break;
                           }
                        }
                        else
                        {
                           ipix_val = ival;
                        }
                     }
                     if (ipix_val<=gd->min_thresh)
                     {
                        pixval = 0;
                     }
                     else if (ipix_val>=gd->max_thresh)
                     {
                        pixval = 63;
                     }
                     else
                     {
                        pixval = 63*(ipix_val-gd->min_thresh)/range;
                     }
                     if (XDL_pw_depth==8)
                     {
                        gd->mag_data[jy*gd->magwsize+jx] = pix[pixval];
                     }
                     else
                     {
                        XPutPixel (gd->mag_image, jx, jy, pix[pixval]);
                     }
                  }
               }
            }
            else
            {

               /* Monochrome display option with 3x3 half-tone simulation of 
                  a 10 level gray scale */

               nmono = (2*max_offset + 1) * gd->mag;
               if (gd->type==1) 
               {
                  bgdata = (unsigned char *) gd->bg_data;
               }
               else if (gd->type==2)
               {
                  i2_bgdata = (unsigned short *) gd->bg_data;
               }
               else
               {
                  int_bgdata = (int *) gd->bg_data;
               }
               for (xoff= -max_offset,kx=0,lx=0;xoff<=max_offset;
                    ++xoff,kx+=gd->nxp,lx+=gd->mag)
               {
                  ibx = gd->xc + xoff;
                  for (yoff= -max_offset,ky=0,ly=0;yoff<=max_offset;
                       ++yoff,ky+=gd->nyp,ly+=gd->mag)
                  {
                     iby = gd->yc + yoff;
                     ioff = (kx+isx)*nyel + ky + isy;
                     rval = 0.0;
                     for (ixx=0;ixx<ncx;++ixx)
                     {
                        for (iyy=0;iyy<ncy;++iyy)
                        {
                           if (gd->type==1)
                           {
                              rval += (float) read_byte_image[ioff+iyy];
                           }
                           else if (gd->type==2)
                           {
                              rval += (float) read_i2_image[ioff+iyy];
                           }
                           else
                           {
                              rval += (float) read_int_image[ioff+iyy];
                           }
                        }
                        ioff += nyel;
                     }
                     ival = (long) rval/(float)(ncx*ncy);
                     jjx = lx + ix;
                     jjy = ly + iy;
                     if (gd->bg_opt==0)
                     {
                        ipix_val = ival;
                     }
                     else
                     {
                        if (gd->bg_present==1)
                        {
                           switch (gd->bg_opt)
                           {
                              case 1:
                                 ipix_val = ival;
                                 if (gd->type==1)
                                 {
                                    ipix_val -= bgdata[iby*gd->img_width+ibx];
                                 }
                                 else if (gd->type==2)
                                 {
                                    ipix_val-=i2_bgdata[iby*gd->img_width+ibx];
                                 }
                                 else
                                 {
                                    ipix_val-=int_bgdata[iby*gd->img_width+ibx];
                                 }
                                 if (ipix_val<0) ipix_val = 0;
                                 break;
                              case 2:
                                 if (gd->type==1)
                                 {
                                    ipix_val =  bgdata[iby*gd->img_width+ibx];
                                 }
                                 else if (gd->type==2)
                                 {
                                    ipix_val = i2_bgdata[iby*gd->img_width+ibx];
                                 }
                                 else
                                 {
                                    ipix_val=int_bgdata[iby*gd->img_width+ibx];
                                 }
                              break;
                           }
                        }
                        else
                        {
                           ipix_val = ival;
                        }
                     }
                     gd->monodata[jjy*nmono+jjx] = ipix_val;
                  }
               }

               /* Find average values over 3*3 pixels and set grayscale halftone
                  bits in magnifying window image data */
 
               for (jy=0;jy<nmono;jy+=3)
               {
                  ii = jy + ym - max_offset*gd->mag;
                  for (jx=0;jx<nmono;jx+=3)
                  {
                     jj = jx + xm - max_offset*gd->mag;
                     jp_save = jy*nmono + jx;
                     ipix_val = 0;
                     for (k=0;k<3;++k)
                     {
                        jp = jp_save;
                        for (l=0;l<3;++l)
                        {
                           ipix_val += gd->monodata[jp];
                           jp++;
                        }
                        jp_save += nmono;
                     }
                     ipix_val /= 9;
                     if (ipix_val<=gd->min_thresh)
                     {
                        pixval = 0;
                     }
                     else if (ipix_val>=gd->max_thresh)
                     {
                        pixval = 9;
                     }
                     else
                     {
                        pixval = 9*(ipix_val-gd->min_thresh)/range;
                     }
                     grayp = &grscal[9*pixval];
       
                     for (k=0;k<3;++k)
                     {
                        if ((ii+k<0)||(ii+k)>=gd->magwsize) continue;
                        for (l=0;l<3;++l)
                        {
                           if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                           if (grayp[3*k+l]==0)
                           {
                              XPutPixel (gd->mag_image, jj+l, ii+k, 
                                         XDL_whitepixel);
                           }
                           else
                           {
                              XPutPixel (gd->mag_image, jj+l, ii+k, 
                                         XDL_blackpixel);
                           }
                        }
                     }
                  }
               }
            }
         }
      }
      xdl_panel_label_set (gd->cursor_position," ",0,0);
   }

   /* Display the magnifying window image data */

   XPutImage (XDL_display, gd->wmag, gd->paint_gc,
              gd->mag_image, 0, 0, 0, 0, gd->magwsize, gd->magwsize);

   /* Draw overlay symbols and vectors if required */

   if (gd->ovly1==0&&gd->ovly2==0) return;
   xs_min = gd->xc - max_offset - MAX_SYMB_SIZE;
   xs_max = gd->xc + max_offset + MAX_SYMB_SIZE;
   ys_min = gd->yc - max_offset - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
   ys_max = gd->yc + max_offset + MAX_SYMB_SIZE + abs(gd->y_ovly_offset);
   xdl_film_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 1, xm, ym);
   xv_min = gd->xc - max_offset;
   xv_max = gd->xc + max_offset;
   yv_min = gd->yc - max_offset - abs(gd->y_ovly_offset);
   yv_max = gd->yc + max_offset + abs(gd->y_ovly_offset);
   xdl_film_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, 1, xm, ym);
   
   return;
}

/*
*************************************
**     xdl_film_image_setdata      **
*************************************

Purpose: Set up pixels array from input data

Author:  John W. Campbell, July 1990

*/

void xdl_film_image_setdata (gd, type)

XDl_film_image *gd;       /* Pointer top global data area */
int type;                 /* data type 1=film, 2=image-plate  unsigned (i2),
                                       3=signed integer data */

{
   int ip;                  /* Offset into imgdata/img/bytedata/i2data arrays */
   int pixval;              /* Pixel value */
   long ipix_val;           /* Image pixel value */
   unsigned long * pix;     /* Temporary pointer to current array of pixels */
   int ih, iw;              /* Loop counters through image height/width */
   int range;               /* Pixel max_thresh-min_thresh */
   unsigned char *bytedata; /* Pointer to film image data */
   unsigned short *i2data;  /* Pointer to image plate 'i2' data */
   int *int_data;           /* Pointer to signed integer data */
   unsigned char *bgdata;   /* Pointer to film background data */
   unsigned short *i2_bgdata; /* Pointer to background data for i2 data */
   int *int_bgdata;           /* Pointer to background data for int data */

   pix = gd->graypixels;
   xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);
   switch (type)
   {
      /* Film data */
      case 1:
         bytedata = (unsigned char *) gd->imgdata;
         bgdata = (unsigned char *) gd->bg_data;
         ip = -1;
         range = gd->max_thresh - gd->min_thresh;
         for (ih=0;ih<gd->img_height;++ih)
         {
            for (iw=0;iw<gd->img_width;++iw)
            {
               ip++;
               if (gd->bg_opt==0)
               {
                  ipix_val = (long) bytedata[ip];
               }
               else
               {
                  if (gd->bg_present==1)
                  {
                     switch (gd->bg_opt)
                     {
                        case 1:
                           ipix_val = (long) bytedata[ip];
                           ipix_val -= (long) bgdata[ip];
                           if (ipix_val<0) ipix_val = 0;
                           break;
                        case 2:
                           ipix_val = (long) bgdata[ip];
                           break;
                     }
                  }
                  else
                  {
                     ipix_val = (long) bytedata[ip];
                  }
               }
               if (ipix_val<=gd->min_thresh)
               {
                  pixval = 0;
               }
               else if (ipix_val>=gd->max_thresh)
               {
                  pixval = 63;
               }
               else
               {
                  pixval = 63*(ipix_val-gd->min_thresh)/range;
               }
               if (XDL_pw_depth==8)
               {
                  gd->img[ip] = pix[pixval];
               }
               else
               {
                  XPutPixel (gd->img_image, iw, ih, pix[pixval]);
               }
            }
         }
         break;

      /* Image plate data */
      case 2:
         i2data = (unsigned short *) gd->imgdata;
         i2_bgdata = (unsigned short *) gd->bg_data;
         ip = -1;
         range = gd->max_thresh - gd->min_thresh;
         for (ih=0;ih<gd->img_height;++ih)
         {
            for (iw=0;iw<gd->img_width;++iw)
            {
               ip++;
               if (gd->bg_opt==0)
               {
                  ipix_val = (long) i2data[ip];
               }
               else
               {
                  if (gd->bg_present==1)
                  {
                     switch (gd->bg_opt)
                     {
                        case 1:
                           ipix_val = (long) i2data[ip];
                           ipix_val -= (long) i2_bgdata[ip];
                           if (ipix_val<0) ipix_val = 0;
                           break;
                        case 2:
                           ipix_val = (long) i2_bgdata[ip];
                           break;
                     }
                  }
                  else
                  {
                     ipix_val = (long) i2data[ip];
                  }
               }
               if (ipix_val<=gd->min_thresh)
               {
                  pixval = 0;
               }
               else if (ipix_val>=gd->max_thresh)
               {
                  pixval = 63;
               }
               else
               {
                  pixval = 63*(ipix_val-gd->min_thresh)/range;
               }
               if (XDL_pw_depth==8)
               {
                  gd->img[ip] = pix[pixval];
               }
               else
               {
                  XPutPixel (gd->img_image, iw, ih, pix[pixval]);
               }
            }
         }
         break;

      /* Signed integer data */
      case 3:
         int_data = (int *) gd->imgdata;
         int_bgdata = (int *) gd->bg_data;
         ip = -1;
         range = gd->max_thresh - gd->min_thresh;
         for (ih=0;ih<gd->img_height;++ih)
         {
            for (iw=0;iw<gd->img_width;++iw)
            {
               ip++;
               if (gd->bg_opt==0)
               {
                  ipix_val = (long) int_data[ip];
               }
               else
               {
                  if (gd->bg_present==1)
                  {
                     switch (gd->bg_opt)
                     {
                        case 1:
                           ipix_val = (long) int_data[ip];
                           ipix_val -= (long) int_bgdata[ip];
                           if (ipix_val<0) ipix_val = 0;
                           break;
                        case 2:
                           ipix_val = (long) int_bgdata[ip];
                           break;
                     }
                  }
                  else
                  {
                     ipix_val = (long) int_data[ip];
                  }
               }
               if (ipix_val<=gd->min_thresh)
               {
                  pixval = 0;
               }
               else if (ipix_val>=gd->max_thresh)
               {
                  pixval = 63;
               }
               else
               {
                  pixval = 63*(ipix_val-gd->min_thresh)/range;
               }
               if (XDL_pw_depth==8)
               {
                  gd->img[ip] = pix[pixval];
               }
               else
               {
                  XPutPixel (gd->img_image, iw, ih, pix[pixval]);
               }
            }
         }
         break;
   }
   xdl_panel_label_set (gd->cursor_position," ",0,0);
   return;
}

/*
**************************************
**     xdl_film_image_set_freeze    **
**************************************

Purpose: Draw rectangle on film image display to mark magnifying window when
         frozen; Save parameters of area that needs to be restored to remove
         this rectangle.

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_set_freeze (gd)

XDl_film_image *gd;     /* Pointer to the global data area */

{
   XDl_film_image_freeze *fz;       /* Pointer to freeze details structure */
   int xmin, xmax, ymin, ymax;      /* Limits of magnifying area on display
                                       and limits of area to restore adjusted
                                       for boundary violations */
   int max_offset;                  /* Limits of magnifying window from
                                       centre position (film image display
                                       pixels */

   fz = &(gd->freeze);
   fz->on = 1;
   max_offset = (gd->magwsize+gd->mag-1)/(2*gd->mag);
   xmin = gd->xc - max_offset;
   xmax = gd->xc + max_offset;
   ymin = gd->yc - max_offset;
   ymax = gd->yc + max_offset;
   XSetForeground (XDL_display, gd->paint_gc, gd->freeze_pixel);
   XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                   xmin, ymin, (xmax-xmin), (ymax-ymin));
   if (xmin<0) xmin = 0;
   if (xmax>gd->img_width) xmax = gd->img_width;
   if (ymin<0) ymin = 0;
   if (ymax>gd->img_height) ymax = gd->img_height;
   fz->xo = xmin;
   fz->yo = ymin;
   fz->width = xmax - xmin + 1;
   fz->height = ymax - ymin + 1;
   return;
}

/*
******************************************
**     xdl_film_image_restore_freeze    **
******************************************

Purpose: Restore area in film image area which had frozen magnifying window
         rectangle marked.

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_restore_freeze (gd)

XDl_film_image *gd;     /* Pointer to the global data area */

{
   XDl_film_image_freeze *fz;       /* Pointer to freeze details structure */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       freeze box */
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       freeze box */
   int xv_min, xv_max;              /* x limits (in main film display area)
                                       for vectors which may appear in the
                                       freeze box */
   int yv_min, yv_max;              /* y limits (in main film display area)
                                       for vectors which may appear in the
                                       freeze box */

   fz = &(gd->freeze);
   if (fz->on==0) return;
   fz->on = 0;
   XPutImage (XDL_display, gd->wdisp, gd->paint_gc, gd->img_image,
              fz->xo, fz->yo, fz->xo, fz->yo, fz->width, fz->height);
   if (gd->ovly1==0&&gd->ovly2==0) return;
   if (gd->num_symb==0) return;
   xs_min = fz->xo - MAX_SYMB_SIZE;
   xs_max = fz->xo + fz->width + MAX_SYMB_SIZE - 1;
   ys_min = fz->yo - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
   ys_max = fz->yo + fz->height + MAX_SYMB_SIZE - 1 + abs(gd->y_ovly_offset);
   xdl_film_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);
   xv_min = fz->xo;
   xv_max = fz->xo + fz->width - 1;
   yv_min = fz->yo;
   yv_max = fz->yo + fz->height - 1;
   xdl_film_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, 0, 0, 0);
   return;
}

/*
**************************************
**     xdl_film_image_set_rect      **
**************************************

Purpose: Draw rectangle on film image display to mark area being selected
         with dragged button1 down (first saves rows of pixels which will
         be overwritten)

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_set_rect (gd, x, y)

XDl_film_image *gd;     /* Pointer to the global data area */
int x;                  /* Current x position of pointer */
int y;                  /* Current y position of pointer */
{
   int w;
   int h;

   if (x>=gd->sel_rect_xo)
   {
      gd->sel_rect_x1 = gd->sel_rect_xo;
      gd->sel_rect_x2 = x;
   }
   else
   {
      gd->sel_rect_x1 = x;
      gd->sel_rect_x2 = gd->sel_rect_xo;
   }
   if (y>=gd->sel_rect_yo)
   {
      gd->sel_rect_y1 = gd->sel_rect_yo;
      gd->sel_rect_y2 = y;
   }
   else
   {
      gd->sel_rect_y1 = y;
      gd->sel_rect_y2 = gd->sel_rect_yo;
   }

   /* Save lines of pixels where rectangle will be drawn */

   w = gd->sel_rect_x2 - gd->sel_rect_x1 + 1;
   h = gd->sel_rect_y2 - gd->sel_rect_y1 + 1;
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_t_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y1, w, 1, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_b_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y2, w, 1, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_l_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y1, 1, h, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_r_line, gd->paint_gc,
              gd->sel_rect_x2, gd->sel_rect_y1, 1, h, 0, 0);

   /* Draw rectangle */

   XSetForeground (XDL_display, gd->paint_gc, gd->freeze_pixel);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y1,
             gd->sel_rect_x2, gd->sel_rect_y1);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y2,
             gd->sel_rect_x2, gd->sel_rect_y2);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y1,
             gd->sel_rect_x1, gd->sel_rect_y2);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x2, gd->sel_rect_y1,
             gd->sel_rect_x2, gd->sel_rect_y2);
   return;
}

/*
****************************************
**     xdl_film_image_restore_rect    **
****************************************

Purpose: Restore pixels to remove rectangle drawn via xdl_film_image_set_rect

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_restore_rect (gd)

XDl_film_image *gd;     /* Pointer to the global data area */

{
   int w;
   int h;

   w = gd->sel_rect_x2 - gd->sel_rect_x1 + 1;
   h = gd->sel_rect_y2 - gd->sel_rect_y1 + 1;
   XCopyArea (XDL_display, gd->sel_rect_t_line, gd->wdisp, gd->paint_gc,
              0, 0, w, 1, gd->sel_rect_x1, gd->sel_rect_y1);
   XCopyArea (XDL_display, gd->sel_rect_b_line, gd->wdisp, gd->paint_gc,
              0, 0, w, 1, gd->sel_rect_x1, gd->sel_rect_y2);
   XCopyArea (XDL_display, gd->sel_rect_l_line, gd->wdisp, gd->paint_gc,
              0, 0, 1, h, gd->sel_rect_x1, gd->sel_rect_y1);
   XCopyArea (XDL_display, gd->sel_rect_r_line, gd->wdisp, gd->paint_gc,
              0, 0, 1, h, gd->sel_rect_x2, gd->sel_rect_y1);
   return;
}

/*
***************************************
**     xdl_film_image_intensity      **
***************************************

Purpose: Read intensity value from file at selected position

Author:  John W. Campbell, August 1990

*/

void xdl_film_image_intensity (gd)

XDl_film_image *gd;     /* Pointer to global data area */

{
   int x_file_off;          /* X offset of selected position */
   int y_file_off;          /* Y offset of selected position */
   long offset;             /* Offset in file of required data item */
   int intens;              /* Intensity */
   char str[20];            /* Panel label string for intensity value */
   int i;                   /* Temp. variable */
   unsigned char *read_byte_buf;  /* Pointer to file read buffer
                                     (gd->read_buf) for film data */
   unsigned short *read_i2_buf;   /* Pointer to file read buffer
                                     (gd->read_buf) for 'i2' data */
   int *read_int_buf;             /* Pointer to file read buffer
                                     (gd->read_buf) for int data */
   unsigned char *byte_full_data; /* Pointer to full image film data */
   unsigned short *i2_full_data;  /* Pointer to full image image plate data */
   int *int_full_data;            /* Pointer to full image int data */
   unsigned char *bgdata;         /* Pointer to film background data */
   unsigned short *i2_bgdata;     /* Pointer to background data for i2 data */
   int *int_bgdata;               /* Pointer to background data for int data */
   int x_disp_offset;             /* x position in display/background image */
   int y_disp_offset;             /* y position in display/background image */
   int bgval;                     /* Background value */
   unsigned short i2_val;         /* Element from full image-plate data */
   int int_val;                   /* Element from full integer data */
   unsigned char *bval;           /* Temp pointer for byte swap */
   unsigned char tempc;           /* Temp character for byte swap */

   switch (gd->order)
   {
      case 1:
         offset = (gd->xpix-1)*(gd->yrasts) + gd->ypix - 1;
         break;
      case 2:
         offset = (gd->xpix-1)*(gd->yrasts) + gd->yrasts - gd->ypix;
         break;
      case 3:
         offset = (gd->xrasts-gd->xpix)*(gd->yrasts) + gd->ypix - 1;
         break;
      case 4:
         offset = (gd->xrasts-gd->xpix)*(gd->yrasts) + gd->yrasts - gd->ypix;
         break;
      case 5:
         offset = (gd->ypix-1)*(gd->xrasts) + gd->xpix - 1;
         break;
      case 6:
         offset = (gd->ypix-1)*(gd->xrasts) + gd->xrasts - gd->xpix;
         break;
      case 7:
         offset = (gd->yrasts-gd->ypix)*(gd->xrasts) + gd->xpix - 1;
         break;
      case 8:
         offset = (gd->yrasts-gd->ypix)*(gd->xrasts) + gd->xrasts - gd->xpix;
         break;
   }
   if (gd->full_image==1)
   {
      switch (gd->type)
      {
         case 1:
            byte_full_data = (unsigned char *) gd->full_imgdata;
            intens = (int) byte_full_data[offset];
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               bgdata = (unsigned char *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval = (int) bgdata[y_disp_offset*(gd->img_width)
                                    + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;

            case 2:
            i2_full_data = (unsigned short *) gd->full_imgdata;
            i2_val = i2_full_data[offset];
            if (gd->swap)
            {
               bval = (unsigned char *) &i2_val;
               tempc = bval[0];
               bval[0] = bval[1];
               bval[1] = tempc;
            }
            intens = (int) i2_val;
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               i2_bgdata = (unsigned short *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval = (int) i2_bgdata[y_disp_offset*(gd->img_width)
                                       + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;

            case 3:
            int_full_data = (int *) gd->full_imgdata;
            int_val = int_full_data[offset];
            if (gd->swap)
            {
               bval = (unsigned char *) &int_val;
               tempc = bval[0];
               bval[3] = bval[0];
               bval[0] = tempc;
               tempc = bval[1];
               bval[1] = bval[2];
               bval[2] = tempc;
            }
            intens = int_val;
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               int_bgdata = (int *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval =  int_bgdata[y_disp_offset*(gd->img_width)
                                       + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;
      }
   }
   else
   {
      if (gd->fd<0)
      {
         xdl_panel_label_set (gd->cursor_position, "Cannot read file", 0, 0);
         return;
      }
      switch (gd->type)
      {
         case 1:
            read_byte_buf = (unsigned char *) gd->read_buf;
            lseek (gd->fd, offset, 0);
            i = read (gd->fd, read_byte_buf, 1);
            if (i!=1)
            {
               xdl_panel_label_set (gd->cursor_position, "Error reading file", 
                                    0, 0);
               return;
            }
            intens = (int) read_byte_buf[0];
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               bgdata = (unsigned char *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval = (int) bgdata[y_disp_offset*(gd->img_width)
                                    + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;
         case 2:
            read_i2_buf = (unsigned short *) gd->read_buf;
            read_byte_buf = (unsigned char *) gd->read_buf;
            offset *= 2;
            lseek (gd->fd, offset, 0);
            i = read (gd->fd, read_i2_buf, 2);
            if (i!=2)
            {
               xdl_panel_label_set (gd->cursor_position, "Error reading file", 
                                    0, 0);
               return;
            }
            if (gd->swap)
            {
               tempc = read_byte_buf[0];
               read_byte_buf[0] = read_byte_buf[1];
               read_byte_buf[1] = tempc;
            }
            intens = (int) read_i2_buf[0];
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               i2_bgdata = (unsigned short *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval = (int) i2_bgdata[y_disp_offset*(gd->img_width)
                                       + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;
         case 3:
            read_int_buf = (int *) gd->read_buf;
            read_byte_buf = (unsigned char *) gd->read_buf;
            offset *= 4;
            lseek (gd->fd, offset, 0);
            i = read (gd->fd, read_int_buf, 4);
            if (i!=4)
            {
               xdl_panel_label_set (gd->cursor_position, "Error reading file", 
                                    0, 0);
               return;
            }
            if (gd->swap)
            {
               tempc = read_byte_buf[0];
               read_byte_buf[0] = read_byte_buf[3];
               read_byte_buf[3] = tempc;
               tempc = read_byte_buf[1];
               read_byte_buf[1] = read_byte_buf[2];
               read_byte_buf[2] = tempc;
            }
            intens = read_int_buf[0];
            if (gd->bg_opt>0&&gd->bg_present==1)
            {
               int_bgdata = (int *) gd->bg_data;
               x_disp_offset = (gd->xpix-gd->xo_pix)/(gd->nxp);
               y_disp_offset = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 
                               - gd->ypix;
               y_disp_offset /= gd->nyp;
               bgval = int_bgdata[y_disp_offset*(gd->img_width)
                                       + x_disp_offset];
               if (gd->bg_opt==1)
               {
                  intens -= bgval;
                  if (intens<0) intens = 0;
               }
               else if (gd->bg_opt==2)
               {
                  intens = bgval;
               }
            }
            break;
      }
   }
   sprintf (str, "Intensity %d", intens);
   xdl_panel_label_set (gd->cursor_position, str, 0, 0);
   return;
}

/*
*************************************
**     xdl_film_image_drawsymb     **
*************************************

Purpose: Draw an overlay symbol

Author:  John W. Campbell, August  1990

*/

void xdl_film_image_drawsymb (gd, wid, x, y, symbol, color)

XDl_film_image *gd;       /* Pointer top global data area */
Window wid;               /* Window id (film or magnifying window) */
int x;                    /* x offset for symbol centre */
int y;                    /* y offset for symbol centre */
int symbol;               /* symbol type  1 to 10,  Vertical cross - increasing
                                                    size
                                          11 to 20, Cross - increasing size
                                          21 to 30, Square - increasing size */
int color;                /* overlay colour number (1-6) */

{
   int symtyp;            /*symbol type 0=vertical cross, 1=cross, 2=square*/
   int symsize;           /*size of symbol from 0 to 9*/

   symtyp = (symbol-1) / 10;
   symsize = (symbol-1) % 10;

   XSetForeground (XDL_display, gd->paint_gc, gd->ovly_pixels[color-1]);

   if (symsize==0)
   {
      XDrawPoint (XDL_display, wid, gd->paint_gc, x, y);
   }
   else
   {
   switch (symtyp)
      {
         case 0:
         /* Vertical Cross */
            XDrawLine (XDL_display, wid, gd->paint_gc, x-symsize, y,
                       x+symsize, y);
            XDrawLine (XDL_display, wid, gd->paint_gc, x, y-symsize,
                       x, y+symsize);
            break;
         case 1:
         /* Cross */
            XDrawLine (XDL_display, wid, gd->paint_gc, x-symsize, y-symsize,
                       x+symsize, y+symsize);
            XDrawLine (XDL_display, wid, gd->paint_gc, x+symsize, y-symsize,
                       x-symsize, y+symsize);
            break;
         case 2:
         /* Square */
            XDrawRectangle (XDL_display, wid, gd->paint_gc,
                            x-symsize, y-symsize, 2*symsize, 2*symsize);
            break;
      }
   }
   return;
}

/*
*************************************
**     xdl_film_image_drawvec      **
*************************************

Purpose: Draw an overlay vector

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_drawvec (gd, wid, x1, y1, x2, y2, color)

XDl_film_image *gd;       /* Pointer top global data area */
Window wid;               /* Window id (film or magnifying window) */
int x1;                   /* x offset for start of vector */
int y1;                   /* y offset for start of vector */
int x2;                   /* x offset for end of vector */
int y2;                   /* y offset for end of vector */
int color;                /* overlay colour number (1-6) */

{

   XSetForeground (XDL_display, gd->paint_gc, gd->ovly_pixels[color-1]);

   XDrawLine (XDL_display, wid, gd->paint_gc, x1, y1, x2, y2);

   return;
}

/*
*************************************
**     xdl_film_image_dsppos       **
*************************************

Purpose: Get pixel position within display window (and offset 0 to nxp or 0
         to nyp)

Author:  John W. Campbell, December 1991

*/

void xdl_film_image_dsppos (xyoff, nxyp, xy_disp, xy_p)

int xyoff;       /* x or y pixel offset wrt origin of displayed section (R)*/
int nxyp;        /* No. of x or y points compressed into a pixel (R)*/
int *xy_disp;    /* Returns offset within main display area (W)*/
int *xy_p;       /* Returns offset within main pixel (W)*/
{
   if (xyoff>=0)
   {
      *xy_disp = xyoff / nxyp;
      *xy_p = xyoff%nxyp;
   }
   else
   {
      *xy_disp = (xyoff + 1 - nxyp) / nxyp;
      *xy_p = xyoff%nxyp;
      if (*xy_p<0) *xy_p += nxyp;
   }

   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Get selected pixel position - xdl_film_image_getxy
The routine xdl_film_image_getxy (xdlf_film_image_getxy) is used to
return the last selected input pixel (spot) position to the calling
program. It will normally be called after the return from a call to
xdl_get_events (xdlf_get_events) indicates that a film image view-object
has returned input.
-end*/
/*
**********************************
**     xdl_film_image_getxy     ** 
**********************************

Purpose: Return the last selected spot position

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_GETXY (IVH, IXPIX, IYPIX, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXPIX    (W)  Returns X pixel position (see xpix_img)
IYPIX    (W)  Returns Y pixel position (see ypix_img)
IERR     (W)  Returns status from xdl_film_image_getxy call
-end*/

/*-C:*/
int xdl_film_image_getxy (vh, xpix_img, ypix_img)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *xpix_img;   /* Returns the pixel 'x' position (from 1 up - refers 
                    to complete uncompressed image) (W) */
int *ypix_img;   /* Returns the pixel 'y' position (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_getxy",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_getxy", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   *xpix_img = gd->xpix_img;
   *ypix_img = gd->ypix_img;
   return 0;
}

/* Fortran binding: xdlf_film_image_getxy */

void xdlf_film_image_getxy (ivh, ixpix, iypix, ierr)
int *ivh, *ixpix, *iypix, *ierr;
{
   *ierr = xdl_film_image_getxy (*ivh, ixpix, iypix);
   return;
}

/*-Routine: Get pixel position from an overlay - xdl_film_image_getxy_ovly
The routine xdl_film_image_getxy_ovly (xdlf_film_image_getxy_ovly) is used 
to return the last selected input pixel (e.g. symbol) position from an
overlay to the calling program. It differs from xdl_film_image_getxy
(xdlf_film_image_getxy) in that it makes an allowance for any offsetting
of the overlay currently in force. It will normally be called after the 
return from a call to xdl_get_events (xdlf_get_events) indicates that a film 
image view-object has returned input.
-end*/
/*
***************************************
**     xdl_film_image_getxy_ovly     ** 
***************************************

Purpose: Return the last selected symbol position (allows for any
         offsetting of the overlay)

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_GETXY_OVLY (IVH, IOV, IXPIX, IYPIX, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IOV      (R)  Overlay number (1 o 2) (see ov_num)
IXPIX    (W)  Returns X pixel position (see xpix_img)
IYPIX    (W)  Returns Y pixel position (see ypix_img)
IOV      (R)
IERR     (W)  Returns status from xdl_film_image_getxy_ovly call
-end*/

/*-C:*/
int xdl_film_image_getxy_ovly (vh, ov_num, xpix_img, ypix_img)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int ov_num;      /* Overlay number (1 or 2) 
                    Note: Overlay 2 is never offset.
                          Overlay 1 will be offset if the overlay type
                          is on/off/offset and the offset option has
                          been selected  (R)*/
int *xpix_img;   /* Returns the pixel 'x' position (from 1 up - refers 
                    to complete uncompressed image); any overlay offset 
                    is subtracted from the value before it is returned 
                    (W) */
int *ypix_img;   /* Returns the pixel 'y' position (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */
   int xs_off;             /* Overlay offset in x */
   int ys_off;             /* Overlay offset in y */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_getxy_ovly",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_getxy_ovly", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   xs_off = 0;
   ys_off = 0;
   if (ov_num==1&&gd->ovly_offset==1)
   {
      xs_off = gd->x_ovly_offset;
      ys_off = gd->y_ovly_offset;
   }
   *xpix_img = gd->xpix_img - xs_off*(gd->nxp);
   *ypix_img = gd->ypix_img + ys_off*(gd->nyp);
   return 0;
}

/* Fortran binding: xdlf_film_image_getxy_ovly */

void xdlf_film_image_getxy_ovly (ivh, iov, ixpix, iypix, ierr)
int *ivh, *iov, *ixpix, *iypix, *ierr;
{
   *ierr = xdl_film_image_getxy_ovly (*ivh, *iov, ixpix, iypix);
   return;
}

/*-Routine: Get selected rectangle - xdl_film_image_getrect
The routine xdl_film_image_getrect (xdlf_film_image_getrect) is used to
return the last selected rectangle coordinates in pixels to the calling
program. It will normally be called after the return from a call to
xdl_get_events (xdlf_get_events) indicates that a film image view-object
has returned input.
-end*/
/*
**********************************
**     xdl_film_image_getrect   ** 
**********************************

Purpose: Return the last selected rectangle

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_GETRECT (IVH, IX1PIX, IY1PIX, 
        +          IX2PIX, IY2PIX, IVECT, ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IX1PIX   (W)  Returns X1 pixel position (see xpix_dn)
IY1PIX   (W)  Returns Y1 pixel position (see ypix_dn)
IX2PIX   (W)  Returns X2 pixel position (see xpix_up)
IY2PIX   (W)  Returns Y2 pixel position (see ypix_up)
IVECT    (R)  Vector identifier for drawing box at selected
              position (1-65535); 0 if no box is to be 
              drawn (see also description of 
              xdlf_film_image_drawvect routine)
ICOLR    (R)  Vector colour (1 to 6) if needed (see color)
IOV      (R)  Overlay number 1 or 2 if needed (see ov_num)
IERR     (W)  Returns status from xdl_film_image_getrect call
-end*/

/*-C:*/
int xdl_film_image_getrect (vh, xpix_dn, ypix_dn, xpix_up, ypix_up,
                            vect_id, color, ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *xpix_dn;    /* Returns the pixel 'x' position (from 1 up - refers 
                    to complete uncompressed image) of rectangle corner
                    where Button1 was pressed (W) */
int *ypix_dn;    /* Returns the pixel 'y' position where Button1 was
                    pressed (W) */
int *xpix_up;    /* Returns the pixel 'x' position (from 1 up - refers 
                    to complete uncompressed image) of rectangle corner
                    where Button1 was released (W) */
int *ypix_up;    /* Returns the pixel 'y' position where Button1 was
                    released (W) */
int vect_id;     /* Vector identifier (1-65535) to draw a box on the
                    image display at the selected rectangle position;
                    0 if no box is to be drawn. (see xdl_film_image_vect) */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default) if needed*/
int ov_num;      /* Overlay number 1 or 2 (R) if needed*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_getrect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_getrect", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   *xpix_dn = gd->xpix_dn;
   *ypix_dn = gd->ypix_dn;
   *xpix_up = gd->xpix_up;
   *ypix_up = gd->ypix_up;

   if (vect_id>0&&vect_id<=65535)
   {
      xdl_film_image_vect (vh, vect_id, gd->xpix_dn, gd->ypix_dn,
                                        gd->xpix_up, gd->ypix_dn,
                                        color, ov_num, 0);
      xdl_film_image_vect (vh, vect_id, gd->xpix_dn, gd->ypix_dn,
                                        gd->xpix_dn, gd->ypix_up,
                                        color, ov_num, 0);
      xdl_film_image_vect (vh, vect_id, gd->xpix_up, gd->ypix_dn,
                                        gd->xpix_up, gd->ypix_up,
                                        color, ov_num, 0);
      xdl_film_image_vect (vh, vect_id, gd->xpix_dn, gd->ypix_up,
                                        gd->xpix_up, gd->ypix_up,
                                        color, ov_num, 1);
   }
   return 0;
}

/* Fortran binding: xdlf_film_image_getrect */

void xdlf_film_image_getrect (ivh, ix1pix, iy1pix, ix2pix, iy2pix, 
                              ivect, icolr, iov, ierr)
int *ivh, *ix1pix, *iy1pix, *ix2pix, *iy2pix, *ivect, *icolr, *iov, *ierr;
{
   *ierr = xdl_film_image_getrect (*ivh, ix1pix, iy1pix, ix2pix, iy2pix,
                              *ivect, *icolr, *iov);
   return;
}

/*-Routine: Set background image - xdl_film_image_background
The routine xdl_film_image_background (xdlf_film_image_background) is used to 
pass a background image to the film image view-object. This has the same
format and size as the corrsponding compressed image. If bg_typ (IBG) was set 
to 1 when the xdl_film_image (xdlf_film_image) routine was called then the 
background options will be handled via the background choice menu on the control
panel; otherwise the background image will be ignored.
-end*/
/*
*************************************
**     xdl_film_image_background   ** 
*************************************

Purpose: Pass a background image to the film image view-object

Author:  John W. Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_BACKGROUND (IVH, IBGDATA)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IBGDATA  (R)  Array holding background image data (same format
              and size as for a compressed image as described
              in the call to the main xdlf_film_image routine.)
              (e.g. pass as in integer array with the data
              packed as needed)
              Note: a compressed background image is given
              here even if a full image was passed to the
              main routine.
              (bg_data)
IERR     (W)  Returns status from xdl_film_background call
-end*/

/*-C:*/
int xdl_film_image_background (vh, bg_data)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char * bg_data;  /* Background image data; the size of the image is the
                    size of the compressed image passed to the 
                    xdl_film_image routine (or formed by it from the full 
                    image). The data format is the same as for the main 
                    image. (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_background",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_background", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (gd->bg_typ==0) return 0;
   gd->bg_data = bg_data;
   gd->bg_present = 1;
   return 0;
}
/* Fortran binding: xdlf_film_image_background */

void xdlf_film_image_background (ivh, ibgdata, ierr)
int *ivh, *ibgdata, *ierr;
{
   *ierr = xdl_film_image_background (*ivh, (char *) ibgdata);
   return;
}

/*-Routine: Set a new image - xdl_film_image_newimg
The routine xdl_film_newimg (xdlf_film_image_newimg) is used to reset
the image in the display to a new image. This image must be of the same
type and have the same dimensions as the original image.
-end*/
/*
*************************************
**     xdl_film_image_newimg       ** 
*************************************

Purpose: Reset image with a new image of the same type and dimensions as
         the original image

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_NEWIMG (IVH, IMGDATA, IFULL, IMGFULL,
        +                             IFD, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
IMGDATA   (R)   Array holding compressed data (e.g. pass as an 
                integer array with data packed as needed)  
                (see imgdata)
IFULL     (R)   1 = full image will be passed to the routine (in
                    addition to the compressed image section)
                0 = full image will not be passed.
                (see full)
IMGFULL   (R)   Array holding full image data (e.g. pass as an
                integer array with data packed as needed). Dummy
                array if IFULL=0.
                (see img_full)
IFD       (R)   File descriptor of film image file (-1 if none to
                be read). Ignored if ifull=1. (see fd)
IERR      (W)  Returns status from xdl_film_image_newimg call
-end*/

/*-C:*/
int xdl_film_image_newimg (vh, imgdata, full, img_full, fd)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char *imgdata;            /* The new image (see xdl_film_image call for
                             full details (R)*/
int       full;           /* 1 = pass full image to the routine (img_full)
                                 as well as the compressed image. This
                                 is used for magnifyiing window data and
                                 pixel intensities if given.
                             0 = do not pass the full image
                             (R)*/
char      *img_full;      /* Full image data (byte for film or unsigned
                             i2 for image plate) if required (see full);
                             otherwise null. (see xdl_film_image call for
                             full details (R)*/
int       fd;             /* File descriptor for the new film or image 
                             plate file.
                             May be -1 if no file is to be read.
                             Not used if full=1.
                             (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_newimg",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_newimg", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->imgdata = imgdata;
   if (full!=0) full = 1;
   gd->full_image = full;
   if (full)
   {
      gd->full_imgdata = img_full;
   }
   else
   {
      gd->full_imgdata = NULL;
   }
   gd->fd = fd;
   gd->bg_present = 0;
   gd->bg_opt = 0;
   gd->bg_data = NULL;
   if (gd->bg_typ>0) xdl_panel_choice_reset (gd->bg_choice, 1);
   gd->magnify_on = 1;
   gd->freeze.on = 0;
   gd->read_file = 0;
   gd->xpix = -1;
   gd->ypix = -1;
   gd->xpix_dn = -1;
   gd->ypix_dn = -1;
   gd->xpix_up = -1;
   gd->ypix_up = -1;
   gd->xpix_img = -1;
   gd->ypix_img = -1;
   gd->xpix_img_up = -1;
   gd->ypix_img_up = -1;
   gd->sel_rect = 0;
   gd->sel_rect_xo = 0;
   gd->sel_rect_yo = 0;
   gd->sel_rect_x1 = 0;
   gd->sel_rect_y1 = 0;
   gd->sel_rect_x2 = 0;
   gd->sel_rect_y2 = 0;
/* PRE clear vectors as well */
   if (gd->vects!=NULL) free (gd->vects);
   gd->vects = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
/* -PRE */
   xdl_film_image_setdata (gd, gd->type);
   xdl_film_image_refresh (gd,1);
   xdl_panel_label_set (gd->cursor_position," ",0,0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_newimg */

void xdlf_film_image_newimg (ivh, imgdata, ifull, imgfull, ifd, ierr)
int *ivh, *imgdata, *ifull, *imgfull, *ifd, *ierr;
{
   *ierr = xdl_film_image_newimg (*ivh, (char *) imgdata, *ifull, 
                                  (char *) imgfull, *ifd);
   return;
}
/*-Routine: Reset the file descriptor - xdl_film_image_reset_fd
The routine xdl_film_image_fd (xdlf_film_image_reset_fd) is used to reset
the file descriptor for a film or image plate file which has been closed
and reopened since the film image view-object was created. An option is
available to indicate that the file has been closed and not reopened.
-end*/
/*
*************************************
**     xdl_film_image_reset_fd     ** 
*************************************

Purpose: Reset file descriptor if film image file has been closed and reopened

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_RESET_FD (IVH, IFD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IFD      (R)  New file descriptor (or -1 to indicate that the
              file is now closed and no longer available for 
              reading) (see fd)
IERR     (W)  Returns status from xdl_film_image_reset_fd call
-end*/

/*-C:*/
int xdl_film_image_reset_fd (vh, fd)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int fd;          /* New file descriptor (or -1 to indicate that the file 
                    is now closed and no longer available for reading (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_reset_fd",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_reset_fd", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->fd = fd;
   return 0;
}

/* Fortran binding: xdlf_film_image_reset_fd */

void xdlf_film_image_reset_fd (ivh, ifd, ierr)
int *ivh, *ifd, *ierr;
{
   *ierr = xdl_film_image_reset_fd (*ivh, *ifd);
   return;
}

/*-Routine: Reset overlay offset values - xdl_film_image_ovly_offset
The routine xdl_film_image_ovly_offset (xdlf_film_image_ovly_offset) is
used to reset the amount by which an overlay is offset when the overlay
offset option is selected via the film image view-object control panel.
The default when the film image view-object is created is four pixels
on the main image display area to the left and down (xoff=-4, yoff=4).
-end*/
/*
****************************************
**     xdl_film_image_ovly_offset     ** 
****************************************

Purpose: Reset overlay offset values (New values will be used the next time
         'offset' is selected from the Overlay menu.

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_OVLY_OFFSET (IVH, IXOFF, IYOFF, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXOFF    (R)  Overlay x offset (see xoff)
IYOFF    (R)  Overlay y offset (see yoff)
IERR     (W)  Returns status from xdl_film_image_ovly_offset call
-end*/
/*-C:*/
int xdl_film_image_ovly_offset (vh, xoff, yoff)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int xoff;        /* New x overlay offset (in pixels of main film 
                    display) (R)*/
int yoff;        /* New y overlay offset (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_ovly_offset",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_ovly_offset", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   gd->x_ovly_offset = xoff;
   gd->y_ovly_offset = yoff;
   return 0;
}

/* Fortran binding: xdlf_film_image_ovly_offset */

void xdlf_film_image_ovly_offset (ivh, ixoff, iyoff, ierr)
int *ivh, *ixoff, *iyoff, *ierr;
{
   *ierr = xdl_film_image_ovly_offset (*ivh, *ixoff, *iyoff);
   return;
}

/*-Routine: Reset an overlay colour - xdl_film_image_set_colour
The routine xdl_film_image_set_colour (xdlf_film_image_set_colour) is
used to reset one of the 6 colours used for the overlay symbols to a
user selected colour or the colour used to indicate the position of
a 'frozen' magnifying window.
-end*/
/*
***************************************
**     xdl_film_image_set_colour     ** 
***************************************

Purpose: Reset an overlay colour (for symbols etc.)

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_SET_COLOUR (IVH, ICOLR, IRED, IGREEN, 
        +                                 IBLUE, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ICOLR    (R)  Overlay colour number (1 to 7) (see icolr)
IRED     (R)  Red component 0 to 65535 (see red)
IGREEN   (R)  Green component 0 to 65535 (see green)
IBLUE    (R)  Blue component 0 to 65535 (see blue)
IERR     (W)  Returns status from xdl_film_image_set_colour call
-end*/

/*-C:*/
int xdl_film_image_set_colour (vh, icolr, red, blue, green)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int icolr;       /* Overlay colour number 1 to 6 for symbols or 7 
                    for the magnifying window position indicating 
                    box (R)*/
int red;         /* Red component 0 to 65535 (R)*/
int green;       /* Green component 0 to 65535 (R)*/
int blue;        /* Blue component 0 to 65535 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */
   XColor colr_cell;       /* Temporary Xcolor structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_set_colour",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_set_colour", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   colr_cell.flags = DoRed|DoGreen|DoBlue;
   colr_cell.red = red;
   colr_cell.green = green;
   colr_cell.blue = blue;
   if (XDL_csets_alloc==1)
   {
      colr_cell.pixel = XDL_colorsets[gd->cset].pixels[63+icolr];
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   else
   {
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
   }
   gd->ovly_pixels[icolr-1] = colr_cell.pixel;
   if (XDL_csets_alloc==0) xdl_film_image_refresh (gd,1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_set_colour */

void xdlf_film_image_set_colour (ivh, icolr, ired, igreen, iblue, ierr)
int *ivh, *icolr, *ired, *igreen, *iblue, *ierr;
{
   *ierr = xdl_film_image_set_colour (*ivh, *icolr, *ired, *igreen, *iblue);
   return;
}

/*-Routine: Reset the user defined colour map - xdl_film_image_set_colormap
The routine xdl_film_image_set_colormap (xdlf_film_image_set_colormap) is
used to reset the 'user defined' 64 colour colour map used to display the
image. The use of the map is selected via the colour choice menu on the
film image view object. Note that the contrast slider will have no effect
when the user defined colour map is selected.
-end*/
/*
*****************************************
**     xdl_film_image_set_colormap     ** 
*****************************************

Purpose: Reset user defined colormap for image display (64 colours)

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_SET_COLORMAP (IVH, IRED, IGREEN, 
        +                                 IBLUE, IERR)
-end*/
 
/*-Parameters:
IVH          (R)  View-object handle (see vh)
IRED(64)     (R)  64 Red components 0 to 65535 (see red)
IGREEN(64)   (R)  64 Green components 0 to 65535 (see green)
IBLUE(64)    (R)  64 Blue components 0 to 65535 (see blue)
IERR         (W)  Returns status from xdl_film_image_set_colormap
                  call
-end*/

/*-C:*/
int xdl_film_image_set_colormap (vh, red, blue, green)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int red[];       /* 64 Red components 0 to 65535 (R)*/
int green[];     /* 64 Green components 0 to 65535 (R)*/
int blue[];      /* 64 Blue components 0 to 65535 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */
   int i;                  /* Temp/loop variable */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_set_colormap",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_set_colormap",
                             vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   for (i=0;i<64;++i)
   {
      if (red[i]<0) red[i] = 0;
      if (red[i]>65535) red[i] = 65535;
      if (green[i]<0) green[i] = 0;
      if (green[i]>65535) green[i] = 65535;
      if (blue[i]<0) blue[i] = 0;
      if (blue[i]>65535) blue[i] = 65535;
      gd->user_rgb[0][i] = red[i];
      gd->user_rgb[1][i] = green[i];
      gd->user_rgb[2][i] = blue[i];
   }
   if (gd->color_opt==6) xdl_film_image_cp2 (iv, 6);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_set_colormap */

void xdlf_film_image_set_colormap (ivh, ired, igreen, iblue, ierr)
int *ivh, *ired, *igreen, *iblue, *ierr;
{
   *ierr = xdl_film_image_set_colormap (*ivh, ired, igreen, iblue);
   return;
}

/*-Routine: Store and display an overlay symbol - xdl_film_image_symbol
The routine xdl_film_image_symbol (xdlf_film_image_symbol) is used to
store details of an overlay symbol and to display that symbol if
overlays are currently being displayed. Two sets of overlay symbols
may be stored. (If many symbols are to be added then, if possible,
use the routine xdl_film_image_symbols (xdlf_film_image_symbols) instead)
-end*/
/*
**********************************
**     xdl_film_image_symbol    ** 
**********************************

Purpose: Store and display a symbol

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_SYMBOL (IVH, IXPIX, IYPIX, ISYMB, 
        +                             ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXPIX    (R)  X pixel position in full image (see x_pixel)
IYPIX    (R)  Y pixel position in full image (see y_pixel)
ISYMB    (R)  Symbol type (see symbol)
ICOLR    (R)  Symbol colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
IERR     (W)  Returns status from xdl_film_image_symbol call
-end*/

/*-C:*/
int xdl_film_image_symbol (vh, x_pixel, y_pixel, symbol, color, ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int x_pixel;     /* The pixel 'x' position (from 1 up - refers to 
                    complete uncompressed image (R) */
int y_pixel;     /* The pixel 'y' position (R) */
int symbol;      /* The symbol type
                    = 1 to 10   vertical cross - increasing size 
                                                 1 to 19 pixels
                                                 in steps of 2.
                    = 11 to 20  cross - increasing size 1 to 19 pixels 
                                        height.
                    = 21 to 30  square - increasing size 1 to 19 pixels.
                                Note: 1, 11 and 21 are single points */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default)*/
int ov_num;      /* Overlay number 1 or 2 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new symbol's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_film_image *gd;       /* Pointer to global data area */
   XDl_film_image_symbol *p; /* Temp. pointer to this structure */
   int x;                    /* x pixel position offset in main film display 
                                area (display area pixels) */
   int y;                    /* y pixel position offset in main film display 
                                area (display area pixels) */
   int xoff;                 /* x Offset of pixel (film image pixels) in display
                                area*/
   int yoff;                 /* Offset of pixel (film image pixels) from
                                top of film display area */
   int x_p;                  /* x pixel offset within main pixel 0 to nxp-1 */
   int y_p;                  /* y pixel offset within main pixel 0 to nyp-1 */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int iposn;                /* Position (offset) in symbols list for new
                                symbol */
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   int xs_off;               /* Overlay symbol x offset */
   int ys_off;               /* Overlay symbol y offset */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_symbol", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;
   if (gd->num_symb>=gd->max_alloc_symbols)
   {
      if (gd->symbols==NULL)
      {
         p = (XDl_film_image_symbol *) malloc (
           (gd->max_alloc_symbols+NSYMB_ALLOC) * sizeof(XDl_film_image_symbol));
         if (p==0) return 4;
      }
      else
      {
         p = (XDl_film_image_symbol *) realloc (gd->symbols,
           (gd->max_alloc_symbols+NSYMB_ALLOC) * sizeof(XDl_film_image_symbol));
         if (p==0) return 4;
      }
      gd->symbols = p;
      gd->max_alloc_symbols += NSYMB_ALLOC;
   }

   /* Find position in sorted list of symbols for the new entry */

   iposn = gd->num_symb;
   if (gd->num_symb>0)
   {
      for (i=0;i<gd->num_symb;++i)
      {
         if (y_pixel>gd->symbols[i].y_pixel) {iposn = i; break;}
         if (y_pixel==gd->symbols[i].y_pixel)
         {
            if (x_pixel<gd->symbols[i].x_pixel) {iposn = i; break;}
         }
      }
   }

   /* Shift symbols if required */

   if (iposn<gd->num_symb)
   {
      tp = (char *) gd->symbols;
      ssiz = sizeof (XDl_film_image_symbol);
      for (i=(gd->num_symb+1)*ssiz-1;i>=iposn*ssiz;--i) tp[i] = tp[i-ssiz];
   }

   gd->num_symb += 1;

   /* Save details for the new symbol */
         
   gd->symbols[iposn].x_pixel = x_pixel;
   gd->symbols[iposn].y_pixel = y_pixel;
   xoff = x_pixel - gd->xo_pix;
   xdl_film_image_dsppos(xoff,gd->nxp,&x,&x_p);
   gd->symbols[iposn].x_nxp_offset = x_p;
   gd->symbols[iposn].x_disp_offset = x;
   yoff = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 - y_pixel;
   xdl_film_image_dsppos(yoff,gd->nyp,&y,&y_p);
   gd->symbols[iposn].y_nyp_offset = y_p;
   gd->symbols[iposn].y_disp_offset = y;
   gd->symbols[iposn].symb = symbol;
   gd->symbols[iposn].color_flags = color;
   if (ov_num==2) gd->symbols[iposn].color_flags |= 0X10;
   if (gd->ovly_typ==2||ov_num==2) gd->symbols[iposn].color_flags |= 0X20;
     
   /* Update y-search offsets array */

   y_start = (y * (Y_SEARCH_MAX-1)) / gd->img_height;
   if (y_start<0) y_start = 0;
   if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
   if (y_start+1<Y_SEARCH_MAX)
   {
       for (i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i] += 1;
   }

   /* Draw symbol if required */

   if (gd->ovly1==0&&ov_num==1) return 0;
   if (gd->ovly2==0&&ov_num==2) return 0;
   xs_off = 0;
   ys_off = 0;
   if (gd->ovly_offset==1&&(gd->symbols[iposn].color_flags&0X20)==0)
   {
       xs_off = gd->x_ovly_offset;
       ys_off = gd->y_ovly_offset;
   }
   xdl_film_image_drawsymb (gd, gd->wdisp, 
                            x + xs_off, y + ys_off,
                            symbol, color);
   xdl_film_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_symbol */

void xdlf_film_image_symbol (ivh, ixpix, iypix, isymb, icolr, iov, ierr)
int *ivh, *ixpix, *iypix, *isymb, *icolr, *iov, *ierr;
{
   *ierr = xdl_film_image_symbol (*ivh, *ixpix, *iypix, *isymb, *icolr, *iov);
   return;
}
/*-Routine: Store and display overlay symbols - xdl_film_image_symbols
The routine xdl_film_image_symbols (xdlf_film_image_symbols) is used to
store details of overlay symbols and to display those symbols if
overlays are currently being displayed. The symbols are added to an
internal symbols list and then sorted. Use this routine when possible
if may symbols are to be added rather than using individual calls to 
xdl_film_image_symbol (xdlf_film_image_symbol). Two sets of overlay 
symbols may be used.
-end*/
/*
***********************************
**     xdl_film_image_symbols    ** 
***********************************

Purpose: Store and display a list of symbols

Author:  John W. Campbell, July 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_SYMBOLS (IVH, NSYMB, IXPIX, IYPIX, ISYMB, 
        +                             ICOLR, IOV, NT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NSYMB    (R)  No. of symbols to store (see nsymb)
IXPIX    (R)  Array of X pixel positions in full image 
              (NSYMB values) (see x_pixel)
IYPIX    (R)  Array of Y pixel positions in full image
              (NSYMB values) (see y_pixel)
ISYMB    (R)  Array of Symbol types or Symbol type as indicated
              by NT (see symbol)
ICOLR    (R)  Array of Symbol colours or Symbol colour as 
              indicated by NT (1 to 6) (see color)
IOV      (R)  Array of overlay numbers (1 or 2) or overlay
              number as indicated by NT (see ov_num)
NT       (R)  =1 give a single symbol type, colour and overlay 
                 number
              =0 give arrays of NSYMB symbols, colours and overlay
                 numbers (see nt)
IERR     (W)  Returns status from xdl_film_image_symbol call
-end*/

/*-C:*/
int xdl_film_image_symbols (vh, nsymb, x_pixel, y_pixel, symbol, color, 
                            ov_num, nt)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int nsymb;       /* The number of symbols to be added */
int x_pixel[];   /* Array of pixel 'x' positions (from 1 up - refers to 
                    complete uncompressed image) (R) */
int y_pixel[];   /* Array of pixel 'y' positions (R) */
int symbol[];    /* Array of symbol types (nsymb values or 1 value, see nt)
                    = 1 to 10   vertical cross - increasing size 
                                                 1 to 19 pixels
                                                 in steps of 2.
                    = 11 to 20  cross - increasing size 1 to 19 pixels 
                                        height.
                    = 21 to 30  square - increasing size 1 to 19 pixels.
                                Note: 1, 11 and 21 are single points (R) */
int color[];     /* Array of Overlay colour types (nsymb values or 1 value,
                    see nt)
                    1 to 6; Red, Yellow, Green, Cyan,Blue, Magenta 
                    (by default but may be reset) (R)*/
int ov_num[];    /* Array of overlay numbers (1 or 2) (nsymb values or 1
                    value, see nt) (R)*/
int nt;          /* =0 give nsymb values for symbol types and colours
                    =1 give a single value for the symbol type and colour 
                       and use this for all the symbols being added (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new symbol's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_film_image *gd;       /* Pointer to global data area */
   XDl_film_image_symbol *p; /* Temp. pointer to this structure */
   XDl_film_image_symbol *s; /* Temp pointer to symbols */
   int x;                    /* x pixel position offset in main film display 
                                area (display area pixels) */
   int y;                    /* y pixel position offset in main film display 
                                area (display area pixels) */
   int xoff;                 /* x Offset of pixel (film image pixels) in display
                                area*/
   int yoff;                 /* Offset of pixel (film image pixels) from
                                top of film display area */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int x_p;                  /* x pixel offset within main pixel 0 to nxp-1 */
   int y_p;                  /* y pixel offset within main pixel 0 to nyp-1 */
   int iposn;                /* Position (offset) in symbols list for new
                                symbol */
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int is;                   /* Loop counter through symbols */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   int y_start_prev;         /* Previous value of y_start */
   int j,l,it,ir;            /* Indices for sort */
   unsigned long tempyx;     /* Temporary saved value for sort */
   int nxt;                  /* Number of allocations for extending
                                symbol table memory */
   int y_max;                /* Set to at least max y pixel possible */
   int xs_off;               /* Overlay symbol x offset */
   int ys_off;               /* Overlay symbol y offset */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_symbol", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (nt!=0) nt=1;
   if (nsymb<=0) return 0;

   /* Get extra memory for new symbols if required */

   if (gd->num_symb+nsymb>gd->max_alloc_symbols)
   {
      nxt = (gd->num_symb+nsymb-gd->max_alloc_symbols-1)/NSYMB_ALLOC + 1;
      {
         if (gd->symbols==NULL)
         {
            p = (XDl_film_image_symbol *) malloc (
              (gd->max_alloc_symbols+nxt*NSYMB_ALLOC) * 
                   sizeof(XDl_film_image_symbol));
            if (p==0) return 4;
         }
         else
         {
            p = (XDl_film_image_symbol *) realloc (gd->symbols,
              (gd->max_alloc_symbols+nxt*NSYMB_ALLOC) * 
                   sizeof(XDl_film_image_symbol));
            if (p==0) return 4;
         }
         gd->symbols = p;
         gd->max_alloc_symbols += nxt*NSYMB_ALLOC;
      }
   }

   /* Add new symbols to the list */

   for(is=0;is<nsymb;++is)
   {
      iposn=gd->num_symb;
      gd->num_symb += 1;

      /* Save details for the new symbol */
         
      gd->symbols[iposn].x_pixel = x_pixel[is];
      gd->symbols[iposn].y_pixel = y_pixel[is];
      xoff = x_pixel[is] - gd->xo_pix;
      xdl_film_image_dsppos(xoff,gd->nxp,&x,&x_p);
      gd->symbols[iposn].x_nxp_offset = x_p;
      gd->symbols[iposn].x_disp_offset = x;
      yoff = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 - y_pixel[is];
      xdl_film_image_dsppos(yoff,gd->nyp,&y,&y_p);
      gd->symbols[iposn].y_nyp_offset = y_p;
      gd->symbols[iposn].y_disp_offset = y;
      if (nt==0)
      {
         gd->symbols[iposn].symb = symbol[is];
         gd->symbols[iposn].color_flags = color[is];
         if (ov_num[is]==2) gd->symbols[iposn].color_flags |= 0X10;
         if (gd->ovly_typ==2||ov_num[is]==2) 
                 gd->symbols[iposn].color_flags |= 0X20;
      }
      else
      {
         gd->symbols[iposn].symb = symbol[0];
         gd->symbols[iposn].color_flags = color[0];
         if (ov_num[0]==2) gd->symbols[iposn].color_flags |= 0X10;
         if (gd->ovly_typ==2||ov_num[0]==2) 
                 gd->symbols[iposn].color_flags |= 0X20;
      }
      if (gd->ovly1==0&&(gd->symbols[iposn].color_flags&0X10)==0) continue;
      if (gd->ovly2==0&&(gd->symbols[iposn].color_flags&0X10)!=0) continue;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(gd->symbols[iposn].color_flags&0X20)==0)
      {
         xs_off = gd->x_ovly_offset;
         ys_off = gd->y_ovly_offset;
      }
      xdl_film_image_drawsymb (gd, gd->wdisp, 
                            x + xs_off, y + ys_off,
                            gd->symbols[iposn].symb, 
                            (gd->symbols[iposn].color_flags&0XF));
   }

   /* Sort symbols list */

   s=gd->symbols;
   y_max=(gd->img_height+1)*(gd->nyp);
   for(i=0;i<gd->num_symb;++i)
   {
      s[i].indx = i;
      s[i].temp = ((y_max-s[i].y_pixel)<<16)+s[i].x_pixel;
   }
   l=gd->num_symb/2;
   ir=gd->num_symb-1;
   for (;;)
   {
      if(l>0){
         l--;
         it = s[l].indx;
         tempyx = s[it].temp;
      }
      else
      {
         it = s[ir].indx;
         tempyx = s[it].temp;
         s[ir].indx = s[0].indx;
         ir--;
         if (ir==0){
            s[0].indx =it;
            break;
         }
      }
      i = l;
      j = 2*l+ 1;
      while (j<=ir)
      {
         if (j<ir)
         {
            if (s[s[j].indx].temp<s[s[j+1].indx].temp)
            {
               j++;
            }
            else if (s[s[j].indx].temp==s[s[j+1].indx].temp &&
                     s[j].indx<s[j+1].indx)
            {
               j++;
            }
         }
         if (tempyx<s[s[j].indx].temp)
         {
            s[i].indx=s[j].indx;
            i = j;
            j = j + i + 1;
         }
         else if (tempyx==s[s[j].indx].temp&&it<s[j].indx)
         {
            s[i].indx =s[j].indx;
            i = j;
            j = j + i + 1;
         }
         else
         {
            j = ir + 1;
         }
      }
      s[i].indx = it;
   }

   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].x_pixel;
   for(i=0;i<gd->num_symb;++i)s[i].x_pixel=(unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].y_pixel;
   for(i=0;i<gd->num_symb;++i)s[i].y_pixel=(unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].x_disp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].x_disp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].y_disp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].y_disp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].x_nxp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].x_nxp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].y_nyp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].y_nyp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].symb;
   for(i=0;i<gd->num_symb;++i)s[i].symb=(unsigned char)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].color_flags;
   for(i=0;i<gd->num_symb;++i)s[i].color_flags=(unsigned char)s[s[i].indx].temp;

   /* Update y-search offsets array */

   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i]=0;
   y_start_prev=0;
   for (is=0;is<gd->num_symb;++is)
   {
      y_start = (gd->symbols[is].y_disp_offset * 
                (Y_SEARCH_MAX-1)) / gd->img_height;
      if (y_start<0) y_start = 0;
      if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
      if (y_start>y_start_prev)
      {
         if (y_start-1<Y_SEARCH_MAX)
         {
            for(i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i]=is;
         }
      }
      y_start_prev=y_start;
   }

   /* Refresh magnifying window if required */

   if (gd->ovly1==0&&gd->ovly2==0) return 0;
   xdl_film_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_symbols */

void xdlf_film_image_symbols (ivh, nsymb, ixpix, iypix, isymb, icolr, 
                             ov_num, nt, ierr)
int *ivh, *nsymb, *ixpix, *iypix, *isymb, *icolr, *ov_num, *nt, *ierr;
{
   *ierr = xdl_film_image_symbols (*ivh, *nsymb, ixpix, iypix, isymb, icolr,
                                  ov_num, *nt);
   return;
}

/*-Routine: Clear overlay symbols - xdl_film_image_clear_symbols
The routine xdl_film_image_clear_symbols (xdlf_film_image_clear_symbols) is
used to clear all overlay symbols from the symbols list and from the
display.
-end*/
/*
******************************************
**     xdl_film_image_clear_symbols     ** 
******************************************

Purpose: Clear table of symbols and clear symbols from screen

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_CLEAR_SYMBOLS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_film_image_clear_symbols 
              call
-end*/

/*-C:*/
int xdl_film_image_clear_symbols (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */
   int i;                  /* Loop counter */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_clear_symbols",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_clear_symbols", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (gd->symbols!=NULL) free (gd->symbols);
   gd->symbols = NULL;
   gd->max_alloc_symbols = 0;
   gd->num_symb = 0;
   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i] = 0;
   xdl_film_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_clear_symbols */

void xdlf_film_image_clear_symbols (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_film_image_clear_symbols (*ivh);
   return;
}

/*-Routine: Delete an overlay symbol - xdl_film_image_del_symbol
The routine xdl_film_image_del_symbol (xdlf_film_image_del_symbol) is used
to delete an overlay symbol from the symbols list and remove it from the
display. If there are many symbols to delete it may be quicker to clear
all symbols (xdl_film_image_clear_symbols (xdlf_film_image_clear_symbols))
and create a new list using xdl_film_image_symbols (xdlf_film_image_symbols)
-end*/
/*
***************************************
**     xdl_film_image_del_symbol     ** 
***************************************

Purpose: Delete a symbol

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_DEL_SYMBOL (IVH, IXPIX, IYPIX, ISYMB,
        +                                 ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXPIX    (R)  X pixel position of symbol (full film image pixel
              numbering) (see x_pixel)
IYPIX    (R)  Y pixel position (see y_pixel)
ISYMB    (R)  If > 0, only delete the symbol if it is of the 
              type ISYMB (see symbol)
ICOLR    (R)  If > 0, only delete the symbol if it is of the 
              colour ICOLR (see color)
IOV      (R)  If > 0, only delete the symbol if it is of the
              overlay number IOV (see ov_num)
IERR     (W)  Returns status from xdl_film_image_del_symbol call
-end*/

/*-C:*/
int xdl_film_image_del_symbol (vh, x_pixel, y_pixel, symbol, color, 
                               ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int x_pixel;     /* x pixel position of the symbol (from 1 up - refers 
                    to complete uncompressed image) (R)*/
int y_pixel;     /* y pixel position of the symbol (from 1 up - refers 
                    to complete uncompressed image) (R)*/
int symbol;      /* If > 0 then only delete the symbol if it is of the
                    type 'symbol' (see xdl_film_image_symbol routine for
                    details of symbol types) (R)*/
int color;       /* If > 0 then only delete the symbol if it is of the 
                    colour 'color' (see xdl_film_image_symbol routine 
                    for details of the colour types) (R)*/
int ov_num;      /* If > 0 then only delete the symbol if it is of
                    the overlay number ov_num (1 or 2) (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Symbol not found
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_film_image *gd;       /* Pointer to global data area */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int iposn;                /* Position (offset) in symbols list for
                                symbol to be deleted*/
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   XDl_film_image_symbol *smb;      /* Temp. pointer to symbol structure */
   int x_d, y_d;                    /* x, y pixel offsets of symbol in main
                                       film display area */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted symbol area (also vectors)*/
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted symbol area (also vectors) */
   int s_start;                     /* Start position in symbols list for search
                                       */
   int match;                       /* Valid match found flag =1 yes, =0 no */
   int ovn;                         /* Overlay number */
   int xs_off;                      /* Overlay symbol x offset */
   int ys_off;                      /* Overlay symbol y offset */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_del_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_del_symbol", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (gd->num_symb==0) return 4;

   for (iposn=0;iposn<gd->num_symb;++iposn)
   {
      smb = &(gd->symbols[iposn]);
      match = 0;
      if (x_pixel==smb->x_pixel&&y_pixel==smb->y_pixel) match = 1;
      if (color>0&&color!=(smb->color_flags&0XF)) match = 0;
      if (symbol>0&&symbol!=smb->symb) match = 0;
      ovn = 1; if ((smb->color_flags&0X10)!=0) ovn=2;
      if (ov_num>0&&ov_num!=ovn) match = 0;
      if (match)
      {
         xs_min = smb->x_disp_offset - MAX_SYMB_SIZE;
         xs_max = smb->x_disp_offset + MAX_SYMB_SIZE;
         ys_min = smb->y_disp_offset - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
         ys_max = smb->y_disp_offset + MAX_SYMB_SIZE + abs(gd->y_ovly_offset);
         y_start = (smb->y_disp_offset * (Y_SEARCH_MAX-1)) / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         if (y_start+1<Y_SEARCH_MAX)
         {
            for (i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i] -= 1;
         }
         if (iposn<gd->num_symb-1)
         {
            tp = (char *) gd->symbols;
            ssiz = sizeof (XDl_film_image_symbol);
            for (i=iposn*ssiz;i<(gd->num_symb*ssiz-ssiz);++i)
                 tp[i] = tp[i+ssiz];
         }
         gd->num_symb -= 1;
         XPutImage (XDL_display, gd->wdisp, gd->paint_gc,
                    gd->img_image, xs_min, ys_min, xs_min, ys_min,
                    (xs_max - xs_min + 1), (ys_max - ys_min + 1));
         if (gd->magnify_on==0) xdl_film_image_set_freeze (gd);
         if (gd->ovly1==0&&gd->ovly2==0) return 0;

         xdl_film_image_rdv (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);

         xs_min -= MAX_SYMB_SIZE;
         xs_max += MAX_SYMB_SIZE;
         ys_min -= MAX_SYMB_SIZE;
         ys_max += MAX_SYMB_SIZE;

         xdl_film_image_magw (gd);

         if (gd->num_symb==0) return 0;
         y_start = ((ys_min)*(Y_SEARCH_MAX-1)) 
                   / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start >= Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         s_start = gd->y_search[y_start];
         if (s_start<gd->num_symb)
         {
            for (i=s_start;i<gd->num_symb;++i)
            {
               smb = &(gd->symbols[i]);
               if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
               if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
               xs_off = 0;
               ys_off = 0;
               if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
               {
                  xs_off = gd->x_ovly_offset;
                  ys_off = gd->y_ovly_offset;
               }
               x_d = smb->x_disp_offset + xs_off;
               y_d = smb->y_disp_offset + ys_off;
               if (x_d<xs_min) continue;
               if (x_d>xs_max) continue;
               if (y_d<ys_min) continue;
               if (y_d>ys_max) break;
               xdl_film_image_drawsymb (gd, gd->wdisp, x_d, y_d,
                                        smb->symb, (smb->color_flags&0XF));
            }
         }
         XFlush (XDL_display);
         return 0;
      }
   }
   XFlush (XDL_display);
   return 4;
}

/* Fortran binding: xdlf_film_image_del_symbol */

void xdlf_film_image_del_symbol (ivh, ixpix, iypix, isymb, icolr, iov, ierr)
int *ivh, *ixpix, *iypix, *isymb, *icolr, *iov, *ierr;
{
   *ierr = xdl_film_image_del_symbol (*ivh, *ixpix, *iypix, *isymb, *icolr,
                                      *iov);
   return;
}
/*-Routine: Store and display an overlay vector - xdl_film_image_vect
The routine xdl_film_image_vect (xdlf_film_image_vect) is used to
store details of an overlay vector and to display that vector if
overlays are currently being displayed. Two sets of overlay vectors
may be stored. The user supplies a vector identifier number for a
vector and this is used to identify a vector or set of vectors for 
deletion if required.
-end*/
/*
**********************************
**     xdl_film_image_vect      ** 
**********************************

Purpose: Store and display an overlay vector

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_VECT (IVH, IVECT, IX1PIX, IY1PIX, 
        +                           IX2PIX, IY2PIX,
        +                           ICOLR, IOV, MAGUPD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  User selected number to identify vector
              (0-65535). The same number may be used
              for more than one vector to identify a
              set of vectors (see vect_id)
IX1PIX   (R)  X start pixel position in full image (see x1_pixel)
IY1PIX   (R)  Y start pixel position in full image (see y1pixel)
IX2PIX   (R)  X end pixel position in full image (see x2_pixel)
IY2PIX   (R)  Y end pixel position in full image (see y2pixel)
ICOLR    (R)  Vector colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
MAGUPD   (R)  =1 Update magnifying window after drawing vector,
              =0 do not (see magw_upd)
IERR     (W)  Returns status from xdl_film_image_vect call
-end*/

/*-C:*/
int xdl_film_image_vect (vh, vect_id, x1_pixel, y1_pixel, 
                         x2_pixel, y2_pixel, color, ov_num, magw_upd)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int vect_id;     /* User selected number to identify vector (0-65535). 
                    The same number may be used for more than one vector to 
                    identify a set of vectors (R)*/
int x1_pixel;    /* The vector start pixel 'x' position (from 1 up - refers 
                    to complete uncompressed image (R) */
int y1_pixel;    /* The vector start pixel 'y' position (R) */
int x2_pixel;    /* The vector end pixel 'x' position (R) */
int y2_pixel;    /* The vector end pixel 'y' position (R) */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default)*/
int ov_num;      /* Overlay number 1 or 2 (R)*/
int magw_upd;    /* =1 Update the magnifying window after drawing the 
                       vector; this should be used when a single vector 
                       is drawn or when the final vector of a series of 
                       vectors is drawn.
                    =0 Do not; this enables a series of vectors to be
                       drawn much more rapidly; it should be used when
                       a series of vectors is to be drawn for all but
                       the final vector (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new vector's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_film_image *gd;       /* Pointer to global data area */
   XDl_film_image_vect *p;   /* Temp. pointer to this structure */
   int x, y;                 /* Temp x, y positions */
   int x1,x2;                /* x pixel position offsets in main film display 
                                area (display area pixels) */
   int y1,y2;                /* y pixel position offsets in main film display 
                                area (display area pixels) */
   int xoff1,xoff2;          /* x Offsets of pixels (film image pixels) in 
                                display area*/
   int yoff1,yoff2;          /* Offsets of pixels (film image pixels) from
                                top of film display area */
   int iposn;                /* Position (offset) in vectors list for new
                                vector */
   int x1_p;                 /* x Offset within main display pixel 
                                (0 to nxp-1)*/
   int x2_p;                 /* x Offset within main display pixel 
                                (0 to nxp-1)*/
   int y1_p;                 /* y Offset within main display pixel 
                                (0 to nyp-1)*/
   int y2_p;                 /* y Offset within main display pixel 
                                (0 to nyp-1)*/
   int xs_off;               /* Overlay vector x offset */
   int ys_off;               /* Overlay vector y offset */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_vect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_vect", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;
   if (gd->num_vect>=gd->max_alloc_vects)
   {
      if (gd->vects==NULL)
      {
         p = (XDl_film_image_vect *) malloc (
           (gd->max_alloc_vects+NVECT_ALLOC) * sizeof(XDl_film_image_vect));
         if (p==0) return 4;
      }
      else
      {
         p = (XDl_film_image_vect *) realloc (gd->vects,
           (gd->max_alloc_vects+NVECT_ALLOC) * sizeof(XDl_film_image_vect));
         if (p==0) return 4;
      }
      gd->vects = p;
      gd->max_alloc_vects += NVECT_ALLOC;
   }

   iposn = gd->num_vect;
   gd->num_vect += 1;

   /* Save details for the new vector */
         
   gd->vects[iposn].vect_id = vect_id;
   xoff1 = x1_pixel - gd->xo_pix;
   xdl_film_image_dsppos(xoff1,gd->nxp,&x1,&x1_p);
   xoff2 = x2_pixel - gd->xo_pix;
   xdl_film_image_dsppos(xoff2,gd->nxp,&x2,&x2_p);
   yoff1 = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 - y1_pixel;
   xdl_film_image_dsppos(yoff1,gd->nyp,&y1,&y1_p);
   yoff2 = (gd->nyp)*(gd->img_height) + gd->yo_pix - 1 - y2_pixel;
   xdl_film_image_dsppos(yoff2,gd->nyp,&y2,&y2_p);

   /*If y2<y1 swap ends of vector before storage */

   if (y2<y1)
   {
      x = x1;
      y = y1;
      x1 = x2;
      y1 = y2;
      x2 = x;
      y2 = y;
      x = x1_p;
      x1_p = x2_p;
      x2_p = x;
      y = y1_p;
      y1_p = y2_p;
      y2_p = y;
   }
   gd->vects[iposn].x1_disp_offset = x1;
   gd->vects[iposn].y1_disp_offset = y1;
   gd->vects[iposn].x1_nxp_offset = x1_p;
   gd->vects[iposn].y1_nyp_offset = y1_p;
   gd->vects[iposn].x2_disp_offset = x2;
   gd->vects[iposn].y2_disp_offset = y2;
   gd->vects[iposn].x2_nxp_offset = x2_p;
   gd->vects[iposn].y2_nyp_offset = y2_p;
   gd->vects[iposn].color_flags = color;
   if (ov_num==2) gd->vects[iposn].color_flags |= 0X10;
   if (gd->ovly_typ==2||ov_num==2) gd->vects[iposn].color_flags |= 0X20;
     
   /* Draw vector if required */

   if (gd->ovly1==0&&ov_num==1) return 0;
   if (gd->ovly2==0&&ov_num==2) return 0;
   xs_off = 0;
   ys_off = 0;
   if (gd->ovly_offset==1&&(gd->vects[iposn].color_flags&0X20)==0)
   {
       xs_off = gd->x_ovly_offset;
       ys_off = gd->y_ovly_offset;
   }
   xdl_film_image_drawvec (gd, gd->wdisp, 
                           x1 + xs_off, y1 + ys_off,
                           x2 + xs_off, y2 + ys_off,
                           color);
   if (magw_upd) xdl_film_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_vect */

void xdlf_film_image_vect (ivh, ivect, ix1pix, iy1pix, ix2pix, iy2pix, 
                           icolr, iov, magupd, ierr)
int *ivh, *ivect, *ix1pix, *iy1pix, *ix2pix, *iy2pix, *icolr, *iov, *magupd;
int *ierr;
{
   *ierr = xdl_film_image_vect (*ivh, *ivect, *ix1pix, *iy1pix, 
                                *ix2pix, *iy2pix, *icolr, *iov, *magupd);
   return;
}

/*-Routine: Clear overlay vectors - xdl_film_image_clear_vectors
The routine xdl_film_image_clear_vects (xdlf_film_image_clear_vects) is
used to clear all overlay vectors from the vectors list and from the
display.
-end*/
/*
******************************************
**     xdl_film_image_clear_vects       ** 
******************************************

Purpose: Clear table of vectors and clear vectors from screen

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_CLEAR_VECTS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_film_image_clear_vects
              call
-end*/

/*-C:*/
int xdl_film_image_clear_vects (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */
   int i;                  /* Loop counter */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_clear_vects",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_clear_vects", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (gd->vects!=NULL) free (gd->vects);
   gd->vects = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
   xdl_film_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_clear_vects */

void xdlf_film_image_clear_vects (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_film_image_clear_vects (*ivh);
   return;
}

/*-Routine: Delete an overlay vector - xdl_film_image_del_vect
The routine xdl_film_image_del_vect (xdlf_film_image_del_vect) is used
to delete an overlay vector (or set of vectors with a common vector
identifier) from the vectors list and remove it (them) from the
display. If there are many vectors to delete it may be quicker to clear
all vectors (xdl_film_image_clear_vects (xdlf_film_image_clear_vects))
and create a new list using xdl_film_image_vect (xdlf_film_image_vect)
-end*/
/*
***************************************
**     xdl_film_image_del_vector     **
***************************************

Purpose: Delete an overlay vector or set of vectors

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_DEL_VECT (IVH, IVECT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  The vectot identifier of the vector or set of
              vectors to be deleted (see vect_id)
IERR     (W)  Returns status from xdl_film_image_del_vect call
-end*/

/*-C:*/
int xdl_film_image_del_vect (vh, vect_id)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int vect_id;     /* Vector identifier for the vector or set of vectors
                    to be deleted (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Vector not found
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_film_image *gd;       /* Pointer to global data area */
   char *tp1,*tp2;            /* Temp character pointers, used in copying 
                                vector structures */
   int iposn;                /* Position (offset) in vectors list for
                                current accepted vector */
   int i;                    /* Loop counter */
   int ii;                   /* Loop counter through vectors */
   int match;                /* Matching vector(s) found */
   int ssiz;                 /* Size of vectors structure */
   XDl_film_image_vect *vec;        /* Temp. pointer to vector structure */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted vectors area (also vectors)*/
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted vector area (also vectors) */
   int xs_off;                      /* Overlay x offset */
   int ys_off;                      /* Overlay y offset */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_del_vect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_del_vect", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (gd->num_vect==0) return 4;

   xs_min = gd->img_width - 1;
   xs_max = 0;
   ys_min = gd->img_height - 1;
   ys_max = 0;

   iposn = 0;
   match = 0;
   for (ii=0;ii<gd->num_vect;++ii)
   {
      vec = &(gd->vects[ii]);
      if (vec->vect_id==vect_id)
      {
         match = 1;
         if (vec->x1_disp_offset<xs_min) xs_min = vec->x1_disp_offset;
         if (vec->x1_disp_offset>xs_max) xs_max = vec->x1_disp_offset;
         if (vec->y1_disp_offset<ys_min) ys_min = vec->y1_disp_offset;
         if (vec->y1_disp_offset>ys_max) ys_max = vec->y1_disp_offset;
         if (vec->x2_disp_offset<xs_min) xs_min = vec->x2_disp_offset;
         if (vec->x2_disp_offset>xs_max) xs_max = vec->x2_disp_offset;
         if (vec->y2_disp_offset<ys_min) ys_min = vec->y2_disp_offset;
         if (vec->y2_disp_offset>ys_max) ys_max = vec->y2_disp_offset;
         continue;
      }
      if (match)
      {
         tp1 = (char *) &gd->vects[iposn];
         tp2 = (char *) &gd->vects[ii];
         ssiz = sizeof (XDl_film_image_vect);
         for (i=0;i<ssiz;++i) tp1[i] = tp2[i];
      }
      iposn++;
   }
   gd->num_vect = iposn;

   if (match==0) return 4;

   XPutImage (XDL_display, gd->wdisp, gd->paint_gc,
              gd->img_image, xs_min, ys_min, xs_min, ys_min,
              (xs_max - xs_min + 1), (ys_max - ys_min + 1));
   if (gd->magnify_on==0) xdl_film_image_set_freeze (gd);
   if (gd->ovly1==0&&gd->ovly2==0) return 0;

   xdl_film_image_rdv (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);

   xs_min -= MAX_SYMB_SIZE;
   xs_max += MAX_SYMB_SIZE;
   ys_min -= MAX_SYMB_SIZE;
   ys_max += MAX_SYMB_SIZE;

   xdl_film_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);

   xdl_film_image_magw (gd);

   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_del_vect */

void xdlf_film_image_del_vect (ivh, ivect, ierr)
int *ivh, *ivect, *ierr;
{
   *ierr = xdl_film_image_del_vect (*ivh, *ivect);
   return;
}

/*-Routine: Reset active strip message - xdl_film_image_input_message
The routine xdl_film_image_input_message (xdlf_film_image_input_message) is
used to reset the message which will appear in the active strip when a
pixel position or selected rectangle is to be input. When the film image 
view-object is created, the default message is 'Input spot positions'.
-end*/
/*
******************************************
**     xdl_film_image_input_message     ** 
******************************************

Purpose: Reset 'Input spot positions' message for the active strip area

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_INPUT_MESSAGE (IVH, XDLSTR(MESSAGE), 
        +                                    LEN, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MESSAGE  (R)  Character string containing the message 
              (see message)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the message string - must be >0 
              (cf len) (max 60 chars) 
IERR     (W)  Returns status from xdl_film_image_input_message 
              call
-end*/

/*-C:*/
int xdl_film_image_input_message (vh, message, len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *message;   /* Message string (max of 60 chars long) (R)*/
int len;         /* Length of the message string; if 0 then length 
                    will be found assuming a null terminated string 
                    (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_film_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_film_image_input_message",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1081)
   {
      xdl_view_type_errmsg ("xdl_film_image_input_message", vh, "xdl_film_image");
      return 2;
   }
   gd = (XDl_film_image *) XDL_view_objects[iv]->gd;
   if (len==0) len = strlen (message);
   if (len>60) len = 60;
   strncpy (gd->input_message, message, len);
   gd->input_message[len] = 0;
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_film_image_input_message */

void xdlf_film_image_input_message (ivh, message, len, ierr)
int *ivh, *message, *len, *ierr;
{
   *ierr = xdl_film_image_input_message (*ivh, XDL_pointers[*message], *len);
   return;
}

/*-Routine: Calculate the size requirements - xdl_film_image_getsize
The routine xdl_film_image_getsize (xdlf_film_image_getsize) is used to
calculate the size of a film image view-object given the size of the
image to be displayed.
-end*/
/*
*************************************
**     xdl_film_image_getsize      **
*************************************

Purpose: Return the size requirements of the film image view-object

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_FILM_IMAGE_GETSIZE (IMGW, IMGH, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
IMGW       (R)  Displayed film image width (see img_width)
IMGH       (R)  Displayed film image height (see img_height)
IWIDTH     (W)  Returns the width required (see w)
IHEIGHT    (W)  Returns the height required (see h)
-end*/

/*-C:*/
void xdl_film_image_getsize (img_width, img_height, w, h)
/*end*/
 
/*-Parameters:*/
int img_width;           /* Displayed image width (R)*/
int img_height;          /* Displayed image height (R)*/
int *w;                  /* Returns the width required (W)*/
int *h;                  /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int width;   /* Calc width */
   int height;  /* Calc height */
   int ytop;    /* Height required for top of object items */

   /* Calculate the size requirements */

   width = 355 + MAGWSIZE_W;
   if ((img_width+2+2*BORDER_WIDTH+2)>width) 
      width = img_width + 2 + 2*BORDER_WIDTH + 2;
   ytop = 66 + BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   if (MAGWSIZE_H+2+ACT_STRIP_HEIGHT>ytop) ytop = MAGWSIZE_H+2+ACT_STRIP_HEIGHT;
   height = img_height + 2 + 2*BORDER_HEIGHT + 2 + ytop;
   *w = width;
   *h = height;
   return;
}

/* Fortran binding: xdlf_film_image_getsize */

void xdlf_film_image_getsize (imgw, imgh, iwidth, iheight)
int *imgw, *imgh, *iwidth, *iheight;
{
   xdl_film_image_getsize (*imgw, *imgh, iwidth, iheight);
   return;
}
