/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Graph Window View-Object Routines
-end*/
 
/*-Intro:
The graphics window view-object provides an area for the output of
graphics. A number of routines are supplied to give access to a basic
set of line plotting functions including drawing of rectangles, polygons
and arcs. Filled areas may also be drawn. Symbol and text plotting functions 
are also available and there are two simple axis drawing routines. An option 
is available to produce Postscript files from the plots in order to obtain 
hard copies. Sections of the plot may be identified by an graphics object group 
identifier enabling parts of the plot to be deleted or redrawn under program 
control. Simple cursor input is provided. A number of options are available 
for defining the mapping of the user coordinates onto the window and for 
handling possible re-sizing of the graphics window. When plotting, a current
graphics style is used; this determines the colour, the line width and
the X-font to be used. This style may be changed when required.
-end*/
 
/*-Section: Creating and Handling a Graph Window View-Object
-end*/
 

/*Routines List:
The I/O window view-object has the following routines available:

   xdl_graph_win               Create a new graphics window view-object
   xdl_graph_win_map           Set mapping option
   xdl_graph_win_style         Set style parameters
   xdl_graph_win_set_colour    Set a colour
   xdl_graph_win_line          Draw a line
   xdl_graph_win_polyline      Draw a polyline
   xdl_graph_win_stext         Draw a text string with scaleable font
   xdl_graph_win_xtext         Draw a text string using a standard font
   xdl_graph_win_symbol        Draw a scaleable symbol
   xdl_graph_win_polysymbol    Draw scaleable polysymbols
   xdl_graph_win_rectangle     Draw a rectangle (may be filled)
   xdl_graph_win_polygon       Draw a polygon (may be filled)
   xdl_graph_win_arc           Draw an arc (may be filled)
   xdl_graph_win_ax1           Draw horizontal axis
   xdl_graph_win_ax2           Draw vertical axis
   xdl_graph_win_getcurs       Get cursor position from a graphic window
   xdl_graph_win_input_message Change message in active strip
   xdl_graph_win_setid         Reset current object group id
   xdl_graph_win_delete        Delete groups of graphics objects
   xdl_graph_win_getsize       Calculate size requirements
*/

/*-Routine: Create a graphics window view-object - xdl_graph_win
The routine xdl_graph_win (xdlf_graph_win) is used to create a new graphics
window view-object. The user may specify the size of the window in
pixels. The required options for handling the possible re-sizing of the
graphics window are selected. On a colour display a default colour map
of 8 colours (black, white, red, yellow, green, cyan, blue and
magenta) is always available. If required, the user may define an
additional colour map to be used to provide additional colours. Such
colours need to be set via calls to the routine xdl_graph_win_set_colour
(xdlf_graph_win_set_colour) before being used.
-end*/
/*
************************
**    xdl_graph_win   **
************************

Purpose: Display a window for program graphics

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN (IVH, IVHPARENT, IX, IY, ICSET, NCOLRS, 
        +                     IGWIDTH, IGHEIGHT, 
        +                     MINW, MINH, IEXPAND, IPOSN, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number for optional additional
                colorset of NCOLRS colours (0 if not needed) 
                (see cset)
NCOLRS    (R)   No. of colours in additional colorset
IGWIDTH   (R)   Required width of graphics area in pixels
                excluding border (see gr_width)
IGHEIGHT  (R)   Required height of graphics area in pixels
                excluding border (see gr_height)
MINW      (R)   Minimum width of view-object in pixels 
                (see min_width)
MINH      (R)   Minimum height of view-object in pixels 
                (see min_height)
IEXPAND   (R)   Flag for handling of expanded/contracted
                graphics area.
                = 0, do not expand/contract
                = 1, expand/contract to fit window size but
                     preserve ratio of axes
                = 2, expand/contact in both directions to
                     fit window
IPOSN     (R)   = 0, Position in centre of window
                = 1, Position at bottom left of window
IERR      (W)   Returns status from xdl_graph_win call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"
#include <math.h>

#define MARG_W 2
#define MARG_H 2
#define MIN_ACT_STRIP 150      /* Minimum width for active strip */
#define NOBJ_ALLOC   100       /* No. of graphics objects to define at a time */
#define STYLE_ALLOC  20        /* No. of graphics style structures to be
                                  allocated at a time */
#define POINTS_ALLOC 50        /* No. of 'X' points to add to 'X' points
                                  list array at a time */

/* Graphic object type definitions */

#define SYMBOL_OBJ       1        /* symbol */
#define POLYSYMBOL_OBJ   2        /* polysymbol */
#define LINE_OBJ         3        /* line */
#define POLYLINE_OBJ     4        /* polyline */
#define RECT_OBJ         5        /* rectangle (unfilled) */
#define POLYGON_OBJ      6        /* polygon (unfilled) */
#define ARC_OBJ          7        /* arc (unfilled) */
#define FILLRECT_OBJ     8        /* rectangle (filled) */
#define FILLPOLYGON_OBJ  9        /* polygon (filled) */
#define FILLSECTARC_OBJ  10       /* arc (sector filled) */
#define FILLCHORDARC_OBJ 11       /* arc (chord filled) */
#define STEXT_OBJ        12       /* scaleable text (internal font) positioned
                                     from bottom left) */
#define STEXTCENT_OBJ    13       /* scaleable text (internal font) centered */
#define TEXT_OBJ         14       /* text (standard fonts). Horizontal
                                     only. Positioned from bottom left */
#define TEXTCENT_OBJ     15       /* text (standard fonts). Horizontal
                                     only. Centered. */

/* Define nearest integer macro */

#define nint(x)  x>=0.0?(int)(x+0.5):(int)(x-0.5)

/* Internal font */

static short qsfont[95][32] = {
/* space !"#$%&'()*+,-./ */
{100},
{6,1,6,2,-6,5,6,15,100},
{4,11,5,14,4,15,-7,11,8,14,7,15,100},
{4,3,4,14,-7,3,7,14,-2,7,9,7,-2,10,9,10,100},
{5,1,5,15,-1,4,3,2,7,2,9,4,9,6,7,8,3,8,1,10,1,12,3,14,7,14,9,12,100},
{2,1,9,15,-2,11,4,11,4,14,2,14,2,11,-7,2,9,2,9,5,7,5,7,2,100},
{9,1,4,12,4,14,5,15,7,15,8,14,8,12,4,8,2,5,2,3,3,1,6,1,9,4,100},
{5,15,6,14,5,11,100},
{7,15,5,13,4,9,4,6,5,3,7,1,100},
{4,15,6,13,7,9,7,6,6,3,4,1,100},
{6,2,6,14,-9,3,2,13,-10,8,1,8,-9,13,2,3,100},
{6,2,6,10,-10,6,2,6,100},
{6,2,6,1,5,1,5,2,6,2,7,0,4,-2,100},
{2,6,10,6,100},
{4,2,5,2,5,3,4,3,4,2,-3,1,6,1,6,4,3,4,3,1,100},
{3,1,7,15,100},
/* 0123456789 */
{2,1,9,15,-2,3,2,13,4,15,7,15,9,13,9,3,7,1,4,1,2,3,100},
{4,12,6,15,6,1,-3,1,9,1,100},
{3,9,2,11,2,13,4,15,8,15,10,12,9,9,7,6,2,1,9,1,100},
{2,15,10,15,7,10,10,8,10,3,7,1,4,1,2,3,100},
{8,15,2,4,10,4,-7,1,7,7,100},
{10,15,2,15,2,8,4,10,7,10,10,7,10,3,7,1,4,1,2,3,100},
{9,15,3,9,2,6,2,3,5,1,8,1,10,4,10,7,8,9,3,9,100},
{2,15,9,15,2,1,-3,8,9,8,100},
{4,8,2,10,2,13,4,15,8,15,10,13,10,9,8,8,4,8,2,6,2,3,4,1,8,1,10,3,10,6,8,8},
{2,1,8,7,9,10,9,13,6,15,3,15,1,12,1,9,3,7,8,7,100},
/* :;<=>?@ */
{4,1,4,2,5,2,5,1,4,1,-4,8,4,9,5,9,5,8,4,8,100},
{6,2,6,1,5,1,5,2,6,2,7,0,4,-2,-5,8,5,9,6,9,6,8,5,8,100},
{10,1,1,6,10,10,100},
{1,4,10,4,-1,8,10,8,100},
{1,1,10,6,1,10,100},
{2,11,2,13,3,15,6,15,8,14,9,12,8,9,5,7,5,4,-5,2,5,1,100},
{10,2,7,1,3,1,1,3,1,7,3,10,8,10,10,7,10,4,7,3,7,8,4,8,3,6,3,5,4,3,7,3},
/* ABCDEFGHIJKLM */
{1,1,1,12,4,15,8,15,10,12,10,1,-1,7,10,7,100},
{1,1,1,15,7,15,9,13,9,10,7,8,1,8,-7,8,9,6,9,3,7,1,1,1,100},
{9,3,7,1,3,1,1,4,1,12,3,15,7,15,9,13,100},
{1,1,1,15,7,15,9,14,10,12,10,4,9,2,7,1,1,1,100},
{10,1,1,1,1,15,10,15,-1,8,7,8,100},
{1,1,1,15,10,15,-1,8,7,8,100},
{10,13,8,15,3,15,1,13,1,3,3,1,8,1,10,3,10,6,6,6,100},
{1,1,1,15,-9,1,9,15,-1,8,9,8,100},
{5,1,5,15,-3,15,7,15,-3,1,7,1,100},
{6,15,6,3,4,1,2,1,1,3,-3,15,9,15,100},
{1,1,1,15,-1,9,9,15,-1,9,9,1,100},
{1,15,1,1,9,1,100},
{1,1,1,15,5,9,9,15,9,1,100},
/* NOPQRSTUVWXYZ */
{1,1,1,15,9,1,9,15,100},
{8,1,3,1,1,4,1,12,3,15,8,15,10,12,10,4,8,1,100},
{1,1,1,15,7,15,9,13,9,9,7,7,1,7,100},
{8,1,3,1,1,4,1,12,3,15,8,15,10,12,10,4,8,1,-6,5,10,1,100},
{1,1,1,15,7,15,9,13,9,9,7,7,1,7,-1,7,10,1,100},
{1,3,3,1,8,1,10,3,10,6,8,8,3,8,1,10,1,13,3,15,8,15,10,13,100},
{5,1,5,15,-1,15,9,15,100},
{1,15,1,4,2,2,4,1,6,1,8,2,9,4,9,15,100},
{1,15,5,1,9,15,100},
{1,15,2,1,5,7,8,1,9,15,100},
{1,1,9,15,-1,15,9,1,100},
{1,15,5,9,5,1,-5,9,9,15,100},
{1,15,9,15,1,1,9,1,100},
/* [\]^_` */
{8,15,4,15,4,1,8,1,100},
{1,15,10,1,100},
{3,15,6,15,6,1,3,1,100},
{3,12,5,15,7,12,100},
{1,-1,10,-1,100},
{5,15,5,14,6,14,6,15,5,15,3,13,6,11,100},
/* abcdefghijklm */
{1,8,3,10,8,10,10,8,10,1,2,1,1,2,1,4,2,5,10,5,100},
{1,15,1,1,8,1,10,3,10,8,8,10,3,10,1,8,100},
{10,9,8,10,4,10,1,8,1,3,4,1,8,1,10,2,100},
{10,15,10,1,3,1,1,3,1,8,3,10,8,10,10,8,100},
{1,6,10,6,10,8,8,10,3,10,1,8,1,3,3,1,8,1,10,3,100},
{4,1,4,13,5,15,8,15,10,13,-2,7,6,7,100},
{10,1,3,1,1,3,1,8,3,10,10,10,10,-2,8,-4,3,-4,1,-2,100},
{1,15,1,1,-1,8,3,10,8,10,10,8,10,1,100},
{3,10,5,10,5,1,-3,1,8,1,-5,13,5,14,100},
{4,10,6,10,6,-1,5,-3,3,-4,1,-3,-6,13,6,14,100},
{1,1,1,15,-1,6,9,12,-4,8,9,1,100},
{5,15,6,15,6,1,-3,1,9,1,100},
{1,1,1,10,-1,8,3,10,4,10,5,8,5,1,-5,8,7,10,8,10,9,8,9,1,100},
/* nopqrstuvwxyz */
{1,10,1,1,-1,8,3,10,8,10,9,7,9,1,100},
{1,3,1,8,3,10,8,10,10,8,10,3,8,1,3,1,1,3,100},
{1,-4,1,10,7,10,9,8,9,3,7,1,1,1,100},
{9,1,3,1,1,3,1,8,3,10,9,10,9,-4,100},
{1,1,1,10,-1,7,3,10,8,10,10,8,10,6,100},
{1,2,3,1,8,1,10,2,10,4,8,5,3,5,1,7,1,8,3,10,8,10,10,9,100},
{5,15,5,3,7,1,9,1,10,3,-3,11,7,11,100},
{1,10,1,3,3,1,7,1,9,3,-9,1,9,10,100},
{1,10,5,1,9,10,100},
{1,10,4,1,6,5,8,1,10,10,100},
{1,1,10,10,-1,10,10,1,100},
{1,10,1,3,3,1,7,1,9,3,-9,10,9,-2,6,-4,3,-4,1,-2,100},
{1,10,10,10,1,1,10,1,100},
/* {|}~ */
{7,1,5,2,4,4,5,7,4,8,5,9,4,12,5,14,7,15,-2,8,4,8,100},
{6,1,6,15,100},
{4,1,6,2,7,4,6,7,7,8,6,9,7,12,6,14,4,15,-7,8,9,8,100},
{1,12,2,14,4,14,7,12,9,12,10,14,100}};

/* Graphics object structure */

typedef struct
{
   unsigned char obj_typ;    /* Object type  SYMBOL_OBJ, LINE_OBJ etc
                                             (see #defines above)
                             */
   char obj_id;              /* Object group id 1-127 (-1 for graphlib) */
   unsigned short style_off; /* Offset in graphics styles array */
   float ax1_coord;          /* First axis 1 coordinate (normalised) */
   float ax2_coord;          /* First axis 2 coordinate (normalised) */
   unsigned short nfdata;    /* No. of additional floating point data items 
                                (e.g. coordinates) */
   unsigned short ncdata;    /* No. of character data items e.g text string
                                characters */
   float* fdata;             /* Pointer to the additional floating point
                                data for the graphics object */
   char *cdata;              /* Pointer to the additional character data
                                for the graphics object */
}
XDl_graphic_object;

typedef struct
{
   unsigned long fg_colr;    /* Foreground colour */
   int      line_width;      /* Line width */
   Font     font_id;         /* Font id for text */
   int      font_off;        /* Offset for this standard font (0-4) */
}
XDl_graphic_style;


/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wgr;                       /* I/o area window id */
   Window wact;                      /* Active strip window */
   Window wpanel;                    /* Control panel window */
   Panel_object button1;             /* 'Postscript' button */
   Panel_object io_panel;            /* I/O object for Postscipt details */
   int on_off;                       /* Input mode on/off status flag 1=on,
                                        0=off */
   GC paint_gc;                      /* Graphics context for paint window */
   char        input_message[61];    /* Message for active strip */
   char        save_message[61];     /* Save message */
   int expand;                       /* Expand flag = 0, fix
                                                    = 1, expand but preserve
                                                         axial ratios 
                                                    = 2, expand both axes
                                                         to fill window */
   int posn;                         /* = 0, position graph in centre
                                        = 1, position at bottom left */
   int gr_width;                     /* Requested graphics area width */
   int gr_height;                    /* Requested graphics area height */
   int graph_width;                  /* Current graphics area width 
                                        (excluding border) */
   int graph_height;                 /* Current graphics area height
                                        (excluding border) */
   float unitsq;                     /* Size of unit square in pixels */
   float sc_ax1;                     /* Resize scaling along axis 1 */
   float sc_ax2;                     /* Resize scaling along axis 2 */
   float ax1_min;                    /* Current mapping axis 1 minimum */
   float ax1_max;                    /* Current mapping axis 1 maximum */
   float ax2_min;                    /* Current mapping axis 2 minimum */
   float ax2_max;                    /* Current mapping axis 2 maximum */
   float ax1_mapsc;                  /* Current mapping scale factor along
                                        axis 1 */
   float ax2_mapsc;                  /* Current mapping scale factor along
                                        axis 2 */
   int ax1_orig;                     /* Origin pixel offset for axis 1 wrt 
                                        bottom left of graphics area */
   int ax2_orig;                     /* Origin pixel offset for axis 2 wrt 
                                        bottom left of graphics area */
   int current_id;                   /* Current object id */
   int num_graph_obj;                /* Number of graphics objects used */
   int num_alloc_graph_obj;          /* Number of graphics objects allocated */
   XDl_graphic_object *graph_obj;    /* Pointer to list of graphics objects */
   int num_graph_style;              /* Number of graphics styles used */
   int num_alloc_graph_style;        /* Number of graphics styles allocated */
   XDl_graphic_style *graph_style;   /* Pointer to list of graphics styles */
   int current_style;                /* Offset in graph styles array for
                                        current style */
   int num_alloc_points;             /* Current capacity of points list */
   XPoint *points;                   /* Pointer to points list */
   int ps_on;                        /* Postscript request in progress flag
                                        =1 in progress, =0 not */
   FILE *fpout;                      /* Postscript o/p file pointer */
   int ps_orient;                    /* Orientation =1 portrait, 
                                        =2 landscape */
   int ps_colour;                    /* Postscript colour flag = 1 yes,
                                        = 0 no (Black and white) */
   char * title;                     /* Pointer to title for PS o/p */
   char * comments[4];               /* Pointers for comments lines for
                                        PS o/p */
   int annotate;                     /* PS extra annotation required */
   int addate;                       /* PS add heading with date */
   int addcomment;                   /* Output comments 1=yes 0=no */
   int ncomment;                     /* No. of comments up to last input
                                        non-blank comment */
   int icomment;                     /* Comment no. of longest comment */
   int lencomm;                      /* Length of longest comment so far */
   int ic;                           /* Counter through comments including
                                        blanks */
   char datstr[30];                  /* Date and time string for PS */
   int ps_current_style;             /* Current style for ps o/p */
   float ps_scale;                   /* Postscript scaling factor */
   int x_sel;                        /* x pixel position selected by
                                        cursor */
   int y_sel;                        /* y pixel position selected by
                                        cursor */
   int panel_y;                      /* Panel y coordinate */
   int panel_w;                      /* Panel width */
   int act_h;                        /* Active strip height */
   int exposed;                      /* Expose has been done flag */
   int cset;                         /* Colour set no.*/
   int ncolrs;                       /* No. colours */
   unsigned long * colr_pixels;      /* Colour pixels */

   /* Daresbury Laboratory SR computing group 2-D graphics package items */
   float gra_vp_xmin;                /* Viewport xmin */
   float gra_vp_xmax;                /* Viewport xmax */
   float gra_vp_ymin;                /* Viewport ymin */
   float gra_vp_ymax;                /* Viewport ymax */
   float gra_current_x;              /* Current point x-coordinate */
   float gra_current_y;              /* Current point y-coordinate */
   float gra_map_xmin;               /* Current mapping xmin */
   float gra_map_ymin;               /* Current mapping ymin */
   float gra_mapx;                   /* Current mapping x scaling*/
   float gra_mapy;                   /* Current mapping y scaling */
   int gra_style_off;                /* Offset in graphic style list in
                                        current use by SR package */
   int gra_cursor_inp;               /* Cursor input in progress */
}  
   XDl_graph_win;

/* Define routine types */
int xdl_graph_win();
void xdl_graph_win_rps();
void xdl_graph_win_rp0();
void xdl_graph_win_rp1();
void xdl_graph_win_ep1();
void xdl_graph_win_rp2();
void xdl_graph_win_cp1();
void xdl_graph_win_on_off();
void xdl_graph_win_tidy_up();
void xdl_graph_win_redraw();
void xdl_graph_win_setscale();
void xdl_graph_win_delobj();
void xdl_graph_win_coord_norm();
void xdl_graph_win_norm_pix();
int xdl_graph_win_obj_alloc();
int xdl_graph_win_style_alloc();
int xdl_graph_win_points_alloc();
void xdl_graph_win_draw_obj();
void xdl_graph_win_draw_symb();
void xdl_graph_win_ps_obj();
void xdl_graph_win_norm_ps();
void xdl_graph_win_postscript();

/*Graphlib interface routines */
int xdl_graph_win_gra_vp();
int xdl_graph_win_gra_map();
int xdl_graph_win_gra_moveto();
int xdl_graph_win_gra_drawto();
int xdl_graph_win_gra_text();
int xdl_graph_win_gra_curs();
int xdl_graph_win_gra_cursinp();
int xdl_graph_win_gra_exposed();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_graph_win xdlf_graph_win_
#define xdlf_graph_win_map xdlf_graph_win_map_
#define xdlf_graph_win_style xdlf_graph_win_style_
#define xdlf_graph_win_set_colour xdlf_graph_win_set_colour_
#define xdlf_graph_win_line xdlf_graph_win_line_
#define xdlf_graph_win_polyline xdlf_graph_win_polyline_
#define xdlf_graph_win_stext xdlf_graph_win_stext_
#define xdlf_graph_win_xtext xdlf_graph_win_xtext_
#define xdlf_graph_win_symbol xdlf_graph_win_symbol_
#define xdlf_graph_win_polysymbol xdlf_graph_win_polysymbol_
#define xdlf_graph_win_rectangle xdlf_graph_win_rectangle_
#define xdlf_graph_win_polygon xdlf_graph_win_polygon_
#define xdlf_graph_win_arc xdlf_graph_win_arc_
#define xdlf_graph_win_ax1 xdlf_graph_win_ax1_
#define xdlf_graph_win_ax2 xdlf_graph_win_ax2_
#define xdlf_graph_win_getcurs xdlf_graph_win_getcurs_
#define xdlf_graph_win_input_message xdlf_graph_win_input_message_
#define xdlf_graph_win_setid xdlf_graph_win_setid_
#define xdlf_graph_win_delete xdlf_graph_win_delete_
#define xdlf_graph_win_getsize xdlf_graph_win_getsize_

#endif

#if  LINKTYP == 2

#define xdlf_graph_win XDLF_GRAPH_WIN
#define xdlf_graph_win_map XDLF_GRAPH_WIN_MAP
#define xdlf_graph_win_style XDLF_GRAPH_WIN_STYLE
#define xdlf_graph_win_set_colour XDLF_GRAPH_WIN_SET_COLOUR
#define xdlf_graph_win_line XDLF_GRAPH_WIN_LINE
#define xdlf_graph_win_polyline XDLF_GRAPH_WIN_POLYLINE
#define xdlf_graph_win_stext XDLF_GRAPH_WIN_STEXT
#define xdlf_graph_win_xtext XDLF_GRAPH_WIN_XTEXT
#define xdlf_graph_win_symbol XDLF_GRAPH_WIN_SYMBOL
#define xdlf_graph_win_polysymbol XDLF_GRAPH_WIN_POLYSYMBOL
#define xdlf_graph_win_rectangle XDLF_GRAPH_WIN_RECTANGLE
#define xdlf_graph_win_polygon XDLF_GRAPH_WIN_POLYGON
#define xdlf_graph_win_arc XDLF_GRAPH_WIN_ARC
#define xdlf_graph_win_ax1 XDLF_GRAPH_WIN_AX1
#define xdlf_graph_win_ax2 XDLF_GRAPH_WIN_AX2
#define xdlf_graph_win_getcurs XDLF_GRAPH_WIN_GETCURS
#define xdlf_graph_win_input_message XDLF_GRAPH_WIN_INPUT_MESSAGE
#define xdlf_graph_win_setid XDLF_GRAPH_WIN_SETID
#define xdlf_graph_win_delete XDLF_GRAPH_WIN_INPUT_DELETE
#define xdlf_graph_win_getsize XDLF_GRAPH_WIN_GETSIZE

#endif

void xdlf_graph_win();
void xdlf_graph_win_map();
void xdlf_graph_win_style();
void xdlf_graph_win_set_colour();
void xdlf_graph_win_line();
void xdlf_graph_win_polyline();
void xdlf_graph_win_stext();
void xdlf_graph_win_xtext();
void xdlf_graph_win_symbol();
void xdlf_graph_win_polysymbol();
void xdlf_graph_win_rectangle();
void xdlf_graph_win_polygon();
void xdlf_graph_win_arc();
void xdlf_graph_win_ax1();
void xdlf_graph_win_ax2();
void xdlf_graph_win_getcurs();
void xdlf_graph_win_input_message();
void xdlf_graph_win_setid();
void xdlf_graph_win_delete();
void xdlf_graph_win_getsize();

/*-C:*/
int xdl_graph_win (int vh, int vh_parent, int x, int y, int cset, 
                   int ncolrs, int gr_width, int gr_height, 
                   int min_width, int min_height, 
                   int expand, int posn)
/*end*/
 
/*-Parameters:
int       vh;              User selected view-object handle (R)
int       vh_parent;       View-object handle for the parent base frame, 
                           if 0 then a base  level frame is created to
                           enclose the graphics window view-object (R)
int       x;               x coordinate for the view-object.
                           If no parent may be -1 to give default x (R)
int       y;               y coordinate for the view-object.
                           If no parent may be -1 to give default y (R)
int       cset;            Colorset number for optional additional 
                           colorset of 'ncolrs' colours. Set to 0 if not
                           required. The colours for this set will be 
                           numbered from 8 upwards (with 0-7 
                           being used for the default colours) (R)
int       ncolrs;          No. of colours in the additional colorset if
                           used (R) 
int       gr_width;        Required width of graphics window in pixels 
                           excluding border (R)
int       gr_height;       Required height of graphics window in pixels 
                           excluding border (R)
int       min_width;       Minimum window width of view-object in pixels. 
                           If this is greater than the width calculated 
                           from the graph area width requirement etc. 
                           then this value will be used as the window 
                           width (R)
int       min_height;      Minimum window height of view-object in 
                           pixels. If this is greater than the height 
                           calculated from the graph area height 
                           requirement etc. then this value will be 
                           used as the window height (R)
int       expand;          Flag for handling of expanded/contracted
                           graphics area.
                           = 0, do not expand/contract
                           = 1, expand/contract to fit window size but
                                preserve ratio of axes
                           = 2, expand/contact in both directions to
                                fit window (R)
int       posn;            = 0, Position in centre of window
                           = 1, Position at bottom left of window (R)
-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Memory allocation error
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   Status st;                  /* Status return from Xlib calls */
   XDl_graph_win * gd;         /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int i;                      /* Loop counter */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int actwidth;               /* Active strip width */
   int panel_x;                /* x position of control panel */
   int panel_y;                /* y position of control panel */
   int panel_w;                /* Width of control panel */
   int panel_h;                /* Height of control panel */
   int area_width;             /* Graphics area width including border */
   int area_height;            /* Graphics area height including border */
   int x_area;                 /* x position of graphics area */
   int y_area;                 /* y position of graphics area */
   char *p;                    /* Temporary pointer */
   int w1, h1;                 /* Button widths/heights */
   int xt, yt, w, h;           /* Temp window x, y, width, height */
   int l;                      /* Temporary axis length */
   float sc1, sc2;             /* Scaling factors */

   err = 0;
   if (expand<0||expand>2) expand = 0;
   if (posn<0||posn>1) posn = 0;

   /* Get size requirements NOTE: Change xdl_graph_win_getsize routine
                                  if size calculation changed          */

   xdl_panel_button_size (2, 2, 2, &w1, &h1);
   panel_w = w1 + 6;
   panel_h = h1 + 6;
   xdl_layout_init ();
   xdl_layout_set_single ("panel", 0, panel_w, panel_h, 2, 1);
   xdl_layout_set_single ("act", 0, MIN_ACT_STRIP, ACT_STRIP_HEIGHT, 0, 1);
   xdl_layout_set_pair (0, 3, "strip", 0, "act", 0, "panel", 0, 0, 0, 4, 75);
   xdl_layout_set_single ("graph", 0, gr_width+2, gr_height+2, 0, 0);
   xdl_layout_set_pair (1, 2, "top", 0, "strip", 0, "graph", 0, 
                        min_width-2*BORDER_WIDTH-2, 
                        min_height-2*BORDER_HEIGHT-2, 2, 10);
   xdl_layout ("top", 0, BORDER_WIDTH, BORDER_HEIGHT, &w, &h);
   width = w + 2*BORDER_WIDTH + 2;
   height = h + 2*BORDER_HEIGHT + 2;

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_graph_win", "xdl_gr",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_graph_win));
   if (p==0) return 2;
   gd = (XDl_graph_win *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, p, 1071, xdl_graph_win_on_off,
                        xdl_graph_win_tidy_up);

   xdl_add_top_callbacks (vh, xdl_graph_win_rp0, NULL);

   XSelectInput (XDL_display, wframe, ExposureMask|StructureNotifyMask); 

   /* Some initialisations */

   gd->wframe = wframe;
   gd->on_off = 0;
   strcpy (gd->input_message, "Input position");
   strcpy (gd->save_message, gd->input_message);
   gd->expand = expand;
   gd->posn = posn;
   gd->ps_on = 0;
   gd->ps_scale = 1.0;
   gd->x_sel = 0;
   gd->y_sel = 0;
   gd->current_id = 1;
   gd->exposed = 0;
   gd->num_graph_obj = 0;
   gd->num_alloc_graph_obj = 0;
   gd->graph_obj = 0;

   /* Allocate default graphic style */

   gd->num_graph_style = 0;
   gd->num_alloc_graph_style = 0;
   gd->graph_style = 0;
   if (xdl_graph_win_style_alloc(gd)==0) return 2;   
   gd->graph_style[0].fg_colr = XDL_blackpixel;
   gd->graph_style[0].line_width = 1;
   gd->graph_style[0].font_id = XDL_fonts[1]->fid;
   gd->graph_style[0].font_off = 1;
   gd->current_style = 0;
   gd->num_graph_style += 1;

   /* 'X' points list */
   gd->points=0;
   gd->num_alloc_points = 0;
   if (xdl_graph_win_points_alloc(gd)==0) return 2;   

   /* Initial colours */

   gd->cset = cset;
   gd->ncolrs = ncolrs;
   p = (char *) malloc((8+ncolrs)*sizeof(unsigned long));
   if (p==0) return 2;
   gd->colr_pixels = (unsigned long *) p;
   gd->colr_pixels[0] = XDL_blackpixel;
   gd->colr_pixels[1] = XDL_whitepixel;
   gd->colr_pixels[2] = XDL_redpixel;
   gd->colr_pixels[3] = XDL_yellowpixel;
   gd->colr_pixels[4] = XDL_greenpixel;
   gd->colr_pixels[5] = XDL_cyanpixel;
   gd->colr_pixels[6] = XDL_bluepixel;
   gd->colr_pixels[7] = XDL_magentapixel;
   if (ncolrs>0)
   {
      if (gd->cset>0&&XDL_color>0&&XDL_csets_alloc==1)
      {
         for (i=0;i<ncolrs;++i) gd->colr_pixels[8+i] 
                            = XDL_colorsets[cset].pixels[i];
      }
      else
      {
         for (i=0;i<ncolrs;++i) gd->colr_pixels[8+i] = XDL_blackpixel;
      }
   }

   /* Create active strip window */

   xdl_layout_get_position ("act", 0, &xt, &yt, &actwidth, &h);

   gd->wact = xdl_create_active_strip (gd->wframe, vh, actwidth, 
                                       xdl_graph_win_rps);

   gd->act_h = h;

   /* Add control panel & panel items if required */

   xdl_layout_get_position ("panel", 0, &panel_x, &panel_y, &w, &h);
   gd->wpanel = xdl_create_paint_window (wframe, 
                panel_x, panel_y, 1, w, h, 0);

   xdl_add_window (vh, gd->wpanel, xdl_graph_win_rp2, NULL);
  
   gd->button1 = xdl_panel_button (vh, gd->wpanel, 2, 2,
                 "PS", 2, 2, 2, 0, 0, 0, 1, 1, xdl_graph_win_cp1);

   gd->panel_y = panel_y;
   gd->panel_w = w;

   /* Create i/o paint window */

   xdl_layout_get_position ("graph", 0, &x_area, &y_area, 
                             &area_width, &area_height);
   gd->wgr = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                              area_width, area_height, cset);

   xdl_add_window (vh, gd->wgr, xdl_graph_win_rp1, xdl_graph_win_ep1);

   gd->gr_width = gr_width;
   gd->gr_height = gr_height;
   gd->graph_width = area_width - 2;
   gd->graph_height = area_height - 2;

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wgr, ExposureMask|StructureNotifyMask|
                                       ButtonPressMask|ButtonReleaseMask);

   /* Set up mapping data etc. */

   l = gr_width-1;
   if (gr_height-1<l) l = gr_height-1;
   gd->unitsq = (float) l;
   gd->sc_ax1 = 1.0;
   gd->sc_ax2 = 1.0;
   gd->ax1_min = 0.0;
   gd->ax1_max = 1.0;
   gd->ax2_min = 0.0;
   gd->ax2_max = 1.0;
   gd->ax1_mapsc = 1.0;
   gd->ax2_mapsc = 1.0;
   gd->ax1_orig = 0;
   gd->ax2_orig = 0;
   xdl_graph_win_setscale (gd);

   /* Daresbury SR computing group 2-D graphics package variables */
   gd->gra_vp_xmin = 0.0;
   gd->gra_vp_xmax = 1.0;
   gd->gra_vp_ymin = 0.0;
   gd->gra_vp_ymax = 1.0;
   gd->gra_current_x = 0.0;
   gd->gra_current_y = 0.0;
   sc1 = ((float) (gd->gr_width-1))/gd->unitsq;
   sc2 = ((float) (gd->gr_height-1))/gd->unitsq;
   gd->gra_map_xmin = 0.0;
   gd->gra_map_ymin = 0.0;
   gd->gra_mapx = sc1;
   gd->gra_mapy = sc2;
   gd->gra_style_off = 0;
   gd->gra_cursor_inp = 0;

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wgr, 0, NULL);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);

   return 0;
}

/* Fortran binding: xdf_graph_win */

void xdlf_graph_win (ivh, ivhparent, ix, iy, icset, ncolrs, igwidth, igheight, 
                     minw, minh, iexpand, iposn, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *ncolrs, *igwidth, *igheight, 
    *minw, *minh, *iexpand, *iposn, *ierr;
{
   *ierr = xdl_graph_win (*ivh, *ivhparent, *ix, *iy, *icset, *ncolrs, 
                          *igwidth, *igheight, *minw, *minh, 
                          *iexpand, *iposn);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_graph_win_rps       **
*********************************

Purpose: Repaint active strip window

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_graph_win * gd;  /* Pointer to global data area */

   gd = (XDl_graph_win *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->input_message, 0);
   }
}
/*
*********************************
**     xdl_graph_win_rp0       **
*********************************

Purpose: Repaint routine for top level window (to allow resize info to be
         passed on)

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_rp0 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_graph_win * gd;     /* Pointer to global data area */
   int width, height;      /* New width, height */
   XEvent xevent;          /* Temp x event */

   gd = (XDl_graph_win *) global;

   if (event->type==ConfigureNotify)
   {
      width = event->xconfigure.width;
      height = event->xconfigure.height;
      if(width-12!=gd->graph_width||height-38!=gd->graph_height)
      {
         XResizeWindow (XDL_display, gd->wgr, width-12, height-38);
         XResizeWindow (XDL_display, gd->wact, 
                        width - 2*BORDER_WIDTH - gd->panel_w - 6,
                        ACT_STRIP_H - 2);
         XMoveWindow (XDL_display, gd->wpanel,
                      width - BORDER_WIDTH -gd->panel_w - 2,
                      gd->panel_y);
      }
   }

   return;
}
/*
*********************************
**     xdl_graph_win_rp1       **
*********************************

Purpose: Repaint graphics area

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_graph_win * gd;     /* Pointer to global data area */
   int width, height;      /* New width, height */
   XEvent xevent;          /* Temporary X event */

   gd = (XDl_graph_win *) global;

   if (event->type==ConfigureNotify)
   {
      width = event->xconfigure.width;
      height = event->xconfigure.height;
      gd->graph_width = width;
      gd->graph_height = height;
      xdl_graph_win_setscale (gd);
      if (gd->exposed) xdl_graph_win_redraw (gd);
   }
   else
   {
      gd->exposed = 1;
      xdl_graph_win_redraw (gd);
   }

   return;
}
/*
*********************************
**     xdl_graph_win_rp2       **
*********************************

Purpose: Repaint the control panel surround

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}
/*
*********************************
**     xdl_graph_win_cp1       **
*********************************

Purpose: Callback for PS panel button

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_cp1 (iv, button)

int iv;
int button;

{
   XDl_graph_win *gd;
   int vh;

   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   vh = XDL_view_objects[iv]->vh;
   gd->io_panel = xdl_panel_io (vh, gd->wgr, 2, 2, 50, 10, 100, 2, 1, 1,
                                gd->graph_width-4, 0, xdl_graph_win_postscript);
   gd->ps_on = 1;
   xdl_panel_io_prompt (gd->io_panel, "Postscript file name: ", 0, 1);
   return;
}
/*
*********************************
**     xdl_graph_win_ep1       **
*********************************

Purpose: Event handling from graphics area

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int x, y;                  /* Mouse position */
   XEvent xevent;             /* X Event structure */

   gd = (XDl_graph_win *) global;

   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1||gd->gra_cursor_inp)
         {
            gd->x_sel = event->xbutton.x;
            gd->y_sel = event->xbutton.y;
            break;
         }
         break;

      case ButtonRelease:
         if (event->xbutton.button==Button1||gd->gra_cursor_inp)
         {
            if (gd->on_off==1)
            {
               XDL_return_data_vh = XDL_view_objects[iv]->vh;
               return;
            }
         }
         break;
   }
   return;
}

/*
**********************************
**     xdl_graph_win_redraw     **
**********************************

Purpose: Redraw graphics window

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_redraw (gd)

XDl_graph_win *gd;
{
   int ioff;

   XClearWindow (XDL_display, gd->wgr);

   if (gd->num_graph_obj==0) return;

   for (ioff=0;ioff<gd->num_graph_obj;++ioff) xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return;
}

/*
***********************************
**     xdl_graph_win_on_off      **
***********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_on_off (iv, i)

int iv;       /* Offset in XDL_view_objects array */
int i;        /* flag =1 on, =0 off */

{
   XDl_graph_win *gd;        /* Pointer to global data area */
   XWindowAttributes attrib; /* Window attributes structure */

   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
   }
   return;
}

/*
************************************
**     xdl_graph_win_tidy_up      **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_graph_win *gd;    /* Pointer to the global data area */
   int i;                /* Loop counter */

   gd = (XDl_graph_win *) global;
   XFreeGC (XDL_display, gd->paint_gc);
   if (gd->points!=NULL) free (gd->points);
   if (gd->graph_style!=NULL) free (gd->graph_style);
   if (gd->num_graph_obj>0)
   {
      for(i=0;i<gd->num_graph_obj;++i)
      {
         if (gd->graph_obj[i].fdata!=NULL) free (gd->graph_obj[i].fdata);
         if (gd->graph_obj[i].cdata!=NULL) free (gd->graph_obj[i].cdata);
      }
   }
   if (gd->graph_obj!=NULL) free (gd->graph_obj);
   if (gd->colr_pixels!=NULL) free (gd->colr_pixels);
   return;
}
/*
************************************
**     xdl_graph_win_setscale     **
************************************

Purpose: Set scaling etc. parameters initially and after a resize

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_setscale (gd)

XDl_graph_win *gd;    /* Pointer to the global data area */

{
   int maxsiz;           /* Maximum dimension of current graph area */
   float sc1, sc2, scal; /* Scale factors */

   /* Get scaling factors for the two axes */

   switch (gd->expand)
   {
      case 0: /* Fixed size */
        gd->sc_ax1 = 1.0;
        gd->sc_ax2 = 1.0;
      break;

      case 1: /* Expand preserving ratios */
         sc1 = (float) (gd->graph_width-1) / (float) (gd->gr_width-1);
         sc2 = (float) (gd->graph_height-1) / (float) (gd->gr_height-1);
         scal = sc1;
         if (sc2<sc1) scal = sc2;
         gd->sc_ax1 = scal;
         gd->sc_ax2 = scal;
      break;

      case 2: /* Expand in both directions */
         gd->sc_ax1 = (float) (gd->graph_width-1) / (float) (gd->gr_width-1);
         gd->sc_ax2 = (float) (gd->graph_height-1) / (float) (gd->gr_height-1);
      break;
   }

   /* Get origin position */

   switch (gd->posn)
   {
      case 0: /* Centre */
         gd->ax1_orig = (int) (gd->graph_width - gd->sc_ax1*gd->gr_width) / 2.0;
         if (gd->ax1_orig<0) gd->ax1_orig = 0;
         gd->ax2_orig = (int) (gd->graph_height - gd->sc_ax2*gd->gr_height) / 2.0;
         if (gd->ax2_orig<0) gd->ax2_orig = 0;
      break;

      case 1: /* Position at bottom left */
         gd->ax1_orig = 0;
         gd->ax2_orig = 0;
      break;
   }

   return;
}
/*
************************************
**     xdl_graph_win_delobj       **
************************************

Purpose: Delete an object or all objects

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_delobj (vh, id_obj)

int vh;               /* View-objects handle Note: assumes already
                         validated (R)*/
int id_obj;           /* Graphics object id (0=all (except graphlib))
                         (-1 for graphlib clear) */

{
   XDl_graph_win *gd;             /* Pointer to global data */
   XDl_graphic_object * obj;      /* Pointer to graphics object */
   int ioff;                      /* Offset into kept objects list */
   char id_min;                   /* Minimum id to delete */
   char id_max;                   /* Maximum id to delete */
   char *tp1, *tp2;               /* Temp pointers for structure copies */
   int ssiz;                      /* Size of graphics object structures */
   int i;                         /* Temp/loop variable */
   int iobj;                      /* Loop through graphics objects list */
   int iv;                        /* Offset into view-objects list */

   iv = xdl_getiv (vh);
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (gd->num_graph_obj<=0) return;
   ioff = -1;
   if (id_obj==0)
   {
      id_min = 1;
      id_max = 127;
   }
   else
   {
      id_min = id_obj;
      id_max = id_obj;
   }

   for (iobj=0;iobj<gd->num_graph_obj;++iobj)
   {
      obj = &gd->graph_obj[iobj];
      if (obj->obj_id>=id_min&&obj->obj_id<=id_max)
      {
         if (obj->fdata!=NULL) free (obj->fdata);
         if (obj->cdata!=NULL) free (obj->cdata);
      }
      else
      {
         ioff++;
         if (ioff<iobj)
         {
            tp1 = (char *) &gd->graph_obj[ioff];
            tp2 = (char *) obj;
            ssiz = sizeof (XDl_graphic_object);
            for (i=0;i<ssiz;++i) tp1[i] = tp2[i];
         }
      }
   }
   gd->num_graph_obj = ioff+1;

   /* Redraw */

   xdl_graph_win_redraw (gd);

   return;
}
/*
************************************
**     xdl_graph_win_coord_norm   **
************************************

Purpose: Convert coordinates to normalised set

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_coord_norm (gd, ax1_coord, ax2_coord, ax1_norm, ax2_norm)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
float ax1_coord;      /* Coordinate (input) along axis 1 (R)*/
float ax2_coord;      /* Coordinate (input) along axis 2 (R)*/
float *ax1_norm;      /* Returned normalised coordinate along axis 1 (W)*/
float *ax2_norm;      /* Returned normalised coordinate along axis 2 (W)*/

{
/*????
      *ax1_norm = gd->ax1_min  + (ax1_coord - gd->ax1_min) * gd->ax1_mapsc;
      *ax2_norm = gd->ax2_min  + (ax2_coord - gd->ax2_min) * gd->ax2_mapsc;
*/
      *ax1_norm = (ax1_coord - gd->ax1_min) * gd->ax1_mapsc;
      *ax2_norm = (ax2_coord - gd->ax2_min) * gd->ax2_mapsc;
      return;
}
/*
************************************
**     xdl_graph_win_norm_pix     **
************************************

Purpose: Convert normalised coordinates to nearest X-windows pixel

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_norm_pix (gd, ax1_norm, ax2_norm, xpix, ypix)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
float ax1_norm;       /* Normalised coordinate along axis 1 (R)*/
float ax2_norm;       /* Normalised coordinate along axis 2 (R)*/
int * xpix;           /* Returned 'x' pixel offset position (W) */
int * ypix;           /* Returned 'y' pixel offset position (W) */

{
      *xpix = nint (gd->sc_ax1 * gd->unitsq * ax1_norm);
      *xpix += gd->ax1_orig;
      *ypix = nint (gd->sc_ax2 * gd->unitsq * ax2_norm);
      *ypix = gd->graph_height - *ypix - gd->ax2_orig - 1;
      return;
}
/*
************************************
**     xdl_graph_win_obj_alloc    **
************************************

Purpose: Allocate another graphics object structure

Author:  John W. Campbell, October 1993

*/

int xdl_graph_win_obj_alloc (gd)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/

{
   XDl_graphic_object *p; /* Temp pointer */

   if (gd->num_graph_obj==gd->num_alloc_graph_obj)
   {
      if (gd->graph_obj==NULL)
      {
         p = (XDl_graphic_object *) malloc (
           (gd->num_alloc_graph_obj+NOBJ_ALLOC) * sizeof(XDl_graphic_object));
         if (p==0) return 0;
      }
      else
      {
         p = (XDl_graphic_object *) realloc (gd->graph_obj,
           (gd->num_alloc_graph_obj+NOBJ_ALLOC) * sizeof(XDl_graphic_object));
         if (p==0) return 0;
      }
      gd->graph_obj = p;
      gd->num_alloc_graph_obj += NOBJ_ALLOC;
   }
      return 1;
}
/*
************************************
**     xdl_graph_win_style_alloc  **
************************************

Purpose: Allocate another graphics style

Author:  John W. Campbell, October 1993

*/

int xdl_graph_win_style_alloc (gd)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/

{
   XDl_graphic_style *p; /* Temp pointer */

   if (gd->num_graph_style==gd->num_alloc_graph_style)
   {
      if (gd->graph_style==NULL)
      {
         p = (XDl_graphic_style *) malloc (
           (gd->num_alloc_graph_style+STYLE_ALLOC) * sizeof(XDl_graphic_style));
         if (p==0) return 0;
      }
      else
      {
         p = (XDl_graphic_style *) realloc (gd->graph_style,
           (gd->num_alloc_graph_style+STYLE_ALLOC) * sizeof(XDl_graphic_style));
         if (p==0) return 0;
      }
      gd->graph_style = p;
      gd->num_alloc_graph_style += STYLE_ALLOC;
   }
      return 1;
}
/*
*************************************
**     xdl_graph_win_points_alloc  **
*************************************

Purpose: Allocate another set of 'X' points

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_points_alloc (gd)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/

{
   XPoint *p; /* Temp pointer */

   if (gd->points==NULL)
   {
      p = (XPoint *) malloc (
        (gd->num_alloc_points+POINTS_ALLOC) * sizeof(XPoint));
      if (p==0) return 0;
   }
   else
   {
      p = (XPoint *) realloc (gd->points,
        (gd->num_alloc_points+POINTS_ALLOC) * sizeof(XPoint));
      if (p==0) return 0;
   }
   gd->points = p;
   gd->num_alloc_points += POINTS_ALLOC;
   return 1;
}
/*
************************************
**     xdl_graph_win_draw_obj     **
************************************

Purpose: Draw a graphics object

Author:  John W. Campbell, October 1993

*/

void xdl_graph_win_draw_obj (gd, obj_off)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
int obj_off;          /* Offset of object in graphics objects list (R)*/
{
   XDl_graphic_object *obj;          /* Graphics object pointer */
   int x1, y1, x2, y2;               /* Coordinates in 'X-windows' pixel
                                        offsets */
   int ioff, joff;                   /* Temp offsets */
   int i,j;                          /* Temp/loop variable */
   int ic;                           /* Character loop variable */
   int draw;                         /* Draw/move flag */
   float dx, dy;                     /* Offsets within character */
   char *str;                        /* Pointer to text string */
   int ixc, iyc;                     /* Offsets within character */
   char ch;                          /* Current character */
   float oax1, oax2;                 /* Origin for text string */
   float size1, size2;               /* Character sizes */
   double angle;                     /* Angle for text string (radians) */
   float ch_ax1, ch_ax2;             /* Character position wrt. text origin */
   float ax1_rot, ax2_rot;           /* Rotated coordinates */
   float sina, cosa;                 /* sine, cosine of text angle */
   int npoints;                      /* No. of points */
   float rad1, rad2;                 /* Radii for arcs */
   int ang1, ang2;                   /* Angle units for arcs */
   int width, height;                /* Width, height for arcs */
   int descent;                      /* Font descent */
   int ascent;                       /* Font ascent */
   int font_off;                     /* Offset into standard fonts array */
   int tw;                           /* Text width */

   if (gd->exposed==0) return;
   if (obj_off<0||obj_off>=gd->num_graph_obj) return;
   obj = &gd->graph_obj[obj_off];

   /* Change graphics context if required */

   if (obj->style_off!=gd->current_style)
   {
      ioff = obj->style_off;
      joff = gd->current_style;
      if (gd->graph_style[ioff].fg_colr!=gd->graph_style[joff].fg_colr)
      {
         XSetForeground (XDL_display, gd->paint_gc, 
                         gd->graph_style[ioff].fg_colr);
      }
      if (gd->graph_style[ioff].line_width!=
               gd->graph_style[joff].line_width)
      {
         XSetLineAttributes (XDL_display, gd->paint_gc, 
                         gd->graph_style[ioff].line_width,
                         LineSolid, CapButt, JoinBevel);
      }
      if (gd->graph_style[ioff].font_id!=
               gd->graph_style[joff].font_id)
      {
         XSetFont (XDL_display, gd->paint_gc, 
                   gd->graph_style[ioff].font_id);
      }
      gd->current_style = ioff;
   }

   /* Now draw the required object */

   switch (obj->obj_typ)
   {
       case LINE_OBJ: /* Line */
          xdl_graph_win_norm_pix (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          xdl_graph_win_norm_pix (gd, obj->fdata[0], obj->fdata[1],
                                  &x2, &y2);
          XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
       break;

       case POLYLINE_OBJ: /* Polyline */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_norm_pix (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          gd->points[0].x = x1;
          gd->points[0].y = y1;
          for (i=1,j=0;i<npoints;++i,j+=2)
          {
              xdl_graph_win_norm_pix (gd, obj->fdata[j], obj->fdata[j+1],
                                  &x1, &y1);
              gd->points[i].x = x1;
              gd->points[i].y = y1;
          }
          XDrawLines (XDL_display, gd->wgr, gd->paint_gc, gd->points, npoints,
                      CoordModeOrigin);
       break;

       case SYMBOL_OBJ: /* Symbol */
          xdl_graph_win_draw_symb (gd, obj->ax1_coord, obj->ax2_coord,
              obj->fdata[0], (int) obj->cdata[0], 0);
       break;

       case POLYSYMBOL_OBJ: /* Polysymbol */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_draw_symb (gd, obj->ax1_coord, obj->ax2_coord,
              obj->fdata[0], (int) obj->cdata[0], 0);
          if (npoints>1)
          {
             for (i=1,j=1;i<npoints;++i,j+=2)
             {
                xdl_graph_win_draw_symb (gd, obj->fdata[j], obj->fdata[j+1],
                                      obj->fdata[0], (int) obj->cdata[0], 0);
             }
          }
       break;

       case RECT_OBJ:      /* Rectangle (unfilled) */
       case FILLRECT_OBJ:  /* Rectangle (filled) */
          xdl_graph_win_norm_pix (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          xdl_graph_win_norm_pix (gd, obj->fdata[0], obj->fdata[1],
                                  &x2, &y2);
          if (x2<x1)
          {
             i = x2;
             x2 = x1;
             x1 = i;
          }
          if (y2<y1)
          {
             i = y2;
             y2 = y1;
             y1 = i;
          }
          if (obj->obj_typ==FILLRECT_OBJ)
          {
             XFillRectangle (XDL_display, gd->wgr, gd->paint_gc, x1, y1, 
                            x2 - x1 + 1, y2 - y1 + 1);
          }
          else
          {
             XDrawRectangle (XDL_display, gd->wgr, gd->paint_gc, x1, y1, 
                            x2 - x1, y2 - y1);
          }
       break;

       case POLYGON_OBJ:      /* Polygon (unfilled) */
       case FILLPOLYGON_OBJ:  /* Polygon (filled) */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_norm_pix (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          x2 = x1;
          y2 = y1;
          gd->points[0].x = x1;
          gd->points[0].y = y1;
          for (i=1,j=0;i<npoints;++i,j+=2)
          {
              xdl_graph_win_norm_pix (gd, obj->fdata[j], obj->fdata[j+1],
                                  &x1, &y1);
              gd->points[i].x = x1;
              gd->points[i].y = y1;
          }
          if (obj->obj_typ==POLYGON_OBJ)
          {
             XDrawLines (XDL_display, gd->wgr, gd->paint_gc, gd->points, 
                         npoints, CoordModeOrigin);
             XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
          }
          else
          {
             XFillPolygon (XDL_display, gd->wgr, gd->paint_gc, gd->points, 
                           npoints, Nonconvex, CoordModeOrigin);
          }   
       break;

       case ARC_OBJ:          /* Arc (unfilled) */
       case FILLSECTARC_OBJ:  /* Arc (sector filled) */
       case FILLCHORDARC_OBJ: /* Arc (chord filled) */
          rad1 = obj->fdata[0];
          rad2 = obj->fdata[1];
          xdl_graph_win_norm_pix (gd, obj->ax1_coord - rad1, 
                                  obj->ax2_coord + rad2,
                                  &x1, &y1);
          width = nint (2.0 * gd->sc_ax1 * gd->unitsq * rad1);
          height = nint (2.0 * gd->sc_ax2 * gd->unitsq * rad2);
          ang1 = (int) 64*obj->fdata[2];
          ang2 = (int) 64*obj->fdata[3];
          if (obj->obj_typ==ARC_OBJ)
          {
             XDrawArc (XDL_display, gd->wgr, gd->paint_gc, x1, y1, 
                       width, height, ang1, ang2);
          }
          else
          {
             if (obj->obj_typ==FILLSECTARC_OBJ)
             {
                 XSetArcMode (XDL_display, gd->paint_gc, ArcPieSlice);
             }
             else
             {
                 XSetArcMode (XDL_display, gd->paint_gc, ArcChord);
             }
             XFillArc (XDL_display, gd->wgr, gd->paint_gc, x1, y1, 
                       width, height, ang1, ang2);
          }
       break;

       case STEXT_OBJ:     /* Text using scaleable internal font */
       case STEXTCENT_OBJ:
          str = (char*) obj->cdata;
          size1 = obj->fdata[0];
          size2 = obj->fdata[1];
          oax1 = obj->ax1_coord;
          oax2 = obj->ax2_coord;
          angle = (double) 0.0174533*obj->fdata[2];
          sina = (float) sin(angle);
          cosa = (float) cos(angle);
          if (obj->obj_typ==STEXTCENT_OBJ)
          {
             ch_ax1 = - size1* ((float) obj->ncdata / 2.0);
             ch_ax2 = - 0.25*size2;
          }
          else
          {
             ch_ax1 = 0.0;
             ch_ax2 = 0.0;
          }
          for (ic=0;ic<obj->ncdata;++ic)
          {
             ch = str[ic];
             if (ch<' '||ch>'~') ch = ' ';
             ch -= ' ';
             for (i=0;i<16;++i)
             {
                if (qsfont[(unsigned) ch][2*i]>=100) break;
                draw = 1;
                ixc = qsfont[(unsigned) ch][2*i];
                iyc = qsfont[(unsigned) ch][2*i+1];
                if (i==0||ixc<0) draw = 0;
                if (ixc<0) ixc = -ixc;
                dx = (size1/12.0) * (float) (ixc-1);
                dy = (size2/20.0) * (float) (iyc-1);
                ax1_rot = (ch_ax1+dx)*cosa - (ch_ax2+dy)*sina;
                ax2_rot = (ch_ax2+dy)*cosa + (ch_ax1+dx)*sina;
                if (draw)
                {
                   xdl_graph_win_norm_pix (gd, oax1+ax1_rot, oax2+ax2_rot,
                                  &x2, &y2);
                   if (x1==x2&&y1==y2)
		   {
                      XDrawPoint (XDL_display, gd->wgr, gd->paint_gc, x1, y1);
		   }
                   else
		   {
                      XDrawLine (XDL_display, gd->wgr, gd->paint_gc,
                                x1, y1, x2, y2);
		   }
                   x1 = x2;
                   y1 = y2;
                }
                else
                {
                   xdl_graph_win_norm_pix (gd, oax1+ax1_rot, oax2+ax2_rot,
                                  &x1, &y1);
                }
             }
             ch_ax1 += size1;
          }
       break;   

       case TEXT_OBJ:     /* Text using standard X font */
       case TEXTCENT_OBJ:
          xdl_graph_win_norm_pix (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          str = (char*) obj->cdata;
          font_off = gd->graph_style[obj->style_off].font_off;
          descent = XDL_fonts[font_off]->descent;
          if (obj->obj_typ==TEXT_OBJ)
          {
             XDrawString (XDL_display, gd->wgr, gd->paint_gc, x1, y1-descent+1,
                          str, obj->ncdata);
          }
          else
          {
             ascent = XDL_fonts[font_off]->ascent;
             tw = XTextWidth (XDL_fonts[font_off], str, obj->ncdata);
             XDrawString (XDL_display, gd->wgr, gd->paint_gc, 
                          x1 - tw/2,
                          y1 + (ascent-descent)/2,
                          str, obj->ncdata);
          }
       break;
   }
   return;
}
/*
************************************
**     xdl_graph_win_draw_symb    **
************************************

Purpose: Draw a symbol for X or PS

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_draw_symb (gd, ax1_coord, ax2_coord, size_norm,
                     symb, ps)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
float ax1_coord;      /* Axis 1 coordinate for symbol centre (R)*/
float ax2_coord;      /* Axis 2 coordinate for symbol centre (R)*/
float size_norm;      /* Symbol size (R)*/
int symb;             /* Symbol type 0 = single pixel point
                                     1 = scaled point
                                     2 = vertical cross
                                     3 = diagonal cross
                                     4 = square
                                     5 = diamond */
int ps;               /* = 1 Postscript, = 0 X (R)*/
{
   float hx, hy;            /* Symbol halfsizes in x, y */
   float hx_ps, hy_ps;      /* Symbol half sizes in PS space */
   float axrat;             /* y/x scaling ratio */
   float xcen_ps, ycen_ps;  /* Symbol centre for Postscript */
   int x1, y1, x2, y2;      /* X pixel offsets */
   int width, height;       /* Width/height in pixels for filled arc */

   if (ps)
   {
      xdl_graph_win_norm_ps (gd, ax1_coord, ax2_coord, &xcen_ps, &ycen_ps);
   }
   hx = size_norm/2.0;
   axrat = (gd->sc_ax2/gd->sc_ax1);
   hy = axrat*hx;
   hx_ps = gd->sc_ax1 * gd->unitsq * hx;
   hy_ps = axrat*hx_ps;


   switch (symb)
   {
      case 0: /* Pixel point */
         if (ps)
         {
            fprintf(gd->fpout,"%.2f %.2f 0.5 P\n", xcen_ps, ycen_ps);
         }
         else
         {
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord, &x1, &y1);
            XDrawPoint (XDL_display, gd->wgr, gd->paint_gc, x1, y1);
         }
      break;

      case 1: /* Scaled point (disc) */
         if (ps)
         {
            fprintf(gd->fpout, "%.2f %.2f %.2f %.3f DISC\n", xcen_ps, ycen_ps, 
                    hx_ps, axrat);
         }
         else
         {
            width = nint (gd->sc_ax1 * gd->unitsq * size_norm);
            height = nint (gd->sc_ax2 * gd->unitsq * size_norm);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord, &x1, &y1);
            XFillArc (XDL_display, gd->wgr, gd->paint_gc, 
                      x1-width/2, y1-height/2, width, height,
                      0, 23040);
         }
      break;

      case 2: /* Vertical cross */
         if (ps)
         {
            fprintf(gd->fpout, "%.2f %.2f %.2f %.2f S1\n", xcen_ps, ycen_ps,
                    hx_ps, hy_ps);
         }
         else
         {
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord+hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord-hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
         }
      break;

      case 3: /* Diagonal cross */
         if (ps)
         {
            fprintf(gd->fpout, "%.2f %.2f %.2f %.2f S2\n", xcen_ps, ycen_ps,
                    hx_ps, hy_ps);
         }
         else
         {
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord-hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord+hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord+hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord-hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
         }
      break;

      case 4: /* Square */
         if (ps)
         {
            fprintf(gd->fpout, "%.2f %.2f %.2f %.2f S3\n", xcen_ps, ycen_ps,
                    hx_ps, hy_ps);
         }
         else
         {
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord-hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord+hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord+hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord+hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord+hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord-hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord-hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord-hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
         }
      break;

      case 5: /* Diamond */
         if (ps)
         {
            fprintf(gd->fpout, "%.2f %.2f %.2f %.2f S4\n", xcen_ps, ycen_ps,
                    hx_ps, hy_ps);
         }
         else
         {
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord+hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord+hx, ax2_coord, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord-hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord-hy, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
            xdl_graph_win_norm_pix (gd, ax1_coord-hx, ax2_coord, &x1, &y1);
            xdl_graph_win_norm_pix (gd, ax1_coord, ax2_coord+hy, &x2, &y2);
            XDrawLine (XDL_display, gd->wgr, gd->paint_gc, x1, y1, x2, y2);
         }
      break;
   }
   return;
}
/*
**************************************
**     xdl_graph_win_postscript     **
**************************************

Purpose: Service postscript option

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_postscript (iv, reply, len, max_rp, iprompt, iret)

int iv;          /* Position (offset) in view-objects list */
char * reply;    /* Reply string */
int len;         /* Length of reply string */
int max_rp;      /* Maximum allowed reply length */
int iprompt;     /* Prompt number */
int iret;        /* Code 1 =  non-blank string
                         0 =  blank string
                         -1 = escape */

{
   XDl_graph_win *gd;        /* Pointer to global data area */
   int vh;                   /* View object handle */
   int xroot, yroot;         /* Root position of image window */
   char str2[26];            /* Temporary string */
   int i,j,k,l;              /* Temp/loop variables */
   int ll;                   /* Character string length */
   int blank;                /* Blank string 1=yes 0=no */
   float xpcen;              /* Page centre in x */
   float ytitl;              /* Page 'y' position for short title (mm) */
   float ydate;              /* Page 'y' position for date/time (mm) */
   float ycomm, dycomm;      /* Page 'y' position & lines offset for comments
                                (mm) */
   float mmscal;             /* Scale factor mm to points */
   time_t tm;                /* Structure for timer routines */
   Window widp;              /* Window id for panel io item */
   float xo_plot;            /* Origin for plot */
   float yo_plot;            /* Origin for plot */
   float sc1;                /* Temp for scale factor calcn */
   float sc2;                /* Temp for scale factor calcn */
   float scale;              /* Scaling factor for plot */
   int ioff;                 /* offset into graphics objects list */

   /* Initialisations */

   mmscal = 2.8346;
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;
   vh = XDL_view_objects[iv]->vh;
   xdl_rootxy (gd->wgr, 2, 30, &xroot, &yroot);

   if (iret==-1) goto psend;


   /* Get name of output postscript file */
   if (iprompt==1)
   {
      if (iret==0) goto psend;

      gd->fpout = fopen (reply,"w");
      if (gd->fpout==0)
      {
         i = xdl_popup_notice (xroot, yroot, 
                          "**Cannot open file**",
                           0, 0, 0, "Retry", 0, "Cancel", 0, 2, 0);
         if (i==2) goto psend;
         xdl_panel_io_prompt (gd->io_panel, "Postscript file name: ", 0, 1);
         return;
      }

      /* Allocate strings memory */
      gd->title = (char *) malloc(81);
      if (gd->title==0)
      {
         xdl_popup_notice (xroot, yroot, 
                        "**Unable to allocate required memory for title**",
                        0, 0, 0, "Continue", 0, 0, 0, 2, 0);
         goto psend;
      }
      for (i=0;i<4;++i)
      {
         gd->comments[i] = (char *) malloc(81);
         if (gd->comments[i]==0)
         {
            xdl_popup_notice (xroot, yroot, 
                          "**Unable to allocate required memory for comments**",
                           0, 0, 0, "Continue", 0, 0, 0, 2, 0);
            goto psend;
         }
      }
      xdl_panel_io_prompt (gd->io_panel, "Portrait or landscape [p]: ", 0, 2);
      return;
   }
   else if (iprompt==2)
   {
      /* Colour or black and white */
      gd->ps_orient = 1;
      str2[0] = 0;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='L'||str2[0]=='l') gd->ps_orient = 2;
      }
      xdl_panel_io_prompt (gd->io_panel, "Colour Postscript [n]: ", 0, 3);
      return;
   }
   else if (iprompt==3)
   {
      /* Short title */
      gd->ps_colour = 0;
      str2[0] = 0;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='Y'||str2[0]=='y') gd->ps_colour = 1;
      }
      xdl_panel_io_prompt (gd->io_panel, "Short title: ", 0, 4);
      return;
   }
   else if (iprompt==4)
   {
      /* Annotation requirements*/
      reply[80]=0;
      strcpy (gd->title,reply);
      xdl_panel_io_prompt (gd->io_panel, 
             "Add further annotation [y]: ", 0, 5);
      return;
   }
   else if (iprompt==5)
   {
      str2[0] = 0;
      gd->annotate = 1;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='n'||str2[0]=='N') gd->annotate = 0;
      }
      if (gd->annotate)
      {
         xdl_panel_io_prompt (gd->io_panel, 
              "Add heading with date [y]: ", 0, 6);
      }
      else
      {
         goto prepare;
      }
      return;
   }
   else if (iprompt==6)
   {
      str2[0] = 0;
      gd->addate = 1;
      if (iret>0)
         {
            j = sscanf(reply,"%s",str2);
            if (str2[0]=='n'||str2[0]=='N') gd->addate = 0;
         }
      if (gd->addate)
      {
         time(&tm);
         strcpy(gd->datstr,ctime(&tm));
         gd->datstr[24] = 0;
      }
      xdl_panel_io_prompt (gd->io_panel, 
              "Add comments [y]: ", 0, 7);
      return;
   }
   else if (iprompt==7)
   {
      str2[0] = 0;
      gd->addcomment = 1;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='n'||str2[0]=='N') gd->addcomment = 0;
      }
      if (gd->addcomment==0) goto prepare;
      gd->ncomment = 0;
      gd->icomment = 1;
      gd->lencomm = 0;
      gd->ic = 0;
      xdl_panel_io_prompt (gd->io_panel, 
              ": ", 0, 8);
      return;
   }
   else if (iprompt>=8&&iprompt<=11)
   {
      reply[80] = 0;
      blank = 1;
      ll = strlen(reply);
      if (ll>0)
      {
         for (l=0;l<ll;++l)
         {
            if (reply[l]!=' ')
            {
               blank = 0;
               break;
            }
         }
      }
      if (blank==0)
      {
         gd->ncomment = gd->ic + 1;
         strcpy (gd->comments[gd->ic],reply);
         if (ll>gd->lencomm)
         {
            gd->icomment = gd->ncomment;
            gd->lencomm = ll;
         }
      }
      gd->ic++;
      if (iprompt==10) goto prepare;
      xdl_panel_io_prompt (gd->io_panel, 
              ": ", 0, iprompt+1);
      return;
   }

   /* Prepare postscript file */
   /* ======================= */

   prepare:

   xdl_panel_io_clear (gd->io_panel);

   /* Initialisation */

   if (gd->ps_orient==1)
   {
      xpcen = mmscal*105;
      ytitl = mmscal*45;
      ydate = mmscal*270;
      ycomm = mmscal*30;
      dycomm = mmscal*5;
   }
   else
   {
      xpcen = mmscal*149;
      ytitl = mmscal*35;
      ydate = mmscal*195;
      ycomm = mmscal*20;
      dycomm = mmscal*5;
   }

   fprintf(gd->fpout,"%%!\n");
   fprintf(gd->fpout,"newpath\n");

   /* Preamble */

   if (gd->ps_orient==2) fprintf(gd->fpout,"595 0 translate 90 rotate\n");

   fprintf(gd->fpout,"/M {moveto} def /L {lineto} def /R {rlineto} def\n");
   fprintf(gd->fpout,"/S {stroke} def\n");

   fprintf(gd->fpout,"/P {/hs exch def /ys exch def /xs exch def\n");
   fprintf(gd->fpout,"xs ys M xs ys hs 0 360 arc fill} def\n");

   fprintf(gd->fpout,"/S1 {/hy exch def /hx exch def\n");
   fprintf(gd->fpout,"/ys exch def /xs exch def\n");
   fprintf(gd->fpout,"xs hx sub ys M xs hx add ys L\n");
   fprintf(gd->fpout,"xs ys hy sub M xs ys hy add L S} def\n");

   fprintf(gd->fpout,"/S2 {/hy exch def /hx exch def\n");
   fprintf(gd->fpout,"/ys exch def /xs exch def\n");
   fprintf(gd->fpout,"xs hx sub ys hy sub M\n");
   fprintf(gd->fpout,"xs hx add ys hy add L\n");
   fprintf(gd->fpout,"xs hx add ys hy sub M\n");
   fprintf(gd->fpout,"xs hx sub ys hy add L S} def\n");

   fprintf(gd->fpout,"/S3 {/hy exch def /hx exch def\n");
   fprintf(gd->fpout,"/ys exch def /xs exch def\n");
   fprintf(gd->fpout,"xs hx sub ys hy sub M\n");
   fprintf(gd->fpout,"xs hx sub ys hy add L\n");
   fprintf(gd->fpout,"xs hx add ys hy add L\n");
   fprintf(gd->fpout,"xs hx add ys hy sub L\n");
   fprintf(gd->fpout,"xs hx sub ys hy sub L S} def\n");

   fprintf(gd->fpout,"/S4 {/hy exch def /hx exch def\n");
   fprintf(gd->fpout,"/ys exch def /xs exch def\n");
   fprintf(gd->fpout,"xs ys hy sub M\n");
   fprintf(gd->fpout,"xs hx sub ys L\n");
   fprintf(gd->fpout,"xs ys hy add L\n");
   fprintf(gd->fpout,"xs hx add ys L\n");
   fprintf(gd->fpout,"xs ys hy sub L S} def\n");

   fprintf(gd->fpout,"/DISC {/yrat exch def /hs exch def\n");
   fprintf(gd->fpout,"/ys exch def /xs exch def\n");
   fprintf(gd->fpout,"gsave xs ys translate 1.0 yrat scale\n");
   fprintf(gd->fpout,"0.0 0.0 hs 0 360 arc fill grestore} def\n");

   /* Short title */

   fprintf(gd->fpout,"/Helvetica findfont 20 scalefont setfont\n");
   fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ytitl);
   fprintf(gd->fpout,"(%s)\n",gd->title);
   fprintf(gd->fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");

   /* Other annotation */

   if (gd->annotate)
   {
      if (gd->addate)  /* Date/time string */
      {
         fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
         fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ydate);
         fprintf(gd->fpout,"(Xdl_view Plot: %s)\n",gd->datstr);
         fprintf(gd->fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");
      }

      if (gd->addcomment&&gd->ncomment>0) /* Comments */
      {
         fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
         fprintf(gd->fpout,"/xoff (%s) stringwidth pop 2 div neg def\n",
                 gd->comments[gd->icomment-1]);
         if (gd->ncomment>3) ycomm += dycomm;
         for (i=0;i<gd->ncomment;++i)
         {
            fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ycomm);
            fprintf(gd->fpout,"xoff 0 rmoveto\n");      
            fprintf(gd->fpout,"(%s) show\n",gd->comments[i]);
            ycomm -= dycomm;
         }
      }
   }
   fprintf(gd->fpout,"stroke\n");

   /* Output the plot now */

   fprintf(gd->fpout,"/Courier findfont 12 scalefont setfont\n");
   if (gd->ps_orient==1)
   {
      sc1 = 560.0 / (float) gd->graph_width;
      sc2 = 580.0 / (float) gd->graph_height;
      scale = sc1;
      if (sc2<scale) scale = sc2;
      xo_plot = 20.0 + (560.0 - (float) gd->graph_width * scale) / 2.0;
      yo_plot = 160.0 + (580.0 - (float) gd->graph_height * scale) / 2.0;
   }
   else
   {
      sc1 = 770.0 / (float) gd->graph_width;
      sc2 = 370.0 / (float) gd->graph_height;
      scale = sc1;
      if (sc2<scale) scale = sc2;
      xo_plot = 15.0 + (770.0 - (float) gd->graph_width * scale) / 2.0;
      yo_plot = 150.0 + (370.0 - (float) gd->graph_height * scale) / 2.0;
   }
   gd->ps_scale = scale;
   fprintf(gd->fpout,"%f %f translate %f %f scale\n", xo_plot, yo_plot,
          scale, scale);
   fprintf(gd->fpout,"newpath 0 0 M %d %d R %d %d R\n", 
           0, gd->graph_height, gd->graph_width, 0);
   fprintf(gd->fpout,"%d %d R %d %d R  closepath clip S\n", 
           0, -gd->graph_height, -gd->graph_width, 0);

   if (gd->num_graph_obj>0)
   {
      gd->ps_current_style = 0;
      for (ioff=0;ioff<gd->num_graph_obj;++ioff) 
          xdl_graph_win_ps_obj (gd, ioff);
   }

   /* Complete output */

   fprintf(gd->fpout,"showpage\n");
   fclose (gd->fpout);

   psend:
   if (gd->title!=0) free(gd->title);
   for(i=0;i<4;++i) if (gd->comments[i]!=0) free(gd->comments[i]);
   xdl_panel_io_wid (gd->io_panel, &widp);
   xdl_delete_panel_item (vh, widp);
   gd->ps_on=0;
   return;
}
/*
************************************
**     xdl_graph_win_ps_obj       **
************************************

Purpose: Postscript o/p for drawing a graphics object

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_ps_obj (gd, obj_off)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
int obj_off;          /* Offset of object in graphics objects list (R)*/
{
   XDl_graphic_object *obj;          /* Graphics object pointer */
   float x1, y1, x2, y2;             /* PS position offsets */
   int ioff, joff;                   /* Temp offsets */
   int i, j;                         /* Temp/loop variables */
   int ic;                           /* Character loop variable */
   int npoints;                      /* No. of points */
   int draw;                         /* Draw/move flag */
   float dx, dy;                     /* Offsets within character */
   char *str;                        /* Pointer to text string */
   int ixc, iyc;                     /* Offsets within character */
   char ch;                          /* Current character */
   float oax1, oax2;                 /* Origin for text string */
   float size1, size2;               /* Character sizes */
   float rad1, rad2;                 /* Radii for arcs */
   double angle;                     /* Angle for text string (radians) */
   float ch_ax1, ch_ax2;             /* Character position wrt. text origin */
   float ax1_rot, ax2_rot;           /* Rotated coordinates */
   float sina, cosa;                 /* sine, cosine of text angle */
   float r,g,b;                      /* Red, green blue values for overlay 
                                        colour */
   XColor colr_cell;                 /* Temp XColor structure */
   static int fsiz[5] ={8,12,14,18,24}; /* Standard Courier font sizes */
   int font_off;                     /* Offset in standard fonts array */

   if (obj_off<0||obj_off>=gd->num_graph_obj) return;
   obj = &gd->graph_obj[obj_off];

   /* Change graphics context if required */

   if (obj->style_off!=gd->ps_current_style)
   {
      ioff = obj->style_off;
      joff = gd->ps_current_style;
      if (gd->ps_colour>0)
      {
         if (gd->graph_style[ioff].fg_colr!=gd->graph_style[joff].fg_colr)
         {
            colr_cell.pixel = gd->graph_style[ioff].fg_colr;
            colr_cell.flags = DoRed|DoGreen|DoBlue;
            XQueryColor (XDL_display, XDL_colormap, &colr_cell);
            r = colr_cell.red/65535.0;
            g = colr_cell.green/65535.0;
            b = colr_cell.blue/65535.0;
            fprintf(gd->fpout,"%.4f %.4f %.4f setrgbcolor\n",r,g,b);
         }
      }
      if (gd->graph_style[ioff].line_width!=
               gd->graph_style[joff].line_width)
      {
          fprintf(gd->fpout,"%d setlinewidth\n",
                  gd->graph_style[ioff].line_width);
      }
      if (gd->graph_style[ioff].font_id!=
               gd->graph_style[joff].font_id)
      {
         for (i=0;i<5;++i)
         {
            if(XDL_fonts[i]->fid==gd->graph_style[ioff].font_id)
            {
               fprintf(gd->fpout,"/Courier findfont %.3f scalefont setfont\n",
                       (float) fsiz[i]/gd->ps_scale);
               break;
            }
         }
      }
      gd->ps_current_style = ioff;
   }

   /* Now draw the object */

   switch (obj->obj_typ)
   {
       case LINE_OBJ: /* Line */
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          xdl_graph_win_norm_ps (gd, obj->fdata[0], obj->fdata[1],
                                  &x2, &y2);
          fprintf(gd->fpout,"%.2f %.2f M %.2f %.2f L S\n", x1, y1, x2, y2);
       break;

       case POLYLINE_OBJ: /* Polyline */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                 &x1, &y1);
          fprintf(gd->fpout,"%.2f %.2f M\n",x1, y1);
          for (i=1,j=0;i<npoints;++i,j+=2)
          {
              xdl_graph_win_norm_ps (gd, obj->fdata[j], obj->fdata[j+1],
                                     &x1, &y1);
              fprintf(gd->fpout,"%.2f %.2f L\n",x1, y1);
          }
          fprintf(gd->fpout,"S\n");
       break;

       case SYMBOL_OBJ: /* Symbol */
          xdl_graph_win_draw_symb (gd, obj->ax1_coord, obj->ax2_coord,
              obj->fdata[0], (int) obj->cdata[0], 1);
       break;

       case POLYSYMBOL_OBJ: /* Polysymbol */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_draw_symb (gd, obj->ax1_coord, obj->ax2_coord,
              obj->fdata[0], (int) obj->cdata[0], 1);
          if (npoints>1)
          {
             for (i=1,j=1;i<npoints;++i,j+=2)
             {
                xdl_graph_win_draw_symb (gd, obj->fdata[j], obj->fdata[j+1],
                                      obj->fdata[0], (int) obj->cdata[0], 1);
             }
          }
       break;

       case RECT_OBJ:      /* Rectangle (unfilled) */
       case FILLRECT_OBJ:  /* Rectangle (filled) */
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          xdl_graph_win_norm_ps (gd, obj->fdata[0], obj->fdata[1],
                                  &x2, &y2);
          fprintf(gd->fpout,"newpath %.2f %.2f M %.2f %.2f L\n", x1, y1,
                  x2, y1);
          fprintf(gd->fpout,"%.2f %.2f L %.2f %.2f L %.2f %.2f L\n", x2, y2,
                  x1, y2, x1, y1);
          if (obj->obj_typ==FILLRECT_OBJ)
          {
             fprintf(gd->fpout,"closepath fill\n");
          }
          else
          {
             fprintf(gd->fpout,"S\n");
          }
       break;

       case POLYGON_OBJ:      /* Polygon (unfilled) */
       case FILLPOLYGON_OBJ:  /* Polygon (filled) */
          npoints = obj->nfdata/2+1;
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                 &x1, &y1);
          fprintf(gd->fpout,"newpath %.2f %.2f M\n", x1, y1);
          x2 = x1;
          y2 = y1;
          for (i=1,j=0;i<npoints;++i,j+=2)
          {
             xdl_graph_win_norm_ps (gd, obj->fdata[j], obj->fdata[j+1],
                                    &x1, &y1);
             fprintf(gd->fpout,"%.2f %.2f L\n", x1, y1);
          }
          if (obj->obj_typ==FILLPOLYGON_OBJ)
          {
             fprintf(gd->fpout,"closepath fill\n");
          }
          else
          {
             fprintf(gd->fpout,"%.2f %.2f L S\n", x2, y2);
          }   
       break;

       case ARC_OBJ:          /* Arc (unfilled) */
       case FILLSECTARC_OBJ:  /* Arc (sector filled) */
       case FILLCHORDARC_OBJ: /* Arc (chord filled) */
          rad1 = obj->fdata[0];
          rad2 = obj->fdata[1];
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          fprintf(gd->fpout,"gsave %.2f %.2f translate 1.0 %.3f scale\n",
                   x1, y1, rad2/rad1);
          if (obj->obj_typ==ARC_OBJ)
          {
             fprintf(gd->fpout,"0.0 0.0 %.3f %.2f %.2f arc S grestore\n",
                     gd->sc_ax1 * gd->unitsq * rad1,
                     obj->fdata[2], obj->fdata[2]+obj->fdata[3]);
          }
          else
          {
             if (obj->obj_typ==FILLSECTARC_OBJ)
             {
                fprintf(gd->fpout,"0.0 0.0 M 0.0 0.0 %.3f %.2f %.2f arc\n",
                        gd->sc_ax1 * gd->unitsq * rad1,
                        obj->fdata[2], obj->fdata[2]+obj->fdata[3]);
                fprintf(gd->fpout,"closepath fill grestore\n");
             }
             else
             {
                fprintf(gd->fpout,"0.0 0.0 %.3f %.2f %.2f arc\n",
                        gd->sc_ax1 * gd->unitsq * rad1,
                        obj->fdata[2], obj->fdata[2]+obj->fdata[3]);
                fprintf(gd->fpout,"closepath fill grestore\n");
             }
          }
       break;

       case STEXT_OBJ:     /* Text using scaleable internal font */
       case STEXTCENT_OBJ:
          str = (char*) obj->cdata;
          size1 = obj->fdata[0];
          size2 = obj->fdata[1];
          oax1 = obj->ax1_coord;
          oax2 = obj->ax2_coord;
          angle = (double) 0.0174533*obj->fdata[2];
          sina = (float) sin(angle);
          cosa = (float) cos(angle);
          if (obj->obj_typ==STEXTCENT_OBJ)
          {
             ch_ax1 = - size1* ((float) obj->ncdata / 2.0);
             ch_ax2 = - 0.25*size2;
          }
          else
          {
             ch_ax1 = 0.0;
             ch_ax2 = 0.0;
          }
          for (ic=0;ic<obj->ncdata;++ic)
          {
             ch = str[ic];
             if (ch<' '||ch>'~') ch = ' ';
             ch -= ' ';
             for (i=0;i<16;++i)
             {
                if (qsfont[(unsigned) ch][2*i]>=100) break;
                draw = 1;
                ixc = qsfont[(unsigned) ch][2*i];
                iyc = qsfont[(unsigned) ch][2*i+1];
                if (i==0||ixc<0) draw = 0;
                if (ixc<0) ixc = -ixc;
                dx = (size1/12.0) * (float) (ixc-1);
                dy = (size2/20.0) * (float) (iyc-1);
                ax1_rot = (ch_ax1+dx)*cosa - (ch_ax2+dy)*sina;
                ax2_rot = (ch_ax2+dy)*cosa + (ch_ax1+dx)*sina;
                if (draw)
                {
                   xdl_graph_win_norm_ps (gd, oax1+ax1_rot, oax2+ax2_rot,
                                  &x2, &y2);
                   fprintf(gd->fpout,"%.2f %.2f M %.2f %.2f L S\n", 
                           x1, y1, x2, y2);
                   x1 = x2;
                   y1 = y2;
                }
                else
                {
                   xdl_graph_win_norm_ps (gd, oax1+ax1_rot, oax2+ax2_rot,
                                  &x1, &y1);
                }
             }
             ch_ax1 += size1;
          }
       break;   

       case TEXT_OBJ:     /* Text using standard X font */
       case TEXTCENT_OBJ:
          xdl_graph_win_norm_ps (gd, obj->ax1_coord, obj->ax2_coord,
                                  &x1, &y1);
          str = (char*) obj->cdata;
          font_off = gd->graph_style[obj->style_off].font_off;
          if (obj->obj_typ==TEXT_OBJ)
          {
             fprintf(gd->fpout,"%.2f %.2f M (%s) show\n", x1, y1, str);
          }
          else
          {
             fprintf(gd->fpout,"%.2f %.2f M (%s)\n", 
                     x1, (int) y1-((float)fsiz[font_off]/gd->ps_scale)/2.0, 
                     str);
             fprintf(gd->fpout,
                     "dup stringwidth pop 2 div neg 0 rmoveto show\n");
          }
       break;
   }
   return;
}
/*
************************************
**     xdl_graph_win_norm_ps      **
************************************

Purpose: Convert normalised coordinates to PS coordinates

Author:  John W. Campbell, November 1993

*/

void xdl_graph_win_norm_ps (gd, ax1_norm, ax2_norm, xpix, ypix)

XDl_graph_win *gd;    /* Pointer to the global data area (R)*/
float ax1_norm;       /* Normalised coordinate along axis 1 */
float ax2_norm;       /* Normalised coordinate along axis 2 */
float * xpix;         /* Returned 'x' offset position (W) */
float * ypix;         /* Returned 'y' offset position (W) */

{
      *xpix = (gd->sc_ax1 * gd->unitsq * ax1_norm);
      *xpix += (float) gd->ax1_orig;
      *ypix = (gd->sc_ax2 * gd->unitsq * ax2_norm);
      *ypix += (float) gd->ax2_orig;
      return;
}

/*===========================================================*/
/*      Daresbury SR group 2-D package service routines      */
/*===========================================================*/
/*
*************************************
**     xdl_graph_win_gra_vp        **
*************************************

Purpose: Set viewport

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_vp (vh, xmin, xmax, ymin, ymax)

int vh;               /* The view-object handle (R)*/
double xmin;          /* Viewport xmin (R)*/
double xmax;          /* Viewport xmax (R)*/
double ymin;          /* Viewport ymin (R)*/
double ymax;          /* Viewport ymax (R)*/

{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   float sc1, sc2;         /* Scaling factors */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_vp",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_vp", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   gd->gra_vp_xmin = (float) xmin;
   gd->gra_vp_xmax = (float) xmax;
   gd->gra_vp_ymin = (float) ymin;
   gd->gra_vp_ymax = (float) ymax;
   sc1 = ((float) (gd->gr_width-1))/gd->unitsq;
   sc2 = ((float) (gd->gr_height-1))/gd->unitsq;
   gd->gra_map_xmin = gd->gra_vp_xmin*sc1;
   gd->gra_map_ymin = gd->gra_vp_ymin*sc2;
   gd->gra_mapx = sc1*(gd->gra_vp_xmax-gd->gra_vp_xmin);
   gd->gra_mapy = sc2*(gd->gra_vp_ymax-gd->gra_vp_ymin);
   return 0;
}
/*
*************************************
**     xdl_graph_win_gra_map       **
*************************************

Purpose: Set mapping

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_map (vh, xmin, xmax, ymin, ymax)

int vh;               /* The view-object handle (R)*/
double xmin;          /* World xmin (R)*/
double xmax;          /* World xmax (R)*/
double ymin;          /* World ymin (R)*/
double ymax;          /* World ymax (R)*/

{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   float sc1, sc2;         /* Scaling factors */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_map",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_map", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;
   sc1 = ((float) (gd->gr_width-1))/gd->unitsq;
   sc2 = ((float) (gd->gr_height-1))/gd->unitsq;
   gd->gra_map_xmin = gd->gra_vp_xmin*sc1;
   gd->gra_map_ymin = gd->gra_vp_ymin*sc2;
   gd->gra_mapx = sc1*(gd->gra_vp_xmax-gd->gra_vp_xmin)/(xmax-xmin);
   gd->gra_mapy = sc2*(gd->gra_vp_ymax-gd->gra_vp_ymin)/(ymax-ymin);
   return 0;
}
/*
*************************************
**     xdl_graph_win_gra_moveto    **
*************************************

Purpose: Move to point

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_moveto (vh, xcoord, ycoord)

int vh;               /* The view-object handle (R)*/
double xcoord;        /* x-coordinate in world space */
double ycoord;        /* y-coordinate in world space */
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_moveto",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_moveto", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   gd->gra_current_x = (float) xcoord;
   gd->gra_current_y = (float) ycoord;
   return 0;
}
/*
*************************************
**     xdl_graph_win_gra_drawto    **
*************************************

Purpose: Draw to given point from the current point

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_drawto (vh, xcoord, ycoord)

int vh;               /* The view-object handle (R)*/
double xcoord;        /* x-coordinate in world space */
double ycoord;        /* y-coordinate in world space */
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects array */
   float *p;               /* Temp pointer to floating point data */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_drawto",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_drawto", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = -1;
   gd->graph_obj[ioff].obj_typ = LINE_OBJ;
   gd->graph_obj[ioff].style_off = gd->gra_style_off;
   gd->graph_obj[ioff].nfdata = 2;
   gd->graph_obj[ioff].fdata = 0;
   p = (float *) malloc (2*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;

   gd->graph_obj[ioff].ax1_coord = gd->gra_map_xmin +
            (gd->gra_current_x - gd->gra_map_xmin) * gd->gra_mapx;
   gd->graph_obj[ioff].ax2_coord = gd->gra_map_ymin +
            (gd->gra_current_y - gd->gra_map_ymin) * gd->gra_mapy;
   p[0] = gd->gra_map_xmin  
          + ((float) xcoord - gd->gra_map_xmin) * gd->gra_mapx;
   p[1] = gd->gra_map_ymin  
          + ((float) ycoord - gd->gra_map_ymin) * gd->gra_mapy;
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   gd->gra_current_x = (float) xcoord;
   gd->gra_current_y = (float) ycoord;
   XFlush (XDL_display);
   return 0;
}
/*
*************************************
**     xdl_graph_win_gra_text      **
*************************************

Purpose: Draw text for SR graphics (at current moveto/drawto position)

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_text (vh, txt, lentxt, xsiz, ysiz, cent)

int vh;               /* The view-object handle (R)*/
char *txt;            /* Text string (may not be null terminated (R)*/
int lentxt;           /* Length of string (may be 0 if string is null
                         terminated (R)*/
double xsiz;          /* Horizontal character size (R)*/
double ysiz;          /* Vertical character size (R)*/
                      /* If both zero, then use default character size */
int cent;             /* =1 centre text, =0 do not (R)*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   char *p;                /* Temp pointer to character data */
   float *pp;              /* Temp pointer to float data */
   int len;                /* String length */
   float size_norm1;       /* Character size (width) in normalised units */
   float size_norm2;       /* Character size (height) in normalised units */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_text",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_text", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   if (xsiz<=0.0&&ysiz<=0.0)
   {
      size_norm1 = 0.02;
      size_norm2 = 0.02;
   }
   else
   {
      size_norm1 = gd->gra_mapx*xsiz;
      size_norm2 = gd->gra_mapy*ysiz;
   }
   len = lentxt;
   if (len==0) len = strlen(txt);
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = -1;
   if (cent)
   {
      gd->graph_obj[ioff].obj_typ = STEXTCENT_OBJ;
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = STEXT_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->gra_style_off;
   gd->graph_obj[ioff].nfdata = 3;
   gd->graph_obj[ioff].fdata = 0;
   pp = (float *) malloc (3*sizeof(float));
   if (pp==0) return 4;
   gd->graph_obj[ioff].fdata = pp;
   pp[0] = size_norm1;
   pp[1] = size_norm2;
   pp[2] = 0.0;
   gd->graph_obj[ioff].cdata = 0;
   p = (char *) malloc (len*sizeof(char));
   if (p==0) return 4;
   gd->graph_obj[ioff].ncdata = len;
   gd->graph_obj[ioff].cdata = p;
   strncpy (p,txt,len);
   gd->graph_obj[ioff].ax1_coord = gd->gra_map_xmin  
          + ((float) gd->gra_current_x - gd->gra_map_xmin) * gd->gra_mapx;
   gd->graph_obj[ioff].ax2_coord = gd->gra_map_ymin  
          + ((float) gd->gra_current_y - gd->gra_map_ymin) * gd->gra_mapy;
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}
/*
***********************************
**     xdl_graph_win_gra_curs    **
***********************************

Purpose: Get cursor input position (graphlib mapping)

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_curs (vh, xcoord, ycoord)

int vh;                /* The view-object handle (R)*/
double *xcoord;        /* Returns x-coordinate in world space (W)*/
double *ycoord;        /* Returns y-coordinate in world space (W)*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   float ax1_norm;         /* Normalised axis 1 coordinate */
   float ax2_norm;         /* Normalised axis 2 coordinate */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_curs",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_curs", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   /* Use current mapping to return coordinates */

   ax1_norm = (float) (gd->x_sel - gd->ax1_orig) / 
              (float) (gd->sc_ax1 * gd->unitsq);
   ax2_norm = (float) (gd->graph_height - gd->y_sel - gd->ax2_orig - 1) / 
              (float) (gd->sc_ax2 * gd->unitsq);

   *xcoord = (double) gd->gra_map_xmin + (ax1_norm - gd->gra_map_xmin) / 
                      gd->gra_mapx;
   *ycoord = (double) gd->gra_map_ymin + (ax2_norm - gd->gra_map_ymin) /
                      gd->gra_mapy;
   return 0;
}
/*
**************************************
**     xdl_graph_win_gra_cursinp    **
**************************************

Purpose: Set / restore context for graphlib cursor input

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_cursinp (vh, iflag)

int vh;                /* The view-object handle (R)*/
int iflag;             /* =1 set graphlib mode, =0 restore normal mode (R)*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_cursinp",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_cursinp", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (iflag)
   {
      strcpy (gd->save_message, gd->input_message);
      strcpy (gd->input_message,"Graphlib cursor input");
      gd->gra_cursor_inp = 1;
   }
   else
   {
      strcpy (gd->input_message, gd->save_message);
      gd->gra_cursor_inp = 0;
   }
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
      XFlush (XDL_display);
   }
   return 0;
}
/*
**************************************
**     xdl_graph_win_gra_exposed    **
**************************************

Purpose: See if graphics window has been exposed

Author:  John W. Campbell, November 1993

*/

int xdl_graph_win_gra_exposed (vh)

int vh;                /* The view-object handle (R)*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_gra_exposed",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_gra_exposed", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   return gd->exposed;
}
/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Set mapping option - xdl_graph_win_map
The routine xdl_graph_win_map (xdlf_graph_win_map) is used to set the current
mapping between coordinate space and display space. 
-end*/
/*
*******************************
**     xdl_graph_win_map     **
*******************************

Purpose: Set mapping option

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_MAP (IVH, AX1_MIN, AX1_MAX, AX2_MIN, AX2_MAX,
        +                         DAX1_MIN, DAX1_MAX, DAX2_MIN, DAX2_MAX,
        +                         ITYP, IERR)
-end*/
 
/*-Parameters:
IVH          (R)  View-object handle (see vh)
AX1_MIN      (R)  Minimum axis 1 coord. in coordinate space
AX1_MAX      (R)  Maximum axis 1 coord. in coordinate space 
AX2_MIN      (R)  Minimum axis 2 coord. in coordinate space
AX2_MAX      (R)  Maximum axis 2 coord. in coordinate space 
DAX1_MIN     (R)  Minimum axis 1 coord. in display space
                  (see ityp below)
DAX1_MAX     (R)  Maximum axis 1 coord. in display space 
                  (see ityp below)
DAX2_MIN     (R)  Minimum axis 2 coord. in display space
                  (see ityp below)
DAX2_MAX     (R)  Maximum axis 2 coord. in display space 
                  (see ityp below)
ITYP         (R)  Display space type = 0 display coordinates
                                         0.0 to 1.0 in axis 1 and
                                         axis 2 represent the full
                                         size of the requested
                                         graphics window when the
                                         graphics window object
                                         was created.
                                     = 1 display coordinates
                                         0.0 to 1.0 in axis 1 and
                                         axis 2 represent the unit
                                         square. 
IERR         (W)  Returns the status from xdl_graph_win_map call
-end*/

/*-C:*/
int xdl_graph_win_map (int vh, float ax1_min, float ax1_max, 
                       float ax2_min, float ax2_max, 
                       float dax1_min, float dax1_max, 
                       float dax2_min, float dax2_max, int ityp)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_min;        Minimum axis 1 coord. in coordinate space (R)
float ax1_max;        Maximum axis 1 coord. in coordinate space (R)
float ax2_min;        Minimum axis 2 coord. in coordinate space (R)
float ax2_max;        Maximum axis 2 coord. in coordinate space (R)
float dax1_min;       Minimum axis 1 coord. in display space
                      (see ityp below) (R)
float dax1_max;       Maximum axis 1 coord. in display space 
                      (see ityp below) (R)
float dax2_min;       Minimum axis 2 coord. in display space
                      (see ityp below) (R)
float dax2_max;       Maximum axis 2 coord. in display space 
                      (see ityp below) (R)
int ityp;             Display space type = 0 display coordinates
                                             0.0 to 1.0 in axis 1 and
                                             axis 2 represent the full
                                             size of the requested
                                             graphics window when the
                                             graphics window object
                                             was created.
                                         = 1 display coordinates
                                             0.0 to 1.0 in axis 1 and
                                             axis 2 represent the unit
                                             square. (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   float sc1;              /* Scaling factor */
   float sc2;              /* Scaling factor */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_map",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_map", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (ityp==0)
   {
      sc1 = ((float) (gd->gr_width-1))/gd->unitsq;
      sc2 = ((float) (gd->gr_height-1))/gd->unitsq;
/*???? 
      gd->ax1_min = dax1_min*sc1;
      gd->ax2_min = dax2_min*sc2;
*/
      gd->ax1_mapsc = sc1*(dax1_max-dax1_min)/(ax1_max-ax1_min);
      gd->ax2_mapsc = sc2*(dax2_max-dax2_min)/(ax2_max-ax2_min);
      gd->ax1_min = ax1_min - dax1_min*sc1/gd->ax1_mapsc;
      gd->ax2_min = ax2_min - dax2_min*sc2/gd->ax2_mapsc;
   }
   else
   {
/*?????
      gd->ax1_min = dax1_min;
      gd->ax2_min = dax2_min;
*/
      gd->ax1_mapsc = (dax1_max-dax1_min)/(ax1_max-ax1_min);
      gd->ax2_mapsc = (dax2_max-dax2_min)/(ax2_max-ax2_min);
      gd->ax1_min = ax1_min - dax1_min/gd->ax1_mapsc;
      gd->ax2_min = ax2_min - dax2_min/gd->ax2_mapsc;
   }
   return 0;
}

/* Fortran binding: xdlf_graph_win_map */

void xdlf_graph_win_map (ivh, ax1_min, ax1_max, ax2_min, ax2_max,
                         dax1_min, dax1_max, dax2_min, dax2_max, ityp, ierr)
int *ivh, *ityp, *ierr;
float *ax1_min, *ax1_max, *ax2_min, *ax2_max, *dax1_min, *dax1_max,
*dax2_min, *dax2_max;
{
   *ierr = xdl_graph_win_map (*ivh, *ax1_min, *ax1_max, *ax2_min, *ax2_max,
                              *dax1_min, *dax1_max, *dax2_min, *dax2_max,
                              *ityp);
   return;
}
/*-Routine: Set style parameters - xdl_graph_win_style
The routine xdl_graph_win_style (xdlf_graph_win_style) is used to set the 
current style required for the plotting. The style parameters are the
colour, the line width and the X-windows font to be used.
-end*/
/*
*******************************
**     xdl_graph_win_style   **
*******************************

Purpose: Set graphics style

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_STYLE (IVH, IFG_COLR, LINW, IFONT, IERR)
-end*/
 
/*-Parameters:
IVH         (R)  View-object handle (see vh)
IFG_COLR    (R)  Foreground colour -1 = current value
                                    0 = black
                                    1 = white
                                    2 = red
                                    3 = yellow
                                    4 = green
                                    5 = cyan
                                    6 = blue
                                    7 = magenta  
                                    8 upwards (colours from additional
                                      colorset if defined). Set via 
                                      xdl_graph_win_set_colour)
LINW        (R)  Line width in pixels (-1 = current value) 
IFONT       (R)  Font type -1 = current font
                            1 = standard font (very small)
                            2 = standard font (small)
                            3 = standard font (medium)
                            4 = standard font (large)
                            5 = standard font (very large)
IERR        (W)  Returns the status from xdl_graph_win_style call
-end*/

/*-C:*/
int xdl_graph_win_style (int vh, int fg_colr, int line_width, 
                         int font_type)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
int fg_colr;          Foreground colour -1 = current value
                                         0 = black
                                         1 = white
                                         2 = red
                                         3 = yellow
                                         4 = green
                                         5 = cyan
                                         6 = blue
                                         7 = magenta  
                                         8 upwards (colours from 
                                           additional colorset if 
                                           defined - set via 
                                           xdl_graph_win_set_colour)
                      (R)
int line_width;       Line width in pixels (-1 = current value) (R)
int font_type;        Font type -1 = current font
                                 1 = standard font (very small)
                                 2 = standard font (small)
                                 3 = standard font (medium)
                                 4 = standard font (large)
                                 5 = standard font (very large)
                      (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Memory allocation failure
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */

   int iv;                   /* Offset in the XDL_view_objects array */
   XDl_graphic_style style;  /* Current style */
   unsigned long colr_cur;   /* Saves current colour */
   int lw_cur;               /* Saves current line width */
   Font font_cur;            /* Saves current font_id */
   int ioff;                 /* Offset for current style */
   int istyle;               /* Loop variable for styles */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_style",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_style", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   /* Set current defaults */
   ioff = gd->current_style;
   style.fg_colr = gd->graph_style[ioff].fg_colr;
   style.line_width = gd->graph_style[ioff].line_width;
   style.font_id = gd->graph_style[ioff].font_id;
   style.font_off = gd->graph_style[ioff].font_off;

   /* Update with new parameters */
   if (fg_colr>=0) 
   {
      colr_cur = style.fg_colr;
      style.fg_colr = gd->colr_pixels[fg_colr];
      if (style.fg_colr!=colr_cur)
      {
         XSetForeground (XDL_display, gd->paint_gc, 
                         style.fg_colr);
      }
   }
   if (line_width>0)
   {
      lw_cur = style.line_width;
      style.line_width = line_width;
      if (style.line_width!=lw_cur)
      {
         XSetLineAttributes (XDL_display, gd->paint_gc, 
                         style.line_width,
                         LineSolid, CapNotLast, JoinMiter);
      }
   }
   if (font_type>0)
   {
      font_cur = style.font_id;
      switch (font_type)
      {
          case 1: 
              style.font_id = XDL_fonts[0]->fid; 
              style.font_off = 0;
          break;
          case 2:
             style.font_id = XDL_fonts[1]->fid; 
             style.font_off = 1;
          break;
          case 3: 
             style.font_id = XDL_fonts[2]->fid; 
             style.font_off = 2;
          break;
          case 4: 
             style.font_id = XDL_fonts[3]->fid; 
             style.font_off = 3;
          break;
          case 5: 
             style.font_id = XDL_fonts[4]->fid; 
             style.font_off = 4;
          break;
      }
      if (style.font_id!=font_cur)
      {
         XSetFont (XDL_display, gd->paint_gc, 
                   style.font_id);
      }
   }

   /* See if style already exists */

   for (istyle=0;istyle<gd->num_graph_style;++istyle)
   {
       if (style.fg_colr!=gd->graph_style[istyle].fg_colr) continue;
       if (style.line_width!=gd->graph_style[istyle].line_width) continue;
       if (style.font_id!=gd->graph_style[istyle].font_id) continue;
       gd->current_style = istyle;
       return 0;
   }

   /* Allocate new graphic style structure */

   if (xdl_graph_win_style_alloc(gd)==0) return 4;   
   istyle = gd->num_graph_style;
   gd->graph_style[istyle].fg_colr = style.fg_colr;
   gd->graph_style[istyle].line_width = style.line_width;
   gd->graph_style[istyle].font_id = style.font_id;
   gd->graph_style[istyle].font_off = style.font_off;
   gd->current_style = istyle;
   gd->num_graph_style += 1;

   return 0;
}

/* Fortran binding: xdlf_graph_win_style */

void xdlf_graph_win_style (ivh, ifg_colr, linw, ifont, ierr)
int *ivh, *ifg_colr, *linw, *ifont, *ierr;
{
   *ierr = xdl_graph_win_style (*ivh, *ifg_colr, *linw, *ifont);
   return;
}
/*-Routine: Reset a colour - xdl_graph_win_set_colour
The routine xdl_graph_win_set_colour (xdlf_graph_win_set_colour) is
used to reset a colour. The routine will only have an effect if a
non-default colorset of additional colours is being used for the graph 
window view object.
-end*/
/*
**************************************
**     xdl_graph_win_set_colour     ** 
**************************************

Purpose: Reset a colour 

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_SET_COLOUR (IVH, ICOLR, IRED, IGREEN, 
        +                                 IBLUE, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ICOLR    (R)  Colour number (8 upwards) (see icolr)
IRED     (R)  Red component 0 to 65535 (see red)
IGREEN   (R)  Green component 0 to 65535 (see green)
IBLUE    (R)  Blue component 0 to 65535 (see blue)
IERR     (W)  Returns status from xdl_image_set_colour call
-end*/

/*-C:*/
int xdl_graph_win_set_colour (int vh, int icolr, int red, int blue, 
                              int green)
/*end*/
 
/*-Parameters:
int vh;          View-object handle (R)
int icolr;       Colour number 8 upwards (R)
int red;         Red component 0 to 65535 (R)
int green;       Green component 0 to 65535 (R)
int blue;        Blue component 0 to 65535 (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_graph_win *gd;      /* Pointer to global data area */
   XColor colr_cell;       /* Temporary Xcolor structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_set_colour",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_set_colour", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;
   if (gd->cset==0) return 0;
   if (icolr<8) return 0;
   if (icolr>8+gd->ncolrs) return 0;
   colr_cell.flags = DoRed|DoGreen|DoBlue;
   colr_cell.red = red;
   colr_cell.green = green;
   colr_cell.blue = blue;
   if (XDL_csets_alloc==1)
   {
      colr_cell.pixel = XDL_colorsets[gd->cset].pixels[icolr-8];
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   else
   {
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
   }
   gd->colr_pixels[icolr] = colr_cell.pixel;
   if (XDL_csets_alloc==0) xdl_graph_win_redraw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_set_colour */

void xdlf_graph_win_set_colour (ivh, icolr, ired, igreen, iblue, ierr)
int *ivh, *icolr, *ired, *igreen, *iblue, *ierr;
{
   *ierr = xdl_graph_win_set_colour (*ivh, *icolr, *ired, *igreen, *iblue);
   return;
}
/*-Routine: Draw a line - xdl_graph_win_line
The routine xdl_graph_win_line (xdlf_graph_win_line) is used to draw a 
line on a graphics window view object. The current colour and line width
are used and the coordinates are based on the current mapping.
-end*/
/*
*******************************
**     xdl_graph_win_line    **
*******************************

Purpose: Draw a line

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_LINE (IVH, AX1_START, AX2_START,
        +                          AX1_END, AX2_END, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_START     (R)  Start position of line axis 1 coordinate 
AX2_START     (R)  Start position of line axis 2 coordinate 
AX1_END       (R)  End position of line axis 1 coordinate 
AX2_END       (R)  End position of line axis 2 coordinate 

                   Note: Coordinates are in 'coordinate' space as set 
                         by  the mapping routine.

IERR          (W)  Returns the status from xdl_graph_win_line call
-end*/

/*-C:*/
int xdl_graph_win_line (int vh, float ax1_start, float ax2_start, 
                        float ax1_end, float ax2_end)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_start;      Start position of line axis 1 coordinate (R)
float ax2_start;      Start position of line axis 2 coordinate (R)
float ax1_end;        End position of line axis 1 coordinate (R)
float ax2_end;        End position of line axis 2 coordinate (R)

                      Coordinates are in 'coordinate' space as set by
                         the mapping routine.

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   float *p;               /* Temp pointer to floating point data */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_line",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_line", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   gd->graph_obj[ioff].obj_typ = LINE_OBJ;
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 2;
   gd->graph_obj[ioff].fdata = 0;
   p = (float *) malloc (2*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   xdl_graph_win_coord_norm (gd, ax1_start, ax2_start, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   xdl_graph_win_coord_norm (gd, ax1_end, ax2_end, 
                             &p[0], &p[1]);

   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_line */

void xdlf_graph_win_line (ivh, ax1_start, ax2_start, ax1_end, ax2_end, ierr)
int *ivh, *ierr;
float *ax1_start, *ax2_start, *ax1_end, *ax2_end;
{
   *ierr = xdl_graph_win_line (*ivh, *ax1_start, *ax2_start, *ax1_end,
                               *ax2_end);
   return;
}
/*-Routine: Draw a polyline - xdl_graph_win_polyline
The routine xdl_graph_win_polyline (xdlf_graph_win_polyline) is used to draw 
lines on a graphics window view object connecting a given series of points.
The current colour and line width are used and the coordinates are based on 
the current mapping.
-end*/
/*
***********************************
**     xdl_graph_win_polyline    **
***********************************

Purpose: Draw a polyline

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_POLYLINE (IVH, AX1_COORDS, AX2_COORDS,
        +                            NPOINTS, IERR)
-end*/
 
/*-Parameters:
IVH            (R)  View-object handle (see vh)
AX1_COORDS     (R)  Axis 1 coordinates
AX2_COORDS     (R)  Axis 2 coordinates
NPOINTS        (R)  No. of points

                    Note: Coordinates are in 'coordinate' space as set
                          by the mapping routine.

IERR           (W)  Returns the status from xdl_graph_win_polyline call
-end*/

/*-C:*/
int xdl_graph_win_polyline (int vh, float * ax1_coords, float * ax2_coords,
                            int  npoints)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coords[];   Axis 1 coordinates (R)
float ax2_coords[];   Axis 2 coordinates (R)
int npoints;          No. of points (at least 2) (R)

                      Coordinates are in 'coordinate' space as set by
                         the mapping routine. 

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   int i, j;               /* Temp counters */
   float *p;               /* Temp pointer to floating point data */


   if (npoints<2) return 0;

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_polyline",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_polyline", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   gd->graph_obj[ioff].obj_typ = POLYLINE_OBJ;
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 0;
   gd->graph_obj[ioff].fdata = 0;
   gd->graph_obj[ioff].nfdata = 2*npoints-2;
   p = (float *) malloc (2*(npoints-1)*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   xdl_graph_win_coord_norm (gd, ax1_coords[0], ax2_coords[0], 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   i = 1;
   j = 0;
   while (i<npoints)
   {
      xdl_graph_win_coord_norm (gd, ax1_coords[i], ax2_coords[i], 
                                &p[j], &p[j+1]);
      i++;
      j += 2;
   }
   while (gd->num_alloc_points<npoints)
   {
      if (xdl_graph_win_points_alloc(gd)==0) return 4;   
   }
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_polyline */

void xdlf_graph_win_polyline (ivh, ax1_coords, ax2_coords, npoints, ierr)
int *ivh, *npoints, *ierr;
float *ax1_coords, *ax2_coords;
{
   *ierr = xdl_graph_win_polyline (*ivh, ax1_coords, ax2_coords, *npoints);
   return;
}
/*-Routine: Draw a text string - xdl_graph_win_stext
The routine xdl_graph_win_stext (xdlf_graph_win_stext) is used to draw a 
text string on a graphics window view object using the internal scaleable
font. The current colour and line width are used and the coordinates are 
based on the current mapping. The current X-windows font definition is not used
by this routine. Two positioning options are available and the text may
be drawn at any angle required.
-end*/
/*
*******************************
**     xdl_graph_win_stext   **
*******************************

Purpose: Draw a text string

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_STEXT (IVH, AX1_COORD, AX2_COORD,
        +                           XDLSTR(TXT), LENTXT, 
        +                           SIZE1, SIZE2, ISIZOPT, ICENT,
        +                           ANGLE, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_COORD     (R)  Start position of text axis 1 coordinate 
AX2_COORD     (R)  Start position of text axis 2 coordinate 

                    Note: Coordinates are in 'coordinate' space as set 
                          by mapping routine.

TXT           (R)  Text string
                    ** Pass address using XDLSTR function **
LENTXT        (R)  Length of the text string (>0)
SIZE1         (R)  Size of the characters along axis 1 (may be 0.0 if
                   size2 > 0.0)
SIZE2         (R)  Size of the characters along axis 2 (may be 0.0 if
                   size1 > 0.0)
ISIZOPT       (R)  Character size (height) definition option
                   = 0 sizes are in unit square units (0.0 - 1.0)
                   = 1 sizes are relative to current axis mapping
                   If a zero size value is given then it will be 
                   calculated from the other size assuming the 
                   standard character shape with a height:width ratio 
                   of 20:12
ICENT         (R)  = 0 position is for bottom left of text
                   = 1 centre text string (both directions)
ANGLE         (R)  Rotate text string anticlockwise by angle degrees
                   from the starting horizontal position (if ICENT is
                   0 the rotation is about the bottom left point of
                   the text string and if ICENT is 1 then it is about
                   the mid point of the string). Note: the size
                   calculations for the text are done before the
                   string is rotated.
IERR          (W)  Returns the status from xdl_graph_win_stext call
-end*/

/*-C:*/
int xdl_graph_win_stext (int vh, float ax1_coord, float ax2_coord, 
                         char * txt, int lentxt, float size1,
                         float size2, int sizopt, int cent, 
                         float angle)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coord;      Start position of text axis 1 coordinate (R)
float ax2_coord;      Start position of text axis 2 coordinate (R)
                      Coordinates are in 'coordinate' space as set by
                      the mapping routine. 
char *txt;            Text string to be drawn (ascii) (R)
int lentxt;           Length of text string (may be 0 if a null
                      terminated string was given) (R)
float size1;          Size of the characters along axis 1 (may be 0.0 if
                      size2 > 0.0) (R)
float size2;          Size of the characters along axis 2 (may be 0.0 if
                      size1 > 0.0) (R)
int sizopt;           Character size definition option

                        = 0 sizes are in unit square units (0.0 - 1.0)
                        = 1 sizes are relative to current axis mapping
  
                      If a zero size value is given then it will be 
                      calculated from the other size assuming the 
                      standard character shape with a height:width 
                      ratio of 20:12 (R)

int cent;             = 0 position is for bottom left of text
                      = 1 centre text string (both directions) (R)
float angle;          Rotate text string anticlockwise by angle degrees
                      from the starting horizontal position (if cent==0
                      the rotation is about the bottom left point of
                      the text string and if cent==1 then it is about
                      the mid point of the string). Note: the size
                      calculations for the text are done before the
                      string is rotated. (R)
-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   char *p;                /* Temp pointer to character data */
   float *pp;              /* Temp pointer to float data */
   int len;                /* String length */
   float size_norm1;       /* Character size (width) in normalised units */
   float size_norm2;       /* Character size (height) in normalised units */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_stext",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_stext", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   switch (sizopt)
   {
      case 0:
         size_norm1 = size1;
         size_norm2 = size2;
      break;

      case 1:
         size_norm1 = gd->ax1_mapsc*size1;
         size_norm2 = gd->ax2_mapsc*size2;
      break;
   }
   if (size_norm1<=0.0) size_norm1 = 0.6*size_norm2;
   if (size_norm2<=0.0) size_norm2 = 1.6667*size_norm1;
   len = lentxt;
   if (len==0) len = strlen(txt);
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   if (cent)
   {
      gd->graph_obj[ioff].obj_typ = STEXTCENT_OBJ;
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = STEXT_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 3;
   gd->graph_obj[ioff].fdata = 0;
   pp = (float *) malloc (3*sizeof(float));
   if (pp==0) return 4;
   gd->graph_obj[ioff].fdata = pp;
   pp[0] = size_norm1;
   pp[1] = size_norm2;
   pp[2] = angle;
   gd->graph_obj[ioff].cdata = 0;
   p = (char *) malloc ((len+1)*sizeof(char));  /* Allow null term byte */
   if (p==0) return 4;
   gd->graph_obj[ioff].ncdata = len;
   gd->graph_obj[ioff].cdata = p;
   strncpy (p,txt,len);
   p[len] = 0;
   
   xdl_graph_win_coord_norm (gd, ax1_coord, ax2_coord, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);

   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_stext */

void xdlf_graph_win_stext (ivh, ax1_coord, ax2_coord, txt, lentxt, size1, 
                           size2, isizopt, icent, angle, ierr)
int *ivh, *txt, *lentxt, *isizopt, *icent, *ierr;
float *ax1_coord, *ax2_coord, *size1, *size2, *angle;
{
   *ierr = xdl_graph_win_stext (*ivh, *ax1_coord, *ax2_coord, 
                               XDL_pointers[*txt], *lentxt,
                               *size1, *size2, *isizopt, *icent, *angle);
   return;
}
/*-Routine: Draw a text string - xdl_graph_win_xtext
The routine xdl_graph_win_xtext (xdlf_graph_win_xtext) is used to draw a 
text string on a graphics window view object using the current xdl_view
standard X-windows font. The text is drawn in the current colour. Only
horizontal text strings may be drawn using this routine. Two positioning
options are available defined using the current coordinate mapping.
-end*/
/*
*******************************
**     xdl_graph_win_xtext   **
*******************************

Purpose: Draw a text string

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_XTEXT (IVH, AX1_COORD, AX2_COORD,
        +                           XDLSTR(TXT), LENTXT, 
        +                           ICENT, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_COORD     (R)  Start position of text axis 1 coordinate 
AX2_COORD     (R)  Start position of text axis 2 coordinate 

                   Note: Coordinates are in 'coordinate' space as set 
                         by mapping routine.

TXT           (R)  Text string
                   ** Pass address using XDLSTR function **
LENTXT        (R)  Length of the text string (>0)
ICENT         (R)  = 0 position is for bottom left of text
                   = 1 centre text string (both directions)
IERR          (W)  Returns the status from xdl_graph_win_xtext call
-end*/

/*-C:*/
int xdl_graph_win_xtext (int vh, float ax1_coord, float ax2_coord, 
                         char * txt, int lentxt, int cent)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coord;      Start position of text axis 1 coordinate (R)
float ax2_coord;      Start position of text axis 2 coordinate (R)
                      Coordinates are in 'coordinate' space as set by
                      mapping routine. 
char *txt;            Text string to be drawn (ascii) (R)
int lentxt;           Length of text string (may be 0 if a null
                      terminated string was given) (R)
int cent;             = 0 position is for bottom left of text
                      = 1 centre text string (both directions) (R)
-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   char *p;                /* Temp pointer to character data */
   int len;                /* String length */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_xtext",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_xtext", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   len = lentxt;
   if (len==0) len = strlen(txt);
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   if (cent)
   {
      gd->graph_obj[ioff].obj_typ = TEXTCENT_OBJ;
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = TEXT_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 0;
   gd->graph_obj[ioff].fdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   p = (char *) malloc ((len+1)*sizeof(char));  /* Allow null term byte */
   if (p==0) return 4;
   gd->graph_obj[ioff].ncdata = len;
   gd->graph_obj[ioff].cdata = p;
   strncpy (p,txt,len);
   p[len] = 0;
   
   xdl_graph_win_coord_norm (gd, ax1_coord, ax2_coord, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);

   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_xtext */

void xdlf_graph_win_xtext (ivh, ax1_coord, ax2_coord, txt, lentxt, icent,
                           ierr)
int *ivh, *txt, *lentxt, *icent, *ierr;
float *ax1_coord, *ax2_coord;
{
   *ierr = xdl_graph_win_xtext (*ivh, *ax1_coord, *ax2_coord, 
                               XDL_pointers[*txt], *lentxt,
                               *icent);
   return;
}
/*-Routine: Draw a symbol - xdl_graph_win_symbol
The routine xdl_graph_win_symbol (xdlf_graph_win_symbol)is used to draw a 
symbol on a graphics window view object using one of the scaleable symbols
available. The current colour and line width are used and the coordinates 
are based on the current mapping.
-end*/
/*
*******************************
**     xdl_graph_win_symbol  **
*******************************

Purpose: Draw a text string

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_SYMBOL (IVH, AX1_COORD, AX2_COORD, ISYMB,
        +                           SIZE, ISIZOPT, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_COORD     (R)  Symbol position axis 1 coordinate 
AX2_COORD     (R)  Symbol position axis 2 coordinate 

                   Note: Coordinates are in 'coordinate' space as set
                         by the mapping routine.

ISYMB         (R)  Symbol type  0 = point (single pixel)
                                1 = variable sized point
                                2 = vertical cross
                                3 = diagonal cross
                                4 = square
                                5 = diamond
SIZE          (R)  Size of the symbol (see ISIZOPT)
ISIZOPT       (R)  Symbol size definition option
                    = 0 size is in unit square units (0.0 - 1.0)
                    = 1 size is relative to current axis 1 axis 
                        mapping
                    = 2 size is relative to current axis 2 axis 
                        mapping
IERR    (W)        Returns the status from xdl_graph_win_symbol call
-end*/

/*-C:*/
int xdl_graph_win_symbol (int vh, float ax1_coord, float ax2_coord, 
                          int symb, float size, int sizopt)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coord;      Symbol position axis 1 coordinate (R)
float ax2_coord;      Symbol position axis 2 coordinate (R)
                      Coordinates are in 'coordinate' space as set by
                      the mapping routine. 
int symb;             Symbol type  0 = point (single pixel)
                                   1 = variable sized point
                                   2 = vertical cross
                                   3 = diagonal cross
                                   4 = square
                                   5 = diamond
                      (R)
float size;           Size of the symbol (see sizopt) (R)
int sizopt;           Symbol size definition option

                          = 0 size is in unit square units (0.0 - 1.0)
                          = 1 size is relative to current axis 1 axis 
                              mapping
                          = 2 size is relative to current axis 2 axis 
                              mapping
                      (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   char *p;                /* Temp pointer to character data */
   float *pp;              /* Temp pointer to float data */
   float size_norm;        /* Symbol size in normalised units */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_symbol", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   switch (sizopt)
   {
      case 0:
         size_norm = size;
      break;

      case 1:
         size_norm = gd->ax1_mapsc*size;
      break;

      case 2:
         size_norm = gd->ax2_mapsc*size;
      break;
   }
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id;
   gd->graph_obj[ioff].obj_typ = SYMBOL_OBJ;
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 1;
   gd->graph_obj[ioff].fdata = 0;
   pp = (float *) malloc (sizeof(float));
   if (pp==0) return 4;
   gd->graph_obj[ioff].fdata = pp;
   pp[0] = size_norm;
   gd->graph_obj[ioff].cdata = 0;
   p = (char *) malloc (sizeof(char));
   if (p==0) return 4;
   gd->graph_obj[ioff].ncdata = 1;
   gd->graph_obj[ioff].cdata = p;
   p[0] = symb;
   
   xdl_graph_win_coord_norm (gd, ax1_coord, ax2_coord, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);

   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_symbol */

void xdlf_graph_win_symbol (ivh, ax1_coord, ax2_coord, isymb, size, isizopt, 
                            ierr)
int *ivh, *isymb, *isizopt, *ierr;
float *ax1_coord, *ax2_coord, *size;
{
   *ierr = xdl_graph_win_symbol (*ivh, *ax1_coord, *ax2_coord, *isymb,
                               *size, *isizopt);
   return;
}
/*-Routine: Draw polysymbols - xdl_graph_win_polysymbol
The routine xdl_graph_win_polysymbol (xdlf_graph_win_polysymbol) is used to 
draw a set of symbols on a graphics window view object for given series of 
points.
-end*/
/*
*************************************
**     xdl_graph_win_polysymbol    **
*************************************

Purpose: Draw polysymbols

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_POLYSYMBOL (IVH, AX1_COORDS, AX2_COORDS,
        +                            NPOINTS, ISYMB, SIZE, ISIZOPT, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_COORDS    (R)  Axis 1 coordinates
AX2_CORDS     (R)  Axis 2 coordinates
NPOINTS       (R)  No. of points
                   Note: Coordinates are in 'coordinate' space as set
                         by the mapping routine.
ISYMB         (R)  Symbol type  0 = point (single pixel)
                                1 = variable sized point
                                2 = vertical cross
                                3 = diagonal cross
                                4 = square
                                5 = diamond
SIZE          (R)  Size of the symbol (see ISIZOPT)
ISIZOPT       (R)  Symbol size definition option
                    = 0 size is in unit square units (0.0 - 1.0)
                    = 1 size is relative to current axis 1 axis 
                        mapping
                    = 2 size is relative to current axis 2 axis 
                        mapping
IERR          (W)  Returns the status from xdl_graph_win_polysymbol call
-end*/

/*-C:*/
int xdl_graph_win_polysymbol (int vh, float * ax1_coords, 
                              float * ax2_coords, int npoints, int symb,
                              float size, int sizopt)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coords[];   Axis 1 coordinates (R)
float ax2_coords[];   Axis 2 coordinates (R)
int npoints;          No. of points (at least 2) (R)
int symb;             Symbol type  0 = point (single pixel)
                                   1 = variable sized point
                                   2 = vertical cross
                                   3 = diagonal cross
                                   4 = square
                                   5 = diamond
                      (R)
float size;           Size of the symbol (see sizopt) (R)
int sizopt;           Symbol size definition option

                          = 0 size is in unit square units (0.0 - 1.0)
                          = 1 size is relative to current axis 1 axis 
                              mapping
                          = 2 size is relative to current axis 2 axis 
                              mapping
                      (R)

                      Coordinates are in 'coordinate' space as set by
                      the mapping routine.

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   int i, j;               /* Temp counters */
   float *p;               /* Temp pointer to floating point data */
   char *pp;               /* Temp pointer to character data */
   float size_norm;        /* Character size (normalised coordinates) */


   if (npoints<1) return 0;

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_polysymbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_polysymbol", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;

   switch (sizopt)
   {
      case 0:
         size_norm = size;
      break;

      case 1:
         size_norm = gd->ax1_mapsc*size;
      break;

      case 2:
         size_norm = gd->ax2_mapsc*size;
      break;
   }
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   gd->graph_obj[ioff].obj_typ = POLYSYMBOL_OBJ;
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 0;
   gd->graph_obj[ioff].fdata = 0;
   gd->graph_obj[ioff].nfdata = 2*npoints-1;
   p = (float *) malloc ((2*npoints-1)*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   p[0] = size_norm;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   pp = (char *) malloc (sizeof(char));
   if (pp==0) return 4;
   gd->graph_obj[ioff].ncdata = 1;
   gd->graph_obj[ioff].cdata = pp;
   pp[0] = symb;
   xdl_graph_win_coord_norm (gd, ax1_coords[0], ax2_coords[0], 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   i = 1;
   j = 1;
   while (i<npoints)
   {
      xdl_graph_win_coord_norm (gd, ax1_coords[i], ax2_coords[i], 
                                &p[j], &p[j+1]);
      i++;
      j += 2;
   }
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_polysymbol */

void xdlf_graph_win_polysymbol (ivh, ax1_coords, ax2_coords, npoints, 
                                isymb, size, isizopt, ierr)
int *ivh, *npoints, *isymb, *isizopt, *ierr;
float *ax1_coords, *ax2_coords, *size;
{
   *ierr = xdl_graph_win_polysymbol (*ivh, ax1_coords, ax2_coords, *npoints,
                                     *isymb, *size, *isizopt);
   return;
}
/*-Routine: Draw a rectangle - xdl_graph_win_rectangle
The routine xdl_graph_win_rectangle (xdlf_graph_win_rectangle) is used to draw 
a rectangle on a graphics window view object. The rectangle may be filled if
desired. For an unfilled rectangle, the current colour and line width
are used and for a filled rectangle, the current colour is used. The 
coordinates are based on the current mapping.
-end*/
/*
***********************************
**     xdl_graph_win_rectangle   **
***********************************

Purpose: Draw a rectangle  (optionally filled)

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_RECTANGLE (IVH, AX1_START, AX2_START,
        +                          AX1_END, AX2_END, IFILL, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_START     (R)  Start corner axis 1 coordinate 
AX2_START     (R)  Start corner axis 2 coordinate 
AX1_END       (R)  End corner axis 1 coordinate 
AX2_END       (R)  End corner axis 2 coordinate 

                   Note: Coordinates are in 'coordinate' space as set 
                         by the mapping routine.

IFILL         (R)  = 0 do not fill, = 1 fill
IERR          (W)  Returns the status from xdl_graph_win_rectangle call
-end*/

/*-C:*/
int xdl_graph_win_rectangle (int vh, float ax1_start, float ax2_start, 
                             float ax1_end, float ax2_end, int fill)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_start;      Start corner axis 1 coordinate (R)
float ax2_start;      Start corner axis 2 coordinate (R)
float ax1_end;        End corner axis 1 coordinate (R)
float ax2_end;        End corner axis 2 coordinate (R)

                      Coordinates are in 'coordinate' space as set by
                      the mapping routine.
int fill;             = 1 fill, = 0 do not (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   float *p;               /* Temp pointer to floating point data */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_rectangle",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_rectangle", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   if (fill)
   {
      gd->graph_obj[ioff].obj_typ = FILLRECT_OBJ;
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = RECT_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 2;
   gd->graph_obj[ioff].fdata = 0;
   p = (float *) malloc (2*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   xdl_graph_win_coord_norm (gd, ax1_start, ax2_start, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   xdl_graph_win_coord_norm (gd, ax1_end, ax2_end, 
                             &p[0], &p[1]);

   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_rectangle */

void xdlf_graph_win_rectangle (ivh, ax1_start, ax2_start, ax1_end, ax2_end, 
                               ifill, ierr)
int *ivh, *ifill, *ierr;
float *ax1_start, *ax2_start, *ax1_end, *ax2_end;
{
   *ierr = xdl_graph_win_rectangle (*ivh, *ax1_start, *ax2_start, *ax1_end,
                               *ax2_end, *ifill);
   return;
}
/*-Routine: Draw a polygon - xdl_graph_win_polygon
The routine xdl_graph_win_polygon (xdlf_graph_win_polygon) is used to draw 
a polygon on a graphics window view object. The polygon may be filled if
desired. The points must define a non-intersecting polygon. If the
last point is not the same as the first point, the polygon will be
automatically closed. For an unfilled polygon, the current colour and line width
are used and for a filled polygon, the current colour is used. The 
coordinates are based on the current mapping.
-end*/
/*
***********************************
**     xdl_graph_win_polygon     **
***********************************

Purpose: Draw a polygon (may be filled)

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_POLYLGON (IVH, AX1_COORDS, AX2_COORDS,
        +                            NPOINTS, IFILL, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_COORDS    (R)  Axis 1 coordinates
AX2_COORDS    (R)  Axis 2 coordinates
NPOINTS       (R)  No. of points

                   Note: Coordinates are in 'coordinate' space as set
                         by the mapping routine.

IFILL         (R)  = 1 fill, = 0 do not.
IERR          (W)  Returns the status from xdl_graph_win_polygon call
-end*/

/*-C:*/
int xdl_graph_win_polygon (int vh, float * ax1_coords, float * ax2_coords,
                           int npoints, int fill)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_coords[];   Axis 1 coordinates (R)
float ax2_coords[];   Axis 2 coordinates (R)
int npoints;          No. of points (at least 3) (R)

                      Coordinates are in 'coordinate' space as set by
                         the mapping routine. 
int fill;             = 1 fill, = 0 do not (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   int i, j;               /* Temp counters */
   float *p;               /* Temp pointer to floating point data */


   if (npoints<3) return 0;

   /* Store without closing point */

   if (ax1_coords[npoints-1]==ax1_coords[0] &&
       ax2_coords[npoints-1]==ax2_coords[0] ) npoints--;

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_polygon",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_polygon", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   if (fill)
   {
      gd->graph_obj[ioff].obj_typ = FILLPOLYGON_OBJ;
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = POLYGON_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 0;
   gd->graph_obj[ioff].fdata = 0;
   gd->graph_obj[ioff].nfdata = 2*npoints-2;
   p = (float *) malloc (2*(npoints-1)*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   xdl_graph_win_coord_norm (gd, ax1_coords[0], ax2_coords[0], 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   i = 1;
   j = 0;
   while (i<npoints)
   {
      xdl_graph_win_coord_norm (gd, ax1_coords[i], ax2_coords[i], 
                                &p[j], &p[j+1]);
      i++;
      j += 2;
   }
   while (gd->num_alloc_points<npoints)
   {
      if (xdl_graph_win_points_alloc(gd)==0) return 4;   
   }
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_polygon */

void xdlf_graph_win_polygon (ivh, ax1_coords, ax2_coords, npoints, ifill, ierr)
int *ivh, *npoints, *ifill, *ierr;
float *ax1_coords, *ax2_coords;
{
   *ierr = xdl_graph_win_polygon (*ivh, ax1_coords, ax2_coords, *npoints,
                                  *ifill);
   return;
}
/*-Routine: Draw an arc - xdl_graph_win_arc
The routine xdl_graph_win_arc (xdlf_graph_win_arc) is used to draw 
an arc on a graphics window view object. The arc may be filled if
desired in one of two ways, either as a sector (pie slice) or as a
filled chord. For an unfilled arc, the current colour and line width
are used and for a filled arc, the current colour is used. The 
coordinates are based on the current mapping.
-end*/
/*
*****************************
**     xdl_graph_win_arc   **
*****************************

Purpose: Draw an arc (optionally filled)

Author:  John W. Campbell, November 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_ARC (IVH, AX1_CEN, AX2_CEN, RAD1, RAD2,
        +                         ANGLE_START, ANGLE_RANGE, IFILL, IERR)
-end*/
 
/*-Parameters:
IVH             (R)  View-object handle (see vh)
AX1_CEN         (R)  Centre position along axis 1
AX2_CEN         (R)  Centre position along axis 2
RAD1            (R)  Radius along axis 1
RAD2            (R)  Radius along axis 2
ANGLE_START     (R)  Start angle (degrees) (anti clockwise 
                     from 3 o'clock)
ANGLE_RANGE     (R)  Angle range (anticlockwise) in degrees
IFILL           (R)  =0 do not fill, 
                     =1 fill as sector (pie slice),
                     =2 fill chord

                     Note: Coordinates & radii are in 'coordinate' space 
                           as set by the mapping routine.

IERR            (W)  Returns the status from xdl_graph_win_arc call
-end*/

/*-C:*/
int xdl_graph_win_arc (int vh, float ax1_cen, float ax2_cen, 
                       float rad1, float rad2, float angle_start,
                       float angle_range, int fill)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_cen;        Centre position along axis 1 (R)
float ax2_cen;        Centre position along axis 2 (R)
float rad1;           Radius along axis 1 (R)
float rad2;           Radius along axis 2 (R)
float angle_start;    Start angle (degrees) (anti clockwise from 3 
                      o'clock) (R)
float angle_range;    Angle range (anticlockwise) in degrees (R)
int fill;             =0 do not fill, 
                      =1 fill as sector (pie slice),
                      =2 fill chord (R)

                      Coordinates & radii are in 'coordinate' space 
                         as set by the mapping routine. 

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int ioff;               /* Offset into graphics objects list for the
                              new object */
   float *p;               /* Temp pointer to floating point data */


   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_arc",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_arc", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   if (xdl_graph_win_obj_alloc(gd)==0) return 4;
   ioff = gd->num_graph_obj;
   gd->graph_obj[ioff].obj_id = gd->current_id; 
   if (fill)
   {
      if (fill==1)
      {
         gd->graph_obj[ioff].obj_typ = FILLSECTARC_OBJ;
      }
      else
      {
         gd->graph_obj[ioff].obj_typ = FILLCHORDARC_OBJ;
      }
   }
   else
   {
      gd->graph_obj[ioff].obj_typ = ARC_OBJ;
   }
   gd->graph_obj[ioff].style_off = gd->current_style;
   gd->graph_obj[ioff].nfdata = 4;
   gd->graph_obj[ioff].fdata = 0;
   p = (float *) malloc (4*sizeof(float));
   if (p==0) return 4;
   gd->graph_obj[ioff].fdata = p;
   gd->graph_obj[ioff].ncdata = 0;
   gd->graph_obj[ioff].cdata = 0;
   xdl_graph_win_coord_norm (gd, ax1_cen, ax2_cen, 
                             &gd->graph_obj[ioff].ax1_coord,
                             &gd->graph_obj[ioff].ax2_coord);
   p[0] = gd->ax1_mapsc*rad1;
   p[1] = gd->ax2_mapsc*rad2;
   p[2] = angle_start;
   p[3] = angle_range;
   gd->num_graph_obj += 1;
   xdl_graph_win_draw_obj (gd, ioff);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_arc */

void xdlf_graph_win_arc (ivh, ax1_cen, ax2_cen, rad1, rad2, angle_start,
                         angle_range, ifill, ierr)
int *ivh, *ifill, *ierr;
float *ax1_cen, *ax2_cen, *rad1, *rad2, *angle_start, *angle_range;
{
   *ierr = xdl_graph_win_arc (*ivh, *ax1_cen, *ax2_cen, *rad1, *rad2,
                              *angle_start, *angle_range, *ifill);
   return;
}
/*-Routine: Draw a horizontal axis - xdl_graph_win_ax1
The routine xdl_graph_win_ax1 (xdlf_graph_win_ax1) is used to draw 
an annotated horizontal graph axis on a graphics window view object. 
The coordinates are based on the current mapping. Tick sizes,
character sizes and annotation intevals are set by the user.
-end*/
/*
*****************************
**     xdl_graph_win_ax1   **
*****************************

Purpose: Draw an annotated horizontal axis

Author:  John W. Campbell, February 1994

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_AX1 (IVH, AX1_ORIG, AX2_ORIG, AXLEN,
        +                         NDIV, AX1_DIV, ND, HTICK, SIZ, 
        +                         ISIZOPT, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_ORIG      (R)  Origin (axis 1) (see note)
AX2_ORIG      (R)  Origin (axis 2) (see note)
AXLEN         (R)  Axis length (see note)
NDIV          (R)  No. of divisions for annotation (if not use AX1_DIV)
AX1_DIV       (R)  Division along axis 1 for annotation if NDIV = 0
                   (see note)
ND            (R)  No. of decimal places for axis values
HTICK         (R)  Height of ticks (see ISIZOPT)
SIZ           (R)  Character height for annotation (see ISIZOPT)
ISIZOPT       (R)  Symbol size definition option
                    = 0 size is in unit square units (0.0 - 1.0)
                    = 1 size is relative to current axis 1 axis 
                        mapping
                    = 2 size is relative to current axis 2 axis 
                        mapping
IERR          (W)  Returns the status from xdl_graph_win_ax1 call

                   Note: Coordinates are in 'coordinate' space 
                         as set by the mapping routine.
-end*/

/*-C:*/
int xdl_graph_win_ax1 (int vh, float ax1_orig, float ax2_orig, float axlen,
                       int  ndiv, float ax1_div, int nd,
                       float htick, float siz, int sizopt)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_orig;       Origin (axis 1) (see note) (R)
float ax2_orig;       Origin (axis 2) (see note) (R)
float axlen;          Axis length (see note) (R)
int ndiv;             No. of divisions for annotation (if 0 then use
                      ax1_div) (R)
float ax1_div;        Division along axis 1 for annotation if ndiv = 0
                      (see note) (R)
int nd;               No. of decimal places for axis values (R)
float htick;          Height of ticks (see sizopt) (R)
float siz;            Character height for annotation (see sizopt) (R)
int sizopt;           Symbol size definition option

                        = 0 size is in unit square units (0.0 - 1.0)
                        = 1 size is relative to current axis 1 axis 
                            mapping
                        = 2 size is relative to current axis 2 axis 
                            mapping (R)

                      Note: Coordinates are in 'coordinate' space 
                            as set by the mapping routine. 

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int i;                  /* Temp/loop variable */
   float h_norm;           /* Tick size in normalised coordinates */
   float siz_norm;         /* Character size in normalised coordinates */
   float axval;            /* Tick position along axis */
   float ch_y;             /* Character y positions */
   char frm[10];           /* Format string */
   char str[30];           /* Number string */
   float offset;           /* Offset for axis annotation */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_ax1",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_ax1", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   xdl_graph_win_line (vh, ax1_orig, ax2_orig, ax1_orig+axlen, ax2_orig);

   if (ndiv<=0&&ax1_div<=0.0) return 0;

   switch (sizopt)
   {
      case 0:
         h_norm = htick;
      break;

      case 1:
         h_norm = gd->ax1_mapsc*htick;
      break;

      case 2:
         h_norm = gd->ax2_mapsc*htick;
      break;
   }

   switch (sizopt)
   {
      case 0:
         siz_norm = siz;
      break;

      case 1:
         siz_norm = gd->ax1_mapsc*siz;
      break;

      case 2:
         siz_norm = gd->ax2_mapsc*siz;
      break;
   }

   /* Tick marks */

   if (ndiv>0)
   {
      for (i=0;i<=ndiv;++i)
      {
         axval = ax1_orig + i*axlen/ndiv;
         xdl_graph_win_line (vh, axval, ax2_orig, 
                             axval, ax2_orig-h_norm/gd->ax2_mapsc);
      }
   }
   else
   {
      offset = 0.0;
      while (offset<=1.001*axlen)
      {
         axval = ax1_orig + offset;
         xdl_graph_win_line (vh, axval, ax2_orig, 
                             axval, ax2_orig-h_norm/gd->ax2_mapsc);
         offset += ax1_div;
      }
   }

      

   /* Values */

   ch_y = ax2_orig - (h_norm+siz_norm)/gd->ax2_mapsc;
   if (ndiv>0)
   {
      for (i=0;i<=ndiv;++i)
      {
         axval = ax1_orig + i*axlen/ndiv;
         if (nd==0)
         {
            sprintf(str,"%d",nint(axval));
         }
         else
         {
            sprintf(frm,"%%.%df",nd);
            sprintf(str,frm,axval);
         }
         xdl_graph_win_stext (vh, axval, ch_y, str, 0, 0.0, siz, sizopt,
                              1, 0.0);
      }
   }
   else
   {
      offset = 0.0;
      while (offset<=1.001*axlen)
      {
         axval = ax1_orig + offset;
         if (nd==0)
         {
            sprintf(str,"%d",nint(axval));
         }
         else
         {
            sprintf(frm,"%%.%df",nd);
            sprintf(str,frm,axval);
         }
         xdl_graph_win_stext (vh, axval, ch_y, str, 0, 0.0, siz, sizopt,
                              1, 0.0);
         offset += ax1_div;
      }
   }

   return 0;
}

/* Fortran binding: xdlf_graph_win_ax1 */

void xdlf_graph_win_ax1 (ivh, ax1_orig, ax2_orig, axlen, ndiv, ax1_div, nd,
                         htick, siz, isizopt, ierr)
int *ivh, *ndiv, *nd, *isizopt, *ierr;
float *ax1_orig, *ax2_orig, *axlen, *ax1_div, *htick, *siz;
{
   *ierr = xdl_graph_win_ax1 (*ivh, *ax1_orig, *ax2_orig, *axlen, *ndiv,
                              *ax1_div, *nd, *htick, *siz, *isizopt);
   return;
}
/*-Routine: Draw a vertical axis - xdl_graph_win_ax2
The routine xdl_graph_win_ax2 (xdlf_graph_win_ax2) is used to draw 
an annotated vertical graph axis on a graphics window view object. 
The coordinates are based on the current mapping. Tick sizes,
character sizes and annotation intevals are set by the user.
-end*/
/*
*****************************
**     xdl_graph_win_ax2   **
*****************************

Purpose: Draw an annotated vertical axis

Author:  John W. Campbell, February 1994

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_AX2 (IVH, AX1_ORIG, AX2_ORIG, AXLEN,
        +                         NDIV, AX2_DIV, ND, HTICK, SIZ, 
        +                         ISIZOPT, IERR)
-end*/
 
/*-Parameters:
IVH           (R)  View-object handle (see vh)
AX1_ORIG      (R)  Origin (axis 1) (see note)
AX2_ORIG      (R)  Origin (axis 2) (see note)
AXLEN         (R)  Axis length (see note)
NDIV          (R)  No. of divisions for annotation (if not use AX2_DIV)
AX2_DIV       (R)  Division along axis 2 for annotation if NDIV = 0
                   (see note)
ND            (R)  No. of decimal places for axis values
HTICK         (R)  Width of ticks (see ISIZOPT)
SIZ           (R)  Character height for annotation (see ISIZOPT)
ISIZOPT       (R)  Symbol size definition option
                    = 0 size is in unit square units (0.0 - 1.0)
                    = 1 size is relative to current axis 1 axis 
                        mapping
                    = 2 size is relative to current axis 2 axis 
                        mapping
IERR          (W)  Returns the status from xdl_graph_win_ax1 call

                   Note: Coordinates are in 'coordinate' space 
                         as set by the mapping routine.
-end*/

/*-C:*/
int xdl_graph_win_ax2 (int vh, float ax1_orig, float ax2_orig, float axlen,
                       int ndiv, float ax2_div, int nd,
                       float wtick, float siz, int sizopt)
/*end*/
 
/*-Parameters:
int vh;               The xdl_graph_win view-object handle (R)
float ax1_orig;       Origin (axis 1) (see note) (R)
float ax2_orig;       Origin (axis 2) (see note) (R)
float axlen;          Axis length (see note) (R)
int ndiv;             No. of divisions for annotation 
                      (if 0 use ax2_div)(R)
float ax2_div;        Division along axis 2 for annotation if ndiv = 0
                      (see note) (R)
int nd;               No. of decimal places for axis values (R)
float wtick;          Width of ticks (see sizopt) (R)
float siz;            Character height for annotation (see sizopt) (R)
int sizopt;           Symbol size definition option

                        = 0 size is in unit square units (0.0 - 1.0)
                        = 1 size is relative to current axis 1 axis 
                            mapping
                        = 2 size is relative to current axis 2 axis 
                            mapping (R)

                      Note: Coordinates are in 'coordinate' space 
                            as set by the mapping routine. 

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate memory
-end*/
{
   XDl_graph_win *gd;      /* Pointer to the global data area */
   int iv;                 /* Offset in the XDL_view_objects array */
   int i;                  /* Temp/loop variable */
   int l;                  /* String length */
   float w_norm;           /* Tick size in normalised coordinates */
   float siz_norm;         /* Character size in normalised coordinates */
   float axval;            /* Tick position along axis */
   float ch_x;             /* Character x positions */
   char frm[10];           /* Format string */
   char str[30];           /* Number string */
   float offset;           /* Offset for annotation */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_ax2",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_ax2", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   xdl_graph_win_line (vh, ax1_orig, ax2_orig, ax1_orig, ax2_orig+axlen);

   if (ndiv<=0&&ax2_div<=0.0) return 0;

   switch (sizopt)
   {
      case 0:
         w_norm = wtick;
      break;

      case 1:
         w_norm = gd->ax1_mapsc*wtick;
      break;

      case 2:
         w_norm = gd->ax2_mapsc*wtick;
      break;
   }

   switch (sizopt)
   {
      case 0:
         siz_norm = siz;
      break;

      case 1:
         siz_norm = gd->ax1_mapsc*siz;
      break;

      case 2:
         siz_norm = gd->ax2_mapsc*siz;
      break;
   }

   /* Tick marks */

   if (ndiv>0)
   {
      for (i=0;i<=ndiv;++i)
      {
         axval = ax2_orig + i*axlen/ndiv;
         xdl_graph_win_line (vh, ax1_orig, axval,
                             ax1_orig-w_norm/gd->ax1_mapsc, axval);
      }
   }
   else
   {
      offset = 0.0;
      while (offset<=axlen)
      {
         axval = ax2_orig + offset;
         xdl_graph_win_line (vh, ax1_orig, axval,
                             ax1_orig-w_norm/gd->ax1_mapsc, axval);
         offset += ax2_div;
      }
   }

   /* Values */

   if (ndiv>0)
   {
      for (i=0;i<=ndiv;++i)
      {
         axval = ax2_orig + i*axlen/ndiv;
         if (nd==0)
         {
            sprintf(str,"%d",nint(axval));
         }
         else
         {
            sprintf(frm,"%%.%df",nd);
            sprintf(str,frm,axval);
         }
         l = strlen(str);
         ch_x = ax1_orig - (w_norm+(l+0.5)*0.6*siz_norm)/gd->ax1_mapsc;
         xdl_graph_win_stext (vh, ch_x, axval, str, 0, 0.0, siz, sizopt,
                              0, 0.0);
      }
   }
   else
   {
      offset = 0.0;
      while (offset<=axlen)
      {
         axval = ax2_orig + offset;
         if (nd==0)
         {
            sprintf(str,"%d",nint(axval));
         }
         else
         {
            sprintf(frm,"%%.%df",nd);
            sprintf(str,frm,axval);
         }
         l = strlen(str);
         ch_x = ax1_orig - (w_norm+(l+0.5)*0.6*siz_norm)/gd->ax1_mapsc;
         xdl_graph_win_stext (vh, ch_x, axval, str, 0, 0.0, siz, sizopt,
                              0, 0.0);
         offset += ax2_div;
      }
   }

   return 0;
}

/* Fortran binding: xdlf_graph_win_ax2 */

void xdlf_graph_win_ax2 (ivh, ax1_orig, ax2_orig, axlen, ndiv, ax2_div, nd,
                         htick, siz, isizopt, ierr)
int *ivh, *ndiv, *nd, *isizopt, *ierr;
float *ax1_orig, *ax2_orig, *axlen, *ax2_div, *htick, *siz;
{
   *ierr = xdl_graph_win_ax2 (*ivh, *ax1_orig, *ax2_orig, *axlen, *ndiv,
                              *ax2_div, *nd, *htick, *siz, *isizopt);
   return;
}
/*-Routine: Get an input cursor position - xdl_graph_win_getcurs
The routine xdl_graph_win_getcurs (xdlf_graph_win_getcurs) is used to
return an selcted cursor position. It should be called
after the return
from a call to xdl_get_events (xdlf_get_events) indicates that a graph
window view-object has returned input. The coordinates returned are
based on the current mapping.
-end*/
/*
*************************************
**     xdl_graph_win_getcurs       **
*************************************

Purpose: Get returned input cursor position from the graph_win

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_GETCURS (IVH, AX1_COORD, AX2_COORD, IERR)
-end*/
 
/*-Parameters:
IVH            (R)  View-object handle (see vh)
AX1_COORD      (W)  Returns the axis 1 coordinate value (current 
                    mapping)
AX2_COORD      (W)  Returns the axis 2 coordinate value (current 
                    mapping)
IERR           (W)  Returns status from xdl_graph_win_getcurs call
-end*/

/*-C:*/
int xdl_graph_win_getcurs (int vh, float * ax1_coord, float * ax2_coord)
/*end*/
 
/*-Parameters:
int vh;                 View-object handle (R)
float *ax1_coord;       Returns the axis 1 coordinate (current mapping) 
                        (W)
float *ax2_coord;       Returns the axis 2 coordinate (current mapping) 
                        (W)
-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;               /* Offset in XDL_view_objects array */
   XDl_graph_win *gd;    /* pointer to the global data area */
   float ax1_norm;       /* Axis 1 normalised coordinate */
   float ax2_norm;       /* Axis 2 normalised coordinate */

   iv = xdl_getiv (vh);

   /* Check validity of the call */

   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_getcurs",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_getcurs", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   /* Use current mapping to return coordinates */

   ax1_norm = (float) (gd->x_sel - gd->ax1_orig) / 
              (float) (gd->sc_ax1 * gd->unitsq);
   ax2_norm = (float) (gd->graph_height - gd->y_sel - gd->ax2_orig - 1) / 
              (float) (gd->sc_ax2 * gd->unitsq);
   *ax1_coord = gd->ax1_min + ax1_norm/gd->ax1_mapsc;
   *ax2_coord = gd->ax2_min + ax2_norm/gd->ax2_mapsc;
   return 0;
}

/* Fortran binding xdlf_graph_win_getcurs */

void xdlf_graph_win_getcurs (ivh, ax1_coord, ax2_coord, ierr)
int *ivh, *ierr;
float *ax1_coord, *ax2_coord;
{
   *ierr = xdl_graph_win_getcurs (*ivh, ax1_coord, ax2_coord);
   return;
}

/*-Routine: Reset active strip message - xdl_graph_win_input_message
The routine xdl_graph_win_input_message (xdlf_graph_win_input_message) is
used to reset the message which will appear in the active strip when cursor
input is requested. When the graphics window view-object is created, the 
default message is 'Input position'.
-end*/
/*
*****************************************
**     xdl_graph_win_input_message     ** 
*****************************************

Purpose: Reset message for the active strip area

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_INPUT_MESSAGE (IVH, XDLSTR(MESSAGE), 
        +                                    LEN, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MESSAGE  (R)  Character string containing the message 
              (see message)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the message string - must be >0 
              (cf len) (max 60 chars) 
IERR     (W)  Returns status from xdl_graph_win_input_message 
              call
-end*/

/*-C:*/
int xdl_graph_win_input_message (int vh, char * message, int len)
/*end*/
 
/*-Parameters:
int vh;          View-object handle (R)
char *message;   Message string (max of 60 chars long) (R)
int len;         Length of the message string; if 0 then length 
                 will be found assuming a null terminated string 
                 (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_graph_win  *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_input_message",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_input_message", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;
   if (len==0) len = strlen (message);
   if (len>60) len = 60;
   strncpy (gd->input_message, message, len);
   gd->input_message[len] = 0;
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_graph_win_input_message */

void xdlf_graph_win_input_message (ivh, message, len, ierr)
int *ivh, *message, *len, *ierr;
{
   *ierr = xdl_graph_win_input_message (*ivh, XDL_pointers[*message], *len);
   return;
}
/*-Routine: Reset current group id - xdl_graph_win_setid
The routine xdl_graph_win_setid (xdlf_graph_win_setid) is
used to reset the current value of the graphics objects group id. This id
is associated with each graphics object drawn and enables selected sets
of graphics objects to be deleted or moved under program control if required.
The default starting id is 1. The valid range is from 1 to 127.
-end*/
/*
*********************************
**     xdl_graph_win_setid     ** 
*********************************

Purpose: Reset current graphics objects group id

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_SETID (IVH, ID, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ID       (R)  Graphics objects group id (1 to 127)
IERR     (W)  Returns status from xdl_graph_win_setid call
-end*/

/*-C:*/
int xdl_graph_win_setid (int vh, int id)
/*end*/
 
/*-Parameters:
int vh;          View-object handle (R)
int id;          New value for graphics objects group id (1-127) (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Invalid group id (not in range 1-127)
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_graph_win  *gd;     /* Pointer to global data area */

   if (id<1||id>127) return 4;

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_setid",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_setid", vh, "xdl_graph_win");
      return 2;
   }
   gd = (XDl_graph_win *) XDL_view_objects[iv]->gd;

   gd->current_id = id;
   return 0;
}

/* Fortran binding: xdlf_graph_win_setid */

void xdlf_graph_win_setid (ivh, id, ierr)
int *ivh, *id, *ierr;
{
   *ierr = xdl_graph_win_setid (*ivh, *id);
   return;
}
/*-Routine: Delete graphics objects - xdl_graph_win_delete
The routine xdl_graph_win_delete (xdlf_graph_win_delete) is
used to delete a set of drawn graphics objects identified by the current
graphics objects group id in force when the objects were drawn. A value
of zero clears all graphics objects. 
-end*/
/*
**********************************
**     xdl_graph_win_delete     ** 
**********************************

Purpose: Delete a group of graphics objects (or clear all)

Author:  John W. Campbell, October 1993

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_DELETE (IVH, ID, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ID       (R)  The id of the group of graphics objects to be 
              deleted (0=all)
IERR     (W)  Returns status from xdl_graph_win_delete call
-end*/

/*-C:*/
int xdl_graph_win_delete (int vh, int id)
/*end*/
 
/*-Parameters:
int vh;          View-object handle (R)
int id;          The id of the group of graphics objects to be deleted
                 (1-127) or 0 to delete all (R)

-end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Invalid id (not in range 0-127)
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_graph_win  *gd;     /* Pointer to global data area */

   if (id<0||id>127) return 4;

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_graph_win_delete",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1071)
   {
      xdl_view_type_errmsg ("xdl_graph_win_delete", vh, "xdl_graph_win");
      return 2;
   }

   xdl_graph_win_delobj (vh, id);
   return 0;
}

/* Fortran binding: xdlf_graph_win_delete */

void xdlf_graph_win_delete (ivh, id, ierr)
int *ivh, *id, *ierr;
{
   *ierr = xdl_graph_win_delete (*ivh, *id);
   return;
}
/*-Routine: Calculate size requirements - xdl_graph_win_getsize
The routine xdl_graph_win_getsize (xdlf_graph_win_getsize) is used to
get the required size for a graphics window view-object given the size
of the graphics area required.
-end*/
/*
************************************
**     xdl_graph_win_getsize      **
************************************

Purpose: Return the size requirements of the graphics window view-object based
         on sizing specified by graph area width and height

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_GRAPH_WIN_GETSIZE (IGWIDTH, IHEIGHT, IWIDTH, 
        +                             IHEIGHT)
-end*/
 
/*-Parameters:
IGWIDTH   (R)   Width of graphics area in pixels
IGHEIGHT  (R)   Height of graphics area in pixels
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_graph_win_getsize (int gr_width, int gr_height, int * w, int * h)
/*end*/
 
/*-Parameters:
int       gr_width;       Required width of graphics area in pixels 
                          (R)
int       gr_height;      Required height of graphics area in 
                          pixels (R)
int *w;                   Returns the width required (W)
int *h;                   Returns the height required (W)

-end*/
/*-Doc:
Return:  None
-end*/
{
   int panel_w;                /* Control panel width */
   int panel_h;                /* Control panel height */
   int w1, h1;                 /* Button widths/heights */
   int xt, yt, wt, ht;         /* Temp window x, y, width, height */

   /* Get size requirements */

   xdl_panel_button_size (2, 2, 2, &w1, &h1);
   panel_w = w1 + 6;
   panel_h = h1 + 6;
   xdl_layout_init ();
   xdl_layout_set_single ("panel", 0, panel_w, panel_h, 2, 1);
   xdl_layout_set_single ("act", 0, MIN_ACT_STRIP, ACT_STRIP_HEIGHT, 0, 1);
   xdl_layout_set_pair (0, 3, "strip", 0, "act", 0, "panel", 0, 0, 0, 4, 75);
   xdl_layout_set_single ("graph", 0, gr_width+2, gr_height+2, 0, 0);
   xdl_layout_set_pair (1, 2, "top", 0, "strip", 0, "graph", 0, 
                        0, 0, 2, 10);
   xdl_layout ("top", 0, BORDER_WIDTH, BORDER_HEIGHT, &wt, &ht);
   *w = wt + 2*BORDER_WIDTH + 2;
   *h = ht + 2*BORDER_HEIGHT + 2;
   return;
}

/* Fortran binding: xdlf_graph_win_getsize */

void xdlf_graph_win_getsize (igwidth, igheight, iwidth, iheight)
int *igwidth, *igheight, *iwidth, *iheight;
{
   xdl_graph_win_getsize (*igwidth, *igheight, iwidth, iheight);
   return;
}
