/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Image Display View-Object Routines
-end*/
 
/*-Intro:
The image view-object is designed ,in particular,for displaying film or 
image-plate images used in Protein Crystallography. It may be used for
measuring spot positions and has facilities for overlaying symbols or
other vectors on top of the displayed image. A magnifying window within 
the view-object shows a magnified version of a portion of the image from the 
main image display area. The image may also be zoomed. A control panel allows 
for a series of adjustments to the display.
-end*/
 
/*-Section: Creating and Handling an Image Display View-Object
-end*/
 

/*Routines List:
The image view-object has the following routines available:

   xdl_image                 Create an image view-object
   xdl_image_getpix          Get selected pixel position
   xdl_image_getpix_ovly     Get pixel position from an overlay
   xdl_image_getrect         Get selected rectangle
   xdl_image_settings        Get settings of display variables
   xdl_image_reset           Reset display variable settings
   xdl_image_axnames         Set local axis names
   xdl_image_display_now     Display image now
   xdl_image_background      Set background image
   xdl_image_newimg          Set new image
   xdl_image_ovly_offset     Reset overlay offset values
   xdl_image_set_colour      Reset an overlay colour
   xdl_image_set_colormap    Reset the user defined colour map
   xdl_image_symbol          Store and display an overlay symbol
   xdl_image_symbols         Store and display overlay symbols
   xdl_image_clear_symbols   Clear overlay symbols
   xdl_image_del_symbol      Delete an overlay symbol
   xdl_image_vect            Store and display an overlay vector
   xdl_image_vects           Store & display a set of overlay vectors
   xdl_image_vects_pos       Store & display a set of overlay vectors
   xdl_image_boxes           Store & display a set of overlay boxes
   xdl_image_clear_vects     Clear overlay vectors
   xdl_image_del_vect        Delete an overlay vector
   xdl_image_text            Output text string to main display
   xdl_image_del_text        Delete a text string
   xdl_image_clear_text      Delete all text strings
   xdl_image_input_message   Reset active strip message
   xdl_image_getsize         Calculate size requirements
*/

/*-Routine: Create an image view-object - xdl_image
The routine xdl_image (xdlf_image) is used to create a new
image view object. The parameters give details of the image to
be displayed.
-end*/
/*
**************************
**    xdl_image         **
**************************

Purpose: Display (and interact with) an image.

Author:  John W. Campbell, August 1993 (based on xdl_film_image view-object)

*/
/*-Fortran:
         CALL XDLF_IMAGE (IVH, IVHPARENT, IX, IY, ICSET, IMGDATA, ITYPE,
        +                 NS_RAS, NF_RAS, NF_OFF, 
        +                 IO_AX1, IO_AX2, NREQ_AX1, NREQ_AX2, NCMP,
        +                 IORDER, JORDER,
        +                 MINVAL, MAXVAL, MINTHR, MAXTHR, MINW, MINH, 
        +                 IBG, IOVLY, ICONTRAST, IDISP_NOW, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number  (see cset)
IMGDATA   (R)   Array holding the full image data (see imgdata)
ITYPE     (R)   1 = unsigned byte data
                2 = unsigned two-byte data (i2)
                3 = signed integer data
                4 = 'squashed i2' data (if stored I<0
                    ipix = -i*8
                5 = 'squashed i2' data if stored I<0
                    ipix = (I+32768)*8
                6 = 'squashed i2' data if stored I<0
                    ipix = (I+32768)*32
                7 = 'squashed i2' data if stored I<0
                    ipix = -(I+1)*256 + 32768
                (see type)
NS_RAS    (R)   Number of rasters in image along slow axis
                (see ns_ras)
NF_RAS    (R)   Number of rasters in image along fast axis
                (see nf_ras)
NF_OFF    (R)   Number of rasters offset between rows of stored
                image data (see nf_off)
IO_AX1    (R)   Origin (local axis 1) pixel for section to be 
                displayed (see o_ax1) (from 1 upwards)
IO_AX2    (R)   Origin (local axis 2) pixel for section to be 
                displayed (see o_ax2) (from 1 upwards)
NREQ_AX1  (R)   Size of section required along local axis 1
                (see nreq_ax1)
NREQ_AX2  (R)   Size of section required along local axis 2
                (see nreq_ax2)
NCMP      (R)   No. of pixels to compress into one pixel
                along each axis (if NCMP>0, pixel averaging will
                be done; give -NCMP to sample rather than average
                (quicker)) (see ncmp)
IORDER    (R)   Order of the data in the input image 
                with respect to two local axes ax1, ax2 e.g.
                (xf, yf) as a number from 1 to 8.
                    1   +ax1 slow   +ax2 fast    (+xf, +yf)
                    2   +ax1 slow   -ax2 fast    (+xf, -yf)
                    3   -ax1 slow   +ax2 fast    (-xf, +yf)
                    4   -ax1 slow   -ax2 fast    (-xf, -yf)
                    5   +ax2 slow   +ax1 fast    (+yf, +xf)
                    6   +ax2 slow   -ax1 fast    (+yf, -xf)
                    7   -ax2 slow   +ax1 fast    (-yf, +xf)
                    8   -ax2 slow   -ax1 fast    (-yf, -xf)
JORDER    (R)   Display order with respect to the two
                local axes (1 to 8) along the X-windows
                axes X horizontal (left to right), Y vertical 
                (top to bottom ) with origin at top left
                1  +ax1 X (horiz)   +ax2 Y (vert)   (+xf, +yf)
                2  +ax1 X (horiz)   -ax2 Y (vert)   (+xf, -yf)
                3  -ax1 X (horiz)   +ax2 Y (vert)   (-xf, +yf)
                4  -ax1 X (horiz)   -ax2 Y (vert)   (-xf, -yf)
                5  +ax2 X (horiz)   +ax1 Y (vert)   (+yf, +xf)
                6  +ax2 X (horiz)   -ax1 Y (vert)   (+yf, -xf)
                7  -ax2 X (horiz)   +ax1 Y (vert)   (-yf, +xf)
                8  -ax2 X (horiz)   -ax1 Y (vert)   (-yf, -xf)
                    
                    (2 was standard for Laue programs)
MINVAL    (R)   Minimum pixel value present (see min_val)
MAXVAL    (R)   Maximum pixel value present (see max_val)
MINTHR    (R)   Minimum threshold for scaling 
                (see min_thresh)
MAXTHR    (R)   Maximum threshold for scaling 
                (see max_thresh)
MINW      (R)   Min. width for view-object or 0 (see min_width)
MINH      (R)   Min. height for view-object or 0 (see min_height)
IBG       (R)   =0 no background choice menu, 
                =1 background choice menu (Set background via
                   xdlf_image_background
                   (see bg_typ)
IOVLY     (R)   = 1 Overlay options are on/off/offset
                = 2 Overlay options are on/off/1 on/2 on
                (see ovly_typ)
ICONTRAST (R)   Start contrast value 1-1024
IDISP_NOW (R)   Display now flag =1 to display immediately; =0
                to wait fo a call to XDLF_IMAGE_DISPLAY_NOW
                before displaying (to enable other parameters
                to be set first) (see disp_now)
IERR      (W)   Returns status from xdl_image call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"

/* Define some parameters */

                          /*Check gd->magwsizes[] if next 3 values changed */
#define MAGWSIZE 128      /*Maximum size of magnified image window*/
#define MAGWSIZE_W 128    /*Width to allow for magnifying window in control
                            panel */
#define MAGWSIZE_H 64     /*Height to allow for magnifying window in control
                            panel */
#define DEFAULT_MAG 4     /*Default magnification for magnified image window*/
#define MIN_MAG 2         /*Minimum magnification for magnifying window*/
#define MAX_MONO_MAG 9    /*Max magnification for monochrome display magnifying
                            window option*/
#define NSYMB_ALLOC 20    /*No. of symbols structures to allocate at a time*/
#define NVECT_ALLOC 20    /*No. of vector structures to allocate at a time*/
#define NTEXT_ALLOC 20    /*No. of text string structures to allocate at a
                            time */
#define MAX_SYMB_SIZE 25  /*Maximum symbol size in pixels*/
#define Y_SEARCH_MAX 64   /*No. of values in 'speed up' y_search array*/

/* Define Integer & Integer*2 data types */

typedef int Intpix;
typedef unsigned short I2pix;
typedef short I2squash;

/* Define nearest integer macro */

#define nint(x)  x>=0.0?(int)(x+0.5):(int)(x-0.5)

/* Define structure for table of symbols */

typedef struct
{
   unsigned short x_pixel;        /* x pixel position wrt X-windows axes */
   unsigned short y_pixel;        /* y_pixel position wrt X-windows axes */
   short x_disp_offset;           /* x offset wrt displayed image - symbol
                                     at pixel centre */
   short y_disp_offset;           /* y offset wrt displayed image - symbol
                                     at pixel centre */
   unsigned char  symb;           /* symbol type number */
   unsigned char  color_flags;    /* symbol colour & flags 
                                     bits 0-3 colour
                                     bit  4   =0 overlay 1, =1 overlay 2
                                     bit  5   =0 allow offset, =1 fixed */
   unsigned int   indx;           /* Reserved for use in sorting (index) */
   unsigned long  temp;           /* Reserved for use in sorting */
}
   XDl_image_symbol;

/* Define structure for table of text strings */

typedef struct
{
   unsigned short x_pixel;        /* x pixel position wrt X-windows axes */
   unsigned short y_pixel;        /* y_pixel position wrt X-windows axes */
   short x_disp_offset;           /* x offset wrt displayed image */
   short y_disp_offset;           /* y offset wrt displayed image */
   unsigned short text_id;        /* Text string identifier */
   char * string;                 /* Pointer to the text string */
   unsigned char  color_flags;    /* vector colour & flags 
                                     bits 0-3 colour
                                     bit  4   =0 overlay 1, =1 overlay 2
                                     bit  5   =0 allow offset, =1 fixed */
   unsigned char font;            /* Font type 1-5 v-small ... v-large */
   unsigned char bold;            /* =0 normal font, =1 bold font */
}
   XDl_image_text;

/* Define structure for table of vectors */

typedef struct
{
   unsigned short x1_pixel;       /* x1 pixel position wrt X-windows axes */
   unsigned short y1_pixel;       /* y1_pixel position wrt X-windows axes */
   unsigned short x2_pixel;       /* x2 pixel position wrt X-windows axes */
   unsigned short y2_pixel;       /* y2_pixel position wrt X-windows axes */
   unsigned short vect_id;        /* Identifier for vector or group of vectors
                                     (user defined; used in delete option) */
   int x1_disp_offset;            /* x offset wrt displayed image 1'st point
                                     (top left of pixel) */
   int y1_disp_offset;            /* y offset wrt displayed image 1'st point
                                     (top left of pixel)
                                     Note: 1'st point always chosen to have
                                           the lower y value */
   int x2_disp_offset;            /* x offset wrt displayed image 2'nd point 
                                     (top left of pixel) */
   int y2_disp_offset;            /* y offset wrt displayed image 2'nd point 
                                     (top left of pixel) */
   short x1_offset;               /* x1 offset flag */   
   short y1_offset;               /* y1 offset flag  */
   short x2_offset;               /* x2 offset flag */
   short y2_offset;               /* y2 offset flag */
   unsigned char  color_flags;    /* vector colour & flags 
                                     bits 0-3 colour
                                     bit  4   =0 overlay 1, =1 overlay 2
                                     bit  5   =0 allow offset, =1 fixed */

                                  /* Offset flags:  0 centre of pixel
                                                   -1 low side
                                                   +1 high side
                                                   -2 outside on low side
                                                   +2 outside on high side */
}
   XDl_image_vect;

/* Define structure for holding details of area frozen in magnifying window */

typedef struct
{
   int            on;             /* =1 frozen area to restore, =0 not */
   int            xo;             /* X origin of rectangle on film display to be
                                     restored */
   int            yo;             /* Y origin */
   int            width;          /* Width of rectangle */
   int            height;         /* Height of rectangle */
}
   XDl_image_freeze;

/* Define structure for the global data for the view-object */

typedef struct
{
   int          vh;                    /* View-object handle */
   int          type;                  /* Image type 1=byte
                                                     2=unsigned i2
                                                     3=signed int
                                                     4=squashed i2 */
   long         (*unsq)();             /* Function to unpack a squashed
                                          image pixel value */
   int          ns_ras;                /* No. of rasters in full image
                                          along slow moving axis */
   int          nf_ras;                /* No. rasters in full image
                                          along fast moving axis */
   int          nf_off;                /* Offset in pixels from start of
                                          one row of full image to the next
                                          (to allow storage of the image in 
                                          a pre-dimenstioned 2-D array */
   int          nf_bgoff;              /* Offset in pixels from start of one
                                          row to start of next row in
                                          background image */
   int          iorder;                /* Order of local axes wrt. image axes 
                                          (1-8)*/
   int          jorder;                /* Order of local axes wrt. displayed
                                          axes (1-8) */
   int          korder;                /* Order of actual image axes wrt.
                                          X-windows x, y (1-8) calc from
                                          iorder, jorder */
   char         ax1_name[5];           /* Name of 1'st axis (max 4 chars) -
                                          null terminated */
   char         ax2_name[5];           /* Name of 2'nd axis (max 4 chars) -
                                          null terminated */
   int          img_width;             /* Displayed image width */
   int          img_height;            /* Displayed image height */
   int          xo_pix;                /* Origin x pixel of image section
                                          (un-zoomed) wrt. the X-windows
                                          axes */
   int          yo_pix;                /* Origin y pixel " */
   int          nxrasts;               /* The number of x rasters in the
                                          full image (along X-windows x-axis */
   int          nyrasts;               /* The number of y rasters in the
                                          full image (along X-windows y-axis */
   int          cur_xo_pix;            /* Origin x pixel of currently 
                                          displayed part of image section
                                          wrt. the X-windows axes */
   int          cur_yo_pix;            /* Origin y pixel of currently displayed
                                          part of the image section " */
   int          ncmp;                  /* No. pixels combined into 1 pixel for
                                          the display (non-zoomed) */
   int          cmp_type;              /* =1 compress by averaging, =0 just
                                          sample */
   int          bg_cmp;                /* Background image compression factor */
   Window       wfilm;                 /* Top level window */
   Window       wact;                  /* Active strip window id */
   Window       wdisp;                 /* Image display area window */
   Window       wmag;                  /* Magnifying area window */
   Window       wpanel;                /* Control panel window */
   Panel_object contrast_slider;       /* Contrast slider */
   Panel_object thresh_slider;         /* Monochrome threshold slider */
   Panel_object cursor_position;       /* Cursor position label */
   Panel_object color_choice;          /* Color choice menu */
   Panel_object mag_choice;            /* Magnification factor menu */
   Panel_object ovly_choice;           /* Overlay on/off etc. choice item */
   Panel_object bg_choice;             /* Background choice menu */
   Panel_object max_text_item;         /* Maximum threshold text item */
   Panel_object min_text_item;         /* Minimum threshold text item */
   Panel_object ps_button;             /* Postscript o/p button */
   Panel_object io_panel;              /* Panel io item (for Postscript 
                                          option) */
   Panel_object zoom_button;           /* Zoom button */
   Panel_object select_label;          /* Zoom select label */
   GC           paint_gc;              /* GC for paint windows */
   Cursor       cursor;                /* Cursor for image and magw windows */
   char         *img;                  /* Array to hold pixel data for the 
                                          XImage */
   void         *imgdata;              /* Pointer to the image data */
   long         *int_arr;              /* Array used when compressing data */
   int          magwsize;              /* Current size for magnifying window */
   int          magwsizes[2];          /* List of magnifying window sizes */
   int          magwsize_opt;          /* Current option for magnifying window
                                          size */
   XImage       *img_image;            /* XImage for the film image */
   XImage       *mag_image;            /* XImage for the magnifying window 
                                          (current) */
   XImage       *mag_images[2];        /* XImages for the magnifying window
                                          small and large */
   unsigned char *mag_data;            /* Magnifying window pixel data */
   int          cset;                  /* Colorset number */
   int          contrast_val;          /* Contast value 1-1024 */
   int          slider_len;            /* Contrast slider length */
   int          thresh_val;            /* Monochrome threshold value 0-63 */
   int          color_opt;             /* Color option type */
   int          mag;                   /* Current magnification factor */
   int          magnify_on;            /* Magnifier on flag =1 on (follows
                                          pointer) =0 off (frozen) */
   int          xc;                    /* Current x offset on image on which
                                          magnifier image is centered */
   int          yc;                    /* Current y offset on image on which
                                          magnifier image is centered */
   int          o_ax1;                 /* Origin pixel (1'st local axis) of
                                          displayed section of image 
                                          (non-zoomed) */
   int          o_ax2;                 /* Origin pixel (2'nd local axisy) */
   int          cur_cmp;               /* Current compression factor
                                          (allowing for zoom if any) */
   int          cur_expand;            /* Current pixel expansion factor
                                          after zoom (=1 if no zoom) */
   int          show_int_expand;       /* Show intensity values if current
                                          expand factor >=  this value and
                                          color option is 1 or 2 */
   int          show_int;              /* =1 allow intensity values option,
                                          =0 disable this option */
   int          ch_width;              /* Character width for printing
                                          intensities on display */
   int          ch_height;             /* Character height for printing
                                          intensities on display */
   int          intensity;             /* Last determined intensity */
   int          nreq_ax1;              /* Requested section length along ax1 */
   int          nreq_ax2;              /* Requested section length along ax2 */
   int          n_ax1;                 /* Actual section length along ax1 */
   int          n_ax2;                 /* Actual section length along ax2 */
   int          ax1pix;                /* Image pixel position (local ax1 axis)
                                          at mouse position 1 upwards */
   int          ax2pix;                /* Image pixel position (local ax2 axis)
                                          at mouse position (1 upwards)*/
   int          ax1pix_dn;             /* Last pixel ax1 on button 1 down */
   int          ax2pix_dn;             /* Last pixel ax2 on button 1 down */
   int          ax1pix_up;             /* Last pixel ax1 on button 1 up */
   int          ax2pix_up;             /* Last pixel ax2 on button 1 up */
   int          ax1pix_img;            /* Returned ax1 pixel no. for selected 
                                          spot or 1'st corner of rectangle*/
   int          ax2pix_img;            /* Returned ax2 pixel no. for selected 
                                          spot or 1'st corner of rectangle */
   int          ax1pix_img_up;         /* Returned ax1 pixel no. of 2'nd corner
                                          of selected rectangle */
   int          ax2pix_img_up;         /* Returned ax2 pixel no. of 2'nd corner
                                          of selected rectangle */
   int          sel_rect;              /* Rectangle being selected on display
                                          =1 yes, =0 no */
   int          sel_rect_xo;           /* x origin of selected rectangle;
                                          button 1 down position */
   int          sel_rect_yo;           /* y origin of selected rectangle;
                                          button 1 down position */
   int          sel_rect_x1;           /* x top left of selected rectangle */
   int          sel_rect_y1;           /* y top left of selected rectangle */
   int          sel_rect_x2;           /* x bottom right of rectangle */
   int          sel_rect_y2;           /* y bottom right of rectangle */
   Pixmap       sel_rect_t_line;       /* Pixmap to save top line of pixels
                                          where selected rectangle will be
                                          drawn */
   Pixmap       sel_rect_b_line;       /* Pixmap to save bottom line */
   Pixmap       sel_rect_l_line;       /* Pixmap to save left line */
   Pixmap       sel_rect_r_line;       /* Pixmap to save right line */
   int          use_image;             /* Flag for magnifying window o/p
                                          = 0 use stored compressed image data, 
                                          = 1 use data from full image */
   long         *monodata;             /* Buffer to hold pixel data for
                                          monochrome display on magnifying
                                          window */
   long         *magw_data;            /* Buffer to hold data read from
                                          full image for preparing magnifying
                                          window o/p */
   long         *magw_bgdata;          /* Buffer to hold background data read 
                                          from background image for preparing 
                                          magnifying window o/p */
   long         min_val;               /* Minimum intensity value in image*/
   long         max_val;               /* Maximum intensity value in image*/
   long         min_thresh;            /* Minimum threshold for scaling (image 
                                          plate) */
   long         max_thresh;            /* Maximum threshold for scaling (image
                                          plate) */
   int          on_off;                /* Active input flag 1=yes, 0=no */
   char         input_message[61];     /* Active strip message */
   int          ovly_typ;              /* Overlay type from input params.
                                          =1 on/off/offset
                                          =2 on/off/ov1 on/ov2 on */
   int          ovly1;                 /* =1 overlay 1 on, =0 off */
   int          ovly2;                 /* =1 overlay 2 on, =0 off */
   int          ovly_offset;           /* =1 offset overlay (if other
                                             conditions are met), =0 do not */
   int          ovly_opt;              /* Number of latest item selected from
                                          overlay menu */
   int          x_ovly_offset;         /* x offset for overlay when it is to be
                                          offset (in pixels on film display
                                          area) */
   int          y_ovly_offset;         /* y offset for overlay */
   int          max_alloc_symbols;     /* Max space currently allocated for
                                          storing symbols information */
   int          num_symb;              /* Number of symbols in  symbols list */
   XDl_image_symbol *symbols;          /* Pointer to array of symbols 
                                          structures */
   int          max_alloc_text;        /* Max space currently allocated for
                                          storing text strings information */
   int          num_text;              /* Number of text strings in text 
                                          strings list */
   XDl_image_text *text;               /* Pointer to array of text strings 
                                          structures */
   int          y_search[Y_SEARCH_MAX];/* Indexing array to speed up search for
                                          symbols */
   int          max_alloc_vects;       /* Max space currently allocated for
                                          storing vectors information */
   int          num_vect;              /* Number of vectors in vectors list */
   XDl_image_vect *vects;              /* Pointer to array of vectors 
                                          structures */
   unsigned long graypixels[64];       /* Pixels for gray scales */
   unsigned long ovly_pixels[7];       /* Pixels for overlay colours */
   unsigned short user_rgb[3][64];     /* User defined rgb values for a
                                          color scale */
   XDl_image_freeze freeze;            /* Details of 'frozen' rectangle */
   unsigned long freeze_pixel;         /* Colour for frozen area rectangle */
   int          bg_typ;                /* =1 include background options
                                          =0 do not */
   int          bg_present;            /* =1 background image has been supplied
                                          =0 background image not yet given */
   int          bg_opt;                /* =0 included (or bg_typ=0)
                                          =1 subtracted
                                          =2 background only */
   void        *bg_data;               /* Pointer to background data */
   int         bytes_per_zpix;         /* No. of bytes per z-pixel in
                                          XImage image data */
   int         mono_grscal;            /* For monochrome display
                                          =1, Simulate grayscale in mag window.
                                          =0, Do not   */
   int         disp_now;               /* Display image now (on refresh) =1 yes,
                                          =0, wait for xdl_image_display_now 
                                          call */
   int         reset;                  /* Reset routine in progress flag
                                          =1 yes, =0 no */
   int         exposed;                /* Set after 1'st expose event to
                                          main display window */
   int         magw_exposed;           /* Set after 1'st expose event to
                                          magnifying window */
   int         zoom_sel;               /* Zoom area request flag */
   int         zoom_on;                /* Image zoomed flag */
   int         zoom_rect;              /* Selected rectangle for possible
                                          zoom displayed */
   int         ps_on;                  /* Postscript option in progress
                                          flag =1 yes, =0 no */
   char *      title;                  /* Pointer to title for Postsript plot */
   char *      comments[4];            /* Pointers to comments for PS */
   FILE *      fpout;                  /* FILE pointer for output postscript 
                                          file */
   int         color_ps;               /* Colour postscript =1 yes, =0 no */
   int         annotate;               /* Additional annotation flag 1=yes 
                                          0=no */
   int         addate;                 /* Output date/time string 1=yes 0=no */
   int         addcomment;             /* Output comments 1=yes 0=no */
   int         ncomment;               /* No. of comments up to last input
                                          non-blank comment */
   int         icomment;               /* Comment no. of longest comment */
   int         lencomm;                /* Length of longest comment so far */
   int         ic;                     /* Counter through comments including
                                          blanks */
   int         inscal;                 /* Input user defined scaling 1=yes 
                                          0=no */
   int         addkey;                 /* Output key 1=yes 0=no */
   float       xwidth;                 /* Width for PS image plot */
   char        datstr[30];             /* Date and time string for PS */
   int         button3_up;             /* Button 3 is up =1 yes =0 no */
}  
   XDl_image;

/* Define routine types */
int xdl_image();
void xdl_image_rps();
void xdl_image_rp1();
void xdl_image_rp2();
void xdl_image_rp3();
void xdl_image_refresh();
void xdl_image_ep1();
void xdl_image_ep2();
void xdl_image_getpix1();
int xdl_image_getpix2();
void xdl_image_xy_to_local();
void xdl_image_xy_to_fs();
void xdl_image_local_to_xy();
void xdl_image_local_to_fs();
void xdl_image_cp1();
void xdl_image_cp2();
void xdl_image_cp3();
void xdl_image_cp4();
void xdl_image_cp5();
void xdl_image_cp6();
void xdl_image_cp7();
void xdl_image_cp8();
void xdl_image_cp9();
void xdl_image_rds();
void xdl_image_rdv();
void xdl_image_rdt();
void xdl_image_magw();
void xdl_image_setdata();
void xdl_image_set_freeze();
void xdl_image_restore_freeze();
void xdl_image_set_rect();
void xdl_image_restore_rect();
void xdl_image_intensity();
long xdl_image_unsq();
long xdl_image_unsq2();
long xdl_image_unsq3();
long xdl_image_unsq4();
void xdl_image_drawsymb();
void xdl_image_drawvec();
void xdl_image_dsppos();
void xdl_image_symbol_offsets();
void xdl_image_vector_offsets();
void xdl_image_text_offsets();
void xdl_image_ps();
void xdl_image_zoom();
void xdl_image_postscript();
void xdl_image_on_off();
void xdl_image_tidy_up();
int xdl_image_getpix();
int xdl_image_getpix_ovly();
int xdl_image_getrect();
int xdl_image_settings();
int xdl_image_reset();
int xdl_image_axnames();
int xdl_image_display_now();
int xdl_image_background();
int xdl_image_newimg();
int xdl_image_ovly_offset();
int xdl_image_set_colour();
int xdl_image_set_colormap();
int xdl_image_clear_symbols();
int xdl_image_del_symbol();
int xdl_image_input_message();
int xdl_image_symbol();
int xdl_image_symbols();
int xdl_image_vect();
int xdl_image_vects();
int xdl_image_vects_pos();
int xdl_image_boxes();
int xdl_image_clear_vects();
int xdl_image_del_vect();
int xdl_image_text();
int xdl_image_del_text();
int xdl_image_clear_text();
void xdl_image_getsize();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_image xdlf_image_
#define xdlf_image_getpix xdlf_image_getpix_
#define xdlf_image_getpix_ovly xdlf_image_getpix_ovly_
#define xdlf_image_getrect xdlf_image_getrect_
#define xdlf_image_settings xdlf_image_settings_
#define xdlf_image_reset xdlf_image_reset_
#define xdlf_image_axnames xdlf_image_axnames_
#define xdlf_image_display_now xdlf_image_display_now_
#define xdlf_image_getsize xdlf_image_getsize_
#define xdlf_image_symbol xdlf_image_symbol_
#define xdlf_image_symbols xdlf_image_symbols_
#define xdlf_image_background xdlf_image_background_
#define xdlf_image_newimg xdlf_image_newimg_
#define xdlf_image_ovly_offset xdlf_image_ovly_offset_
#define xdlf_image_set_colour xdlf_image_set_colour_
#define xdlf_image_set_colormap xdlf_image_set_colormap_
#define xdlf_image_clear_symbols xdlf_image_clear_symbols_
#define xdlf_image_del_symbol xdlf_image_del_symbol_
#define xdlf_image_vect xdlf_image_vect_
#define xdlf_image_vects xdlf_image_vects_
#define xdlf_image_vects_pos xdlf_image_vects_pos_
#define xdlf_image_boxes xdlf_image_boxes_
#define xdlf_image_clear_vects xdlf_image_clear_vects_
#define xdlf_image_del_vect xdlf_image_del_vect_
#define xdlf_image_text xdlf_image_text_
#define xdlf_image_del_text xdlf_image_del_text_
#define xdlf_image_clear_text xdlf_image_clear_text_
#define xdlf_image_input_message xdlf_image_input_message_

#endif

#if  LINKTYP == 2

#define xdlf_image XDLF_IMAGE
#define xdlf_image_getpix XDLF_IMAGE_GETPIX
#define xdlf_image_getpix_ovly XDLF_IMAGE_GETPIX_OVLY
#define xdlf_image_getrect XDLF_IMAGE_GETRECT
#define xdlf_image_settings XDLF_IMAGE_SETTINGS
#define xdlf_image_reset XDLF_IMAGE_RESET
#define xdlf_image_axnames XDLF_IMAGE_AXNAMES
#define xdlf_image_display_now XDLF_IMAGE_DISPLAY_NOW
#define xdlf_image_getsize XDLF_IMAGE_GETSIZE
#define xdlf_image_symbol XDLF_IMAGE_SYMBOL
#define xdlf_image_symbols XDLF_IMAGE_SYMBOLS
#define xdlf_image_background XDLF_IMAGE_BACKGROUND
#define xdlf_image_newimg XDLF_IMAGE_NEWIMG
#define xdlf_image_ovly_offset XDLF_IMAGE_OVLY_OFFSET
#define xdlf_image_set_colour XDLF_IMAGE_SET_COLOUR
#define xdlf_image_set_colormap XDLF_IMAGE_SET_COLORMAP
#define xdlf_image_clear_symbols XDLF_IMAGE_CLEAR_SYMBOLS
#define xdlf_image_del_symbol XDLF_IMAGE_DEL_SYMBOL
#define xdlf_image_vect XDLF_IMAGE_VECT
#define xdlf_image_vects XDLF_IMAGE_VECTS
#define xdlf_image_vects_pos XDLF_IMAGE_VECTS_POS
#define xdlf_image_boxes XDLF_IMAGE_BOXES
#define xdlf_image_clear_vects XDLF_IMAGE_CLEAR_VECTS
#define xdlf_image_del_vect XDLF_IMAGE_DEL_VECT
#define xdlf_image_text XDLF_IMAGE_TEXT
#define xdlf_image_del_text XDLF_IMAGE_DEL_TEXT
#define xdlf_image_clear_text XDLF_IMAGE_CLEAR_TEXT
#define xdlf_image_input_message XDLF_IMAGE_INPUT_MESSAGE

#endif

void xdlf_image();
void xdlf_image_getpix();
void xdlf_image_getpix_ovly();
void xdlf_image_getrect();
void xdlf_image_settings();
void xdlf_image_reset();
void xdlf_image_axnames();
void xdlf_image_display_now();
void xdlf_image_getsize();
void xdlf_image_symbol();
void xdlf_image_symbols();
void xdlf_image_background();
void xdlf_image_newimg();
void xdlf_image_ovly_offset();
void xdlf_image_set_colour();
void xdlf_image_set_colormap();
void xdlf_image_clear_symbols();
void xdlf_image_del_symbol();
void xdlf_image_vect();
void xdlf_image_vects();
void xdlf_image_vects_pos();
void xdlf_image_boxes();
void xdlf_image_clear_vects();
void xdlf_image_del_vect();
void xdlf_image_text();
void xdlf_image_del_text();
void xdlf_image_clear_text();
void xdlf_image_input_message();

/*-C:*/
int xdl_image (vh, vh_parent, x, y, cset, imgdata, type, 
                    ns_ras, nf_ras, nf_off,
                    o_ax1, o_ax2, nreq_ax1, nreq_ax2, ncmp_in,
                    iorder, jorder,
                    min_val, max_val, min_thresh, max_thresh, 
                    min_width, min_height, bg_typ, ovly_typ,
                    contrast, disp_now)
/*end*/
 
/*-Parameters:*/
int       vh;             /* User selected view-object handle (R)*/
int       vh_parent;      /* View-object handle for the parent base frame,
                             if 0 then a base  level frame is created to
                             enclose the image plate image view-object 
                             (R)*/
int       x;              /* x coordinate for the view-object.
                             If no parent may be -1 to give default x (R)*/
int       y;              /* y coordinate for the view-object.
                             If no parent may be -1 to give default y (R)*/
int       cset;           /* Number of the colorset to be used for the 
                             image display; must have 72 or more 
                             colorcells allocated (R)*/
char      *imgdata;       /* Pointer to the image data in byte, i2 or int
                             format as indicated by the 'type' parameter.
                             The image data must remain in place as long
                             as the view object is being used as it will 
                             be accessed again if the scaling thresholds 
                             etc. are changed. (R)*/
int       type;           /* 1 = unsigned byte data
                             2 = unsigned two-byte data
                             3 = signed integer data 
                             4 = 'squashed i2' data (if stored I<0
                                 ipix = -i*8
                             5 = 'squashed i2' data if stored I<0
                                 ipix = (I+32768)*8
                             6 = 'squashed i2' data if stored I<0
                                 ipix = (I+32768)*32
                             7 = 'squashed i2' data if stored I<0
                                 ipix = -(I+1)*256 + 32768
                             (R) */
int       ns_ras;         /* No. of rasters in slow moving direction of
                             full image (R)*/
int       nf_ras;         /* No. of rasters in fast moving direction of
                             full image (R)*/
int       nf_off;         /* Offset between the start of each stored
                             image row in fast axis rasters (to allow 
                             storage of image in a pre-dimensioned 2-D 
                             array (length of 2'nd dimension in 'C' and 
                             1'st dimension in Fortran)) (R)*/
int       o_ax1;          /* The local 'ax1' (e.g.xf) origin pixel number 
                             (pixels numbered from 1 up) for the section of 
                             image to be displayed (R)*/
int       o_ax2;          /* The local 'ax2' (e.g. yf) origin pixel number 
                             (pixels numbered from 1 up) for the section of 
                             image to be displayed (R)*/
int       nreq_ax1;       /* Requested size along the first local axis 
                             (ax1) of the section of the image to be 
                             displayed; the actual size may be reduced by 
                             up to ncmp-1 pixels if nreq_ax1 is not a 
                             multiple of ncmp (see below). The actual size 
                             will be ncmp*(nreq_ax1/ncmp) (R)*/
int       nreq_ax2;       /* Requested size along the second local axis 
                             (ax2) of the section of the image to be 
                             displayed; the actual size may be reduced by 
                             up to ncmp-1 pixels if nreq_ax2 is not a 
                             multiple of ncmp (see below). The actual size 
                             will be ncmp*(nreq_ax2/ncmp) (R)*/
int       ncmp_in;        /* The number of image pixels in both axis
                             direction to be combined into 1 pixel in the
                             displayed image (e.g. 4 will reduce a 
                             2400x2400 image to a 600x600 image. Give 
                             -ncmp if compression by sampling rather than 
                             averaging required (R) */
int       iorder;         /* Order of the data in the input image 
                             with respect to two local axes ax1, ax2 e.g.
                             (xf, yf) as a number from 1 to 8.
                             1   +ax1 slow   +ax2 fast    (+xf, +yf)
                             2   +ax1 slow   -ax2 fast    (+xf, -yf)
                             3   -ax1 slow   +ax2 fast    (-xf, +yf)
                             4   -ax1 slow   -ax2 fast    (-xf, -yf)
                             5   +ax2 slow   +ax1 fast    (+yf, +xf)
                             6   +ax2 slow   -ax1 fast    (+yf, -xf)
                             7   -ax2 slow   +ax1 fast    (-yf, +xf)
                             8   -ax2 slow   -ax1 fast    (-yf, -xf)
                             (R)*/
int       jorder;         /* Display order with respect to the two
                             local axes (1 to 8) along the X-windows
                             axes X horizontal, Y vertical with origin
                             at top left
                              +ax1 X (horiz) +ax2 Y (vert)  (+xf, +yf)
                             2 +ax1 X (horiz) -ax2 Y (vert)  (+xf, -yf)
                             3 -ax1 X (horiz) +ax2 Y (vert)  (-xf, +yf)
                             4 -ax1 X (horiz) -ax2 Y (vert)  (-xf, -yf)
                             5 +ax2 X (horiz) +ax1 Y (vert)  (+yf, +xf)
                             6 +ax2 X (horiz) -ax1 Y (vert)  (+yf, -xf)
                             7 -ax2 X (horiz) +ax1 Y (vert)  (-yf, +xf)
                             8 -ax2 X (horiz) -ax1 Y (vert)  (-yf, -xf)
                             
                             (2 was standard for Laue programs)
                             (R) */
int       min_val;        /* Minimum value of a pixel in the image.
                             Used as a lower limit for min_thresh (R)*/
int       max_val;        /* Maximum value of a pixel in the image.
                             Used as upper limit for max_thresh. (R)*/
int       min_thresh;     /* Minimum threshold value for the initial 
                             intensity scaling. min_val<=min_thesh<max_val. 
                             (R)*/
int       max_thresh;     /* Maximum threshold value for the initial
                             intensity scaling. 
                             min_thresh<max_thresh<=max_val. (R)*/
int       min_width;      /* minimum width for the displayed view-object.
                             The program will calculate the required width
                             for the view-object based on the image width,
                             the space required for the magnified image,
                             slider bars, menus etc. If this is less than
                             min_width then min_width will be used and the
                             view-object will be set out as appropriate. 
                             (R)*/
int       min_height;     /* minimum height for the displayed view object.
                             cf min_width (R) */
int       bg_typ;         /* =0 no background choice menu, 
                             =1 background choice menu (Set background via
                                xdl_image_background routine) (R)*/
int       ovly_typ;       /* = 1 Overlay options are on/off/offset.
                             = 2 Overlay options are on/off/1 on/2 on.
                                 Two separate sets of overlay symbols may
                                 be drawn.
                                 In case 1, the offset only applies to
                                 to overlay 1; In case 2 no offsetting is
                                 done but it is possible to switch between
                                 the two overlays. (R)*/
int       contrast;       /* Starting contrast value (1-1024) (R)*/
int       disp_now;       /* = 1 display image when this routine has been
                                 called.
                             = 0 do not display yet, wait for other 
                                 parameters to be set (e.g. contrast, 
                                 colour options) and a call to the  
                                 xdl_image_display_now routine (R)*/
/*end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
                bit 0 set: Requested parent not found in view-objects list
                bit 1 set: Unable to allocate memory for global data area
                bit 2 set: Unable to allocate memory for internal image 
                           data
                bit 4 set: Unable to allocate memory for magnifying window
                           image areas.
                bit 5 set: Unable to allocate memory for compress array
-end*/
{
   Window wfilm;            /* Window id for primary window */
   Window wparent;          /* Window id of parent window */
   XDl_image * gd;          /* Pointer to view-object's global data structure */
   int i,j,k,kk;            /* Temporary/loop variables */
   char * p;                /* Temporary pointer */
   int err;                 /* Error return flag */
   int iv;                  /* Offset into XDL_view_objects array */
   int width, height;       /* Full width/height of the object */
   int img_x, img_y;        /* Offsets in view-object for positioning the image
                               display area */
   int panel_x;             /* Offset x in view-object for positioning the
                               control panel area or item */
   int panel_y;             /* Offset y for control panel item */
   int font;                /* Font size 1 to 5 */
   int korder;              /* Order of image axes wrt the X-windows axes
                               (1-8) 
                               1   +axf X (horiz)   +axs Y (vert)   
                               2   +axf X (horiz)   -axs Y (vert)
                               3   -axf X (horiz)   +axs Y (vert)   
                               4   -axf X (horiz)   -axs Y (vert)  
                               5   +axs X (horiz)   +axf Y (vert)
                               6   +axs X (horiz)   -axf Y (vert)  
                               7   -axs X (horiz)   +axf Y (vert)   
                               8   -axs X (horiz)   -axf Y (vert)  
                            */
   int n_ax1;               /* Rounded down section size along local axis 1 
                                in pixels */
   int n_ax2;               /* Rounded down section size along local axis 2 
                               in pixels */
   int img_width;           /* Required image width */
   int img_height;          /* Required image height */
   int small_cursor;        /* Small cursor flag =1 yes, =0 no */
   int start_pos;           /* Start position for slider */
   char valstr[12];         /* Value string for numeric text item */
   int ytop;                /* Height required for top of object items */
   int cmp_type;            /* Compression type =1 average, =0 sample */
   XColor colr_cell;        /* Temporary Xcolor structure */
   XColor cursor_fg;        /* Structure for setting cross cursor foreground */
   XColor cursor_bg;        /* Structure for setting cross cursor background */
   Pixmap cursor_source;    /* Pixmap for cursor source */
   Pixmap cursor_mask;      /* Pixmap for cursor mask */
   static unsigned char cursor_source_bits[] = {
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x0F, 0x0F, 0x8F, 0x87, 0x87, 0x07,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00};

   static unsigned char cursor_mask_bits[] = {
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0xFF, 0xFF, 0x8F, 0xFF, 0xFF, 0x07,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x20, 0x00, 0x00, 0x00,   0x00, 0x00, 0x20, 0x00, 0x00, 0x00};
   static unsigned char curs15_source_bits[] ={
      0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x33, 0x66, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static unsigned char curs15_mask_bits[] = {
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x3f, 0x7e, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static char *color_strings[] =
   {
     "Black on white",
     "White on black",
     "Yellow if >Max",
     "Colour 1",
     "Colour 2",
     "User defined",
     "Random colour"
   };                                          /* Colour options menu strings */
   static char *mag_strings[] = 
   {"x2", "x3", "x4", "x5", "x6", "x7", "x8"}; /* Magnifying choice strings */
   static char *mag_strings2[] =
   {"x2", "x3", "x4", "x5", "x6", "x7", "x8", "x3G", "x6G", "x9G"};
                                               /* Magnifying choice strings
                                                  for monochrome case */
   static char *ovly_strings[] =
   {"on", "off", "offset"};                    /* Overlay choice strings for
                                                  overlay type 1*/
   static char *ovly_strings2[] =
   {"on", "off", "ov1 on", "ov2 on"};          /* Overlay choice strings for
                                                  overlay type 2*/
   static char *bg_strings[] =
   {"inc.", "sub.", "only"};                    /* Background choice strings */

   static int kord[8][8] = {
     {5,6,7,8,1,2,3,4},
     {6,5,8,7,3,4,1,2},
     {7,8,5,6,2,1,4,3},
     {8,7,6,5,4,3,2,1},
     {1,2,3,4,5,6,7,8},
     {3,4,1,2,6,5,8,7},
     {2,1,4,3,7,8,5,6},
     {4,3,2,1,8,7,6,5}};
   static int ncmp;

   err = 0;
   font = 2;
   if (type<1||type>7) type = 1;
   if (max_val<=min_val) max_val = min_val+1;
   if (min_thresh>=max_val) min_thresh = max_val - 1;
   if (min_thresh<min_val) min_thresh = min_val;
   if (max_thresh>max_val) max_thresh = max_val;
   if (max_thresh<=min_thresh) max_thresh = min_thresh + 1;
   if (ovly_typ!=2) ovly_typ = 1;
   if (bg_typ!=1) bg_typ = 0;
   if (disp_now!=0) disp_now = 1;
   if (iorder<1||iorder>8) iorder = 1;
   if (jorder<1||jorder>8) jorder = 1;
   if (contrast<1) contrast = 1;
   if (contrast>1024) contrast = 1024;
   cmp_type = 1;
   ncmp = ncmp_in;
   if (ncmp<0)
   {
      ncmp = -ncmp;
      cmp_type = 0;
   }
   /* Calculate the size requirements. *WARNING* update xdl_image_getsize 
      if this section changed */

   korder = kord[iorder-1][jorder-1];
   n_ax1 = ncmp*(nreq_ax1/ncmp);
   n_ax2 = ncmp*(nreq_ax2/ncmp);
   if (jorder<=4)
   {
      img_width = n_ax1/ncmp;
      img_height = n_ax2/ncmp;
   }
   else
   {
      img_width = n_ax2/ncmp;
      img_height = n_ax1/ncmp;
   }
   
   width = 405 + MAGWSIZE_W;
   if ((img_width+2+2*BORDER_WIDTH+2)>width) 
      width = img_width + 2 + 2*BORDER_WIDTH + 2;
   if (min_width>width) width = min_width;
   panel_x = BORDER_WIDTH + MAGWSIZE_W +2 + (width-2-385-MAGWSIZE_W-2)/2;
   img_x = (width-2-img_width-2)/2;
   ytop = 66 + BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   if (MAGWSIZE_H+2+ACT_STRIP_HEIGHT>ytop) ytop = MAGWSIZE_H+2+ACT_STRIP_HEIGHT;
   height = img_height + 2 + 2*BORDER_HEIGHT + 2 + ytop;
   if (min_height>height) height = min_height;
   img_y = ytop + (height-2-img_height-2-ytop)/2;

   /* Create a primary (top level) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      {
         err |= 1;
         return err;
      }
      /* Create frame window as parent frame was given */
      wparent = XDL_view_objects[iv]->wid;
      wfilm = xdl_create_frame_window (wparent, x, y, width, height,
                                       cset);
   }else{
   /* Create base frame window as no parent was specified */
      wfilm = xdl_create_base_frame_window ("xdl_image","xdl_film",
                         x, y, width, height);
   }

   /* Create structure for the routines' global data */

   p = (char *) malloc (sizeof(XDl_image));
   if (p==0) {err |= 2; return err;}
   gd = (XDl_image *) p;

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wfilm, vh_parent, p, 1085, xdl_image_on_off,
                        xdl_image_tidy_up);

   /* Store image related parameters */

   gd->ncmp = ncmp;
   gd->cur_cmp = ncmp;
   gd->cur_expand = 1;
   gd->cmp_type = cmp_type;
   gd->bg_cmp = 1;
   gd->bytes_per_zpix = (XDL_image_bits-1)/8 + 1;
   gd->type = type;
   if (type>3)
   {
      gd->type = 4;
      switch (type)
      {
         case 4:
         gd->unsq = xdl_image_unsq;
         break;
         case 5:
         gd->unsq = xdl_image_unsq2;
         break;
         case 6:
         gd->unsq = xdl_image_unsq3;
         break;
         case 7:
         gd->unsq = xdl_image_unsq4;
         break;
      }
   }
   gd->iorder = iorder;
   gd->jorder = jorder;
   gd->korder = korder;
   strcpy (gd->ax1_name,"ax1");
   strcpy (gd->ax2_name,"ax2");
   gd->imgdata = imgdata;
   gd->ns_ras = ns_ras;
   gd->nf_ras = nf_ras;
   gd->nf_off = nf_off;
   gd->o_ax1 = o_ax1;
   gd->o_ax2 = o_ax2;
   gd->nreq_ax1 = nreq_ax1;
   gd->nreq_ax2 = nreq_ax2;
   gd->n_ax1 = n_ax1;
   gd->n_ax2 = n_ax2;
   gd->img_width = img_width;
   gd->img_height = img_height;
   gd->use_image = 0;
   gd->img = NULL;
   gd->mag_data = NULL;
   gd->magw_data = NULL;
   gd->monodata = NULL;
   gd->magw_bgdata = NULL;
   gd->int_arr = NULL;
   
   /* Get full image size wrt. internal (X-windows x,y) axes */
   if (gd->korder<=4)
   {
      gd->nxrasts = nf_ras;
      gd->nyrasts = ns_ras;
   }
   else
   {
      gd->nxrasts = ns_ras;
      gd->nyrasts = nf_ras;
   }

   /* Get origin pixel wrt. internal (X-windows x,y) axes for full image */
   switch (gd->jorder)
   {
      case 1:
         gd->xo_pix = o_ax1;
         gd->yo_pix = o_ax2;
      break;

      case 2:
         gd->xo_pix = o_ax1;
         gd->yo_pix = gd->nyrasts - o_ax2 - nreq_ax2 + 2;
      break;

      case 3:
         gd->xo_pix = gd->nxrasts - o_ax1 - nreq_ax1 + 2;
         gd->yo_pix = o_ax2;
      break;

      case 4:
         gd->xo_pix = gd->nxrasts - o_ax1 - nreq_ax1 + 2;
         gd->yo_pix = gd->nyrasts - o_ax2 - nreq_ax2 + 2;
      break;

      case 5:
         gd->xo_pix = o_ax2;
         gd->yo_pix = o_ax1;
      break;

      case 6:
         gd->xo_pix = o_ax2;
         gd->yo_pix = gd->nxrasts - o_ax1 - nreq_ax1 + 2;
      break;

      case 7:
         gd->xo_pix = gd->nyrasts - o_ax2 - nreq_ax2 + 2;
         gd->yo_pix = o_ax1;
      break;

      case 8:
         gd->xo_pix = gd->nyrasts - o_ax2 - nreq_ax2 + 2;
         gd->yo_pix = gd->nxrasts - o_ax1 - nreq_ax1 + 2;
      break;
   }
   gd->cur_xo_pix = gd->xo_pix;
   gd->cur_yo_pix = gd->yo_pix;

   /* Allocate space for magnifying window image data */

   gd->mag_data = (unsigned char *) 
                   malloc (gd->bytes_per_zpix*MAGWSIZE*MAGWSIZE);
   if (gd->mag_data==0) {err|=4; return err;}

   /* Some other initialisations */

   gd->vh = vh;
   gd->wfilm = wfilm;
   gd->on_off = 0;
   gd->color_opt = 1;
   gd->contrast_val = contrast;
   gd->thresh_val = 32;
   gd->cset = cset;
   if (XDL_color>0)
   {
      gd->mag = DEFAULT_MAG;
   }
   else
   {
      gd->mag = 3;
   }
   gd->magnify_on = 1;
   gd->magwsizes[0] = MAGWSIZE_H;
   gd->magwsizes[1] = 2*(gd->magwsizes[0]);
   gd->magwsize = gd->magwsizes[0];
   gd->magwsize_opt = 1;
   gd->freeze.on = 0;
   gd->ax1pix = -1;
   gd->ax2pix = -1;
   gd->ax1pix_dn = -1;
   gd->ax2pix_dn = -1;
   gd->ax1pix_up = -1;
   gd->ax2pix_up = -1;
   gd->ax1pix_img = -1;
   gd->ax2pix_img = -1;
   gd->ax1pix_img_up = -1;
   gd->ax2pix_img_up = -1;
   gd->xc = -1;
   gd->yc = -1;
   gd->sel_rect = 0;
   gd->sel_rect_xo = 0;
   gd->sel_rect_yo = 0;
   gd->sel_rect_x1 = 0;
   gd->sel_rect_y1 = 0;
   gd->sel_rect_x2 = 0;
   gd->sel_rect_y2 = 0;
   gd->monodata = NULL;
   gd->mono_grscal = 0;
   gd->min_val = min_val;
   gd->max_val = max_val;
   gd->min_thresh = min_thresh;
   gd->max_thresh = max_thresh;
   gd->bg_typ = bg_typ;
   gd->bg_present = 0;
   gd->bg_opt = 0;
   gd->bg_data = NULL;
   gd->ovly_typ = ovly_typ;
   gd->ovly_opt = 1;
   gd->ovly1 = 1;
   gd->ovly2 = 1;
   gd->ovly_offset = 0;
   gd->x_ovly_offset = -4;
   gd->y_ovly_offset = 4;
   gd->max_alloc_symbols = 0;
   gd->num_symb = 0;
   gd->symbols = NULL;
   gd->max_alloc_text = 0;
   gd->num_text = 0;
   gd->text = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
   gd->vects = NULL;
   gd->disp_now = disp_now;
   gd->reset = 0;
   gd->exposed = 0;
   gd->magw_exposed = 0;
   gd->zoom_sel = 0;
   gd->zoom_on = 0;
   gd->zoom_rect = 0;
   gd->ps_on = 0;
   gd->title = 0;
   for (i=0;i<4;++i) gd->comments[i] = 0;
   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i] = 0;     
   p = strcpy (gd->input_message, "Input spot positions");
   gd->ch_width = XDL_fonts[1]->max_bounds.width;
   gd->ch_height = XDL_fonts[1]->ascent + XDL_fonts[1]->descent;
   gd->show_int_expand = xdl_ndigit(max_val)*gd->ch_width + 4;
   gd->show_int = 1;
   gd->intensity = 0;
   gd->button3_up = 1;

   /* Create active strip area */

   gd->wact = xdl_create_active_strip (wfilm, vh, 
                      width-2*BORDER_WIDTH-2, xdl_image_rps);

   /* Set initial color map */

   if (XDL_color>0)
   {
      if (XDL_csets_alloc==1)
      {
         xdl_set64gray (gd->cset, gd->contrast_val, 1);
         for (i=0;i<64;++i)
         {
            gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
         }
      }
      else
      {
         xdl_get64gray (gd->graypixels, gd->contrast_val, 1);
      }
   }
   else
   {
      for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_whitepixel;
      for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_blackpixel;
   }

   /* Set overlay colours */

   if (XDL_color>1)
   {
      colr_cell.flags = DoRed|DoGreen|DoBlue;
      /* Red */
      colr_cell.red = 65535;
      colr_cell.green = 0;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[64];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[0] = colr_cell.pixel;
      /* Yellow */
      colr_cell.red = 65535;
      colr_cell.green = 65535;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[65];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[1] = colr_cell.pixel;
      /* Green */
      colr_cell.red = 0;
      colr_cell.green = 65535;
      colr_cell.blue = 0;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[66];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[2] = colr_cell.pixel;
      /* Cyan */
      colr_cell.red = 0;
      colr_cell.green = 65535;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[67];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[3] = colr_cell.pixel;
      /* Blue */
      colr_cell.red = 0;
      colr_cell.green = 0;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[68];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[4] = colr_cell.pixel;
      /* Magenta */
      colr_cell.red = 65535;
      colr_cell.green = 0;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[69];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[5] = colr_cell.pixel;
      /* Freeze rectangle colour */
      colr_cell.red = 10000;
      colr_cell.green = 50000;
      colr_cell.blue = 65535;
      if (XDL_csets_alloc==1)
      {
         colr_cell.pixel = XDL_colorsets[cset].pixels[70];
         XStoreColor (XDL_display, XDL_colormap, &colr_cell);
      }
      else
      {
         XAllocColor (XDL_display, XDL_colormap, &colr_cell);
      }
      gd->ovly_pixels[6] = colr_cell.pixel;
      gd->freeze_pixel = colr_cell.pixel;
   }
   else
   {
      for (i=0;i<7;++i) gd->ovly_pixels[i] = XDL_blackpixel;
      gd->freeze_pixel = XDL_blackpixel;
   }

   /* Create film display area paint window */

   gd->wdisp = xdl_create_paint_window (wfilm, img_x, img_y, 1,
                    img_width+2, img_height+2, cset);

   xdl_add_window (vh, gd->wdisp, xdl_image_rp1, xdl_image_ep1);

   /* Prepare for event handling */

   XSelectInput (XDL_display, gd->wdisp, ExposureMask|StructureNotifyMask|
           ButtonPressMask|ButtonReleaseMask|PointerMotionMask|
           LeaveWindowMask);

   /* Set Cross cursor */

   small_cursor = 1;
   p = XGetDefault (XDL_display,"xdl_image","cursor");
   if (p!=NULL)
   {
      if (p[0]=='l') small_cursor = 0;
   }
   if (small_cursor)
   {
      cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                          (char *)curs15_source_bits, 15, 15);

      cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                          (char *)curs15_mask_bits, 15, 15);
   }
   else
   {
      cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                           (char *)cursor_source_bits, 43, 43);

      cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                           (char *)cursor_mask_bits, 43, 43);
   }

   cursor_fg.red = 0;
   cursor_fg.green = 0;
   cursor_fg.blue = 0;
   cursor_fg.flags = DoRed|DoGreen|DoBlue;

   cursor_bg.red = 65535;
   cursor_bg.green = 65535;
   cursor_bg.blue = 65535;
   cursor_bg.flags = DoRed|DoGreen|DoBlue;
   
   if (small_cursor)
   {
      gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                        &cursor_fg, &cursor_bg, 7, 7);
   }
   else
   {
      gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                     &cursor_fg, &cursor_bg, 21, 21);
   }

   XFreePixmap (XDL_display, cursor_source);
   XFreePixmap (XDL_display, cursor_mask);

   XDefineCursor (XDL_display, gd->wdisp, gd->cursor);

   /* Magnifier paint window */

   gd->wmag = xdl_create_paint_window (wfilm, BORDER_WIDTH, 
            BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1, gd->magwsize+2, 
            gd->magwsize+2, cset);

   xdl_add_window (vh, gd->wmag, xdl_image_rp2, xdl_image_ep2);

   /* Set Cross cursor */

   XDefineCursor (XDL_display, gd->wmag, gd->cursor);

   /* Prepare for event handling */
  
   XSelectInput (XDL_display, gd->wmag, ExposureMask|StructureNotifyMask|
           ButtonPressMask|ButtonReleaseMask|PointerMotionMask|LeaveWindowMask);

   /* Paint window for control panel */

   gd->wpanel = xdl_create_paint_window (wfilm, panel_x, 
                BORDER_HEIGHT + ACT_STRIP_HEIGHT, 1, 385, 66, cset);

   xdl_add_window (vh, gd->wpanel, xdl_image_rp3, NULL);

   XSelectInput (XDL_display, gd->wpanel, ExposureMask|StructureNotifyMask);

   /* Add contrast slider unless monochrome*/

   if (XDL_color>0)
   {
      panel_x = 220;
      panel_y = 24;
      gd->slider_len = 64;
      start_pos = (gd->slider_len*gd->contrast_val)/1024;

      gd->contrast_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                            "Contrast", font, gd->slider_len, start_pos, 0,
                            0, 0, xdl_image_cp1);
   }
   else
   {
      /* Add threshold slider for monochrome */
      panel_x = 220;
      panel_y = 24;
      start_pos = 32;

      gd->thresh_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                            "Thresh", font, 64, start_pos, 0,
                            0, 0, xdl_image_cp8);
   }

   /* Add cursor position string */

   panel_x = 230;
   panel_y = 4;

   gd->cursor_position = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                         "Cursor Position", 18, font, 0,  0, 0, 0);

   /* Add color type choice menu */

   panel_x = 4;
   panel_y = 44;

   i=2;
   if (XDL_color>1) i=7;

   gd->color_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Colour ", i, color_strings, 1, font, 0, 0, 0,
                      xdl_image_cp2);

   /* Add magnifying choice menu */

   panel_x = 200;
   panel_y = 44;

   if (XDL_color>0)
   {
      gd->mag_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                       "Mag", 7, mag_strings, 3, font, 0, 0, 0,
                       xdl_image_cp3);
   }
   else
   {
      gd->mag_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                       "Mag", 10, mag_strings2, 1, font, 0, 0, 0,
                       xdl_image_cp3);
   }

   /* Add Overlay on/off choice item */

   panel_x = 4;
   panel_y = 24;

   if (gd->ovly_typ==1)
   {
      gd->ovly_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                        "Overlay", 3, ovly_strings, 1, font, 0, 0, 0,
                        xdl_image_cp6);
   }
   else
   {
      gd->ovly_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                        "Overlay", 4, ovly_strings2, 1, font, 0, 0, 0,
                        xdl_image_cp6);
   }

   /* Add threshold numeric text input items image plate display */

   sprintf (valstr, "%ld", gd->min_thresh);

   gd->min_text_item = xdl_panel_value (vh, gd->wpanel, 4, 4,
                       "Min", valstr, 8, font, 0, 0, 0, xdl_image_cp5);

   sprintf (valstr, "%ld", gd->max_thresh);

   gd->max_text_item = xdl_panel_value (vh, gd->wpanel, 122, 4,
                       "Max", valstr, 8, font, 0, 0, 0, xdl_image_cp4);

   /* Add background choice menu if required */

   if (bg_typ>0)
   {
      panel_x = 150;
      panel_y = 24;
      gd->bg_choice = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                      "Bg", 3, bg_strings, 1, font, 0, 0, 0,
                       xdl_image_cp7);
   }

   /* Add Postscript print button */

   panel_x = 294;
   panel_y = 44;
   gd->ps_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                   "PS", 2, font, 2, 0, 0, 1, 1, 1, xdl_image_ps);

   /* Add Zoom button */

   panel_x = 330;
   panel_y = 44;
   gd->zoom_button = xdl_panel_button (vh, gd->wpanel, panel_x, panel_y,
                   "Zoom", 4, font, 2, 0, 0, 1, 1, 1, xdl_image_cp9);

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wdisp, 0, NULL);

   /* Initialise 'User defined' rgb colorscale */

   kk = 0;
   for (i=1;i<=4;++i)
   {
      for (j=1;j<=4;++j)
      {
         for (k=1;k<=4;++k)
         {
            gd->user_rgb[0][kk] = 16000*i;
            gd->user_rgb[1][kk] = 16000*j;
            gd->user_rgb[2][kk] = 16000*k;
            kk++;
         }
      }
   }

   /* Create pixmaps to store saved lines of pixels for select rectangle
      option. (Use this method rather than xor'ing to give control over
      colour of drawn outline */

   gd->sel_rect_t_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    gd->img_width, 1, XDL_image_depth);
   gd->sel_rect_b_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    gd->img_width, 1, XDL_image_depth);
   gd->sel_rect_l_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    1, gd->img_height, XDL_image_depth);
   gd->sel_rect_r_line = XCreatePixmap (XDL_display, gd->wdisp,
                                    1, gd->img_height, XDL_image_depth);

   /* Create data area for image data ZPixmap */

   gd->img = (char *) malloc ((gd->bytes_per_zpix)*img_width
                             *img_height*sizeof(char));
   if (gd->img==0) {err |= 4; return err;}

   /* Set up XImages for film/image-plate data and magnified data*/

   gd->img_image = XCreateImage (XDL_display, XDL_visual,
                                  XDL_image_depth, ZPixmap, 0, gd->img, img_width, 
                                  img_height, 8, 0);

   gd->mag_images[0] = XCreateImage (XDL_display, XDL_visual, 
                                 XDL_image_depth, ZPixmap, 0, 
                                 (char *) gd->mag_data, 
                                 gd->magwsizes[0], gd->magwsizes[0],
                                 8, 0);

   gd->mag_images[1] = XCreateImage (XDL_display, XDL_visual, 
                                 XDL_image_depth, ZPixmap, 0, 
                                 (char *) gd->mag_data, 
                                 gd->magwsizes[1], gd->magwsizes[1],
                                 8, 0);

   gd->mag_image = gd->mag_images[0];

   /* Allocate arrays for storing magnifying windows image & background
      data (as Intpix) */

   i = (MAGWSIZE+MIN_MAG-1)/(2*MIN_MAG);
   i = 2*i + 1;
   i = i*i*ncmp*ncmp;
   gd->magw_data = (long *) malloc (i*sizeof(long));
   if (gd->magw_data==0) {err |= 16; return err;}
   gd->magw_bgdata = (long *) malloc (i*sizeof(long));
   if (gd->magw_bgdata==0) {err |= 16; return err;}

   /* Set up array for compressing data */

   gd->int_arr = (long *) malloc (((nf_ras/ncmp)+1)*sizeof(long));
   if (gd->int_arr==0) {err |= 32; return err;}

   /* Set up pixels data */

   xdl_image_setdata (gd);

   /* For monochrome display set up extra array for handling magnifying
      window data */

   if (XDL_color==0)
   {
      i = MAGWSIZE + 2*MAX_MONO_MAG;
      gd->monodata = (long *) 
           malloc (i*i*sizeof(long));
      if (gd->monodata==0) {err |= 4; return err;}
   }

   return 0;
}

/* Fortran binding: xdlf_image */

void xdlf_image (ivh, ivhparent, ix, iy, icset, imgdata, itype, 
                 ns_ras, nf_ras, nf_off, io_ax1, io_ax2,
                 nreq_ax1, nreq_ax2, ncmp, iorder, jorder,
                 minval, maxval, minthr, maxthr, minw, minh, 
                 ibg, iovly, icontrast, idisp_now, ierr)
int *ivh, *ivhparent, *ix, *iy, *icset, *imgdata, *itype;
int *ns_ras, *nf_ras, *nf_off, *io_ax1, *io_ax2, *nreq_ax1, *nreq_ax2, *ncmp;
int *iorder, *jorder;
int *minval, *maxval, *minthr, *maxthr, *minw, *minh;
int *ibg, *iovly, *icontrast, *idisp_now, *ierr;
{
   *ierr = xdl_image (*ivh, *ivhparent, *ix, *iy, *icset, 
                      (void *) imgdata, *itype,
                      *ns_ras, *nf_ras, *nf_off, *io_ax1, *io_ax2,
                      *nreq_ax1, *nreq_ax2, *ncmp, *iorder, *jorder,
                      *minval, *maxval, *minthr, *maxthr, *minw, *minh, 
                      *ibg, *iovly, *icontrast, *idisp_now);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_image_rps      **
*********************************

Purpose: Repaint the active strip

Author:  John W. Campbell, July 1990

*/

void xdl_image_rps (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_image * gd;     /* Pointer to the global data area */

   gd = (XDl_image *) global;
   if (gd->on_off==0)
   {
      xdl_active_strip_off (wid);
   }
   else
   {
      xdl_active_strip_on (wid, gd->input_message, 0);
   }
}
/*
**************************
**     xdl_image_ps     **
**************************

Purpose: Callback for Postscript button push

Author:  John W. Campbell, December 1991

*/

void xdl_image_ps (iv, ibutton)

int iv;
int ibutton;

{
   XDl_image *gd;          /* Pointer to global data area */
   int vh;                 /* View-object handle */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   vh = XDL_view_objects[iv]->vh;
   gd->io_panel = xdl_panel_io (vh, gd->wdisp, 2, 2, 50, 10, 100, 2, 1, 1,
                                gd->img_width-4, 0, xdl_image_postscript);
   gd->ps_on = 1;
   xdl_panel_io_prompt (gd->io_panel, "Postscript file name: ", 0, 1);
}
/*
**************************
**     xdl_image_zoom   **
**************************

Purpose: Zoom image

Author:  John W. Campbell, March 1994

*/

void xdl_image_zoom (gd)

XDl_image * gd;

{
   int expanded;           /* Flag to ensure that at least some expansion
                              is done */
   float fac;              /* Required expansion factor based on selected
                              rectangle size */
   float fac1;             /* Height expansion factor */
   float expfac;           /* Expansion factor based on decreasing pixel 
                              compression factor and/or increasing pixel 
                              expansion factor */
   float cmpfac;           /* Composite compression/expansion factor */
   int x;                  /* x mid point of selected rectangle */
   int y;                  /* y mid point of selected rectangle */
   int xpix;               /* x pixel in full image (internal axes) of
                              centre of selected rectangle */
   int ypix;               /* y pixel in full image (internal axes) of
                              centre of selected rectangle */
   int i;                  /* Loop counter through symbols/vectors */
   int is;                 /* Loop counter through symbols */
   int y_start;            /* Y offset 0 to Y_SEARCH_MAX-1 */
   int y_start_prev;       /* Previous value of y_start */
   Window wid;             /* Window id */

   xdl_panel_label_wid (gd->select_label, &wid);
   xdl_delete_panel_item (gd->vh, wid);
   xdl_image_restore_rect (gd);
   x = (gd->sel_rect_x1+gd->sel_rect_x2)/2;
   y = (gd->sel_rect_y1+gd->sel_rect_y2)/2;

   /* Note: Assumes that both gd->cur_cmp and gd->cur_expand are not > 1 */
   if (gd->cur_cmp>1)
   {
      xpix = x*gd->cur_cmp + gd->cur_cmp/2;
      ypix = y*gd->cur_cmp + gd->cur_cmp/2;
   }
   else
   {
      xpix = x/gd->cur_expand;
      ypix = y/gd->cur_expand;
   }
   xpix += gd->cur_xo_pix;
   ypix += gd->cur_yo_pix;
   fac = (float)gd->img_width/(float)abs(gd->sel_rect_x1-gd->sel_rect_x2);
   fac1 = (float)gd->img_height/(float)abs(gd->sel_rect_y1-gd->sel_rect_y2);
   if (fac1<fac) fac = fac1;
   gd->zoom_rect = 0;
   expfac = 1.0;
   expanded = 0;
   if (gd->cur_cmp>1)
   {
      expfac = expfac * (float)(gd->cur_cmp) / (float)(gd->cur_cmp-1);
      gd->cur_cmp--;
      expanded = 1;
      while (gd->cur_cmp>1&&
            (expfac * (float)(gd->cur_cmp) / (float)(gd->cur_cmp-1))
            <=fac)
      {
         expfac = expfac * (float)(gd->cur_cmp) / (float)(gd->cur_cmp-1);
         gd->cur_cmp--;
      }
   }
   if (expanded==0)
   {
      expfac = expfac * (float)(gd->cur_expand+1) / (float)(gd->cur_expand);
      gd->cur_expand++;
   }
   while (expfac * (float)(gd->cur_expand+1) / (float)(gd->cur_expand)<=fac)
   {
      expfac = expfac * (float)(gd->cur_expand+1) / (float)(gd->cur_expand);
      gd->cur_expand++;
   }  
   if (XDL_color==0&&gd->cur_expand>8)
   {
      gd->cur_expand = 3*((gd->cur_expand+2)/3);
   }
   cmpfac = (float) gd->cur_cmp / (float) gd->cur_expand;
   gd->cur_xo_pix = nint (xpix - cmpfac*(gd->img_width/2));
   gd->cur_yo_pix = nint (ypix - cmpfac*(gd->img_height/2));
   if (gd->cur_xo_pix+cmpfac*gd->img_width>gd->nxrasts) gd->cur_xo_pix 
               = gd->nxrasts-cmpfac*gd->img_width;
   if (gd->cur_yo_pix+cmpfac*gd->img_height>gd->nyrasts) gd->cur_yo_pix 
               = gd->nyrasts-cmpfac*gd->img_height;
   if (gd->cur_xo_pix<1) gd->cur_xo_pix = 1;
   if (gd->cur_yo_pix<1) gd->cur_yo_pix = 1;
   gd->zoom_on = 1;

   xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);
   xdl_image_restore_freeze (gd);
   gd->magnify_on = 1;
   xdl_image_setdata (gd);
   if (gd->num_symb>0)
   {
      for (i=0;i<gd->num_symb;++i) xdl_image_symbol_offsets (gd,i);
      /* Update y-search offsets array */
      for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i]=0;
      y_start_prev=0;
      for (is=0;is<gd->num_symb;++is)
      {
         y_start = (gd->symbols[is].y_disp_offset * 
                   (Y_SEARCH_MAX-1)) / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         if (y_start>y_start_prev)
         {
            if (y_start-1<Y_SEARCH_MAX)
            {
               for(i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i]=is;
            }
         }
         y_start_prev=y_start;
      }
   }
   if (gd->num_vect>0)
   {
      for (i=0;i<gd->num_vect;++i) xdl_image_vector_offsets (gd,i);
   }
   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i) xdl_image_text_offsets (gd,i);
   }
   xdl_image_refresh (gd, 1);
   xdl_panel_label_set (gd->cursor_position," ",0,0);
   gd->zoom_sel = 0;
   xdl_panel_button_set (gd->zoom_button, "Zoom", 4, 1, 1);
   return;
}
/*
**********************************
**     xdl_image_postscript     **
**********************************

Purpose: Service postscript option

Author:  John W. Campbell, December 1991

*/

void xdl_image_postscript (iv, reply, len, max_rp, iprompt, iret)

int iv;          /* Position (offset) in view-objects list */
char * reply;    /* Reply string */
int len;         /* Length of reply string */
int max_rp;      /* Maximum allowed reply length */
int iprompt;     /* Prompt number */
int iret;        /* Code 1 =  non-blank string
                         0 =  blank string
                         -1 = escape */

{
   XDl_image *gd;            /* Pointer to global data area */
   int vh;                   /* View object handle */
   int xroot, yroot;         /* Root position of image window */
   char str2[26];            /* Temporary string */
   int i,j,l;              /* Temp/loop variables */
   int ll;                   /* Character string length */
   int blank;                /* Blank string 1=yes 0=no */
   int range;                /* Range between min and max scaling thresholds */
   int ih, iw;               /* Loop counters through compressed image */
   int high;                 /* Black high 1=high 0=low */
   int maxels;               /* Max no. of pixels along an axis of compressed
                                image */
   int xs_off;               /* Overlay  x offset for symbol/vector */
   int ys_off;               /* Overlay  y offset for symbol/vector */
   int xs;                   /* Symbol x position */
   int ys;                   /* Symbol y position */
   int symtyp;               /* Symbol type */
   int symsize;              /* Symbol size */
   int icolr;                /* Current overlay colour number */
   int last_colr;            /* Last overlay colour number */
   int yo;                   /* y origin of freeze box */
   int x1, y1, x2, y2;       /* Ends of overlay vector */
   int xt, yt;               /* Start position for text */
   int max_per_line;         /* Max no. of pixels to o/p per line of hex data */

   float xscal, yscal;       /* x, y postscript scaling factors for image */
   float xpcen, ypcen;       /* Page position for centre of image (mm) */
   float ytitl;              /* Page 'y' position for short title (mm) */
   float ydate;              /* Page 'y' position for date/time (mm) */
   float xorg, yorg;         /* Page origin position for image (points) */
   float xlen, ylen;         /* Image size on page (points) */
   float x1key, x2key, x3key;/* Page 'x' position for parts of key (mm) */
   float ykey, dykey;        /* Page 'y' position & lines offset for key (mm) */
   float ycomm, dycomm;      /* Page 'y' position & lines offset for comments
                                (mm) */
   float mmscal;             /* Scale factor mm to points */
   float xx;                 /* Temp floating point value */
   float r,g,b;              /* Red, green blue values fo overlay colour */

   long ipix;                /* Pixel value (intensity) */
   unsigned char pixval;     /* Pixel value for display (0-255) */
   unsigned char *byte_data; /* Pointer to byte image data*/
   I2pix *i2_data;           /* Pointer to image plate 'i2' data */
   I2squash *i2_squash;      /* Pointer to 'squashed i2' data */
   Intpix *int_data;         /* Pointer to signed integer data */
   unsigned char *byte_bgdata; /* Pointer to byte image background data*/
   I2pix *i2_bgdata;         /* Pointer to background data for i2 data */
   I2squash *i2_bgsquash;    /* Pointer to background 'squashed i2' data */
   Intpix *int_bgdata;       /* Pointer to background data for int data */
   unsigned char rpix[64];   /* Red pixel values 0-255 for colour image */
   unsigned char gpix[64];   /* Green pixel values 0-255 for colour image */
   unsigned char bpix[64];   /* Blue pixel values 0-255 for colour image */

   time_t tm;                /* Structure for timer routines */
   XColor colr_cell;         /* Temp XColor structure */
   XDl_image_symbol *smb;    /* Temp pointer to symbol structure */
   XDl_image_vect *vec;      /* Temp pointer to vector structure */
   XDl_image_text *text;     /* Temp pointer to text string structure */
   XDl_image_freeze *fz;     /* Temp pointer to freeze structure */
   int xpix;                 /* x pixel position wrt. X-windows axes */
   int ypix;                 /* y pixel position wrt. X-windows axes */
   int xpixmax;              /* Maximum displayed x pixel */
   int ypixmax;              /* Maximum displayed y pixel */
   int ax1min;               /* Minimum displayed pixel along local axis 1 */
   int ax1max;               /* Maximum displayed pixel along local axis 1 */
   int ax2min;               /* Minimum displayed pixel along local axis 2 */
   int ax2max;               /* Maximum displayed pixel along local axis 2 */
   int ax1rasts;             /* Total number of rasters along local axis 1 */
   int ax2rasts;             /* Total number of rasters along local axis 2 */
   int fpix;                 /* pixel position along image fast axis */
   int spix;                 /* pixel position along image slow axis */
   int ioff;                 /* Offset into image file */
   int ioff_bg;              /* Offset into background image file */
   float cmpfac;             /* Current image compression factor allowing
                                for compress and expand */
   Window widp;                /* Window id for panel io item */
   int num_xp;                 /* No. of pixels displayed along x */
   int num_yp;                 /* No. of pixels displayed along y */
   int xcen;                   /* x offset at centre of displayed pixel */
   int ycen;                   /* y offset at centre of displayed pixel */
   int ax1pix;                 /* pixel position wrt. local axis 1 */
   int ax2pix;                 /* pixel position wrt. local axis 2 */
   int ix;                     /* Loop counter through displayed x pixels */
   int iy;                     /* Loop counter through displayed y pixels */
   char str[20];               /* Temp string */
   XColor ccell;               /* Colour cell */
   static int fsiz[5] ={8,12,14,18,24}; /* Standard Courier font sizes */

   /* Initialisations */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   vh = XDL_view_objects[iv]->vh;
   xdl_rootxy (gd->wdisp, 2, 30, &xroot, &yroot);

   if (iret==-1) goto psend;


   /* Get name of output postscript file */
   if (iprompt==1)
   {
      if (iret==0) goto psend;

      gd->fpout = fopen (reply,"w");
      if (gd->fpout==0)
      {
         i = xdl_popup_notice (xroot, yroot, 
                          "**Cannot open file**",
                           0, 0, 0, "Retry", 0, "Cancel", 0, 2, 0);
         if (i==2) goto psend;
         xdl_panel_io_prompt (gd->io_panel, "Postscript file name: ", 0, 1);
         return;
      }

      /* Allocate strings memory */
      gd->title = (char *) malloc(81);
      if (gd->title==0)
      {
         xdl_popup_notice (xroot, yroot, 
                        "**Unable to allocate required memory for title**",
                        0, 0, 0, "Continue", 0, 0, 0, 2, 0);
         goto psend;
      }
      for (i=0;i<4;++i)
      {
         gd->comments[i] = (char *) malloc(81);
         if (gd->comments[i]==0)
         {
            xdl_popup_notice (xroot, yroot, 
                          "**Unable to allocate required memory for comments**",
                           0, 0, 0, "Continue", 0, 0, 0, 2, 0);
            goto psend;
         }
         strcpy (gd->comments[i],"");
      }
      xdl_panel_io_prompt (gd->io_panel, "Colour postscript [n]: ", 0, 2);
      return;
   }
   else if (iprompt==2)
   {
      /* Colour or black and white postscript */

      gd->color_ps = 0;
      str2[0] = 0;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='y'||str2[0]=='Y') gd->color_ps = 1;
      }
      xdl_panel_io_prompt (gd->io_panel, 
          "Automatic (a) or user defined (u) scaling [a]: ", 0, 3);
      return;
   }
   else if (iprompt==3)
   {
      /* Scaling of image */

      str2[0] = 0;
      gd->inscal = 0;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='u'||str2[0]=='U') gd->inscal = 1;
      }
      if (gd->inscal)
      {
         xdl_panel_io_prompt (gd->io_panel, 
             "Required width in cms (w/h<=18.0 cms) [15.0]: ", 0, 4);
      }
      else
      {
         xdl_panel_io_prompt (gd->io_panel, 
             "Short title: ", 0, 5);
      }
      return;
   }
   else if (iprompt==4)
   {
      gd->xwidth = 15.0;
      if (iret>0)
      {
         i = sscanf(reply,"%f",&xx);
         if (i>0) gd->xwidth = xx;
         if (gd->xwidth<=0.0) gd->xwidth = 15.0;
      }
      xdl_panel_io_prompt (gd->io_panel, "Short title: ", 0, 5);
      return;
   }
   else if (iprompt==5)
   {
      /* Annotation requirements*/
      reply[80]=0;
      strcpy (gd->title,reply);
      xdl_panel_io_prompt (gd->io_panel, 
             "Add further annotation [y]: ", 0, 6);
      return;
   }
   else if (iprompt==6)
   {
      str2[0] = 0;
      gd->annotate = 1;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='n'||str2[0]=='N') gd->annotate = 0;
      }
      if (gd->annotate)
      {
         xdl_panel_io_prompt (gd->io_panel, 
              "Add heading with date [y]: ", 0, 7);
      }
      else
      {
         goto prepare;
      }
      return;
   }
   else if (iprompt==7)
   {
      str2[0] = 0;
      gd->addate = 1;
      if (iret>0)
         {
            j = sscanf(reply,"%s",str2);
            if (str2[0]=='n'||str2[0]=='N') gd->addate = 0;
         }
      if (gd->addate)
      {
         time(&tm);
         strcpy(gd->datstr,ctime(&tm));
         gd->datstr[24] = 0;
      }
      xdl_panel_io_prompt (gd->io_panel, 
              "Add key [y]: ", 0, 8);
      return;
   }
   else if (iprompt==8)
   {
      str2[0] = 0;
      gd->addkey = 1;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='n'||str2[0]=='N') gd->addkey = 0;
      }
      xdl_panel_io_prompt (gd->io_panel, 
              "Add comments [y]: ", 0, 9);
      return;
   }
   else if (iprompt==9)
   {
      str2[0] = 0;
      gd->addcomment = 1;
      if (iret>0)
      {
         j = sscanf(reply,"%s",str2);
         if (str2[0]=='n'||str2[0]=='N') gd->addcomment = 0;
      }
      if (gd->addcomment==0) goto prepare;
      gd->ncomment = 0;
      gd->icomment = 1;
      gd->lencomm = 0;
      gd->ic = 0;
      xdl_panel_io_prompt (gd->io_panel, 
              ": ", 0, 10);
      return;
   }
   else if (iprompt>=10&&iprompt<=13)
   {
      reply[80] = 0;
      blank = 1;
      ll = strlen(reply);
      if (ll>0)
      {
         for (l=0;l<ll;++l)
         {
            if (reply[l]!=' ')
            {
               blank = 0;
               break;
            }
         }
      }
      if (blank==0)
      {
         gd->ncomment = gd->ic + 1;
         strcpy (gd->comments[gd->ic],reply);
         if (ll>gd->lencomm)
         {
            gd->icomment = gd->ncomment;
            gd->lencomm = ll;
         }
      }
      gd->ic++;
      if (iprompt==13) goto prepare;
      xdl_panel_io_prompt (gd->io_panel, 
              ": ", 0, iprompt+1);
      return;
   }

   /* Prepare postscript file */
   /* ======================= */

   prepare:

   xdl_panel_io_clear (gd->io_panel);

   xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);

   high = 1;
   if (gd->color_opt==2||gd->color_opt==5) high = 0;
   mmscal = 2.8346;
   xpcen = mmscal*105;
   ypcen = mmscal*175;
   ytitl = mmscal*75;
   ydate = mmscal*270;
   x1key = mmscal*8;
   x2key = mmscal*20;
   x3key = mmscal*61;
   ykey = mmscal*35;
   dykey = mmscal*5;
   ycomm = mmscal*60;
   dycomm = mmscal*5;
   cmpfac = (float) gd->cur_cmp / (float) gd->cur_expand;

   /* Initialisation */

   fprintf(gd->fpout,"%%!\n");
   fprintf(gd->fpout,"newpath\n");

   /* Short title */

   fprintf(gd->fpout,"/Helvetica findfont 20 scalefont setfont\n");
   fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ytitl);
   fprintf(gd->fpout,"(%s)\n",gd->title);
   fprintf(gd->fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");

   /* Other annotation */

   if (gd->annotate&&gd->addate)  /* Date/time string */
   {
      fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ydate);
      fprintf(gd->fpout,"(Image Plot: %s)\n",gd->datstr);
      fprintf(gd->fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");
   }

   if (gd->annotate&&gd->addkey) /* Key */
   {
      fprintf(gd->fpout,"/Helvetica findfont 15 scalefont setfont\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x1key,ykey);
      fprintf(gd->fpout,"(Key) show\n");
      fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(gd->fpout,"(Image section details) show\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x3key,ykey);
      xpixmax = (int) (cmpfac*(gd->img_width-1) + cmpfac/2.0 + .5);
      xpixmax += gd->cur_xo_pix -1;
      ypixmax = (int) (cmpfac*(gd->img_height-1) + cmpfac/2.0 + .5);
      ypixmax += gd->cur_yo_pix -1;
      xdl_image_xy_to_local (gd, gd->cur_xo_pix, gd->cur_yo_pix, 
                             &ax1min, &ax2min, 1);
      xdl_image_xy_to_local (gd, xpixmax, ypixmax, &ax1max, &ax2max, 1);
      if (ax1max<ax1min)
      {
         i = ax1min;
         ax1min = ax1max;
         ax1max = i;
      }
      if (ax2max<ax2min)
      {
         i = ax2min;
         ax2min = ax2max;
         ax2max = i;
      }
      if (gd->jorder<=4)
      {
         ax1rasts = gd->nxrasts;
         ax2rasts = gd->nyrasts;
      }
      else
      {
         ax1rasts = gd->nyrasts;
         ax2rasts = gd->nxrasts;
      }
      
      fprintf(gd->fpout,
              "(: %s %d to %d of %d,  %s %d to %d of %d) show\n",
               gd->ax1_name, ax1min, ax1max, ax1rasts,
               gd->ax2_name, ax2min, ax2max, ax2rasts);
      ykey -= dykey;
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(gd->fpout,"(Image orientation) show\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x3key,ykey);
      switch (gd->jorder)
      {
          case 1:
             fprintf(gd->fpout,"(: %s to the right, %s down) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 2:
             fprintf(gd->fpout,"(: %s to the right, %s up) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 3:
             fprintf(gd->fpout,"(: %s to the left, %s down) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 4:
             fprintf(gd->fpout,"(: %s to the left, %s up) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 5:
             fprintf(gd->fpout,"(: %s down, %s to the right) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 6:
             fprintf(gd->fpout,"(: %s up, %s to the right) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 7:
             fprintf(gd->fpout,"(: %s down, %s to the left) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;

          case 8:
             fprintf(gd->fpout,"(: %s up, %s to the left) show\n", 
                     gd->ax1_name, gd->ax2_name);
          break;
      }
      ykey -= dykey;
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x2key,ykey);
      fprintf(gd->fpout,"(Maximum pixel value) show\n");
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x3key,ykey);
      fprintf(gd->fpout,"(: %ld) show\n",gd->max_val);
      ykey -= dykey;
      fprintf(gd->fpout,"%.2f %.2f moveto\n",x2key,ykey);
      if(high)
      {
         fprintf(gd->fpout,"(Scaling thresholds) show\n");
         fprintf(gd->fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(gd->fpout,"(: minimum = %ld, maximum = %ld, black is high) show\n",
                 gd->min_thresh, gd->max_thresh);
      }
      else
      {
         fprintf(gd->fpout,"(Scaling thresholds) show\n");
         fprintf(gd->fpout,"%.2f %.2f moveto\n",x3key,ykey);
         fprintf(gd->fpout,"(: minimum = %ld, maximum = %ld, black is low) show\n",
                 gd->min_thresh, gd->max_thresh);
      }
   }
   
   if (gd->annotate&&gd->addcomment&&gd->ncomment>0) /* Comments */
   {
      fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
      fprintf(gd->fpout,"/xoff (%s) stringwidth pop 2 div neg def\n",
              gd->comments[gd->icomment-1]);
      if (gd->ncomment>3) ycomm += dycomm;
      for (i=0;i<gd->ncomment;++i)
      {
         fprintf(gd->fpout,"%.2f %.2f moveto\n",xpcen,ycomm);
         fprintf(gd->fpout,"xoff 0 rmoveto\n");      
         fprintf(gd->fpout,"(%s) show\n",gd->comments[i]);
         ycomm -= dycomm;
      }
   }
   fprintf(gd->fpout,"stroke\n");

   /* Get image size and draw box */

   maxels = gd->img_width;
   if (gd->img_height>maxels) maxels = gd->img_height;
   if (gd->inscal)
   {
      xscal = 10.0*gd->xwidth*mmscal/(float) gd->img_width;
   }
   else
   {
      xscal = 432.0/(float) maxels;
   }
   yscal = xscal;
   xlen = xscal*(gd->img_width);
   ylen = yscal*(gd->img_height);
   xorg = xpcen - xscal*(gd->img_width/2);
   yorg = ypcen - yscal*(gd->img_height/2);
   fprintf(gd->fpout,"%.2f %.2f moveto\n",xorg,yorg);
   fprintf(gd->fpout,"%.2f 0 rlineto\n",xlen);
   fprintf(gd->fpout,"0 %.2f rlineto\n",ylen);
   fprintf(gd->fpout,"%.2f 0 rlineto\n",-xlen);
   fprintf(gd->fpout,"0 %.2f rlineto\n",-ylen);
   fprintf(gd->fpout,"stroke\n");
   fprintf(gd->fpout,"%.2f %.2f moveto\n",xorg,yorg);
   fprintf(gd->fpout,"%.2f 0 rlineto\n",xlen);
   fprintf(gd->fpout,"0 %.2f rlineto\n",ylen);
   fprintf(gd->fpout,"%.2f 0 rlineto\n",-xlen);
   fprintf(gd->fpout,"0 %.2f rlineto\n",-ylen);
   fprintf(gd->fpout,"closepath clip\n");

   /* Reset position */

   fprintf(gd->fpout,"0 0 moveto\n");
   fprintf(gd->fpout,"save\n");
   fprintf(gd->fpout,"%.2f %.2f translate\n",xorg,yorg);
   fprintf(gd->fpout,"gsave\n");

   /* Output the image */

   fprintf(gd->fpout,"%.2f %.2f scale\n",xlen,ylen);

   if (gd->color_ps)
   {
      for (i=0;i<64;++i)
      {
         colr_cell.pixel = gd->graypixels[i];
         colr_cell.flags = DoRed|DoGreen|DoBlue;
         XQueryColor (XDL_display, XDL_colormap, &colr_cell);
         rpix[i] = colr_cell.red/256;
         gpix[i] = colr_cell.green/256;
         bpix[i] = colr_cell.blue/256;
      }
   }

   if (gd->color_ps)
   {
      fprintf(gd->fpout,"/picstr %d string def\n",3*gd->img_width); 
      fprintf(gd->fpout,"%d %d 8\n",gd->img_width,gd->img_height);
      fprintf(gd->fpout,"[ %d 0 0 %d 0 %d ]\n",gd->img_width,-gd->img_height,
              gd->img_height);
      fprintf(gd->fpout,"{ currentfile picstr readhexstring pop }\n");
      fprintf(gd->fpout,"false 3 colorimage\n");
      max_per_line = 12;
   }
   else
   {
      fprintf(gd->fpout,"/picstr %d string def\n",gd->img_width); 
      fprintf(gd->fpout,"%d %d 8\n",gd->img_width,gd->img_height);
      fprintf(gd->fpout,"[ %d 0 0 %d 0 %d ]\n",gd->img_width,-gd->img_height,
              gd->img_height);
      fprintf(gd->fpout,"{ currentfile picstr readhexstring pop }\n");
      fprintf(gd->fpout,"image\n");
      max_per_line = 40;
   }

   if (gd->type==1)
   {
      byte_data = (unsigned char *) gd->imgdata;
      byte_bgdata = (unsigned char *) gd->bg_data;
   }
   else if (gd->type==2)
   {
      i2_data = (I2pix *) gd->imgdata;
      i2_bgdata = (I2pix *) gd->bg_data;
   }
   else if (gd->type==3)
   {
      int_data = (Intpix *) gd->imgdata;
      int_bgdata = (Intpix *) gd->bg_data;
   }
   else if (gd->type==4)
   {
      i2_squash = (I2squash *) gd->imgdata;
      i2_bgsquash = (I2squash *) gd->bg_data;
   }
  
   range = gd->max_thresh - gd->min_thresh;

   i = 0;
   for (ih=0;ih<gd->img_height;++ih)
   {
      /* Note: Assumes that both gd->cur_cmp and gd->cur_expand are not > 1 */
      if (gd->cur_cmp>1)
      {
      ypix = ih*gd->cur_cmp + gd->cur_cmp/2;
      }
      else
      {
         ypix = ih/gd->cur_expand;
      }
      ypix += gd->cur_yo_pix;
      j = 0;
      for (iw=0;iw<gd->img_width;++iw)
      {
         /* Note: Assumes that both gd->cur_cmp and gd->cur_expand are not > 1 */
         if (gd->cur_cmp>1)
         {
            xpix = iw*gd->cur_cmp + gd->cur_cmp/2;
         }
         else
         {
            xpix = iw/gd->cur_expand;
         }
         xpix += gd->cur_xo_pix;
         xdl_image_xy_to_fs (gd, xpix, ypix, &fpix, &spix, 1);
         ioff = (spix-1)*gd->nf_off + fpix - 1;
         ioff_bg = ((spix-1)/gd->bg_cmp)*gd->nf_bgoff + (fpix-1)/gd->bg_cmp;
         if (gd->type==1)
         {
            if (gd->bg_opt==0)
            {
               ipix = (long) byte_data[ioff];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (long) byte_data[ioff];
                        ipix -= (long) byte_bgdata[ioff_bg];
                        break;
                     case 2:
                        ipix = (long) byte_bgdata[ioff_bg];
                        break;
                  }
               }
               else
               {
                  ipix = (long) byte_data[ioff];
               }
            }
         }
         else if (gd->type==2)
         {
            if (gd->bg_opt==0)
            {
               ipix = (long) i2_data[ioff];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (long) i2_data[ioff];
                        ipix -= (long) i2_bgdata[ioff_bg];
                        break;
                     case 2:
                        ipix = (long) i2_bgdata[ioff_bg];
                        break;
                  }
               }
               else
               {
                  ipix = (long) i2_data[ioff];
               }
            }
         }
         else if (gd->type==3)
         {
            if (gd->bg_opt==0)
            {
               ipix = (long) int_data[ioff];
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (long) int_data[ioff];
                        ipix -= (long) int_bgdata[ioff_bg];
                        break;
                     case 2:
                        ipix = (long) int_bgdata[ioff_bg];
                        break;
                  }
               }
               else
               {
                  ipix = (long) int_data[ioff];
               }
            }
         }
         else if (gd->type==4)
         {
            if (gd->bg_opt==0)
            {
               ipix = (long) gd->unsq(i2_squash[ioff]);
            }
            else
            {
               if (gd->bg_present==1)
               {
                  switch (gd->bg_opt)
                  {
                     case 1:
                        ipix = (long) gd->unsq(i2_squash[ioff]);
                        ipix -= (long) gd->unsq(i2_bgsquash[ioff_bg]);
                        break;
                     case 2:
                        ipix = (long) gd->unsq(i2_bgsquash[ioff_bg]);
                        break;
                  }
               }
               else
               {
                  ipix = (long) gd->unsq(i2_squash[ioff]);
               }
            }
         }

         if (gd->color_ps)
         {
            if (ipix<=gd->min_thresh)
            {
               pixval = 0;
            }
            else if (ipix>=gd->max_thresh)
            {
               pixval = 63;
            }
            else
            {
               pixval = 63*(ipix-gd->min_thresh)/range;
            }
            fprintf(gd->fpout,"%02X%02X%02X",rpix[pixval],gpix[pixval],
                    bpix[pixval]);
         }
         else
         {
            if (ipix<=gd->min_thresh)
            {
               pixval = 0;
            }
            else if (ipix>=gd->max_thresh)
            {
               pixval = 255;
            }
            else
            {
               pixval = 255*(ipix-gd->min_thresh)/range;
            }
            if (high) pixval = 255 - pixval;
            fprintf(gd->fpout,"%02X",pixval);
         }
         j++;
         if (j==max_per_line) {fprintf(gd->fpout,"\n"); j=0;}
         i++; 
      }
      if (gd->img_width%max_per_line!=0) fprintf(gd->fpout,"\n");
   }

   /* Overlays if needed */

   fprintf(gd->fpout,"grestore\n");
   fprintf(gd->fpout,"/M {moveto} def /L {lineto} def /S {stroke} def\n");
   fprintf(gd->fpout,"/color0 {0.0 0.0 0.0 setrgbcolor} def\n");
   if (gd->color_ps)
   {
      for (i=0;i<7;++i)
      {
         colr_cell.pixel = gd->ovly_pixels[i];
         colr_cell.flags = DoRed|DoGreen|DoBlue;
         XQueryColor (XDL_display, XDL_colormap, &colr_cell);
         r = colr_cell.red/65535.0;
         g = colr_cell.green/65535.0;
         b = colr_cell.blue/65535.0;
         fprintf(gd->fpout,"/color%d {%.4f %.4f %.4f setrgbcolor} def\n",
                 i+1,r,g,b);
      }
   }
   else
   {
      for (i=0;i<7;++i)
      {
         if (high)
         {
            fprintf(gd->fpout,"/color%d {0.0 0.0 0.0 setrgbcolor} def\n",i+1);
         }
         else
         {
            fprintf(gd->fpout,"/color%d {0.0 0.0 0.0 setrgbcolor} def\n",i+1);
         }
      }
   }
   /* Output symbols if needed */

   last_colr = 1;
   fprintf(gd->fpout,"color1\n");
   fprintf(gd->fpout,"%.5f %.5f scale\n",xscal,yscal);
   fprintf(gd->fpout,"0.5 0.5 translate\n"); /*Offset by half a pixel*/
   fprintf(gd->fpout,"1 setlinewidth\n");
   if (gd->ovly1>0||gd->ovly2>0)
   {
      if (gd->num_symb>0)
      {
         /*Define symbol drawing routines*/
         fprintf(gd->fpout,"/P { /ys exch def /xs exch def\n");
         fprintf(gd->fpout,"xs 0.5 sub ys M xs 0.5 add L\n");
         fprintf(gd->fpout,"S } def\n");
         fprintf(gd->fpout,"/VC { /hs exch def /ys exch def /xs exch def\n");
         fprintf(gd->fpout,"xs hs sub ys M xs hs add ys L\n");
         fprintf(gd->fpout,"xs ys hs sub M xs ys hs add L\n");
         fprintf(gd->fpout,"S } def\n");
         fprintf(gd->fpout,"/CX { /hs exch def /ys exch def /xs exch def\n");
         fprintf(gd->fpout,"xs hs sub ys hs sub M\n");
         fprintf(gd->fpout,"xs hs add ys hs add L\n");
         fprintf(gd->fpout,"xs hs add ys hs sub M\n");
         fprintf(gd->fpout,"xs hs sub ys hs add L\n");
         fprintf(gd->fpout,"S } def\n");
         fprintf(gd->fpout,"/SQ { /hs exch def /ys exch def /xs exch def\n");
         fprintf(gd->fpout,"xs hs sub ys hs sub M\n");
         fprintf(gd->fpout,"xs hs sub ys hs add L\n");
         fprintf(gd->fpout,"xs hs add ys hs add L\n");
         fprintf(gd->fpout,"xs hs add ys hs sub L\n");
         fprintf(gd->fpout,"xs hs sub ys hs sub L\n");
         fprintf(gd->fpout,"stroke } def\n");

         /*Draw symbols*/

         for (i=0;i<gd->num_symb;++i)
         {
            smb = &(gd->symbols[i]);
            if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            icolr = smb->color_flags&0XF;
            if (icolr!=last_colr)
            {
               fprintf(gd->fpout,"color%d\n",icolr);
               last_colr = icolr;
            }
            xs = smb->x_disp_offset + xs_off;
            ys = smb->y_disp_offset + ys_off;
            ys = gd->img_height - 1 - ys;
            symtyp = (smb->symb) / 10;
            symsize = (smb->symb) % 10;
            if (symsize==0)
            {
               fprintf(gd->fpout,"%d %d P\n",xs,ys);
            }
            else
            {
               switch (symtyp)
               {
                  case 0: /*Vertical cross*/
                     fprintf(gd->fpout,"%d %d %d VC\n",xs,ys,symsize);
                     break;
                  case 1: /*Cross*/
                     fprintf(gd->fpout,"%d %d %d CX\n",xs,ys,symsize);
                     break;
                  case 2: /*Square*/
                     fprintf(gd->fpout,"%d %d %d SQ\n",xs,ys,symsize);
                     break;
               }
            }
         }
      }

   /* Output vectors if needed */

      if (gd->num_vect>0)
      {
         for (i=0;i<gd->num_vect;++i)
         {
            vec = &(gd->vects[i]);
            if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            icolr = vec->color_flags&0XF;
            if (icolr!=last_colr)
            {
               fprintf(gd->fpout,"color%d\n",icolr);
               last_colr = icolr;
            }
            x1 = vec->x1_disp_offset + xs_off;
            y1 = gd->img_height - 1 - vec->y1_disp_offset - ys_off;
            x2 = vec->x2_disp_offset + xs_off;
            y2 = gd->img_height - 1 - vec->y2_disp_offset - ys_off;
            fprintf(gd->fpout,"%d %d M %d %d L S\n",x1,y1,x2,y2);
         }
      }
   }

   /* Output text strings if needed */

   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i)
      {
         text = &(gd->text[i]);
         if (gd->ovly1==0&&(text->color_flags&0X10)==0) continue;
         if (gd->ovly2==0&&(text->color_flags&0X10)!=0) continue;
         xs_off = 0;
         ys_off = 0;
         if (gd->ovly_offset==1&&(text->color_flags&0X20)==0)
         {
            xs_off = gd->x_ovly_offset;
            ys_off = gd->y_ovly_offset;
         }
         icolr = text->color_flags&0XF;
         if (icolr!=last_colr)
         {
            fprintf(gd->fpout,"color%d\n",icolr);
            last_colr = icolr;
         }
         if (text->bold)
         {
            fprintf(gd->fpout,"/Courier-Bold findfont %.3f scalefont setfont\n",
                    (float) fsiz[text->font-1]);
         }
         else
         {
            fprintf(gd->fpout,"/Courier findfont %.3f scalefont setfont\n",
                    (float) fsiz[text->font-1]);
         }
         xt = text->x_disp_offset + xs_off;
         yt = gd->img_height - 1 - text->y_disp_offset - ys_off;
         fprintf(gd->fpout,"%d %d M (%s) show\n",xt,yt,text->string);
      }
   }

   if (gd->magnify_on==0) 
   {
      fz = &(gd->freeze);
      fprintf(gd->fpout,"color7\n");
      yo = gd->img_height - 1 - fz->yo;
      fprintf(gd->fpout,"%d %d M\n",fz->xo, yo);
      fprintf(gd->fpout,"%d %d L\n",fz->xo+fz->width-1, yo);
      fprintf(gd->fpout,"%d %d L\n",fz->xo+fz->width-1, yo-fz->height+1);
      fprintf(gd->fpout,"%d %d L\n",fz->xo, yo-fz->height+1);
      fprintf(gd->fpout,"%d %d L S\n",fz->xo, yo);
   }

   /* Intensity values if needed */

   if (gd->show_int&&gd->cur_expand>=gd->show_int_expand&&gd->color_opt<=3)
   {
      fprintf(gd->fpout,"/Helvetica findfont 12 scalefont setfont\n");
      num_xp = nint ((float)gd->img_width/(float)gd->cur_expand);
      num_yp = nint ((float)gd->img_height/(float)gd->cur_expand);
      for (ix=0;ix<num_xp;++ix)
      {
         xpix = gd->cur_xo_pix + ix;
         xcen = ix*gd->cur_expand + gd->cur_expand/2;
         for (iy=0;iy<num_yp;++iy)
         {
            ypix = gd->cur_yo_pix + iy;
            ycen = iy*gd->cur_expand + gd->cur_expand/2;
            xdl_image_xy_to_local (gd, xpix, ypix, &ax1pix, &ax2pix, 1);
            xdl_image_intensity (gd, ax1pix, ax2pix, 0);
            sprintf (str,"%d",gd->intensity);
            ccell.pixel = XGetPixel(gd->img_image,xcen,ycen);
            XQueryColor (XDL_display, XDL_colormap, &ccell);
            if (ccell.red > 40000)
            {
               fprintf (gd->fpout,"0.0 0.0 0.0 setrgbcolor\n");
            }
            else
            {
               fprintf (gd->fpout,"1.0 1.0 1.0 setrgbcolor\n");
            }
            ycen = gd->img_height - 1 - ycen;
            fprintf (gd->fpout,"%d %d M (%d)\n", xcen, ycen, gd->intensity);
            fprintf(gd->fpout,"dup stringwidth pop 2 div neg 0 rmoveto show\n");
         }
      }
   }

   /* Complete output */

   fprintf(gd->fpout,"color1\n");
   fprintf(gd->fpout,"restore\n");
   fprintf(gd->fpout,"showpage\n");
   fclose (gd->fpout);

   xdl_panel_label_set (gd->cursor_position," ",0,0);
 
   psend:
   if (gd->title!=0) free(gd->title);
   for(i=0;i<4;++i) if (gd->comments[i]!=0) free(gd->comments[i]);
   xdl_panel_io_wid (gd->io_panel, &widp);
   xdl_delete_panel_item (vh, widp);
   gd->ps_on=0;
   return;
}

/*
**********************************
**     xdl_image_on_off         **
**********************************

Purpose: Routine called when switching application input on/off

Author:  John W. Campbell, July 1990

*/

void xdl_image_on_off (iv, i)

int iv;      /* Offset in XDL_view_objects array */
int i;       /* Flag =1 on, =0 off */

{
   XDl_image *gd;           /* Pointer to global data area */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (i==1) 
   {
      gd->on_off = 1;
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   else
   {
      gd->on_off = 0;
      xdl_active_strip_off (gd->wact);
   }
   return;
}

/*
********************************
**     xdl_image_tidy_up      **
********************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, July 1990

*/

void xdl_image_tidy_up (global)

char *global;

{
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Temp/loop variable */

   gd = (XDl_image *) global;
   if (gd->img!=NULL) free ((char *) gd->img);
   if (gd->mag_data!=NULL) free ((char *) gd->mag_data);
   if (gd->monodata!=NULL) free ((char *) gd->monodata);
   if (gd->symbols!=NULL) free ((char *) gd->symbols);
   if (gd->vects!=NULL) free ((char *) gd->vects);
   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i)
      {
         if (gd->text[i].string!=NULL) free ((char *) gd->text[i].string);
      }
   }
   if (gd->text!=NULL) free ((char *) gd->text);
   if (gd->magw_data!=NULL) free ((char *) gd->magw_data);
   if (gd->magw_bgdata!=NULL) free ((char *) gd->magw_bgdata);
   if (gd->int_arr!=NULL) free ((char *) gd->int_arr);
   XFreeCursor (XDL_display, gd->cursor);
   XFreeGC (XDL_display, gd->paint_gc);
   XFreePixmap (XDL_display, gd->sel_rect_t_line);
   XFreePixmap (XDL_display, gd->sel_rect_b_line);
   XFreePixmap (XDL_display, gd->sel_rect_l_line);
   XFreePixmap (XDL_display, gd->sel_rect_r_line);
   return;
}

/*
****************************
**     xdl_image_rp1      **
****************************

Purpose: Repaint displayed image area

Author:  John W. Campbell, July 1990

*/

void xdl_image_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_image * gd;        /* Pointer to global data area */

   gd = (XDl_image *) global;

   gd->exposed = 1;
   xdl_image_refresh (gd, 0);

   return;
}

/*
****************************
**     xdl_image_rp2      **
****************************

Purpose: Repaint the magnifying window on an expose

Author:  John W. Campbell, July 1990

*/

void xdl_image_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XDl_image * gd;      /* Pointer to global data area */

   gd = (XDl_image *) global;
   gd->magw_exposed = 1;
   XClearWindow (XDL_display, wid);  
   if (gd->xc<0) return; 
   xdl_image_magw (gd);
   return;
}

/*
****************************
**     xdl_image_rp3      **
****************************

Purpose: Repaint the control panel canvas surround

Author:  John W. Campbell, July 1990

*/

void xdl_image_rp3 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
********************************
**     xdl_image_refresh      **
********************************

Purpose: Refresh displayed image area and, if required, magnifying window

Author:  John W. Campbell, August 1990

*/

void xdl_image_refresh (gd, magw)

XDl_image *gd;          /* Pointer to global data area */
int magw;               /* =1, refresh magnifying window (if needed)
                           =0, do not */

{
   XDl_image_symbol *smb; /* Temp. pointer to symbol structure */
   XDl_image_vect *vec;   /* Temp. pointer to vector structure */
   XDl_image_text *text;  /* Temp. pointer to text string structure */
   int i;                      /* Loop counter */
   int xs_off;                 /* Overlay symbol x offset */
   int ys_off;                 /* Overlay symbol y offset */
   int num_xp;                 /* No. of pixels displayed along x */
   int num_yp;                 /* No. of pixels displayed along y */
   int xcen;                   /* x offset at centre of displayed pixel */
   int ycen;                   /* y offset at centre of displayed pixel */
   int xpix;                   /* x pixel position */
   int ypix;                   /* y pixel position */
   int ax1pix;                 /* pixel position wrt. local axis 1 */
   int ax2pix;                 /* pixel position wrt. local axis 2 */
   int ix;                     /* Loop counter through displayed x pixels */
   int iy;                     /* Loop counter through displayed y pixels */
   int xx;                     /* Start x position for character o/p */
   int yy;                     /* y position for characters o/p */
   char str[20];               /* Temp string */
   XColor ccell;               /* Colour cell */
   int ndig;                   /* No. of digist in intensity (allowing for -
                                  if present */
   int color;                  /* Colour for text */
   int ll;                     /* Length of text string */

   if (gd->exposed==0) return;
   if (gd->disp_now==0) return;

   XPutImage (XDL_display, gd->wdisp, gd->paint_gc, gd->img_image,
              0, 0, 0, 0, gd->img_width, gd->img_height); 

   if (gd->ovly1>0||gd->ovly2>0)
   {
      if (gd->num_symb>0)
      {
         for (i=0;i<gd->num_symb;++i)
         {
            smb = &(gd->symbols[i]);
            if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            xdl_image_drawsymb (gd, gd->wdisp,
                                     smb->x_disp_offset + xs_off,
                                     smb->y_disp_offset + ys_off,
                                     smb->symb, (smb->color_flags&0XF));
         }
      }
      if (gd->num_vect>0)
      {
         for (i=0;i<gd->num_vect;++i)
         {
            vec = &(gd->vects[i]);
            if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            xdl_image_drawvec (gd, gd->wdisp, gd->cur_expand,
                               vec->x1_disp_offset + xs_off,
                               vec->y1_disp_offset + ys_off,
                               vec->x2_disp_offset + xs_off,
                               vec->y2_disp_offset + ys_off,
                               vec->x1_offset,
                               vec->y1_offset,
                               vec->x2_offset,
                               vec->y2_offset,
                               (vec->color_flags&0XF));
         }
      }
      if (gd->num_text>0)
      {
         for (i=0;i<gd->num_text;++i)
         {
            text = &(gd->text[i]);
            if (gd->ovly1==0&&(text->color_flags&0X10)==0) continue;
            if (gd->ovly2==0&&(text->color_flags&0X10)!=0) continue;
            xs_off = 0;
            ys_off = 0;
            if (gd->ovly_offset==1&&(text->color_flags&0X20)==0)
            {
               xs_off = gd->x_ovly_offset;
               ys_off = gd->y_ovly_offset;
            }
            if (gd->exposed==1)
            {
               color = text->color_flags&0XF;
               ll = strlen(text->string);
               if (color==0)
               {
                  XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
               }
               else
               {
                  XSetForeground (XDL_display, gd->paint_gc, 
                                  gd->ovly_pixels[color-1]);
               }
               if (text->bold)
               {
                  XSetFont (XDL_display,gd->paint_gc,
                            XDL_bold_fonts[text->font-1]->fid);
               }
               else
               {
                  XSetFont (XDL_display,gd->paint_gc,
                            XDL_fonts[text->font-1]->fid);
               }
               XDrawString (XDL_display,gd->wdisp, gd->paint_gc,
                            text->x_disp_offset + xs_off,
                            text->y_disp_offset + ys_off,
                            text->string, ll);
            }
         }
      }
   }

   if (gd->show_int&&gd->cur_expand>=gd->show_int_expand&&gd->color_opt<=3)
   {
      XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);
      num_xp = nint ((float)gd->img_width/(float)gd->cur_expand);
      num_yp = nint ((float)gd->img_height/(float)gd->cur_expand);
      for (ix=0;ix<num_xp;++ix)
      {
         xpix = gd->cur_xo_pix + ix;
         xcen = ix*gd->cur_expand + gd->cur_expand/2;
         for (iy=0;iy<num_yp;++iy)
         {
            ypix = gd->cur_yo_pix + iy;
            ycen = iy*gd->cur_expand + gd->cur_expand/2;
            xdl_image_xy_to_local (gd, xpix, ypix, &ax1pix, &ax2pix, 1);
            xdl_image_intensity (gd, ax1pix, ax2pix, 0);
            ndig = xdl_ndigit(gd->intensity);
            if (gd->intensity<0) ndig++;
            xx=nint(xcen - (float)ndig*gd->ch_width/2.0);
            yy=ycen+gd->ch_height/2;
            sprintf (str,"%d",gd->intensity);
            if (XDL_color>0)
            {
               ccell.pixel = XGetPixel(gd->img_image,xcen,ycen);
               XQueryColor (XDL_display, XDL_colormap, &ccell);
               if (ccell.red > 40000)
               {
                  XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
               }
               else
               {
                  XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
               }
               for (i=0;i<ndig;++i)
               {
                  XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xx, ycen,
                             &str[i], 1);
                  xx += gd->ch_width;
               }
            }
            else
            {
               XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
               XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
               XDrawImageString (XDL_display, gd->wdisp, gd->paint_gc, 
                                 xx, ycen, str, ndig);
            }
         }
      }
      XSetBackground (XDL_display, gd->paint_gc, XDL_whitepixel);
   }

   if (gd->magnify_on==0) xdl_image_set_freeze (gd);

   if (magw==1&&gd->xc>=0) xdl_image_magw (gd);

   return;
}

/*
****************************
**     xdl_image_ep1      **
****************************

Purpose: Event handling for the image display canvas

Author:  John W. Campbell, July 1990

*/

void xdl_image_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   int x, y;                  /* Mouse position */
   XDl_image *gd;             /* Pointer to global data area */
   XEvent xevent;             /* X Event structure */
   char str[20];              /* Position label string */

   gd = (XDl_image *) global;
   switch (event->type)
   {
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_image_getpix1 (gd, x, y, 1);
            gd->ax1pix_dn = gd->ax1pix;
            gd->ax2pix_dn = gd->ax2pix;
            gd->sel_rect = 1;
            gd->sel_rect_xo = x;
            gd->sel_rect_yo = y;
            xdl_image_set_rect (gd, x, y);
            break;
         }
         else if (event->xbutton.button==Button3)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_image_getpix1 (gd, x, y, 0);
            xdl_image_intensity (gd, gd->ax1pix, gd->ax2pix, 1);
            gd->button3_up = 0;
            break;
         }
         else if (event->xbutton.button!=Button2)
         {
             break;
         }
         else  /*Button 2*/
         {
            if (gd->sel_rect) /* Option to cancel select rectangle if needed */
            {
               xdl_image_restore_rect (gd);
               gd->sel_rect = 0;
               break;
            }
            else   /* Normal button2 function; freeze/free magnifying window */
            {
               gd->magnify_on = 1 - gd->magnify_on;
               if (gd->magnify_on==0)
               {
                  xdl_image_set_freeze (gd);
                  gd->use_image = 1;
                  xdl_image_magw (gd);
                  break;
               }
               else
               {
                  xdl_image_restore_freeze (gd);
               }
            }
         }
      /* Note: Fall through to next case if magnify_on has just been set */

      case MotionNotify:
         x = event->xmotion.x;
         y = event->xmotion.y;
         /* Get later motion notify event if any, and remove any intermediate
            ones in the event queue */
         while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
                MotionNotify, &xevent))
         {
            x = xevent.xmotion.x;
            y = xevent.xmotion.y;
         }
         if (x<0) x = 0;
         if (x>=gd->img_width) x = gd->img_width - 1;
         if (y<0) y = 0;
         if (y>=gd->img_height) y = gd->img_height - 1;
         xdl_image_getpix1 (gd, x, y, gd->button3_up);
         if(!gd->button3_up)
	 {
            xdl_image_intensity (gd, gd->ax1pix, gd->ax2pix, 1);
         }
         if (gd->sel_rect)
         {
            xdl_image_restore_rect (gd);
            xdl_image_set_rect (gd, x, y);
         }
         if (gd->magnify_on==0) break;
         gd->xc = x;
         gd->yc = y;
         gd->use_image = 0;
         xdl_image_magw (gd);
         break;
      case LeaveNotify:
         gd->ax1pix = -1;
         gd->ax2pix = -1;
         xdl_panel_label_set (gd->cursor_position, "Cursor position", 0, 0);
         break;
      case ButtonRelease:
         if (event->xbutton.button==Button1&&gd->sel_rect)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            xdl_image_getpix1 (gd, x, y, 1);
            gd->ax1pix_up = gd->ax1pix;
            gd->ax2pix_up = gd->ax2pix;
            gd->sel_rect = 0;
            if (gd->zoom_sel)
            {
               if (abs(gd->sel_rect_x1-gd->sel_rect_x2)<10||
                   abs(gd->sel_rect_y1-gd->sel_rect_y2)<10)
               {
                  xdl_image_restore_rect (gd);
                  gd->zoom_rect = 0;
               }
               else
               {
                  gd->zoom_rect = 1;
               }
               if (gd->zoom_rect) xdl_image_zoom(gd);
            }
            else
            {
               xdl_image_restore_rect (gd);
               if (gd->on_off==1)
               {
                  gd->ax1pix_img = gd->ax1pix_dn;
                  gd->ax2pix_img = gd->ax2pix_dn;
                  gd->ax1pix_img_up = gd->ax1pix_up;
                  gd->ax2pix_img_up = gd->ax2pix_up;
                  XDL_return_data_vh = XDL_view_objects[iv]->vh;
                  return;
               }
            }
         }
         if (event->xbutton.button==Button3)
         {
            sprintf (str, "%s=%d, %s=%d", gd->ax1_name, gd->ax1pix, 
                           gd->ax2_name, gd->ax2pix);
            xdl_panel_label_set (gd->cursor_position, str, 0, 0);
            gd->button3_up = 1;
         }
         break;
   }
   return;
}

/*
****************************
**     xdl_image_ep2      **
****************************

Purpose: Event handling for the magnifying window

Author:  John W. Campbell, July 1990

*/

void xdl_image_ep2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char *global;
void (*callback)();

{
   int x, y;                /* Mouse position */
   int i;                   /* Temp. variable */
   XDl_image *gd;           /* Pointer to global data area */
   XEvent xevent;           /* Temp. XEvent structure */
   char str[20];            /* Position label string */

   gd = (XDl_image *) global;

   switch (event->type)
   {
      case MotionNotify:
         x = event->xmotion.x;
         y = event->xmotion.y;
         /* Get later motion notify event if any, and remove any intermediate
            ones in the event queue */
         while (XCheckTypedWindowEvent (XDL_display, gd->wmag,
                MotionNotify, &xevent))
         {
            x = xevent.xmotion.x;
            y = xevent.xmotion.y;
         }
         xdl_image_getpix2 (gd, x, y, gd->button3_up);
         if (!gd->button3_up)
	 {
            xdl_image_intensity (gd, gd->ax1pix, gd->ax2pix, 1);
         }
         break;
      case ButtonPress:
         if (event->xbutton.button==Button1)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            i = xdl_image_getpix2 (gd, x, y, 1);
            if (i>0) return;
            if (gd->on_off==1)
            {
               gd->ax1pix_img = gd->ax1pix;
               gd->ax2pix_img = gd->ax2pix;
               XDL_return_data_vh = XDL_view_objects[iv]->vh;
               return;
            }
            break;
         }
         else if (event->xbutton.button==Button2)
         {
            if (gd->magnify_on==0) xdl_image_restore_freeze(gd);
            gd->magwsize_opt = 3 - gd->magwsize_opt;
            gd->magwsize = gd->magwsizes[gd->magwsize_opt-1];
            gd->mag_image = gd->mag_images[gd->magwsize_opt-1];
            XResizeWindow (XDL_display, gd->wmag, gd->magwsize, gd->magwsize);
            if (gd->magnify_on==0) xdl_image_set_freeze (gd);
            xdl_image_magw (gd);
            break;
         }
         else if (event->xbutton.button==Button3)
         {
            x = event->xbutton.x;
            y = event->xbutton.y;
            i = xdl_image_getpix2 (gd, x, y, 0);
            if (i>0) return;
            xdl_image_intensity (gd, gd->ax1pix, gd->ax2pix, 1);
            gd->button3_up = 0;
            break;
         }
         break;
      case LeaveNotify:
         gd->ax1pix = -1;
         gd->ax2pix = -1;
         xdl_panel_label_set (gd->cursor_position, "Cursor position", 0, 0);
         break;
      case ButtonRelease:
         if (event->xbutton.button==Button3)
         {
            gd->button3_up = 1;
            sprintf (str, "%s=%d, %s=%d", gd->ax1_name, gd->ax1pix, 
                           gd->ax2_name, gd->ax2pix);
            xdl_panel_label_set (gd->cursor_position, str, 0, 0);
         }
         break;
   }
   return;
}

/*
******************************
**     xdl_image_getpix1    **
******************************

Purpose: Get film image pixel position from x,y mouse position in main film
         display area and show value; Sets gd->ax1pix, gd->ax2pix.

Author:  John W. Campbell, August 1993

*/

void xdl_image_getpix1 (gd, x, y, show)

XDl_image *gd;        /* Pointer to global data area */
int x;                /* Mouse x position in film display area */
int y;                /* Mouse y position in film display area */
int show;             /* Display position =1 yes =0 no */

{
   char str[20];             /* String to hold position label string */
   int xpix;                 /* 'x' pixel value wrt internal (X-window) axes */
   int ypix;                 /* 'y' pixel value wrt internal (X-window) axes */


   /* Note: Assumes that both gd->cur_cmp and gd->cur_expand are not > 1 */
   if (gd->cur_cmp>1)
   {
      xpix = x*gd->cur_cmp + gd->cur_cmp/2;
      ypix = y*gd->cur_cmp + gd->cur_cmp/2;
   }
   else
   {
      xpix = x/gd->cur_expand;
      ypix = y/gd->cur_expand;
   }
   xpix += gd->cur_xo_pix;
   ypix += gd->cur_yo_pix;
   xdl_image_xy_to_local (gd, xpix, ypix, &gd->ax1pix, &gd->ax2pix, 1);

   if (show)
   {
      sprintf (str, "%s=%d, %s=%d", gd->ax1_name, gd->ax1pix, 
               gd->ax2_name, gd->ax2pix);
      xdl_panel_label_set (gd->cursor_position, str, 0, 0);
   }
   return;
}
/*
*********************************
**     xdl_image_xy_to_local   **
*********************************

Purpose: Convert internal image pixel position (X-windows axes) to local
         axes pixel position (pixel or offsets)

Author:  John W. Campbell, August 1993

*/

void xdl_image_xy_to_local (gd, xpix, ypix, ax1_pix, ax2_pix, typ)

XDl_image *gd;        /* Pointer to global data area (R)*/
int xpix;             /* Pixel 'x' position wrt. internal axes (R)*/
int ypix;             /* Pixel 'y' position wrt. internal axes (R)*/
int *ax1_pix;         /* Pixel local axis 1 position (W)*/
int *ax2_pix;         /* Pixel local axis 2 position (W)*/
int typ;              /* =+1 pixel to pixel, =-1 offset to offset */

{
   switch (gd->jorder) /* along X-windows x, y */
   {
       case 1:  /* +ax1, +ax2 */
          *ax1_pix = xpix;
          *ax2_pix = ypix;
       break;

       case 2:  /* +ax1, -ax2 */
          *ax1_pix = xpix;
          *ax2_pix = gd->nyrasts - ypix + typ;
       break;         

       case 3:  /* -ax1, +ax2 */
          *ax1_pix = gd->nxrasts - xpix + typ;
          *ax2_pix = ypix;
       break;

       case 4:  /* -ax1, -ax2 */
          *ax1_pix = gd->nxrasts - xpix + typ;
          *ax2_pix = gd->nyrasts - ypix + typ;
       break;

       case 5:  /* +ax2, +ax1 */
          *ax1_pix = ypix;
          *ax2_pix = xpix;
       break;

       case 6:  /* +ax2, -ax1 */
          *ax1_pix = gd->nyrasts - ypix + typ;
          *ax2_pix = xpix;
       break;         

       case 7:  /* -ax2, +ax1 */
          *ax1_pix = ypix;
          *ax2_pix = gd->nxrasts - xpix + typ;
       break;

       case 8:  /* -ax2, -ax1 */
          *ax1_pix = gd->nyrasts - ypix + typ;
          *ax2_pix = gd->nxrasts - xpix + typ;
       break;
   }
   return;
}
/*
*********************************
**     xdl_image_xy_to_fs      **
*********************************

Purpose: Convert internal image pixel position (X-windows axes) to 
         actual image (fast and slow axis) pixel position - actual
         positions or offsets

Author:  John W. Campbell, August 1993

*/

void xdl_image_xy_to_fs (gd, xpix, ypix, axf_pix, axs_pix, typ)

XDl_image *gd;        /* Pointer to global data area (R)*/
int xpix;             /* Pixel 'x' position wrt. internal axes (R)*/
int ypix;             /* Pixel 'y' position wrt. internal axes (R)*/
int *axf_pix;         /* Pixel fast axis (W)*/
int *axs_pix;         /* Pixel slow axis (W)*/
int typ;              /* =+1 pixel to pixel, =-1 offset to offset (R) */

{
   switch (gd->korder) /* along X-windows x, y */
   {
       case 1:  /* +axf, +axs */
          *axf_pix = xpix;
          *axs_pix = ypix;
       break;

       case 2:  /* +axf, -axs */
          *axf_pix = xpix;
          *axs_pix = gd->nyrasts - ypix + typ;
       break;         

       case 3:  /* -axf, +axs */
          *axf_pix = gd->nxrasts - xpix + typ;
          *axs_pix = ypix;
       break;

       case 4:  /* -axf, -axs */
          *axf_pix = gd->nxrasts - xpix + typ;
          *axs_pix = gd->nyrasts - ypix + typ;
       break;

       case 5:  /* +axs, +axf */
          *axf_pix = ypix;
          *axs_pix = xpix;
       break;

       case 6:  /* +axs, -axf */
          *axf_pix = gd->nyrasts - ypix + typ;
          *axs_pix = xpix;
       break;         

       case 7:  /* -axs, +axf */
          *axf_pix = ypix;
          *axs_pix = gd->nxrasts - xpix + typ;
       break;

       case 8:  /* -axs, -axf */
          *axf_pix = gd->nyrasts - ypix + typ;
          *axs_pix = gd->nxrasts - xpix + typ;
       break;
   }
   return;
}
/*
*********************************
**     xdl_image_local_to_fs   **
*********************************

Purpose: Convert pixel position in local coordinates to actual image 
        (slow and fast axis) pixel positions or offsets

Author:  John W. Campbell, August 1993

*/

void xdl_image_local_to_fs (gd, ax1_pix, ax2_pix, axf_pix, axs_pix, typ)

XDl_image *gd;        /* Pointer to global data area (R)*/
int ax1_pix;          /* Pixel local axis 1 position (R)*/
int ax2_pix;          /* Pixel local axis 2 position (R)*/
int *axf_pix;         /* Pixel position (fast axis) (W)*/
int *axs_pix;         /* Pixel position (slow axis) (W)*/
int typ;              /* =+1 pixel to pixel, =-1 offset to offset */

{
   switch (gd->iorder) 
   {
       case 1:  /* +ax1, +ax2 */
          *axs_pix = ax1_pix;
          *axf_pix = ax2_pix;
       break;

       case 2:  /* +ax1, -ax2 */
          *axs_pix = ax1_pix;
          *axf_pix = gd->nf_ras - ax2_pix  + typ;
       break;         

       case 3:  /* -ax1, +ax2 */
          *axs_pix = gd->ns_ras - ax1_pix + typ;
          *axf_pix = ax2_pix;
       break;

       case 4:  /* -ax1, -ax2 */
          *axs_pix = gd->ns_ras - ax1_pix + typ;
          *axf_pix = gd->nf_ras - ax2_pix + typ;
       break;

       case 5:  /* +ax2, +ax1 */
          *axs_pix = ax2_pix;
          *axf_pix = ax1_pix;
       break;

       case 6:  /* +ax2, -ax1 */
          *axs_pix = ax2_pix;
          *axf_pix = gd->nf_ras - ax1_pix + typ;
       break;         

       case 7:  /* -ax2, +ax1 */
          *axs_pix = gd->ns_ras - ax2_pix + typ;
          *axf_pix = ax1_pix;
       break;

       case 8:  /* -ax2, -ax1 */
          *axs_pix = gd->ns_ras - ax2_pix + typ;
          *axf_pix = gd->nf_ras - ax1_pix + typ;
       break;
   }
   return;
}
/*
*********************************
**     xdl_image_local_to_xy   **
*********************************

Purpose: Convert pixel position in local coordinates to internal image 
        (X-windows x,y axes) pixel positions or offsets

Author:  John W. Campbell, August 1993

*/

void xdl_image_local_to_xy (gd, ax1_pix, ax2_pix, xpix, ypix, typ)

XDl_image *gd;        /* Pointer to global data area (R)*/
int ax1_pix;          /* Pixel local axis 1 position (R)*/
int ax2_pix;          /* Pixel local axis 2 position (R)*/
int *xpix;            /* Pixel position (X-windows x) (W)*/
int *ypix;            /* Pixel position (X-windows y) (W)*/
int typ;              /* =+1 pixel to pixel, =-1 offset to offset */

{
   switch (gd->jorder) 
   {
       case 1:  /* +ax1, +ax2 */
          *xpix = ax1_pix;
          *ypix = ax2_pix;
       break;

       case 2:  /* +ax1, -ax2 */
          *xpix = ax1_pix;
          *ypix = gd->nyrasts - ax2_pix  + typ;
       break;         

       case 3:  /* -ax1, +ax2 */
          *xpix = gd->nxrasts - ax1_pix + typ;
          *ypix = ax2_pix;
       break;

       case 4:  /* -ax1, -ax2 */
          *xpix = gd->nxrasts - ax1_pix + typ;
          *ypix = gd->nyrasts - ax2_pix + typ;
       break;

       case 5:  /* +ax2, +ax1 */
          *xpix = ax2_pix;
          *ypix = ax1_pix;
       break;

       case 6:  /* +ax2, -ax1 */
          *xpix = ax2_pix;
          *ypix = gd->nxrasts - ax1_pix + typ;
       break;         

       case 7:  /* -ax2, +ax1 */
          *xpix = gd->nyrasts - ax2_pix + typ;
          *ypix = ax1_pix;
       break;

       case 8:  /* -ax2, -ax1 */
          *xpix = gd->nyrasts - ax2_pix + typ;
          *ypix = gd->nxrasts - ax1_pix + typ;
       break;
   }
   return;
}

/*
*******************************
**     xdl_image_getpix2     **
*******************************

Purpose: Get and display image pixel position from magnifying window
         mouse x, y position. Set gd->ax1pix, gd->ax2pix

Author:  John W. Campbell, July 1990

*/

int xdl_image_getpix2 (gd, x, y, show)

XDl_image *gd;             /* Pointer to global data area */
int x;                     /* Mouse x position */
int y;                     /* Mouse y position */
int show;                  /* Show pixel position =1 yes, -0 no */

{
   int xm, ym;              /* Centre pixel position on magnifying window */
   int xpix, ypix;          /* Intermediate values in pixel position calcn. */
   int dx, dy, delta;       /* Offsets from centre pixel */
   int pix1;                /* Temp pixel position within 1 to gd->mag */
   float a;                 /* Used in pixel position calcn */
   char str[20];            /* String to hold position label */
   float cmpfac;            /* Current compression factor */

   if (gd->xc<0) return 1;
   cmpfac = (float) gd->cur_cmp /(float) gd->cur_expand;
   xm = (gd->magwsize-1)/2 - (gd->mag-1)/2;
   ym = xm;
   dx = x-xm;
   if (dx>=0)
   {
      delta = dx/(gd->mag);
      pix1 = dx%(gd->mag) + 1;
   }else{
      delta = (dx+1)/(gd->mag) - 1;
      pix1 = gd->mag + (dx+1)%(gd->mag);
   }
   a = ((float) (pix1-1)) * (cmpfac/(float)(gd->mag));
   xpix = (int) (cmpfac*(gd->xc+delta) + a) + 1; 
   dy = y-ym;
   if (dy>=0)
   {
      delta = dy/(gd->mag);
      pix1 = dy%(gd->mag) + 1;
   }else{
      delta = (dy+1)/(gd->mag) - 1;
      pix1 = gd->mag + (dy+1)%(gd->mag);
   }
   a = ((float) (pix1-1)) * (cmpfac/(float)(gd->mag));
   ypix = (int) (cmpfac*(gd->yc+delta) + a) + 1;
   if (xpix<1) return 1;
   if (xpix>cmpfac*(gd->img_width)) return 1;
   if (ypix<1) return 1;
   if (ypix>cmpfac*(gd->img_height)) return 1;
   xpix += gd->cur_xo_pix -1;
   ypix += gd->cur_yo_pix -1;
   xdl_image_xy_to_local (gd, xpix, ypix, &gd->ax1pix, &gd->ax2pix, 1);
   if (show)
   {
      sprintf (str, "%s=%d, %s=%d", gd->ax1_name, gd->ax1pix, 
               gd->ax2_name, gd->ax2pix);
      xdl_panel_label_set (gd->cursor_position, str, 0, 0);
   }
 
   return 0;
}
/*
****************************
**     xdl_image_cp1      **
****************************

Purpose: Callback for control panel contrast slider

Author:  John W. Campbell, July 1990

*/

void xdl_image_cp1 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_image *gd;          /* Pointer to global data area */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->contrast_val=(1023*value)/slider_len + 1;
   xdl_image_cp2 (iv,gd->color_opt);
   return;
}

/*
****************************
**     xdl_image_cp2      **
****************************

Purpose: Callback for control panel color choice menu

Author:  John W. Campbell, July 1990

*/

void xdl_image_cp2 (iv, item)

int iv;
int item;

{
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Temp/loop variable */
   XColor colr_cell;       /* Temp XColor structure */
   int fac;                /* Scaling factor for random numbers */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->color_opt=item;
   if (XDL_color>0)
   {
      switch(gd->color_opt)
      {
         case 1:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 2:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray (gd->cset, gd->contrast_val, 2);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray (gd->graypixels, gd->contrast_val, 2);
            }
            break;
         case 3:
            if (XDL_csets_alloc==1)
            {
               xdl_set64gray_yel (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64gray_yel (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 4:
            if (XDL_csets_alloc==1)
            {
               xdl_set64color (gd->cset, gd->contrast_val, 1);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64color (gd->graypixels, gd->contrast_val, 1);
            }
            break;
         case 5:
            if (XDL_csets_alloc==1)
            {
               xdl_set64color (gd->cset, gd->contrast_val, 2);
               for (i=0;i<64;++i)
               {
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               xdl_get64color (gd->graypixels, gd->contrast_val, 2);
            }
            break;
         case 6:  /* User defined color map */
            if (XDL_csets_alloc==1)
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
                  colr_cell.red = gd->user_rgb[0][i];
                  colr_cell.green = gd->user_rgb[1][i];
                  colr_cell.blue = gd->user_rgb[2][i];
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XStoreColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.red = gd->user_rgb[0][i];
                  colr_cell.green = gd->user_rgb[1][i];
                  colr_cell.blue = gd->user_rgb[2][i];
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XAllocColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = colr_cell.pixel;
               }
            }
            break;
         case 7:  /* Random Colour */
            fac = 2;
            if (XDL_csets_alloc==1)
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
                  colr_cell.red = fac * xdl_rand16();
                  colr_cell.green = fac * xdl_rand16();
                  colr_cell.blue = fac * xdl_rand16();
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XStoreColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = XDL_colorsets[gd->cset].pixels[i];
               }
            }
            else
            {
               for (i=0;i<64;++i)
               {
                  colr_cell.red = fac * xdl_rand16();
                  colr_cell.green = fac * xdl_rand16();
                  colr_cell.blue = fac * xdl_rand16();
                  colr_cell.flags = DoRed|DoGreen|DoBlue;
                  XAllocColor (XDL_display, XDL_colormap, &colr_cell);
                  gd->graypixels[i] = colr_cell.pixel;
               }
            }
            break;
      }
   }
   else
   {
      switch (gd->color_opt)
      {
         case 1:
           for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_whitepixel;
           for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_blackpixel;
           break;
         case 2:
           for (i=0;i<gd->thresh_val;++i) gd->graypixels[i] = XDL_blackpixel;
           for (i=gd->thresh_val;i<64;++i) gd->graypixels[i] = XDL_whitepixel;
           break;
      }
   }
   if (XDL_csets_alloc==0||XDL_color==0&&gd->reset==0)
   {
       xdl_image_setdata (gd);
       xdl_image_refresh (gd,1);
       return;
   }
   if (gd->show_int&&gd->cur_expand>=gd->show_int_expand&&gd->reset==0)
   {
       xdl_image_refresh (gd,0);
   }
   return;
}

/*
****************************
**     xdl_image_cp3      **
****************************

Purpose: Callback for control panel magnification factor menu

Author:  John W. Campbell, July 1990

*/

void xdl_image_cp3 (iv, item)

int iv;
int item;

{
   XDl_image *gd;          /* Pointer to global data area */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (XDL_color>0)
   {
      gd->mag = item + 1;
   }
   else
   {
      gd->mono_grscal = 0;
      gd->mag = item + 1;
      if (item>7)
      {
         gd->mono_grscal = 1;
         gd->mag = 3*(item-7);
      }
   }
   if (gd->xc<0) return;
   xdl_image_magw (gd);
   xdl_image_restore_freeze (gd);
   if (gd->magnify_on==0) xdl_image_set_freeze (gd);
   return;
}

/*
****************************
**     xdl_image_cp4      **
****************************

Purpose: Callback for control panel maximum threshold numeric text item

Author:  John W. Campbell, July 1990

*/

void xdl_image_cp4 (iv, str, len, maxv_len)

int iv;
char *str;
int len;
int maxv_len;

{
   XDl_image *gd;          /* Pointer to global data area */
   char valstr[12];        /* Parameter value string */
   int value;              /* Parameter value as an integer */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   sscanf (str, "%d", &value);
   if (value>gd->max_val) value = gd->max_val;
   if (value<=gd->min_thresh)
   {
      gd->max_thresh = gd->min_thresh + 1;
   }
   else
   {
      gd->max_thresh = value;
   }
   sprintf (valstr, "%ld", gd->max_thresh);
   xdl_panel_value_set (gd->max_text_item, valstr, 0);
   if (gd->reset==0)
   {
      xdl_image_setdata (gd);
      xdl_image_refresh (gd, 1);
   }
   return;
}

/*
****************************
**     xdl_image_cp5      **
****************************

Purpose: Callback for control panel minimum threshold numeric text item

Author:  John W. Campbell, July 1990

*/

void xdl_image_cp5 (iv, str, len, maxv_len)

int iv;
char * str;
int len;
int maxv_len;

{
   XDl_image *gd;          /* Pointer to global data area */
   char valstr[12];        /* Parameter value string */
   int value;              /* Parameter value as an integer */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   sscanf (str, "%d", &value);
   if (value<0) value = 0;
   if (value>=gd->max_thresh)
   {
      gd->min_thresh = gd->max_thresh - 1;
   }
   else
   {
      gd->min_thresh = value;
   }
   sprintf (valstr, "%ld", gd->min_thresh);
   xdl_panel_value_set (gd->min_text_item, valstr, 0);
   if (gd->reset==0)
   {
      xdl_image_setdata (gd);
      xdl_image_refresh (gd, 1);
   }
   return;
}

/*
****************************
**     xdl_image_cp6      **
****************************

Purpose: Callback for control panel overlay type menu

Author:  John W. Campbell, August 1990

*/

void xdl_image_cp6 (iv, item)

int iv;
int item;

{
   XDl_image *gd;          /* Pointer to global data area */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   switch (item)
   {
      case 1:
         if (gd->ovly_opt==1) return;
         gd->ovly_opt = 1;
         gd->ovly1 = 1;
         gd->ovly2 = 1;
         gd->ovly_offset = 0;
         break;
      case 2:
         if (gd->ovly_opt==2) return;
         gd->ovly_opt = 2;
         gd->ovly1 = 0;
         gd->ovly2 = 0;
         gd->ovly_offset = 0;
         break;
      case 3:
         if (gd->ovly_opt==3) return;
         gd->ovly_opt = 3;
         if (gd->ovly_typ==1)
         {
            gd->ovly1 = 1;
            gd->ovly2 = 1;
            gd->ovly_offset = 1;
         }
         else
         {
            gd->ovly1 = 1;
            gd->ovly2 = 0;
            gd->ovly_offset = 0;
         }
         break;
      case 4:
         if (gd->ovly_opt==4) return;
         gd->ovly_opt = 4;
         gd->ovly1 = 0;
         gd->ovly2 = 1;
         gd->ovly_offset = 0;
         break;
   }
   xdl_image_refresh (gd, 1);
   return;
}


/*
****************************
**     xdl_image_cp7      **
****************************

Purpose: Callback for background type menu

Author:  John W. Campbell, August 1991

*/

void xdl_image_cp7 (iv, item)

int iv;
int item;

{
   XDl_image *gd;          /* Pointer to global data area */
   int xroot, yroot;       /* Root coordinates for error notice */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   switch (item)
   {
      case 1:
         if (gd->bg_opt==0) return;
         gd->bg_opt = 0;
         break;
      case 2:
         if (gd->bg_opt==1) return;
         if (gd->bg_present==0)
         {
            xdl_view_object_rootxy (XDL_view_objects[iv]->vh,240,30,
                                    &xroot,&yroot);
            xdl_popup_notice (xroot,yroot,"**No background defined**",0,
                              0,-1,"Continue",0,0,-1,2,0);
            xdl_panel_choice_reset (gd->bg_choice, 1);
            gd->bg_opt = 0;
            return;
         }
         gd->bg_opt = 1;
         break;
      case 3:
         if (gd->bg_opt==2) return;
         if (gd->bg_present==0)
         {
            xdl_view_object_rootxy (XDL_view_objects[iv]->vh,240,30,
                                    &xroot,&yroot);
            xdl_popup_notice (xroot,yroot,"**No background defined**",0,
                              0,-1,"Continue",0,0,-1,2,0);
            xdl_panel_choice_reset (gd->bg_choice, 1);
            gd->bg_opt = 0;
            return;
         }
         gd->bg_opt = 2;
         break;
   }
   xdl_image_setdata (gd);
   xdl_image_refresh (gd, 1);
   return;
}
/*
****************************
**     xdl_image_cp8      **
****************************

Purpose: Callback for control panel monochrome threshold slider

Author:  John W. Campbell, Sept 1991

*/

void xdl_image_cp8 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_image *gd;          /* Pointer to global data area */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->thresh_val=(63*value)/slider_len;
   xdl_image_cp2 (iv,gd->color_opt);
   return;
}
/*
**************************
**     xdl_image_cp9    **
**************************

Purpose: Callback for Zoom button push

Author:  John W. Campbell, March 1994

*/

void xdl_image_cp9 (iv, ibutton)

int iv;
int ibutton;

{
   XDl_image *gd;          /* Pointer to global data area */
   int vh;                 /* View-object handle */
   int expanded;           /* Flag to ensure that at least some expansion
                              is done */
   float fac;              /* Required expansion factor based on selected
                              rectangle size */
   float fac1;             /* Height expansion factor */
   float expfac;           /* Expansion factor based on decreasing pixel 
                              compression factor and/or increasing pixel 
                              expansion factor */
   float cmpfac;           /* Composite compression/expansion factor */
   int x;                  /* x mid point of selected rectangle */
   int y;                  /* y mid point of selected rectangle */
   int xpix;               /* x pixel in full image (internal axes) of
                              centre of selected rectangle */
   int ypix;               /* y pixel in full image (internal axes) of
                              centre of selected rectangle */
   int i;                  /* Loop counter through symbols/vectors */
   int is;                 /* Loop counter through symbols */
   int y_start;            /* Y offset 0 to Y_SEARCH_MAX-1 */
   int y_start_prev;       /* Previous value of y_start */
   Window wid;             /* Window id */

   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   vh = XDL_view_objects[iv]->vh;
   if (gd->zoom_sel==0)
   {
       xdl_panel_button_set (gd->zoom_button, "Full", 4, 1, 1);
       gd->select_label = xdl_panel_label (vh, gd->wdisp, 10, 10, 
                          ">>> Select Zoom Area <<<",
                          25, 2, 1, 0, 0, 1);
       gd->zoom_sel = 1;
       return;
   }
   /* Restore full image */

   xdl_panel_label_wid (gd->select_label, &wid);
   xdl_delete_panel_item (vh, wid);
   xdl_panel_button_set (gd->zoom_button, "Zoom", 4, 1, 1);
   gd->zoom_sel = 0;
   if (gd->zoom_on==0) return;

   gd->zoom_on = 0;
   gd->cur_xo_pix = gd->xo_pix;
   gd->cur_yo_pix = gd->yo_pix;
   gd->cur_cmp = gd->ncmp;
   gd->cur_expand = 1;

   xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);
   xdl_image_restore_freeze (gd);
   gd->magnify_on = 1;
   xdl_image_setdata (gd);
   if (gd->num_symb>0)
   {
      for (i=0;i<gd->num_symb;++i) xdl_image_symbol_offsets (gd,i);
      /* Update y-search offsets array */
      for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i]=0;
      y_start_prev=0;
      for (is=0;is<gd->num_symb;++is)
      {
         y_start = (gd->symbols[is].y_disp_offset * 
                   (Y_SEARCH_MAX-1)) / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         if (y_start>y_start_prev)
         {
            if (y_start-1<Y_SEARCH_MAX)
            {
               for(i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i]=is;
            }
         }
         y_start_prev=y_start;
      }
   }
   if (gd->num_vect>0)
   {
      for (i=0;i<gd->num_vect;++i) xdl_image_vector_offsets (gd,i);
   }
   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i) xdl_image_text_offsets (gd,i);
   }
   xdl_image_refresh (gd, 1);
   xdl_panel_label_set (gd->cursor_position," ",0,0);
   return;
}
/*
*************************
**     xdl_image_rds   **
*************************

Purpose: Redraw symbols

Author:  John W. Campbell, December 1991

*/

void xdl_image_rds (gd, xs_min, xs_max, ys_min, ys_max, magw, xm, ym)
XDl_image *gd;         /* Pointer to the global data area (R)*/
int xs_min;            /* Limits of area to be refilled (max symbol width
                          already allowed for therefore search for symbol
                          centres within this area (R)*/
int xs_max;            /* " */
int ys_min;            /* " */
int ys_max;            /* " */
int magw;              /* =0 drawing in main window, =1 draw in magnifying 
                             window (R)*/
int xm;                /* x centre position in magnifying window (only
                            needed if magw==1) (R)*/
int ym;                /* y centre position in magnifying window (only
                            needed if magw==1) (R)*/

{
   XDl_image_symbol *smb;      /* Temp. pointer to symbol structure */
   int x_d, y_d;                    /* x, y pixel offsets of symbol in main
                                       film display area */
   int x_s, y_s;                    /* Symbol position x, y offsets in 
                                       magnifying window */
   int y_start;                     /* Y offset 0 to Y_SEARCH_MAX-1 for symbols
                                       search  start */
   int s_start;                     /* Start position in symbols list for search
                                       */
   int xs_off;                      /* Overlay symbol x offset */
   int ys_off;                      /* Overlay symbol y offset */
   int i;                           /* Temp/loop variable */
   int exp2;                        /* Current expand factor / 2 */
   float cmpfac;                    /* Composite compression/expansion factor*/
   float xp, yp;                    /* Pixel at centre of mag window */
 
   /* Find and draw symbols as needed */

   cmpfac = (float) gd->cur_cmp / (float) gd->cur_expand;
   xp = gd->cur_xo_pix + cmpfac*gd->xc;
   yp = gd->cur_yo_pix + cmpfac*gd->yc;
   y_start = ((ys_min)*(Y_SEARCH_MAX-1)) 
             / gd->img_height;
   if (y_start<0) y_start = 0;
   if (y_start >= Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
   s_start = gd->y_search[y_start];
   if (s_start<gd->num_symb)
   {
      for (i=s_start;i<gd->num_symb;++i)
      {
         smb = &(gd->symbols[i]);
         if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
         if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
         xs_off = 0;
         ys_off = 0;
         if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
         {
            xs_off = gd->x_ovly_offset;
            ys_off = gd->y_ovly_offset;
         }
         x_d = smb->x_disp_offset + xs_off;
         y_d = smb->y_disp_offset + ys_off;
         if (x_d<xs_min) continue;
         if (x_d>xs_max) continue;
         if (y_d<ys_min) continue;
         if (y_d>ys_max) break;
         if (magw)
         {
            x_s = (int) xm + (smb->x_pixel-xp+0.5)*gd->mag/cmpfac;
            y_s = (int) ym + (smb->y_pixel-yp+0.5)*gd->mag/cmpfac; 
            xdl_image_drawsymb (gd, gd->wmag, x_s, y_s,
                                     smb->symb, (smb->color_flags&0XF));
         }
         else
         {
            xdl_image_drawsymb (gd, gd->wdisp, x_d, y_d,
                                     smb->symb, (smb->color_flags&0XF));
         }
      }
   }
   return;
}
/*
*************************
**     xdl_image_rdv   **
*************************

Purpose: Redraw overlayed vectors

Author:  John W. Campbell, December 1991

*/

void xdl_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, magw, xm, ym)
XDl_image *gd;         /* Pointer to the global data area (R)*/
int xv_min;            /* Limits of area to be refilled wrt main display (R)*/
int xv_max;            /* " */
int yv_min;            /* " */
int yv_max;            /* " */
int magw;              /* =0 drawing in main window, =1 draw in magnifying 
                             window (R)*/
int xm;                /* x centre position in magnifying window (only
                            needed if magw==1) (R)*/
int ym;                /* y centre position in magnifying window (only
                            needed if magw==1) (R)*/

{
   XDl_image_vect *vec;             /* Temp. pointer to vectorstructure */
   int x1_d, y1_d, x2_d, y2_d;      /* x, y pixel offsets of vector in main
                                       film display area */
   int x1_s, y1_s, x2_s, y2_s;      /* Vector position x, y offsets in 
                                       magnifying window */
   int xs_off;                      /* Overlay vector x offset */
   int ys_off;                      /* Overlay vector y offset */
   int i;                           /* Temp/loop variable */
   float cmpfac;                    /* Composite compression/expansion factor*/
   float xp, yp;                    /* Pixel position at centre of mag 
                                       window */
   int pixsiz;                      /* Pixel display size on mag window */

   /* Find and draw vectors as needed */

   if (gd->num_vect==0) return;
   cmpfac = (float) gd->cur_cmp / (float) gd->cur_expand;
   xp = gd->cur_xo_pix + cmpfac*gd->xc;
   yp = gd->cur_yo_pix + cmpfac*gd->yc;

   pixsiz = (int) gd->mag/cmpfac;
   for (i=0;i<gd->num_vect;++i)
   {
      vec = &(gd->vects[i]);
      if (gd->ovly1==0&&(vec->color_flags&0X10)==0) continue;
      if (gd->ovly2==0&&(vec->color_flags&0X10)!=0) continue;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(vec->color_flags&0X20)==0)
      {
         xs_off = gd->x_ovly_offset;
         ys_off = gd->y_ovly_offset;
      }
      x1_d = vec->x1_disp_offset + xs_off;
      y1_d = vec->y1_disp_offset + ys_off;

      if (y1_d>yv_max) continue;
      x2_d = vec->x2_disp_offset + xs_off;
      y2_d = vec->y2_disp_offset + ys_off;

      if (y2_d<yv_min) continue;
      if (x1_d<=x2_d)
      {
         if (x1_d>xv_max) continue;
         if (x2_d<xv_min) continue;
      }
      else
      {
         if (x2_d>xv_max) continue;
         if (x1_d<xv_min) continue;
      }
      if (magw)
      {
         x1_s = (int) xm + (vec->x1_pixel-xp)*gd->mag/cmpfac; 
         y1_s = (int) ym + (vec->y1_pixel-yp)*gd->mag/cmpfac; 
         x2_s = (int) xm + (vec->x2_pixel-xp)*gd->mag/cmpfac; 
         y2_s = (int) ym + (vec->y2_pixel-yp)*gd->mag/cmpfac; 
         xdl_image_drawvec (gd, gd->wmag, pixsiz,
                            x1_s, y1_s, x2_s, y2_s,
                            vec->x1_offset,
                            vec->y1_offset,
                            vec->x2_offset,
                            vec->y2_offset,
                            (vec->color_flags&0XF));
      }
      else
      {
         xdl_image_drawvec (gd, gd->wdisp, gd->cur_expand,
                            x1_d, y1_d, x2_d, y2_d,
                            vec->x1_offset,
                            vec->y1_offset,
                            vec->x2_offset,
                            vec->y2_offset,
                           (vec->color_flags&0XF));
      }
   }
   return;
}
/*
*************************
**     xdl_image_rdt   **
*************************

Purpose: Redraw overlayed text strings

Author:  John W. Campbell, Aug 1993

*/

void xdl_image_rdt (gd, yt_min, yt_max)
XDl_image *gd;         /* Pointer to the global data area (R)*/
int yt_min;            /* Limits in y of area to be refilled wrt main display 
                          (allowing for character height (R)*/
int yt_max;            /* " */

{
   XDl_image_text *text;            /* Temp. pointer to text string structure */
   int x_d, y_d;                    /* x, y pixel offsets of textin main
                                       film display area */
   int xs_off;                      /* Overlay vector x offset */
   int ys_off;                      /* Overlay vector y offset */
   int i;                           /* Temp/loop variable */
   int color;                       /* Color for text */
   int ll;                          /* Length of character string */

   /* Find and draw text strings as needed */

   if (gd->num_text==0) return;
   for (i=0;i<gd->num_text;++i)
   {
      text = &(gd->text[i]);
      if (gd->ovly1==0&&(text->color_flags&0X10)==0) continue;
      if (gd->ovly2==0&&(text->color_flags&0X10)!=0) continue;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(text->color_flags&0X20)==0)
      {
         xs_off = gd->x_ovly_offset;
         ys_off = gd->y_ovly_offset;
      }
      x_d = text->x_disp_offset + xs_off;
      y_d = text->y_disp_offset + ys_off;
      if (y_d<yt_min||y_d>yt_max) continue;
      if (gd->exposed==1)
      {
         color = text->color_flags&0XF;
         ll = strlen(text->string);
         if (color==0)
         {
            XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
         }
         else
         {
            XSetForeground (XDL_display, gd->paint_gc, 
                            gd->ovly_pixels[color-1]);
         }
         if (text->bold)
         {
            XSetFont (XDL_display,gd->paint_gc,
                      XDL_bold_fonts[text->font-1]->fid);
         }
         else
         {
            XSetFont (XDL_display,gd->paint_gc,
                      XDL_fonts[text->font-1]->fid);
         }
         XDrawString (XDL_display,gd->wdisp, gd->paint_gc,
                      x_d, y_d, text->string, ll);
      }
   }
   return;
}

/*
*****************************
**     xdl_image_magw      **
*****************************

Purpose: Paint the magnifying window 
         (for a monochrome display a 10 level 3*3 halftone grayscale is
          simulated)

Author:  John W. Campbell, July 1990

*/

void xdl_image_magw (gd)

XDl_image *gd;           /* Pointer to the global data area */

{
   int x, y;              /* Selected pixel position from main image */
   int i, j, k, l, ix, iy, jx, jy, kx, ky, lx, ly, ii, jj, kk, ll, jjx, jjy;
                          /* Temp/loop variables */
   int ixx, iyy;          /* Loop variables */
   int max_offset;        /* Maximum possible offset from centre of the
                             magnifying window */
   int xm, ym;            /* Centre position in magnifying window */
   int xoff, yoff;        /* x and y offset loop counters */
   unsigned char idat;    /* Pixel from main image */
   unsigned char *byte_data; /* Pointer to byte image data */
   I2pix *i2_data;        /* Pointer to image plate 'i2' data */
   I2squash *i2_squash;   /* Pointer to 'squashed i2' data */
   Intpix *int_data;      /* Pointer to signed integer data */
   int nxel, nyel;        /* Number of x, y data points */
   long range;            /* Threshold range */
   int nfx, nfy;          /* No. x,y points in file */
   int ncx, ncy;          /* No. x,y pixels to combine into 1 */
   int isx, isy;          /* Pixel position in read in image (1 to nxp or 
                             1 to nyp corresponding to a magnifying window 
                             pixel position 1 to gd->mag */
   int ns, nf;            /* Start and finish of overlapping pixels */
   long xoff_start;       /* Offset (x) into image data */
   long yoff_start;       /* Offset (y) into image data */
   int ioff;              /* Offset into data or magw_data array */
   int ioff_bg;           /* Offset into backgound or magw_bgdata array */
   int joff;              /* Offset into image data array */
   int joff_bg;           /* Offset into image background array */
   int axf_off;           /* Fast axis offset into original image */
   int axs_off;           /* Slow axis offset into original image */
   int xtyp, ytyp;        /* Flag xtyp = 1, 2, 3 if nxp=mag, nxp>mag, nxp<mag
                             and cf for ytyp */
   float x_fr, y_fr, fr1, fr2, rs, rf; /* Used in calculating pixels required
                                          for non-simple cases */
   double rval;           /* Total intensity of pixels to be combined into 1 */
   long ival;             /* Pixel intensity to be o/p */
   unsigned long *pix;    /* Pointer to pixels array */
   long pixval;           /* Pixel value */
   unsigned char * img;   /* Temp pointer to stored image data */
   unsigned char * byte_bgdata; /* Pointer to byte background image data */
   I2pix *i2_bgdata;      /* Pointer to i2 background data */
   I2squash *i2_bgsquash; /* Pointer to 'squashed i2' data */
   Intpix *int_bgdata;    /* Pointer to integer background data */
   int xs_min, xs_max;    /* x limits (in main film display area)
                             for symbols which may appear in the
                             magnifying window */
   int ys_min, ys_max;    /* y limits (in main film display area)
                             for symbols which may appear in the
                             magnifying window */
   int xv_min, xv_max;    /* x limits (in main film display area)
                             for vectorswhich may appear in the
                             magnifying window */
   int yv_min, yv_max;    /* y limits (in main film display area)
                             for vectors which may appear in the
                             magnifying window */
   unsigned long pixel;   /* Pixel number */
   int kp, kp_save;       /* Indices into gd->mag_data array */
   int jp, jp_save;       /* Indices into gd->monodata array */
   int nmono;             /* No. of elements in a row of data
                             in gd->monodata array */
   long ipix_val;          /* Pixel value */
   int xpix;               /* x pixel value */
   int ypix;               /* y pixel value */
   int fpix;               /* fast axis pixel value */
   int spix;               /* slow axis pixel value */
   static unsigned char grscal[] = 
     { 0,0,0,0,0,0,0,0,0,
       0,0,0,0,1,0,0,0,0,
       0,0,0,1,1,0,0,0,0,
       0,0,0,1,1,0,0,1,0,
       0,0,0,1,1,1,0,1,0,
       0,0,1,1,1,1,0,1,0,
       0,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,1,
       1,1,1,1,1,1,1,1,1 };          /* Grayscale emulations 3*3 half-tone */
   unsigned char *grayp;             /* Pointer to grayscale half-tone pixels
                                        for current pixel value */
   int xr;                           /* No. of x rasters - 1 */
   int yr;                           /* No. of y rasters - 1 */

   if (gd->magw_exposed==0) return;
   max_offset = (gd->magwsize+gd->mag-1)/(2*gd->mag);
   xm = (gd->magwsize-1)/2 - (gd->mag-1)/2;
   ym = xm;
   x = gd->xc;
   y = gd->yc;
   
   if (gd->use_image==0||gd->cur_cmp==1||gd->cur_expand>1)
   {
      if (XDL_color>0||gd->mono_grscal==0) 
      {
      /* Expand data from stored image (Colour/grayscale display or
         monochrome not using simulated grayscale) */

         img = (unsigned char *) gd->img_image->data;
         if (XDL_pw_depth==8)
         {
            for (yoff= -max_offset;yoff<=max_offset;++yoff)
            {
               iy = y + yoff;
               ii = ym + gd->mag*yoff;
               for (xoff= -max_offset;xoff<=max_offset;++xoff)
               {
                  ix = x + xoff;
                  jj = xm + gd->mag*xoff;
                  if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
                  {
                     idat = XDL_whitepixel;
                  }
                  else
                  {
                     idat = img[iy*(gd->img_width)+ix];
                  }
                  kp_save = ii * gd->magwsize + jj;
                  for(k=0;k<gd->mag;++k)
                  {
                     kp = kp_save;
                     if ((ii+k)<0||(ii+k)>=gd->magwsize) 
                                           {kp_save+=gd->magwsize; continue;}
                     for(l=0;l<gd->mag;++l)
                     {
                        if ((jj+l)<0||(jj+l)>=gd->magwsize) {kp++; continue;}
                        gd->mag_data[kp]=idat;
                        kp++;
                     }
                     kp_save += gd->magwsize;
                  }
               }
            }
         }
         else
         {
            for (yoff= -max_offset;yoff<=max_offset;++yoff)
            {
               iy = y + yoff;
               ii = ym + gd->mag*yoff;
               for (xoff= -max_offset;xoff<=max_offset;++xoff)
               {
                  ix = x + xoff;
                  jj = xm + gd->mag*xoff;
                  if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
                  {
                     pixel = XDL_whitepixel;
                  }
                  else
                  {
                     pixel = XGetPixel(gd->img_image,ix,iy);
                  }
                  for(k=0;k<gd->mag;++k)
                  {
                     if ((ii+k)<0||(ii+k)>=gd->magwsize) continue;
                     for(l=0;l<gd->mag;++l)
                     {
                        if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                        XPutPixel(gd->mag_image,jj+l,ii+k,pixel);
                     }
                  }
               }
            }
         }
      }
      else
      {
         /* Expand data from stored XImage data for monochrome
            display using 3x3 halftone 10 level gray scale */

         range = gd->max_thresh - gd->min_thresh;
         int_data = (Intpix *) gd->imgdata;
         i2_data = (I2pix *) gd->imgdata;
         i2_squash = (I2squash *) gd->imgdata;
         byte_data = (unsigned char *) gd->imgdata;
         int_bgdata = (Intpix *) gd->bg_data;
         i2_bgdata = (I2pix *) gd->bg_data;
         i2_bgsquash = (I2squash *) gd->bg_data;
         byte_bgdata = (unsigned char *) gd->bg_data;

         for (yoff= -max_offset,jjy=0;yoff<=max_offset;++yoff,++jjy)
         {
            iy = y + yoff;
            ii = ym + gd->mag*yoff;
            for (xoff= -max_offset,jjx=0;xoff<=max_offset;++xoff,++jjx)
            {
               ix = x + xoff;
               jj = xm + gd->mag*xoff;
               if ((ix<0)||(ix>=gd->img_width)||(iy<0)||(iy>=gd->img_height))
               {
                  ipix_val = 0;
               }
               else
               {
                  /* Note: Assumes that both gd->cur_cmp and gd->cur_expand are not > 1 */
                  if (gd->cur_cmp>1)
                  {
                     xpix = ix*gd->cur_cmp + gd->cur_cmp/2;
                     ypix = iy*gd->cur_cmp + gd->cur_cmp/2;
                  }
                  else
                  {
                     xpix = ix/gd->cur_expand;
                     ypix = iy/gd->cur_expand;
                  }
                  xpix += gd->cur_xo_pix;
                  ypix += gd->cur_yo_pix;
                  xdl_image_xy_to_fs (gd, xpix, ypix, &fpix, &spix, 1);
                  ioff = (spix-1)*gd->nf_off + fpix - 1;
                  ioff_bg = ((spix-1)/gd->bg_cmp)*gd->nf_bgoff + 
                            (fpix-1)/gd->bg_cmp;
                  if (gd->type==1)
                  {
                     ival = byte_data[ioff];
                  }
                  else if (gd->type==2)
                  {
                     ival = i2_data[ioff];
                  }
                  else if (gd->type==3)
                  {
                     ival = int_data[ioff];
                  }
                  else if (gd->type==4)
                  {
                     ival = gd->unsq(i2_squash[ioff]);
                  }
                  if (gd->bg_opt==0)
                  {
                     ipix_val = ival;
                  }
                  else
                  {
                     if (gd->bg_present==1)
                     {
                        switch (gd->bg_opt)
                        {
                           case 1:
                              ipix_val = ival;
                              if (gd->type==1)
                              {
                                 ipix_val -= byte_bgdata[ioff_bg];
                              }
                              else if (gd->type==2)
                              {
                                 ipix_val -= i2_bgdata[ioff_bg];
                              }
                              else if (gd->type==3)
                              {
                                 ipix_val -= int_bgdata[ioff_bg];
                              }
                              else if (gd->type==4)
                              {
                                 ipix_val -= gd->unsq(i2_bgsquash[ioff_bg]);
                              }
                              if (ipix_val<0) ipix_val = 0;
                              break;
                           case 2:
                              if (gd->type==1)
                              {
                                 ipix_val = byte_bgdata[ioff_bg];
                              }
                              else if (gd->type==2)
                              {
                                 ipix_val = i2_bgdata[ioff_bg];
                              }
                              else if (gd->type==3)
                              {
                                 ipix_val = int_bgdata[ioff_bg];
                              }
                              else if (gd->type==4)
                              {
                                 ipix_val = gd->unsq(i2_bgsquash[ioff_bg]);
                              }
                              break;
                        }
                     }
                     else
                     {
                        ipix_val = ival;
                     }
                  }
               }
               if (ipix_val<=gd->min_thresh)
               {
                  pixval = 0;
               }
               else if (ipix_val>=gd->max_thresh)
               {
                  pixval = 9;
               }
               else
               {
                  pixval = 9*(ipix_val-gd->min_thresh)/range;
               }
               grayp = &grscal[9*pixval];
               for (k=0;k<gd->mag;++k)
               {
                  if ((ii+k)<0||(ii+k)>=gd->magwsize) continue;
                  kk = k%3;
                  for(l=0;l<gd->mag;++l)
                  {
                     if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                     ll = l%3;
                     if (grayp[3*kk+ll]==0)
                     {
                        XPutPixel (gd->mag_image, jj+l, ii+k, XDL_whitepixel);
                     }
                     else
                     {
                        XPutPixel (gd->mag_image, jj+l, ii+k, XDL_blackpixel);
                     }
                  }
               }
            }
         }
      }
   }

   else

   /* Get image data from full image */  /*Note: assumes expand == 1*/

   {
      xdl_panel_label_set (gd->cursor_position," W O R K I N G ",0,1);
      range = gd->max_thresh - gd->min_thresh;
      int_data = (Intpix *) gd->imgdata;
      i2_data = (I2pix *) gd->imgdata;
      i2_squash = (I2squash *) gd->imgdata;
      byte_data = (unsigned char *) gd->imgdata;
      int_bgdata = (Intpix *) gd->bg_data;
      i2_bgdata = (I2pix *) gd->bg_data;
      i2_bgsquash = (I2squash *) gd->bg_data;
      byte_bgdata = (unsigned char *) gd->bg_data;

      nfx = gd->nxrasts;
      nfy = gd->nyrasts;
      nxel = (2*max_offset + 1) * gd->cur_cmp;
      nyel = (2*max_offset + 1) * gd->cur_cmp;
      /* Get x, y offsets for top left pixel of section in full image */
      xoff_start = (gd->xc - max_offset) * (gd->cur_cmp) + gd->cur_xo_pix - 1;
      yoff_start = (gd->yc - max_offset) * (gd->cur_cmp) + gd->cur_yo_pix - 1;
      xoff = xoff_start;
      ioff = 0;
      for (ix=0;ix<nxel;++ix)
      {
         if (xoff<0||xoff>=nfx)
         {
            for (i=0;i<nyel;++i)
            {
               gd->magw_data[ioff] = 0;
               gd->magw_bgdata[ioff] = 0;
               ioff ++;
            }
         }
         else
         {
            yoff = yoff_start;
            for (iy=0;iy<nyel;++iy)
            {
               if (yoff<0||yoff>=nfy)
               {
                  gd->magw_data[ioff] = 0;
                  gd->magw_bgdata[ioff] = 0;
               }
               else
               {
                  xdl_image_xy_to_fs (gd, xoff, yoff, &axf_off, &axs_off, -1);
                  joff = axs_off*gd->nf_off + axf_off;
                  joff_bg = (axs_off/gd->bg_cmp)*gd->nf_bgoff
                            + axf_off/gd->bg_cmp;
                  switch (gd->type)
                  {
                      case 1:
                         gd->magw_data[ioff] = byte_data[joff];
                         if (gd->bg_opt>0) gd->magw_bgdata[ioff] 
                                           = byte_bgdata[joff_bg];
                      break;

                      case 2:
                         gd->magw_data[ioff] = i2_data[joff];
                         if (gd->bg_opt>0) gd->magw_bgdata[ioff] 
                                           = i2_bgdata[joff_bg];
                      break;

                      case 3:
                         gd->magw_data[ioff] = int_data[joff];
                         if (gd->bg_opt>0) gd->magw_bgdata[ioff] 
                                           = int_bgdata[joff_bg];
                      break;

                      case 4:
                         gd->magw_data[ioff] = gd->unsq(i2_squash[joff]);
                         if (gd->bg_opt>0) gd->magw_bgdata[ioff] 
                                           = gd->unsq(i2_bgsquash[joff_bg]);
                      break;
                  }
               }
               ioff++;
               yoff++; 
            }
         }
         xoff++;
      }

      /* Get case types for x (nxp=mag, nxp>mag, nxp<mag) */
      xtyp = 1;
      ncx = 1;
      x_fr = (float) (gd->cur_cmp) / (float) (gd->mag);
      if (gd->cur_cmp>gd->mag)
      {
         xtyp = 2;
         ncx = gd->cur_cmp/gd->mag;
      }
      if (gd->cur_cmp<gd->mag)
      {
         xtyp = 3;
         ncx = 1;
      }
      /* Get case types for y (nyp=mag, nyp>mag, nyp<mag) */
      ytyp = 1;
      ncy = 1;
      y_fr = (float) (gd->cur_cmp) / (float) (gd->mag);
      if (gd->cur_cmp>gd->mag)
      {
         ytyp = 2;
         ncy = gd->cur_cmp/gd->mag;
      }
      if (gd->cur_cmp<gd->mag)
      {
         ytyp = 3;
         ncy = 1;
      }

      /* Form image data */
      pix = gd->graypixels;
      for (ix=0;ix<gd->mag;++ix)
      {
         switch (xtyp)
         {
            case 1:
               isx = ix;
               break;
            case 2:
               rs = (float) ix*x_fr;
               rf = (float) (ix+1)*x_fr;
               ns = (int) rs;
               nf = (int) rf;
               if ((nf-ns)>ncx)
               {
                  isx = ns + 1;
               }
               else
               {
                  fr1 = (float) (ns+1) - rs;
                  fr2 = rf - (float) (nf);
                  isx = ns;
                  if (fr2>fr1) isx++;
               }
               break;
            case 3:
               isx = (int) ((float)ix + 0.5) * x_fr;
         }
         for (iy=0;iy<gd->mag;++iy)
         {
            switch (ytyp)
            {
               case 1:
                  isy = iy;
                  break;
               case 2:
                  rs = (float) iy*y_fr;
                  rf = (float) (iy+1)*y_fr;
                  ns = (int) rs;
                  nf = (int) rf;
                  if ((nf-ns)>ncy)
                  {
                     isy = ns + 1;
                  }
                  else
                  {
                     fr1 = (float) (ns+1) - rs;
                     fr2 = rf - (float) (nf);
                     isy = ns;
                     if (fr2>fr1) isy++;
                  }
                  break;
               case 3:
                  isy = (int) ((float)iy + 0.5) * y_fr;
            }

            if (XDL_color>0||gd->mono_grscal==0)
            {

               /* Colour/grayscale display options or monochrome not using
                  simulated grayscale */
               for (xoff= -max_offset,kx=0;xoff<=max_offset;
                    ++xoff,kx+=gd->cur_cmp)
               {
                  for (yoff= -max_offset,ky=0;yoff<=max_offset;
                       ++yoff,ky+=gd->cur_cmp)
                  {
                     ioff = (kx+isx)*nyel + ky + isy;
                     ioff_bg = ioff;
                     rval = 0.0;
                     for (ixx=0;ixx<ncx;++ixx)
                     {
                        for (iyy=0;iyy<ncy;++iyy)
                        {
                           rval += (float) gd->magw_data[ioff+iyy];
                        }
                        ioff += nyel;
                     }
                     ival = (long) rval/(float)(ncx*ncy);
                     jy = ym + gd->mag*yoff + iy;
                     jx = xm + gd->mag*xoff + ix;
                     if (jy<0||jy>=gd->magwsize||jx<0||jx>=gd->magwsize) 
                         continue;
                     if (gd->bg_opt==0)
                     {
                        ipix_val = ival;
                     }
                     else
                     {
                        if (gd->bg_present==1)
                        {
                           switch (gd->bg_opt)
                           {
                              case 1:
                                 ipix_val = ival;
                                 ipix_val -= gd->magw_bgdata[ioff_bg];
                                 break;
                              case 2:
                                 ipix_val =  gd->magw_bgdata[ioff_bg];
                                 break;
                           }
                        }
                        else
                        {
                           ipix_val = ival;
                        }
                     }
                     if (ipix_val<=gd->min_thresh)
                     {
                        pixval = 0;
                     }
                     else if (ipix_val>=gd->max_thresh)
                     {
                        pixval = 63;
                     }
                     else
                     {
                        pixval = 63*(ipix_val-gd->min_thresh)/range;
                     }
                     if (XDL_pw_depth==8)
                     {
                        gd->mag_data[jy*gd->magwsize+jx] = pix[pixval];
                     }
                     else
                     {
                        XPutPixel (gd->mag_image, jx, jy, pix[pixval]);
                     }
                  }
               }
            }
            else
            {

               /* Monochrome display option with 3x3 half-tone simulation of 
                  a 10 level gray scale */

               nmono = (2*max_offset + 1) * gd->mag;
               for (xoff= -max_offset,kx=0,lx=0;xoff<=max_offset;
                    ++xoff,kx+=gd->cur_cmp,lx+=gd->mag)
               {
                  for (yoff= -max_offset,ky=0,ly=0;yoff<=max_offset;
                       ++yoff,ky+=gd->cur_cmp,ly+=gd->mag)
                  {
                     ioff = (kx+isx)*nyel + ky + isy;
                     ioff_bg = ioff;
                     rval = 0.0;
                     for (ixx=0;ixx<ncx;++ixx)
                     {
                        for (iyy=0;iyy<ncy;++iyy)
                        {
                           rval += (float) gd->magw_data[ioff+iyy];
                        }
                        ioff += nyel;
                     }
                     ival = (long) rval/(float)(ncx*ncy);
                     jjx = lx + ix;
                     jjy = ly + iy;
                     if (gd->bg_opt==0)
                     {
                        ipix_val = ival;
                     }
                     else
                     {
                        if (gd->bg_present==1)
                        {
                           switch (gd->bg_opt)
                           {
                              case 1:
                                 ipix_val = ival;
                                 ipix_val -= gd->magw_bgdata[ioff_bg];
                              break;

                              case 2:
                                 ipix_val =  gd->magw_bgdata[ioff_bg];
                              break;
                           }
                        }
                        else
                        {
                           ipix_val = ival;
                        }
                     }
                     gd->monodata[jjy*nmono+jjx] = ipix_val;
                  }
               }

               /* Find average values over 3*3 pixels and set grayscale halftone
                  bits in magnifying window image data */
 
               for (jy=0;jy<nmono;jy+=3)
               {
                  ii = jy + ym - max_offset*gd->mag;
                  for (jx=0;jx<nmono;jx+=3)
                  {
                     jj = jx + xm - max_offset*gd->mag;
                     jp_save = jy*nmono + jx;
                     ipix_val = 0;
                     for (k=0;k<3;++k)
                     {
                        jp = jp_save;
                        for (l=0;l<3;++l)
                        {
                           ipix_val += gd->monodata[jp];
                           jp++;
                        }
                        jp_save += nmono;
                     }
                     ipix_val /= 9;
                     if (ipix_val<=gd->min_thresh)
                     {
                        pixval = 0;
                     }
                     else if (ipix_val>=gd->max_thresh)
                     {
                        pixval = 9;
                     }
                     else
                     {
                        pixval = 9*(ipix_val-gd->min_thresh)/range;
                     }
                     grayp = &grscal[9*pixval];
       
                     for (k=0;k<3;++k)
                     {
                        if ((ii+k<0)||(ii+k)>=gd->magwsize) continue;
                        for (l=0;l<3;++l)
                        {
                           if ((jj+l)<0||(jj+l)>=gd->magwsize) continue;
                           if (grayp[3*k+l]==0)
                           {
                              XPutPixel (gd->mag_image, jj+l, ii+k, 
                                         XDL_whitepixel);
                           }
                           else
                           {
                              XPutPixel (gd->mag_image, jj+l, ii+k, 
                                         XDL_blackpixel);
                           }
                        }
                     }
                  }
               }
            }
         }
      }
      xdl_panel_label_set (gd->cursor_position," ",0,0);
   }

   /* Display the magnifying window image data */

   XPutImage (XDL_display, gd->wmag, gd->paint_gc,
              gd->mag_image, 0, 0, 0, 0, gd->magwsize, gd->magwsize);

   /* Draw overlay symbols and vectors if required */

   if (gd->ovly1==0&&gd->ovly2==0) return;
   xs_min = gd->xc - max_offset - MAX_SYMB_SIZE;
   xs_max = gd->xc + max_offset + MAX_SYMB_SIZE;
   ys_min = gd->yc - max_offset - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
   ys_max = gd->yc + max_offset + MAX_SYMB_SIZE + abs(gd->y_ovly_offset);
   xdl_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 1, xm, ym);
   xv_min = gd->xc - max_offset - gd->cur_expand - 1;
   xv_max = gd->xc + max_offset + gd->cur_expand + 1;
   yv_min = gd->yc - max_offset - abs(gd->y_ovly_offset) - gd->cur_expand - 1;
   yv_max = gd->yc + max_offset + abs(gd->y_ovly_offset) + gd->cur_expand + 1;
   xdl_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, 1, xm, ym);
   
   return;
}

/*
********************************
**     xdl_image_setdata      **
********************************

Purpose: Set up pixels array from input data

Author:  John W. Campbell, July 1990

*/

void xdl_image_setdata (gd)

XDl_image *gd;            /* Pointer top global data area */

{
   int cmp;                 /* Current compression factor */
   int expand;              /* Current expand factor */
   int img_width;           /* Displayed image width */
   int img_height;          /* Displayed image height */
   int nn;                  /* Total no. of pixels compressed into one pixel */
   int cmp_type;            /* Compress type =1 average, =0 sample */
   long range;              /* Threshold range */
   long maxint;             /* Max pixel value allowed for compression by 
                               averaging (to avoid overflows) */
   int nx_disp;             /* No. of compressed pixels to be accessed
                               from image along 'x' direction */
   int ny_disp;             /* No. of compressed pixels to be accessed
                               from image along 'y' direction */
   int ns_disp;             /* No. of compressed pixels to be accessed
                               from image along slow direction */
   int nf_disp;             /* No. of compressed pixels to be accessed
                               from image along fast direction */
   int xo_off;              /* 'x' offset within internal image of
                               curently displayed origin pixel */
   int yo_off;              /* 'y' offset within internal image of
                               curently displayed origin pixel */
   int o_s_off;             /* Start offset along image slow axis for accessing
                               image data */
   int o_f_off;             /* Start offset along image fast axis for accessing
                               image data */
   int i_start;             /* Start pixel offset along slow axis (allowing
                               for offset (cmp/2) if sampling */
   int j_start;             /* Start pixel offset along fast axis (allowing
                               for offset (cmp/2) if sampling */
   int i;                   /* Loop counter - slow axis */
   int j;                   /* Loop counter - fast axis */
   int ioff;                /* Offset (for central of compressed pixel)
                               along slow axis */
   int joff;                /* Offset (for central of compressed pixel)
                               along fast axis */
   int i_bg_off;            /* Offset into background image for slow axis */
   unsigned long *pix;      /* Pointer to current array of colour pixels */
   int pixval;              /* Pixel value */
   int ii;                  /* Loop counter over compressed pixel or
                               expand  elements */
   int jj;                  /* Loop counter over compressed pixel or
                               expand elements */
   int k;                   /* Offset into image data */
   int xoff;                /* 'x' offset for pixel wrt. X-windows axes */
   int yoff;                /* 'y' offset for pixel wrt. X-windows axes */
   int bg_off;              /* Offset into background image array */
   long intens;             /* Pixel intensity (averaged/sampled) allowing
                               for background option */
   int ip;                  /* Offset into image data (for XImage) */
   int ipstart;             /* 'ip' value at start of row */
   int nxpix;               /* No. of x pixels to display * expand factor
                               - may go beyond image width */
   int nypix;               /* No. of y pixels to display * expand factor 
                               - may go beyond image height*/
   Intpix *int_data;        /* Pointer to image data if of type integer */
   I2pix *i2_data;          /* Pointer to image data if of type (i2) */
   I2squash *i2_squash;     /* Pointer to image data for 'squashed i2' data */
   unsigned char *byte_data;/* Pointer to byte image data */
   Intpix *int_bgdata;      /* Pointer to background data if of type integer */
   I2pix *i2_bgdata;        /* Pointer to background data if of type (i2) */
   I2squash *i2_bgsquash;   /* Pointer to background 'squashed i2' data */
   unsigned char *byte_bgdata; /*Pointer to byte image background data */
   long *int_arr;           /* Pointer to array for compressing data */
   int cmph;                /* Half of compression factor */
   static unsigned char grscal[] = 
     { 0,0,0,0,0,0,0,0,0,
       0,0,0,0,1,0,0,0,0,
       0,0,0,1,1,0,0,0,0,
       0,0,0,1,1,0,0,1,0,
       0,0,0,1,1,1,0,1,0,
       0,0,1,1,1,1,0,1,0,
       0,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,0,
       1,0,1,1,1,1,1,1,1,
       1,1,1,1,1,1,1,1,1 };          /* Grayscale emulations 3*3 half-tone */
   unsigned char *grayp;             /* Pointer to grayscale half-tone pixels
                                        for current pixel value */
   int iii;                          /* Counter for grayscale simulation */
   int jjj;                          /* Counter for grayscale simulation */

   /* Initialisations */

   cmp = gd->cur_cmp;
   cmph = cmp/2;
   expand = gd->cur_expand;
   img_width = gd->img_width;
   img_height = gd->img_height;
   nn = cmp*cmp;
   cmp_type = gd->cmp_type;
   range = gd->max_thresh - gd->min_thresh;
   if (cmp_type==1)
   {
      maxint = 2147483647/nn; 
      if (gd->max_val>maxint) cmp_type = 0;
   }
   nx_disp = (img_width-1)/expand + 1;
   ny_disp = (img_height-1)/expand + 1;
   nxpix = nx_disp*expand;
   nypix = ny_disp*expand;
   if (gd->korder<=4)
   {
      nf_disp = nx_disp;
      ns_disp = ny_disp;
   }
   else
   {
      nf_disp = ny_disp;
      ns_disp = nx_disp;
   }
   int_data = (Intpix *) gd->imgdata;
   i2_data = (I2pix *) gd->imgdata;
   i2_squash = (I2squash *) gd->imgdata;
   byte_data = (unsigned char *) gd->imgdata;
   int_bgdata = (Intpix *) gd->bg_data;
   i2_bgdata = (I2pix *) gd->bg_data;
   i2_bgsquash = (I2squash *) gd->bg_data;
   byte_bgdata = (unsigned char *) gd->bg_data;
   int_arr = gd->int_arr;

   /* Calculate start offsets for accessing image data */

   /* Note: May be sligthly beyond displayed image area because of
            allowing for compression etc. */

   xo_off = gd->cur_xo_pix - 1;
   yo_off = gd->cur_yo_pix - 1;

   switch (gd->korder)
   {
       case 1:
          o_f_off = xo_off;
          o_s_off = yo_off;
       break;

       case 2:
          o_f_off = xo_off;
          o_s_off = gd->nyrasts - yo_off - ny_disp*cmp;
       break;

       case 3:
          o_f_off = gd->nxrasts - xo_off - nx_disp*cmp;
          o_s_off = yo_off;
       break;

       case 4:
          o_f_off = gd->nxrasts - xo_off - nx_disp*cmp;
          o_s_off = gd->nyrasts - yo_off - ny_disp*cmp;
       break;

       case 5:
          o_f_off = yo_off;
          o_s_off = xo_off;
       break;

       case 6:
          o_f_off = gd->nyrasts - yo_off - ny_disp*cmp;
          o_s_off = xo_off;
       break;

       case 7:
          o_f_off = yo_off;
          o_s_off = gd->nxrasts - xo_off - nx_disp*cmp;
       break;

       case 8:
          o_f_off = gd->nyrasts - yo_off - ny_disp*cmp;
          o_s_off = gd->nxrasts - xo_off - nx_disp*cmp;
       break;
   }

   i_start = o_s_off;
   j_start = o_f_off;

   pix = gd->graypixels;

   for (i=0,ioff=i_start;i<ns_disp;++i,ioff+=cmp)
   {
      i_bg_off = (ioff+cmph)/gd->bg_cmp;

      /*Compress by averaging*/

      if (cmp_type==1&&cmp>1)
      {
         switch (gd->type) /*Repeat some code to avoid switch in inner loop */
         {
             case 1:
               for (j=0;j<nf_disp;++j) int_arr[j] = 0;
               for (ii=0;ii<cmp;++ii)
               {
                  k = (ioff+ii)*gd->nf_off + j_start;
                  for (j=0;j<nf_disp;++j)
                  {
                     for (jj=0;jj<cmp;++jj,++k)
                     {
                        int_arr[j] += byte_data[k];
                     }
                  }
               }
               break;

             case 2:
               for (j=0;j<nf_disp;++j) int_arr[j] = 0;
               for (ii=0;ii<cmp;++ii)
               {
                  k = (ioff+ii)*gd->nf_off + j_start;
                  for (j=0;j<nf_disp;++j)
                  {
                     for (jj=0;jj<cmp;++jj,++k)
                     {
                        int_arr[j] += i2_data[k];
                     }
                  }
               }
               break;

             case 3:
               for (j=0;j<nf_disp;++j) int_arr[j] = 0;
               for (ii=0;ii<cmp;++ii)
               {
                  k = (ioff+ii)*gd->nf_off + j_start;
                  for (j=0;j<nf_disp;++j)
                  {
                     for (jj=0;jj<cmp;++jj,++k)
                     {
                        int_arr[j] += int_data[k];
                     }
                  }
               }
               break;

             case 4:
               for (j=0;j<nf_disp;++j) int_arr[j] = 0;
               for (ii=0;ii<cmp;++ii)
               {
                  k = (ioff+ii)*gd->nf_off + j_start;
                  for (j=0;j<nf_disp;++j)
                  {
                     for (jj=0;jj<cmp;++jj,++k)
                     {
                        int_arr[j] += gd->unsq(i2_squash[k]);
                     }
                  }
               }
               break;
         }
         for (j=0;j<nf_disp;++j) int_arr[j] = int_arr[j]/nn;
      }

      /* No compression needed or just sample */
   
      else
      {
         k = (ioff+cmph)*gd->nf_off + j_start + cmph;
         switch (gd->type)
         {
            case 1:
               for (j=0;j<nf_disp;++j,k+=cmp) int_arr[j] = byte_data[k];
               break;

            case 2:
               for (j=0;j<nf_disp;++j,k+=cmp) int_arr[j] = i2_data[k];
               break;

            case 3:
               for (j=0;j<nf_disp;++j,k+=cmp) int_arr[j] = int_data[k];
               break;

            case 4:
               for (j=0;j<nf_disp;++j,k+=cmp) int_arr[j] = gd->unsq(i2_squash[k]);
               break;
         }
      }
      for (j=0,joff=j_start;j<nf_disp;++j,joff+=cmp)
      {
         switch (gd->korder)
         {
            /* If expanded then xoff,yoff are for top left corner */
            case 1:
               xoff = j*expand;
               yoff = i*expand;
            break;

            case 2:
               xoff = j*expand;
               yoff = nypix - i*expand - expand;
            break;

            case 3:
               xoff = nxpix - j*expand - expand;
               yoff = i*expand;
            break;

            case 4:
               xoff = nxpix - j*expand - expand;
               yoff = nypix - i*expand - expand;
            break;

            case 5:
               xoff = i*expand;
               yoff = j*expand;
            break;

            case 6:
               xoff = i*expand;
               yoff = nypix - j*expand - expand;
            break;

            case 7:
               xoff = nxpix - i*expand - expand;
               yoff = j*expand;
            break;

            case 8:
               xoff = nxpix - i*expand - expand;
               yoff = nypix - j*expand - expand;
            break;
         }

         if (gd->bg_opt==0||gd->bg_present==0)
         {
            intens = int_arr[j];
         }
         else
         {
            bg_off = i_bg_off*gd->nf_bgoff + (joff+cmph)/gd->bg_cmp;
            switch (gd->type)
            {
               case 1:
                  switch (gd->bg_opt)
                  {
                     case 1:
                        intens = int_arr[j] - byte_bgdata[bg_off];
                        break;
                     case 2:
                        intens = byte_bgdata[bg_off];
                        break;
                  }
                  break;

               case 2:
                  switch (gd->bg_opt)
                  {
                     case 1:
                        intens = int_arr[j] - i2_bgdata[bg_off];
                        break;
                     case 2:
                        intens = i2_bgdata[bg_off];
                        break;
                  }
                  break;

               case 3:
                  switch (gd->bg_opt)
                  {
                     case 1:
                        intens = int_arr[j] - int_bgdata[bg_off];
                        break;
                     case 2:
                        intens = int_bgdata[bg_off];
                        break;
                  }
                  break;

               case 4:
                  switch (gd->bg_opt)
                  {
                     case 1:
                        intens = int_arr[j] - gd->unsq(i2_bgsquash[bg_off]);
                        break;
                     case 2:
                        intens = gd->unsq(i2_bgsquash[bg_off]);
                        break;
                  }
                  break;
            }
         }
         if (intens<=gd->min_thresh)
         {
            pixval = 0;
         }
         else if (intens>=gd->max_thresh)
         {
            pixval = 63;
         }
         else
         {
            pixval = 63*(intens-gd->min_thresh)/range;
         }
         if (expand==1)
         {
            if (XDL_pw_depth==8)
            {
               ip = yoff*img_width+xoff;
               gd->img[ip] = pix[pixval];
            }
            else
            {
               XPutPixel (gd->img_image, xoff, yoff, pix[pixval]);
            }
         }
         else
         {
            /* Need to check against max x, y, if expand>1 */
            if (XDL_pw_depth==8&&XDL_color>0)
            {
               ipstart = yoff*img_width+xoff;
               for (ii=0;ii<expand;++ii)
               {
                  ip=ipstart;
                  if (yoff+ii>=0&&yoff+ii<img_height)
                  {
                     for (jj=0;jj<expand;++jj,++ip)
                     {
                        if (xoff+jj>=0&&xoff+jj<img_width)
                        { 
                            gd->img[ip] = pix[pixval];
                        }
                     }
                  }
                  ipstart += img_width;
               }
            }
            else
            {
               if (XDL_color>0||expand<9)
               {
                  for (ii=0;ii<expand;++ii)
                  {
                     if (yoff+ii>=0&&yoff+ii<img_height)
                     {
                        for (jj=0;jj<expand;++jj)
                        {
                           if (xoff+jj>=0&&xoff+jj<img_width) 
                                              XPutPixel (gd->img_image, 
                                              xoff+jj, yoff+ii, pix[pixval]);
                        }
                     }
                  }
               }
               else
               {
                  pixval = pixval/7;
                  grayp = &grscal[9*pixval];
                  for (ii=0;ii<expand;++ii)
                  {
                     if (yoff+ii>=0&&yoff+ii<img_height)
                     {
                        iii = ii%3;
                        for (jj=0;jj<expand;++jj)
                        {
                           if (xoff+jj>=0&&xoff+jj<img_width) 
                           {
                              jjj = jj%3;
                              if (grayp[3*iii+jjj]==0)
                              {
                                 XPutPixel (gd->img_image, xoff+jj, yoff+ii, 
                                            XDL_whitepixel);
                              }
                              else
                              {
                                 XPutPixel (gd->img_image, xoff+jj, yoff+ii, 
                                            XDL_blackpixel); 
                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }
   return;
}

/*
*********************************
**     xdl_image_set_freeze    **
*********************************

Purpose: Draw rectangle on image display to mark magnifying window when
         frozen; Save parameters of area that needs to be restored to remove
         this rectangle.

Author:  John W. Campbell, August 1990

*/

void xdl_image_set_freeze (gd)

XDl_image *gd;          /* Pointer to the global data area */

{
   XDl_image_freeze *fz;            /* Pointer to freeze details structure */
   int xmin, xmax, ymin, ymax;      /* Limits of magnifying area on display
                                       and limits of area to restore adjusted
                                       for boundary violations */
   int max_offset;                  /* Limits of magnifying window from
                                       centre position (image display
                                       pixels */

   fz = &(gd->freeze);
   fz->on = 1;
   max_offset = (gd->magwsize+gd->mag-1)/(2*gd->mag);
   xmin = gd->xc - max_offset;
   xmax = gd->xc + max_offset;
   ymin = gd->yc - max_offset;
   ymax = gd->yc + max_offset;
   XSetForeground (XDL_display, gd->paint_gc, gd->freeze_pixel);
   XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                   xmin, ymin, (xmax-xmin), (ymax-ymin));
   if (xmin<0) xmin = 0;
   if (xmax>gd->img_width) xmax = gd->img_width;
   if (ymin<0) ymin = 0;
   if (ymax>gd->img_height) ymax = gd->img_height;
   fz->xo = xmin;
   fz->yo = ymin;
   fz->width = xmax - xmin + 1;
   fz->height = ymax - ymin + 1;
   return;
}

/*
*************************************
**     xdl_image_restore_freeze    **
*************************************

Purpose: Restore area in image area which had frozen magnifying window
         rectangle marked.

Author:  John W. Campbell, August 1990

*/

void xdl_image_restore_freeze (gd)

XDl_image *gd;          /* Pointer to the global data area */

{
   XDl_image_freeze *fz;            /* Pointer to freeze details structure */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       freeze box */
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       freeze box */
   int xv_min, xv_max;              /* x limits (in main film display area)
                                       for vectors which may appear in the
                                       freeze box */
   int yv_min, yv_max;              /* y limits (in main film display area)
                                       for vectors which may appear in the
                                       freeze box */
   int yt_min, yt_max;              /* y limits (in main film display area
                                       for text which may appear in the
                                       freeze box */

   fz = &(gd->freeze);
   if (fz->on==0) return;
   fz->on = 0;
   XPutImage (XDL_display, gd->wdisp, gd->paint_gc, gd->img_image,
              fz->xo, fz->yo, fz->xo, fz->yo, fz->width, fz->height);
   if (gd->ovly1==0&&gd->ovly2==0) return;
   if (gd->num_symb==0) return;
   xs_min = fz->xo - MAX_SYMB_SIZE;
   xs_max = fz->xo + fz->width + MAX_SYMB_SIZE - 1;
   ys_min = fz->yo - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
   ys_max = fz->yo + fz->height + MAX_SYMB_SIZE - 1 + abs(gd->y_ovly_offset);
   xdl_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);
   xv_min = fz->xo - gd->cur_expand - 1;
   xv_max = fz->xo + fz->width - 1 + gd->cur_expand + 1;
   yv_min = fz->yo - gd->cur_expand - 1;
   yv_max = fz->yo + fz->height - 1 + gd->cur_expand + 1;
   xdl_image_rdv (gd, xv_min, xv_max, yv_min, yv_max, 0, 0, 0);
   yt_min = fz->yo;
   yt_max = fz->yo + fz->height - 1 + gd->ch_height;
   xdl_image_rdt (gd, yt_min, yt_max);
   if (gd->show_int&&gd->cur_expand>=gd->show_int_expand)
   {
       xdl_image_refresh (gd,0);
   }
   return;
}

/*
*********************************
**     xdl_image_set_rect      **
*********************************

Purpose: Draw rectangle on image display to mark area being selected
         with dragged button1 down (first saves rows of pixels which will
         be overwritten)

Author:  John W. Campbell, December 1991

*/

void xdl_image_set_rect (gd, x, y)

XDl_image *gd;          /* Pointer to the global data area */
int x;                  /* Current x position of pointer */
int y;                  /* Current y position of pointer */
{
   int w;
   int h;

   if (x>=gd->sel_rect_xo)
   {
      gd->sel_rect_x1 = gd->sel_rect_xo;
      gd->sel_rect_x2 = x;
   }
   else
   {
      gd->sel_rect_x1 = x;
      gd->sel_rect_x2 = gd->sel_rect_xo;
   }
   if (y>=gd->sel_rect_yo)
   {
      gd->sel_rect_y1 = gd->sel_rect_yo;
      gd->sel_rect_y2 = y;
   }
   else
   {
      gd->sel_rect_y1 = y;
      gd->sel_rect_y2 = gd->sel_rect_yo;
   }

   /* Save lines of pixels where rectangle will be drawn */

   w = gd->sel_rect_x2 - gd->sel_rect_x1 + 1;
   h = gd->sel_rect_y2 - gd->sel_rect_y1 + 1;
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_t_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y1, w, 1, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_b_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y2, w, 1, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_l_line, gd->paint_gc,
              gd->sel_rect_x1, gd->sel_rect_y1, 1, h, 0, 0);
   XCopyArea (XDL_display, gd->wdisp, gd->sel_rect_r_line, gd->paint_gc,
              gd->sel_rect_x2, gd->sel_rect_y1, 1, h, 0, 0);

   /* Draw rectangle */

   XSetForeground (XDL_display, gd->paint_gc, gd->freeze_pixel);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y1,
             gd->sel_rect_x2, gd->sel_rect_y1);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y2,
             gd->sel_rect_x2, gd->sel_rect_y2);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x1, gd->sel_rect_y1,
             gd->sel_rect_x1, gd->sel_rect_y2);
   XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
             gd->sel_rect_x2, gd->sel_rect_y1,
             gd->sel_rect_x2, gd->sel_rect_y2);
   return;
}

/*
***********************************
**     xdl_image_restore_rect    **
***********************************

Purpose: Restore pixels to remove rectangle drawn via xdl_image_set_rect

Author:  John W. Campbell, December 1991

*/

void xdl_image_restore_rect (gd)

XDl_image *gd;          /* Pointer to the global data area */

{
   int w;
   int h;

   w = gd->sel_rect_x2 - gd->sel_rect_x1 + 1;
   h = gd->sel_rect_y2 - gd->sel_rect_y1 + 1;
   XCopyArea (XDL_display, gd->sel_rect_t_line, gd->wdisp, gd->paint_gc,
              0, 0, w, 1, gd->sel_rect_x1, gd->sel_rect_y1);
   XCopyArea (XDL_display, gd->sel_rect_b_line, gd->wdisp, gd->paint_gc,
              0, 0, w, 1, gd->sel_rect_x1, gd->sel_rect_y2);
   XCopyArea (XDL_display, gd->sel_rect_l_line, gd->wdisp, gd->paint_gc,
              0, 0, 1, h, gd->sel_rect_x1, gd->sel_rect_y1);
   XCopyArea (XDL_display, gd->sel_rect_r_line, gd->wdisp, gd->paint_gc,
              0, 0, 1, h, gd->sel_rect_x2, gd->sel_rect_y1);
   return;
}
/*
**********************************
**     xdl_image_intensity      **
**********************************

Purpose: Get intensity value at selected position

Author:  John W. Campbell, August 1990

*/

void xdl_image_intensity (gd, ax1pix, ax2pix, show)

XDl_image *gd;          /* Pointer to global data area */
int ax1pix;             /* Pixel position along local axis 1 */
int ax2pix;             /* Pixel position along local axis 2 */
int show;               /* =1 show on panel, =0 do not */



{
   long intens;              /* Intensity */
   char str[20];            /* Panel label string for intensity value */
   unsigned char *byte_data;/* Pointer to full image byte data */
   I2pix * i2_data;         /* Pointer to full image (i2) data */
   I2squash *i2_squash;     /*Pointer to full image 'squashed i2' data */
   Intpix * int_data;       /* Pointer to full image integer type data */
   unsigned char *byte_bgdata; /* Pointer to background image byte data */
   I2pix * i2_bgdata;       /* Pointer to background image (i2) data */
   I2squash *i2_bgsquash;   /* Pointer to background 'squashed i2' data */
   Intpix * int_bgdata;     /* Pointer to background image integer type data */
   long bgval;              /* Background value */
   unsigned short i2_val;   /* Element from full image-plate data */
   int int_val;             /* Element from full integer data */
   int spix;                /* Pixel position along slow axis */
   int fpix;                /* Pixels position along fast axis */
   int ioff;                /* Offset into full image array */
   int ioff_bg;             /* Offset into background image array */

   xdl_image_local_to_fs (gd, ax1pix, ax2pix, &fpix, &spix, 1);

   ioff = (spix-1)*gd->nf_off + fpix - 1;
   ioff_bg = ((spix-1)/gd->bg_cmp)*gd->nf_bgoff + (fpix-1)/gd->bg_cmp;

   switch (gd->type)
   {
      case 1:
         byte_data = (unsigned char *) gd->imgdata;
         intens = (long) byte_data[ioff];
      break;

      case 2:
         i2_data = (I2pix *) gd->imgdata;
         intens = (long) i2_data[ioff];
      break;

      case 3:
         int_data = (Intpix *) gd->imgdata;
         intens = (long) int_data[ioff];
      break;

      case 4:
         i2_squash = (I2squash *) gd->imgdata;
         intens = (long) gd->unsq(i2_squash[ioff]);
      break;
   }
   if (gd->bg_opt>0&&gd->bg_present==1)
   {
      switch (gd->type)
      {
         case 1:
            byte_bgdata = (unsigned char *) gd->bg_data;
            bgval = (long) byte_bgdata[ioff_bg];
         break;

         case 2:
            i2_bgdata = (I2pix *) gd->bg_data;
            bgval = (long) i2_bgdata[ioff_bg];
         break;

         case 3:
            int_bgdata = (Intpix *) gd->bg_data;
            bgval = (long) int_bgdata[ioff_bg];
         break;

         case 4:
            i2_bgsquash = (I2squash *) gd->bg_data;
            bgval = (long) gd->unsq(i2_bgsquash[ioff_bg]);
         break;
      }
      if (gd->bg_opt==1)
      {
         intens -= bgval;
         if (intens<0) intens = 0;
      }
      else if (gd->bg_opt==2)
      {
         intens = bgval;
      }
   }
   gd->intensity = intens;
   if (show==0) return;
   sprintf (str, "Intensity %ld", intens);
   xdl_panel_label_set (gd->cursor_position, str, 0, 0);
   return;
}
/*
****************************
**     xdl_image_unsq     **
****************************

Purpose: Get intensity value from 'squashed i2' pixel (type 4)

Author:  John W. Campbell, September 1996

*/

long xdl_image_unsq (ipx)

I2squash ipx;               /* Stored pixel value */
{
   if (ipx>=0) return (long) ipx;
   return (long) -ipx*8;
}

/*
****************************
**     xdl_image_unsq2    **
****************************

Purpose: Get intensity value from 'squashed i2' pixel (type 5 - RAXIS II)

Author:  John W. Campbell, September 1996

*/

long xdl_image_unsq2 (ipx)

I2squash ipx;               /* Stored pixel value */
{
   if (ipx>=0) return (long) ipx;
   return (long) (ipx+32768)*8;
}

/*
****************************
**     xdl_image_unsq3    **
****************************

Purpose: Get intensity value from 'squashed i2' pixel (type 6 - RAXIS IV)

Author:  John W. Campbell, September 1996

*/

long xdl_image_unsq3 (ipx)

I2squash ipx;               /* Stored pixel value */
{
   if (ipx>=0) return (long) ipx;
   return (long) (ipx+32768)*32;
}

/*
****************************
**     xdl_image_unsq4    **
****************************

Purpose: Get intensity value from 'squashed i2' pixel (type 7 - MAC SCIENCE)

Author:  John W. Campbell, September 1996

*/

long xdl_image_unsq4 (ipx)

I2squash ipx;               /* Stored pixel value */
{
   if (ipx>=0) return (long) ipx;
   return (long) -(ipx+1)*256 + 32768;
}

/*
********************************
**     xdl_image_drawsymb     **
********************************

Purpose: Draw an overlay symbol

Author:  John W. Campbell, August  1990

*/

void xdl_image_drawsymb (gd, wid, x, y, symbol, color)

XDl_image *gd;            /* Pointer top global data area */
Window wid;               /* Window id (film or magnifying window) */
int x;                    /* x offset for symbol centre */
int y;                    /* y offset for symbol centre */
int symbol;               /* symbol type  1 to 10,  Vertical cross - increasing
                                                    size
                                          11 to 20, Cross - increasing size
                                          21 to 30, Square - increasing size */
int color;                /* overlay colour number (1-6) */

{
   int symtyp;            /*symbol type 0=vertical cross, 1=cross, 2=square*/
   int symsize;           /*size of symbol from 0 to 9*/

   if (gd->exposed==0) return;
   symtyp = (symbol-1) / 10;
   symsize = (symbol-1) % 10;

   XSetForeground (XDL_display, gd->paint_gc, gd->ovly_pixels[color-1]);

   if (symsize==0)
   {
      XDrawPoint (XDL_display, wid, gd->paint_gc, x, y);
   }
   else
   {
   switch (symtyp)
      {
         case 0:
         /* Vertical Cross */
            XDrawLine (XDL_display, wid, gd->paint_gc, x-symsize, y,
                       x+symsize, y);
            XDrawLine (XDL_display, wid, gd->paint_gc, x, y-symsize,
                       x, y+symsize);
            break;
         case 1:
         /* Cross */
            XDrawLine (XDL_display, wid, gd->paint_gc, x-symsize, y-symsize,
                       x+symsize, y+symsize);
            XDrawLine (XDL_display, wid, gd->paint_gc, x+symsize, y-symsize,
                       x-symsize, y+symsize);
            break;
         case 2:
         /* Square */
            XDrawRectangle (XDL_display, wid, gd->paint_gc,
                            x-symsize, y-symsize, 2*symsize, 2*symsize);
            break;
      }
   }
   return;
}

/*
********************************
**     xdl_image_drawvec      **
********************************

Purpose: Draw an overlay vector

Author:  John W. Campbell, December 1991

*/

void xdl_image_drawvec (gd, wid, pixsiz, x1, y1, x2, y2, x1_off, y1_off,
                        x2_off, y2_off, color)

XDl_image *gd;            /* Pointer top global data area */
Window wid;               /* Window id (film or magnifying window) */
int pixsiz;               /* Current display pIxel size */
int x1;                   /* x offset for start of vector */
int y1;                   /* y offset for start of vector */
int x2;                   /* x offset for end of vector */
int y2;                   /* y offset for end of vector */
short x1_off;             /* x1 offset flag */
short y1_off;             /* y1 offset flag */
short x2_off;             /* x2 offset flag */
short y2_off;             /* y2 offset flag */
int color;                /* overlay colour number (1-6) */

{
   int xx1;
   int yy1;
   int xx2;
   int yy2;

   if (gd->exposed==0) return;

   XSetForeground (XDL_display, gd->paint_gc, gd->ovly_pixels[color-1]);

   xx1 = x1;
   yy1 = y1;
   xx2 = x2;
   yy2 = y2;
   switch (x1_off)
   {
      case -2: xx1 = xx1 - 1; break;
      case  0: xx1 = xx1 + pixsiz/2; break;
      case  1: xx1 = xx1 + pixsiz - 1; break;
      case  2: xx1 = xx1 + pixsiz; break;
   }
   switch (y1_off)
   {
      case -2: yy1 = yy1 - 1; break;
      case  0: yy1 = yy1 + pixsiz/2; break;
      case  1: yy1 = yy1 + pixsiz - 1; break;
      case  2: yy1 = yy1 + pixsiz; break;
   }
   switch (x2_off)
   {
      case -2: xx2 = xx2 - 1; break;
      case  0: xx2 = xx2 + pixsiz/2; break;
      case  1: xx2 = xx2 + pixsiz - 1; break;
      case  2: xx2 = xx2 + pixsiz; break;
   }
   switch (y2_off)
   {
      case -2: yy2 = yy2 - 1; break;
      case  0: yy2 = yy2 + pixsiz/2; break;
      case  1: yy2 = yy2 + pixsiz - 1; break;
      case  2: yy2 = yy2 + pixsiz; break;
   }
   if (xx1<0&&xx2<0) return;
   if (xx1>gd->img_width&&xx2>gd->img_width) return;
   if (yy1<0&&yy2<0) return;
   if (yy1>gd->img_height&&yy2>gd->img_height) return;
   if (xx1>10000)
   {
      yy1 = yy2 + ((float)(10000-xx2)/(float)(xx1-xx2))*(yy1-yy2);
      xx1 = 10000;
   }
   if (xx1<-10000)
   {
      yy1 = yy2 + ((float)(-10000-xx2)/(float)(xx1-xx2))*(yy1-yy2);
      xx1 = -10000;
   }
   if (yy1>10000)
   {
      xx1 = xx2 + ((float)(10000-yy2)/(float)(yy1-yy2))*(xx1-xx2);
      yy1 = 10000;
   }
   if (yy1<-10000)
   {
      xx1 = xx2 + ((float)(-10000-yy2)/(float)(yy1-yy2))*(xx1-xx2);
      yy1 = -10000;
   }
   if (xx2>10000)
   {
      yy2 = yy1 + ((float)(10000-xx1)/(float)(xx2-xx1))*(yy2-yy1);
      xx2 = 10000;
   }
   if (xx2<-10000)
   {
      yy2 = yy1 + ((float)(-10000-xx1)/(float)(xx2-xx1))*(yy2-yy1);
      xx2 = -10000;
   }
   if (yy2>10000)
   {
      xx2 = xx1 + ((float)(10000-yy1)/(float)(yy2-yy1))*(xx2-xx1);
      yy2 = 10000;
   }
   if (yy2<-10000)
   {
      xx2 = xx1 + ((float)(-10000-yy1)/(float)(yy2-yy1))*(xx2-xx1);
      yy2 = -10000;
   }

   XDrawLine (XDL_display, wid, gd->paint_gc, xx1, yy1, xx2, yy2);

   return;
}
/*
********************************
**     xdl_image_dsppos       **
********************************

Purpose: Get pixel position within display window

Author:  John W. Campbell, December 1991

*/

void xdl_image_dsppos (xyoff, nxyp, expand, xy_disp)

int xyoff;       /* x or y pixel offset wrt origin of displayed section (R)*/
int nxyp;        /* No. of x or y points compressed into a pixel (R)*/
int expand;      /* Current expand factor */
int *xy_disp;    /* Returns offset within main display area (W)*/
{
   /* Assumes that not both nxyp and expand are > 1 */
   if (expand==1)
   {
      if (xyoff>=0)
      {
         *xy_disp = xyoff / nxyp;
      }
      else
      {
         *xy_disp = (xyoff + 1 - nxyp) / nxyp;
      }
   }
   else
   {
      *xy_disp = xyoff*expand;
   }
   return;
}
/*
*************************************
**     xdl_image_symbol_offsets    **
*************************************

Purpose: Get display image offsets for a symbol based on current origin,
         compression etc.

Author:  John W. Campbell, August 1993

*/

void xdl_image_symbol_offsets (gd, iposn)

XDl_image *gd;
int iposn;         /* Offset into symbols list */
{
   int xoff;                 /* x Offset of pixel (image pixels) in display
                                area*/
   int yoff;                 /* Offset of pixel (image pixels) from
                                top of film display area */
   int x;                    /* x pixel position offset in main film display 
                                area (display area pixels) */
   int y;                    /* y pixel position offset in main film display 
                                area (display area pixels) */
   int exp2;                 /* Current expand size / 2*/

   exp2 = gd->cur_expand/2;
   xoff = gd->symbols[iposn].x_pixel - gd->cur_xo_pix;
   xdl_image_dsppos(xoff,gd->cur_cmp,gd->cur_expand,&x);
   gd->symbols[iposn].x_disp_offset = x + exp2;
   yoff = gd->symbols[iposn].y_pixel - gd->cur_yo_pix;
   xdl_image_dsppos(yoff,gd->cur_cmp,gd->cur_expand,&y);
   gd->symbols[iposn].y_disp_offset = y + exp2;
   return;
}
/*
*************************************
**     xdl_image_vector_offsets    **
*************************************

Purpose: Get display image offsets for a vector based on current origin,
         compression etc.

Author:  John W. Campbell, August 1993

*/

void xdl_image_vector_offsets (gd, iposn)

XDl_image *gd;
int iposn;         /* Offset into vectors list */
{
   int x, y;                 /* Temp x, y positions */
   int x1,x2;                /* x pixel position offsets in main film display 
                                area (display area pixels) */
   int y1,y2;                /* y pixel position offsets in main film display 
                                area (display area pixels) */
   int xoff1,xoff2;          /* x Offsets of pixels (image pixels) in 
                                display area*/
   int yoff1,yoff2;          /* Offsets of pixels (image pixels) from
                                top of film display area */
   int x1_p;                 /* x1 Offset flag */
   int x2_p;                 /* x2 Offset flag */
   int y1_p;                 /* y1 Offset flag */
   int y2_p;                 /* y2 Offset flag */

   xoff1 = gd->vects[iposn].x1_pixel - gd->cur_xo_pix;
   xdl_image_dsppos(xoff1,gd->cur_cmp,gd->cur_expand,&x1);
   xoff2 = gd->vects[iposn].x2_pixel- gd->cur_xo_pix;
   xdl_image_dsppos(xoff2,gd->cur_cmp,gd->cur_expand,&x2);
   yoff1 = gd->vects[iposn].y1_pixel- gd->cur_yo_pix;
   xdl_image_dsppos(yoff1,gd->cur_cmp,gd->cur_expand,&y1);
   yoff2 = gd->vects[iposn].y2_pixel- gd->cur_yo_pix;
   xdl_image_dsppos(yoff2,gd->cur_cmp,gd->cur_expand,&y2);
   x1_p =  gd->vects[iposn].x1_offset;
   y1_p =  gd->vects[iposn].y1_offset;
   x2_p =  gd->vects[iposn].x2_offset;
   y2_p =  gd->vects[iposn].y2_offset;

   /*If y2<y1 swap ends of vector before storage */

   if (y2<y1)
   {
      x = x1;
      y = y1;
      x1 = x2;
      y1 = y2;
      x2 = x;
      y2 = y;
      x = x1_p;
      x1_p = x2_p;
      x2_p = x;
      y = y1_p;
      y1_p = y2_p;
      y2_p = y;
      x = gd->vects[iposn].x1_pixel;
      y = gd->vects[iposn].y1_pixel;
      gd->vects[iposn].x1_pixel = gd->vects[iposn].x2_pixel;
      gd->vects[iposn].y1_pixel = gd->vects[iposn].y2_pixel;
      gd->vects[iposn].x2_pixel = x;
      gd->vects[iposn].y2_pixel = y;
   }
   gd->vects[iposn].x1_disp_offset = x1;
   gd->vects[iposn].y1_disp_offset = y1;
   gd->vects[iposn].x1_offset = x1_p;
   gd->vects[iposn].y1_offset = y1_p;
   gd->vects[iposn].x2_disp_offset = x2;
   gd->vects[iposn].y2_disp_offset = y2;
   gd->vects[iposn].x2_offset = x2_p;
   gd->vects[iposn].y2_offset = y2_p;

   return;
}
/*
***********************************
**     xdl_image_text_offsets    **
***********************************

Purpose: Get display image offsets for a text stringbased on current origin,
         compression etc.

Author:  John W. Campbell, August 1993

*/

void xdl_image_text_offsets (gd, iposn)

XDl_image *gd;
int iposn;         /* Offset into text strings list */
{
   int xoff;                 /* x Offset of pixel (image pixels) in display
                                area*/
   int yoff;                 /* Offset of pixel (image pixels) from
                                top of film display area */
   int x;                    /* x pixel position offset in main film display 
                                area (display area pixels) */
   int y;                    /* y pixel position offset in main film display 
                                area (display area pixels) */
   int x_p;                  /* x pixel offset within main pixel 0 to nxp-1 */
   int y_p;                  /* y pixel offset within main pixel 0 to nyp-1 */
   int exp2;                 /* Current expand size / 2*/

   exp2 = gd->cur_expand/2;
   xoff = gd->text[iposn].x_pixel - gd->cur_xo_pix;
   xdl_image_dsppos(xoff,gd->cur_cmp,gd->cur_expand,&x);
   gd->text[iposn].x_disp_offset = x + exp2;
   yoff = gd->text[iposn].y_pixel - gd->cur_yo_pix;
   xdl_image_dsppos(yoff,gd->cur_cmp,gd->cur_expand,&y);
   gd->text[iposn].y_disp_offset = y + exp2;
   return;
}

/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Get selected pixel position - xdl_image_getpix
The routine xdl_image_getpix (xdlf_image_getpix) is used to
return the last selected input pixel (spot) position to the calling
program. It will normally be called after the return from a call to
xdl_get_events (xdlf_get_events) indicates that an image view-object
has returned input.
-end*/
/*
*****************************
**     xdl_image_getpix    ** 
*****************************

Purpose: Return the last selected spot position

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_GETPIX (IVH, IAX1PIX, IAX2PIX, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IAX1PIX  (W)  Returns local axis 1 pixel position 
              (see ax1pix_img)
IAX2PIX  (W)  Returns local axis 2 pixel position 
              (see ax2pix_img)
IERR     (W)  Returns status from xdl_image_getpix call
-end*/

/*-C:*/
int xdl_image_getpix (vh, ax1pix_img, ax2pix_img)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *ax1pix_img; /* Returns the local axis 1 pixel position 
                    (from 1 up - refers to complete uncompressed 
                    image) (W) */
int *ax2pix_img; /* Returns the local axis 2 pixel position (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_getpix",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_getpix", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   *ax1pix_img = gd->ax1pix_img;
   *ax2pix_img = gd->ax2pix_img;
   return 0;
}

/* Fortran binding: xdlf_image_getpix */

void xdlf_image_getpix (ivh, iax1pix, iax2pix, ierr)
int *ivh, *iax1pix, *iax2pix, *ierr;
{
   *ierr = xdl_image_getpix (*ivh, iax1pix, iax2pix);
   return;
}

/*-Routine: Get pixel position from an overlay - xdl_image_getpix_ovly
The routine xdl_image_getpix_ovly (xdlf_image_getpix_ovly) is used 
to return the last selected input pixel (e.g. symbol) position from an
overlay to the calling program. It differs from xdl_image_getpix
(xdlf_image_getpix) in that it makes an allowance for any offsetting
of the overlay currently in force. It will normally be called after the 
return from a call to xdl_get_events (xdlf_get_events) indicates that an 
image view-object has returned input.
-end*/
/*
**********************************
**     xdl_image_getpix_ovly    ** 
**********************************

Purpose: Return the last selected symbol position (allows for any
         offsetting of the overlay)

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_GETPIX_OVLY (IVH, IOV, IXPIX, IYPIX, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IOV      (R)  Overlay number (1 o 2) (see ov_num)
IAX1PIX  (W)  Returns local axis 1 pixel position 
              (see ax1pix_img)
IAX2PIX  (W)  Returns local axis 2 pixel position 
              (see ax2pix_img)
IERR     (W)  Returns status from xdl_image_getpix_ovly call
-end*/

/*-C:*/
int xdl_image_getpix_ovly (vh, ov_num, ax1pix_img, ax2pix_img)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int ov_num;      /* Overlay number (1 or 2) 
                    Note: Overlay 2 is never offset.
                          Overlay 1 will be offset if the overlay type
                          is on/off/offset and the offset option has
                          been selected  (R)*/
int *ax1pix_img; /* Returns the local axis 1 pixel position (from 1 up 
                    - refers to complete uncompressed image); any 
                    overlay offset is subtracted from the value before 
                    it is returned (W) */
int *ax2pix_img; /* Returns the local axis 2 pixel position (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   float curfac;           /* Current image to display pixel ratio */
   int xpix;               /* Pixel x position wrt X-windows axes */
   int ypix;               /* Pixel y position wrt X-windows axes */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_getpix_ovly",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_getpix_ovly", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;

   curfac = (float) gd->cur_cmp / (float) gd->cur_expand;
   if (ov_num==1&&gd->ovly_offset==1)
   {
      xdl_image_local_to_xy (gd, gd->ax1pix, gd->ax2pix, &xpix, &ypix, 1);
      xpix = nint (xpix - gd->x_ovly_offset*curfac);
      ypix = nint (ypix - gd->y_ovly_offset*curfac);
      xdl_image_xy_to_local (gd, xpix, ypix, ax1pix_img, ax2pix_img, 1);
   }
   else
   {
      *ax1pix_img = gd->ax1pix_img;
      *ax2pix_img = gd->ax2pix_img;
   }
   return 0;
}

/* Fortran binding: xdlf_image_getpix_ovly */

void xdlf_image_getpix_ovly (ivh, iov, iax1pix, iax2pix, ierr)
int *ivh, *iov, *iax1pix, *iax2pix, *ierr;
{
   *ierr = xdl_image_getpix_ovly (*ivh, *iov, iax1pix, iax2pix);
   return;
}

/*-Routine: Get selected rectangle - xdl_image_getrect
The routine xdl_image_getrect (xdlf_image_getrect) is used to
return the last selected rectangle coordinates in pixels to the calling
program. It will normally be called after the return from a call to
xdl_get_events (xdlf_get_events) indicates that an image view-object
has returned input.
-end*/
/*
*****************************
**     xdl_image_getrect   ** 
*****************************

Purpose: Return the last selected rectangle

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_GETRECT (IVH, IAX1PIX1, IAX2PIX1, 
        +          IAX1PIX2, IAX2PIX2, IVECT, ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IAX1PIX1 (W)  Returns 1'st local axis 1 pixel position 
              (see ax1pix_dn)
IAX2PIX1 (W)  Returns 1'st local axis 2 pixel position 
              (see ax2pix_dn)
IAX1PIX2 (W)  Returns 2'nd local axis 1 pixel position 
              (see ax1pix_up)
IAX2PIX2 (W)  Returns 2'nd local axis 2 pixel position 
              (see ax2pix_up)
IVECT    (R)  Vector identifier for drawing box at selected
              position (1-65535); 0 if no box is to be 
              drawn (see also description of 
              xdlf_image_drawvect routine)
ICOLR    (R)  Vector colour (1 to 6) if needed (see color)
IOV      (R)  Overlay number 1 or 2 if needed (see ov_num)
IERR     (W)  Returns status from xdl_image_getrect call
-end*/

/*-C:*/
int xdl_image_getrect (vh, ax1pix_dn, ax2pix_dn, ax1pix_up, ax2pix_up,
                            vect_id, color, ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *ax1pix_dn;  /* Returns the local axis 1 pixel position (from 1 up 
                    - refers to complete uncompressed image) of rectangle 
                    corner where Button1 was pressed (W) */
int *ax2pix_dn;  /* Returns the local axis 2 pixel position where Button1 
                    was pressed (W) */
int *ax1pix_up;  /* Returns the local axis 1 pixel position (from 1 up 
                    - refers to complete uncompressed image) of rectangle 
                    corner where Button1 was released (W) */
int *ax2pix_up;  /* Returns the local axis 2 pixel position where Button1 
                    was released (W) */
int vect_id;     /* Vector identifier (1-65535) to draw a box on the
                    image display at the selected rectangle position;
                    0 if no box is to be drawn. (see xdl_image_vect) */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default) if needed*/
int ov_num;      /* Overlay number 1 or 2 (R) if needed*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_getrect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_getrect", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   *ax1pix_dn = gd->ax1pix_dn;
   *ax2pix_dn = gd->ax2pix_dn;
   *ax1pix_up = gd->ax1pix_up;
   *ax2pix_up = gd->ax2pix_up;

   if (gd->zoom_rect)
   {
      xdl_image_restore_rect (gd);
      gd->zoom_rect = 0;
   }

   if (vect_id>0&&vect_id<=65535)
   {
      xdl_image_vect (vh, vect_id, gd->ax1pix_dn, gd->ax2pix_dn,
                                        gd->ax1pix_up, gd->ax2pix_dn,
                                        color, ov_num, 0);
      xdl_image_vect (vh, vect_id, gd->ax1pix_dn, gd->ax2pix_dn,
                                        gd->ax1pix_dn, gd->ax2pix_up,
                                        color, ov_num, 0);
      xdl_image_vect (vh, vect_id, gd->ax1pix_up, gd->ax2pix_dn,
                                        gd->ax1pix_up, gd->ax2pix_up,
                                        color, ov_num, 0);
      xdl_image_vect (vh, vect_id, gd->ax1pix_dn, gd->ax2pix_up,
                                        gd->ax1pix_up, gd->ax2pix_up,
                                        color, ov_num, 1);
   }
   return 0;
}

/* Fortran binding: xdlf_image_getrect */

void xdlf_image_getrect (ivh, iax1pix1, iax2pix1, iax1pix2, iax2pix2, 
                              ivect, icolr, iov, ierr)
int *ivh, *iax1pix1, *iax2pix1, *iax1pix2, *iax2pix2, *ivect, *icolr, *iov, *ierr;
{
   *ierr = xdl_image_getrect (*ivh, iax1pix1, iax2pix1, iax1pix2, iax2pix2,
                              *ivect, *icolr, *iov);
   return;
}

/*-Routine: Get display settings - xdl_image_settings
The routine xdl_image_settings (xdlf_image_settings) is used to obtain the
current display settings so that they can be used for example for the display
of another image.
-end*/
/*
*****************************
**     xdl_image_settings  ** 
*****************************

Purpose: Get current display settings

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_SETTINGS (IVH, MIN_THR, MAX_THR, MONO_THR,
        +                          ICONTRAST, ICOLOR_OPT, IOVLY_OPT,
        +                          IBG_OPT, MAG_OPT, ISHOW_INT, IERR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
MIN_THR    (W)  Returns the minimum scaling threshold 
                (see min_thr)
MAX_THR    (W)  Returns the maximum scaling threshold 
                (see max_thr)
MONO_THR   (W)  Returns the current threshold for monochome 
                display  0-63 (see mono_thr)
ICONTRAST  (W)  Returns the contrast value 1-1024 (see contrast)
ICOLOR_OPT (W)  Returns the colour choice option 1-7 for colour,
                1-2 for monochrome  (see color_opt)
IOVLY_OPT  (W)  Returns overlay option 1-3 or 1-4 (see ovly_opt)
IBG_OPT    (W)  Returns the background option 1-3 (see bg_opt)
MAG_OPT    (W)  Returns the current magnification option 1-7
                for colour or 1-10 for monochrome (see mag_opt)
ISHOW_INT  (W)  Returns flag for whether intensity values are 
                to be shown on zooming 0-1 (see show_int)
IERR       (W)  Returns status from xdl_image_settings call
-end*/

/*-C:*/
int xdl_image_settings (vh, min_thr, max_thr, mono_thr, contrast,
                        color_opt, ovly_opt, bg_opt, mag_opt, show_int)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int *min_thr;    /* Returns the minimum scaling threshold (W) */
int *max_thr;    /* Returns the maximum scaling threshold (W) */
int *mono_thr;   /* Returns the current threshold for monochome 
                    display  0-63 (W)*/
int *contrast;   /* Returns the contrast value 1-1024 (W) */
int *color_opt;  /* Returns the colour choice option 1-7 for colour,
                    1-2 for monochrome (W) */
int *ovly_opt;   /* Returns overlay option 1-3 or 1-4 (W) */
int *bg_opt;     /* Returns the background option 1-3 (W) */
int *mag_opt;    /* Returns the current magnification option 1-7
                    for colour or 1-10 for monochrome (W) */
int *show_int;   /* Returns flag for whether intensity values are to be
                    shown on zooming 0-1 (W) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_settings",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_settings", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   *min_thr = gd->min_thresh;
   *max_thr = gd->max_thresh;
   *mono_thr = gd->thresh_val;
   *contrast = gd->contrast_val;
   *color_opt = gd->color_opt;
   *ovly_opt = gd->ovly_opt;
   *bg_opt = gd->bg_opt;
   *mag_opt = gd->magwsize_opt;
   *show_int = gd->show_int;
   return 0;
}

/* Fortran binding: xdlf_image_settings */

void xdlf_image_settings (ivh, min_thr, max_thr, mono_thr, icontrast,
                          icolor_opt, iovly_opt, ibg_opt, mag_opt,
                          ishow_int, ierr)
int *ivh, *min_thr, *max_thr, *mono_thr, *icontrast, *icolor_opt;
int *iovly_opt, *ibg_opt, *mag_opt, *ishow_int, *ierr;
{
   *ierr = xdl_image_settings (*ivh, min_thr, max_thr, mono_thr, icontrast,
                               icolor_opt, iovly_opt, ibg_opt, mag_opt,
                               ishow_int);
   return;
}
/*-Routine: Reset current display settings - xdl_image_reset
The routine xdl_image_reset (xdlf_image_reset) is used to set new values
for the display settings such as the scaling thresholds, contrast level
and colour options.
-end*/
/*
*****************************
**     xdl_image_reset     ** 
*****************************

Purpose: Reset current display settings

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_RESET (IVH, MIN_THR, MAX_THR, MONO_THR,
        +                       ICONTRAST, ICOLOR_OPT, IOVLY_OPT,
        +                       IBG_OPT, MAG_OPT, ISHOW_INT, IERR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
MIN_THR    (R)  Sets the minimum scaling threshold (see min_thr)
MAX_THR    (R)  Sets the maximum scaling threshold (see max_thr)
MONO_THR   (R)  Sets the current threshold for monochome 
                display  0-63 (see mono_thr)
ICONTRAST  (R)  Sets the contrast value 1-1024 (see contrast)
ICOLOR_OPT (R)  Sets the colour choice option 1-7 for colour,
                1-2 for monochrome  (see color_opt)
                    1 = Black on white
                    2 = White on black
                    3 = Yellow if sat.
                    4 = Colour 1
                    5 = Colour 2
                    6 = User defined
                    7 = Random
IOVLY_OPT  (R)  Sets overlay option 1-3 or 1-4 (see ovly_opt)
                1'st case 1=on, 2=off, 3=offset
                2'nd case 1=on, 2=off, 3=ov1 on, 4=ov2 on
IBG_OPT    (R)  Sets the background option 1-3 (see bg_opt)
                1=included, 2=subtracted, 3=background only 
MAG_OPT    (R)  Sets the current magnification option 1-7
                for colour or 1-10 for monochrome (see mag_opt)
                1=x2, 2=x3, 3=X4, 4=x5, 5=x6, 6=x7, 7=x8
                8=x3G, 9=x6G, 10=x9G (monochrome simulated gray) 
ISHOW_INT  (R)  Sets flag for whether intensity values are 
                to be shown on zooming 0-1 0=no, 1-yes
                (see show_int)
IERR       (W)  Returns status from xdl_image_reset call
-end*/

/*-C:*/
int xdl_image_reset (vh, min_thr, max_thr, mono_thr, contrast,
                        color_opt, ovly_opt, bg_opt, mag_opt, show_int)
/*end*/
 
/*-Parameters:*/
int vh;         /* View-object handle (R)*/
int min_thr;    /* Sets the minimum scaling threshold (ignore if <=
                   -10,000,000) (R) */
int max_thr;    /* Sets the maximum scaling threshold (ignore if <=
                   -10,000,000 (R) */
int mono_thr;   /* Sets the current threshold for monochome 
                    display  0-63  (ignore if < 0) (R)*/
int contrast;   /* Sets the contrast value 1-1024 (R) */
int color_opt;  /* Sets the colour choice option 1-7 
                    1-7 for colour, 1-2 for monochrome (ignore if < 0) 
                    1 = Black on white
                    2 = White on black
                    3 = Yellow if sat.
                    4 = Colour 1
                    5 = Colour 2
                    6 = User defined
                    7 = Random
                    (R) */
int ovly_opt;   /* Sets overlay option 1-3 (overlay type 1 on
                   initial call or 1-4 for overlay type 2 on
                   initial call to xdl_image.
                   1'st case 1=on, 2=off, 3=offset
                   2'nd case 1=on, 2=off, 3=ov1 on, 4=ov2 on
                   (ignore if < 0) 
                   (R) */
int bg_opt;     /* Sets the background option 1-3 (ignore if < 0) 
                   1=included, 2=subtracted, 3=background only (R) */
int mag_opt;    /* Sets the current magnification option 1-7
                   for colour or 1-10 for monochrome (ignore if < 0) 
                   1=x2, 2=x3, 3=X4, 4=x5, 5=x6, 6=x7, 7=x8
                   8=x3G, 9=x6G, 10=x9G  (monochrome simulated gray)
                   (R) */
int show_int;   /* Sets flag for whether intensity values are to be
                   shown on zooming 0-1 0=no, 1=yes (ignore if < 0) 
                   (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int colr;               /* =1 colour reset required, =0 not */
   int setdata;            /* =1 call to xdl_image_setdata required, =0 not */
   int islide;             /* Slider position */
   char str[20];           /* Temp string */
   int ll;                 /* String length */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_reset",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_reset", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->reset = 1;
   setdata = 0;
   colr = 0;
   if (min_thr>-10000000)
   {
      sprintf (str,"%d",min_thr);
      ll = strlen (str);
      xdl_image_cp5 (iv, str, ll, 8);
      setdata = 1;
   }
   if (max_thr>-10000000)
   {
      sprintf (str,"%d",max_thr);
      ll = strlen (str);
      xdl_image_cp4 (iv, str, ll, 8);
      setdata = 1;
   }
   if (mono_thr>=0&&XDL_color==0)
   {
      if (mono_thr>63) mono_thr = 63;
      gd->thresh_val = mono_thr;
      xdl_panel_slider_reset (gd->thresh_slider, (mono_thr*64)/63);
      colr = 1;
      setdata = 1;
   }
   if (contrast>=0&&XDL_color>0)
   {
       if (contrast<1) contrast = 1;
       if (contrast>1024) contrast = 1024;
       gd->contrast_val = contrast;
       islide = (gd->slider_len*gd->contrast_val)/1024;
       xdl_panel_slider_reset (gd->contrast_slider, islide);
       colr = 1;
       setdata = 1;
   }
   if (gd->color_opt>=0)
   {
      if (color_opt<1) color_opt = 1;
      if (XDL_color>0&&color_opt>7) color_opt = 7;
      if (XDL_color==0&&color_opt>2) color_opt = 2;
      gd->color_opt = color_opt;
      xdl_panel_choice_reset (gd->color_choice, color_opt);
      colr = 1;
      setdata = 1;
   }
   if (ovly_opt>=0)
   {
      if (ovly_opt<1) ovly_opt = 1;
      if (gd->ovly_typ==1&&ovly_opt>3) ovly_opt = 3;
      if (gd->ovly_typ==2&&ovly_opt>4) ovly_opt = 4;
      xdl_panel_choice_reset (gd->ovly_choice, ovly_opt);
      xdl_image_cp6 (iv, ovly_opt);
   }
   if (bg_opt>=0&&gd->bg_typ>0)
   {
      if (bg_opt<1) bg_opt = 1;
      if (bg_opt>3) bg_opt = 3;
      switch (bg_opt)
      {
         case 1:
            gd->bg_opt = 0;
            break;
         case 2:
            if (gd->bg_present==0)
            {
               gd->bg_opt = 0;
            }
            else
            {
               gd->bg_opt = 1;
            }
            break;
         case 3:
            if (gd->bg_present==0)
            {
               gd->bg_opt = 0;
            }
            else
            {
               gd->bg_opt = 2;
            }
            break;
      }
      xdl_panel_choice_reset (gd->bg_choice, gd->bg_opt+1);
      setdata = 1;
   }
   if (mag_opt>=0)
   {
      if (mag_opt<1) mag_opt = 1;
      if (XDL_color>0&&mag_opt>7) mag_opt = 7;
      if (XDL_color==0&&mag_opt>10) mag_opt = 10;
      xdl_panel_choice_reset (gd->mag_choice, mag_opt);
      xdl_image_cp3 (iv, mag_opt);
   }
   if (show_int>=0)
   {
      if (show_int>1) show_int = 1;
      gd->show_int = show_int;
   }
   if (colr) xdl_image_cp2 (iv, gd->color_opt);
   if (setdata) xdl_image_setdata (gd);
   gd->reset = 0;
   if (gd->disp_now==1) xdl_image_refresh (gd, 1);
   return 0;
}

/* Fortran binding: xdlf_image_reset */

void xdlf_image_reset (ivh, min_thr, max_thr, mono_thr, icontrast,
                       icolor_opt, iovly_opt, ibg_opt, mag_opt,
                       ishow_int, ierr)
int *ivh, *min_thr, *max_thr, *mono_thr, *icontrast, *icolor_opt;
int *iovly_opt, *ibg_opt, *mag_opt, *ishow_int, *ierr;
{
   *ierr = xdl_image_reset (*ivh, *min_thr, *max_thr, *mono_thr, *icontrast,
                            *icolor_opt, *iovly_opt, *ibg_opt, *mag_opt,
                            *ishow_int);
   return;
}
/*-Routine: Set local axis names - xdl_image_axnames
The routine xdl_image_axnames (xdlf_image_axnames) is used to reset the
names of the local axes (the default starting values are 'ax1' and 'ax2'
for the two axes.
-end*/
/*
*****************************
**     xdl_image_axnames   ** 
*****************************

Purpose: Reset local axis names

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_AXNAMES (IVH, XDLSTR(NAME1), LEN1,
        +                         XDLSTR(NAME2), LEN2, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NAME1    (R)  Character string containing the name of the 1'st
              local axis (max 4 characters in length)
              ** Pass address using the XDLSTR function **
LEN1     (R)  Length of the name string - must be >0 
              (cf len1) (max 4 chars) 
NAME2    (R)  Character string containing the name of the 2'nd
              local axis (max 4 characters in length)
              ** Pass address using the XDLSTR function **
LEN2     (R)  Length of the name string - must be >0 
              (cf len1) (max 4 chars) 
IERR     (W)  Returns status from xdl_image_axnames call
-end*/

/*-C:*/
int xdl_image_axnames (vh, name1, len1, name2, len2)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *name1;     /* Name of 1'st local axis (max 4 chars long) (R)*/
int len1;        /* Length of name (may give 0 if a null terminated string
                    is passed) (R)*/
char *name2;     /* Name of 2'nd local axis (max 4 chars long) (R)*/
int len2;        /* Length of name (may give 0 if a null terminated string
                    is passed) (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int ll;                 /* String length */
   int l;                  /* Loop counter */
   int n;                  /* Significant length of string excluding trailing
                              blanks */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_axnames",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_axnames", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   ll = len1;
   if (ll==0) ll = strlen(name1);
   if (ll>4) ll=4;
   gd->ax1_name[0] = 0;
   if (ll>0)
   {
      n = 0;
      for (l=ll;l>=0;--l)
      {
         if (name1[l]!=' ') {n=l; break;}
      }
      if (n>0) strncat (gd->ax1_name,name1,n);
   }
   ll = len2;
   if (ll==0) ll = strlen(name2);
   if (ll>4) ll=4;
   gd->ax2_name[0] = 0;
   if (ll>0)
   {
      n = 0;
      for (l=ll;l>=0;--l)
      {
         if (name2[l]!=' ') {n=l; break;}
      }
      if (n>0) strncat (gd->ax2_name,name2,n);
   }
   return 0;
}

/* Fortran binding: xdlf_image_axnames */

void xdlf_image_axnames (ivh, name1, len1, name2, len2, ierr)
int *ivh, *name1, *len1, *name2, *len2, *ierr;
{
   *ierr = xdl_image_axnames (*ivh, XDL_pointers[*name1], *len1, 
                              XDL_pointers[*name2], *len2);
   return;
}
/*-Routine: Display the image now - xdl_image_display_now
The routine xdl_image_display_now (xdlf_image_display_now) is used to 
display the image. It will normally be used if non-default display
settings are required and disp_now (IDISP_NOW) was set to zero on
the initial call to xdl_image (xdlf_image)
-end*/
/*
********************************
**     xdl_image_display_now  ** 
********************************

Purpose: Display the image now

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_DISPLAY_NOW (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_image_display_now call
-end*/

/*-C:*/
int xdl_image_display_now (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_display_now",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_display_now", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->disp_now = 1;
   xdl_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_display_now */

void xdlf_image_display_now (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_image_display_now (*ivh);
   return;
}

/*-Routine: Set background image - xdl_image_background
The routine xdl_image_background (xdlf_image_background) is used to 
pass a background image to the image view-object. This has the same
data order as the original image but is compressed by a factor
(bg_cmp (IBG_CMP)) in the direction of both image axes. If bg_typ (IBG) was set 
to 1 when the xdl_image (xdlf_image) routine was called then the 
background options will be handled via the background choice menu on the control
panel; otherwise the background image will be ignored.
-end*/
/*
********************************
**     xdl_image_background   ** 
********************************

Purpose: Pass a background image to the image view-object

Author:  John W. Campbell, August 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_BACKGROUND (IVH, IBGDATA, IBG_CMP, NF_BGOFF)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IBGDATA  (R)  Array holding background image data (same format
              and size as for a compressed image as described
              in the call to the main xdlf_image routine.)
              (e.g. pass as in integer array with the data
              packed as needed)
              Note: a compressed background image is given
              here even if a full image was passed to the
              main routine.
              (bg_data)
IBG_CMP  (R)  Compression factor in background image (fast & slow
              axes ) 
NF_BGOFF (R)  Offset in pixels between stored rows of background
              image (to allow for storage in pre-dimensioned 2-D
              arrays.
IERR     (W)  Returns status from xdl_film_background call
-end*/

/*-C:*/
int xdl_image_background (vh, bg_data, bg_cmp, nf_bgoff)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char * bg_data;  /* Background image data; the size of the image is the
                    size of the compressed image passed to the 
                    xdl_image routine (or formed by it from the full 
                    image). The data format is the same as for the main 
                    image. (R)*/
int bg_cmp;      /* Compression factor in background image (fast & slow
                    axes (R)*/
int nf_bgoff;    /* Offset in pixels between stored rows of background
                    image (to allow for storage in pre-dimensioned 2-D
                    arrays (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_background",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_background", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->bg_typ==0) return 0;
   gd->bg_data = bg_data;
   gd->bg_cmp = bg_cmp;
   gd->nf_bgoff = nf_bgoff;
   gd->bg_present = 1;
   return 0;
}
/* Fortran binding: xdlf_image_background */

void xdlf_image_background (ivh, ibgdata, ibg_cmp, nf_bgoff, ierr)
int *ivh, *ibgdata, *ibg_cmp, *nf_bgoff, *ierr;
{
   *ierr = xdl_image_background (*ivh, (char *) ibgdata, *ibg_cmp, *nf_bgoff);
   return;
}

/*-Routine: Set a new image - xdl_image_newimg
The routine xdl_film_newimg (xdlf_image_newimg) is used to reset
the image in the display to a new image. This image must be of the same
type and have the same dimensions as the original image.
-end*/
/*
********************************
**     xdl_image_newimg       ** 
********************************

Purpose: Reset image with a new image of the same type and dimensions as
         the original image

Author:  John W. Campbell, March 1992

*/
/*-Fortran:
         CALL XDLF_IMAGE_NEWIMG (IVH, IMGDATA, IDISP_NOW, IERR)
-end*/
 
/*-Parameters:
IVH       (R)  View-object handle (see vh)
IMGDATA   (R)   Array holding compressed data (e.g. pass as an 
                integer array with data packed as needed)  
                (see imgdata)
IDISP_NOW (R)   Display flag =1 display immediately, =0 wait for 
                XDLF_IMAGE_DISPLAY_NOW call
IERR      (W)   Returns status from xdl_image_newimg call
-end*/

/*-C:*/
int xdl_image_newimg (vh, imgdata, disp_now)
/*end*/
 
/*-Parameters:*/
int vh;                   /* View-object handle (R)*/
char *imgdata;            /* The new image (see xdl_image call for
                             full details (R)*/
int disp_now;             /* Display now flag =1, =0 wait for 
                             xdl_image_display_now call (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Loop variable */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_newimg",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_newimg", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->imgdata = imgdata;
   gd->bg_present = 0;
   gd->bg_opt = 0;
   gd->bg_data = NULL;
   if (gd->bg_typ>0) xdl_panel_choice_reset (gd->bg_choice, 1);
   gd->magnify_on = 1;
   gd->freeze.on = 0;
   gd->use_image = 0;
   gd->ax1pix = -1;
   gd->ax2pix = -1;
   gd->ax1pix_dn = -1;
   gd->ax2pix_dn = -1;
   gd->ax1pix_up = -1;
   gd->ax2pix_up = -1;
   gd->ax1pix_img = -1;
   gd->ax2pix_img = -1;
   gd->ax1pix_img_up = -1;
   gd->ax2pix_img_up = -1;
   gd->sel_rect = 0;
   gd->sel_rect_xo = 0;
   gd->sel_rect_yo = 0;
   gd->sel_rect_x1 = 0;
   gd->sel_rect_y1 = 0;
   gd->sel_rect_x2 = 0;
   gd->sel_rect_y2 = 0;
   gd->zoom_rect = 0;
   if (gd->symbols!=NULL) free (gd->symbols);
   gd->symbols = NULL;
   gd->max_alloc_symbols = 0;
   gd->num_symb = 0;
   if (gd->vects!=NULL) free (gd->vects);
   gd->vects = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i)
      {
         if (gd->text[i].string!=NULL) free ((char *) gd->text[i].string);
      }
   }
   if (gd->text!=NULL) free ((char *) gd->text);
   gd->text = NULL;
   gd->max_alloc_text = 0;
   gd->num_text = 0;
   xdl_image_setdata (gd);
   gd->zoom_rect = 0;
   gd->zoom_on = 0;
   if (gd->zoom_sel) xdl_panel_button_set (gd->zoom_button, "Zoom", 4, 1, 1);
   gd->zoom_sel = 0;
   gd->disp_now = disp_now;
   if (gd->disp_now)
   {
      xdl_image_refresh (gd,1);
   }
   else
   {
      XClearWindow (XDL_display, gd->wdisp);
   }
   xdl_panel_label_set (gd->cursor_position," ",0,0);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_newimg */

void xdlf_image_newimg (ivh, imgdata, disp_now, ierr)
int *ivh, *imgdata, *disp_now, *ierr;
{
   *ierr = xdl_image_newimg (*ivh, (char *) imgdata, *disp_now);
   return;
}
/*-Routine: Reset overlay offset values - xdl_image_ovly_offset
The routine xdl_image_ovly_offset (xdlf_image_ovly_offset) is
used to reset the amount by which an overlay is offset when the overlay
offset option is selected via the image view-object control panel.
The default when the image view-object is created is four pixels
on the main image display area to the left and down (xoff=-4, yoff=4).
-end*/
/*
***********************************
**     xdl_image_ovly_offset     ** 
***********************************

Purpose: Reset overlay offset values (New values will be used the next time
         'offset' is selected from the Overlay menu.

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_OVLY_OFFSET (IVH, IXOFF, IYOFF, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IXOFF    (R)  Overlay x offset (see xoff)
IYOFF    (R)  Overlay y offset (see yoff)
IERR     (W)  Returns status from xdl_image_ovly_offset call
-end*/
/*-C:*/
int xdl_image_ovly_offset (vh, xoff, yoff)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int xoff;        /* New x overlay offset (in pixels of main film 
                    display) (R)*/
int yoff;        /* New y overlay offset (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;            /* Offset in XDL_view_objects array */
   XDl_image *gd;     /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_ovly_offset",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_ovly_offset", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   gd->x_ovly_offset = xoff;
   gd->y_ovly_offset = yoff;
   return 0;
}

/* Fortran binding: xdlf_image_ovly_offset */

void xdlf_image_ovly_offset (ivh, ixoff, iyoff, ierr)
int *ivh, *ixoff, *iyoff, *ierr;
{
   *ierr = xdl_image_ovly_offset (*ivh, *ixoff, *iyoff);
   return;
}

/*-Routine: Reset an overlay colour - xdl_image_set_colour
The routine xdl_image_set_colour (xdlf_image_set_colour) is
used to reset one of the 6 colours used for the overlay symbols to a
user selected colour or the colour used to indicate the position of
a 'frozen' magnifying window.
-end*/
/*
**********************************
**     xdl_image_set_colour     ** 
**********************************

Purpose: Reset an overlay colour (for symbols etc.)

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_SET_COLOUR (IVH, ICOLR, IRED, IGREEN, 
        +                                 IBLUE, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ICOLR    (R)  Overlay colour number (1 to 7) (see icolr)
IRED     (R)  Red component 0 to 65535 (see red)
IGREEN   (R)  Green component 0 to 65535 (see green)
IBLUE    (R)  Blue component 0 to 65535 (see blue)
IERR     (W)  Returns status from xdl_image_set_colour call
-end*/

/*-C:*/
int xdl_image_set_colour (vh, icolr, red, blue, green)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int icolr;       /* Overlay colour number 1 to 6 for symbols or 7 
                    for the magnifying window position indicating 
                    box (R)*/
int red;         /* Red component 0 to 65535 (R)*/
int green;       /* Green component 0 to 65535 (R)*/
int blue;        /* Blue component 0 to 65535 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   XColor colr_cell;       /* Temporary Xcolor structure */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_set_colour",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_set_colour", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   colr_cell.flags = DoRed|DoGreen|DoBlue;
   colr_cell.red = red;
   colr_cell.green = green;
   colr_cell.blue = blue;
   if (XDL_csets_alloc==1)
   {
      colr_cell.pixel = XDL_colorsets[gd->cset].pixels[63+icolr];
      XStoreColor (XDL_display, XDL_colormap, &colr_cell);
   }
   else
   {
      XAllocColor (XDL_display, XDL_colormap, &colr_cell);
   }
   gd->ovly_pixels[icolr-1] = colr_cell.pixel;
   if (XDL_csets_alloc==0) xdl_image_refresh (gd,1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_set_colour */

void xdlf_image_set_colour (ivh, icolr, ired, igreen, iblue, ierr)
int *ivh, *icolr, *ired, *igreen, *iblue, *ierr;
{
   *ierr = xdl_image_set_colour (*ivh, *icolr, *ired, *igreen, *iblue);
   return;
}

/*-Routine: Reset user defined colour map - xdl_image_set_colormap
The routine xdl_image_set_colormap (xdlf_image_set_colormap) is
used to reset the 'user defined' 64 colour colour map used to display the
image. The use of the map is selected via the colour choice menu on the
image view object. Note that the contrast slider will have no effect
when the user defined colour map is selected.
-end*/
/*
************************************
**     xdl_image_set_colormap     ** 
************************************

Purpose: Reset user defined colormap for image display (64 colours)

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_SET_COLORMAP (IVH, IRED, IGREEN, 
        +                                 IBLUE, IERR)
-end*/
 
/*-Parameters:
IVH          (R)  View-object handle (see vh)
IRED(64)     (R)  64 Red components 0 to 65535 (see red)
IGREEN(64)   (R)  64 Green components 0 to 65535 (see green)
IBLUE(64)    (R)  64 Blue components 0 to 65535 (see blue)
IERR         (W)  Returns status from xdl_image_set_colormap
                  call
-end*/

/*-C:*/
int xdl_image_set_colormap (vh, red, green, blue)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int red[];       /* 64 Red components 0 to 65535 (R)*/
int green[];     /* 64 Green components 0 to 65535 (R)*/
int blue[];      /* 64 Blue components 0 to 65535 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Temp/loop variable */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_set_colormap",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_set_colormap",
                             vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   for (i=0;i<64;++i)
   {
      if (red[i]<0) red[i] = 0;
      if (red[i]>65535) red[i] = 65535;
      if (green[i]<0) green[i] = 0;
      if (green[i]>65535) green[i] = 65535;
      if (blue[i]<0) blue[i] = 0;
      if (blue[i]>65535) blue[i] = 65535;
      gd->user_rgb[0][i] = red[i];
      gd->user_rgb[1][i] = green[i];
      gd->user_rgb[2][i] = blue[i];
   }
   if (gd->color_opt==6) xdl_image_cp2 (iv, 6);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_set_colormap */

void xdlf_image_set_colormap (ivh, ired, igreen, iblue, ierr)
int *ivh, *ired, *igreen, *iblue, *ierr;
{
   *ierr = xdl_image_set_colormap (*ivh, ired, igreen, iblue);
   return;
}

/*-Routine: Store and display an overlay symbol - xdl_image_symbol
The routine xdl_image_symbol (xdlf_image_symbol) is used to
store details of an overlay symbol and to display that symbol if
overlays are currently being displayed. Two sets of overlay symbols
may be stored. (If many symbols are to be added then, if possible,
use the routine xdl_image_symbols (xdlf_image_symbols) instead)
-end*/
/*
*****************************
**     xdl_image_symbol    ** 
*****************************

Purpose: Store and display a symbol

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_SYMBOL (IVH, IAX1PIX, IAX2PIX, ISYMB, 
        +                             ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IAX1PIX  (R)  Local axis 1 pixel position in full image 
              (see ax1_pixel)
IAX2PIX  (R)  Local axis 2 pixel position in full image 
              (see ax2_pixel)
ISYMB    (R)  Symbol type (see symbol)
ICOLR    (R)  Symbol colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
IERR     (W)  Returns status from xdl_image_symbol call
-end*/

/*-C:*/
int xdl_image_symbol (vh, ax1_pixel, ax2_pixel, symbol, color, ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int ax1_pixel;   /* The local axis 1 pixel position (from 1 up 
                    - refers to complete uncompressed image (R) */
int ax2_pixel;   /* The local axis 2 pixel position (R) */
int symbol;      /* The symbol type
                    = 1 to 10   vertical cross - increasing size 
                                                 1 to 19 pixels
                                                 in steps of 2.
                    = 11 to 20  cross - increasing size 1 to 19 pixels 
                                        height.
                    = 21 to 30  square - increasing size 1 to 19 pixels.
                                Note: 1, 11 and 21 are single points */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default)*/
int ov_num;      /* Overlay number 1 or 2 (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new symbol's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_symbol *p;      /* Temp. pointer to this structure */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int iposn;                /* Position (offset) in symbols list for new
                                symbol */
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   int xs_off;               /* Overlay symbol x offset */
   int ys_off;               /* Overlay symbol y offset */
   int x_pixel;              /* x pixel position wrt. X-windows axes */
   int y_pixel;              /* y pixel position wrt. X-windows axes */
   int y;                    /* y offset */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_symbol", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;
   if (gd->num_symb>=gd->max_alloc_symbols)
   {
      if (gd->symbols==NULL)
      {
         p = (XDl_image_symbol *) malloc (
           (gd->max_alloc_symbols+NSYMB_ALLOC) * sizeof(XDl_image_symbol));
         if (p==0) return 4;
      }
      else
      {
         p = (XDl_image_symbol *) realloc (gd->symbols,
           (gd->max_alloc_symbols+NSYMB_ALLOC) * sizeof(XDl_image_symbol));
         if (p==0) return 4;
      }
      gd->symbols = p;
      gd->max_alloc_symbols += NSYMB_ALLOC;
   }
   /* Get pixel position wrt. X-windows axes */

   xdl_image_local_to_xy (gd, ax1_pixel, ax2_pixel, &x_pixel, &y_pixel, 1);

   /* Find position in sorted list of symbols for the new entry */

   iposn = gd->num_symb;

   if (gd->num_symb>0)
   {
      for (i=0;i<gd->num_symb;++i)
      {
         if (y_pixel<gd->symbols[i].y_pixel) {iposn = i; break;}
         if (y_pixel==gd->symbols[i].y_pixel)
         {
            if (x_pixel<gd->symbols[i].x_pixel) {iposn = i; break;}
         }
      }
   }

   /* Shift symbols if required */

   if (iposn<gd->num_symb)
   {
      tp = (char *) gd->symbols;
      ssiz = sizeof (XDl_image_symbol);
      for (i=(gd->num_symb+1)*ssiz-1;i>=iposn*ssiz;--i) tp[i] = tp[i-ssiz];
   }

   gd->num_symb += 1;

   /* Save details for the new symbol */

   gd->symbols[iposn].x_pixel = x_pixel;
   gd->symbols[iposn].y_pixel = y_pixel;
   xdl_image_symbol_offsets (gd, iposn);
   gd->symbols[iposn].symb = symbol;
   gd->symbols[iposn].color_flags = color;
   if (ov_num==2) gd->symbols[iposn].color_flags |= 0X10;
   if (gd->ovly_typ==2||ov_num==2) gd->symbols[iposn].color_flags |= 0X20;
     
   /* Update y-search offsets array */

   y = gd->symbols[iposn].y_disp_offset;
   y_start = (y * (Y_SEARCH_MAX-1)) / gd->img_height;
   if (y_start<0) y_start = 0;
   if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
   if (y_start+1<Y_SEARCH_MAX)
   {
       for (i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i] += 1;
   }

   /* Draw symbol if required */

   if (gd->ovly1==0&&ov_num==1) return 0;
   if (gd->ovly2==0&&ov_num==2) return 0;
   xs_off = 0;
   ys_off = 0;
   if (gd->ovly_offset==1&&(gd->symbols[iposn].color_flags&0X20)==0)
   {
       xs_off = gd->x_ovly_offset;
       ys_off = gd->y_ovly_offset;
   }
   xdl_image_drawsymb (gd, gd->wdisp, 
                       gd->symbols[iposn].x_disp_offset + xs_off, 
                       gd->symbols[iposn].y_disp_offset + ys_off,
                       symbol, color);
   xdl_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_symbol */

void xdlf_image_symbol (ivh, iax1pix, iax2pix, isymb, icolr, iov, ierr)
int *ivh, *iax1pix, *iax2pix, *isymb, *icolr, *iov, *ierr;
{
   *ierr = xdl_image_symbol (*ivh, *iax1pix, *iax2pix, *isymb, *icolr, *iov);
   return;
}
/*-Routine: Store and display overlay symbols - xdl_image_symbols
The routine xdl_image_symbols (xdlf_image_symbols) is used to
store details of overlay symbols and to display those symbols if
overlays are currently being displayed. The symbols are added to an
internal symbols list and then sorted. Use this routine when possible
if may symbols are to be added rather than using individual calls to 
xdl_image_symbol (xdlf_image_symbol). Two sets of overlay 
symbols may be used.
-end*/
/*
******************************
**     xdl_image_symbols    ** 
******************************

Purpose: Store and display a list of symbols

Author:  John W. Campbell, July 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_SYMBOLS (IVH, NSYMB, IAX1PIX, IAX2PIX, ISYMB, 
        +                             ICOLR, IOV, NT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
NSYMB    (R)  No. of symbols to store (see nsymb)
IAX1PIX  (R)  Array of local axis 1 pixel positions in full 
              image (NSYMB values) (see ax1_pixel)
IAX2PIX  (R)  Array of local axis 2 pixel positions in full 
              image (NSYMB values) (see ax2_pixel)
ISYMB    (R)  Array of Symbol types or Symbol type as indicated
              by NT (see symbol)
ICOLR    (R)  Array of Symbol colours or Symbol colour as 
              indicated by NT (1 to 6) (see color)
IOV      (R)  Array of overlay numbers (1 or 2) or overlay
              number as indicated by NT (see ov_num)
NT       (R)  =1 give a single symbol type, colour and overlay 
                 number
              =0 give arrays of NSYMB symbols, colours and overlay
                 numbers (see nt)
IERR     (W)  Returns status from xdl_image_symbol call
-end*/

/*-C:*/
int xdl_image_symbols (vh, nsymb, ax1_pixel, ax2_pixel, symbol, color, 
                            ov_num, nt)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int nsymb;       /* The number of symbols to be added */
int ax1_pixel[]; /* Array of local axis 1 pixel positions (from 1 up 
                    - refers to complete uncompressed image) (R) */
int ax2_pixel[]; /* Array of local axis 2 pixel positions (R) */
int symbol[];    /* Array of symbol types (nsymb values or 1 value, see nt)
                    = 1 to 10   vertical cross - increasing size 
                                                 1 to 19 pixels
                                                 in steps of 2.
                    = 11 to 20  cross - increasing size 1 to 19 pixels 
                                        height.
                    = 21 to 30  square - increasing size 1 to 19 pixels.
                                Note: 1, 11 and 21 are single points (R) */
int color[];     /* Array of Overlay colour types (nsymb values or 1 value,
                    see nt)
                    1 to 6; Red, Yellow, Green, Cyan,Blue, Magenta 
                    (by default but may be reset) (R)*/
int ov_num[];    /* Array of overlay numbers (1 or 2) (nsymb values or 1
                    value, see nt) (R)*/
int nt;          /* =0 give nsymb values for symbol types and colours
                    =1 give a single value for the symbol type and colour 
                       and use this for all the symbols being added (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new symbol's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_symbol *p;      /* Temp. pointer to this structure */
   XDl_image_symbol *s;      /* Temp pointer to symbols */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int iposn;                /* Position (offset) in symbols list for new
                                symbol */
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int is;                   /* Loop counter through symbols */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   int y_start_prev;         /* Previous value of y_start */
   int j,l,it,ir;            /* Indices for sort */
   unsigned long tempyx;     /* Temporary saved value for sort */
   int nxt;                  /* Number of allocations for extending
                                symbol table memory */
   int y_max;                /* Set to at least max y pixel possible */
   int xs_off;               /* Overlay symbol x offset */
   int ys_off;               /* Overlay symbol y offset */
   int x_pixel;              /* x pixel position wrt. X-windows axes */
   int y_pixel;              /* y pixel position wrt. X-windows axes */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_symbol", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (nt!=0) nt=1;
   if (nsymb<=0) return 0;

   /* Get extra memory for new symbols if required */

   if (gd->num_symb+nsymb>gd->max_alloc_symbols)
   {
      nxt = (gd->num_symb+nsymb-gd->max_alloc_symbols-1)/NSYMB_ALLOC + 1;
      {
         if (gd->symbols==NULL)
         {
            p = (XDl_image_symbol *) malloc (
              (gd->max_alloc_symbols+nxt*NSYMB_ALLOC) * 
                   sizeof(XDl_image_symbol));
            if (p==0) return 4;
         }
         else
         {
            p = (XDl_image_symbol *) realloc (gd->symbols,
              (gd->max_alloc_symbols+nxt*NSYMB_ALLOC) * 
                   sizeof(XDl_image_symbol));
            if (p==0) return 4;
         }
         gd->symbols = p;
         gd->max_alloc_symbols += nxt*NSYMB_ALLOC;
      }
   }

   /* Add new symbols to the list */

   for(is=0;is<nsymb;++is)
   {
      iposn=gd->num_symb;
      gd->num_symb += 1;

      /* Save details for the new symbol */

      xdl_image_local_to_xy (gd, ax1_pixel[is], ax2_pixel[is],
          &x_pixel, &y_pixel, 1);
      gd->symbols[iposn].x_pixel = x_pixel;
      gd->symbols[iposn].y_pixel = y_pixel;
      xdl_image_symbol_offsets (gd, iposn);
      if (nt==0)
      {
         gd->symbols[iposn].symb = symbol[is];
         gd->symbols[iposn].color_flags = color[is];
         if (ov_num[is]==2) gd->symbols[iposn].color_flags |= 0X10;
         if (gd->ovly_typ==2||ov_num[is]==2) 
                 gd->symbols[iposn].color_flags |= 0X20;
      }
      else
      {
         gd->symbols[iposn].symb = symbol[0];
         gd->symbols[iposn].color_flags = color[0];
         if (ov_num[0]==2) gd->symbols[iposn].color_flags |= 0X10;
         if (gd->ovly_typ==2||ov_num[0]==2) 
                 gd->symbols[iposn].color_flags |= 0X20;
      }
      if (gd->ovly1==0&&(gd->symbols[iposn].color_flags&0X10)==0) continue;
      if (gd->ovly2==0&&(gd->symbols[iposn].color_flags&0X10)!=0) continue;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(gd->symbols[iposn].color_flags&0X20)==0)
      {
         xs_off = gd->x_ovly_offset;
         ys_off = gd->y_ovly_offset;
      }
      xdl_image_drawsymb (gd, gd->wdisp, 
                          gd->symbols[iposn].x_disp_offset + xs_off,
                          gd->symbols[iposn].y_disp_offset + ys_off,
                          gd->symbols[iposn].symb, 
                          (gd->symbols[iposn].color_flags&0XF));
   }

   /* Sort symbols list */

   s=gd->symbols;
   y_max=(gd->img_height+1)*(gd->cur_cmp);
   for(i=0;i<gd->num_symb;++i)
   {
      s[i].indx = i;
      s[i].temp = ((s[i].y_pixel)<<16)+s[i].x_pixel;
   }
   l=gd->num_symb/2;
   ir=gd->num_symb-1;
   for (;;)
   {
      if(l>0){
         l--;
         it = s[l].indx;
         tempyx = s[it].temp;
      }
      else
      {
         it = s[ir].indx;
         tempyx = s[it].temp;
         s[ir].indx = s[0].indx;
         ir--;
         if (ir==0){
            s[0].indx =it;
            break;
         }
      }
      i = l;
      j = 2*l+ 1;
      while (j<=ir)
      {
         if (j<ir)
         {
            if (s[s[j].indx].temp<s[s[j+1].indx].temp)
            {
               j++;
            }
            else if (s[s[j].indx].temp==s[s[j+1].indx].temp &&
                     s[j].indx<s[j+1].indx)
            {
               j++;
            }
         }
         if (tempyx<s[s[j].indx].temp)
         {
            s[i].indx=s[j].indx;
            i = j;
            j = j + i + 1;
         }
         else if (tempyx==s[s[j].indx].temp&&it<s[j].indx)
         {
            s[i].indx =s[j].indx;
            i = j;
            j = j + i + 1;
         }
         else
         {
            j = ir + 1;
         }
      }
      s[i].indx = it;
   }

   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].x_pixel;
   for(i=0;i<gd->num_symb;++i)s[i].x_pixel=(unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].y_pixel;
   for(i=0;i<gd->num_symb;++i)s[i].y_pixel=(unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].x_disp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].x_disp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].y_disp_offset;
   for(i=0;i<gd->num_symb;++i)s[i].y_disp_offset=
                              (unsigned short)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].symb;
   for(i=0;i<gd->num_symb;++i)s[i].symb=(unsigned char)s[s[i].indx].temp;
   for(i=0;i<gd->num_symb;++i)s[i].temp=(unsigned long)s[i].color_flags;
   for(i=0;i<gd->num_symb;++i)s[i].color_flags=(unsigned char)s[s[i].indx].temp;

   /* Update y-search offsets array */

   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i]=0;
   y_start_prev=0;
   for (is=0;is<gd->num_symb;++is)
   {
      y_start = (gd->symbols[is].y_disp_offset * 
                (Y_SEARCH_MAX-1)) / gd->img_height;
      if (y_start<0) y_start = 0;
      if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
      if (y_start>y_start_prev)
      {
         if (y_start-1<Y_SEARCH_MAX)
         {
            for(i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i]=is;
         }
      }
      y_start_prev=y_start;
   }

   /* Refresh magnifying window if required */

   if (gd->ovly1==0&&gd->ovly2==0) return 0;
   xdl_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_symbols */

void xdlf_image_symbols (ivh, nsymb, iax1pix, iax2pix, isymb, icolr, 
                             ov_num, nt, ierr)
int *ivh, *nsymb, *iax1pix, *iax2pix, *isymb, *icolr, *ov_num, *nt, *ierr;
{
   *ierr = xdl_image_symbols (*ivh, *nsymb, iax1pix, iax2pix, isymb, icolr,
                                  ov_num, *nt);
   return;
}

/*-Routine: Clear overlay symbols - xdl_image_clear_symbols
The routine xdl_image_clear_symbols (xdlf_image_clear_symbols) is
used to clear all overlay symbols from the symbols list and from the
display.
-end*/
/*
*************************************
**     xdl_image_clear_symbols     ** 
*************************************

Purpose: Clear table of symbols and clear symbols from screen

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_CLEAR_SYMBOLS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_image_clear_symbols 
              call
-end*/

/*-C:*/
int xdl_image_clear_symbols (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Loop counter */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_clear_symbols",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_clear_symbols", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->symbols!=NULL) free (gd->symbols);
   gd->symbols = NULL;
   gd->max_alloc_symbols = 0;
   gd->num_symb = 0;
   for (i=0;i<Y_SEARCH_MAX;++i) gd->y_search[i] = 0;
   xdl_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_clear_symbols */

void xdlf_image_clear_symbols (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_image_clear_symbols (*ivh);
   return;
}

/*-Routine: Delete an overlay symbol - xdl_image_del_symbol
The routine xdl_image_del_symbol (xdlf_image_del_symbol) is used
to delete an overlay symbol from the symbols list and remove it from the
display. If there are many symbols to delete it may be quicker to clear
all symbols (xdl_image_clear_symbols (xdlf_image_clear_symbols))
and create a new list using xdl_image_symbols (xdlf_image_symbols)
-end*/
/*
**********************************
**     xdl_image_del_symbol     ** 
**********************************

Purpose: Delete a symbol

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_DEL_SYMBOL (IVH, IAX1PIX, IAX2PIX, ISYMB,
        +                                 ICOLR, IOV, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IAX1PIX  (R)  Local axis 1 pixel position of symbol (full 
              image pixel numbering) (see ax1_pixel)
IAX2PIX  (R)  Local axis 2 pixel position (see ax2_pixel)
ISYMB    (R)  If > 0, only delete the symbol if it is of the 
              type ISYMB (see symbol)
ICOLR    (R)  If > 0, only delete the symbol if it is of the 
              colour ICOLR (see color)
IOV      (R)  If > 0, only delete the symbol if it is of the
              overlay number IOV (see ov_num)
IERR     (W)  Returns status from xdl_image_del_symbol call
-end*/

/*-C:*/
int xdl_image_del_symbol (vh, ax1_pixel, ax2_pixel, symbol, color, 
                               ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int ax1_pixel;   /* Local axis 1 pixel position of the symbol (from 1 up
                    - refers to complete uncompressed image) (R)*/
int ax2_pixel;   /* Local axis 2 pixel position of the symbol (from 1 up 
                    - refers to complete uncompressed image) (R)*/
int symbol;      /* If > 0 then only delete the symbol if it is of the
                    type 'symbol' (see xdl_image_symbol routine for
                    details of symbol types) (R)*/
int color;       /* If > 0 then only delete the symbol if it is of the 
                    colour 'color' (see xdl_image_symbol routine 
                    for details of the colour types) (R)*/
int ov_num;      /* If > 0 then only delete the symbol if it is of
                    the overlay number ov_num (1 or 2) (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Symbol not found
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   char *tp;                 /* Temp character pointer, used in copying symbols 
                                */
   int iposn;                /* Position (offset) in symbols list for
                                symbol to be deleted*/
   int i;                    /* Loop counter */
   int ssiz;                 /* Size of symbols structure */
   int y_start;              /* Y offset 0 to Y_SEARCH_MAX-1 */
   XDl_image_symbol *smb;      /* Temp. pointer to symbol structure */
   int x_d, y_d;                    /* x, y pixel offsets of symbol in main
                                       film display area */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted symbol area (also vectors)*/
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted symbol area (also vectors) */
   int s_start;                     /* Start position in symbols list for search
                                       */
   int match;                       /* Valid match found flag =1 yes, =0 no */
   int ovn;                         /* Overlay number */
   int xs_off;                      /* Overlay symbol x offset */
   int ys_off;                      /* Overlay symbol y offset */
   int x_pixel;                     /* x pixel position wrt. X-windows axes */
   int y_pixel;                     /* y pixel position wrt. X-windows axes */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_del_symbol",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_del_symbol", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->num_symb==0) return 4;

   xdl_image_local_to_xy (gd, ax1_pixel, ax2_pixel, &x_pixel, &y_pixel, 1);

   for (iposn=0;iposn<gd->num_symb;++iposn)
   {
      smb = &(gd->symbols[iposn]);
      match = 0;
      if (x_pixel==smb->x_pixel&&y_pixel==smb->y_pixel) match = 1;
      if (color>0&&color!=(smb->color_flags&0XF)) match = 0;
      if (symbol>0&&symbol!=smb->symb) match = 0;
      ovn = 1; if ((smb->color_flags&0X10)!=0) ovn=2;
      if (ov_num>0&&ov_num!=ovn) match = 0;
      if (match)
      {
         xs_min = smb->x_disp_offset - MAX_SYMB_SIZE;
         xs_max = smb->x_disp_offset + MAX_SYMB_SIZE;
         ys_min = smb->y_disp_offset - MAX_SYMB_SIZE - abs(gd->y_ovly_offset);
         ys_max = smb->y_disp_offset + MAX_SYMB_SIZE + abs(gd->y_ovly_offset);
         y_start = (smb->y_disp_offset * (Y_SEARCH_MAX-1)) / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start>=Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         if (y_start+1<Y_SEARCH_MAX)
         {
            for (i=y_start+1;i<Y_SEARCH_MAX;++i) gd->y_search[i] -= 1;
         }
         if (iposn<gd->num_symb-1)
         {
            tp = (char *) gd->symbols;
            ssiz = sizeof (XDl_image_symbol);
            for (i=iposn*ssiz;i<(gd->num_symb*ssiz-ssiz);++i)
                 tp[i] = tp[i+ssiz];
         }
         gd->num_symb -= 1;
         if (xs_min<0) xs_min = 0;
         if (ys_min<0) ys_min = 0;
         if (xs_max>gd->img_width - 1) xs_max = gd->img_width - 1;
         if (ys_max>gd->img_height - 1) ys_max = gd->img_height - 1;
         XPutImage (XDL_display, gd->wdisp, gd->paint_gc,
                    gd->img_image, xs_min, ys_min, xs_min, ys_min,
                    (xs_max - xs_min + 1), (ys_max - ys_min + 1));
         if (gd->magnify_on==0) xdl_image_set_freeze (gd);
         if (gd->ovly1==0&&gd->ovly2==0) return 0;

         xdl_image_rdv (gd, xs_min - gd->cur_expand - 1, 
                        xs_max + gd->cur_expand + 1, 
                        ys_min - gd->cur_expand - 1, 
                        ys_max + gd->cur_expand + 1, 0, 0, 0);
         xdl_image_rdt (gd, ys_min, ys_max+gd->ch_height);

         xs_min -= MAX_SYMB_SIZE;
         xs_max += MAX_SYMB_SIZE;
         ys_min -= MAX_SYMB_SIZE;
         ys_max += MAX_SYMB_SIZE;

         xdl_image_magw (gd);

         if (gd->num_symb==0) return 0;
         y_start = ((ys_min)*(Y_SEARCH_MAX-1)) 
                   / gd->img_height;
         if (y_start<0) y_start = 0;
         if (y_start >= Y_SEARCH_MAX) y_start = Y_SEARCH_MAX - 1;
         s_start = gd->y_search[y_start];
         if (s_start<gd->num_symb)
         {
            for (i=s_start;i<gd->num_symb;++i)
            {
               smb = &(gd->symbols[i]);
               if (gd->ovly1==0&&(smb->color_flags&0X10)==0) continue;
               if (gd->ovly2==0&&(smb->color_flags&0X10)!=0) continue;
               xs_off = 0;
               ys_off = 0;
               if (gd->ovly_offset==1&&(smb->color_flags&0X20)==0)
               {
                  xs_off = gd->x_ovly_offset;
                  ys_off = gd->y_ovly_offset;
               }
               x_d = smb->x_disp_offset + xs_off;
               y_d = smb->y_disp_offset + ys_off;
               if (x_d<xs_min) continue;
               if (x_d>xs_max) continue;
               if (y_d<ys_min) continue;
               if (y_d>ys_max) break;
               xdl_image_drawsymb (gd, gd->wdisp, x_d, y_d,
                                        smb->symb, (smb->color_flags&0XF));
            }
         }
         XFlush (XDL_display);
         return 0;
      }
   }
   XFlush (XDL_display);
   return 4;
}

/* Fortran binding: xdlf_image_del_symbol */

void xdlf_image_del_symbol (ivh, iax1pix, iax2pix, isymb, icolr, iov, ierr)
int *ivh, *iax1pix, *iax2pix, *isymb, *icolr, *iov, *ierr;
{
   *ierr = xdl_image_del_symbol (*ivh, *iax1pix, *iax2pix, *isymb, *icolr,
                                      *iov);
   return;
}
/*-Routine: Store and display an overlay vector - xdl_image_vect
The routine xdl_image_vect (xdlf_image_vect) is used to
store details of an overlay vector and to display that vector if
overlays are currently being displayed. Two sets of overlay vectors
may be stored. The user supplies a vector identifier number for a
vector and this is used to identify a vector or set of vectors for 
deletion if required.
-end*/
/*
*****************************
**     xdl_image_vect      ** 
*****************************

Purpose: Store and display an overlay vector

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_VECT (IVH, IVECT, IAX1PIX1, IAX2PIX1, 
        +                           IAX1PIX2, IAX2PIX2,
        +                           ICOLR, IOV, MAGUPD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  User selected number to identify vector
              (0-65535). The same number may be used
              for more than one vector to identify a
              set of vectors (see vect_id)
IAX1PIX1 (R)  Local axis 1 start pixel position in full image 
              (see ax1_1_pixel)
IAX2PIX1 (R)  Local axis 2 start pixel position in full image 
              (see ax2_1_pixel)
IAX1PIX2 (R)  Local axis 1 end pixel position in full image 
              (see ax2_2_pixel)
IAX2PIX2 (R)  Local axis 2 end pixel position in full image 
              (see ax2_2_pixel)
ICOLR    (R)  Vector colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
MAGUPD   (R)  =1 Update magnifying window and flush X buffer
                 after drawing vector,
              =0 do not (see magw_upd)
IERR     (W)  Returns status from xdl_image_vect call
-end*/

/*-C:*/
int xdl_image_vect (vh, vect_id, ax1_1_pixel, ax2_1_pixel, 
                         ax1_2_pixel, ax2_2_pixel, color, ov_num, magw_upd)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int vect_id;     /* User selected number to identify vector (0-65535). 
                    The same number may be used for more than one vector to 
                    identify a set of vectors (R)*/
int ax1_1_pixel; /* The vector start local axis 1 pixel position (from 1 
                    up - refers to complete uncompressed image (R) */
int ax2_1_pixel; /* The vector start local axis 2 pixel position (R) */
int ax1_2_pixel; /* The vector end local axis 1 pixel position (R) */
int ax2_2_pixel; /* The vector end local axis 2 pixel position (R) */
int color;       /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                    Blue, Magenta (by default)*/
int ov_num;      /* Overlay number 1 or 2 (R)*/
int magw_upd;    /* =1 Update the magnifying window and flush X buffer
                       after drawing the  vector; this should be used 
                       when a single vector is drawn or when the final 
                       vector of a series of vectors is drawn.
                    =0 Do not; this enables a series of vectors to be
                       drawn much more rapidly; it should be used when
                       a series of vectors is to be drawn for all but
                       the final vector (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new vector's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_vect *p;        /* Temp. pointer to this structure */
   int iposn;                /* Position (offset) in vectors list for new
                                vector */
   int xs_off;               /* Overlay vector x offset */
   int ys_off;               /* Overlay vector y offset */
   int x1_pixel;             /* x1 pixel wrt X-windows axes */
   int y1_pixel;             /* y1 pixel wrt X-windows axes */
   int x2_pixel;             /* x2 pixel wrt X-windows axes */
   int y2_pixel;             /* y2 pixel wrt X-windows axes */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_vect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_vect", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;
   if (gd->num_vect>=gd->max_alloc_vects)
   {
      if (gd->vects==NULL)
      {
         p = (XDl_image_vect *) malloc (
           (gd->max_alloc_vects+NVECT_ALLOC) * sizeof(XDl_image_vect));
         if (p==0) return 4;
      }
      else
      {
         p = (XDl_image_vect *) realloc (gd->vects,
           (gd->max_alloc_vects+NVECT_ALLOC) * sizeof(XDl_image_vect));
         if (p==0) return 4;
      }
      gd->vects = p;
      gd->max_alloc_vects += NVECT_ALLOC;
   }

   iposn = gd->num_vect;
   gd->num_vect += 1;

   /* Save details for the new vector */

   xdl_image_local_to_xy(gd, ax1_1_pixel, ax2_1_pixel, &x1_pixel, &y1_pixel,1);
   xdl_image_local_to_xy(gd, ax1_2_pixel, ax2_2_pixel, &x2_pixel, &y2_pixel,1);
   gd->vects[iposn].vect_id = vect_id;
   gd->vects[iposn].x1_pixel = x1_pixel;
   gd->vects[iposn].y1_pixel = y1_pixel;
   gd->vects[iposn].x2_pixel = x2_pixel;
   gd->vects[iposn].y2_pixel = y2_pixel;
   gd->vects[iposn].x1_offset = 0;
   gd->vects[iposn].y1_offset = 0;
   gd->vects[iposn].x2_offset = 0;
   gd->vects[iposn].y2_offset = 0;
   xdl_image_vector_offsets (gd, iposn);
   gd->vects[iposn].color_flags = color;
   if (ov_num==2) gd->vects[iposn].color_flags |= 0X10;
   if (gd->ovly_typ==2||ov_num==2) gd->vects[iposn].color_flags |= 0X20;
     
   /* Draw vector if required */

   if (gd->ovly1==0&&ov_num==1) return 0;
   if (gd->ovly2==0&&ov_num==2) return 0;
   xs_off = 0;
   ys_off = 0;
   if (gd->ovly_offset==1&&(gd->vects[iposn].color_flags&0X20)==0)
   {
       xs_off = gd->x_ovly_offset;
       ys_off = gd->y_ovly_offset;
   }
   xdl_image_drawvec (gd, gd->wdisp, gd->cur_expand,
                      gd->vects[iposn].x1_disp_offset + xs_off, 
                      gd->vects[iposn].y1_disp_offset + ys_off,
                      gd->vects[iposn].x2_disp_offset + xs_off, 
                      gd->vects[iposn].y2_disp_offset + ys_off,
                      gd->vects[iposn].x1_offset,
                      gd->vects[iposn].y1_offset,
                      gd->vects[iposn].x2_offset,
                      gd->vects[iposn].y2_offset,
                      color);
   if (magw_upd)
   {
      xdl_image_magw (gd);
      XFlush (XDL_display);
   }
   return 0;
}

/* Fortran binding: xdlf_image_vect */

void xdlf_image_vect (ivh, ivect, iax1pix1, iax2pix1, iax1pix2, iax2pix2, 
                           icolr, iov, magupd, ierr)
int *ivh, *ivect, *iax1pix1, *iax2pix1, *iax1pix2, *iax2pix2, 
    *icolr, *iov, *magupd;
int *ierr;
{
   *ierr = xdl_image_vect (*ivh, *ivect, *iax1pix1, *iax2pix1, 
                                *iax1pix2, *iax2pix2, *icolr, *iov, *magupd);
   return;
}
/*-Routine: Store and display overlay vectors - xdl_image_vects
The routine xdl_image_vects (xdlf_image_vects) is used to
store details of a set of overlay vectors and to display those vectors if
overlays are currently being displayed. Two sets of overlay vectors
may be stored. The user supplies a vector identifier set number for the
vectors and this is used to identify the set of vectors for 
deletion if required.
-end*/
/*
*****************************
**     xdl_image_vects     ** 
*****************************

Purpose: Store and display a set of overlay vectors

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_VECTS (IVH, IVECT, NUM_VECT, IAX1PIX1, IAX2PIX1, 
        +                           IAX1PIX2, IAX2PIX2,
        +                           ICOLR, IOV, MAGUPD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  User selected number to identify vector set
              (0-65535). (see vect_id)
NUM_VECT (R)  The number of vectors
IAX1PIX1 (R)  Local axis 1 start pixel positions in full image 
              (see ax1_1_pixel) (array of NUM_VECT values)
IAX2PIX1 (R)  Local axis 2 start pixel positions in full image 
              (see ax2_1_pixel) (array of NUM_VECT values)
IAX1PIX2 (R)  Local axis 1 end pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_VECT values)
IAX2PIX2 (R)  Local axis 2 end pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_VECT values)
ICOLR    (R)  Vector colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
MAGUPD   (R)  =1 Update magnifying window after drawing vector,
              =0 do not (see magw_upd)
IERR     (W)  Returns status from xdl_image_vects call
-end*/

/*-C:*/
int xdl_image_vects (vh, vect_id, num_vect, ax1_1_pixel, ax2_1_pixel, 
                         ax1_2_pixel, ax2_2_pixel, color, ov_num, magw_upd)
/*end*/
 
/*-Parameters:*/
int vh;           /* View-object handle (R)*/
int vect_id;      /* User selected number to identify vector (0-65535). 
                     The same number may be used for more than one vector to 
                     identify a set of vectors (R)*/
int num_vect;     /* Number of vectors to be added */
int *ax1_1_pixel; /* The vector start local axis 1 pixel positions (from 1 
                     up - refers to complete uncompressed image 
                     (array of num_vect values (R) */
int *ax2_1_pixel; /* The vector start local axis 2 pixel positions
                     (array of num_vect values  (R) */
int *ax1_2_pixel; /* The vector end local axis 1 pixel positions
                     (array of num_vect values  (R) */
int *ax2_2_pixel; /* The vector end local axis 2 pixel positions
                     (array of num_vect values  (R) */
int color;        /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                     Blue, Magenta (by default)*/
int ov_num;       /* Overlay number 1 or 2 (R)*/
int magw_upd;     /* =1 Update the magnifying window after drawing the 
                        vectors; this should be used when all the
                        required vectors have been defined.
                     =0 Do not (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new vector's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_vect *p;        /* Temp. pointer to this structure */
   int iposn;                /* Position (offset) in vectors list for new
                                vector */
   int xs_off;               /* Overlay vector x offset */
   int ys_off;               /* Overlay vector y offset */
   int x1_pixel;             /* x1 pixel wrt X-windows axes */
   int y1_pixel;             /* y1 pixel wrt X-windows axes */
   int x2_pixel;             /* x2 pixel wrt X-windows axes */
   int y2_pixel;             /* y2 pixel wrt X-windows axes */
   int ivec;                 /* Vectors loop variable */
   int nxt;                  /* Number of extra allocations of vectors
                                required */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_vects",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_vects", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;

   if (gd->num_vect+num_vect>gd->max_alloc_vects)
   {
      nxt = (gd->num_vect+num_vect-gd->max_alloc_vects-1)/NVECT_ALLOC + 1;
      {
         if (gd->vects==NULL)
         {
            p = (XDl_image_vect *) malloc (
              (gd->max_alloc_vects+nxt*NVECT_ALLOC) * 
                   sizeof(XDl_image_vect));
            if (p==0) return 4;
         }
         else
         {
            p = (XDl_image_vect *) realloc (gd->vects,
              (gd->max_alloc_vects+nxt*NVECT_ALLOC) * 
                   sizeof(XDl_image_vect));
            if (p==0) return 4;
         }
         gd->vects = p;
         gd->max_alloc_vects += nxt*NVECT_ALLOC;
      }
   }

   for (ivec=0;ivec<num_vect;++ivec)
   {
      iposn = gd->num_vect;
      gd->num_vect += 1;

      /* Save details for the new vector */

      xdl_image_local_to_xy(gd, ax1_1_pixel[ivec], ax2_1_pixel[ivec],
                            &x1_pixel, &y1_pixel,1);
      xdl_image_local_to_xy(gd, ax1_2_pixel[ivec], ax2_2_pixel[ivec],
                            &x2_pixel, &y2_pixel,1);
      gd->vects[iposn].vect_id = vect_id;
      gd->vects[iposn].x1_pixel = x1_pixel;
      gd->vects[iposn].y1_pixel = y1_pixel;
      gd->vects[iposn].x2_pixel = x2_pixel;
      gd->vects[iposn].y2_pixel = y2_pixel;
      gd->vects[iposn].x1_offset = 0;
      gd->vects[iposn].y1_offset = 0;
      gd->vects[iposn].x2_offset = 0;
      gd->vects[iposn].y2_offset = 0;
      xdl_image_vector_offsets (gd, iposn);
      gd->vects[iposn].color_flags = color;
      if (ov_num==2) gd->vects[iposn].color_flags |= 0X10;
      if (gd->ovly_typ==2||ov_num==2) gd->vects[iposn].color_flags |= 0X20;
     
      /* Draw vector if required */
   
      if (gd->ovly1==0&&ov_num==1) return 0;
      if (gd->ovly2==0&&ov_num==2) return 0;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(gd->vects[iposn].color_flags&0X20)==0)
      {
          xs_off = gd->x_ovly_offset;
          ys_off = gd->y_ovly_offset;
      }
      xdl_image_drawvec (gd, gd->wdisp, gd->cur_expand, 
                         gd->vects[iposn].x1_disp_offset + xs_off, 
                         gd->vects[iposn].y1_disp_offset + ys_off,
                         gd->vects[iposn].x2_disp_offset + xs_off, 
                         gd->vects[iposn].y2_disp_offset + ys_off,
                         gd->vects[iposn].x1_offset,
                         gd->vects[iposn].y1_offset,
                         gd->vects[iposn].x2_offset,
                         gd->vects[iposn].y2_offset,
                         color);
   }
   if (magw_upd) xdl_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_vects */

void xdlf_image_vects (ivh, ivect, num_vect, 
                      iax1pix1, iax2pix1, iax1pix2, iax2pix2, 
                      icolr, iov, magupd, ierr)
int *ivh, *ivect, *num_vect, *iax1pix1, *iax2pix1, *iax1pix2, *iax2pix2, 
    *icolr, *iov, *magupd;
int *ierr;
{
   *ierr = xdl_image_vects (*ivh, *ivect, *num_vect, iax1pix1, iax2pix1, 
                                iax1pix2, iax2pix2, *icolr, *iov, *magupd);
   return;
}
/*-Routine: Store and display overlay vectors - xdl_image_vects_pos
The routine xdl_image_vects_pos (xdlf_image_vects_pos) is used to
store details of a set of overlay vectors and to display those vectors if
overlays are currently being displayed. Two sets of overlay vectors
may be stored. The user supplies a vector identifier set number for the
vectors and this is used to identify the set of vectors for 
deletion if required. In contrast to xdl_image_vects (xdlf_image_vects) which
always positions the vector ends at the centre of a displayed pixel, this 
routine allows the user some further options for positioning the vector
ends with respect to the displayed pixels.
-end*/
/*
*********************************
**     xdl_image_vects_pos     ** 
*********************************

Purpose: Store and display a set of overlay vectors

Author:  John W. Campbell, July 1995

*/
/*-Fortran:
         CALL XDLF_IMAGE_VECTS_POS (IVH, IVECT, NUM_VECT, 
        +                           IAX1PIX1, IAX2PIX1, 
        +                           IAX1PIX2, IAX2PIX2,
        +                           IAX1POS1, IAX2POS1, 
        +                           IAX1POS2, IAX2POS2,
        +                           ICOLR, IOV, MAGUPD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  User selected number to identify vector set
              (0-65535). (see vect_id)
NUM_VECT (R)  The number of vectors
IAX1PIX1 (R)  Local axis 1 start pixel positions in full image 
              (see ax1_1_pixel) (array of NUM_VECT values)
IAX2PIX1 (R)  Local axis 2 start pixel positions in full image 
              (see ax2_1_pixel) (array of NUM_VECT values)
IAX1PIX2 (R)  Local axis 1 end pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_VECT values)
IAX2PIX2 (R)  Local axis 2 end pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_VECT values)
IAX1POS1 (R)  Position option flags for local axis 1 start pixels
              (array of NUM_VECT values)
IAX2POS1 (R)  Position option flags for local axis 2 start pixels
              (array of NUM_VECT values)
IAX1POS2 (R)  Position option flags for local axis 1 end pixels
              (array of NUM_VECT values)
IAX2POS2 (R)  Position option flags for local axis 2 end pixels
              (array of NUM_VECT values)
ICOLR    (R)  Vector colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
MAGUPD   (R)  =1 Update magnifying window after drawing vector,
              =0 do not (see magw_upd)
IERR     (W)  Returns status from xdl_image_vects_pos call

              Note: Position flags   = 0   Centre of displayed pixel
                                     =-1   Low side of displayed pixel
                                     =+1   High side of displayed pixel
                                     =-2   Just outside low side
                                     =+2   Just outside high side
-end*/

/*-C:*/
int xdl_image_vects_pos (vh, vect_id, num_vect, ax1_1_pixel, ax2_1_pixel, 
                       ax1_2_pixel, ax2_2_pixel, ax1_1_pos, ax2_1_pos,
                       ax1_2_pos, ax2_2_pos, color, ov_num, magw_upd)
/*end*/
 
/*-Parameters:*/
int vh;           /* View-object handle (R)*/
int vect_id;      /* User selected number to identify vector (0-65535). 
                     The same number may be used for more than one vector to 
                     identify a set of vectors (R)*/
int num_vect;     /* Number of vectors to be added */
int *ax1_1_pixel; /* The vector start local axis 1 pixel positions (from 1 
                     up - refers to complete uncompressed image 
                     (array of num_vect values (R) */
int *ax2_1_pixel; /* The vector start local axis 2 pixel positions
                     (array of num_vect values  (R) */
int *ax1_2_pixel; /* The vector end local axis 1 pixel positions
                     (array of num_vect values  (R) */
int *ax2_2_pixel; /* The vector end local axis 2 pixel positions
                     (array of num_vect values  (R) */
int *ax1_1_pos;   /* Position option flags for local axis 1 start pixels
                     (array of num_vect values  (R) */
int *ax2_1_pos;   /* Position option flags for local axis 2 start pixels
                     (array of num_vect values  (R) */
int *ax1_2_pos;   /* Position option flags for local axis 1 end pixels
                     (array of num_vect values  (R) */
int *ax2_2_pos;   /* Position option flags for local axis 2 end pixels
                     (array of num_vect values  (R) */
int color;        /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                     Blue, Magenta (by default)*/
int ov_num;       /* Overlay number 1 or 2 (R)*/
int magw_upd;     /* =1 Update the magnifying window after drawing the 
                        vectors; this should be used when all the
                        required vectors have been defined.
                     =0 Do not (R) */

         /*   Note: Position flags   = 0   Centre of displayed pixel
                                     =-1   Low side of displayed pixel
                                     =+1   High side of displayed pixel
                                     =-2   Just outside low side
                                     =+2   Just outside high side  */
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new vector's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_vect *p;        /* Temp. pointer to this structure */
   int iposn;                /* Position (offset) in vectors list for new
                                vector */
   int xs_off;               /* Overlay vector x offset */
   int ys_off;               /* Overlay vector y offset */
   int x1_pixel;             /* x1 pixel wrt X-windows axes */
   int y1_pixel;             /* y1 pixel wrt X-windows axes */
   int x2_pixel;             /* x2 pixel wrt X-windows axes */
   int y2_pixel;             /* y2 pixel wrt X-windows axes */
   int ax1_1_off;            /* Local axis 1 start offset flag */
   int ax2_1_off;            /* Local axis 2 start offset flag */
   int ax1_2_off;            /* Local axis 1 end offset flag */
   int ax2_2_off;            /* offset flag */
   int x1_off;               /* x1 offset flag */
   int y1_off;               /* y1 offset flag */
   int x2_off;               /* x2 offset flag */
   int y2_off;               /* y2 offset flag */
   int ivec;                 /* Vectors loop variable */
   int nxt;                  /* Number of extra allocations of vectors
                                required */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_vects_pos",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_vects_pos", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;

   if (gd->num_vect+num_vect>gd->max_alloc_vects)
   {
      nxt = (gd->num_vect+num_vect-gd->max_alloc_vects-1)/NVECT_ALLOC + 1;
      {
         if (gd->vects==NULL)
         {
            p = (XDl_image_vect *) malloc (
              (gd->max_alloc_vects+nxt*NVECT_ALLOC) * 
                   sizeof(XDl_image_vect));
            if (p==0) return 4;
         }
         else
         {
            p = (XDl_image_vect *) realloc (gd->vects,
              (gd->max_alloc_vects+nxt*NVECT_ALLOC) * 
                   sizeof(XDl_image_vect));
            if (p==0) return 4;
         }
         gd->vects = p;
         gd->max_alloc_vects += nxt*NVECT_ALLOC;
      }
   }

   for (ivec=0;ivec<num_vect;++ivec)
   {
      iposn = gd->num_vect;
      gd->num_vect += 1;

      /* Save details for the new vector */

      xdl_image_local_to_xy(gd, ax1_1_pixel[ivec], ax2_1_pixel[ivec],
                            &x1_pixel, &y1_pixel,1);
      xdl_image_local_to_xy(gd, ax1_2_pixel[ivec], ax2_2_pixel[ivec],
                            &x2_pixel, &y2_pixel,1);
      gd->vects[iposn].vect_id = vect_id;
      gd->vects[iposn].x1_pixel = x1_pixel;
      gd->vects[iposn].y1_pixel = y1_pixel;
      gd->vects[iposn].x2_pixel = x2_pixel;
      gd->vects[iposn].y2_pixel = y2_pixel;
      ax1_1_off = ax1_1_pos[ivec];
      ax2_1_off = ax2_1_pos[ivec];
      ax1_2_off = ax1_2_pos[ivec];
      ax2_2_off = ax2_2_pos[ivec];
      if (ax1_1_off<-2) ax1_1_off = 0;
      if (ax1_1_off>2) ax1_1_off = 0;
      if (ax2_1_off<-2) ax2_1_off = 0;
      if (ax2_1_off>2) ax2_1_off = 0;
      if (ax1_2_off<-2) ax1_2_off = 0;
      if (ax1_2_off>2) ax1_2_off = 0;
      if (ax2_2_off<-2) ax2_2_off = 0;
      if (ax2_2_off>2) ax2_2_off = 0;
      switch (gd->jorder) 
      {
          case 1:  /* +ax1, +ax2 */
             x1_off = ax1_1_off;
             y1_off = ax2_1_off;
             x2_off = ax1_2_off;
             y2_off = ax2_2_off;
          break;

          case 2:  /* +ax1, -ax2 */
             x1_off = ax1_1_off;
             y1_off = -ax2_1_off;
             x2_off = ax1_2_off;
             y2_off = -ax2_2_off;
          break;         

          case 3:  /* -ax1, +ax2 */
             x1_off = -ax1_1_off;
             y1_off = ax2_1_off;
             x2_off = -ax1_2_off;
             y2_off = ax2_2_off;
          break;

          case 4:  /* -ax1, -ax2 */
             x1_off = -ax1_1_off;
             y1_off = -ax2_1_off;
             x2_off = -ax1_2_off;
             y2_off = -ax2_2_off;
          break;

          case 5:  /* +ax2, +ax1 */
             x1_off = ax2_1_off;
             y1_off = ax1_1_off;
             x2_off = ax2_2_off;
             y2_off = ax1_2_off;
          break;

          case 6:  /* +ax2, -ax1 */
             x1_off = ax2_1_off;
             y1_off = -ax1_1_off;
             x2_off = ax2_2_off;
             y2_off = -ax1_2_off;
          break;         

          case 7:  /* -ax2, +ax1 */
             x1_off = -ax2_1_off;
             y1_off = ax1_1_off;
             x2_off = -ax2_2_off;
             y2_off = ax1_2_off;
          break;

          case 8:  /* -ax2, -ax1 */
             x1_off = -ax2_1_off;
             y1_off = -ax1_1_off;
             x2_off = -ax2_2_off;
             y2_off = -ax1_2_off;
          break;
      }
      gd->vects[iposn].x1_offset = x1_off;
      gd->vects[iposn].y1_offset = y1_off;
      gd->vects[iposn].x2_offset = x2_off;
      gd->vects[iposn].y2_offset = y2_off;
      xdl_image_vector_offsets (gd, iposn);
      gd->vects[iposn].color_flags = color;
      if (ov_num==2) gd->vects[iposn].color_flags |= 0X10;
      if (gd->ovly_typ==2||ov_num==2) gd->vects[iposn].color_flags |= 0X20;
     
      /* Draw vector if required */
   
      if (gd->ovly1==0&&ov_num==1) return 0;
      if (gd->ovly2==0&&ov_num==2) return 0;
      xs_off = 0;
      ys_off = 0;
      if (gd->ovly_offset==1&&(gd->vects[iposn].color_flags&0X20)==0)
      {
          xs_off = gd->x_ovly_offset;
          ys_off = gd->y_ovly_offset;
      }
      xdl_image_drawvec (gd, gd->wdisp, gd->cur_expand, 
                         gd->vects[iposn].x1_disp_offset + xs_off, 
                         gd->vects[iposn].y1_disp_offset + ys_off,
                         gd->vects[iposn].x2_disp_offset + xs_off, 
                         gd->vects[iposn].y2_disp_offset + ys_off,
                         gd->vects[iposn].x1_offset,
                         gd->vects[iposn].y1_offset,
                         gd->vects[iposn].x2_offset,
                         gd->vects[iposn].y2_offset,
                         color);
   }
   if (magw_upd) xdl_image_magw (gd);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_vects_pos */

void xdlf_image_vects_pos (ivh, ivect, num_vect, 
                      iax1pix1, iax2pix1, iax1pix2, iax2pix2, 
                      iax1pos1, iax2pos1, iax1pos2, iax2pos2,
                      icolr, iov, magupd, ierr)
int *ivh, *ivect, *num_vect, *iax1pix1, *iax2pix1, *iax1pix2, *iax2pix2, 
    *icolr, *iov, *magupd, *iax1pos1, *iax2pos1, *iax1pos2, *iax2pos2;
int *ierr;
{
   *ierr = xdl_image_vects_pos (*ivh, *ivect, *num_vect, 
                                iax1pix1, iax2pix1, 
                                iax1pix2, iax2pix2, 
                                iax1pos1, iax2pos1, 
                                iax1pos2, iax2pos2, 
                                *icolr, *iov, *magupd);
   return;
}

/*-Routine: Store and display overlay boxes - xdl_image_boxes
The routine xdl_image_boxes (xdlf_image_boxes) is used to
store details of a set of overlay boxes surrounding a rectangle of pixels
and to display those boxes if overlays are currently being displayed. 
The routine saves each box as a set of vectors (see also xdl_image_vect
and xdlf_image_vect etc.). The user supplies a vector identifier set number 
for the box vectors and this is used to identify the set of vectors for 
deletion if required.
-end*/
/*
*****************************
**     xdl_image_boxes     ** 
*****************************

Purpose: Store and display a set of boxes surrounding a set of pixels

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_BOXES (IVH, IVECT, NUM_BOX, IAX1PIX1, IAX2PIX1, 
        +                       IAX1PIX2, IAX2PIX2, IPOS, 
        +                       ICOLR, IOV, MAGUPD, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  User selected number to identify vector set associated 
              with the boxes (0-65535). (see vect_id)
NUM_BOX  (R)  The number of boxes
IAX1PIX1 (R)  Local axis 1 start corner pixel positions in full image 
              (see ax1_1_pixel) (array of NUM_BOX values)
IAX2PIX1 (R)  Local axis 2 start corner pixel positions in full image 
              (see ax2_1_pixel) (array of NUM_BOX values)
IAX1PIX2 (R)  Local axis 1 end corner pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_BOX values)
IAX2PIX2 (R)  Local axis 2 end corner pixel positions in full image 
              (see ax2_2_pixel) (array of NUM_BOX values)
IPOS     (R)  = 1 position just within pixel boundaries
              = 2 position just outside pixel boundaries
ICOLR    (R)  Vector colour (1 to 6) (see color)
IOV      (R)  Overlay number 1 or 2 (see ov_num)
MAGUPD   (R)  =1 Update magnifying window after drawing vector,
              =0 do not (see magw_upd)
IERR     (W)  Returns status from xdl_image_boxes call
-end*/

/*-C:*/
int xdl_image_boxes (vh, vect_id, num_box, ax1_1_pixel, ax2_1_pixel, 
                         ax1_2_pixel, ax2_2_pixel, ipos,
                         color, ov_num, magw_upd)
/*end*/
 
/*-Parameters:*/
int vh;           /* View-object handle (R)*/
int vect_id;      /* User selected number to identify vector (0-65535). 
                     The same number may be used for more than one vector to 
                     identify a set of vectors (R)*/
int num_box;      /* Number of boxes to be added */
int *ax1_1_pixel; /* The vector start corner local axis 1 pixel positions 
                     (from 1 up - refers to complete uncompressed image 
                     (array of num_box values (R) */
int *ax2_1_pixel; /* The vector startcorner local axis 2 pixel positions
                     (array of num_box values  (R) */
int *ax1_2_pixel; /* The vector end corner local axis 1 pixel positions
                     (array of num_box values  (R) */
int *ax2_2_pixel; /* The vector end corner local axis 2 pixel positions
                     (array of num_box values  (R) */
int ipos;         /*  = 1 position just within pixel boundaries
                      = 2 position just outside pixel boundaries (R) */
int color;        /* Overlay colour type 1 to 6; Red, Yellow, Green, Cyan,
                     Blue, Magenta (by default)*/
int ov_num;       /* Overlay number 1 or 2 (R)*/
int magw_upd;     /* =1 Update the magnifying window after drawing the 
                        boxes; this should be used when all the
                        required boxes have been defined.
                     =0 Do not (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set:  Unable to allocate additional space for
                              storing the new vector's details
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   int ax1_1[200];           /* Vector list */
   int ax2_1[200];           /* Vector list */
   int ax1_2[200];           /* Vector list */
   int ax2_2[200];           /* Vector list */
   int pos1_1[200];          /* Vector list - position flags */ 
   int pos2_1[200];          /* Vector list - position flags */ 
   int pos1_2[200];          /* Vector list - position flags */ 
   int pos2_2[200];          /* Vector list - position flags */
   int nbl;                  /* No. of blocks of 50 boxes to store */
   int ib;                   /* Loop counter for boxes blocks */
   int nleft;                /* No. of boxes left to o/p at 
                                start of new block */
   int nmax;                 /* No. of boxes in current block */
   int n;                    /* Counter through boxes in current block */
   int i;                    /* Counter through boxes arrays */
   int j;                    /* Counter through vectors arrays */
   int mg;                   /* Magnifying window flag to use for
                                current block */
   int x1, y1, x2, y2, tmp;  /* Temp. coordinates */
   int ir;                   /* Return code from xdl_image_vects_pos call */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_boxes",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_boxes", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<1||color>6) color = 1;
   if (ipos<1||ipos>2) ipos = 1;

   if (num_box<=0) return 0;
   nbl = (num_box-1)/50 + 1;
   i=-1;
   nleft = num_box;
   for (ib=0;ib<nbl;++ib)
   {
      mg = 0;
      if (ib==nbl-1) mg = magw_upd;
      nmax = 50;
      if (nleft<50) nmax = nleft;
      j = -1;
      for (n=0;n<nmax;++n)
      {
         i++;
         x1 = ax1_1_pixel[i];
         y1 = ax2_1_pixel[i];
         x2 = ax1_2_pixel[i];
         y2 = ax2_2_pixel[i];
         if (x2<x1)
	 {
            tmp = x1;
            x1 = x2;
            x2 = tmp;
	 }
         if (y2<y1)
	 {
            tmp = y1;
            y1 = y2;
            y2 = tmp;
	 }
         j++;
         ax1_1[j] = x1;
         ax2_1[j] = y1;
         ax1_2[j] = x1;
         ax2_2[j] = y2;
         pos1_1[j] = -ipos;
         pos2_1[j] = -ipos;
         pos1_2[j] = -ipos;
         pos2_2[j] = ipos;
         j++;
         ax1_1[j] = x1;
         ax2_1[j] = y2;
         ax1_2[j] = x2;
         ax2_2[j] = y2;
         pos1_1[j] = -ipos;
         pos2_1[j] = ipos;
         pos1_2[j] = ipos;
         pos2_2[j] = ipos;
         j++;
         ax1_1[j] = x2;
         ax2_1[j] = y2;
         ax1_2[j] = x2;
         ax2_2[j] = y1;
         pos1_1[j] = ipos;
         pos2_1[j] = ipos;
         pos1_2[j] = ipos;
         pos2_2[j] = -ipos;
         j++;
         ax1_1[j] = x2;
         ax2_1[j] = y1;
         ax1_2[j] = x1;
         ax2_2[j] = y1;
         pos1_1[j] = ipos;
         pos2_1[j] = -ipos;
         pos1_2[j] = -ipos;
         pos2_2[j] = -ipos;
      }
      ir =xdl_image_vects_pos (vh, vect_id, 4*nmax, ax1_1, ax2_1, ax1_2, ax2_2,
                           pos1_1, pos2_1, pos1_2, pos2_2, color, ov_num, mg);
      if (ir!=0) return ir;
      nleft -= nmax;      
   }
   return 0;
}

/* Fortran binding: xdlf_image_boxes */

void xdlf_image_boxes (ivh, ivect, num_box, 
                      iax1pix1, iax2pix1, iax1pix2, iax2pix2, 
                      ipos, icolr, iov, magupd, ierr)
int *ivh, *ivect, *num_box, *iax1pix1, *iax2pix1, *iax1pix2, *iax2pix2, 
    *ipos, *icolr, *iov, *magupd;
int *ierr;
{
   *ierr = xdl_image_boxes (*ivh, *ivect, *num_box, iax1pix1, iax2pix1, 
                                iax1pix2, iax2pix2, *ipos, *icolr, 
                                *iov, *magupd);
   return;
}

/*-Routine: Clear overlay vectors - xdl_image_clear_vectors
The routine xdl_image_clear_vects (xdlf_image_clear_vects) is
used to clear all overlay vectors from the vectors list and from the
display.
-end*/
/*
*************************************
**     xdl_image_clear_vects       ** 
*************************************

Purpose: Clear table of vectors and clear vectors from screen

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_CLEAR_VECTS (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_image_clear_vects
              call
-end*/

/*-C:*/
int xdl_image_clear_vects (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Loop counter */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_clear_vects",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_clear_vects", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->vects!=NULL) free (gd->vects);
   gd->vects = NULL;
   gd->max_alloc_vects = 0;
   gd->num_vect = 0;
   xdl_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_clear_vects */

void xdlf_image_clear_vects (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_image_clear_vects (*ivh);
   return;
}

/*-Routine: Delete an overlay vector - xdl_image_del_vect
The routine xdl_image_del_vect (xdlf_image_del_vect) is used
to delete an overlay vector (or set of vectors with a common vector
identifier) from the vectors list and remove it (them) from the
display. If there are many vectors to delete it may be quicker to clear
all vectors (xdl_image_clear_vects (xdlf_image_clear_vects))
and create a new list using xdl_image_vect (xdlf_image_vect)
-end*/
/*
********************************
**     xdl_image_del_vect     **
********************************

Purpose: Delete an overlay vector or set of vectors

Author:  John W. Campbell, December 1991

*/
/*-Fortran:
         CALL XDLF_IMAGE_DEL_VECT (IVH, IVECT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IVECT    (R)  The vector identifier of the vector or set of
              vectors to be deleted (see vect_id)
IERR     (W)  Returns status from xdl_image_del_vect call
-end*/

/*-C:*/
int xdl_image_del_vect (vh, vect_id)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int vect_id;     /* Vector identifier for the vector or set of vectors
                    to be deleted (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Vector not found
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   char *tp1,*tp2;           /* Temp character pointers, used in copying 
                                vector structures */
   int iposn;                /* Position (offset) in vectors list for
                                current accepted vector */
   int i;                    /* Loop counter */
   int ii;                   /* Loop counter through vectors */
   int match;                /* Matching vector(s) found */
   int ssiz;                 /* Size of vectors structure */
   XDl_image_vect *vec;        /* Temp. pointer to vector structure */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted vectors area (also vectors)*/
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted vector area (also vectors) */
   int xs_off;                      /* Overlay x offset */
   int ys_off;                      /* Overlay y offset */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_del_vect",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_del_vect", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->num_vect==0) return 4;

   xs_min = gd->img_width - 1;
   xs_max = 0;
   ys_min = gd->img_height - 1;
   ys_max = 0;

   iposn = 0;
   match = 0;
   for (ii=0;ii<gd->num_vect;++ii)
   {
      vec = &(gd->vects[ii]);
      if (vec->vect_id==vect_id)
      {
         match = 1;
         if (vec->x1_disp_offset<xs_min) xs_min = vec->x1_disp_offset;
         if (vec->x1_disp_offset>xs_max) xs_max = vec->x1_disp_offset;
         if (vec->y1_disp_offset<ys_min) ys_min = vec->y1_disp_offset;
         if (vec->y1_disp_offset>ys_max) ys_max = vec->y1_disp_offset;
         if (vec->x2_disp_offset<xs_min) xs_min = vec->x2_disp_offset;
         if (vec->x2_disp_offset>xs_max) xs_max = vec->x2_disp_offset;
         if (vec->y2_disp_offset<ys_min) ys_min = vec->y2_disp_offset;
         if (vec->y2_disp_offset>ys_max) ys_max = vec->y2_disp_offset;
         continue;
      }
      if (match)
      {
         tp1 = (char *) &gd->vects[iposn];
         tp2 = (char *) &gd->vects[ii];
         ssiz = sizeof (XDl_image_vect);
         for (i=0;i<ssiz;++i) tp1[i] = tp2[i];
      }
      iposn++;
   }
   gd->num_vect = iposn;

   if (match==0) return 4;

   if (xs_min<0) xs_min = 0;
   if (ys_min<0) ys_min = 0;
   if (xs_max>gd->img_width - 1) xs_max = gd->img_width - 1;
   if (ys_max>gd->img_height - 1) ys_max = gd->img_height - 1;
   XPutImage (XDL_display, gd->wdisp, gd->paint_gc,
              gd->img_image, xs_min, ys_min, xs_min, ys_min,
              (xs_max - xs_min + 1), (ys_max - ys_min + 1));
   if (gd->magnify_on==0) xdl_image_set_freeze (gd);
   if (gd->ovly1==0&&gd->ovly2==0) return 0;

   xdl_image_rdv (gd, xs_min - gd->cur_expand - 1, 
                  xs_max + gd->cur_expand + 1, 
                  ys_min - gd->cur_expand - 1, 
                  ys_max + gd->cur_expand + 1, 0, 0, 0);
   xdl_image_rdt (gd, ys_min, ys_max+gd->ch_height);

   xs_min -= MAX_SYMB_SIZE;
   xs_max += MAX_SYMB_SIZE;
   ys_min -= MAX_SYMB_SIZE;
   ys_max += MAX_SYMB_SIZE;

   xdl_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);

   xdl_image_magw (gd);

   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_del_vect */

void xdlf_image_del_vect (ivh, ivect, ierr)
int *ivh, *ivect, *ierr;
{
   *ierr = xdl_image_del_vect (*ivh, *ivect);
   return;
}
/*-Routine: Output a text string - xdl_image_text
The routine xdl_image_text (xdlf_image_text) is used to output a text string
to the main display area.
-end*/
/*
*************************
**     xdl_image_text  ** 
*************************

Purpose: Output a text string to the main display area

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_TEXT (IVH, ITEXT, IAX1PIXEL, IAX2PIXEL,
        +                      XDLSTR(STRING), LEN, ICOLOR, IFONT, 
        +                      IBOLD, IOV_NUM, IERR)
-end*/
 
/*-Parameters:
IVH        (R)  View-object handle (see vh)
ITEXT      (R)  User selected text string identifier (used
                to identify a string or set of strings for
                deletion) (0-65535) 
IAX1PIXEL  (R)  Pixel position wrt. local axis 1 for start of text
                string output
IAX2PIXEL  (R)  Pixel position wrt. local axis 2 for start of text
                string output
STRING     (R)  The text string 
                ** Pass address using the XDLSTR function **
LEN        (R)  The length of the string; must be > 0 (cf len)
ICOLOR     (R)  Overlay colour type =0 for black or 1 to 6; Red, 
                Yellow, Green, Cyan, Blue, Magenta (by default)
IFONT      (R)  Font no. 1-5 very-small, small, medium, large,
                very-large
IBOLD      (R)  = 0 normal text, =1 bold text
IOV_NUM    (R)  Overlay number 1 or 2 
IERR       (W)  Returns status from xdl_image_text call
-end*/

/*-C:*/
int xdl_image_text (vh, text_id, ax1_pixel, ax2_pixel, string, len,
                    color, font_type, bold, ov_num)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int text_id;     /* User selected text string identifier (used
                    to identify a string or set of strings for
                    deletion) (0-20000) (R) */
int ax1_pixel;   /* Pixel position wrt. local axis 1 for start of text
                    string output (R)*/
int ax2_pixel;   /* Pixel position wrt. local axis 2 for start of text
                    string output (R)*/
char * string;   /* The text string */
int len;         /* The length of the text string; may give 0 if the
                    string is null terminated (R)*/
int color;       /* Overlay colour type =0 for black or 1 to 6; Red, 
                    Yellow, Green, Cyan, Blue, Magenta (by default)*/
int font_type;   /* Font type number 1-5, very-small, small, medium,
                    large, very-large (R)*/
int bold;        /* =0 normal text, =1 bold text */
int ov_num;      /* Overlay number 1 or 2 (R)*/
/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   XDl_image_text *p;        /* Temp. pointer to this structure */
   int iposn;                /* Position (offset) in text strings list for new
                                text string */
   int xs_off;               /* Overlay x offset */
   int ys_off;               /* Overlay y offset */
   int x_pixel;              /* x pixel wrt X-windows axes */
   int y_pixel;              /* y pixel wrt X-windows axes */
   int ll;                   /* Length of string */
   int n;                    /* Significant length of string excluding
                                trailing blanks */
   int l;                    /* Loop counter */
   char * pp;                /* Temp string pointer */

   if (ov_num!=1&&ov_num!=2) ov_num=1;
   if (font_type<1) font_type = 1;
   if (font_type>5) font_type = 5;
   if (bold!=0&&bold!=1) bold = 0;
   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_text",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_text", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (color<0||color>6) color = 0;
   if (gd->num_text>=gd->max_alloc_text)
   {
      if (gd->text==NULL)
      {
         p = (XDl_image_text *) malloc (
           (gd->max_alloc_text+NTEXT_ALLOC) * sizeof(XDl_image_text));
         if (p==0) return 4;
      }
      else
      {
         p = (XDl_image_text *) realloc (gd->text,
           (gd->max_alloc_text+NTEXT_ALLOC) * sizeof(XDl_image_text));
         if (p==0) return 4;
      }
      gd->text = p;
      gd->max_alloc_text += NTEXT_ALLOC;
   }

   iposn = gd->num_text;
   gd->num_text += 1;

   /* Save details for the new text string */

   xdl_image_local_to_xy(gd, ax1_pixel, ax2_pixel, &x_pixel, &y_pixel,1);
   gd->text[iposn].text_id = text_id;
   gd->text[iposn].x_pixel = x_pixel;
   gd->text[iposn].y_pixel = y_pixel;
   xdl_image_text_offsets (gd, iposn);
   gd->text[iposn].string = NULL;
   ll = len;
   if (ll==0) ll = strlen(string);
   n = 0;
   if (ll>0)
   {
      n = 0;
      for (l=ll-1;l>=0;--l)
      {
         if (string[l]!=' ') {n=l+1; break;}
      }
   }
   pp = malloc ((n+1)*sizeof(char));
   if (pp!=0)
   {
      gd->text[iposn].string = pp;
      pp[0] = 0;
      strncat (pp, string, n);
   }
   gd->text[iposn].color_flags = color;
   if (ov_num==1) gd->text[iposn].color_flags |= 0X10;
   if (gd->ovly_typ==2||ov_num==2) gd->text[iposn].color_flags |= 0X20;
   gd->text[iposn].font = font_type;
   gd->text[iposn].bold = bold;
     
   /* Draw text string if required */

   if (gd->ovly1==0&&ov_num==1) return 0;
   if (gd->ovly2==0&&ov_num==2) return 0;
   xs_off = 0;
   ys_off = 0;
   if (gd->ovly_offset==1&&(gd->text[iposn].color_flags&0X20)==0)
   {
       xs_off = gd->x_ovly_offset;
       ys_off = gd->y_ovly_offset;
   }
   ll = strlen(gd->text[iposn].string);
   if (color==0)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   }
   else
   {
      XSetForeground (XDL_display, gd->paint_gc, 
                      gd->ovly_pixels[color-1]);
   }
   if (bold)
   {
      XSetFont (XDL_display,gd->paint_gc,
                XDL_bold_fonts[font_type-1]->fid);
   }
   else
   {
      XSetFont (XDL_display,gd->paint_gc,
                XDL_fonts[font_type-1]->fid);
   }
   XDrawString (XDL_display,gd->wdisp, gd->paint_gc,
                gd->text[iposn].x_disp_offset + xs_off,
                gd->text[iposn].y_disp_offset + ys_off,
                gd->text[iposn].string, ll);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_text */

void xdlf_image_text (ivh, itext, iax1pixel, iax2pixel, string, len,
                      icolor, ifont, ibold, iov_num, ierr)
int *ivh, *itext, *iax1pixel, *iax2pixel, *string, *len, *icolor, *ifont,
*ibold;
int *iov_num, *ierr;
{
   *ierr = xdl_image_text (*ivh, *itext, *iax1pixel, *iax2pixel,
                           XDL_pointers[*string], *len, *icolor, *ifont, 
                            *ibold, *iov_num);
   return;
}
/*-Routine: Delete a text string - xdl_image_del_text
The routine xdl_image_del_text (xdlf_image_del_text) is used
to delete an overlay text string (or set of text strings with a common text
identifier) from the text strings list and remove it (them) from the
display. If there are many text strings to delete it may be quicker to clear
all the text strings (xdl_image_clear_text (xdlf_image_clear_text))
and create a new set using xdl_image_text (xdlf_image_text)
-end*/
/*
********************************
**     xdl_image_del_text     **
********************************

Purpose: Delete an overlay text stringor set of text string

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_DEL_TEXT(IVH, ITEXT, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
ITEXT    (R)  The text string identifier of the text string 
              or set of text strings to be deleted (see text_id)
IERR     (W)  Returns status from xdl_image_del_text call
-end*/

/*-C:*/
int xdl_image_del_text (vh, text_id)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
int text_id;     /* Text identifier for the text string or set of text
                    strings to be deleted (R) */

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
                  bit 2 set;  Vector not found
-end*/
{
   int iv;                   /* Offset in XDL_view_objects array */
   XDl_image *gd;            /* Pointer to global data area */
   char *tp1,*tp2;           /* Temp character pointers, used in copying 
                                text structures */
   int iposn;                /* Position (offset) in text strings list for
                                current accepted text string */
   int i;                    /* Loop counter */
   int ii;                   /* Loop counter through text strings */
   int match;                /* Matching text string(s) found */
   int ssiz;                 /* Size of text strings structure */
   XDl_image_text *text;        /* Temp. pointer to text string structure */
   int xs_min, xs_max;              /* x limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted text strings area 
                                       (also vectors or other text strings)*/
   int ys_min, ys_max;              /* y limits (in main film display area)
                                       for symbols which may appear in the
                                       deleted text strings area 
                                       (also vectors or other text strings) */
   int xs_off;                      /* Overlay x offset */
   int ys_off;                      /* Overlay y offset */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_del_text",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_del_text", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->num_text==0) return 4;

   xs_min = gd->img_width - 1;
   xs_max = gd->img_width - 1;  /*Length of text not calculated */
   ys_min = gd->img_height - 1;
   ys_max = 0;

   iposn = 0;
   match = 0;
   for (ii=0;ii<gd->num_text;++ii)
   {
      text = &(gd->text[ii]);
      if (text->text_id==text_id)
      {
         match = 1;
         if (text->x_disp_offset<xs_min) xs_min = text->x_disp_offset;
         if (text->y_disp_offset-gd->ch_height<ys_min) 
                   ys_min = text->y_disp_offset-gd->ch_height;
         if (text->y_disp_offset>ys_max) ys_max = text->y_disp_offset;
         continue;
      }
      if (match)
      {
         tp1 = (char *) &gd->text[iposn];
         tp2 = (char *) &gd->text[ii];
         ssiz = sizeof (XDl_image_text);
         for (i=0;i<ssiz;++i) tp1[i] = tp2[i];
      }
      iposn++;
   }
   gd->num_text = iposn;

   if (match==0) return 4;

   if (xs_min<0) xs_min = 0;
   if (ys_min<0) ys_min = 0;
   if (xs_max>gd->img_width - 1) xs_max = gd->img_width - 1;
   if (ys_max>gd->img_height - 1) ys_max = gd->img_height - 1;
   XPutImage (XDL_display, gd->wdisp, gd->paint_gc,
              gd->img_image, xs_min, ys_min, xs_min, ys_min,
              (xs_max - xs_min + 1), (ys_max - ys_min + 1));
   if (gd->magnify_on==0) xdl_image_set_freeze (gd);
   if (gd->ovly1==0&&gd->ovly2==0) return 0;

   xdl_image_rdt (gd, ys_min, ys_max+gd->ch_height);
   xdl_image_rdv (gd, xs_min - gd->cur_expand - 1, 
                  xs_max + gd->cur_expand + 1, 
                  ys_min - gd->cur_expand - 1, 
                  ys_max + gd->cur_expand + 1, 0, 0, 0);

   xs_min -= MAX_SYMB_SIZE;
   xs_max += MAX_SYMB_SIZE;
   ys_min -= MAX_SYMB_SIZE;
   ys_max += MAX_SYMB_SIZE;

   xdl_image_rds (gd, xs_min, xs_max, ys_min, ys_max, 0, 0, 0);

   xdl_image_magw (gd);

   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_del_text */

void xdlf_image_del_text (ivh, itext, ierr)
int *ivh, *itext, *ierr;
{
   *ierr = xdl_image_del_text (*ivh, *itext);
   return;
}
/*-Routine: Clear all text strings - xdl_image_clear_text
The routine xdl_image_clear_text (xdlf_image_clear_text) is used to clear
(delete) all text strings which were previously output to the main display
window.
-end*/
/*
*******************************
**     xdl_image_clear_text  ** 
*******************************

Purpose: Clear all text strings from main display

Author:  John W. Campbell, August 1993

*/
/*-Fortran:
         CALL XDLF_IMAGE_CLEAR_TEXT (IVH, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
IERR     (W)  Returns status from xdl_image_clear_text call
-end*/

/*-C:*/
int xdl_image_clear_text (vh)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */
   int i;                  /* Temp/loop variable */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_clear_text",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_clear_text", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (gd->num_text>0)
   {
      for (i=0;i<gd->num_text;++i)
      {
         if (gd->text[i].string!=NULL) free ((char *) gd->text[i].string);
      }
   }
   if (gd->text!=NULL) free ((char *) gd->text);
   gd->text = NULL;
   gd->max_alloc_text = 0;
   gd->num_text = 0;
   xdl_image_refresh (gd, 1);
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_clear_text */

void xdlf_image_clear_text (ivh, ierr)
int *ivh, *ierr;
{
   *ierr = xdl_image_clear_text (*ivh);
   return;
}

/*-Routine: Reset active strip message - xdl_image_input_message
The routine xdl_image_input_message (xdlf_image_input_message) is
used to reset the message which will appear in the active strip when a
pixel position or selected rectangle is to be input. When the image 
view-object is created, the default message is 'Input spot positions'.
-end*/
/*
*************************************
**     xdl_image_input_message     ** 
*************************************

Purpose: Reset 'Input spot positions' message for the active strip area

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_INPUT_MESSAGE (IVH, XDLSTR(MESSAGE), 
        +                                    LEN, IERR)
-end*/
 
/*-Parameters:
IVH      (R)  View-object handle (see vh)
MESSAGE  (R)  Character string containing the message 
              (see message)
              ** Pass address using the XDLSTR function **
LEN      (R)  Length of the message string - must be >0 
              (cf len) (max 60 chars) 
IERR     (W)  Returns status from xdl_image_input_message 
              call
-end*/

/*-C:*/
int xdl_image_input_message (vh, message, len)
/*end*/
 
/*-Parameters:*/
int vh;          /* View-object handle (R)*/
char *message;   /* Message string (max of 60 chars long) (R)*/
int len;         /* Length of the message string; if 0 then length 
                    will be found assuming a null terminated string 
                    (R)*/

/*end*/
/*-Doc:
Return:  Status = 0 OK, >0 error
                  bit 0 set:  View-object handle not found
                  bit 1 set:  View handle does not match view object
-end*/
{
   int iv;                 /* Offset in XDL_view_objects array */
   XDl_image *gd;          /* Pointer to global data area */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_image_input_message",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1085)
   {
      xdl_view_type_errmsg ("xdl_image_input_message", vh, "xdl_image");
      return 2;
   }
   gd = (XDl_image *) XDL_view_objects[iv]->gd;
   if (len==0) len = strlen (message);
   if (len>60) len = 60;
   strncpy (gd->input_message, message, len);
   gd->input_message[len] = 0;
   if (gd->on_off==1) 
   {
      xdl_active_strip_on (gd->wact, gd->input_message, 0);
   }
   XFlush (XDL_display);
   return 0;
}

/* Fortran binding: xdlf_image_input_message */

void xdlf_image_input_message (ivh, message, len, ierr)
int *ivh, *message, *len, *ierr;
{
   *ierr = xdl_image_input_message (*ivh, XDL_pointers[*message], *len);
   return;
}

/*-Routine: Calculate the size requirements - xdl_image_getsize
The routine xdl_image_getsize (xdlf_image_getsize) is used to
calculate the size of an image view-object given the size of the
image to be displayed.
-end*/
/*
********************************
**     xdl_image_getsize      **
********************************

Purpose: Return the size requirements of the image view-object

Author:  John W. Campbell, August 1990

*/
/*-Fortran:
         CALL XDLF_IMAGE_GETSIZE (NREQ_AX1, NREQ_AX2, IORDER, JORDER,
                                  NCMP, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
NREQ_AX1   (R)  Section size along local axis 1 (see nreq_ax1)
NREQ_AX2   (R)  Section size along local axis 2 (see nreq_ax1)
IORDER     (R)  Image data order wrt. local axes (1-8)
                (see XDLF_IMAGE routine) 
JORDER     (R)  Display axis order wrt. local axes (1-8)
                (see XDLF_IMAGE routine) 
NCMP       (R)  No. of pixels compressed into 1 pixel
                along each axis
IWIDTH     (W)  Returns the width required (see w)
IHEIGHT    (W)  Returns the height required (see h)
-end*/

/*-C:*/
void xdl_image_getsize (nreq_ax1, nreq_ax2, iorder, jorder, ncmp, w, h)
/*end*/
 
/*-Parameters:*/
int nreq_ax1;            /* Section size along local axis 1 (R) */
int nreq_ax2;            /* Section size along local axis 2 (R) */
int iorder;              /* Image data order wrt. local axes (1-8)
                            (see xdl_image routine) (R)*/
int jorder;              /* Display axis order wrt. local axes (1-8)
                            (see xdl_image routine) (R)*/
int ncmp;                /* No. of pixels compressed into 1 pixel
                            along each axis */
int *w;                  /* Returns the width required (W)*/
int *h;                  /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int width;      /* Calc width */
   int height;     /* Calc height */
   int ytop;       /* Height required for top of object items */
   int n_ax1;      /* Rounded down section size along local axis 1 in pixels */
   int n_ax2;      /* Rounded down section size along local axis 2 in pixels */
   int img_width;  /* Displayed image width */
   int img_height; /* Displayed image height */
   int korder;     /* Order of image axes wrt. X-windows axes*/
   static int kord[8][8] = {
     {5,6,7,8,1,2,3,4},
     {6,5,8,7,3,4,1,2},
     {7,8,5,6,2,1,4,3},
     {8,7,6,5,4,3,2,1},
     {1,2,3,4,5,6,7,8},
     {3,4,1,2,6,5,8,7},
     {2,1,4,3,7,8,5,6},
     {4,3,2,1,8,7,6,5}};

   /* Calculate the size requirements */

   if (iorder<1||iorder>8) iorder = 1;
   if (jorder<1||jorder>8) jorder = 1;
   if (ncmp<0) ncmp = -ncmp;
   korder = kord[iorder-1][jorder-1];
   n_ax1 = ncmp*(nreq_ax1/ncmp);
   n_ax2 = ncmp*(nreq_ax2/ncmp);
   if (jorder<=4)
   {
      img_width = n_ax1/ncmp;
      img_height = n_ax2/ncmp;
   }
   else
   {
      img_width = n_ax2/ncmp;
      img_height = n_ax1/ncmp;
   }

   width = 405 + MAGWSIZE_W;
   if ((img_width+2+2*BORDER_WIDTH+2)>width) 
      width = img_width + 2 + 2*BORDER_WIDTH + 2;
   ytop = 66 + BORDER_HEIGHT + ACT_STRIP_HEIGHT;
   if (MAGWSIZE_H+2+ACT_STRIP_HEIGHT>ytop) ytop = MAGWSIZE_H+2+ACT_STRIP_HEIGHT;
   height = img_height + 2 + 2*BORDER_HEIGHT + 2 + ytop;
   *w = width;
   *h = height;
   return;
}

/* Fortran binding: xdlf_image_getsize */

void xdlf_image_getsize (nreq_ax1, nreq_ax2, iorder, jorder, ncmp,
                         iwidth, iheight)
int *nreq_ax1, *nreq_ax2, *iorder, *jorder, *ncmp, *iwidth, *iheight;
{
   xdl_image_getsize (*nreq_ax1, *nreq_ax2, *iorder, *jorder, *ncmp,
                      iwidth, iheight);
   return;
}
