/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Laue Simulations View-Object Routines
-end*/
 
/*-Intro:
The Laue simulation view-object is used to display the simulation of a
Laue pattern (the pattern prediction itself must be done externally
to the routine). The view-object allows either for colour coded
simulations or for black and white simulations where the soft limits can be
modified via sliders to show the effects that they have on the patterns.
The indices of selected spots may be displayed. Labels may be drawn on
the black and white simulations. The view-object also provides a hard copy 
(Postscript file) option. As an alternative, the simulation may also be 
displayed as a gnomonic projection if desired.
-end*/
 
/*-Section: Creating and Handling a Laue Simulations View-Object
-end*/
 

/*Routines List:
The Laue simulation view-object has the following routines available:

   xdl_laue_sim                       Create a Laue simulation view-object
   xdl_laue_sim_getsize               Calculate size requirements
   xdl_laue_sim_new                   Display a new simulation
*/

/*-Routine: Create a Laue simulation view-object - xdl_laue_sim
The routine xdl_laue_sim (xdlf_laue_sim) is used to create a new Laue
simulation display view-object. The simulation itself is calculated
externally to the routine and the details are passed to the routine
for display. The simulation may also be displayed as a gnomonic
projection if desired.
-end*/
/*
************************
**    xdl_laue_sim    **
************************

Purpose: Display a simulated Laue pattern

Author:  John W. Campbell, July 1992

*/
/*-Fortran:
         CALL XDLF_LAUE_SIM (IVH, IVHPARENT, IX, IY, ICSET, ITYPE, ISIZE,
        +                    NSPOT, IHKL, XF, YF, ALAM, DTHR, MULTI,FLAGS, 
        +                    NODHKLS, IDXLAM, IDXDMIN, IDXNOD, FILMR, 
        +                    GNOMR, ALMIN, ALMAX, DMIN, D_LMIN, D_LMAX, 
        +                    D_DMIN, CELL, PHIS, LATT, CTOF,
        +                    MINW, MINH, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
ITYPE     (R)   Simulation type =1 Colour, = 2 Monochrome with
                soft limits options etc. (see type)
ISIZE     (R)   Half width/height for displayed pattern in 
                pixels (see size)
NSPOT     (R)   Number of spots predicted (see nspot)
IHKL      (R)   Array of NSPOT packed indices (see hkl)
XF        (R)   Array of NSPOT 'xf' coordinates (see xf)
YF        (R)   Array of NSPOT 'yf' coordinates (see yf)
ALAM      (R)   Array of NSPOT lambda values (see lam)
DTHR      (R)   Array of NSPOT (1/dmin)**2 thresholds (see dthr)
MULTI     (R)   Array of NSPOT multiplicict data flags (see multi)
FLAGS     (R)   Array of NSPOT reflection flags (see flags)
NODHKLS   (R)   Array of NSPOT packed nodal indices (see nodhkls)
IDXLAM    (R)   Index array for lambda sorted data;
                NSPOT values from 1 to NSPOT (see idxlam)
IDXDMIN   (R)   Index array for dmin sorted data;
                NSPOT values from 1 to NSPOT (see idxdmin)
IDXNOD    (R)   Index array for packed nodal index values 
                (see idxnod)
FILMR     (R)   Pattern radius in mm (see filmr)
GNOMR     (R)   Minimum radius in mm (>0.1) from image centre for 
                conversion of plot to a gnomonic projection. For
                a normal Laue simulation, give a value of 0.0
ALMIN     (R)   Lambda minimum of prediction (see lmin)
ALMAX     (R)   Lambda maximum of prediction (see lmax)
DMIN      (R)   Dmin value of prediction (see dmin)
D_LMIN    (R)   Interval for lambda-min slider (see d_lmin)
D_LMAX    (R)   Interval for lambda-max slider (see d_lmax)
D_DMIN    (R)   Interval for dmin slider (see d_dmin)
CELL      (R)   Array of the 6 real cell dimensions (see cell)
PHIS      (R)   Array of 4 setting angles, Phix, Phiy, Phiz and
                spindle (see phis)
LATT      (R)   4 integer array; crystal system, lattice type,
                rotation axis, beam axis (see latt)
CTOF      (R)   Crystal to film distance (mm) (see ctof)
MINW      (R)   Minimum width for view-object or 0 
                (see min_width)
MINH      (R)   Minimum height for view-object or 0 
                (see min_height)
IERR      (W)   Returns status from xdl_laue_sim call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"
#include <math.h>
#ifdef VMS
#include <file.h>
#else
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>
#endif

#define PANEL_1A_WIDTH 180    /*Width required for panel 'a' (reflection 
                                details area) including 1 pixel border for 
                                type 1 display*/
#define PANEL_1B_WIDTH 400    /*Width required for panel 'b' (controls)
                                including 1 pixel border for type 1 display*/
#define PANEL_1_HEIGHT 70     /*Height of panels including 1 pixel 
                                border for display type 1*/
#define PANEL_2A_WIDTH 180    /*Width required for panel 'a' (reflection 
                                details area) including 1 pixel border for 
                                type 2 display*/
#define PANEL_2B_WIDTH 415    /*Width required for panel 'b' (controls)
                                including 1 pixel border for type 2 display*/
#define PANEL_2_HEIGHT 70     /*Height of panels including 1 pixel 
                                border for display type 2*/
#define PANEL_PS_HEIGHT 22    /*Height of Postscript control panel including
                                1 pixel border*/
#define SEP_H_MIN 4           /*Minimum separation between contol panels and
                                simulation display area*/
#define MAX_LABELS 100        /* Maximum number of labels allowed */
#define SAV_SIZ 100           /* Size of pixmap to save area for moving 
                                 labels */
#define MARK_SIZ 25           /* Size of pixmap for marked area around spot */
#define TWO20 1048576         /* 2**20 */
#define TWO10 1024            /* 2**10 */
#define TWO12 4096            /* 2**12 */
#define TWO4 16               /* 2**4 */
#define CMS_TO_POINTS 2.8346  /* Conversion factor mm's to points */

/* Define structure for index labels */

typedef struct
{
   int x1;          /* x position, top left of label */
   int y1;          /* y position, top left of label */
   int x2;          /* x position, bottom right of label */
   int y2;          /* y position, bottom right of label */
   int xcoff;       /* x centre of label offset from x1 */   
   int ycoff;       /* y centre of label offset from y1 */   
   int idx;         /* Index in reflections list of spot being labelled */
   char hklstr[15]; /* Label 'hkl' indices string */
   char disp;       /* Displayed status =1 displayed, =0 'silent' */
   short h;         /* Displayed 'h' index */
   short k;         /* Displayed 'k' index */
   short l;         /* Displayed 'l' index */
   float lamnod;    /* Lambda value for nodal hkl */
   float dnod;      /* Dmin value for nodal hkl */
}
XDl_laue_sim_label;

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wdisp;                     /* Paint window id for simulation */
   Window wpanel;                    /* Paint window for control panel */
   Window wrefln;                    /* Paint window for reflection details
                                        area */
   Window wpanel_ps;                 /* Paint window for Postscript panel */
   Panel_object lmin_slider;         /* Lambda min panel slider */
   Panel_object lmax_slider;         /* Lambda max panel slider */
   Panel_object dmin_slider;         /* Dmin panel slider */
   Panel_object lmin_curval;         /* Label with current lambda-min value */
   Panel_object lmax_curval;         /* Label with current lambda-max value */
   Panel_object dmin_curval;         /* Label with current dmin value */
   Panel_object reflab[5];           /* Reflection details label items */
   Panel_object sym2_size;           /* Symbol sizes menu for type 2 plots */
   Panel_object highlight2_opt;      /* Plot type highlight options menu for 
                                        type 2 plots */
   Panel_object labels2_opt;         /* Plot type 2 labels options menu */
   Panel_object colorcode_choice;    /* Colour coding choice for type 1 plots */
   Panel_object spatial_choice;      /* Spatials selection for type 1 plots */
   Panel_object spot_type_choice;    /* Spot type choice for type 1 plots */
   Panel_object key_choice;          /* Display key choice for type 1 plots */
   Panel_object spotsize_choice;     /* Spot size choice for type 1 plots */
   Panel_object nodal_sel;           /* Nodal index value selection  */
   Panel_object psbutton;            /* Postscript print button */
   Panel_object ps_io;               /* Postscript details i/o item */
   int         cset;                 /* Colorset number */
   float       lmin_interval;        /* Lambda interval for lambda-min slider */
   float       lmax_interval;        /* Lambda interval for lambda-max slider */
   float       dmin_interval;        /* Dmin interval for panel slider */
   GC          paint_gc;             /* GC for paint window */
   Cursor      cursor;               /* Display area cursor */
   int         color_type;           /* Display colour type flag for interactive
                                        option =1 writeable colour map; 
                                               =0 non-writeable colour map */
   int         type;                 /* Display type 1=colour, 2= monochrome
                                        with sliders etc. */
   int         half_size;            /* Display area half width/height */
   int         nspot;                /* Number of spots */
   int         *hkl;                 /* Pointer to 'hkl' packed indices array */
   float       *xf;                  /* Pointer to 'xf' coordinates array */
   float       *yf;                  /* Pointer to 'yf' coordinates array */
   float       *lam;                 /* Pointer to lambda values array */
   float       *dthr;                /* Pointer to (1/dmin)**2 thresholds 
                                        array */
   int         *multi;               /* Pointer to input multiplicities
                                        array */
   int         *flags;               /* Pointer to reflections flags array */
   int         *nodhkls;             /* Pointer to packed nodal indices array */
   int         *idxlam;              /* Pointer to lambdas index array */
   int         *idxdmin;             /* Pointer to dmin index array */
   int         *idxnod;              /* Pointer to nodal indices index array */
   float       filmr;                /* Predicted pattern radius in mm */
   float       lmin;                 /* Lambda min. for prediction */
   float       lmax;                 /* Lambda max. for prediction */
   float       dmin;                 /* Dmin for prediction */
   short       *xpix;                /* x pixel spot positions array */
   short       *ypix;                /* y pixel spot positions array */
   unsigned char *mult;              /* Multiplicities array */
   unsigned char *max_harmonic;      /* Maximum harmonic array */
   unsigned char *harm_inc;          /* Harmonics increment array */
   unsigned char *colr;              /* color indices (in pixels array) array 
                                        when colour map can be modified. 
                                        0=do not not draw.
                                        or: 0=do not draw, 100=white, 
                                        101=black */
   int         cur_slider;           /* Currently active slider
                                        =1 lambda-min
                                        =2 lambda-max
                                        =3 dmin                      */
   float       cur_lmin;             /* Current lmin value */
   float       cur_lmax;             /* Current lmax value */
   float       cur_dmin;             /* Current dmin value */
   int         cur_lmin_slider_val;  /* Current lmin slider value */
   int         cur_lmax_slider_val;  /* Current lmax slider value */
   int         cur_dmin_slider_val;  /* Current dmin slider value */
   unsigned long pixels[53];         /* Pixels for interactive display */
   unsigned long colrpix[8];         /* Pixels for type 1 colour plot display */
   int         recalc_mult;          /* Re-calculate multiplicities flag
                                        1=yes, 0=no (type 2 plots) */
   int         size_sym2;            /* Current size (1-3) for symbols for
                                        type 2 plots */
   int         opt_highlight2;       /* Highlight option for type 2 plots
                                        1=none, 2=multiples, 3=nodals */
   int         opt_labels2;          /* Labels option for type 2 plots
                                        1=off, 
                                        2=min. hkl
                                        3=max. hkl
                                        4=nodal hkl */
   int         last_labtyp;          /* Last label type 2-4 cf opt_labels2
                                        =0 if not yet set */
   int         nodidx_cur;           /* Current nodal index */
   int         disp_ref;             /* Offset in reflections list of reflection
                                        with indices currently displayed; -1 if
                                        none */
   int         color_coding;         /* Colour coding for plot type 1
                                        =1 wavelength
                                        =2 multiplicity
                                        =3 wavlength/multiplicity
                                        =4 nodality */
   int         spatials;             /* Spatials flag for plot type 1
                                        =1 include
                                        =2 exclude
                                        =3 only */
   int         spot_type;            /* Spot type for plot type 1
                                        =1 all
                                        =2 singles
                                        =3 multiples */
   int         key_opt;              /* Key option for plot type 1
                                        =1 on
                                        =2 off */
   int         spotsize;             /* Spot size for plot type 1
                                        =1 small
                                        =2 medium
                                        =3 large */
   int         ps_on;                /* Postscript being processed currently
                                        =1 yes, =0 no */
   float       ps_r[8];              /* Colour red components for Postscript */
   float       ps_g[8];              /* Colour green components for PS */
   float       ps_b[8];              /* Colour blue components for PS */
   FILE        *psfil;               /* File pointer for Postscript file */
   float       ps_xc;                /* X centre for Postscript plot */
   float       ps_yc;                /* Y centre for Postscript plot */
   float       mm_to_p;              /* Conversion factor, mm to points */
   float       ps_scal;              /* Scale to apply to coordinates to scale 
                                        up to plot size in mm */
   float       ps_pixsiz;            /* Pixel size for PS plot (points)*/
   char        *ps_comment;          /* Pointer to comment for postscript file
                                        (last reply from panel i/o item) */
   float       plot_rad;             /* Plot radius in mm for PS plots */
   float       ps_spot_rad;          /* Spot radius in mm for PS plots */
   float       ps_symb_hs;           /* Symbol size in mm for PS plots */
   int         ps_color;             /* Flag =1 use coloured symbols
                                        on type 2 PS plot, =0 do not */
   int         ps_addkey;            /* Flag =1 output key on type 1
                                        PS plot, =0 do not */
   XDl_laue_sim_label *labels[MAX_LABELS]; /* Pointers to index labels 
                                              structures */
   int num_labels;                   /* Number of labels */
   int move_label;                   /* =1 label moving in progress, =0 not
                                        (for type 2 plots) */
   int cur_label;                    /* Offset into labels array for current
                                        label (i.e. new label or label being
                                        moved */
   float cell[6];                    /* Cell parameters */
   float phis[4];                    /* Setting params Phix, Phiy, Phiz &
                                        Spindle */
   int latt[4];                      /* Crystal system, Lattice type, 
                                        Rotn axis, Beam axis */
   float ctof;                       /* Crystal to film distance */
   Pixmap save_area;                 /* Save area for label moving */
   int area_saved;                   /* Area saved flag 1=yes, 0=no */
   int sav_x;                        /* x position of saved area */
   int sav_y;                        /* y position of saved area */
   int sav_w;                        /* width of saved area */
   int sav_h;                        /* height of saved area */
   Pixmap mark_area;                 /* Save area for marked reflection */
   int mark_saved;                   /* Mark area saved flag 1=yes, 0=no */
   int mark_x;                       /* x position of saved mark area */
   int mark_y;                       /* y position of saved mark area */
   int mark_w;                       /* width of saved mark area */
   int mark_h;                       /* height of save mark area */

   int gnom;                         /* Gnomonic projection flag 1 yes, 
                                        0 no */
   float gdist;                      /* Distance for gnomonic projection */
   float gnomr;                      /* Gnomonic radius as input */
}  
   XDl_laue_sim;

/* Define routine types */

void xdl_laue_sim_rp1();
void xdl_laue_sim_rp2();
void xdl_laue_sim_rp3();
void xdl_laue_sim_rp4();
void xdl_laue_sim_rp5();
void xdl_laue_sim_ep1();
void xdl_laue_sim_cp1();
void xdl_laue_sim_cp2();
void xdl_laue_sim_cp3();
void xdl_laue_sim_cp4();
void xdl_laue_sim_cp5();
void xdl_laue_sim_cp6();
void xdl_laue_sim_cp7();
void xdl_laue_sim_cp8();
void xdl_laue_sim_cp9();
void xdl_laue_sim_cp10();
void xdl_laue_sim_cp11();
void xdl_laue_sim_cp12();
void xdl_laue_sim_cp13();
void xdl_laue_sim_cp14();
void xdl_laue_sim_tidy_up();
void xdl_laue_sim_setcolors();
void xdl_laue_sim_mult();
void xdl_laue_sim_redraw1();
void xdl_laue_sim_redraw2();
void xdl_laue_sim_symb1();
void xdl_laue_sim_drawkey();
void xdl_laue_sim_symb2();
void xdl_laue_sim_digit();
void xdl_laue_sim_label();
void xdl_laue_sim_mark();
int xdl_laue_sim_isalabel();
int xdl_laue_sim_relabel();
void xdl_laue_sim_labhkl();
void xdl_laue_sim_nodidx();
void xdl_laue_sim_reset_labels();
void xdl_laue_sim_save_mark();
void xdl_laue_sim_restore_mark();
void xdl_laue_sim_save_area();
void xdl_laue_sim_restore_area();
void xdl_laue_sim_hsv_to_rgb();
void xdl_laue_sim_ps();
void xdl_laue_sim_pshead();
void xdl_laue_sim_psdraw1();
void xdl_laue_sim_psdraw2();
void xdl_laue_sim_pslabel();
void xdl_laue_sim_psdigit();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_laue_sim xdlf_laue_sim_
#define xdlf_laue_sim_getsize xdlf_laue_sim_getsize_
#define xdlf_laue_sim_new xdlf_laue_sim_new_

#endif

#if  LINKTYP == 2

#define xdlf_laue_sim XDLF_LAUE_SIM
#define xdlf_laue_sim_getsize XDLF_LAUE_SIM_GETSIZE
#define xdl_laue_sim_new XDLF_LAUE_SIM_NEW

#endif

void xdlf_laue_sim();
void xdlf_laue_sim_getsize();
void xdlf_laue_sim_new();

/*-C:*/
int xdl_laue_sim (int vh, int vh_parent, int x, int y, int cset, int type, 
                  int size, int nspot, int * hkl, float * xf, float * yf, 
                  float * lam, float * dthr, int * multi, int * flags, 
                  int * nodhkls, int * idxlam, int * idxdmin, int * idxnod, 
                  float filmr, float gnomr, float lmin, float lmax,  
                  float dmin, float d_lmin, float d_lmax, float d_dmin, 
                  float * cell, float * phis, int * latt, float ctof,
                  int min_width, int min_height)
/*end*/
 
/*-Parameters:
int       vh;              User selected view-object handle (R)
int       vh_parent;       View-object handle for the parent base frame, 
                           if 0 then a base  level frame is created to
                           enclose the view-object (R)
int       x;               x coordinate for the view-object.
                           If no parent may be -1 to give default x (R)
int       y;               y coordinate for the view-object.
                           If no parent may be -1 to give default y (R)
int       cset;            Number of the colorset to be used; must
                           have 53 or more colorcells allocated (R) 
int       type;            Simulation type =1 Colour, =2 Monochrome with
                           soft limit sliders etc.
int       size;            Pattern display area half width/height in
                           pixels (R)
int       nspot;           Number of spots predicted (R)
int       hkl[];           Array of 'nspot' 'hkl' packed indices 
                           (as LRL) (R)
float     xf[];            Array of 'nspot' 'xf' coordinates in mm from
                           the pattern centre (R) 
float     yf[];            Array of 'nspot' 'yf' coordinates in mm from
                           the pattern centre (R) 
float     lam[];           Array of 'nspot' lambda values (R)
float     dthr[];          Array of 'nspot' (1/dmin)**2 threshold values
                           (R)
int       multi[];         Array of 'nspot' multiplicity flags (format
                           as for 'LRL' data 
                               = 1 single
                               > 1 2**20*mult + 2**12*min-hrm 
                                   + 2**4*max-hrm + harm_inc 
int       flags[];         Array of 'nspot' reflection flags
                           Flags format as for 'LRL' data
                           Bit 0 set = spatial overlap; this is the only
                                       bit used by this routine.
                           (R)
int       nodhkls[];       Array of 'nspot' packed nodal indices; packed
                           as 2**20(h+512)+2**10(k+512)+l+512 (R)
int       idxlam[];        Index array for data sorted by lambda, 'nspot'
                           values from 1 to nspot (R)
int       idxdmin[];       Index array for data sorted by dmin 'nspot'
                           values from 1 to nspot (R)
int       idxnod[];        Index array for data sorted by packed nodal
                           index from 1 to nspot (R)
float     filmr;           Pattern radius in mm (R)
float     gnomr;           Minimum radius in mm (>0.1) from image 
                           centre for conversion of plot to a gnomonic 
                           projection. For a normal Laue simulation, 
                           give a value of 0.0 (R) 
float     lmin;            Lambda minimum of prediction (R)
float     lmax;            Lambda maximum of prediction (R)
float     dmin;            Dmin value of prediction (R)
float     d_lmin;          Lambda interval in Angstoms for lambda-min
                           slider (R)
float     d_lmax;          Lambda interval in Angstoms for lambda-max
                           slider (R)
float     d_dmin;          Dmin interval in Angstoms for dmin slider 
                           (R)
float     cell[6];         Real cell dimensions a, c, c, alpha, beta, 
                           gamma (R)
float     phis[4];         Setting angles phix, phiy, phiz, spindle (R)
int       latt[4];         Crystal system, lattice, rotation axis,
                           beam axis (R)
float     ctof;            Crystal to film distance (mm) 
int       min_width;       The program will calculate the width required
                           for the view-object based on the display area
                           width, control panel width etc. If this is
                           less than min_width then min_width will be
                           used instead and the layout altered to reflect
                           this (R)
int       min_height;      minimum height cf min_width (R)

 Note: Data arrays hkl[],.....idxnod[] must remain in place during
         the lifetime of the view-object 

-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Unable to allocate memory for x, y pixel positions
                    arrays.
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_laue_sim * gd;          /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int i;                      /* Temp/loop variable */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int area_width;             /* Simulation display area width */
   int panel_1a_width;         /* Width for 'a' panel for type 1
                                  display (show reflection details) */
   int panel_1b_width;         /* Width for 'b' panel for type 1
                                  display */
   int panel_1_height;         /* Height of panels for type 1 display */
   int panel_2a_width;         /* Width for 'a' panel for type 2
                                  display (show reflection details) */
   int panel_2b_width;         /* Width for 'b' panel for type 2
                                  display (control area) */
   int panel_2_height;         /* Height of panels for type 2 display */
   int panel_ps_width;         /* Postscript control panel width */
   int panel_ps_height;        /* Postscript control panel height */
   int area_height;            /* Simulation display area height */
   int panels_width;           /* Control panels width needed */
   int panels_height;          /* Control panels height needed */
   int x_area;                 /* x position of display area within 
                                  the view-object*/
   int y_area;                 /* y position of display area within
                                  the view-object */
   int x_panel_1a;             /* x position of control panel 'a' for type 1 */
   int y_panel_1a;             /* y position of control panel 'a' for type 1 */
   int x_panel_2a;             /* x position of control panel 'a' for type 2 */
   int y_panel_2a;             /* y position of control panel 'a' for type 2 */
   int x_panel_1b;             /* x position of control panel 'b' for type 1 */
   int y_panel_1b;             /* y position of control panel 'b' for type 1 */
   int x_panel_2b;             /* x position of control panel 'b' for type 2 */
   int y_panel_2b;             /* y position of control panel 'b' for type 2 */
   int x_panel_ps;             /* x position of Postscript control panel */
   int y_panel_ps;             /* y position of Postscript control panel */
   int ytop;                   /* Top allowed y position for display area */
   float d;                    /* Temporary distance value */
   float r1;                   /* Laue spot distance from centre */
   float r2;                   /* Gnomonic spot distance from centre */
   float twoth;                /* 2-theta value */
   float thg;                  /* Gnomonic theta angle */
   float ratio;                /* r2/r1 ratio */
   float piov2 = 1.5707963;    /* Pi/2 */
   int font;                   /* Font type for control panel items */
   int panel_x;                /* x position for panel object */
   int panel_y;                /* y position for panel object */
   int slider_width;           /* slider width */
   int start_pos;              /* Slider start position */
   int edge;                   /* Edge offset for panel positioning */
   char str[20];               /* Temp string */
   XColor cursor_fg;        /* Structure for setting cross cursor foreground */
   XColor cursor_bg;        /* Structure for setting cross cursor background */
   XColor colr_cell;        /* Structure for setting colour display colours */
   float hue, sat, val;     /* HSV (Hue/saturation/value) colour components */
   float r, g, b;           /* RGB (Red/Green/Blue) colour components */
   Pixmap cursor_source;    /* Pixmap for cursor source */
   Pixmap cursor_mask;      /* Pixmap for cursor mask */
   static unsigned char curs15_source_bits[] ={
      0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x33, 0x66, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static unsigned char curs15_mask_bits[] = {
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x3f, 0x7e, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static char *sym2_strings[] = 
   {
      "Small",
      "Medium",
      "Large"
   };
   static char *highlight2_strings[] =
   {
      "None",
      "Mults",
      "Nodals"
   };
   static char *labels2_strings[] =
   {
      "Off",
      "Min. hkl",
      "Max. hkl",
      "Nodal hkl"
   };
   static char *color_coding_strings[] = 
   {
      "Wavelength",
      "Multiplicity",
      "Wav./mult.",
      "Nodality"
   };
   static char *spot_type_strings[] =
   {
      "All spots",
      "Singles",
      "Multiples"
   };
   static char *spatials_strings[] =
   {
      "Included",
      "Excluded",
      "Spatials only"
   };
   static char *spotsize_strings[] =
   {
      "Small",
      "Medium",
      "Large"
   };
   static char *key_opt_strings[] =
   {
      "On",
      "Off"
   };
   static float hues[] = {0.0, 40.0, 60.0, 80.0, 160.0, 180.0, 200.0, 240.0};

   /* Initialisations */

   err = 0;
   font = 2;

   if (type<1||type>2) type =1;

   /* Get size requirements */

   area_width = 2*size + 1 + 2;
   area_height = 2*size + 1 + 2;
   switch (type)
   {
      case 1:
         panel_1a_width = PANEL_1A_WIDTH;
         panel_1b_width = PANEL_1B_WIDTH;
         panel_1_height = PANEL_1_HEIGHT;
         panels_width = panel_1a_width + 4 + panel_1b_width;
         panels_height = panel_1_height;
         break;
     
      case 2:
         panel_2a_width = PANEL_2A_WIDTH;
         panel_2b_width = PANEL_2B_WIDTH;
         panel_2_height = PANEL_2_HEIGHT;
         panels_width = panel_2a_width + 4 + panel_2b_width;
         panels_height = panel_2_height;
         break;
   }
   if (panels_width>area_width)
   {
      width = panels_width + 2*BORDER_WIDTH + 2;
   }
   else
   {
      width = area_width + 2*BORDER_WIDTH + 2;
   }
   height = area_height + SEP_H_MIN + panels_height 
            + 4 + PANEL_PS_HEIGHT
            + 2*BORDER_HEIGHT + 2;
   if (width<min_width) width = min_width;
   if (height<min_height) height = min_height;

   /* Get positions for panels */

   switch(type)
   {   
      case 1:
         x_panel_1a = (width - 2 - panels_width)/2;
         y_panel_1a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         edge = (width - 2 - panel_1a_width - panel_1b_width)/4;
         x_panel_1a = edge;
         y_panel_1a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         x_panel_1b = width - 2 - edge - panel_1b_width;
         y_panel_1b = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         break;
      case 2:
         edge = (width - 2 - panel_2a_width - panel_2b_width)/4;
         x_panel_2a = edge;
         y_panel_2a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         x_panel_2b = width - 2 - edge - panel_2b_width;
         y_panel_2b = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         break;
   }
   x_area = (width - 2 - area_width)/2;
   x_panel_ps = BORDER_WIDTH;
   y_panel_ps = BORDER_HEIGHT;
   panel_ps_width = width - 2*BORDER_WIDTH - 2;
   panel_ps_height = PANEL_PS_HEIGHT;
   ytop = BORDER_HEIGHT + panels_height + 4 + PANEL_PS_HEIGHT + SEP_H_MIN;
   y_area = ytop + (height - 2 - area_height - ytop)/2;

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_laue_sim", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_laue_sim *) malloc (sizeof(XDl_laue_sim));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 1061, NULL,
                        xdl_laue_sim_tidy_up);

   /* Some initialisations for global data area */

   gd->wframe = wframe;
   gd->cset = cset;
   gd->type = type;
   gd->half_size = size;
   gd->filmr = filmr;
   gd->lmin = lmin;
   gd->lmax = lmax;
   gd->dmin = dmin;
   gd->nspot = nspot;
   gd->hkl = hkl;
   gd->xf = xf;
   gd->yf = yf;
   gd->lam = lam;
   gd->dthr = dthr;
   gd->multi = multi;
   gd->flags = flags;
   gd->nodhkls = nodhkls;
   gd->idxlam = idxlam;
   gd->idxdmin = idxdmin;
   gd->idxnod = idxnod;
   gd->lmin_interval = d_lmin;
   gd->lmax_interval = d_lmax;
   gd->dmin_interval = d_dmin;
   gd->cur_slider = 1;
   gd->cur_lmin = lmin;
   gd->cur_lmax = lmax;
   gd->cur_dmin = dmin;
   gd->color_type = 0;
   if (XDL_color>0&&XDL_csets_alloc==1) gd->color_type = 1;
   gd->recalc_mult = 1;
   gd->size_sym2 = 2;
   gd->opt_highlight2 = 1;
   gd->opt_labels2 = 1;
   gd->last_labtyp = 0;
   gd->nodidx_cur = 4;
   gd->disp_ref = -1;
   gd->color_coding = 1;
   gd->spot_type = 1;
   gd->spatials = 1;
   gd->spotsize = 2;
   gd->key_opt = 1;
   gd->num_labels = 0;
   gd->move_label = 0;
   gd->cur_label = -1;
   gd->area_saved = 0;
   gd->mark_saved = 0;
   gd->ps_on = 0;
   gd->psfil = 0;
   gd->mm_to_p = 2.8346;
   for (i=0;i<6;++i) gd->cell[i] = cell[i];
   for (i=0;i<4;++i) gd->phis[i] = phis[i];
   for (i=0;i<4;++i) gd->latt[i] = latt[i];
   gd->ctof = ctof;
   gd->gnom = 0;
   gd->gnomr = 0.0;
   gd->gdist = 0.0;
   if (gnomr>0.1)
   {
      gd->gnom = 1;
      thg = piov2 - 0.5*atanf(gnomr/ctof);
      gd->gdist = filmr/tanf(thg);
      gd->gnomr = gnomr;
   }

   /* Set colour pixels array for colour display */

   if (type==1)
   {
      if (XDL_color>0)
      {
         for (i=0;i<8;++i)
         {
            hue = hues[i];
            sat = 1.0;
            val = 1.0;
            xdl_laue_sim_hsv_to_rgb (hue, sat, val, &r, &g, &b);
            gd->ps_r[i] = r;
            gd->ps_g[i] = g;
            gd->ps_b[i] = b;
            colr_cell.red = 65535 * r;
            colr_cell.green = 65535 * g;
            colr_cell.blue = 65535 * b;
            colr_cell.flags = DoRed|DoGreen|DoBlue;
            if (XDL_csets_alloc==1)
            {
               colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
               XStoreColor (XDL_display, XDL_colormap, &colr_cell);
            }
            else
            {
               XAllocColor (XDL_display, XDL_colormap, &colr_cell);
            }
            gd->colrpix[i] = colr_cell.pixel;
         }
      }
      else
      {
         for (i=0;i<8;++i) gd->colrpix[i] = XDL_whitepixel;
      }
   }

   /* Set pixels array if Interactive type with writeable colour map */

   if (type==2&&gd->color_type==1)
   {
     for (i=0;i<53;++i) gd->pixels[i]=XDL_colorsets[gd->cset].pixels[i];
   }


   /* Allocate memory to store spot positions in pixels */

   gd->xpix = (short *) malloc (nspot*sizeof(short));
   if (gd->xpix==0) {err |= 2; return err;}
   gd->ypix = (short *) malloc (nspot*sizeof(short));
   if (gd->ypix==0) {free ((char *)gd->xpix); err |= 2; return err;}
   gd->mult = (unsigned char *) malloc (nspot*sizeof(unsigned char));
   if (gd->mult==0)
   { 
      free ((char *)gd->xpix); 
      free ((char *)gd->ypix);
      err |= 2;
      return err;
   }
   gd->max_harmonic = (unsigned char *) malloc (nspot*sizeof(unsigned char));
   if (gd->max_harmonic==0)
   { 
      free ((char *)gd->xpix); 
      free ((char *)gd->ypix);
      free ((char *)gd->mult);
      err |= 2;
      return err;
   }
   gd->harm_inc = (unsigned char *) malloc (nspot*sizeof(unsigned char));
   if (gd->max_harmonic==0)
   { 
      free ((char *)gd->xpix); 
      free ((char *)gd->ypix);
      free ((char *)gd->mult);
      free ((char *)gd->max_harmonic);
      err |= 2;
      return err;
   }
   gd->colr = (unsigned char *) malloc (nspot*sizeof(unsigned char));
   if (gd->colr==0)
   { 
      free ((char *)gd->xpix); 
      free ((char *)gd->ypix);
      free ((char *)gd->mult);
      free ((char *)gd->max_harmonic);
      free ((char *)gd->harm_inc);
      err |= 2;
      return err;
   }

   /* Get spot pixel positions (Note: reversal of sign needed for y) */

   for (i=0;i<nspot;++i)
   {
      if (gd->gnom)
      {
         r1 = sqrtf (gd->xf[i]*gd->xf[i]+gd->yf[i]*gd->yf[i]);
         if (r1>=gnomr)
	 {
            twoth = atanf (r1/gd->ctof);
            thg = piov2 - 0.5*twoth;
            r2 = gd->gdist*tanf(thg);
            ratio = r2/r1;
            d = ratio*((gd->xf[i]/gd->filmr)*(float)gd->half_size);
            gd->xpix[i] = gd->half_size + ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
            d = ratio*((gd->yf[i]/gd->filmr)*(float)gd->half_size);
            gd->ypix[i] = gd->half_size - ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
         }
         else
	 {
            gd->xpix[i] = -1;
            gd->ypix[i] = -1;
         }
      }
      else
      {
         d = (gd->xf[i]/gd->filmr)*(float)gd->half_size;
         gd->xpix[i] = gd->half_size + ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
         d = (gd->yf[i]/gd->filmr)*(float)gd->half_size;
         gd->ypix[i] = gd->half_size - ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
      }
   }

   /* Calculate multiplicities for colour plot type 1 */

   if (type==1)
   {
      gd->cur_lmin = lmin;
      gd->cur_lmax = lmax;
      gd->cur_dmin = dmin;
      xdl_laue_sim_mult (gd);
   }

   /* Set up type 2 colours if needed */
  
   if (type==2)
   {
      xdl_laue_sim_setcolors (gd, 1, 0);
   }

   /* Create paint window for Laue simulation display area*/

   gd->wdisp = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                                         area_width, area_height, cset);

   xdl_add_window (vh, gd->wdisp, xdl_laue_sim_rp1, xdl_laue_sim_ep1);

   /* Prepare for event handling */

   switch (type)
   {
      case 1:
         XSelectInput (XDL_display, gd->wdisp, ExposureMask|StructureNotifyMask|
                      ButtonPressMask);
         break;
      case 2:
         XSelectInput (XDL_display, gd->wdisp, ExposureMask|StructureNotifyMask|
                      ButtonPressMask|ButtonReleaseMask|PointerMotionMask);
         break;
   }

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wdisp, 0, NULL);

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);

   /* Create display area cursor */

   cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                        (char *) curs15_source_bits, 15, 15);

   cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                        (char *) curs15_mask_bits, 15, 15);

   cursor_fg.red = 0;
   cursor_fg.green = 0;
   cursor_fg.blue = 0;
   cursor_fg.flags = DoRed|DoGreen|DoBlue;

   cursor_bg.red = 65535;
   cursor_bg.green = 65535;
   cursor_bg.blue = 65535;
   cursor_bg.flags = DoRed|DoGreen|DoBlue;
   
   gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                     &cursor_fg, &cursor_bg, 7, 7);

   XFreePixmap (XDL_display, cursor_source);
   XFreePixmap (XDL_display, cursor_mask);

   XDefineCursor (XDL_display, gd->wdisp, gd->cursor);

   /* Create Pixmap save areas (for label moving etc.) */

   if (type==2)
   {
      gd->save_area = XCreatePixmap (XDL_display, gd->wdisp, SAV_SIZ, SAV_SIZ,
                      XDL_pw_depth);
   }
   gd->mark_area = XCreatePixmap (XDL_display, gd->wdisp, MARK_SIZ, MARK_SIZ,
                   XDL_pw_depth);

   /* Create Panel paint windows */
   /*----------------------------*/

   /* Postscript options control panel */

   gd->wpanel_ps = xdl_create_paint_window (wframe, x_panel_ps, y_panel_ps,
                           1, panel_ps_width, panel_ps_height, cset);
   xdl_add_window (vh, gd->wpanel_ps, xdl_laue_sim_rp5, NULL);

   XSelectInput (XDL_display, gd->wpanel_ps, ExposureMask|
                 StructureNotifyMask|ButtonPressMask);

   panel_x = 4;
   panel_y = 2;
   gd->psbutton = xdl_panel_button (vh, gd->wpanel_ps, panel_x, panel_y,
                         "PS", 2, 2, 2, 0, 0, 1, 1, 1, xdl_laue_sim_cp13);

   panel_x = 32;
   panel_y = 3;
   gd->ps_io = xdl_panel_io (vh, gd->wpanel_ps, panel_x, panel_y,
                            40, 20, 200, 2, 0, 1, 
                            panel_ps_width-panel_x-2, 0, 
                            xdl_laue_sim_cp14);


   /* Other control panels */

   switch(type)
   {
      case 1:
         /* Set up reflection details panel area */

         gd->wrefln = xdl_create_paint_window (wframe, x_panel_1a, y_panel_1a,
                                1, panel_1a_width, panel_1_height, cset);
         xdl_add_window (vh, gd->wrefln, xdl_laue_sim_rp3, NULL);

         XSelectInput (XDL_display, gd->wrefln, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         panel_x = 2;
         panel_y = 4;
         gd->reflab[0] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         "Reflection Details", 25, font, 0, 0, 0, 1);
         panel_y += 15;
         gd->reflab[1] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[2] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[3] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[4] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);

         /* Control panel */

         gd->wpanel = xdl_create_paint_window (wframe, x_panel_1b, y_panel_1b,
                                 1, panel_1b_width, panel_1_height, cset);
         xdl_add_window (vh, gd->wpanel, xdl_laue_sim_rp2, NULL);

         /* Prepare for event handling */

         XSelectInput (XDL_display, gd->wpanel, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         /* Control panel items */

         /* Colour coding choices */
         panel_x = 4;
         panel_y = 4;
         gd->colorcode_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Colour coding", 4, color_coding_strings, 
                          gd->color_coding, font, 
                          0, 0, 0, xdl_laue_sim_cp6);

         /* Spot type choices */
         panel_x = 4;
         panel_y = 24;
         gd->spot_type_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spot types   ", 3, spot_type_strings, 
                          gd->spot_type, font, 
                          0, 0, 0, xdl_laue_sim_cp7);

         /* Spatials choices */
         panel_x = 4;
         panel_y = 44;
         gd->spatial_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spatials     ", 3, spatials_strings, 
                          gd->spatials, font, 
                          0, 0, 0, xdl_laue_sim_cp8);

         /* Spot size choices */
         panel_x = 200;
         panel_y = 4;
         gd->spotsize_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spot size", 3, spotsize_strings, 
                          gd->spotsize, font, 
                          0, 0, 0, xdl_laue_sim_cp9);

         /* Key choices */
         panel_x = 200;
         panel_y = 24;
         gd->key_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Key      ", 2, key_opt_strings, 
                          gd->key_opt, font, 
                          0, 0, 0, xdl_laue_sim_cp10);
         /* Nodal index value */
         panel_x = 266;
         panel_y = 44;
         gd->nodal_sel = xdl_panel_value (vh, gd->wpanel,
                        panel_x, panel_y, "Nidx", "4", 2, 2,
                        0, 0, 0, xdl_laue_sim_cp12);
         break;
      case 2:
         /* Set up reflection details panel area */

         gd->wrefln = xdl_create_paint_window (wframe, x_panel_2a, y_panel_2a,
                                1, panel_2a_width, panel_2_height, cset);
         xdl_add_window (vh, gd->wrefln, xdl_laue_sim_rp3, NULL);

         XSelectInput (XDL_display, gd->wrefln, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         panel_x = 2;
         panel_y = 4;
         gd->reflab[0] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         "Reflection Details", 25, font, 0, 0, 0, 1);
         panel_y += 15;
         gd->reflab[1] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[2] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[3] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[4] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);

         /* Set up controls panel area */

         gd->wpanel = xdl_create_paint_window (wframe, x_panel_2b, y_panel_2b,
                                  1, panel_2b_width, panel_2_height, cset);
         xdl_add_window (vh, gd->wpanel, xdl_laue_sim_rp4, NULL);

         XSelectInput (XDL_display, gd->wpanel, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         /* Set up Control Panel Items */

         /* Lambda-min slider & label */
         panel_x = 2;
         panel_y = 4;
         slider_width = 100;
         start_pos = 0;
         gd->cur_lmin_slider_val = start_pos;
         gd->lmin_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                           "Lambda-min", font, slider_width, start_pos, 
                           0, 0, 0, xdl_laue_sim_cp1);
         panel_x += 195;
         sprintf (str,"%.3f",gd->cur_lmin);
         gd->lmin_curval = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                           str, 6, font, 0, 0, 0, 0);
         /* Lambda-max slider & label */
         panel_x = 4;
         panel_y = 24;
         slider_width = 100;
         start_pos = 100;
         gd->cur_lmax_slider_val = start_pos;
         gd->lmax_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                           "Lambda-max", font, slider_width, start_pos, 
                           0, 0, 0, xdl_laue_sim_cp2);
         panel_x += 195;
         sprintf (str,"%.3f",gd->cur_lmax);
         gd->lmax_curval = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                           str, 6, font, 0, 0, 0, 0);
         /* Dmin slider & label */
         panel_x = 4;
         panel_y = 44;
         slider_width = 100;
         start_pos = 0;
         gd->cur_dmin_slider_val = start_pos;
         gd->dmin_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                           "Dmin      ", font, slider_width, start_pos, 
                           0, 0, 0, xdl_laue_sim_cp3);
         panel_x += 195;
         sprintf (str,"%.3f",gd->cur_dmin);
         gd->dmin_curval = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                           str, 6, font, 0, 0, 0, 0);

         /* Symbol size choices */
         panel_x = 240;
         panel_y = 4;
         gd->sym2_size = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                         "Symbols", 3, sym2_strings, gd->size_sym2, font, 
                          0, 0, 0, xdl_laue_sim_cp4);
         /* Highlight options choices */
         panel_x = 240;
         panel_y = 24;
         gd->highlight2_opt = xdl_panel_choice (vh, gd->wpanel, 
                          panel_x, panel_y,
                         "Highlight", 3, highlight2_strings, 
                          gd->opt_highlight2, font, 0, 0, 0,
                          xdl_laue_sim_cp5);
         /* Labels options choices */
         panel_x = 240;
         panel_y = 44;
         gd->labels2_opt = xdl_panel_choice (vh, gd->wpanel, 
                          panel_x, panel_y,
                         "Labels", 4, labels2_strings, 
                          gd->opt_labels2, font, 0, 0, 0,
                          xdl_laue_sim_cp11);
         /* Nodal index value */
         panel_x = 358;
         panel_y = 4;
         gd->nodal_sel = xdl_panel_value (vh, gd->wpanel,
                        panel_x, panel_y, "Nidx", "4", 2, 2,
                        0, 0, 0, xdl_laue_sim_cp12);
         break;
   }

   return 0;
}

/* Fortran binding: xdlf_laue_sim */

void xdlf_laue_sim (ivh, ivhparent, ix, iy, icset, itype, isize, nspot,
                    ihkl, xf, yf, alam, dthr, multi, flags, nodhkls,
                    idxlam, idxdmin, idxnod, filmr, gnomr,
                    almin, almax, dmin, d_lmin, d_lmax, d_dmin,
                    cell, phis, latt, ctof,
                    minw, minh, ierr)

int *ivh, *ivhparent, *ix, *iy, *icset, *itype, *isize, *nspot;
int *ihkl, *multi, *flags, *nodhkls, *idxlam, *idxdmin, *idxnod, *latt;
int *minw, *minh, *ierr;
float *xf, *yf, *alam, *dthr, *filmr, *gnomr, *almin, *almax; 
float *dmin, *cell, *phis;
float *d_lmin, *d_lmax, *d_dmin, *ctof;

{
   *ierr = xdl_laue_sim (*ivh, *ivhparent, *ix, *iy, *icset, *itype,
                         *isize, *nspot, ihkl, xf, yf, alam, dthr, 
                         multi, flags, nodhkls, idxlam, idxdmin, idxnod, 
                         *filmr, *gnomr, *almin, *almax, *dmin, 
                         *d_lmin, *d_lmax, *d_dmin, cell, phis, latt, *ctof,
                         *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_laue_sim_rp1        **
*********************************

Purpose: Repaint display area paint window 

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_laue_sim * gd;     /* Pointer to global data area */
   int i;                 /* Temp/loop variable */

   gd = (XDl_laue_sim *) global;

   /* Repaint code */

   switch (gd->type)
   {
      case 1:
         xdl_laue_sim_redraw1 (gd);
         break;

      case 2:
         xdl_laue_sim_redraw2 (gd);
         break;
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_rp2        **
*********************************

Purpose: Repaint control panel area for type 1 display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_laue_sim_rp3        **
*********************************

Purpose: Repaint reflection details panel area for type 1 or 2 display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_rp3 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_laue_sim_rp4        **
*********************************

Purpose: Repaint control panel area for type 2 display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_rp4 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_laue_sim_rp5        **
*********************************

Purpose: Repaint postscript control panel area

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_rp5 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_laue_sim_ep1        **
*********************************

Purpose: Event handling from display area paint window

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_laue_sim *gd;      /* Pointer to the global data area */
   int i, k;              /* Temp/loop variables */
   int j;                 /* Array index */
   int x, y;              /* Mouse x, y position */
   int xd, yd;            /* x, y offsets from current spot */
   int dsq;               /* Distance squared from mouse position to
                             current spot */
   int dsqmin;            /* Current minimum value of dsq */
   int ii;                /* Index in list of reflections of selected 
                             reflection */
   int mult;              /* Multiplicity */
   int k1, k2;            /* Range of harmonics for selected reflection */
   char str[51];          /* Temp. string for labels */
   int llen;              /* Label length */
   int wl;                /* Width of labels including border */
   int xx, yy;            /* Temp x, y offsets */
   int xmin, ymin;        /* Min x, y values required to save labelling area */
   int xmax, ymax;        /* Max x, y values required to save labelling area */
   int xspot, yspot;      /* Spot position */
   XDl_laue_sim_label *lt;/* Temp pointer to label structure */
   XEvent xevent;         /* X event structure */
   float cur_dmin2;       /* Current value of 1/dmin**2 */
   int hlab, klab, llab;  /* Indices for label */
   float dmin;            /* dmin value */
   int h_n, k_n, l_n;     /* Nodal indices */
   int harmonic;          /* Harmonic number */
   int max_xy;            /* maximum x, y pixel position within window */
   int jj;                /* Temp variable for unpacking indices */
   int hh, kk, ll;        /* Unpacked indices */

   gd = (XDl_laue_sim *) global;
   cur_dmin2 = 1/((gd->cur_dmin)*(gd->cur_dmin));

  switch (gd->type)
  {
     case 1:
        switch (event->type)
        {
            case ButtonPress:
               if (event->xbutton.button==Button1)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->recalc_mult) xdl_laue_sim_mult(gd);
                  if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                  dsqmin = 1000;
                  ii = 0;
                  for (i=0;i<gd->nspot;++i)
                  {
                     j = gd->idxlam[i] - 1;
                     if (gd->spatials!=1)
                     {
                        if (gd->spatials==2&&(gd->flags[j]&0X1)!=0) continue;
                        if (gd->spatials==3&&(gd->flags[j]&0X1)==0) continue;
                     }
                     xd = x - gd->xpix[j];
                     yd = y - gd->ypix[j];
                     dsq = xd*xd + yd*yd;
                     if (dsq<dsqmin)
                     {
                        ii = j;
                        dsqmin = dsq;
                     }
                  }
                  if (dsqmin<=18)
                  {
                     gd->disp_ref = ii;
                     jj = gd->hkl[ii];
                     hh = jj/TWO20;
                     jj = jj - TWO20*hh;
                     hh -= 512;
                     kk = jj/TWO10;
                     ll = jj-TWO10*kk - 512;
                     kk -= 512;
                     sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                     xdl_panel_label_set(gd->reflab[1],str,0,0);
                     sprintf (str,"lambda: %.3f",gd->lam[ii]);
                     xdl_panel_label_set(gd->reflab[2],str,0,0);
                     mult = (int) gd->mult[ii];
                     k2 = (int) gd->max_harmonic[ii];
                     k1 = k2 - mult + 1;
                     if (mult>1)
                     {
                        sprintf (str,"mult:   %d (%d-%d)",mult,k1,k2);
                     }
                     else
                     {
                        sprintf (str,"mult:   %d",mult);
                     }
                     xdl_panel_label_set(gd->reflab[3],str,0,0);
                     sprintf (str,"xf, yf: %.2f, %.2f",gd->xf[ii],gd->yf[ii]);
                     xdl_panel_label_set(gd->reflab[4],str,0,0);
                     xdl_laue_sim_save_mark (gd, ii);
                     xdl_laue_sim_mark (gd, ii, 1);
                  }
                  else
                  {
                     gd->disp_ref = -1;
                     for (k=0;k<4;++k) 
                     xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  }
               }
               else if (event->xbutton.button==Button2 ||
                        event->xbutton.button==Button3)
               {
                  if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                  if (gd->disp_ref>=0)
                  {
                     gd->disp_ref = -1;
                     for (k=0;k<4;++k) 
                     xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  }
               }
               break;
        }
        break;

     case 2:
        switch (event->type)
        {
            case ButtonPress:
               if (event->xbutton.button==Button1)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (xdl_laue_sim_isalabel (gd, x, y, 0)) break;
                  if (gd->recalc_mult) xdl_laue_sim_mult(gd);
                  if (gd->area_saved) xdl_laue_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                  dsqmin = 1000;
                  ii = 0;
                  for (i=0;i<gd->nspot;++i)
                  {
                     j = gd->idxlam[i] - 1;
                     if (gd->lam[j]>=gd->cur_lmin &&
                         gd->lam[j]<=gd->cur_lmax &&
                         gd->dthr[j]<=cur_dmin2)
                     {
                        xd = x - gd->xpix[j];
                        yd = y - gd->ypix[j];
                        dsq = xd*xd + yd*yd;
                        if (dsq<dsqmin)
                        {
                           ii = j;
                           dsqmin = dsq;
                        }
                     }
                  }
                  if (dsqmin<=18)
                  {
                     gd->disp_ref = ii;
                     jj = gd->hkl[ii];
                     hh = jj/TWO20;
                     jj = jj - TWO20*hh;
                     hh -= 512;
                     kk = jj/TWO10;
                     ll = jj-TWO10*kk - 512;
                     kk -= 512;
                     sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                     xdl_panel_label_set(gd->reflab[1],str,0,0);
                     sprintf (str,"lambda: %.3f",gd->lam[ii]);
                     xdl_panel_label_set(gd->reflab[2],str,0,0);
                     mult = (int) gd->mult[ii];
                     k2 = (int) gd->max_harmonic[ii];
                     k1 = k2 - mult + 1;
                     if (mult>1)
                     {
                        sprintf (str,"mult:   %d (%d-%d)",mult,k1,k2);
                     }
                     else
                     {
                        sprintf (str,"mult:   %d",mult);
                     }
                     xdl_panel_label_set(gd->reflab[3],str,0,0);
                     sprintf (str,"xf, yf: %.2f, %.2f",gd->xf[ii],gd->yf[ii]);
                     xdl_panel_label_set(gd->reflab[4],str,0,0);
                     xdl_laue_sim_save_mark (gd, ii);
                     xdl_laue_sim_mark (gd, ii, 1);
                  }
                  else
                  {
                     gd->disp_ref = -1;
                     for (k=0;k<4;++k) 
                     xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  }
               }
               else if (event->xbutton.button==Button2)
               {
                  if (gd->opt_labels2==1) break;
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->recalc_mult) xdl_laue_sim_mult(gd);
                  if (gd->area_saved) xdl_laue_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                  if (xdl_laue_sim_isalabel (gd, x, y, 1))
                  {
                     lt = gd->labels[gd->cur_label];
                     xx = lt->x2 - lt->x1;
                     yy = lt->y2 - lt->y1;
                     lt->x1 = x - lt->xcoff;
                     lt->y1 = y - lt->ycoff;
                     lt->x2 = lt->x1 + xx;
                     lt->y2 = lt->y1 + yy;
                     xspot = gd->xpix[lt->idx];
                     yspot = gd->ypix[lt->idx];
                     if (lt->x1<xspot) { xmin=lt->x1; }else{ xmin=xspot; }
                     if (lt->y1<yspot) { ymin=lt->y1; }else{ ymin=yspot; }
                     if (lt->x2>xspot) { xmax=lt->x2; }else{ xmax=xspot; }
                     if (lt->y2>yspot) { ymax=lt->y2; }else{ ymax=yspot; }
                     if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
                     {
                        xdl_laue_sim_save_area (gd, xmin, ymin,
                            (xmax-xmin+1), (ymax-ymin+1));
                        xdl_laue_sim_label (gd, gd->cur_label, 1);
                     }
                     else
                     {
                        llen = strlen(lt->hklstr);
                        wl = 7*llen+4-6;
                        xdl_laue_sim_save_area (gd, lt->x1, lt->y1, wl, 12);
                        xdl_laue_sim_save_mark (gd, lt->idx);
                        xdl_laue_sim_mark (gd, lt->idx, 2);
                        xdl_laue_sim_label (gd, gd->cur_label, 0);
                     }
                     gd->move_label = 1;
                  }
                  else
                  {
                     dsqmin = 1000;
                     ii = 0;
                     for (i=0;i<gd->nspot;++i)
                     {
                        j = gd->idxlam[i] - 1;
                        if (gd->lam[j]>=gd->cur_lmin &&
                            gd->lam[j]<=gd->cur_lmax &&
                            gd->dthr[j]<=cur_dmin2)
                        {
                           xd = x - gd->xpix[j];
                           yd = y - gd->ypix[j];
                           dsq = xd*xd + yd*yd;
                           if (dsq<dsqmin)
                           {
                              ii = j;
                              dsqmin = dsq;
                           }
                        }
                     }
                     if (dsqmin<=18)
                     {
                        gd->disp_ref = ii;
                        jj = gd->hkl[ii];
                        hh = jj/TWO20;
                        jj = jj - TWO20*hh;
                        hh -= 512;
                        kk = jj/TWO10;
                        ll = jj-TWO10*kk - 512;
                        kk -= 512;
                        sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                        xdl_panel_label_set(gd->reflab[1],str,0,0);
                        sprintf (str,"lambda: %.3f",gd->lam[ii]);
                        xdl_panel_label_set(gd->reflab[2],str,0,0);
                        mult = (int) gd->mult[ii];
                        k2 = (int) gd->max_harmonic[ii];
                        k1 = k2 - mult + 1;
                        if (mult>1)
                        {
                           sprintf (str,"mult:   %d (%d-%d)",mult,k1,k2);
                        }
                        else
                        {
                           sprintf (str,"mult:   %d",mult);
                        }
                        xdl_panel_label_set(gd->reflab[3],str,0,0);
                        sprintf (str,"xf, yf: %.2f, %.2f",gd->xf[ii],gd->yf[ii]);
                        xdl_panel_label_set(gd->reflab[4],str,0,0);
                        if (gd->num_labels<MAX_LABELS)
                        {
                           lt= (XDl_laue_sim_label *) 
                               malloc(sizeof(XDl_laue_sim_label));
                           if (lt!=0)
                           {
                              gd->labels[gd->num_labels] = lt;
                              gd->num_labels += 1;
                              lt->idx = ii;
                              xspot = gd->xpix[ii];
                              yspot = gd->ypix[ii];
                              xdl_laue_sim_nodidx (gd, ii, 
                                   &h_n, &k_n, &l_n, &harmonic);
                              lt->lamnod = harmonic*gd->lam[ii];
                              dmin = 1.0/sqrt ((double) gd->dthr[ii]);
                              lt->dnod = harmonic*dmin;
                              xdl_laue_sim_labhkl (gd, ii, &hlab, &klab,
                                                   &llab);
                              sprintf (lt->hklstr,"%d %d %d",
                                      hlab, klab, llab);
                              lt->h = hlab;
                              lt->k = klab;
                              lt->l = llab;
                              lt->disp = 1;
                              llen = strlen(lt->hklstr);
                              wl = 7*llen+4-6;
                              lt->xcoff = wl/2;
                              lt->ycoff = 5;
                              lt->x1 = xspot - lt->xcoff;
                              lt->y1 = yspot - lt->ycoff;
                              lt->x2 = lt->x1 + wl - 1;
                              lt->y2 = lt->y1 + 11;
                              gd->cur_label = gd->num_labels - 1;
                              if (lt->x1<xspot) 
                                 { xmin=lt->x1; }else{ xmin=xspot; }
                              if (lt->y1<yspot) 
                                 { ymin=lt->y1; }else{ ymin=yspot; }
                              if (lt->x2>xspot) 
                                 { xmax=lt->x2; }else{ xmax=xspot; }
                              if (lt->y2>yspot) 
                                 { ymax=lt->y2; }else{ ymax=yspot; }
                              if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
                              {
                                 xdl_laue_sim_save_area (gd, xmin, ymin,
                                     (xmax-xmin+1), (ymax-ymin+1));
                                 xdl_laue_sim_label (gd, gd->cur_label, 1);
                              }
                              else
                              {
                                 xdl_laue_sim_save_area (gd, xmin, ymin,
                                      wl, 12);
                                 xdl_laue_sim_save_mark (gd, ii);
                                 xdl_laue_sim_mark (gd, ii, 2);
                                 xdl_laue_sim_label (gd, gd->cur_label, 0);
                              }
                              gd->move_label = 1;
                           }
                        }
                     }
                     else
                     {
                        gd->disp_ref = -1;
                        for (k=0;k<4;++k) 
                        xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                     }
                  }
               }
               else if (event->xbutton.button==Button3)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->area_saved) xdl_laue_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                  gd->disp_ref = -1;
                  for (k=0;k<4;++k) 
                  xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  if (xdl_laue_sim_isalabel (gd, x, y, 1))
                  {
                     gd->num_labels -= 1;
                     gd->move_label = 0;
                  }
               }
               break;
            case ButtonRelease:
               if (event->xbutton.button==Button2)
               {
                  if (gd->move_label)
                  {
                     gd->move_label = 0;
                     if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
                     if (gd->area_saved) xdl_laue_sim_restore_area (gd);
                     xdl_laue_sim_label (gd, gd->cur_label, 1);
                     if (gd->disp_ref>=0)
                     {
                        xdl_laue_sim_save_mark (gd, gd->disp_ref);
                        xdl_laue_sim_mark (gd, gd->disp_ref, 1);
                     }
                  }
                  gd->move_label = 0;
                  gd->cur_label = -1;
               }
               break;
            case MotionNotify:
               x = event->xmotion.x;
               y = event->xmotion.y;
               while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
                      MotionNotify, &xevent))
               {
                  x = xevent.xmotion.x;
                  y = xevent.xmotion.y;
               }
               max_xy = 2*gd->half_size;
               if (x<0) x = 0;
               if (x>max_xy) x = max_xy;
               if (y<0) y = 0;
               if (y>max_xy) y = max_xy;
               if (gd->move_label==0) break;
               if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
               if (gd->area_saved) xdl_laue_sim_restore_area (gd);
               lt = gd->labels[gd->cur_label];
               xx = lt->x2 - lt->x1;
               yy = lt->y2 - lt->y1;
               wl = xx + 1;
               lt->x1 = x - lt->xcoff;
               lt->y1 = y - lt->ycoff;
               lt->x2 = lt->x1 + xx;
               lt->y2 = lt->y1 + yy;
               xspot = gd->xpix[lt->idx];
               yspot = gd->ypix[lt->idx];
               if (lt->x1<xspot) { xmin=lt->x1; }else{ xmin=xspot; }
               if (lt->y1<yspot) { ymin=lt->y1; }else{ ymin=yspot; }
               if (lt->x2>xspot) { xmax=lt->x2; }else{ xmax=xspot; }
               if (lt->y2>yspot) { ymax=lt->y2; }else{ ymax=yspot; }
               if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
               {
                  xdl_laue_sim_save_area (gd, xmin, ymin,
                      (xmax-xmin+1), (ymax-ymin+1));
                  xdl_laue_sim_label (gd, gd->cur_label, 1);
               }
               else
               {
                  xdl_laue_sim_save_area (gd, lt->x1, lt->y1, wl, 12);
                  xdl_laue_sim_save_mark (gd, lt->idx);
                  xdl_laue_sim_mark (gd, lt->idx, 2);
                  xdl_laue_sim_label (gd, gd->cur_label, 0);
               }
               break;
         }
         break;
   }

   return;
}

/*
*********************************
**     xdl_laue_sim_cp1        **
*********************************

Purpose: Callback for control panel lambda-min slider

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp1 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int recalc;           /* Flag =1 recalculate colours etc. if
                                    this slider was not the last one accessed
                                 =0 reset the colour map only (if writeable
                                    colourmap available) */
   char str[20];         /* Label string */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   gd->cur_lmin_slider_val = value;
   gd->cur_lmin = gd->lmin + ((50*value)/100)*gd->lmin_interval;
   sprintf (str,"%.3f",gd->cur_lmin);
   xdl_panel_label_set (gd->lmin_curval, str, 0, 0);
   if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
   if (gd->disp_ref>=0)
   {
      gd->disp_ref = -1;
      for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
   }
   recalc = 0; 
   if (gd->cur_slider!=1)
   {
      recalc = 1;
      gd->cur_slider = 1;
   }
   gd->recalc_mult = 1;
   if (xdl_laue_sim_relabel(gd)) recalc = 1;
   xdl_laue_sim_setcolors (gd, recalc, 1);
   return;
}

/*
*********************************
**     xdl_laue_sim_cp2        **
*********************************

Purpose: Callback for control panel lambda-max slider

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp2 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int recalc;           /* Flag =1 recalculate colours etc. if
                                    this slider was not the last one accessed
                                 =0 reset the colour map only (if writeable
                                    colourmap available */
   char str[20];         /* Label string */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   gd->cur_lmax_slider_val = value;
   gd->cur_lmax = gd->lmax - ((50*(100-value))/100)*gd->lmax_interval;
   sprintf (str,"%.3f",gd->cur_lmax);
   xdl_panel_label_set (gd->lmax_curval, str, 0, 0);
   if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
   if (gd->disp_ref>=0)
   {
      gd->disp_ref = -1;
      for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
   }
   recalc = 0; 
   if (gd->cur_slider!=2)
   {
      recalc = 1;
      gd->cur_slider = 2;
   }
   gd->recalc_mult = 1;
   if (xdl_laue_sim_relabel(gd)) recalc = 1;
   xdl_laue_sim_setcolors (gd, recalc, 1);
   return;
}

/*
*********************************
**     xdl_laue_sim_cp3        **
*********************************

Purpose: Callback for control panel 'dmin' slider

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp3 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int recalc;           /* Flag =1 recalculate colours etc. if
                                    this slider was not the last one accessed
                                 =0 reset the colour map only (if writeable
                                    colourmap available */
   char str[20];         /* Label string */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   gd->cur_dmin_slider_val = value;
   gd->cur_dmin = gd->dmin + ((50*value)/100)*gd->dmin_interval;
   sprintf (str,"%.3f",gd->cur_dmin);
   xdl_panel_label_set (gd->dmin_curval, str, 0, 0);
   if (gd->mark_saved) xdl_laue_sim_restore_mark (gd);
   if (gd->disp_ref>=0)
   {
      gd->disp_ref = -1;
      for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
   }
   recalc = 0; 
   if (gd->cur_slider!=3)
   {
      recalc = 1;
      gd->cur_slider = 3;
   }
   gd->recalc_mult = 1;
   if (xdl_laue_sim_relabel(gd)) recalc = 1;
   xdl_laue_sim_setcolors (gd, recalc, 1);
   return;
}
/*
*********************************
**     xdl_laue_sim_cp4        **
*********************************

Purpose: Callback for control panel symbol size choice menu (plot type 2)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp4 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->size_sym2)
   {
      gd->size_sym2 = item;
      xdl_laue_sim_redraw2 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp5        **
*********************************

Purpose: Callback for control panel highlight option choice menu (plot type 2)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp5 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->opt_highlight2)
   {
      gd->opt_highlight2 = item;
      xdl_laue_sim_setcolors (gd, 1, 1);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp6        **
*********************************

Purpose: Callback for colour coding type choice for type 1 plots

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp6 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->color_coding)
   {
      gd->color_coding = item;
      xdl_laue_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp7        **
*********************************

Purpose: Callback for spot types choice for type 1 plots

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp7 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spot_type)
   {
      if (gd->disp_ref>=0)
      {
         gd->disp_ref = -1;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->spot_type = item;
      xdl_laue_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp8        **
*********************************

Purpose: Callback for spatials choice for plot type 1

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp8 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spatials)
   {
      if (gd->disp_ref>=0)
      {
         gd->disp_ref = -1;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->spatials = item;
      xdl_laue_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp9        **
*********************************

Purpose: Callback for spot size choice for plot type 1

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp9 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spotsize)
   {
      gd->spotsize = item;
      xdl_laue_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp10       **
*********************************

Purpose: Callback for key choice for plot type 1

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp10 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->key_opt)
   {
      if (gd->disp_ref>=0)
      {
         gd->disp_ref = -1;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->key_opt = item;
      xdl_laue_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp11       **
*********************************

Purpose: Callback for labels option choice for plot type 2

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp11 (iv, item)

int iv;
int item;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->opt_labels2)
   {
      gd->opt_labels2 = item;
      if (gd->opt_labels2>1)
      {
         if (gd->opt_labels2!=gd->last_labtyp)
         {
            xdl_laue_sim_relabel (gd);
            gd->last_labtyp = gd->opt_labels2;
         }
      }
      xdl_laue_sim_redraw2 (gd);
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp12       **
*********************************

Purpose: Callback for nodal index value (types 1 or 2)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp12 (iv, valstr, len, max_len)

int iv;
char * valstr;
int len;
int max_len;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */
   int nod;              /* New nodal index */
   char str[5];          /* Temporary string */
   int ll;               /* String length */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;

   sscanf (valstr,"%d",&nod);
   if (nod<0) nod = 0;
   if (nod>12) nod = 12;
   sprintf (str, "%d", nod);
   ll = strlen (str);
   xdl_panel_value_set (gd->nodal_sel, str, ll);
   if (nod!=gd->nodidx_cur)
   {
      gd->nodidx_cur = nod;
      switch (gd->type)
      {
         case 1:
            if (gd->color_coding==4) xdl_laue_sim_redraw1 (gd);
            break;
         case 2:
            if (gd->opt_highlight2==3) xdl_laue_sim_redraw2 (gd);
            break;
      }
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_cp13       **
*********************************

Purpose: Callback for postscript print button (types 1 or 2)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp13 (iv, button)

int iv;
int button;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;

   gd->ps_on = 1;
   xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
   return;
}
/*
*********************************
**     xdl_laue_sim_cp14       **
*********************************

Purpose: Callback for postscript i/o item (types 1 or 2)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_cp14 (iv, reply, len, max_rp, flag, iret)

int iv;
char *reply;
int len;
int max_rp;
int flag;
int iret;

{
   XDl_laue_sim *gd;     /* Pointer to global data area */

   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;

   xdl_laue_sim_ps (gd, reply, len, flag, iret);

   return;
}
/*
************************************
**     xdl_laue_sim_tidy_up       **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_laue_sim *gd;    /* Pointer to the global data area */
   int i;               /* Temp/loop variable */

   gd = (XDl_laue_sim *) global;

   XFreeGC (XDL_display, gd->paint_gc);
   if (gd->xpix!=NULL) free ((char *)gd->xpix);
   if (gd->ypix!=NULL) free ((char *)gd->ypix);
   if (gd->mult!=NULL) free ((char *)gd->mult);
   if (gd->max_harmonic!=NULL) free ((char *)gd->max_harmonic);
   if (gd->harm_inc!=NULL) free ((char *)gd->harm_inc);
   if (gd->colr!=NULL) free ((char *)gd->colr);
   if (gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i)
      {
         if (gd->labels[i]!=NULL) free ((char *)gd->labels[i]);
      }
   }
   if (gd->type==2) XFreePixmap (XDL_display, gd->save_area);
   XFreePixmap (XDL_display, gd->mark_area);
   if (gd->ps_on>0&&gd->psfil!=0) fclose (gd->psfil);
   return;
}
/*
***********************************
**     xdl_laue_sim_setcolors    **
***********************************

Purpose: Set color indices array and colour map for interactive display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_setcolors (gd, recalc, draw)

XDl_laue_sim *gd;    /* Pointer to the global data area */
int recalc;          /* flag =1 recalculate pixel values,
                             =0 reset colour map only (if writeable
                                colour map; else treat as case 1) */
int draw;            /* flag =1 redraw display if needed
                             =0 do not redraw display */
{
   int i;                /* Temp/loop variable */
   int k;                /* Bin number */
   int ithr;             /* Threshold for current soft limit */
   int black_pix;        /* Black pixel flag 1=black, 0=white */
   float dmin_tmp;       /* Tempoaray dmin value */
   float range;          /* Range for soft limit being considered */
   XColor wh_colr_cell;  /* Xcolor structure for white pixel */
   XColor bl_colr_cell;  /* Xcolor structure for black pixel */
   float cur_dmin2;      /* Current value of dmin**2 */

   cur_dmin2 = 1/((gd->cur_dmin)*(gd->cur_dmin));
   gd->pixels[0] = XDL_whitepixel;
   if (gd->color_type==0||gd->opt_highlight2!=1) recalc = 1;
   switch (gd->cur_slider)
   {
      case 1:
         range = 50.0*(gd->lmin_interval);
         ithr = (int) (50.0*(gd->cur_lmin-gd->lmin)/range) + 2;
         break;
      case 2:
         range = 50.0*(gd->lmax_interval);
         ithr = (int) (50.0*(gd->cur_lmax-gd->lmax+range)/range) + 2;
         break;
      case 3:
         range = 50.0*(gd->dmin_interval);
         ithr = (int) (50.0*(gd->cur_dmin-gd->dmin)/range) + 2;
         break;
   }

   if (recalc)
   {
      switch (gd->cur_slider)
      {
         case 1:
            for (i=0;i<gd->nspot;++i)
            {
               if (gd->lam[i]>gd->cur_lmax||gd->dthr[i]>cur_dmin2)
               {
                  gd->colr[i] = 0;
               }
               else
               {
                  k = (int) (50.0*(gd->lam[i]-gd->lmin)/range) + 2;
                  if (k<1) k = 1;
                  if (k>52) k = 52;
                  if (gd->color_type==1&&gd->opt_highlight2==1)
                  {
                     gd->colr[i] = k;
                  }
                  else
                  {
                     if (k<ithr) {gd->colr[i]=100;}else{gd->colr[i]=101;}
                  }
               }
            }
            break;

         case 2:
            for (i=0;i<gd->nspot;++i)
            {
               if (gd->lam[i]<gd->cur_lmin||gd->dthr[i]>cur_dmin2)
               {
                  gd->colr[i] = 0;
               }
               else
               {
                  k = (int) (50.0*(gd->lam[i]-gd->lmax+range)/range) + 2;
                  if (k<1) k = 1;
                  if (k>52) k = 52;
                  if (gd->color_type==1&&gd->opt_highlight2==1)
                  {
                     gd->colr[i] = k;
                  }
                  else
                  {
                     if (k<ithr) {gd->colr[i]=101;}else{gd->colr[i]=100;}
                  }
               }
            }
            break;

         case 3:
            for (i=0;i<gd->nspot;++i)
            {
               if (gd->lam[i]<gd->cur_lmin||gd->lam[i]>gd->cur_lmax)
               {
                  gd->colr[i] = 0;
               }
               else
               {
                  dmin_tmp = sqrt ((double) (1.0/gd->dthr[i]));
                  k = (int) (50.0*(dmin_tmp-gd->dmin)/range) + 2;
                  if (k<1) k = 1;
                  if (k>52) k = 52;
                  if (gd->color_type==1&&gd->opt_highlight2==1)
                  {
                     gd->colr[i] = k;
                  }
                  else
                  {
                     if (k<ithr) {gd->colr[i]=100;}else{gd->colr[i]=101;}
                  }
               }
            }
            break;
      }
   }

   /* Set pixel values for modifying colour map */

   if (gd->color_type==1&&gd->opt_highlight2==1)
   {
      wh_colr_cell.pixel = XDL_whitepixel;
      XQueryColor (XDL_display, XDL_colormap, &wh_colr_cell);
      bl_colr_cell.pixel = XDL_blackpixel;
      XQueryColor (XDL_display, XDL_colormap, &bl_colr_cell);
      for (i=1;i<=52;++i)
      {
         if (gd->cur_slider==2)
         {
            black_pix = 0;
            if (i<ithr) black_pix = 1;
         }
         else
         {
            black_pix = 1;
            if (i<ithr) black_pix = 0;
         }
         if (black_pix)
         {
            bl_colr_cell.pixel = gd->pixels[i];
            XStoreColor (XDL_display, XDL_colormap, &bl_colr_cell);
         }
         else
         {
            wh_colr_cell.pixel = gd->pixels[i];
            XStoreColor (XDL_display, XDL_colormap, &wh_colr_cell);
         }
      }
   }
   if ((recalc||gd->opt_highlight2!=1)&&draw) xdl_laue_sim_redraw2 (gd);

   return;
}
/*
************************************
**     xdl_laue_sim_redraw1       **
************************************

Purpose: Routine to redraw colour coded option display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_redraw1 (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */

{
   int i, j;          /* Temp/loop variables */
   int k;             /* index to current reflection */
   int nodal;         /* Nodal flag 1=yes, 0=no */
   int harmonic;      /* Harmonic number */
   int h_n, k_n, l_n; /* Nodal indices */
   float range;       /* Lambda range */
   int lam;           /* Lambda range loop counter */
   int mul;           /* Multiplicity loop counter */
   int nod;           /* Nodals loop counter */
   int x, y;          /* Spot position */
   int ir;            /* Lambda bin (0 to 7) for current reflection */
   int im;            /* Multiplicity (0 to 3) bin for current reflection */
   int colr;          /* Colour (/code) for xdl_laue_sim_symb1 */


   XClearWindow (XDL_display, gd->wdisp);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc, 0, 0,
              2*(gd->half_size)+1, 2*(gd->half_size)+1);

   gd->mark_saved = 0;
   range = gd->lmax - gd->lmin;

   switch (gd->color_coding)
   {
      case 1: /* Colour coding by wavelength */
      case 3: /* As above but multiples white */
         for (lam=0;lam<8;++lam)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               ir = (int) (8*(gd->lam[k]-gd->lmin)/range);
               if (ir<0) ir = 0;
               if (ir>7) ir = 7;
               if (ir!=lam) continue;
               x = (int) gd->xpix[k];
               y = (int) gd->ypix[k];
               colr = 7 - lam;
               if (gd->color_coding==3&&gd->mult[k]>1) colr = 100;
               xdl_laue_sim_symb1 (gd, x, y, colr, gd->spotsize);
            }
         }
         if (gd->key_opt==1) xdl_laue_sim_drawkey (gd);
         break;
      case 2: /* Colour coded by multiplicity */
         for (mul=0;mul<4;++mul)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               im = gd->mult[k] - 1;
               if (im>3) im = 3;
               if (im!=mul) continue;
               x = (int) gd->xpix[k];
               y = (int) gd->ypix[k];
               colr = 6 - 2*im;
               xdl_laue_sim_symb1 (gd, x, y, colr, gd->spotsize);
            }
         }
         if (gd->key_opt==1) xdl_laue_sim_drawkey (gd);
         break;
      case 4: /* Colour coded by nodality */
         for (nod=0;nod<2;++nod)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               nodal = 1;
               xdl_laue_sim_nodidx (gd, k, &h_n, &k_n, &l_n, &harmonic);
               if (h_n<0) h_n = -h_n;
               if (k_n<0) k_n = -k_n;
               if (l_n<0) l_n = -l_n;
               if (h_n>gd->nodidx_cur||k_n>gd->nodidx_cur||l_n>gd->nodidx_cur) 
                  nodal = 0;
               if (nodal!=nod) continue;
               x = (int) gd->xpix[k];
               y = (int) gd->ypix[k];
               colr = 6 - 4*nod;
               xdl_laue_sim_symb1 (gd, x, y, colr, gd->spotsize);
            }
         }
         if (gd->key_opt==1) xdl_laue_sim_drawkey (gd);
         break;
   }
   /* Draw mark for last 'listed' reflection */

   if (gd->disp_ref>=0)
   {
      xdl_laue_sim_save_mark (gd, gd->disp_ref);
      xdl_laue_sim_mark (gd, gd->disp_ref, 1);
   }

   return;
}
/*
************************************
**     xdl_laue_sim_redraw2       **
************************************

Purpose: Routine to redraw interactive option display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_redraw2 (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */

{
   int i, j;          /* Temp/loop variables */
   int k;             /* index to current reflection */
   int styp;          /* Symbol type (see xdl_laue_sim_symb2) */
   int colr;          /* Colour flag for xdl_laue_sim_symb2 */
   int nodal;         /* Nodal flag 1=yes, 0=no */
   int h_n, k_n, l_n; /* Nodal indices */
   int harmonic;      /* Harmonic number */

   XClearWindow (XDL_display, gd->wdisp);

   gd->area_saved = 0;
   gd->mark_saved = 0;

   if (gd->opt_highlight2==2&&gd->recalc_mult) xdl_laue_sim_mult(gd);
   for(i=0;i<gd->nspot;++i)
   {
      switch (gd->cur_slider)
      {
         case 1: /*Draw in increasing order of lambda*/
            k = gd->idxlam[i] - 1;
            break;
         case 2: /*Draw in decreasing order of lambda*/
            j = gd->nspot - 1 - i;
            k = gd->idxlam[j] - 1;
            break;
         case 3: /*Draw in increasing order of 'd'. Note: index array gives
                   (1/d**2) ordering therefore use reverse order*/
            j = gd->nspot - 1 - i;
            k = gd->idxdmin[j] - 1;
            break;
      }
      if (gd->colr[k]==0) continue;
      styp = gd->size_sym2;
      colr = (int) gd->colr[k];
      switch (gd->opt_highlight2)
      {
         case 1:
            if (colr==100) continue;
            break;
         case 2:
            if (colr==100) continue;
            if (XDL_color==2)
            {
               colr = 101;
               if (gd->mult[k]==2){colr = 102; styp = 3 + gd->size_sym2;}
               if (gd->mult[k]>2){colr = 103; styp = 6 + gd->size_sym2;}
            }
            else
            {
               colr = 101;
               if (gd->mult[k]==2) styp = 3 + gd->size_sym2;
               if (gd->mult[k]>2) styp = 6 + gd->size_sym2;
            }
            break;
         case 3:
            if (colr==100) continue;
            nodal = 1;
            xdl_laue_sim_nodidx (gd, k, &h_n, &k_n, &l_n, &harmonic);
            if (h_n<0) h_n = -h_n;
            if (k_n<0) k_n = -k_n;
            if (l_n<0) l_n = -l_n;
            if (h_n>gd->nodidx_cur||k_n>gd->nodidx_cur||l_n>gd->nodidx_cur) 
               nodal = 0;
            if (XDL_color==2)
            {
               colr = 101;
               if (nodal){colr = 103; styp = 3 + gd->size_sym2;}
            }
            else
            {
               colr = 101;
               if (nodal) styp = 3 + gd->size_sym2;
            }
            break;
      }
      xdl_laue_sim_symb2 (gd, (int)gd->xpix[k], (int)gd->ypix[k],
                          colr, styp);
   }

   /* Draw labels if required and if present */

   if (gd->opt_labels2>1&&gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i) xdl_laue_sim_label (gd, i, 1);
   }

   /* Draw mark for last 'listed' reflection */

   if (gd->disp_ref>=0)
   {
      xdl_laue_sim_save_mark (gd, gd->disp_ref);
      xdl_laue_sim_mark (gd, gd->disp_ref, 1);
   }

   return;
}

/*
**********************************
**    xdl_laue_sim_symb1        **
**********************************

Purpose: Draw symbol for type 1 (colour) simulation display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_symb1 (gd,xpix,ypix,colr,symtyp)

XDl_laue_sim *gd;      /* Pointer to global data area */
int xpix;              /* Symbol x position */
int ypix;              /* Symbol y position */
int colr;              /* Index in colrpix (<100) or 100=white */
int symtyp;            /* Symbol type 1 = small disc (point)
                                      2 = medium disc
                                      3 = large disc
                       */
                            

{
   if (xpix<0||ypix<0) return;
   if (colr<100)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
   }
   else
   {
      switch (colr)
      {
         case 100:
            XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
            break;
      }
   }
   switch(symtyp)
   {
      case 1: /* Small 'disc' (Point) */
         XDrawPoint (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix);
         break;
      case 2: /* Medium 'disc' */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-1, ypix-1, 3, 3, 0, 23040);
         break;
      case 3: /* Large 'disc' */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-2, ypix-2, 5, 5, 0, 23040);
         break;
   }
   return;
}
/*
**********************************
**    xdl_laue_sim_drawkey      **
**********************************

Purpose: Draw keys for type 1 (colour) plots

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_drawkey (gd)

XDl_laue_sim *gd;      /* Pointer to global data area */

{
   int i;              /* Temp/loop variable */
   int xspot;          /* x coordinate for coloured spot */
   int yspot;          /* y coordinate for coloured spot */
   int xstring;        /* x coordinate for string */
   int ystring;        /* y coordinate for string */
   int len;            /* String length */
   char str[30];       /* Temp string for text output */
   float lam1;         /* Minimum lambda for the range */
   float lam2;         /* Maximum lambda for the range */
   float dlam;         /* Width of lambda range */

   XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XDrawString (XDL_display, gd->wdisp, gd->paint_gc,
                    4, 10, "Key", 3);
   switch (gd->color_coding)
   {
      case 1: /* Colour coded by wavelength */
      case 3: /* Colour coded by wavelength, multiples white */
        if (gd->color_coding==1)
        {
           XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                           4, 15, 114, 122);
           XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
           XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                           5, 16, 113, 121);
        }
        else
        {
           XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                           4, 15, 114, 137);
           XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
           XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                           5, 16, 113, 136);
        }
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        dlam = (gd->lmax - gd->lmin) / 8.0;
        for(i=0;i<8;++i)
        {
           XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[7-i]);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           lam1 = gd->lmin + i*dlam;
           lam2 = lam1 + dlam;
           if (i==7) lam2 = gd->lmax;
           sprintf (str,"%.3f-%.3f A",lam1,lam2);
           len = strlen(str);
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                        ystring, str, len);
           yspot += 15;
           ystring += 15;
        }
        if (gd->color_coding==3)
        {
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                        ystring, "multiples", 9);
        }
        break;
        
      case 2: /* Colour coded by multiplicity */
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 64, 64);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 63, 63);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<4;++i)
        {
           XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[6-2*i]);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           switch (i)
           {
              case 0:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "single", 6);
                 break;
              case 1:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "double", 6);
                 break;
              case 2:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "triple", 6);
                 break;
              case 3:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "> 3", 3);
                 break;
           }
           yspot += 15;
           ystring += 15;
        }
        break;
      case 4: /* Colour coded by nodal/non-nodal */
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 85, 31);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 84, 30);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<2;++i)
        {
           XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[6-4*i]);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           switch (i)
           {
              case 0:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "non-nodal", 9);
                 break;
              case 1:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "nodal", 5);
                 break;
           }
           yspot += 15;
           ystring += 15;
        }
        break;
   }
   return;
}
/*
**********************************
**    xdl_laue_sim_symb2        **
**********************************

Purpose: Draw symbol for type 2 simulation display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_symb2 (gd,xpix,ypix,colr,symtyp)

XDl_laue_sim *gd;      /* Pointer to global data area */
int xpix;              /* Symbol x position */
int ypix;              /* Symbol y position */
int colr;              /* Index in pixels array for symbol colour (<100)
                          or 101 = black
                             102 = blue
                             103 = red   */
int symtyp;            /* Symbol type 1 = small disc (point)
                                      2 = medium disc
                                      3 = large disc
                                      4 = small cross
                                      5 = medium cross
                                      6 = large cross
                                      7 = small star
                                      8 = medium star
                                      9 = large star
                       */
                            

{
   if (xpix<0||ypix<0) return;
   if (colr<100)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->pixels[colr]);
   }
   else
   {
      switch (colr)
      {
         case 101:
            XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
            break;
         case 102:
            XSetForeground (XDL_display, gd->paint_gc, XDL_bluepixel);
            break;
         case 103:
            XSetForeground (XDL_display, gd->paint_gc, XDL_redpixel);
            break;
      }
   }
   switch(symtyp)
   {
      case 1: /* Small disc (Point) */
         XDrawPoint (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix);
         break;

      case 2: /* Medium disc */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-1, ypix-1, 3, 3, 0, 23040);
         break;

      case 3: /* Large disc */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-2, ypix-2, 5, 5, 0, 23040);
         break;

      case 4: /* Small cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix,
                    xpix+2, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-2,
                    xpix, ypix+2);
         break;
      case 5: /* Medium cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix,
                    xpix+3, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-3,
                    xpix, ypix+3);
         break;
      case 6: /* Large cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix,
                    xpix+4, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-4,
                    xpix, ypix+4);
         break;
      case 7: /* Small star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix,
                    xpix+2, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-2,
                    xpix, ypix+2);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix-2,
                    xpix+2, ypix+2);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix+2,
                    xpix+2, ypix-2);
         break;
      case 8: /* Medium star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix,
                    xpix+3, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-3,
                    xpix, ypix+3);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix-3,
                    xpix+3, ypix+3);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix+3,
                    xpix+3, ypix-3);
         break;
      case 9: /* Large star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix,
                    xpix+4, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-4,
                    xpix, ypix+4);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix-4,
                    xpix+4, ypix+4);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix+4,
                    xpix+4, ypix-4);
         break;
   }
   return;
}
/*
********************************
**     xdl_laue_sim_mult      **
********************************

Purpose: Get current multiplicities and maximum harmonic numbers

Author:  John W. Campbell, July 1993 (re-coded for use with Laue Reflection
         List 'LRL')

*/

void xdl_laue_sim_mult (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */

{
   int i;              /* Temp/loop variable */
   int ii;             /* Temp/loop variable */
   int h_n, k_n, l_n;  /* Nodal indices */
   int minharm;        /* Minimum harmonic number*/
   int maxharm;        /* Maximum harmonic increment*/
   int harminc;        /* Harmonics increment */
   int mult;           /* Multiplicity */
   int nmult;          /* Multiplicity counter */
   int harm;           /* Temp harmonic number */
   int max_harm;       /* Maximum harmonic */
   float cur_dmin2;    /* Current value of 1/dmin**2 */
   float lam;          /* Temp lambda value */
   float dthr;         /* Temp dmin_thresholds2 (1/d**2) value */

   cur_dmin2 = 1/((gd->cur_dmin)*(gd->cur_dmin));

   /* Get multiplicities */

   for (i=0;i<gd->nspot;++i)
   {
       if (gd->multi[i]==1)
       {
          xdl_laue_sim_nodidx (gd, i, &h_n, &k_n, &l_n, &harm);
          gd->mult[i] = 1;
          gd->max_harmonic[i] = harm;
          gd->harm_inc[i] = 1;   /*dummy*/
       }
       else
       {
          ii = gd->multi[i];
          mult = ii/TWO20;
          ii -= TWO20*mult;
          minharm = ii/TWO12;
          ii -= TWO12*minharm;
          maxharm = ii/TWO4;
          harminc = ii - TWO4*maxharm;
          nmult = 0;
          max_harm = 0;
          for (ii=minharm;ii<=maxharm;ii+=harminc)
          {
             lam = gd->lam[i]*minharm/ii;
             dthr = gd->dthr[i]*ii/minharm;
             if (lam>=gd->cur_lmin &&
                 lam<=gd->cur_lmax &&
                 dthr<=cur_dmin2)
             {
                nmult++;
                if (ii>max_harm) max_harm = ii;
             }
          }
          gd->mult[i] = nmult;
          gd->max_harmonic[i] = max_harm;
          gd->harm_inc[i] = harminc;
      }
   }
   
   gd->recalc_mult = 0;
   
   return;
}
/*
********************************
**     xdl_laue_sim_digit     **
********************************

Purpose: Draw digit on display area

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_digit (gd, digit, xch, ych)

XDl_laue_sim *gd;    /* Pointer to global data area */
int digit;           /* Digit number 0 to 9 */
int xch;             /* x position (bottom left corner of digit) */
int ych;             /* y position (bottom left corner of digit) */

{
   static short digits[10][16][2]={
   {{1,2},{1,7},{2,8},{4,8},{5,7},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{3,8},{3,1},{-1,1},{5,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{1,2},{1,1},
    {5,1},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},{2,5},{-4,5},
    {5,4},{5,2},{4,1},{2,1},{1,2},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,3},{5,3},{-4,1},{4,5},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,8},{1,5},{4,5},{5,4},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,2},{2,1},{4,1},{5,2},{5,3},{4,4},
    {2,4},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,8},{5,8},{5,6},{1,2},{1,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{2,5},{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},
    {2,5},{1,4},{1,2},{2,1},{4,1},{5,2},{5,4},{4,5}},

   {{1,1},{5,5},{5,7},{4,8},{2,8},{1,7},{1,6},{2,5},
    {4,5},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
   };
   int i;                   /* Temp/loop variable */
   int dx1, dy1, dx2, dy2;  /* Pixel point offsets for lines within character */
   int x1, y1, x2, y2;      /* Coordinates for ends of lines */

   for(i=0;i<16;++i)
   {
      if (digits[digit][i][0]==0) break;
      if (i==0||digits[digit][i][0]<0)
      {
         dx1 = digits[digit][i][0]; if (dx1<0) dx1 = -dx1;
         dy1 = digits[digit][i][1]; if (dy1<0) dy1 = -dy1;
         continue;
      }
      dx2 = digits[digit][i][0];
      dy2 = digits[digit][i][1];
      x1 = xch + dx1 - 1;
      y1 = ych - dy1 + 1;
      x2 = xch + dx2 - 1;
      y2 = ych - dy2 + 1;
      XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, x1, y1, x2, y2);
      dx1 = dx2;
      dy1 = dy2;
   }
     
   return;
}
/*
********************************
**     xdl_laue_sim_label     **
********************************

Purpose: Draw an indices label

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_label (gd, loff, iatt)

XDl_laue_sim *gd;    /* Pointer to global data area */
int loff;            /* Offset in labels list */
int iatt;            /* =1 draw attachment line, =0 do not */
{
   XDl_laue_sim_label *lt;   /* Pointer to current labels structure */
   int l;                    /* Temp/loop variable */
   int ll;                   /* Length of character string */
   int digit;                /* Current digit number 0-9 */
   int xch, ych;             /* Current character position */
   int xc, yc;               /* Box corner for attached line */
   int xspot, yspot;         /* Spot position */

   lt = gd->labels[loff];
   if (lt->disp==0) return;
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc, lt->x1, lt->y1,
                  (lt->x2-lt->x1), (lt->y2-lt->y1));
   XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc, lt->x1+1, lt->y1+1,
                  (lt->x2-lt->x1-1), (lt->y2-lt->y1-1));
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   ll = strlen (lt->hklstr);
   if (ll<=0) return;
   xch = lt->x1 + 2;
   ych = lt->y2 - 2;
   for (l=0;l<ll;++l)
   {
      if (lt->hklstr[l]=='-')
      {
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xch, ych-4,
                   xch+4, ych-4);
         xch += 7;
      }
      else if (lt->hklstr[l]>='0'&&lt->hklstr[l]<='9')
      {
         digit = lt->hklstr[l] - '0';
         xdl_laue_sim_digit (gd, digit, xch, ych);
         xch += 7;
      }
      else
      {
         xch += 4;
      }
   }
   /* Draw attaching line to spot */

   if (iatt==1)
   {
      xspot = gd->xpix[lt->idx];
      yspot = gd->ypix[lt->idx];
      if (xspot>=lt->x1&&xspot<=lt->x2&&yspot>=lt->y1&&yspot<=lt->y2) return;
      xc = lt->x2;
      if (abs(xspot-lt->x1)<abs(xspot-lt->x2)) xc = lt->x1;
      yc = lt->y2;
      if (abs(yspot-lt->y1)<abs(yspot-lt->y2)) yc = lt->y1;
      XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xc, yc, xspot, yspot);
   }
   return;
}
/*
******************************
**     xdl_laue_sim_mark    **
******************************

Purpose: Mark spot just indexed (or temp mark for labelling)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_mark (gd, idx, type)

XDl_laue_sim *gd;    /* Pointer to global data area */
int idx;             /* Index for spot to be marked */
int type;            /* =1, mark for indexed spot
                        =2, temp. mark for labelling */
{
   if (gd->type==1)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   }
   else
   {
      if (XDL_color==2&&type==1)
      {
         XSetForeground (XDL_display, gd->paint_gc, XDL_redpixel);
      }
      else
      {
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      }
   }
   
   switch (type)
   {
      case 1:
         XDrawArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  gd->xpix[idx]-8, gd->ypix[idx]-8, 17, 17, 0, 23040);
         break;
      case 2:
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
                   (gd->xpix[idx]-9), gd->ypix[idx],
                   (gd->xpix[idx]+9), gd->ypix[idx]);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
                    gd->xpix[idx], (gd->ypix[idx]-9),
                    gd->xpix[idx], (gd->ypix[idx]+9));
         break;
   }
   return;
}
/*
**********************************
**     xdl_laue_sim_isalabel    **
**********************************

Purpose: See if mouse x, y is within a displayed label; if so take
         appropriate action.

Author:  John W. Campbell, July 1992

*/

int xdl_laue_sim_isalabel (gd, x, y, iopt)

XDl_laue_sim *gd;    /* Pointer to global data area */
int x;               /* Mouse x position */
int y;               /* Mouse y position */
int iopt;            /* =1, if label found, shift to end of labels list and
                            redraw plot without last label.
                        =0, do not */
{
   XDl_laue_sim_label *lt;       /* Temporary pointer to labels structure */
   int i;                        /* Temp/loop variable */
   int ii;                       /* Index to selected label (-1=none) */
   int sav_disp_ref;             /* Temp copy of gd->disp_ref */

   if (gd->opt_labels2==1) return 0;
   if (gd->num_labels==0) return 0;
   ii = -1;
   for (i=0;i<gd->num_labels;++i)
   {
      lt = gd->labels[i];
      if (lt->disp==0) continue;
      if (x>=lt->x1&&x<=lt->x2&&y>=lt->y1&&y<=lt->y2) ii = i;
   }
   if (ii<0) return 0;

   if (iopt==0) return 1;

   /* Shift selected label to the end of the labels list */

   if (ii<gd->num_labels-1)
   {
      lt = gd->labels[ii];
      for (i=ii;i<gd->num_labels-1;++i) gd->labels[i] = gd->labels[i+1];
      gd->labels[gd->num_labels-1] = lt;
   }
   
   /* Redraw plot without the last label (temp. reset gd->num_labels)
      and without mark if label attached to the marked reflection  */

   gd->num_labels -= 1;
   sav_disp_ref = gd->disp_ref;
   if (gd->disp_ref==lt->idx) gd->disp_ref = -1;
   xdl_laue_sim_redraw2 (gd);
   gd->disp_ref = sav_disp_ref;
   gd->num_labels += 1;
   
   gd->cur_label = gd->num_labels - 1;
   return 1;
}
/*
**********************************
**     xdl_laue_sim_relabel     **
**********************************

Purpose: See if labelling requirements change when sliders moved

Author:  John W. Campbell, July 1992

*/

int xdl_laue_sim_relabel (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */
{
   XDl_laue_sim_label *lt;       /* Temporary pointer to labels structure */
   int change;                   /* Display status changed flag 1=yes, 0=no */
   int relab;                    /* Recalculate labels flag 1=yes, 0=no */
   int display;                  /* Display current label 1=yes, 0=no */
   int i, j;                     /* Temp/loop variables */
   int ii;                       /* Offset in reflections array */
   int lam_ok;                   /* Lambda within range flag =1 ok, =0 not */
   int d_ok;                     /* Dmin within range flag =1 ok, =0 not */
   int h, k, l;                  /* Reflection indices */
   int min_harm;                 /* Minimum harmonic for the spot */
   int max_harm;                 /* Maximum harmonic for the spot */
   float lam;                    /* Lambda value */
   float dmin;                   /* Dmin value */

   if (gd->num_labels==0) return 0;
   if (gd->opt_labels2==1) return 0;
   if (gd->recalc_mult) xdl_laue_sim_mult (gd);

   change = 0;
   relab = 0;
   for (i=0;i<gd->num_labels;++i)
   {
      lt = gd->labels[i];
      ii = lt->idx;
      max_harm = gd->max_harmonic[ii];
      min_harm = max_harm - (gd->mult[ii]-1)*(gd->harm_inc[ii]);
      lam_ok = 0;
      for (j=min_harm;j<=max_harm;j+=gd->harm_inc[ii])
      {
          lam = lt->lamnod/ ((float) j);
          if (lam>=gd->cur_lmin&&lam<=gd->cur_lmax)
          {
             lam_ok = 1;
             break;
          }
      }
      display = lam_ok;
      if (display)
      {
         d_ok = 0;
         for (j=min_harm;j<=max_harm;j+=gd->harm_inc[ii])
         {
             dmin = lt->dnod/((float) j);
             if (dmin>=gd->cur_dmin)
             {
                d_ok = 1;
                break;
             }
         }
         display = d_ok;
      }
      if (lt->disp!=display)
      {
         change = 1;
         lt->disp = display;
      }
      xdl_laue_sim_labhkl (gd, ii, &h, &k, &l);
      if (h!=lt->h||k!=lt->k||l!=lt->l)
      {
         change = 1;
         relab = 1;
      }
   }
   if (relab) xdl_laue_sim_reset_labels (gd);
   return change;
}
/*
**********************************
**     xdl_laue_sim_labhkl      **
**********************************

Purpose: Get required indices for spot label

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_labhkl (gd, idx, h, k, l)

XDl_laue_sim *gd;    /* Pointer to global data area */
int idx;             /* Offset in reflections arrays */
int *h;              /* Returns required 'h' index */
int *k;              /* Returns required 'k' index */
int *l;              /* Returns required 'l' index */
{
   int jj;              /* Temp value for decoding nodal indices */
   int h_n, k_n, l_n;   /* Nodal indices */
   int harmonic;        /* Harmonic number */

   if (gd->recalc_mult) xdl_laue_sim_mult(gd);
   xdl_laue_sim_nodidx (gd, idx, &h_n, &k_n, &l_n, &harmonic);
   switch (gd->opt_labels2)
   {
      case 2:
        *h = (gd->max_harmonic[idx]
             -(gd->mult[idx]-1)*(gd->harm_inc[idx])) * h_n;
        *k = (gd->max_harmonic[idx]
             -(gd->mult[idx]-1)*(gd->harm_inc[idx])) * k_n;
        *l = (gd->max_harmonic[idx]
             -(gd->mult[idx]-1)*(gd->harm_inc[idx])) * l_n;
        break;
      case 3:
        *h = gd->max_harmonic[idx] * h_n;
        *k = gd->max_harmonic[idx] * k_n;
        *l = gd->max_harmonic[idx] * l_n;
        break;
      case 4:
        *h = h_n;
        *k = k_n;
        *l = l_n;
        break;
   }
   return;
}
/*
**********************************
**     xdl_laue_sim_nodidx      **
**********************************

Purpose: Get nodal indices & harmonic number for a reflection 

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_nodidx (gd, idx, h_n, k_n, l_n, harmonic)

XDl_laue_sim *gd;    /* Pointer to global data area */
int idx;             /* Offset in reflections arrays */
int *h_n;            /* Returns required 'h' index */
int *k_n;            /* Returns required 'k' index */
int *l_n;            /* Returns required 'l' index */
int *harmonic;       /* Returns the harmonic number */
{
   int jj;              /* Temp value for decoding indices */
   int h, k, l;         /* Spot indices */

   jj = gd->hkl[idx];
   h = jj/TWO20;
   jj = jj - TWO20*h;
   h -= 512;
   k = jj/TWO10;
   l = jj-TWO10*k - 512;
   k -= 512;
   jj = gd->nodhkls[idx];
   *h_n = jj/TWO20;
   jj = jj - TWO20*(*h_n);
   *h_n -= 512;
   *k_n = jj/TWO10;
   *l_n = jj-TWO10*(*k_n) - 512;
   *k_n -= 512;
   *harmonic = 0;
   if (*h_n!=0)
   {
      *harmonic = h/(*h_n);
   }
   else if(*k_n!=0)
   {
      *harmonic = k/(*k_n);
   }
   else if (*l_n!=0)
   {
      *harmonic = l/(*l_n);
   }
   return;
}
/*
*************************************
**     xdl_laue_sim_reset_labels   **
*************************************

Purpose: Reset labels when index type for labels changed
         or if indices changed when slider moved.

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_reset_labels (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */
{
   XDl_laue_sim_label *lt;       /* Temporary pointer to labels structure */
   int i;                        /* Temp/loop variable */
   int h, k, l;                  /* New indices */
   int wlnew, wlold;             /* Label widths new and old */
   int ll;                       /* label string length */
   int xspot;                    /* Spot x position */

   if (gd->num_labels==0) return;
   for (i=0;i<gd->num_labels;++i)
   {
      lt = gd->labels[i];
      xdl_laue_sim_labhkl (gd, lt->idx, &h, &k, &l);
      lt->h = h;
      lt->k = k;
      lt->l = l;
      sprintf (lt->hklstr,"%d %d %d",h, k, l);
      ll = strlen(lt->hklstr);
      wlnew = 7*ll+4-6;
      wlold = lt->x2 - lt->x1 + 1;
      lt->xcoff = wlnew/2;
      xspot = gd->xpix[lt->idx];

      /* Keep attached corner at same x value */

      if (abs(xspot-lt->x1)<abs(xspot-lt->x2))
      {
         lt->x2 += wlnew - wlold;
      }
      else
      {
         lt->x1 += wlold - wlnew;
      }
   }
   return;
}
/*
***********************************
**     xdl_laue_sim_save_mark    **
***********************************

Purpose: Save area to be marked

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_save_mark (gd, idx)

XDl_laue_sim *gd;    /* Pointer to global data area */
int idx;             /* Index for spot to be marked */
{
   int max_x;      /* Max allowed x value */
   int max_y;      /* Max allowed y value */

   max_x = 2*(gd->half_size + 1) - MARK_SIZ + 1;
   max_y = max_x;
   gd->mark_x = gd->xpix[idx] - MARK_SIZ/2;
   if (gd->mark_x<0) gd->mark_x = 0;
   if (gd->mark_x>max_x) gd->mark_x = max_x;
   gd->mark_y = gd->ypix[idx] - MARK_SIZ/2;
   if (gd->mark_y<0) gd->mark_y = 0;
   if (gd->mark_y>max_y) gd->mark_y = max_y;
   gd->mark_w = MARK_SIZ;
   gd->mark_h = MARK_SIZ;
   XCopyArea (XDL_display, gd->wdisp, gd->mark_area,
              gd->paint_gc, gd->mark_x, gd->mark_y, 
              gd->mark_w, gd->mark_h, 0, 0);
   gd->mark_saved = 1;
   return;
}
/*
**************************************
**     xdl_laue_sim_restore_mark    **
**************************************

Purpose: Restore area which was marked

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_restore_mark (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */
{
   XCopyArea (XDL_display, gd->mark_area, gd->wdisp, 
              gd->paint_gc, 0, 0, gd->mark_w, gd->mark_h, 
              gd->mark_x, gd->mark_y);
   gd->mark_saved = 0;
   return;
}
/*
***********************************
**     xdl_laue_sim_save_area    **
***********************************

Purpose: Save area (for labelling)

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_save_area (gd, x, y, w, h)

XDl_laue_sim *gd;    /* Pointer to global data area */
int x;               /* x coordinate of top left corner of area */
int y;               /* y coordinate of top left corner of area */
int w;               /* width to be saved */
int h;               /* height to be saved */
{
   int max_x;      /* Max allowed x value */
   int max_y;      /* Max allowed y value */

   max_x = 2*(gd->half_size + 1) - w + 1;
   max_y = 2*(gd->half_size + 1) - h + 1;
   gd->sav_x = x;
   if (gd->sav_x<0) gd->sav_x = 0;
   if (gd->sav_x>max_x) gd->sav_x = max_x;
   gd->sav_y = y;
   if (gd->sav_y<0) gd->sav_y = 0;
   if (gd->sav_y>max_y) gd->sav_y = max_y;
   gd->sav_w = w;
   gd->sav_h = h;
   XCopyArea (XDL_display, gd->wdisp, gd->save_area,
              gd->paint_gc, gd->sav_x, gd->sav_y, 
              gd->sav_w, gd->sav_h, 0, 0);
   gd->area_saved = 1;
   return;
}
/*
**************************************
**     xdl_laue_sim_restore_area    **
**************************************

Purpose: Restore saved area

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_restore_area (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */
{
   XCopyArea (XDL_display, gd->save_area, gd->wdisp, 
              gd->paint_gc, 0, 0, gd->sav_w, gd->sav_h, 
              gd->sav_x, gd->sav_y);
   gd->area_saved = 0;
   return;
}
/*
*************************************
**     xdl_laue_sim_hsv_to_rgb     **
*************************************

Purpose: Convert HSV colour to RGB colour

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_hsv_to_rgb (h, s, v, r, g, b)

float h;             /* Hue 0.0 to 360.0 (R)*/
float s;             /* Saturation 0.0 to 1.0 (R)*/
float v;             /* Value 0.0 to 1.0 (R)*/
float *r;            /* Returned red value 0.0 to 1.0 (W)*/
float *g;            /* Returned green value 0.0 to 1.0 (W)*/
float *b;            /* Returned blue value 0.0 to 1.0 (W)*/

{
   float p1, p2, p3, fr, iv;
   int i;

   if (h==360.0) h = 0.0;
   h = h/60.0;
   iv = (float) floor ((double) h);
   fr = h - iv;
   p1 = v * (1.0-s);
   p2 = v * (1.0 - (s*fr));
   p3 = v * (1.0 - (s * (1.0-fr)));
   i = (int) iv;
   switch(i)
   {
      case 0:
         *r = v;
         *g = p3;
         *b = p1;
         break;
      case 1:
         *r = p2;
         *g = v;
         *b = p1;
         break;
      case 2:
         *r = p1;
         *g = v;
         *b = p3;
         break;
      case 3:
         *r = p1;
         *g = p2;
         *b = v;
         break;
      case 4:
         *r = p3;
         *g = p1;
         *b = v;
         break;
      case 5:
         *r = v;
         *g = p1;
         *b = p2;
         break;
   }
   return;
}
/*
*****************************
**     xdl_laue_sim_ps     **
*****************************

Purpose: Postscript output processing

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_ps (gd, reply, len, prompt_num, iret)

XDl_laue_sim *gd;    /* Pointer to global data area */
char *reply;         /* Reply to last prompt */
int len;             /* Length of reply */
int prompt_num;      /* Postscript prompt number */
int iret;            /* Return flag =0 blank string
                                    =1 non-blank string
                                    =-1 escape */
{
   int xroot;       /* Root x position for error message */
   int yroot;       /* Root y position for error message */
   int i, j;        /* Temp/loop variables */
   int prompt_only; /* =1 prompt without processing,=0 process then prompt */
   static float spsiz[3] = {0.35, 0.75, 1.5};  /* Default spot sizes in mm */
   static float smsiz[3] = {1.5, 2.0, 2.5};     /* Default symbol sizes in mm */
   float val;       /* Temporary value */
   char prompt[61]; /* Temporary prompt string */

   /* Process current reply */
    
   prompt_only = 0;
   xdl_rootxy (gd->wpanel_ps, 200, 20, &xroot, &yroot);
   if (iret==-1)
   {
      if (gd->ps_on>0) fclose (gd->psfil);
      gd->ps_on = 0;
      xdl_panel_io_clear (gd->ps_io);
      return;
   }
   if (prompt_num==1)
   {
      if (iret==0)
      {
         gd->ps_on = 0;
         xdl_panel_io_clear (gd->ps_io);
         return;
      }
      gd->psfil = fopen (reply, "w");
      if (gd->psfil==0)
      {
        i = xdl_popup_notice (xroot, yroot, 
                             "** Cannot open file **", 0, 
                             NULL, -1,
                             "Retry", 0, "Cancel", 0, 2, 1);
        xdl_panel_io_clear (gd->ps_io);
        if (i==1)
        {
           gd->ps_on = 1;
           xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
        }
        else
        {
           gd->ps_on = 0;
        }
        return;
      }
      xdl_laue_sim_pshead (gd);
   }
   labl:
   switch (gd->type)
   {
      case 1:
         switch (prompt_num)
         {
             case 1:
               prompt_num ++;
               sprintf (prompt,"Spot diameter in mm [%.2f]:",
                       spsiz[gd->spotsize-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 2:
               if (prompt_only==0)
               {
                  gd->ps_spot_rad = spsiz[gd->spotsize-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.1||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.1 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_spot_rad = val/2.0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                                   "Add key [y]:",
                                   0, prompt_num);
               break;
            case 3:
               if (prompt_only==0)
               {
                  gd->ps_addkey = 1;
                  if (iret>0)
                  {
                     if (reply[0]!='y'&&reply[0]!='Y'&&
                         reply[0]!='n'&&reply[0]!='N')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 'y' or 'n'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='n'||reply[0]=='N') gd->ps_addkey = 0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, "Comment:", 0, prompt_num);
               break;
            case 4:
               gd->ps_comment = reply;
               xdl_laue_sim_psdraw1 (gd);
               xdl_panel_io_clear (gd->ps_io);
               gd->ps_on = 0;
         }
         break;

      case 2:
         switch (prompt_num)
         {
            case 1:
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                      "Standard (s) or absolute (a) scale [s]:", 39,
                      prompt_num);
               break;
            case 2:
               if (prompt_only==0)
               {
                  gd->plot_rad = 90.0;
                  if (iret>0)
                  {
                     if (reply[0]!='s'&&reply[0]!='S'&&
                         reply[0]!='a'&&reply[0]!='A')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 's' or 'a'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='a'||reply[0]=='A') gd->plot_rad = gd->filmr;
                  }
                  if (gd->plot_rad>90.0)
                  {
                     i = xdl_popup_notice (xroot, yroot, 
                         "*Warning*", 0, 
                         "Plot too large - will be clipped", 0,
                         "Continue", 0, NULL, -1, 2, 1);
                  }
               }
               prompt_num ++;
               sprintf (prompt,"Spot diameter in mm [%.2f]:",
                       spsiz[gd->size_sym2-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 3:
               if (prompt_only==0)
               {
                  gd->ps_spot_rad = spsiz[gd->size_sym2-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.1||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.1 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_spot_rad = val/2.0;
                  }
               }
               prompt_num ++;
               if (gd->opt_highlight2==1)
               {
                  gd->ps_color = 0;
                  gd->ps_symb_hs = smsiz[gd->size_sym2-1]/2.0;
                  prompt_num ++;  /* Skip next two prompts */
                  prompt_only = 1; goto labl;
               }
               sprintf (prompt,"Symbol size mm [%.2f]:",
                       smsiz[gd->size_sym2-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 4:
               if (prompt_only==0)
               {
                  gd->ps_symb_hs = smsiz[gd->size_sym2-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.5||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.5 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_symb_hs = val/2.0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                                   "Use coloured symbols [n]:",
                                   0, prompt_num);
               break;
            case 5:
               if (prompt_only==0)
               {
                  gd->ps_color = 0;
                  if (iret>0)
                  {
                     if (reply[0]!='y'&&reply[0]!='Y'&&
                         reply[0]!='n'&&reply[0]!='N')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 'y' or 'n'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='y'||reply[0]=='Y') gd->ps_color = 1;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, "Comment:", 0, prompt_num);
               break;
            case 6:
               gd->ps_comment = reply;
               xdl_laue_sim_psdraw2 (gd);
               xdl_panel_io_clear (gd->ps_io);
               gd->ps_on = 0;
         }
         break;
   }
   return;
}
/*
*********************************
**     xdl_laue_sim_pshead     **
*********************************

Purpose: Postscript header info output

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_pshead (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */
{
   int i;       /*Temp/loop variable */

   fprintf (gd->psfil,"%%!\nnewpath\n");
   fprintf (gd->psfil,"/M {moveto} def\n");
   fprintf (gd->psfil,"/L {lineto} def\n");
   fprintf (gd->psfil,"/R {rlineto} def\n");
   fprintf (gd->psfil,"/S {stroke} def\n");
   fprintf (gd->psfil,"/mm_to_p 2.835 def\n");

   /* Spot print */
   fprintf (gd->psfil,"\n%% Output spot (xs ys rad_mm S1) %%\n");
   fprintf (gd->psfil,"/S1 {/rad_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/rad {rad_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs ys M xs ys rad 0 360 arc fill S} def\n");

   /* Vertical cross print */
   fprintf (gd->psfil,"\n%% Output vertical cross (xs ys hs_mm S2) %%\n");
   fprintf (gd->psfil,"/S2 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"xs ys hs sub M xs ys hs add L\n");
   fprintf (gd->psfil,"S} def\n");

   /* Star print */
   fprintf (gd->psfil,"\n%% Star (xs ys hs_mm S3) %%\n");
   fprintf (gd->psfil,"/S3 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"xs ys hs sub M xs ys hs add L\n");
   fprintf (gd->psfil,"xs hs sub ys hs sub M xs hs add ys hs add L\n");
   fprintf (gd->psfil,"xs hs sub ys hs add M xs hs add ys hs sub L\n");
   fprintf (gd->psfil,"S} def\n");

   fprintf (gd->psfil,"\n1 setlinewidth\n");

   /* Colour table */
   fprintf (gd->psfil,"/black {0 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/white {1 1 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/red {1 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/blue {0 0 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/prcolr   {0.8 0.8 0.8 setrgbcolor} def\n");
   if (gd->type==1)
   {
      for (i=0;i<8;++i)
      {
         fprintf (gd->psfil, "/color%d {%.3f %.3f %.3f setrgbcolor} def\n",
                  i, gd->ps_r[i], gd->ps_g[i], gd->ps_b[i]);
      }
   }

   /*Break into lines procedure */
   fprintf (gd->psfil,"/wordbreak ( ) def /BreakIntoLines\n");
   fprintf (gd->psfil,
            "{/wrlin exch def /linewidth exch def /textstring exch def\n");
   fprintf (gd->psfil,"/breakwidth wordbreak stringwidth pop def\n");
   fprintf (gd->psfil,"/curwidth 0 def /lastwordbreak 0 def\n");
   fprintf (gd->psfil,"/startchar 0 def /restoftext textstring def\n");
   fprintf (gd->psfil,"{restoftext wordbreak search\n");
   fprintf (gd->psfil,"{/nextword exch def pop /restoftext exch def\n");
   fprintf (gd->psfil,"/wordwidth nextword stringwidth pop def\n");
   fprintf (gd->psfil,"curwidth wordwidth add linewidth gt\n");
   fprintf (gd->psfil,"{textstring startchar lastwordbreak startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin /startchar lastwordbreak def\n");
   fprintf (gd->psfil,"/curwidth wordwidth breakwidth add def}\n");
   fprintf (gd->psfil,"{/curwidth curwidth wordwidth add\n");
   fprintf (gd->psfil,"breakwidth add def } ifelse\n");
   fprintf (gd->psfil,
            "/lastwordbreak lastwordbreak nextword length add 1 add def}\n");
   fprintf (gd->psfil,"{pop exit} ifelse }loop\n");
   fprintf (gd->psfil,"/lastchar textstring length def\n");
   fprintf (gd->psfil,"textstring startchar lastchar startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin } def\n");

   return;
}
/*
************************************
**     xdl_laue_sim_psdraw1       **
************************************

Purpose: Postscript output for colour coded option display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_psdraw1 (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */

{
   int i, j;          /* Temp/loop variables */
   int k;             /* index to current reflection */
   int nodal;         /* Nodal flag 1=yes, 0=no */
   int harmonic;      /* Harmonic number */
   int h_n, k_n, l_n; /* Nodal indices */
   float range;       /* Lambda range */
   float dlam;        /* Lambda interval */
   int lam;           /* Lambda range loop counter */
   int mul;           /* Multiplicity loop counter */
   int nod;           /* Nodals loop counter */
   float xs, ys;      /* Spot position (points) */
   float xdisc, ydisc;/* Key colour disc position (points) */
   float xstr, ystr;  /* Key string position */
   float lam1, lam2;  /* Wavelength limits for range */
   int ir;            /* Lambda bin (0 to 7) for current reflection */
   int im;            /* Multiplicity (0 to 3) bin for current reflection */
   int colr;          /* Colour (/code) for xdl_laue_sim_symb1 */
   float r1;                   /* Laue spot distance from centre */
   float r2;                   /* Gnomonic spot distance from centre */
   float twoth;                /* 2-theta value */
   float thg;                  /* Gnomonic theta angle */
   float ratio;                /* r2/r1 ratio */
   float piov2 = 1.5707963;    /* Pi/2 */

   /* Simulation area */

   gd->ps_xc = 297.6; 
   gd->ps_yc = 520.0;
   gd->plot_rad = 90.0;
   gd->ps_scal = gd->plot_rad/gd->filmr;
   fprintf (gd->psfil,"initclip clippath 0 0 0 setrgbcolor fill\n");

   range = gd->lmax - gd->lmin;

   switch (gd->color_coding)
   {
      case 1: /* Colour coding by wavelength */
      case 3: /* As above but multiples white */
         for (lam=0;lam<8;++lam)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               ir = (int) (8*(gd->lam[k]-gd->lmin)/range);
               if (ir<0) ir = 0;
               if (ir>7) ir = 7;
               if (ir!=lam) continue;
               ratio = 1.0;
               if (gd->gnom)
	       {
                  r1 = sqrtf (gd->xf[k]*gd->xf[k]+gd->yf[k]*gd->yf[k]);
                  if (r1<gd->gnomr) continue;
                  twoth = atanf (r1/gd->ctof);
                  thg = piov2 - 0.5*twoth;
                  r2 = gd->gdist*tanf(thg);
                  ratio = r2/r1;
               }
               xs = ratio*gd->ps_scal*gd->xf[k]*gd->mm_to_p + gd->ps_xc;
               ys = ratio*gd->ps_scal*gd->yf[k]*gd->mm_to_p + gd->ps_yc;
               colr = 7 - lam;
               if (gd->color_coding==3&&gd->mult[k]>1)
               {
                  fprintf (gd->psfil, "white %.2f %.2f %.3f S1 S\n",
                           xs, ys, gd->ps_spot_rad);
               }
               else
               {
                  fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
               }
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            if (gd->color_coding==3)
            {
               fprintf (gd->psfil,
               "(Colour coded by wavelength; multiples coded white) show\n");
            }
            else
            {
               fprintf (gd->psfil,
              "(Colour coded by wavelength; highest wavelength for multiples)");
               fprintf (gd->psfil," show\n");
            }
            dlam = range/8.0;
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");
            for(i=0;i<8;++i)
            {
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        7-i, xdisc, ydisc);
               lam1 = gd->lmin + i*dlam;
               lam2 = lam1 + dlam;
               if (i==7) lam2 = gd->lmax;
               fprintf (gd->psfil, "prcolr\n");
               fprintf (gd->psfil, "%.2f %.2f M (%.3f to %.3f Angstroms) show\n",
                       xstr, ystr, lam1, lam2);
               ydisc -= 15.0;
               ystr -= 15.0;
            }
            if (gd->color_coding==3)
            {
               fprintf (gd->psfil, "white %.2f %.2f 1.5 S1 S\n", 
                        xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               fprintf (gd->psfil, "%.2f %.2f M (multiple) show\n", xstr, ystr);
               ydisc -= 15.0;
               ystr -= 15.0;
           }
         }
         break;
      case 2: /* Colour coded by multiplicity */
         for (mul=0;mul<4;++mul)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               im = gd->mult[k] - 1;
               if (im>3) im = 3;
               if (im!=mul) continue;
               ratio = 1.0;
               if (gd->gnom)
	       {
                  r1 = sqrtf (gd->xf[k]*gd->xf[k]+gd->yf[k]*gd->yf[k]);
                  if (r1<gd->gnomr) continue;
                  twoth = atanf (r1/gd->ctof);
                  thg = piov2 - 0.5*twoth;
                  r2 = gd->gdist*tanf(thg);
                  ratio = r2/r1;
               }
               xs = ratio*gd->ps_scal*gd->xf[k]*gd->mm_to_p + gd->ps_xc;
               ys = ratio*gd->ps_scal*gd->yf[k]*gd->mm_to_p + gd->ps_yc;
               colr = 6 - 2*im;
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                        colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil, "(Colour coded by multiplicity) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            for(i=0;i<4;++i)
            {
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        6-2*i, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "%.2f %.2f M (single) show\n",
                              xstr, ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "%.2f %.2f M (double) show\n",
                              xstr, ystr);
                     break;
                  case 2:
                     fprintf (gd->psfil, "%.2f %.2f M (triple) show\n",
                              xstr, ystr);
                     break;
                  case 3:
                     fprintf (gd->psfil, "%.2f %.2f M (> 3) show\n",
                              xstr, ystr);
                     break;
               }
               ydisc -= 15;
               ystr -= 15;
            }
         }
         break;
      case 4: /* Colour coded by nodality */
         for (nod=0;nod<2;++nod)
         {
            for (i=0;i<gd->nspot;++i)
            {
               k = gd->idxlam[i] - 1;
               if (gd->spot_type!=1)
               {
                  if (gd->spot_type==2&&gd->mult[k]>1) continue;
                  if (gd->spot_type==3&&gd->mult[k]<2) continue;
               }
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&(gd->flags[k]&0X1)!=0) continue;
                  if (gd->spatials==3&&(gd->flags[k]&0X1)==0) continue;
               }
               nodal = 1;
               xdl_laue_sim_nodidx (gd, k, &h_n, &k_n, &l_n, &harmonic);
               if (h_n<0) h_n = -h_n;
               if (k_n<0) k_n = -k_n;
               if (l_n<0) l_n = -l_n;
               if (h_n>gd->nodidx_cur||k_n>gd->nodidx_cur||l_n>gd->nodidx_cur) 
                  nodal = 0;
               if (nodal!=nod) continue;
               ratio = 1.0;
               if (gd->gnom)
	       {
                  r1 = sqrtf (gd->xf[k]*gd->xf[k]+gd->yf[k]*gd->yf[k]);
                  if (r1<gd->gnomr) continue;
                  twoth = atanf (r1/gd->ctof);
                  thg = piov2 - 0.5*twoth;
                  r2 = gd->gdist*tanf(thg);
                  ratio = r2/r1;
               }
               xs = ratio*gd->ps_scal*gd->xf[k]*gd->mm_to_p + gd->ps_xc;
               ys = ratio*gd->ps_scal*gd->yf[k]*gd->mm_to_p + gd->ps_yc;
               colr = 6 - 4*nod;
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                        colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil, 
               "(Plot of nodals versus non-nodals \\(nodal index = %d\\)) show\n",
                gd->nodidx_cur);
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            for(i=0;i<2;++i)
            {
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        6-4*i, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "prcolr %.2f %.2f M (non-nodal) show\n",
                              xstr, ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "prcolr %.2f %.2f M (nodal) show\n",
                              xstr, ystr);
                     break;
               }
               ydisc -= 15;
               ystr -= 15;
            }
         }
         break;
   }
   /* Add remainder of key if required */
   if (gd->ps_addkey)
   {
      fprintf (gd->psfil, "prcolr\n");
      switch (gd->spot_type)
      {
         case 1:
            fprintf (gd->psfil, "100 %.2f M (Singles and multiples;) show\n",
                    ystr);
            break;

         case 2:
            fprintf (gd->psfil, "100 %.2f M (Singles only;) show\n",
                    ystr);
            break;

         case 3:
            fprintf (gd->psfil, "100 %.2f M (Multiples only;) show\n",
                    ystr);
            break;
      }
      switch (gd->spatials)
      {
         case 1:
            fprintf (gd->psfil, "( Spatial overlaps included) show\n");
            break;

         case 2:
            fprintf (gd->psfil, "( Spatial overlaps excluded) show\n");
            break;

         case 3:
            fprintf (gd->psfil, "( Spatial overlaps only) show\n");
            break;
      }
   }

   /* Comment */

   if (strlen(gd->ps_comment)>0)
   {
      fprintf (gd->psfil, "prcolr\n");
      fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 56 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->ps_comment);
      fprintf (gd->psfil,"490 {50 ycomm M show /ycomm ycomm 14 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }

   /* Show page */

   fprintf (gd->psfil,"showpage\n");
   fclose (gd->psfil);

   return;
}
/*
************************************
**     xdl_laue_sim_psdraw2       **
************************************

Purpose: Routine to perform postcript o/p for type 2 display

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_psdraw2 (gd)

XDl_laue_sim *gd;    /* Pointer to global data area */

{
   int i, j;          /* Temp/loop variables */
   int styp;          /* Symbol type 1=spot, 2=cross, 3=star */
   int colr;          /* Colour flag for xdl_laue_sim_symb2 */
   int nodal;         /* Nodal flag 1=yes, 0=no */
   int h_n, k_n, l_n; /* Nodal indices */
   int harmonic;      /* Harmonic number */
   float cur_dmin2;   /* Current value of 1/dmin**2 */
   float xorig;       /* y origin of box around plot */
   float yorig;       /* y origin of box around plot */
   float xlen;        /* x length of box around plot */
   float ylen;        /* y length of box around plot */
   float xs;          /* Spot/symbol x coordinate (Postscript) */
   float ys;          /* Spot/symbol y coordinate (Postscript) */
   float xc1;         /* Annotation column x coordinate */
   float xc2;         /* Annotation column x coordinate */
   float xc3;         /* Annotation column x coordinate */
   float xc4;         /* Annotation column x coordinate */
   float xc5;         /* Annotation column x coordinate */
   float xc6;         /* Annotation column x coordinate */
   float ytop;        /* Annotation top y value */
   float dy;          /* Line spacing for annotation */
   float yy;          /* Annotation y coordinate */
   float r1;                   /* Laue spot distance from centre */
   float r2;                   /* Gnomonic spot distance from centre */
   float twoth;                /* 2-theta value */
   float thg;                  /* Gnomonic theta angle */
   float ratio;                /* r2/r1 ratio */
   float piov2 = 1.5707963;    /* Pi/2 */
   time_t tm;         /* Time value */
   char datstr[30];   /* Date & time string */
   static char *sys_str[] = {"Triclinic",
                            "Monoclinic",
                            "Orthorhombic",
                            "Tetragonal",
                            "Hexagonal",
                            "Rhombohedral",
                            "Cubic"};
   static char *latt_str[] = {"P","A","B","C","I","F","R"};
   static char *ax_str[] = {"-c*","-b*","-a*"," ","+a*","+b*","+c*"};

   /* Plot heading */

   time(&tm);
   strcpy(datstr,ctime(&tm));
   datstr[24] = 0;
   fprintf (gd->psfil,"/Helvetica findfont 16 scalefont setfont\n");
   fprintf (gd->psfil,"95.0 780.0 M (Laue Simulation - LAUEGEN %s) show\n",
            datstr);

   /* Simulation area */

   gd->ps_xc = 297.6; 
   gd->ps_yc = 520.0;
   gd->ps_scal = gd->plot_rad/gd->filmr;
   gd->ps_pixsiz = (gd->plot_rad/(float)gd->half_size)*gd->mm_to_p;
   xorig = gd->ps_xc - gd->plot_rad*gd->mm_to_p;
   yorig = gd->ps_yc - gd->plot_rad*gd->mm_to_p;
   if (gd->plot_rad<=90.0)
   {
      xlen = 2*gd->plot_rad*gd->mm_to_p;
      xorig = gd->ps_xc - gd->plot_rad*gd->mm_to_p;
      yorig = gd->ps_yc - gd->plot_rad*gd->mm_to_p;
   }
   else
   {
      xlen = 180.0*gd->mm_to_p;
      xorig = gd->ps_xc - 90.0*gd->mm_to_p;
      yorig = gd->ps_yc - 90.0*gd->mm_to_p;
   }
   ylen = xlen;
   fprintf (gd->psfil,"gsave\n");
   fprintf (gd->psfil,"%.3f %.3f M\n",xorig,yorig);
   fprintf (gd->psfil,"%.3f 0 R\n",xlen);
   fprintf (gd->psfil,"0 %.3f R\n",ylen);
   fprintf (gd->psfil,"%.3f 0 R\n",-xlen);
   fprintf (gd->psfil,"0 %.3f R\n",-ylen);
   fprintf (gd->psfil,"S\n");
   fprintf (gd->psfil,"%.3f %.3f M\n",xorig,yorig);
   fprintf (gd->psfil,"%.3f 0 R\n",xlen);
   fprintf (gd->psfil,"0 %.3f R\n",ylen);
   fprintf (gd->psfil,"%.3f 0 R\n",-xlen);
   fprintf (gd->psfil,"0 %.3f R\n",-ylen);
   fprintf (gd->psfil,"closepath clip newpath\n");
   cur_dmin2 = 1/((gd->cur_dmin)*(gd->cur_dmin));
   if (gd->opt_highlight2==2&&gd->recalc_mult) xdl_laue_sim_mult(gd);
   for(i=0;i<gd->nspot;++i)
   {
      if (gd->lam[i]<gd->cur_lmin||gd->lam[i]>gd->cur_lmax) continue;
      if (gd->dthr[i]>cur_dmin2) continue;
      styp = 1;
      switch (gd->opt_highlight2)
      {
         case 2:
            if (gd->mult[i]==2) styp = 2;
            if (gd->mult[i]>2) styp = 3;
            break;
         case 3:
            nodal = 1;
            xdl_laue_sim_nodidx (gd, i, &h_n, &k_n, &l_n, &harmonic);
            if (h_n<0) h_n = -h_n;
            if (k_n<0) k_n = -k_n;
            if (l_n<0) l_n = -l_n;
            if (h_n>gd->nodidx_cur||k_n>gd->nodidx_cur||l_n>gd->nodidx_cur) 
               nodal = 0;
            if (nodal) styp = 2;
            break;
      }
      ratio = 1.0;
      if (gd->gnom)
      {
         r1 = sqrtf (gd->xf[i]*gd->xf[i]+gd->yf[i]*gd->yf[i]);
         if (r1<gd->gnomr) continue;
         twoth = atanf (r1/gd->ctof);
         thg = piov2 - 0.5*twoth;
         r2 = gd->gdist*tanf(thg);
         ratio = r2/r1;
      }
      xs = ratio*gd->ps_scal*gd->xf[i]*gd->mm_to_p + gd->ps_xc;
      ys = ratio*gd->ps_scal*gd->yf[i]*gd->mm_to_p + gd->ps_yc;

      fprintf (gd->psfil,"black\n");
      switch (styp)
      {
         case 1:
            if (gd->ps_color) fprintf (gd->psfil,"black ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xs, ys, gd->ps_spot_rad);
            break;
         case 2:
            if (gd->ps_color&&gd->opt_highlight2==2)fprintf (gd->psfil,"blue ");
            if (gd->ps_color&&gd->opt_highlight2==3)fprintf (gd->psfil,"red ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xs, ys, gd->ps_symb_hs);
            break;
         case 3:
            if (gd->ps_color) fprintf (gd->psfil,"red ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S3 S\n", 
            xs, ys, gd->ps_symb_hs);
            break;
      }

   }

   /* Draw labels if required and if present */

   if (gd->opt_labels2>1&&gd->num_labels>0)
   {
      fprintf (gd->psfil,"/linwsav currentlinewidth def\n");
      if (gd->plot_rad<90.0)
      {
         fprintf (gd->psfil,"%.2f setlinewidth\n", (gd->plot_rad/90.0));
      }
      for (i=0;i<gd->num_labels;++i) xdl_laue_sim_pslabel (gd, i);
      fprintf (gd->psfil,"linwsav setlinewidth\n");
   }

   fprintf (gd->psfil,"grestore\n");

   /* Annotation */

   fprintf (gd->psfil,"/Helvetica findfont 15 scalefont setfont\n");
   xc1 = 40.0; 
   xc2 = 100.0; 
   xc3 = 220.0; 
   xc4 = 295.0;
   xc5 = 450.0;
   xc6 = 460.0;
   dy = 15.0;
   ytop = 220.0;
   yy = ytop;
   fprintf (gd->psfil,"%.2f %.2f M (a:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[0]);
   yy = ytop - dy;
   fprintf (gd->psfil,"%.2f %.2f M (b:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[1]);
   yy = ytop - 2.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M (c:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[2]);
   yy = ytop - 3.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M (alpha:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[3]);
   yy = ytop - 4.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M (beta:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[4]);
   yy = ytop - 5.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M (gamma:) show %.2f %.2f M (%.2f) show\n",
            xc1, yy, xc2, yy, gd->cell[5]);
   yy = ytop - 6.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (System:) show %.2f %.2f M (%s \\(%s\\)) show\n",
            xc1, yy, xc2, yy, sys_str[gd->latt[0]-1], latt_str[gd->latt[1]-1]);
   yy = ytop - 7.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Setting:) show %.2f %.2f M (%s, %s) show\n",
            xc1, yy, xc2, yy, ax_str[gd->latt[2]+3], ax_str[gd->latt[3]+3]);
   yy = ytop;
   fprintf (gd->psfil,
            "%.2f %.2f M (Lam-min:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->cur_lmin);
   yy = ytop - dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Lam-max:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->cur_lmax);
   yy = ytop - 2.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Dmin:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->cur_dmin);
   yy = ytop - 3.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (c_to_f:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->ctof);
   yy = ytop - 4.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Phix:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->phis[0]);
   yy = ytop - 5.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Phiy:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->phis[1]);
   yy = ytop - 6.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Phiz:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->phis[2]);
   yy = ytop - 7.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (Spindle:) show %.2f %.2f M (%.2f) show\n",
            xc3, yy, xc4, yy, gd->phis[3]);

   switch (gd->opt_highlight2)
   {
      case 1: /* No highlighting */
         break;

      case 2: /* Multiples highlighted */
         fprintf (gd->psfil,"/Helvetica findfont 12 scalefont setfont\n");
         dy = 12.0;
         yy = ytop + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"black ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xc5, yy, gd->ps_spot_rad);
         yy = ytop - dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"blue ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xc5, yy, gd->ps_symb_hs);
         yy = ytop - 2.0*dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"red ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S3 S\n", 
            xc5, yy, gd->ps_symb_hs);
         fprintf (gd->psfil,"black\n");
         yy = ytop;
         fprintf (gd->psfil,"%.2f %.2f M (Singles) show\n", xc6, yy);
         yy = ytop - dy;
         fprintf (gd->psfil,"%.2f %.2f M (Doubles) show\n", xc6, yy);
         yy = ytop - 2.0*dy;
         fprintf (gd->psfil,"%.2f %.2f M (Higher multiples) show\n", xc6, yy);
         break;

      case 3: /* Nodals highlighted */
         fprintf (gd->psfil,"/Helvetica findfont 12 scalefont setfont\n");
         dy = 12.0;
         yy = ytop + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"black ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xc5, yy, gd->ps_spot_rad);
         yy = ytop - dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"red ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xc5, yy, gd->ps_symb_hs);
         fprintf (gd->psfil,"black\n");
         yy = ytop;
         fprintf (gd->psfil,"%.2f %.2f M (Non-nodals) show\n", xc6, yy);
         yy = ytop - dy;
         fprintf (gd->psfil,"%.2f %.2f M (Nodals) show\n", xc6, yy);
         yy = ytop - 2.0*dy;
         fprintf (gd->psfil,"%.2f %.2f M (Nodal index = %d) show\n",
                  xc5, yy, gd->nodidx_cur);
         break;
   }

   /* Comment */

   if (strlen(gd->ps_comment)>0)
   {
      fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 80 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->ps_comment);
      fprintf (gd->psfil,"510 {40 ycomm M show /ycomm ycomm 14 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }

   /* Show page */

   fprintf (gd->psfil,"showpage\n");
   fclose (gd->psfil);
   return;
}
/*
********************************
**     xdl_laue_sim_pslabel   **
********************************

Purpose: Draw an indices label for postscript plot

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_pslabel (gd, loff)

XDl_laue_sim *gd;    /* Pointer to global data area */
int loff;            /* Offset in labels list */
{
   XDl_laue_sim_label *lt;   /* Pointer to current labels structure */
   int l;                    /* Temp/loop variable */
   int ll;                   /* Length of character string */
   int digit;                /* Current digit number 0-9 */
   float xch, ych;           /* Current character position (points)*/
   int xc, yc;               /* Box corner for attached line (pixels)*/
   int xspot, yspot;         /* Spot position (pixels)*/
   float x1;                 /* x corner of label (points)*/
   float y1;                 /* y corner of label (points)*/
   float xsp;                /* Spot x (points) */
   float ysp;                /* Spot y (points) */
   float w;                  /* label width (points) */
   float h;                  /* label height (points) */
   float r1;                 /* Laue spot distance from centre */
   float r2;                 /* Gnomonic spot distance from centre */
   float twoth;              /* 2-theta value */
   float thg;                /* Gnomonic theta angle */
   float ratio;              /* r2/r1 ratio */
   float piov2 = 1.5707963;  /* Pi/2 */

   lt = gd->labels[loff];
   if (lt->disp==0) return;
   x1 = (lt->x1-gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   y1 = (gd->half_size-lt->y1)*gd->ps_pixsiz + gd->ps_yc;
   w = (lt->x2-lt->x1)*gd->ps_pixsiz;
   h = (lt->y2-lt->y1)*gd->ps_pixsiz;
   fprintf (gd->psfil,"white ");
   fprintf (gd->psfil,"%.2f %.2f M\n",x1,y1);
   fprintf (gd->psfil,"%.3f 0 R\n",w);
   fprintf (gd->psfil,"0 %.3f R\n",-h);
   fprintf (gd->psfil,"%.3f 0 R\n",-w);
   fprintf (gd->psfil,"0 %.3f R\n",h);
   fprintf (gd->psfil,"closepath fill\n");
   fprintf (gd->psfil,"black ");
   fprintf (gd->psfil,"%.2f %.2f M\n",x1,y1);
   fprintf (gd->psfil,"%.3f 0 R\n",w);
   fprintf (gd->psfil,"0 %.3f R\n",-h);
   fprintf (gd->psfil,"%.3f 0 R\n",-w);
   fprintf (gd->psfil,"0 %.3f R\n",h);
   fprintf (gd->psfil,"S\n");
   ll = strlen (lt->hklstr);
   if (ll<=0) return;
   xch = (lt->x1 + 2 - gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   ych = (gd->half_size - lt->y2 + 2)*gd->ps_pixsiz + gd->ps_yc;

   for (l=0;l<ll;++l)
   {
      if (lt->hklstr[l]=='-')
      {
         fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n",
                 xch, ych+4.0*gd->ps_pixsiz, xch+4.0*gd->ps_pixsiz,
                 ych+4.0*gd->ps_pixsiz);
         xch += 7.0*gd->ps_pixsiz;
      }
      else if (lt->hklstr[l]>='0'&&lt->hklstr[l]<='9')
      {
         digit = lt->hklstr[l] - '0';
         xdl_laue_sim_psdigit (gd, digit, xch, ych);
         xch += 7.0*gd->ps_pixsiz;
      }
      else
      {
         xch += 4.0*gd->ps_pixsiz;
      }
   }

   /* Draw attaching line to spot */

   xspot = gd->xpix[lt->idx];
   yspot = gd->ypix[lt->idx];
   if (xspot>=lt->x1&&xspot<=lt->x2&&yspot>=lt->y1&&yspot<=lt->y2) return;
   xc = lt->x2;
   if (abs(xspot-lt->x1)<abs(xspot-lt->x2)) xc = lt->x1;
   yc = lt->y2;
   if (abs(yspot-lt->y1)<abs(yspot-lt->y2)) yc = lt->y1;
   x1 = (xc-gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   y1 = (gd->half_size-yc)*gd->ps_pixsiz + gd->ps_yc;
   ratio = 1.0;
   if (gd->gnom)
   {
      r1 = sqrtf (gd->xf[lt->idx]*gd->xf[lt->idx]
                  +gd->yf[lt->idx]*gd->yf[lt->idx]);
      twoth = atanf (r1/gd->ctof);
      thg = piov2 - 0.5*twoth;
      r2 = gd->gdist*tanf(thg);
      ratio = r2/r1;
   }
   xsp = ratio*gd->ps_scal*gd->xf[lt->idx]*gd->mm_to_p + gd->ps_xc;
   ysp = ratio*gd->ps_scal*gd->yf[lt->idx]*gd->mm_to_p + gd->ps_yc;
   fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", x1, y1, xsp, ysp);

   return;
}
/*
**********************************
**     xdl_laue_sim_psdigit     **
**********************************

Purpose: Draw digit for label for Postscript output

Author:  John W. Campbell, July 1992

*/

void xdl_laue_sim_psdigit (gd, digit, xch, ych)

XDl_laue_sim *gd;    /* Pointer to global data area */
int digit;           /* Digit number 0 to 9 */
float xch;           /* x position (bottom left corner of digit) (points)*/
float ych;           /* y position (bottom left corner of digit) (points)*/

{
   static short digits[10][16][2]={
   {{1,2},{1,7},{2,8},{4,8},{5,7},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{3,8},{3,1},{-1,1},{5,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{1,2},{1,1},
    {5,1},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},{2,5},{-4,5},
    {5,4},{5,2},{4,1},{2,1},{1,2},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,3},{5,3},{-4,1},{4,5},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,8},{1,5},{4,5},{5,4},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,2},{2,1},{4,1},{5,2},{5,3},{4,4},
    {2,4},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,8},{5,8},{5,6},{1,2},{1,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{2,5},{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},
    {2,5},{1,4},{1,2},{2,1},{4,1},{5,2},{5,4},{4,5}},

   {{1,1},{5,5},{5,7},{4,8},{2,8},{1,7},{1,6},{2,5},
    {4,5},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
   };
   int i;                   /* Temp/loop variable */
   int dx1, dy1, dx2, dy2;  /* Pixel point offsets for lines within character */
   float x1, y1, x2, y2;    /* Coordinates for ends of lines */

   for(i=0;i<16;++i)
   {
      if (digits[digit][i][0]==0) break;
      if (i==0||digits[digit][i][0]<0)
      {
         dx1 = digits[digit][i][0]; if (dx1<0) dx1 = -dx1;
         dy1 = digits[digit][i][1]; if (dy1<0) dy1 = -dy1;
         continue;
      }
      dx2 = digits[digit][i][0];
      dy2 = digits[digit][i][1];
      x1 = xch + (dx1 - 1)*gd->ps_pixsiz;
      y1 = ych + (dy1 - 1)*gd->ps_pixsiz;
      x2 = xch + (dx2 - 1)*gd->ps_pixsiz;
      y2 = ych + (dy2 - 1)*gd->ps_pixsiz;
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L\n",x1, y1, x2, y2);
      dx1 = dx2;
      dy1 = dy2;
   }
   fprintf (gd->psfil, "S\n");
     
   return;
}


/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Calculate size requirements - xdl_laue_sim_getsize
The routine xdl_laue_sim_getsize (xdlf_laue_sim_getsize) is used to
get the required size for a Laue simulation display view-object.
-end*/
/*
************************************
**     xdl_laue_sim_getsize       **
************************************

Purpose: Return the size requirements of the Laue simulation display
view-object.

Author:  John W. Campbell, July 1992

*/
/*-Fortran:
         CALL XDLF_LAUE_SIM_GETSIZE (ITYPE, ISIZE, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
ITYPE     (R)   Simulation type =1 Colour, = 2 Monochrome with
                soft limits options etc. (see type)
ISIZE     (R)   Half width/height for displayed pattern in 
                pixels (see size)
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_laue_sim_getsize (type, size, w, h)
/*end*/
 
/*-Parameters:*/
int       type;           /* Simulation type =1 Colour, =2 Monochrome with
                             soft limit sliders etc.*/
int       size;           /* Pattern display area half width/height in
                             pixels (R)*/
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int area_width;             /* Simulation display area width */
   int area_height;            /* Simulation display area height */
   int panel_1a_width;         /* Width for 'a' panel for type 1
                                  display (show reflection details) */
   int panel_1b_width;         /* Width for 'b' panel for type 1
                                  display */
   int panel_1_height;         /* Height of panels for type 1 display */
   int panel_2a_width;         /* Width for 'a' panel for type 2
                                  display (show reflection details) */
   int panel_2b_width;         /* Width for 'b' panel for type 2
                                  display (control area) */
   int panel_2_height;         /* Height of panels for type 2 display */
   int panels_width;           /* Control panels width needed */
   int panels_height;          /* Control panels height needed */

   /* Calculate the size requirements */

   if (type<1||type>2) type =1;

   /* Get size requirements */

   area_width = 2*size + 1 + 2;
   area_height = 2*size + 1 + 2;
   switch (type)
   {
      case 1:
         panel_1a_width = PANEL_1A_WIDTH;
         panel_1b_width = PANEL_1B_WIDTH;
         panel_1_height = PANEL_1_HEIGHT;
         panels_width = panel_1a_width + 4 + panel_1b_width;
         panels_height = panel_1_height;
         break;
     
      case 2:
         panel_2a_width = PANEL_2A_WIDTH;
         panel_2b_width = PANEL_2B_WIDTH;
         panel_2_height = PANEL_2_HEIGHT;
         panels_width = panel_2a_width + 4 + panel_2b_width;
         panels_height = panel_2_height;
         break;
   }
   if (panels_width>area_width)
   {
      *w = panels_width + 2*BORDER_WIDTH + 2;
   }
   else
   {
      *w = area_width + 2*BORDER_WIDTH + 2;
   }
   *h = area_height + SEP_H_MIN + panels_height 
        + 4 + PANEL_PS_HEIGHT
        + 2*BORDER_HEIGHT + 2;

   return;
}

/* Fortran binding: xdlf_laue_sim_getsize */

void xdlf_laue_sim_getsize (itype, isize, iwidth, iheight)
int *itype, *isize, *iwidth, *iheight;
{
   xdl_laue_sim_getsize (*itype, *isize, iwidth, iheight);
   return;
}
/*-Routine: Display a new simulation - xdl_laue_sim_new
The routine xdl_laue_sim_new (xdlf_laue_sim_new) is used to display a
new Laue simulation in a Laue simulation view-object. The currently
selected options for the view-object are unchanged. If, for type 2 plots,
the lambda limits or dmin value are changed then new current values for 
lambda-min, lambda-max and dmin will be calculated based on the current slider
positions. All current labels will be removed.
-end*/
/*
****************************
**    xdl_laue_sim_new    **
****************************

Purpose: Replace the current simulation with a new simulation

Author:  John W. Campbell, July 1992

*/
/*-Fortran:
         CALL XDLF_LAUE_SIM_NEW (IVH, NSPOT, IHKL, XF, YF, 
        +                    ALAM, DTHR, MULTI, FLAGS,
        +                    NODHKLS, IDXLAM, IDXDMIN, IDXNOD, FILMR, 
        +                    GNOMR, ALMIN, ALMAX, DMIN, 
        +                    CELL, PHIS, LATT, CTOF, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
NSPOT     (R)   Number of spots predicted (see nspot)
IHKL      (R)   Array of NSPOT 'packed' indices (see hkl)
IK        (R)   Array of NSPOT 'k' indices (see k)
IL        (R)   Array of NSPOT 'l' indices (see l)
XF        (R)   Array of NSPOT 'xf' coordinates (see xf)
YF        (R)   Array of NSPOT 'yf' coordinates (see yf)
ALAM      (R)   Array of NSPOT lambda values (see lam)
DTHR      (R)   Array of NSPOT (1/dmin)**2 thresholds (see dthr)
MULTI     (R)   Array of NSPOT multiplicities flags (see multi)
FLAGS     (R)   Array of NSPOT reflection flags (see flags)
NODHKLS   (R)   Array of NSPOT packed nodal indices (see nodhkls)
IDXLAM    (R)   Index array for lambda sorted data;
                NSPOT values from 1 to NSPOT (see idxlam)
IDXDMIN   (R)   Index array for dmin sorted data;
                NSPOT values from 1 to NSPOT (see idxdmin)
IDXNOD    (R)   Index array for packed nodal index values 
                (see idxnod)
FILMR     (R)   Pattern radius in mm (see filmr)
GNOMR     (R)   Minimum radius in mm (>0.1) from image centre for 
                conversion of plot to a gnomonic projection. For
                a normal Laue simulation, give a value of 0.0
ALMIN     (R)   Lambda minimum of prediction (see lmin)
ALMAX     (R)   Lambda maximum of prediction (see lmax)
DMIN      (R)   Dmin value of prediction (see dmin)
CELL      (R)   Array of the 6 real cell dimensions (see cell)
PHIS      (R)   Array of 4 setting angles, Phix, Phiy, Phiz and
                spindle (see phis)
LATT      (R)   4 integer array; crystal system, lattice type,
                rotation axis, beam axis (see latt)
CTOF      (R)   Crystal to film distance (mm) (see ctof)
IERR      (W)   Returns status from xdl_laue_sim_new call
-end*/

/*-C:*/
int xdl_laue_sim_new (int vh, int nspot, int * hkl, float * xf, float * yf, 
                      float * lam, float * dthr, int * multi, int * flags, 
                      int * nodhkls, int * idxlam, int * idxdmin, 
                      int * idxnod, float filmr, float gnomr,
                      float lmin, float lmax, float dmin, float * cell, 
                      float * phis, int * latt, float ctof)
/*end*/
 
/*-Parameters:
int       vh;              User selected view-object handle (R)
int       nspot;           Number of spots predicted (R)
int       hkl[];           Array of 'nspot' 'hkl' packed indices (as LRL)
                           2**20(h+512)+2**10*(k+512)+l+512 (R)
float     xf[];            Array of 'nspot' 'xf' coordinates in mm from
                           the pattern centre (R) 
float     yf[];            Array of 'nspot' 'yf' coordinates in mm from
                           the pattern centre (R) 
float     lam[];           Array of 'nspot' lambda values (R)
float     dthr[];          Array of 'nspot' (1/dmin)**2 threshold values 
                           (R)
int       multi[];         Array of 'nspot' multiplicities flags as for
                           'LRL' data (see main routine) (R)
int       flags[];         Array of 'nspot' reflection flags
                           Flags format as for LRL' data
                           Bit 0 set = spatial overlap; this is the only
                                       bit used by this routine.
                           (R)
int       nodhkls[];       Array of 'nspot' packed nodal indices; packed
                           as 2**20(h+512)+2**10(k+512)+l+512 (R)
int       idxlam[];        Index array for data sorted by lambda, 'nspot'
                           values from 1 to nspot (R)
int       idxdmin[];       Index array for data sorted by dmin 'nspot'
                           values from 1 to nspot (R)
int       idxnod[];        Index array for data sorted by packed nodal
                           index from 1 to nspot (R)
float     filmr;           Pattern radius in mm (R)
float     gnomr;           Minimum radius in mm (>0.1) from image 
                           centre for conversion of plot to a gnomonic 
                           projection. For a normal Laue simulation, 
                           give a value of 0.0 (R) 
float     lmin;            Lambda minimum of prediction (R)
float     lmax;            Lambda maximum of prediction (R)
float     dmin;            Dmin value of prediction (R)
float     cell[6];         Real cell dimensions a, c, c, alpha, beta, 
                           gamma (R)
float     phis[4];         Setting angles phix, phiy, phiz, spindle (R)
int       latt[4];         Crystal system, lattice, rotation axis,
                           beam axis (R)
float     ctof;            Crystal to film distance (mm) 

-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: View-object handle not found
         bit 1 set: View handle does not match view object
         bit 2 set: Unable to allocate required memory
-end*/
{
   int i;                           /* Temp/loop variable */
   int iv;                          /* Offset in XDL_view_objects array */
   XDl_laue_sim *gd;                /* Pointer tothe global data area */
   char str[20];                    /* Temporary character string */
   char *p;                         /* Temp pointer */
   float d;                         /* Temporary distance value */
   float r1;                   /* Laue spot distance from centre */
   float r2;                   /* Gnomonic spot distance from centre */
   float twoth;                /* 2-theta value */
   float thg;                  /* Gnomonic theta angle */
   float ratio;                /* r2/r1 ratio */
   float piov2 = 1.5707963;    /* Pi/2 */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_laue_sim_new",vh); return 1;}
   if (XDL_view_objects[iv]->type!=1061)
   {
      xdl_view_type_errmsg ("xdl_laue_sim_new", vh, "xdl_laue_sim");
      return 2;
   }
   gd = (XDl_laue_sim *) XDL_view_objects[iv]->gd;

   /* Re-allocate memory to store spot positions in pixels, mults, 
      max_harmonics, colr */

   p = (char *) realloc (gd->xpix, nspot*sizeof(short));
   if (p==0) return 4;
   gd->xpix = (short *) p;
   p = (char *) realloc (gd->ypix, nspot*sizeof(short));
   if (p==0) return 4;
   gd->ypix = (short *) p;
   p = (char *) realloc (gd->mult, nspot*sizeof(unsigned char));
   if (p==0) return 4;
   gd->mult = (unsigned char *) p;
   p = (char *) realloc (gd->max_harmonic, nspot*sizeof(unsigned char));
   if (p==0) return 4;
   gd->max_harmonic = (unsigned char *) p;
   p = (char *) realloc (gd->harm_inc, nspot*sizeof(unsigned char));
   if (p==0) return 4;
   gd->harm_inc = (unsigned char *) p;
   p = (char *) realloc (gd->colr, nspot*sizeof(unsigned char));
   if (p==0) return 4;
   gd->colr = (unsigned char *) p;

   /* Reset parameters */

   gd->nspot = nspot;
   gd->filmr = filmr;
   gd->lmin = lmin;
   gd->lmax = lmax;
   gd->dmin = dmin;
   gd->hkl = hkl;
   gd->xf = xf;
   gd->yf = yf;
   gd->lam = lam;
   gd->dthr = dthr;
   gd->multi = multi;
   gd->flags = flags;
   gd->nodhkls = nodhkls;
   gd->idxlam = idxlam;
   gd->idxdmin = idxdmin;
   gd->idxnod = idxnod;
   gd->gnom = 0;
   gd->gnomr = 0.0;
   gd->gdist = 0.0;
   if (gnomr>0.1)
   {
      gd->gnom = 1;
      thg = piov2 - 0.5*atanf(gnomr/ctof);
      gd->gdist = filmr/tanf(thg);
      gd->gnomr = gnomr;
   }

   /* Get spot pixel positions (Note: reversal of sign needed for y) */

   for (i=0;i<nspot;++i)
   {
      if (gd->gnom)
      {
         r1 = sqrtf (gd->xf[i]*gd->xf[i]+gd->yf[i]*gd->yf[i]);
         if (r1>=gnomr)
	 {
            twoth = atanf (r1/gd->ctof);
            thg = piov2 - 0.5*twoth;
            r2 = gd->gdist*tanf(thg);
            ratio = r2/r1;
            d = ratio*((gd->xf[i]/gd->filmr)*(float)gd->half_size);
            gd->xpix[i] = gd->half_size + ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
            d = ratio*((gd->yf[i]/gd->filmr)*(float)gd->half_size);
            gd->ypix[i] = gd->half_size - ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
         }
         else
	 {
            gd->xpix[i] = -1;
            gd->ypix[i] = -1;
         }
      }
      else
      {
         d = (gd->xf[i]/gd->filmr)*(float)gd->half_size;
         gd->xpix[i] = gd->half_size + ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
         d = (gd->yf[i]/gd->filmr)*(float)gd->half_size;
         gd->ypix[i] = gd->half_size - ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
      }
   }

   /* Remove labels */

   if (gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i)
      {
         if (gd->labels[i]!=NULL) free ((char *)gd->labels[i]);
      }
   }
   gd->num_labels = 0;
   gd->move_label = 0;

   if (gd->type==2)
   {
      gd->cur_lmin = gd->lmin 
                     + ((50*gd->cur_lmin_slider_val)/100)*gd->lmin_interval;
      sprintf (str,"%.3f",gd->cur_lmin);
      xdl_panel_label_set (gd->lmin_curval, str, 0, 0);
      gd->cur_lmax = gd->lmax 
                  - ((50*(100-gd->cur_lmax_slider_val))/100)*gd->lmax_interval;
      sprintf (str,"%.3f",gd->cur_lmax);
      xdl_panel_label_set (gd->lmax_curval, str, 0, 0);
      gd->cur_dmin = gd->dmin 
                     + ((50*gd->cur_dmin_slider_val)/100)*gd->dmin_interval;
      sprintf (str,"%.3f",gd->cur_dmin);
      xdl_panel_label_set (gd->dmin_curval, str, 0, 0);
   }

   gd->disp_ref = -1;

   if (gd->type==1)
   {   
      xdl_laue_sim_mult (gd);
   }
   else
   {
      gd->recalc_mult = 1;
      xdl_laue_sim_setcolors (gd, 1, 0);
   }

   /* Draw new plot */

   switch (gd->type)
   {
      case 1:
         xdl_laue_sim_redraw1 (gd);
         break;

      case 2:
         xdl_laue_sim_redraw2 (gd);
         break;
   }
   return 0;
}

/* Fortran binding: xdlf_laue_sim_new */

void xdlf_laue_sim_new (ivh, nspot, ihkl, xf, yf, alam, dthr, 
                        multi, flags, nodhkls,
                        idxlam, idxdmin, idxnod, filmr, gnomr,
                        almin, almax, dmin, 
                        cell, phis, latt, ctof, ierr)

int *ivh, *nspot;
int *ihkl, *multi, *flags, *nodhkls, *idxlam, *idxdmin, *idxnod, *latt;
int *ierr;
float *xf, *yf, *alam, *dthr, *filmr, *gnomr, *almin, *almax, *dmin; 
float *cell, *phis;
float *ctof;

{
   *ierr = xdl_laue_sim_new (*ivh, *nspot, ihkl, xf, yf, alam, dthr, 
                            multi, flags, nodhkls, idxlam, idxdmin, idxnod, 
                            *filmr, *gnomr, *almin, *almax, *dmin, 
                            cell, phis, latt, *ctof);
   return;
}
