/* Copyright 1996-1999, CLRC Daresbury Laboratory.
   For licence conditions, see the file COPYING in the distribution.
 */

/*-Title: The Rotation Image Simulation View-Object Routines
-end*/
 
/*-Intro:
The rotation pattern simulation view-object is used to display the simulation 
of a single crystal monochromatic X-ray rotation diffraction pattern (the 
pattern prediction itself must be done externally to the routine and the
reflections stored in a list using the appropriate 'rpf' functions). 
The view-object allows either for colour coded simulations or for interactive
black and white simulations where various parameters may be modified via a 
slider to show the effects that they have on the patterns.
The indices of selected spots may be displayed. Labels may be drawn on
the interactive black and white simulations. The view-object also provides a 
hard copy (Postscript file) option.
-end*/
 
/*-Section: Creating and Handling a Rotation Simulation View-Object
-end*/
 

/*Routines List:
The rotation pattern simulation view-object has the following routines 
available:

   xdl_rot_sim              Create a rotation pattern simulation view-object
   xdl_rot_sim_getsize      Calculate size requirements
   xdl_rot_sim_new          Display a new simulation
*/

/*-Routine: Create a rotation simulation view-object - xdl_rot_sim
The routine xdl_rot_sim (xdlf_rot_sim) is used to create a new rotation
image simulation display view-object. The simulation itself is calculated
externally to the routine.
-end*/
/*
************************
**    xdl_rot_sim     **
************************

Purpose: Display a simulated rotation diffraction pattern

Author:  John W. Campbell, June 1995

*/
/*-Fortran:
         CALL XDLF_ROT_SIM  (IVH, IVHPARENT, IX, IY, ICSET, ITYPE, ISIZE,
        +                    IORD, MINDX, ICRYS, IMGNUM, FILMR, SCAX,
        +                    ROTS, ROTE, ISYN, ETA, DIVV, DIVH, DELAMB,
        +                    DELCOR, IAX_V, IAX_H, NWMAX, CUR_ROTR,
        +                    CUR_ETA, CUR_DIVV, CUR_DIVH, CUR_DELAMB,
        +                    CELL, UMAT, DELPHI, WAVE, LSYS, CTOD,
        +                    NAME1, LEN1, NAME2, LEN2, MINW, MINH, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IVHPARENT (R)   View-object handle for parent 0=none 
                (see vh_parent)
IX        (R)   X position (may be -1 if no parent) (see x)
IY        (R)   Y position (may be -1 if no parent) (see y)
ICSET     (R)   Colorset number (see cset)
ITYPE     (R)   Simulation type =1 Colour, = 2 Monochrome with
                soft limits options etc. (see type)
ISIZE     (R)   Half width/height for displayed pattern in 
                pixels (see size)
IORD      (R)   Order of local detector axes flag. First axis
                along X windows 'x' direction on screen (to
                right), second axis along X-windows 'y' direction 
                (down).
                1  +ax1 +ax2
                2  +ax1 -ax2
                3  -ax1 +ax2
                4  -ax1 -ax2
                5  +ax2 +ax1
                6  +ax2 -ax1
                7  -ax2 +ax1
                8  -ax2 -ax1
MINDX     (R)   Index to spot list (as returned from 
                rpff_reflist_init call)
ICRYS     (R)   Crystal number
IMGNUM    (R)   Image number
FILMR     (R)   Pattern radius in mm (see filmr)
SCAX(3)   (R)   Rotation (scan) axis vector as used in the 
                prediction 
ROTS      (R)   Rotation start for current image (degrees) as used
                in the prediction
ROTE      (R)   Rotation end for current image (>ROTS) (degrees) 
                as used in the prediction
ISYN      (R)   Flag =1 synchrotron, =0 lab source as used in the
                prediction
ETA       (R)   Mosaicity (halfwidth, radians) as used in the 
                prediction
DIVV      (R)   Verical divergence (half width, radians) as used
                in the prediction
DIVH      (R)   Horizontal divergence (half width, radians) as
                used in the prediction
DELAMB    (R)   Delta-lambda/lambda as used in the prediction    
DELCOR    (R)   Correlated dispersion (half width, radians as used in
                the prediction
IAX_V     (R)   No. of vertical axis (lab frame) 1=X, 2=Y, 3=Z
                as used in the prediction
IAX_H     (R)   No. of horizontal axis (lab frame) 1=X, 2=Y, 3=Z
                as used in the prediction
NWMAX     (R)   Maximum no. of images to treat as a non too-wide
                partial as used in the prediction
CUR_ROTR  (R)   Current rotation range for type 2 plots (must
                be <= predicted range)
CUR_ETA   (R)   Current 'eta' value for type 2 plots (must
                be <= value used in the prediction)
CUR_DIVV  (R)   Current 'divv' value for type 2 plots (must
                be <= value used in the prediction)
CUR_DIVH  (R)   Current 'divh' value for type 2 plots (must
                be <= value used in the prediction)
CUR_DELAMB(R)   Current 'delamb' value for type 2 plots (must
                be <= value used in the prediction)
CELL(6)   (R)   Real cell parameters in Angstroms and degrees
                for annotation of type 2 Postscript plots
UMAT(3,3) (R)   'U' matrix for annotation of type 2 Postscript 
                plots
DELPHI(3) (R)   Missetting angles in degrees for annotation of 
                type 2 Postscript plots
WAVE      (R)   Wavelength in Angstroms for annotation of type 2 
                Postscript plots
LSYS(2)   (R)   Crystal system 1-7, Lattice type 1-7
CTOD      (R)   Crystal to detector distance (mm)
MINW      (R)   Minimum width for view-object or 0 
                (see min_width)
MINH      (R)   Minimum height for view-object or 0 
                (see min_height)
NAME1     (R)   Character string containing the name of the 1'st
                decector axis (max 4 characters in length)
                ** Pass address using the XDLSTR function **
LEN1      (R)   Length of the name string - must be >0 
                (cf len1) (max 4 chars) 
NAME2     (R)   Character string containing the name of the 2'nd
                detector axis (max 4 characters in length)
                ** Pass address using the XDLSTR function **
LEN2      (R)   Length of the name string - must be >0 
                (cf len1) (max 4 chars) 
IERR      (W)   Returns status from xdl_rot_sim call
-end*/

/* Include XDL external data and headers */

#include "xdl_view_ext.h"
#include "xdl_panel_itm.h"
#include "prl.h"
#include <math.h>
#ifdef VMS
#include <file.h>
#else
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fcntl.h>
#endif

#define PANEL_1A_WIDTH 180    /*Width required for panel 'a' (reflection 
                                details area) including 1 pixel border for 
                                type 1 display*/
#define PANEL_1B_WIDTH 400    /*Width required for panel 'b' (controls)
                                including 1 pixel border for type 1 display*/
#define PANEL_1_HEIGHT 70     /*Height of panels including 1 pixel 
                                border for display type 1*/
#define PANEL_2A_WIDTH 180    /*Width required for panel 'a' (reflection 
                                details area) including 1 pixel border for 
                                type 2 display*/
#define PANEL_2B_WIDTH 415    /*Width required for panel 'b' (controls)
                                including 1 pixel border for type 2 display*/
#define PANEL_2_HEIGHT 70     /*Height of panels including 1 pixel 
                                border for display type 2*/
#define PANEL_PS_HEIGHT 22    /*Height of Postscript control panel including
                                1 pixel border*/
#define SEP_H_MIN 4           /*Minimum separation between contol panels and
                                simulation display area*/
#define MAX_LABELS 100        /* Maximum number of labels allowed */
#define SAV_SIZ 100           /* Size of pixmap to save area for moving 
                                 labels */
#define MARK_SIZ 25           /* Size of pixmap for marked area around spot */
#define TWO20 1048576         /* 2**20 */
#define TWO10 1024            /* 2**10 */
#define TWO12 4096            /* 2**12 */
#define TWO4 16               /* 2**4 */
#define CMS_TO_POINTS 2.8346  /* Conversion factor mm's to points */

#define nint(x)  x>=0.0?(int)(x+0.5):(int)(x-0.5)

/* Define structure for index labels */

typedef struct
{
   int x1;          /* x position, top left of label */
   int y1;          /* y position, top left of label */
   int x2;          /* x position, bottom right of label */
   int y2;          /* y position, bottom right of label */
   int xcoff;       /* x centre of label offset from x1 */   
   int ycoff;       /* y centre of label offset from y1 */   
   int idx;         /* Index in reflections list of spot being labelled */
   char hklstr[15]; /* Label 'hkl' indices string */
   char disp;       /* Displayed status =1 displayed, =0 'silent' */
   short h;         /* Displayed 'h' index */
   short k;         /* Displayed 'k' index */
   short l;         /* Displayed 'l' index */
   
}
XDl_rot_sim_label;

/* Define structure for current variable ordering */

typedef struct
{
   int idx;       /* Index of reflection in reflections list (from 1 upwards)*/
   float var;     /* Value of the current variable */
}
XDl_rot_sim_varidx;

/* Define structure for the global data for the view-object */

typedef struct
{
   Window wframe;                    /* Primary window id */
   Window wdisp;                     /* Paint window id for simulation */
   Window wpanel;                    /* Paint window for control panel */
   Window wrefln;                    /* Paint window for reflection details
                                        area */

   Panel_object var_slider;          /* Variable parameter panel slider */
   Panel_object var_curval;          /* Label with current variable parameter
                                        value */

   Window wpanel_ps;                 /* Paint window for Postscript panel */
   Panel_object reflab[5];           /* Reflection details label items */
   Panel_object slide_type;          /* Slider variable type menu for type 2
                                        plots */
   Panel_object sym2_size;           /* Symbol sizes menu for type 2 plots */
   Panel_object highlight2_opt;      /* Plot type highlight options menu for 
                                        type 2 plots */
   Panel_object labels2_opt;         /* Plot type 2 labels options menu */
   Panel_object colorcode_choice;    /* Colour coding choice for type 1 plots 
                                        */
   Panel_object spatial_choice;      /* Spatials selection for type 1 plots */
   Panel_object spot_type_choice;    /* Spot type choice for type 1 plots */
   Panel_object key_choice;          /* Display key choice for type 1 plots */
   Panel_object spotsize_choice;     /* Spot size choice for type 1 plots */
   Panel_object partial_sel;         /* Partial 'n' selection */
   Panel_object psbutton;            /* Postscript print button */
   Panel_object ps_io;               /* Postscript details i/o item */
   int          cset;                /* Colorset number */

   GC          paint_gc;             /* GC for paint window */
   Cursor      cursor;               /* Display area cursor */
   int         color_type;           /* Display colour type flag for 
                                        interactive
                                        option =1 writeable colour map; 
                                               =0 non-writeable colour map */
   int         type;                 /* Display type 1=colour, 2= monochrome
                                        with sliders etc. */
   int         half_size;            /* Display area half width/height */
   float       filmr;                /* Predicted pattern radius in mm */
   int         iord;                 /* Axis order on display wrt X-windows
                                        'x' and 'y' axes.
                                        1  +ax1 +ax2
                                        2  +ax1 -ax2
                                        3  -ax1 +ax2
                                        4  -ax1 -ax2
                                        5  +ax2 +ax1
                                        6  +ax2 -ax1
                                        7  -ax2 +ax1
                                        8  -ax2 -ax1 */
   char        ax1_name[5];          /* Name of first detector axis */
   char        ax2_name[5];          /* Name of second detector axis */
   float       fac ;                 /* Conversion factor mm offset to pixel 
                                        offset */
   int         mindx;                /* Index to reflections list */
   int         icrys;                /* Crystal number */
   int         imgnum;               /* Image number */

   unsigned char *colr;              /* color indices (in pixels array) array 
                                        when colour map can be modified. 
                                        0=do not not draw.
                                        or: 0=do not draw, 100=white, 
                                        101=black. Note the position of
                                        a reflection in the reflections list
                                        (numbered from 1 upwards) is used to
                                        index this array. there are 'nref+1'
                                        elements in the array and element 0
                                        is unused. (also 102=blue, 103=red,
                                        104=green)
                                        */
   XDl_rot_sim_varidx *varidx;       /* Variable value and index array for
                                        sorting data for interactive plots */

   int         cur_slider;           /* Currently active slider variable
                                        =1 phirange
                                        =2 eta
                                        =3 divv
                                        =4 divh
                                        =5 delamb                    */
   int         nspot;                /* No. of spots for current image */
   float       scax[3];              /* Rotation (scan) axis unit vector */
   float       rots;                 /* Rotation start for current image 
                                        (degrees) as used in the prediction */
   float       rote;                 /* Rotation end for current image (>ROTS) 
                                        (degrees) as used in the prediction */
   int         isyn;                 /* Flag =1 synchrotron, =0 lab source 
                                        as used in the prediction*/
   float       divh;                 /* Horizontal divergence (half width) 
                                        (radians) as used in the prediction */
   float       divv;                 /* Verical divergence (half width) 
                                        (radians) as used in the prediction */
   float       eta;                  /* Mosaicity (radians) as used in the
                                        prediction */
   float       delamb;               /* Delta-lambda/lambda as used in the
                                        prediction */    
   float       delcor;               /* Correlated dispersion (half width) as
                                        used in the prediction(R)*/
   int         iax_h;                /* No. of horizontal axis (lab frame) 
                                        1=X, 2=Y, 3=Z as used in the
                                        prediction */
   int         iax_v;                /* No. of vertical axis (lab frame) 
                                        1=X, 2=Y, 3=Z as used in the
                                        prediction */
   int         nwmax;                /* Maximum no. of images to treat as a 
                                        non too-wide partial as used in the 
                                        prediction */
   float       phirange;             /* Rotation range (degrees) */
   float       cur_phirange;         /* Current value of varaible rotation 
                                        range */
   float       cur_divh;             /* Current value of variable horizontal
                                        divergence */
   float       cur_divv;             /* Current value of variable vertical
                                        divergence */
   float       cur_eta;              /* Currernt value of variable eta */
   float       cur_var;              /* Current value of current slider */
   float       cur_delamb;           /* Current value of variable delamb */

   float       cur_phirange_pos;     /* Current slider position of variable 
                                        rotation range */
   float       cur_divh_pos;         /* Current slider position of variable 
                                        horizontal divergence */
   float       cur_divv_pos;         /* Current slider position of variable 
                                        vertical divergence */
   float       cur_eta_pos;          /* Current slider position of variable 
                                        eta */
   float       cur_delamb_pos;       /* Current slider position of variable 
                                        delamb */

   float       cell[6];              /* Real cell parameters for type 2
                                        Postscript plots annotation */
   float       umat[9];              /* 'U' matrix  for type 2 Postscript 
                                        plots annotation */
   float       delphi[3];            /* Missetting angles for type 2
                                        Postscript plots annotation */
   float       wave;                 /* Wavelength for type 2 Postscript
                                        plots annotation */
   int         isys;                 /* Crystal system (1-7) for type 2
                                        Postscript plots annotation */
   int         latt;                 /* Lattice type (1-7) for type 2
                                        Postscript plots annotation */
   float       ctod;                 /* Crystal to detector distance (mm)
                                        for type 2 Postscript plots
                                        annotation */

   unsigned long pixels[53];         /* Pixels for interactive display */
   unsigned long colrpix[8];         /* Pixels for type 1 colour plot 
                                        display */
   int         size_sym2;            /* Current size (1-3) for symbols for
                                        type 2 plots */
   int         opt_highlight2;       /* Highlight option for type 2 plots
                                        1=none, 2=fulls, 3=partials */
   int         opt_labels2;          /* Labels option for type 2 plots
                                        1=off, 
                                        2=on */
   int         partn_cur;            /* Current partial 'n' flag */
   int         disp_ref;             /* Number (>0) in reflections list of 
                                        reflection with indices currently 
                                        displayed; 0 if none */
   int         color_coding;         /* Colour coding for plot type 1
                                        =1 Fulls/partials
                                        =2 Partiality 1 
                                        =3 Partiality 2  
                                        =4 All the same
                                        =5 Overlap status */
   int         spatials;             /* Spatials flag for plot type 1
                                        =1 include
                                        =2 exclude
                                        =3 only */
   int         spot_type;            /* Spot type for plot type 1
                                        =1 All
                                        =2 Fulls
                                        =3 Partials */
   int         key_opt;              /* Key option for plot type 1
                                        =1 on
                                        =2 off */
   int         spotsize;             /* Spot size for plot type 1
                                        =1 small
                                        =2 medium
                                        =3 large */
   int         ps_on;                /* Postscript being processed currently
                                        =1 yes, =0 no */
   float       ps_r[8];              /* Colour red components for Postscript */
   float       ps_g[8];              /* Colour green components for PS */
   float       ps_b[8];              /* Colour blue components for PS */
   FILE        *psfil;               /* File pointer for Postscript file */
   float       ps_xc;                /* X centre for Postscript plot */
   float       ps_yc;                /* Y centre for Postscript plot */
   float       mm_to_p;              /* Conversion factor, mm to points */
   float       ps_scal;              /* Scale to apply to coordinates to scale 
                                        up to plot size in mm */
   float       ps_pixsiz;            /* Pixel size for PS plot (points)*/
   char        *ps_comment;          /* Pointer to comment for postscript file
                                        (last reply from panel i/o item) */
   float       plot_rad;             /* Plot radius in mm for PS plots */
   float       ps_spot_rad;          /* Spot radius in mm for PS plots */
   float       ps_symb_hs;           /* Symbol size in mm for PS plots */
   int         ps_color;             /* Flag =1 use coloured symbols
                                        on type 2 PS plot, =0 do not */
   int         ps_addkey;            /* Flag =1 output key on type 1
                                        PS plot, =0 do not */
   XDl_rot_sim_label *labels[MAX_LABELS]; /* Pointers to index labels 
                                              structures */
   int num_labels;                   /* Number of labels */
   int move_label;                   /* =1 label moving in progress, =0 not
                                        (for type 2 plots) */
   int cur_label;                    /* Offset into labels array for current
                                        label (i.e. new label or label being
                                        moved */
   Pixmap save_area;                 /* Save area for label moving */
   int area_saved;                   /* Area saved flag 1=yes, 0=no */
   int sav_x;                        /* x position of saved area */
   int sav_y;                        /* y position of saved area */
   int sav_w;                        /* width of saved area */
   int sav_h;                        /* height of saved area */
   Pixmap mark_area;                 /* Save area for marked reflection */
   int mark_saved;                   /* Mark area saved flag 1=yes, 0=no */
   int mark_x;                       /* x position of saved mark area */
   int mark_y;                       /* y position of saved mark area */
   int mark_w;                       /* width of saved mark area */
   int mark_h;                       /* height of save mark area */
}  
   XDl_rot_sim;

/* Define routine types */

void xdl_rot_sim_rp1();
void xdl_rot_sim_rp2();
void xdl_rot_sim_rp3();
void xdl_rot_sim_rp4();
void xdl_rot_sim_rp5();
void xdl_rot_sim_ep1();
void xdl_rot_sim_cp2();
void xdl_rot_sim_cp3();
void xdl_rot_sim_cp4();
void xdl_rot_sim_cp5();
void xdl_rot_sim_cp6();
void xdl_rot_sim_cp7();
void xdl_rot_sim_cp8();
void xdl_rot_sim_cp9();
void xdl_rot_sim_cp10();
void xdl_rot_sim_cp11();
void xdl_rot_sim_cp12();
void xdl_rot_sim_cp13();
void xdl_rot_sim_cp14();
void xdl_rot_sim_tidy_up();
void xdl_rot_sim_setcolors();
void xdl_rot_sim_redraw1();
void xdl_rot_sim_redraw2();
void xdl_rot_sim_symb1();
void xdl_rot_sim_drawkey();
void xdl_rot_sim_symb2();
void xdl_rot_sim_digit();
void xdl_rot_sim_label();
void xdl_rot_sim_mark();
int xdl_rot_sim_isalabel();
int xdl_rot_sim_relabel();
void xdl_rot_sim_labhkl();
void xdl_rot_sim_save_mark();
void xdl_rot_sim_restore_mark();
void xdl_rot_sim_save_area();
void xdl_rot_sim_restore_area();
void xdl_rot_sim_hsv_to_rgb();
void xdl_rot_sim_ps();
void xdl_rot_sim_pshead();
void xdl_rot_sim_psdraw1();
void xdl_rot_sim_psdraw2();
void xdl_rot_sim_pslabel();
void xdl_rot_sim_psdigit();
void xdl_rot_sim_pix();
void xdl_rot_sim_setlab4();
void xdl_rot_sim_varmin();
void xdl_rot_sim_rflags();
void xdl_rot_sim_vunit();
void xdl_rot_sim_vcross();
int xdl_rot_sim_compar();

/* Define Fortran binding names */

#if    LINKTYP == 1

#define xdlf_rot_sim xdlf_rot_sim_
#define xdlf_rot_sim_getsize xdlf_rot_sim_getsize_
#define xdlf_rot_sim_new xdlf_rot_sim_new_

#endif

#if  LINKTYP == 2

#define xdlf_rot_sim XDLF_ROT_SIM
#define xdlf_rot_sim_getsize XDLF_ROT_SIM_GETSIZE
#define xdl_rot_sim_new XDLF_ROT_SIM_NEW

#endif

void xdlf_rot_sim();
void xdlf_rot_sim_getsize();
void xdlf_rot_sim_new();

/*-C:*/
int xdl_rot_sim (int vh, int vh_parent, int x, int y, int cset, int type, 
                 int size, int iord, int mindx, int icrys, int imgnum, 
		 float filmr, float * scax, float rots, float rote, 
                 int isyn, float eta, float divv, float divh, float delamb, 
                 float delcor, int iax_v, int iax_h, int nwmax, 
                 float cur_rotr, float cur_eta, float cur_divv, 
                 float cur_divh, float cur_delamb, 
                 float * cell, float * umat, float * delphi, 
                 float wave, int * lsys, float ctod,
                 char * name1, int len1, char * name2, int len2, 
                 int min_width, int min_height)
/*end*/
 
/*-Parameters:
int       vh;              User selected view-object handle (R)
int       vh_parent;       View-object handle for the parent base frame, 
                           if 0 then a base  level frame is created to
                           enclose the view-object (R)
int       x;               x coordinate for the view-object.
                           If no parent may be -1 to give default x (R)
int       y;               y coordinate for the view-object.
                           If no parent may be -1 to give default y (R)
int       cset;            Number of the colorset to be used; must
                           have 53 or more colorcells allocated (R) 
int       type;            Simulation type =1 Colour, =2 Monochrome with
                           soft limit sliders etc.
int       size;            Pattern display area half width/height in
                           pixels (R)
int       iord;            Order of local detector axes flag. First axis
                           along X windows 'x' direction on screen (to
                           right), second axis along X-windows 'y' 
                           direction (down).
                             1  +ax1 +ax2
                             2  +ax1 -ax2
                             3  -ax1 +ax2
                             4  -ax1 -ax2
                             5  +ax2 +ax1
                             6  +ax2 -ax1
                             7  -ax2 +ax1
                             8  -ax2 -ax1
                             (R)
 int       mindx;          Reflection list index as returned from
                           prl_init (R)
int       icrys;           Crystal number (R)
int       imgnum;          Image number (R)
float     filmr;           Pattern radius in mm (R)
float     scax[3];         Rotation (scan) axis vector as used in the
                           prediction (R)
float     rots;            Rotation start for current image (degrees) as
                           used in the prediction (R)
float     rote;            Rotation end for current image (>ROTS) 
                           (degrees) as usedin the prediction (R)
int       isyn;            Flag =1 synchrotron, =0 lab source as used
                           in the prediction (R)
float     eta;             Mosaicity (half width, radians) as used in the 
                           prediction (R)
float     divv;            Verical divergence (half width) (radians) as
                           used in the prediction (R)
float     divh;            Horizontal divergence (half width) (radians)
                           as used in the prediction (R)
float     delamb;          Delta-lambda/lambda as used in the
                           prediction (R)    
float     delcor;          Correlated dispersion (half width) as used
                           in the prediction (R)
int       iax_v;           No. of vertical axis (lab frame) 
                           1=X, 2=Y, 3=Z as used in the prediction (R)
int       iax_h;           No. of horizontal axis (lab frame) 
                           1=X, 2=Y, 3=Z  as used in the prediction (R)
int       nwmax;           Maximum no. of images to treat as a non 
                           too-wide partial as used in the 
                           prediction (R) 
float     cur_rotr;        Current rotation range for type 2 plots (must
                           be <= predicted range) (R)
float     cur_eta;         Current 'eta' value for type 2 plots (must
                           be <= value used in the prediction) (R)
float     cur_divv;        Current 'divv' value for type 2 plots (must
                           be <= value used in the prediction) (R)
float     cur_divh;        Current 'divh' value for type 2 plots (must
                           be <= value used in the prediction) (R)
float     cur_delamb;      Current 'delamb' value for type 2 plots (must
                           be <= value used in the prediction) (R)
float     cell[6];         Real cell parameters in Angstroms and degrees
                           for annotation of type 2 Postscript plots 
                           (R)
float     umat[9];         'U' matrix for annotation of type 2 Postscript 
                           plots (R)
float     delphi[3];       Missetting angles in degrees for annotation of 
                           type 2 Postscript plots (R)
float     wave;            Wavelength in Angstroms for annotation of 
                           type 2 Postscript plots (R)
int       lsys[2];         Crystal system 1-7, lattice type 1-7 for
                           annotation of type 2 Postscript plots (R)
float     ctod;            Crystal to detector distance in mm for
                           annotation of type 2 Postscript plots (R)
char      *name1;          Name of 1'st detector axis (max 4 chars long) 
                           (R)
int       len1;            Length of name (may give 0 if a null terminated 
                           string is passed) (R)
char      *name2;          Name of 2'nd detector axis (max 4 chars long)
                           (R)
int       len2;            Length of name (may give 0 if a null terminated 
                           string is passed) (R)
int       min_width;       The program will calculate the width required
                           for the view-object based on the display area
                           width, control panel width etc. If this is
                           less than min_width then min_width will be
                           used instead and the layout altered to reflect
                           this (R)
int       min_height;      minimum height cf min_width (R)


-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: Requested parent not found in the view-objects list
         bit 1 set: Unable to allocate memory for x, y pixel positions
                    arrays.
-end*/
{
   Window wframe;              /* Top level window id*/
   Window wparent;             /* Parent window id */
   XDl_rot_sim * gd;           /* Pointer to view-object's global data 
                                  structure */
   int err;                    /* Error return flag */
   int iv;                     /* Offset into XDL_view_objects array */
   int i;                      /* Temp/loop variable */
   int j;                      /* Temp/loop variable */
   int l;                      /* Temp/loop variable */
   int n;                      /* Temp/loop variable */
   int ll;                     /* Length of string */
   int width;                  /* View-object width */
   int height;                 /* View-object height */
   int area_width;             /* Simulation display area width */
   int panel_1a_width;         /* Width for 'a' panel for type 1
                                  display (show reflection details) */
   int panel_1b_width;         /* Width for 'b' panel for type 1
                                  display */
   int panel_1_height;         /* Height of panels for type 1 display */
   int panel_2a_width;         /* Width for 'a' panel for type 2
                                  display (show reflection details) */
   int panel_2b_width;         /* Width for 'b' panel for type 2
                                  display (control area) */
   int panel_2_height;         /* Height of panels for type 2 display */
   int panel_ps_width;         /* Postscript control panel width */
   int panel_ps_height;        /* Postscript control panel height */
   int area_height;            /* Simulation display area height */
   int panels_width;           /* Control panels width needed */
   int panels_height;          /* Control panels height needed */
   int x_area;                 /* x position of display area within 
                                  the view-object*/
   int y_area;                 /* y position of display area within
                                  the view-object */
   int x_panel_1a;             /* x position of control panel 'a' for type 1 */
   int y_panel_1a;             /* y position of control panel 'a' for type 1 */
   int x_panel_2a;             /* x position of control panel 'a' for type 2 */
   int y_panel_2a;             /* y position of control panel 'a' for type 2 */
   int x_panel_1b;             /* x position of control panel 'b' for type 1 */
   int y_panel_1b;             /* y position of control panel 'b' for type 1 */
   int x_panel_2b;             /* x position of control panel 'b' for type 2 */
   int y_panel_2b;             /* y position of control panel 'b' for type 2 */
   int x_panel_ps;             /* x position of Postscript control panel */
   int y_panel_ps;             /* y position of Postscript control panel */
   int ytop;                   /* Top allowed y position for display area */
   float d;                    /* Temporary distance value */
   int font;                   /* Font type for control panel items */
   int panel_x;                /* x position for panel object */
   int panel_y;                /* y position for panel object */
   int slider_width;           /* slider width */
   int start_pos;              /* Slider start position */
   int edge;                   /* Edge offset for panel positioning */
   char str[20];               /* Temp string */
   int nref;                   /* No. of reflections in the reflections list */
   XColor cursor_fg;        /* Structure for setting cross cursor foreground */
   XColor cursor_bg;        /* Structure for setting cross cursor background */
   XColor colr_cell;        /* Structure for setting colour display colours */
   float hue, sat, val;     /* HSV (Hue/saturation/value) colour components */
   float r, g, b;           /* RGB (Red/Green/Blue) colour components */
   Pixmap cursor_source;    /* Pixmap for cursor source */
   Pixmap cursor_mask;      /* Pixmap for cursor mask */
   static unsigned char curs15_source_bits[] ={
      0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x33, 0x66, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static unsigned char curs15_mask_bits[] = {
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x00, 0x00, 0x3f, 0x7e, 0x00, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00,
      0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00};

   static char *var_strings[] = 
   {
      "Rot range",
      "Eta (deg.)",
      "Divv (deg.)",
      "Divh (deg.)",
      "Delamb (x1000)"
   };

   static char *sym2_strings[] = 
   {
      "Small",
      "Medium",
      "Large"
   };
   static char *highlight2_strings[] =
   {
      "None",
      "Fulls",
      "Partials"
   };
   static char *labels2_strings[] =
   {
      "Off",
      "On"
   };
   static char *color_coding_strings[] = 
   {
      "Fulls/partials",
      "Partiality 1",
      "Partiality 2",
      "All the same",
      "Overlap status"
   };
   static char *spot_type_strings[] =
   {
      "All spots",
      "Fulls",
      "Partials"
   };
   static char *spatials_strings[] =
   {
      "Included",
      "Excluded",
      "Spatials only"
   };
   static char *spotsize_strings[] =
   {
      "Small",
      "Medium",
      "Large"
   };
   static char *key_opt_strings[] =
   {
      "On",
      "Off"
   };
  static float hues[] = {0.0, 25.0, 60.0, 100.0, 160.0, 180.0, 200.0, 240.0};

   /* Initialisations */

   err = 0;
   font = 2;

   if (type<1||type>2) type =1;

   /* Get size requirements */

   area_width = 2*size + 1 + 2;
   area_height = 2*size + 1 + 2;
   switch (type)
   {
      case 1:
         panel_1a_width = PANEL_1A_WIDTH;
         panel_1b_width = PANEL_1B_WIDTH;
         panel_1_height = PANEL_1_HEIGHT;
         panels_width = panel_1a_width + 4 + panel_1b_width;
         panels_height = panel_1_height;
         break;
     
      case 2:
         panel_2a_width = PANEL_2A_WIDTH;
         panel_2b_width = PANEL_2B_WIDTH;
         panel_2_height = PANEL_2_HEIGHT;
         panels_width = panel_2a_width + 4 + panel_2b_width;
         panels_height = panel_2_height;
         break;
   }
   if (panels_width>area_width)
   {
      width = panels_width + 2*BORDER_WIDTH + 2;
   }
   else
   {
      width = area_width + 2*BORDER_WIDTH + 2;
   }
   height = area_height + SEP_H_MIN + panels_height 
            + 4 + PANEL_PS_HEIGHT
            + 2*BORDER_HEIGHT + 2;
   if (width<min_width) width = min_width;
   if (height<min_height) height = min_height;

   /* Get positions for panels */

   switch(type)
   {   
      case 1:
         x_panel_1a = (width - 2 - panels_width)/2;
         y_panel_1a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         edge = (width - 2 - panel_1a_width - panel_1b_width)/4;
         x_panel_1a = edge;
         y_panel_1a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         x_panel_1b = width - 2 - edge - panel_1b_width;
         y_panel_1b = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         break;
      case 2:
         edge = (width - 2 - panel_2a_width - panel_2b_width)/4;
         x_panel_2a = edge;
         y_panel_2a = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         x_panel_2b = width - 2 - edge - panel_2b_width;
         y_panel_2b = BORDER_HEIGHT + PANEL_PS_HEIGHT + 4;
         break;
   }
   x_area = (width - 2 - area_width)/2;
   x_panel_ps = BORDER_WIDTH;
   y_panel_ps = BORDER_HEIGHT;
   panel_ps_width = width - 2*BORDER_WIDTH - 2;
   panel_ps_height = PANEL_PS_HEIGHT;
   ytop = BORDER_HEIGHT + panels_height + 4 + PANEL_PS_HEIGHT + SEP_H_MIN;
   y_area = ytop + (height - 2 - area_height - ytop)/2;

   /* Create the top level frame (or base frame) window for the routine */

   /* Check that parent_vh present if requested */

   if (vh_parent>0)
   {
      iv = xdl_getiv (vh_parent);
      if (iv<0)
      /* Create frame window as parent window was given */
      {
         err |= 1;
         return err;
      }
      wparent = XDL_view_objects[iv]->wid;
      wframe = xdl_create_frame_window (wparent, x, y, width, height, cset);
   }else{
   /* Create base frame window as no parent frame window was specified */
      wframe = xdl_create_base_frame_window ("xdl_rot_sim", "xdl_view",
                          x, y, width, height);
   }

   /* Create structure for the routines' global data */

   gd = (XDl_rot_sim *) malloc (sizeof(XDl_rot_sim));

   /* Add new view-object to the list of view-objects */

   xdl_new_view_object (vh, wframe, vh_parent, (char *) gd, 2111, NULL,
                        xdl_rot_sim_tidy_up);

   /* Some initialisations for global data area */

   gd->wframe = wframe;
   gd->cset = cset;
   gd->type = type;
   gd->half_size = size;
   gd->filmr = filmr;
   gd->fac = ((float)gd->half_size)/gd->filmr;
   gd->mindx = mindx;
   gd->icrys = icrys;
   gd->imgnum = imgnum;
   gd->scax[0] = scax[0];
   gd->scax[1] = scax[1];
   gd->scax[2] = scax[2];
   xdl_rot_sim_vunit (gd->scax);
   gd->rots = rots;
   gd->rote = rote;
   gd->isyn = isyn;
   gd->divh = divh;
   gd->divv = divv;
   gd->eta = eta;
   gd->delamb = delamb;
   gd->delcor = delcor;
   gd->iax_h = iax_h;
   gd->iax_v = iax_v;
   gd->nwmax = nwmax;
   gd->phirange = rote - rots;
   gd->cur_phirange = gd->phirange;
   if (cur_rotr>=0.0&&cur_rotr<=gd->phirange) gd->cur_phirange = cur_rotr;
   gd->cur_divh = gd->divh;
   if (cur_divh>=0.0&&cur_divh<=gd->divh) gd->cur_divh = cur_divh;
   gd->cur_divv = gd->divv;
   if (cur_divv>=0.0&&cur_divv<=gd->divv) gd->cur_divv = cur_divv;
   gd->cur_eta = gd->eta;
   if (cur_eta>=0.0&&cur_eta<=gd->eta) gd->cur_eta = cur_eta;
   gd->cur_delamb = gd->delamb;
   if (cur_delamb>=0.0&&cur_delamb<=gd->delamb) gd->cur_delamb = cur_delamb;
   gd->cur_slider = 1;
   gd->cur_var = gd->cur_phirange;
   gd->cur_phirange_pos = 100;
   if (gd->phirange>0.0) gd->cur_phirange_pos 
                         = nint(100*gd->cur_phirange/gd->phirange);
   gd->cur_eta_pos = 100;
   if (gd->eta>0.0) gd->cur_eta_pos 
                         = nint(100*gd->cur_eta/gd->eta);
   gd->cur_divv_pos = 100;
   if (gd->divv>0.0) gd->cur_divv_pos 
                         = nint(100*gd->divv/gd->divv);
   gd->cur_divh_pos = 100;
   if (gd->divh>0.0) gd->cur_divh_pos 
                         = nint(100*gd->cur_divh/gd->divh);
   gd->cur_delamb_pos = 100;
   if (gd->delamb>0.0) gd->cur_delamb_pos 
                         = nint(100*gd->cur_delamb/gd->delamb);
   for(i=0;i<6;++i) gd->cell[i] = cell[i];
   for(i=0;i<9;++i) gd->umat[i] = umat[i];
   for(i=0;i<3;++i) gd->delphi[i] = delphi[i];
   gd->wave = wave;
   gd->isys = lsys[0];
   gd->latt = lsys[1];
   gd->ctod = ctod;
   gd->iord = iord;
   if (gd->iord<1||gd->iord>8) gd->iord = 1;
   ll = len1;
   if (ll==0) ll = strlen(name1);
   if (ll>4) ll=4;
   gd->ax1_name[0] = 0;
   if (ll>0)
   {
      n = 0;
      for (l=ll;l>=0;--l)
      {
         if (name1[l]!=' ') {n=l; break;}
      }
      if (n>0) strncat (gd->ax1_name,name1,n);
   }
   ll = len2;
   if (ll==0) ll = strlen(name2);
   if (ll>4) ll=4;
   gd->ax2_name[0] = 0;
   if (ll>0)
   {
      n = 0;
      for (l=ll;l>=0;--l)
      {
         if (name2[l]!=' ') {n=l; break;}
      }
      if (n>0) strncat (gd->ax2_name,name2,n);
   }
   gd->color_type = 0;
   if (XDL_color>0&&XDL_csets_alloc==1) gd->color_type = 1;
   gd->size_sym2 = 2;
   gd->opt_highlight2 = 1;
   gd->opt_labels2 = 1;
   gd->partn_cur = 2;
   gd->disp_ref = 0;
   gd->color_coding = 1;
   gd->spot_type = 1;
   gd->spatials = 1;
   gd->spotsize = 2;


   gd->key_opt = 1;
   gd->num_labels = 0;
   gd->move_label = 0;
   gd->cur_label = -1;
   gd->area_saved = 0;
   gd->mark_saved = 0;
   gd->ps_on = 0;
   gd->psfil = 0;
   gd->mm_to_p = 2.8346;

   /* Set colour pixels array for colour display */

   if (type==1)
   {
      if (XDL_color>0)
      {
         for (i=0;i<8;++i)
         {
            hue = hues[i];
            sat = 1.0;
            val = 1.0;
            xdl_rot_sim_hsv_to_rgb (hue, sat, val, &r, &g, &b);
            gd->ps_r[i] = r;
            gd->ps_g[i] = g;
            gd->ps_b[i] = b;
            colr_cell.red = 65535 * r;
            colr_cell.green = 65535 * g;
            colr_cell.blue = 65535 * b;
            colr_cell.flags = DoRed|DoGreen|DoBlue;
            if (XDL_csets_alloc==1)
            {
               colr_cell.pixel = XDL_colorsets[gd->cset].pixels[i];
               XStoreColor (XDL_display, XDL_colormap, &colr_cell);
            }
            else
            {
               XAllocColor (XDL_display, XDL_colormap, &colr_cell);
            }
            gd->colrpix[i] = colr_cell.pixel;
         }
      }
      else
      {
         for (i=0;i<8;++i) gd->colrpix[i] = XDL_whitepixel;
      }
   }

   /* Set pixels array if Interactive type with writeable colour map */

   if (type==2&&gd->color_type==1)
   {
     for (i=0;i<53;++i) gd->pixels[i]=XDL_colorsets[gd->cset].pixels[i];
   }

   /* Allocate arrays for colours and variable sorting */

   gd->colr = 0;
   nref = prl_numrefs (mindx);
   i = nref;
   if (i<1) i=1;
   gd->colr = (unsigned char *) malloc ((i+1)*sizeof(unsigned char));
   if (gd->colr==0)
   { 
      err |= 2;
      return err;
   }
   gd->varidx = (XDl_rot_sim_varidx *) malloc (i*sizeof(XDl_rot_sim_varidx));
   if (gd->varidx==0)
   {
      free (gd->colr);
      err |= 2;
      return err;
   }


   /* Set up type 2 colours if needed */
  
   if (type==2)
   {
      xdl_rot_sim_setcolors (gd, 1, 0);
   }

   /* Create paint window for rotation pattern simulation display area*/

   gd->wdisp = xdl_create_paint_window (wframe, x_area, y_area, 1, 
                                         area_width, area_height, cset);

   xdl_add_window (vh, gd->wdisp, xdl_rot_sim_rp1, xdl_rot_sim_ep1);

   /* Prepare for event handling */

   switch (type)
   {
      case 1:
         XSelectInput (XDL_display, gd->wdisp, 
                      ExposureMask|StructureNotifyMask|
                      ButtonPressMask);
         break;
      case 2:
         XSelectInput (XDL_display, gd->wdisp, 
                      ExposureMask|StructureNotifyMask|
                      ButtonPressMask|ButtonReleaseMask|PointerMotionMask);
         break;
   }

   /* Create and initialise GC */

   gd->paint_gc = XCreateGC (XDL_display, gd->wdisp, 0, NULL);

   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XSetFont (XDL_display, gd->paint_gc, XDL_fonts[1]->fid);

   /* Create display area cursor */

   cursor_source = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                        (char *) curs15_source_bits, 15, 15);

   cursor_mask = XCreateBitmapFromData (XDL_display, gd->wdisp,
                                        (char *) curs15_mask_bits, 15, 15);

   cursor_fg.red = 0;
   cursor_fg.green = 0;
   cursor_fg.blue = 0;
   cursor_fg.flags = DoRed|DoGreen|DoBlue;

   cursor_bg.red = 65535;
   cursor_bg.green = 65535;
   cursor_bg.blue = 65535;
   cursor_bg.flags = DoRed|DoGreen|DoBlue;
   
   gd->cursor = XCreatePixmapCursor (XDL_display, cursor_source, cursor_mask,
                                     &cursor_fg, &cursor_bg, 7, 7);

   XFreePixmap (XDL_display, cursor_source);
   XFreePixmap (XDL_display, cursor_mask);

   XDefineCursor (XDL_display, gd->wdisp, gd->cursor);

   /* Create Pixmap save areas (for label moving etc.) */

   if (type==2)
   {
      gd->save_area = XCreatePixmap (XDL_display, gd->wdisp, SAV_SIZ, SAV_SIZ,
                      XDL_pw_depth);
   }
   gd->mark_area = XCreatePixmap (XDL_display, gd->wdisp, MARK_SIZ, MARK_SIZ,
                   XDL_pw_depth);

   /* Create Panel paint windows */
   /*----------------------------*/

   /* Postscript options control panel */

   gd->wpanel_ps = xdl_create_paint_window (wframe, x_panel_ps, y_panel_ps,
                           1, panel_ps_width, panel_ps_height, cset);
   xdl_add_window (vh, gd->wpanel_ps, xdl_rot_sim_rp5, NULL);

   XSelectInput (XDL_display, gd->wpanel_ps, ExposureMask|
                 StructureNotifyMask|ButtonPressMask);

   panel_x = 4;
   panel_y = 2;
   gd->psbutton = xdl_panel_button (vh, gd->wpanel_ps, panel_x, panel_y,
                         "PS", 2, 2, 2, 0, 0, 1, 1, 1, xdl_rot_sim_cp13);

   panel_x = 32;
   panel_y = 3;
   gd->ps_io = xdl_panel_io (vh, gd->wpanel_ps, panel_x, panel_y,
                            40, 20, 200, 2, 0, 1, 
                            panel_ps_width-panel_x-2, 0, 
                            xdl_rot_sim_cp14);


   /* Other control panels */

   switch(type)
   {
      case 1:
         /* Set up reflection details panel area */

         gd->wrefln = xdl_create_paint_window (wframe, x_panel_1a, y_panel_1a,
                                1, panel_1a_width, panel_1_height, cset);
         xdl_add_window (vh, gd->wrefln, xdl_rot_sim_rp3, NULL);

         XSelectInput (XDL_display, gd->wrefln, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         panel_x = 2;
         panel_y = 4;
         gd->reflab[0] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         "Reflection Details", 25, font, 0, 0, 0, 1);
         panel_y += 15;
         gd->reflab[1] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[2] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[3] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[4] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);

         /* Control panel */

         gd->wpanel = xdl_create_paint_window (wframe, x_panel_1b, y_panel_1b,
                                 1, panel_1b_width, panel_1_height, cset);
         xdl_add_window (vh, gd->wpanel, xdl_rot_sim_rp2, NULL);

         /* Prepare for event handling */

         XSelectInput (XDL_display, gd->wpanel, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         /* Control panel items */

         /* Colour coding choices */
         panel_x = 4;
         panel_y = 4;
         gd->colorcode_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Colour coding", 5, color_coding_strings, 
                          gd->color_coding, font, 
                          0, 0, 0, xdl_rot_sim_cp6);

         /* Spot type choices */
         panel_x = 4;
         panel_y = 24;
         gd->spot_type_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spot types   ", 3, spot_type_strings, 
                          gd->spot_type, font, 
                          0, 0, 0, xdl_rot_sim_cp7);

         /* Spatials choices */
         panel_x = 4;
         panel_y = 44;
         gd->spatial_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spatials     ", 3, spatials_strings, 
                          gd->spatials, font, 
                          0, 0, 0, xdl_rot_sim_cp8);

         /* Spot size choices */
         panel_x = 230;
         panel_y = 4;
         gd->spotsize_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Spot size", 3, spotsize_strings, 
                          gd->spotsize, font, 
                          0, 0, 0, xdl_rot_sim_cp9);

         /* Key choices */
         panel_x = 230;
         panel_y = 24;
         gd->key_choice = xdl_panel_choice (vh, gd->wpanel, 
                         panel_x, panel_y,
                         "Key      ", 2, key_opt_strings, 
                          gd->key_opt, font, 
                          0, 0, 0, xdl_rot_sim_cp10);
         /* Partial 'n'  value */
         panel_x = 230;
         panel_y = 44;
         gd->partial_sel = xdl_panel_value (vh, gd->wpanel,
                        panel_x, panel_y, "Partial 'n'", "2", 2, 2,
                        0, 0, 0, xdl_rot_sim_cp12);
         break;

      case 2:
         /* Set up reflection details panel area */

         gd->wrefln = xdl_create_paint_window (wframe, x_panel_2a, y_panel_2a,
                                1, panel_2a_width, panel_2_height, cset);
         xdl_add_window (vh, gd->wrefln, xdl_rot_sim_rp3, NULL);

         XSelectInput (XDL_display, gd->wrefln, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         panel_x = 2;
         panel_y = 4;
         gd->reflab[0] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         "Reflection Details", 25, font, 0, 0, 0, 1);
         panel_y += 15;
         gd->reflab[1] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[2] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[3] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);
         panel_y += 12;
         gd->reflab[4] = xdl_panel_label (vh, gd->wrefln, panel_x, panel_y,
                         " ", 25, font, 0, 0, 0, 0);

         /* Set up controls panel area */

         gd->wpanel = xdl_create_paint_window (wframe, x_panel_2b, y_panel_2b,
                                  1, panel_2b_width, panel_2_height, cset);
         xdl_add_window (vh, gd->wpanel, xdl_rot_sim_rp4, NULL);

         XSelectInput (XDL_display, gd->wpanel, ExposureMask|
                       StructureNotifyMask|ButtonPressMask);

         /* Set up Control Panel Items */

         /* Variable slider & label */
         panel_x = 4;
         panel_y = 44;
         slider_width = 100;
         start_pos = nint(100*gd->cur_phirange/gd->phirange);
         gd->var_slider = xdl_panel_slider (vh, gd->wpanel, panel_x, panel_y,
                           "", font, slider_width, start_pos, 
                           0, 0, 0, xdl_rot_sim_cp3);
         panel_x += 135;
         sprintf (str,"%.3f",gd->cur_phirange);
         gd->var_curval = xdl_panel_label (vh, gd->wpanel, panel_x, panel_y,
                          str, 6, font, 0, 0, 0, 0);

         /* Slider variable type choices */
         panel_x = 4;
         panel_y = 24;
         gd->slide_type = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                         "Slider variable", 5, var_strings, 
                          gd->cur_slider, font, 
                          0, 0, 0, xdl_rot_sim_cp2);

         /* Symbol size choices */
         panel_x = 240;
         panel_y = 4;
         gd->sym2_size = xdl_panel_choice (vh, gd->wpanel, panel_x, panel_y,
                         "Symbols", 3, sym2_strings, gd->size_sym2, font, 
                          0, 0, 0, xdl_rot_sim_cp4);
         /* Highlight options choices */
         panel_x = 240;
         panel_y = 24;
         gd->highlight2_opt = xdl_panel_choice (vh, gd->wpanel, 
                          panel_x, panel_y,
                         "Highlight", 3, highlight2_strings, 
                          gd->opt_highlight2, font, 0, 0, 0,
                          xdl_rot_sim_cp5);
         /* Labels options choices */
         panel_x = 240;
         panel_y = 44;
         gd->labels2_opt = xdl_panel_choice (vh, gd->wpanel, 
                          panel_x, panel_y,
                         "Labels", 2, labels2_strings, 
                          gd->opt_labels2, font, 0, 0, 0,
                          xdl_rot_sim_cp11);
         /* Partial 'n' value */
         panel_x = 4;
         panel_y = 4;
         gd->partial_sel = xdl_panel_value (vh, gd->wpanel,
                        panel_x, panel_y, "Partial 'n'", "2", 2, 2,
                        0, 0, 0, xdl_rot_sim_cp12);
         break;
   }

   return 0;
}

/* Fortran binding: xdlf_rot_sim */

void xdlf_rot_sim (ivh, ivhparent, ix, iy, icset, itype, isize, iord, mindx,
                   icrys, imgnum, filmr, scax, rots, rote, isyn, 
                   eta, divv, divh,
                   delamb, delcor, iax_v, iax_h, nwmax, cur_rotr,
                   cur_eta, cur_divv, cur_divh, cur_delamb, cell,
                   umat, delphi, wave, lsys, ctod, 
                   name1, len1, name2, len2, minw, minh, ierr)

int *ivh, *ivhparent, *ix, *iy, *icset, *itype, *isize, *iord, *mindx;
int *icrys, *imgnum, *isyn, *iax_h, *iax_v, *nwmax, lsys[2];
int *name1, *len1, *name2, *len2, *minw, *minh, *ierr;
float *filmr, *rots, *rote, *divh, *divv, *eta, *delamb, *delcor, *scax;
float *cur_rotr, *cur_eta, *cur_divv, *cur_divh, *cur_delamb;
float *cell, umat[9], *delphi, *wave, *ctod;

{
   float um[3][3];
   float * umm;
   int i,j,k;

   k = 0;
   for (i=0;i<3;++i)
   {
      for (j=0;j<3;++j)
      {
         um[j][i] = umat[k];
         k++;
      }
   }
   umm = (float *) um;
   *ierr = xdl_rot_sim (*ivh, *ivhparent, *ix, *iy, *icset, *itype, 
                        *isize, *iord, *mindx, *icrys, *imgnum, *filmr, scax,
                        *rots, *rote, *isyn, *eta, *divv, *divh,
                        *delamb, *delcor, *iax_v, *iax_h, *nwmax,
                        *cur_rotr, *cur_eta, *cur_divv, *cur_divh, 
                        *cur_delamb, cell, umm, delphi, *wave, lsys, *ctod,
                        XDL_pointers[*name1], *len1, 
                        XDL_pointers[*name2], *len2,
                        *minw, *minh);
   return;
}

/*==================================================*/
/*         'Private' SERVICE ROUTINES               */
/*==================================================*/

/*
*********************************
**     xdl_rot_sim_rp1         **
*********************************

Purpose: Repaint display area paint window 

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rp1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_rot_sim * gd;          /* Pointer to global data area */
   int i;                     /* Temp/loop variable */

   gd = (XDl_rot_sim *) global;


   /* Repaint code */

   switch (gd->type)
   {
      case 1:
         xdl_rot_sim_redraw1 (gd);
         break;

      case 2:
         xdl_rot_sim_redraw2 (gd);
         break;
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_rp2         **
*********************************

Purpose: Repaint control panel area for type 1 display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rp2 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_rot_sim_rp3         **
*********************************

Purpose: Repaint reflection details panel area for type 1 or 2 display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rp3 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_rot_sim_rp4         **
*********************************

Purpose: Repaint control panel area for type 2 display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rp4 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_rot_sim_rp5         **
*********************************

Purpose: Repaint postscript control panel area

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rp5 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   /* Repaint code */

   XSetWindowBackground (XDL_display, wid, XDL_whitepixel);
   XClearWindow (XDL_display, wid);  
   return;
}

/*
*********************************
**     xdl_rot_sim_ep1         **
*********************************

Purpose: Event handling from display area paint window

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_ep1 (wid, event, iv, global, callback)

Window wid;
XEvent *event;
int iv;
char * global;
void (*callback)();

{
   XDl_rot_sim *gd;       /* Pointer to the global data area */
   int i, k;              /* Temp/loop variables */
   int x, y;              /* Mouse x, y position */
   int xd, yd;            /* x, y offsets from current spot */
   int dsq;               /* Distance squared from mouse position to
                             current spot */
   int dsqmin;            /* Current minimum value of dsq */
   int ii;                /* Index in list of reflections of selected 
                             reflection */
   char str[51];          /* Temp. string for labels */
   int llen;              /* Label length */
   int wl;                /* Width of labels including border */
   int xx, yy;            /* Temp x, y offsets */
   int xmin, ymin;        /* Min x, y values required to save labelling area */
   int xmax, ymax;        /* Max x, y values required to save labelling area */
   int xspot, yspot;      /* Spot position in pixels */
   XDl_rot_sim_label *lt;/* Temp pointer to label structure */
   XEvent xevent;         /* X event structure */
   int hlab, klab, llab;  /* Indices for label */
   int max_xy;            /* maximum x, y pixel position within window */
   int hh, kk, ll;        /* Unpacked indices */
   int ks;                /* Symmetry flag */
   int nref;              /* Total no. of reflections in reflections list */
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */

   gd = (XDl_rot_sim *) global;

  switch (gd->type)
  {
     case 1:
        switch (event->type)
        {
            case ButtonPress:
               if (event->xbutton.button==Button1)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                  dsqmin = 10000;
                  ii = 0;
                  nref = prl_numrefs (gd->mindx);
                  if (nref>0)
		  {
                     for (i=0;i<nref;++i)
                     {
                        prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                          &hh, &kk, &ll, &ks ,&xf, &yf, &nimg,
                                          &ifirst, &ir_flag, &inr_flag, &spat);
                        if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
                        if (ir_flag==0) continue;
                        if (gd->spatials!=1)
                        {
                           if (gd->spatials==2&&spat) continue;
                           if (gd->spatials==3&&spat==0) continue;
                        }
                        xdl_rot_sim_pix (gd, i+1, &xspot, &yspot);
                        xd = x - xspot;
                        yd = y - yspot;
                        dsq = xd*xd + yd*yd;
                        if (dsq<dsqmin)
                        {
                           ii = i+1;
                           dsqmin = dsq;
                        }
                     }
                     if (ii>0&&dsqmin<=18)
                     {
                        gd->disp_ref = ii;
                        prl_getref(gd->mindx, ii, &icrys, &imgnum,
                                        &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                        &ifirst, &ir_flag, &inr_flag, &spat);
                        sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                        xdl_panel_label_set(gd->reflab[1],str,0,0);
                        sprintf (str,"%s, %s: %.2f, %.2f",gd->ax1_name,
                                 gd->ax2_name, xf, yf);
                        xdl_panel_label_set(gd->reflab[2],str,0,0);
                        if (gd->type==2) xdl_rot_sim_rflags (gd, ii, &ir_flag,
                                                             &nimg, &ifirst);
                        xdl_rot_sim_setlab4 (ir_flag, nimg, ifirst, str);
                        xdl_panel_label_set(gd->reflab[3],str,0,0);
                        if (gd->type==1)
			{
                           strcpy (str,"non-overlapped spot");
                           if (spat) strcpy (str,"overlapped spot");
                           xdl_panel_label_set(gd->reflab[4],str,0,0);
			}
                        xdl_rot_sim_save_mark (gd, ii);
                        xdl_rot_sim_mark (gd, ii, 1);
                     }
                     else
                     {
                        gd->disp_ref = 0;
                        for (k=0;k<4;++k) 
                        xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                     }
		  }
               }
               else if (event->xbutton.button==Button2 ||
                        event->xbutton.button==Button3)
               {
                  if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                  if (gd->disp_ref>=0)
                  {
                     gd->disp_ref = 0;
                     for (k=0;k<4;++k) 
                     xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  }
               }
               break;
        }
        break;

     case 2:
        switch (event->type)
        {
            case ButtonPress:
               if (event->xbutton.button==Button1)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (xdl_rot_sim_isalabel (gd, x, y, 0)) break;
                  if (gd->area_saved) xdl_rot_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                  dsqmin = 10000;
                  ii = 0;
                  nref = prl_numrefs (gd->mindx);
                  if (nref>0)
		  {
                     for (i=0;i<nref;++i)
                     {
                        prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                          &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                          &ifirst, &ir_flag, &inr_flag, &spat);
                        if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
                        xdl_rot_sim_rflags (gd, i+1, &ir_flag, &nimg, &ifirst);
                        if (ir_flag==0) continue;
                        xdl_rot_sim_pix (gd, i+1, &xspot, &yspot);
                        xd = x - xspot;
                        yd = y - yspot;
                        dsq = xd*xd + yd*yd;
                        if (dsq<dsqmin)
                        {
                           ii = i+1;
                           dsqmin = dsq;
                        }
                     }
                     if (ii>0&&dsqmin<=18)
                     {
                        gd->disp_ref = ii;
                        prl_getref(gd->mindx, ii, &icrys, &imgnum,
                                       &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                       &ifirst, &ir_flag, &inr_flag, &spat);
                        sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                        xdl_panel_label_set(gd->reflab[1],str,0,0);
                        sprintf (str,"%s, %s: %.2f, %.2f",gd->ax1_name,
                                 gd->ax2_name, xf, yf);
                        xdl_panel_label_set(gd->reflab[2],str,0,0);
                        xdl_rot_sim_rflags (gd, ii, &ir_flag, &nimg, &ifirst);
                        xdl_rot_sim_setlab4 (ir_flag, nimg, ifirst, str);
                        xdl_panel_label_set(gd->reflab[3],str,0,0);
                        if (gd->type==1)
			{
                           strcpy (str,"non-overlapped spot");
                           if (spat) strcpy (str,"overlapped spot");
                           xdl_panel_label_set(gd->reflab[4],str,0,0);
			}
                        xdl_rot_sim_save_mark (gd, ii);
                        xdl_rot_sim_mark (gd, ii, 1);
                     }
                     else
                     {
                        gd->disp_ref = 0;
                        for (k=0;k<4;++k) 
                        xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                     }
		  }
               }
               else if (event->xbutton.button==Button2)
               {
                  if (gd->opt_labels2==1) break;
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->area_saved) xdl_rot_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                  if (xdl_rot_sim_isalabel (gd, x, y, 1))
                  {
                     lt = gd->labels[gd->cur_label];
                     xx = lt->x2 - lt->x1;
                     yy = lt->y2 - lt->y1;
                     lt->x1 = x - lt->xcoff;
                     lt->y1 = y - lt->ycoff;
                     lt->x2 = lt->x1 + xx;
                     lt->y2 = lt->y1 + yy;
                     xdl_rot_sim_pix (gd, lt->idx, &xspot, &yspot);
                     if (lt->x1<xspot) { xmin=lt->x1; }else{ xmin=xspot; }
                     if (lt->y1<yspot) { ymin=lt->y1; }else{ ymin=yspot; }
                     if (lt->x2>xspot) { xmax=lt->x2; }else{ xmax=xspot; }
                     if (lt->y2>yspot) { ymax=lt->y2; }else{ ymax=yspot; }
                     if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
                     {
                        xdl_rot_sim_save_area (gd, xmin, ymin,
                            (xmax-xmin+1), (ymax-ymin+1));
                        xdl_rot_sim_label (gd, gd->cur_label, 1);
                     }
                     else
                     {
                        llen = strlen(lt->hklstr);
                        wl = 7*llen+4-6;
                        xdl_rot_sim_save_area (gd, lt->x1, lt->y1, wl, 12);
                        xdl_rot_sim_save_mark (gd, lt->idx);
                        xdl_rot_sim_mark (gd, lt->idx, 2);
                        xdl_rot_sim_label (gd, gd->cur_label, 0);
                     }
                     gd->move_label = 1;
                  }
                  else
                  {
                     dsqmin = 10000;
                     ii = 0;
                     nref = prl_numrefs (gd->mindx);
                     if (nref>0)
		     {
                        for (i=0;i<nref;++i)
                        {
                           prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                          &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                          &ifirst, &ir_flag, &inr_flag, &spat);
                           if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
                           xdl_rot_sim_rflags (gd, i+1, &ir_flag, &nimg, 
                                               &ifirst);
                           if (ir_flag==0) continue;
                           xdl_rot_sim_pix (gd, i+1, &xspot, &yspot);
                           xd = x - xspot;
                           yd = y - yspot;
                           dsq = xd*xd + yd*yd;
                           if (dsq<dsqmin)
                           {
                              ii = i+1;
                              dsqmin = dsq;
                           }
                        }
                        if (dsqmin<=18)
                        {
                           gd->disp_ref = ii;
                           prl_getref(gd->mindx, ii, &icrys, &imgnum,
                                          &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                          &ifirst, &ir_flag, &inr_flag, &spat);
                           sprintf (str,"h k l:  %d %d %d", hh, kk, ll);
                           xdl_panel_label_set(gd->reflab[1],str,0,0);
                           sprintf (str,"%s, %s: %.2f, %.2f",gd->ax1_name,
                                    gd->ax2_name, xf, yf);
                           xdl_panel_label_set(gd->reflab[2],str,0,0);
                           xdl_rot_sim_rflags (gd, ii, 
                                              &ir_flag, &nimg, &ifirst);
                           xdl_rot_sim_setlab4 (ir_flag, nimg, ifirst, str);
                           xdl_panel_label_set(gd->reflab[3],str,0,0);
                           if (gd->type==1)
			   {
                              strcpy (str,"non-overlapped spot");
                              if (spat) strcpy (str,"overlapped spot");
                              xdl_panel_label_set(gd->reflab[4],str,0,0);
			   }
                           if (gd->num_labels<MAX_LABELS)
                           {
                              lt= (XDl_rot_sim_label *) 
                                  malloc(sizeof(XDl_rot_sim_label));
                              if (lt!=0)
                              {
                                 gd->labels[gd->num_labels] = lt;
                                 gd->num_labels += 1;
                                 lt->idx = ii;
                                 xdl_rot_sim_pix (gd, ii, &xspot, &yspot);
                                 xdl_rot_sim_labhkl (gd, ii, &hlab, &klab,
                                                      &llab);
                                 sprintf (lt->hklstr,"%d %d %d",
                                         hlab, klab, llab);
                                 lt->h = hlab;
                                 lt->k = klab;
                                 lt->l = llab;
                                 lt->disp = 1;
                                 llen = strlen(lt->hklstr);
                                 wl = 7*llen+4-6;
                                 lt->xcoff = wl/2;
                                 lt->ycoff = 5;
                                 lt->x1 = xspot - lt->xcoff;
                                 lt->y1 = yspot - lt->ycoff;
                                 lt->x2 = lt->x1 + wl - 1;
                                 lt->y2 = lt->y1 + 11;
                                 gd->cur_label = gd->num_labels - 1;
                                 if (lt->x1<xspot) 
                                    { xmin=lt->x1; }else{ xmin=xspot; }
                                 if (lt->y1<yspot) 
                                    { ymin=lt->y1; }else{ ymin=yspot; }
                                 if (lt->x2>xspot) 
                                    { xmax=lt->x2; }else{ xmax=xspot; }
                                 if (lt->y2>yspot) 
                                    { ymax=lt->y2; }else{ ymax=yspot; }
                                 if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
                                 {
                                    xdl_rot_sim_save_area (gd, xmin, ymin,
                                        (xmax-xmin+1), (ymax-ymin+1));
                                    xdl_rot_sim_label (gd, gd->cur_label, 1);
                                 }
                                 else
                                 {
                                    xdl_rot_sim_save_area (gd, xmin, ymin,
                                         wl, 12);
                                    xdl_rot_sim_save_mark (gd, ii);
                                    xdl_rot_sim_mark (gd, ii, 2);
                                    xdl_rot_sim_label (gd, gd->cur_label, 0);
                                 }
                                 gd->move_label = 1;
                              }
                           }
                        }
                        else
                        {
                           gd->disp_ref = 0;
                           for (k=0;k<4;++k) 
                           xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
			}
                     }
                  }
               }
               else if (event->xbutton.button==Button3)
               {
                  x = event->xbutton.x;
                  y = event->xbutton.y;
                  if (gd->area_saved) xdl_rot_sim_restore_area (gd);
                  if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                  gd->disp_ref = 0;
                  for (k=0;k<4;++k) 
                  xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
                  if (xdl_rot_sim_isalabel (gd, x, y, 1))
                  {
                     gd->num_labels -= 1;
                     gd->move_label = 0;
                  }
               }
               break;
            case ButtonRelease:
               if (event->xbutton.button==Button2)
               {
                  if (gd->move_label)
                  {
                     gd->move_label = 0;
                     if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
                     if (gd->area_saved) xdl_rot_sim_restore_area (gd);
                     xdl_rot_sim_label (gd, gd->cur_label, 1);
                     if (gd->disp_ref>0)
                     {
                        xdl_rot_sim_save_mark (gd, gd->disp_ref);
                        xdl_rot_sim_mark (gd, gd->disp_ref, 1);
                     }
                  }
                  gd->move_label = 0;
                  gd->cur_label = -1;
               }
               break;
            case MotionNotify:
               x = event->xmotion.x;
               y = event->xmotion.y;
               while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
                      MotionNotify, &xevent))
               {
                  x = xevent.xmotion.x;
                  y = xevent.xmotion.y;
               }
               max_xy = 2*gd->half_size;
               if (x<0) x = 0;
               if (x>max_xy) x = max_xy;
               if (y<0) y = 0;
               if (y>max_xy) y = max_xy;
               if (gd->move_label==0) break;
               if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
               if (gd->area_saved) xdl_rot_sim_restore_area (gd);
               lt = gd->labels[gd->cur_label];
               xx = lt->x2 - lt->x1;
               yy = lt->y2 - lt->y1;
               wl = xx + 1;
               lt->x1 = x - lt->xcoff;
               lt->y1 = y - lt->ycoff;
               lt->x2 = lt->x1 + xx;
               lt->y2 = lt->y1 + yy;
               xdl_rot_sim_pix (gd, lt->idx, &xspot, &yspot);
               if (lt->x1<xspot) { xmin=lt->x1; }else{ xmin=xspot; }
               if (lt->y1<yspot) { ymin=lt->y1; }else{ ymin=yspot; }
               if (lt->x2>xspot) { xmax=lt->x2; }else{ xmax=xspot; }
               if (lt->y2>yspot) { ymax=lt->y2; }else{ ymax=yspot; }
               if ((xmax-xmin)<SAV_SIZ&&(ymax-ymin)<SAV_SIZ)
               {
                  xdl_rot_sim_save_area (gd, xmin, ymin,
                      (xmax-xmin+1), (ymax-ymin+1));
                  xdl_rot_sim_label (gd, gd->cur_label, 1);
               }
               else
               {
                  xdl_rot_sim_save_area (gd, lt->x1, lt->y1, wl, 12);
                  xdl_rot_sim_save_mark (gd, lt->idx);
                  xdl_rot_sim_mark (gd, lt->idx, 2);
                  xdl_rot_sim_label (gd, gd->cur_label, 0);
               }
               break;
         }
         break;
   }

   return;
}

/*
*********************************
**     xdl_rot_sim_cp2         **
*********************************

Purpose: Callback for control panel slider type menu (plot type 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp2 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;              /* Pointer to global data area */
   char str[20];                 /* Temp character string */
   int islide;                   /* Slider position */
   static float rtod = 57.29578; /* Convert radians to degrees */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->cur_slider)
   {
      gd->cur_slider = item;
      switch (gd->cur_slider)
      {
         case 1:
         gd->cur_var = gd->cur_phirange;
         islide = gd->cur_phirange_pos;
         sprintf (str,"%.3f",gd->cur_phirange);
         break;

         case 2:
         gd->cur_var = gd->cur_eta;
         islide = gd->cur_eta_pos;
         sprintf (str,"%.4f",gd->cur_eta*2.0*rtod);
         break;

         case 3:
         gd->cur_var = gd->cur_divv;
         islide = gd->cur_divv_pos;
         sprintf (str,"%.4f",gd->cur_divv*2.0*rtod);
         break;

         case 4:
         gd->cur_var = gd->cur_divh;
         islide = gd->cur_divh_pos;
         sprintf (str,"%.4f",gd->cur_divh*2.0*rtod);
         break;

         case 5:
         gd->cur_var = gd->cur_delamb;
         islide = gd->cur_delamb_pos;
         sprintf (str,"%.4f",gd->cur_delamb*1000.0);
         break;
      }
      xdl_panel_slider_reset (gd->var_slider, islide);
      xdl_panel_label_set (gd->var_curval, str, 0, 0);
      xdl_rot_sim_setcolors (gd, 1, 1);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp3         **
*********************************

Purpose: Callback for control panel variable slider

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp3 (iv, value, slider_len)

int iv;
int value;
int slider_len;

{
   XDl_rot_sim *gd;              /* Pointer to global data area */
   int recalc;                   /* Flag =1 recalculate colours etc. if
                                    this slider was not the last one accessed
                                    =0 reset the colour map only (if writeable
                                    colourmap available */
   char str[20];                 /* Label string */
   int k;                        /* Temp/loop variable */
   static float rtod = 57.29578; /* Convert radians to degrees */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;

   recalc = 0;
   switch (gd->cur_slider)
   {
      case 1:
      gd->cur_var = gd->phirange*(value/2)/50.0;
      gd->cur_phirange = gd->cur_var;
      gd->cur_phirange_pos = value;
      sprintf (str,"%.3f",gd->cur_phirange);
      break;

      case 2:
      gd->cur_var = gd->eta*(value/2)/50.0;
      gd->cur_eta = gd->cur_var;
      gd->cur_eta_pos = value;
      sprintf (str,"%.4f",gd->cur_eta*2.0*rtod);
      break;

      case 3:
      gd->cur_var = gd->divv*(value/2)/50.0;
      gd->cur_divv = gd->cur_var;
      gd->cur_divv_pos = value;
      sprintf (str,"%.4f",gd->cur_divv*2.0*rtod);
      break;

      case 4:
      gd->cur_var = gd->divh*(value/2)/50.0;
      gd->cur_divh = gd->cur_var;
      gd->cur_divh_pos = value;
      sprintf (str,"%.4f",gd->cur_divh*2.0*rtod);
      break;

      case 5:
      gd->cur_var = gd->delamb*(value/2)/50.0;
      gd->cur_delamb = gd->cur_var;
      gd->cur_delamb_pos = value;
      sprintf (str,"%.4f",gd->cur_delamb*1000.0);
      break;
   }
   xdl_panel_label_set (gd->var_curval, str, 0, 0);
   if (gd->mark_saved) xdl_rot_sim_restore_mark (gd);
   if (gd->disp_ref>0)
   {
      gd->disp_ref = 0;
      for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
   }
   if (xdl_rot_sim_relabel(gd)) recalc = 1;
   xdl_rot_sim_setcolors (gd, recalc, 1);
   return;
}
/*
*********************************
**     xdl_rot_sim_cp4         **
*********************************

Purpose: Callback for control panel symbol size choice menu (plot type 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp4 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->size_sym2)
   {
      gd->size_sym2 = item;
      xdl_rot_sim_redraw2 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp5         **
*********************************

Purpose: Callback for control panel highlight option choice menu (plot type 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp5 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->opt_highlight2)
   {
      gd->opt_highlight2 = item;
      xdl_rot_sim_setcolors (gd, 1, 1);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp6         **
*********************************

Purpose: Callback for colour coding type choice for type 1 plots

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp6 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->color_coding)
   {
      gd->color_coding = item;
      xdl_rot_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp7         **
*********************************

Purpose: Callback for spot types choice for type 1 plots

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp7 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spot_type)
   {
      if (gd->disp_ref>0)
      {
         gd->disp_ref = 0;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->spot_type = item;
      xdl_rot_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp8         **
*********************************

Purpose: Callback for spatials choice for plot type 1

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp8 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;      /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spatials)
   {
      if (gd->disp_ref>0)
      {
         gd->disp_ref = 0;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->spatials = item;
      xdl_rot_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp9         **
*********************************

Purpose: Callback for spot size choice for plot type 1

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp9 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->spotsize)
   {
      gd->spotsize = item;
      xdl_rot_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp10        **
*********************************

Purpose: Callback for key choice for plot type 1

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp10 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;      /* Pointer to global data area */
   int k;                /* Temp/loop variable */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->key_opt)
   {
      if (gd->disp_ref>0)
      {
         gd->disp_ref = 0;
         for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);
      }
      gd->key_opt = item;
      xdl_rot_sim_redraw1 (gd);
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp11        **
*********************************

Purpose: Callback for labels option choice for plot type 2

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp11 (iv, item)

int iv;
int item;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;
   if (item!=gd->opt_labels2)
   {
      gd->opt_labels2 = item;
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp12        **
*********************************

Purpose: Callback for Partial 'n' value (types 1 or 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp12 (iv, valstr, len, max_len)

int iv;
char * valstr;
int len;
int max_len;

{
   XDl_rot_sim *gd;      /* Pointer to global data area */
   int i;                /* Temp/loop variable */
   int np;               /* New partial 'n' value */
   char str[5];          /* Temporary string */
   int ll;               /* String length */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;

   i = sscanf (valstr,"%d",&np);
   if (i<1) np = gd->partn_cur;
   if (np<2) np = 2;
   if (np>gd->nwmax) np = gd->nwmax;
   sprintf (str, "%d", np);
   ll = strlen (str);
   xdl_panel_value_set (gd->partial_sel, str, ll);

   if (np!=gd->partn_cur)
   {
      gd->partn_cur = np;
      switch (gd->type)
      {
         case 1:
            if (gd->color_coding==2) xdl_rot_sim_redraw1 (gd);
            break;
         case 2:
            if (gd->opt_highlight2==3) xdl_rot_sim_redraw2 (gd);
            break;
      }
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_cp13        **
*********************************

Purpose: Callback for postscript print button (types 1 or 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp13 (iv, button)

int iv;
int button;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;

   gd->ps_on = 1;
   xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
   return;
}
/*
*********************************
**     xdl_rot_sim_cp14        **
*********************************

Purpose: Callback for postscript i/o item (types 1 or 2)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_cp14 (iv, reply, len, max_rp, flag, iret)

int iv;
char *reply;
int len;
int max_rp;
int flag;
int iret;

{
   XDl_rot_sim *gd;     /* Pointer to global data area */

   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;

   xdl_rot_sim_ps (gd, reply, len, flag, iret);

   return;
}
/*
************************************
**     xdl_rot_sim_tidy_up        **
************************************

Purpose: Routine called to tidy up before view-object is deleted

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_tidy_up (global)

char * global;    /* Pointer to global data area */

{
   XDl_rot_sim *gd;     /* Pointer to the global data area */
   int i;               /* Temp/loop variable */

   gd = (XDl_rot_sim *) global;

   XFreeGC (XDL_display, gd->paint_gc);
   if (gd->colr!=NULL) free ((char *)gd->colr);
   if (gd->varidx!=NULL) free ((char *)gd->varidx);
   if (gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i)
      {
         if (gd->labels[i]!=NULL) free ((char *)gd->labels[i]);
      }
   }
   if (gd->type==2) XFreePixmap (XDL_display, gd->save_area);
   XFreePixmap (XDL_display, gd->mark_area);
   if (gd->ps_on>0&&gd->psfil!=0) fclose (gd->psfil);
   return;
}
/*
***********************************
**     xdl_rot_sim_setcolors     **
***********************************

Purpose: Set color indices array and colour map for interactive display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_setcolors (gd, recalc, draw)

XDl_rot_sim *gd;     /* Pointer to the global data area */
int recalc;          /* flag =1 recalculate pixel values,
                             =0 reset colour map only (if writeable
                                colour map; else treat as case 1) */
int draw;            /* flag =1 redraw display if needed
                             =0 do not redraw display */
{
   int i;                 /* Temp/loop variable */
   int k;                 /* Bin number */
   int ithr;              /* Threshold for current soft limit */
   int black_pix;         /* Black pixel flag 1=black, 0=white */
   float range;           /* Range for variablebeing considered */
   float drange;          /* Step in range - a little bit */
   XColor wh_colr_cell;   /* Xcolor structure for white pixel */
   XColor bl_colr_cell;   /* Xcolor structure for black pixel */
   int hh, kk, ll;        /* Unpacked indices */
   int ks;                /* Symmetry flag */
   int nref;              /* Total no. of reflections in reflections list */
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */
   float var_min;         /* Minimum value of current variable which allows
                             appearance of spot */
   int irf;               /* Flag for spot recorded within range of current 
                             variable 1=yes, 0=no */
   int jr_flag;           /* Reflection recorded flag */
   int jimg;              /* No. of images spot is spread over */
   int jfirst;            /* First image of spot (offset) */

   gd->pixels[0] = XDL_whitepixel;

   switch (gd->cur_slider)
   {
      case 1:
      range = gd->phirange;
      break;

      case 2:
      range = gd->eta;
      break;

      case 3:
      range = gd->divv;
      break;

      case 4:
      range = gd->divh;
      break;

      case 5:
      range = gd->delamb;
      break;
   }
   drange = 0.999*(range/50.0);
   ithr = (int) (50.0*(gd->cur_var+drange)/range) + 2;

   /* Special case for DIVH if delcor is non-zero */

   if (gd->cur_slider==4&&gd->delcor!=0.0) 
   {
      recalc = 1;
      nref = prl_numrefs (gd->mindx);
      gd->nspot = 0;
      if (nref>0)
      {
         for (i=0;i<nref;++i)
         {
            gd->colr[i+1] = 0;
            prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                               &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                               &ifirst, &ir_flag, &inr_flag, &spat);
            if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
            if (ir_flag==0) continue;
            gd->varidx[gd->nspot].idx = i+1;
            gd->varidx[gd->nspot].var = 0.0;
            gd->nspot += 1;
            xdl_rot_sim_rflags (gd, i+1, &jr_flag, &jimg, &jfirst);
            if (jr_flag!=0) gd->colr[i+1] = 2; 
	 }
      }
   }
   else
   {
      /* Normal case */

      if (recalc)
      {
         nref = prl_numrefs (gd->mindx);
         gd->nspot = 0;
         if (nref>0)
         {
            for (i=0;i<nref;++i)
            {
               gd->colr[i+1] = 0;
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                  &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                  &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               xdl_rot_sim_varmin (gd, gd->cur_slider, i+1, &var_min, &irf);
               if (irf==0) continue;
               gd->varidx[gd->nspot].idx = i+1;
               gd->varidx[gd->nspot].var = var_min;
               gd->nspot += 1;
               k = (int) 50.0*(var_min+drange)/range + 2;
               if (k<2) k = 2;
               if (k>52) k = 52;
               gd->colr[i+1]=k;
            }
	 }
      }
      /* Sort data in descending order */

      if (gd->nspot>1) qsort (gd->varidx, gd->nspot, 
                              sizeof (XDl_rot_sim_varidx),
                              xdl_rot_sim_compar);
   }


   /* Set pixel values for modifying colour map */

   if (gd->color_type==1&&gd->opt_highlight2==1)
   {
      wh_colr_cell.pixel = XDL_whitepixel;
      XQueryColor (XDL_display, XDL_colormap, &wh_colr_cell);
      bl_colr_cell.pixel = XDL_blackpixel;
      XQueryColor (XDL_display, XDL_colormap, &bl_colr_cell);
      for (i=1;i<=52;++i)
      {
         black_pix = 0;
         if (i<=ithr) black_pix = 1;

         if (black_pix)
         {
            bl_colr_cell.pixel = gd->pixels[i];
            XStoreColor (XDL_display, XDL_colormap, &bl_colr_cell);
         }
         else
         {
            wh_colr_cell.pixel = gd->pixels[i];
            XStoreColor (XDL_display, XDL_colormap, &wh_colr_cell);
         }
      }
   }

   if ((recalc||gd->opt_highlight2!=1)&&draw==1) xdl_rot_sim_redraw2 (gd);

   return;
}
/*
************************************
**     xdl_rot_sim_redraw1        **
************************************

Purpose: Routine to redraw colour coded option display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_redraw1 (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */

{
   int i;                 /* Temp/loop variables */
   int x, y;              /* Spot position */
   int colr;              /* Colour (/code) for xdl_rot_sim_symb1 */
   int hh, kk, ll;        /* Unpacked indices */
   int ks;                /* Symmetry flag */
   int nref;              /* Total no. of reflections in reflections list */
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */
   XEvent xevent;         /* X Event structure */

   while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
          Expose, &xevent));

   XClearWindow (XDL_display, gd->wdisp);
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc, 0, 0,
              2*(gd->half_size)+1, 2*(gd->half_size)+1);

   gd->mark_saved = 0;

   switch (gd->color_coding)
   {
      case 1: /* Fulls/partials */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xdl_rot_sim_pix (gd, i+1, &x, &y);
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
		  case 3:
                  colr = 2;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               xdl_rot_sim_symb1 (gd, x, y, colr, gd->spotsize);
	    }
            if (gd->key_opt==1) xdl_rot_sim_drawkey (gd);
    	 }
         break;

      case 2: /* Partiality 1: Fulls, partials<=n, partials>n, wide,  cusp */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xdl_rot_sim_pix (gd, i+1, &x, &y);
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
                  colr = 4;
                  if (nimg>gd->partn_cur) colr = 2;
                  break;

		  case 3:
                  colr = 1;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               xdl_rot_sim_symb1 (gd, x, y, colr, gd->spotsize);
	    }
            if (gd->key_opt==1) xdl_rot_sim_drawkey (gd);
    	 }
         break;

      case 3: /* Partiality 2: Fulls, p2, p3, p4, p>4, wide, cusp */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xdl_rot_sim_pix (gd, i+1, &x, &y);
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
                  colr = 6-nimg;
                  if (colr<2) colr = 2;
                  break;

		  case 3:
                  colr = 1;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               xdl_rot_sim_symb1 (gd, x, y, colr, gd->spotsize);
	    }
            if (gd->key_opt==1) xdl_rot_sim_drawkey (gd);
    	 }
         break;

      case 4: /* All the same */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xdl_rot_sim_pix (gd, i+1, &x, &y);
               colr = 5;
               xdl_rot_sim_symb1 (gd, x, y, colr, gd->spotsize);
	    }
            if (gd->key_opt==1) xdl_rot_sim_drawkey (gd);
    	 }
         break;

      case 5: /* Overlap status */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xdl_rot_sim_pix (gd, i+1, &x, &y);
               switch (spat)
	       {
	          case 0:
                  colr = 6;
                  break;
    
		  case 1:
                  colr = 2;
                  break;
	       }
               xdl_rot_sim_symb1 (gd, x, y, colr, gd->spotsize);
	    }
            if (gd->key_opt==1) xdl_rot_sim_drawkey (gd);
    	 }
         break;

   }
   /* Draw mark for last 'listed' reflection */

   if (gd->disp_ref>0)
   {
      xdl_rot_sim_save_mark (gd, gd->disp_ref);
      xdl_rot_sim_mark (gd, gd->disp_ref, 1);
   }

   return;
}
/*
************************************
**     xdl_rot_sim_redraw2        **
************************************

Purpose: Routine to redraw interactive option display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_redraw2 (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */

{
   int i, j;              /* Temp/loop variables */
   int k;                 /* index to current reflection */
   int styp;              /* Symbol type (see xdl_rot_sim_symb2) */
   int colr;              /* Colour flag for xdl_rot_sim_symb2 */
   int hh, kk, ll;        /* Unpacked indices */
   int ks;                /* Symmetry flag */
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */
   int xpix;              /* 'x' pixel position */
   int ypix;              /* 'x' pixel position */
   int reftyp;            /* reflection type flag =0full, 
                             1=partial<=gd->partn_cur,
                             2=partial>gd->partn_cur, 3=cusp */
   XEvent xevent;         /* X Event structure */

   while (XCheckTypedWindowEvent (XDL_display, gd->wdisp,
          Expose, &xevent));

   XClearWindow (XDL_display, gd->wdisp);

   gd->area_saved = 0;
   gd->mark_saved = 0;
  if (gd->nspot>0)
  {
     for (i=0;i<gd->nspot;++i)
     {
        k = gd->varidx[i].idx;
        prl_getref(gd->mindx, k, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
        if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
        if (ir_flag==0) continue;
        xdl_rot_sim_pix (gd, k, &xpix, &ypix);
        if (gd->colr[k]==0) continue;
         styp = gd->size_sym2;
         colr = (int) gd->colr[k];
         switch (gd->opt_highlight2)
         {
            case 1:
               if (colr==100) continue;
               break;
            case 2:
               if (colr==100) continue;
               xdl_rot_sim_rflags (gd, k, &ir_flag, &nimg, &ifirst);
               if (ir_flag==0) continue;
               if (XDL_color==2)
               {
                  colr = 101;
                  if (ir_flag==1){colr = 102; styp = 3 + gd->size_sym2;}
               }
               else
               {
                  colr = 101;
                  if (ir_flag==1) styp = 3 + gd->size_sym2;
               }
               break;
            case 3:
               if (colr==100) continue;
               xdl_rot_sim_rflags (gd, k, &ir_flag, &nimg, &ifirst);
               reftyp = 0;
               if (ir_flag==0) continue;
               if (ir_flag==2)
	       {
                  reftyp = 1;
                  if (nimg>gd->partn_cur) reftyp= 2;
	       }
               if (ir_flag==3) reftyp = 2;
               if (ir_flag==4) reftyp = 3;
               if (XDL_color==2)
               {
                  colr = 101;
                  if (reftyp==1){colr = 102; styp = 3 + gd->size_sym2;}
                  if (reftyp==2){colr = 103; styp = 6 + gd->size_sym2;}
                  if (reftyp==3){colr = 104; styp = 9 + gd->size_sym2;}
               }
               else
               {
                  colr = 101;
                  if (reftyp==1) styp = 3 + gd->size_sym2;
                  if (reftyp==2) styp = 6 + gd->size_sym2;
                  if (reftyp==3) styp = 9 + gd->size_sym2;
               }
               break;
         }
         xdl_rot_sim_symb2 (gd, xpix, ypix, colr, styp);
      }
   }

   /* Draw labels if required and if present */

   if (gd->opt_labels2>1&&gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i) xdl_rot_sim_label (gd, i, 1);
   }

   /* Draw mark for last 'listed' reflection */

   if (gd->disp_ref>0)
   {
      xdl_rot_sim_save_mark (gd, gd->disp_ref);
      xdl_rot_sim_mark (gd, gd->disp_ref, 1);
   }

   return;
}

/*
**********************************
**    xdl_rot_sim_symb1         **
**********************************

Purpose: Draw symbol for type 1 (colour) simulation display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_symb1 (gd,xpix,ypix,colr,symtyp)

XDl_rot_sim *gd;      /* Pointer to global data area */
int xpix;              /* Symbol x position */
int ypix;              /* Symbol y position */
int colr;              /* Index in colrpix (<100) or 100=white */
int symtyp;            /* Symbol type 1 = small disc (point)
                                      2 = medium disc
                                      3 = large disc
                       */
                            

{
   if (colr<100)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
   }
   else
   {
      switch (colr)
      {
         case 100:
            XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
            break;
      }
   }
   switch(symtyp)
   {
      case 1: /* Small 'disc' (Point) */
         XDrawPoint (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix);
         break;
      case 2: /* Medium 'disc' */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-1, ypix-1, 3, 3, 0, 23040);
         break;
      case 3: /* Large 'disc' */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-2, ypix-2, 5, 5, 0, 23040);
         break;
   }
   return;
}
/*
**********************************
**    xdl_rot_sim_drawkey       **
**********************************

Purpose: Draw keys for type 1 (colour) plots

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_drawkey (gd)

XDl_rot_sim *gd;      /* Pointer to global data area */

{
   int i;              /* Temp/loop variable */
   int xspot;          /* x coordinate for coloured spot */
   int yspot;          /* y coordinate for coloured spot */
   int xstring;        /* x coordinate for string */
   int ystring;        /* y coordinate for string */
   int len;            /* String length */
   char str[30];       /* Temp string for text output */
   int colr;           /* Flag for colour  */

   XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XDrawString (XDL_display, gd->wdisp, gd->paint_gc,
                    4, 10, "Key", 3);

   switch (gd->color_coding)
   {
      case 1: /* Colour coded: Fulls/Partials/Cusp */
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 70, 50);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 69, 49);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<3;++i)
        {
           switch (i)
	   {
	      case 0:
              colr = 6;
              break;
    
	      case 1:
              colr = 2;
              break;

	      case 2:
              colr = 0;
              break;
	   }
           XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           switch (i)
           {
              case 0:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "full", 4);
                 break;
              case 1:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "partial", 7);
                 break;
              case 2:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "cusp", 4);
                 break;
           }
           yspot += 15;
           ystring += 15;
        }
        break;

      case 2: /* Partiality 1: fulls, partials<=n, partials>n, too-wide, cusp*/
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 106, 80);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 105, 79);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<5;++i)
        {
           switch (i)
	   {
	      case 0:
              colr = 6;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "full", 4);
              break;
    
	      case 1:
              colr = 4;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "partial <=n", 11);
              break;

	      case 2:
              colr = 2;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                       xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "partial >n", 10);
              break;

	      case 3:
              colr = 1;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "wide partial", 12);
              break;

	      case 4:
              colr = 0;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "cusp partial", 12);
              break;
	   }
           yspot += 15;
           ystring += 15;
	}
        break;        

      case 3: /* Partiality 2: fulls, part 2, part 3, part>3, too-wide, cusp*/
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 106, 94);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 105, 93);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<6;++i)
        {
           switch (i)
	   {
	      case 0:
              colr = 6;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "full", 4);
              break;
    
	      case 1:
              colr = 4;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "partial =2", 10);
              break;

	      case 2:
              colr = 3;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "partial =3", 10);
              break;

	      case 3:
              colr = 2;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                       xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "partial >3", 10);
              break;

	      case 4:
              colr = 1;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "wide partial", 12);
              break;

	      case 5:
              colr = 0;
              XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
              XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                        xspot-4, yspot-4, 9, 9, 0, 23040);
              XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
              XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                           ystring, "cusp partial", 12);
              break;
	   }
           yspot += 15;
           ystring += 15;
	}
        break;

      case 4: /* All the same */
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 115, 19);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 114, 18);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28;
        colr = 5; 
        XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
        XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xspot-4, yspot-4, 9, 9, 0, 23040);
        XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
        XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                     ystring, "recorded spot", 13);
        break;
        
      case 5: /* Overlap status */
        XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        4, 15, 120, 33);
        XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
        XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc,
                        5, 16, 119, 32);
        xspot = 12;
        yspot = 23;
        xstring = 22;
        ystring = 28; 
        for(i=0;i<2;++i)
        {
           switch (i)
	   {
	      case 0:
              colr = 6;
              break;
    
	      case 1:
              colr = 2;
              break;
	   }
           XSetForeground (XDL_display, gd->paint_gc, gd->colrpix[colr]);
           XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                     xspot-4, yspot-4, 9, 9, 0, 23040);
           XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
           switch (i)
           {
              case 0:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "not-overlapped", 14);
                 break;
              case 1:
                 XDrawString (XDL_display, gd->wdisp, gd->paint_gc, xstring,
                              ystring, "overlapped", 10);
                 break;
           }
           yspot += 15;
           ystring += 15;
        }
        break;
   }
   return;
}
/*
**********************************
**    xdl_rot_sim_symb2         **
**********************************

Purpose: Draw symbol for type 2 simulation display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_symb2 (gd,xpix,ypix,colr,symtyp)

XDl_rot_sim *gd;      /* Pointer to global data area */
int xpix;              /* Symbol x position */
int ypix;              /* Symbol y position */
int colr;              /* Index in pixels array for symbol colour (<100)
                          or 101 = black
                             102 = blue
                             103 = red 
                             104 = green  */
int symtyp;            /* Symbol type 1 = small disc (point)
                                      2 = medium disc
                                      3 = large disc
                                      4 = small vertical cross
                                      5 = medium vertical cross
                                      6 = large vertical cross
                                      7 = small diagonal cross
                                      8 = medium diagonal cross
                                      9 = large diagonal cross
                                      10 = small star
                                      11 = medium star
                                      12 = large star
                       */
                            

{
   if (colr<100)
   {
      XSetForeground (XDL_display, gd->paint_gc, gd->pixels[colr]);
   }
   else
   {
      switch (colr)
      {
         case 101:
            XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
            break;
         case 102:
            XSetForeground (XDL_display, gd->paint_gc, XDL_bluepixel);
            break;
         case 103:
            XSetForeground (XDL_display, gd->paint_gc, XDL_redpixel);
            break;
         case 104:
            XSetForeground (XDL_display, gd->paint_gc, XDL_greenpixel);
            break;
       }
   }
   switch(symtyp)
   {
      case 1: /* Small disc (Point) */
         XDrawPoint (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix);
         break;

      case 2: /* Medium disc */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-1, ypix-1, 3, 3, 0, 23040);
         break;

      case 3: /* Large disc */
         XFillArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-2, ypix-2, 5, 5, 0, 23040);
         break;

      case 4: /* Small verticalcross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix,
                    xpix+2, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-2,
                    xpix, ypix+2);
         break;
      case 5: /* Medium vertical cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix,
                    xpix+3, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-3,
                    xpix, ypix+3);
         break;
      case 6: /* Large vertical cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix,
                    xpix+4, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-4,
                    xpix, ypix+4);
         break;
      case 7: /* Small  diagonal cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix-2,
                    xpix+2, ypix+2);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix+2, ypix-2,
                    xpix-2, ypix+2);
         break;
      case 8: /* Medium diagonal cross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix-3,
                    xpix+3, ypix+3);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix+3, ypix-3,
                    xpix-3, ypix+3);
         break;
      case 9: /* Large  diagonalcross */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix-4,
                    xpix+4, ypix+4);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix+4, ypix-4,
                    xpix-4, ypix+4);
         break;
      case 10: /* Small star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix,
                    xpix+2, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-2,
                    xpix, ypix+2);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix-2,
                    xpix+2, ypix+2);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-2, ypix+2,
                    xpix+2, ypix-2);
         break;
      case 11: /* Medium star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix,
                    xpix+3, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-3,
                    xpix, ypix+3);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix-3,
                    xpix+3, ypix+3);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-3, ypix+3,
                    xpix+3, ypix-3);
         break;
      case 12: /* Large star */
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix,
                    xpix+4, ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix, ypix-4,
                    xpix, ypix+4);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix-4,
                    xpix+4, ypix+4);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xpix-4, ypix+4,
                    xpix+4, ypix-4);
         break;
   }
   return;
}
/*
********************************
**     xdl_rot_sim_digit      **
********************************

Purpose: Draw digit on display area

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_digit (gd, digit, xch, ych)

XDl_rot_sim *gd;    /* Pointer to global data area */
int digit;           /* Digit number 0 to 9 */
int xch;             /* x position (bottom left corner of digit) */
int ych;             /* y position (bottom left corner of digit) */

{
   static short digits[10][16][2]={
   {{1,2},{1,7},{2,8},{4,8},{5,7},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{3,8},{3,1},{-1,1},{5,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{1,2},{1,1},
    {5,1},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},{2,5},{-4,5},
    {5,4},{5,2},{4,1},{2,1},{1,2},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,3},{5,3},{-4,1},{4,5},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,8},{1,5},{4,5},{5,4},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,2},{2,1},{4,1},{5,2},{5,3},{4,4},
    {2,4},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,8},{5,8},{5,6},{1,2},{1,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{2,5},{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},
    {2,5},{1,4},{1,2},{2,1},{4,1},{5,2},{5,4},{4,5}},

   {{1,1},{5,5},{5,7},{4,8},{2,8},{1,7},{1,6},{2,5},
    {4,5},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
   };
   int i;                   /* Temp/loop variable */
   int dx1, dy1, dx2, dy2;  /* Pixel point offsets for lines within character */
   int x1, y1, x2, y2;      /* Coordinates for ends of lines */

   for(i=0;i<16;++i)
   {
      if (digits[digit][i][0]==0) break;
      if (i==0||digits[digit][i][0]<0)
      {
         dx1 = digits[digit][i][0]; if (dx1<0) dx1 = -dx1;
         dy1 = digits[digit][i][1]; if (dy1<0) dy1 = -dy1;
         continue;
      }
      dx2 = digits[digit][i][0];
      dy2 = digits[digit][i][1];
      x1 = xch + dx1 - 1;
      y1 = ych - dy1 + 1;
      x2 = xch + dx2 - 1;
      y2 = ych - dy2 + 1;
      XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, x1, y1, x2, y2);
      dx1 = dx2;
      dy1 = dy2;
   }
     
   return;
}
/*
********************************
**     xdl_rot_sim_label      **
********************************

Purpose: Draw an indices label

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_label (gd, loff, iatt)

XDl_rot_sim *gd;     /* Pointer to global data area */
int loff;            /* Offset in labels list */
int iatt;            /* =1 draw attachment line, =0 do not */
{
   XDl_rot_sim_label *lt;   /* Pointer to current labels structure */
   int l;                    /* Temp/loop variable */
   int ll;                   /* Length of character string */
   int digit;                /* Current digit number 0-9 */
   int xch, ych;             /* Current character position */
   int xc, yc;               /* Box corner for attached line */
   int xspot, yspot;         /* Spot position */

   lt = gd->labels[loff];
   if (lt->disp==0) return;
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   XDrawRectangle (XDL_display, gd->wdisp, gd->paint_gc, lt->x1, lt->y1,
                  (lt->x2-lt->x1), (lt->y2-lt->y1));
   XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   XFillRectangle (XDL_display, gd->wdisp, gd->paint_gc, lt->x1+1, lt->y1+1,
                  (lt->x2-lt->x1-1), (lt->y2-lt->y1-1));
   XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
   ll = strlen (lt->hklstr);
   if (ll<=0) return;
   xch = lt->x1 + 2;
   ych = lt->y2 - 2;
   for (l=0;l<ll;++l)
   {
      if (lt->hklstr[l]=='-')
      {
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xch, ych-4,
                   xch+4, ych-4);
         xch += 7;
      }
      else if (lt->hklstr[l]>='0'&&lt->hklstr[l]<='9')
      {
         digit = lt->hklstr[l] - '0';
         xdl_rot_sim_digit (gd, digit, xch, ych);
         xch += 7;
      }
      else
      {
         xch += 4;
      }
   }
   /* Draw attaching line to spot */

   if (iatt==1)
   {
      xdl_rot_sim_pix (gd, lt->idx, &xspot, &yspot);
      if (xspot>=lt->x1&&xspot<=lt->x2&&yspot>=lt->y1&&yspot<=lt->y2) return;
      xc = lt->x2;
      if (abs(xspot-lt->x1)<abs(xspot-lt->x2)) xc = lt->x1;
      yc = lt->y2;
      if (abs(yspot-lt->y1)<abs(yspot-lt->y2)) yc = lt->y1;
      XDrawLine (XDL_display, gd->wdisp, gd->paint_gc, xc, yc, xspot, yspot);
   }
   return;
}
/*
******************************
**     xdl_rot_sim_mark     **
******************************

Purpose: Mark spot just indexed (or temp mark for labelling)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_mark (gd, idx, type)

XDl_rot_sim *gd;     /* Pointer to global data area */
int idx;             /* Index for spot to be marked */
int type;            /* =1, mark for indexed spot
                        =2, temp. mark for labelling */
{
   int xpix;
   int ypix;

   xdl_rot_sim_pix (gd, idx, &xpix, &ypix);
   if (gd->type==1)
   {
      XSetForeground (XDL_display, gd->paint_gc, XDL_whitepixel);
   }
   else
   {
      if (XDL_color==2&&type==1)
      {
         XSetForeground (XDL_display, gd->paint_gc, XDL_redpixel);
      }
      else
      {
         XSetForeground (XDL_display, gd->paint_gc, XDL_blackpixel);
      }
   }
   
   switch (type)
   {
      case 1:
         XDrawArc (XDL_display, gd->wdisp, gd->paint_gc, 
                  xpix-8, ypix-8, 17, 17, 0, 23040);
         break;
      case 2:
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
                   (xpix-9), ypix,
                   (xpix+9), ypix);
         XDrawLine (XDL_display, gd->wdisp, gd->paint_gc,
                    xpix, (ypix-9),
                    xpix, (ypix+9));
         break;
   }
   return;
}
/*
**********************************
**     xdl_rot_sim_isalabel     **
**********************************

Purpose: See if mouse x, y is within a displayed label; if so take
         appropriate action.

Author:  John W. Campbell, June 1995

*/

int xdl_rot_sim_isalabel (gd, x, y, iopt)

XDl_rot_sim *gd;    /* Pointer to global data area */
int x;               /* Mouse x position */
int y;               /* Mouse y position */
int iopt;            /* =1, if label found, shift to end of labels list and
                            redraw plot without last label.
                        =0, do not */
{
   XDl_rot_sim_label *lt;       /* Temporary pointer to labels structure */
   int i;                        /* Temp/loop variable */
   int ii;                       /* Index to selected label (-1=none) */
   int sav_disp_ref;             /* Temp copy of gd->disp_ref */

   if (gd->opt_labels2==1) return 0;
   if (gd->num_labels==0) return 0;
   ii = -1;
   for (i=0;i<gd->num_labels;++i)
   {
      lt = gd->labels[i];
      if (lt->disp==0) continue;
      if (x>=lt->x1&&x<=lt->x2&&y>=lt->y1&&y<=lt->y2) ii = i;
   }
   if (ii<0) return 0;

   if (iopt==0) return 1;

   /* Shift selected label to the end of the labels list */

   if (ii<gd->num_labels-1)
   {
      lt = gd->labels[ii];
      for (i=ii;i<gd->num_labels-1;++i) gd->labels[i] = gd->labels[i+1];
      gd->labels[gd->num_labels-1] = lt;
   }
   
   /* Redraw plot without the last label (temp. reset gd->num_labels)
      and without mark if label attached to the marked reflection  */

   gd->num_labels -= 1;
   sav_disp_ref = gd->disp_ref;
   if (gd->disp_ref==lt->idx) gd->disp_ref = 0;
   xdl_rot_sim_redraw2 (gd);
   gd->disp_ref = sav_disp_ref;
   gd->num_labels += 1;
   
   gd->cur_label = gd->num_labels - 1;
   return 1;
}
/*
**********************************
**     xdl_rot_sim_relabel      **
**********************************

Purpose: See if labelling requirements change when sliders moved

Author:  John W. Campbell, June 1995

*/

int xdl_rot_sim_relabel (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */
{
   XDl_rot_sim_label *lt;       /* Temporary pointer to labels structure */
   int change;                   /* Display status changed flag 1=yes, 0=no */
   int display;                  /* Display current label 1=yes, 0=no */
   int i, j;                     /* Temp/loop variables */
   int ii;                       /* Offset in reflections array */
   int ir_flag;                  /* Spot recorded flag */
   int nimg;                     /* No. of images */
   int ifirst;                   /* Offset of first image */

   if (gd->num_labels==0) return 0;
   if (gd->opt_labels2==1) return 0;

   change = 0;
   for (i=0;i<gd->num_labels;++i)
   {
      lt = gd->labels[i];
      ii = lt->idx;
      display = 0;
      xdl_rot_sim_rflags (gd, ii, &ir_flag, &nimg, &ifirst);
      if (ir_flag>0) display = 1;
      if (lt->disp!=display)
      {
         change = 1;
         lt->disp = display;
      }
   }
   return change;
}
/*
**********************************
**     xdl_rot_sim_labhkl       **
**********************************

Purpose: Get required indices for spot label

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_labhkl (gd, idx, h, k, l)

XDl_rot_sim *gd;     /* Pointer to global data area */
int idx;             /* Index in reflections arrays */
int *h;              /* Returns required 'h' index */
int *k;              /* Returns required 'k' index */
int *l;              /* Returns required 'l' index */
{
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   int ks;                /* Symmetry flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */

   prl_getref(gd->mindx, idx, &icrys, &imgnum,
                      h, k, l, &ks, &xf, &yf, &nimg,
                      &ifirst, &ir_flag, &inr_flag, &spat);
   return;
}
/*
***********************************
**     xdl_rot_sim_save_mark     **
***********************************

Purpose: Save area to be marked

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_save_mark (gd, idx)

XDl_rot_sim *gd;    /* Pointer to global data area */
int idx;             /* Index for spot to be marked */
{
   int max_x;      /* Max allowed x value */
   int max_y;      /* Max allowed y value */
   int xpix;       /* x pixel position */
   int ypix;       /* y pixel position */

   xdl_rot_sim_pix (gd, idx, &xpix, &ypix);
   max_x = 2*(gd->half_size + 1) - MARK_SIZ + 1;
   max_y = max_x;
   gd->mark_x = xpix - MARK_SIZ/2;
   if (gd->mark_x<0) gd->mark_x = 0;
   if (gd->mark_x>max_x) gd->mark_x = max_x;
   gd->mark_y = ypix - MARK_SIZ/2;
   if (gd->mark_y<0) gd->mark_y = 0;
   if (gd->mark_y>max_y) gd->mark_y = max_y;
   gd->mark_w = MARK_SIZ;
   gd->mark_h = MARK_SIZ;
   XCopyArea (XDL_display, gd->wdisp, gd->mark_area,
              gd->paint_gc, gd->mark_x, gd->mark_y, 
              gd->mark_w, gd->mark_h, 0, 0);
   gd->mark_saved = 1;
   return;
}
/*
**************************************
**     xdl_rot_sim_restore_mark     **
**************************************

Purpose: Restore area which was marked

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_restore_mark (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */
{
   XCopyArea (XDL_display, gd->mark_area, gd->wdisp, 
              gd->paint_gc, 0, 0, gd->mark_w, gd->mark_h, 
              gd->mark_x, gd->mark_y);
   gd->mark_saved = 0;
   return;
}
/*
***********************************
**     xdl_rot_sim_save_area     **
***********************************

Purpose: Save area (for labelling)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_save_area (gd, x, y, w, h)

XDl_rot_sim *gd;    /* Pointer to global data area */
int x;               /* x coordinate of top left corner of area */
int y;               /* y coordinate of top left corner of area */
int w;               /* width to be saved */
int h;               /* height to be saved */
{
   int max_x;      /* Max allowed x value */
   int max_y;      /* Max allowed y value */

   max_x = 2*(gd->half_size + 1) - w + 1;
   max_y = 2*(gd->half_size + 1) - h + 1;
   gd->sav_x = x;
   if (gd->sav_x<0) gd->sav_x = 0;
   if (gd->sav_x>max_x) gd->sav_x = max_x;
   gd->sav_y = y;
   if (gd->sav_y<0) gd->sav_y = 0;
   if (gd->sav_y>max_y) gd->sav_y = max_y;
   gd->sav_w = w;
   gd->sav_h = h;
   XCopyArea (XDL_display, gd->wdisp, gd->save_area,
              gd->paint_gc, gd->sav_x, gd->sav_y, 
              gd->sav_w, gd->sav_h, 0, 0);
   gd->area_saved = 1;
   return;
}
/*
**************************************
**     xdl_rot_sim_restore_area     **
**************************************

Purpose: Restore saved area

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_restore_area (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */
{
   XCopyArea (XDL_display, gd->save_area, gd->wdisp, 
              gd->paint_gc, 0, 0, gd->sav_w, gd->sav_h, 
              gd->sav_x, gd->sav_y);
   gd->area_saved = 0;
   return;
}
/*
*************************************
**     xdl_rot_sim_hsv_to_rgb      **
*************************************

Purpose: Convert HSV colour to RGB colour

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_hsv_to_rgb (h, s, v, r, g, b)

float h;             /* Hue 0.0 to 360.0 (R)*/
float s;             /* Saturation 0.0 to 1.0 (R)*/
float v;             /* Value 0.0 to 1.0 (R)*/
float *r;            /* Returned red value 0.0 to 1.0 (W)*/
float *g;            /* Returned green value 0.0 to 1.0 (W)*/
float *b;            /* Returned blue value 0.0 to 1.0 (W)*/

{
   float p1, p2, p3, fr, iv;
   int i;

   if (h==360.0) h = 0.0;
   h = h/60.0;
   iv = (float) floor ((double) h);
   fr = h - iv;
   p1 = v * (1.0-s);
   p2 = v * (1.0 - (s*fr));
   p3 = v * (1.0 - (s * (1.0-fr)));
   i = (int) iv;
   switch(i)
   {
      case 0:
         *r = v;
         *g = p3;
         *b = p1;
         break;
      case 1:
         *r = p2;
         *g = v;
         *b = p1;
         break;
      case 2:
         *r = p1;
         *g = v;
         *b = p3;
         break;
      case 3:
         *r = p1;
         *g = p2;
         *b = v;
         break;
      case 4:
         *r = p3;
         *g = p1;
         *b = v;
         break;
      case 5:
         *r = v;
         *g = p1;
         *b = p2;
         break;
   }
   return;
}
/*
*****************************
**     xdl_rot_sim_ps      **
*****************************

Purpose: Postscript output processing

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_ps (gd, reply, len, prompt_num, iret)

XDl_rot_sim *gd;    /* Pointer to global data area */
char *reply;         /* Reply to last prompt */
int len;             /* Length of reply */
int prompt_num;      /* Postscript prompt number */
int iret;            /* Return flag =0 blank string
                                    =1 non-blank string
                                    =-1 escape */
{
   int xroot;       /* Root x position for error message */
   int yroot;       /* Root y position for error message */
   int i, j;        /* Temp/loop variables */
   int prompt_only; /* =1 prompt without processing,=0 process then prompt */
   static float spsiz[3] = {0.35, 0.75, 1.5};  /* Default spot sizes in mm */
   static float smsiz[3] = {1.5, 2.0, 2.5};    /* Default symbol sizes in mm */
   float val;       /* Temporary value */
   char prompt[61]; /* Temporary prompt string */

   /* Process current reply */
    
   prompt_only = 0;
   xdl_rootxy (gd->wpanel_ps, 200, 20, &xroot, &yroot);
   if (iret==-1)
   {
      if (gd->ps_on>0) fclose (gd->psfil);
      gd->ps_on = 0;
      xdl_panel_io_clear (gd->ps_io);
      return;
   }
   if (prompt_num==1)
   {
      if (iret==0)
      {
         gd->ps_on = 0;
         xdl_panel_io_clear (gd->ps_io);
         return;
      }
      gd->psfil = fopen (reply, "w");
      if (gd->psfil==0)
      {
        i = xdl_popup_notice (xroot, yroot, 
                             "** Cannot open file **", 0, 
                             NULL, -1,
                             "Retry", 0, "Cancel", 0, 2, 1);
        xdl_panel_io_clear (gd->ps_io);
        if (i==1)
        {
           gd->ps_on = 1;
           xdl_panel_io_prompt (gd->ps_io, "Postscript file name:", 0, 1);
        }
        else
        {
           gd->ps_on = 0;
        }
        return;
      }
      xdl_rot_sim_pshead (gd);
   }
   labl:
   switch (gd->type)
   {
      case 1:
         switch (prompt_num)
         {
             case 1:
               prompt_num ++;
               sprintf (prompt,"Spot diameter in mm [%.2f]:",
                       spsiz[gd->spotsize-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 2:
               if (prompt_only==0)
               {
                  gd->ps_spot_rad = spsiz[gd->spotsize-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.1||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.1 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_spot_rad = val/2.0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                                   "Add key [y]:",
                                   0, prompt_num);
               break;
            case 3:
               if (prompt_only==0)
               {
                  gd->ps_addkey = 1;
                  if (iret>0)
                  {
                     if (reply[0]!='y'&&reply[0]!='Y'&&
                         reply[0]!='n'&&reply[0]!='N')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 'y' or 'n'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='n'||reply[0]=='N') gd->ps_addkey = 0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, "Comment:", 0, prompt_num);
               break;
            case 4:
               gd->ps_comment = reply;
               xdl_rot_sim_psdraw1 (gd);
               xdl_panel_io_clear (gd->ps_io);
               gd->ps_on = 0;
         }
         break;

      case 2:
         switch (prompt_num)
         {
            case 1:
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                      "Standard (s) or absolute (a) scale [s]:", 39,
                      prompt_num);
               break;
            case 2:
               if (prompt_only==0)
               {
                  gd->plot_rad = 90.0;
                  if (iret>0)
                  {
                     if (reply[0]!='s'&&reply[0]!='S'&&
                         reply[0]!='a'&&reply[0]!='A')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 's' or 'a'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='a'||reply[0]=='A') gd->plot_rad = gd->filmr;
                  }
                  if (gd->plot_rad>90.0)
                  {
                     i = xdl_popup_notice (xroot, yroot, 
                         "*Warning*", 0, 
                         "Plot too large - will be clipped", 0,
                         "Continue", 0, NULL, -1, 2, 1);
                  }
               }
               prompt_num ++;
               sprintf (prompt,"Spot diameter in mm [%.2f]:",
                       spsiz[gd->size_sym2-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 3:
               if (prompt_only==0)
               {
                  gd->ps_spot_rad = spsiz[gd->size_sym2-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.1||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.1 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_spot_rad = val/2.0;
                  }
               }
               prompt_num ++;
               if (gd->opt_highlight2==1)
               {
                  gd->ps_color = 0;
                  gd->ps_symb_hs = smsiz[gd->size_sym2-1]/2.0;
                  prompt_num ++;  /* Skip next two prompts */
                  prompt_only = 1; goto labl;
               }
               sprintf (prompt,"Symbol size mm [%.2f]:",
                       smsiz[gd->size_sym2-1]);
               xdl_panel_io_prompt (gd->ps_io, prompt, 0, prompt_num);
               break;
            case 4:
               if (prompt_only==0)
               {
                  gd->ps_symb_hs = smsiz[gd->size_sym2-1]/2.0;
                  if (iret>0)
                  {
                     i = sscanf (reply,"%f",&val);
                     if (i==0)
                     {
                        j = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Invalid number format", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (val<0.5||val>10.0)
                     {
                        i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Value must be in range 0.5 to 10.0", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                        prompt_num --; prompt_only = 1; goto labl;
                     }
                     gd->ps_symb_hs = val/2.0;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, 
                                   "Use coloured symbols [n]:",
                                   0, prompt_num);
               break;
            case 5:
               if (prompt_only==0)
               {
                  gd->ps_color = 0;
                  if (iret>0)
                  {
                     if (reply[0]!='y'&&reply[0]!='Y'&&
                         reply[0]!='n'&&reply[0]!='N')
                     {
                         i = xdl_popup_notice (xroot, yroot, 
                             "**Error**", 0, 
                             "Reply must be 'y' or 'n'", 0,
                             "Continue", 0, NULL, -1, 2, 1);
                         prompt_num --; prompt_only = 1; goto labl;
                     }
                     if (reply[0]=='y'||reply[0]=='Y') gd->ps_color = 1;
                  }
               }
               prompt_num ++;
               xdl_panel_io_prompt (gd->ps_io, "Comment:", 0, prompt_num);
               break;
            case 6:
               gd->ps_comment = reply;
               xdl_rot_sim_psdraw2 (gd);
               xdl_panel_io_clear (gd->ps_io);
               gd->ps_on = 0;
         }
         break;
   }
   return;
}
/*
*********************************
**     xdl_rot_sim_pshead      **
*********************************

Purpose: Postscript header info output

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_pshead (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */
{
   int i;       /*Temp/loop variable */

   fprintf (gd->psfil,"%%!\nnewpath\n");
   fprintf (gd->psfil,"/M {moveto} def\n");
   fprintf (gd->psfil,"/L {lineto} def\n");
   fprintf (gd->psfil,"/R {rlineto} def\n");
   fprintf (gd->psfil,"/S {stroke} def\n");
   fprintf (gd->psfil,"/mm_to_p 2.835 def\n");

   /* Spot print */
   fprintf (gd->psfil,"\n%% Output spot (xs ys rad_mm S1) %%\n");
   fprintf (gd->psfil,"/S1 {/rad_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/rad {rad_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs ys M xs ys rad 0 360 arc fill S} def\n");

   /* Vertical cross print */
   fprintf (gd->psfil,"\n%% Output vertical cross (xs ys hs_mm S2) %%\n");
   fprintf (gd->psfil,"/S2 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"xs ys hs sub M xs ys hs add L\n");
   fprintf (gd->psfil,"S} def\n");

   /* Diagonal cross print */
   fprintf (gd->psfil,"\n%% Output diagonal  cross (xs ys hs_mm S3) %%\n");
   fprintf (gd->psfil,"/S3 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys hs sub M xs hs add ys hs add L\n");
   fprintf (gd->psfil,"xs hs add ys hs sub M xs hs sub ys hs add L\n");
   fprintf (gd->psfil,"S} def\n");

   /* Star print */
   fprintf (gd->psfil,"\n%% Star (xs ys hs_mm S4) %%\n");
   fprintf (gd->psfil,"/S4 {/hs_mm exch def /ys exch def /xs exch def\n");
   fprintf (gd->psfil,"/hs {hs_mm mm_to_p mul} def\n");
   fprintf (gd->psfil,"xs hs sub ys M xs hs add ys L\n");
   fprintf (gd->psfil,"xs ys hs sub M xs ys hs add L\n");
   fprintf (gd->psfil,"xs hs sub ys hs sub M xs hs add ys hs add L\n");
   fprintf (gd->psfil,"xs hs sub ys hs add M xs hs add ys hs sub L\n");
   fprintf (gd->psfil,"S} def\n");

   fprintf (gd->psfil,"\n1 setlinewidth\n");

   /* Colour table */
   fprintf (gd->psfil,"/black {0 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/white {1 1 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/red {1 0 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/blue {0 0 1 setrgbcolor} def\n");
   fprintf (gd->psfil,"/green {0 1 0 setrgbcolor} def\n");
   fprintf (gd->psfil,"/prcolr   {0.8 0.8 0.8 setrgbcolor} def\n");
   if (gd->type==1)
   {
      for (i=0;i<8;++i)
      {
         fprintf (gd->psfil, "/color%d {%.3f %.3f %.3f setrgbcolor} def\n",
                  i, gd->ps_r[i], gd->ps_g[i], gd->ps_b[i]);
      }
   }

   /*Break into lines procedure */
   fprintf (gd->psfil,"/wordbreak ( ) def /BreakIntoLines\n");
   fprintf (gd->psfil,
            "{/wrlin exch def /linewidth exch def /textstring exch def\n");
   fprintf (gd->psfil,"/breakwidth wordbreak stringwidth pop def\n");
   fprintf (gd->psfil,"/curwidth 0 def /lastwordbreak 0 def\n");
   fprintf (gd->psfil,"/startchar 0 def /restoftext textstring def\n");
   fprintf (gd->psfil,"{restoftext wordbreak search\n");
   fprintf (gd->psfil,"{/nextword exch def pop /restoftext exch def\n");
   fprintf (gd->psfil,"/wordwidth nextword stringwidth pop def\n");
   fprintf (gd->psfil,"curwidth wordwidth add linewidth gt\n");
   fprintf (gd->psfil,"{textstring startchar lastwordbreak startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin /startchar lastwordbreak def\n");
   fprintf (gd->psfil,"/curwidth wordwidth breakwidth add def}\n");
   fprintf (gd->psfil,"{/curwidth curwidth wordwidth add\n");
   fprintf (gd->psfil,"breakwidth add def } ifelse\n");
   fprintf (gd->psfil,
            "/lastwordbreak lastwordbreak nextword length add 1 add def}\n");
   fprintf (gd->psfil,"{pop exit} ifelse }loop\n");
   fprintf (gd->psfil,"/lastchar textstring length def\n");
   fprintf (gd->psfil,"textstring startchar lastchar startchar sub\n");
   fprintf (gd->psfil,"getinterval wrlin } def\n");

   return;
}
/*
************************************
**     xdl_rot_sim_psdraw1        **
************************************

Purpose: Postscript output for colour coded option display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_psdraw1 (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */

{
   int i, j;              /* Temp/loop variables */
   int k;                 /* index to current reflection */
   float xs, ys;          /* Spot position (points) */
   float dxs, dys;        /* Spot position offsets from centre (points) */
   float xdisc, ydisc;    /* Key colour disc position (points) */
   float xstr, ystr;      /* Key string position */
   int colr;              /* Colour (/code) for xdl_rot_sim_symb1 */
   int hh, kk, ll;        /* Unpacked indices */
   int ks;                /* Symmetry flag */
   int nref;              /* Total no. of reflections in reflections list */
   int nimg;              /* No. of images spot is spread over */
   int ifirst;            /* First image of spot (offset) */
   int icrys;             /* Crystal number of spot */
   int imgnum;            /* Image number of spot */
   int ir_flag;           /* Spot recorded flag */
   int inr_flag;          /* Spot not-recorded flag */
   int spat;              /* Spatially overlapped flag */
   float xf;              /* Spot 'xf' position in mm */
   float yf;              /* Spot 'yf' position in mm */

   /* Simulation area */

   gd->ps_xc = 297.6; 
   gd->ps_yc = 520.0;
   gd->plot_rad = 90.0;
   gd->ps_scal = gd->plot_rad/gd->filmr;
   fprintf (gd->psfil,"initclip clippath 0 0 0 setrgbcolor fill\n");


   switch (gd->color_coding)
   {
      case 1: /* Fulls/partials */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               dxs = gd->ps_scal*xf*gd->mm_to_p;
               dys = gd->ps_scal*yf*gd->mm_to_p;
               /* Note 'y' for PS is opposite direction from X-windows 'y' */
               switch (gd->iord)
	       {
	          case 1:
                  xs = gd->ps_xc + dxs;
                  ys = gd->ps_yc - dys;
                  break;

	          case 2:
                  xs = gd->ps_xc + dxs;
                  ys = gd->ps_yc + dys;
                  break;

	          case 3:
                  xs = gd->ps_xc - dxs;
                  ys = gd->ps_yc - dys;
                  break;

	          case 4:
                  xs = gd->ps_xc - dxs;
                  ys = gd->ps_yc + dys;
                  break;

	          case 5:
                  xs = gd->ps_xc + dys;
                  ys = gd->ps_yc - dxs;
                  break;

	          case 6:
                  xs = gd->ps_xc + dys;
                  ys = gd->ps_yc + dxs;
                  break;

	          case 7:
                  xs = gd->ps_xc - dys;
                  ys = gd->ps_yc - dxs;
                  break;

	          case 8:
                  xs = gd->ps_xc - dys;
                  ys = gd->ps_yc + dxs;
                  break;
               }
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
		  case 3:
                  colr = 2;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil,
               "(Colour coded showing fulls and partials) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");


            for(i=0;i<3;++i)
            {
               switch (i)
	       {
	          case 0:
                  colr = 6;
                  break;
    
	          case 1:
                  colr = 2;
                  break;

	          case 2:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        colr, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "%.2f %.2f M (full) show\n",
                              xstr,ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "%.2f %.2f M (partial) show\n",
                              xstr,ystr);
                     break;
                  case 2:
                     fprintf (gd->psfil, "%.2f %.2f M (cusp) show\n",
                              xstr,ystr);
                     break;
               }
               ydisc -= 15.0;
               ystr -= 15.0;
            }
	 }
         break;

      case 2: /* Partiality 1  */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xs = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
               ys = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
                  colr = 4;
                  if (nimg>gd->partn_cur) colr = 2;
                  break;

		  case 3:
                  colr = 1;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil,
               "(Colour coded showing fulls and classes of partials) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");


            for(i=0;i<5;++i)
            {
               switch (i)
	       {
	          case 0:
                  colr = 6;
                  break;
    
	          case 1:
                  colr = 4;
                  break;

	          case 2:
                  colr = 2;
                  break;

		  case 3:
                  colr = 1;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        colr, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "%.2f %.2f M (full) show\n",
                              xstr,ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "%.2f %.2f M (partial <=n) show\n",
                              xstr,ystr);
                     break;
                  case 2:
                     fprintf (gd->psfil, "%.2f %.2f M (partial >n) show\n",
                              xstr,ystr);
                     break;
                  case 3:
                     fprintf (gd->psfil, "%.2f %.2f M (wide partial) show\n",
                              xstr,ystr);
                     break;
                  case 4:
                     fprintf (gd->psfil, "%.2f %.2f M (cusp partial) show\n",
                              xstr,ystr);
                     break;
               }
               ydisc -= 15.0;
               ystr -= 15.0;
            }
	 }
         fprintf (gd->psfil, "100.0 %.2f M (Partial 'n' = %d) show\n", ystr,
                  gd->partn_cur);
         ystr -= 15.0;
         break;

      case 3: /* Partiality 2  */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xs = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
               ys = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
               switch (ir_flag)
	       {
	          case 1:
                  colr = 6;
                  break;
    
		  case 2:
                  colr = 6-nimg;
                  if (colr<2) colr = 2;
                  break;

		  case 3:
                  colr = 1;
                  break;

		  case 4:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil,
               "(Colour coded showing fulls and classes of partials) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");


            for(i=0;i<6;++i)
            {
               switch (i)
	       {
	          case 0:
                  colr = 6;
                  break;
    
	          case 1:
                  colr = 4;
                  break;

	          case 2:
                  colr = 3;
                  break;

		  case 3:
                  colr = 2;
                  break;

		  case 4:
                  colr = 1;
                  break;

		  case 5:
                  colr = 0;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        colr, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "%.2f %.2f M (full) show\n",
                              xstr,ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "%.2f %.2f M (partial =2) show\n",
                              xstr,ystr);
                     break;
                  case 2:
                     fprintf (gd->psfil, "%.2f %.2f M (partial =3) show\n",
                              xstr,ystr);
                     break;
                  case 3:
                     fprintf (gd->psfil, "%.2f %.2f M (partial >3) show\n",
                              xstr,ystr);
                     break;
                  case 4:
                     fprintf (gd->psfil, "%.2f %.2f M (wide partial) show\n",
                              xstr,ystr);
                     break;
                  case 5:
                     fprintf (gd->psfil, "%.2f %.2f M (cusp partial) show\n",
                              xstr,ystr);
                     break;
               }
               ydisc -= 15.0;
               ystr -= 15.0;
            }
	 }
         break;

      case 4: /* All the same */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xs = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
               ys = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
               colr = 5;
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil,
               "(All recorded spots) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        colr, xdisc, ydisc);
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "%.2f %.2f M (recorded spot) show\n",
                     xstr,ystr);
            ydisc -= 15.0;
            ystr -= 15.0;
	 }
         break;

      case 5: /* Overlap status */
         nref = prl_numrefs (gd->mindx);
         if (nref>0)
	 {
            for (i=0;i<nref;++i)
            {
               prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                                 &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                                 &ifirst, &ir_flag, &inr_flag, &spat);
               if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
               if (ir_flag==0) continue;
               if (gd->spot_type==2&&ir_flag!=1) continue;
               if (gd->spot_type==3&&ir_flag<2) continue;
               if (gd->spatials!=1)
               {
                  if (gd->spatials==2&&spat) continue;
                  if (gd->spatials==3&&spat==0) continue;
               }
               xs = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
               ys = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
               switch (spat)
	       {
	          case 0:
                  colr = 6;
                  break;
    
		  case 1:
                  colr = 2;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f %.3f S1 S\n",
                           colr, xs, ys, gd->ps_spot_rad);
            }
         }
         if (gd->ps_addkey)
         {
            fprintf (gd->psfil, "prcolr\n");
            fprintf (gd->psfil, "/Helvetica findfont 12 scalefont setfont\n");
            fprintf (gd->psfil, "50 220 M (Key:) show\n");
            fprintf (gd->psfil, "100 220 M\n");
            fprintf (gd->psfil,
               "(Colour coded showing overlap status) show\n");
            xdisc = 105.0;
            ydisc = 209.0;
            xstr = 120.0;
            ystr = 205.0;
            fprintf (gd->psfil, "prcolr\n");


            for(i=0;i<2;++i)
            {
               switch (i)
	       {
	          case 0:
                  colr = 6;
                  break;
    
	          case 1:
                  colr = 2;
                  break;
	       }
               fprintf (gd->psfil, "color%d %.2f %.2f 1.5 S1 S\n", 
                        colr, xdisc, ydisc);
               fprintf (gd->psfil, "prcolr\n");
               switch (i)
               {
                  case 0:
                     fprintf (gd->psfil, "%.2f %.2f M (not-overlapped) show\n",
                              xstr,ystr);
                     break;
                  case 1:
                     fprintf (gd->psfil, "%.2f %.2f M (overlapped) show\n",
                              xstr,ystr);
                     break;
               }
               ydisc -= 15.0;
               ystr -= 15.0;
            }
	 }
         break;
   }
   /* Add remainder of key if required */
   if (gd->ps_addkey)
   {
      fprintf (gd->psfil, "prcolr\n");
      switch (gd->spot_type)
      {
         case 1:
            fprintf (gd->psfil, "100 %.2f M (All spots;) show\n",
                    ystr);
            break;

         case 2:
            fprintf (gd->psfil, "100 %.2f M (Fulls only;) show\n",
                    ystr);
            break;

         case 3:
            fprintf (gd->psfil, "100 %.2f M (Partials only;) show\n",
                    ystr);
            break;
      }
      switch (gd->spatials)
      {
         case 1:
            fprintf (gd->psfil, "( Spatial overlaps included) show\n");
            break;

         case 2:
            fprintf (gd->psfil, "( Spatial overlaps excluded) show\n");
            break;

         case 3:
            fprintf (gd->psfil, "( Spatial overlaps only) show\n");
            break;
      }
   }

   /* Comment */

   if (strlen(gd->ps_comment)>0)
   {
      fprintf (gd->psfil, "prcolr\n");
      fprintf (gd->psfil,"/Helvetica findfont 14 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 56 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->ps_comment);
      fprintf (gd->psfil,"490 {50 ycomm M show /ycomm ycomm 14 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }

   /* Show page */

   fprintf (gd->psfil,"showpage\n");
   fclose (gd->psfil);

   return;
}
/*
************************************
**     xdl_rot_sim_psdraw2        **
************************************

Purpose: Routine to perform postcript o/p for type 2 display

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_psdraw2 (gd)

XDl_rot_sim *gd;    /* Pointer to global data area */

{
   int i, j;          /* Temp/loop variables */
   int styp;          /* Symbol type 1=spot, 2=vert cross, 3 = diag cross,
                         4=star */
   int colr;          /* Colour flag for xdl_rot_sim_symb2 */
   float xorig;       /* y origin of box around plot */
   float yorig;       /* y origin of box around plot */
   float xlen;        /* x length of box around plot */
   float ylen;        /* y length of box around plot */
   float xs;          /* Spot/symbol x coordinate (Postscript) */
   float ys;          /* Spot/symbol y coordinate (Postscript) */
   float dxs;         /* Spot/symbol x coordinate offset (Postscript) */
   float dys;         /* Spot/symbol y coordinate offset (Postscript) */
   float xc1;         /* Annotation column x coordinate */
   float xc5;         /* Annotation column x coordinate */
   float xc6;         /* Annotation column x coordinate */
   float ytop;        /* Annotation top y value */
   float dy;          /* Line spacing for annotation */
   float yy;          /* Annotation y coordinate */
   int hh, kk, ll;    /* Unpacked indices */
   int ks;            /* Symmetry flag */
   int nref;          /* Total no. of reflections in reflections list */
   int nimg;          /* No. of images spot is spread over */
   int ifirst;        /* First image of spot (offset) */
   int icrys;         /* Crystal number of spot */
   int imgnum;        /* Image number of spot */
   int ir_flag;       /* Spot recorded flag */
   int inr_flag;      /* Spot not-recorded flag */
   int spat;          /* Spatially overlapped flag */
   float xf;          /* Spot 'xf' position in mm */
   float yf;          /* Spot 'yf' position in mm */
   time_t tm;         /* Time value */
   char datstr[30];   /* Date & time string */
   static float rtod = 57.29578; /* Convert radians to degrees */
   static char *sys_str[] = {"Triclinic",
                            "Monoclinic",
                            "Orthorhombic",
                            "Tetragonal",
                            "Hexagonal",
                            "Rhombohedral",
                            "Cubic"};
   static char *latt_str[] = {"P","A","B","C","I","F","R"};

   /* Plot heading */

   time(&tm);
   strcpy(datstr,ctime(&tm));
   datstr[24] = 0;
   fprintf (gd->psfil,"/Helvetica findfont 16 scalefont setfont\n");
   fprintf (gd->psfil,"95.0 780.0 M (Rotation Pattern Simulation %s) show\n",
            datstr);

   /* Simulation area */

   gd->ps_xc = 297.6; 
   gd->ps_yc = 520.0;
   gd->ps_scal = gd->plot_rad/gd->filmr;
   gd->ps_pixsiz = (gd->plot_rad/(float)gd->half_size)*gd->mm_to_p;
   xorig = gd->ps_xc - gd->plot_rad*gd->mm_to_p;
   yorig = gd->ps_yc - gd->plot_rad*gd->mm_to_p;
   if (gd->plot_rad<=90.0)
   {
      xlen = 2*gd->plot_rad*gd->mm_to_p;
      xorig = gd->ps_xc - gd->plot_rad*gd->mm_to_p;
      yorig = gd->ps_yc - gd->plot_rad*gd->mm_to_p;
   }
   else
   {
      xlen = 180.0*gd->mm_to_p;
      xorig = gd->ps_xc - 90.0*gd->mm_to_p;
      yorig = gd->ps_yc - 90.0*gd->mm_to_p;
   }
   ylen = xlen;
   fprintf (gd->psfil,"gsave\n");
   fprintf (gd->psfil,"%.3f %.3f M\n",xorig,yorig);
   fprintf (gd->psfil,"%.3f 0 R\n",xlen);
   fprintf (gd->psfil,"0 %.3f R\n",ylen);
   fprintf (gd->psfil,"%.3f 0 R\n",-xlen);
   fprintf (gd->psfil,"0 %.3f R\n",-ylen);
   fprintf (gd->psfil,"S\n");
   fprintf (gd->psfil,"%.3f %.3f M\n",xorig,yorig);
   fprintf (gd->psfil,"%.3f 0 R\n",xlen);
   fprintf (gd->psfil,"0 %.3f R\n",ylen);
   fprintf (gd->psfil,"%.3f 0 R\n",-xlen);
   fprintf (gd->psfil,"0 %.3f R\n",-ylen);
   fprintf (gd->psfil,"closepath clip newpath\n");

   nref = prl_numrefs (gd->mindx);
   for (i=0;i<nref;++i)
   {
      prl_getref(gd->mindx, i+1, &icrys, &imgnum,
                         &hh, &kk, &ll, &ks, &xf, &yf, &nimg,
                         &ifirst, &ir_flag, &inr_flag, &spat);
      if (icrys!=gd->icrys||imgnum!=gd->imgnum) continue;
      if (gd->type==2) xdl_rot_sim_rflags (gd, i+1, &ir_flag,
                                            &nimg, &ifirst);
      if (ir_flag==0) continue;
      styp = 1;
      switch (gd->opt_highlight2)
      {
         case 2:
            if (ir_flag==1) styp = 2;
            break;
         case 3:
            if (ir_flag==2)
	    {
               styp = 2;
               if (nimg>gd->partn_cur) styp = 3;
	    }
            if (ir_flag==3) styp = 3;
            if (ir_flag==4) styp = 4;
            break;
      }
      xs = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
      ys = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
      dxs = gd->ps_scal*xf*gd->mm_to_p;
      dys = gd->ps_scal*yf*gd->mm_to_p;
      /* Note 'y' for PS is opposite direction from X-windows 'y' */
      switch (gd->iord)
      {
         case 1:
         xs = gd->ps_xc + dxs;
         ys = gd->ps_yc - dys;
         break;

         case 2:
         xs = gd->ps_xc + dxs;
         ys = gd->ps_yc + dys;
         break;

	 case 3:
         xs = gd->ps_xc - dxs;
         ys = gd->ps_yc - dys;
         break;

	 case 4:
         xs = gd->ps_xc - dxs;
         ys = gd->ps_yc + dys;
         break;

	 case 5:
         xs = gd->ps_xc + dys;
         ys = gd->ps_yc - dxs;
         break;

	 case 6:
         xs = gd->ps_xc + dys;
         ys = gd->ps_yc + dxs;
         break;

	 case 7:
         xs = gd->ps_xc - dys;
         ys = gd->ps_yc - dxs;
         break;

	 case 8:
         xs = gd->ps_xc - dys;
         ys = gd->ps_yc + dxs;
         break;
      }

      fprintf (gd->psfil,"black\n");
      switch (styp)
      {
         case 1:
            if (gd->ps_color) fprintf (gd->psfil,"black ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xs, ys, gd->ps_spot_rad);
            break;
         case 2:
            if (gd->ps_color) fprintf (gd->psfil,"blue ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xs, ys, gd->ps_symb_hs);
            break;
         case 3:
            if (gd->ps_color) fprintf (gd->psfil,"red ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S3 S\n",
                     xs, ys, gd->ps_symb_hs);
            break;
         case 4:
            if (gd->ps_color) fprintf (gd->psfil,"green ");
            fprintf (gd->psfil,"%.2f %.2f %.3f S4 S\n", 
            xs, ys, gd->ps_symb_hs);
            break;
      }

   }

   /* Draw labels if required and if present */

   if (gd->opt_labels2>1&&gd->num_labels>0)
   {
      fprintf (gd->psfil,"/linwsav currentlinewidth def\n");
      if (gd->plot_rad<90.0)
      {
         fprintf (gd->psfil,"%.2f setlinewidth\n", (gd->plot_rad/90.0));
      }
      for (i=0;i<gd->num_labels;++i) xdl_rot_sim_pslabel (gd, i);
      fprintf (gd->psfil,"linwsav setlinewidth\n");
   }

   fprintf (gd->psfil,"grestore\n");

   /* Annotation */

   fprintf (gd->psfil,"/Helvetica findfont 13 scalefont setfont\n");
   xc1 = 40.0; 
   xc5 = 450.0;
   xc6 = 460.0;
   dy = 13.0;
   ytop = 220.0;
   yy = ytop;
   fprintf (gd->psfil,"%.2f %.2f M\n", xc1, yy);
   fprintf (gd->psfil," (crystal system: %s, lattice type: %s) show\n",
            sys_str[gd->isys-1], latt_str[gd->latt-1]);
   yy = ytop - dy;
   fprintf (gd->psfil,"%.2f %.2f M\n", xc1, yy);
   fprintf (gd->psfil," (cell: %.2f %.2f %.2f %.2f %.2f %.2f) show\n",
            gd->cell[0], gd->cell[1], gd->cell[2], 
            gd->cell[3], gd->cell[4], gd->cell[5]);
   yy = ytop - 2.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M\n", xc1, yy);
   fprintf (gd->psfil,
         " (umat: %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f) show\n",
            gd->umat[0], gd->umat[1], gd->umat[2], 
            gd->umat[3], gd->umat[4], gd->umat[5],
            gd->umat[6], gd->umat[7], gd->umat[8]);
   yy = ytop -3.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M\n", xc1, yy);
   fprintf (gd->psfil,
     " (missetting angles (deg): %.3f %.3f %.3f) show\n",
       gd->delphi[0], gd->delphi[1], gd->delphi[2]); 
   yy = ytop -4.0*dy;
   fprintf (gd->psfil,"%.2f %.2f M\n", xc1, yy);
   fprintf (gd->psfil,
     " (rotation start: %.3f deg, range: %.3f deg) show\n",
       gd->rots, gd->cur_phirange); 
   yy = ytop - 5.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (crystal to detector distance: %.2f mm) show \n",
            xc1, yy, gd->ctod);
   yy = ytop - 6.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (wavelength: %.2f Angstroms) show \n",
            xc1, yy, gd->wave);
   yy = ytop - 7.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M\n",xc1, yy);
   fprintf (gd->psfil,
            "(mosaicity: %.4f deg, divv: %.4f deg, divh: %.4f deg) show\n",
            gd->cur_eta*2.0*rtod, gd->cur_divv*2.0*rtod, 
            gd->cur_divh*2.0*rtod);
   yy = ytop - 8.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (delcor: %.5f) show\n", xc1, yy, gd->delcor);
   yy = ytop - 9.0*dy;
   fprintf (gd->psfil,
            "%.2f %.2f M (delamb: %.5f) show\n", xc1, yy, gd->cur_delamb);
   yy = ytop - 10.0*dy;
   switch (gd->iord)
   {
      case 1:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the right, %s down) show\n", 
            xc1, yy, gd->ax1_name, gd->ax2_name);
      break;

      case 2:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the right, %s up) show\n", 
            xc1, yy, gd->ax1_name, gd->ax2_name);
      break;

      case 3:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the left, %s down) show\n", 
            xc1, yy, gd->ax1_name, gd->ax2_name);
      break;

      case 4:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the left, %s up) show\n", 
            xc1, yy, gd->ax1_name, gd->ax2_name);
      break;

      case 5:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the right, %s down) show\n", 
            xc1, yy, gd->ax2_name, gd->ax1_name);
      break;

      case 6:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the right, %s up) show\n", 
            xc1, yy, gd->ax2_name, gd->ax1_name);
      break;

      case 7:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the left, %s down) show\n", 
            xc1, yy, gd->ax2_name, gd->ax1_name);
      break;

      case 8:
      fprintf (gd->psfil,
            "%.2f %.2f M (axis %s to the left, %s up) show\n", 
            xc1, yy, gd->ax2_name, gd->ax1_name);
      break;
   }

   switch (gd->opt_highlight2)
   {
      case 1: /* No highlighting */
         break;

      case 2: /* Fulls highlighted */
         fprintf (gd->psfil,"/Helvetica findfont 12 scalefont setfont\n");
         dy = 12.0;
         yy = ytop + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"blue ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xc5, yy, gd->ps_symb_hs);
         yy = ytop - dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"black ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xc5, yy, gd->ps_spot_rad);
         fprintf (gd->psfil,"black\n");
         yy = ytop;
         fprintf (gd->psfil,"%.2f %.2f M (Fulls) show\n", xc6, yy);
         yy = ytop - dy;
         fprintf (gd->psfil,"%.2f %.2f M (Partials) show\n", xc6, yy);
         break;

      case 3: /* Partials highlighted */
         fprintf (gd->psfil,"/Helvetica findfont 12 scalefont setfont\n");
         dy = 12.0;
         yy = ytop + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"black ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S1 S\n", 
                     xc5, yy, gd->ps_spot_rad);
         yy = ytop - dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"blue ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S2 S\n",
                     xc5, yy, gd->ps_symb_hs);
         yy = ytop - 2*dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"red ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S3 S\n",
                     xc5, yy, gd->ps_symb_hs);
         yy = ytop - 3*dy + 6.0;
         if (gd->ps_color) fprintf (gd->psfil,"green ");
         fprintf (gd->psfil,"%.2f %.2f %.3f S4 S\n",
                     xc5, yy, gd->ps_symb_hs);
         fprintf (gd->psfil,"black\n");
         yy = ytop;
         fprintf (gd->psfil,"%.2f %.2f M (Fulls) show\n", xc6, yy);
         yy = ytop - dy;
         fprintf (gd->psfil,"%.2f %.2f M (Partials <= %d) show\n", xc6, yy,
                  gd->partn_cur);
         yy = ytop - 2.0*dy;
         fprintf (gd->psfil,"%.2f %.2f M (Partials > %d) show\n",
                  xc6, yy, gd->partn_cur);
         yy = ytop - 3.0*dy;
         fprintf (gd->psfil,"%.2f %.2f M (Cusp partials) show\n",
                  xc6, yy);
         break;
   }

   /* Comment */

   if (strlen(gd->ps_comment)>0)
   {
      fprintf (gd->psfil,"/Helvetica findfont 15 scalefont setfont\n");
      fprintf (gd->psfil,"/ycomm 60 def\n");
      fprintf (gd->psfil,"(%s)\n",gd->ps_comment);
      fprintf (gd->psfil,"510 {40 ycomm M show /ycomm ycomm 15 sub def}\n");
      fprintf (gd->psfil,"BreakIntoLines\n");
   }

   /* Show page */

   fprintf (gd->psfil,"showpage\n");
   fclose (gd->psfil);
   return;
}
/*
********************************
**     xdl_rot_sim_pslabel    **
********************************

Purpose: Draw an indices label for postscript plot

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_pslabel (gd, loff)

XDl_rot_sim *gd;    /* Pointer to global data area */
int loff;            /* Offset in labels list */
{
   XDl_rot_sim_label *lt;   /* Pointer to current labels structure */
   int l;                    /* Temp/loop variable */
   int ll;                   /* Length of character string */
   int digit;                /* Current digit number 0-9 */
   float xch, ych;           /* Current character position (points)*/
   int xc, yc;               /* Box corner for attached line (pixels)*/
   int xspot, yspot;         /* Spot position (pixels)*/
   float x1;                 /* x corner of label (points)*/
   float y1;                 /* y corner of label (points)*/
   float xsp;                /* Spot x (points) */
   float ysp;                /* Spot y (points) */
   float w;                  /* label width (points) */
   float h;                  /* label height (points) */
   float xf;                 /* Spot 'x' coordinate in mm */
   float yf;                 /* Spot 'y' coordinate in mm */

   lt = gd->labels[loff];
   if (lt->disp==0) return;
   x1 = (lt->x1-gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   y1 = (gd->half_size-lt->y1)*gd->ps_pixsiz + gd->ps_yc;
   w = (lt->x2-lt->x1)*gd->ps_pixsiz;
   h = (lt->y2-lt->y1)*gd->ps_pixsiz;
   fprintf (gd->psfil,"white ");
   fprintf (gd->psfil,"%.2f %.2f M\n",x1,y1);
   fprintf (gd->psfil,"%.3f 0 R\n",w);
   fprintf (gd->psfil,"0 %.3f R\n",-h);
   fprintf (gd->psfil,"%.3f 0 R\n",-w);
   fprintf (gd->psfil,"0 %.3f R\n",h);
   fprintf (gd->psfil,"closepath fill\n");
   fprintf (gd->psfil,"black ");
   fprintf (gd->psfil,"%.2f %.2f M\n",x1,y1);
   fprintf (gd->psfil,"%.3f 0 R\n",w);
   fprintf (gd->psfil,"0 %.3f R\n",-h);
   fprintf (gd->psfil,"%.3f 0 R\n",-w);
   fprintf (gd->psfil,"0 %.3f R\n",h);
   fprintf (gd->psfil,"S\n");
   ll = strlen (lt->hklstr);
   if (ll<=0) return;
   xch = (lt->x1 + 2 - gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   ych = (gd->half_size - lt->y2 + 2)*gd->ps_pixsiz + gd->ps_yc;

   for (l=0;l<ll;++l)
   {
      if (lt->hklstr[l]=='-')
      {
         fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n",
                 xch, ych+4.0*gd->ps_pixsiz, xch+4.0*gd->ps_pixsiz,
                 ych+4.0*gd->ps_pixsiz);
         xch += 7.0*gd->ps_pixsiz;
      }
      else if (lt->hklstr[l]>='0'&&lt->hklstr[l]<='9')
      {
         digit = lt->hklstr[l] - '0';
         xdl_rot_sim_psdigit (gd, digit, xch, ych);
         xch += 7.0*gd->ps_pixsiz;
      }
      else
      {
         xch += 4.0*gd->ps_pixsiz;
      }
   }

   /* Draw attaching line to spot */

   xdl_rot_sim_pix (gd, lt->idx, &xspot, &yspot);
   if (xspot>=lt->x1&&xspot<=lt->x2&&yspot>=lt->y1&&yspot<=lt->y2) return;
   xc = lt->x2;
   if (abs(xspot-lt->x1)<abs(xspot-lt->x2)) xc = lt->x1;
   yc = lt->y2;
   if (abs(yspot-lt->y1)<abs(yspot-lt->y2)) yc = lt->y1;
   x1 = (xc-gd->half_size)*gd->ps_pixsiz + gd->ps_xc;
   y1 = (gd->half_size-yc)*gd->ps_pixsiz + gd->ps_yc;
   prl_getcoords (gd->mindx, lt->idx, &xf, &yf);
   xsp = gd->ps_scal*xf*gd->mm_to_p + gd->ps_xc;
   ysp = gd->ps_scal*yf*gd->mm_to_p + gd->ps_yc;
   fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L S\n", x1, y1, xsp, ysp);

   return;
}
/*
**********************************
**     xdl_rot_sim_psdigit      **
**********************************

Purpose: Draw digit for label for Postscript output

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_psdigit (gd, digit, xch, ych)

XDl_rot_sim *gd;    /* Pointer to global data area */
int digit;           /* Digit number 0 to 9 */
float xch;           /* x position (bottom left corner of digit) (points)*/
float ych;           /* y position (bottom left corner of digit) (points)*/

{
   static short digits[10][16][2]={
   {{1,2},{1,7},{2,8},{4,8},{5,7},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{3,8},{3,1},{-1,1},{5,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{1,2},{1,1},
    {5,1},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},{2,5},{-4,5},
    {5,4},{5,2},{4,1},{2,1},{1,2},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,3},{5,3},{-4,1},{4,5},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,8},{1,5},{4,5},{5,4},{5,2},{4,1},{2,1},
    {1,2},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{5,8},{1,4},{1,2},{2,1},{4,1},{5,2},{5,3},{4,4},
    {2,4},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{1,8},{5,8},{5,6},{1,2},{1,1},{0,0},{0,0},{0,0},
    {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}},

   {{2,5},{1,6},{1,7},{2,8},{4,8},{5,7},{5,6},{4,5},
    {2,5},{1,4},{1,2},{2,1},{4,1},{5,2},{5,4},{4,5}},

   {{1,1},{5,5},{5,7},{4,8},{2,8},{1,7},{1,6},{2,5},
    {4,5},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0}}
   };
   int i;                   /* Temp/loop variable */
   int dx1, dy1, dx2, dy2;  /* Pixel point offsets for lines within character */
   float x1, y1, x2, y2;    /* Coordinates for ends of lines */

   for(i=0;i<16;++i)
   {
      if (digits[digit][i][0]==0) break;
      if (i==0||digits[digit][i][0]<0)
      {
         dx1 = digits[digit][i][0]; if (dx1<0) dx1 = -dx1;
         dy1 = digits[digit][i][1]; if (dy1<0) dy1 = -dy1;
         continue;
      }
      dx2 = digits[digit][i][0];
      dy2 = digits[digit][i][1];
      x1 = xch + (dx1 - 1)*gd->ps_pixsiz;
      y1 = ych + (dy1 - 1)*gd->ps_pixsiz;
      x2 = xch + (dx2 - 1)*gd->ps_pixsiz;
      y2 = ych + (dy2 - 1)*gd->ps_pixsiz;
      fprintf (gd->psfil,"%.2f %.2f M %.2f %.2f L\n",x1, y1, x2, y2);
      dx1 = dx2;
      dy1 = dy2;
   }
   fprintf (gd->psfil, "S\n");
     
   return;
}

/*
**********************************
**     xdl_rot_sim_pix          **
**********************************

Purpose: Get pixel coordinates for a predicted spot

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_pix (gd, idx, xpix, ypix)

XDl_rot_sim *gd;    /* Pointer to global data area */
int idx;            /* Index in reflections list */
int * xpix;         /* Returns 'x' pixel position */
int * ypix;         /* Returns 'y' pixel position */
{
   float xf;   /* Spot 'x' coordinate in mm */
   float yf;   /* Spot 'y' coordinate in mm */
   int d;      /* Distance from centre */
   int dx;     /* 'xf' pixel offset (nearest) */
   int dy;     /* 'yf' pixel offset (nearest) */

   prl_getcoords (gd->mindx, idx, &xf, &yf);
   d = gd->fac*xf;
   dx = ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
   d = gd->fac*yf;
   dy = ((d>0.0)?(int)(d+0.5):(int)(d-0.5));
   switch (gd->iord)
   {
      case 1:
      *xpix = gd->half_size + dx;
      *ypix = gd->half_size + dy;
      break;

      case 2:
      *xpix = gd->half_size + dx;
      *ypix = gd->half_size - dy;
      break;

      case 3:
      *xpix = gd->half_size - dx;
      *ypix = gd->half_size + dy;
      break;

      case 4:
      *xpix = gd->half_size - dx;
      *ypix = gd->half_size - dy;
      break;

      case 5:
      *xpix = gd->half_size + dy;
      *ypix = gd->half_size + dx;
      break;

      case 6:
      *xpix = gd->half_size + dy;
      *ypix = gd->half_size - dx;
      break;

      case 7:
      *xpix = gd->half_size - dy;
      *ypix = gd->half_size + dx;
      break;

      case 8:
      *xpix = gd->half_size - dy;
      *ypix = gd->half_size - dx;
      break;
   }
   return;   
}

/*
**********************************
**     xdl_rot_sim_setlab4      **
**********************************

Purpose: Set string for 4'th label line

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_setlab4 (ir_flag, nimg, ifirst, str)

int ir_flag;        /* Recorded reflection flag */
int nimg;           /* No. of images reflection spread over */
int ifirst;         /* First image (offset) */
char *str;          /* String for  return */
{
   str[0] = 0;
   switch (ir_flag)
   {
      case 1:
      strcat (str,"fully recorded");
      break;

      case 2:
      sprintf (str, "partial %d (offset %d)", nimg, ifirst);
      break;

      case 3:
      strcat (str,"too-wide partial");
      break;

      case 4:
      strcat (str,"cusp partial");
      break;
   }
   return;   
}
/*
**********************************
**     xdl_rot_sim_varmin       **
**********************************

Purpose: Get minimum value of the current variable parameter for which a spot
         will be present

Author:  John W. Campbell, June 1995

***NOTE for DIVH this routine only applies if delcor==0.0  ***

*/

void xdl_rot_sim_varmin (gd, ityp, idx, var_min, irf)

XDl_rot_sim *gd;    /* Pointer to global data area */
int ityp;           /* variable parameter type 1 = phi range width
                                               2 = eta 
                                               3 = divv
                                               4 = divh
                                               5 = delamb */
int idx;            /* Index in reflections list */
float *var_min;     /* Returns the minimum value for the variable */
int *irf;           /* Returns 1 if spot recorded, 0 if not recorded */
{
   int i;             /* Temp/loop variable */
   int ir_flag;       /* Reflection recorded flag */
   int nimg;          /* No. of imagesreflection spread over */
   int ifirst;        /* Offset of first image wrt the current image */
   float phi;         /* Phi value */
   float xr[3];       /* Reciprocal lattice coordinates */
   float rlorf;       /* Lorentz factor */
   float vsq;         /* Vertical displacement**2 */
   float hsq;         /* Horizontal displacement**2 */
   float diverg;      /* Current divergence */
   float esynh;       /* Synchrotron horizontal divergence component */
   float esynv;       /* Synchrotron vertical divergence component */
   float dstar2;      /* dstar**2 */
   float dst4;        /* 0.25 dstar2**2 */
   float rspot;       /* Spot radius */
   float phiw;        /* Reflection phi width */
   static float rtod = 57.29578; /* Radians to degrees conversion factor */
   int iax_h;         /* Horizontal axis */
   int iax_v;         /* Vertical axis */
   float divh;        /* Current horizontal divergence */
   float divv;        /* Current vertical divergence */
   float delamb;      /* Current delta(lambda)/lambda */
   float delcor;      /* Current correlated dispersion */
   float eta;         /* Current eta value */
   float phiwmin;     /* Minimum HALF phi width*/
   float rspotmin;    /* Minimum spot radius */
   float rspot0;      /* Spot radius when current parameter is zero */
   float fac;         /* Stored factor */
   float divergmin;   /* Minimum divergence */
   float a;           /* Temp variable */
   float rspotmax;    /* Maximum spot radius */
   float g[3];        /* Vector perpendicular to rotation axis to spot */
   float d1sq;        /* Distance**2 */
   float csimin2;     /* Cusp spot distance criterion */
   float sav_var;     /* Save current variable value */
  
   iax_h = gd->iax_h;
   iax_v = gd->iax_v;
   divh = gd->cur_divh;
   divv = gd->cur_divv;
   eta = gd->cur_eta;
   delamb = gd->cur_delamb;
   delcor = gd->delcor;

   prl_getphrc (gd->mindx, idx, &phi, &xr[0], &xr[1], &xr[2], &rlorf);
   vsq = xr[iax_v-1]*xr[iax_v-1];
   hsq = xr[iax_h-1]*xr[iax_h-1];
   dstar2 = xr[0]*xr[0] + xr[1]*xr[1] + xr[2]*xr[2];
   dst4 = 0.25*dstar2*dstar2;
   fac = sqrt(dstar2-dst4);

   *irf = 1;
   *var_min = 0.0;
   switch (ityp)
   {
      case 1: /* Rotation range */
      if (phi>gd->rots)
      {
         if (gd->isyn==0)
         {
            diverg = sqrt ((divh*divh*hsq+divv*divv*vsq)/(vsq+hsq));
         }
         else
         {
            esynh = delcor*dstar2 + xr[iax_h-1]*divh;
            esynv = divv*xr[iax_v-1];
            diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
         }
         rspot = (diverg+eta)*fac + 0.25*delamb*dstar2;
         rspotmax = rspot;
         phiw = 2.0*rspot*rlorf*rtod;
         *var_min = phi - gd->rots - phiw/2.0;
         if (*var_min<0.0) *var_min = 0.0;
         if (*var_min>gd->phirange) {*irf = 0; return;}
      }
      else if (phi<gd->rots)
      {
         if (gd->isyn==0)
         {
            diverg = sqrt ((divh*divh*hsq+divv*divv*vsq)/(vsq+hsq));
         }
         else
         {
            esynh = delcor*dstar2 + xr[iax_h-1]*divh;
            esynv = divv*xr[iax_v-1];
            diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
         }
         rspot = (diverg+eta)*fac + 0.25*delamb*dstar2;
         rspotmax = rspot;
         phiw = 2.0*rspot*rlorf*rtod;
         if (phiw/2<(gd->rots-phi)) {*irf = 0; return;}
      }
      break; 

      case 2: /* eta */
      if (phi<gd->rots||phi>gd->rots+gd->cur_phirange)
      {
         if (gd->isyn==0)
         {
            diverg = sqrt ((divh*divh*hsq+divv*divv*vsq)/(vsq+hsq));
         }
         else
         {
            esynh = delcor*dstar2 + xr[iax_h-1]*divh;
            esynv = divv*xr[iax_v-1];
            diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
         }
         phiwmin= 0.0;
         if (phi<gd->rots)
	 {
            phiwmin = gd->rots - phi;
	 }
         else if (phi>gd->rots+gd->cur_phirange)
	 {
            phiwmin = phi - gd->rots - gd->cur_phirange;
	 }
         if (phiwmin==0.0) break;
         rspotmin = phiwmin/(rlorf*rtod);
         rspot0 = (diverg)*fac + 0.25*delamb*dstar2;
         if (rspot0>rspotmin) break;
         *var_min = (rspotmin -  0.25*delamb*dstar2)/fac - diverg;
         if (*var_min>gd->eta) *irf = 0;
         rspotmax = (diverg+gd->eta)*fac + 0.25*delamb*dstar2;
      }
      break;

      case 3: /* divv */
      case 4: /* divh - only applies if delcor==0.0*/
      if (phi<gd->rots||phi>gd->rots+gd->cur_phirange)
      {
         if (phi<gd->rots)
	 {
            phiwmin = gd->rots - phi;
	 }
         else if (phi>gd->rots+gd->cur_phirange)
	 {
            phiwmin = phi - gd->rots - gd->cur_phirange;
	 }
         if (phiwmin==0.0) break;
         rspotmin = phiwmin/(rlorf*rtod);
         divergmin = (rspotmin-0.25*delamb*dstar2)/fac - eta;
         if (divergmin<=0.0) break;
         switch (ityp)
	 {
	    case 3:
            if (gd->isyn==0)
            {
               a = divergmin*divergmin*(vsq+hsq) - divh*divh*hsq;
               if (a<=0.0) break;
               *var_min = sqrt(a/vsq);
               if (*var_min>gd->divv) *irf = 0;
            }
            else
            {
               esynh = delcor*dstar2 + xr[iax_h-1]*divh;
               a = divergmin*divergmin*(vsq+hsq) - esynh*esynh;
               if (a<=0.0) break;
               *var_min = sqrt(a/vsq);
               if (*var_min>gd->divv) *irf = 0;
            }
            break;

	    case 4:
            if (gd->isyn==0)
            {
               a = divergmin*divergmin*(vsq+hsq) - divv*divv*vsq;
               if (a<=0.0) break;
               *var_min = sqrt(a/hsq);
               if (*var_min>gd->divh) *irf = 0;
            }
            else
            {
               esynv = divv*xr[iax_v-1];
               a = divergmin*divergmin*(vsq+hsq) - esynv*esynv;
               if (a<=0.0) break;
               *var_min = sqrt(a/hsq);
               if (*var_min>gd->divh) *irf = 0;
            }
            switch (ityp)
	    {
               case 3:
               if (gd->isyn==0)
               {
                  diverg = sqrt ((divh*divh*hsq+gd->divv*gd->divv*vsq)
                                /(vsq+hsq));
               }
               else
               {
                  esynh = delcor*dstar2 + xr[iax_h-1]*divh;
                  esynv = gd->divv*xr[iax_v-1];
                  diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
               }
               break;

               case 4:
               if (gd->isyn==0)
               {
                  diverg = sqrt ((gd->divh*gd->divh*hsq+divv*divv*vsq)
                                /(vsq+hsq));
               }
               else
               {
                  esynh = delcor*dstar2 + xr[iax_h-1]*gd->divh;
                  esynv = divv*xr[iax_v-1];
                  diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
               }
               break;
	    }
            rspotmax = (diverg+eta)*fac + 0.25*delamb*dstar2;
            break;
         }
      }
      break;

      case 5: /* delamb */
      if (phi<gd->rots||phi>gd->rots+gd->cur_phirange)
      {
         if (gd->isyn==0)
         {
            diverg = sqrt ((divh*divh*hsq+divv*divv*vsq)/(vsq+hsq));
         }
         else
         {
            esynh = delcor*dstar2 + xr[iax_h-1]*divh;
            esynv = divv*xr[iax_v-1];
            diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
         }
         phiwmin= 0.0;
         if (phi<gd->rots)
	 {
            phiwmin = gd->rots - phi;
	 }
         else if (phi>gd->rots+gd->cur_phirange)
	 {
            phiwmin = phi - gd->rots - gd->cur_phirange;
	 }
         if (phiwmin==0.0) break;
         rspotmin = phiwmin/(rlorf*rtod);
         rspot0 = (diverg+eta)*fac;
         if (rspot0>rspotmin) break;
         *var_min = (rspotmin -  (diverg+eta)*fac)/(0.25*dstar2);
         if (*var_min>gd->delamb) *irf = 0;
         rspotmax = (diverg+eta)*fac + 0.25*gd->delamb*dstar2;
      }
      break;
   }

   csimin2 = rspotmax*dstar2 + dst4;
   xdl_rot_sim_vcross (g, xr, gd->scax);
   d1sq = g[0]*g[0] + g[1]*g[1] + g[2]*g[2];

   /* Check for potential cusp reflections */
   if (d1sq<csimin2)
   {
      if (gd->cur_slider==1)
      {
         *irf = 1;
         if (d1sq<dst4) *irf = 0;
      }
      else
      {
         switch (gd->cur_slider)
         {
	    case 2: /* eta */
            sav_var = gd->cur_eta;
            *irf = 0;
            for (i=0;i<=50;++i)
	    {
               gd->cur_eta = 0.02*i*gd->eta;
               xdl_rot_sim_rflags (gd, idx, &ir_flag, &nimg, &ifirst);
               if (ir_flag!=0) {*irf = 1; *var_min = gd->cur_eta; break;}
	    }
            gd->cur_eta = sav_var;
            break;

	    case 3: /* divv */
            sav_var = gd->cur_divv;
            *irf = 0;
            for (i=0;i<=50;++i)
	    {
               gd->cur_divv = 0.02*i*gd->divv;
               xdl_rot_sim_rflags (gd, idx, &ir_flag, &nimg, &ifirst);
               if (ir_flag!=0) {*irf = 1; *var_min = gd->cur_divv; break;}
	    }
            gd->cur_divv = sav_var;
            break;

	    case 4: /* divh */
            sav_var = gd->cur_divh;
            *irf = 0;
            for (i=0;i<=50;++i)
	    {
               gd->cur_divh = 0.02*i*gd->divh;
               xdl_rot_sim_rflags (gd, idx, &ir_flag, &nimg, &ifirst);
               if (ir_flag!=0) {*irf = 1; *var_min = gd->cur_divh; break;}
	    }
            gd->cur_divh = sav_var;
            break;

	    case 5: /* delamb */
            sav_var = gd->cur_delamb;
            *irf = 0;
            for (i=0;i<=50;++i)
	    {
               gd->cur_delamb = 0.02*i*gd->delamb;
               xdl_rot_sim_rflags (gd, idx, &ir_flag, &nimg, &ifirst);
               if (ir_flag!=0) {*irf = 1; *var_min = gd->cur_delamb; break;}
	    }
            gd->cur_delamb = sav_var;
            break;
         }
      }
   }
   return;   
}

/*
**********************************
**     xdl_rot_sim_rflags       **
**********************************

Purpose: Get recorded spot type flags for a spot given the current
         slider parameters

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_rflags (gd, idx, ir_flag, nimg, ifirst)

XDl_rot_sim *gd;    /* Pointer to global data area */
int idx;            /* Index in reflections list */
int * ir_flag;      /* Recorded flag = 0 not-recorded
                                     = 1 full
                                     = 2 good partial (<=nwmax images)
                                     = 3 too-wide partial (>nwmax images)
                                     = 4 within cusp but part visible
                    */
int * nimg;         /* No. of images for a partial 0 if too wide */
int *ifirst;        /* Offset of first image from current image */
{
   int iax_h;         /* Horizontal axis */
   int iax_v;         /* Vertical axis */
   int isoff;         /* Start offset */
   int ieoff;         /* End offset */
   float divh;        /* Current horizontal divergence */
   float divv;        /* Current vertical divergence */
   float eta;         /* Current eta value */
   float delamb;      /* Current delta(lambda)/lambda */
   float delcor;      /* Current correlated dispersion */
   float phi;         /* Phi value */
   float xr[3];       /* Reciprocal lattice coordinates */
   float rlorf;       /* Lorentz factor */
   float vsq;         /* Vertical displacement**2 */
   float hsq;         /* Horizontal displacement**2 */
   float dstar2;      /* dstar**2 */
   float dst4;        /* 0.25 dstar2**2 */
   float fac;         /* Stored factor */
   float esynh;       /* Synchrotron horizontal divergence component */
   float esynv;       /* Synchrotron vertical divergence component */
   float diverg;      /* Current divergence */
   float rspot;       /* Spot radius */
   float phiw;        /* Reflection phi width */
   float phis;        /* Phi start */
   float phie;        /* Phi end */
   float d1sq;        /* Distance**2 */
   float g[3];        /* Vector perpendicular to rotation axis to spot */
   float csimin2;     /* Cusp spot distance criterion */
   static float rtod = 57.29578; /* Radians to degrees conversion factor */
   

  *ir_flag = 0;
  *nimg = 0;
  *ifirst = 0;

   iax_h = gd->iax_h;
   iax_v = gd->iax_v;
   divh = gd->cur_divh;
   divv = gd->cur_divv;
   eta = gd->cur_eta;
   delamb = gd->cur_delamb;
   delcor = gd->delcor;
   prl_getphrc (gd->mindx, idx, &phi, &xr[0], &xr[1], &xr[2], &rlorf);
   vsq = xr[iax_v-1]*xr[iax_v-1];
   hsq = xr[iax_h-1]*xr[iax_h-1];
   dstar2 = xr[0]*xr[0] + xr[1]*xr[1] + xr[2]*xr[2];
   dst4 = 0.25*dstar2*dstar2;
   fac = sqrt(dstar2-dst4);

   if (gd->isyn==0)
   {
      diverg = sqrt ((divh*divh*hsq+divv*divv*vsq)/(vsq+hsq));
   }
   else
   {
      esynh = delcor*dstar2 + xr[iax_h-1]*divh;
      esynv = divv*xr[iax_v-1];
      diverg = sqrt ((esynh*esynh + esynv*esynv) /(vsq+hsq));
   }
   rspot = (diverg+eta)*fac + 0.25*delamb*dstar2;
   phiw = 2.0*rspot*rlorf*rtod;
   phis = phi - 0.5*phiw;
   phie = phis + phiw;
   if (phis>gd->rots+gd->cur_phirange||phie<gd->rots) return;
   isoff = 1;
   lab20:
   isoff --;
   if (isoff<-gd->nwmax) goto lab30;
   if ((gd->rots+isoff*gd->cur_phirange)>phis) goto lab20;
   ieoff = -1;
   lab25:
   ieoff ++;
   if (ieoff>gd->nwmax) goto lab30;
   if ((gd->rots+gd->cur_phirange+ieoff*gd->cur_phirange)<phie) goto lab25;
   *nimg = ieoff-isoff+1;
   if (*nimg>gd->nwmax) goto lab30;
   *ifirst = isoff;
   *ir_flag = 1;
   if (*nimg>1) *ir_flag = 2;
   goto lab35;
   lab30:
   *nimg = 0;
   *ir_flag = 3;
   lab35:
   csimin2 = rspot*dstar2 + dst4;
   xdl_rot_sim_vcross (g, xr, gd->scax);
   d1sq = g[0]*g[0] + g[1]*g[1] + g[2]*g[2];
   if (d1sq<csimin2)
   {
      *ir_flag = 4;
      if (d1sq<dst4) *ir_flag = 0;
   }
   return;
}
/*
**********************************
**     xdl_rot_sim_vunit        **
**********************************

Purpose: Scale vector to a unit vector

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_vunit (v)

float v[3];
{
   float vmod;   /* Vector magnitude */

   vmod = v[0]*v[0] + v[1]*v[1] + v[2]*v[2];
   vmod = sqrt(vmod);
   if (vmod==0.0) return;
   v[0] = v[0]/vmod;
   v[1] = v[1]/vmod;
   v[2] = v[2]/vmod;
   return;
}
/*
**********************************
**     xdl_rot_sim_vcross       **
**********************************

Purpose: Cross product of two vectors a = (b x c)

Author:  John W. Campbell, June 1995

*/

void xdl_rot_sim_vcross (a, b, c)

float a[3];
float b[3];
float c[3];
{
   a[0] = b[1]*c[2] - c[1]*b[2];
   a[1] = b[2]*c[0] - c[2]*b[0];
   a[2] = b[0]*c[1] - c[0]*b[1];
   return;
}

/*
**********************************
**     xdl_rot_sim_compar       **
**********************************

Purpose: Comparison function for sort on current variable (in descending
         order)

Author:  John W. Campbell, June 1995

*/

int  xdl_rot_sim_compar (void *el1, void *el2)
{
   XDl_rot_sim_varidx *v1;   /* Pointer to first variable structure */
   XDl_rot_sim_varidx *v2;   /* Pointer to second variable structure */

   v1 = (XDl_rot_sim_varidx *) el1;
   v2 = (XDl_rot_sim_varidx *) el2;
   if (v1->var>v2->var) return -1;
   if (v1->var<v2->var) return 1;
   return 0;
}
 
/*===========================================================*/
/*         'Public' APPLICATION CALLABLE ROUTINES            */
/*===========================================================*/

/*-Routine: Calculate size requirements - xdl_rot_sim_getsize
The routine xdl_rot_sim_getsize (xdlf_rot_sim_getsize) is used to
get the required size for a rotation pattern simulation display view-object.
-end*/
/*
************************************
**     xdl_rot_sim_getsize        **
************************************

Purpose: Return the size requirements of the rotation pattern simulation 
         display view-object.

Author:  John W. Campbell, June 1995

*/
/*-Fortran:
         CALL XDLF_ROT_SIM_GETSIZE (ITYPE, ISIZE, IWIDTH, IHEIGHT)
-end*/
 
/*-Parameters:
ITYPE     (R)   Simulation type =1 Colour, = 2 Monochrome with
                soft limits options etc. (see type)
ISIZE     (R)   Half width/height for displayed pattern in 
                pixels (see size)
IWIDTH    (W)   Returns the width required (see w)
IHEIGHT   (W)   Returns the height required (see h)
-end*/

/*-C:*/
void xdl_rot_sim_getsize (type, size, w, h)
/*end*/
 
/*-Parameters:*/
int       type;           /* Simulation type =1 Colour, =2 Monochrome with
                             soft limit sliders etc.*/
int       size;           /* Pattern display area half width/height in
                             pixels (R)*/
int *w;                   /* Returns the width required (W)*/
int *h;                   /* Returns the height required (W)*/

/*end*/
/*-Doc:
Return:  None
-end*/
{
   int area_width;             /* Simulation display area width */
   int area_height;            /* Simulation display area height */
   int panel_1a_width;         /* Width for 'a' panel for type 1
                                  display (show reflection details) */
   int panel_1b_width;         /* Width for 'b' panel for type 1
                                  display */
   int panel_1_height;         /* Height of panels for type 1 display */
   int panel_2a_width;         /* Width for 'a' panel for type 2
                                  display (show reflection details) */
   int panel_2b_width;         /* Width for 'b' panel for type 2
                                  display (control area) */
   int panel_2_height;         /* Height of panels for type 2 display */
   int panels_width;           /* Control panels width needed */
   int panels_height;          /* Control panels height needed */

   /* Calculate the size requirements */

   if (type<1||type>2) type =1;

   /* Get size requirements */

   area_width = 2*size + 1 + 2;
   area_height = 2*size + 1 + 2;
   switch (type)
   {
      case 1:
         panel_1a_width = PANEL_1A_WIDTH;
         panel_1b_width = PANEL_1B_WIDTH;
         panel_1_height = PANEL_1_HEIGHT;
         panels_width = panel_1a_width + 4 + panel_1b_width;
         panels_height = panel_1_height;
         break;
     
      case 2:
         panel_2a_width = PANEL_2A_WIDTH;
         panel_2b_width = PANEL_2B_WIDTH;
         panel_2_height = PANEL_2_HEIGHT;
         panels_width = panel_2a_width + 4 + panel_2b_width;
         panels_height = panel_2_height;
         break;
   }
   if (panels_width>area_width)
   {
      *w = panels_width + 2*BORDER_WIDTH + 2;
   }
   else
   {
      *w = area_width + 2*BORDER_WIDTH + 2;
   }
   *h = area_height + SEP_H_MIN + panels_height 
        + 4 + PANEL_PS_HEIGHT
        + 2*BORDER_HEIGHT + 2;

   return;
}

/* Fortran binding: xdlf_rot_sim_getsize */

void xdlf_rot_sim_getsize (itype, isize, iwidth, iheight)
int *itype, *isize, *iwidth, *iheight;
{
   xdl_rot_sim_getsize (*itype, *isize, iwidth, iheight);
   return;
}
/*-Routine: Display a new simulation - xdl_rot_sim_new
The routine xdl_rot_sim_new (xdlf_rot_sim_new) is used to display a
new rotation pattern simulation in a rotation simulation view-object. 
The current simulation type (colour/interactive) and the currently selected 
options for the view-object are unchanged (apart from the orientation
if desired). Any labels will be removed.
-end*/
/*
****************************
**    xdl_rot_sim_new     **
****************************

Purpose: Replace the current simulation with a new simulation

Author:  John W. Campbell, June 1995

*/
/*-Fortran:
         CALL XDLF_ROT_SIM_NEW (IVH, IORD, NAME1, LEN1, NAME2, LEN2,
        +                       MINDX, ICRYS, IMGNUM, 
        +                       FILMR, SCAX, ROTS, ROTE, ISYN, ETA, 
        +                       DIVV, DIVH, DELAMB, DELCOR, 
        +                       IAX_V, IAX_H, NWMAX, CELL, UMAT, 
        +                       DELPHI, WAVE, LSYS, CTOD, IERR)
-end*/
 
/*-Parameters:
IVH       (R)   View-object handle (see vh)
IORD      (R)   Order of local detector axes flag 1-8 (0 if no change in
                which case axis names will also be left unchanged)
NAME1     (R)   Character string containing the name of the 1'st
                decector axis (max 4 characters in length)
                ** Pass address using the XDLSTR function **
LEN1      (R)   Length of the name string - must be >0 
                (cf len1) (max 4 chars) 
NAME2     (R)   Character string containing the name of the 2'nd
                detector axis (max 4 characters in length)
                ** Pass address using the XDLSTR function **
LEN2      (R)   Length of the name string - must be >0 
                (cf len1) (max 4 chars) 
MINDX     (R)   Index to spot list (as returned from 
                rpff_reflist_init call)
ICRYS     (R)   Crystal number
IMGNUM    (R)   Image number
FILMR     (R)   Pattern radius in mm (see filmr)
SCAX(3)   (R)   Rotation (scan) axis vector as used in the 
                prediction 
ROTS      (R)   Rotation start for current image (degrees) as used
                in the prediction
ROTE      (R)   Rotation end for current image (>ROTS) (degrees) 
                as used in the prediction
ISYN      (R)   Flag =1 synchrotron, =0 lab source as used in the
                prediction
ETA       (R)   Mosaicity (radians) as used in the prediction
DIVV      (R)   Verical divergence (half width) (radians) as used
                in the prediction
DIVH      (R)   Horizontal divergence (half width) (radians) as
                used in the prediction
DELAMB    (R)   Delta-lambda/lambda as used in the prediction    
DELCOR    (R)   Correlated dispersion (half width) as used in
                the prediction
IAX_V     (R)   No. of vertical axis (lab frame) 1=X, 2=Y, 3=Z
                as used in the prediction
IAX_H     (R)   No. of horizontal axis (lab frame) 1=X, 2=Y, 3=Z
                as used in the prediction
NWMAX     (R)   Maximum no. of images to treat as a non too-wide
                partial as used in the prediction
CELL(6)   (R)   Real cell parameters in Angstroms and degrees
                for annotation of type 2 Postscript plots
UMAT(3,3) (R)   'U' matrix for annotation of type 2 Postscript 
                plots
DELPHI(3) (R)   Missetting angles in degrees for annotation of 
                type 2 Postscript plots
WAVE      (R)   Wavelength in Angstroms for annotation of type 2 
                Postscript plots
LSYS(2)   (R)   Crystal system 1-7, Lattice type 1-7
CTOD      (R)   Crystal to detector distance (mm)
IERR      (W)   Returns status from xdl_rot_sim_new call
-end*/

/*-C:*/
int xdl_rot_sim_new (int vh, int iord, char * name1, int len1, 
                     char * name2, int len2, int mindx, int icrys, 
                     int imgnum, float filmr, float * scax, 
                     float rots, float rote, int isyn, float eta, 
                     float divv, float divh, float delamb, float delcor, 
                     int iax_v, int iax_h, int nwmax, float * cell, 
                     float * umat, float *delphi, float wave, int * lsys, 
                     float ctod) 
/*end*/
 
/*-Parameters:
int       vh;              User selected view-object handle (R)
int       iord;            Order of local detector axes flag 1-8 (if
                           0 then axis order and axis names will
                           remain unchanged) (R)
char      *name1;          Name of 1'st detector axis (max 4 chars long) 
                           (R)
int       len1;            Length of name (may give 0 if a null terminated 
                           string is passed) (R)
char      *name2;          Name of 2'nd detector axis (max 4 chars long)
                           (R)
int       len2;            Length of name (may give 0 if a null terminated 
                           string is passed) (R)
int       mindx;           Reflection list index as returned from
                           prl_init (R)
int       icrys;           Crystal number (R)
int       imgnum;          Image number (R)
float     filmr;           Pattern radius in mm (R)
float     scax[3];         Rotation (scan) axis vector as used in the
                           prediction (R)
float     rots;            Rotation start for current image (degrees) as
                           used in the prediction (R)
float     rote;            Rotation end for current image (>ROTS) 
                           (degrees) as usedin the prediction (R)
int       isyn;            Flag =1 synchrotron, =0 lab source as used
                           in the prediction (R)
float     eta;             Mosaicity (radians) as used in the 
                           prediction (R)
float     divv;            Verical divergence (half width) (radians) as
                           used in the prediction (R)
float     divh;            Horizontal divergence (half width) (radians)
                           as used in the prediction (R)
float     delamb;          Delta-lambda/lambda as used in the
                           prediction (R)    
float     delcor;          Correlated dispersion (half width) as used
                           in the prediction (R)
int       iax_v;           No. of vertical axis (lab frame) 
                           1=X, 2=Y, 3=Z as used in the prediction (R)
int       iax_h;           No. of horizontal axis (lab frame) 
                           1=X, 2=Y, 3=Z  as used in the prediction (R)
int       nwmax;           Maximum no. of images to treat as a non 
                           too-wide partial as used in the 
                           prediction (R) 
float     cell[6];         Real cell parameters in Angstroms and degrees
                           for annotation of type 2 Postscript plots 
                           (R)
float     umat[9];         'U' matrix for annotation of type 2 Postscript 
                           plots (R)
float     delphi[3];       Missetting angles in degrees for annotation of 
                           type 2 Postscript plots (R)
float     wave;            Wavelength in Angstroms for annotation of 
                           type 2 Postscript plots (R)
int       lsys[2];         Crystal system 1-7, lattice type 1-7 for
                           annotation of type 2 Postscript plots (R)
float     ctod;            Crystal to detector distance in mm for
                           annotation of type 2 Postscript plots (R)

-end*/
/*-Doc:
Return:  Status flag =0 OK, >0 error
         bit 0 set: View-object handle not found
         bit 1 set: View handle does not match view object
         bit 2 set: Unable to allocate required memory
-end*/
{
   int i;                           /* Temp/loop variable */
   int k;                           /* Temp/loop variable */
   int l;                           /* Temp/loop variable */
   int ll;                          /* String length */
   int n;                           /* No. non-blank characters in string */
   int iv;                          /* Offset in XDL_view_objects array */
   XDl_rot_sim *gd;                 /* Pointer tothe global data area */
   char str[20];                    /* Temporary character string */
   char *p;                         /* Temp pointer */
   float d;                         /* Temporary distance value */
   int nref;                        /* No. of reflections in reflections list*/
   int islide;                      /* Slider position */
   static float rtod = 57.29578;    /* Convert radians to degrees */

   iv = xdl_getiv (vh);
   if (iv<0) {xdl_view_iv_errmsg ("xdl_rot_sim_new",vh); return 1;}
   if (XDL_view_objects[iv]->type!=2111)
   {
      xdl_view_type_errmsg ("xdl_rot_sim_new", vh, "xdl_rot_sim");
      return 2;
   }
   gd = (XDl_rot_sim *) XDL_view_objects[iv]->gd;

   /* Re-allocate array to store spot 'color' flags & sort list*/

   nref = prl_numrefs (mindx);
   if (nref>0)
   {
      p = (char *) realloc (gd->colr, (nref+1)*sizeof(unsigned char));
      if (p==0) return 4;
      gd->colr = (unsigned char *) p; 
      p = (char *) realloc (gd->varidx, nref*sizeof(XDl_rot_sim_varidx));
      if (p==0) return 4;
      gd->varidx = (XDl_rot_sim_varidx *) p; 
   }

   /* Reset parameters */

   gd->mindx = mindx;
   gd->icrys = icrys;
   gd->imgnum = imgnum;
   gd->filmr = filmr;
   gd->fac = ((float)gd->half_size)/gd->filmr;
   gd->scax[0] = scax[0];
   gd->scax[1] = scax[1];
   gd->scax[2] = scax[2];
   xdl_rot_sim_vunit (gd->scax);
   gd->rots = rots;
   gd->rote = rote;
   gd->isyn = isyn;
   gd->divh = divh;
   gd->divv = divv;
   gd->eta = eta;
   gd->delamb = delamb;
   gd->delcor = delcor;
   gd->iax_h = iax_h;
   gd->iax_v = iax_v;
   gd->nwmax = nwmax;
   gd->phirange = rote - rots;
   for(i=0;i<6;++i) gd->cell[i] = cell[i];
   for(i=0;i<9;++i) gd->umat[i] = umat[i];
   for(i=0;i<3;++i) gd->delphi[i] = delphi[i];
   gd->wave = wave;
   gd->isys = lsys[0];
   gd->latt = lsys[1];
   gd->ctod = ctod;

   if (iord>=1&&iord<=8)
   {
      gd->iord = iord;
      ll = len1;
      if (ll==0) ll = strlen(name1);
      if (ll>4) ll=4;
      gd->ax1_name[0] = 0;
      if (ll>0)
      {
         n = 0;
         for (l=ll;l>=0;--l)
         {
            if (name1[l]!=' ') {n=l; break;}
         }
         if (n>0) strncat (gd->ax1_name,name1,n);
      }
      ll = len2;
      if (ll==0) ll = strlen(name2);
      if (ll>4) ll=4;
      gd->ax2_name[0] = 0;
      if (ll>0)
      {
         n = 0;
         for (l=ll;l>=0;--l)
         {
            if (name2[l]!=' ') {n=l; break;}
         }
         if (n>0) strncat (gd->ax2_name,name2,n);
      }
   }
   /* Remove labels */

   if (gd->num_labels>0)
   {
      for (i=0;i<gd->num_labels;++i)
      {
         if (gd->labels[i]!=NULL) free ((char *)gd->labels[i]);
      }
   }
   gd->num_labels = 0;
   gd->move_label = 0;

   /* Reset slider values for type 2 plots */

   islide = gd->cur_phirange_pos;
   gd->cur_phirange = gd->phirange*(islide/2)/50.0;
   if (gd->cur_slider==1)
   {
      gd->cur_var = gd->cur_phirange;
      sprintf (str,"%.3f",gd->cur_phirange);
   }

   islide = gd->cur_eta_pos;
   gd->cur_eta = gd->eta*(islide/2)/50.0;
   if (gd->cur_slider==2)
   {
      gd->cur_var = gd->cur_eta;
      sprintf (str,"%.4f",gd->cur_eta*2.0*rtod);
   }

   islide = gd->cur_divv_pos;
   gd->cur_divv = gd->divv*(islide/2)/50.0;
   if (gd->cur_slider==3)
   {
      gd->cur_var = gd->cur_divv;
      sprintf (str,"%.4f",gd->cur_divv*2.0*rtod);
   }

   islide = gd->cur_divh_pos;
   gd->cur_divh = gd->divh*(islide/2)/50.0;
   if (gd->cur_slider==4)
   {
      gd->cur_var = gd->cur_divh;
      sprintf (str,"%.4f",gd->cur_divh*2.0*rtod);
   }

   islide = gd->cur_delamb_pos;
   gd->cur_delamb = gd->delamb*(islide/2)/50.0;
   if (gd->cur_slider==5)
   {
      gd->cur_var = gd->cur_delamb;
      sprintf (str,"%.4f",gd->cur_delamb*1000.0);
   }

   if (gd->type==2)
   {
      xdl_panel_slider_reset (gd->var_slider, islide);
      xdl_panel_label_set (gd->var_curval, str, 0, 0);
   }

   gd->disp_ref = 0;
   for (k=0;k<4;++k) xdl_panel_label_set(gd->reflab[k+1]," ",0,0);

   if (gd->type==2)
   {
      xdl_rot_sim_setcolors (gd, 1, 0);
   }

   /* Draw new plot */

   switch (gd->type)
   {
      case 1:
         xdl_rot_sim_redraw1 (gd);
         break;

      case 2:
         xdl_rot_sim_redraw2 (gd);
         break;
   }
   return 0;
}

/* Fortran binding: xdlf_rot_sim_new */

void xdlf_rot_sim_new (ivh, iord, name1, len1, name2, len2,
                       mindx, icrys, imgnum, filmr, scax, 
                       rots, rote, isyn, eta, divv, divh,
                       delamb, delcor, iax_v, iax_h, nwmax, cell,
                       umat, delphi, wave, lsys, ctod, ierr)

int *ivh, *iord, *name1, *len1, *name2, *len2;
int *mindx, *icrys, *imgnum, *isyn, *iax_v, *iax_h, lsys[2], *nwmax;
int *ierr;
float *filmr, *rots, *rote, *divh, *divv, *eta, *delamb, *delcor, *scax;
float *cell, umat[9], *delphi, *wave, *ctod;
{
   float um[3][3];
   float * umm;
   int i,j,k;

   k = 0;
   for (i=0;i<3;++i)
   {
      for (j=0;j<3;++j)
      {
         um[j][i] = umat[k];
         k++;
      }
   }
   umm = (float *) um;
   *ierr = xdl_rot_sim_new (*ivh, *iord, 
                        XDL_pointers[*name1], *len1, 
                        XDL_pointers[*name2], *len2,
                        *mindx, *icrys, *imgnum, *filmr, scax,
                        *rots, *rote, *isyn, *eta, *divv, *divh,
                        *delamb, *delcor, *iax_v, *iax_h, *nwmax,
                        cell, umm, delphi, *wave, lsys, *ctod);
   return;
}
